package com.hisense.hiaposcmp;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.res.Configuration;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.WindowManager;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.multidex.MultiDex;

import com.alipay.xdevicemanager.api.XDeviceManager;
import com.hisense.hiaposcmp.boot.CrashHandler;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.hs330r.GlobalDefines;
import com.hisense.hiaposcmp.util.DeviceInfoUtils;
import com.hisense.hiaposcmp.util.IFlyTtsUtil;
import com.hisense.hipos.BankServiceHelper;
import com.hisense.pos.hs6500.HS650DeviceHelper;
import com.hisense.pos.print.hk316.HI316DeviceHelper;
import com.hisense.pos.sunmit2.printer.SunmiPrintHelper;
import com.histonepos.npsdk.api.IServiceManager;
import com.histonepos.npsdk.bind.IServiceConnector;
import com.histonepos.npsdk.bind.SmartService;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import me.jessyan.autosize.AutoSizeConfig;
import me.jessyan.autosize.unit.Subunits;


/**
 * @author caokai           2017-01-10
 * @author dengyunpeng      2017-07-17  HIPOSAPP-117 删除包含vnd.android.package-archive代码
 * @author zhangsai         2017-10-24
 * @author dukunbiao(null)  2019-07-22  重构
 * @author wangyaogang      2020-01-03  HIAPOSCMP-464 标准版打印功能开发
 */
public class App extends Application {
    private Activity mCurrentActivity;
    private Set<Activity> allActivities = new HashSet<>();
    private static App mInstance;
    private WindowManager.LayoutParams wmParams = new WindowManager.LayoutParams();

    public static XDeviceManager mXDeviceManager;

    public boolean isPaying = false;

    public WindowManager.LayoutParams getMywmParams() {
        return wmParams;
    }

    public static App getInstance() {
        return mInstance;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        // PAD 设备白名单
        if (TextUtils.equals(DeviceInfoUtils.getPackageInfo(this).packageName, "com.hisense.hipos.likelai")) {
            List<String> whiteListDevice = new ArrayList<>();
            whiteListDevice.add("APOS A8");
//        boolean isPadDevice = isPad(this);
            if (!whiteListDevice.contains(Build.MODEL)) {
                Toast.makeText(this, "不支持的设备", Toast.LENGTH_SHORT).show();
                System.exit(0);
            }
        }
        mInstance = this;
        initActivityLifecycleCallbacks();

        CrashHandler crashHandler = CrashHandler.getInstance();
        crashHandler.init(getApplicationContext());
        Config.init();

        HS650DeviceHelper.getInstance().init(getApplicationContext());
        HS650DeviceHelper.getInstance().bindService();

        if (mXDeviceManager == null) {
            mXDeviceManager = new XDeviceManager(getApplicationContext());
            mXDeviceManager.initContext();
        }

        initAutoSize();

        HI316DeviceHelper.getInstance().init(getApplicationContext());

        IFlyTtsUtil.initIflyTts(getApplicationContext());

        BankServiceHelper.bindService(getApplicationContext());

        //商米T2内置打印机
        SunmiPrintHelper.getInstance().initSunmiPrinterService(getApplicationContext());

        SmartService.getInstance().register(this, new IServiceConnector() {

            @Override
            public void onServiceDisconnected() {

            }

            @Override
            public void onServiceConnected(IServiceManager serviceManager) {
                Log.i("PosPad_Activity", "onServiceConnected");
                GlobalDefines.sm = serviceManager;
                HI316DeviceHelper.setSm(serviceManager);
                if (GlobalDefines.sm != null) {
					//Toast.makeText(getContext(), "Bind SmartService Success", Toast.LENGTH_SHORT).show();
                }
            }
        });
    }

    /**
     * 可以在 pt、in、mm 这三个冷门单位中，选择一个作为副单位，
     * 副单位是用于规避修改 DisplayMetrics#density 所造成的对于其他使用
     * dp 布局的系统控件或三方库控件的不良影响，使用副单位后可直接填写
     * 设计图上的像素尺寸，不需要再将像素转化为 dp
     * setSupportDP(false) 可保证原有页面不受新配置的影响
     */
    private void initAutoSize() {
        AutoSizeConfig.getInstance().getUnitsManager()
                .setSupportDP(false)
                .setSupportSP(false)
                .setSupportSubunits(Subunits.MM);
    }

    private void initActivityLifecycleCallbacks() {
        registerActivityLifecycleCallbacks(new ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
                allActivities.add(activity);
                setCurrentActivity(activity);
            }

            @Override
            public void onActivityStarted(Activity activity) {

            }

            @Override
            public void onActivityResumed(Activity activity) {
                setCurrentActivity(activity);
            }

            @Override
            public void onActivityPaused(Activity activity) {

            }

            @Override
            public void onActivityStopped(Activity activity) {

            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(Activity activity) {
                if (activity.equals(mCurrentActivity)) {
                    setCurrentActivity(null);
                }
                allActivities.remove(activity);

                if (allActivities.size() == 0) {
                    BankServiceHelper.unBindService(getApplicationContext());
                }
            }
        });
    }

    private void setCurrentActivity(@NonNull Activity mCurrentActivity) {
        this.mCurrentActivity = mCurrentActivity;
    }

    public Activity getCurrentActivity() {
        return mCurrentActivity;
    }

    public static Activity getContext() {
        return getInstance().getCurrentActivity();
    }

    public static Context getAppContext() {
        return getInstance().getApplicationContext();
    }

    public void exit() {
        for (Activity activity : allActivities) {
            if (activity != null) {
                activity.finish();
            }
        }
    }


    @Override
    public void onTerminate() {
        super.onTerminate();

        HS650DeviceHelper.getInstance().unbindService();
        HI316DeviceHelper.getInstance().unBindDeviceService();
        if (mXDeviceManager != null) {
            mXDeviceManager.uninitContext();
            mXDeviceManager = null;
        }

        BankServiceHelper.unBindService(getApplicationContext());

        //商米T2内置打印机
        SunmiPrintHelper.getInstance().deInitSunmiPrinterService(getApplicationContext());
    }

    @Override
    protected void attachBaseContext(Context base) {
        super.attachBaseContext(base);
        MultiDex.install(this);
    }

    public boolean isPad(Context context) {
        return (context.getResources().getConfiguration().screenLayout
                & Configuration.SCREENLAYOUT_SIZE_MASK)
                >= Configuration.SCREENLAYOUT_SIZE_LARGE;
    }


}
