package com.hisense.hiaposcmp.activity;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;

import androidx.fragment.app.Fragment;

import butterknife.ButterKnife;

/**
 * Fragment 基类，用于Fragment的统一管理。
 *
 * @author dukunbiao(null)  2019-12-30
 * @author wangyaogang      2020-01-10   抽象成懒加载模式
 */
public class BaseFragment extends Fragment {

    private int layoutId = -1;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        displayInjectLayout();
        View rootView = inflater.inflate(layoutId, container, false);
        ButterKnife.bind(this, rootView);

        return rootView;
    }

    private void displayInjectLayout() {
        Class<?> clazz = this.getClass();
        if (clazz.isAnnotationPresent(ActivityLayoutInject.class)) {
            ActivityLayoutInject inject = clazz.getAnnotation(ActivityLayoutInject.class);
            layoutId = inject.value();
        }
    }

    /**
     * 缩放动画
     *
     * @param view 待缩放控件
     */
    public void executeAnimation(View view) {
        Animation scaleAnimation = AnimationUtils.loadAnimation(getContext(), R.anim.scale_anim);
        view.startAnimation(scaleAnimation);
    }
}
