package com.hisense.hiaposcmp.activity;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Handler;
import android.os.PersistableBundle;
import android.view.MotionEvent;

import com.hisense.common.boot.SingleCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.Cashier;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.boot.ActivityCollector;
import com.hisense.hiaposcmp.boot.BootReceiver;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.dialog.AlertAliDialog;
import com.hisense.hiaposcmp.dialog.AlertStdDialog;
import com.hisense.hiaposcmp.dialog.AlertWxDialog;
import com.hisense.hiaposcmp.dialog.AutoExitAliDialog;
import com.hisense.hiaposcmp.dialog.AutoExitWxDialog;
import com.hisense.hiaposcmp.dialog.HintAliDialog;
import com.hisense.hiaposcmp.dialog.LoadingDialog;
import com.hisense.hiaposcmp.pos.payment.BocPaymentFactory;
import com.hisense.hiaposcmp.pos.payment.CcbPaymentFactory;
import com.hisense.hiaposcmp.pos.payment.IcbcPaymentFactory;
import com.hisense.hiaposcmp.pos.payment.UmsPaymentFactory;
import com.hisense.hiaposcmp.pos.payment.UnionCcbPaymentFactory;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.service.payment.AggregatePaymentChannel;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.DeviceInfoUtils;
import com.hisense.hiaposcmp.util.MidConfigUtils;
import com.hisense.hiaposcmp.util.SysBarUtils;
import com.hisense.hiaposcmp.view.IView.IBaseView;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.user.User;
import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.boc.BocPaymentServiceProvider;
import com.hisense.pos.payment.ccb.CcbPaymentServiceProvider;
import com.hisense.pos.payment.icbc.IcbcPaymentServiceProvider;
import com.hisense.pos.payment.ums.UmsPaymentServiceProvider;

import java.lang.ref.WeakReference;

/**
 * @author zhuguangsheng on 2018/7/4.
 * @author dukunbiao(null)  2019-07-22  重构
 * @author guolina          2019-10-23  重构
 * @author xieliyan         2019-11-07  全屏控制
 * @author guolina          2019-12-17  HIAPOSCMP-442  退货流水useCode和useName应该为登陆时的账户密码
 * @author wangyaogang      2020-01-06  HIAPOSCMP-473  增加标准版提示框
 * @author guolina          2020-03-10  HIAPOSCMP-577  邹平圣豪-易收银的微信、支付宝两个支付方式合并为一个
 */
public abstract class BaseMvpActivity<V extends IBaseView, T extends BasePresenter<V>> extends BaseActivity implements IBaseView {

    protected T mPresenter;
    protected boolean isNotShowExitDialog = false;

    public LoadingDialog loadingDialog;
    private ProgressDialog dialog;
    public CountDownTimer timer;


    private static final int MILLIS_IN_FUTURE = 120000;
    private static final int COUNT_DOWN_INTERVAL = 1000;

    private static User mLoginUser;
    protected MainSaleModel mainSaleModel = MainSaleModel.getInstance();

    public MainSaleModel getMainSaleModel() {
        return mainSaleModel;
    }

    public SaleModel getSale() {
        if (mainSaleModel.getSaleModel() == null) {
            return new SaleModel();
        }
        return mainSaleModel.getSaleModel();
    }

    public ReturnSaleModel getReturnSale() {
        if (mainSaleModel.getReturnSaleModel() == null) {
            return new ReturnSaleModel();
        }
        return mainSaleModel.getReturnSaleModel();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initTimer();
    }

    private void init() {
        mPresenter = createPresenter();
        mPresenter.attachView((V) this);
    }

    protected abstract T createPresenter();

    private void initTimer() {
        timer = new CountDownTimer(MILLIS_IN_FUTURE, COUNT_DOWN_INTERVAL) {
            @Override
            public void onTick(long millisUntilFinished) {
            }

            @Override
            public void onFinish() {
                try{
                    setExitDialog();
                } catch(Exception e){
                    e.printStackTrace();
                    LoggerUtils.getLogger().error("setExitDialog 自动返回欢迎页时出现异常");
                }

            }
        };
    }

    private void setExitDialog() {
        if (isNotShowExitDialog()) {
            return;
        }
        if (!mPresenter.isViewAttached()) {
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (CommonUtils.isWxStyle()) {
            AutoExitWxDialog myDialog = new AutoExitWxDialog(mPresenter.getView().getContext());
            if (getActivity(myDialog.getContext()).isFinishing()) {
                LoggerUtils.getLogger().debug("调用activity出错");
                return;
            }
            myDialog.setYesOnclickListener(() -> {
                timeStart();
                myDialog.dismiss();
            });
            myDialog.setNoOnclickListener(() -> {
                MainSaleModel.getInstance().reset();
                CommonTools.gotoWelcomeWxActivity((BaseActivity) mPresenter.getView().getContext());
                myDialog.dismiss();
            });

            myDialog.show();
        } else if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            AutoExitAliDialog myDialog = new AutoExitAliDialog(mPresenter.getView().getContext());
            if (getActivity(myDialog.getContext()).isFinishing()) {
                LoggerUtils.getLogger().debug("调用activity出错");
                return;
            }
            myDialog.setMessage("检测到该界面长时间没有操作，30s后将关闭...");
            myDialog.setYesOnclickListener(null, () -> {
                timeStart();
                myDialog.dismiss();
            });
            myDialog.show();
        }
    }

    private boolean isNotShowExitDialog() {
        return isNotShowExitDialog || (!CommonUtils.isUniformStyle());
    }

    public void timeStart() {
        new Handler(getMainLooper()).post(() -> timer.start());
    }

    @Override
    public boolean dispatchTouchEvent(MotionEvent ev) {
        try {
            if (ev.getAction() == MotionEvent.ACTION_UP) {
                if (!isNotShowExitDialog()) {
                    timer.cancel();
                    timeStart();
                }
            } else {
                timer.cancel();
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("获取触摸动作异常:" + e.toString());
        }

        return super.dispatchTouchEvent(ev);
    }

    @Override
    protected void onPause() {
        super.onPause();
        timer.cancel();
    }

    @Override
    protected void onStop() {
        super.onStop();
        timer.cancel();
    }

    @Override
    protected void onResume() {
        super.onResume();
        // 应用闪退后，Context中的对象无法自动恢复。因此，在MainActivity的onResume方法中重新初始化对象。
        com.hisense.hipos.webapi.Context.cashier(new User(Config.user_id, Config.user_code, Config.user_name));
        com.hisense.crm.webapi.Context.cashier(new Cashier(Config.user_code, Config.user_name));
        if (!isNotShowExitDialog()) {
            timeStart();
        }
    }

    @Override
    protected void onDestroy() {
        if (mPresenter != null) {
            mPresenter.detachView();
        }

        super.onDestroy();
        timer.cancel();
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        if (CommonUtils.isUniformStyle() || CommonUtils.isStdStyle() ){
            SysBarUtils.banStatusBar(this);
            SysBarUtils.hideSystemUi(this);
        }
    }

    @Override
    public void onSaveInstanceState(Bundle outState, PersistableBundle outPersistentState) {
        super.onSaveInstanceState(outState, outPersistentState);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (mPresenter == null) {
            mPresenter = createPresenter();
        }
    }

    @Override
    public Activity getActivity() {
        WeakReference<Activity> weakReference = new WeakReference<>(this);
        return weakReference.get();
    }

    @Override
    public Context getContext() {
        return this;
    }

    @Override
    public void finishThisActivity() {
        this.finish();
    }

    @Override
    public void performQuit() {
        ActivityCollector.finishAll();
        exitApp();
    }

    @Override
    public boolean performQuitBack(String value) {
        if (CommonUtils.performQuitBack(value)) {
            performQuit();
            return true;
        }

        return false;
    }

    @Override
    public void showLoadingDialog() {

        runOnUiThread(() -> {
            
            try {


                if (loadingDialog == null) {
                    loadingDialog = new LoadingDialog(getContext());
                }


                loadingDialog.show();
            } catch (Exception e) {
                //重置为null,即使偶尔内存泄漏，也不能让对话框下次还不出现
                loadingDialog = null;
                loadingDialog = new LoadingDialog(getContext());
                LoggerUtils.getLogger().error(e);
            }
        });
    }

    @Override
    public void hideLoadingDialog() {
        if (loadingDialog != null) {
            runOnUiThread(() -> {
                try {
                    loadingDialog.dismiss();
                    loadingDialog = null;
                } catch (Exception e) {
                    LoggerUtils.getLogger().error(e);
                }
            });
        }
    }

    @Override
    public void showProgressDialog(String text) {
        hideProgressDialog();
        showProgressDialogNoHide(text);
    }

    @Override
    public void showProgressDialogNoHide(String text) {
        runOnUiThread(() -> {
            try {
                if (dialog == null) {
                    dialog = new ProgressDialog(this);
                }
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                dialog.setMessage(text);
                dialog.setIndeterminate(false);
                dialog.setCancelable(false);
                dialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error(e);
            }
        });
    }

    @Override
    public void hideProgressDialog() {
        runOnUiThread(() -> {
            try {
                if (dialog != null) {
                    dialog.dismiss();
                    dialog = null;
                }
            } catch (Exception e) {
                LoggerUtils.getLogger().error(e);
            }
        });
    }

    @Override
    public void showHintDialog(String msg) {
        showHintDialog("注意", msg);
    }

    @Override
    public void showHintDialog(String title, String msg) {
        if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            showAliStyleHintDialog(title, msg);
        } else if (CommonUtils.isWxStyle()) {
            showWxStyleHintDialog(title, msg, false);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintDialog(title, msg);
        } else {
            showAliStyleHintDialog(title, msg, (m) -> {

            });
        }
    }

    @Override
    public void showHintDialog(String msg, SingleCallback callback) {
        if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            showAliStyleHintDialog("提示", msg, callback);
        } else if (CommonUtils.isWxStyle()) {
            showWxStyleHintDialog("提示", msg, callback);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintDialog("提示", msg, callback);
        } else {
            showAliStyleHintDialog("提示", msg, callback);
        }
    }

    @Override
    public void showHintDialog(String title, String msg, boolean isExit) {
        if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            showAliStyleHintDialog(title, msg);
        } else if (CommonUtils.isWxStyle()) {
            showWxStyleHintDialog(title, msg, isExit);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintDialog(title, msg);
        } else {
            showAliStyleHintDialog(title, msg, (m) -> {

            });
        }
    }

    @Override
    public void showHintConfirm(String msg, UniCallback callback) {
        showHintConfirm("提示", msg, callback);
    }

    @Override
    public void showHintConfirm(String title, String msg, UniCallback callback) {
        if (CommonUtils.isWxStyle()) {
            showWxStyleHintConfirm(title, msg, "确定", "取消", callback, true);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintConfirm(title, msg, "确定", "取消", callback);
        } else {
            showAliStyleHintConfirm(title, msg, "确定", "取消", callback);
        }
    }

    @Override
    public void showHintConfirm(String title, String msg, String yesBtn, String noBtn, UniCallback callback) {
        if (CommonUtils.isWxStyle()) {
            showWxStyleHintConfirm(title, msg, yesBtn, noBtn, callback, true);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        } else {
            showAliStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        }
    }

    @Override
    public void showHintConfirm(String style, String title, String msg, String yesBtn, String noBtn, UniCallback callback) {
        if (CommonUtils.isWxStyle(style)) {
            showWxStyleHintConfirm(title, msg, yesBtn, noBtn, callback, true);
        } else if (CommonUtils.isStdStyle(style)) {
            showStdStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        } else {
            showAliStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        }
    }

    public void showAliStyleHintDialog(String title, String message) {
        if (!mPresenter.isViewAttached()) {
            if (message != null && title != null) {
                CommonTools.toastLong(title + "\n" + message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null || title == null) {
            LoggerUtils.getLogger().info("prompt call error, message or title null!");
            return;
        }

        runOnUiThread(() -> {
            try {
                HintAliDialog commHintDialog = new HintAliDialog(mPresenter.getView().getContext());
                if (getActivity(commHintDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                commHintDialog.show();
                commHintDialog.setBasicSytle(HintAliDialog.STYLE_FAILURE);
                commHintDialog.setTextMainHead(title);
                commHintDialog.setReason(message);
            } catch (Exception e) {
                LoggerUtils.getLogger().error("调用AntHintDialog出错:" + e.toString());
                CommonTools.toastLong(title + "\n" + message);
            }
        });
    }

    public void showAliStyleHintDialog(String title, String message, SingleCallback callback) {
        runOnUiThread(() -> {
            AlertAliDialog alertDialog = new AlertAliDialog(this, false);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener("确定", () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.show();
        });
    }

    public void showAliStyleHintConfirm(String title, String message, String yesBtn, String noBtn, UniCallback callback) {
        runOnUiThread(() -> {
            AlertAliDialog alertDialog = new AlertAliDialog(this, true);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener(yesBtn, () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.setNoOnclickListener(noBtn, () -> {
                alertDialog.dismiss();
                callback.onError("");
            });
            alertDialog.show();
        });
    }

    public void showWxStyleHintDialog(String title, String message, boolean isExit) {
        if (!mPresenter.isViewAttached()) {
            if (message != null && title != null) {
                CommonTools.toastLong(title + "\n" + message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null || title == null) {
            LoggerUtils.getLogger().info("prompt call error, message or title null!");
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertWxDialog myDialog = new AlertWxDialog(mPresenter.getView().getContext(), false, true, isExit);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener("确定", () -> myDialog.dismiss());
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("调用WxAlertDialog出错:" + e.toString());
                CommonTools.toastLong(title + "\n" + message);
            }
        });
    }

    public void showWxStyleHintDialog(String title, String message, SingleCallback callback) {
        if (!mPresenter.isViewAttached()) {
            if (message != null) {
                CommonTools.toastLong(message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null) {
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertWxDialog myDialog = new AlertWxDialog(mPresenter.getView().getContext(), false, true);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener("确定", () -> {
                    myDialog.dismiss();
                    if (callback != null) {
                        callback.onSuccess("");
                    }
                });
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("showWxStyleHintDialog异常:" + e.toString());
                CommonTools.toastLong(message);
            }
        });
    }

    public void showWxStyleHintConfirm(String title, String message, String yesBtn, String noBtn, UniCallback callback, boolean isClick) {
        if (!mPresenter.isViewAttached()) {
            if (message != null) {
                CommonTools.toastLong(message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null) {
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertWxDialog myDialog = new AlertWxDialog(mPresenter.getView().getContext(), true, isClick);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener(yesBtn, () -> {
                    myDialog.dismiss();
                    if (callback != null) {
                        callback.onSuccess("");
                    }
                });
                myDialog.setNoOnclickListener(noBtn, () -> {
                    myDialog.dismiss();
                    if (callback != null) {
                        callback.onError("");
                    }
                });
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("showWxStyleHintDialog异常:" + e.toString());
                CommonTools.toastLong(message);
            }
        });
    }

    public void showStdStyleHintDialog(String title, String message) {
        if (!mPresenter.isViewAttached()) {
            if (message != null && title != null) {
                CommonTools.toastLong(title + "\n" + message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null || title == null) {
            LoggerUtils.getLogger().info("prompt call error, message or title null!");
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertStdDialog myDialog = new AlertStdDialog(mPresenter.getView().getContext(), false);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().error("调用AlertStdDialog出错,activity消失");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener("确定", () -> myDialog.dismiss());
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("调用AlertStdDialog出错:" + e.toString());
                CommonTools.toastLong(title + "\n" + message);
            }
        });
    }

    public void showStdStyleHintDialog(String title, String message, SingleCallback callback) {
        runOnUiThread(() -> {
            AlertStdDialog alertDialog = new AlertStdDialog(this, false);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener("确定", () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.show();
        });
    }

    public void showStdStyleHintConfirm(String title, String message, String yesBtn, String noBtn, UniCallback callback) {
        runOnUiThread(() -> {
            AlertStdDialog alertDialog = new AlertStdDialog(this, true);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener(yesBtn, () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.setNoOnclickListener(noBtn, () -> {
                alertDialog.dismiss();
                callback.onError("");
            });
            alertDialog.show();
        });
    }

    @Override
    public void isNotShowExitDialog(boolean bool) {
        isNotShowExitDialog = bool;

        if (!isNotShowExitDialog()) {
            timer.cancel();
            timeStart();
        } else {
            timer.cancel();
        }
    }


    protected void setUser(User user) {
        mLoginUser = user;
    }

    public static boolean hasPrivilege(String code) {
        return mLoginUser != null && mLoginUser.hasPrivilege(code);
    }

    /**
     * 返回一个新的支付服务提供程序。
     *
     * @param activity Activity
     * @return 支付服务提供程序
     */
    protected static IPaymentServiceProvider getPaymentServiceProvider(Activity activity) {
        IPaymentServiceProvider paymentProvider = null;

        if (GeneralSettings.INSTANCE.getPaymentChannel().isCCB()) {
            paymentProvider = new CcbPaymentServiceProvider(activity);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isICBC()) {
            paymentProvider = new IcbcPaymentServiceProvider(activity);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS()) {
            paymentProvider = new UmsPaymentServiceProvider(activity);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
            paymentProvider = new BocPaymentServiceProvider(activity);
        }
        return paymentProvider;
    }

    /**
     * 返回一个新的支付提供程序。
     *
     * @param activity        Activity
     * @param payMethodRemark 支付方式备注
     * @return 支付提供程序
     */
    protected static IPaymentProvider getPaymentProvider(Activity activity, String payMethodRemark) {
        IPaymentProvider paymentProvider = null;

        // TODO：使用非支付通道的聚合支付，暂不支持 IPaymentProvider 模式。2018-11-27 zhangsai
        if (!AggregatePaymentChannel.isCHANNEL()) {
            if (PayMethod.isAliPay(payMethodRemark) || PayMethod.isWxPay(payMethodRemark) || PayMethod.isMobilePay(payMethodRemark)) {
                return null;
            }
        }

        if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS()) {
            // 银联商务
            paymentProvider = UmsPaymentFactory.createProvider(activity, payMethodRemark);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isCCB()) {
            if (PayMethod.isUccbScan(payMethodRemark) || PayMethod.isUccbCard(payMethodRemark)) {
                paymentProvider = UnionCcbPaymentFactory.createProvider(activity, payMethodRemark);
            } else {
                // 建行收单
                paymentProvider = CcbPaymentFactory.createProvider(activity, payMethodRemark);
            }

        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isICBC()) {
            paymentProvider = IcbcPaymentFactory.createProvider(activity, payMethodRemark);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
            paymentProvider = BocPaymentFactory.createProvider(activity, payMethodRemark);
        }

        return paymentProvider;
    }

    protected boolean isHisenseDevice() {
        LoggerUtils.getLogger().info("当前flavor:" + MidConfigUtils.getFlavorCustom());
        LoggerUtils.getLogger().info("设备序列号:" + DeviceInfoUtils.getSystemBuildSerial());

        if (!BuildConfig.DEBUG && !Config.isHisenseDevice()) {
            showHintDialog("非海信设备不能运行此应用", (msg) -> {
                LoggerUtils.getLogger().debug("慧收银-非海信设备退出");
                App.getInstance().exit();
            });
            return false;
        }
        return true;
    }

    private Activity getActivity(Context context) {
        while (!(context instanceof Activity) && context instanceof ContextWrapper) {
            context = ((ContextWrapper) context).getBaseContext();
        }
        if (context instanceof Activity) {
            return (Activity) context;
        } else {
            return null;
        }
    }

    protected void exitApp() {
        BootReceiver.setBootUp(false);

        finishApp();
        finishAffinity();
        System.exit(0);
    }

    private void finishApp() {
        Intent startMain = new Intent(Intent.ACTION_MAIN);
        startMain.addCategory(Intent.CATEGORY_HOME);
        startMain.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(startMain);
    }
}
