package com.hisense.hiaposcmp.activity.ali;

import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SettingConstants;
import com.hisense.hiaposcmp.constant.pay.AliConstants;
import com.hisense.hiaposcmp.pos.scanner.MipcaActivityCapture;
import com.hisense.hiaposcmp.presenter.pay.PayPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.HomeWatcher;
import com.hisense.hiaposcmp.view.IView.pay.IPayView;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.pay.PayUi;
import com.hisense.hipos.payment.PayMethod;

import butterknife.BindView;

/**
 * 支付
 *
 * @author liaoyuping
 * @author xieliyan  2018-06-14
 * @author dukunbiao(null)  2019-07-25  重构
 * @author xieliyan 2020-01-13 控制多次点击
 */
@ActivityLayoutInject(R.layout.activity_pay_ant)
public class PayAliActivity extends BaseMvpActivity<IPayView, PayPresenter> implements IPayView, View.OnClickListener, AdapterView.OnItemClickListener {

    @BindView(R.id.imvClose)
    public ImageView mImvClose;

    @BindView(R.id.antWFTotal)
    public TextView textViewWf;

    @BindView(R.id.antYFTotal)
    public TextView textViewYf;

    @BindView(R.id.antYYFTotal)
    public TextView textViewYyf;

    @BindView(R.id.layoutShowCost)
    public LinearLayout layoutShowCost;

    @BindView(R.id.gvw_often)
    public GridView mGridViewOften;

    private LinearLayout layoutSelect;
    private HomeWatcher mHomeWatcher;

    private static boolean isClickPay = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //PayMethodService.createAliLayout(this, this);

        init();
        initPresenter();
        mPresenter.initSaleService();
    }

    @Override
    protected PayPresenter createPresenter() {
        return new PayPresenter();
    }

    private void init() {
        mGridViewOften.setOnItemClickListener(this);
        mImvClose.setOnClickListener(this);
        //点击home按键
        mHomeWatcher = new HomeWatcher(this);
        mHomeWatcher.setOnHomePressedListener(new HomeWatcher.OnHomePressedListener() {
            @Override
            public void onHomePressed() {
                if (getSale().paySize() > 0) {
                    CommonTools.toastLong(PayAliActivity.this, "已经进行了支付，不能返回");
                    return;
                }

                mPresenter.cancelPayFinish();
            }

            @Override
            public void onHomeLongPressed() {
                if (getSale().paySize() > 0) {
                    CommonTools.toastLong(PayAliActivity.this, "已经进行了支付，不能返回");
                    return;
                }

                mPresenter.cancelPayFinish();
            }
        });
        mHomeWatcher.startWatch();

        layoutShowCost.setVisibility(View.GONE);
    }

    private void initPresenter() {
        mPresenter.initMiYaConfig();
        mPresenter.initZoloz();
        mPresenter.initPayMethodList();
        mPresenter.getPayMethod(mGridViewOften);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK || keyCode == KeyEvent.KEYCODE_HOME || keyCode == KeyEvent.KEYCODE_MENU) {
            if (getSale().paySize() > 0) {
                CommonTools.toastLong(PayAliActivity.this, "已经进行了支付，不能返回");
                return false;
            } else {
                mPresenter.cancelPayFinish();
            }
        }

        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void onClick(View v) {

        if (isFastClick()) {
            return;
        }

        //处理支付方式按键
        if (v instanceof LinearLayout) {
            if (isClickPay) {
                LoggerUtils.getLogger().debug("已点击一次支付方式" + isClickPay);
                return;
            }
            isClickPay = true;
            if (layoutSelect != null) {
                layoutSelect.setBackgroundResource(R.drawable.border);
            }
            layoutSelect = (LinearLayout) v;
            layoutSelect.setBackgroundResource(R.drawable.border_red);

            mPresenter.updateTotal();
            PayMethod payMethod = (PayMethod) layoutSelect.getTag();
            mPresenter.pay(payMethod);
        } else {
            if (v.getId() == R.id.imvClose) {
                if (getSale().paySize() > 0) {
                    showHintDialog("已经进行了支付，不能返回");
                } else {
                    mPresenter.cancelPayFinish();
                }
            }
        }
    }

    /**
     * 普通手机摄象头扫码付
     */
    private void performStandardCameraScanPay() {
        Intent intent = new Intent();
        intent.setClass(PayAliActivity.this, MipcaActivityCapture.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SCAN);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        hideLoadingDialog();
        isClickPay = false;
        if (requestCode == ActivityConstants.REQUEST_PAY_TO_SCAN && resultCode == ActivityConstants.RESULT_PAY_TO_SCAN) {
            scanSuccessPay(data);
        } else if ((requestCode == ActivityConstants.REQUEST_PAY_TO_SCAN) && (resultCode == ActivityConstants.RESULT_PAY_TO_SUCCESS)) {
            //Todo 普通hi98扫描测试使用
            Bundle bundle = data.getExtras();
            String scanCode = bundle.getString("result");
            String signed = "";

            mPresenter.payPay(scanCode, null, signed, AliConstants.ali_pay_subject());
        } else if ((requestCode == ActivityConstants.REQUEST_PAY_TO_SUCCESS) && (resultCode == ActivityConstants.RESULT_PAY_TO_SUCCESS)) {
            saleSubmit();
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    private void scanSuccessPay(Intent data) {
        //处理支付宝扫码界面得到条码返回
        Bundle bundle = data.getExtras();
        if (bundle != null && bundle.containsKey("barcode")) {
            String scanCode = bundle.getString("barcode");
            LoggerUtils.getLogger().debug("获取支付条码:" + scanCode);

            String signed = mPresenter.signed(scanCode);

            if (SettingConstants.PAY_ANT_CHANNEL_MIYA.equals(GeneralSettings.INSTANCE.getPayAntChannel())) {
                mPresenter.payMiYaAll(scanCode);
            } else {
                if (signed == null) {
                    showHintDialog("机具签名失败，返回为null");
                } else {
                    mPresenter.payPay(scanCode, null, signed, AliConstants.ali_pay_subject());
                }
            }
        } else {
            showHintDialog("支付码为空，请重新扫码");
        }
    }

    private void saleSubmit() {
        mPresenter.submit();
    }

    @Override
    public void updateTotal() {
        textViewWf.setText(getSale().getTotalUnPaid());
        textViewYf.setText(getSale().getTotalRecv());
        textViewYyf.setText(getSale().getTotalPaid());
    }

    @Override
    public void paySuccess(SalePayModel payModel) {

    }

    @Override
    public void gotoCancelPayChannel(SalePayModel payModel) {

    }

    @Override
    public void cancelPaySuccess(SalePayModel payModel) {

    }

    @Override
    public void cancelPayError() {

    }

    @Override
    public void cancelPayFinish() {
        finishThisActivity();
    }

    @Override
    public void whenPayDone() {
        this.setResult(ActivityConstants.RESULT_SALE_TO_PAY);
        this.finish();
    }

    @Override
    protected void onResume() {
        super.onResume();
        isClickPay = false;
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mHomeWatcher.stopWatch();
    }

    /**
     * 支付扫码界面
     */
    @Override
    public void gotoScanActivity() {
        if (BuildConfig.DEBUG) {
            mPresenter.cashPay(mPresenter.getSale().getTotalUnPaid());
            return;
        }

        Bundle bundle = new Bundle();
        bundle.putString("needPayCost", textViewYf.getText().toString());
        bundle.putString("payedCost", textViewYyf.getText().toString());
        bundle.putString("payType", getSale().getPayMethod().remark());

        Intent intent = new Intent();
        intent.setClass(PayAliActivity.this, PayScanAliActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);

        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SCAN);

        LoggerUtils.getLogger().debug("支付金额:" + getSale().getTotalUnPaid());
    }

    /**
     * 支付成功界面
     */
    @Override
    public void afterPaySuccess(Bundle bundle) {
        Intent intent = new Intent(PayAliActivity.this, PaySuccessAliActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SUCCESS);
    }

    /**
     * 支付失败界面
     */
    @Override
    public void afterPayFailure(String msg) {
        Bundle bundle = new Bundle();
        bundle.putString("msg", msg);
        Intent intent = new Intent(PayAliActivity.this, PayFailureAliActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(intent);
    }

    @Override
    public void gotoCouponUseActivity() {

    }

    @Override
    public void gotoScrmCouponActivity() {

    }

    @Override
    public void gotoCardPayActivity() {

    }

    @Override
    public void gotoPointsPayActivity() {

    }

    @Override
    public void gotoOtherPayActivity() {

    }

    @Override
    public void gotoPayChannel(String totalPaid) {

    }

    @Override
    public void setIsClickPayFalse() {
        isClickPay = false;
    }

    @Override
    public void onItemClick(AdapterView<?> adapterView, View view, int position, long id) {
        if (isClickPay) {
            LoggerUtils.getLogger().debug("已点击一次支付方式" + isClickPay);
            return;
        }
        if (isFastClick()) {
            LoggerUtils.getLogger().debug("支付方式快速点击");
            return;
        }
        isClickPay = true;
        PayUi payUi = null;
        payUi = mPresenter.getPayUiOften().get(position);
        if (payUi == null) {
            CommonTools.toastShort("暂不支持该支付方式");
            return;
        }

        LoggerUtils.getLogger().debug("支付方式:" + payUi.getPayRemark());
        mPresenter.updateTotal();
        PayMethod payMethod = new PayMethod(payUi.getPayCode(), payUi.getPayName(), payUi.getPayRemark(), payUi.getPayRemark());
        mPresenter.pay(payMethod);
    }
}
