package com.hisense.hiaposcmp.activity.ali;

import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.adapter.SaleItemAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.bean.BaseEventBean;
import com.hisense.hiaposcmp.boot.WrapContentLinearLayoutManager;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.EventActions;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.presenter.sale.SalePresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.IFlyTtsUtil;
import com.hisense.hiaposcmp.view.IView.sale.ISaleView;
import com.hisense.hipos.model.BaseSalePluModel;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import butterknife.BindView;
import butterknife.OnClick;
import jp.wasabeef.recyclerview.animators.SlideInRightAnimator;

/**
 * 销售界面
 *
 * @author zhuguangsheng on 2018/5/25.
 * @author xieliyan         2018-06-02
 * @author zhuguangsheng    2018-02-02  HIPOSAPP-555 点击大购物袋小购物袋按钮，反应比较慢，平均两次加进购物车一次
 * @author xieliyan         2018-06-19  HIPOSAPP-557 称重商品禁止更改数量
 * @author xieliyan         2018-09-07  重构
 * @author dukunbiao(null)  2019-07-24  重构
 * @author xieliyan         2020-01-02 HIAPOSCMP-313 商品查询异常时语音播报
 */
@ActivityLayoutInject(R.layout.activity_sale_ant)
public class SaleAliActivity extends BaseMvpActivity<ISaleView, SalePresenter> implements ISaleView, ScanKeyEventHelper.OnScanSuccessListener {

    @BindView(R.id.ll_shopping_cart_prompt)
    public LinearLayout llShoppingCartPrompt;

    @BindView(R.id.ll_top_prompt)
    public LinearLayout llTopPrompt;

    @BindView(R.id.tv_top_prompt)
    public TextView tvTopPrompt;

    @BindView(R.id.ll_quit)
    public LinearLayout llQuit;

    @BindView(R.id.civ_quit)
    public ImageView civQuit;

    @BindView(R.id.tv_quit)
    public TextView tvQuit;

    @BindView(R.id.ll_top_print_ticket)
    public LinearLayout llPrintTicket;

    @BindView(R.id.tv_total_count)
    public TextView tvTotalCount;

    @BindView(R.id.tv_total_discount)
    public TextView tvTotalDiscount;

    @BindView(R.id.ll_big_bag)
    public LinearLayout llBigBag;

    @BindView(R.id.ll_middle_bag)
    public LinearLayout llMiddleBag;

    @BindView(R.id.ll_small_bag)
    public LinearLayout llSmallBag;

    @BindView(R.id.ll_manual_input_barcode)
    public LinearLayout llManualInputBarcode;

    @BindView(R.id.rv_shopping_cart)
    public RecyclerView mRecyclerView;

    @BindView(R.id.btn_pay)
    public Button btnPay;

    @BindView(R.id.ll_discount)
    public LinearLayout llDiscount;

    @BindView(R.id.imvLogo)
    public ImageView imvLogo;

    private SaleItemAdapter saleItemAdapter;
    private ScanKeyEventHelper mScanKeyEventHelper;

    boolean isPluDeleteEnabled = true;
    private boolean isScan = true;
    private static boolean payAntActivity;

    @Override
    protected SalePresenter createPresenter() {
        return new SalePresenter();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        IFlyTtsUtil.init(this);
        IFlyTtsUtil.setParams();
        payAntActivity = false;
        EventBus.getDefault().register(this);

        initAdapter();
        initScanKeyEventHelper();
        initPresenter();
        initBags();
        initButton();
        initLogo();

        processIntent(getIntent());
    }

    private void initAdapter() {
        mRecyclerView.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        saleItemAdapter = new SaleItemAdapter(mRecyclerView, false);
        mRecyclerView.setAdapter(saleItemAdapter);
        saleItemAdapter.setOnItemClickListener((view, viewName, position) -> processViewClick(viewName, position));

        //防崩溃java.lang.IndexOutOfBoundsException: Inconsistency detected（自带bug）
        mRecyclerView.setLayoutManager(new WrapContentLinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));

        //动画
        mRecyclerView.setItemAnimator(new SlideInRightAnimator());
        mRecyclerView.getItemAnimator().setAddDuration(300);
        mRecyclerView.getItemAnimator().setRemoveDuration(50);
        mRecyclerView.getItemAnimator().setMoveDuration(50);
        mRecyclerView.getItemAnimator().setChangeDuration(50);
    }

    private void initScanKeyEventHelper() {
        mScanKeyEventHelper = new ScanKeyEventHelper(this, true);
    }

    private void initPresenter() {
        saleItemAdapter.clearPlu();
        mPresenter.updateAdapterAndUi();
    }

    @Override
    public void updateAdapter() {
        if (saleItemAdapter != null) {
            saleItemAdapter.setData(mPresenter.getSale().getPlus());
            saleItemAdapter.notifyDataSetChanged();
        }
    }

    private void initBags() {
        llMiddleBag.setVisibility(View.VISIBLE);
    }

    private void initButton() {
        if (StringUtils.isEmpty(GeneralSettings.INSTANCE.getBigBagCode())) {
            llBigBag.setVisibility(View.INVISIBLE);
        }
        if (StringUtils.isEmpty(GeneralSettings.INSTANCE.getMiddleBagCode())) {
            llMiddleBag.setVisibility(View.INVISIBLE);
        }
        if (StringUtils.isEmpty(GeneralSettings.INSTANCE.getSmallBagCode())) {
            llSmallBag.setVisibility(View.INVISIBLE);
        }
    }

    private void initLogo() {
        if (LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH) == null) {
            imvLogo.setVisibility(View.GONE);
            tvTopPrompt.setVisibility(View.VISIBLE);
        } else {
            imvLogo.setVisibility(View.VISIBLE);
            imvLogo.setImageBitmap(LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH));
            tvTopPrompt.setVisibility(View.GONE);
        }
    }

    private void processIntent(Intent intent) {
        Bundle bundle;
        String barcode;
        try {
            bundle = intent.getExtras();
            if (null == bundle) {
                return;
            }
            if (bundle.containsKey("barcode")) {
                barcode = bundle.getString("barcode");
                EventBus.getDefault().post(new BaseEventBean(EventActions.SCAN_GOT_BARCODE, barcode));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onDestroy() {
        EventBus.getDefault().unregister(this);
        IFlyTtsUtil.destroy();
        super.onDestroy();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
        payAntActivity = false;
        isScan = true;
        initScanKeyEventHelper();
        try {
            mPresenter.updateAllUi();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    public void onlineFail(String msg) {

    }

    @Override
    public void reSubmitFail(String msg) {

    }

    @Override
    public void reSubmitSuccess(String msg) {

    }

    /**
     * 拦截键盘输入
     */
    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        mScanKeyEventHelper.analysisKeyEvent(event);
        return true;
    }

    /**
     * 监听事件
     *
     * @param viewName 商品条目的操作名称
     * @param position 商品条目的位置（从0计数）
     */
    private void processViewClick(SaleItemAdapter.ViewName viewName, int position) {
        switch (viewName) {
            case DELETE:
                processDelete(position);
                break;
            case INCREASE:
                pluAdjust(position, true);
                break;
            case DECREASE:
                pluAdjust(position, false);
                break;
            default:
                break;
        }
    }

    @Override
    public void addPlu(int position, BaseSalePluModel salePluModel) {
        saleItemAdapter.addPlu(position, salePluModel);
    }

    @Override
    public void removePlu(int position) {
        saleItemAdapter.removePlu(position);
//        updateAdapter();
    }

    @Override
    public void changeQuantity(int position) {
        saleItemAdapter.changeQuantity(position);
        updateAdapter();
    }

    private void processDelete(int position) {
        if (isFastClick()) {
            return;
        }

        if (GeneralSettings.INSTANCE.isDeleteCheckPm()) {
            Intent intent = new Intent();
            intent.setClass(this, PermissionCheckAliActivity.class);
            intent.putExtra("position", position);
            startActivityForResult(intent, ActivityConstants.REQUEST_PLU_DELETE_TO_PERMISSION);
        } else {
            showDeleteConfirmDialog(position);
        }
    }

    private void pluAdjust(int position, boolean add) {
        if (isFastClick() || !isPluDeleteEnabled) {
            return;
        }

        mPresenter.pluAdjust(position, add);
    }

    @OnClick(R.id.ll_quit)
    public void onClickQuit() {
        if (isFastClick()) {
            return;
        }

        if (mPresenter.getSale() == null) {
            showGiveUpDialog();
            return;
        }

        if (!ListUtils.isEmpty(mPresenter.getSale().getPlus()) && GeneralSettings.INSTANCE.isClearCheckPm()) {
            CommonTools.goTo(this, PermissionCheckAliActivity.class, ActivityConstants.REQUEST_SALE_CANCEL_TO_PERMISSION);
        } else {
            showGiveUpDialog();
        }
    }

    @OnClick(R.id.ll_top_print_ticket)
    public void onClickPrint() {
        if (isFastClick()) {
            return;
        }

        CommonTools.goTo(SaleAliActivity.this, PermissionCheckAliActivity.class, ActivityConstants.REQUEST_REPRINT_TO_PERMISSION);
    }

    @OnClick(R.id.ll_big_bag)
    public void onClickBig() {
        if (isFastClick()) {
            return;
        }

        mPresenter.addShoppingBag(GeneralSettings.INSTANCE.getBigBagCode());
    }

    @OnClick(R.id.ll_middle_bag)
    public void onClickMiddle() {
        if (isFastClick()) {
            return;
        }

        mPresenter.addShoppingBag(GeneralSettings.INSTANCE.getMiddleBagCode());
    }

    @OnClick(R.id.ll_small_bag)
    public void onClickSmall() {
        if (isFastClick()) {
            return;
        }

        mPresenter.addShoppingBag(GeneralSettings.INSTANCE.getSmallBagCode());
    }

    @OnClick(R.id.ll_manual_input_barcode)
    public void onClickInput() {
        if (isFastClick()) {
            return;
        }

        CommonTools.goTo(SaleAliActivity.this, PluCodeInputAliActivity.class);
    }

    @OnClick(R.id.btn_pay)
    public void onClickPay() {
        if (isFastClick()) {
            return;
        }

        if (payAntActivity) {
            LoggerUtils.getLogger().debug("是否已点击去付款:" + payAntActivity);
            return;
        }

        mPresenter.promotionCalc();
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(BaseEventBean event) {
        switch (event.getActionName()) {
            case SCAN_GOT_BARCODE:
                mPresenter.pluQuery(event.getMsgDetail());
                break;
            case ON_MANUAL_BARCODE_INPUT_COMPLETE:
                mPresenter.pluQuery(event.getMsgDetail());
                break;
            case ON_MANUAL_PERMISSION_INPUT_COMPLETE:
                mPresenter.rePrintBill(event.getMsgDetail());
                break;
            default:
                break;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == ActivityConstants.REQUEST_SALE_TO_PAY && resultCode == ActivityConstants.RESULT_SALE_TO_PAY) {
            payAntActivity = false;
            isScan = true;
            mPresenter.resetSaleModel();
        } else if (resultCode == ActivityConstants.RESULT_PERMISSION_SUCCESS) {
            if (data == null || data.getStringExtra("permissionCode") == null) {
                showHintDialog("获取授权码错误，请重试");
            } else if (!data.getStringExtra("permissionCode").equals(GeneralSettings.INSTANCE.GetPermissionCode())) {
                showHintDialog("授权码不正确");
            } else {
                if (requestCode == ActivityConstants.REQUEST_SALE_CANCEL_TO_PERMISSION) {
                    showGiveUpDialog();
                } else if (requestCode == ActivityConstants.REQUEST_PLU_DELETE_TO_PERMISSION) {
                    processDelete(data.getIntExtra("position", 0));
                } else if (requestCode == ActivityConstants.REQUEST_REPRINT_TO_PERMISSION) {
                    mPresenter.rePrintBill(data.getStringExtra("permissionCode"));
                }
            }
        }

        updateAdapter();
        updateTotal();
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void updateTotal(int count, String totalDsc, String totalRecv) {
        tvTotalCount.setText(String.valueOf(count));
        tvTotalDiscount.setText(totalDsc);

        String btnText = "¥ " + totalRecv + "    确认支付";
        btnPay.setText(btnText);
    }

    public void updateTotal() {
        if (mPresenter.getSale() != null) {
            updateTotal(mPresenter.getSale().calcCount(), mPresenter.getSale().getTotalDsc(), mPresenter.getSale().getTotalRecv());
        } else {
            updateTotal(0, "0.00", "0.00");
        }
    }

    @Override
    public void updateVipInfo(Vip vip) {
        if (vip != null) {
            tvTopPrompt.setText("您已登录成功");
            tvQuit.setText("退出登录");
            civQuit.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.logout));
        } else {
            tvTopPrompt.setText("欢迎使用");
            tvQuit.setText("重新开始");
            civQuit.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.restart));
        }
    }

    @Override
    public void hideShoppingCartPrompt() {
        llShoppingCartPrompt.setVisibility(View.GONE);
    }

    @Override
    public void showShoppingCartPrompt() {
        llShoppingCartPrompt.setVisibility(View.VISIBLE);
    }

    public void showGiveUpDialog() {
        showHintConfirm("是否放弃此次交易,重新再来", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                mPresenter.resetSaleModel();
            }

            @Override
            public void onError(String err) {

            }
        });
    }

    /**
     * 删除商品对话框
     *
     * @param position 商品条目的位置（从0计数）
     */
    private void showDeleteConfirmDialog(final int position) {
        BaseSalePluModel pluModel = mPresenter.getItem(position);

        showHintConfirm("是否删除此商品:\n" + pluModel.getPluName(), new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                mPresenter.pluDelete(position);
            }

            @Override
            public void onError(String err) {

            }
        });
    }

    @Override
    public void onScanSuccess(String barcode) {
        CommonUtils.startBeepBgm(this, barcode);

        if (!isScan) {
            LoggerUtils.getLogger().info("点击了去付款按钮，不允许继续扫码 isScan:" + isScan);
            return;
        }

        LoggerUtils.getLogger().info("扫描内容:" + barcode + "扫码状态isScan:" + isScan);

        if (GeneralSettings.INSTANCE.isCheckScanCode()) {
            if (CommonUtils.checkBarcode(barcode)) {
                return;
            }
        }

        timeStart();

        EventBus.getDefault().post(new BaseEventBean(EventActions.SCAN_GOT_BARCODE, barcode));
    }

    @Override
    public void callPayActivity() {
        payAntActivity = true;
        isScan = false;

        Intent intent = new Intent(getContext(), PayAliActivity.class);
        Bundle payBundle = new Bundle();
        intent.putExtras(payBundle);
        startActivityForResult(intent, ActivityConstants.REQUEST_SALE_TO_PAY);
    }

    @Override
    public void promotionCalcSuccess(View view) {

    }

    @Override
    public void callGiftActivity() {

    }

    @Override
    public void scrollListToBottom() {
        try {
            int position = saleItemAdapter.getItemCount() - 1;
            if (position >= 0) {
                mRecyclerView.smoothScrollToPosition(position);
            }
        } catch (Exception e) {
            CommonTools.toastLong("滚动失败:" + e.getMessage());
        }
    }

    @Override
    public void scrollListToPosition(int position) {

    }

    @Override
    public void setPluDeleteEnabled(boolean isPluDeleteEnabled) {
        this.isPluDeleteEnabled = isPluDeleteEnabled;
    }

    @Override
    public void backToWelcomeActivity() {
        CommonTools.goToWithFinishMe(SaleAliActivity.this, WelcomeAliActivity.class);
    }

    @Override
    public void pluQueryNoCode(BaseSalePluModel pluModel) {

    }

    @Override
    public void showBagsPopupWindow() {

    }

    @Override
    public void hideBagsPopupWindow() {

    }

    @Override
    public void checkoutSuccess() {

    }

    @Override
    public void checkoutError() {

    }

    @Override
    public void submitSuccess() {

    }

    @Override
    public void submitFail(String msg) {

    }
}
