package com.hisense.hiaposcmp.activity.aliland;

import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.core.content.FileProvider;

import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.AppContext;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.ali.WelcomeAliActivity;
import com.hisense.hiaposcmp.activity.std.AuthStdActivity;
import com.hisense.hiaposcmp.activity.std.ConfigStdActivity;
import com.hisense.hiaposcmp.activity.std.LoginStdActivity;
import com.hisense.hiaposcmp.activity.std.MainStdActivity;
import com.hisense.hiaposcmp.activity.wx.WelcomeWxActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.boot.BootReceiver;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.FeatureConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SettingConstants;
import com.hisense.hiaposcmp.constant.pay.AliConstants;
import com.hisense.hiaposcmp.constant.pay.WxConstants;
import com.hisense.hiaposcmp.db.DBCopyManService;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.auth.LoginPresenter;
import com.hisense.hiaposcmp.print.services.PrintImageHelper;
import com.hisense.hiaposcmp.print.services.PrintTemplateService;
import com.hisense.hiaposcmp.service.log.UploadLogService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.util.SysBarUtils;
import com.hisense.hiaposcmp.view.IView.ILoginView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.auth.LoginResultData;

import org.apache.commons.lang3.StringUtils;

import java.io.File;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 支付宝自助横屏版登录界面（适用8字连锁项目商米T2机型）
 * @author lcy  2020-06-05
 * @Description 该类内容由LoginStdActivity拷贝过来，去掉onCreate中海信设备的判断，另外加载自助横屏版布局。
 */
@ActivityLayoutInject(R.layout.activity_login_std_ali_land)
public class LoginAliLandActivity extends BaseMvpActivity<ILoginView, LoginPresenter> implements ILoginView, KeyBoardStdView.OnKeyboardListener{

    @BindView(R.id.edtUserName)
    EditText edtUserName;

    @BindView(R.id.edtUserPsw)
    EditText edtUserPsw;

    @BindView(R.id.btnLogin)
    Button btnLogin;

    @BindView(R.id.edtLoginStdKeyBoard)
    KeyBoardStdView edtLoginStdKeyBoard;

    @BindView(R.id.txtUser)
    TextView txtUser;

    @BindView(R.id.linear_inisetting)
    LinearLayout linearInisetting;

    public AlipayClient alipayClient;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initValue();
        initPayValue();
        initService();
        CommonUtils.cleanLog();
    }

    @Override
    protected LoginPresenter createPresenter() {
        return new LoginPresenter();
    }

    private void init() {
        SoftInput.disable(edtUserName);
        SoftInput.disable(edtUserPsw);
        edtLoginStdKeyBoard.setOnKeyboardListener(this);

        isNotShowExitDialog = true;
        Config.init();
        mPresenter.getInvoiceUrl();
        if (CommonUtils.isWriteOk()) {
            LoggerUtils.getLogger().info("当前渠道:" + Custom.current().getName());
        }
    }

    private void initValue() {
        if (StringUtils.isEmpty(Config.org_name)) {
            txtUser.setText(Config.org_code);
        } else {
            txtUser.setText(Config.org_code + "-" + Config.org_name);
        }
    }

    private void initPayValue() {
        if (CommonUtils.isWxStyle()) {
            initWxValue();
        } else if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            initAliValue();
        }

        FeatureConstants.initFeatureConstants();
        SettingConstants.initSettingConstants();
        Config.init();

        LoggerUtils.getLogger().info("test  getAipAppId:" + GeneralSettings.INSTANCE.getAipAppId() + "  getAipKey:" + GeneralSettings.INSTANCE.getAipKey() + "   getAipUrl:" + GeneralSettings.INSTANCE.getAipUrl());
        LoggerUtils.getLogger().info("加密授权url:" + GeneralSettings.INSTANCE.getPayAuthUrl() + "  加密授权key:" + GeneralSettings.INSTANCE.getPayAuthKey());

        if (!NetUtils.isNetworkAvailable()) {
            showHintDialog("网络错误，请检查网络");
            return;
        }

        if (!("".equals(GeneralSettings.INSTANCE.getCashierCode()) || "".equals(GeneralSettings.INSTANCE.getCashierPwd()))) {
            edtUserName.setText(GeneralSettings.INSTANCE.getCashierCode());
            edtUserPsw.setText(GeneralSettings.INSTANCE.getCashierPwd());
            new Handler().postDelayed(() -> {
                CommonTools.toastLong("准备自动跳转登录界面");
                mPresenter.login(GeneralSettings.INSTANCE.getCashierCode(), GeneralSettings.INSTANCE.getCashierPwd());
            }, 2000);
        }
    }

    private void initAliValue() {
        AliConstants.init();

        //初始化阿里人脸识别client
        alipayClient = new DefaultAlipayClient(
                AliConstants.ALI_PAY_URL, AliConstants.ali_pay_app_id(),
                AliConstants.ali_pay_app_private_key(), AliConstants.ALI_PAY_FORMAT,
                AliConstants.ALI_PAY_CHARSET, AliConstants.ali_pay_ali_public_key(),
                AliConstants.ALI_PAY_SIGN_TYPE);
    }

    public void initWxValue() {
        WxConstants.init();
    }

    private void initService() {
        //启动数据库拷贝服务
        startDbCopyManService();
        //启动日志上传服务
        startLogUploadService();
    }

    @OnClick(R.id.btnLogin)
    public void onClickLogin() {
        if (CommonUtils.isUniformStyle()) {
            mPresenter.checkApiVersion();
            return;
        }

        switch (FeatureConstants.isPayAuth) {
            case FeatureConstants.IS_ENABLE_FEATURE:
                mPresenter.preLogin();
                break;
            case FeatureConstants.IS_DISABLE_FEATURE:
                mPresenter.checkApiVersion();
                break;
        }
    }

    @OnClick({R.id.linear_inisetting, R.id.edtUserName, R.id.edtUserPsw})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.linear_inisetting:
                onClickConfig();
                break;
            case R.id.edtUserName:
                edtUserName.setFocusable(true);
                edtUserName.setFocusableInTouchMode(true);
                edtUserName.requestFocus();
                break;
            case R.id.edtUserPsw:
                edtUserPsw.setFocusable(true);
                edtUserPsw.setFocusableInTouchMode(true);
                edtUserPsw.requestFocus();
                break;
            default:
                break;
        }
    }

    public void onClickConfig() {
        boolean isConfiged = CommonTools.loginConfig(this).getBoolean(Config.IS_SETTING, false);
        if (!CommonUtils.isUniformStyle() && isConfiged) {
            AuthStdActivity.authToServiceSettings(this);
        } else {
            //慧收银 或者 未配置的易收银可进入此判断,理论上此处只有慧收银可进入
            Intent intent = new Intent(this, ConfigStdActivity.class);
            intent.putExtra("isFromLogin", true);
            startActivity(intent);
        }
    }

    @Override
    public void installApk(File file) {
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        Uri fileUri;

        //Android7.0以上
        if (Build.VERSION.SDK_INT >= 24) {
            fileUri = FileProvider.getUriForFile(this,
                    BuildConfig.APPLICATION_ID + ".provider", file);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        } else {
            fileUri = Uri.fromFile(file);
        }

        intent.setDataAndType(fileUri, "application/vnd.android.package-archive");
        startActivity(intent);
    }

    @Override
    public void checkApiVersionSuccess() {
        mPresenter.login(edtUserName.getText().toString(), edtUserPsw.getText().toString());
    }

    @Override
    public void loginSuccess(LoginResultData loginResult) {
        Config.user_code = loginResult.getUserCode();
        Config.user_id = loginResult.getUserId();
        Config.user_name = loginResult.getUserName();
        Config.org_name = Context.orgName();
        Config.shop_id = loginResult.getShopId();

        Config.save();
        Config.init();

        // 用户登录时重置 APP 运行的上下文环境，避免不同用户登录时产生冲突
        AppContext.reset();

        jumpUi();
    }

    @Override
    public void queryPrivilegesSuccess(LoginResultData loginResult, User user) {

    }

    private void jumpUi() {
        LoggerUtils.getLogger().debug("登录成功，当前版本为:" + getVersion());

        if (CommonUtils.isWxStyle()) {
            CommonTools.goTo(LoginAliLandActivity.this, WelcomeWxActivity.class);
        } else if (CommonUtils.isAliStyle()) {
            CommonTools.goTo(LoginAliLandActivity.this, WelcomeAliActivity.class);
        } else if(CommonUtils.isAliLandStyle()){
            CommonTools.goTo(LoginAliLandActivity.this, WelcomeAliLandActivity.class);
        } else if (CommonUtils.isStdStyle()) {
            CommonTools.goTo(LoginAliLandActivity.this, MainStdActivity.class);
        } else {
            showHintDialog("暂未支持该界面风格");
            return;
        }

        hideLoadingDialog();
        LoginAliLandActivity.this.finish();
        BootReceiver.setBootUp(false);
    }

    @Override
    public void onStart() {
        super.onStart();
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onResume() {
        super.onResume();

        initValue();

        //FIXME null 再议
        if (Config.isConfiged()) {
            PrintTemplateService.checkUpdate(LoginAliLandActivity.this);
            mPresenter.shopList();
            PrintImageHelper.checkUpdate(LoginAliLandActivity.this);
        }

        SysBarUtils.banStatusBar(this);

        //检查更新
        mPresenter.appCheckUpdate(getPackageName(), getVersion());
    }

    @Override
    protected void onPause() {
        super.onPause();

        SysBarUtils.unBanStatusBar(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            CommonTools.showConfirmDialog("确定要退出吗？", (dialog, which) -> {
                exitApp();
            });
        }

        return false;
    }

    private void startDbCopyManService() {
        Intent intent = new Intent(this, DBCopyManService.class);
        stopService(intent);
        startService(intent);
    }

    private void startLogUploadService() {
        Intent intent = new Intent(this, UploadLogService.class);
        stopService(intent);
        startService(intent);
    }

    @Override
    protected void onScan(String s) {
        super.onScan(s);
    }

    @Override
    public void onInsertKeyEvent(String text) {
        if (edtUserName.isFocused()) {
            KeyBoardStdView.addChar(edtUserName, text);
        } else if (edtUserPsw.isFocused()) {
            KeyBoardStdView.addChar(edtUserPsw, text);
        }
    }

    @Override
    public void onDeleteKeyEvent() {
        if (edtUserName.isFocused()) {
            KeyBoardStdView.deleteChar(edtUserName);
        } else if (edtUserPsw.isFocused()) {
            KeyBoardStdView.deleteChar(edtUserPsw);
        }
    }

    @Override
    public void onClearKeyEvent() {
        if (edtUserName.isFocused()) {
            KeyBoardStdView.clearChar(edtUserName);
        } else if (edtUserPsw.isFocused()) {
            KeyBoardStdView.clearChar(edtUserPsw);
        }
    }

    @Override
    public void onDoneKeyEvent() {
        if (edtUserPsw.isFocused()) {
            if (CommonUtils.isUniformStyle()) {
                mPresenter.checkApiVersion();
                return;
            }
            switch (FeatureConstants.isPayAuth) {
                case FeatureConstants.IS_ENABLE_FEATURE:
                    mPresenter.preLogin();
                    break;
                case FeatureConstants.IS_DISABLE_FEATURE:
                    mPresenter.checkApiVersion();
                    break;
            }
        }
    }
}
