package com.hisense.hiaposcmp.activity.aliland;

import android.content.Intent;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.CommonConstants;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.presenter.pay.PaySuccessPresenter;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.util.IFlyTtsUtil;
import com.hisense.hiaposcmp.view.IView.pay.IPaySuccessView;
import com.hisense.hipos.model.SalePayModel;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 支付成功展示界面
 *
 * @author dukunbiao(null)  2018-06-19
 * @author dukunbiao(null)  2019-07-25  重构
 * @author xieliyan         2020-01-02 HIAPOSCMP-467 打印小票
 * @author xieliyan         2020-01-02 HIAPOSCMP-313 支付完成语音播报
 */
@ActivityLayoutInject(R.layout.activity_pay_success_ant_landscape)
public class PaySuccessAliLandActivity extends BaseMvpActivity<IPaySuccessView, PaySuccessPresenter> implements IPaySuccessView {

    @BindView(R.id.paySuccessTvReceipt)
    public TextView paySuccessTvReceipt;

    @BindView(R.id.paySuccessTvAccount)
    public TextView paySuccessTvAccount;

    @BindView(R.id.paySuccessTvNumber)
    public TextView paySuccessTvNumber;

    @BindView(R.id.paySuccessTvTotal)
    public TextView paySuccessTvTotal;

    @BindView(R.id.paySuccessLinearMDiscount)
    public LinearLayout paySuccessLinearMDiscount;

    @BindView(R.id.paySuccessLinearDiscount)
    public LinearLayout paySuccessLinearDiscount;

    @BindView(R.id.paySuccessTvMDiscount)
    public TextView paySuccessTvMDiscount;

    @BindView(R.id.paySuccessTvDiscount)
    public TextView paySuccessTvDiscount;

    @BindView(R.id.paySuccessBtnClose)
    public Button paySuccessBtnClose;

    @BindView(R.id.txtShow)
    public TextView txtShow;

    @BindView(R.id.imvLogo)
    public ImageView imvLogo;

    private String amount = "";
    private String count = "";
    private String payName = "";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        IFlyTtsUtil.init(this);
        IFlyTtsUtil.setParams();
        mPresenter.print();
        init();
        initLogo();
        IFlyTtsUtil.start(payName + "支付成功，支付" + amount + "元，" + count + "请拿好小票，欢迎下次购物");
        beginCountDown();
    }

    @Override
    protected PaySuccessPresenter createPresenter() {
        return new PaySuccessPresenter();
    }

    private void init() {
        Intent intent = this.getIntent();
        Bundle bundle = intent.getExtras();
        if (bundle == null) {
            return;
        }

        SalePayModel payModel = (SalePayModel) bundle.getSerializable("payModel");

        paySuccessTvReceipt.setText("￥" + payModel.getTotalPaid());
        paySuccessTvNumber.setText("共" + mPresenter.getSale().calcCount() + "件商品");
        paySuccessTvTotal.setText("￥" + payModel.getTotal());

        amount = payModel.getTotalPaid();
        count = paySuccessTvNumber.getText().toString() + ",";
        payName = payModel.getPayName();

        if (mPresenter.getSale().getPayMethod().isWxPay()) {
            txtShow.setText("微信付款成功");

            paySuccessTvAccount.setText("");
        } else {
            txtShow.setText("支付宝付款成功");

            paySuccessTvAccount.setText("支付宝账号" + payModel.getBuyerLogonId());
        }

        if (BigDecimalUtils.isGreater(payModel.getTotalDscSelf(), "0")) {
            paySuccessTvMDiscount.setText("-￥" + payModel.getTotalDscSelf());
        } else {
            paySuccessLinearMDiscount.setVisibility(View.INVISIBLE);
        }

        if (BigDecimalUtils.isGreater(payModel.getTotalDscChannel(), "0")) {
            paySuccessTvDiscount.setText("-￥" + payModel.getTotalDscChannel());
        } else {
            paySuccessLinearDiscount.setVisibility(View.INVISIBLE);
        }
    }

    private void beginCountDown() {
        new CountDownTimer(CommonConstants.PAY_SUCCESS_MILLIS_IN_FUTURE, CommonConstants.PAY_SUCCESS_COUNT_DOWN_INTERVAL) {

            @Override
            public void onTick(long millisUntilFinished) {
                //重置倒计时文本
                paySuccessBtnClose.setText("关闭   (" + String.format("%ds", millisUntilFinished / 1000) + ")");
            }

            @Override
            public void onFinish() {
                finishSelf();
            }
        }.start();
    }

    private void initLogo() {
        //设置完成界面LOGO
        if (LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH) == null) {
            imvLogo.setVisibility(View.GONE);
        } else {
            imvLogo.setVisibility(View.VISIBLE);
            imvLogo.setImageBitmap(LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH));
        }
    }

    @OnClick(R.id.paySuccessBtnClose)
    public void onClickClose() {
        finishSelf();
    }

    private String unSensitive(String str) {
        String result;

        try {
            if (!str.contains("@")) {
                result = str.substring(0, 3) + "******" + str.substring(9, 11);
            } else {
                result = str.substring(0, 3) + "***" + str.substring(str.indexOf("@"));
            }
        } catch (Exception e) {
            result = str;
        }

        return result;
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        return true;
    }

    protected void finishSelf() {
        Intent intent = new Intent();
        PaySuccessAliLandActivity.this.setResult(ActivityConstants.RESULT_PAY_TO_SUCCESS, intent);
        PaySuccessAliLandActivity.this.finish();
        finish();
    }
}
