package com.hisense.hiaposcmp.activity.aliland;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.PermissionCheckPresenter;
import com.hisense.hiaposcmp.view.IView.IPermissionView;
import com.hisense.hiaposcmp.view.KeyBoardAliView;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * @author liaoyuping on 2018/6/25.
 * @author dukunbiao(null)  2019-07-24  重构
 */
@ActivityLayoutInject(R.layout.activity_permissioncheck_ant_landscape)
public class PermissionCheckAliLandActivity extends BaseMvpActivity<IPermissionView, PermissionCheckPresenter> implements IPermissionView, KeyBoardAliView.IOnKeyboardListener {

    @BindView(R.id.img_permCheckQuit)
    public ImageView imgPermCheckQuit;

    @BindView(R.id.keyboar_ant_permcheck)
    public KeyBoardAliView myKeyboard;

    @BindView(R.id.edt_inputPrem)
    public EditText edtInputCode;

    @BindView(R.id.txt_permCheckTip)
    public TextView txtTip;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @Override
    protected PermissionCheckPresenter createPresenter() {
        return new PermissionCheckPresenter();
    }

    private void init() {
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
        myKeyboard.setIOnKeyboardListener(this);

        txtTip.setText("需要授权 \n        请联系管理员扫描或录入授权码");

        edtInputCode.setOnKeyListener((v, keyCode, event) -> {
            if (keyCode == KeyEvent.KEYCODE_ENTER) {
                onDoneKeyEvent();

                return true;
            }
            return false;
        });

        edtInputCode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (StringUtils.isEmpty(s.toString())) {
                    edtInputCode.setFocusable(true);
                    edtInputCode.requestFocus();
                }
            }
        });
    }

    @OnClick(R.id.img_permCheckQuit)
    public void onClickQuit() {
        finish();
    }

    @Override
    public void onInsertKeyEvent(String text) {
        edtInputCode.append(text);
    }

    @Override
    public void onDeleteKeyEvent() {
        String inputStr = edtInputCode.getText().toString().trim();
        if (inputStr.length() >= 1) {
            edtInputCode.setText(inputStr.substring(0, inputStr.length() - 1));
            edtInputCode.setSelection(edtInputCode.getText().toString().length());
        }
    }

    @Override
    public void onDoneKeyEvent() {
        if (isFastClick()) {
            return;
        }

        response();
        finish();
    }

    private String getText() {
        return edtInputCode.getText().toString().trim();
    }

    private void response() {
        Intent mIntent = new Intent();
        if (getIntent().getExtras() != null) {
            if (getIntent().getExtras().containsKey("position")) {
                mIntent.putExtra("position", getIntent().getExtras().getInt("position"));
            }
        }
        mIntent.putExtra("permissionCode", getText());
        setResult(ActivityConstants.RESULT_PERMISSION_SUCCESS, mIntent);
    }
}
