package com.hisense.hiaposcmp.activity.aliland;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.common.boot.GlobalCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.common.AuthActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.constant.CommonConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.pos.print.PrintProvider;
import com.hisense.hiaposcmp.presenter.WelcomePresenter;
import com.hisense.hiaposcmp.service.sale.SaleService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.IWelcomeView;
import com.hisense.pos.hs6500.HS650PrintProvider;
import com.hisense.pos.print.IPrintProvider;
import com.hisense.pos.sunmit2.printer.SunmiPrintProvider;

import org.greenrobot.eventbus.EventBus;

import androidx.annotation.Nullable;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 蚂蚁金服 欢迎界面
 * Created by zhuguangsheng on 2018/5/23.
 * edit by wangwenyue
 *
 * @author xieliyan 2018-07-05 HIAPOS-3 检测未传流水自动上传
 * @author dukunbiao(null)  2019-07-22  重构
 */
@ActivityLayoutInject(R.layout.activity_welcome_ant_landscape)
public class WelcomeAliLandActivity extends BaseMvpActivity<IWelcomeView, WelcomePresenter> implements IWelcomeView {

    @BindView(R.id.btn_vip)
    public Button btnVipLogin;

    @BindView(R.id.btn_novip)
    public Button btnNoVipLogin;

    @BindView(R.id.ll_logo)
    public LinearLayout llLogo;

    @BindView(R.id.iv_test)
    public ImageView ivTest;

    @BindView(R.id.tvPrinterInfo)
    public TextView tvPrinterInfo;

    @BindView(R.id.tv_debug)
    public TextView tvDebug;

    long[] mHits = new long[CommonConstants.CLICK_COUNTS];

    private ScanKeyEventHelper mScanKeyEventHelper;
    private boolean printerPaperOut = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

//        //隐藏导航栏
//        WindowManager.LayoutParams params = getWindow().getAttributes();
//
//        params.systemUiVisibility = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION|View.SYSTEM_UI_FLAG_IMMERSIVE;
//
//        getWindow().setAttributes(params);
        hideBottomUIMenu();
        initView();
        startSaleService();
        mPresenter.initTimer();
    }

    /**
     * 隐藏虚拟按键，并且全屏
     */
    protected void hideBottomUIMenu() {
        //隐藏虚拟按键，并且全屏
        if (Build.VERSION.SDK_INT > 11 && Build.VERSION.SDK_INT < 19) { // lower api
            View v = this.getWindow().getDecorView();
            v.setSystemUiVisibility(View.GONE);
        } else if (Build.VERSION.SDK_INT >= 19) {
            //for new api versions.
            View decorView = getWindow().getDecorView();
            int uiOptions = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY | View.SYSTEM_UI_FLAG_FULLSCREEN;
            decorView.setSystemUiVisibility(uiOptions);
        }
    }

    @Override
    protected WelcomePresenter createPresenter() {
        return new WelcomePresenter();
    }

    private void initView() {
        isNotShowExitDialog = true;

        if (GeneralSettings.INSTANCE.isUseVip()) {
            btnVipLogin.setVisibility(View.VISIBLE);
            btnNoVipLogin.setText("非会员");
        } else {
            btnVipLogin.setVisibility(View.GONE);
            btnNoVipLogin.setText("开始交易");
        }

        if (isApkInDebug(this)) {
            tvDebug.setVisibility(View.VISIBLE);
        } else {
            tvDebug.setVisibility(View.GONE);
        }
    }

    @OnClick(R.id.btn_novip)
    public void onClickNoVip() {
        if (isFastClick()) {
            return;
        }

        mPresenter.createNewSale(new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                CommonTools.goTo(WelcomeAliLandActivity.this, SaleAliLandActivity.class);
                onPause();
            }

            @Override
            public void onError(String msg) {
                showHintDialog("网络错误,请重试", msg);
            }
        });
    }

    @OnClick(R.id.btn_vip)
    public void onClickToVip() {
        if (isFastClick()) {
            return;
        }

        mPresenter.createNewSale(new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                CommonTools.goTo(WelcomeAliLandActivity.this, VipLoginAliLandActivity.class);
                onPause();
            }

            @Override
            public void onError(String msg) {
                showHintDialog("网络错误,请重试", msg);
            }
        });
    }

    private void onScanToSale(final String barcode) {
        mPresenter.createNewSale(new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                Bundle bundle = new Bundle();
                bundle.putString("barcode", barcode);
                CommonTools.goTo(WelcomeAliLandActivity.this, SaleAliLandActivity.class, bundle);
                onPause();
            }

            @Override
            public void onError(String msg) {
                showHintDialog("网络错误,请重试", "无法创建新订单" + msg);
            }
        });
    }

    @OnClick(R.id.ll_logo)
    public void navigateBackstage() {
        System.arraycopy(mHits, 1, mHits, 0, mHits.length - 1);
        //实现左移，然后最后一个位置更新距离开机的时间，如果最后一个时间和最开始时间小于DURATION，即连续5次点击
        mHits[mHits.length - 1] = SystemClock.uptimeMillis();
        if (mHits[0] >= (SystemClock.uptimeMillis() - CommonConstants.CLICK_DURATION)) {
            //达到点击次数，清理数组
            for (int i = 0; i < mHits.length; i++) {
                mHits[i] = 0;
            }
            AuthActivity.authToManagement(this);
        }
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        mScanKeyEventHelper.analysisKeyEvent(event);
        return true;
    }

    private void scanResult() {
        mScanKeyEventHelper = new ScanKeyEventHelper((barcode) -> {
            if (printerPaperOut) {
                showHintDialog("打印机缺纸或异常", "请联系管理员");
                LoggerUtils.getLogger().debug("打印机缺纸或异常");
                return;
            }
            onScanToSale(barcode);
        });
    }

    @Override
    protected void onPause() {
        super.onPause();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
        scanResult();

        showLoadingDialog();
        CommonUtils.reInitSaleModel(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                hideLoadingDialog();
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                showHintDialog(err);
            }
        });

        initView();
        if (BuildConfig.DEBUG) {
            printerPaperOut = false;
        } else {
            getPrintStatus();
        }
        mPresenter.timeStart();

    }

    @Override
    protected void onDestroy() {
        EventBus.getDefault().unregister(this);
        mScanKeyEventHelper.onDestroy();
        mPresenter.timeCancel();
        super.onDestroy();
    }

    /**
     * 判断apk是否是debug模式
     *
     * @param context
     * @return
     */
    public static boolean isApkInDebug(Context context) {
        try {
            ApplicationInfo info = context.getApplicationInfo();
            return (info.flags & ApplicationInfo.FLAG_DEBUGGABLE) != 0;
        } catch (Exception e) {
            return false;
        }
    }

    @OnClick(R.id.tvPrinterInfo)
    public void getPrintStatus() {
        printerPaperOut = false;
        tvPrinterInfo.setText("");

        if (GeneralSettings.INSTANCE.isUsePrinter()) {
            try {
                IPrintProvider hs650PrintProvider = null;
                if(GeneralSettings.INSTANCE.getDeviceType().isHSX()){
                    hs650PrintProvider = new HS650PrintProvider();
                } else if(GeneralSettings.INSTANCE.getDeviceType().isSunmiT2()){
                    hs650PrintProvider = new SunmiPrintProvider();
                } else{
                    hs650PrintProvider = new HS650PrintProvider();
                }
                if (hs650PrintProvider.initPrinter(GeneralSettings.INSTANCE.getPrinterRate(), GeneralSettings.INSTANCE.getPrinterPort(),
                        GeneralSettings.INSTANCE.getDeviceType().isHS650() ? 0 : 1, GeneralSettings.INSTANCE.getPrinterFreeLine())) {
                    if (hs650PrintProvider.getStatus() == 1) {
                        printerPaperOut = true;
                        tvPrinterInfo.setText("打印机缺纸或异常，请联系管理员");
                        LoggerUtils.getLogger().debug("打印机缺纸或异常");
                    }
                    hs650PrintProvider.closePrinter();
                } else {
                    printerPaperOut = true;
                    tvPrinterInfo.setText("打印机初始化失败，请联系管理员");
                    LoggerUtils.getLogger().debug("打印机初始化失败");
                }
            } catch (Exception e) {
                tvPrinterInfo.setText("打印机异常，请联系管理员");
                LoggerUtils.getLogger().debug("打印机获取缺纸状态异常" + e.toString());
            }
        }
    }

    @Override
    public void startSaleService() {
        Intent intent = new Intent(this, SaleService.class);
        startService(intent);
    }

    @Override
    public void checkPrintStatus() {
        getPrintStatus();
    }
}
