/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.common;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.std.ConfigStdActivity;
import com.hisense.hiaposcmp.activity.std.SettingStdActivity;
import com.hisense.hiaposcmp.activity.std.refund.ReturnStdActivity;
import com.hisense.hiaposcmp.activity.ysy.SettingActivity;
import com.hisense.hiaposcmp.activity.ysy.refund.SaleReturnActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.layout.CombinedEditText;
import com.hisense.hiaposcmp.presenter.auth.LoginPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.ILoginView;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.hipos.user.Privileges;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.auth.LoginResultData;

import java.io.File;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 系统设置登录对话框。
 *
 * @author xieliyan 2017-07-14
 * @author zhangsai 2017-10-20 HIPOSAPP-186   升级用户身份验证的WebAPI调用
 * @author dukunbiao(null)  2019-07-25        重构
 * @author guolina  2019-07-25 HIAPOSCMP-370  修复登录页面多次点击设置按钮弹出2个设置窗口
 */
@ActivityLayoutInject(R.layout.activity_auth)
public class AuthActivity extends BaseMvpActivity<ILoginView, LoginPresenter> implements ILoginView {

    @BindView(R.id.txtUserCode)
    public CombinedEditText txtUserCode;

    @BindView(R.id.txtPassword)
    public CombinedEditText txtPassword;

    private static final int GOTO_RETURN = 1;
    private static final int GOTO_SETTINGS = 2;
    private static final int GOTO_SERVICE_SETTINGS = 3;
    private static final int GOTO_MANAGEMENT = 4;

    public static final String ADMIN_REFUND = "9996";
    public static final String ADMIN_SETTING = "9999";

    private int mFlag;
    private static final String DEFAULT_PSD = "654321";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        initValue();
    }

    @Override
    protected LoginPresenter createPresenter() {
        return new LoginPresenter();
    }

    private void initValue() {
        Bundle bundle = getIntent().getExtras();
        if (bundle != null) {
            mFlag = bundle.getInt("flag");
        }

        String mUserCode = "";
        if (mFlag == GOTO_RETURN) {
            mUserCode = ADMIN_REFUND;
        } else if (mFlag == GOTO_SETTINGS) {
            mUserCode = ADMIN_SETTING;
        } else if (mFlag == GOTO_MANAGEMENT) {
            mUserCode = ADMIN_SETTING;
        } else if (mFlag == GOTO_SERVICE_SETTINGS) {
            mUserCode = ADMIN_SETTING;
        }

        txtUserCode.setText(mUserCode);

        KeyBoardView keyBoard = txtUserCode.getKeyboard();
        txtPassword.setKeyboard(keyBoard);
        txtPassword.requestFocus();
    }

    @OnClick(R.id.btnOk)
    public void onClickOk() {

        if (mFlag == GOTO_SERVICE_SETTINGS
                && txtPassword.getText().toString().trim().equals(DEFAULT_PSD)) {
            Intent intent;
            if (CommonUtils.isStdStyle()) {
                intent = new Intent(this, ConfigStdActivity.class);
            } else {
                intent = new Intent(this, ConfigActivity.class);
            }

            intent.putExtra("isFromLogin", true);
            startActivity(intent);
            finish();
        } else {
            // TODO 国美大屏自助临时跳转
            mPresenter.login(txtUserCode.getText().toString(), txtPassword.getText().toString());
            //CommonTools.goTo(AuthActivity.this, PosManageActivity.class);
            //finish();
        }
    }

    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        finish();
    }

    /**
     * 用户认证后转到退货界面。
     *
     * @param activity 调起认证的 Activity
     */
    public static void authToReturn(Activity activity) {
        authToActivity(activity, GOTO_RETURN);
    }

    /**
     * 用户认证后转到系统设置界面。
     *
     * @param activity 调起认证的 Activity
     */
    public static void authToSystemSettings(Activity activity) {
        authToActivity(activity, GOTO_SETTINGS);
    }

    /**
     * 用户认证后转到服务地址设置页面
     *
     * @param activity 调起认证的 Activity
     */
    public static void authToServiceSettings(Activity activity) {
        authToActivity(activity, GOTO_SERVICE_SETTINGS);
    }

    /**
     * 用户认证后转到管理界面
     *
     * @param activity 调起认证的 Activity
     */
    public static void authToManagement(Activity activity) {
        authToActivity(activity, GOTO_MANAGEMENT);
    }

    /**
     * 用户认证后转到 {@param value} 指定的界面。
     *
     * @param activity 调起认证的 Activity
     * @param flag     通过认证后跳转的 Activity 的标识
     */
    private static void authToActivity(Activity activity, int flag) {
        Bundle bundle = new Bundle();
        bundle.putInt("flag", flag);

        Intent intent = new Intent();
        intent.putExtras(bundle);
        intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        intent.setClass(activity, AuthActivity.class);
        activity.startActivity(intent);
    }

    @Override
    public void loginSuccess(LoginResultData loginResult) {
        mPresenter.queryPrivileges(loginResult);
    }

    @Override
    public void queryPrivilegesSuccess(LoginResultData loginResult, User user) {
        if (user == null) {
            showNoPrivilege(loginResult);
            return;
        }

        if (mFlag == GOTO_RETURN && user.hasPrivilege(Privileges.SALE_RETURN)) {
            goToReturn();
            finish();
        } else if (mFlag == GOTO_SETTINGS && user.hasPrivilege(Privileges.SETTING)) {
            if (CommonUtils.isStdStyle()) {
                CommonTools.goTo(AuthActivity.this, SettingStdActivity.class);
            } else {
                CommonTools.goTo(AuthActivity.this, SettingActivity.class);
            }

            finish();
        } else if (mFlag == GOTO_SERVICE_SETTINGS && user.hasPrivilege(Privileges.SETTING)) {
            Intent intent;
            if (CommonUtils.isStdStyle()) {
                intent = new Intent(this, ConfigStdActivity.class);
            } else {
                intent = new Intent(this, ConfigActivity.class);
            }
            intent.putExtra("isFromLogin", true);
            startActivity(intent);
            finish();
        } else if (mFlag == GOTO_MANAGEMENT && user.hasPrivilege(Privileges.SETTING)) {
            CommonTools.goTo(AuthActivity.this, PosManageActivity.class);
            finish();
        } else {
            showNoPrivilege(loginResult);
        }
    }

    private void showNoPrivilege(LoginResultData loginResult) {
        String privilege = "";
        if (mFlag == GOTO_RETURN) {
            privilege = "退货";
        } else if (mFlag == GOTO_SETTINGS || mFlag == GOTO_SERVICE_SETTINGS) {
            privilege = "修改设置";
        }
        showHintDialog("用户[" + loginResult.getUserCode() + "]没有" + privilege + "权限");
    }

    private void goToReturn() {
        Bundle bundle = new Bundle();
        bundle.putString("authNo", txtUserCode.getText().toString());
        Intent intent = new Intent();
        intent.putExtras(bundle);
        if (CommonUtils.isStdStyle()) {
            intent.setClass(AuthActivity.this, ReturnStdActivity.class);
        } else {
            intent.setClass(AuthActivity.this, SaleReturnActivity.class);
        }

        startActivity(intent);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    public void installApk(File file) {

    }

    @Override
    public void checkApiVersionSuccess() {

    }
}
