package com.hisense.hiaposcmp.activity.common;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.ProgressBar;

import com.hisense.common.boot.SingleCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.JSONUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.aliland.LoginAliLandActivity;
import com.hisense.hiaposcmp.activity.std.ConfigStdActivity;
import com.hisense.hiaposcmp.activity.std.LoginStdActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.auth.algorithm.Des;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.presenter.BootUpPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.FileUtils;
import com.hisense.hiaposcmp.util.MidConfigUtils;
import com.hisense.hiaposcmp.view.IView.IBootUpView;
import com.qw.soul.permission.SoulPermission;
import com.qw.soul.permission.bean.Special;
import com.qw.soul.permission.callbcak.GoAppDetailCallBack;
import com.qw.soul.permission.callbcak.SpecialPermissionListener;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import butterknife.BindView;

import static com.hisense.common.util.DateUtils.dateFormat;

/**
 * 从中间件获取慧收银配置信息
 *
 * @author xieliyan
 * @author dukunbiao(null)  2019-07-25  重构
 * @author xieliyan         2019-12-24  HIAPOSCMP-446  启动页加载时间
 * @author wangyaogang      2019-12-24  HIAPOSCMP-465  标准版-6.0+权限申请,打开设置页授权
 */
@ActivityLayoutInject(R.layout.activity_bootup)
public class BootUpActivity extends BaseMvpActivity<IBootUpView, BootUpPresenter> implements IBootUpView, View.OnFocusChangeListener {

    @BindView(R.id.pb_loading)
    public ProgressBar progressBar;

    private static final int COUNTDOWN_MILLISEC = 150 * 1000;
    private static final int COUNTDOWN_INTEVAL = 1000;

    private Handler mainHandler = new Handler();
    private CountDownTimerUtil mTimerUtil;
    private String appName = "";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        init();
    }

    @Override
    protected BootUpPresenter createPresenter() {
        return new BootUpPresenter();
    }

    private void init() {
        appName = getApplicationInfo().loadLabel(getPackageManager()).toString();
        isNotShowExitDialog = true;

        MidConfigUtils.initMidConfigParams();
    }

    @Override
    protected void onResume() {
        super.onResume();

        checkAndRequestSystemAlert();
    }

    /**
     * 先检查有无悬浮窗权限,没有则申请,有再检查一般权限,如读写,没有则申请,有执行原操作
     */
    private void checkAndRequestSystemAlert() {
        boolean checkResult = SoulPermission.getInstance().checkSpecialPermission(Special.SYSTEM_ALERT);
        if (checkResult) {
            Log.d("TradeLog", "已有 System Alert ");
            List<String> permissions = new ArrayList<>();
           /* if (!SoulPermission.getInstance().checkSinglePermission(Manifest.permission.READ_PHONE_STATE).isGranted()) {
                permissions.add("电话");
            }
            if (!SoulPermission.getInstance().checkSinglePermission(Manifest.permission.CAMERA).isGranted()) {
                permissions.add("相机");
            }
            if (!SoulPermission.getInstance().checkSinglePermission(Manifest.permission.ACCESS_COARSE_LOCATION).isGranted()) {
                permissions.add("位置信息");
            }*/
            if (!SoulPermission.getInstance().checkSinglePermission(Manifest.permission.WRITE_EXTERNAL_STORAGE).isGranted()) {
                permissions.add("存储空间");
            }
            if (permissions.isEmpty()) {
                LoggerUtils.getLogger().debug("权限都有了继续下一步");
                LoggerUtils.getLogger().debug("DEVICE-TAG 设备型号：" + Build.MODEL);
                doInResume();
            } else {
                Log.d("TradeLog", "某一个权限没有,打开设置");
                showSettingDialog(BootUpActivity.this, permissions);
            }
        } else {
            Log.d("TradeLog", "没有 System Alert ,去申请");
            showHintConfirm("standard", appName, getString(R.string.message_permission_alert_window),
                    getString(R.string.message_btn_yes),
                    getString(R.string.message_btn_no),
                    new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            Log.d("TradeLog", "checkAndRequestSystemAlert");
                            //if you want do noting or no need all the callbacks you may use SimpleSpecialPermissionAdapter instead
                            SoulPermission.getInstance().checkAndRequestPermission(Special.SYSTEM_ALERT, new SpecialPermissionListener() {
                                @Override
                                public void onGranted(Special permission) {
                                    Log.d("TradeLog", "System Alert onGranted:");
                                }

                                @Override
                                public void onDenied(Special permission) {
                                    Log.d("TradeLog", "System Alert onDenied:");
                                }
                            });
                        }

                        @Override
                        public void onError(String err) {
                            Log.d("TradeLog", "拒绝checkAndRequestSystemAlert");
                            showHintDialog("由于您没有授予权限,程序即将退出", new SingleCallback() {
                                @Override
                                public void onSuccess(String msg) {
                                    finish();
                                }
                            });
                        }
                    });
        }
    }

    /**
     * 打开设置页 ,授予一般权限
     */
    public void showSettingDialog(Context context, final List<String> permissions) {
        String message = context.getString(R.string.message_permission_always_failed,
                TextUtils.join("\n", permissions));
        showHintConfirm("standard", appName, message,
                getString(R.string.message_btn_yes),
                getString(R.string.message_btn_no),
                new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        Log.d("TradeLog", "AndPermission_showSettingDialog允许");
                        goApplicationSettings();
                    }

                    @Override
                    public void onError(String err) {
                        Log.d("TradeLog", "AndPermission_showSettingDialog拒绝");
                        showHintDialog(getString(R.string.message_auth_fail), new SingleCallback() {
                            @Override
                            public void onSuccess(String msg) {
                                finish();
                            }
                        });
                    }
                });
    }

    public void goApplicationSettings() {
        SoulPermission.getInstance().goApplicationSettings(new GoAppDetailCallBack() {
            @Override
            public void onBackFromAppDetail(Intent data) {
                //if you need to know when back from app detail , 第一次允许拿不到结果....所以不使用这个回调
            }
        });
    }

    private void doInResume() {
        LoggerUtils.getLogger().debug("doInResume");
        mainHandler.post(() -> {
            int result = mPresenter.fetchMidConfig();

            //当未配置地址时直接跳转到配置界面
            if (result == -1) {
                mainHandler.post(() -> jumpLogic());
                return;
            }

            //配置地址时展示进度条
            initCountDown();
        });
    }

    @Override
    protected void onPause() {
        super.onPause();
        try {
            mTimerUtil.cancel();
        } catch (Exception e) {

        }
    }

    @Override
    public CountDownTimerUtil countTimerUtil() {
        return mTimerUtil;
    }

    private void initCountDown() {
        mTimerUtil = new CountDownTimerUtil(COUNTDOWN_MILLISEC, COUNTDOWN_INTEVAL) {
            @Override
            public void onTick(long millisUntilFinished) {
                int progress = (int) ((COUNTDOWN_MILLISEC - millisUntilFinished) * 100 / COUNTDOWN_MILLISEC);
                progressBar.setProgress(progress);
            }

            @Override
            public void onFinish() {
                progressBar.setProgress(100);
                mainHandler.post(() -> jumpLogic());
            }
        }.start();
    }

    private void jumpLogic() {
        if (JSONUtils.isEmpty(MidConfigUtils.loadMidConfigParams())) {
            gotoConfigActivity();
        } else {
            gotoLoginActivity();
        }
    }

    private void gotoConfigActivity() {
        isFirstInstall();
        if (CommonUtils.isStdStyle() || CommonUtils.isUniformStyle()) {
            CommonTools.goTo(this, ConfigStdActivity.class);
        } else {
            CommonTools.goTo(this, ConfigActivity.class);
        }
    }

    private void gotoLoginActivity() {
        isFirstInstall();
        if (CommonUtils.isStdStyle() || CommonUtils.isUniformStyle()) {
            if (CommonUtils.isAliLandStyle()){
                CommonTools.goToWithFinishMe(this, LoginAliLandActivity.class);
            } else {
                CommonTools.goToWithFinishMe(this, LoginStdActivity.class);
            }
        } else {
            CommonTools.goToWithFinishMe(this, LoginActivity.class);
        }
    }

    private void isFirstInstall() {
        String filePth = Environment.getExternalStorageDirectory().getPath() + "/android/data/installed.txt";
        try {
            FileUtils.writeFileContent(filePth, Des.encrypt2Base64("ilovehisense", dateFormat(new Date())));
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("加密异常" + e.toString());
        }
    }

    @Override
    public void onFocusChange(View view, boolean b) {

    }

    @Override
    public void configDownloadFail() {
        gotoConfigActivity();
    }
}
