/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.common;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.common.boot.UniCallback;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.presenter.ConfigPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IConfigView;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 登录设置。
 *
 * @author fujiaxin 2017-03-14
 * @author zhangsai 2017-10-25 HIPOSAPP-192 保存设置时检查服务地址
 * @author zhangsai 2017-11-30 HIPOSAPP-205
 * @author zhangsai 2017-12-21 HIPOSAPP-252 增加重置功能，防止用户误操作导致必须重复输入设置信息的问题
 * @author dukunbiao(null)  2019-07-25  重构
 * @author liuyazeng2020-07-14 HIAPOSCMP-890 配置地址添加二维码扫描
 */
@ActivityLayoutInject(R.layout.activity_login_config)
public class ConfigActivity extends BaseMvpActivity<IConfigView, ConfigPresenter> implements IConfigView, View.OnFocusChangeListener {

    @BindView(R.id.linearOrgCode)
    public LinearLayout linearOrgCode;

    @BindView(R.id.txtOrgCode)
    public EditText txtOrgCode;

    @BindView(R.id.linearShopCode)
    public LinearLayout linearShopCode;

    @BindView(R.id.txtShopCode)
    public EditText txtShopCode;

    @BindView(R.id.linearPosNo)
    public LinearLayout linearPosNo;

    @BindView(R.id.txtPosNo)
    public EditText txtPosNo;

    @BindView(R.id.txtServiceUrl)
    public EditText txtServiceUrl;

    @BindView(R.id.txtVersion)
    public TextView txtVersion;

    @BindView(R.id.imgOrgCode)
    public ImageView imgOrgCode;

    @BindView(R.id.imgShopCode)
    public ImageView imgShopCode;

    @BindView(R.id.imgPosNo)
    public ImageView imgPosNo;

    @BindView(R.id.linearServiceUrl)
    public LinearLayout linearServiceUrl;

    @BindView(R.id.imgServiceUrl)
    public ImageView imgServiceUrl;

    @BindView(R.id.btnGoBack)
    public LinearLayout btnGoBack;

    @BindView(R.id.lytReset)
    public LinearLayout lytReset;

    @BindView(R.id.btnConfig)
    public Button btnConfig;

    private boolean isFromLogin;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initValue();
        if (mPresenter.isRootRight()) {
            showHintConfirm("非可信环境", "检测到非可信环境,是否退出应用", new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    exitApp();
                }

                @Override
                public void onError(String err) {

                }
            });
        }
    }

    @Override
    protected ConfigPresenter createPresenter() {
        return new ConfigPresenter();
    }

    private void init() {
        isNotShowExitDialog = true;

        txtShopCode.setOnFocusChangeListener(this);
        txtOrgCode.setOnFocusChangeListener(this);
        txtPosNo.setOnFocusChangeListener(this);
        txtServiceUrl.setOnFocusChangeListener(this);
        txtVersion.setText(this.getVersion() + " 渠道:" + Custom.current().getName());
    }

    /**
     * 加载设置信息。
     */
    private void initValue() {
        boolean isNotFirstConfig = CommonTools.loginConfig(this).getBoolean(Config.IS_SETTING, false);
        if (isNotFirstConfig) {
            // 如果非首次设置，则显示返回按钮。首次设置时，返回按钮不显示。
            btnGoBack.setVisibility(View.VISIBLE);
            lytReset.setVisibility(View.VISIBLE);
        }

        txtOrgCode.setText(CommonTools.loginConfig(this).getString(Config.ORG_CODE, ""));
        txtShopCode.setText(CommonTools.loginConfig(this).getString(Config.SHOP_CODE, ""));
        txtPosNo.setText(CommonTools.loginConfig(this).getString(Config.POS_NO, ""));
        txtServiceUrl.setText(CommonTools.loginConfig(this).getString(Config.SERVER_URL, ""));
        mScanner = getScanner(this);
        isFromLogin = false;
        Intent intent = this.getIntent();
        Bundle bundle = intent.getExtras();
        if (bundle != null && bundle.containsKey("isFromLogin")) {
            linearOrgCode.setVisibility(View.GONE);
            linearShopCode.setVisibility(View.GONE);
            linearPosNo.setVisibility(View.GONE);
            btnConfig.setText("下一步");

            isFromLogin = bundle.getBoolean("isFromLogin");
        }

        if (bundle != null && bundle.containsKey("pos")) {
            int pos = getIntent().getIntExtra("pos", 0);

            linearServiceUrl.setVisibility(View.GONE);
            setFocus(pos);
        }
    }

    private void setFocus(int pos) {
        switch (pos) {
            default:
            case 1:
                txtOrgCode.requestFocus();
                txtOrgCode.setSelection(txtOrgCode.getText().length());
                break;
            case 2:
                txtShopCode.requestFocus();
                txtShopCode.setSelection(txtShopCode.getText().length());
                break;
            case 3:
                txtPosNo.requestFocus();
                txtPosNo.setSelection(txtPosNo.getText().length());
                break;
        }
    }

    @Override
    public void onFocusChange(View v, boolean b) {
        switch (v.getId()) {
            case R.id.txtOrgCode:
                imgOrgCode.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            case R.id.txtShopCode:
                imgShopCode.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            case R.id.txtPosNo:
                imgPosNo.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            case R.id.txtServiceUrl:
                imgServiceUrl.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            default:
                break;
        }
    }

    @OnClick(R.id.imageScan)
    public void onScanClick() {
        scan();
    }

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                txtServiceUrl.setText(s);
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }


    /**
     * 接收Activity结果值
     *
     * @param requestCode 请求编号。
     * @param resultCode  返回编号。
     * @param data        返回数据。
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            txtServiceUrl.setText(mScanner.getScanResult(data));
        }
    }

    @OnClick(R.id.btnGoBack)
    public void onClickBack() {
        onBackPressed();
        finish();
    }

    @OnClick(R.id.btnConfig)
    public void onClickSave() {
        checkAndSave();
    }

    @OnClick(R.id.lytReset)
    public void onClickReset() {
        resetValue();
    }

    /**
     * 重新加载设置信息。
     */
    private void resetValue() {
        txtOrgCode.setText(CommonTools.loginConfig(this).getString(Config.ORG_CODE, ""));
        txtShopCode.setText(CommonTools.loginConfig(this).getString(Config.SHOP_CODE, ""));
        txtPosNo.setText(CommonTools.loginConfig(this).getString(Config.POS_NO, ""));
        txtServiceUrl.setText(CommonTools.loginConfig(this).getString(Config.SERVER_URL, ""));
    }

    @OnClick(R.id.lytClearOrgCode)
    public void onClickClearOrgCode() {
        if (imgOrgCode.getVisibility() == View.VISIBLE) {
            txtOrgCode.setText("");
        }
    }

    @OnClick(R.id.lytClearShopCode)
    public void onClickClearShopCode() {
        if (imgShopCode.getVisibility() == View.VISIBLE) {
            txtShopCode.setText("");
        }
    }

    @OnClick(R.id.lytClearPosNo)
    public void onClickClearPosNo() {
        if (imgPosNo.getVisibility() == View.VISIBLE) {
            txtPosNo.setText("");
        }
    }

    @OnClick(R.id.lytClearServiceUrl)
    public void onClickClearServiceUrl() {
        if (imgServiceUrl.getVisibility() == View.VISIBLE) {
            txtServiceUrl.setText("");
        }
    }

    @Override
    public void serviceOnlineSuccess() {
        if (isFromLogin) {
            isFromLogin = false;

            linearOrgCode.setVisibility(View.VISIBLE);
            linearShopCode.setVisibility(View.VISIBLE);
            linearPosNo.setVisibility(View.VISIBLE);
            txtOrgCode.requestFocus();
            txtOrgCode.setSelection(txtOrgCode.getText().toString().length());
            btnConfig.setText("确定");
        } else {
            finish();
        }
    }

    /**
     * 检查用户输入并保存设置。
     */
    private void checkAndSave() {
        if (isFromLogin) {
            mPresenter.checkAndSave(txtServiceUrl.getText().toString().trim());
        } else {
            mPresenter.checkAndSave(txtOrgCode.getText().toString().trim(), txtShopCode.getText().toString().trim(),
                    txtPosNo.getText().toString().trim(), txtServiceUrl.getText().toString().trim());
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

}
