package com.hisense.hiaposcmp.activity.common;

import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.view.KeyEvent;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.core.content.FileProvider;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.AppContext;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.ysy.MainActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.boot.BootReceiver;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.FeatureConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SettingConstants;
import com.hisense.hiaposcmp.db.DBCopyManService;
import com.hisense.hiaposcmp.layout.CombinedEditText;
import com.hisense.hiaposcmp.presenter.auth.LoginPresenter;
import com.hisense.hiaposcmp.print.services.PrintImageHelper;
import com.hisense.hiaposcmp.print.services.PrintTemplateService;
import com.hisense.hiaposcmp.service.log.UploadLogService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.SysBarUtils;
import com.hisense.hiaposcmp.view.IView.ILoginView;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.auth.LoginResultData;

import org.apache.commons.lang3.StringUtils;

import java.io.File;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 登录
 *
 * @author fujiaxin         2017-01-11
 * @author fujiaxin         2017-03-14
 * @author fujiaxin         2017-09-12  HIPOSAPP-154 版本更新前首先判断数据表中是否有该列
 * @author xieliyan         2017-09-27  HIPOSAPP-160 刷卡（调用HI98驱动）：获取射频卡信息
 * @author zhangsai         2017-10-20  HIPOSAPP-186 升级用户身份验证的WebAPI调用
 * @author zhangsai         2017-10-24  HIPOSAPP-189 升级打印模板更新的WebAPI调用
 * @author zhangsai         2017-10-26  HIPOSAPP-193 退出时提示用户
 * @author zhangsai         2017-10-30  HIPOSAPP-189 升级打印模板更新的WebAPI调用
 * @author xieliyan         2017-11-22  HIPOSAPP-224 射频卡密码获取方式更改（删除）
 * @author zhangsai         2017-12-21  HIPOSAPP-250 删除获取结账日期的代码
 * @author zhangsai         2018-03-28  HIPOSAPP-369 登录前检查 WebAPI 版本，不兼容时强制更新
 * @author lizhixin         2018-05-11  HIPOSAPP-445 检测设备，非海信设备不能运行app
 * @author xieliyan         2018-07-09  HIAPOS-12
 * @author dukunbiao(null)  2019-07-22  重构
 * @author guolina          2019-10-21  HIPOSAPP-1317 利客来未上传流水自动上传
 * @author guolina          2019-11-28  HIAPOSCMP-286 修复初次安装登陆界面软键盘点击完成不隐藏
 * @author liuyazeng        2020-07-17  HIAPOSCMP-845 登录界面支持显示款台号
 */
@ActivityLayoutInject(R.layout.activity_login)
public class LoginActivity extends BaseMvpActivity<ILoginView, LoginPresenter> implements ILoginView, ScanKeyEventHelper.OnScanSuccessListener {

    @BindView(R.id.linearConfig)
    public ImageView linearConfig;

    @BindView(R.id.txtOrgCode)
    public TextView txtOrgCode;

    @BindView(R.id.edtUserCode)
    public CombinedEditText edtUserCode;

    @BindView(R.id.edtPassword)
    public CombinedEditText edtPassword;

    @BindView(R.id.btnLogin)
    public Button btnLogin;

    @BindView(R.id.txtVersion)
    public TextView txtVersion;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (!hasConfig()) {
            onClickConfig();
            return;
        }
        init();
        initValue();
        initPayValue();
        initService();
        CommonUtils.cleanLog();
    }

    @Override
    protected LoginPresenter createPresenter() {
        return new LoginPresenter();
    }

    private void init() {
        isNotShowExitDialog = true;
        Config.init();
        mPresenter.getInvoiceUrl();
        LoggerUtils.getLogger().info("当前渠道:" + Custom.current().getName());
        BootReceiver.setBootUp(false);
    }

    private void initValue() {
        KeyBoardView keyboard = edtUserCode.getKeyboard();
        edtPassword.setKeyboard(keyboard);

        if (StringUtils.isEmpty(Config.org_name)) {
            txtOrgCode.setText(Config.org_code);
        } else {
            if (StringUtils.isEmpty(Config.pos_no)) {
                txtOrgCode.setText(Config.org_code + "-" + Config.org_name);
            } else {
                txtOrgCode.setText(Config.org_code + "-" + Config.pos_no + "-" + Config.org_name);
            }
        }

        txtVersion.setText(this.getVersion());
    }

    private void initPayValue() {
        FeatureConstants.initFeatureConstants();
        SettingConstants.initSettingConstants();
        Config.init();
        LoggerUtils.getLogger().info("test  getAipAppId:" + GeneralSettings.INSTANCE.getAipAppId() + "  getAipKey:" + GeneralSettings.INSTANCE.getAipKey() + "   getAipUrl:" + GeneralSettings.INSTANCE.getAipUrl());
        LoggerUtils.getLogger().info("加密授权url:" + GeneralSettings.INSTANCE.getPayAuthUrl() + "  加密授权key:" + GeneralSettings.INSTANCE.getPayAuthKey());
    }

    private void initService() {
        //启动数据库拷贝服务
        startDbCopyManService();
        //启动日志上传服务
        startLogUploadService();
    }

    private boolean hasConfig() {
        return CommonTools.loginConfig(this).getBoolean(Config.IS_SETTING, false);
    }

    @OnClick(R.id.btnLogin)
    public void onClickLogin() {
        if (CommonUtils.isUniformStyle()) {
            mPresenter.checkApiVersion();
            return;
        }
        switch (FeatureConstants.isPayAuth) {
            case FeatureConstants.IS_ENABLE_FEATURE:
                mPresenter.preLogin();
                break;
            case FeatureConstants.IS_DISABLE_FEATURE:
                mPresenter.checkApiVersion();
                break;
        }

    }

    @OnClick(R.id.linearConfig)
    public void onClickConfig() {
        boolean isConfiged = CommonTools.loginConfig(this).getBoolean(Config.IS_SETTING, false);
        if (!CommonUtils.isUniformStyle() && isConfiged) {
            AuthActivity.authToServiceSettings(this);
        } else {
            Intent intent = new Intent(this, ConfigActivity.class);
            intent.putExtra("isFromLogin", true);
            startActivity(intent);
        }
    }

    @Override
    public void installApk(File file) {
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        Uri fileUri;

        //Android7.0以上
        if (Build.VERSION.SDK_INT >= 24) {
            fileUri = FileProvider.getUriForFile(this,
                    BuildConfig.APPLICATION_ID + ".provider", file);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        } else {
            fileUri = Uri.fromFile(file);
        }

        intent.setDataAndType(fileUri, "application/vnd.android.package-archive");
        startActivity(intent);
    }

    @Override
    public void checkApiVersionSuccess() {
        mPresenter.login(edtUserCode.getText().toString(), edtPassword.getText().toString());
    }

    @Override
    public void loginSuccess(LoginResultData loginResult) {
        Config.user_code = loginResult.getUserCode();
        Config.user_id = loginResult.getUserId();
        Config.user_name = loginResult.getUserName();
        Config.org_name = Context.orgName();
        Config.shop_id = loginResult.getShopId();

        Config.save();
        Config.init();

        // 用户登录时重置 APP 运行的上下文环境，避免不同用户登录时产生冲突
        AppContext.reset();

        jumpUi();
    }

    @Override
    public void queryPrivilegesSuccess(LoginResultData loginResult, User user) {

    }

    /**
     * UI跳转控制，区分不同版本
     */
    private void jumpUi() {
        LoggerUtils.getLogger().debug("登录成功，当前版本为:" + getVersion());

        CommonTools.goTo(LoginActivity.this, MainActivity.class);
        hideLoadingDialog();
        LoginActivity.this.finish();
        BootReceiver.setBootUp(false);
    }

    @Override
    public void onStart() {
        super.onStart();

       /* if (RootTools.isDeviceRooted()) {
            showHintDialog("检测到系统有root风险，继续使用将造成数据不安全", (msg) ->
                    ActivityCollector.finishAll()
            );
        }*/
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onResume() {
        super.onResume();
        Config.init();
        initValue();

        //FIXME null 再议
        if (Config.isConfiged()) {
            PrintTemplateService.checkUpdate(LoginActivity.this);
            mPresenter.shopList();
            PrintImageHelper.checkUpdate(LoginActivity.this);
        }

        SysBarUtils.banStatusBar(this);

        //检查更新
        mPresenter.appCheckUpdate(getPackageName(), getVersion());
    }

    @Override
    protected void onPause() {
        super.onPause();

        SysBarUtils.unBanStatusBar(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            CommonTools.showConfirmDialog("确定要退出吗？", (dialog, which) -> {
                exitApp();
            });
        }

        return false;
    }

    private void startDbCopyManService() {
        Intent intent = new Intent(this, DBCopyManService.class);
        stopService(intent);
        startService(intent);
    }

    private void startLogUploadService() {
        Intent intent = new Intent(this, UploadLogService.class);
        stopService(intent);
        startService(intent);
    }

    @Override
    public void onScanSuccess(String barcode) {

    }
}
