package com.hisense.hiaposcmp.activity.common;

import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.bigkoo.pickerview.builder.TimePickerBuilder;
import com.bigkoo.pickerview.view.TimePickerView;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.presenter.UpLogPresenter;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.IUpLogView;

import java.util.Calendar;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 设备日志上传（手动）
 *
 * @author xieliyan 2019-06-25
 * @author dukunbiao(null)  2019-07-25  重构
 */
@ActivityLayoutInject(R.layout.activity_up_log)
public class UpLogActivity extends BaseMvpActivity<IUpLogView, UpLogPresenter> implements IUpLogView {

    @BindView(R.id.tv_time)
    public TextView mTvTime;

    @BindView(R.id.btn_time)
    public Button mBtnTime;

    @BindView(R.id.btn_upload)
    public Button mBtnUpload;

    @BindView(R.id.tv_tip)
    public TextView mTvTip;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (CommonUtils.isStdStyle()) {
            mBtnTime.setBackground(getResources().getDrawable(R.drawable.selector_btn_std));
            mBtnUpload.setBackground(getResources().getDrawable(R.drawable.selector_btn_std));
        }
        mPresenter.initHttpUtils();
    }

    @Override
    protected UpLogPresenter createPresenter() {
        return new UpLogPresenter();
    }

    @OnClick(R.id.btnGoBack)
    public void onClickBack() {
        finish();
    }

    @OnClick(R.id.btn_time)
    public void onClickTime(View view) {
        mTvTip.setText("");
        initTimePicker(mTvTime);
    }

    @OnClick(R.id.btn_upload)
    public void onClickUpload(View view) {
        mTvTip.setText("");

        try {
            mPresenter.startUploadRoutine(mTvTime.getText().toString());
        } catch (Exception e) {
            LoggerUtils.getLogger().error(e.toString());
            showHintDialog(e.getMessage());
        }
    }

    private void initTimePicker(final TextView tvTime) {
        Calendar startDate = Calendar.getInstance();
        startDate.set(2019, 0, 1);
        //时间选择器
        TimePickerView pvTime = new TimePickerBuilder(this, (date, v) -> {
            // 这里回调过来的v,就是show()方法里面所添加的 View 参数，如果show的时候没有添加参数，v则为null
            tvTime.setText(DateUtils.dateFormat(date, "yyyy-MM-dd"));
        })
                //年月日时分秒 的显示与否，不设置则默认全部显示
                .setType(new boolean[]{true, true, true, false, false, false})
                .setSubmitColor(Color.parseColor("#F04B63"))
                .setCancelColor(Color.parseColor("#666666"))
                .setDividerColor(Color.DKGRAY)
                .setContentTextSize(21)
                .setDate(Calendar.getInstance())
                .isCenterLabel(true)
                .setRangDate(startDate, Calendar.getInstance())
                .setLabel("年", "月", "日", "", "", "")
                .isCyclic(false)
                .setDecorView(null)
                .isDialog(true)
                .build();
        pvTime.show();
    }

    @Override
    public void upLogResult(String msg) {
        mTvTip.setText(msg);
    }
}
