package com.hisense.hiaposcmp.activity.common.authorization.ldk_rus;

import android.os.Looper;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.model.AuthorizationInfo;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.model.FeatureInfo;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.safenet.patch.Product;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import static com.hisense.hiaposcmp.activity.common.authorization.Aladdin.HaspStatus.HASP_INV_FORMAT;
import static com.hisense.hiaposcmp.activity.common.authorization.Aladdin.HaspStatus.HASP_INV_SCOPE;
import static com.hisense.hiaposcmp.activity.common.authorization.Aladdin.HaspStatus.HASP_STATUS_OK;


public class ProductKey {

    private ActivateThread activateThread = null;
    /**
     * set the URL here or input the URL manual
     */
    public String url = "http://10.16.8.201:8081";
    public String pk = "19d204e8-4c25-4d5a-a018-b77e0c2a9105";
    /**
     * the jsessionid for initiating WS calls to the EMS server
     */
    protected String jSessionId = null;
    /**
     * flag indicating if registration is needed
     */
    protected int registrationStat = 0;
    protected String redirectToUserReg = "";

    public static final String scope = new String(
            "<haspscope>\n" +
                    " <license_manager hostname=\"localhost\" />\n" +
                    "</haspscope>\n");

    public static final String host_fingerprint = new String("<haspformat format=\"host_fingerprint\"/>");

    public static final String updateinfo = new String("<haspformat format=\"updateinfo\"/>");
    public static final String haspscope = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
            "<haspscope>" +
            "    <license_manager ip=\"localhost\">" +
            "        <product id=\"61\" />" +
            "    </license_manager>" +
            "</haspscope>";

    public static final String haspinfo = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
            "<haspformat root=\"hasp_info\">" +
            "    <hasp>" +
            "        <attribute name=\"id\" />" +
            "        <attribute name=\"type\" />" +
            "            <product>" +
            "                <attribute name=\"id\" />" +
            "                    <feature>" +
            "                        <attribute name=\"id\" />" +
            "                        <element name=\"license\" />" +
            "                    </feature>" +
            "            </product>" +
            "    </hasp>" +
            "</haspformat>";


    public void activate(boolean isNew, UniCallback uniCallback) {
        //   Looper.prepare();
        activateThread = new ActivateThread(isNew, uniCallback);
        //    Looper.loop();
        activateThread.start();
    }


    class ActivateThread extends Thread {
        boolean isFirst;
        UniCallback mUniCallback;

        ActivateThread(boolean isNew, UniCallback uniCallback) {
            isFirst = isNew;
            mUniCallback = uniCallback;
        }


        @Override
        public void run() {
            boolean r;

            try {
                url = GeneralSettings.INSTANCE.getPayAuthUrl();
                pk = GeneralSettings.INSTANCE.getPayAuthKey();

                r = customerLogin();

                if (r) {
                    r = doActivation(isFirst);

                    if (r) {
                        mUniCallback.onSuccess("激活成功");
                        LoggerUtils.getLogger().debug("Activation successfully.");
                    } else {
                        mUniCallback.onError("激活失败");
                        LoggerUtils.getLogger().debug("Activation failed.");
                    }

                } else {
                    mUniCallback.onError("激活失败，未配置授权地址");
                    LoggerUtils.getLogger().debug("Activation failed.未配置授权地址");
                }


            } catch (Exception e) {
                mUniCallback.onError("激活失败" + e.toString());
                LoggerUtils.getLogger().debug("Error activating: " + e.getMessage());
            }


        }


    }


    /**
     * Logs in with product key and sets the jsessionid and the registration required flag
     *
     * @return true if login succeeded
     * @throws Exception
     */
    protected boolean customerLogin() {
        //logging in
        String[] loginRes = null;

        try {

            if (StringUtils.isEmpty(url) || StringUtils.isEmpty(pk)) {
                LoggerUtils.getLogger().debug("加密授权地址配置错误url" + url);
                return false;
            }
            loginRes = Activation.customerLogin(url, pk);

            //validate login response
            if (loginRes[0].isEmpty()) {
                LoggerUtils.getLogger().debug("Error Login failed. No login data returned from server.");
                return false;
            }

            //validate and set jsessionid and registration flag
            jSessionId = loginRes[0];
            if (jSessionId == null) {
                LoggerUtils.getLogger().debug("Error Login failed. Session not created.");
                return false;
            }

            if (loginRes[1] != null) {
                registrationStat = Integer.parseInt(loginRes[1]);
            }

            if (loginRes[2] != null) {
                redirectToUserReg = loginRes[2];
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    /**
     * This method does the following operation which are needed for activating:<br/>
     * 1. Get fingerprint or Reads the C2V of the key.<br/>
     * 2. Sends the fingerprint or C2V to the server and receives the license.</br>
     * 3. Install the license or updates the key with the license.
     */
    protected boolean doActivation(boolean isNew) {
        boolean r = false;

        pk = pk.trim();

        String c2v = collectInformation(isNew);

        boolean ispassedServer = false;

        if (c2v != null) {
            try {

                String v2c;
                if (isNew) {
                    v2c = Activation.getLicense(url, jSessionId, c2v, pk);
                } else {
                    v2c = Activation.updateLicense(url, jSessionId, c2v); //get v2cp
                }

                ispassedServer = true;

                if (v2c.contains("<hasp_info>")) {
                    r = Activation.updateKeyWithLicense(v2c);

                    LoggerUtils.getLogger().debug("Key updated successfully.");
                }
            } catch (Exception e) {
                if (ispassedServer) {
                    LoggerUtils.getLogger().debug("Runtime Error: " + e.getMessage());
                } else {
                    LoggerUtils.getLogger().debug("EMS Error: " + e.getMessage());
                }

                r = false;
            }
        } else {
            r = false;
        }

        return r;
    }

    /**
     * Get fingerprint for install SL or get C2V for update protection key
     *
     * @return fingerprint or C2V if successful
     */
    public String collectInformation(boolean isNew) {
        String info;
        String text;
        String format;
        int status;

        text = "Collect information: ";

        if (isNew) {
            format = host_fingerprint;
        } else {
            format = updateinfo;
        }


        Product product = new Product();
        info = product.GetInfo(scope, format);
        status = product.getLastError();
        switch (status) {
            case HASP_STATUS_OK:
                text += "OK\n";
                break;
            case HASP_INV_FORMAT:
                text += "Invalid format";
                break;
            case HASP_INV_SCOPE:
                text += "Invalid scope";
                break;
            default:
                text += "Failed with status:" + status;
                break;
        }

        if (status == HASP_STATUS_OK) {
            return info;
        } else {
            LoggerUtils.getLogger().debug(text);
            return null;
        }
    }

    public String authInfo() {
        Product product = new Product();
        String info = product.GetInfo(haspscope, haspinfo);
        return info;
    }

    public AuthorizationInfo getPayAuthInfo(UniCallback callback) {

        try {
            SAXReader reader = new SAXReader();
            Product product = new Product();
            String info = product.GetInfo(haspscope, haspinfo);
            LoggerUtils.getLogger().debug("info:" + info);
            Document document = null;
            document = reader.read(new ByteArrayInputStream(info.getBytes("UTF-8")));
            Element rootElement = document.getRootElement();

            Element productElement = rootElement.element("hasp").element("product");


            AuthorizationInfo authorizationInfo = new AuthorizationInfo();
            authorizationInfo.setHaspId(rootElement.element("hasp").attributeValue("id"));
            authorizationInfo.setProductId(productElement.attributeValue("id"));


            Iterator features = productElement.elementIterator();

            List<FeatureInfo> featureInfoList = new ArrayList<>();

            while (features.hasNext()) {

                Element feature = (Element) features.next();
                Element licenseElement = feature.element("license");

                FeatureInfo featureInfo = new FeatureInfo();
                featureInfo.setFeatureId(feature.attributeValue("id"));
                featureInfo.setLicenseType(licenseElement.elementText("license_type"));
                featureInfo.setExpDate(licenseElement.elementText("exp_date"));
                featureInfoList.add(featureInfo);

            }

            authorizationInfo.setFeature(featureInfoList);
            GeneralSettings.INSTANCE.saveAuthorization(authorizationInfo.toString());
            callback.onSuccess("");
            return authorizationInfo;
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("解析授权返回的xml数据异常" + e.toString());
            callback.onSuccess("解析授权返回的xml数据异常" + e.toString());
        }

        return null;


    }


}
