/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RelativeLayout;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.ConfigPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IConfigView;
import com.hisense.hiaposcmp.view.KeyBoardView;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 登录设置。
 *
 * @author guolina 2020-01-07
 */
@ActivityLayoutInject(R.layout.activity_login_config_std)
public class ConfigStdActivity extends BaseMvpActivity<IConfigView, ConfigPresenter> implements IConfigView, View.OnFocusChangeListener {

    @BindView(R.id.imgClose)
    ImageView imgClose;

    @BindView(R.id.edtOrgCode)
    EditText edtOrgCode;

    @BindView(R.id.imgOrgCodeClear)
    ImageView imgOrgCodeClear;

    @BindView(R.id.edtShopCode)
    EditText edtShopCode;

    @BindView(R.id.imgShopCodeClear)
    ImageView imgShopCodeClear;

    @BindView(R.id.edtPosNo)
    EditText edtPosNo;

    @BindView(R.id.imgPosNoClear)
    ImageView imgPosNoClear;

    @BindView(R.id.edtServiceUrl)
    EditText edtServiceUrl;

    @BindView(R.id.imgServiceUrlClear)
    ImageView imgServiceUrlClear;

    @BindView(R.id.btnReset)
    Button btnReset;

    @BindView(R.id.btnOK)
    Button btnOK;

    @BindView(R.id.rlOrgeCode)
    RelativeLayout rlOrgeCode;

    @BindView(R.id.rlShopCode)
    RelativeLayout rlShopCode;

    @BindView(R.id.rlPosNo)
    RelativeLayout rlPosNo;

    @BindView(R.id.rlServiceUrl)
    RelativeLayout rlServiceUrl;

    private boolean isFromLogin;
    KeyBoardView keyBoardView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initValue();
    }

    private void init() {
        SoftInput.disable(edtShopCode);
        SoftInput.disable(edtOrgCode);
        SoftInput.disable(edtPosNo);
        SoftInput.disable(edtServiceUrl);
        isNotShowExitDialog = true;

        edtShopCode.setOnFocusChangeListener(this);
        edtOrgCode.setOnFocusChangeListener(this);
        edtPosNo.setOnFocusChangeListener(this);
        edtServiceUrl.setOnFocusChangeListener(this);

    }

    @Override
    protected ConfigPresenter createPresenter() {
        return new ConfigPresenter();
    }


    /**
     * 加载设置信息。
     */
    private void initValue() {
        boolean isNotFirstConfig = CommonTools.loginConfig(this).getBoolean(Config.IS_SETTING, false);
        if (isNotFirstConfig) {
            // 如果非首次设置，则显示返回按钮。首次设置时，返回按钮不显示。
            imgClose.setVisibility(View.VISIBLE);
            btnReset.setVisibility(View.VISIBLE);
        }

        edtOrgCode.setText(CommonTools.loginConfig(this).getString(Config.ORG_CODE, ""));
        edtShopCode.setText(CommonTools.loginConfig(this).getString(Config.SHOP_CODE, ""));
        edtPosNo.setText(CommonTools.loginConfig(this).getString(Config.POS_NO, ""));
        edtServiceUrl.setText(CommonTools.loginConfig(this).getString(Config.SERVER_URL, ""));

        isFromLogin = false;
        Intent intent = this.getIntent();
        Bundle bundle = intent.getExtras();
        if (bundle != null && bundle.containsKey("isFromLogin")) {
            rlOrgeCode.setVisibility(View.GONE);
            rlShopCode.setVisibility(View.GONE);
            rlPosNo.setVisibility(View.GONE);
            btnOK.setText("下一步");

            keyBoardView = new KeyBoardView(edtServiceUrl, false);
            keyBoardView.showKeyboard();

            isFromLogin = bundle.getBoolean("isFromLogin");
        }

        if (bundle != null && bundle.containsKey("pos")) {
            int pos = getIntent().getIntExtra("pos", 0);

            rlServiceUrl.setVisibility(View.GONE);
            setFocus(pos);

        }
        if (rlOrgeCode.getVisibility() == View.VISIBLE) {
            keyBoardView = new KeyBoardView(edtOrgCode);
            keyBoardView.showKeyboard();
        }
    }

    private void setFocus(int pos) {
        switch (pos) {
            default:
            case 1:
                edtOrgCode.requestFocus();
                edtOrgCode.setSelection(edtOrgCode.getText().length());

                keyBoardView = new KeyBoardView(edtOrgCode);
                keyBoardView.showKeyboard();
                break;
            case 2:
                edtShopCode.requestFocus();
                edtShopCode.setSelection(edtShopCode.getText().length());
                keyBoardView = new KeyBoardView(edtShopCode);
                keyBoardView.showKeyboard();
                break;
            case 3:
                edtPosNo.requestFocus();
                edtPosNo.setSelection(edtPosNo.getText().length());
                keyBoardView = new KeyBoardView(edtPosNo);
                keyBoardView.showKeyboard();
                break;
        }
    }

    @Override
    public void onFocusChange(View v, boolean b) {
        if (keyBoardView != null) {
            keyBoardView.change((EditText) v);
        }
        switch (v.getId()) {
            case R.id.edtOrgCode:
                imgOrgCodeClear.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            case R.id.edtShopCode:
                imgShopCodeClear.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            case R.id.edtPosNo:
                imgPosNoClear.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            case R.id.edtServiceUrl:
                imgServiceUrlClear.setVisibility(b ? View.VISIBLE : View.GONE);
                break;
            default:
                break;
        }
    }

    @OnClick(R.id.imgClose)
    public void onClickBack() {
        onBackPressed();
        finish();
    }

    @OnClick(R.id.btnOK)
    public void onClickSave() {
        checkAndSave();
    }

    @OnClick(R.id.btnReset)
    public void onClickReset() {
        resetValue();
    }

    /**
     * 重新加载设置信息。
     */
    private void resetValue() {
        edtOrgCode.setText(CommonTools.loginConfig(this).getString(Config.ORG_CODE, ""));
        edtShopCode.setText(CommonTools.loginConfig(this).getString(Config.SHOP_CODE, ""));
        edtPosNo.setText(CommonTools.loginConfig(this).getString(Config.POS_NO, ""));
        edtServiceUrl.setText(CommonTools.loginConfig(this).getString(Config.SERVER_URL, ""));
    }

    @OnClick(R.id.imgOrgCodeClear)
    public void onClickClearOrgCode() {
        if (imgOrgCodeClear.getVisibility() == View.VISIBLE) {
            edtOrgCode.setText("");
        }
    }

    @OnClick(R.id.imgShopCodeClear)
    public void onClickClearShopCode() {
        if (imgShopCodeClear.getVisibility() == View.VISIBLE) {
            edtShopCode.setText("");
        }
    }

    @OnClick(R.id.imgPosNoClear)
    public void onClickClearPosNo() {
        if (imgPosNoClear.getVisibility() == View.VISIBLE) {
            edtPosNo.setText("");
        }
    }

    @OnClick(R.id.imgServiceUrlClear)
    public void onClickClearServiceUrl() {
        if (imgServiceUrlClear.getVisibility() == View.VISIBLE) {
            edtServiceUrl.setText("");
        }
    }

    @Override
    public void serviceOnlineSuccess() {
        if (isFromLogin) {
            isFromLogin = false;

            rlOrgeCode.setVisibility(View.VISIBLE);
            rlShopCode.setVisibility(View.VISIBLE);
            rlPosNo.setVisibility(View.VISIBLE);
            edtOrgCode.requestFocus();
            edtOrgCode.setSelection(edtOrgCode.getText().toString().length());
            btnOK.setText("确定");
        } else {
            finish();
        }
    }

    /**
     * 检查用户输入并保存设置。
     */
    private void checkAndSave() {
        if (isFromLogin) {
            mPresenter.checkAndSave(edtServiceUrl.getText().toString().trim());
        } else {
            mPresenter.checkAndSave(edtOrgCode.getText().toString().trim(), edtShopCode.getText().toString().trim(),
                    edtPosNo.getText().toString().trim(), edtServiceUrl.getText().toString().trim());
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
