/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std;

import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.std.query.QueryStdActivity;
import com.hisense.hiaposcmp.activity.std.sale.SaleStdActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.cache.Cache;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.MainPresenter;
import com.hisense.hiaposcmp.service.payment.PayMethodService;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.HomeMenuButton;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.IMainView;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.Privileges;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.webapi.Callback;

import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 主界面
 *
 * @author guolina  2019-12-18  新建标准版主界面
 */
@ActivityLayoutInject(R.layout.activity_main_std)
public class MainStdActivity extends BaseMvpActivity<IMainView, MainPresenter> implements IMainView {

    @BindView(R.id.linear_more)
    LinearLayout linearMore;

    @BindView(R.id.linear_back)
    LinearLayout linearBack;

    @BindView(R.id.btnSale)
    HomeMenuButton btnSale;

    @BindView(R.id.btnPreSale)
    HomeMenuButton btnPreSale;

    @BindView(R.id.btnQuery)
    HomeMenuButton btnQuery;

    @BindView(R.id.btnReturn)
    HomeMenuButton btnReturn;

    @BindView(R.id.btnAbout)
    HomeMenuButton btnAbout;

    @BindView(R.id.btnChangePsw)
    HomeMenuButton btnChangePsw;

    @BindView(R.id.btnReport)
    HomeMenuButton btnReport;

    @BindView(R.id.btnAccounting)
    HomeMenuButton btnAccounting;

    @BindView(R.id.btnSetting)
    HomeMenuButton btnSetting;

    @BindView(R.id.txtMsgContent)
    TextView txtMsgContent;

    @BindView(R.id.imgPreSales)
    ImageView imgPreSales;

    @BindView(R.id.rlPreSales)
    RelativeLayout rlPreSales;

    @BindView(R.id.imgQuery)
    ImageView imgQuery;

    @BindView(R.id.rlQuery)
    RelativeLayout rlQuery;

    @BindView(R.id.imgReturn)
    ImageView imgReturn;

    @BindView(R.id.rlReturn)
    RelativeLayout rlReturn;

    @BindView(R.id.imgAbout)
    ImageView imgAbout;

    @BindView(R.id.rlAbout)
    RelativeLayout rlAbout;

    @BindView(R.id.imgChangePsw)
    ImageView imgChangePsw;

    @BindView(R.id.rlChangePsw)
    RelativeLayout rlChangePsw;

    @BindView(R.id.imgReport)
    ImageView imgReport;

    @BindView(R.id.rlReport)
    RelativeLayout rlReport;

    @BindView(R.id.imgAccounting)
    ImageView imgAccounting;

    @BindView(R.id.rlAccounting)
    RelativeLayout rlAccounting;

    @BindView(R.id.imgSetting)
    ImageView imgSetting;

    @BindView(R.id.rlSetting)
    RelativeLayout rlSetting;

    private int mCurrentCardVersion = -1;

    private IPaymentServiceProvider paymentServiceProvider;
    private SaleApplicationService saleApplicationService;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        init();
    }

    @Override
    protected MainPresenter createPresenter() {
        return new MainPresenter();
    }

    private void init() {
        isNotShowExitDialog = true;

        showLoadingDialog();
        PayMethodService.getAll(this, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                showHintDialog(throwable.getMessage());
                hideLoadingDialog();
            }
        });

        saleApplicationService = new SaleApplicationService(this);

        if (!saleApplicationService.deleteUploadedLocalData()) {
            LoggerUtils.getLogger().info("本地无三个月前已上传流水");
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
            CommonTools.exit(this, LoginStdActivity.class);

            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (paymentServiceProvider != null && paymentServiceProvider.requestCode() == requestCode) {
            try {
                paymentServiceProvider.handleResult(data);
            } catch (Exception ex) {
                showHintDialog(ex.getMessage());
                LoggerUtils.getLogger().error(ex.getMessage(), ex);
            }
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    protected void onResume() {
        super.onResume();
        mPresenter.startSaleUploadAuto();

        showLoadingDialog();
        CommonUtils.reInitSaleModel(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                hideLoadingDialog();
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                showHintDialog(err);
            }
        });

        //获取射频卡信息
        if (mCurrentCardVersion != GeneralSettings.INSTANCE.getIcCardVersion().value()) {
            mPresenter.getIcCardPassword();
        }

        paymentServiceProvider = getPaymentServiceProvider(this);
        if (Context.cashier() != null) {
            mPresenter.queryPrivileges(Context.cashier().id());
        }
    }

    @Override
    public void serviceOnlineSuccess() {
        CommonTools.goTo(this, SaleStdActivity.class, "tranType", String.valueOf(TranType.Sale.value()));
    }

    @Override
    public void getIcCardPasswordSuccess(String password) {
        mCurrentCardVersion = GeneralSettings.INSTANCE.getIcCardVersion().value();
        Cache.icCardPassword(password);
    }

    @Override
    public void queryPrivilegesSuccess(User user) {
        super.setUser(user);
    }

    @Override
    public void queryPrivilegesFail() {
        super.setUser(null);
    }

    @Override
    public void getAccountingDateSuccess(String accDate) {
        mPresenter.confirmAccounting(accDate);
    }

    @Override
    public void accountingSuccess(String s) {
        LoggerUtils.getLogger().debug("允许结算:" + GeneralSettings.INSTANCE.isSettleAfterAccount());
        if (GeneralSettings.INSTANCE.isSettleAfterAccount()) {
            try {
                paymentServiceProvider.settle(null);
            } catch (Exception ex) {
                showHintDialog(ex.getMessage());
                LoggerUtils.getLogger().error(ex.getMessage(), ex);
            }
        }
    }

    @OnClick({R.id.linear_more, R.id.linear_back, R.id.btnSale, R.id.btnPreSale, R.id.btnQuery, R.id.btnReturn, R.id.btnAbout, R.id.btnChangePsw, R.id.btnReport, R.id.btnAccounting, R.id.btnSetting})
    public void onViewClicked(View view) {
        switch (view.getId()) {

            case R.id.linear_more:
                showHintDialog("暂无更多内容");
                break;
            case R.id.linear_back:
                if (!mPresenter.uploadLocalData()) {
                    CommonTools.goTo(this, LoginStdActivity.class);
                    finish();
                }
                break;
            case R.id.btnSale:
                if (hasPrivilege(Privileges.SALE)) {
                    mPresenter.serviceOnline();
                } else {
                    showHintDialog("请在后台开通销售权限");
                }

                break;
            case R.id.btnPreSale:
                if (hasPrivilege(Privileges.SALE)) {
                    CommonTools.goTo(this, SaleStdActivity.class, "tranType", String.valueOf(TranType.PreSale.value()));
                } else {
                    showHintDialog("请在后台开通销售权限");
                }

                break;
            case R.id.btnQuery:
                CommonTools.goTo(this, QueryStdActivity.class);
                break;
            case R.id.btnReturn:

                //    CommonTools.goTo(this, ReturnStdActivity.class);

                AuthStdActivity.authToReturn(this);

                break;
            case R.id.btnAbout:
                CommonTools.goTo(this, AboutStdActivity.class);
                break;
            case R.id.btnChangePsw:
                CommonTools.goTo(this, ChangePasswordStdActivity.class);
                break;
            case R.id.btnReport:
                CommonTools.goTo(this, ReportStdActivity.class);
                break;
            case R.id.btnAccounting:
                if (!saleApplicationService.uploadLocalData()) {
                    mPresenter.account();
                }
                break;
            case R.id.btnSetting:
                AuthStdActivity.authToSystemSettings(this);
                break;
        }
    }
}
