/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.std.fragment.ReportAllFragment;
import com.hisense.hiaposcmp.activity.std.fragment.ReportCashierFragment;
import com.hisense.hiaposcmp.activity.std.fragment.ReportDepFragment;
import com.hisense.hiaposcmp.activity.std.fragment.ReportGoodsFragment;
import com.hisense.hiaposcmp.activity.std.fragment.ReportPresaleFragment;
import com.hisense.hiaposcmp.activity.std.fragment.ReportShopFragment;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.presenter.ReportPresenter;
import com.hisense.hiaposcmp.view.AutoFlowLayout;
import com.hisense.hiaposcmp.view.FlowAdapter;
import com.hisense.hiaposcmp.view.IView.IReportView;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.Privileges;
import com.hisense.hipos.webapi.report.DepReportResultData;
import com.hisense.hipos.webapi.report.PluReportResultData;

import java.util.Arrays;
import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 报表界面
 *
 * @author guolina  2019-12-26  新建标准版报表
 */

@ActivityLayoutInject(R.layout.activity_report_std)
public class ReportStdActivity extends BaseMvpActivity<IReportView, ReportPresenter> implements IReportView {

    /**
     * 会员的图标
     */
    @BindView(R.id.tvStdTitleVipIcon)
    TextView tvStdTitleVipIcon;

    /**
     * 正中的标题
     */
    @BindView(R.id.tvStdTitleItem)
    TextView mTvStdTitleItem;

    /**
     * 标题栏  时间
     */
    @BindView(R.id.tvStdTitleDate)
    TextView tvStdTitleDate;
    /**
     * 标题栏 门店名称
     */
    @BindView(R.id.tvStdTitleOrgName)
    TextView tvStdTitleOrgName;
    /**
     * 标题栏 营业员名称
     */
    @BindView(R.id.tvStdTitleCashierName)
    TextView tvStdTitleCashierName;
    /**
     * 会员登录(隐藏)
     */
    @BindView(R.id.tvStdTitleVipLogin)
    public TextView tvStdTitleVipLogin;

    /**
     * 切换会员(隐藏)
     */
    @BindView(R.id.tvStdTitleVipChange)
    TextView tvStdTitleVipChange;

    @BindView(R.id.tvTipSaleNo)
    TextView mTvTipSaleNo;

    @BindView(R.id.btnBack)
    Button btnBack;

    @BindView(R.id.afl_report)
    AutoFlowLayout aflReport;

    List<String> reportTexts;
    private LayoutInflater mLayoutInflater;
    FragmentManager fm;
    private ReportAllFragment reportAllFragment;
    private ReportCashierFragment reportCashierFragment;
    private ReportShopFragment reportShopFragment;
    private ReportPresaleFragment reportPresaleFragment;
    private ReportDepFragment reportDepFragment;
    private ReportGoodsFragment reportgoodsFragment;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        mTvStdTitleItem.setText("报表");
        tvStdTitleVipLogin.setVisibility(View.GONE);
        tvStdTitleVipChange.setVisibility(View.GONE);
        tvStdTitleDate.setText(DateUtils.dateFormatWithWeek("MM月dd日    ww   aa hh:mm"));
        tvStdTitleOrgName.setText(com.hisense.hipos.webapi.Context.orgName());
        tvStdTitleCashierName.setText(Config.user_name);
        tvStdTitleVipIcon.setVisibility(View.GONE);
        mTvTipSaleNo.setVisibility(View.GONE);

        fm = getSupportFragmentManager();
        mLayoutInflater = LayoutInflater.from(this);
        aflReport.setDefSelected(0);
        reportTexts = Arrays.asList(new String[]{"总交易报表", "收银员报表", "专柜报表", "预销售报表", "部门报表", "商品销售汇总报表"});
        aflReport.setAdapter(new FlowAdapter(reportTexts) {
            @Override
            public View getView(int position) {
                String reportText = reportTexts.get(position);

                View item = mLayoutInflater.inflate(R.layout.item_std_report, null);
                TextView tvReport = item.findViewById(R.id.tv_report);
                tvReport.setText(reportText);

                return item;
            }
        });
        aflReport.setOnItemClickListener(new AutoFlowLayout.OnItemClickListener() {
            @Override
            public void onItemClick(int position, View view) {
                aflReport.select();
                String reportText = reportTexts.get(position);

                LoggerUtils.getLogger().debug("reportText:" + reportText);
                gotoReportFragment(position);
            }
        });
        if (hasPrivilege(Privileges.REPORT_SALE)) {
            gotoReportFragment(0);
        } else {
            showHintDialog("没有总交易报表查询权限");
        }


    }


    private void gotoReportFragment(int index){
        FragmentTransaction ft = fm.beginTransaction();
        hideFragment(ft);
        switch (index) {
            case 0:
                if (hasPrivilege(Privileges.REPORT_SALE)) {
                    if(reportAllFragment ==null){
                        reportAllFragment = new ReportAllFragment();
                        ft.add(R.id.flReport, reportAllFragment);
                    }else{
                        ft.show(reportAllFragment);
                    }
                } else {
                    showHintDialog("没有总交易报表查询权限");
                    return;
                }
                break;

            case 1:
                if (hasPrivilege(Privileges.REPORT_CASHIER)) {
                    if(reportCashierFragment ==null){
                        reportCashierFragment = new ReportCashierFragment();
                        ft.add(R.id.flReport, reportCashierFragment);
                    }else{
                        ft.show(reportCashierFragment);
                    }
                } else {
                    showHintDialog("没有收银员报表查询权限");
                    return;
                }


                break;

            case 2:
                if (hasPrivilege(Privileges.REPORT_DEPARTMENT)) {
                    if(reportShopFragment ==null){
                        reportShopFragment = new ReportShopFragment();
                        ft.add(R.id.flReport, reportShopFragment);
                    }else{
                        ft.show(reportShopFragment);
                    }
                } else {
                    showHintDialog("没有专柜报表查询权限");
                    return;
                }

                break;

            case 3:
                if (hasPrivilege(Privileges.REPORT_PRE_SALE)) {
                    if(reportPresaleFragment ==null){
                        reportPresaleFragment = new ReportPresaleFragment();
                        ft.add(R.id.flReport, reportPresaleFragment);
                    }else{
                        ft.show(reportPresaleFragment);
                    }

                } else {
                    showHintDialog("没有预销售报表查询权限");
                    return;
                }

                break;

            case 4:
                if(reportDepFragment ==null){
                    reportDepFragment = new ReportDepFragment();
                    ft.add(R.id.flReport, reportDepFragment);
                }else{
                    ft.show(reportDepFragment);
                }

                break;

            case 5:
                if(reportgoodsFragment ==null){
                    reportgoodsFragment = new ReportGoodsFragment();
                    ft.add(R.id.flReport, reportgoodsFragment);
                }else{
                    ft.show(reportgoodsFragment);
                }

                break;

        }
        ft.commit();
    }
    //用于隐藏fragment
    private void hideFragment(FragmentTransaction ft){
        if(reportAllFragment!=null){
            ft.hide(reportAllFragment);
        }
        if(reportCashierFragment!=null){
            ft.hide(reportCashierFragment);
        }
        if(reportShopFragment!=null){
            ft.hide(reportShopFragment);
        }
        if(reportPresaleFragment!=null){
            ft.hide(reportPresaleFragment);
        }
        if(reportDepFragment!=null){
            ft.hide(reportDepFragment);
        }
        if(reportgoodsFragment!=null){
            ft.hide(reportgoodsFragment);
        }
    }


    ReportPresenter presenter;
    @Override
    protected ReportPresenter createPresenter() {
        presenter= new ReportPresenter();
        return presenter;
    }


    @Override
    public void showCashierData(boolean isShowData) {

    }

    @Override
    public void showSaleReport(SaleReport report) {

    }

    @Override
    public void showDepReport(DepReportResultData report) {

    }

    @Override
    public void showPluReport(PluReportResultData report) {

    }

    @Override
    public void showShopSaleReport(ShopSaleReport report) {

    }

    @Override
    public void accountingDateSaleSuccess(String accDate) {

    }


    @OnClick({R.id.btnBack, R.id.linearStdTitleHome, R.id.afl_report})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.btnBack:
            case R.id.linearStdTitleHome:
                finish();
                break;
            case R.id.afl_report:
                break;
        }
    }
    @Override
    public TranType tranType() {
        return TranType.Sale;
    }
}
