package com.hisense.hiaposcmp.activity.std.fragment;

import android.app.DatePickerDialog;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpFragment;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.report.ReportConstants;
import com.hisense.hiaposcmp.dialog.TimePicker;
import com.hisense.hiaposcmp.presenter.ReportPresenter;
import com.hisense.hiaposcmp.view.IView.IReportView;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.report.DepReportResultData;
import com.hisense.hipos.webapi.report.PluReportResultData;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import butterknife.BindView;

/**
 * 收银员报表
 *
 * @author guolina  2019-12-26  新建标准版收银员报表
 */
@ActivityLayoutInject(R.layout.fragment_report_cashier)
public class ReportCashierFragment extends BaseMvpFragment<IReportView, ReportPresenter> implements IReportView {


    @BindView(R.id.lvPayments)
    ListView lvPayments;
    @BindView(R.id.txtDatePicker)
    TextView txtDatePicker;
    @BindView(R.id.ivDatePicker)
    ImageView ivDatePicker;
    @BindView(R.id.btnPrint)
    Button btnPrint;
    @BindView(R.id.txtCashierName)
    TextView txtCashierName;
    @BindView(R.id.txtDep)
    TextView txtDep;
    @BindView(R.id.txtDate)
    TextView txtDate;
    @BindView(R.id.txtTotal)
    TextView txtTotal;
    @BindView(R.id.txtRealTotal)
    TextView txtRealTotal;
    @BindView(R.id.txtTotalDsc)
    TextView txtTotalDsc;
    @BindView(R.id.txtTotalCount)
    TextView txtTotalCount;
    @BindView(R.id.txtEmpty)
    TextView txtEmpty;
    @BindView(R.id.txtTotalReturned)
    TextView txtTotalReturned;
    @BindView(R.id.txtTotalTransReturned)
    TextView txtTotalTransReturned;
    private String accDate = null;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        return view;
    }

    @Override
    protected void initData() {
        txtDatePicker.setText(TimePicker.getTime(new Date()));
        updateCashier();
        btnPrint.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.printSalePort(ReportConstants.REPORT_CASHIER);
            }
        });

        ivDatePicker.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int year = Integer.parseInt(accDate.substring(0, 4));
                int month = Integer.parseInt(accDate.substring(5, 7));
                int day = Integer.parseInt(accDate.substring(8, 10));

                new DatePickerDialog(getContext(),
                        new DatePickerDialog.OnDateSetListener() {
                            @Override
                            public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {

                                accDate = year + "-" + mPresenter.appendZero(String.valueOf(month + 1)) + "-" + mPresenter.appendZero(String.valueOf(dayOfMonth));
                                txtDatePicker.setText(accDate);
                                mPresenter.getCashierReport(accDate);
                            }
                        }
                        , year, month - 1, day).show();
            }
        });
        mPresenter.accountingDateSale(ReportConstants.REPORT_CASHIER, null);
    }
    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        if (!hidden) {
            txtTotal.setText("");
            txtRealTotal.setText("");
            txtTotalDsc.setText("");
            txtTotalCount.setText("");
            txtTotalReturned.setText("");
            txtTotalTransReturned.setText("");
            mPresenter.accountingDateSale(ReportConstants.REPORT_CASHIER, null);
        }
    }
    @Override
    protected ReportPresenter createPresenter() {
        return new ReportPresenter();
    }

    @Override
    public void showCashierData(boolean isShowData) {
        if (isShowData) {
            lvPayments.setVisibility(View.VISIBLE);
            txtEmpty.setVisibility(View.GONE);
        } else {
            lvPayments.setVisibility(View.GONE);
            txtEmpty.setVisibility(View.VISIBLE);
            mPresenter.setCashierReport(null);
        }
    }

    @Override
    public void showSaleReport(SaleReport report) {
        if (null == report) {
            lvPayments.setAdapter(mPresenter.saleReportadapter());
            txtTotal.setText("");
            txtRealTotal.setText("");
            txtTotalDsc.setText("");

            txtTotalCount.setText("");
            txtTotalReturned.setText("");
            txtTotalTransReturned.setText("");
        } else {
            lvPayments.setAdapter(mPresenter.saleReportadapter());
            txtTotal.setText(String.format(Locale.CHINESE, "%.2f", report.total()));
            txtRealTotal.setText(String.format(Locale.CHINESE, "%.2f", report.totalReceived()));
            txtTotalDsc.setText(String.format(Locale.CHINESE, "%.2f", report.totalDiscount()));

            txtTotalCount.setText(String.valueOf(report.totalTrans()));
            txtTotalReturned.setText(String.format(Locale.CHINESE, "%.2f", report.totalReturned()));
            txtTotalTransReturned.setText(String.valueOf(report.totalTransReturned()));
        }
    }

    @Override
    public void showDepReport(DepReportResultData report) {

    }

    @Override
    public void showPluReport(PluReportResultData report) {

    }

    @Override
    public void showShopSaleReport(ShopSaleReport report) {

    }

    @Override
    public void accountingDateSaleSuccess(String accDate) {
        if (!accDate.equals(this.accDate)) {
            txtDatePicker.setText(accDate);
            this.accDate = accDate;
        }
    }

    /**
     * 更新收银员信息
     */
    private void updateCashier() {
        showCashierData(false);
        txtCashierName.setText(String.format(Locale.CHINESE, "%s(%s)",
                Context.cashier().name(), Context.cashier().code()));
        if (Context.orgName().isEmpty()) {
            txtDep.setText(Context.shopCode());
        } else {
            txtDep.setText(Context.shopCode() + "-[" + Context.orgName() + "]");
        }
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date(System.currentTimeMillis());
        txtDate.setText(formatter.format(date));
    }

    @Override
    public TranType tranType() {
        return TranType.Sale;
    }
}
