package com.hisense.hiaposcmp.activity.std.fragment;

import android.app.DatePickerDialog;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpFragment;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.report.ReportConstants;
import com.hisense.hiaposcmp.dialog.TimePicker;
import com.hisense.hiaposcmp.presenter.ReportPresenter;
import com.hisense.hiaposcmp.view.IView.IReportView;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.report.DepReportResultData;
import com.hisense.hipos.webapi.report.PluReportResultData;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import butterknife.BindView;

/**
 * 商品销售汇总报表
 *
 * @author guolina  2019-12-26  新建标准版商品交易汇总报表
 */
@ActivityLayoutInject(R.layout.fragment_report_goods)
public class ReportGoodsFragment extends BaseMvpFragment<IReportView, ReportPresenter> implements IReportView {


    @BindView(R.id.lvPlus)
    ListView lvPlus;
    @BindView(R.id.txtEmpty)
    TextView txtEmpty;
    @BindView(R.id.txtTotal)
    TextView txtTotal;
    @BindView(R.id.txtDatePicker)
    TextView txtDatePicker;
    @BindView(R.id.ivDatePicker)
    ImageView ivDatePicker;
    @BindView(R.id.txtDepCode)
    TextView txtDepCode;
    @BindView(R.id.btnPrint)
    Button btnPrint;
    @BindView(R.id.rlTotal)
    RelativeLayout rlTotal;
    private String accDate = null;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        return view;
    }

    @Override
    protected void initData() {
        txtDatePicker.setText(TimePicker.getTime(new Date()));
        txtDepCode.setText(Context.shopCode());
        showCashierData(false);

        btnPrint.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mPresenter.printDepPort(ReportConstants.REPORT_PLU, accDate);
            }
        });

        ivDatePicker.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int year = Integer.parseInt(accDate.substring(0, 4));
                int month = Integer.parseInt(accDate.substring(5, 7));
                int day = Integer.parseInt(accDate.substring(8, 10));

                new DatePickerDialog(getContext(),
                        new DatePickerDialog.OnDateSetListener() {
                            @Override
                            public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {

                                accDate = year + "-" + mPresenter.appendZero(String.valueOf(month + 1)) + "-" + mPresenter.appendZero(String.valueOf(dayOfMonth));
                                txtDatePicker.setText(accDate);
                                mPresenter.getPluReport(accDate, GeneralSettings.INSTANCE.isNoShopLimits() ? "*" : Context.shopCode());
                            }
                        }
                        , year, month - 1, day).show();
            }
        });
        mPresenter.accountingDateSale(ReportConstants.REPORT_PLU, null);
    }
    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        if (!hidden) {
            mPresenter.setPluSaleReport(null);
            txtTotal.setText("");
            mPresenter.accountingDateSale(ReportConstants.REPORT_PLU, null);
        }
    }
    @Override
    protected ReportPresenter createPresenter() {
        return new ReportPresenter();
    }

    @Override
    public void showCashierData(boolean isShowData) {
        if (isShowData) {
            lvPlus.setVisibility(View.VISIBLE);
            rlTotal.setVisibility(View.VISIBLE);
            txtEmpty.setVisibility(View.GONE);
        } else {
            lvPlus.setVisibility(View.GONE);
            rlTotal.setVisibility(View.GONE);
            txtEmpty.setVisibility(View.VISIBLE);
            mPresenter.setPluSaleReport(null);
        }
    }

    @Override
    public void showSaleReport(SaleReport report) {
    }

    @Override
    public void showDepReport(DepReportResultData report) {
    }

    @Override
    public void showPluReport(PluReportResultData report) {
        if (report == null) {
            mPresenter.setPluSaleReport(null);
            lvPlus.setAdapter(mPresenter.pluAdapter());
            txtTotal.setText("");
        } else {
            mPresenter.setPluSaleReport(report);
            lvPlus.setAdapter(mPresenter.pluAdapter());
            txtTotal.setText(String.format(Locale.CHINA, "%.2f", Double.parseDouble(report.totalRecv)));
        }

    }

    @Override
    public void showShopSaleReport(ShopSaleReport report) {

    }

    @Override
    public void accountingDateSaleSuccess(String accDate) {
        if (!accDate.equals(this.accDate)) {
            txtDatePicker.setText(accDate);
            this.accDate = accDate;
        }
    }
    @Override
    public TranType tranType() {
        return TranType.Sale;
    }
}

