/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std.pay;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.Card;
import com.hisense.crm.card.VipCard;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.activity.ysy.card.CardReturnQueryActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hiaposcmp.tool.UpdateTextButton;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.hipos.payment.PayMethod;

import java.util.Locale;

import butterknife.BindView;

import static com.hisense.hiaposcmp.constant.ActivityConstants.RESULT_CARD_BALANCE_QUERY_CANCEL;
import static com.hisense.hiaposcmp.constant.ActivityConstants.RESULT_CARD_BALANCE_QUERY_COMMIY;

/**
 * 卡退款查询
 *
 * @author xieliyan    2017-05-19
 * @author xieliyan    2017-12-21 HIPOSAPP-254 控制支付金额的大小更改
 * @author zhangsai    2018-02-02 HIPOSAPP-315
 * @author guolina     2018-04-02 HIPOSAPP-365 数据传输未分类导致的储值卡退款闪退；参数未传输导致的无法获取会员服务和签名错误
 * @author zhangsai    2018-06-11 HIPOSAPP-509 增加退货时最大可退金额的限制
 * @author wangyaogang 2019-03-07 HIPOSAPP-1040 修复储值卡/积分卡支付时,金额正确但提示"请输入正确的金额"的问题
 * @author guolina     2019-04-02 HIPOSAPP-1072 修复部分退货时储值卡积分支付可退金额错误
 * @author wangyaogang 2020-01-15 HIAPOSCMP-496 标准版-储值卡/积分支付页面改造
 * @author xieliyan    2020-03-31 HIAPOSCMP-612 SCRM余额退款，输入密码后，退款金额输入0.可进行退款为0，应该校验退款金额不能为0
 */
@ActivityLayoutInject(R.layout.activity_card_pay_query_std)
public class CardReturnQueryStdActivity extends BaseActivity {

    private String fSsTotal = "0";
    private String maxRefund = "0";

    @BindView(R.id.usableTotal)
    public TextView mTxtUsableTotal;

    @BindView(R.id.CardTypeName)
    public TextView mTxtCardTypeName;

    @BindView(R.id.cardFaceNo)
    public TextView mTxtCardFaceNo;

    @BindView(R.id.expireDate)
    public TextView mTxtExpireDate;

    @BindView(R.id.payTotal)
    public EditText mTxtPayTotal;

    @BindView(R.id.commit_action)
    public Button mBtnCommit;

    @BindView(R.id.cancel_action)
    public Button mBtnCancel;

    @BindView(R.id.txtReturn)
    public TextView mTxtReturn;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        SoftInput.disable(mTxtPayTotal);
        KeyBoardView keyboard = new KeyBoardView(mTxtPayTotal);
        keyboard.showKeyboard();

        NumberHandler.setPoint(mTxtPayTotal, 2);

        mBtnCommit.setText("确定退款");
        mTxtReturn.setText("应退金额");

        Bundle bundle = this.getIntent().getExtras();
        String typeTag = bundle.getString("typeTag");

        if ("Card".equals(typeTag)) {
            Card card = (Card) bundle.getSerializable("card");
            mTxtUsableTotal.setText(String.format(Locale.CHINESE, "%.2f", card.usableTotal()));
            mTxtCardTypeName.setText(card.cardType().name());
            mTxtCardFaceNo.setText(card.cardFaceNo());
            mTxtExpireDate.setText(card.expireDate());
        } else {
            String remark = bundle.getString("remark");
            VipCard card = (VipCard) bundle.getSerializable("card");
            if (PayMethod.isScrmRemain(remark)) {
                mTxtUsableTotal.setText(String.format(Locale.CHINESE, "%.2f", card.usableTotal()));
            } else {
                mTxtUsableTotal.setText(String.format(Locale.CHINESE, "%.2f", card.usableBalance()));
            }

            mTxtCardTypeName.setText(card.cardType().name());
            mTxtCardFaceNo.setText(card.cardFaceNo());
            mTxtExpireDate.setText(card.expireDate());
        }

        maxRefund = bundle.getString("maxRefund");
        fSsTotal = bundle.getString("fSsTotal");
        mTxtPayTotal.setText(fSsTotal);

        UpdateTextButton.setOnTextChange(mTxtPayTotal, fSsTotal);

        mTxtPayTotal.setInputType(EditorInfo.TYPE_CLASS_PHONE);
        mTxtPayTotal.setSelection(mTxtPayTotal.getText().length());
        mBtnCancel.setOnClickListener((view) -> {
            setResult(RESULT_CARD_BALANCE_QUERY_CANCEL);
            finish();
        });
        mBtnCommit.setOnClickListener(new onClick());
    }

    /**
     * 确定按钮监听
     */
    public class onClick implements View.OnClickListener {

        @Override
        public void onClick(View view) {
            if (isFastClick()) {
                return;
            }
            if (StringUtils.isEmpty(mTxtPayTotal.getText().toString())) {
                CommonTools.toastShort(CardReturnQueryStdActivity.this, "请输入退款金额");
                return;
            }
            String cardYfTotal = mTxtPayTotal.getText().toString();
            String wfTotal = BigDecimalUtils.subtract(fSsTotal, cardYfTotal);
            if (BigDecimalUtils.isLess0(wfTotal)||BigDecimalUtils.isLessAndEquals0(cardYfTotal)) {
                CommonTools.toastShort(CardReturnQueryStdActivity.this, "请输入正确的金额");
                return;
            }
            LoggerUtils.getLogger().debug("maxRefund:" + maxRefund);
            if (BigDecimalUtils.isGreater(cardYfTotal, maxRefund)) {
                CommonTools.toastShort(String.format(Locale.CHINESE, "退款金额超出最大可退金额: %s", maxRefund));
                return;
            }
            Intent mIntent = new Intent();
            mIntent.putExtra("wfTotal", wfTotal);
            mIntent.putExtra("cardYfTotal", cardYfTotal);
            setResult(RESULT_CARD_BALANCE_QUERY_COMMIY, mIntent);
            finish();
        }
    }
}
