/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std.pay;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.ysy.card.CardSecurityInputActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.VipLoginPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IVipLoginView;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.hipos.payment.PayMethod;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 会员券查询界面
 *
 * @author xieliyan    2017-11-24 HIPOSAPP-235
 * @author xieliyan    2017-12-20 HIPOSAPP-243 不输入密码不允许查询
 * @author xieliyan    2018-03-07 HIPOSAPP-348 一段式赠券单笔添加
 * @author guolina     2019-02-25 HIPOSAPP-1029 去掉TicketPluBean.java文件，优化代码
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author wangyaogang 2019-06-12 HIPOSAPP-1126 会员查询时去掉密码输入框,不校验密码
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 * @author dukunbiao(null)  2019-10-17  重构
 * @author wangyaogang 2019-10-17 HIPOSAPP-1325 会员券页面点击会员按钮时，若已登入会员，scrm显示vipCardNo，crm显示cardFaceNo。
 */
@ActivityLayoutInject(R.layout.activity_vip_coupon_query_std)
public class CouponQueryStdActivity extends BaseMvpActivity<IVipLoginView, VipLoginPresenter> implements IVipLoginView {

    @BindView(R.id.edtCardNo)
    public EditText edtCardNo;

    @BindView(R.id.btnQuery)
    public Button btnQuery;

    private String remark;

    private CardNo cardNo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @Override
    protected VipLoginPresenter createPresenter() {
        return new VipLoginPresenter();
    }

    private void init() {
        SoftInput.disable(edtCardNo);
        KeyBoardView keyboard = new KeyBoardView(edtCardNo);
        keyboard.showKeyboard();

        btnQuery.setEnabled(true);

        mPresenter.initSelectedPayMethod();
        remark = mPresenter.getSale().getPayMethod().remark();
        if (mPresenter.getSale().isVip()) {
            if (PayMethod.isScrmCoupon(remark)) {
                if (!StringUtils.isEmpty(mPresenter.getSale().getVip().card().vipCardNo())) {
                    edtCardNo.setText(mPresenter.getSale().getVip().card().vipCardNo());
                }
            } else {
                if (!StringUtils.isEmpty(mPresenter.getSale().getVip().card().cardFaceNo())) {
                    edtCardNo.setText(mPresenter.getSale().getVip().card().cardFaceNo());
                }
            }
        }
    }

    @OnClick(R.id.btnQuery)
    public void onClickQuery() {
        if (StringUtils.isEmpty(edtCardNo.getText().toString())) {
            CommonTools.toastShort(CouponQueryStdActivity.this, "请输入会员卡号/手机号");
            return;
        }

        btnQuery.setEnabled(false);


        if (PayMethod.isScrmCoupon(remark)) {
            cardNo = CardNo.MobileOrDefault(edtCardNo.getText().toString().trim(), CardNoType.VipCode);
        } else {
            cardNo = CardNo.MobileOrDefault(edtCardNo.getText().toString().trim());
        }
        mPresenter.vipQuery(cardNo);
    }

    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        finish();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
        btnQuery.setEnabled(true);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == ActivityConstants.REQUEST_TO_CARD_SECURITY_INPUT && resultCode == ActivityConstants.RESULT_TO_CARD_SECURITY_INPUT) {
            if (data == null) {
                return;
            }

            CardSecurity cardSecurity = (CardSecurity) data.getSerializableExtra(CardSecurityInputActivity.CARD_SECURITY);
            mPresenter.verifyVip(cardSecurity, cardNo);
        }
    }

    @Override
    public void vipLoginSuccess(Vip vip) {
        mPresenter.queryVerifyType(vip);
    }

    @Override
    public void vipLoginFail(String msg) {
        btnQuery.setEnabled(true);
        showHintDialog(msg);
    }

    @Override
    public void verifyTypeSuccess(Vip vip) {
        Intent intent = new Intent();

        if (PayMethod.isScrmCoupon(remark)) {
            intent.putExtra("vipCardNo", vip.card().vipCardNo());
        } else {
            intent.putExtra("cardFaceNo", vip.card().cardFaceNo());
        }

        setResult(ActivityConstants.RESULT_VIP_COUPON_CALCULATE, intent);
        finish();
    }

    @Override
    public void verifyTypeFail(String msg) {
        btnQuery.setEnabled(true);
        showHintDialog(msg);
    }
}
