package com.hisense.hiaposcmp.activity.std.pay;

import android.content.Intent;
import android.os.Bundle;
import android.os.SystemClock;
import android.view.KeyEvent;
import android.view.View;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.EmptyPresenter;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.view.IView.IEmptyView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;
import com.hisense.hiaposcmp.view.MoneyView;
import com.hisense.hipos.payment.PayMethod;

import butterknife.BindView;

/**
 * @author wangyaogang 2019-12-27 HIAPOSCMP-455 标准版支付功能开发,聚合扫码页
 */
@ActivityLayoutInject(R.layout.activity_pay_std_dialog)
public class PayStdDialogActivity extends BaseMvpActivity<IEmptyView, EmptyPresenter> implements IEmptyView, ScanKeyEventHelper.OnScanSuccessListener {


    @BindView(R.id.tvDialogPayNo)
    public TextView mTvDialogPayNo;
    private ScanKeyEventHelper mScanKeyEventHelper;
    private MoneyView mEtDialogInputAmount;//支付金额
    private EditText mEtStdDialogPayno; //支付号
    private PayMethod mSelectedPayMethod;
    private String fSsTotal;
    private String activity;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initSelectedPayMethod(this.getIntent());
        init();
    }

    public void initSelectedPayMethod(Intent intent) {
        Bundle bundle = intent.getExtras();
        if (bundle == null) {
            mSelectedPayMethod = getSale().getPayMethod();
        } else {
            mSelectedPayMethod = (PayMethod) bundle.getSerializable("payMethod");

            if (bundle.containsKey("fSsTotal")) {
                fSsTotal = bundle.getString("fSsTotal");
            }
            if (bundle.containsKey("activity")) {
                activity = bundle.getString("activity");
            }
        }
    }

    @Override
    protected EmptyPresenter createPresenter() {
        return new EmptyPresenter();
    }


    private void init() {

        /**
         * 中间支付名称
         */
        TextView mTvStdDialogPayname = findViewById(R.id.tv_std_dialog_payname);
        mTvStdDialogPayname.setText(mSelectedPayMethod.name() + "支付");
        /**
         * 右上角关闭按钮
         */
        RelativeLayout mRlStdDialogCancel = findViewById(R.id.rl_std_dialog_cancel);
        mRlStdDialogCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        /**
         * 金额框
         */
        mEtDialogInputAmount = findViewById(R.id.et_dialog_input_amount);
        SoftInput.disable(mEtDialogInputAmount);
        mEtDialogInputAmount.setText(fSsTotal);
        /**
         * 金额框清空
         */
        RelativeLayout mRlStdDialogClear = findViewById(R.id.rl_std_dialog_clear);
        mRlStdDialogClear.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mEtDialogInputAmount.setText("");
                mEtDialogInputAmount.setFocusable(true);
                mEtDialogInputAmount.setFocusableInTouchMode(true);
                mEtDialogInputAmount.requestFocus();
            }
        });

        /**
         * 付款码
         */

        mEtStdDialogPayno = findViewById(R.id.et_std_dialog_payno);
        if ("RefundStdActivity".equals(activity)) {
            mTvDialogPayNo.setText("商户订单号");
        }
        SoftInput.disable(mEtStdDialogPayno);
        SystemClock.sleep(100);
        mEtStdDialogPayno.setFocusable(true);
        mEtStdDialogPayno.setFocusableInTouchMode(true);
        mEtStdDialogPayno.requestFocus();

        /**
         * 自定义键盘
         */
        KeyBoardStdView mEdtPayKeyboard = findViewById(R.id.edt_pay_keyboard);
        mEdtPayKeyboard.setOnKeyboardListener(new KeyBoardStdView.OnKeyboardListener() {
            @Override
            public void onInsertKeyEvent(String text) {
                if (mEtDialogInputAmount.isFocused()) {
                    KeyBoardStdView.addChar(mEtDialogInputAmount, text);
                } else if (mEtStdDialogPayno.isFocused()) {
                    KeyBoardStdView.addChar(mEtStdDialogPayno, text);
                }
            }

            @Override
            public void onDeleteKeyEvent() {
                if (mEtDialogInputAmount.isFocused()) {
                    KeyBoardStdView.deleteChar(mEtDialogInputAmount);
                } else if (mEtStdDialogPayno.isFocused()) {
                    KeyBoardStdView.deleteChar(mEtStdDialogPayno);
                }
            }

            @Override
            public void onClearKeyEvent() {
                if (mEtDialogInputAmount.isFocused()) {
                    KeyBoardStdView.clearChar(mEtDialogInputAmount);
                } else if (mEtStdDialogPayno.isFocused()) {
                    KeyBoardStdView.clearChar(mEtStdDialogPayno);
                }
            }

            @Override
            public void onDoneKeyEvent() {
                backToPay();
            }
        });

    }

    private void backToPay() {
        String inputMoney = mEtDialogInputAmount.getText().toString();
        String payNo = mEtStdDialogPayno.getText().toString();
        if (StringUtils.isEmpty(inputMoney)) {
            showHintDialog("请输入金额");
            return;
        }
        if (BigDecimalUtils.isEquals0(inputMoney)) {
            showHintDialog("金额不能为0");
            return;
        }
        if (!"RefundStdActivity".equals(activity)) {
            if (BigDecimalUtils.isGreater(inputMoney, getSale().getTotalUnPaid())) {
                showHintDialog("支付金额不能大于未支付金额");
                return;
            }
        }

        if (StringUtils.isEmpty(payNo)) {
            showHintDialog("请输入支付号");
            return;
        }

        Intent intent = new Intent();
        intent.putExtra("inputMoney", inputMoney);
        intent.putExtra("payNo", payNo);
        setResult(ActivityConstants.RESULT_NTE_PAY_SUCCESS, intent);
        finish();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK || keyCode == KeyEvent.KEYCODE_HOME || keyCode == KeyEvent.KEYCODE_MENU) {
            LoggerUtils.getLogger().info("标准版聚合弹出页不允许直接返回");
            return false;
        }

        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        mEtStdDialogPayno.setFocusable(true);
        mEtStdDialogPayno.setFocusableInTouchMode(true);
        mEtStdDialogPayno.requestFocus();
        mEtStdDialogPayno.setSelection(mEtStdDialogPayno.getText().length());
        if (event.getKeyCode() == KeyEvent.KEYCODE_ENTER && isFastClick()) {
            //屏蔽连扫的第二个条码
            LoggerUtils.getLogger().info("屏蔽条码:" + event.getKeyCode());
        } else {
            mScanKeyEventHelper.analysisKeyEvent(event);
        }

        return super.dispatchKeyEvent(event);
    }

    @Override
    public void onScanSuccess(String barcode) {
        LoggerUtils.getLogger().info("ac_onScanSuccess:" + barcode);
        if (mEtStdDialogPayno != null && mEtDialogInputAmount != null) {
            mEtStdDialogPayno.setText(barcode);
            mEtStdDialogPayno.setSelection(mEtStdDialogPayno.getText().length());
            backToPay();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mScanKeyEventHelper = new ScanKeyEventHelper(this);
    }
}
