/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std.pay;

import android.content.Intent;
import android.os.Bundle;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.VipCard;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hiaposcmp.tool.UpdateTextButton;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.hipos.payment.PayMethod;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 积分余额查询显示
 *
 * @author zhenghao    2017-03-30
 * @author xieliyan    2017-10-30 HIPOSAPP-196 确定按钮点击一次后置为不可用
 * @author xieliyan    2017-12-21 HIPOSAPP-254 控制支付金额的大小更改
 * @author wangyaogang 2019-03-07 HIPOSAPP-1040 修复储值卡/积分卡支付时,金额正确但提示"请输入正确的金额"的问题
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author dukunbiao(null)  2019-10-16  重构
 * @author wangyaogang 2020-01-15 HIAPOSCMP-496 标准版-储值卡/积分支付页面改造
 * @author wangyaogang 2020-03-30 HIAPOSCMP-613 scrm余额使用usableTotal, scrm积分和crm积分使用usableBalance进行支付判断
 */
@ActivityLayoutInject(R.layout.activity_vip_points_pay_std)
public class PointsPayShowStdActivity extends BaseActivity {

    @BindView(R.id.textYF)
    public EditText textYF;

    @BindView(R.id.btnCommit)
    public Button mBtnCommit;

    @BindView(R.id.textJfTotal)
    public TextView txtJfTotal;

    @BindView(R.id.textJfBalace)
    public TextView txtJfBalance;

    @BindView(R.id.cancel_action)
    public Button btnCancel;

    @BindView(R.id.textVipName)
    public TextView txtVipName;

    @BindView(R.id.textCardTypeName)
    public TextView txtCardTypeName;

    @BindView(R.id.textCardFaceNo)
    public TextView txtCardFaceNo;

    private String total = "0";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        SoftInput.disable(textYF);
        KeyBoardView keyboard = new KeyBoardView(textYF);
        keyboard.showKeyboard();
        NumberHandler.setPoint(textYF, 2);//只允许输入两位小数
        Bundle bundle = this.getIntent().getExtras();
        if (bundle == null) {
            CommonTools.toastShort("获取卡信息失败，请重新操作");
            return;
        }

        VipCard card = (VipCard) bundle.getSerializable("card");
        if (card == null) {
            CommonTools.toastShort("获取卡信息失败，请重新操作");
            return;
        }

        String remark = bundle.getString("remark");
        if (remark == null) {
            CommonTools.toastShort("获取支付信息失败，请重新操作");
            return;
        }

        txtJfTotal.setText(String.format(Locale.CHINESE, "%.2f", card.usablePoints()));
        if (PayMethod.isScrmRemain(remark)) {
            txtJfBalance.setText(String.format(Locale.CHINESE, "%.2f", card.usableTotal()));
        } else {
            txtJfBalance.setText(String.format(Locale.CHINESE, "%.2f", card.usableBalance()));
        }

        txtVipName.setText(card.vipName());
        txtCardTypeName.setText(card.cardType().name());

        total = bundle.getString("fSsTotal");
        txtCardFaceNo.setText(card.cardFaceNo());
        double usableTotal;
        try {
            usableTotal = Double.parseDouble(txtJfBalance.getText().toString());
        } catch (NumberFormatException e) {
            usableTotal = 0;
        }
        UpdateTextButton.updateTotalText(textYF, total, usableTotal);
        UpdateTextButton.setOnTextChange(textYF, total, usableTotal);
        textYF.setInputType(EditorInfo.TYPE_CLASS_PHONE);
        textYF.setSelection(textYF.getText().length());
    }

    @OnClick(R.id.cancel_action)
    public void onClickCancel() {
        if (isFastClick()) {
            return;
        }
        finish();
    }

    @OnClick(R.id.btnCommit)
    public void onClickCommit() {
        if (isFastClick()) {
            return;
        }
        if (StringUtils.isEmpty(textYF.getText().toString())) {
            CommonTools.toastShort(PointsPayShowStdActivity.this, "请输入支付金额");
            return;
        }

        mBtnCommit.setEnabled(false);
        String cardYfTotal = textYF.getText().toString();
        String wfTotal = BigDecimalUtils.subtract(total, cardYfTotal);
        String canUse = BigDecimalUtils.subtract(txtJfBalance.getText().toString(), cardYfTotal);
        if (BigDecimalUtils.isLess0(wfTotal) || BigDecimalUtils.isLess0(canUse)) {
            LoggerUtils.getLogger().debug("超出余额");
            mBtnCommit.setEnabled(true);
            CommonTools.toastShort(PointsPayShowStdActivity.this, "请输入正确的金额");
            return;
        }

        Intent mIntent = new Intent();
        mIntent.putExtra("wfTotal", wfTotal);
        mIntent.putExtra("cardYfTotal", cardYfTotal);
        setResult(ActivityConstants.RESULT_TO_POINTS_PAY_SHOW, mIntent);
        finish();
    }
}
