/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.std.pay;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.StartActivityService;
import com.hisense.hiaposcmp.activity.ysy.card.CardBaseMvpActivity;
import com.hisense.hiaposcmp.activity.ysy.card.CardSecurityInputActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.pay.PointsPayPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.pay.IPointsPayView;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import butterknife.BindView;
import butterknife.OnClick;

import static com.hisense.hiaposcmp.constant.ActivityConstants.REQUEST_CARD_BALANCE_QUERY;
import static com.hisense.hiaposcmp.constant.ActivityConstants.RESULT_CARD_BALANCE_QUERY_COMMIY;

/**
 * 积分支付刷卡读取信息
 *
 * @author zhenghao    2017-03-09
 * @author xieliyan    2017-09-27 HIPOSAPP-160 刷卡（适配HI98）
 * @author xieliyan    2017-10-11 HIPOSAPP-165 积分支付时，不输入密码不允许支付
 * @author xieliyan    2017-10-31 HIPOSAPP-201 使用枚举实现单例模式:更改调用方法
 * @author zhangsai    2017-11-15 HIPOSAPP-212
 * @author xieliyan    2017-11-22 HIPOSAPP-224 支持射频卡2.0
 * @author xieliyan    2018-01-19 HIPOSAPP-286 优化文本编辑控件
 * @author xieliyan    2018-04-09 HIPOSAPP-382 修复由于App.getContext空指针异常导致的闪退问题
 * @author guolina     2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author wangyaogang 2019-06-04 HIPOSAPP-1156 积分支付时会员认证方式也根据配置来控制
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 手动录入会员号时此处不再校验密码
 * @author dukunbiao(null)  2019-10-16  重构
 * @author wangyaogang 2020-01-15 HIAPOSCMP-496 标准版-储值卡/积分支付页面改造
 */
@ActivityLayoutInject(R.layout.activity_mult_card_query_std)
public class PointsPayStdActivity extends CardBaseMvpActivity<IPointsPayView, PointsPayPresenter> implements IPointsPayView {

    @BindView(R.id.commit_action)
    public Button btnCommitAction;

    @BindView(R.id.edt_card_no)
    EditText mEdtCardNo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @Override
    protected PointsPayPresenter createPresenter() {
        return new PointsPayPresenter();
    }

    private void init() {
        mScanner = getScanner(this);
        mPresenter.initSaleRepository();

        SoftInput.disable(mEdtCardNo);
        KeyBoardView keyboard = new KeyBoardView(mEdtCardNo);
        keyboard.showKeyboard();

        if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
            mEdtCardNo.setFocusable(false);
            mEdtCardNo.setFocusableInTouchMode(false);
            mEdtCardNo.setHint("刷卡");
            keyboard.hideKeyboard();
        }

        mPresenter.initSelectedPayMethod(this.getIntent());
    }

    @OnClick(R.id.btnScan)
    public void onClickScan() {
//        showHintDialog("暂不支持,应该弹出一个等待扫码页面");
        if (isFastClick()) {
            return;
        }
        scan();
    }

    @OnClick(R.id.cancel_action)
    public void onClickCancel() {
        if (isFastClick()) {
            return;
        }
        btnCommitAction.setEnabled(true);
        setResult(99);
        finish();
    }

    @OnClick(R.id.commit_action)
    public void onClickCommit() {
        if (isFastClick()) {
            return;
        }
        String sCardNo = mEdtCardNo.getText().toString().trim();
        if (StringUtils.isEmpty(sCardNo)) {
            if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
                CommonTools.toastShort(PointsPayStdActivity.this, "请刷卡");
            } else {
                CommonTools.toastShort(PointsPayStdActivity.this, "请输入卡号");
            }
            return;
        }

        btnCommitAction.setEnabled(false);

        mPresenter.queryVerifyType(CardNo.MobileOrDefault(sCardNo, CardNoType.CardFaceNo));
    }

    @Override
    protected void onCardRead(CardNo cardNo) {
        mPresenter.queryVerifyType(cardNo);
    }

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();

                mPresenter.queryVerifyType(CardNo.PayCode(s));
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            CardNo cardNo = CardNo.PayCode(mScanner.getScanResult(data));

            mPresenter.queryVerifyType(cardNo);
        } else if ((requestCode == ActivityConstants.REQUEST_TO_POINTS_PAY_SHOW) && (resultCode == ActivityConstants.RESULT_TO_POINTS_PAY_SHOW)) {
            mPresenter.pointsPay(data);
        } else if ((requestCode == ActivityConstants.REQUEST_TO_CARD_SECURITY_INPUT) && (resultCode == ActivityConstants.RESULT_TO_CARD_SECURITY_INPUT)) {
            CardSecurity cardSecurity = (CardSecurity) data.getSerializableExtra(CardSecurityInputActivity.CARD_SECURITY);

            mPresenter.pointsQuery(cardSecurity);
        } else if (requestCode == REQUEST_CARD_BALANCE_QUERY && resultCode == RESULT_CARD_BALANCE_QUERY_COMMIY) {
            mPresenter.pointsRefund(data);
        }

        btnCommitAction.setEnabled(true);
    }

    @Override
    public void needVerifyType(boolean iseCard, VerifyType verifyType) {
        StartActivityService.startCardSecurityInputActivity(this, iseCard, verifyType);
    }

    @Override
    public void pointsQuerySuccess(VipCard card) {
        StartActivityService.gotoPointsPayShowActivity(this, card, mPresenter.getSale().getTotalUnPaid(), mPresenter.getPayMethod().remark());
    }

    @Override
    public void pointsQueryRefund(VipCard card, String fSsTotal, String maxRefund) {
        StartActivityService.gotoCardReturnQueryActivity(this, "vipCard", null, card, fSsTotal, maxRefund, mPresenter.getPayMethod().remark());
    }

    @Override
    public void pointsQueryError() {
        btnCommitAction.setEnabled(true);
        PointsPayStdActivity.super.restartListen();
    }

    @Override
    public void pointsPaySuccess() {
        setResult(ActivityConstants.RESULT_PAY_TO_POINTS_SUCCESS);
        finish();
    }

    @Override
    public void pointsRefundSuccess() {
        setResult(ActivityConstants.RESULT_REFUND_TO_POINTS_SUCCESS);
        finish();
    }
}
