package com.hisense.hiaposcmp.activity.std.query;


import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.Card;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpFragment;
import com.hisense.hiaposcmp.activity.StartActivityService;
import com.hisense.hiaposcmp.activity.ysy.card.CardSecurityInputActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.query.CardAndPointsQueryPresenter;
import com.hisense.hiaposcmp.view.IView.query.ICardAndPointsQueryView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;
import com.hisense.hipos.payment.PayMethod;

import org.apache.commons.lang3.StringUtils;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;


/**
 * @author wangyaogang 2020-01-10 HIAPOSCMP-474 查询功能
 */
@ActivityLayoutInject(R.layout.fragment_card_balance_query)
public class PointsBalanceQueryFragment extends BaseMvpFragment<ICardAndPointsQueryView, CardAndPointsQueryPresenter> implements ICardAndPointsQueryView {
    public final static String TAG = "PointsBalanceQueryFragment";
    /**
     * 中间 卡类型
     */
    @BindView(R.id.tv_card_type)
    TextView mTvCardType;
    /**
     * 中间 卡号
     */
    @BindView(R.id.tv_card_no)
    TextView mTvCardNo;
    /**
     * 中间 有效期
     */
    @BindView(R.id.tv_expiry_date)
    TextView mTvExpiryDate;
    /**
     * 右侧 会员名
     */
    @BindView(R.id.tv_user_name)
    TextView mTvUserName;
    /**
     * 右侧 手机号
     */
    @BindView(R.id.tv_user_phone)
    TextView mTvUserPhone;
    /**
     * 右侧 卡号
     */
    @BindView(R.id.tv_right_card_no)
    TextView mTvRightCardNo;
    /**
     * 右侧 生日
     */
    @BindView(R.id.tv_right_birthday)
    TextView mTvRightBirthday;
    /**
     * 右侧 卡类型
     */
    @BindView(R.id.tv_right_card_type)
    TextView mTvRightCardType;
    /**
     * 右侧 卡号录入框
     */
    @BindView(R.id.et_right_input_no)
    EditText mEtRightInputNo;
    /**
     * 右侧 键盘
     */
    @BindView(R.id.edt_pay_keyboard)
    KeyBoardStdView mEdtPayKeyboard;
    /**
     * 右侧 查询按钮
     */
    @BindView(R.id.ll_card_query)
    LinearLayout mLlCardQuery;
    /**
     * 中间  余额
     */
    @BindView(R.id.tv_total_amount)
    TextView mTvTotalAmount;

    private CardNo cardNo;
    private String payMethod = PayMethod.VIP_POINTS;

    public PointsBalanceQueryFragment() {
        // Required empty public constructor
    }


    @Override
    protected void initData() {
        SoftInput.disable(mEtRightInputNo);
        mEdtPayKeyboard.setOnKeyboardListener(new KeyBoardStdView.OnKeyboardListener() {
            @Override
            public void onInsertKeyEvent(String text) {
                KeyBoardStdView.addChar(mEtRightInputNo, text);
            }

            @Override
            public void onDeleteKeyEvent() {
                KeyBoardStdView.deleteChar(mEtRightInputNo);
            }

            @Override
            public void onClearKeyEvent() {
                KeyBoardStdView.clearChar(mEtRightInputNo);
            }

            @Override
            public void onDoneKeyEvent() {
                verify();
            }
        });
        mEtRightInputNo.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                String result = replaceSymbol(s);
                if (result != null) {
                    mEtRightInputNo.setText(result);
                    mEtRightInputNo.setSelection(mEtRightInputNo.length());
                }
            }
        });
    }

    @Override
    protected CardAndPointsQueryPresenter createPresenter() {
        return new CardAndPointsQueryPresenter();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return super.onCreateView(inflater, container, savedInstanceState);
    }

    /**
     * A 拿到刷卡结果,调用 F,执行后续流程
     *
     * @param cardNo 卡信息
     */
    public void verify(CardNo cardNo) {
        LoggerUtils.getLogger().debug("积分_取得刷卡结果");
        clearData();
        this.cardNo = cardNo;
        mPresenter.verify(cardNo, payMethod);
    }

    @Override
    public void verifySuccess(VerifyType verifyType) {
        if (!verifyType.needVerify()) {
            mPresenter.queryPoints(cardNo, null);
        } else {
            if (getActivity() != null) {
                StartActivityService.startCardSecurityInputActivity(getActivity(), false, verifyType);
            } else {
                LoggerUtils.getLogger().debug("积分查询:getActivity() == null");
            }
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == ActivityConstants.REQUEST_TO_CARD_SECURITY_INPUT && resultCode == ActivityConstants.RESULT_TO_CARD_SECURITY_INPUT) {
            CardSecurity cardSecurity = (CardSecurity) data.getSerializableExtra(CardSecurityInputActivity.CARD_SECURITY);
            mPresenter.queryPoints(cardNo, cardSecurity);
        }
    }

    @Override
    public void queryCardSuccess(Card card) {

    }

    @Override
    public void queryPointsSuccess(VipCard card) {
        mEtRightInputNo.setText("");
        mTvCardType.setText(String.format(Locale.CHINESE, "%s-%s", card.cardType().code(), card.cardType().name()));
        mTvRightCardType.setText(String.format(Locale.CHINESE, "%s-%s", card.cardType().code(), card.cardType().name()));

        mTvCardNo.setText(card.cardFaceNo());
        mTvRightCardNo.setText(card.cardFaceNo());

        mTvExpiryDate.setText(card.expireDate());

        mTvTotalAmount.setText(String.format(Locale.CHINESE, "¥ %.2f", card.usableBalance()));

        mTvUserName.setText(card.vipName());
        mTvUserPhone.setVisibility(View.GONE);
    }

    @Override
    public void showErrorMsg(String msg) {
        showHintDialog(msg);
        if (getActivity() != null) {
            ((QueryStdActivity) getActivity()).restart();
        }
    }

    private void clearData() {
        mTvCardType.setText("");
        mTvRightCardType.setText("");
        mTvCardNo.setText("");
        mTvRightCardNo.setText("");
        mTvExpiryDate.setText("");
        mTvTotalAmount.setText("");
        mTvUserName.setText("");
    }

    @OnClick(R.id.ll_card_query)
    public void onViewClicked() {
        if (isFastClick()) {
            return;
        }
        if (getActivity() != null) {
            ((QueryStdActivity) getActivity()).executeAnimation(mLlCardQuery);
        }
        verify();
    }

    private void verify() {
        clearData();
        String no = mEtRightInputNo.getText().toString().trim();
        if (StringUtils.isEmpty(no)) {
            showHintDialog("请输入手机号或会员卡号");
            return;
        }
        cardNo = CardNo.MobileOrDefault(no);
        mPresenter.verify(cardNo, payMethod);
    }
}
