package com.hisense.hiaposcmp.activity.std.query;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.ysy.card.CardBaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.presenter.EmptyPresenter;
import com.hisense.hiaposcmp.view.IView.IEmptyView;
import com.hisense.hipos.user.Privileges;

import androidx.fragment.app.Fragment;
import butterknife.BindView;
import butterknife.OnClick;

/**
 * @author wangyaogang 2020-01-10 HIAPOSCMP-474 查询功能
 */
@ActivityLayoutInject(R.layout.activity_query_std)
public class QueryStdActivity extends CardBaseMvpActivity<IEmptyView, EmptyPresenter> implements IEmptyView {

    /**
     * 会员的图标
     */
    @BindView(R.id.tvStdTitleVipIcon)
    TextView tvStdTitleVipIcon;

    /**
     * 正中的标题
     */
    @BindView(R.id.tvStdTitleItem)
    TextView mTvStdTitleItem;

    /**
     * 标题栏  时间
     */
    @BindView(R.id.tvStdTitleDate)
    TextView tvStdTitleDate;
    /**
     * 标题栏 门店名称
     */
    @BindView(R.id.tvStdTitleOrgName)
    TextView tvStdTitleOrgName;
    /**
     * 标题栏 营业员名称
     */
    @BindView(R.id.tvStdTitleCashierName)
    TextView tvStdTitleCashierName;
    /**
     * 会员登录(隐藏)
     */
    @BindView(R.id.tvStdTitleVipLogin)
    public TextView tvStdTitleVipLogin;

    /**
     * 切换会员(隐藏)
     */
    @BindView(R.id.tvStdTitleVipChange)
    TextView tvStdTitleVipChange;

    @BindView(R.id.tvTipSaleNo)
    TextView mTvTipSaleNo;

    /**
     * 储值卡查询
     */
    @BindView(R.id.tv_card_balance)
    TextView mTvCardBalance;
    /**
     * 会员查询
     */
    @BindView(R.id.tv_vip_query)
    TextView mTvVipQuery;
    /**
     * 积分查询
     */
    @BindView(R.id.tv_points_balance)
    TextView mTvPointsBalance;
    /**
     * 流水查询
     */
    @BindView(R.id.tv_sale_query)
    TextView mTvSaleQuery;

    private TextView[] mClickViews = new TextView[4];

    /**
     * 当前选择的fragment
     */
    private Fragment currentFragment;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        init();
    }

    private final BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (action != null && action.equals(Intent.ACTION_TIME_TICK)) {
                tvStdTitleDate.setText(DateUtils.dateFormatWithWeek("MM月dd日    ww   aa hh:mm"));
            }
        }
    };

    private void init() {
        isNotShowExitDialog(true);
        mTvStdTitleItem.setText("查询");
        IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_TIME_TICK);
        registerReceiver(receiver, filter);

        tvStdTitleVipLogin.setVisibility(View.GONE);
        tvStdTitleVipChange.setVisibility(View.GONE);
        tvStdTitleDate.setText(DateUtils.dateFormatWithWeek("MM月dd日    ww   aa hh:mm"));
        tvStdTitleOrgName.setText(com.hisense.hipos.webapi.Context.orgName());
        tvStdTitleCashierName.setText(Config.user_name);
        tvStdTitleVipIcon.setVisibility(View.GONE);
        mTvTipSaleNo.setVisibility(View.GONE);

        mClickViews[0] = mTvCardBalance;
        mClickViews[1] = mTvVipQuery;
        mClickViews[2] = mTvPointsBalance;
        mClickViews[3] = mTvSaleQuery;

        if (BaseMvpActivity.hasPrivilege(Privileges.QUERY_CARD_BALANCE)) {
            updateViewState(mTvCardBalance);
            showFragment(CardBalanceQueryFragment.TAG);
        } else {
            showHintDialog("没有储值卡余额查询权限");
        }
    }

    @Override
    protected EmptyPresenter createPresenter() {
        return new EmptyPresenter();
    }

    @OnClick({R.id.ll_pay_back, R.id.linearStdTitleHome})
    public void onBackClicked(View view) {
        finish();
    }

    @OnClick({R.id.tv_card_balance, R.id.tv_vip_query, R.id.tv_points_balance, R.id.tv_sale_query})
    public void onViewClicked(View view) {
        updateViewState(view);
        switch (view.getId()) {
            case R.id.tv_card_balance:
            default:
                if (BaseMvpActivity.hasPrivilege(Privileges.QUERY_CARD_BALANCE)) {
                    showFragment(CardBalanceQueryFragment.TAG);
                } else {
                    showHintDialog("没有储值卡余额查询权限");
                }
                break;
            case R.id.tv_vip_query:
                if (BaseMvpActivity.hasPrivilege(Privileges.REPORT_VIP_BALANCE)) {
                    showFragment(VipQueryFragment.TAG);
                } else {
                    showHintDialog("没有会员查询权限");
                }
                break;
            case R.id.tv_points_balance:
                if (BaseMvpActivity.hasPrivilege(Privileges.QUERY_VIP_POINTS)) {
                    showFragment(PointsBalanceQueryFragment.TAG);
                } else {
                    showHintDialog("没有积分余额查询权限");
                }
                break;
            case R.id.tv_sale_query:
                if (BaseMvpActivity.hasPrivilege(Privileges.QUERY_SALE)) {
                    showFragment(SaleQueryFragment.TAG);
                } else {
                    showHintDialog("没有流水查询权限");
                }
                break;
        }
    }

    /**
     * 更新视图状态
     *
     * @param view 选中的view
     */
    private void updateViewState(View view) {
        for (TextView clickView : mClickViews) {
            if (clickView == view) {
                clickView.setSelected(true);
            } else {
                clickView.setSelected(false);
            }
        }
    }

    /**
     * 通过show/hide 的方法 切换fragment
     *
     * @param tag 标记
     */
    private void showFragment(String tag) {
        if (currentFragment != null) {
            //todo  使用remove ,每次都是新创建,不保留历史数据
            getSupportFragmentManager().beginTransaction().remove(currentFragment).commit();
//            getSupportFragmentManager().beginTransaction().hide(currentFragment).commit();
        }
        currentFragment = getSupportFragmentManager().findFragmentByTag(tag);
        if (currentFragment == null) {
            switch (tag) {
                case CardBalanceQueryFragment.TAG:
                default:
                    currentFragment = new CardBalanceQueryFragment();
                    break;
                case VipQueryFragment.TAG:
                    currentFragment = new VipQueryFragment();
                    break;
                case PointsBalanceQueryFragment.TAG:
                    currentFragment = new PointsBalanceQueryFragment();
                    break;
                case SaleQueryFragment.TAG:
                    currentFragment = new SaleQueryFragment();
                    break;
            }
            getSupportFragmentManager().beginTransaction().add(R.id.fl_query_container, currentFragment, tag).commit();
        } else {
            getSupportFragmentManager().beginTransaction().show(currentFragment).commit();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        LoggerUtils.getLogger().debug("QueryStdActivity收到回调:" + requestCode);
        if (currentFragment != null) {
            currentFragment.onActivityResult(requestCode, resultCode, data);
        }
    }

    @Override
    protected void onCardRead(CardNo cardNo) {
        if (currentFragment != null) {
            if (currentFragment instanceof CardBalanceQueryFragment) {
                ((CardBalanceQueryFragment) currentFragment).verify(cardNo);
            } else if (currentFragment instanceof PointsBalanceQueryFragment) {
                ((PointsBalanceQueryFragment) currentFragment).verify(cardNo);
            } else if (currentFragment instanceof VipQueryFragment) {
                ((VipQueryFragment) currentFragment).query(cardNo);
            }
        }
    }

    /**
     * F 调用 A 重启刷卡监听
     */
    public void restart() {
        super.restartListen();
    }
}
