package com.hisense.hiaposcmp.activity.std.query;


import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpFragment;
import com.hisense.hiaposcmp.adapter.QueryStdPayedMethodAdapter;
import com.hisense.hiaposcmp.adapter.QueryStdPluAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.query.SaleQueryPresenter;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.view.IView.query.ISaleQueryView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.webapi.pay.PayService;
import com.hisense.webapi.Callback;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;


/**
 * @author wangyaogang 2020-01-10 HIAPOSCMP-474 查询功能
 */
@ActivityLayoutInject(R.layout.fragment_sale_query)
public class SaleQueryFragment extends BaseMvpFragment<ISaleQueryView, SaleQueryPresenter> implements ISaleQueryView {
    public final static String TAG = "SaleQueryFragment";
    @BindView(R.id.rcv_plu_list)
    RecyclerView mRcvPluList;
    @BindView(R.id.tv_right_ss_total)
    TextView mTvRightSsTotal;
    @BindView(R.id.rcv_payed_list)
    RecyclerView mRcvPayedList;
    @BindView(R.id.et_right_input_no)
    EditText mEtRightInputNo;
    @BindView(R.id.rl_std_clear)
    RelativeLayout mRlStdClear;
    @BindView(R.id.edt_pay_keyboard)
    KeyBoardStdView mEdtPayKeyboard;
    @BindView(R.id.ll_receipt_print)
    LinearLayout mLlReceiptPrint;
    @BindView(R.id.ll_sale_print)
    LinearLayout mLlSalePrint;

    //存储海信支付的支付信息
    private List<SalePayModel> lstSalePayment = new ArrayList<>();

    // 查出的交易流水
    private BaseSaleModel sale;
    private QueryStdPayedMethodAdapter mQueryStdPayedMethodAdapter;
    private QueryStdPluAdapter mQueryStdPluAdapter;

    public SaleQueryFragment() {
        // Required empty public constructor
    }

    @Override
    protected void initData() {
        LoggerUtils.getLogger().debug("流水查询");
        SoftInput.disable(mEtRightInputNo);
        mEtRightInputNo.requestFocus();
        mEdtPayKeyboard.setOnKeyboardListener(new KeyBoardStdView.OnKeyboardListener() {
            @Override
            public void onInsertKeyEvent(String text) {
                KeyBoardStdView.addChar(mEtRightInputNo, text);
            }

            @Override
            public void onDeleteKeyEvent() {
                KeyBoardStdView.deleteChar(mEtRightInputNo);
            }

            @Override
            public void onClearKeyEvent() {
                KeyBoardStdView.clearChar(mEtRightInputNo);
            }

            @Override
            public void onDoneKeyEvent() {
                querySale();
            }
        });
        mEtRightInputNo.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                String result = replaceSymbol(s);
                if (result != null) {
                    mEtRightInputNo.setText(result);
                    mEtRightInputNo.setSelection(mEtRightInputNo.length());
                    mEtRightInputNo.requestFocus();
                }
            }
        });

        getMainSaleModel().reset();
        showData(false);

        //查找最大流水号
        mPresenter.getMaxSaleNo();
        //获取发票配置
        mPresenter.getInvoiceConfig();
    }

    @Override
    protected SaleQueryPresenter createPresenter() {
        return new SaleQueryPresenter();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return super.onCreateView(inflater, container, savedInstanceState);
    }

    /**
     * 查询流水。
     */
    private void querySale() {
        mRcvPluList.setAdapter(null);
        mRcvPayedList.setAdapter(null);
        showData(false);

        mPresenter.querySale(mEtRightInputNo.getText().toString());
    }

    /**
     * 更新显示流水数据。
     *
     * @param sale 流水
     */
    private void updateView(BaseSaleModel sale) {
        boolean isReturn = sale instanceof ReturnSaleModel;

        updateSaleItemsView(sale, isReturn);
        updatePaymentView(sale);
        mTvRightSsTotal.setText(String.format(Locale.CHINESE, "%.2f", Double.parseDouble(sale.getTotalRecv())));
    }

    /**
     * 更新交易商品信息。
     *
     * @param sale     交易流水
     * @param isReturn 是否退货
     */
    private void updateSaleItemsView(final BaseSaleModel sale, boolean isReturn) {
        if (getContext() == null) {
            LoggerUtils.getLogger().debug("更新商品时,getContext() == null");
            return;
        }
        LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mQueryStdPluAdapter = new QueryStdPluAdapter(getContext(), sale.getPlus());
        mRcvPluList.setLayoutManager(layoutManager);
        mRcvPluList.setAdapter(mQueryStdPluAdapter);
    }

    /**
     * 更新流水支付信息。
     *
     * @param sale 交易流水
     */
    private void updatePaymentView(final BaseSaleModel sale) {
        if (lstSalePayment.size() > 0) {
            lstSalePayment.clear();
        }
        if (getContext() == null) {
            LoggerUtils.getLogger().debug("更新支付列表时,getContext() == null");
            return;
        }
        if (sale != null && sale.getPays().size() > 0) {
            LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
            layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
            mQueryStdPayedMethodAdapter = new QueryStdPayedMethodAdapter(getContext(), sale.getPays());
            mRcvPayedList.setLayoutManager(layoutManager);
            mRcvPayedList.setAdapter(mQueryStdPayedMethodAdapter);

            for (SalePayModel salePay : sale.getPays()) {
                //海信支付的微信, 海信支付的
                if (salePay.getRemark() != null && GeneralSettings.INSTANCE.printHXPayReceipt()) {
                   /* if ((AggregatePaymentChannel.isHXPAY() && PayMethod.isWxPay(salePay.getRemark()))
                            || (AggregatePaymentChannel.isHXPAY() && PayMethod.isAliPay(salePay.getRemark()) && !CommonUtils.isTimesSquare(getContext()))) {
                        lstSalePayment.add(salePay);
                    }*/
                    //标准版默认只有海信支付
                    if (PayMethod.isWxPay(salePay.getRemark()) || PayMethod.isAliPay(salePay.getRemark())) {
                        lstSalePayment.add(salePay);
                    }
                }
            }
        }
    }

    /**
     * 更新布局。
     * todo 改成置空数据,刷新一下
     *
     * @param show true：布局需要显示 false：不需要显示
     */
    private void showData(final boolean show) {
        if (show) {
            mRcvPluList.setVisibility(View.VISIBLE);
            mRcvPayedList.setVisibility(View.VISIBLE);
            mTvRightSsTotal.setVisibility(View.VISIBLE);
        } else {
            mRcvPluList.setVisibility(View.INVISIBLE);
            mRcvPayedList.setVisibility(View.INVISIBLE);
            mTvRightSsTotal.setVisibility(View.INVISIBLE);
        }
    }

    @OnClick({R.id.rl_std_clear, R.id.ll_receipt_print, R.id.ll_sale_print})
    public void onViewClicked(View view) {
        if (isFastClick()) {
            LoggerUtils.getLogger().debug("流水查询快速点击");
            return;
        }
        if (getActivity() != null) {
            ((QueryStdActivity) getActivity()).executeAnimation(view);
        }
        switch (view.getId()) {
            case R.id.rl_std_clear:
                mEtRightInputNo.setText("");
                mEtRightInputNo.requestFocus();
                break;
            case R.id.ll_receipt_print:
                //先打印海信支付签购单,再银联支付签购单? 且需要锁,明确知道前者已打印完
                reprintHXReceipt();
                break;
            case R.id.ll_sale_print:
                //重打小票
                mPresenter.reprintTrade(sale);
                break;
            default:
                break;
        }
    }

    private void printReceipt(final int pos) {
        try {
            final SalePayModel salePayment = lstSalePayment.get(pos);
            LoggerUtils.getLogger().debug("海信支付重打印签购单:" + salePayment.getPayCode());
            showLoadingDialog();
            new Thread(new Runnable() {
                @Override
                public void run() {
                    PrinterHelper.getInstance().printWithStringList(PayService.getPrintContent(sale, null, salePayment), new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            dismissAndCallback(pos);
                        }

                        @Override
                        public void onError(String err) {
                            dismissAndCallback(pos);
                        }
                    });
                }
            }).start();

        } catch (Exception e) {
            LoggerUtils.getLogger().debug("海信支付打印重打印签购单异常:" + e.toString());
            dismissAndCallback(pos);
        }
    }

    private void dismissAndCallback(final int pos) {
        LoggerUtils.getLogger().debug("海信支付打印重打印签购单dismissAndCallback");
        hideLoadingDialog();
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (pos == lstSalePayment.size() - 1) {
                    LoggerUtils.getLogger().debug("海信支付打印完,打印银联签购单");
                } else {
                    printReceipt(pos + 1);
                }
            }
        });
    }

    /**
     * 打印海信支付签购单
     */
    private void reprintHXReceipt() {
        if (lstSalePayment.isEmpty()) {
            LoggerUtils.getLogger().debug("无海信支付,打印银联签购单");
            showHintDialog("未找到可打印签购单的支付方式");
            return;
        }
        printReceipt(0);
    }


    @Override
    public void getMaxSaleNoSuccess(String maxSaleNo) {
        mEtRightInputNo.setText(maxSaleNo);
        mEtRightInputNo.setSelection(mEtRightInputNo.length());
        mEtRightInputNo.requestFocus();
    }

    @Override
    public void querySaleSuccess(BaseSaleModel sale) {
        if (sale.isVip()) {
            showLoadingDialog();
            CardNo cardNo = CardNo.cardNo(sale.getVip().getCard().cardFaceNo(), sale.getVip().getCard().cardNoType());
            VipService.vipQuery(cardNo, null, sale.getSaleNo(), new Callback<Vip>() {
                @Override
                public void onSuccess(Vip vip) {
                    hideLoadingDialog();
                    sale.setVip(vip);
                    updateUi(sale);
                }

                @Override
                public void onError(Throwable throwable) {
                    LoggerUtils.getLogger().error("流水查询中-会员查询失败：" + throwable.getMessage());
                    hideLoadingDialog();
                    updateUi(sale);
                }
            });
        } else {
            updateUi(sale);
        }
    }

    /**
     * 刷新页面显示
     *
     * @param sale
     */
    private void updateUi(BaseSaleModel sale) {
        this.sale = sale;
        showData(true);
        updateView(sale);
    }

    @Override
    public void querySaleFail(String msg) {
        this.sale = null;
        if (msg != null) {
            showHintDialog(msg);
        }
        showData(false);
    }
}
