package com.hisense.hiaposcmp.activity.std.query;


import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.vip.VipPointsProject;
import com.hisense.crm.vip.VipSex;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpFragment;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.VipLoginPresenter;
import com.hisense.hiaposcmp.view.IView.IVipLoginView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;


/**
 * @author wangyaogang 2020-01-10 HIAPOSCMP-474 查询功能
 */
@ActivityLayoutInject(R.layout.fragment_vip_query)
public class VipQueryFragment extends BaseMvpFragment<IVipLoginView, VipLoginPresenter> implements IVipLoginView {
    public final static String TAG = "VipQueryFragment";
    @BindView(R.id.tv_mid_user_name)
    TextView mTvMidUserName;
    @BindView(R.id.iv_user_gender)
    ImageView mIvUserGender;
    @BindView(R.id.tv_vip_type)
    TextView mTvVipType;
    @BindView(R.id.tv_vip_state)
    TextView mTvVipState;
    @BindView(R.id.tv_vip_card_state)
    TextView mTvVipCardState;
    @BindView(R.id.tv_vip_birth)
    TextView mTvVipBirth;
    @BindView(R.id.tv_vip_phone)
    TextView mTvVipPhone;
    @BindView(R.id.tv_vip_project)
    TextView mTvVipProject;
    @BindView(R.id.tv_vip_points)
    TextView mTvVipPoints;
    @BindView(R.id.tv_user_name)
    TextView mTvUserName;
    @BindView(R.id.tv_user_phone)
    TextView mTvUserPhone;
    @BindView(R.id.tv_right_card_no)
    TextView mTvRightCardNo;
    @BindView(R.id.tv_right_birthday)
    TextView mTvRightBirthday;
    @BindView(R.id.tv_right_card_type)
    TextView mTvRightCardType;
    @BindView(R.id.et_right_input_no)
    EditText mEtRightInputNo;
    @BindView(R.id.edt_pay_keyboard)
    KeyBoardStdView mEdtPayKeyboard;
    @BindView(R.id.ll_card_query)
    LinearLayout mLlCardQuery;

    public VipQueryFragment() {
        // Required empty public constructor
    }

    @Override
    protected void initData() {
        if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
            mEtRightInputNo.setHint("请扫描会员码");
        }
        SoftInput.disable(mEtRightInputNo);
        mEdtPayKeyboard.setOnKeyboardListener(new KeyBoardStdView.OnKeyboardListener() {
            @Override
            public void onInsertKeyEvent(String text) {
                KeyBoardStdView.addChar(mEtRightInputNo, text);
            }

            @Override
            public void onDeleteKeyEvent() {
                KeyBoardStdView.deleteChar(mEtRightInputNo);
            }

            @Override
            public void onClearKeyEvent() {
                KeyBoardStdView.clearChar(mEtRightInputNo);
            }

            @Override
            public void onDoneKeyEvent() {
                query();
            }
        });
        mEtRightInputNo.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                String result = replaceSymbol(s);
                if (result != null) {
                    mEtRightInputNo.setText(result);
                    mEtRightInputNo.setSelection(mEtRightInputNo.length());
                }
            }
        });
    }

    @Override
    protected VipLoginPresenter createPresenter() {
        return new VipLoginPresenter();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return super.onCreateView(inflater, container, savedInstanceState);
    }

    public void query(CardNo cardNo) {
        LoggerUtils.getLogger().debug("会员查询_取得刷卡结果");
        clearData();
        mPresenter.vipQuery(cardNo);
    }

    @Override
    public void vipLoginSuccess(Vip vip) {
        mEtRightInputNo.setText("");
        mTvMidUserName.setText(vip.name());
        // 性别
        if (vip.sex() == VipSex.Female) {
            mIvUserGender.setImageResource(R.drawable.ic_std_woman);
        } else {
            mIvUserGender.setImageResource(R.drawable.ic_std_man);
        }
        mTvVipType.setText(String.format("%s - %s", vip.card().cardType().code(), vip.card().cardType().name()));
        mTvVipState.setText(vip.status().trueName());
        mTvVipCardState.setText(vip.card().status().name());
        mTvVipBirth.setText(vip.birthday());
        mTvVipPhone.setText(vip.mobile());
        if (!ListUtils.isEmpty(vip.projects())) {
            StringBuilder sb = new StringBuilder();
            for (VipPointsProject project : vip.projects()) {
                sb.append(project.name() + "：" + project.usablePoints() + ";");
            }
            mTvVipProject.setText(sb.toString());
        }
        mTvVipPoints.setText(String.format(Locale.CHINESE, "%.2f", vip.card().usablePoints()));

        mTvUserName.setText(vip.name());
        mTvUserPhone.setText(vip.mobile());
        mTvRightBirthday.setText(vip.birthday());
        mTvRightCardNo.setText(vip.card().cardFaceNo());
        mTvRightCardType.setText(String.format("%s - %s", vip.card().cardType().code(), vip.card().cardType().name()));
    }

    @Override
    public void vipLoginFail(String msg) {
        showHintDialog(msg);
        if (getActivity() != null) {
            ((QueryStdActivity) getActivity()).restart();
        }
    }

    private void clearData() {
        mTvMidUserName.setText("");
        mTvVipType.setText("");
        mTvVipState.setText("");
        mTvVipCardState.setText("");
        mTvVipBirth.setText("");
        mTvVipPhone.setText("");
        mTvVipProject.setText("");
        mTvVipPoints.setText("");

        mTvUserName.setText("");
        mTvUserPhone.setText("");
        mTvRightBirthday.setText("");
        mTvRightCardNo.setText("");
        mTvRightCardType.setText("");
    }

    @Override
    public void verifyTypeSuccess(Vip vip) {

    }

    @Override
    public void verifyTypeFail(String msg) {

    }

    @OnClick(R.id.ll_card_query)
    public void onViewClicked() {
        if (isFastClick()) {
            return;
        }
        if (getActivity() != null) {
            ((QueryStdActivity) getActivity()).executeAnimation(mLlCardQuery);
        }
        query();
    }

    private void query() {
        clearData();
        String cardNo = mEtRightInputNo.getText().toString().trim();
        mPresenter.vipQuery(cardNo);
    }
}
