package com.hisense.hiaposcmp.activity.std.refund;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;

import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.std.MainStdActivity;

import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.RefundViewModel;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.refund.RefundPresenter;
import com.hisense.hiaposcmp.service.payment.PayMethodService;

import com.hisense.hiaposcmp.view.AutoFlowLayout;
import com.hisense.hiaposcmp.view.FlowAdapter;
import com.hisense.hiaposcmp.view.IView.refund.IRefundView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;
import com.hisense.hiaposcmp.view.MoneyView;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.webapi.Callback;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

import static com.hisense.hipos.payment.PayMethod.getAllInPayRemark;

/**
 * 标准版退款页  	HIAPOSCMP-487
 *
 * @author xieliyan 2019-01-07 	HIAPOSCMP-487 标准版退款功能开发
 * @author xieliyan 2019-01-07 	HIAPOSCMP-487 屏蔽无用支付方式
 */
@ActivityLayoutInject(R.layout.activity_refund_std)
public class RefundStdActivity extends BaseMvpActivity<IRefundView, RefundPresenter> implements IRefundView {


    /**
     * 会员的图标
     */
    @BindView(R.id.tvStdTitleVipIcon)
    TextView tvStdTitleVipIcon;

    /**
     * 所有支付方式
     */
    @BindView(R.id.afl_pay_method)
    AutoFlowLayout mAflPayMethod;
    /**
     * 已支付列表
     */
    @BindView(R.id.rcv_payed)
    RecyclerView mRcvPayed;
    /**
     * 正中的标题
     */
    @BindView(R.id.tvStdTitleItem)
    TextView mTvStdTitleItem;

    /**
     * 自定义键盘
     */
    @BindView(R.id.edt_pay_keyboard)
    KeyBoardStdView mEdtPayKeyboard;
    /**
     * 标题栏  时间
     */
    @BindView(R.id.tvStdTitleDate)
    TextView tvStdTitleDate;
    /**
     * 标题栏 流水号
     */
    @BindView(R.id.tvStdTitleSaleNo)
    TextView tvStdTitleSaleNo;
    /**
     * 标题栏 门店名称
     */
    @BindView(R.id.tvStdTitleOrgName)
    TextView tvStdTitleOrgName;
    /**
     * 标题栏 营业员名称
     */
    @BindView(R.id.tvStdTitleCashierName)
    TextView tvStdTitleCashierName;
    /**
     * 会员登录(隐藏)
     */
    @BindView(R.id.tvStdTitleVipLogin)
    public TextView tvStdTitleVipLogin;

    /**
     * 会员信息父布局
     */
    @BindView(R.id.linearStdTitleVipInfo)
    public LinearLayout linearStdTitleVipInfo;
    /**
     * 会员名
     */
    @BindView(R.id.tvStdTitleVipName)
    TextView tvStdTitleVipName;
    /**
     * 会员类别
     */
    @BindView(R.id.tvStdTitleVipLevel)
    TextView tvStdTitleVipLevel;
    /**
     * 会员生日
     */
    @BindView(R.id.tvStdTitleVipBirth)
    TextView tvStdTitleVipBirth;
    /**
     * 积分
     */
    @BindView(R.id.tvStdTitlePoints)
    TextView tvStdTitlePoints;
    /**
     * 余额
     */
    @BindView(R.id.tvStdTitleBalance)
    TextView tvStdTitleBalance;
    /**
     * 切换会员(隐藏)
     */
    @BindView(R.id.tvStdTitleVipChange)
    TextView tvStdTitleVipChange;
    /**
     * 右侧 返回交易
     */
    @BindView(R.id.llRefundBack)
    LinearLayout mLlRefundBack;
    /**
     * 应退金额
     */
    @BindView(R.id.tvShouldRefund)
    TextView mTvShouldRefund;
    /**
     * 合计金额
     */
    @BindView(R.id.tvTotalAmount)
    TextView mTvTotalAmount;
    /**
     * 已退金额
     */
    @BindView(R.id.tvPaidAmount)
    TextView mTvPaidAmount;
    @BindView(R.id.linearStdTitleHome)
    LinearLayout mLinearStdTitleHome;
    /**
     * 底部撤销支付
     */
    @BindView(R.id.llyRefund)
    LinearLayout mLlyRefund;

    @BindView(R.id.txtRefundAmount)
    MoneyView mTxtRefundAmount;

    private Context mContext;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        LoggerUtils.getLogger().debug("----------onCreate-----------PayStdActivity----------");
        //记录当前处于支付过程中
        mContext = this;

        init();
        initPresenter();
        initPayMethod();

        initValue();
    }

    private final BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (action != null && action.equals(Intent.ACTION_TIME_TICK)) {
                tvStdTitleDate.setText(DateUtils.dateFormatWithWeek("MM月dd日    ww   aa hh:mm"));
            }
        }
    };

    private void initValue() {
        IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_TIME_TICK);
        registerReceiver(receiver, filter);

        tvStdTitleVipLogin.setVisibility(View.GONE);
        tvStdTitleVipChange.setVisibility(View.GONE);
        tvStdTitleDate.setText(DateUtils.dateFormatWithWeek("MM月dd日    ww   aa hh:mm"));
        tvStdTitleOrgName.setText(Config.org_name);
        tvStdTitleCashierName.setText(Config.user_name);
        tvStdTitleSaleNo.setText(mPresenter.getSale().getSaleNo());
        updateVipInfo(mPresenter.getReturnSale().getVip());


    }

    private void updateVipInfo(Vip vip) {
        if (vip == null) {
            tvStdTitleVipIcon.setVisibility(View.GONE);
            return;
        }
        linearStdTitleVipInfo.setVisibility(View.VISIBLE);

        tvStdTitleVipName.setText(vip.name());
        tvStdTitleVipLevel.setText(vip.card().cardType().name());
        tvStdTitleVipBirth.setText(vip.birthday());

        String points = "0";
        String balance = "0";
        if (vip.projects().size() > 0) {
            points = BigDecimalUtils.decimalFormat(vip.projects().get(0).usablePoints());
            balance = BigDecimalUtils.decimalFormat(vip.projects().get(0).usableBalance());

        } else if (vip.card() != null && vip.card().usablePoints() > 0) {
            points = BigDecimalUtils.decimalFormat(vip.card().usablePoints());
            balance = BigDecimalUtils.decimalFormat(vip.card().usableBalance());

        }

        tvStdTitlePoints.setText(points);
        tvStdTitleBalance.setText(balance);
    }

    private void initPresenter() {
        //初始化赢商通配置
        mPresenter.initYstConfig();
        //初始化米雅配置
        mPresenter.initMiYaConfig();
        //初始化ReturnViewModel
        mPresenter.initPayments();
        //退款后刷新页面
        mPresenter.refreshAfterRefund(null);

        //初始化数据库服务
        mPresenter.initSaleRepository();
        //清空优惠和积分
        mPresenter.clearPoints();
    }

    private void init() {
        isNotShowExitDialog(true);
        mLinearStdTitleHome.setVisibility(View.INVISIBLE);
        mTvStdTitleItem.setText("退款");

        //右侧金额输入框,屏蔽系统键盘,使用自定义
        SoftInput.disable(mTxtRefundAmount);
        mEdtPayKeyboard.setOnKeyboardListener(new KeyBoardStdView.OnKeyboardListener() {
            @Override
            public void onInsertKeyEvent(String text) {
                KeyBoardStdView.addChar(mTxtRefundAmount, text);
            }

            @Override
            public void onDeleteKeyEvent() {
                KeyBoardStdView.deleteChar(mTxtRefundAmount);
            }

            @Override
            public void onClearKeyEvent() {
                KeyBoardStdView.clearChar(mTxtRefundAmount);
            }

            @Override
            public void onDoneKeyEvent() {

            }
        });

        mTxtRefundAmount.setText(mTvShouldRefund.getText().toString());


    }


    /**
     * 初始化 所有支付方式
     */
    private void initPayMethod() {
        showLoadingDialog();
        PayMethodService.getAll(mContext, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                List<PayMethod> supportedMethods=new ArrayList<>();
                for(PayMethod payMethod:payMethods){
                    if(payMethod.isCash()||payMethod.isWxPay()||payMethod.isAliPay()||payMethod.isCard()||payMethod.isVipPoints()||payMethod.isScrmRemain()||payMethod.isScrmVipPoints()){
                        supportedMethods.add(payMethod);
                    }
                }

                mAflPayMethod.setAdapter(new FlowAdapter(supportedMethods) {
                    @Override
                    public View getView(int position) {
                        PayMethod method = supportedMethods.get(position);
                        View item = LayoutInflater.from(mContext).inflate(R.layout.item_std_pay_method, null);
                        TextView tvPaymethod = item.findViewById(R.id.tv_paymethod);
                        tvPaymethod.setText(method.name());
                        ImageView ivPaymethod = item.findViewById(R.id.iv_paymethod);
                        ivPaymethod.setImageDrawable(ContextCompat.getDrawable(mContext, PayMethodService.getIconByCode(method)));
                        return item;
                    }
                });
                mAflPayMethod.setOnItemClickListener(new AutoFlowLayout.OnItemClickListener() {
                    @Override
                    public void onItemClick(int position, View view) {

                        mAflPayMethod.select();
                        PayMethod method = supportedMethods.get(position);
                        LoggerUtils.getLogger().debug("Pay支付方式:" + method);
                        PayMethodService.updateFrequency(mContext, method);

                        mPresenter.refundStd(method);
                    }
                });

                for (SalePayModel payment : getReturnSale().getSaleModel().getPays()) {
                    for (PayMethod method : supportedMethods) {
                        if (payment.getPayCode().equals(method.code())
                                || (PayMethod.YI_DIAN_TONG.equalsIgnoreCase(getAllInPayRemark(payment.getRemark())) &&
                                PayMethod.YI_DIAN_TONG.equalsIgnoreCase(getAllInPayRemark(method.remark())))) {
                            mPresenter.getListPayments().add(new RefundViewModel(payment));
                            payment.setPayType(method.payType());
                            payment.setRemark(method.remark());

                            break;
                        }
                    }
                }
                setAdapter();
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
            }
        });

    }


    @Override
    protected RefundPresenter createPresenter() {
        return new RefundPresenter();
    }

    @Override
    public void setAdapter() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(this);
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        mRcvPayed.setLayoutManager(layoutManager);
        mRcvPayed.setAdapter(mPresenter.stdAdapter());

    }

    @Override
    public View.OnClickListener paymentClick() {
        return null;
    }

    @Override
    public void setRefundingText(String refunding) {
        mTxtRefundAmount.setText(refunding);
        mTxtRefundAmount.setSelection(refundTextValue().length());
    }

    @Override
    public String refundTextValue() {
        return mTxtRefundAmount.getText().toString().trim();
    }

    /**
     * 返回监听
     */
    @OnClick(R.id.llRefundBack)
    public void onClickBack() {
        if (mainSaleModel.getReturnSaleModel().getPays().size() > 0) {
            showHintDialog("已经进行了退货，不能返回");
        } else {
            onBackPressed();
        }
    }

    @Override
    public void goToOtherRefund() {

    }

    @Override
    public void goToCouponRefund(int requestCode, String maxRefund) {

    }

    @Override
    public void refreshTotal() {
        try {

            mTvShouldRefund.setText(getReturnSale().getTotalUnPaid());// 未退
            mTvPaidAmount.setText(getReturnSale().getTotalPaid());    // 已退
            mTvTotalAmount.setText(getReturnSale().getTotalRecv());
            setRefundingText(getReturnSale().getTotalUnPaid());
//            String maxRefund = mPresenter.getMaxRefund(mPresenter.selectedPayMethod());
//            String refund = BigDecimalUtils.ternaryOperatorLess(String.valueOf(maxRefund), getReturnSale().getTotalUnPaid());
//            setRefundingText(refund);//退款
//            UpdateTextButton.setOnTextChange(mTxtRefundAmount, refund);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("更新金额异常" + e.toString());
        }
    }

    @Override
    public void refreshAfterRefund() {
        mPresenter.getStdAdapter().notifyDataSetChanged();
        refreshTotal();
        if (BigDecimalUtils.isEquals(getReturnSale().getTotalRecv(), getReturnSale().getTotalPaid())) {
            mTxtRefundAmount.setEnabled(false);
//            mBtnFinish.setBackgroundResource(R.color.color_amount);
//            mBtnFinish.setText("完成");
//            mBtnFinish.setEnabled(false);

            complete();
        }


    }

    @Override
    public void openCamera() {

    }

    @Override
    public IPaymentProvider getPaymentProvider() {
        return getPaymentProvider(this, mPresenter.selectedPayMethod().remark());
    }

    @Override
    public void goToUnionRefundConfirm(double amount, String traceNo, String refNo) {

    }

    @Override
    public void goToMainActivity() {
        Intent intent = new Intent(RefundStdActivity.this, MainStdActivity.class);
        startActivity(intent);
        finish();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == ActivityConstants.REQUEST_INTE_PAY_INPUT && resultCode == ActivityConstants.RESULT_NTE_PAY_SUCCESS) {
            if (data != null) {

                String inputMoney = data.getStringExtra("inputMoney");
                String outTradeNo = data.getStringExtra("payNo");
                LoggerUtils.getLogger().debug("标准版-聚合支付成功返回:" + inputMoney + "||" + outTradeNo);
                mPresenter.oldSalePaymentStd(outTradeNo);
                if (null == mPresenter.oldSalePayment()) {
                    showHintDialog("未找到该笔交易，请确认商户订单号是否正确");
                    return;
                }
                mPresenter.payRefundStd(outTradeNo, inputMoney);

            }
        }else if ((requestCode == ActivityConstants.REQUEST_PAY_TO_CARD && resultCode == ActivityConstants.RESULT_REFUND_TO_CARD_SUCCESS) ||
                (requestCode == ActivityConstants.REQUEST_PAY_TO_POINTS && resultCode == ActivityConstants.RESULT_REFUND_TO_POINTS_SUCCESS)) {
            mPresenter.oldSalePayment(null);
            SalePayModel salePayModel = getReturnSale().getLastPay();
            getReturnSale().removeLastPay();
            mPresenter.refreshAfterRefund(salePayModel);
        }
    }

    /**
     * 退款完成，上传退货流水。
     */
    private void complete() {


        if (BigDecimalUtils.isEquals(mTvPaidAmount.getText().toString(), mTvTotalAmount.getText().toString())) {
            if (mPresenter.hasUploaded()) {
                LoggerUtils.getLogger().debug("已上传退货流水");
                goToMainActivity();
                return;
            }
            LoggerUtils.getLogger().debug("准备上传退货流水");
            mPresenter.hasUploaded(true);

            mPresenter.uploadSale();
            // mBtnFinish.setEnabled(true);


        } else {
            //  mBtnFinish.setEnabled(false);
        }
    }

    @Override
    protected void onStart() {
        LoggerUtils.getLogger().debug("----------onStart-----------PayStdActivity----------");
        super.onStart();
    }

    @Override
    protected void onResume() {
        LoggerUtils.getLogger().debug("----------onResume-----------PayStdActivity----------");
        super.onResume();
    }

    @Override
    protected void onPause() {
        LoggerUtils.getLogger().debug("----------onPause-----------PayStdActivity----------");
        super.onPause();
    }

    @Override
    protected void onStop() {
        LoggerUtils.getLogger().debug("----------onStop-----------PayStdActivity----------");
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        LoggerUtils.getLogger().debug("----------onDestroy-----------PayStdActivity----------");

        super.onDestroy();
    }
}
