package com.hisense.hiaposcmp.activity.wx;


import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.Card;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.pay.CardPayPresenter;
import com.hisense.hiaposcmp.view.IView.ICardPayView;
import com.hisense.hiaposcmp.view.KeyBoardWxViewV2;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 动态码支付
 *
 * @author xieliyan 2019-03-13
 * @author dukunbiao(null)  2019-07-25  重构
 */
@ActivityLayoutInject(R.layout.activity_wechat_card_pay)
public class CardPayWxActivity extends BaseMvpActivity<ICardPayView, CardPayPresenter> implements ICardPayView, KeyBoardWxViewV2.IOnKeyboardListener {

    @BindView(R.id.imv_viplogin_cancel)
    public ImageView imvBack;

    @BindView(R.id.edt_viplogin_input)
    public EditText edtTextInput;

    @BindView(R.id.imv_viplogin_textdelete)
    public ImageView imvTextDel;

    @BindView(R.id.tv_viplogin_prompt)
    public TextView tvVipPrompt;

    @BindView(R.id.keyboard_wechat_viplogin)
    public KeyBoardWxViewV2 mKeyBroad;

    @BindView(R.id.btn_viplogin_enter)
    public Button btnEnter;

    private String mVipNum = "";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initListener();
        initBtn();
    }

    private void initListener() {
        edtTextInput.setOnKeyListener((v, keyCode, event) -> {
            if (keyCode == KeyEvent.KEYCODE_ENTER && event.getAction() == KeyEvent.ACTION_DOWN) {
                mPresenter.cardQuery(edtTextInput.getText().toString());
            }

            return false;
        });
    }

    @Override
    protected CardPayPresenter createPresenter() {
        return new CardPayPresenter();
    }

    private void init() {
        mKeyBroad.setIOnKeyboardListener(this);

        edtTextInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                if (StringUtils.isEmpty(s.toString())) {
                    edtTextInput.setFocusable(true);
                    edtTextInput.requestFocus();
                }
                mVipNum = s.toString();
                initBtn();
            }
        });
        edtTextInput.setInputType(InputType.TYPE_NULL);

        mPresenter.initSelectedPayMethod(this.getIntent());
    }

    private void initBtn() {
        if (StringUtils.isEmpty(mVipNum)) {
            imvTextDel.setVisibility(View.INVISIBLE);
            btnEnter.setBackgroundResource(R.drawable.button_wx_login_unpress);
            tvVipPrompt.setVisibility(View.INVISIBLE);
        } else {
            imvTextDel.setVisibility(View.VISIBLE);
            btnEnter.setBackgroundResource(R.drawable.button_wx_login);
        }
    }

    @Override
    public void onInsertKeyEvent(String text) {
        edtTextInput.append(text);
    }

    @Override
    public void onDeleteKeyEvent() {
        String inputStr = edtTextInput.getText().toString().trim();
        if (inputStr.length() >= 1) {
            edtTextInput.setText(inputStr.substring(0, inputStr.length() - 1));
        }
    }

    @OnClick(R.id.imv_viplogin_cancel)
    public void onClickCancel() {
        if (isFastClick()) {
            return;
        }

        setResult(ActivityConstants.RESULT_PAY_TO_CARD_CANCEL);
        finish();
    }

    @OnClick(R.id.imv_viplogin_textdelete)
    public void onClickDelete() {
        edtTextInput.setText("");
    }

    @OnClick(R.id.btn_viplogin_enter)
    public void onClickEnter() {
        if (isFastClick()) {
            return;
        }

        mPresenter.cardQuery(edtTextInput.getText().toString().trim());
    }

    @Override
    public void needVerifyType(boolean iseCard, VerifyType verifyType) {

    }

    @Override
    public void cardQuerySuccess(Card card) {
        mPresenter.cardPay(card);
    }

    @Override
    public void cardPaySuccess() {
        setResult(ActivityConstants.RESULT_PAY_TO_CARD_SUCCESS);
        finish();
    }

    @Override
    protected void onResume() {
        super.onResume();
        edtTextInput.setText("");
    }

    @Override
    public void cardQueryRefund(Card card, String fSsTotal, String maxRefund) {

    }

    @Override
    public void cardRefundSuccess() {

    }

    @Override
    public void queryVerifyTypeFail(Throwable throwable) {
    }
}
