package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import androidx.annotation.Nullable;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.pay.CouponPresenter;
import com.hisense.hiaposcmp.view.IView.ICouponView;
import com.hisense.hiaposcmp.view.KeyBoardWxViewV2;
import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.UsableCoupon;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 微信商品条码输入界面
 *
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-25  重构
 */
@ActivityLayoutInject(R.layout.activity_wechat_coupon)
public class CouponUseWxActivity extends BaseMvpActivity<ICouponView, CouponPresenter> implements ICouponView, KeyBoardWxViewV2.IOnKeyboardListener {

    @BindView(R.id.imb_codeinput_cancel)
    public ImageView btnBack;

    @BindView(R.id.edt_codeinput_input)
    public EditText edtCodeInput;

    @BindView(R.id.edt_check_input_input)
    public EditText edtCheckCodeInput;

    @BindView(R.id.imv_codeinput_textdelete)
    public ImageView imvCodeDel;

    @BindView(R.id.imv_check_textdelete)
    public ImageView imvCheckCodeDel;

    @BindView(R.id.keyboar_wechat_codeinput)
    public KeyBoardWxViewV2 mKeybroad;

    @BindView(R.id.btn_codeinput_enter)
    public Button btnCodeEnter;

    private EditText mCurrentEdtView;
    private ImageView mCurrentImgDelete;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initBtn();
    }

    @Override
    protected CouponPresenter createPresenter() {
        return new CouponPresenter();
    }

    private void init() {
        mKeybroad.setIOnKeyboardListener(this);
        edtCodeInput.setOnFocusChangeListener(mFocusChangedListener);
        edtCheckCodeInput.setOnFocusChangeListener(mFocusChangedListener);
        mCurrentEdtView = edtCodeInput;
        mCurrentImgDelete = imvCheckCodeDel;

        edtCodeInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                initTextDelBtn(edtCodeInput, imvCodeDel);
            }
        });
        edtCheckCodeInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                initTextDelBtn(edtCheckCodeInput, imvCheckCodeDel);
            }
        });

        edtCodeInput.setInputType(InputType.TYPE_NULL);
        edtCheckCodeInput.setInputType(InputType.TYPE_NULL);

        mPresenter.initSelectedPayMethod(this.getIntent());
    }

    private void initTextDelBtn(EditText mCurrentEdtView, ImageView mCurrentImgDelete) {
        if (mCurrentEdtView.getText().toString().trim().equals("")) {
            mCurrentImgDelete.setVisibility(View.INVISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login_unpress);
        } else {
            mCurrentImgDelete.setVisibility(View.VISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login);
        }
    }

    private void initBtn() {
        if (StringUtils.isEmpty(edtCodeInput.getText().toString())) {
            imvCodeDel.setVisibility(View.INVISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login_unpress);
        } else {
            imvCodeDel.setVisibility(View.VISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login);
        }

        if (StringUtils.isEmpty(edtCheckCodeInput.getText().toString())) {
            imvCheckCodeDel.setVisibility(View.INVISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login_unpress);
        } else {
            imvCheckCodeDel.setVisibility(View.VISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login);
        }
    }

    @OnClick(R.id.imb_codeinput_cancel)
    public void onClickCancel() {
        setResult(ActivityConstants.RESULT_PAY_TO_COUPON_CANCEL);
        finish();
    }

    @OnClick(R.id.imv_codeinput_textdelete)
    public void onClickCodeDelete() {
        edtCodeInput.setText("");
    }

    @OnClick(R.id.imv_check_textdelete)
    public void onClickCheckDelete() {
        edtCheckCodeInput.setText("");
    }

    @OnClick(R.id.btn_codeinput_enter)
    public void onClickEnter() {
        if (isFastClick()) {
            return;
        }

        mPresenter.couponCalc(edtCodeInput.getText().toString(), edtCheckCodeInput.getText().toString());
    }

    @Override
    public void onInsertKeyEvent(String text) {
        mCurrentEdtView.append(text);
    }

    @Override
    public void onDeleteKeyEvent() {
        String inputStr = mCurrentEdtView.getText().toString().trim();
        if (inputStr.length() >= 1) {
            mCurrentEdtView.setText(inputStr.substring(0, inputStr.length() - 1));
        }
    }

    private View.OnFocusChangeListener mFocusChangedListener = new View.OnFocusChangeListener() {
        @Override
        public void onFocusChange(View view, boolean hasFocus) {
            if (hasFocus) {
                mCurrentEdtView = (EditText) view;
                if (mCurrentEdtView.equals(edtCodeInput)) {
                    mCurrentImgDelete = imvCodeDel;
                } else if (mCurrentEdtView.equals(edtCheckCodeInput)) {
                    mCurrentImgDelete = imvCheckCodeDel;
                }
            }
        }
    };

    @Override
    public void couponCalcSuccess(int position, UsableCoupon usableCoupon) {
        mPresenter.couponUse1(usableCoupon);
    }

    @Override
    public void needVerifyType(VerifyType verifyType) {

    }

    @Override
    public void queryVipCouponsSuccess(List<Coupon> coupons) {

    }

    @Override
    public void couponUse1Success(Collection<UsableCoupon> usableCoupons, Collection<Coupon> coupons) {
        Intent intent = new Intent();
        intent.putExtra("usableCoupon", (Serializable) usableCoupons);

        setResult(ActivityConstants.RESULT_PAY_TO_COUPON_SUCCESS, intent);
        finish();
    }

    @Override
    public void calcCouponsSuccess() {

    }

    @Override
    protected void onResume() {
        super.onResume();
        edtCodeInput.setText("");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
