package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.adapter.SaleProAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.boot.FinishActivityManager;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.pay.WxConstants;
import com.hisense.hiaposcmp.presenter.pay.PayPresenter;
import com.hisense.hiaposcmp.print.printer.impl.LocalPOSPrinter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.pay.IPayView;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.pay.PayUi;
import com.hisense.hipos.payment.PayMethod;

import java.util.Collection;

import butterknife.BindView;
import butterknife.OnClick;
import jp.wasabeef.recyclerview.animators.SlideInRightAnimator;

/**
 * 支付界面（自定义：目前用于永辉和万科）
 *
 * @author xieliyan 2018-12-05
 * @author xieliyan 2018-12-07 支持万科业务逻辑
 * @author dukunbiao(null)  2019-07-25  重构
 * @author xieliyan 2020-01-13 控制多次点击
 */
@ActivityLayoutInject(R.layout.activity_pay_custom)
public class PayCustomActivity extends BaseMvpActivity<IPayView, PayPresenter> implements IPayView, AdapterView.OnItemClickListener {

    @BindView(R.id.list_sale_pro)
    public RecyclerView mRecyclerView;

    @BindView(R.id.txt_pro_total)
    public TextView txtProTotal;

    @BindView(R.id.gvw_often)
    public GridView mGridViewOften;

    @BindView(R.id.gvw_other)
    public GridView mGridViewOther;

    @BindView(R.id.imgWxPayExit)
    public ImageView mImageExit;

    @BindView(R.id.unPayTotal)
    public TextView txtUnPayTotal;

    @BindView(R.id.total)
    public TextView txtTotal;

    private SaleProAdapter mSaleProAdapter;

    private static boolean isClickPay = false;
    public static String WxFacePayVersion="2.13";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        FinishActivityManager.addActivity(this);
        LoggerUtils.getLogger().debug("支付界面被创建");
        WxFacePayVersion=getWxFacePayVersion();
        LoggerUtils.getLogger().debug("微信人脸版本号"+WxFacePayVersion);
        init();
        initPresenter();
        initAdapter();
        initValue();
    }

    @Override
    protected PayPresenter createPresenter() {
        return new PayPresenter();
    }

    private void init() {
        mGridViewOften.setOnItemClickListener(this);
        mGridViewOther.setOnItemClickListener(this);
    }

    private void initPresenter() {
        mPresenter.initPayMethodList();
        mPresenter.getPayMethod(mGridViewOften, mGridViewOther);
        mPresenter.assignAdapter(mSaleProAdapter);
        mPresenter.updateDscUi();
        mPresenter.initSaleService();
    }

    private void initAdapter() {
        mRecyclerView.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        mSaleProAdapter = new SaleProAdapter(mRecyclerView);
        mRecyclerView.setAdapter(mSaleProAdapter);

        mRecyclerView.setItemAnimator(new SlideInRightAnimator());
        mRecyclerView.getItemAnimator().setAddDuration(300);
        mRecyclerView.getItemAnimator().setRemoveDuration(50);
        mRecyclerView.getItemAnimator().setMoveDuration(50);
        mRecyclerView.getItemAnimator().setChangeDuration(50);
    }

    private void initValue() {
        LocalPOSPrinter.isPrint = true;

        updateTotal();
    }

    @Override
    public void whenPayDone() {
        setResult(ActivityConstants.RESULT_SALE_TO_PAY);
        FinishActivityManager.finishActivity();
    }

    @Override
    public void gotoScanActivity() {
        if (BuildConfig.DEBUG) {
            mPresenter.cashPay(mPresenter.getSale().getTotalUnPaid());
            return;
        }

        LoggerUtils.getLogger().debug("支付金额" + mPresenter.getSale().getTotalUnPaid());

        Bundle bundle = new Bundle();
        bundle.putString("totalPaid", mPresenter.getSale().getTotalUnPaid());

        Intent intent = new Intent();
        intent.setClass(this, PayScanWxActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SCAN);
    }

    @Override
    public void afterPaySuccess(Bundle bundle) {
        Intent intent = new Intent(this, PaySuccessWxActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SUCCESS);
    }

    @Override
    public void afterPayFailure(String msg) {

    }

    @OnClick(R.id.imgWxPayExit)
    public void onClickCancelPay() {
        if (ListUtils.isEmpty(mPresenter.getSale().getPays())) {
            mPresenter.cancelPayFinish();
        } else {
            mPresenter.cancelPay();
        }
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        if (isClickPay) {
            LoggerUtils.getLogger().debug("已点击一次支付方式" + isClickPay);
            return;
        }
        if (isFastClick()) {
            LoggerUtils.getLogger().debug("支付方式快速点击");
            return;
        }
        isClickPay = true;
        PayUi payUi = null;
        if (parent.getId() == R.id.gvw_often) {
            payUi = mPresenter.getPayUiOften().get(position);
        } else if (parent.getId() == R.id.gvw_other) {
            payUi = mPresenter.getPayUiOther().get(position);
        }

        if (payUi == null) {
            CommonTools.toastShort("暂不支持该支付方式");
            return;
        }

        LoggerUtils.getLogger().debug("支付方式:" + payUi.getPayRemark());
        mPresenter.updateTotal();
        PayMethod payMethod = new PayMethod(payUi.getPayCode(), payUi.getPayName(), payUi.getPayRemark(), payUi.getPayRemark());
        mPresenter.pay(payMethod);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        //吉麦隆市场问题新增注释
        LoggerUtils.getLogger().debug("回调onActivityResult");
        super.onActivityResult(requestCode, resultCode, data);
        isClickPay = false;
        hideLoadingDialog();
        if (requestCode == ActivityConstants.REQUEST_PAY_TO_SCAN && resultCode == ActivityConstants.RESULT_PAY_TO_SCAN) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入REQUEST_PAY_TO_SCAN");
            payPay(data);
        } else if (requestCode == ActivityConstants.REQUEST_PAY_TO_SUCCESS && resultCode == ActivityConstants.RESULT_PAY_TO_SUCCESS) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入RESULT_PAY_TO_SUCCESS");
            saleSubmit();
        } else if (requestCode == ActivityConstants.REQUEST_PAY_TO_COUPON && resultCode == ActivityConstants.RESULT_PAY_TO_COUPON_SUCCESS) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入RESULT_PAY_TO_COUPON_SUCCESS");
            couponUseSuccess(data);
        } else if (requestCode == ActivityConstants.REQUEST_PAY_TO_CARD && resultCode == ActivityConstants.RESULT_PAY_TO_CARD_SUCCESS) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入RESULT_PAY_TO_CARD_SUCCESS");
            mPresenter.paySuccess();
        }
    }

    private void saleSubmit() {
        mPresenter.submit();

    }

    private void payPay(Intent data) {
        //处理微信扫码界面得到条码返回
        Bundle bundle = data.getExtras();

        if (bundle != null && bundle.containsKey("barcode")) {
            mPresenter.payPay(bundle.getString("barcode"), null, null, WxConstants.wx_pay_store_name());
        } else {
            showHintDialog("支付失败", "获取支付条码失败");
        }
    }

    private void couponUseSuccess(Intent data) {
        Bundle bundle = data.getExtras();
        if (bundle == null) {
            return;
        }
        Collection<UsableCoupon> usableCoupons = (Collection<UsableCoupon>) bundle.getSerializable("usableCoupon");
        if (usableCoupons == null || usableCoupons.size() == 0) {
            return;
        }

        mPresenter.paySuccess();
    }

    @Override
    public void gotoCouponUseActivity() {
        Bundle bundle = new Bundle();
        bundle.putSerializable("payMethod", mPresenter.selectedPayMethod());

        Intent intent = new Intent(this, CouponUseWxActivity.class);
        intent.putExtras(bundle);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_COUPON);
    }

    @Override
    public void gotoScrmCouponActivity() {

    }

    @Override
    public void gotoCardPayActivity() {
        Bundle bundle = new Bundle();
        bundle.putSerializable("payMethod", mPresenter.selectedPayMethod());

        Intent intent = new Intent(this, CardPayWxActivity.class);
        intent.putExtras(bundle);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_CARD);
    }

    @Override
    public void gotoPointsPayActivity() {

    }

    @Override
    public void gotoOtherPayActivity() {

    }

    @Override
    public void gotoPayChannel(String totalPaid) {

    }

    @Override
    public void updateTotal() {
        LoggerUtils.getLogger().debug("saleModel:" + mPresenter.getSale());

        txtUnPayTotal.setText(mPresenter.getSale().getTotalUnPaid());
        txtTotal.setText(mPresenter.getSale().getTotalPaid());
        txtProTotal.setText(mPresenter.getSale().getTotalDsc());
    }

    @Override
    public void paySuccess(SalePayModel payModel) {

    }

    @Override
    public void gotoCancelPayChannel(SalePayModel payModel) {

    }

    @Override
    public void cancelPaySuccess(SalePayModel payModel) {

    }

    @Override
    public void cancelPayError() {

    }

    @Override
    public void cancelPayFinish() {
        finishThisActivity();
    }

    @Override
    protected void onPause() {
        super.onPause();
        LoggerUtils.getLogger().debug("支付界面被暂停");
    }

    @Override
    protected void onStop() {
        super.onStop();
        LoggerUtils.getLogger().debug("支付界面被停止");
    }

    @Override
    protected void onResume() {
        isClickPay = false;
        super.onResume();
        LoggerUtils.getLogger().debug("支付界面被恢复");
    }

    @Override
    protected void onDestroy() {
        mPresenter.quit();
        super.onDestroy();
        LoggerUtils.getLogger().debug("支付界面被销毁");
    }

    private String getWxFacePayVersion() {
        String packageName = "com.tencent.wxpayface";
        try {
            PackageInfo packInfo = getPackageManager().getPackageInfo(packageName, 0);
            return packInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("获取微信人脸版本号异常"+e.toString());
        }
        return "";
    }
    @Override
    public void setIsClickPayFalse() {
        isClickPay = false;
    }

}
