package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.CommonConstants;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.presenter.pay.PaySuccessPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.util.IFlyTtsUtil;
import com.hisense.hiaposcmp.view.IView.pay.IPaySuccessView;
import com.hisense.hipos.model.SalePayModel;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * @author liaoyuping on    2018/7/5.
 * @author dukunbiao(null)  2019-07-25  重构
 * @author xieliyan         2020-01-02 HIAPOSCMP-467 打印小票
 * @author xieliyan         2020-01-02 HIAPOSCMP-313 支付完成语音播报
 */
@ActivityLayoutInject(R.layout.activity_pay_success_wx)
public class PaySuccessWxActivity extends BaseMvpActivity<IPaySuccessView, PaySuccessPresenter> implements IPaySuccessView {

    @BindView(R.id.txtPayCost)
    public TextView mTxtPayCost;

    @BindView(R.id.txtCost)
    public TextView mTxtCost;

    @BindView(R.id.txtDis)
    public TextView mTxtDis;

    @BindView(R.id.txtJF)
    public TextView mTxtJF;

    @BindView(R.id.lytHome)
    public RelativeLayout mLytHome;

    @BindView(R.id.txtLastTime)
    public TextView mTxtLastTime;

    @BindView(R.id.lytCurrPoint)
    public LinearLayout lytCurrPoint;

    @BindView(R.id.img_logo)
    public ImageView imgLogo;


    private String amount = "";
    private String count = "";
    private String payName = "";


    @Override
    protected PaySuccessPresenter createPresenter() {
        return new PaySuccessPresenter();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        IFlyTtsUtil.init(App.getAppContext());
        IFlyTtsUtil.setParams();
        LoggerUtils.getLogger().debug("支付成功界面被创建");
        //保存本地流水
        mPresenter.uSaveData();
        mPresenter.print();
        init();
        initLogo();
        initCountDown();
        //暂无无人店项目，屏蔽开门
        //openDoor();
        IFlyTtsUtil.start(payName + "支付成功，支付" + amount + "元，" + count + "请拿好小票，欢迎下次购物");
        initCountDown();
    }

    private void init() {
        mTxtPayCost.setText("");
        mTxtCost.setText("");
        mTxtDis.setText("");
        mTxtJF.setText("");
        mTxtLastTime.setText(CommonConstants.PAY_SUCCESS_COUNT_DOWN_INTERVAL / 1000 + "s");

        Intent intent = this.getIntent();
        Bundle bundle = intent.getExtras();
        if (bundle == null) {
            CommonTools.toastLong(this, "获取支付信息失败");
            return;
        }

        SalePayModel payModel = (SalePayModel) bundle.getSerializable("payModel");

        mTxtPayCost.setText("￥" + payModel.getTotalPaid());
        mTxtCost.setText("￥" + payModel.getTotal());
        mTxtDis.setText("￥" + BigDecimalUtils.add(payModel.getTotalDscChannel(), payModel.getTotalDscSelf()));
        amount = payModel.getTotalPaid();
        count = "共" + mPresenter.getSale().calcCount() + "件商品，";
        payName = payModel.getPayName();
    }

    private void initLogo() {
        try {
            if (LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH) == null) {
                imgLogo.setImageDrawable(getResources().getDrawable(R.drawable.hisense_wechat));
            } else {
                imgLogo.setImageBitmap(LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH));
            }
        } catch (Exception e) {
            imgLogo.setImageDrawable(getResources().getDrawable(R.drawable.hisense_wechat));
            LoggerUtils.getLogger().error("加载图片异常:" + e.toString());
        }
    }

    private void initCountDown() {
        new CountDownTimer(CommonConstants.PAY_SUCCESS_MILLIS_IN_FUTURE, CommonConstants.PAY_SUCCESS_COUNT_DOWN_INTERVAL) {
            @Override
            public void onTick(long millisUntilFinished) {
                mTxtLastTime.setText(millisUntilFinished / 1000 + "s");
            }

            @Override
            public void onFinish() {
                finishSelf();
            }
        }.start();
    }

    @OnClick(R.id.lytHome)
    public void onClickHome() {
        finishSelf();
    }

    protected void finishSelf() {
        Intent intent = new Intent();
        LoggerUtils.getLogger().debug("支付成功界面倒计时结束");
        PaySuccessWxActivity.this.setResult(ActivityConstants.RESULT_PAY_TO_SUCCESS, intent);
        LoggerUtils.getLogger().debug("支付成功界面执行setResult");
        PaySuccessWxActivity.this.finish();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        LoggerUtils.getLogger().debug("支付成功界面被销毁");
        IFlyTtsUtil.destroy();
        LoadSDCard.releaseImageViewResouce(imgLogo);
    }
}
