package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.pay.WxConstants;
import com.hisense.hiaposcmp.presenter.pay.PayPresenter;
import com.hisense.hiaposcmp.service.payment.PayMethodService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.HomeWatcher;
import com.hisense.hiaposcmp.view.IView.pay.IPayView;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.webapi.Callback;

import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 支付
 *
 * @author liaoyuping on    2018-07-03
 * @author dukunbiao(null)  2018-07-16  增加微信支付业务
 * @author xieliyan         2018-08-16  增加称重防损
 * @author dukunbiao(null)  2019-07-24  重构
 */
@ActivityLayoutInject(R.layout.activity_pay_wx)
public class PayWxActivity extends BaseMvpActivity<IPayView, PayPresenter> implements IPayView {

    @BindView(R.id.imgWxPayExit)
    public ImageView mImvClose;

    @BindView(R.id.linFacePay)
    public LinearLayout linFacePay;

    @BindView(R.id.linQRPay)
    public LinearLayout linQRPay;

    private HomeWatcher mHomeWatcher;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        PayMethodService.createWxLayout(this, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                for (int i = 0; i < payMethods.size(); i++) {
                    PayMethod payMethod = payMethods.get(i);
                    if (payMethod.isWxPay()) {
                        linQRPay.setTag(payMethod);
                    }
                    if (payMethod.isWxFacePay()) {
                        linFacePay.setTag(payMethod);
                    }
                }
            }

            @Override
            public void onError(Throwable throwable) {

            }
        });

        init();
        mPresenter.initSaleService();
    }

    private void init() {
        mHomeWatcher = new HomeWatcher(this);
        mHomeWatcher.setOnHomePressedListener(new HomeWatcher.OnHomePressedListener() {
            @Override
            public void onHomePressed() {
                if (mPresenter.getSale().paySize() > 0) {
                    CommonTools.toastLong(PayWxActivity.this, "已经进行了支付，不能返回");
                    return;
                }

                mPresenter.cancelPayFinish();
            }

            @Override
            public void onHomeLongPressed() {
                if (mPresenter.getSale().paySize() > 0) {
                    CommonTools.toastLong(PayWxActivity.this, "已经进行了支付，不能返回");
                    return;
                }

                mPresenter.cancelPayFinish();
            }
        });
        mHomeWatcher.startWatch();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK || keyCode == KeyEvent.KEYCODE_HOME
                || keyCode == KeyEvent.KEYCODE_MENU) {
            if (mPresenter.getSale().paySize() > 0) {
                CommonTools.toastLong(PayWxActivity.this, "已经进行了支付，不能返回");
                return false;
            }

            mPresenter.cancelPayFinish();
        }

        return super.onKeyDown(keyCode, event);
    }

    @OnClick(R.id.imgWxPayExit)
    public void onClickExit() {
        if (isFastClick()) {
            return;
        }

        if (mPresenter.getSale().paySize() > 0) {
            showHintDialog("提示", "已经进行了支付，不能返回");
            return;
        }

        mPresenter.cancelPayFinish();
    }

    @OnClick({R.id.linQRPay, R.id.linFacePay})
    public void onClickPay(View v) {
        linQRPay.setBackgroundColor(Color.parseColor("#FFFFFF"));
        linFacePay.setBackgroundColor(Color.parseColor("#FFFFFF"));

        v.setBackgroundColor(Color.parseColor("#F1F1F1"));

        if (BigDecimalUtils.isLess(mPresenter.getSale().getTotalUnPaid(), "0.001")) {
            CommonTools.toastLong(this, "支付金额为0，不允许继续支付");
            return;
        }
        mPresenter.updateTotal();
        mPresenter.getSale().setPayMethod((PayMethod) v.getTag());

        mPresenter.pay((PayMethod) v.getTag());
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == ActivityConstants.REQUEST_PAY_TO_SCAN && resultCode == ActivityConstants.RESULT_PAY_TO_SCAN) {
            //处理微信扫码界面得到条码返回
            Bundle bundle = data.getExtras();
            if (bundle != null && bundle.containsKey("barcode")) {
                String scanCode = bundle.getString("barcode");
                mPresenter.payPay(scanCode, null, null, WxConstants.wx_pay_store_name());
            } else {
                showHintDialog("获取支付条码失败");
            }
        } else if (requestCode == ActivityConstants.REQUEST_PAY_TO_SCAN && resultCode == RESULT_CANCELED) {
            //从扫码支付界面取消返回
            hideLoadingDialog();
        } else if (requestCode == ActivityConstants.REQUEST_PAY_TO_SUCCESS && resultCode == ActivityConstants.RESULT_PAY_TO_SUCCESS) {
            //从支付完成界面返回
            saleSubmit();
        }
    }

    private void saleSubmit() {
        mPresenter.submit();
    }

    @Override
    protected PayPresenter createPresenter() {
        return new PayPresenter();
    }

    @Override
    public void whenPayDone() {
        this.setResult(ActivityConstants.RESULT_SALE_TO_PAY);
        this.finish();
    }

    @Override
    public void gotoScanActivity() {
        LoggerUtils.getLogger().debug("支付金额:" + mPresenter.getSale().getTotalUnPaid());

        Bundle bundle = new Bundle();
        bundle.putString("totalPaid", mPresenter.getSale().getTotalUnPaid());

        Intent intent = new Intent();
        intent.setClass(PayWxActivity.this, PayScanWxActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SCAN);
    }

    @Override
    public void afterPaySuccess(Bundle bundle) {
        Intent intent = new Intent(this, PaySuccessWxActivity.class);
        intent.putExtras(bundle);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivityForResult(intent, ActivityConstants.REQUEST_PAY_TO_SUCCESS);
    }

    @Override
    public void afterPayFailure(String msg) {
        //跳转支付失败界面
    }

    @Override
    public void updateTotal() {

    }

    @Override
    public void paySuccess(SalePayModel payModel) {

    }

    @Override
    public void gotoCancelPayChannel(SalePayModel payModel) {

    }

    @Override
    public void cancelPaySuccess(SalePayModel payModel) {

    }

    @Override
    public void cancelPayError() {

    }

    @Override
    public void cancelPayFinish() {
        setResult(ActivityConstants.RESULT_GIFT_TO_PAY);
        finishThisActivity();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mHomeWatcher.stopWatch();
    }

    @Override
    public void gotoCouponUseActivity() {

    }

    @Override
    public void gotoScrmCouponActivity() {

    }

    @Override
    public void gotoCardPayActivity() {

    }

    @Override
    public void gotoPointsPayActivity() {

    }

    @Override
    public void gotoOtherPayActivity() {

    }

    @Override
    public void gotoPayChannel(String totalPaid) {

    }

    @Override
    public void setIsClickPayFalse() {

    }
}
