package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.sale.PluCodeInputPresenter;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.view.IView.sale.IPluCodeInputView;
import com.hisense.hiaposcmp.view.KeyBoardWxViewV2;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 微信商品条码输入界面
 *
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-24  重构
 */
@ActivityLayoutInject(R.layout.activity_wechat_codeinput)
public class PluCodeInputWxActivity extends BaseMvpActivity<IPluCodeInputView, PluCodeInputPresenter> implements IPluCodeInputView, KeyBoardWxViewV2.IOnKeyboardListener {

    @BindView(R.id.imb_codeinput_cancel)
    public ImageView btnBack;

    @BindView(R.id.edt_codeinput_input)
    public EditText edtCodeInput;

    @BindView(R.id.imv_codeinput_textdelete)
    public ImageView imvCodeDel;

    @BindView(R.id.tv_codeinput_prompt)
    public TextView tvCodePrompt;

    @BindView(R.id.keyboar_wechat_codeinput)
    public KeyBoardWxViewV2 mKeybroad;

    @BindView(R.id.btn_codeinput_enter)
    public Button btnCodeEnter;

    private String mPluCode = "";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        LoggerUtils.getLogger().debug("商品条码输入界面被创建");
        init();
        initValue();
    }

    @Override
    protected PluCodeInputPresenter createPresenter() {
        return new PluCodeInputPresenter();
    }

    private void init() {
        mKeybroad.setIOnKeyboardListener(this);

        edtCodeInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                mPluCode = s.toString();
                initValue();
            }
        });

        edtCodeInput.setInputType(InputType.TYPE_NULL);
    }

    /**
     * 初始化界面中的部分按钮和文字
     */
    private void initValue() {
        if (StringUtils.isEmpty(mPluCode)) {
            imvCodeDel.setVisibility(View.INVISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login_unpress);
            tvCodePrompt.setVisibility(View.INVISIBLE);
        } else {
            imvCodeDel.setVisibility(View.VISIBLE);
            btnCodeEnter.setBackgroundResource(R.drawable.button_wx_login);
        }
    }

    @OnClick(R.id.imb_codeinput_cancel)
    public void onClickCancel() {
        finish();
    }

    @OnClick(R.id.imv_codeinput_textdelete)
    public void onClickDelete() {
        edtCodeInput.setText("");
    }

    @OnClick(R.id.btn_codeinput_enter)
    public void onClickEnter() {
        if (isFastClick()) {
            return;
        }

        mPluCode = edtCodeInput.getText().toString().trim();
        if (StringUtils.isEmpty(mPluCode)) {
            showHintDialog("商品条码不能为空");
        } else {
            tvCodePrompt.setVisibility(View.INVISIBLE);
            Intent intent = new Intent();
            intent.putExtra("barcode", mPluCode);
            setResult(ActivityConstants.RESULT_PLU_SCAN_TO_INPUT, intent);
            finish();
        }
    }

    @Override
    public void onInsertKeyEvent(String text) {
        edtCodeInput.append(text);
    }

    @Override
    public void onDeleteKeyEvent() {
        String inputStr = edtCodeInput.getText().toString().trim();
        if (inputStr.length() >= 1) {
            edtCodeInput.setText(inputStr.substring(0, inputStr.length() - 1));
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        LoggerUtils.getLogger().debug("商品条码输入界面被恢复");
        edtCodeInput.setText("");
    }
}
