package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.VideoView;

import androidx.annotation.Nullable;

import com.google.zxing.BarcodeFormat;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.bean.BaseEventBean;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.sale.PluCodeScanPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.sale.IPluCodeScanView;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 微信商品扫描界面
 *
 * @author wangwenyue       2018-09-10
 * @author xieliyan         2018-09-10  (济南银座：富基）
 * @author dukunbiao(null)  2019-07-24  重构
 */
@ActivityLayoutInject(R.layout.activity_wechat_goodsscan)
public class PluCodeScanWxActivity extends BaseMvpActivity<IPluCodeScanView, PluCodeScanPresenter> implements IPluCodeScanView, ScanKeyEventHelper.OnScanSuccessListener {

    @BindView(R.id.btn_goodsscan_back)
    public Button btnBack;

    @BindView(R.id.rl_vipinfo)
    public RelativeLayout rlVipInfo;

    @BindView(R.id.tv_vipname)
    public TextView tvTitle;

    @BindView(R.id.tv_cardnum)
    public TextView tvCardNum;

    @BindView(R.id.tv_title_cardnum)
    public TextView tvTitleCardNum;

    @BindView(R.id.tv_title_integral)
    public TextView tvTitleIntegral;

    @BindView(R.id.tv_integral)
    public TextView tvIntegral;

    @BindView(R.id.ll_codeinput_manual)
    public LinearLayout llCodeInput;

    @BindView(R.id.vv_codescan_video)
    public VideoView vvCodeScan;

    @BindView(R.id.tv_goodsscan_title)
    public TextView tvScanTitle;

    private ScanKeyEventHelper mScanKeyEventHelper;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        LoggerUtils.getLogger().debug("商品扫码界面被创建");
        EventBus.getDefault().register(this);

        init();
    }

    @Override
    protected PluCodeScanPresenter createPresenter() {
        return new PluCodeScanPresenter();
    }

    private void init() {
        if (mPresenter.getSale().getVip() == null) {
            rlVipInfo.setVisibility(View.GONE);
        } else {
            initVipInfo();
        }
    }

    private void initVipInfo() {
        String vipName = mPresenter.getSale().getVip().name();
        String vipCardFaceNo = mPresenter.getSale().getVip().card().cardFaceNo();
        String vipCardNo = mPresenter.getSale().getVip().card().vipCardNo();
        double vipPoint = mPresenter.getSale().getVip().card().usablePoints();

        if (StringUtils.isEmpty(vipName)) {
            tvTitle.setText("您好，会员");
        } else {
            tvTitle.setText("您好，" + vipName + " 会员");
        }

        if (vipPoint == 0) {
            tvTitleIntegral.setVisibility(View.GONE);
            tvIntegral.setVisibility(View.GONE);
        } else {
            tvTitleIntegral.setVisibility(View.VISIBLE);
            tvIntegral.setVisibility(View.VISIBLE);
            tvIntegral.setText(String.valueOf(vipPoint));
        }

        if (!StringUtils.isEmpty(vipCardFaceNo)) {
            tvCardNum.setText(vipCardFaceNo);
            tvCardNum.setVisibility(View.VISIBLE);
        } else {
            tvCardNum.setVisibility(View.GONE);
            tvTitleCardNum.setVisibility(View.GONE);
        }
    }

    @OnClick(R.id.btn_goodsscan_back)
    public void onClickBack() {
        showHintConfirm("是否退出结账?", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                CommonTools.gotoWelcomeWxActivity(PluCodeScanWxActivity.this);
                finish();
            }

            @Override
            public void onError(String err) {

            }
        });
    }

    @OnClick(R.id.ll_codeinput_manual)
    public void onClickManual() {
        Intent intent = new Intent();
        intent.setClass(PluCodeScanWxActivity.this, PluCodeInputWxActivity.class);
        startActivityForResult(intent, ActivityConstants.REQUEST_PLU_SCAN_TO_INPUT);
        onPause();
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        mScanKeyEventHelper.analysisKeyEvent(event);
        return true;
    }

    @Override
    public void onScanSuccess(String barcode) {
        CommonUtils.startBeepBgm(this, barcode);
        LoggerUtils.getLogger().debug("商品扫码界面接收到barcode"+barcode);
        if (GeneralSettings.INSTANCE.isCheckScanCode()) {
            if (CommonUtils.checkBarcode(barcode)) {
                return;
            }
        }

        transmitCode(barcode);
    }

    private void transmitCode(String barcode) {
        Bundle bundle = new Bundle();
        bundle.putString("barcode", barcode);
        LoggerUtils.getLogger().debug("商品扫码界面跳转到销售界面");
        CommonTools.goTo(PluCodeScanWxActivity.this, SaleWxActivity.class, bundle);
        finish();
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(Object event) {
        if (event instanceof BaseEventBean) {
            switch (((BaseEventBean) event).getActionName()) {
                case ON_NO_VIP_LOGIN:
                    rlVipInfo.setVisibility(View.GONE);
                    break;
                case ON_VIP_LOGIN:
                    initVipInfo();
                    break;
                default:
                    break;
            }
        }
    }

    private void initVideoView() {
        vvCodeScan.setVideoURI(Uri.parse("android.resource://" + getPackageName() + "/" + R.raw.wechat_video_scan));
        vvCodeScan.requestFocus();
        vvCodeScan.setOnErrorListener((mp, what, extra) -> {
            vvCodeScan.stopPlayback(); //播放异常，则停止播放，防止弹窗使界面阻塞
            return true;
        });
        vvCodeScan.setOnCompletionListener((mp) -> {
            vvCodeScan.start();
            mp.setLooping(true);
        });

        try {
            vvCodeScan.start();
        } catch (Exception e) {
            LoggerUtils.getLogger().error("播放引导视频异常:" + e.toString());
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        LoggerUtils.getLogger().debug("商品扫码界面被恢复");
        initVideoView();
        mScanKeyEventHelper = new ScanKeyEventHelper(this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        LoggerUtils.getLogger().debug("商品扫码界面被暂停");
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mScanKeyEventHelper.onDestroy();
        LoggerUtils.getLogger().debug("商品扫码界面被销毁");
        EventBus.getDefault().unregister(this);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == ActivityConstants.REQUEST_PLU_SCAN_TO_INPUT && resultCode == ActivityConstants.RESULT_PLU_SCAN_TO_INPUT) {
            Intent intent = new Intent();
            intent.putExtra("barcode", data.getExtras().getString("barcode"));
            LoggerUtils.getLogger().debug(" 商品扫描界面跳转到商品条码输入界面"+ data.getExtras().getString("barcode"));
            intent.setClass(PluCodeScanWxActivity.this, SaleWxActivity.class);
            startActivity(intent);
        }
    }
}
