package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.PreferenceActivity;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDelegate;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.common.TemplateEditActivity;
import com.hisense.hiaposcmp.boot.ActivityCollector;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hipos.template.models.PrintTemplateType;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 系统设置，业务设置
 *
 * @author liaoyuping
 * @author xieliyan 2018-06-27 HIPOSAPP-598 美食林增加一码配置功能
 * @author dukunbiao(null)  2019-07-25  重构
 * FIXME null need 重构
 */
public class PosSettingActivity extends PreferenceActivity {

    private AppCompatDelegate delegate;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        getDelegate().installViewFactory();
        getDelegate().onCreate(savedInstanceState);
        super.onCreate(savedInstanceState);

        try {
            setContentView(R.layout.activity_setting_sspos);
            addPreferencesFromResource(R.xml.setting_hsyuniform);

            findViewById(R.id.btnGoBackSSPos).setOnClickListener((v) -> {
                refreshConfigs();
                finish();
            });

            Map<String, String> deviceTypeMap = new HashMap<>();
            deviceTypeMap.put("HS650", "海信HS650系列");
            deviceTypeMap.put("HS550", "海信HS550系列");
            deviceTypeMap.put("SUNMIT2", "商米T2系列");
            bindPreferenceSummary("deviceType", "%s", deviceTypeMap);
            Map<String, String> pinterRateMap = new HashMap<>();
            pinterRateMap.put("9600", "9600");
            pinterRateMap.put("19200", "19200");
            pinterRateMap.put("38400", "38400");
            pinterRateMap.put("115200", "115200");
            bindPreferenceSummary("printerRate", "%s", pinterRateMap);

            Map<String, String> printerPortMap = new HashMap<>();
            printerPortMap.put("0", "COM1");
            printerPortMap.put("1", "COM2");
            printerPortMap.put("2", "COM3");
            printerPortMap.put("3", "COM4");
            printerPortMap.put("4", "COM5");
            bindPreferenceSummary("printerPort", "%s", printerPortMap);
        } catch (Exception e) {
            LoggerUtils.getLogger().error("bind Error:" + e.toString());
        }
    }

    private void bindPreferenceSummary(final String key, final String format, final Map<String, String> values) {
        ListPreference listPreference = (ListPreference) findPreference(key);
        listPreference.setSummary(String.format(Locale.CHINESE, format, listPreference.getEntry()));

        listPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            String value = (String) newValue;
            if (values != null) {
                value = values.get(newValue);
            }
            preference.setSummary(String.format(Locale.CHINESE, format, value));

            return true;
        });
    }

    /**
     * 编辑小票模板
     *
     * @param view 视图对象
     */
    public void editVoucherTemplate(View view) {
        enterEditTemplate(PrintTemplateType.Receipt);
    }

    /**
     * 编辑赠券模板
     *
     * @param view 视图对象
     */
    public void editCouponTemplate(View view) {
        enterEditTemplate(PrintTemplateType.Coupon);
    }

    /**
     * 打开编辑票据模板页面
     *
     * @param type 模板类型
     */
    private void enterEditTemplate(PrintTemplateType type) {
        Intent intent = new Intent(this, TemplateEditActivity.class);
        intent.putExtra("type", type);
        startActivity(intent);
    }

    /**
     * 获取 delegate 对象。
     *
     * @return delegate 对象
     */
    private AppCompatDelegate getDelegate() {
        if (delegate == null) {
            delegate = AppCompatDelegate.create(this, null);
        }
        return delegate;
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        getDelegate().onPostCreate(savedInstanceState);
    }

    @Override
    public void setContentView(int layoutResID) {
        super.setContentView(layoutResID);
        getDelegate().setContentView(layoutResID);
    }

    @Override
    protected void onPostResume() {
        super.onPostResume();
        getDelegate().onPostResume();
        refreshConfigs();
    }

    private void refreshConfigs() {
        Config.cd_Key = GeneralSettings.INSTANCE.getCdKey();
        Config.big_bag = GeneralSettings.INSTANCE.getBigBagCode();
        Config.small_bag = GeneralSettings.INSTANCE.getSmallBagCode();
        Config.pay_ip = "";
        Config.pay_port = "";
        Config.is_vpay = "false";
        Config.permission_code = GeneralSettings.INSTANCE.GetPermissionCode();
        Config.face_limit = String.valueOf(GeneralSettings.INSTANCE.getAliFacePayLimit());
        Config.update_url = GeneralSettings.INSTANCE.getOtaServerIp();
        Config.update_port = GeneralSettings.INSTANCE.getOtaServerPort();
        Config.operator_code = GeneralSettings.INSTANCE.getOperator();

        Config.save();
    }

    @Override
    protected void onPause() {
        super.onPause();
        refreshConfigs();
    }

    @Override
    protected void onStop() {
        super.onStop();
        getDelegate().onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        ActivityCollector.removeActivity(this);
        getDelegate().onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }
}
