package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.adapter.GiftItemAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.FeatureConstants;
import com.hisense.hiaposcmp.presenter.sale.ProGiftPresenter;
import com.hisense.hiaposcmp.view.IView.sale.IProGiftView;

import java.util.ArrayList;
import java.util.HashMap;

import butterknife.BindView;
import butterknife.OnClick;
import jp.wasabeef.recyclerview.animators.SlideInRightAnimator;

/**
 * 换购界面
 *
 * @author xieliyan         2018-11-14
 * @author dukunbiao(null)  2019-07-24  重构
 */
@ActivityLayoutInject(R.layout.activity_purchase_gift_wx)
public class ProGiftWxActivity extends BaseMvpActivity<IProGiftView, ProGiftPresenter> implements IProGiftView {

    @BindView(R.id.btn_quit)
    public Button btnQuit;

    @BindView(R.id.rv_shopping_cart)
    public RecyclerView mRecyclerView;

    @BindView(R.id.tv_summary_text)
    public TextView tvSummaryText;

    @BindView(R.id.tv_total_money)
    public TextView tvTotalMoney;

    @BindView(R.id.btn_pay)
    public Button btnPay;

    private GiftItemAdapter giftItemAdapter;
    private int countSale = 0;

    @Override
    protected ProGiftPresenter createPresenter() {
        return new ProGiftPresenter();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        initAdapter();
        initPresenter();
        onItemClicked();
    }

    private void initAdapter() {
        mRecyclerView.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        giftItemAdapter = new GiftItemAdapter(mRecyclerView);
        mRecyclerView.setAdapter(giftItemAdapter);

        mRecyclerView.setItemAnimator(new SlideInRightAnimator());
        mRecyclerView.getItemAnimator().setAddDuration(300);
        mRecyclerView.getItemAnimator().setRemoveDuration(50);
        mRecyclerView.getItemAnimator().setMoveDuration(50);
        mRecyclerView.getItemAnimator().setChangeDuration(50);
    }

    private void initPresenter() {
        countSale = mPresenter.countSale();
        mPresenter.assignAdapter(giftItemAdapter);
        mPresenter.clearAdapter();
        mPresenter.setAdapter();
        mPresenter.updateAllTotal();
    }

    private void onItemClicked() {
        giftItemAdapter.setOnItemClickListener((view, viewName, position) -> processViewClick(viewName, position));
    }

    private void processViewClick(GiftItemAdapter.ViewName viewName, int position) {
        switch (viewName) {
            case INCREASE:
                mPresenter.increase(position);
                break;
            case DECREASE:
                mPresenter.decrease(position);
                break;
            default:
                break;
        }
    }

    @OnClick(R.id.btn_quit)
    public void onClickQuit() {
        if (isFastClick()) {
            return;
        }

        mPresenter.delete(countSale);
        finish();
    }

    @OnClick(R.id.btn_pay)
    public void onClickPay() {
        if (isFastClick()) {
            return;
        }

        Intent intent = new Intent(this, PayCustomActivity.class);
        startActivityForResult(intent, ActivityConstants.REQUEST_GIFT_TO_PAY);
    }

    @Override
    public void updateAllTotal(int totalCount, String totalDiscount, String totalPay) {
        tvSummaryText.setText("共" + totalCount + "商品，" + "已优惠￥" + totalDiscount + "," + "合计");
        tvTotalMoney.setText(totalPay);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == ActivityConstants.REQUEST_GIFT_TO_PAY && resultCode == ActivityConstants.RESULT_SALE_TO_PAY) {
            if (data.getExtras() == null) {
                LoggerUtils.getLogger().info("支付返回的结果为空");
                return;
            }

            ArrayList<HashMap<String, Object>> dataList = (ArrayList<HashMap<String, Object>>) data.getSerializableExtra("PayList");
            Intent intent = new Intent();
            intent.putExtra("PayList", dataList);
            this.setResult(ActivityConstants.RESULT_SALE_TO_PAY, intent);
            this.finish();
        } else if (requestCode == ActivityConstants.REQUEST_GIFT_TO_PAY && resultCode == ActivityConstants.RESULT_GIFT_TO_PAY) {
            showHintDialog("提示", "您已放弃支付");
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
