/***************************************************************************************************
 * copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.sale.SaleItemPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hiaposcmp.view.IView.sale.ISaleItemView;
import com.hisense.hiaposcmp.view.KeyBoardStdView;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.pos.scanner.IScanner;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;
import butterknife.OnFocusChange;
import butterknife.OnTextChanged;
import butterknife.OnTouch;

/**
 * 价格录入对话框。
 *
 * @author dukunbiao(null)  2020-02-14
 * @author guolina          2020-03-09 HIAPOSCMP-566  微信自助允许录入商品价格需求
 */
@ActivityLayoutInject(R.layout.activity_sale_item_price_wx)
public class SaleItemPriceWxActivity extends BaseMvpActivity<ISaleItemView, SaleItemPresenter> implements ISaleItemView, KeyBoardStdView.OnKeyboardListener {

    @BindView(R.id.tvSaleItemPluName)
    public TextView tvSaleItemPluName;

    @BindView(R.id.edtSaleItemTagPrice)
    public EditText edtSaleItemTagPrice;

    @BindView(R.id.tvSaleItemDiscount)
    public TextView tvSaleItemDiscount;

    @BindView(R.id.tvSaleItemPrice)
    public TextView tvSaleItemPrice;

    @BindView(R.id.tvSaleItemQuantity)
    public TextView tvSaleItemQuantity;

    @BindView(R.id.edtSaleItemQuantity)
    public EditText edtSaleItemQuantity;

    @BindView(R.id.edtSaleItemSpec)
    public EditText edtSaleItemSpec;

    @BindView(R.id.tvSaleItemTotal)
    public TextView tvSaleItemTotal;

    @BindView(R.id.btnSaleItemConfirm)
    public Button btnSaleItemConfirm;

    @BindView(R.id.keyBrdSaleStdKeyBoard)
    public KeyBoardStdView keyBrdSaleStdKeyBoard;

    private boolean isHandleTextChanged = false;

    private BaseSalePluModel mPluModel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initData();
        mScanner = getScanner(this);
    }

    private void init() {
        Bundle bundle = this.getIntent().getExtras();
        BaseSalePluModel pluModel = (BaseSalePluModel) bundle.getSerializable("plu");
        if (pluModel == null) {
            showHintDialog("获取商品信息失败，请重试");
            return;
        }

        mPluModel = pluModel;
        mPresenter.initRangeValidator(mPluModel);

        edtSaleItemQuantity.setSelection(edtSaleItemQuantity.getText().length());

        SoftInput.disable(edtSaleItemTagPrice);
        SoftInput.disable(edtSaleItemQuantity);
        SoftInput.disable(edtSaleItemSpec);

        keyBrdSaleStdKeyBoard.setOnKeyboardListener(this);
        btnSaleItemConfirm.setEnabled(true);
    }

    private void initData() {
        tvSaleItemPluName.setText(mPluModel.getPluCode() + "--" + mPluModel.getPluName());
        tvSaleItemQuantity.setText(mPluModel.isWeight() ? "重量" : "件数");

        edtSaleItemTagPrice.setText("");
        tvSaleItemPrice.setText("");
        edtSaleItemQuantity.setText("1");
        edtSaleItemSpec.setText(mPluModel.getSpec());
        tvSaleItemTotal.setText("");
        edtSaleItemTagPrice.requestFocus();
        NumberHandler.setPoint(edtSaleItemTagPrice, 2);//只允许输入两位小数
    }

    @Override
    protected SaleItemPresenter createPresenter() {
        return new SaleItemPresenter();
    }

    @OnClick(R.id.btnSaleItemReset)
    public void onClickReset() {
        initData();
    }

    @OnClick(R.id.btnSaleItemConfirm)
    public void onClickConfirm() {
        try {
            if (!btnSaleItemConfirm.isEnabled()) {
                return;
            }

            btnSaleItemConfirm.setEnabled(false);
            edtSaleItemTagPrice.setText(BigDecimalUtils.decimalFormat(edtSaleItemTagPrice.getText().toString()));
            mPresenter.checkNoCodePlu(mPluModel, edtSaleItemTagPrice, tvSaleItemPrice, tvSaleItemDiscount, edtSaleItemQuantity, tvSaleItemTotal, edtSaleItemSpec);
        } finally {
            btnSaleItemConfirm.setEnabled(true);
        }
    }

    @OnClick(R.id.ivSaleItemCancel)
    public void onClickCancel() {
        setResult(RESULT_CANCELED, null);
        finish();
    }

    @OnTouch({R.id.edtSaleItemTagPrice, R.id.edtSaleItemQuantity, R.id.edtSaleItemSpec})
    public void onTouch(View arg0, MotionEvent arg1) {
        EditText edt = (EditText) arg0;

        int inType = edt.getInputType();

        edt.onTouchEvent(arg1);
        edt.setInputType(inType);
        edt.setSelection(edt.length());
    }

    @OnFocusChange({R.id.edtSaleItemTagPrice, R.id.edtSaleItemQuantity, R.id.edtSaleItemSpec})
    public void onFocusChange() {
        if (edtSaleItemSpec.isFocused()) {
            SoftInput.openSoftKeyBoard(this);
        } else {
            SoftInput.closeSoftKeyBoard(this);
        }
    }


    @OnTextChanged(R.id.tvSaleItemPrice)
    public void onTextChangedPrice(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        String err = mPresenter.priceValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        calcDiscount();
    }

    @OnTextChanged(R.id.edtSaleItemTagPrice)
    public void onTextChangedTagPrice(CharSequence s) {
        if (isHandleTextChanged) {
            return;
        }
        if (StringUtils.isEmpty(s.toString())) {
            tvSaleItemPrice.setText("");
            tvSaleItemTotal.setText("");
            return;
        }

        String err = mPresenter.tagPriceValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        calcSalePrice();
    }

    @OnTextChanged(R.id.edtSaleItemQuantity)
    public void onTextChangedQuantity(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        String err = mPresenter.quantityValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        try {
            double value1 = Double.valueOf(s.toString());
            double value2 = Double.valueOf(String.format(Locale.CHINESE, "%.3f", value1));

            // 如果录入超出 3 位小数，则重新格式化，保留 3 位小数
            if (Double.compare(value1, value2) != 0) {
                isHandleTextChanged = true;
                edtSaleItemQuantity.setText(String.format(Locale.CHINESE, "%.3f", value1));
                edtSaleItemQuantity.setSelection(edtSaleItemQuantity.length());
            }

            // 称重商品或等价，显示“件数”；非称重商品，显示“重量”
            tvSaleItemQuantity.setText(mPluModel.isWeight() || (Double.compare(value2, Math.round(value2)) == 0) ? "件数" : "重量");

            calcTotal();
        } finally {
            isHandleTextChanged = false;
        }
    }

    @OnTextChanged(R.id.edtSaleItemSpec)
    public void onTextChangedSpec(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        if (edtSaleItemSpec.getText().toString().length() > 20) {
            CommonTools.toastShort("规格最长为20个字符");
            KeyBoardStdView.deleteChar(edtSaleItemSpec);
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            edtSaleItemSpec.setText(mScanner.getScanResult(data));
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    /**
     * 使用单价、折扣计算售价。
     */
    private void calcSalePrice() {
        if (edtSaleItemTagPrice.length() == 0 || tvSaleItemDiscount.length() == 0) {
            return;
        }

        try {
            isHandleTextChanged = true;

            String rate = BigDecimalUtils.subtract("100", tvSaleItemDiscount.getText().toString());
            if (BigDecimalUtils.isLess(rate, "0")) {
                CommonTools.toastLong("折扣率不能小于 0");
                return;
            }
            if (BigDecimalUtils.isGreater(rate, "100")) {
                CommonTools.toastLong("折扣率不能大于 100");
                return;
            }

            String price = StringUtils.changeF2Y(BigDecimalUtils.multiply(edtSaleItemTagPrice.getText().toString(), tvSaleItemDiscount.getText().toString()));
            tvSaleItemPrice.setText(price);

            calcTotal();
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
        } finally {
            isHandleTextChanged = false;
        }
    }

    /**
     * 使用单价、售价计算折扣。
     */
    private void calcDiscount() {
        if (edtSaleItemTagPrice.length() == 0 || tvSaleItemPrice.length() == 0) {
            return;
        }

        try {
            isHandleTextChanged = true;
            if (BigDecimalUtils.isGreater(tvSaleItemPrice.getText().toString(), edtSaleItemTagPrice.getText().toString())) {
                CommonTools.toastLong("售价不能大于商品标价");
                tvSaleItemPrice.setText(tvSaleItemPrice.getText().toString().substring(0, tvSaleItemPrice.getText().toString().length() - 1));
                return;
            }

            String rate = BigDecimalUtils.divide(BigDecimalUtils.multiply(tvSaleItemPrice.getText().toString(), "100"), edtSaleItemTagPrice.getText().toString(), 4);
            tvSaleItemDiscount.setText(BigDecimalUtils.decimalFormat(rate, 2));

            calcTotal();
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
        } finally {
            isHandleTextChanged = false;
        }
    }

    private void calcTotal() {
        if (StringUtils.isEmpty(tvSaleItemPrice.getText().toString()) || StringUtils.isEmpty(edtSaleItemQuantity.getText().toString())) {
            return;
        }

        String total = BigDecimalUtils.multiply(tvSaleItemPrice.getText().toString(), edtSaleItemQuantity.getText().toString());
        tvSaleItemTotal.setText(BigDecimalUtils.decimalFormat(total));
    }

    @Override
    public void checkNoCodePluSuccess(BaseSalePluModel pluModel) {
        Bundle bundle = new Bundle();
        bundle.putSerializable("plu", pluModel);
        Intent intent = new Intent();
        intent.putExtras(bundle);

        setResult(ActivityConstants.RESULT_SALE_TO_SET_PLU_PRICE, intent);
        finish();
    }

    @Override
    public void onInsertKeyEvent(String text) {
        EditText editText = getFocusedView();
        if (editText == null) {
            return;
        }

        KeyBoardStdView.addChar(editText, text);
    }

    @Override
    public void onDeleteKeyEvent() {
        EditText editText = getFocusedView();
        if (editText == null) {
            return;
        }

        KeyBoardStdView.deleteChar(editText);
    }

    @Override
    public void onClearKeyEvent() {
        EditText editText = getFocusedView();
        if (editText == null) {
            return;
        }

        KeyBoardStdView.clearChar(editText);
    }

    @Override
    public void onDoneKeyEvent() {
        EditText editText = null;
        if (edtSaleItemTagPrice.isFocused()) {
            editText = edtSaleItemQuantity;
        } else if (edtSaleItemQuantity.isFocused()) {
            editText = edtSaleItemSpec;
            SoftInput.openSoftKeyBoard(this);
        }
        if (edtSaleItemSpec.isFocused()) {
            SoftInput.closeSoftKeyBoard(this);
        }

        if (editText != null) {
            editText.requestFocus();
            editText.setSelection(editText.getText().toString().length());
        }
    }

    private EditText getFocusedView() {
        if (edtSaleItemTagPrice.isFocused()) {
            return edtSaleItemTagPrice;
        }
        if (edtSaleItemQuantity.isFocused()) {
            return edtSaleItemQuantity;
        }
        if (edtSaleItemSpec.isFocused()) {
            return edtSaleItemSpec;
        }

        return null;
    }
}
