package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.adapter.SaleItemAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.bean.BagBean;
import com.hisense.hiaposcmp.bean.BaseEventBean;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.EventActions;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.popup.BagPopupWindow;
import com.hisense.hiaposcmp.presenter.sale.SalePresenter;
import com.hisense.hiaposcmp.service.wx.WxService;
import com.hisense.hiaposcmp.service.wx.dto.WxDto;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.IFlyTtsUtil;
import com.hisense.hiaposcmp.view.IView.sale.ISaleView;
import com.hisense.hipos.model.BaseSalePluModel;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;
import jp.wasabeef.recyclerview.animators.SlideInRightAnimator;

/**
 * 微信销售主界面
 * Created by zhuguangsheng on 2018/7/2.
 *
 * @author zhuguangsheng    取消商品-光升，取消交易-光升，商品查询-光升
 * @author xieliyan         调整业务流程
 * @author xieliyan         2018-09-10  济南银座（富基）
 * @author dukunbiao(null)  2019-07-23  重构
 * @author xieliyan         2020-01-02 HIAPOSCMP-313 商品查询异常时语音播报
 * @author guolina          2020-01-09 HIAPOSCMP-490 慧收银增加整单促销接口
 * @author guolina          2020-03-09 HIAPOSCMP-566  微信自助允许录入商品价格需求
 */
@ActivityLayoutInject(R.layout.activity_sale_wx)
public class SaleWxActivity extends BaseMvpActivity<ISaleView, SalePresenter> implements ISaleView, ScanKeyEventHelper.OnScanSuccessListener {

    @BindView(R.id.btn_test)
    public Button btnTest;

    @BindView(R.id.ll_add_bag)
    public LinearLayout llAddBag;

    @BindView(R.id.ll_manual_input_barcode)
    public LinearLayout llManualInputBarcode;

    @BindView(R.id.btn_quit)
    public Button btnQuit;

    @BindView(R.id.rv_shopping_cart)
    public RecyclerView mRecyclerView;

    @BindView(R.id.tv_summary_text)
    public TextView tvSummaryText;

    @BindView(R.id.tv_total_money)
    public TextView tvTotalMoney;

    @BindView(R.id.btn_pay)
    public Button btnPay;

    private SaleItemAdapter saleItemAdapter;

    private ScanKeyEventHelper mScanKeyEventHelper;

    private boolean isScan = true;
    private static boolean payCustomActivity;
    boolean isPluDeleteEnabled = true;

    BagPopupWindow bagPopupWindow;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        IFlyTtsUtil.init(this);
        IFlyTtsUtil.setParams();
        initAdapter();
        initPresenter();
        LoggerUtils.getLogger().debug("销售界面被创建");
        initValue();

        EventBus.getDefault().register(this);
    }

    @Override
    protected SalePresenter createPresenter() {
        return new SalePresenter();
    }

    private void initAdapter() {
        mRecyclerView.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        saleItemAdapter = new SaleItemAdapter(mRecyclerView, true);
        mRecyclerView.setAdapter(saleItemAdapter);
        saleItemAdapter.setOnItemClickListener((view, viewName, position) -> processViewClick(viewName, position));

        //动画
        mRecyclerView.setItemAnimator(new SlideInRightAnimator());
        mRecyclerView.getItemAnimator().setAddDuration(300);
        mRecyclerView.getItemAnimator().setRemoveDuration(50);
        mRecyclerView.getItemAnimator().setMoveDuration(50);
        mRecyclerView.getItemAnimator().setChangeDuration(50);
    }

    private void initPresenter() {
        saleItemAdapter.clearPlu();
        mPresenter.updateAdapterAndUi();
    }

    @Override
    public void updateAdapter() {
        if (saleItemAdapter != null) {
            saleItemAdapter.setData(mPresenter.getSale().getPlus());
            saleItemAdapter.notifyDataSetChanged();
        }
    }

    @Override
    public void onlineFail(String msg) {

    }

    @Override
    public void reSubmitFail(String msg) {

    }

    @Override
    public void reSubmitSuccess(String msg) {

    }

    private void initValue() {
        payCustomActivity = false;
        try {
            Thread.sleep(200);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        processIntent(getIntent());

        if (GeneralSettings.INSTANCE.isUseWxFacePay()) {
            if (!WxDto.getWxDto().isInit()) {
                WxService.initWxPayFace(getApplicationContext());
            }
        }

        String shoppingBagSmall = GeneralSettings.INSTANCE.getSmallBagCode();
        String shoppingBagMiddle = GeneralSettings.INSTANCE.getMiddleBagCode();
        String shoppingBagBig = GeneralSettings.INSTANCE.getBigBagCode();
        List<BagBean> listBags = new ArrayList<>();
        if (!StringUtils.isEmpty(shoppingBagBig)) {
            BagBean bagBean1 = new BagBean(shoppingBagBig, "大购物袋");
            listBags.add(bagBean1);
        }
        if (!StringUtils.isEmpty(shoppingBagMiddle)) {
            BagBean bagBean2 = new BagBean(shoppingBagMiddle, "中购物袋");
            listBags.add(bagBean2);
        }
        if (!StringUtils.isEmpty(shoppingBagSmall)) {
            BagBean bagBean3 = new BagBean(shoppingBagSmall, "小购物袋");
            listBags.add(bagBean3);
        }

        bagPopupWindow = new BagPopupWindow(this, listBags, sItemClickCallback);

        if (ListUtils.isEmpty(listBags)) {
            llAddBag.setVisibility(View.INVISIBLE);
        }
    }

    private void processIntent(Intent intent) {
        Bundle bundle;
        String barcode;
        try {
            bundle = intent.getExtras();
            if (bundle == null) {
                return;
            }

            if (bundle.containsKey("barcode")) {

                barcode = bundle.getString("barcode");
                LoggerUtils.getLogger().error("触发商品查询:" + barcode);
                mPresenter.pluQuery(barcode);
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("添加商品异常:" + e.toString());
        }
    }

    @OnClick(R.id.btn_quit)
    public void onClickQuit() {
        if (isFastClick()) {
            return;
        }

        if (GeneralSettings.INSTANCE.isUsePermission() && !mPresenter.getSale().getPlus().isEmpty()) {
            CommonTools.goTo(SaleWxActivity.this, PermissionCheckWxActivity.class, ActivityConstants.REQUEST_SALE_CANCEL_TO_PERMISSION);
        } else {
            showGiveUpDialog();
        }
    }

    @Override
    @OnClick(R.id.ll_add_bag)
    public void showBagsPopupWindow() {
        if (isFastClick()) {
            return;
        }

        bagPopupWindow.showUp(llAddBag);
    }

    @OnClick(R.id.ll_manual_input_barcode)
    public void onClickInput() {
        if (isFastClick()) {
            return;
        }

        Intent intent = new Intent();
        intent.setClass(SaleWxActivity.this, PluCodeInputWxActivity.class);
        startActivityForResult(intent, ActivityConstants.REQUEST_PLU_SALE_TO_INPUT);
    }

    @OnClick(R.id.btn_pay)
    public void onClickPay() {
        if (isFastClick()) {
            LoggerUtils.getLogger().debug("在销售界面快速点击支付");
            return;
        }
        LoggerUtils.getLogger().debug("有效点击支付payCustomActivity" + payCustomActivity);
        if (payCustomActivity) {
            LoggerUtils.getLogger().debug("是否已点击去付款:" + payCustomActivity);
            return;
        }

        mPresenter.promotionCalc();

    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(Object event) {
        if (event instanceof BaseEventBean) {
            String barCode = ((BaseEventBean) event).getMsgDetail();
            switch (((BaseEventBean) event).getActionName()) {
                //扫描枪条码
                case SCAN_GOT_BARCODE:
                    mPresenter.pluQuery(barCode);
                    break;
                //FIXME null 异常分支？
                case ON_MANUAL_BARCODE_INPUT_COMPLETE:
                    LoggerUtils.getLogger().debug("异常分支");
                    break;
                case ON_MANUAL_PERMISSION_INPUT_COMPLETE:
                    mPresenter.rePrintBill(barCode);
                    break;
                default:
                    break;
            }
        }
    }

    private void processViewClick(SaleItemAdapter.ViewName viewName, int position) {
        switch (viewName) {
            case ITEM:
                setSelected(position);
                break;
            case DELETE:
                pluDeleteConfirm(position);
                break;
            case INCREASE:
                pluAdjust(position, true);
                break;
            case DECREASE:
                pluAdjust(position, false);
                break;
            default:
                break;
        }
    }

    @Override
    public void addPlu(int position, BaseSalePluModel salePluModel) {
        saleItemAdapter.addPlu(position, salePluModel);
    }

    @Override
    public void removePlu(int position) {
        saleItemAdapter.removePlu(position);
//        updateAdapter();
    }

    @Override
    public void changeQuantity(int position) {
        saleItemAdapter.changeQuantity(position);
        updateAdapter();
    }

    private synchronized void setSelected(int position) {
        saleItemAdapter.setItemSelected(position);
//        updateAdapter();
    }

    private void pluDeleteConfirm(int position) {
        if (isFastClick()) {
            return;
        }

        if (GeneralSettings.INSTANCE.isUsePermission()) {
            Intent intent = new Intent(this, PermissionCheckWxActivity.class);
            Bundle bundle = new Bundle();
            bundle.putInt("position", position);
            intent.putExtras(bundle);

            startActivityForResult(intent, ActivityConstants.REQUEST_PLU_DELETE_TO_PERMISSION);
        } else {
            pluDelete(position);
        }
    }

    private void pluDelete(int position) {
        try {
            BaseSalePluModel pluModel = mPresenter.getItem(position);

            showHintConfirm("是否删除此商品:" + pluModel.getPluName(), new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    mPresenter.pluDelete(position);
                }

                @Override
                public void onError(String err) {

                }
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("刪除商品异常:" + e.toString());
        }
    }

    private void pluAdjust(int position, boolean add) {
        if (isFastClick(100) || !isPluDeleteEnabled) {
            return;
        }

        mPresenter.pluAdjust(position, add);
    }

    @Override
    public void hideShoppingCartPrompt() {

    }

    @Override
    public void showShoppingCartPrompt() {

    }

    @Override
    public void updateTotal(int count, String totalDsc, String totalRecv) {
        tvSummaryText.setText("共" + count + "商品，" + "已优惠￥" + totalDsc + "," + "合计");
        tvTotalMoney.setText(totalRecv);
    }

    public void updateTotal() {
        if (mPresenter.getSale() != null) {
            updateTotal(mPresenter.getSale().calcCount(), mPresenter.getSale().getTotalDsc(), mPresenter.getSale().getTotalRecv());
        } else {
            updateTotal(0, "0.00", "0.00");
        }
    }

    @Override
    public void updateVipInfo(Vip vip) {

    }

    public void showGiveUpDialog() {
        showHintConfirm("是否退出结账？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                mPresenter.saleCancel();
            }

            @Override
            public void onError(String err) {

            }
        });
    }

    @Override
    public void callPayActivity() {
        payCustomActivity = true;
        isScan = false;
        Intent intent = new Intent(this, PayCustomActivity.class);
        Bundle payBundle = new Bundle();
        intent.putExtras(payBundle);

        startActivityForResult(intent, ActivityConstants.REQUEST_SALE_TO_PAY);
    }

    @Override
    public void promotionCalcSuccess(View view) {

    }

    @Override
    public void callGiftActivity() {
        payCustomActivity = true;
        isScan = false;

        Intent intent = new Intent(this, ProGiftWxActivity.class);
        startActivityForResult(intent, ActivityConstants.REQUEST_SALE_TO_GIFT);
    }

    @Override
    public void scrollListToBottom() {
        try {
            int position = saleItemAdapter.getItemCount() - 1;
            if (position >= 0) {
                mRecyclerView.smoothScrollToPosition(position);
            }
        } catch (Exception e) {
            CommonTools.toastLong("滚动失败:" + e.getMessage());
        }
    }

    @Override
    public void scrollListToPosition(int position) {

    }

    @Override
    public void setPluDeleteEnabled(boolean isPluDeleteEnabled) {
        this.isPluDeleteEnabled = isPluDeleteEnabled;
    }

    @Override
    public void backToWelcomeActivity() {
        CommonTools.goToWithFinishMe(SaleWxActivity.this, WelcomeWxActivity.class);
    }

    @Override
    public void pluQueryNoCode(BaseSalePluModel pluModel) {
        Bundle bundle = new Bundle();
        bundle.putSerializable("plu", pluModel);

        Intent intent = new Intent(this, SaleItemPriceWxActivity.class);
        intent.putExtras(bundle);

        startActivityForResult(intent, ActivityConstants.REQUEST_SALE_TO_SET_PLU_PRICE);
    }

    static BagPopupWindow.ItemClickCallback sItemClickCallback = (barcode) ->
            EventBus.getDefault().post(new BaseEventBean(EventActions.SCAN_GOT_BARCODE, barcode));

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        mScanKeyEventHelper.analysisKeyEvent(event);
        try {
            if (event.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
                timer.cancel();
                timeStart();
            } else {
                timer.cancel();
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("拦截扫码动作异常:" + e.toString());
        }

        return true;
    }

    @Override
    public void onScanSuccess(String barcode) {
        CommonUtils.startBeepBgm(this, barcode);

        if (!isScan) {
            LoggerUtils.getLogger().info("点击了去付款按钮，不允许继续扫码 isScan:" + isScan);
            return;
        }

        LoggerUtils.getLogger().info("扫描内容:" + barcode + "扫码状态isScan:" + isScan);

        if (GeneralSettings.INSTANCE.isCheckScanCode()) {
            if (CommonUtils.checkBarcode(barcode)) {
                return;
            }
        }

        EventBus.getDefault().post(new BaseEventBean(EventActions.SCAN_GOT_BARCODE, barcode));
    }

    @Override
    public void hideBagsPopupWindow() {

    }

    @Override
    public void checkoutSuccess() {

    }

    @Override
    public void checkoutError() {

    }

    @Override
    public void submitSuccess() {

    }

    @Override
    public void submitFail(String msg) {

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        //吉麦隆市场问题新增注释
        LoggerUtils.getLogger().debug("回调onActivityResult");
        if (requestCode == ActivityConstants.REQUEST_PLU_DELETE_TO_PERMISSION && resultCode == ActivityConstants.RESULT_PERMISSION_SUCCESS) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入REQUEST_PLU_DELETE_TO_PERMISSION--RESULT_PERMISSION_SUCCESS");
            permissionPluDelete(data);
        } else if (requestCode == ActivityConstants.REQUEST_PLU_SALE_TO_INPUT && resultCode == ActivityConstants.RESULT_PLU_SCAN_TO_INPUT) {
            LoggerUtils.getLogger().debug("商品条码输入界面返回到销售界面");
            processIntent(data);
        } else if ((requestCode == ActivityConstants.REQUEST_SALE_TO_PAY || requestCode == ActivityConstants.REQUEST_SALE_TO_GIFT)
                && resultCode == ActivityConstants.RESULT_SALE_TO_PAY) {
            payCustomActivity = false;
            mPresenter.resetSaleModel();
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入RESULT_SALE_TO_PAY");
        } else if (requestCode == ActivityConstants.REQUEST_SALE_CANCEL_TO_PERMISSION && resultCode == ActivityConstants.RESULT_PERMISSION_SUCCESS) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入REQUEST_SALE_CANCEL_TO_PERMISSION--RESULT_PERMISSION_SUCCESS");
            permissionSaleCancel(data);
        } else if (requestCode == ActivityConstants.REQUEST_SALE_TO_SET_PLU_PRICE && resultCode == ActivityConstants.RESULT_SALE_TO_SET_PLU_PRICE) {
            //吉麦隆市场问题新增注释
            LoggerUtils.getLogger().debug("回调onActivityResult进入RESULT_SALE_TO_SET_PLU_PRICE");
            addPluNoCode(data);
        }

        updateAdapter();
        updateTotal();
        super.onActivityResult(requestCode, resultCode, data);
    }

    private void addPluNoCode(Intent data) {
        Bundle bundle = data.getExtras();
        if (bundle == null) {
            return;
        }

        BaseSalePluModel pluModel = (BaseSalePluModel) bundle.getSerializable("plu");
        mPresenter.checkPluVipDiscount(pluModel);
    }

    private void permissionPluDelete(Intent intent) {
        Bundle bundle = intent.getExtras();

        if (mPresenter.checkPermissionCode(intent)) {
            pluDelete(bundle.getInt("position"));
        }
    }

    private void permissionSaleCancel(Intent intent) {
        if (mPresenter.checkPermissionCode(intent)) {
            showGiveUpDialog();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        LoggerUtils.getLogger().debug("销售界面被恢复");
        payCustomActivity = false;
        isScan = true;
        mScanKeyEventHelper = new ScanKeyEventHelper(this);
    }

    @Override
    protected void onPause() {
        mScanKeyEventHelper.onDestroy();
        LoggerUtils.getLogger().debug("销售界面被暂停");
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        mScanKeyEventHelper.onDestroy();
        LoggerUtils.getLogger().debug("销售界面被销毁");
        EventBus.getDefault().unregister(this);
        if (GeneralSettings.INSTANCE.isUseWxFacePay()) {
            WxService.releaseWxPayFace(getApplicationContext());
        }
        super.onDestroy();
    }
}
