package com.hisense.hiaposcmp.activity.wx;

import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.bean.BaseEventBean;
import com.hisense.hiaposcmp.constant.EventActions;
import com.hisense.hiaposcmp.presenter.VipLoginPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IVipLoginView;
import com.hisense.hiaposcmp.view.KeyBoardWxViewV2;

import org.greenrobot.eventbus.EventBus;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-24  重构
 */
@ActivityLayoutInject(R.layout.activity_wechat_viplogin)
public class VipLoginWxActivity extends BaseMvpActivity<IVipLoginView, VipLoginPresenter> implements IVipLoginView, KeyBoardWxViewV2.IOnKeyboardListener {

    @BindView(R.id.imv_viplogin_cancel)
    public ImageView imvBack;

    @BindView(R.id.edt_viplogin_input)
    public EditText edtTextInput;

    @BindView(R.id.imv_viplogin_textdelete)
    public ImageView imvTextDel;

    @BindView(R.id.tv_viplogin_prompt)
    public TextView tvVipPrompt;

    @BindView(R.id.keyboard_wechat_viplogin)
    public KeyBoardWxViewV2 mKeybroad;

    @BindView(R.id.btn_viplogin_enter)
    public Button btnEnter;

    private String mVipNum = "";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        initBtn();
    }

    @Override
    protected void onResume() {
        super.onResume();
        edtTextInput.setText("");
        edtTextInput.setFocusable(true);
    }

    @Override
    protected VipLoginPresenter createPresenter() {
        return new VipLoginPresenter();
    }

    private void init() {
        mKeybroad.setIOnKeyboardListener(this);

        edtTextInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                if (StringUtils.isEmpty(s.toString())) {
                    edtTextInput.setFocusable(true);
                    edtTextInput.requestFocus();
                }
                mVipNum = s.toString();
                initBtn();
            }
        });

        edtTextInput.setInputType(InputType.TYPE_NULL);
        initListener();
    }

    private void initBtn() {
        if (StringUtils.isEmpty(mVipNum)) {
            imvTextDel.setVisibility(View.INVISIBLE);
            btnEnter.setBackgroundResource(R.drawable.button_wx_login_unpress);
            tvVipPrompt.setVisibility(View.INVISIBLE);
        } else {
            imvTextDel.setVisibility(View.VISIBLE);
            btnEnter.setBackgroundResource(R.drawable.button_wx_login);
        }
    }

    @Override
    public void onInsertKeyEvent(String text) {
        edtTextInput.append(text);
    }

    @Override
    public void onDeleteKeyEvent() {
        String inputStr = edtTextInput.getText().toString().trim();
        if (inputStr.length() >= 1) {
            edtTextInput.setText(inputStr.substring(0, inputStr.length() - 1));
        }
    }

    @OnClick(R.id.imv_viplogin_cancel)
    public void onClickCancel() {
        CommonTools.goTo(VipLoginWxActivity.this, WelcomeWxActivity.class);
        finish();
    }

    @OnClick(R.id.imv_viplogin_textdelete)
    public void onClickDelete() {
        edtTextInput.setText("");
    }

    @OnClick(R.id.btn_viplogin_enter)
    public void onClickEnter() {
        if (isFastClick()) {
            return;
        }

        try {
            mPresenter.vipQuery(edtTextInput.getText().toString().trim());
        } catch (NullPointerException e) {
            hideLoadingDialog();
            tvVipPrompt.setVisibility(View.VISIBLE);
            tvVipPrompt.setText("网络状态异常，查询失败");
        }
    }

    private void initListener() {
        edtTextInput.setOnKeyListener((v, keyCode, event) -> {
                    if (keyCode == KeyEvent.KEYCODE_ENTER && event.getAction() == KeyEvent.ACTION_DOWN) {
                        String vipNo = edtTextInput.getText().toString();
                        if (!StringUtils.isNumeric(String.valueOf(vipNo.charAt(0)))) {
                            vipNo = vipNo.substring(1);
                            edtTextInput.setText(vipNo);
                        }
                        mPresenter.vipQuery(vipNo);
                    }

                    return false;
                }
        );
    }

    @Override
    public void vipLoginSuccess(Vip vip) {
        mPresenter.getSale().setVip(vip);
        EventBus.getDefault().post(new BaseEventBean(EventActions.ON_VIP_LOGIN, ""));
        CommonTools.goTo(VipLoginWxActivity.this, PluCodeScanWxActivity.class);
        finish();
    }

    @Override
    public void vipLoginFail(String msg) {
        tvVipPrompt.setVisibility(View.VISIBLE);
        tvVipPrompt.setText(msg);
    }

    @Override
    public void verifyTypeSuccess(Vip vip) {

    }

    @Override
    public void verifyTypeFail(String msg) {

    }
}
