package com.hisense.hiaposcmp.activity.wx;

import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.os.Bundle;
import android.os.RemoteException;
import android.os.SystemClock;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.VideoView;

import androidx.annotation.Nullable;

import com.hisense.common.boot.GlobalCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.activity.common.AuthActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.bean.BaseEventBean;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.CommonConstants;
import com.hisense.hiaposcmp.constant.EventActions;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.presenter.WelcomePresenter;
import com.hisense.hiaposcmp.service.sale.SaleService;
import com.hisense.hiaposcmp.service.wx.WxService;
import com.hisense.hiaposcmp.service.wx.dto.WxDto;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.IWelcomeView;
import com.tencent.wxpayface.IWxPayfaceCallback;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 微信欢迎界面
 *
 * @author wangwenyue 2018-07-04
 * @author xieliyan   2018-08-14 增加称重防损提示
 * @author dukunbiao(null)  2019-07-22  重构
 */
@ActivityLayoutInject(R.layout.activity_wechat_welcome)
public class WelcomeWxActivity extends BaseMvpActivity<IWelcomeView, WelcomePresenter> implements IWelcomeView, ScanKeyEventHelper.OnScanSuccessListener {

    @BindView(R.id.btn_login)
    public Button btnLogin;

    @BindView(R.id.btn_viplogin)
    public Button btnVipLogin;

    @BindView(R.id.ll_viplogin)
    public LinearLayout llVipLogin;

    @BindView(R.id.vv_operationshow)
    public VideoView vvVideo;

    @BindView(R.id.imv_logo)
    public ImageView imvLogo;

    @BindView(R.id.tv_support)
    public TextView tvSupport;

    @BindView(R.id.lytErrorList)
    public FrameLayout lytErrorList;

    @BindView(R.id.btnReCheck)
    public TextView btnReCheck;

    @BindView(R.id.txtError)
    public TextView txtError;

    @BindView(R.id.ll_reprint_ticket)
    public LinearLayout llReprintTicket;

    @BindView(R.id.lyt_video)
    public LinearLayout lytVideo;

    @BindView(R.id.ll_minigo_use)
    public LinearLayout llMiNiGoUse;

    @BindView(R.id.btnUse)
    public Button btnUse;

    private ScanKeyEventHelper mScanKeyEventHelper;

    private long[] mHits = new long[CommonConstants.CLICK_COUNTS];

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        LoggerUtils.getLogger().debug("欢迎界面被创建");
        EventBus.getDefault().register(this);

        init();
        initLogo();
    }

    @Override
    protected WelcomePresenter createPresenter() {
        return new WelcomePresenter();
    }

    private void init() {
        isNotShowExitDialog = true;

        startSaleService();
        mPresenter.initTimer();

    }

    @Override
    public void startSaleService() {
        Intent intent = new Intent(this, SaleService.class);
        startService(intent);
    }

    private void initLogo() {
        try {
            if (LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH) == null) {
                imvLogo.setImageDrawable(getResources().getDrawable(R.drawable.hisense_wechat));
            } else {
                imvLogo.setImageBitmap(LoadSDCard.getImageFromSDCard(SDCardConstants.SSPOS_LOGO_PATH));
            }
        } catch (Exception e) {
            imvLogo.setImageDrawable(getResources().getDrawable(R.drawable.hisense_wechat));
            LoggerUtils.getLogger().error("加载Logo异常:" + e.toString());
        }
    }

    private void initView() {
        if (GeneralSettings.INSTANCE.isUseVip()) {
            llVipLogin.setVisibility(View.VISIBLE);
            btnVipLogin.setVisibility(View.VISIBLE);

            LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(360, 120);
            btnLogin.setLayoutParams(layoutParams);
            layoutParams.setMargins(0, 0, 20, 0);
            btnVipLogin.setLayoutParams(layoutParams);
        } else {
            llVipLogin.setVisibility(View.GONE);
            btnVipLogin.setVisibility(View.GONE);

            LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(720, 120);
            btnLogin.setLayoutParams(layoutParams);
            btnVipLogin.setLayoutParams(layoutParams);
        }
    }

    private void initVideoView() {
        vvVideo.setVideoURI(Uri.parse(loadVideo()));
        vvVideo.requestFocus();
        try {
            vvVideo.start();
            vvVideo.setOnErrorListener((mp, what, extra) -> {
                LoggerUtils.getLogger().error("播放异常，停止播放，防止弹窗使界面阻塞");
                vvVideo.stopPlayback();
                return true;
            });
            vvVideo.setOnCompletionListener((mp) -> {
                vvVideo.setVideoURI(Uri.parse(loadVideo()));
                vvVideo.requestFocus();
                vvVideo.start();
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("视频播放异常:" + e.toString());
        }
    }

    private String loadVideo() {
        String uriString = "android.resource://" + getPackageName() + "/" + R.raw.wechat_video_home;
        File file = new File(SDCardConstants.SSPOS_VIDEO_PATH);
        if (file.exists()) {
            uriString = "file://" + SDCardConstants.SSPOS_VIDEO_PATH;
            LoggerUtils.getLogger().info("加载本地视频，路径:" + uriString);
        }
        return uriString;
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == ActivityConstants.REQUEST_REPRINT_TO_PERMISSION && resultCode == ActivityConstants.RESULT_PERMISSION_SUCCESS) {
            Bundle bundle = data.getExtras();
            if (bundle != null && bundle.containsKey("permissionCode")) {
                String permissionCode = bundle.getString("permissionCode");
                String setPermissionCode = GeneralSettings.INSTANCE.GetPermissionCode();
                if (permissionCode != null && permissionCode.equals(setPermissionCode)) {
                    mPresenter.printTicketAgain();
                } else {
                    showHintDialog("请注意", "授权码错误");
                }
            }
        }
    }

    /**
     * 点击登录商品扫描界面
     */
    @OnClick(R.id.btn_login)
    public void onClickToPluScan() {
        if (isFastClick()) {
            return;
        }
        LoggerUtils.getLogger().debug("欢迎界面点击登录");
        mPresenter.createNewSale(new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                CommonTools.goTo(WelcomeWxActivity.this, PluCodeScanWxActivity.class);
                EventBus.getDefault().post(new BaseEventBean(EventActions.ON_NO_VIP_LOGIN, ""));
            }

            @Override
            public void onError(String message) {
                showHintDialog("请注意", message);
            }
        });
    }

    @OnClick({R.id.ll_viplogin, R.id.btn_viplogin})
    public void onClickToVip() {
        if (isFastClick()) {
            return;
        }

        mPresenter.createNewSale(new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                CommonTools.goTo(WelcomeWxActivity.this, VipLoginWxActivity.class);
            }

            @Override
            public void onError(String message) {
                showHintDialog("请注意", message);
            }
        });
    }

    @OnClick(R.id.imv_logo)
    public void onClickBackstage() {
        System.arraycopy(mHits, 1, mHits, 0, mHits.length - 1);
        //实现左移，然后最后一个位置更新距离开机的时间，如果最后一个时间和最开始时间小于DURATION，即连续5次点击
        mHits[mHits.length - 1] = SystemClock.uptimeMillis();
        if (mHits[0] >= (SystemClock.uptimeMillis() - CommonConstants.CLICK_DURATION)) {
            //达到点击次数，清理数组
            for (int i = 0; i < mHits.length; i++) {
                mHits[i] = 0;
            }
            AuthActivity.authToManagement(this);
        }
    }

    @OnClick(R.id.btnReCheck)
    public void onClickCheckPrintStatus() {
        if (isFastClick()) {
            return;
        }

        txtError.setText("");

        String msg = mPresenter.getPrintStatus();

        if (StringUtils.isEmpty(msg)) {
            lytErrorList.setVisibility(View.GONE);
        } else {
            lytErrorList.setVisibility(View.VISIBLE);

            txtError.setText(msg);
        }
    }

    @OnClick(R.id.btnUse)
    public void onClickStartApp() {
        // 通过包名获取此APP详细信息，包括Activities、services、versioncode、name等等
        PackageInfo packageinfo = null;
        try {
            packageinfo = getContext().getPackageManager().getPackageInfo("com.hisense.minigo", 0);
        } catch (PackageManager.NameNotFoundException e) {
            LoggerUtils.getLogger().error("打开核销程序异常:" + e.toString());
        }
        if (packageinfo == null) {
            showHintDialog("未安装核销程序");
            return;
        }

        // 创建一个类别为CATEGORY_LAUNCHER的该包名的Intent
        Intent resolveIntent = new Intent(Intent.ACTION_MAIN, null);
        resolveIntent.addCategory(Intent.CATEGORY_LAUNCHER);
        resolveIntent.setPackage(packageinfo.packageName);

        // 通过getPackageManager()的queryIntentActivities方法遍历
        List<ResolveInfo> resolveInfoList = getContext().getPackageManager().queryIntentActivities(resolveIntent, 0);
        ResolveInfo resolveinfo = resolveInfoList.iterator().next();
        if (resolveinfo != null) {
            // packagename = 参数packname
            String packageName = resolveinfo.activityInfo.packageName;
            // 这个就是我们要找的该APP的LAUNCHER的Activity[组织形式：packagename.mainActivityname]
            String className = resolveinfo.activityInfo.name;
            // LAUNCHER Intent
            CommonUtils.startApp(packageName, className, getContext());
        }
    }

    @OnClick(R.id.ll_reprint_ticket)
    public void onClickReprint() {
        if (isFastClick()) {
            return;
        }

        CommonTools.goTo(WelcomeWxActivity.this, PermissionCheckWxActivity.class, ActivityConstants.REQUEST_REPRINT_TO_PERMISSION);
    }

    @Override
    public void checkPrintStatus() {
        onClickCheckPrintStatus();
    }

    /**
     * 拦截键盘输入
     */
    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        mScanKeyEventHelper.analysisKeyEvent(event);
        return true;
    }

    @Override
    public void onScanSuccess(String barcode) {

        LoggerUtils.getLogger().debug("接收到扫码" + barcode);

        CommonUtils.startBeepBgm(this, barcode);

        if (GeneralSettings.INSTANCE.isCheckScanCode()) {
            if (CommonUtils.checkBarcode(barcode)) {
                return;
            }
        }

        try {
            LoggerUtils.getLogger().debug("在欢迎界面触发扫码" + barcode);
            onScanToSale(barcode);
        } catch (NullPointerException e) {
            hideLoadingDialog();
            showHintDialog("服务器连接异常");
        }
    }

    private void onScanToSale(String barcode) {
        mPresenter.createNewSale(new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                Bundle bundle = new Bundle();
                bundle.putString("barcode", barcode);
                CommonTools.goTo(WelcomeWxActivity.this, SaleWxActivity.class, bundle);
                LoggerUtils.getLogger().debug("从欢迎界面跳转到销售界面" + barcode);
                onPause();
            }

            @Override
            public void onError(String message) {
                showHintDialog("请注意", message);
            }
        });
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(Object event) {

    }

    @Override
    protected void onPause() {
        super.onPause();
        LoggerUtils.getLogger().debug("欢迎界面暂停");
        mScanKeyEventHelper.onDestroy();
        closeBannerState();
    }

    @Override
    protected void onResume() {
        super.onResume();
        LoggerUtils.getLogger().debug("返回到欢迎界面");
        if (GeneralSettings.INSTANCE.isUse()) {
            llMiNiGoUse.setVisibility(View.VISIBLE);
        } else {
            llMiNiGoUse.setVisibility(View.GONE);
        }

        showLoadingDialog();
        CommonUtils.reInitSaleModel(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                hideLoadingDialog();
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                showHintDialog(err);
            }
        });

        initVideoView();
        initView();
        mScanKeyEventHelper = new ScanKeyEventHelper(this);
        onClickCheckPrintStatus();
        mPresenter.timeStart();

        openBannerState();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        LoggerUtils.getLogger().debug("欢迎界面destroy");
        EventBus.getDefault().unregister(this);
        mScanKeyEventHelper.onDestroy();
        mPresenter.timeCancel();
        LoadSDCard.releaseImageViewResouce(imvLogo);
    }

    /**
     * 打开运营位
     */
    private void openBannerState() {
        if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
            LoggerUtils.getLogger().debug("刷脸支付开关未打开");
            return;
        }
        if (!WxDto.getWxDto().isInit()) {
            WxService.initWxPayFace(getApplicationContext(), new IWxPayfaceCallback() {
                @Override
                public void response(Map map2) throws RemoteException {
                    LoggerUtils.getLogger().debug("微信人脸初始化成功，进而打开运营位");
                    Map map = new HashMap();
                    //map.put("banner_state", 0);
                    map.put("banner_state", 1);//暂时关闭运营位
                    WxService.updateWxpayfaceBannerState(map, new IWxPayfaceCallback() {
                        @Override
                        public void response(Map map) throws RemoteException {
                            LoggerUtils.getLogger().debug("打开运营位成功");
                        }
                    });
                }
            });
        } else {
            LoggerUtils.getLogger().debug("微信人脸已经初始化，直接打开运营位");
            Map map = new HashMap();
            //map.put("banner_state", 0);
            map.put("banner_state", 1);//暂时关闭运营位
            WxService.updateWxpayfaceBannerState(map, new IWxPayfaceCallback() {
                @Override
                public void response(Map map) throws RemoteException {
                    LoggerUtils.getLogger().debug("打开运营位成功");

                }
            });
        }
    }

    /**
     * 关闭运营位
     */
    private void closeBannerState() {
        if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
            LoggerUtils.getLogger().debug("微信刷脸支付开关未打开");
            return;
        }
        if (!WxDto.getWxDto().isInit()) {
            WxService.initWxPayFace(getApplicationContext(), new IWxPayfaceCallback() {
                @Override
                public void response(Map map2) throws RemoteException {
                    LoggerUtils.getLogger().debug("微信人脸初始化成功，进而关闭运营位");
                    Map map = new HashMap();
                    map.put("banner_state", 1);
                    WxService.updateWxpayfaceBannerState(map, new IWxPayfaceCallback() {
                        @Override
                        public void response(Map map) throws RemoteException {
                            LoggerUtils.getLogger().debug("关闭运营位成功");
                        }
                    });
                }
            });
        } else {
            LoggerUtils.getLogger().debug("微信人脸已经初始化，直接关闭运营位");
            Map map = new HashMap();
            map.put("banner_state", 1);
            WxService.updateWxpayfaceBannerState(map, new IWxPayfaceCallback() {
                @Override
                public void response(Map map) throws RemoteException {
                    LoggerUtils.getLogger().debug("关闭运营位成功");
                }
            });
        }
    }
}
