/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.wx.refund;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import androidx.annotation.Nullable;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 券退款
 *
 * @author xieliyan    2017-05-18
 * @author xieliyan    2018-04-03 HIPOSAPP-376 修复选择会员券、满赠券退货闪退的问题
 * @author zhangsai    2018-06-11 HIPOSAPP-509 增加退货时最大可退金额的限制
 * @author wangyaogang 2019-02-28 HIPOSAPP-1031 优化扫描结果传值逻辑
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author wangyaogang 2019-07-01 HIPOSAPP-1189 使用 IScanner 扫描器实例扫描
 * @author xieliyan    2019-11-01 HIAPOSCMP-254 退款重构
 */
@ActivityLayoutInject(R.layout.activity_coupon_refund_wx)
public class CouponRefundWxActivity extends BaseActivity {
    /**
     * 券号
     */
    @BindView(R.id.edtCouponNo)
    public EditText edtCouponNo;
    /**
     * 退款金额
     */
    @BindView(R.id.edtRefundMoney)
    public EditText edtRefundMoney;
    /**
     * 取消
     */
    @BindView(R.id.btnCancel)
    public Button btnCancel;
    /**
     * /确认
     */
    @BindView(R.id.btnConfirm)
    public Button btnConfirm;
    /**
     * 扫描
     */
    @BindView(R.id.imgScan)
    public ImageView imgScan;
    /**
     * 输入退款金额
     */
    private String msqValue = "0";
    /**
     * 可退款最大金额
     */
    private String maxRefund = "0";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mScanner = getScanner(this);
        Bundle bundle = getIntent().getExtras();
        edtRefundMoney.setText(String.format("%.2f", Double.parseDouble(bundle.get("fYTui").toString())));
        msqValue = edtRefundMoney.getText().toString();
        maxRefund = bundle.getString("maxRefund");
    }

    /**
     * 扫码监听
     */
    @OnClick(R.id.imgScan)
    public void onClickScan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                edtCouponNo.setText(s);
                edtCouponNo.setSelection(edtCouponNo.getText().toString().length());
            }

            @Override
            public void onError(final Throwable t) {
                CommonTools.toastShort(t.getMessage());
            }
        });
    }

    /**
     * 取消监听
     */
    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        setResult(ActivityConstants.RESULT_REFUND_COUPON_CANCEL);
        finish();
    }

    /**
     * 确认监听
     */
    @OnClick(R.id.btnConfirm)
    public void onClickConfirm() {

        if (StringUtils.isEmpty(edtCouponNo.getText().toString())) {
            CommonTools.toastShort(CouponRefundWxActivity.this, "券号不能为空");
            return;
        }
        if (StringUtils.isEmpty(edtRefundMoney.getText().toString())) {
            CommonTools.toastShort(CouponRefundWxActivity.this, "应退金额不能为空");
            return;
        }

        if (BigDecimalUtils.isLess(msqValue, edtRefundMoney.getText().toString())) {
            CommonTools.toastShort(CouponRefundWxActivity.this, "应退金额过大");
            edtRefundMoney.setText(msqValue);
            return;
        }
        if (BigDecimalUtils.isGreater(edtRefundMoney.getText().toString(), String.valueOf(maxRefund))) {
            CommonTools.toastShort(CouponRefundWxActivity.this, "应退金额过大");
            edtRefundMoney.setText(msqValue);
            return;
        }

        Intent intent = new Intent();
        intent.putExtra("qNo", edtCouponNo.getText().toString());
        intent.putExtra("qValue", edtRefundMoney.getText().toString());
        setResult(ActivityConstants.RESULT_REFUND_COUPON_CONFIRM, intent);
        finish();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            edtCouponNo.setText(mScanner.getScanResult(data));
            edtCouponNo.setSelection(edtCouponNo.getText().toString().length());
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
