/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy;

import android.content.Intent;
import android.os.Bundle;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceScreen;
import android.view.View;

import androidx.appcompat.app.AppCompatDelegate;

import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.common.ConfigActivity;
import com.hisense.hiaposcmp.activity.common.TemplateEditActivity;
import com.hisense.hiaposcmp.activity.common.UpLogActivity;
import com.hisense.hiaposcmp.boot.ActivityCollector;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.device.DeviceType;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hipos.template.models.PrintTemplateType;
import com.hisense.webapi.WebApi;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 设置。
 *
 * @author guanguangjin 2017-02-23
 * @author xieliyan     2018-01-05
 * @author lizhixin     2018-06-12 HIPOSAPP-520 加入工行收单支付通道
 * @author wangyaogang  2018-11-26 HIPOSAPP-836 系统设置流程调整,增加新的设置选项
 * @author guolina      2018-12-07 HIPOSAPP-874 tradelog和crash更改存放路径并自动清理过期日志
 * @author wangyaogang  2018-12-04 HIPOSAPP-837 增加微信,支付宝直连支付方式
 * @author guolina      2018-12-25 HIPOSAPP-850 获取新流水号兼容POS中间件,规则可以在中间件设置
 * @author guolina      2019-01-15 HIPOSAPP-937 连网超时时间可配置
 * @author zhangsai     2019-01-30 HIPOSAPP-1002 增加“扫描器类型”配置项
 * @author wangyaogang  2019-02-20 HIPOSAPP-1024 修复摄像头类型已选择"激光扫描头"时,仍可修改设备类型为非HI98的问题
 * @author guolina      2019-02-25 HIPOSAPP-1022 专柜联打印联数支持设置
 * @author wangyaogang  2019-05-09 HIPOSAPP-1086 支持银联扫码支付
 * @author wangyaogang 2019-05-13 HIPOSAPP-1087 控制每台机器收款的小数位
 * @author wangyaogang 2019-05-15 HIPOSAPP-1087 优化[控制每台机器收款的小数位]逻辑,去掉isNeedRound
 * @author wangyaogang 2019-05-28 HIPOSAPP-1117 会员认证方式可配置
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 储值卡/积分卡查询增加校验码选项,且各开关不再互斥
 * @author wangyaogang 2019-06-24 HIPOSAPP-1090 设置中增加礼品券模板
 * FIXME null need 重构
 * @author guolina     2019-10-21 HIPOSAPP-1317 利客来未上传流水自动上传
 */
public class SettingActivity extends PreferenceActivity {

    private AppCompatDelegate delegate;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        getDelegate().installViewFactory();
        getDelegate().onCreate(savedInstanceState);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_setting);

        addPreferencesFromResource(R.xml.setting);

        ActivityCollector.AddActivity(this);

        this.findViewById(R.id.btnGoBack).setOnClickListener((v) -> {
            finish();
        });

        EditTextPreference interval = (EditTextPreference) findPreference(Config.PAPER_CURT_INTERVAL);
        interval.setSummary(interval.getText() + " 秒");
        interval.setOnPreferenceChangeListener((preference, newValue) -> {
            preference.setSummary(newValue + " 秒");
            return true;
        });

        EditTextPreference terminalId = (EditTextPreference) findPreference(GeneralSettings.Keys.TERMINAL_ID);
        terminalId.setSummary(terminalId.getText());
        terminalId.setOnPreferenceChangeListener((preference, newValue) -> {
            preference.setSummary(newValue.toString());
            return true;
        });

        EditTextPreference timeOfUpload = (EditTextPreference) findPreference(GeneralSettings.Keys.TIME_OF_UPLOAD);
        timeOfUpload.setSummary(timeOfUpload.getText());
        timeOfUpload.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object newValue) {
                preference.setSummary(newValue.toString());
                return true;
            }
        });
        bindPreferenceSummary(GeneralSettings.Keys.TOTAL_REDUCTION_WAY, "%s", null);
        bindPreferenceSummary(GeneralSettings.Keys.RETURN_SALE_DAYS, "%s 天", null);
        bindPreferenceSummary(GeneralSettings.Keys.LOG_FILE_EXPIRED_DAYS, "%s 天", null);
        bindPreferenceSummary(GeneralSettings.Keys.RECEIPT_PRINT_BY_SHOP, "%s", R.array.receiptPrintType, R.array.receiptPrintTypeValues);
        bindPreferenceSummary(GeneralSettings.Keys.TIMEOUT, "%s 秒", null, null, (preference, newValue) -> {
            String value = (String) newValue;
            CommonTools.loginConfig(App.getInstance()).edit()
                    .putString(GeneralSettings.Keys.TIMEOUT, value)
                    .commit();
            WebApi.init(Config.getUrl(Config.base_url) + "/", Integer.parseInt(value), TimeUnit.SECONDS);
            return true;
        });

        // 设备类型
        bindPreferenceSummary(GeneralSettings.Keys.DEVICE_TYPE, "%s", R.array.deviceType, R.array.deviceTypeValues, null,
                (pre, newValue) -> {
                    // 如果选择的设备类型不支持当前的扫描器类型，则提示用户
                    if (!DeviceType.valueOf((String) newValue).isHI98() &&
                            GeneralSettings.INSTANCE.getScannerType().isLaser()) {
                        CommonTools.toastShort(String.format(Locale.CHINESE, "%s不支持%s",
                                getDeviceTypeName(DeviceType.valueOf((String) newValue)),
                                getResources().getStringArray(R.array.scannerTypeNames)[1]));
                        return false;
                    } else {
                        return true;
                    }
                });

        // 扫描器类型
        bindPreferenceSummary(GeneralSettings.Keys.SCANNER_TYPE, "%s", R.array.scannerTypeNames, R.array.scannerTypeValues,
                getResources().getStringArray(R.array.scannerTypeNames)[0], (preference, newValue) -> {
                    // 如果选择的设备类型不支持当前的扫描器类型，则不允许修改配置项，同时提示用户
                    if (getResources().getStringArray(R.array.scannerTypeValues)[1].equals(newValue) &&
                            !GeneralSettings.INSTANCE.getDeviceType().isHI98()) {
                        // 激光扫码+非HI98时，返回false
                        CommonTools.toastShort(String.format(Locale.CHINESE, "%s不支持%s",
                                getDeviceTypeName(),
                                getResources().getStringArray(R.array.scannerTypeNames)[1]));
                        return false;
                    } else {
                        return true;
                    }
                });

        // 打印字体大小
        bindPreferenceSummary(GeneralSettings.Keys.PRINTER_TEXT_SIZE, "%s",
                R.array.printer_text_size_entries, R.array.printer_text_size_values);

        // IC卡版本
        bindPreferenceSummary(GeneralSettings.Keys.IC_CARD_VERSION, "%s",
                R.array.cardVersion, R.array.cardVersionValues);

        // 计算精度
        bindPreferenceSummary(GeneralSettings.Keys.CURRENCY_PRECISION, "%s",
                R.array.currencyPrecision, R.array.currencyPrecisionValues);

        // 会员认证方式
        bindPreferenceSummary(GeneralSettings.Keys.AUTH_METHOD, "%s",
                R.array.authMethod, R.array.authMethodValues);

        // 支付通道
        bindPreferenceSummary(GeneralSettings.Keys.PAYMENT_CHANNEL, "%s",
                R.array.paymentChannel, R.array.paymentChannelValue);

        // 聚合支付通道
        bindPreferenceSummary(GeneralSettings.Keys.AGGREGATE_PAYMENT_CHANNEL, "%s",
                R.array.aggregatePayChannel, R.array.aggregatePayChannelValue);
    }

    @Override
    public boolean onPreferenceTreeClick(PreferenceScreen preferenceScreen, Preference preference) {
        if ("upLog".equals(preference.getKey())) {
            CommonTools.gotoActivity(UpLogActivity.class);
        } else if ("orgCode".equals(preference.getKey())) {
            Intent intent = new Intent(this, ConfigActivity.class);
            intent.putExtra("pos", 1);
            startActivity(intent);
        } else if ("shopCode".equals(preference.getKey())) {
            Intent intent = new Intent(this, ConfigActivity.class);
            intent.putExtra("pos", 2);
            startActivity(intent);
        } else if ("posNo".equals(preference.getKey())) {
            Intent intent = new Intent(this, ConfigActivity.class);
            intent.putExtra("pos", 3);
            startActivity(intent);
        }

        return super.onPreferenceTreeClick(preferenceScreen, preference);
    }

    /**
     * 返回设备名称。
     */
    private String getDeviceTypeName() {
        return getDeviceTypeName(GeneralSettings.INSTANCE.getDeviceType());
    }

    /**
     * 返回设备名称。
     *
     * @param deviceType 设备类型
     */
    private String getDeviceTypeName(DeviceType deviceType) {
        return getEntryNameByValue(deviceType.toString(),
                R.array.deviceType, R.array.deviceTypeValues);
    }

    /**
     * 返回配置项值 {@param value} 对应的名称。
     *
     * @param value       配置项的当前值
     * @param entries     配置项名称集合
     * @param entryValues 配置项的值集合
     * @return 配置项对应的名称
     */
    private String getEntryNameByValue(String value, int entries, int entryValues) {
        if (value != null) {
            for (int i = 0; i < App.getContext().getResources().getStringArray(entries).length; i++) {
                if (value.equals(App.getContext().getResources().getStringArray(entryValues)[i])) {
                    return App.getContext().getResources().getStringArray(entries)[i];
                }
            }
        }

        return "";
    }

    @Override
    protected void onResume() {
        super.onResume();
        setNumber(Config.ORG_CODE);
        setNumber(Config.SHOP_CODE);
        setNumber(Config.POS_NO);
    }

    /**
     * 设置门店号等
     *
     * @param key
     */
    private void setNumber(String key) {
        Preference preference = findPreference(key);
        final String value = CommonTools.loginConfig(this).getString(key, "");
        preference.setSummary(value);
    }

    /**
     * 绑定配置项说明。
     *
     * @param key    配置项的键
     * @param format 配置项说明的格式
     * @param values 配置项的值集合
     */
    private void bindPreferenceSummary(final String key, final String format, final Map<String, String> values) {
        this.bindPreferenceSummary(key, format, values, null, null);
    }

    /**
     * 绑定配置项说明。
     *
     * @param key         配置项的键
     * @param format      配置项说明的格式
     * @param entries     配置项名称集合
     * @param entryValues 配置项的值集合
     */
    private void bindPreferenceSummary(final String key, final String format, int entries, int entryValues) {
        this.bindPreferenceSummary(key, format, entries, entryValues, null, null);
    }

    /**
     * 绑定配置项说明。
     *
     * @param key          配置项的键
     * @param format       配置项说明的格式
     * @param entries      配置项名称集合
     * @param entryValues  配置项的值集合
     * @param defaultValue 配置项的默认值
     * @param listener     监听配置项变化的监听器
     */
    private void bindPreferenceSummary(final String key, final String format, int entries, int entryValues,
                                       final String defaultValue,
                                       final Preference.OnPreferenceChangeListener listener) {
        Map<String, String> map = new HashMap<>();
        for (int i = 0; i < App.getContext().getResources().getStringArray(entries).length; i++) {
            map.put(
                    App.getContext().getResources().getStringArray(entryValues)[i],
                    App.getContext().getResources().getStringArray(entries)[i]);
        }
        this.bindPreferenceSummary(key, format, map, defaultValue, listener);
    }

    /**
     * 绑定配置项说明。
     *
     * @param key          配置项的键
     * @param format       配置项说明的格式
     * @param values       配置项的键值图
     * @param defaultValue 配置项的默认值
     * @param listener     监听配置项变化的监听器
     */
    private void bindPreferenceSummary(final String key, final String format, final Map<String, String> values,
                                       final String defaultValue,
                                       final Preference.OnPreferenceChangeListener listener) {
        ListPreference preference = (ListPreference) findPreference(key);
        CharSequence entry = preference.getEntry();
        if (entry != null) {
            preference.setSummary(String.format(Locale.CHINESE, format, preference.getEntry()));
        } else {
            preference.setSummary(String.format(Locale.CHINESE, format, defaultValue));
        }

        preference.setOnPreferenceChangeListener((pre, newValue) -> {
            if (listener != null) {
                if (!listener.onPreferenceChange(pre, newValue)) {
                    return false;
                }
            }

            String value = (String) newValue;
            if (values != null) {
                value = values.get(newValue);
            }
            pre.setSummary(String.format(Locale.CHINESE, format, value));

            return true;
        });
    }

    /**
     * 编辑小票模板
     *
     * @param view 视图对象
     */
    public void editVoucherTemplate(View view) {
        enterEditTemplate(PrintTemplateType.Receipt);
    }

    /**
     * 编辑赠券模板
     *
     * @param view 视图对象
     */
    public void editCouponTemplate(View view) {
        enterEditTemplate(PrintTemplateType.Coupon);
    }

    /**
     * 编辑礼品券模板
     *
     * @param view 视图对象
     */
    public void editGiftCouponTemplate(View view) {
        enterEditTemplate(PrintTemplateType.GiftCoupon);
    }

    /**
     * 打开编辑票据模板页面
     *
     * @param type 模板类型
     */
    private void enterEditTemplate(PrintTemplateType type) {
        Intent intent = new Intent(this, TemplateEditActivity.class);
        intent.putExtra("type", type);
        startActivity(intent);
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        getDelegate().onPostCreate(savedInstanceState);
    }

    @Override
    public void setContentView(int layoutResID) {
        super.setContentView(layoutResID);
        getDelegate().setContentView(layoutResID);
    }

    @Override
    protected void onPostResume() {
        super.onPostResume();
        getDelegate().onPostResume();
    }

    @Override
    protected void onStop() {
        super.onStop();
        getDelegate().onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        ActivityCollector.removeActivity(this);
        getDelegate().onDestroy();
    }

    /**
     * 获取 delegate 对象。
     *
     * @return delegate 对象
     */
    private AppCompatDelegate getDelegate() {
        if (delegate == null) {
            delegate = AppCompatDelegate.create(this, null);
        }
        return delegate;
    }
}
