/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.card;

import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.TextView;

import com.hisense.common.boot.UniCallback;
import com.hisense.crm.card.Card;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.StartActivityService;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.layout.CombinedEditText;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.query.CardAndPointsQueryPresenter;
import com.hisense.hiaposcmp.view.IView.query.ICardAndPointsQueryView;
import com.hisense.hiaposcmp.view.KeyBoardView;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import org.apache.commons.lang3.StringUtils;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 储值卡余额查询。
 *
 * @author xieliyan    2017-03-29
 * @author xieliyan    2017-09-25 HIPOSAPP-160 调用HI98驱动刷卡（磁条/射频）
 * @author xieliyan    2017-10-31 HIPOSAPP-201 使用枚举实现单例模式:更改调用方法
 * @author xieliyan    2017-11-22 HIPOSAPP-224 支持射频卡2.0
 * @author xieliyan    2017-12-26 HIPOSAPP-257 查询时增加加载动画
 * @author xieliyan    2018-01-04 HIPOSAPP-265 查询时增加密码
 * @author zhangsai    2018-01-31 HIPOSAPP-311 优化储值卡余额查询
 * @author zhangsai    2018-02-02 HIPOSAPP-315 继承自 CardBaseActivity，由后者提供刷卡服务
 * @author zhangsai    2018-03-29 HIPOSAPP-363 修复刷卡时卡号类型错误导致无法完成查询的问题
 * @author guolina     2018-08-07 HIPOSAPP-713 进入储值卡余额查询界面软键盘应该默认为弹出状态
 * @author wangyaogang 2018-08-07 HIPOSAPP-942 修复点击"储值卡余额查询"、"积分余额查询"程序崩溃的问题
 * @author wangyaogang 2018-12-29 HIPOSAPP-875 会员/储值卡余额/积分余额查询支持条码扫描
 * @author wangyaogang 2019-01-03 HIPOSAPP-875 优化键盘使用逻辑
 * @author zhangsai    2019-01-30 HIPOSAPP-1002 使用 IScanner 扫描器实例扫描
 * @author wangyaogang 2019-02-14 HIPOSAPP-1018 修复需要输入密码或校验位进行储值卡或者积分卡余额查询时无响应问题
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 储值卡查询接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 储值卡查询页面增加校验码选项
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 储值卡余额查询更改为根据接口返回结果确定验证类型,校验参数封装为CardSecurity
 * @author wangyaogang 2019-07-19 HIPOSAPP-1216 储值卡传值优化,卡类型为7时,isVerifyPwd值取决于开关状态,非7默认1
 * @author xieliyan    2019-08-05 HIPOSAPP-1277 对接SCRM-获取会员卡验证方式:请求参数增加payMethod
 * @author wangyaogang 2019-09-05 HIPOSAPP-1287 查询验证方式出错时重新启动刷卡监听
 * @author wangyaogang 2019-11-12 HIAPOSCMP-267 解决扫码按钮无效问题
 * @author wangyaogang 2019-11-15 HIAPOSCMP-288 监听键盘完成按钮
 */
@ActivityLayoutInject(R.layout.activity_vip_query)
public class CardBalanceCardAndPointsQueryActivity extends CardBaseMvpActivity<ICardAndPointsQueryView, CardAndPointsQueryPresenter> implements ICardAndPointsQueryView {

    @BindView(R.id.txtTitle)
    TextView mTxtTitle;

    @BindView(R.id.txtCardNo)
    CombinedEditText txtCardNo;

    @BindView(R.id.txtPassword)
    CombinedEditText txtPassword;

    @BindView(R.id.txtFaceNo)
    CombinedEditText txtFaceNo;

    @BindView(R.id.txtVerify)
    CombinedEditText txtVerify;

    private CardNo cardNo;
    private String payMethod = PayMethod.CARD;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        mScanner = getScanner(this);
    }

    private void init() {
        mTxtTitle.setText("储值余额查询");

        txtPassword.getEdit().setOnTouchListener((v, event) -> {
            if (event.getAction() == MotionEvent.ACTION_UP) {
                SoftInput.closeSoftKeyBoard(this);
                KeyBoardView keyboard = txtPassword.getKeyboard();
                txtPassword.setKeyboard(keyboard);
            }
            return false;
        });

        txtCardNo.getEdit().setOnTouchListener(mOnTouchListener);
        txtVerify.getEdit().setOnTouchListener(mOnTouchListener);
        txtFaceNo.getEdit().setOnTouchListener(mOnTouchListener);

        txtCardNo.getEdit().setImeOptions(EditorInfo.IME_ACTION_DONE);
        txtCardNo.getEdit().setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int actionId, KeyEvent keyEvent) {
                if (actionId == EditorInfo.IME_ACTION_DONE) {
                    veriy();
                }
                return false;
            }
        });
    }

    @Override
    protected CardAndPointsQueryPresenter createPresenter() {
        return new CardAndPointsQueryPresenter();
    }

    View.OnTouchListener mOnTouchListener = new View.OnTouchListener() {
        @Override
        public boolean onTouch(View v, MotionEvent event) {
            if (event.getAction() == MotionEvent.ACTION_UP) {
                KeyBoardView keyboard = txtPassword.getKeyboard();
                keyboard.hideKeyboard();
            }
            return false;
        }
    };

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                txtCardNo.setText(s);
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    /**
     * 接收Activity结果值
     *
     * @param requestCode 请求编号。
     * @param resultCode  返回编号。
     * @param data        返回数据。
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            txtCardNo.setText(mScanner.getScanResult(data));
        }
        if (requestCode == ActivityConstants.REQUEST_TO_CARD_SECURITY_INPUT && resultCode == ActivityConstants.RESULT_TO_CARD_SECURITY_INPUT) {
            CardSecurity cardSecurity = (CardSecurity) data.getSerializableExtra(CardSecurityInputActivity.CARD_SECURITY);
            mPresenter.queryCard(cardNo, cardSecurity);
        }
    }

    @Override
    protected void onCardRead(CardNo cardNo) {
        this.cardNo = cardNo;
        mPresenter.verify(cardNo, payMethod);
    }

    @OnClick(R.id.txtCardNo)
    public void onViewClicked() {
        scan();
    }

    @OnClick({R.id.btnGoBack, R.id.btnLogin})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.btnGoBack:
                onBackPressed();
                break;
            case R.id.btnLogin:
                veriy();
                break;
            default:
                break;
        }
    }

    private void veriy() {
        String no = txtCardNo.getText().toString().trim();
        if (StringUtils.isEmpty(no)) {
            showHintDialog("请输入手机号或会员卡号");
            return;
        }
        cardNo = CardNo.MobileOrDefault(no);
        mPresenter.verify(cardNo, payMethod);
    }

    @Override
    public void verifySuccess(VerifyType verifyType) {
        if (!verifyType.needVerify()) {
            mPresenter.queryCard(cardNo, null);
        } else {
            StartActivityService.startCardSecurityInputActivity(CardBalanceCardAndPointsQueryActivity.this, false, verifyType);
        }
    }

    @Override
    public void queryCardSuccess(Card card) {
        Bundle bundle = new Bundle();
        bundle.putSerializable("card", card);

        Intent login = new Intent(this, CardBalanceActivity.class);
        login.putExtras(bundle);

        startActivity(login);
        finish();
    }

    @Override
    public void queryPointsSuccess(VipCard card) {

    }

    @Override
    public void showErrorMsg(String msg) {
        showHintDialog(msg);
        CardBalanceCardAndPointsQueryActivity.super.restartListen();
    }
}
