/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.card;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;

import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.cache.Cache;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1CardBlockPasswordCallback;
import com.hisense.pos.card.M1CardReaderProvider;
import com.hisense.pos.card.boc.BOCCardReader;
import com.hisense.pos.card.ccb.CcbCardReader;
import com.hisense.pos.card.hi98.Hi98CardReader;
import com.hisense.pos.card.hk316.Hk316CardReader;
import com.hisense.pos.card.icbc.ICBCCardReader;
import com.hisense.pos.card.ums.UmsCardReader;
import com.hisense.pos.print.boc.BOCDeviceHelper;
import com.hisense.pos.print.ccb.CCBDeviceHelper;
import com.hisense.pos.print.icbc.ICBCDeviceHelper;
import com.hisense.webapi.Callback;
import com.landi.bocapilibrary.DeviceService;

/**
 * 提供磁条卡/射频卡读卡的 Activity 基类。
 *
 * @author zhangsai    2018-02-02 HIPOSAPP-315
 * @author xieliyan    2018-04-09 HIPOSAPP-382 修复由于App.getContext空指针异常导致的闪退问题
 * @author zhangsai    2018-01-08 HIPOSAPP-649 增加未获取射频卡密码提示
 * @author zhangsai    2018-01-08 HIPOSAPP-676 修复联迪设备上面会员登录偶发性闪退
 * @author wangyaogang 2019-05-24 HIPOSAPP-1118 因温州时代-银联商务的K9设备正常使用,仍提示错误,注掉提示信息
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 储值卡/积分卡操作增加校验码选项
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 更改为根据接口返回结果确定验证类型,校验参数封装为CardSecurity
 * @author wangyaogang 2020-01-10 HIAPOSCMP-482 支持316磁条卡/非接卡刷卡功能
 */
public class CardBaseActivity extends BaseActivity {

    private ICardReader mCardReader;
    private ProgressDialog dialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (Config.isHk316() || Config.isHt616()) {
            mCardReader = new Hk316CardReader(new HiposM1CardReaderProvider(),
                    GeneralSettings.INSTANCE.getIcCardVersion().value());
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS()) {
            if (GeneralSettings.INSTANCE.getDeviceType().isLANDI()) {
                mCardReader = new UmsCardReader(
                        GeneralSettings.INSTANCE.getIcCardVersion().value(),
                        new HiposM1CardReaderProvider());
            } else if (GeneralSettings.INSTANCE.getDeviceType().isHI98()) {
                mCardReader = new Hi98CardReader(
                        new HiposM1CardReaderProvider(),
                        GeneralSettings.INSTANCE.getIcCardVersion().value());
            }
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isCCB()) {
            mCardReader = new CcbCardReader(
                    CCBDeviceHelper.getInstance().getDeviceService(),
                    new HiposM1CardReaderProvider(),
                    GeneralSettings.INSTANCE.getIcCardVersion().value());
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isICBC()) {
            //工行读卡服务
            mCardReader = null;
            mCardReader = new ICBCCardReader(
                    ICBCDeviceHelper.getInstance().getDeviceService(),
                    new HiposM1CardReaderProvider(),
                    GeneralSettings.INSTANCE.getIcCardVersion().value(),
                    GeneralSettings.INSTANCE.getDeviceType().value());
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
            if (TextUtils.isEmpty(Cache.icCardPassword())) {
                CommonTools.toastLong("无法获取射频卡密码,请退出重试!");
                return;
            }
            //showProgressDialog("初始化...");
            BOCDeviceHelper.getInstance().bind();
            LoggerUtils.getLogger().debug("1服务状态:" + DeviceService.Holder.getInstance().isConnection());
            mHandler.sendMessageDelayed(mHandler.obtainMessage(), 100);
        }

        readCard();
    }

    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            if (DeviceService.Holder.getInstance().isConnection()) {
                hideProgressDialog();
                //中行版设备读卡服务
                LoggerUtils.getLogger().debug("服务状态:" + DeviceService.Holder.getInstance().isConnection());
                mCardReader = new BOCCardReader(
                        new HiposM1CardReaderProvider(),
                        GeneralSettings.INSTANCE.getIcCardVersion().value());

                readCard();
            } else {
                mHandler.sendMessageDelayed(mHandler.obtainMessage(), 100);
            }
        }
    };

    /**
     * 启动读卡线程
     */
    private void readCard() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                if (mCardReader != null) {
                    try {
                        mCardReader.open(CardBaseActivity.this);

                        if (TextUtils.isEmpty(Cache.icCardPassword())) {
                            //CommonTools.toastLong("暂时无法使用射频卡,请退出重试!");
                            LoggerUtils.getLogger().debug("射频卡密码为空,无法使用射频卡");
                        }

                        mCardReader.read(Cache.icCardPassword(), (final String s) -> {
                            LoggerUtils.getLogger().debug("===卡号====" + s);
                            runOnUiThread(() -> onCardRead(CardNo.Default(s)));
                            if (!CommonUtils.isStdStyle()) {
                                mCardReader.stop();
                            }
                        });
                    } catch (Exception ex) {
                        onError("onCreate", ex);
                    }
                }
            }
        }).start();

    }

    public void showProgressDialog(String text) {
        hideProgressDialog();
        dialog = new ProgressDialog(this);
        dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        dialog.setMessage(text);
        dialog.setIndeterminate(false);
        dialog.setCancelable(false);
        dialog.show();
    }

    public void hideProgressDialog() {
        if (dialog != null) {
            dialog.dismiss();
            dialog = null;
        }
    }

    /**
     * 刷卡结束关闭刷卡设备
     */
    @Override
    protected void onDestroy() {
        LoggerUtils.getLogger().debug("cardbase_onDestroy");
        if (mCardReader != null) {
            try {
                mCardReader.close();
            } catch (Exception ex) {
                onError("onDestroy", ex);
            }
        }
        if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
            BOCDeviceHelper.getInstance().unBind();
        }
        super.onDestroy();
    }

    /**
     * 当查询界面暂停时停止读卡。
     */
    @Override
    protected void onPause() {
        super.onPause();

        if (mCardReader != null) {
            try {
                mCardReader.stop();
            } catch (Exception ex) {
                onError("onPause", ex);
            }
        }
    }

    /**
     * 当查询界面恢复时可再次调用刷卡
     */
    @Override
    protected void onResume() {
        super.onResume();

        if (mCardReader != null) {
            try {
                mCardReader.start();
            } catch (Exception ex) {
                onError("onResume", ex);
            }
        }
    }

    protected void onError(String prefix, Exception ex) {
        //因温州时代-银联商务 K9设备 正常使用,仍提示错误信息,注掉
        //CommonTools.toastShort(ex.getMessage());
        LoggerUtils.getLogger().debug(prefix + ": " + ex.toString());
    }

    protected void onCardRead(CardNo cardNo) {

    }

    protected void restartListen() {
        if (mCardReader != null) {
            mCardReader.start();
        } else {
            LoggerUtils.getLogger().debug("mCardReader = null");
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == 3) {
            this.restartListen();
        }
    }

    private class HiposM1CardReaderProvider implements M1CardReaderProvider {
        @Override
        public void getBlockPassword(final String icNo, final M1CardBlockPasswordCallback callback) {
            // 在 UI 线程运行，方便调用 Loading 等 UI 控件显示状态。但是此处显示 Loading 的效果不好，用户会
            // 以为已经完成读卡，把卡拿开。而实际上，此时只是刚刚读取第一个区块，还没有真正读取卡号区块。
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    VipService.getMifare1CardPassword(icNo, Context.cashier(), new Callback<String>() {
                        @Override
                        public void onSuccess(String s) {
                            callback.onSuccess(s);
                        }

                        @Override
                        public void onError(Throwable throwable) {
                            callback.onSuccess(null);
                        }
                    });
                }
            });
        }
    }
}
