package com.hisense.hiaposcmp.activity.ysy.card;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.os.Bundle;
import android.os.PersistableBundle;

import com.hisense.common.boot.SingleCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.boot.ActivityCollector;
import com.hisense.hiaposcmp.boot.BootReceiver;
import com.hisense.hiaposcmp.dialog.AlertAliDialog;
import com.hisense.hiaposcmp.dialog.AlertStdDialog;
import com.hisense.hiaposcmp.dialog.AlertWxDialog;
import com.hisense.hiaposcmp.dialog.HintAliDialog;
import com.hisense.hiaposcmp.dialog.LoadingDialog;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.SysBarUtils;
import com.hisense.hiaposcmp.view.IView.IBaseView;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;

import java.lang.ref.WeakReference;

/**
 * @author dukunbiao(null)  2019-07-22  重构
 * @author xieliyan         2019-11-07  全屏控制
 * @author wangyaogang      2020-01-06  HIAPOSCMP-473  增加标准版提示框
 */
public abstract class CardBaseMvpActivity<V extends IBaseView, T extends BasePresenter<V>> extends CardBaseActivity implements IBaseView {

    protected T mPresenter;

    public LoadingDialog loadingDialog;
    private ProgressDialog dialog;

    private static long lastClickTime = 0;

    private static final int SPACE_TIME = 1000;

    protected MainSaleModel mainSaleModel = MainSaleModel.getInstance();

    public MainSaleModel getMainSaleModel() {
        return mainSaleModel;
    }

    public SaleModel getSale() {
        if (mainSaleModel.getSaleModel() == null) {
            return new SaleModel();
        }
        return mainSaleModel.getSaleModel();
    }

    public ReturnSaleModel getReturnSale() {
        if (mainSaleModel.getReturnSaleModel() == null) {
            return new ReturnSaleModel();
        }
        return mainSaleModel.getReturnSaleModel();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        mPresenter = createPresenter();
        mPresenter.attachView((V) this);
    }

    protected abstract T createPresenter();

    @Override
    protected void onDestroy() {
        if (mPresenter != null) {
            mPresenter.detachView();
        }

        super.onDestroy();
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        if (CommonUtils.isUniformStyle() || CommonUtils.isStdStyle()) {
            SysBarUtils.banStatusBar(this);
            SysBarUtils.hideSystemUi(this);
        }
    }

    @Override
    public void onSaveInstanceState(Bundle outState, PersistableBundle outPersistentState) {
        super.onSaveInstanceState(outState, outPersistentState);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (mPresenter == null) {
            mPresenter = createPresenter();
        }
    }

    @Override
    public Activity getActivity() {
        WeakReference<Activity> weakReference = new WeakReference<>(this);
        return weakReference.get();
    }

    @Override
    public Context getContext() {
        return this;
    }

    @Override
    public void finishThisActivity() {
        this.finish();
    }

    @Override
    public void performQuit() {
        ActivityCollector.finishAll();
        exitApp();
    }

    @Override
    public boolean performQuitBack(String value) {
        if (CommonUtils.performQuitBack(value)) {
            performQuit();
            return true;
        }

        return false;
    }

    @Override
    public void showLoadingDialog() {
        if (loadingDialog == null) {
            loadingDialog = new LoadingDialog(getContext());
        }
        runOnUiThread(() -> {
            try {
                loadingDialog.show();
            } catch (Exception e) {
                //重置为null,即使偶尔内存泄漏，也不能让对话框下次还不出现
                loadingDialog = null;
                loadingDialog = new LoadingDialog(getContext());
                LoggerUtils.getLogger().error(e);
            }
        });
    }

    @Override
    public void hideLoadingDialog() {
        if (loadingDialog != null) {
            runOnUiThread(() -> {
                try {
                    loadingDialog.dismiss();
                    loadingDialog = null;
                } catch (Exception e) {
                    LoggerUtils.getLogger().error(e);
                }
            });
        }
    }

    @Override
    public void showProgressDialog(String text) {
        hideProgressDialog();
        showProgressDialogNoHide(text);
    }

    @Override
    public void showProgressDialogNoHide(String text) {
        runOnUiThread(() -> {
            try {
                if (dialog == null) {
                    dialog = new ProgressDialog(this);
                }
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                dialog.setMessage(text);
                dialog.setIndeterminate(false);
                dialog.setCancelable(false);
                dialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error(e);
            }
        });
    }

    @Override
    public void hideProgressDialog() {
        runOnUiThread(() -> {
            try {
                if (dialog != null) {
                    dialog.dismiss();
                    dialog = null;
                }
            } catch (Exception e) {
                LoggerUtils.getLogger().error(e);
            }
        });
    }

    @Override
    public void showHintDialog(String msg) {
        showHintDialog("注意", msg);
    }

    @Override
    public void showHintDialog(String title, String msg) {
        if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            showAliStyleHintDialog(title, msg);
        } else if (CommonUtils.isWxStyle()) {
            showWxStyleHintDialog(title, msg, false);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintDialog(title, msg);
        } else {
            showAliStyleHintDialog(title, msg, (m) -> {

            });
        }
    }

    @Override
    public void showHintDialog(String msg, SingleCallback callback) {
        if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            showAliStyleHintDialog("提示", msg, callback);
        } else if (CommonUtils.isWxStyle()) {
            showWxStyleHintDialog("提示", msg, callback);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintDialog("提示", msg, callback);
        } else {
            showAliStyleHintDialog("提示", msg, callback);
        }
    }

    @Override
    public void showHintDialog(String title, String msg, boolean isExit) {
        if (CommonUtils.isAliStyle() || CommonUtils.isAliLandStyle()) {
            showAliStyleHintDialog(title, msg);
        } else if (CommonUtils.isWxStyle()) {
            showWxStyleHintDialog(title, msg, isExit);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintDialog(title, msg);
        } else {
            showAliStyleHintDialog(title, msg, (m) -> {

            });
        }
    }

    @Override
    public void showHintConfirm(String msg, UniCallback callback) {
        showHintConfirm("提示", msg, callback);
    }

    @Override
    public void showHintConfirm(String title, String msg, UniCallback callback) {
        if (CommonUtils.isWxStyle()) {
            showWxStyleHintConfirm(title, msg, "确定", "取消", callback, true);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintConfirm(title, msg, "确定", "取消", callback);
        } else {
            showAliStyleHintConfirm(title, msg, "确定", "取消", callback);
        }
    }

    @Override
    public void showHintConfirm(String title, String msg, String yesBtn, String noBtn, UniCallback callback) {
        if (CommonUtils.isWxStyle()) {
            showWxStyleHintConfirm(title, msg, yesBtn, noBtn, callback, true);
        } else if (CommonUtils.isStdStyle()) {
            showStdStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        } else {
            showAliStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        }
    }

    @Override
    public void showHintConfirm(String style, String title, String msg, String yesBtn, String noBtn, UniCallback callback) {
        if (CommonUtils.isWxStyle(style)) {
            showWxStyleHintConfirm(title, msg, yesBtn, noBtn, callback, true);
        } else if (CommonUtils.isStdStyle(style)) {
            showStdStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        } else {
            showAliStyleHintConfirm(title, msg, yesBtn, noBtn, callback);
        }
    }

    public void showAliStyleHintDialog(String title, String message) {
        if (!mPresenter.isViewAttached()) {
            if (message != null && title != null) {
                CommonTools.toastLong(title + "\n" + message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null || title == null) {
            LoggerUtils.getLogger().info("prompt call error, message or title null!");
            return;
        }

        runOnUiThread(() -> {
            try {
                HintAliDialog commHintDialog = new HintAliDialog(mPresenter.getView().getContext());
                if (getActivity(commHintDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                commHintDialog.show();
                commHintDialog.setBasicSytle(HintAliDialog.STYLE_FAILURE);
                commHintDialog.setTextMainHead(title);
                commHintDialog.setReason(message);
            } catch (Exception e) {
                LoggerUtils.getLogger().error("调用AntHintDialog出错:" + e.toString());
                CommonTools.toastLong(title + "\n" + message);
            }
        });
    }

    public void showAliStyleHintDialog(String title, String message, SingleCallback callback) {
        runOnUiThread(() -> {
            AlertAliDialog alertDialog = new AlertAliDialog(this, false);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener("确定", () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.show();
        });
    }

    public void showAliStyleHintConfirm(String title, String message, String yesBtn, String noBtn, UniCallback callback) {
        runOnUiThread(() -> {
            AlertAliDialog alertDialog = new AlertAliDialog(this, true);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener(yesBtn, () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.setNoOnclickListener(noBtn, () -> {
                alertDialog.dismiss();
                callback.onError("");
            });
            alertDialog.show();
        });
    }

    public void showWxStyleHintDialog(String title, String message, boolean isExit) {
        if (!mPresenter.isViewAttached()) {
            if (message != null && title != null) {
                CommonTools.toastLong(title + "\n" + message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null || title == null) {
            LoggerUtils.getLogger().info("prompt call error, message or title null!");
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertWxDialog myDialog = new AlertWxDialog(mPresenter.getView().getContext(), false, true, isExit);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener("确定", () -> myDialog.dismiss());
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("调用WxAlertDialog出错:" + e.toString());
                CommonTools.toastLong(title + "\n" + message);
            }
        });
    }

    public void showWxStyleHintDialog(String title, String message, SingleCallback callback) {
        if (!mPresenter.isViewAttached()) {
            if (message != null) {
                CommonTools.toastLong(message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null) {
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertWxDialog myDialog = new AlertWxDialog(mPresenter.getView().getContext(), false, true);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener("确定", () -> {
                    myDialog.dismiss();
                    if (callback != null) {
                        callback.onSuccess("");
                    }
                });
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("showWxStyleHintDialog异常:" + e.toString());
                CommonTools.toastLong(message);
            }
        });
    }

    public void showWxStyleHintConfirm(String title, String message, String yesBtn, String noBtn, UniCallback callback, boolean isClick) {
        if (!mPresenter.isViewAttached()) {
            if (message != null) {
                CommonTools.toastLong(message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null) {
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertWxDialog myDialog = new AlertWxDialog(mPresenter.getView().getContext(), true, isClick);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().debug("调用activity出错");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener(yesBtn, () -> {
                    myDialog.dismiss();
                    if (callback != null) {
                        callback.onSuccess("");
                    }
                });
                myDialog.setNoOnclickListener(noBtn, () -> {
                    myDialog.dismiss();
                    if (callback != null) {
                        callback.onError("");
                    }
                });
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("showWxStyleHintDialog异常:" + e.toString());
                CommonTools.toastLong(message);
            }
        });
    }

    public void showStdStyleHintDialog(String title, String message) {
        if (!mPresenter.isViewAttached()) {
            if (message != null && title != null) {
                CommonTools.toastLong(title + "\n" + message);
            }
            LoggerUtils.getLogger().debug("activity被finish");
            return;
        }

        if (message == null || title == null) {
            LoggerUtils.getLogger().info("prompt call error, message or title null!");
            return;
        }

        runOnUiThread(() -> {
            try {
                AlertStdDialog myDialog = new AlertStdDialog(mPresenter.getView().getContext(), false);
                if (getActivity(myDialog.getContext()).isFinishing()) {
                    LoggerUtils.getLogger().error("调用AlertStdDialog出错,activity消失");
                    return;
                }
                myDialog.setTitle(title);
                myDialog.setMessage(message);
                myDialog.setYesOnclickListener("确定", () -> myDialog.dismiss());
                myDialog.show();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("调用AlertStdDialog出错:" + e.toString());
                CommonTools.toastLong(title + "\n" + message);
            }
        });
    }

    public void showStdStyleHintDialog(String title, String message, SingleCallback callback) {
        runOnUiThread(() -> {
            AlertStdDialog alertDialog = new AlertStdDialog(this, false);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener("确定", () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.show();
        });
    }

    public void showStdStyleHintConfirm(String title, String message, String yesBtn, String noBtn, UniCallback callback) {
        runOnUiThread(() -> {
            AlertStdDialog alertDialog = new AlertStdDialog(this, true);
            alertDialog.setTitle(title);
            alertDialog.setMessage(message);
            alertDialog.setYesOnclickListener(yesBtn, () -> {
                alertDialog.dismiss();
                callback.onSuccess("");
            });
            alertDialog.setNoOnclickListener(noBtn, () -> {
                alertDialog.dismiss();
                callback.onError("");
            });
            alertDialog.show();
        });
    }

    @Override
    public void isNotShowExitDialog(boolean bool) {

    }

    public static boolean isFastClick() {
        long currentTime = System.currentTimeMillis();
        boolean isFastClick;
        isFastClick = (currentTime - lastClickTime < SPACE_TIME);
        lastClickTime = currentTime;
        LoggerUtils.getLogger().debug("快速点击" + lastClickTime);
        return isFastClick;
    }

    public static boolean isFastClick(int spaceTime) {
        long currentTime = System.currentTimeMillis();
        boolean isFastClick;
        isFastClick = (currentTime - lastClickTime < spaceTime);
        lastClickTime = currentTime;
        LoggerUtils.getLogger().debug("快速点击" + lastClickTime);
        return isFastClick;
    }

    private Activity getActivity(Context context) {
        while (!(context instanceof Activity) && context instanceof ContextWrapper) {
            context = ((ContextWrapper) context).getBaseContext();
        }
        if (context instanceof Activity) {
            return (Activity) context;
        } else {
            return null;
        }
    }

    protected void exitApp() {
        BootReceiver.setBootUp(false);

        finishApp();
        finishAffinity();
        System.exit(0);
    }

    private void finishApp() {
        Intent startMain = new Intent(Intent.ACTION_MAIN);
        startMain.addCategory(Intent.CATEGORY_HOME);
        startMain.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(startMain);
    }
}
