/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.card;

import android.content.Intent;
import android.os.Bundle;
import android.text.InputType;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RelativeLayout;

import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 卡号校验码对话框。
 *
 * @author xieliyan    2017-04-07
 * @author zhangsai    2017-11-15 HIPOSAPP-212
 * @author xieliyan    2017-12-08
 * @author xieliyan
 * @author wangyaogang 2019-05-29 HIPOSAPP-1135 可配置付款码查询时是否需要密码
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 调整页面为校验码,卡面后几位,密码三个选项,根据开关设置显示对应内容
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 储值卡/积分卡更改为根据接口确定验证类型(动态付款码仍根据本地开关确定),校验参数封装为CardSecurity
 */
@ActivityLayoutInject(R.layout.activity_card_security_input)
public class CardSecurityInputActivity extends BaseActivity {

    public static final String CARD_SECURITY = "cardSecurity";

    @BindView(R.id.rl_Verify)
    public RelativeLayout rlVerify;

    @BindView(R.id.rl_FaceNo)
    public RelativeLayout rlFaceNo;

    @BindView(R.id.rl_password)
    public RelativeLayout rlPassword;

    @BindView(R.id.edtVerify)
    public EditText edtVerify;

    @BindView(R.id.edtFaceNo)
    public EditText edtFaceNo;

    @BindView(R.id.edtPassword)
    public EditText edtPassword;

    @BindView(R.id.btnCommit)
    public Button btnCommit;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        boolean iseCard = getIntent().getBooleanExtra("iseCard", false);
        VerifyType verifyType = (VerifyType) getIntent().getSerializableExtra("verifyType");

        if (iseCard) {
            rlPassword.setVisibility(View.VISIBLE);
            edtPassword.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
        } else {
            if (verifyType.isVerifyCode()) {
                rlVerify.setVisibility(View.VISIBLE);
                edtVerify.setHint(String.format("请输入%s位校验码", verifyType.verifyCodeLength()));
                edtVerify.setInputType(InputType.TYPE_CLASS_NUMBER);
            }
            if (verifyType.isVerifyFaceNoCheck()) {
                rlFaceNo.setVisibility(View.VISIBLE);
                edtFaceNo.setHint(String.format("请输入卡面后%s位", verifyType.cardFaceNoCheckLength()));
                edtFaceNo.setInputType(InputType.TYPE_CLASS_NUMBER);
            }
            if (verifyType.isVerifyPassword()) {
                rlPassword.setVisibility(View.VISIBLE);
                edtPassword.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
            }
        }
    }

    @OnClick(R.id.btnCommit)
    public void onClickCommit() {
        btnCommit.setEnabled(false);

        CardSecurity security = new CardSecurity.Builder()
                .cardVerifyCode(edtVerify.getText().toString().trim())
                .cardFaceNoCheck(edtFaceNo.getText().toString().trim())
                .password(edtPassword.getText().toString().trim())
                .build();
        Intent mIntent = new Intent();
        mIntent.putExtra(CardSecurityInputActivity.CARD_SECURITY, security);
        setResult(ActivityConstants.RESULT_TO_CARD_SECURITY_INPUT, mIntent);
        finish();
    }

    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        btnCommit.setEnabled(true);
        finish();
    }

}
