/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.pay;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.TextView;

import com.hisense.crm.card.Card;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.StartActivityService;
import com.hisense.hiaposcmp.activity.ysy.card.CardBaseMvpActivity;
import com.hisense.hiaposcmp.activity.ysy.card.CardSecurityInputActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.pay.CardPayPresenter;
import com.hisense.hiaposcmp.view.IView.ICardPayView;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 刷卡
 *
 * @author zhenghao    2017-03-20
 * @author xieliyan    2017-09-26 HIPOSAPP-160 刷磁条卡（HI98）
 * @author xieliyan    2017-09-28 HIPOSAPP-160 射频卡（HI98）
 * @author xieliyan    2017-10-31 HIPOSAPP-201 使用枚举实现单例模式:更改调用方法
 * @author xieliyan    2017-11-22 HIPOSAPP-224 支持射频卡2.0
 * @author xieliyan    2018-01-16 HIPOSAPP-278 调用联迪驱动刷卡
 * @author xieliyan    2018-04-09 HIPOSAPP-382 修复由于App.getContext空指针异常导致的闪退问题
 * @author guolina     2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author dukunbiao(null)  2019-10-16  重构
 * @author wangyaogang 2019-04-09 HIAPOSCMP-400 付款码扫码返回不关闭页面
 */
@ActivityLayoutInject(R.layout.activity_swing_card)
public class CardPayActivity extends CardBaseMvpActivity<ICardPayView, CardPayPresenter> implements ICardPayView {

    @BindView(R.id.hintText)
    public TextView textViewHint;

    @BindView(R.id.btnScan)
    public Button btnScan;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @Override
    protected CardPayPresenter createPresenter() {
        return new CardPayPresenter();
    }

    private void init() {
        textViewHint.setText("请刷卡");
        mScanner = getScanner(this);

        mPresenter.initSelectedPayMethod(this.getIntent());
    }

    @OnClick(R.id.btnScan)
    public void onClickScan() {
        scan();
    }

    @Override
    protected void onCardRead(CardNo cardNo) {
        mPresenter.queryVerifyType(cardNo);
    }

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();

                mPresenter.queryVerifyType(CardNo.PayCode(s));
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
                CardPayActivity.super.restartListen();
            }
        });
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            mPresenter.queryVerifyType(CardNo.PayCode(mScanner.getScanResult(data)));
        } else if ((requestCode == ActivityConstants.REQUEST_TO_CARD_SECURITY_INPUT) && (resultCode == ActivityConstants.RESULT_TO_CARD_SECURITY_INPUT)) {
            CardSecurity cardSecurity = (CardSecurity) data.getSerializableExtra(CardSecurityInputActivity.CARD_SECURITY);

            mPresenter.cardQuery(cardSecurity);
        } else if (requestCode == ActivityConstants.REQUEST_TO_CARD_PAY_SHOW && resultCode == ActivityConstants.RESULT_TO_CARD_PAY_SHOW) {
            mPresenter.cardPay(data);
        } else if (requestCode == ActivityConstants.REQUEST_CARD_BALANCE_QUERY && resultCode == ActivityConstants.RESULT_CARD_BALANCE_QUERY_COMMIY) {
            mPresenter.cardRefund(data);
        } else {
            finish();
        }
    }

    @Override
    public void needVerifyType(boolean iseCard, VerifyType verifyType) {
        StartActivityService.startCardSecurityInputActivity(this, iseCard, verifyType);
    }

    @Override
    public void cardQuerySuccess(Card card) {
        StartActivityService.gotoCardPayShowActivity(this, card, mPresenter.getSale().getTotalUnPaid());
    }

    @Override
    public void cardQueryRefund(Card card, String fSsTotal, String maxRefund) {
        StartActivityService.gotoCardReturnQueryActivity(this, "Card", card, null, fSsTotal, maxRefund, "");
    }

    @Override
    public void cardPaySuccess() {
        setResult(ActivityConstants.RESULT_PAY_TO_CARD_SUCCESS);
        finish();
    }

    @Override
    public void cardRefundSuccess() {
        setResult(ActivityConstants.RESULT_REFUND_TO_CARD_SUCCESS);
        finish();
    }

    @Override
    public void queryVerifyTypeFail(Throwable throwable) {
        CardPayActivity.super.restartListen();
    }
}
