/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.pay;

import android.content.Intent;
import android.os.Bundle;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.Card;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hiaposcmp.tool.UpdateTextButton;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 储值余额查询显示
 *
 * @author zhenghao    2017-03-30
 * @author xieliyan    2017-10-30 HIPOSAPP-196 确定按钮点击一次后置为不可用
 * @author xieliyan    2017-12-21 HIPOSAPP-254 控制支付金额的大小更改
 * @author wangyaogang 2019-03-07 HIPOSAPP-1040 修复储值卡/积分卡支付时,金额正确但提示"请输入正确的金额"的问题
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author dukunbiao(null)  2019-10-16  重构
 */
@ActivityLayoutInject(R.layout.activity_card_pay_query)
public class CardPayShowActivity extends BaseActivity {

    @BindView(R.id.payTotal)
    public EditText txtPayTotal;

    @BindView(R.id.commit_action)
    public Button btnCommit;

    @BindView(R.id.usableTotal)
    public TextView txtUsableTotal;

    @BindView(R.id.CardTypeName)
    public TextView txtCardTypeName;

    @BindView(R.id.cardFaceNo)
    public TextView txtCardFaceNo;

    @BindView(R.id.expireDate)
    public TextView txtExpireDate;

    @BindView(R.id.cancel_action)
    public Button btnCancel;

    private String total = "0";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        Bundle bundle = this.getIntent().getExtras();
        if (bundle == null) {
            CommonTools.toastShort("获取卡信息失败，请重新操作");
            return;
        }

        Card card = (Card) bundle.getSerializable("card");
        if (card == null) {
            CommonTools.toastShort("获取卡信息失败，请重新操作");
            return;
        }

        txtUsableTotal.setText(String.format(Locale.CHINESE, "%.2f", card.usableTotal()));
        txtCardTypeName.setText(card.cardType().name());
        txtCardFaceNo.setText(card.cardFaceNo());
        txtExpireDate.setText(card.expireDate());

        total = bundle.getString("fSsTotal");
        final double usableTotal = card.usableTotal();
        UpdateTextButton.updateTotalText(txtPayTotal, total, usableTotal);
        UpdateTextButton.setOnTextChange(txtPayTotal, total, usableTotal);
        txtPayTotal.setInputType(EditorInfo.TYPE_CLASS_PHONE);
        txtPayTotal.setSelection(txtPayTotal.getText().length());
        NumberHandler.setPoint(txtPayTotal, 2);//只允许输入两位小数
    }

    @OnClick(R.id.cancel_action)
    public void onClickCancel() {
        finish();
    }

    @OnClick(R.id.commit_action)
    public void onClickCommit() {
        if (StringUtils.isEmpty(txtPayTotal.getText().toString())) {
            CommonTools.toastShort(CardPayShowActivity.this, "请输入支付金额");
            return;
        }

        btnCommit.setEnabled(false);
        String cardYfTotal = txtPayTotal.getText().toString();
        String wfTotal = BigDecimalUtils.subtract(total, cardYfTotal);
        String canUse = BigDecimalUtils.subtract(txtUsableTotal.getText().toString(), cardYfTotal);
        if (BigDecimalUtils.isLess0(wfTotal) || BigDecimalUtils.isLess0(canUse)) {
            LoggerUtils.getLogger().debug("超出余额");
            btnCommit.setEnabled(true);
            CommonTools.toastShort(CardPayShowActivity.this, "请输入正确的金额");
            return;
        }

        Intent mIntent = new Intent();
        mIntent.putExtra("wfTotal", wfTotal);
        mIntent.putExtra("cardYfTotal", cardYfTotal);
        setResult(ActivityConstants.RESULT_TO_CARD_PAY_SHOW, mIntent);
        finish();
    }
}
