/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.pay;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.vip.VerifyType;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.pay.CouponPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.ICouponView;
import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import java.util.Collection;
import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 添加单张赠券
 *
 * @author xieliyan    2018-03-07 HIPOSAPP-348  一段式赠券单笔添加
 * @author guolina     2019-01-09 HIPOSAPP-967  万科会员券校验码可以为空
 * @author guolina     2019-01-16 HIPOSAPP-978  券计算过程增加等待窗口
 * @author guolina     2019-02-25 HIPOSAPP-1029 去掉TicketPluBean.java文件，优化代码
 * @author wangyaogang 2019-02-28 HIPOSAPP-1031 优化扫描结果传值逻辑
 * @author guolina     2019-03-04 HIPOSAPP-1038 修复非会员录入会员券号点击确定时的闪退
 * @author wangyaogang 2019-07-01 HIPOSAPP-1189 使用 IScanner 扫描器实例扫描
 * @author wangyaogang 2019-07-03 HIPOSAPP-1130 添加单张券之前判断券类型是否一致
 */
@ActivityLayoutInject(R.layout.activity_coupon_addition)
public class CouponAdditionActivity extends BaseMvpActivity<ICouponView, CouponPresenter> implements ICouponView {

    @BindView(R.id.edtCouponNo)
    public EditText edtCouponNo;

    @BindView(R.id.edtCheckCode)
    public EditText edtCheckCode;

    @BindView(R.id.btnCancel)
    public Button btnCancel;

    @BindView(R.id.btnConfirm)
    public Button btnConfirm;

    @BindView(R.id.imgScan)
    public ImageView imgScan;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mScanner = getScanner(this);
    }

    @Override
    protected CouponPresenter createPresenter() {
        return new CouponPresenter();
    }

    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        finish();
    }

    @OnClick(R.id.btnConfirm)
    public void onClickConfirm() {
        if (StringUtils.isEmpty(edtCouponNo.getText().toString())) {
            CommonTools.toastShort(this, "请输入券号");
            return;
        }

        Coupon coupon = new Coupon.Builder().coupon(edtCouponNo.getText().toString(), edtCheckCode.getText().toString()).build();
        mPresenter.couponCalc(coupon, -1);
    }

    @OnClick(R.id.imgScan)
    public void onClickScan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                edtCouponNo.setText(s);
                edtCouponNo.setSelection(edtCouponNo.getText().toString().length());
            }

            @Override
            public void onError(final Throwable t) {
               showHintDialog(t.getMessage());
            }
        });
    }

    /**
     * 调用Activity后的处理。
     *
     * @param requestCode 请求编号
     * @param resultCode  返回编号
     * @param data        返回数据
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            edtCouponNo.setText(mScanner.getScanResult(data));
            edtCouponNo.setSelection(edtCouponNo.getText().toString().length());
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    public void couponCalcSuccess(int position, UsableCoupon usableCoupon) {
        Intent intent = new Intent();
        intent.putExtra("usableCoupon", usableCoupon);

        setResult(ActivityConstants.RESULT_COUPON_ADDITION, intent);
        finish();
    }

    @Override
    public void needVerifyType(VerifyType verifyType) {

    }

    @Override
    public void queryVipCouponsSuccess(List<Coupon> coupons) {

    }

    @Override
    public void couponUse1Success(Collection<UsableCoupon> usableCoupons, Collection<Coupon> coupons) {

    }

    @Override
    public void calcCouponsSuccess() {

    }
}
