/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.activity.ysy.pay;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.pay.OtherPayPresenter;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hiaposcmp.view.IView.pay.IOtherPayView;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 其他支付方式录入
 *
 * @author guolina     2019-03-14 HIPOSAPP-1052
 * @author wangyaogang 2019-06-10 HIPOSAPP-1119 使用其他支付方式时,默认带出未支付金额
 * @author wangyaogang 2019-07-01 HIPOSAPP-1189 使用 IScanner 扫描器实例扫描
 * @author dukunbiao(null)  2019-10-18  重构
 */
@ActivityLayoutInject(R.layout.activity_other_paymethod)
public class OtherPayActivity extends BaseMvpActivity<IOtherPayView, OtherPayPresenter> implements IOtherPayView {

    @BindView(R.id.edtCouponNo)
    public EditText edtPayNo;

    @BindView(R.id.edtCheckCode)
    public EditText edtAmount;

    @BindView(R.id.btnCancel)
    public Button btnCancel;

    @BindView(R.id.btnConfirm)
    public Button btnConfirm;

    @BindView(R.id.imgScan)
    public ImageView imgScan;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @Override
    protected OtherPayPresenter createPresenter() {
        return new OtherPayPresenter();
    }

    private void init() {
        NumberHandler.setPoint(edtAmount, 2);
        mScanner = getScanner(this);
        edtAmount.setText(mPresenter.getSale().getTotalUnPaid());
    }

    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        finish();
    }

    @OnClick(R.id.btnConfirm)
    public void onClickConfirm() {
        mPresenter.otherPay(edtAmount.getText().toString().trim(), edtPayNo.getText().toString().trim());
    }

    @OnClick(R.id.imgScan)
    public void onClickScan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                edtPayNo.setText(s);
                edtPayNo.setSelection(edtPayNo.getText().toString().length());
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    /**
     * 调用Activity后的处理。
     *
     * @param requestCode 请求编号
     * @param resultCode  返回编号
     * @param data        返回数据
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            edtPayNo.setText(mScanner.getScanResult(data));
            edtPayNo.setSelection(edtPayNo.getText().toString().length());
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    public void otherPaySuccess() {
        Intent intent = new Intent();
        setResult(ActivityConstants.RESULT_OTHER_PAYMETHOD, intent);
        finish();
    }
}
