/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.pay;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.tool.CommonTools;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 交易完成界面
 *
 * @author xieliyan    2017-11-17
 * @author xieliyan    2018-01-08 HIPOSAPP-266 避免线程阻塞问题
 * @author zhangsai    2018-01-18 HIPOSAPP-282 自动关闭时间由5秒改为3秒，减少收银员等待时间
 * @author guolina     2018-05-30 HIPOSAPP-473 修复点击系统返回键造成的支付无响应问题
 * @author zhangsai    2019-01-16 HIPOSAPP-982 删除 Handler；优化代码
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 */
@ActivityLayoutInject(R.layout.activity_pay_finished)
public class PayFinishedActivity extends BaseActivity {

    private static final String MESSAGE_RESOURCE_ID = "MESSAGE_RESOURCE_ID";

    // 最长等待时间（秒）
    private static final int MAX_SECONDS_UNTIL_FINISHED = 3;

    @BindView(R.id.txtAutoFinish)
    public TextView txtAutoFinish;

    @BindView(R.id.txtMessage)
    public TextView txtMessage;

    private CountDownThread mCountDownThread;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        Bundle bundle = getIntent().getExtras();
        if (bundle != null) {
            int resId = bundle.getInt(MESSAGE_RESOURCE_ID, R.string.msg_pay_finished);
            txtMessage.setText(getResources().getString(resId));
        }

        mCountDownThread = new CountDownThread(this, MAX_SECONDS_UNTIL_FINISHED);
        new Thread(mCountDownThread).start();
    }

    /**
     * 计数。
     *
     * @param secondsUntilFinished 结束前需等待的秒数
     */
    private void countDown(final int secondsUntilFinished) {
        runOnUiThread(() -> {
            txtAutoFinish.setText(String.format(Locale.CHINESE, "(%d 秒后关闭)", secondsUntilFinished));
        });
    }

    @Override
    public void onBackPressed() {
        if (mCountDownThread != null) {
            mCountDownThread.cancel();
            mCountDownThread = null;
        }

        // 后退时也要给予提示信号
        this.setResult(RESULT_OK);
        super.onBackPressed();
    }

    @OnClick(R.id.btnOK)
    public void onClickOk(View v) {
        this.finish();
    }

    @Override
    public void finish() {
        if (mCountDownThread != null) {
            mCountDownThread.cancel();
            mCountDownThread = null;
        }

        this.setResult(RESULT_OK);
        super.finish();
    }

    /**
     * 等待计数线程。
     */
    private static class CountDownThread implements Runnable {
        private static final int ONE_SECOND = 1000;

        private PayFinishedActivity activity;
        // 当前等待时间
        private int currentWaitingTimeInSeconds;
        // 是否取消等待。true 表示已取消。默认为 false
        private boolean isCancelled;

        CountDownThread(PayFinishedActivity activity, int maxWaitingTimeInSeconds) {
            this.activity = activity;
            this.currentWaitingTimeInSeconds = maxWaitingTimeInSeconds;
        }

        @Override
        public void run() {
            try {
                while (currentWaitingTimeInSeconds >= 0) {
                    if (isCancelled) {
                        break;
                    }
                    if (currentWaitingTimeInSeconds > 0) {
                        this.activity.countDown(currentWaitingTimeInSeconds--);
                        Thread.sleep(ONE_SECOND);
                    } else {
                        this.activity.finish();
                    }
                }

            } catch (Exception e) {
                CommonTools.toastShort(this.activity, e.getMessage());
            }
        }

        /**
         * 取消等待。
         */
        public void cancel() {
            isCancelled = true;
            this.activity = null;
        }
    }

    /**
     * 支付完成。
     *
     * @param activity    发起请求的活动
     * @param requestCode 请求代码
     */
    public static void afterPay(Activity activity, int requestCode) {
        LoggerUtils.getLogger().debug("PayFinishedActivity：afterPay");
        Intent intent = new Intent(activity, PayFinishedActivity.class);
        intent.putExtra(MESSAGE_RESOURCE_ID, R.string.msg_pay_finished);
        activity.startActivityForResult(intent, requestCode);
    }

    /**
     * 退款完成。
     *
     * @param activity    发起请求的活动
     * @param requestCode 请求代码
     */
    public static void afterRefund(Activity activity, int requestCode) {
        LoggerUtils.getLogger().debug("PayFinishedActivity：afterRefund");
        Intent intent = new Intent(activity, PayFinishedActivity.class);
        intent.putExtra(MESSAGE_RESOURCE_ID, R.string.msg_refund_finished);
        activity.startActivityForResult(intent, requestCode);
    }
}
