/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.pay;

import android.app.AlertDialog;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.presenter.PaymentChannelPresenter;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.view.IView.IPaymentChannelView;
import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import org.json.JSONException;
import org.json.JSONObject;

import java.math.BigDecimal;
import java.util.Map;
import java.util.regex.Pattern;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 支付通道模块。
 *
 * @author xieliyan    2017-04-13
 * @author zhangsai    2017-11-15
 * @author xieliyan    2017-12-27
 * @author xieliyan    2018-01-02 HIPOSAPP-262 支持建行收单打印/余额查询
 * @author lizhixin    2018-07-04 HIPOSAPP-532 支持工行日结
 * @author lizhixin    2018-07-04 HIPOSAPP-646 支持工行余额查询
 * @author lizhixin    2018-07-12 HIPOSAPP-533 对接工行交易汇总并支持打印
 * @author wangyaogang 2018-11-08 HIPOSAPP-813 增加中行支付渠道
 * @author zhangsai    2018-12-07 HIPOSAPP-888 使用IPaymentServiceProvider提供打印/结算/换班等收单服务
 * @author wangyaogang 2018-12-24 HIPOSAPP-926 日结后进行结算流程优化,增加可配置选项
 * @author guolina     2019-10-25 重构
 * @author wangyaogang 2020-04-15 HIAPOSCMP-673 银联商务支持班次查询
 * @author wangyaogang 2020-04-22 HIAPOSCMP-657 对接银商POS通订单查询接口
 * @author wangyaogang 2020-04-22 HIAPOSCMP-658 对接银商交易明细查询接口
 */
@ActivityLayoutInject(R.layout.activity_payment_channel)
public class PaymentChannelActivity extends BaseMvpActivity<IPaymentChannelView, PaymentChannelPresenter> implements IPaymentChannelView {

    @BindView(R.id.lytPrint)
    LinearLayout lytPrint;

    @BindView(R.id.lytQuery)
    LinearLayout lytQuery;

    @BindView(R.id.lytChange)
    LinearLayout lytChange;

    @BindView(R.id.lytReprint)
    LinearLayout lytReprint;

    @BindView(R.id.lytCCB)
    LinearLayout lytCCB;

    @BindView(R.id.txtTitle)
    TextView txtTitle;

    @BindView(R.id.lytQueryBatchNo)
    LinearLayout lytQueryBatchNo;

    @BindView(R.id.lytTradeQuery)
    LinearLayout lytTradeQuery;

    @BindView(R.id.lytScanTradeQuery)
    LinearLayout lytScanTradeQuery;


    private IPaymentServiceProvider paymentServiceProvider;

    //银商交易查询，使用凭证号可查所有
    private final static int TRADE_QUERY = 1;
    //银商交易查询，特指扫码支付
    private final static int SCAN_TRADE_QUERY = 2;

    //transid
    private final static String TRANS_DETAIL = "交易明细";
    //主要用于pos通
    private final static String QUERY_ORDER = "订单查询";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mScanner = getScanner(this);
        paymentServiceProvider = getPaymentServiceProvider(this);
        updateUi();
    }

    @Override
    protected PaymentChannelPresenter createPresenter() {
        return new PaymentChannelPresenter();
    }

    private void updateUi() {
        if (GeneralSettings.INSTANCE.getPaymentChannel().isCCB()) {
            lytChange.setVisibility(View.GONE);
            lytQueryBatchNo.setVisibility(View.GONE);
            lytScanTradeQuery.setVisibility(View.GONE);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isICBC()) {
            //工行收单
            lytChange.setVisibility(View.GONE);
            lytReprint.setVisibility(View.GONE);
            lytQueryBatchNo.setVisibility(View.GONE);
            lytTradeQuery.setVisibility(View.GONE);
            lytScanTradeQuery.setVisibility(View.GONE);
        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
            lytChange.setVisibility(View.GONE);
            lytPrint.setVisibility(View.GONE);
            lytQuery.setVisibility(View.GONE);
            lytReprint.setVisibility(View.GONE);
            lytQueryBatchNo.setVisibility(View.GONE);
            lytTradeQuery.setVisibility(View.GONE);
            lytScanTradeQuery.setVisibility(View.GONE);
        } else {
            //此处为银商
            lytCCB.setVisibility(View.GONE);
        }

        txtTitle.setText(paymentServiceProvider.providerName());
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            if (ed_trans_no != null) {
                ed_trans_no.setText(mScanner.getScanResult(data));
                query();
            }
        }
        if (paymentServiceProvider.requestCode() == requestCode) {
            try {
                NetUtils.printExtras(data == null ? null : data.getExtras());
            } catch (Exception e) {
                NetUtils.printDetail("打印收单行返回信息", e);
            }
            try {
                Map map = paymentServiceProvider.handleResult(data);
                //失败，已经进catch 提示了，此处只处理成功的  两个交易查询
                //暂定自己不进行解析展示
//                showResult(map);
            } catch (Exception ex) {
                showHintDialog(ex.getMessage());
                LoggerUtils.getLogger().error(ex.getMessage(), ex);
            }
        }

    }

    /**
     * 展示返回结果
     *
     * @param map
     * @throws JSONException
     */
    private void showResult(Map map) throws JSONException {
        if ("0".equals(map.get("resultCode"))) {
            String transId = ((String) map.get("transId"));
            if (!TextUtils.isEmpty(transId)) {
                transId = transId.replaceAll("\\s*", "");
                if (TRANS_DETAIL.equalsIgnoreCase(transId)
                        || QUERY_ORDER.equalsIgnoreCase(transId)) {
                    String sJson = (String) map.get("transData");
                    JSONObject transData = new JSONObject(sJson);
                    StringBuilder sb = new StringBuilder();
                    sb.append("凭证号：" + transData.get("traceNo") + "\n");
                    String cardNo = (String) transData.get("cardNo");
                    if (QUERY_ORDER.equalsIgnoreCase(transId)) {
                        JSONObject memInfo = new JSONObject(transData.get("memInfo").toString());
                        sb.append("商户单号：" + memInfo.getString("orderNo") + "\n");
                    } else if (TRANS_DETAIL.equalsIgnoreCase(transId)
                            && isValidNumber(cardNo)
                            && BigDecimalUtils.isEquals0(cardNo)) {
                        JSONObject memInfo = new JSONObject(transData.get("memInfo").toString());
                        sb.append("商户单号：" + memInfo.getString("orderNo") + "\n");
                    } else {
                        sb.append("卡号：" + cardNo + "\n");
                    }
                    sb.append("交易时间：" + transData.get("date") + " " + transData.get("time") + "\n");
                    sb.append("交易类型：" + transData.get("transName") + "\n");
                    if (QUERY_ORDER.equalsIgnoreCase(transId)) {
                        sb.append("交易金额：¥ " + new BigDecimal((String) transData.get("actualPayAmt")).divide(new BigDecimal("100")).toPlainString() + "\n");
                    } else {
                        sb.append("交易金额：¥ " + transData.get("amt") + "\n");
                    }
                    sb.append("------------------------------------------------------ " + "\n");
                    sb.append(transData.get("resDesc"));
                    showHintDialog("", sb.toString());
                }
            }
        }
    }

    /**
     * 是否纯数字
     *
     * @param cardNo
     * @return
     */
    private boolean isValidNumber(String cardNo) {
        if (TextUtils.isEmpty(cardNo) || cardNo.trim().length() == 0) {
            return false;
        }
        Pattern pattern = Pattern.compile("^[-+]?[\\d]*$");
        return pattern.matcher(cardNo).matches();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @OnClick({R.id.btnGoBack, R.id.lytPrint, R.id.lytQuery, R.id.lytShift, R.id.lytSettlement,
            R.id.lytReprint, R.id.lytQueryBatchNo, R.id.lytTradeQuery, R.id.lytScanTradeQuery})
    public void onViewClicked(View view) {
        if (isFastClick()) {
            return;
        }
        switch (view.getId()) {
            case R.id.btnGoBack:
                onBackPressed();
                break;
            case R.id.lytPrint:
                mPresenter.printTransSummary(paymentServiceProvider);
                break;
            case R.id.lytQuery:
                mPresenter.queryBalance(paymentServiceProvider, PaymentChannelActivity.this);
                break;
            case R.id.lytShift:
                paymentServiceProvider.shift();
                break;
            case R.id.lytQueryBatchNo:
                paymentServiceProvider.queryBatchNo();
                break;
            case R.id.lytTradeQuery: {
//                showQueryDialog(TRADE_QUERY);
                try {
                    paymentServiceProvider.tradeQuery("", false, new PaymentHandler() {
                        @Override
                        public void onFinish(Object... args) {

                        }
                    });
                } catch (Exception ex) {
                    showHintDialog(ex.getMessage());
                    LoggerUtils.getLogger().error(ex.getMessage(), ex);
                }
            }
            break;
            case R.id.lytScanTradeQuery: {
//                showQueryDialog(SCAN_TRADE_QUERY);
                try {
                    paymentServiceProvider.scanTradeQuery("", false, new PaymentHandler() {
                        @Override
                        public void onFinish(Object... args) {

                        }
                    });
                } catch (PaymentProviderException ex) {
                    showHintDialog(ex.getMessage());
                    LoggerUtils.getLogger().error(ex.getMessage(), ex);
                }
            }
            break;
            case R.id.lytSettlement:
                mPresenter.settlement(paymentServiceProvider, PaymentChannelActivity.this);
                break;
            case R.id.lytReprint:
                paymentServiceProvider.reprintSettlement();
                break;
            default:
                break;
        }
    }

    /**
     * 展示弹框
     */
    private AlertDialog queryDialog; //交易查询的弹框
    private EditText ed_trans_no;
    private CheckBox cb_print;
    private TextView tv_error_tip;
    private int queryType = 1;

    private void showQueryDialog(int queryType) {
        this.queryType = queryType;
        LoggerUtils.getLogger().debug("showQueryDialog");
        View view = LayoutInflater.from(PaymentChannelActivity.this).inflate(R.layout.dialog_trade_query, null, false);
        queryDialog = new AlertDialog.Builder(PaymentChannelActivity.this).setView(view).create();
        queryDialog.setCanceledOnTouchOutside(false);

        ed_trans_no = (EditText) view.findViewById(R.id.ed_trans_no);
        cb_print = (CheckBox) view.findViewById(R.id.cb_print);
        tv_error_tip = (TextView) view.findViewById(R.id.tv_error_tip);
        LinearLayout ll_print = (LinearLayout) view.findViewById(R.id.ll_print);
        ll_print.setVisibility(View.GONE);

        //打开系统键盘
        SoftInput.openSoftKeyBoard(PaymentChannelActivity.this);

        Button btn_cancel = (Button) view.findViewById(R.id.btn_cancel);
        Button btn_determine = (Button) view.findViewById(R.id.btn_determine);

        RelativeLayout rlScanVip = (RelativeLayout) view.findViewById(R.id.rlScanVip);
        if (queryType == SCAN_TRADE_QUERY) {
            rlScanVip.setVisibility(View.VISIBLE);
            ed_trans_no.setHint("请输入商户订单号");
        } else {
            rlScanVip.setVisibility(View.INVISIBLE);
            ed_trans_no.setHint("请输入凭证号");
        }

        rlScanVip.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //扫描, 输入商户订单号时需要
                scan();
            }
        });

        btn_determine.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                query();
            }
        });

        btn_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                queryDialog.dismiss();
            }
        });
        queryDialog.show();
    }

    private void query() {
        LoggerUtils.getLogger().debug("查询类型：" + queryType);
        tv_error_tip.setVisibility(View.GONE);
        tv_error_tip.setText("");
        //取凭证号 ，调接口查询
        String transNo = ed_trans_no.getText().toString().trim();
        if (TextUtils.isEmpty(transNo)) {
            tv_error_tip.setVisibility(View.VISIBLE);
            if (queryType == SCAN_TRADE_QUERY) {
                tv_error_tip.setText("请输入商户订单号");
            } else {
                tv_error_tip.setText("请输入凭证号");
            }
            return;
        }

        switch (queryType) {
            case SCAN_TRADE_QUERY: {
                // pos通查询
                try {
                    paymentServiceProvider.scanTradeQuery(transNo, !cb_print.isChecked(), new PaymentHandler() {
                        @Override
                        public void onFinish(Object... args) {

                        }
                    });
                } catch (PaymentProviderException ex) {
                    showHintDialog(ex.getMessage());
                    LoggerUtils.getLogger().error(ex.getMessage(), ex);
                }
            }
            break;
            case TRADE_QUERY:
            default: {
                try {
                    paymentServiceProvider.tradeQuery(transNo, !cb_print.isChecked(), new PaymentHandler() {
                        @Override
                        public void onFinish(Object... args) {

                        }
                    });
                } catch (Exception ex) {
                    showHintDialog(ex.getMessage());
                    LoggerUtils.getLogger().error(ex.getMessage(), ex);
                }
            }
            break;
        }

        if (queryDialog != null) {
            queryDialog.dismiss();
        }
    }

    /**
     * 扫描方法
     */
    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                ed_trans_no.setText(s);
                query();
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }
}
