package com.hisense.hiaposcmp.activity.ysy.refund;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.RefundViewModel;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.OnClick;

import static com.hisense.hiaposcmp.constant.ActivityConstants.RESULT_OTHER_PAYMETHOD_REFUND;

/**
 * 其他支付方式录入
 *
 * @author guolina     2019-03-14 HIPOSAPP-1052
 * @author zhangsai    2019-06-13 HIPOSAPP-1169 ReturnSale.payments() 返回类型改为 ISalePayment 集合
 * @author wangyaogang 2019-06-13 HIPOSAPP-1176 点击金额框时,根据输入的支付号计算最大可退金额并显示在金额框内
 * @author wangyaogang 2019-07-01 HIPOSAPP-1189 使用 IScanner 扫描器实例扫描
 * @author xieliyan    2019-11-01 HIAPOSCMP-254 退款重构
 */
@ActivityLayoutInject(R.layout.activity_other_paymethod)
public class OtherPayMethodRefundActivity extends BaseActivity {

    /**
     * 券号
     */
    @BindView(R.id.edtCouponNo)
    public EditText mEdtPayNo;
    /**
     * 券校验码
     */
    @BindView(R.id.edtCheckCode)
    public EditText mEdtAmount;
    /**
     * 取消
     */
    @BindView(R.id.btnCancel)
    public Button mBtnCancel;
    /**
     * 确认
     */
    @BindView(R.id.btnConfirm)
    public Button mBtnConfirm;
    /**
     * 扫码
     */
    @BindView(R.id.imgScan)
    public ImageView mImgScan;
    /**
     * 未付金额
     */
    public String mTotalUnpaid;
    /**
     * 支付/退款集合
     */
    private ArrayList<RefundViewModel> listPayments;
    /**
     * 退货流水
     */
    private ReturnSaleModel mReturnSale;
    /**
     * 最大可退金额
     */
    private String mMaxRefund;
    /**
     * 当前支付方式
     */
    private PayMethod mPayMethod;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        NumberHandler.setPoint(mEdtAmount, 2);
        mScanner = getScanner();

        Bundle bundle = getIntent().getExtras();
        if (bundle != null) {
            mTotalUnpaid = bundle.getString("total_unpaid");
            mPayMethod = (PayMethod) getIntent().getSerializableExtra("paymethod");
            listPayments = (ArrayList<RefundViewModel>) getIntent().getSerializableExtra("listPayments");
            mReturnSale = (ReturnSaleModel) getIntent().getSerializableExtra("returnSale");
        }
    }

    /**
     * 取消
     */
    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        setResult(RESULT_CANCELED);
        finish();
    }

    /**
     * 确认
     */
    @OnClick(R.id.btnConfirm)
    public void onClickConfirm() {
        String payNo = mEdtPayNo.getText().toString();
        mMaxRefund = getOtherMaxRefund(mPayMethod, payNo);
        if (StringUtils.isEmpty(mEdtAmount.getText().toString())) {
            CommonTools.toastShort(OtherPayMethodRefundActivity.this, "请输入退款金额");
            return;
        }
        if (BigDecimalUtils.isGreater(mEdtAmount.getText().toString(), mTotalUnpaid)) {
            CommonTools.toastShort(OtherPayMethodRefundActivity.this, "退款金额不得大于未退金额" + mTotalUnpaid);
            return;
        }
        if (BigDecimalUtils.isGreater(mEdtAmount.getText().toString(), mMaxRefund)) {
            CommonTools.toastShort(OtherPayMethodRefundActivity.this, "退款金额不得大于可退金额" + mMaxRefund);
            return;
        }
        if (BigDecimalUtils.isEquals0(mEdtAmount.getText().toString())) {
            CommonTools.toastShort(OtherPayMethodRefundActivity.this, "输入金额不得为0");
            return;
        }
        Intent intent = new Intent();
        intent.putExtra("payNo", mEdtPayNo.getText().toString());
        intent.putExtra("amount", mEdtAmount.getText().toString());
        setResult(RESULT_OTHER_PAYMETHOD_REFUND, intent);
        finish();
    }

    /**
     * 扫码
     */
    @OnClick(R.id.imgScan)
    public void onClickScan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                mEdtPayNo.setText(s);
                mEdtPayNo.setSelection(mEdtPayNo.getText().toString().length());
            }

            @Override
            public void onError(final Throwable t) {
                CommonTools.toastShort(t.getMessage());
            }
        });
    }

    /**
     * 返回其他支付方式的合计可退金额。
     *
     * @param payMethod 支付方式
     * @return 其他支付方式的最大可退金额
     */
    private String getOtherMaxRefund(PayMethod payMethod, String payNo) {
        return BigDecimalUtils.subtract(getOtherTotalReceived(payMethod, payNo), getOtherTotalRefund(payMethod, payNo));
    }

    /**
     * 返回其他支付方式的合计支付金额。
     *
     * @param payMethod 支付方式
     * @return 其他支付方式的合计支付金额
     */
    private String getOtherTotalReceived(PayMethod payMethod, String payNo) {
        String total = "0";

        if (payMethod != null) {
            for (int i = 0; i < listPayments.size(); i++) {
                if (listPayments.get(i).payment().getRemark().equals(payMethod.remark())
                        && listPayments.get(i).payCode().equals(payMethod.code())
                        && listPayments.get(i).payNo().equals(payNo)) {

                    total = BigDecimalUtils.add(total, listPayments.get(i).totalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 返回其他支付方式的合计退款金额。
     *
     * @param payMethod 支付方式
     * @return 其他支付方式的合计退款金额
     */
    private String getOtherTotalRefund(PayMethod payMethod, String payNo) {
        String total = "0";

        if (payMethod != null) {
            for (SalePayModel payment : mReturnSale.getPays()) {
                if (payment.getRemark().equals(payMethod.remark())
                        && payment.getPayCode().equals(payMethod.code())
                        && payment.getBuyerLogonId().equals(payNo)) {
                    total = BigDecimalUtils.add(total, payment.getTotalPaid());

                }
            }
        }

        return total;
    }

    /**
     * 调用Activity后的处理。
     *
     * @param requestCode 请求编号
     * @param resultCode  返回编号
     * @param data        返回数据
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            mEdtPayNo.setText(mScanner.getScanResult(data));
            mEdtPayNo.setSelection(mEdtPayNo.getText().toString().length());
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }
}
