package com.hisense.hiaposcmp.activity.ysy.refund;

import android.content.Intent;
import android.os.Bundle;
import android.text.InputType;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.hisense.common.util.DateUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.webapi.WebApi;

import java.util.Date;
import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;

import static com.hisense.hiaposcmp.constant.ActivityConstants.RESULT_UMS_REFUND_CANCEL;
import static com.hisense.hiaposcmp.constant.ActivityConstants.RESULT_UMS_REFUND_COMMIT;
import static com.hisense.hiaposcmp.constant.refund.RefundConstants.PARAM_AMOUNT;
import static com.hisense.hiaposcmp.constant.refund.RefundConstants.PARAM_REF_NO;
import static com.hisense.hiaposcmp.constant.refund.RefundConstants.PARAM_SALE_TIME;
import static com.hisense.hiaposcmp.constant.refund.RefundConstants.PARAM_TRACE_NO;

/**
 * 当日撤销/隔日退货选择对话框。
 *
 * <p>默认选择当日撤销。如果判断非当日退货，则自动选择隔日退货。</p>
 *
 * <p>确定时，如果选择当日撤销，则返回 TRACE_NO，REF_NO 为空；如果选择隔日退货，则返回 REF_NO，TRACE_NO 为空。</p>
 *
 * @author xieliyan 2017-05-10
 * @author xieliyan 2018-01-04 HIPOSAPP-262 支持建行收单
 * @author xieliyan 2018-02-01 HIPOSAPP-216 退货的"当日/非当日退款"改为由 Android 绘制
 * @author zhangsai 2018-02-01 HIPOSAPP-898 根据选择当日撤销/隔日退货返回交易凭证号/交易参考号
 * @author xieliyan 2019-10-31
 * @author guolina  2019-10-30 HIPOSAPP-1332 当日撤销金额不可以编辑，解决隔日退款金额无法输入小数和凭证号错误
 */
@ActivityLayoutInject(R.layout.dialog_sale_refund)
public class UnionRefundActivity extends BaseActivity {
    /**
     * 当日撤销
     */
    @BindView(R.id.rlSameDate)
    public RelativeLayout mRlSameDate;
    /**
     * 隔日退款
     */
    @BindView(R.id.rlDiffDate)
    public RelativeLayout mRlDiffDate;
    /**
     * 隔日退货
     */
    @BindView(R.id.txtDiffDate)
    public TextView mTxtDiffDate;
    /**
     * 当日撤销
     */
    @BindView(R.id.txtSameDate)
    public TextView mTxtSameDate;
    /**
     * 取消
     */
    @BindView(R.id.btnCancel)
    public Button mBtnCancel;
    /**
     * 确定
     */
    @BindView(R.id.btnCommit)
    public Button mBtnCommit;
    /**
     * 退款金额
     */
    @BindView(R.id.edtRefundAmount)
    public EditText mEdtRefundAmount;
    /**
     * 凭证号or参考号
     */
    @BindView(R.id.edtRefundNo)
    public EditText mEdtRefundNo;
    /**
     * 凭证号or参考号 标题
     */
    @BindView(R.id.txtRefundTitle)
    public TextView mTxtRefundTitle;
    /**
     * 隔日退货
     */
    @BindView(R.id.imgDiffDate)
    public ImageView mImgDiffDate;
    /**
     * 当日撤销
     */
    @BindView(R.id.imgSameDate)
    public ImageView mImgSameDate;
    /**
     * 交易凭证号
     */
    private String mTraceNo = "";
    /**
     * 交易参考号
     */
    private String mRefNo = "";
    /**
     * 退款金额
     */
    private Double mRefundAmount = 0.00;
    /**
     * 是否当日
     */
    private boolean mIsSameDateSelected = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Bundle bundle = this.getIntent().getExtras();
        mRefundAmount = bundle.getDouble(PARAM_AMOUNT);
        mTraceNo = bundle.getString(PARAM_TRACE_NO);
        mRefNo = bundle.getString(PARAM_REF_NO);

        mEdtRefundNo.setText(mTraceNo);
        mEdtRefundNo.setInputType(InputType.TYPE_NUMBER_FLAG_DECIMAL);

        mEdtRefundAmount.setText(String.format(Locale.CHINESE, "%.2f", mRefundAmount));
        NumberHandler.setPoint(mEdtRefundAmount, 2);//只允许输入两位小数
        mEdtRefundAmount.setEnabled(false);

        // 默认选择当日撤销。如果判断非当日退货，则自动选择隔日退货
        Date saleDate = DateUtils.dateParseWhitDefault(bundle.getString(PARAM_SALE_TIME));
        mIsSameDateSelected = WebApi.formatDate(new Date()).equals(WebApi.formatDate(saleDate));
        if (!mIsSameDateSelected) {
            selectDiffDate();
        }
    }

    /**
     * 隔日退货
     */
    @OnClick(R.id.rlDiffDate)
    public void onClickDiffDate() {
        selectDiffDate();
    }

    /**
     * 当日撤销
     */
    @OnClick(R.id.rlSameDate)
    public void onClickSameDate() {
        selectSameDate();
    }

    /**
     * 取消
     */
    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        setResult(RESULT_UMS_REFUND_CANCEL);
        finish();
    }

    /**
     * 确认
     */
    @OnClick(R.id.btnCommit)
    public void onClickCommit() {
        Bundle bundle = new Bundle();
        if (mIsSameDateSelected) {
            bundle.putString(PARAM_REF_NO, "");
            bundle.putString(PARAM_TRACE_NO, mEdtRefundNo.getText().toString());
        } else {
            bundle.putString(PARAM_REF_NO, mEdtRefundNo.getText().toString());
            bundle.putString(PARAM_TRACE_NO, "");
        }
        bundle.putString(PARAM_AMOUNT, mEdtRefundAmount.getText().toString());

        Intent intent = new Intent();
        intent.putExtras(bundle);

        setResult(RESULT_UMS_REFUND_COMMIT, intent);
        finish();
    }

    /**
     * 选择当日撤销。
     */
    private void selectSameDate() {
        updateDiffDateUI(false);
        updateSameDateUI(true);
        mIsSameDateSelected = true;
        mTxtRefundTitle.setText(getResources().getText(R.string.title_trans_trace_no));
        mEdtRefundNo.setText(mTraceNo);
        mEdtRefundAmount.setText(String.format(Locale.CHINESE, "%.2f", mRefundAmount));
        mEdtRefundAmount.setEnabled(false);
    }

    /**
     * 选择隔日退货。
     */
    private void selectDiffDate() {
        mEdtRefundAmount.setEnabled(true);
        updateDiffDateUI(true);
        updateSameDateUI(false);
        mIsSameDateSelected = false;
        mTxtRefundTitle.setText(getResources().getText(R.string.title_trans_ref_no));
        mEdtRefundNo.setText(mRefNo);
        mEdtRefundAmount.setText(String.format(Locale.CHINESE, "%.2f", mRefundAmount));
    }

    /**
     * 更新隔日退货UI。
     *
     * @param selected 是否选中隔日退货。true 表示是，false 表示否。
     */
    private void updateDiffDateUI(boolean selected) {
        mRlDiffDate.setBackground(ContextCompat.getDrawable(App.getContext(),
                selected ? R.drawable.corner_right_circle_down : R.drawable.corner_right_circle_up));
        mImgDiffDate.setBackground(ContextCompat.getDrawable(App.getContext(),
                selected ? R.drawable.ic_payment_channel_refund_other_day_down : R.drawable.ic_payment_channel_refund_other_day_up));
        mTxtDiffDate.setTextColor(ContextCompat.getColor(this,
                selected ? R.color.button_text : R.color.fontDefault));
    }

    /**
     * 更新当日撤销UI。
     *
     * @param selected 是否选中当日撤销。true 表示是，false 表示否。
     */
    private void updateSameDateUI(boolean selected) {
        mRlSameDate.setBackground(ContextCompat.getDrawable(App.getContext(),
                selected ? R.drawable.corner_left_circle_down : R.drawable.corner_left_circle_up));
        mImgSameDate.setBackground(ContextCompat.getDrawable(this,
                selected ? R.drawable.ic_payment_channel_refund_today_down : R.drawable.ic_payment_channel_refund_today_up));
        mTxtSameDate.setTextColor(ContextCompat.getColor(this,
                selected ? R.color.button_text : R.color.fontDefault));
    }

}
