/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.refund.viewModel;

import com.hisense.hipos.model.SalePayModel;

import java.io.Serializable;

/**
 * 退款的 ViewModel。
 *
 * @author zhangsai 2018-12-21 HIPOSAPP-699
 * @author zhangsai 2019-06-12 HIPOSAPP-1141 新增 totalUnRefund 属性
 * @author zhangsai 2019-06-13 HIPOSAPP-1169 删除 totalRefund/totalUnRefund 属性
 */

public class RefundViewModel implements Serializable {
    /**
     * 原交易流水的一笔支付信息
     */
    private SalePayModel payModel;

    /**
     * 初始化一个 {@code RefundViewModel} 的实例。
     *
     * @param payment 原交易流水的一笔支付信息
     */
    public RefundViewModel(SalePayModel payment) {
        this.payModel = payment;
    }


    /**
     * 返回原交易流水的支付信息。
     *
     * @return 原交易流水的支付信息
     */
    public SalePayModel payment() {
        return this.payModel;
    }

    /**
     * 返回支付方式名称。
     *
     * @return 支付方式名称
     */
    public String payName() {
        return this.payModel.getPayName();
    }

    /**
     * 返回支付方式编码。
     *
     * @return 支付方式编码
     */
    public String payCode() {
        return this.payModel.getPayCode();
    }

    /**
     * 返回支付号。
     *
     * @return 支付号
     */
    public String payNo() {
        return this.payModel.getBuyerLogonId();
    }

    /**
     * 返回交易参考号。
     *
     * @return 交易参考号
     */
    public String refNo() {
        return this.payModel.getTradeNo();
    }


    /**
     * 返回合计支付金额。(此处为实收金额，正常情况下实收=支付金额，
     * 但当有找零的情况时，实收+找零=支付金额
     *
     * @return 合计支付金额
     */
    public String totalPaid() {
        return this.payModel.getTotalRecv();
    }

    /**
     * 返回合计退款金额。
     *
     * @return 合计退款金额
     */
    public double totalRefund() {
        return this.payModel.totalRefund();
    }


}
