/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.refund.viewModel;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;


import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.service.payment.PayMethodService;

import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Locale;

/**
 * 退货支付列表适配器。
 *
 * @author xieliyan 2017-05-12
 * @author xieliyan 2017-07-02 增加银联钱包图标
 * @author zhangsai 2017-11-15 HIPOSAPP-212
 * @author zhangsai 2018-12-21 HIPOSAPP-699 优化退货逻辑
 */
public class SaleRefundItemsAdapter extends BaseAdapter {

    private LayoutInflater mInflater;

    private ArrayList<RefundViewModel> viewModels;

    private View.OnClickListener mOnClickListener;

    public SaleRefundItemsAdapter(Context context, ArrayList<RefundViewModel> viewModels, View.OnClickListener listener) {
        mInflater = LayoutInflater.from(context);
        this.viewModels = viewModels;
        this.mOnClickListener = listener;
    }

    @Override
    public int getCount() {
        return viewModels.size();
    }

    @Override
    public Object getItem(int position) {
        return viewModels.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(int i, View view, ViewGroup parent) {
        PayViewHolder holder;
        if (view == null) {
            view = mInflater.inflate(R.layout.sale_refund_list_item, null);
            holder = new PayViewHolder();
            holder.txtPayName = view.findViewById(R.id.txtPayName);
            holder.txtPaid = view.findViewById(R.id.txtPaid);
            holder.imgPayMethod = view.findViewById(R.id.imgPayMethod);
            holder.txtPayNo = view.findViewById(R.id.txtPayNo);
            holder.txtRefund = view.findViewById(R.id.txtRefund);
            view.setTag(holder);

            view.setOnClickListener(this.mOnClickListener);
        } else {
            holder = (PayViewHolder) view.getTag();
        }

        holder.txtPayName.setText(viewModels.get(i).payName());
        holder.txtPaid.setText(viewModels.get(i).totalPaid());
        holder.txtRefund.setText(String.format(Locale.CHINESE, "-%s", viewModels.get(i).totalRefund()));

        holder.imgPayMethod.setBackgroundResource(
                PayMethodService.getIconByCode(viewModels.get(i).payCode()));
        holder.imgPayMethod.invalidate();

        if (StringUtils.isWhitespace(viewModels.get(i).payNo())) {
            holder.txtPayNo.setVisibility(View.GONE);
        } else {
            holder.txtPayNo.setVisibility(View.VISIBLE);
            holder.txtPayNo.setText(viewModels.get(i).payNo());
        }

        return view;
    }

    private static class PayViewHolder {
        ImageView imgPayMethod;
        TextView txtPayName;
        TextView txtPaid;
        TextView txtPayNo;
        TextView txtRefund;
    }
}
