/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.report;

import android.os.Bundle;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.report.ReportConstants;
import com.hisense.hiaposcmp.presenter.ReportPresenter;
import com.hisense.hiaposcmp.view.IView.IReportView;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.report.DepReportResultData;
import com.hisense.hipos.webapi.report.PluReportResultData;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 收银员报表。
 *
 * @author fujiaxin 2017-03-14
 * @author xieliyan 2017-10-31 HIPOSAPP-201 使用枚举实现单例模式:更改调用方法
 * @author zhangsai 2017-12-21 HIPOSAPP-250 改由通过 SaleService 获取当前结账日期
 * @author zhangsai 2018-07-16 HIPOSAPP-668 更改支付方式图标调用方法
 * @author zhangsai 2018-12-28 HIPOSAPP-946 改用 ReportService 查询销售报表
 */
@ActivityLayoutInject(R.layout.activity_report_sale_cashier)
public class CashierReportActivity extends BaseMvpActivity<IReportView, ReportPresenter> implements IReportView {

    /**
     * 返回按钮
     */
    @BindView(R.id.llGoBack)
    public LinearLayout llGoBack;
    /**
     * 打印按钮
     */
    @BindView(R.id.llPrint)
    public LinearLayout llPrint;
    /**
     * 页面容器
     */
    @BindView(R.id.llReportContainer)
    public LinearLayout llReportContainer;
    /**
     * 页面数据位空时的提示信息
     */
    @BindView(R.id.txtEmpty)
    public TextView txtEmpty;
    /**
     * 支付方式列表
     */
    @BindView(R.id.lvwPayments)
    public ListView lvwPayments;
    /**
     * 收银员
     */
    @BindView(R.id.txtCashier)
    public TextView txtCashier;
    /**
     * 专柜
     */
    @BindView(R.id.txtShop)
    public TextView txtShop;
    /**
     * 专柜容器
     */
    @BindView(R.id.llShop)
    public LinearLayout llShop;
    /**
     * 合计应收金额
     */
    @BindView(R.id.txtTotal)
    public TextView txtTotal;
    /**
     * 合计实收金额
     */
    @BindView(R.id.txtTotalRecv)
    public TextView txtTotalRecv;
    /**
     * 合计优惠金额
     */
    @BindView(R.id.txtTotalDsc)
    public TextView txtTotalDsc;
    /**
     * 合计退款金额
     */
    @BindView(R.id.txtTotalReturned)
    public TextView txtTotalReturned;
    /**
     * 合计交易笔数
     */
    @BindView(R.id.txtTotalTrans)
    public TextView txtTotalTrans;
    /**
     * 合计退货笔数
     */
    @BindView(R.id.txtTotalTransReturned)
    public TextView txtTotalTransReturned;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        updateCashier();
        mPresenter.accountingDateCashier();
    }

    @Override
    protected ReportPresenter createPresenter() {
        return new ReportPresenter();
    }
    /**
     * 返回监听
     */
    @OnClick(R.id.llGoBack)
    public void onClickBack() {
        finish();
    }
    /**
     * 打印监听
     */
    @OnClick(R.id.llPrint)
    public void onClickPrint() {
        mPresenter.printSalePort(ReportConstants.REPORT_CASHIER);
    }
    /**
     * 更新收银员信息
     */
    private void updateCashier() {
        showCashierData(false);
        txtCashier.setText(String.format(Locale.CHINESE, "%s(%s)",
                Context.cashier().name(), Context.cashier().code()));
        txtShop.setText(Context.shopCode());
        // HIPOSAPP-113 全场收银时隐藏收银员信息
        if (GeneralSettings.INSTANCE.isNoShopLimits()) {
            llShop.setVisibility(View.GONE);
        }
    }
    /**
     * 显示收银相关数据
     */
    @Override
    public void showCashierData(final boolean isShowData) {
        if (isShowData) {
            llReportContainer.setVisibility(View.VISIBLE);
            txtEmpty.setVisibility(View.GONE);
        } else {
            llReportContainer.setVisibility(View.GONE);
            txtEmpty.setVisibility(View.VISIBLE);
            mPresenter.setCashierReport(null);

        }
    }

    /**
     * 显示报表。
     *
     * @param report 报表
     */
    @Override
    public void showSaleReport(SaleReport report) {
        mPresenter.setCashierReport(report);

        lvwPayments.setAdapter(mPresenter.saleReportadapter());

        if (null == report) {
            txtTotal.setText("");
            txtTotalRecv.setText("");
            txtTotalReturned.setText("");
            txtTotalDsc.setText("");
            txtTotalTrans.setText("");
            txtTotalTransReturned.setText("");
        } else {
            txtTotal.setText(String.format(Locale.CHINESE, "%.2f", report.total()));
            txtTotalRecv.setText(String.format(Locale.CHINESE, "%.2f", report.totalReceived()));
            txtTotalReturned.setText(String.format(Locale.CHINESE, "%.2f", report.totalReturned()));
            txtTotalDsc.setText(String.format(Locale.CHINESE, "%.2f", report.totalDiscount()));
            txtTotalTrans.setText(String.valueOf(report.totalTrans()));
            txtTotalTransReturned.setText(String.valueOf(report.totalTransReturned()));
        }

    }

    @Override
    public void showDepReport(DepReportResultData report) {

    }

    @Override
    public void showPluReport(PluReportResultData report) {

    }

    @Override
    public void showShopSaleReport(ShopSaleReport report) {

    }

    @Override
    public void accountingDateSaleSuccess(String accDate) {

    }

    @Override
    public TranType tranType() {
        return TranType.Sale;
    }
}
