/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.report;

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.bigkoo.pickerview.builder.TimePickerBuilder;
import com.bigkoo.pickerview.listener.OnTimeSelectListener;
import com.bigkoo.pickerview.view.TimePickerView;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.report.ReportConstants;
import com.hisense.hiaposcmp.presenter.ReportPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IReportView;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.report.DepReportResultData;
import com.hisense.hipos.webapi.report.PluReportResultData;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 总交易报表/预销售报表
 *
 * @author fujiaxin 2017-03-14
 * @author xieliyan 2017-09-01
 * @author xieliyan 2017-09-01 HIPOSAPP-153 调用预销售接口时删除trantype字段
 * @author zhangsai 2017-12-21 HIPOSAPP-250 改由通过 SaleService 获取当前结账日期
 * @author zhangsai 2018-07-16 HIPOSAPP-668 更改支付方式图标调用方法
 * @author zhangsai 2018-12-28 HIPOSAPP-946 改用 ReportService 查询销售报表
 */
@ActivityLayoutInject(R.layout.activity_report_sale)
public class SaleReportActivity extends BaseMvpActivity<IReportView, ReportPresenter> implements IReportView {

    // 标题
    @BindView(R.id.txtTitle)
    public TextView txtTitle;

    // 返回按钮
    @BindView(R.id.llGoBack)
    public LinearLayout llGoBack;

    // 打印按钮
    @BindView(R.id.llPrint)
    public LinearLayout llPrint;

    //页面容器
    @BindView(R.id.llContainer)
    public LinearLayout llContainer;

    // 暂无数据
    @BindView(R.id.txtEmpty)
    public TextView txtEmpty;

    // 支付方式列表
    @BindView(R.id.lvwPayments)
    public ListView lvwPayments;

    // 合计应收金额
    @BindView(R.id.txtTotal)
    public TextView txtTotal;

    // 合计实收金额
    @BindView(R.id.txtTotalRecv)
    public TextView txtTotalRecv;

    // 合计优惠金额
    @BindView(R.id.txtTotalDsc)
    public TextView txtTotalDsc;

    // 合计退款金额
    @BindView(R.id.txtTotalReturned)
    public TextView txtTotalReturned;

    // 合计交易笔数
    @BindView(R.id.txtTotalTrans)
    public TextView txtTotalTrans;

    // 合计退货笔数
    @BindView(R.id.txtTotalTransReturned)
    public TextView txtTotalTransReturned;

    // 查询报表
    @BindView(R.id.btn_query)
    Button mBtnQuery;

    //开始日期
    @BindView(R.id.tx_begin_time)
    TextView mTxBeginTime;

    // 交易类型
    private TranType mTranType;
    // 标题
    private String mTitle;
    private android.content.Context mContext;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        changeTitle();
        showCashierData(false);
        mContext = this;
        mTxBeginTime.setText(getTime(new Date()));
        //mPresenter.accountingDateSale(mTitle, mTranType);
    }

    @Override
    protected ReportPresenter createPresenter() {
        return new ReportPresenter();
    }

    @OnClick({R.id.llGoBack,R.id.llPrint,R.id.btn_query,R.id.tx_begin_time})
    public void onViewClicked(View v) {
        switch (v.getId()) {
            case R.id.llGoBack:
                this.finish();
                break;
            case R.id.llPrint:
                mPresenter.printSalePort(ReportConstants.REPORT_SALE, mTxBeginTime.getText().toString());
                break;
            case R.id.btn_query:
                getReport();
                break;
            case R.id.tx_begin_time:
                initTimePicker(mTxBeginTime);
                break;
            default:
                break;
        }
    }
    private void getReport(){
        showLoadingDialog();
        String beginTime = mTxBeginTime.getText().toString();
        // 判断日期
        if (TextUtils.isEmpty(beginTime)) {
            hideLoadingDialog();
            CommonTools.toastShort(getString(R.string.select_begin_time));
            return;
        }
        mPresenter.getSaleReport(beginTime,mTranType);
    }
    //yyyy-MM-dd 可根据需要自行截取数据显示
    private String getTime(Date date) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA);
        return format.format(date);
    }

    //上次选择日期
    private Date lastDate;

    private void initTimePicker(final TextView tvTime) {
        Calendar startDate = Calendar.getInstance();
        startDate.set(2010, 0, 1);

        //等同于保存上次选择日期
        Calendar nowDate = Calendar.getInstance();
        if (lastDate != null) {
            nowDate.setTime(lastDate);
        }
        // .isDialog(true)      // 加上这句 就在中间显示了
        //时间选择器
        TimePickerView pvTime = new TimePickerBuilder(mContext, new OnTimeSelectListener() {
            @Override
            public void onTimeSelect(Date date, View v) {
                // 这里回调过来的v,就是show()方法里面所添加的 View 参数，如果show的时候没有添加参数，v则为null
                //选择一个日期时,判断另一个日期是否录入,若录入,则比较时间先后,进行提示
                lastDate = date;
                tvTime.setText(getTime(date));
            }
        })
                //年月日时分秒 的显示与否，不设置则默认全部显示
                .setType(new boolean[]{true, true, true, false, false, false})
                .setSubmitColor(Color.parseColor("#F04B63"))
                .setCancelColor(Color.parseColor("#666666"))
                .setDividerColor(Color.DKGRAY)
                .setContentTextSize(21)
                .setDate(nowDate)
                .isCenterLabel(true)
                .setRangDate(startDate, Calendar.getInstance())
                .setLabel("年", "月", "日", "", "", "")
                .isCyclic(false)
                .setDecorView(null)
                .build();
        pvTime.show();
    }
    @Override
    public void showCashierData(boolean isShowData) {
        if (isShowData) {
            llContainer.setVisibility(View.VISIBLE);
            txtEmpty.setVisibility(View.GONE);
        } else {
            llContainer.setVisibility(View.GONE);
            txtEmpty.setVisibility(View.VISIBLE);
            mPresenter.setCashierReport(null);
        }
    }

    @Override
    public void showSaleReport(SaleReport report) {
        mPresenter.setCashierReport(report);

        lvwPayments.setAdapter(mPresenter.saleReportadapter());

        if (null == report) {
            txtTotal.setText("");
            txtTotalRecv.setText("");
            txtTotalReturned.setText("");
            txtTotalDsc.setText("");

            txtTotalTrans.setText("");
            txtTotalTransReturned.setText("");
        } else {
            txtTotal.setText(String.format(Locale.CHINESE, "%.2f", report.total()));
            txtTotalRecv.setText(String.format(Locale.CHINESE, "%.2f", report.totalReceived()));
            txtTotalReturned.setText(String.format(Locale.CHINESE, "%.2f", report.totalReturned()));
            txtTotalDsc.setText(String.format(Locale.CHINESE, "%.2f", report.totalDiscount()));

            txtTotalTrans.setText(String.valueOf(report.totalTrans()));
            txtTotalTransReturned.setText(String.valueOf(report.totalTransReturned()));
        }

    }

    @Override
    public void showDepReport(DepReportResultData report) {

    }

    @Override
    public void showPluReport(PluReportResultData report) {

    }

    @Override
    public void showShopSaleReport(ShopSaleReport report) {

    }

    @Override
    public void accountingDateSaleSuccess(String accDate) {

    }

    private void changeTitle() {
        Bundle bundle = this.getIntent().getExtras();

        if (bundle != null) {
            mTranType = TranType.valueOf(bundle.getInt("tranType"));
        }

        if (mTranType.equals(TranType.Sale)) {
            mTitle = ReportConstants.REPORT_SALE;
        } else if (mTranType.equals(TranType.PreSale)) {
            mTitle = ReportConstants.REPORT_PRESALE;
        }
        txtTitle.setText(mTitle);
    }

    @Override
    public TranType tranType() {
        return mTranType;
    }

    @Override
    public Activity getActivity() {
        return this;
    }
}
