package com.hisense.hiaposcmp.activity.ysy.sale;

import android.os.Bundle;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.adapter.GiftTokenItemAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.layout.SwitchViewWatcher;
import com.hisense.hiaposcmp.presenter.sale.GiftPresenter;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.view.IView.sale.IGiftView;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.sale.CertiDtlQueryResultData;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 礼品券录入界面
 *
 * @author yuminghao      2020-09-28
 */
@ActivityLayoutInject(R.layout.activity_sale_gift)
public class SaleGiftActivity extends BaseMvpActivity<IGiftView, GiftPresenter> implements IGiftView {

    @BindView(R.id.edtPlu)
    public EditText edtPlu;

    @BindView(R.id.txtSaleNo)
    public TextView txtSaleNo;

    @BindView(R.id.lytClearPlu)
    public RelativeLayout lytClearPlu;

    // 礼品券信息显示
    @BindView(R.id.lytAssistant)
    public LinearLayout lytAssistant;
    @BindView(R.id.txt_gift_token_type)
    public TextView giftType;
    @BindView(R.id.txt_gift_token_state)
    public TextView giftState;

    @BindView(R.id.salesItem)
    public ListView lvwSaleItems;

    private String certiCode = "";
    private GiftTokenItemAdapter saleItemAdapter;
    private List<BaseSalePluModel> mSaleItems = null;       // 当前交易商品。交易结束后置清空
    SalePayModel salePayModel;         // 支付方式

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initValue();
        initEvent();
    }

    @Override
    protected GiftPresenter createPresenter() {
        return new GiftPresenter();
    }

    private void initValue() {
        // 初始化流水号
        mPresenter.createNewSale(null, TranType.Sale, new UniCallback() {
            @Override
            public void onSuccess(String s) {
//                updateVipInfo(mPresenter.getSale().getVip());
                txtSaleNo.setText(mPresenter.getSale().getSaleNo());
            }

            @Override
            public void onError(String message) {
                showHintDialog(message);
            }
        });
        // 初始化商品列表
        mSaleItems = new ArrayList<>();
        saleItemAdapter = new GiftTokenItemAdapter(this, mSaleItems);
        lvwSaleItems.setAdapter(saleItemAdapter);
        // 隐藏券信息
        lytAssistant.setVisibility(View.GONE);
    }

    private void initEvent() {
        //绑定输入商品编码查询事件
        edtPlu.addTextChangedListener(new SwitchViewWatcher(lytClearPlu));
        edtPlu.requestFocus();
        edtPlu.setImeOptions(EditorInfo.IME_ACTION_DONE);
        edtPlu.setOnEditorActionListener((textView, actionId, keyEvent) -> {
            if (actionId == EditorInfo.IME_ACTION_DONE) {
                //调用服务获取商品信息
                if (!StringUtils.isEmpty(edtPlu.getText().toString())) {
                    certiCode = edtPlu.getText().toString();
                    mPresenter.giftQueryWithCode(edtPlu.getText().toString().trim());
                }
            }
            return false;
        });
    }

    @OnClick(R.id.lytClearPlu)
    void clearEdtPlu() {
        edtPlu.setText("");
    }

    @OnClick(R.id.btnGoBack)
    public void onClickGoBack() {
        onBackPressed();
    }

    @OnClick(R.id.btn_cancel)
    public void onCancel() {
        mPresenter.getSale().clearPlu();
        mSaleItems.clear();
        saleItemAdapter.notifyDataSetChanged();
    }

    @OnClick(R.id.btn_charge_off)
    public void onCharge() {
        if (ListUtils.isEmpty(getSale().getPlus()) || certiCode.equals("")) {
            Toast.makeText(this, "请输入有效的礼品券码", Toast.LENGTH_SHORT).show();
            return;
        }
        mPresenter.consumeCerti(certiCode);
    }

    @Override
    public void onQuerySuccess(CertiDtlQueryResultData certiDtlQueryResultData) {
        // 查询成功，暂存支付方式
        if (salePayModel == null){
            salePayModel = new SalePayModel();
        }
        salePayModel.setPayCode(certiDtlQueryResultData.lpPayCode);
        salePayModel.setPayName(certiDtlQueryResultData.lpPayName);
        salePayModel.setPayType("");
        salePayModel.setBuyerLogonId("");
        salePayModel.setTotalPaid(certiDtlQueryResultData.apportionType.equals("0")? getSale().getTotal() : "0.00");
        salePayModel.setTotalRecv(certiDtlQueryResultData.apportionType.equals("0")? getSale().getTotal() : "0.00");
        salePayModel.setRemark("");
        salePayModel.setUdp1("");
        salePayModel.setUdp2("");
        salePayModel.setUdp3("");
        salePayModel.setUdp4("");
        salePayModel.setUdp5("");
        // 查询成功，更新礼品券信息和商品列表
        updateGiftInfo(certiDtlQueryResultData);
    }

    @Override
    public void onQueryFail() {
        // 查询失败，删除相关信息
        lytAssistant.setVisibility(View.GONE);
        saleItemAdapter.clear();
        salePayModel = null;
    }

    @Override
    public void updateGiftInfo(CertiDtlQueryResultData certiDtlQueryResultData) {
        // 初始化搜索框
        edtPlu.setText("");
        // 设置搜索结果
        lytAssistant.setVisibility(View.VISIBLE);
        giftType.setText(certiDtlQueryResultData.getRealCertiType());
        giftState.setText(certiDtlQueryResultData.getRealCertiStatus());
        saleItemAdapter.setData(getSale().getPlus());
    }

    @Override
    public void consumeSuccess() {
        try {
            List<SalePayModel> salePayModels = new ArrayList<>();
            salePayModels.add(salePayModel);
            getSale().setPays(salePayModels);
            mPresenter.saleSubmit();
        } catch (Exception ex) {
            LoggerUtils.getLogger().error("流水上传异常：" + ex.toString());
            //获取新的流水号
            SaleNoService.abandon();
            mPresenter.resetSaleModel();
            submitFail("流水上传异常：" + ex.getMessage());
            showHintDialog("流水上传失败,请点击右上角上传流水重试");
        }
    }

    @Override
    public void consumeFail() {

    }

    @Override
    public void onlineFail(String message) {

    }

    @Override
    public void submitSuccess() {
        certiCode = "";
        salePayModel = null;
        initValue();
    }

    @Override
    public void submitFail(String message) {
        showHintDialog(message);
        initValue();
    }
}
