/***************************************************************************************************
 * copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.sale;

import android.content.Intent;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.sale.SaleItemPresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.NumberHandler;
import com.hisense.hiaposcmp.view.IView.sale.ISaleItemView;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import java.util.Locale;

import butterknife.BindView;
import butterknife.OnClick;
import butterknife.OnTextChanged;
import butterknife.OnTouch;

/**
 * 价格录入对话框。
 *
 * @author caokai      2017-03-02
 * @author xieliyan    2017-09-01 HIPOSAPP-136 限制折扣输入不能大于100
 * @author zhangsai    2018-01-05 HIPOSAPP-267
 * @author zhangsai    2018-01-18 HIPOSAPP-281 商品单价与售价一样格式化为保留一位小数
 * @author xieliyan    2018-02-11 HIPOSAPP-319 修复异常关闭易收银程序，打印小票中包含之前未交易的商品信息的问题
 * @author zhangsai    2018-02-11 HIPOSAPP-325 支持录入称重商品重量
 * @author xieliyan    2018-02-11 HIPOSAPP-326 控制商品输入界面确定按钮的点击次数
 * @author zhangsai    2018-02-08 HIPOSAPP-335 购买数量支持小数
 * @author xieliyan    2018-04-10 HIPOSAPP-384 价格录入时按照四舍五入规则计数
 * @author zhangsai    2018-08-20 HIPOSAPP-749 直接录入称重商品编码打开对话框时提示错误
 * @author guolina     2018-12-21 HIPOSAPP-839 小票增加促销打印
 * @author guolina     2019-01-14 HIPOSAPP-935 大码商品增加规格录入
 * @author guolina     2019-02-12 HIPOSAPP-1008 称重商品跳转SaleItemPriceActivity后自动显示重量
 * @author guolina     2019-02-14 HIPOSAPP-995  规格录入支持扫码
 * @author guolina     2019-03-04 HIPOSAPP-1035 大码商品去掉规格缓存
 * @author wangyaogang 2019-05-15 HIPOSAPP-1087 优化[控制每台机器收款的小数位]逻辑,去掉isNeedRound
 * @author wangyaogang 2019-07-09 HIPOSAPP-1191 调整价格录入对话框,不再遮挡底部按钮
 * @author dukunbiao(null)  2019-10-31  重构
 * @author guolina     2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 * @author wangyaogang 2020-04-22 HIAPOSCMP-697 规格字段录入长度限制
 */
@ActivityLayoutInject(R.layout.activity_sale_item_price)
public class SaleItemPriceActivity extends BaseMvpActivity<ISaleItemView, SaleItemPresenter> implements ISaleItemView {

    @BindView(R.id.edtTagPrice)
    public EditText edtTagPrice;   // 单价

    @BindView(R.id.edtQuantity)
    public EditText edtQuantity;   // 购买数量

    @BindView(R.id.edtPrice)
    public EditText edtPrice;      // 售价

    @BindView(R.id.edtDiscount)
    public EditText edtDiscount;   // 折扣

    @BindView(R.id.edtSpec)
    public EditText edtSpec;       // 规格

    @BindView(R.id.edtDscTotal)
    public EditText edtDscTotal;      // 单项优惠

    @BindView(R.id.txtTotal)
    public TextView txtTotal;      // 合计

    @BindView(R.id.btnConfirm)
    public Button btnConfirm;      // 确定

    @BindView(R.id.lytSpec)
    public RelativeLayout lytSpec;

    @BindView(R.id.ivScanSpec)
    public ImageView ivScanSpec;    //规格扫描框

    @BindView(R.id.txtPluCode)
    public TextView txtPluCode;

    @BindView(R.id.lytDscTotal)
    public LinearLayout lytDscTotal;

    @BindView(R.id.txtQuantity)
    public TextView txtQuantity;

    private boolean isHandleTextChanged = false; // 控制更改文本时，不要产生重复调用

    private BaseSalePluModel mPluModel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
        mScanner = getScanner(this);
    }

    private void init() {
        Bundle bundle = this.getIntent().getExtras();
        BaseSalePluModel pluModel = (BaseSalePluModel) bundle.getSerializable("plu");
        if (pluModel == null) {
            showHintDialog("获取商品信息失败，请重试");
            return;
        }

        mPluModel = pluModel;
        mPresenter.initRangeValidator(mPluModel);

        txtPluCode.setText(pluModel.getPluCode() + "--" + pluModel.getPluName());
        txtQuantity.setText(pluModel.isWeight() ? "重量" : "件数");

        edtDiscount.setInputType(EditorInfo.TYPE_CLASS_PHONE);
        edtDiscount.setSelection(edtDiscount.getText().length());

        edtPrice.setInputType(EditorInfo.TYPE_CLASS_PHONE);
        edtPrice.setSelection(edtPrice.getText().length());

        edtQuantity.setInputType(EditorInfo.TYPE_CLASS_PHONE);
        edtQuantity.setSelection(edtQuantity.getText().length());

        btnConfirm.setEnabled(true);

        edtPrice.setText("");
        edtTagPrice.setText("");
        edtDiscount.setText("100");
        edtSpec.setText("");
        edtTagPrice.requestFocus();
        if (Double.parseDouble(pluModel.getTagPrice()) != 0 || (pluModel.isWeight() && GeneralSettings.INSTANCE.isChangePrice())) {
            edtTagPrice.setText(pluModel.getTagPrice());
            edtTagPrice.setEnabled(false);
            edtTagPrice.setTextColor(getResources().getColor(R.color.fontDefault));
        } else if (pluModel.isWeight()) {
            lytSpec.setVisibility(View.GONE);

            edtTagPrice.setText(pluModel.getTagPrice());
            edtTagPrice.setEnabled(false);
            edtTagPrice.setTextColor(getResources().getColor(R.color.fontDefault));

            edtPrice.setText(pluModel.getTagPrice());
            edtPrice.setEnabled(false);
            edtPrice.setTextColor(getResources().getColor(R.color.fontDefault));

            edtDiscount.setEnabled(false);
            edtDiscount.setTextColor(getResources().getColor(R.color.fontDefault));

            edtQuantity.setText(pluModel.getWeight());
        }

        if (GeneralSettings.INSTANCE.isChangePrice()) {
            edtPrice.setEnabled(false);
            NumberHandler.setPoint(edtDscTotal, 2);//只允许输入两位小数

        } else {
            lytDscTotal.setVisibility(View.GONE);
        }
    }

    @Override
    protected SaleItemPresenter createPresenter() {
        return new SaleItemPresenter();
    }

    @OnClick(R.id.ivScanSpec)
    public void onClickScan() {
        scan();
    }

    @OnClick(R.id.btnConfirm)
    public void onClickConfirm() {
        try {
            if (!btnConfirm.isEnabled()) {
                return;
            }

            btnConfirm.setEnabled(false);

            mPresenter.checkNoCodePlu(mPluModel, edtTagPrice, edtPrice, edtDiscount, edtQuantity, txtTotal, edtSpec, edtDscTotal);
        } finally {
            btnConfirm.setEnabled(true);
        }
    }

    @OnClick(R.id.btnCancel)
    public void onClickCancel() {
        setResult(RESULT_CANCELED, null);
        finish();
    }

    @OnTouch({R.id.edtDiscount, R.id.edtPrice, R.id.edtTagPrice, R.id.edtQuantity, R.id.edtSpec, R.id.edtDscTotal})
    public void onTouch(View arg0, MotionEvent arg1) {
        EditText edt = (EditText) arg0;

        int inType = edt.getInputType();

        edt.onTouchEvent(arg1);
        edt.setInputType(inType);
        edt.setSelection(edt.length());
    }

    @OnTextChanged(R.id.edtDiscount)
    public void onTextChangedDiscount(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        String err = mPresenter.discountValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        calcSalePrice();
    }

    @OnTextChanged(R.id.edtPrice)
    public void onTextChangedPrice(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        String err = mPresenter.priceValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        calcDiscount();
    }

    @OnTextChanged(R.id.edtTagPrice)
    public void onTextChangedTagPrice(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        String err = mPresenter.tagPriceValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        calcSalePrice();
    }

    @OnTextChanged(R.id.edtDscTotal)
    public void onTextChangedDscTotal(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }
        if (StringUtils.isEmpty(edtQuantity.getText().toString()) || StringUtils.isEmpty(edtTagPrice.getText().toString())) {
            return;
        }
        if (null != edtDscTotal) {
            if (GeneralSettings.INSTANCE.isMaximumDiscount()) {
                double multiply = BigDecimalUtils.multiply(Double.parseDouble(BigDecimalUtils.multiply(edtTagPrice.getText().toString(), edtQuantity.getText().toString(), 2)), 0.2, 2);
                if (BigDecimalUtils.isGreaterAndEquals(edtDscTotal.getText().toString(), String.valueOf(multiply))) {
                    showHintDialog("单项优惠不得大于" + multiply);
                    edtDscTotal.setText("");
                    return;
                }
            }

            if (BigDecimalUtils.isGreaterAndEquals(edtDscTotal.getText().toString(), BigDecimalUtils.multiply(edtTagPrice.getText().toString(), edtQuantity.getText().toString(), 2))) {
                showHintDialog("单项优惠不得大于" + BigDecimalUtils.multiply(edtTagPrice.getText().toString(), edtQuantity.getText().toString(), 2));
                edtDscTotal.setText("");
                return;
            }
        }

        calcTotalbyDsc();
    }


    @OnTextChanged(R.id.edtQuantity)
    public void onTextChangedQuantity(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }

        String err = mPresenter.quantityValidator.validate(s);
        if (err != null) {
            CommonTools.toastShort(err);
            return;
        }

        try {
            double value1 = Double.valueOf(s.toString());
            double value2 = Double.valueOf(String.format(Locale.CHINESE, "%.3f", value1));

            // 如果录入超出 3 位小数，则重新格式化，保留 3 位小数
            if (Double.compare(value1, value2) != 0) {
                isHandleTextChanged = true;
                edtQuantity.setText(String.format(Locale.CHINESE, "%.3f", value1));
                edtQuantity.setSelection(edtQuantity.length());
            }

            // 称重商品或等价，显示“件数”；非称重商品，显示“重量”
            txtQuantity.setText(mPluModel.isWeight() || (Double.compare(value2, Math.round(value2)) == 0) ? "件数" : "重量");

            calcTotal();
        } finally {
            isHandleTextChanged = false;
        }
    }

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                edtSpec.setText(s);
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    @OnTextChanged(R.id.edtSpec)
    public void onTextChangedSpec(CharSequence s) {
        if (isHandleTextChanged || StringUtils.isEmpty(s.toString())) {
            return;
        }
        String spec = edtSpec.getText().toString();
        if (spec.length() > 20) {
            CommonTools.toastShort("规格最长为20个字符");
            edtSpec.setText(spec.substring(0, 20));
            edtSpec.setSelection(edtSpec.length());
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            edtSpec.setText(mScanner.getScanResult(data));
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    /**
     * 使用单价、折扣计算售价。
     */
    private void calcSalePrice() {
        if (edtTagPrice.length() == 0 || edtDiscount.length() == 0) {
            return;
        }

        try {
            isHandleTextChanged = true;

            String rate = BigDecimalUtils.subtract("100", edtDiscount.getText().toString());
            if (BigDecimalUtils.isLess(rate, "0")) {
                CommonTools.toastLong("折扣率不能小于 0");
                return;
            }
            if (BigDecimalUtils.isGreater(rate, "100")) {
                CommonTools.toastLong("折扣率不能大于 100");
                return;
            }

            String price = StringUtils.changeF2Y(BigDecimalUtils.multiply(edtTagPrice.getText().toString(), edtDiscount.getText().toString()));
            edtPrice.setText(price);
            if (GeneralSettings.INSTANCE.isChangePrice()) {
                edtDscTotal.setText("0.00");
            }

            calcTotal();
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
        } finally {
            isHandleTextChanged = false;
        }
    }

    /**
     * 使用单价、折扣计算售价，不改单项优惠。
     */
    private void calcPrice() {
        if (edtTagPrice.length() == 0 || edtDiscount.length() == 0) {
            return;
        }

        try {
            isHandleTextChanged = true;

            String rate = BigDecimalUtils.subtract("100", edtDiscount.getText().toString());
            if (BigDecimalUtils.isLess(rate, "0")) {
                CommonTools.toastLong("折扣率不能小于 0");
                return;
            }
            if (BigDecimalUtils.isGreater(rate, "100")) {
                CommonTools.toastLong("折扣率不能大于 100");
                return;
            }

            String price = StringUtils.changeF2Y(BigDecimalUtils.multiply(edtTagPrice.getText().toString(), edtDiscount.getText().toString()));
            edtPrice.setText(price);

            // calcTotal();
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
        } finally {
            isHandleTextChanged = false;
        }
    }

    /**
     * 使用单价、售价计算折扣。
     */
    private void calcDiscount() {
        if (edtTagPrice.length() == 0 || edtPrice.length() == 0) {
            return;
        }

        try {
            isHandleTextChanged = true;
            if (BigDecimalUtils.isGreater(edtPrice.getText().toString(), edtTagPrice.getText().toString())) {
                CommonTools.toastLong("售价不能大于商品标价");
                edtPrice.setText(edtPrice.getText().toString().substring(0, edtPrice.getText().toString().length() - 1));
                edtPrice.setSelection(edtPrice.getText().toString().length());
                return;
            }

            String rate = BigDecimalUtils.divide(BigDecimalUtils.multiply(edtPrice.getText().toString(), "100"), edtTagPrice.getText().toString(), 4);
            edtDiscount.setText(BigDecimalUtils.decimalFormat(rate, 2));

            calcTotal();
        } catch (Exception ex) {
            if (!GeneralSettings.INSTANCE.isZeroPriceSale()) {
                showHintDialog(ex.getMessage());
            }
        } finally {
            isHandleTextChanged = false;
        }
    }

    private void calcTotal() {
        if (StringUtils.isEmpty(edtPrice.getText().toString()) || StringUtils.isEmpty(edtQuantity.getText().toString())) {
            return;
        }
        String total;
        if (GeneralSettings.INSTANCE.isChangePrice() && !edtDscTotal.getText().toString().isEmpty()) {
            total = BigDecimalUtils.subtract(BigDecimalUtils.multiply(edtPrice.getText().toString(), edtQuantity.getText().toString()), edtDscTotal.getText().toString());
        } else {
            total = BigDecimalUtils.multiply(edtPrice.getText().toString(), edtQuantity.getText().toString());
        }
        txtTotal.setText(BigDecimalUtils.decimalFormat(total));
    }

    /**
     * 使用单项优惠计算合计。
     */
    private void calcTotalbyDsc() {
        if (StringUtils.isEmpty(edtDscTotal.getText().toString()) || StringUtils.isEmpty(edtQuantity.getText().toString()) || StringUtils.isEmpty(edtPrice.getText().toString())) {
            return;
        }
        try {
            isHandleTextChanged = true;
            edtDiscount.setText("100");
            calcPrice();
            String total = BigDecimalUtils.subtract(BigDecimalUtils.multiply(edtPrice.getText().toString(), edtQuantity.getText().toString(), 2), edtDscTotal.getText().toString());

            String err2 = mPresenter.totalValidator.validate(total);
            if (err2 != null) {
                CommonTools.toastShort("输入单项优惠过大");
                edtDscTotal.setText("");
                total = BigDecimalUtils.multiply(edtPrice.getText().toString(), edtQuantity.getText().toString(), 2);
                txtTotal.setText(BigDecimalUtils.decimalFormat(total));
                return;
            }
            txtTotal.setText(BigDecimalUtils.decimalFormat(total));

        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
        } finally {
            isHandleTextChanged = false;
        }

    }

    @Override
    public void checkNoCodePluSuccess(BaseSalePluModel pluModel) {
        Bundle bundle = new Bundle();
        bundle.putSerializable("plu", pluModel);
        Intent intent = new Intent();
        intent.putExtras(bundle);

        setResult(ActivityConstants.RESULT_SALE_TO_SET_PLU_PRICE, intent);
        finish();
    }
}
