/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.vip;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.hisense.crm.vip.Vip;
import com.hisense.crm.vip.VipSex;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.adapter.VipPointsProjectAdapter;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;

import org.apache.commons.lang3.StringUtils;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 会员信息（显示会员查询结果）。
 *
 * @author xieliyan   2017-03-27
 * @author wangwenyue 2017-06-09 增加会员状态和会员卡状态
 * @author zhangsai   2017-11-15 HIPOSAPP-212
 * @author zhangsai   2017-12-27 HIPOSAPP-258 使用序列化后的 Vip 实例初始化界面
 */
@ActivityLayoutInject(R.layout.activity_vip_info)
public class VipInfoActivity extends BaseActivity {

    @BindView(R.id.lytTitle)
    LinearLayout lytTitle;

    @BindView(R.id.lytAvatarBackground)
    LinearLayout lytAvatarBackground;

    @BindView(R.id.imgAvatar)
    ImageView imgAvatar;

    @BindView(R.id.txtVipName)
    TextView mTxtVipName;

    @BindView(R.id.imgSex)
    ImageView imgSex;

    @BindView(R.id.txtHint)
    TextView txtHint;

    @BindView(R.id.txtVipType)
    TextView mTxtVipType;

    @BindView(R.id.txtVipCardStatus)
    TextView txtVipCardStatus;

    @BindView(R.id.txtBirthday)
    TextView txtBirthday;

    @BindView(R.id.txtCare)
    TextView txtCare;

    @BindView(R.id.txtTel)
    TextView txtTel;
    @BindView(R.id.lvwVipPoints)
    ListView lvwVipPoints;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    private void init() {
        Bundle bundle = this.getIntent().getExtras();
        Vip vip = (Vip) bundle.getSerializable("vip");
        String vipName = vip.name();
        if (!TextUtils.isEmpty(vipName)) {
            vipName = vipName.length() == 2 ? vipName.substring(0, 1) + "*" : vipName.length() >= 3 ? vipName.substring(0, 1) + "*" + vipName.substring(2) : vipName;
        }
        // 会员姓名
        mTxtVipName.setText(vipName);

        // 性别
        if (vip.sex() == VipSex.Female) {
            imgSex.setImageResource(R.drawable.ic_female);
        } else {
            imgSex.setImageResource(R.drawable.ic_male);
        }

        // 会员类别
        mTxtVipType.setText(String.format("%s - %s", vip.card().cardType().code(), vip.card().cardType().name()));

        // 会员提示信息
        if (StringUtils.isEmpty(vip.hintMessage())) {
            txtHint.setVisibility(View.GONE);
        } else {
            txtHint.setText(vip.hintMessage());
        }

        // 会员卡状态
        txtVipCardStatus.setText(vip.card().status().name());
        if (vip.card().status().isRegistered() ||
                vip.card().status().isSold() ||
                vip.status().isUnReviewed()) {
            imgAvatar.setImageResource(R.drawable.vip_unchecked);
            lytTitle.setBackgroundColor(0xffe0e0e0);
            lytAvatarBackground.setBackgroundColor(0xffe0e0e0);
        } else if (!vip.card().status().isNormal() ||
                vip.status().isDropped() ||
                vip.status().isUnknown()) {
            imgAvatar.setImageResource(R.drawable.vip_dropped);
            lytTitle.setBackgroundColor(0xffffcad2);
            lytAvatarBackground.setBackgroundColor(0xffffcad2);
            txtVipCardStatus.setBackgroundColor(0xfff04b63);
        }

        // 生日
        if (StringUtils.isWhitespace(vip.birthday())) {
            txtBirthday.setTextColor(0xfff04b63);
        } else {
            txtBirthday.setText(vip.birthday());
        }

        // 会员关怀信息
        if (StringUtils.isEmpty(vip.careMessage())) {
            txtCare.setVisibility(View.GONE);
        } else {
            txtCare.setText(vip.careMessage());
        }
        String mobile = vip.mobile();
        if (mobile != null && mobile.length() > 7) {
            mobile = mobile.substring(0, 3) + "****" + mobile.substring(7);
        }
        // 电话
        txtTel.setText(mobile);

        // 积分方案
        VipPointsProjectAdapter queryMemberAdapter = new VipPointsProjectAdapter(this, vip.projects());
        lvwVipPoints.setAdapter(queryMemberAdapter);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @OnClick(R.id.btnGoBack)
    public void onViewClicked() {
        onBackPressed();
    }
}
