/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.vip;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.RelativeLayout;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.ysy.card.CardBaseMvpActivity;
import com.hisense.hiaposcmp.activity.ysy.sale.SaleActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.CombinedEditText;
import com.hisense.hiaposcmp.presenter.VipLoginPresenter;
import com.hisense.hiaposcmp.view.IView.IVipLoginView;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 会员登陆
 *
 * @author caokai      2017-03-16 HIPOSAPP-21
 * @author xieliyan    2017-10-17 HIPOSAPP-180 显示当前会员卡类别：获取会员卡类型
 * @author zhangsai    2018-01-24 HIPOSAPP-263 在会员查询界面查询其它状态的会员，如：已注销；提示：注销
 * @author guolina     2018-04-09 HIPOSAPP-378 添加易收银会员注册
 * @author xieliyan    2018-04-11 HIPOSAPP-383 增加会员状态的判断
 * @author guolina     2018-04-11 HIPOSAPP-378 界面优化
 * @author xieliyan    2018-04-20 HIPOSAPP-411 未注册手机号登录时判断是否注册会员
 * @author xieliyan    2018-05-15 HIPOSAPP-451 会员注册开关启用时，未注册手机号登录时判断是否注册会员
 * @author guolina     2018-07-23 HIPOSAPP-680 修复页面销毁后dialog仍在运行时的闪退
 * @author guolina     2018-08-20 HIPOSAPP-750 会员登陆界面增加扫描功能
 * @author guolina     2018-12-03 HIPOSAPP-859 会员登陆支持摄像头扫码
 * @author guolina     2018-12-29 HIPOSAPP-953 不启用"会员注册"时对齐标题
 * @author zhangsai    2019-01-30 HIPOSAPP-1002 使用 IScanner 扫描器实例扫描
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author wangyaogang 2019-05-28 HIPOSAPP-1117 会员认证方式可配置
 * @author wangyaogang 2019-06-06 HIPOSAPP-1162 支持会员动态码登录,仅在会员认证方式为会员码时生效
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 * @author dukunbiao(null)  2019-09-29  重构
 * @author guolina     2020-04-27 HIAPOSCMP-732 会员登录界面刷卡出错后无法继续刷卡
 * @author liuyazeng    2019-09-25 HIPOSAPP-1417 会员手机号码登录、会员扫码登录可配置
 */
@ActivityLayoutInject(R.layout.activity_vip_login)
public class VipLoginActivity extends CardBaseMvpActivity<IVipLoginView, VipLoginPresenter> implements IVipLoginView {

    @BindView(R.id.txtCardNo)
    public CombinedEditText txtCardNo;

    @BindView(R.id.relativeRegister)
    public RelativeLayout rytRegister;

    private boolean mIsFromSale = false;            // 是否从销售界面进入
    private String mTranType = "";                  // 是否预销售

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        init();
    }

    @Override
    protected VipLoginPresenter createPresenter() {
        return new VipLoginPresenter();
    }

    private void init() {
        Intent intent = this.getIntent();
        if (intent != null) {
            mTranType = intent.getStringExtra("tranType");

            String afterLogin = intent.getStringExtra("afterLogin");
            if ("1".equals(afterLogin)) {
                mIsFromSale = true;
            }
        }

        mScanner = getScanner(this);

        if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
            txtCardNo.getEdit().setFocusable(false);
            txtCardNo.getEdit().setFocusableInTouchMode(false);
            txtCardNo.getEdit().setHint("请扫描会员码");
        }

        if (GeneralSettings.INSTANCE.isVipRegisterEnabled()) {
            rytRegister.setVisibility(View.VISIBLE);
        } else {
            rytRegister.setVisibility(View.INVISIBLE);
        }
        if (GeneralSettings.INSTANCE.isScanVisibility()) {
            txtCardNo.setDefaultIconVisibility(View.GONE);
        }
        if (GeneralSettings.INSTANCE.isMobileLogin()) {
            txtCardNo.setHint("会员卡号/刷卡");
        }
    }

    @OnClick(R.id.txtCardNo)
    public void onClickCardNo() {
        scan();
    }

    @OnClick(R.id.relativeRegister)
    public void onClickRegister() {
        Intent intent = new Intent(VipLoginActivity.this, VipRegisterActivity.class);
        intent.putExtra("isFromSale", mIsFromSale);
        intent.putExtra("tranType", mTranType);
        startActivity(intent);
        finish();
    }

    @OnClick(R.id.btnGoBack)
    public void onClickGoBack() {
        if (mIsFromSale) {
            onBackPressed();
        } else {
            finish();
        }
    }

    @OnClick(R.id.btnVip)
    public void onClickVip() {
        String cardNo = txtCardNo.getText().toString().trim();
        if (GeneralSettings.INSTANCE.isMobileLogin()) {
            String REGEX_MOBILE = "^1\\d{10}$";
            Pattern REGEX_MOBILE_PATTERN = Pattern.compile(REGEX_MOBILE);
            Matcher matcher = REGEX_MOBILE_PATTERN.matcher(cardNo);
            if (matcher.matches()) {
                showHintDialog("禁止手机号登录会员");
                return;
            }
        }
        mPresenter.vipQuery(cardNo);

    }

    @OnClick(R.id.btnNonVip)
    public void onClickNonVip() {
        vipLoginSuccess(null);
    }

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                txtCardNo.setText(s);
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    @Override
    protected void onCardRead(CardNo cardNo) {
        mPresenter.vipQuery(cardNo);
    }

    /**
     * 接收Activity结果值
     *
     * @param requestCode 请求编号。
     * @param resultCode  返回编号。
     * @param data        返回数据。
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            txtCardNo.setText(mScanner.getScanResult(data));
        }
    }

    @Override
    public void vipLoginSuccess(Vip vip) {
        Intent intent;

        if (mIsFromSale) {
            intent = new Intent();
        } else {
            intent = new Intent(VipLoginActivity.this, SaleActivity.class);
        }

        // 如果使用会员登录，则传入会员信息。非会员登录，无需传入此值。
        if (vip != null) {
            intent.putExtra("vip", vip);
        }

        if (mIsFromSale) {
            VipLoginActivity.this.setResult(ActivityConstants.RESULT_SALE_TO_VIP_LOGIN, intent);
        } else {
            intent.putExtra("tranType", mTranType);
            startActivity(intent);
        }

        finish();
    }

    @Override
    public void vipLoginFail(String msg) {
        showHintDialog(msg);
        VipLoginActivity.super.restartListen();
        if (!StringUtils.isEmpty(txtCardNo.getText().toString())) {


            if (GeneralSettings.INSTANCE.isVipRegisterEnabled() &&
                    CardNo.MobileOrDefault(txtCardNo.getText().toString()).getType() == CardNoType.Mobile) {
                register(txtCardNo.getText().toString());
            }
        }
    }

    @Override
    public void verifyTypeSuccess(Vip vip) {

    }

    @Override
    public void verifyTypeFail(String msg) {

    }

    private void register(final String phoneNo) {
        showHintConfirm("是否注册会员？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                Intent intent = new Intent(VipLoginActivity.this, VipRegisterActivity.class);
                intent.putExtra("isFromSale", mIsFromSale);
                intent.putExtra("tranType", mTranType);
                intent.putExtra("phoneNo", phoneNo);
                startActivity(intent);
            }

            @Override
            public void onError(String err) {
                Intent intent = new Intent(VipLoginActivity.this, SaleActivity.class);
                startActivity(intent);
            }
        });
    }
}
