/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.activity.ysy.vip;


import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.crm.card.CardNo;
import com.hisense.crm.vip.Vip;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.ysy.card.CardBaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.CombinedEditText;
import com.hisense.hiaposcmp.presenter.VipLoginPresenter;
import com.hisense.hiaposcmp.view.IView.IVipLoginView;
import com.hisense.pos.scanner.IScanner;
import com.hisense.pos.scanner.ScannerCallback;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 会员查询。
 *
 * @author xieliyan    2017-03-28
 * @author wangwenyue
 * @author xieliyan    2017-09-27 HIPOSAPP-160
 * @author xieliyan    2017-09-25 HIPOSAPP-160 刷卡：磁条/射频（调用HI98驱动）
 * @author xieliyan    2017-09-27
 * @author xieliyan    2017-10-31 HIPOSAPP-201 使用枚举实现单例模式:更改调用方法
 * @author xieliyan    2017-12-26 HIPOSAPP-257 查询时增加加载动画
 * @author zhangsai    2017-12-27 HIPOSAPP-258 使用 VipService 异步查询会员信息
 * @author xieliyan    2018-01-16 HIPOSAPP-278 调用联迪驱动刷卡
 * @author zhangsai    2018-01-24 HIPOSAPP-263 在会员查询界面查询其它状态的会员，如：已注销；提示：注销
 * @author xieliyan    2018-04-09 HIPOSAPP-382 修复由于App.getContext空指针异常导致的闪退问题
 * @author wangyaogang 2018-12-29 HIPOSAPP-875 会员/储值卡余额/积分余额查询支持条码扫描
 * @author wangyaogang 2019-01-03 HIPOSAPP-875 优化键盘使用逻辑
 * @author zhangsai    2019-01-30 HIPOSAPP-1002 使用 IScanner 扫描器实例扫描
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author wangyaogang 2019-06-06 HIPOSAPP-1162 支持会员动态码登录,仅在会员认证方式为会员码时生效
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 隐藏非必要控件
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 * @author wangyaogang 2019-11-12 HIAPOSCMP-267 解决扫码按钮无效问题
 * @author wangyaogang 2019-11-15 HIAPOSCMP-288 监听键盘完成按钮
 */
@ActivityLayoutInject(R.layout.activity_vip_query)
public class VipQueryActivity extends CardBaseMvpActivity<IVipLoginView, VipLoginPresenter> implements IVipLoginView {

    @BindView(R.id.txtCardNo)
    CombinedEditText txtCardNo;

    @BindView(R.id.lytPassword)
    LinearLayout mLytPassword;

    @BindView(R.id.lytFaceNo)
    LinearLayout mLytFaceNo;

    @BindView(R.id.lytVerify)
    LinearLayout mLytVerify;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mScanner = getScanner(this);
        init();
    }

    @Override
    protected VipLoginPresenter createPresenter() {
        return new VipLoginPresenter();
    }

    private void init() {
        if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
            txtCardNo.getEdit().setFocusable(false);
            txtCardNo.getEdit().setFocusableInTouchMode(false);
            txtCardNo.getEdit().setHint("请扫描会员码");
        }

        txtCardNo.getEdit().setImeOptions(EditorInfo.IME_ACTION_DONE);
        txtCardNo.getEdit().setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int actionId, KeyEvent keyEvent) {
                if (actionId == EditorInfo.IME_ACTION_DONE) {
                    String cardNo = txtCardNo.getText().toString().trim();
                    mPresenter.vipQuery(cardNo);
                }
                return false;
            }
        });

        mLytPassword.setVisibility(View.GONE);
        mLytVerify.setVisibility(View.GONE);
        mLytFaceNo.setVisibility(View.GONE);
    }

    @Override
    protected void onCardRead(CardNo cardNo) {
        mPresenter.vipQuery(cardNo);
    }

    private void scan() {
        mScanner.scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mScanner.close();
                txtCardNo.setText(s);
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (IScanner.SCAN_REQUEST == requestCode && RESULT_OK == resultCode) {
            txtCardNo.setText(mScanner.getScanResult(data));
        }
    }

    @OnClick(R.id.txtCardNo)
    public void onViewClicked() {
        scan();
    }

    @OnClick({R.id.btnGoBack, R.id.btnLogin})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.btnGoBack:
                onBackPressed();
                break;
            case R.id.btnLogin:
                String cardNo = txtCardNo.getText().toString().trim();
                mPresenter.vipQuery(cardNo);
                break;
            default:
                break;
        }
    }

    @Override
    public void vipLoginSuccess(Vip vip) {
        Intent intent = new Intent(VipQueryActivity.this, VipInfoActivity.class);
        intent.putExtra("vip", vip);
        startActivity(intent);
        finish();
    }

    @Override
    public void vipLoginFail(String msg) {
        VipQueryActivity.super.restartListen();
        showHintDialog(msg);
    }

    @Override
    public void verifyTypeSuccess(Vip vip) {

    }

    @Override
    public void verifyTypeFail(String msg) {

    }
}
