package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hipos.plu.PluGiftAction;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 加价换购的购物车销售列表
 *
 * @author xieliyan 2018-11-24
 */

public class GiftItemAdapter extends RecyclerView.Adapter<GiftItemAdapter.ViewHolder> implements View.OnClickListener {
    private Context mContext;

    private List<PluGiftAction> dataList = new ArrayList<>();
    private List<PluGiftAction> dataListSelected = new ArrayList<>();
    private Map<Integer, PluGiftAction> selectedMap = new HashMap<>();
    private RecyclerView mRecyclerView;

    public GiftItemAdapter(RecyclerView recyclerView) {
        this.mContext = recyclerView.getContext();
        this.mRecyclerView = recyclerView;
    }

    private OnRecyclerViewItemClickListener mOnItemClickListener;

    public void setOnItemClickListener(GiftItemAdapter.OnRecyclerViewItemClickListener listener) {
        this.mOnItemClickListener = listener;
    }

    /**
     * 点击监听回调接口
     */
    public interface OnRecyclerViewItemClickListener {
        void onClick(View view, GiftItemAdapter.ViewName viewName, int position);
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext)
                .inflate(R.layout.item_list_shopping_gift_wx, parent, false);
        GiftItemAdapter.ViewHolder viewHolder = new GiftItemAdapter.ViewHolder(view);
        return viewHolder;
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        PluGiftAction item;
        try {
            item = dataList.get(position);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        holder.tvPluName.setText(item.pluName());
        //原价
        holder.originPrice.setText(item.dscPrice());
        if (item.quantity() == null || item.quantity().equals("")) {
            item.setQuantity("0");
        }
        holder.quantity.setText(item.quantity());

        holder.itemView.setTag(position);
        holder.btnDecrease.setTag(position);
        holder.btnIncrease.setTag(position);

        updateIncreaseButtonState(holder, position);
        updateDecreaseButtonState(holder);
        updateTotal(holder, position);
    }

    @Override
    public int getItemCount() {
        return dataList.size();
    }

    /**
     * 设置
     *
     * @param list 换购行为列表
     */
    public void setData(List<PluGiftAction> list) {
        if (dataList.size() > 0) {
            dataList.clear();
        }
        dataList.addAll(list);
        notifyDataSetChanged();
    }

    /**
     * 设置
     *
     * @param pluGiftAction 换购行为
     * @param position      位置
     */
    public void set(PluGiftAction pluGiftAction, int position) {
        try {
            dataList.set(position, pluGiftAction);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 选中的换购商品
     *
     * @param pluGiftAction 换购行为
     * @param position      位置
     */
    public void setDataListSelected(int position, PluGiftAction pluGiftAction) {
        selectedMap.put(position, pluGiftAction);
    }

    /**
     * 移除换购商品
     *
     * @param position 位置
     */
    public void removeSelected(int position) {
        if (selectedMap.size() > 0) {
            PluGiftAction pluGiftAction = selectedMap.get(position);

            if (BigDecimalUtils.isLessAndEquals(pluGiftAction.getQuantity(), "0")) {
                selectedMap.remove(position);
            }
        }

        if (dataListSelected.size() > 0) {
            PluGiftAction pluGiftAction = dataListSelected.get(position);

            dataListSelected.get(position).setQuantity(BigDecimalUtils.subtract(pluGiftAction.quantity(), "1"));
            if (BigDecimalUtils.isLessAndEquals(pluGiftAction.getQuantity(), "0")) {
                dataListSelected.remove(pluGiftAction);
            }
        }
    }

    /**
     * 改变数量
     *
     * @param position 在列表中的位置，从0数
     * @param quantity 数量整数
     */
    public void changeQuantity(int position, int quantity) {
        PluGiftAction pluGiftAction = dataList.get(position);
        pluGiftAction.setQuantity(String.valueOf(quantity));
        notifyDataSetChanged();
    }

    /**
     * 全清空
     */
    public void clear() {
        try {
            dataList.clear();
            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onClick(View v) {
        int position;
        position = (int) v.getTag();

        if (mOnItemClickListener != null) {
            switch (v.getId()) {
                case R.id.root_layout:
                    mOnItemClickListener.onClick(v, ViewName.ITEM, position);
                    break;

                case R.id.btn_decrease:
                    mOnItemClickListener.onClick(v, GiftItemAdapter.ViewName.DECREASE, position);
                    break;
                case R.id.btn_increase:
                    mOnItemClickListener.onClick(v, GiftItemAdapter.ViewName.INCREASE, position);
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * item里面有多个控件可以点击
     */
    public enum ViewName {
        /**
         * 条目
         */
        ITEM,
        /**
         * 增加
         */
        INCREASE,
        /**
         * 减少
         */
        DECREASE
    }

    /**
     * 更新调整状态（减）
     *
     * @param holder 类
     */
    private void updateDecreaseButtonState(GiftItemAdapter.ViewHolder holder) {
        String lowLevel = "0";

        if (holder.quantity.getText() == null || holder.quantity.getText().toString().equals("")) {
            return;
        }
        holder.btnDecrease.setEnabled(compare(lowLevel, holder.quantity.getText().toString()));
        holder.btnDecrease.setBackground(mContext.getResources().getDrawable(R.drawable.button_wx_gift_sub));
    }

    /**
     * 更新调整状态（加）
     *
     * @param holder   类
     * @param position 位置
     */
    private void updateIncreaseButtonState(GiftItemAdapter.ViewHolder holder, int position) {

        PluGiftAction pluGiftAction = dataList.get(position);

        if (holder.quantity.getText() == null || holder.quantity.getText().toString().equals("")) {
            return;
        }
        if (getTotalQuantity() >= Double.valueOf(pluGiftAction.total())) {
            holder.btnIncrease.setEnabled(false);
            holder.btnIncrease.setBackground(mContext.getResources().getDrawable(R.drawable.button_wx_gift_add));

            return;
        }
        String dscTotal;
        String base = "0.0";
        if (pluGiftAction.saleMaxIn().equals(base)) {
            dscTotal = pluGiftAction.baseDscNum();
        } else {
            dscTotal = pluGiftAction.saleMaxIn();
        }
        holder.btnIncrease.setEnabled(compare(holder.quantity.getText().toString(), dscTotal));

        holder.btnIncrease.setBackground(mContext.getResources().getDrawable(R.drawable.button_wx_gift_add));

    }

    /**
     * 更新 {@code item} 指定商品的合计
     *
     * @param holder   类
     * @param position 位置
     */
    private void updateTotal(GiftItemAdapter.ViewHolder holder, int position) {
        PluGiftAction pluGiftAction = dataList.get(position);

        holder.originPrice.setText(pluGiftAction.dscPrice());

        String itemTotal;
        String itemPurchase;
        if (isNum(pluGiftAction)) {
            itemTotal = pluGiftAction.baseConNum() + "件";
            holder.tvItemTotal.setText(itemTotal);
            if (compare("0", pluGiftAction.saleMaxIn()) && compare(pluGiftAction.saleMaxIn(), pluGiftAction.baseDscNum())) {
                itemPurchase = pluGiftAction.saleMaxIn() + "件";
                holder.tvItemPurchase.setText(itemPurchase);
            } else {
                itemPurchase = pluGiftAction.baseDscNum() + "件";
                holder.tvItemPurchase.setText(itemPurchase);
            }
        } else {
            itemTotal = "¥" + pluGiftAction.baseConNum();
            holder.tvItemTotal.setText(itemTotal);
            if (compare("0", pluGiftAction.saleMaxIn()) && compare(pluGiftAction.saleMaxIn(), pluGiftAction.baseDscNum())) {
                itemPurchase = "¥" + pluGiftAction.saleMaxIn();
                holder.tvItemPurchase.setText(itemPurchase);
            } else {
                itemPurchase = "¥" + pluGiftAction.baseDscNum();
                holder.tvItemPurchase.setText(itemPurchase);
            }
        }
    }

    /**
     * 换购类型是否为数量
     *
     * @param pluGiftAction 换购行为
     */
    private boolean isNum(PluGiftAction pluGiftAction) {
        return pluGiftAction.baseDscType().equals("NUM");

    }

    /**
     * 比较大小
     *
     * @param value1 数值1
     * @param value2 数值2
     */
    private boolean compare(String value1, String value2) {
        return Double.valueOf(value1) < Double.valueOf(value2);
    }

    /**
     * 返回已选中的换购商品的数量
     *
     * @return total 已换购数量
     */
    private double getTotalQuantity() {
        double total = 0;
        for (PluGiftAction selectedAction : selectedMap.values()) {
            total = BigDecimalUtils.add(total, Double.valueOf(selectedAction.quantity()));
        }
        return total;
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private LinearLayout rootLayout;
        private TextView tvPluName;
        private TextView originPrice;
        private TextView quantity;
        private Button btnDecrease;
        private Button btnIncrease;

        private TextView tvItemTotal;
        private TextView tvItemPurchase;

        public ViewHolder(View itemView) {
            super(itemView);
            rootLayout = itemView.findViewById(R.id.root_layout);
            tvPluName = itemView.findViewById(R.id.tv_plu_name);
            originPrice = itemView.findViewById(R.id.tv_origin_price);
            quantity = itemView.findViewById(R.id.txt_quantity);
            btnDecrease = itemView.findViewById(R.id.btn_decrease);
            btnIncrease = itemView.findViewById(R.id.btn_increase);

            tvItemTotal = itemView.findViewById(R.id.tv_item_total);
            tvItemPurchase = itemView.findViewById(R.id.tv_item_purchase);
            itemView.setOnClickListener(GiftItemAdapter.this);

            btnDecrease.setOnClickListener(GiftItemAdapter.this);
            btnIncrease.setOnClickListener(GiftItemAdapter.this);
        }
    }
}
