package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.graphics.Paint;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.common.util.ListUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hipos.model.BaseSalePluModel;

import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Locale;

/**
 * 礼品券兑换界面商品列表
 *
 * @author yuminghao 2020-09-30
 */
public class GiftTokenItemAdapter extends BaseAdapter {

    private Context mContext;
    private List<BaseSalePluModel> saleItems;   // 商品列表

    public GiftTokenItemAdapter(Context context, List<BaseSalePluModel> saleItems) {
        this.mContext = context;
        this.saleItems = saleItems;
    }

    public void setData(List<BaseSalePluModel> pluModels) {
        if (!ListUtils.isEmpty(pluModels)) {
            saleItems.clear();
            saleItems.addAll(pluModels);
            notifyDataSetChanged();
        }
    }

    public void clear(){
        saleItems.clear();
        notifyDataSetChanged();
    }

    @Override
    public int getCount() {
        return this.saleItems.size();
    }

    @Override
    public BaseSalePluModel getItem(int position) {
        return this.saleItems.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        final BaseSalePluModel item = getItem(position);

        final ViewHolder holder;
        if (convertView == null) {
            LayoutInflater li = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = li.inflate(R.layout.item_sale_gift_item, parent, false);

            holder = createViewHolder(convertView);
        } else {
            holder = (ViewHolder) convertView.getTag();
        }

        holder.txtPluName.setText(item.getPluName());
        holder.txtPluCode.setText(item.getPluCode());

        if (GeneralSettings.INSTANCE.isNoShopLimits() && !StringUtils.isEmpty(item.getShopName())) {
            holder.txtShopName.setText(String.format("(%s)", item.getShopName()));
        }

        if (item.isWeightOrEquivalent()) {
            // 称重商品，计重
            holder.txtPluWeight.setText(String.format(Locale.CHINESE, "%.3f %s", Double.parseDouble(item.getQuantity()), item.getUnit()));
            holder.lytPluSpec.setVisibility(View.GONE);
            holder.lytPluUnit.setVisibility(View.GONE);
            holder.lytPluWeight.setVisibility(View.VISIBLE);
        } else if (item.isWeight() && item.isCountQuantity()) {
            // 称重商品，计件
            this.setPluSpec(holder, item.getSpec());
            this.setPluUnit(holder, item.getUnit());
            holder.lytPluWeight.setVisibility(View.GONE);
        } else{
            // 非称重商品，录入的整数件数
            this.setPluSpec(holder, item.getSpec());
            this.setPluUnit(holder, item.getUnit());
            holder.lytPluWeight.setVisibility(View.GONE);
        }

        holder.imgDscNormal.setVisibility(item.isNormalDsc() ? View.VISIBLE : View.GONE);
        holder.imgDscVip.setVisibility(item.isVipDsc() ? View.VISIBLE : View.GONE);

        updateTotal(holder, item);
        holder.txtCount.setText(item.getQuantity());

        return convertView;
    }

    /**
     * 设置商品规格。
     *
     * @param holder 商品视图的 Holder
     * @param value  商品规格
     */
    private void setPluSpec(ViewHolder holder, String value) {
        holder.txtPluSpec.setText(value);
        holder.lytPluSpec.setVisibility(StringUtils.isWhitespace(value) ? View.GONE : View.VISIBLE);
    }

    /**
     * 设置商品计量单位。
     *
     * @param holder 商品视图的 Holder
     * @param value  商品计量单位
     */
    private void setPluUnit(ViewHolder holder, String value) {
        holder.txtPluUnit.setText(value);
        holder.lytPluUnit.setVisibility(StringUtils.isWhitespace(value) ? View.GONE : View.VISIBLE);
    }

    /**
     * 更新 {@code item} 指定商品的数量、合计。
     *
     * @param holder 商品视图的 Holder
     * @param item   更新的商品
     */
    private void updateTotal(ViewHolder holder, BaseSalePluModel item) {
        // 只有当存在优惠时，才显示(优惠前)合计
        if (item.isHasTotalDsc()) {
            holder.txtTotal.setText(item.getTotal());
        } else {
            holder.txtTotal.setText("");
        }

        holder.txtTotalDue.setText(item.getTotalRecv());
        holder.txtCount.setText(String.format(Locale.CHINESE, "%d", item.count()));
    }

    /**
     * 创建一个新的视图 Holder，并将其缓存在 {@code convertView} 的 Tag 中。
     *
     * @param convertView 视图
     * @return 新创建的视图 Holder
     */
    private ViewHolder createViewHolder(View convertView) {
        ViewHolder holder = new ViewHolder();

        holder.txtPluName = convertView.findViewById(R.id.txtPluName);
        holder.txtPluCode = convertView.findViewById(R.id.txtPluCode);
        holder.txtShopName = convertView.findViewById(R.id.txtShopName);

        holder.imgDscNormal = convertView.findViewById(R.id.imgDscNormal);
        holder.imgDscVip = convertView.findViewById(R.id.imgDscVip);

        holder.txtPluSpec = convertView.findViewById(R.id.txtPluSpec);
        holder.txtPluUnit = convertView.findViewById(R.id.txtPluUnit);
        holder.txtPluWeight = convertView.findViewById(R.id.txtPluWeight);
        holder.lytPluSpec = convertView.findViewById(R.id.lytPluSpec);
        holder.lytPluUnit = convertView.findViewById(R.id.lytPluUnit);
        holder.lytPluWeight = convertView.findViewById(R.id.lytPluWeight);

        holder.txtCount = convertView.findViewById(R.id.txtCount);
        holder.txtTotalDue = convertView.findViewById(R.id.txtTotalDue);
        holder.txtTotal = convertView.findViewById(R.id.txtTotal);
        holder.txtTotal.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG);

        convertView.setTag(holder);

        return holder;
    }

    /**
     * 视图 Holder。
     */
    private static class ViewHolder {
        TextView txtPluName;        // 商品名称
        TextView txtPluCode;        // 商品编码
        TextView txtShopName;       // 专柜名称

        TextView txtPluUnit;        // 单位
        LinearLayout lytPluUnit;    // 单位布局
        TextView txtPluSpec;        // 规格
        LinearLayout lytPluSpec;    // 规格布局
        TextView txtPluWeight;      // 重量
        LinearLayout lytPluWeight;  // 重量布局

        TextView txtTotal;          // 应收合计(优惠前)
        TextView txtTotalDue;       // 实收合计(优惠后)
        TextView txtCount;          // 购买件数

        ImageView imgDscNormal;     // 普通商品优惠
        ImageView imgDscVip;        // 会员优惠
    }
}
