/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.hisense.common.util.ListUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.service.payment.PayMethodService;
import com.hisense.hipos.model.SalePayModel;

import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 支付方式列表适配器。
 *
 * @author zhenghao 2017-03-10
 * @author zhangsai 2017-11-15 HIPOSAPP-212
 * @author xieliyan 2017-12-15 HIPOSAPP-235
 * @author zhangsai 2018-12-08 HIPOSAPP-888 修改布局样式，改用ArrayList<Payment>显示支付信息
 * @author zhangsai 2018-12-08 HIPOSAPP-892 优化支付撤销逻辑，改用ArrayList<SalePayItem>显示支付信息
 * @author xieliyan 2020-04-17 HIAPOSCMP-696 测试反馈支付方式高度应该保持一致
 */
public class PayItemsAdapter extends BaseAdapter {

    private LayoutInflater mInflater;
    private ArrayList<SalePayModel> payments;

    public PayItemsAdapter(Context context, ArrayList<SalePayModel> payments) {
        mInflater = LayoutInflater.from(context);
        this.payments = payments;
    }

    public void setData(List<SalePayModel> payModels) {
        if (!ListUtils.isEmpty(payModels)) {
            payments.clear();
            payments.addAll(payModels);
            notifyDataSetChanged();
        }
    }

    @Override
    public int getCount() {
        return payments.size();
    }

    @Override
    public Object getItem(int i) {
        return null;
    }

    @Override
    public long getItemId(int i) {
        return 0;
    }

    @Override
    public View getView(int i, View view, ViewGroup viewGroup) {
        PayViewHolder holder;
        if (view == null) {
            view = mInflater.inflate(R.layout.pay_item_list, null);
            holder = new PayViewHolder();
            holder.imgPayMethod = view.findViewById(R.id.imgPayMethod);
            holder.txtPayName = view.findViewById(R.id.txtPayName);
            holder.txtPayNo = view.findViewById(R.id.txtPayNo);
            holder.txtTotal = view.findViewById(R.id.txtTotal);

            view.setTag(holder);
        } else {
            holder = (PayViewHolder) view.getTag();
        }

        holder.txtPayName.setText(payments.get(i).getPayName());
        holder.txtTotal.setText(payments.get(i).getTotalPaid());
        holder.imgPayMethod.setBackgroundResource(
                PayMethodService.getIconByCode(payments.get(i).getPayCode()));
        holder.imgPayMethod.invalidate();

//        if (StringUtils.isWhitespace(payments.get(i).getBuyerLogonId())) {
//            holder.txtPayNo.setVisibility(View.GONE);
//        } else {
            holder.txtPayNo.setVisibility(View.VISIBLE);
            holder.txtPayNo.setText(payments.get(i).getBuyerLogonId());
     //   }

        return view;
    }

    private static class PayViewHolder {
        ImageView imgPayMethod; // 支付图标
        TextView txtPayName;    // 支付方式名称
        TextView txtTotal;      // 支付金额
        TextView txtPayNo;      // 支付号
    }
}
