package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.ListUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hipos.model.SalePayModel;

import java.util.ArrayList;
import java.util.List;

/**
 * {功能描述}
 *
 * @author wangyaogang 2019-12-27 HIAPOSCMP-455 标准版支付功能开发
 */
public class PayedMethodStdAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private OnItemClickListener mOnItemClickListener;
    private LayoutInflater mLayoutInflater;
    private ArrayList<SalePayModel> payments;

    public PayedMethodStdAdapter(Context context, ArrayList<SalePayModel> payments) {
        mLayoutInflater = LayoutInflater.from(context);
        this.payments = payments;
    }

    public void setOnItemClickListener(OnItemClickListener mOnItemClickListener) {
        this.mOnItemClickListener = mOnItemClickListener;
    }

    public void setData(List<SalePayModel> payModels) {
        if (!ListUtils.isEmpty(payModels)) {
            payments.clear();
            payments.addAll(payModels);
            notifyDataSetChanged();
        }
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {

        return new ContentViewHolder(mLayoutInflater.inflate(R.layout.item_std_payed_method, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        if (holder instanceof ContentViewHolder) {
            SalePayModel payModel = payments.get(position);

            ((ContentViewHolder) holder).tvPayedNo.setText(payModel.getBuyerLogonId());
            ((ContentViewHolder) holder).tvPayedName.setText(payModel.getPayName());
            ((ContentViewHolder) holder).tvPayedAmount.setText(payModel.getTotalPaid());
            if (mOnItemClickListener != null) {
                holder.itemView.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        int position = holder.getLayoutPosition();
                        mOnItemClickListener.onItemClick(holder.itemView, position);
                    }
                });
            }
        }
    }

    @Override
    public int getItemCount() {
        return payments.size();
    }

    //内容 ViewHolder
    public static class ContentViewHolder extends RecyclerView.ViewHolder {
        private TextView tvPayedNo, tvPayedName, tvPayedAmount;

        public ContentViewHolder(View itemView) {
            super(itemView);
            tvPayedNo = itemView.findViewById(R.id.item_tv_payed_no);
            tvPayedName = itemView.findViewById(R.id.item_tv_payed_name);
            tvPayedAmount = itemView.findViewById(R.id.item_tv_payed_amount);
        }
    }

    public interface OnItemClickListener {
        void onItemClick(View view, int position);
    }
}
