package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.ReturnItemViewModel;

import java.util.List;

/**
 * 销售商品列表adapter
 *
 * @author dukunbiao(null)  2019-12-20  新建
 */
public class ReturnItemStdAdapter extends RecyclerView.Adapter<ReturnItemStdAdapter.ViewHolder> implements View.OnClickListener {

    private Context mContext;

    private List<ReturnItemViewModel> mSaleItems;

    private ViewHolder mViewHolder;

    public ReturnItemStdAdapter(RecyclerView recyclerView, List<ReturnItemViewModel> data) {
        this.mContext = recyclerView.getContext();
        this.mSaleItems = data;
    }

    private OnRecyclerItemClickListener mOnItemClickListener;

    public void setOnItemClickListener(OnRecyclerItemClickListener listener) {
        this.mOnItemClickListener = listener;
    }


    @Override
    public int getItemCount() {
        return this.mSaleItems.size();
    }

    @Override
    @NonNull
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(R.layout.item_list_return_std_plu, parent, false);
        return new ReturnItemStdAdapter.ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        mViewHolder = holder;
        ReturnItemViewModel saleItem;
        try {
            saleItem = this.mSaleItems.get(position);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
        //更新商品选中状态
        if (saleItem.isChecked()) {
            holder.mImgViewReturnPluSelect.setBackgroundResource(R.drawable.btn_std_plu_item_selected);
        } else {
            holder.mImgViewReturnPluSelect.setBackgroundResource(R.drawable.btn_std_plu_item_unselected);
        }


        holder.mTvReturnStdPluName.setText(saleItem.pluName());
        //holder.mTvReturnStdPluCount.setText(String.valueOf(saleItem.count()));
        holder.mTvReturnStdPluCount.setText(saleItem.count());
        holder.mTvReturnStdPluUnit.setText(saleItem.unit());
        holder.mTvReturnStdPluSpec.setText(saleItem.spec());

        holder.mTvReturnStdPluTotalRecv.setText(saleItem.total());
        holder.mTvSaleStdPluTotalRefund.setText(saleItem.totalRefund());
        LoggerUtils.getLogger().debug("退货金额" + holder.mTvSaleStdPluTotalRefund.getText().toString());

        holder.itemView.setTag(position);
        holder.mImgViewReturnPluSelect.setTag(position);
        holder.mBtnReturnStdPluDecrease.setTag(position);
        holder.mBtnReturnStdPluDecrease.setEnabled(saleItem.testReturnLess());
        holder.mBtnReturnStdPluIncrease.setTag(position);
        holder.mBtnReturnStdPluIncrease.setEnabled(saleItem.testReturnMore());
        if (saleItem.isChecked()) {
            holder.mImgViewReturnPluSelect.setBackgroundResource(R.drawable.btn_std_plu_item_selected);
        } else {
            holder.mImgViewReturnPluSelect.setBackgroundResource(R.drawable.btn_std_plu_item_unselected);
        }

        if (saleItem.testReturnLess()) {
            //    holder.mBtnReturnStdPluIncrease.setBackgroundResource(R.drawable.border_radius_left_enable);
            holder.mBtnReturnStdPluIncrease.setTextColor(Color.parseColor("#ffffff"));
        } else {
            //   holder.mBtnReturnStdPluDecrease.setBackgroundResource(R.drawable.border_radius_left);
            holder.mBtnReturnStdPluDecrease.setTextColor(Color.parseColor("#999999"));
        }

        if (saleItem.testReturnMore()) {
            //  holder.mBtnReturnStdPluIncrease.setBackgroundResource(R.drawable.border_radius_right_enable);
            holder.mBtnReturnStdPluIncrease.setTextColor(Color.parseColor("#ffffff"));
        } else {
            //    holder.mBtnReturnStdPluDecrease.setBackgroundResource(R.drawable.border_radius_right);
            holder.mBtnReturnStdPluDecrease.setTextColor(Color.parseColor("#999999"));
        }
    }

    @Override
    public void onClick(View v) {
        int position = (int) v.getTag();
        if (mOnItemClickListener != null) {
            switch (v.getId()) {
                case R.id.llyReturnStdPluItem:
                case R.id.imgViewReturnPluSelect:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.ITEM, position);
                    break;
                case R.id.btnReturnStdPluDecrease:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.DECREASE, position);
                    break;
                case R.id.btnReturnStdPluIncrease:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.INCREASE, position);
                    break;
                default:
                    break;
            }
        }
    }

    public void btnDecressState(boolean isEnable) {
        mViewHolder.mBtnReturnStdPluDecrease.setEnabled(isEnable);
    }

    public void btnIncressState(boolean isEnable) {
        mViewHolder.mBtnReturnStdPluIncrease.setEnabled(isEnable);
    }

    public void returnPluUnSelected() {
        mViewHolder.mImgViewReturnPluSelect.setBackgroundResource(R.drawable.btn_std_plu_item_unselected);
    }

    public void returnPluSelected() {
        mViewHolder.mImgViewReturnPluSelect.setBackgroundResource(R.drawable.btn_std_plu_item_selected);
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private LinearLayout mLlyReturnStdPluItem;
        private ImageView mImgViewReturnPluSelect;
        private TextView mTvReturnStdPluName;
        private TextView mTvReturnStdPluUnit;
        private TextView mTvReturnStdPluSpec;
        private Button mBtnReturnStdPluDecrease;
        private TextView mTvReturnStdPluCount;
        private Button mBtnReturnStdPluIncrease;
        private TextView mTvReturnStdPluTotalRecv;
        private TextView mTvSaleStdPluTotalRefund;

        public ViewHolder(View itemView) {
            super(itemView);
            mLlyReturnStdPluItem = itemView.findViewById(R.id.llyReturnStdPluItem);
            mImgViewReturnPluSelect = itemView.findViewById(R.id.imgViewReturnPluSelect);
            mTvReturnStdPluName = itemView.findViewById(R.id.tvReturnStdPluName);
            mTvReturnStdPluUnit = itemView.findViewById(R.id.tvReturnStdPluUnit);
            mTvReturnStdPluSpec = itemView.findViewById(R.id.tvReturnStdPluSpec);
            mBtnReturnStdPluDecrease = itemView.findViewById(R.id.btnReturnStdPluDecrease);
            mTvReturnStdPluCount = itemView.findViewById(R.id.tvReturnStdPluCount);
            mBtnReturnStdPluIncrease = itemView.findViewById(R.id.btnReturnStdPluIncrease);
            mTvReturnStdPluTotalRecv = itemView.findViewById(R.id.tvReturnStdPluTotalRecv);
            mTvSaleStdPluTotalRefund = itemView.findViewById(R.id.tvSaleStdPluTotalRefund);


            mLlyReturnStdPluItem.setOnClickListener(ReturnItemStdAdapter.this);
            mImgViewReturnPluSelect.setOnClickListener(ReturnItemStdAdapter.this);
            mBtnReturnStdPluDecrease.setOnClickListener(ReturnItemStdAdapter.this);
            mBtnReturnStdPluIncrease.setOnClickListener(ReturnItemStdAdapter.this);
        }
    }
}
