package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.graphics.Paint;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.hisense.common.util.ListUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hipos.model.BaseSalePluModel;

import java.util.ArrayList;
import java.util.List;

/**
 * 购物车销售列表adapter
 *
 * @author zhuguangsheng on 2018/7/2.
 * @author dukunbiao(null)  2019-07-24  重构
 */
public class SaleItemAdapter extends RecyclerView.Adapter<SaleItemAdapter.ViewHolder> implements View.OnClickListener {

    private Context mContext;
    private List<BaseSalePluModel> dataList = new ArrayList<>();
    private RecyclerView mRecyclerView;
    private int selectedPos = 0;
    private boolean isWxStyle;

    public void setData(List<BaseSalePluModel> pluModels) {
        if (!ListUtils.isEmpty(pluModels)) {
            dataList.clear();
            dataList.addAll(pluModels);
            notifyDataSetChanged();
        }
    }

    /**
     * 在列表指定位置添加一项
     *
     * @param salePluModel
     * @param position
     */
    public void addPlu(int position, BaseSalePluModel salePluModel) {
        try {
            dataList.add(position, salePluModel);
            notifyItemChanged(position);
            selectedPos = position;

            //添加也需要整体刷新，否则原来不该有的高亮无法清除
            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 设置
     *
     * @param pluModel
     * @param position
     */
    public void setPlu(int position, BaseSalePluModel pluModel) {
        try {
            dataList.set(position, pluModel);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 改变数量
     *
     * @param position 在列表中的位置，从0数
     */
    public void changeQuantity(int position) {
        selectedPos = position;
    }

    /**
     * 移除
     *
     * @param position
     */
    public void removePlu(int position) {
        try {
            dataList.remove(position);
            notifyItemChanged(position);

            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 全清空
     */
    public void clearPlu() {
        try {
            dataList.clear();
            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setItemSelected(int position) {
        this.selectedPos = position;
        notifyDataSetChanged();
    }

    public SaleItemAdapter(RecyclerView recyclerView, boolean isWxStyle) {
        this.mContext = recyclerView.getContext();
        this.mRecyclerView = recyclerView;
        this.isWxStyle = isWxStyle;
    }

    private OnRecyclerViewItemClickListener mOnItemClickListener;

    public void setOnItemClickListener(SaleItemAdapter.OnRecyclerViewItemClickListener listener) {
        this.mOnItemClickListener = listener;
    }

    /**
     * 点击监听回调接口
     */
    public interface OnRecyclerViewItemClickListener {
        void onClick(View view, SaleItemAdapter.ViewName viewName, int position);
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        int layout;
        if (isWxStyle) {
            layout = R.layout.item_list_shopping_cart_wx;
        } else if(CommonUtils.isAliLandStyle()){
            layout = R.layout.item_list_shopping_cart_ant_land;
        } else {
            layout = R.layout.item_list_shopping_cart_ant;
        }

        View view = LayoutInflater.from(mContext).inflate(layout, parent, false);
        SaleItemAdapter.ViewHolder viewHolder = new SaleItemAdapter.ViewHolder(view);
        return viewHolder;
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.itemView.setSelected(selectedPos == position);

        BaseSalePluModel pluModel;
        try {
            pluModel = dataList.get(position);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        if (isWxStyle) {
            LinearLayout rootLayout = holder.itemView.findViewById(R.id.root_layout);
            if (selectedPos == position) {
                rootLayout.setBackgroundResource(R.drawable.shape_wx_sale_item_selected);
            } else {
                rootLayout.setBackgroundResource(R.drawable.shape_wx_sale_item_normal);
            }
        }

        holder.tvPluName.setText(pluModel.getPluName());
        holder.originPrice.setText(pluModel.getTagPrice());
        holder.originPrice.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG);
        holder.quantity.setText(String.valueOf(pluModel.count()));
        holder.discount.setText(pluModel.getTotalDsc());
        holder.tvTotal.setText(pluModel.getTotalRecv());

        holder.itemView.setTag(position);
        holder.btnDeleteItem.setTag(position);
        holder.btnDecrease.setTag(position);
        holder.btnIncrease.setTag(position);

        if (pluModel.isWeightOrEquivalent()) {
            updateWeightTotal(holder, position);
        }
        updateDecreaseButtonState(holder, position);
        updateTotal(holder, position);
    }

    @Override
    public int getItemCount() {
        return dataList.size();
    }

    @Override
    public void onClick(View v) {
        int position;
        position = (int) v.getTag();
        if (mOnItemClickListener != null) {
            switch (v.getId()) {
                case R.id.root_layout:
                    mOnItemClickListener.onClick(v, ViewName.ITEM, position);
                    break;
                case R.id.btn_delete_item:
                    mOnItemClickListener.onClick(v, SaleItemAdapter.ViewName.DELETE, position);
                    break;
                case R.id.btn_decrease:
                    mOnItemClickListener.onClick(v, SaleItemAdapter.ViewName.DECREASE, position);
                    break;
                case R.id.btn_increase:
                    mOnItemClickListener.onClick(v, SaleItemAdapter.ViewName.INCREASE, position);
                    break;
                default:
                    break;
            }
        }
    }

    public class ViewHolder extends RecyclerView.ViewHolder {//implements AnimateViewHolder
        private LinearLayout rootLayout;
        private TextView tvPluName;
        private Button btnDeleteItem;
        private TextView originPrice;
        private TextView discount;
        private TextView quantity;
        private Button btnDecrease;
        private Button btnIncrease;
        private TextView tvTotal;

        public ViewHolder(View itemView) {
            super(itemView);
            rootLayout = itemView.findViewById(R.id.root_layout);
            tvPluName = itemView.findViewById(R.id.tv_plu_name);
            btnDeleteItem = itemView.findViewById(R.id.btn_delete_item);
            originPrice = itemView.findViewById(R.id.tv_origin_price);
            discount = itemView.findViewById(R.id.tv_discount);
            quantity = itemView.findViewById(R.id.tv_quantity);
            btnDecrease = itemView.findViewById(R.id.btn_decrease);
            btnIncrease = itemView.findViewById(R.id.btn_increase);
            tvTotal = itemView.findViewById(R.id.tv_item_sum);

            itemView.setOnClickListener(SaleItemAdapter.this);
            btnDeleteItem.setOnClickListener(SaleItemAdapter.this);
            btnDecrease.setOnClickListener(SaleItemAdapter.this);
            btnIncrease.setOnClickListener(SaleItemAdapter.this);
        }
    }

    /**
     * item里面有多个控件可以点击
     */
    public enum ViewName {
        ITEM,
        DELETE,
        INCREASE,
        DECREASE
    }

    private void updateDecreaseButtonState(SaleItemAdapter.ViewHolder holder, int position) {
        int lowLevel = 1;

        BaseSalePluModel pluModel = dataList.get(position);

        if (pluModel.count() > lowLevel) {
            holder.btnDecrease.setBackground(mContext.getResources().getDrawable(R.drawable.button_wx_sub));
        } else {
            holder.btnDecrease.setBackground(mContext.getResources().getDrawable(R.drawable.button_wx_sub));
        }
    }

    /**
     * 更新 {@code item} 指定商品的合计
     *
     * @param holder
     * @param position
     */
    private void updateTotal(SaleItemAdapter.ViewHolder holder, int position) {
        BaseSalePluModel pluModel = dataList.get(position);

        holder.tvTotal.setText(pluModel.getTotalRecv());
    }

    /**
     * 更新 {@code item} 指定商品的合计
     *
     * @param holder
     * @param position
     */
    private void updateWeightTotal(SaleItemAdapter.ViewHolder holder, int position) {
        BaseSalePluModel pluModel = dataList.get(position);

        holder.originPrice.setText(pluModel.getTotal());
    }
}
