package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.graphics.Paint;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.hisense.hiaposcmp.R;
import com.hisense.hipos.model.BaseSalePluModel;

import java.util.ArrayList;
import java.util.List;

/**
 * 销售商品列表adapter
 *
 * @author dukunbiao(null)  2019-12-20  新建
 */
public class SaleItemStdAdapter extends RecyclerView.Adapter<SaleItemStdAdapter.ViewHolder> implements View.OnClickListener {

    private Context mContext;
    private List<BaseSalePluModel> pluModels = new ArrayList<>();
    private RecyclerView mRecyclerView;
    private boolean isShowPreKey;
    private int selectedPos = 0;

    public SaleItemStdAdapter(RecyclerView recyclerView) {
        this.mContext = recyclerView.getContext();
        this.mRecyclerView = recyclerView;
        this.isShowPreKey = true;
    }

    private OnRecyclerItemClickListener mOnItemClickListener;

    public void setOnItemClickListener(OnRecyclerItemClickListener listener) {
        this.mOnItemClickListener = listener;
    }

    public List<BaseSalePluModel> getPluModels() {
        return pluModels;
    }

    public void setData(List<BaseSalePluModel> pluModels) {
        if (pluModels != null) {
            this.pluModels.clear();
            this.pluModels.addAll(pluModels);
            notifyDataSetChanged();
        }
    }

    public void addPlu(int position, BaseSalePluModel pluModel) {
        try {
            this.pluModels.add(position, pluModel);
            notifyItemChanged(position);
            this.selectedPos = position;

            //添加也需要整体刷新，否则原来不该有的高亮无法清除
            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setPlu(int position, BaseSalePluModel pluModel) {
        try {
            this.pluModels.set(position, pluModel);
            notifyItemChanged(position);
            this.selectedPos = position;
            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void changeQuantity(int position) {
        this.selectedPos = position;
    }

    public void removePlu(int position) {
        try {
            this.pluModels.remove(position);
            notifyItemChanged(position);

            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void clearPlu() {
        try {
            this.pluModels.clear();
            notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setItemSelected(int position) {
        this.selectedPos = position;
        notifyDataSetChanged();
    }

    public void updateUi(boolean isShowPreKey) {
        this.isShowPreKey = isShowPreKey;
        notifyDataSetChanged();
    }

    @Override
    public int getItemCount() {
        return this.pluModels.size();
    }

    @Override
    @NonNull
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(R.layout.item_list_sale_std_plu, parent, false);
        return new SaleItemStdAdapter.ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.itemView.setSelected(selectedPos == position);

        BaseSalePluModel pluModel;
        try {
            pluModel = this.pluModels.get(position);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        if (selectedPos == position) {
            holder.linearSaleStdPluItem.setBackgroundResource(R.drawable.shape_std_sale_item_selected);
        } else {
            holder.linearSaleStdPluItem.setBackgroundResource(R.drawable.shape_std_sale_item_normal);
        }

        holder.tvSaleStdPluName.setText(pluModel.getPluName());
        //holder.tvSaleStdPluCount.setText(String.valueOf(pluModel.count()));
        holder.tvSaleStdPluCount.setText(pluModel.getQuantity());
        holder.tvSaleStdPluPrice.setText(pluModel.getTagPrice());
        holder.tvSaleStdPluTotalRecv.setText(pluModel.getTotalRecv());

        if (pluModel.isHasTotalDsc()) {
            holder.tvSaleStdPluTotal.setVisibility(View.VISIBLE);
            holder.tvSaleStdPluTotal.setText(pluModel.getTotal());
            holder.tvSaleStdPluTotal.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG);
        } else {
            holder.tvSaleStdPluTotal.setVisibility(View.GONE);
            holder.tvSaleStdPluTotal.setText(pluModel.getTotal());
        }

        holder.itemView.setTag(position);
        holder.btnSaleStdPluDel.setTag(position);
        holder.btnSaleStdPluDecrease.setTag(position);
        holder.btnSaleStdPluIncrease.setTag(position);
        holder.tvSaleStdPluCount.setTag(position);

//        if (pluModel.isWeightOrEquivalent()) {
//            holder.btnSaleStdPluDecrease.setEnabled(false);
//            holder.btnSaleStdPluIncrease.setEnabled(false);
//            //updateWeightTotal(holder, position);
//        }
        updateUiByPreKey(holder);
    }

    private void updateWeightTotal(SaleItemStdAdapter.ViewHolder holder, int position) {
        BaseSalePluModel pluModel = this.pluModels.get(position);

        holder.tvSaleStdPluPrice.setText(pluModel.getTotal());
    }

    private void updateUiByPreKey(SaleItemStdAdapter.ViewHolder holder) {
        if (isShowPreKey) {
            holder.linearSaleStdPluCount.getLayoutParams().width = 215;
            holder.linearSaleStdPluPrice.getLayoutParams().width = 63;
            holder.linearSaleStdPluTotal.getLayoutParams().width = 63;
        } else {
            holder.linearSaleStdPluCount.getLayoutParams().width = 355;
            holder.linearSaleStdPluPrice.getLayoutParams().width = 63;
            holder.linearSaleStdPluTotal.getLayoutParams().width = 290;
        }
    }

    @Override
    public void onClick(View v) {
        int position = (int) v.getTag();
        if (mOnItemClickListener != null) {
            switch (v.getId()) {
                case R.id.linearSaleStdPluItem:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.ITEM, position);
                    break;
                case R.id.btnSaleStdPluDel:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.DELETE, position);
                    break;
                case R.id.btnSaleStdPluDecrease:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.DECREASE, position);
                    break;
                case R.id.btnSaleStdPluIncrease:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.INCREASE, position);
                    break;
                case R.id.tvSaleStdPluCount:
                    mOnItemClickListener.onClick(v, OnRecyclerItemClickListener.RecyclerItemViewName.CHANGE_QUALITY, position);
                    break;
                default:
                    break;
            }
        }
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private LinearLayout linearSaleStdPluItem;
        private RelativeLayout btnSaleStdPluDel;
        private TextView tvSaleStdPluName;
        private Button btnSaleStdPluDecrease;
        private TextView tvSaleStdPluCount;
        private Button btnSaleStdPluIncrease;
        private TextView tvSaleStdPluPrice;
        private TextView tvSaleStdPluTotalRecv;
        private TextView tvSaleStdPluTotal;

        private LinearLayout linearSaleStdPluCount;
        private LinearLayout linearSaleStdPluPrice;
        private LinearLayout linearSaleStdPluTotal;

        public ViewHolder(View itemView) {
            super(itemView);
            linearSaleStdPluItem = itemView.findViewById(R.id.linearSaleStdPluItem);
            btnSaleStdPluDel = itemView.findViewById(R.id.btnSaleStdPluDel);
            tvSaleStdPluName = itemView.findViewById(R.id.tvSaleStdPluName);
            btnSaleStdPluDecrease = itemView.findViewById(R.id.btnSaleStdPluDecrease);
            tvSaleStdPluCount = itemView.findViewById(R.id.tvSaleStdPluCount);
            btnSaleStdPluIncrease = itemView.findViewById(R.id.btnSaleStdPluIncrease);
            tvSaleStdPluPrice = itemView.findViewById(R.id.tvSaleStdPluPrice);
            tvSaleStdPluTotalRecv = itemView.findViewById(R.id.tvSaleStdPluTotalRecv);
            tvSaleStdPluTotal = itemView.findViewById(R.id.tvSaleStdPluTotal);

            linearSaleStdPluCount = itemView.findViewById(R.id.linearSaleStdPluCount);
            linearSaleStdPluPrice = itemView.findViewById(R.id.linearSaleStdPluPrice);
            linearSaleStdPluTotal = itemView.findViewById(R.id.linearSaleStdPluTotal);

            linearSaleStdPluItem.setOnClickListener(SaleItemStdAdapter.this);
            btnSaleStdPluDel.setOnClickListener(SaleItemStdAdapter.this);
            btnSaleStdPluDecrease.setOnClickListener(SaleItemStdAdapter.this);
            btnSaleStdPluIncrease.setOnClickListener(SaleItemStdAdapter.this);
            tvSaleStdPluCount.setOnClickListener(SaleItemStdAdapter.this);
        }
    }
}
