/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.adapter;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;

import com.hisense.common.util.ListUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.ysy.sale.SaleActivity;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.layout.SoftInput;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePluDscModel;

import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Locale;

/**
 * 销售商品列表适配器。
 *
 * @author caokai      2017-01-12
 * @author caokai      2017-01-18
 * @author caokai      2017-04-14
 * @author zhangsai    2018-01-08
 * @author zhangsai    2018-01-10 HIPOSAPP-270 显示商品优惠类型和优惠前合计金额
 * @author zhangsai    2018-02-08 HIPOSAPP-321 支持称重商品
 * @author zhangsai    2018-02-08 HIPOSAPP-335 购买数量为小数时按称重商品处理
 * @author zhangsai    2018-12-19 HIPOSAPP-698 交易商品类型改为 ISaleItem
 * @author wangyaogang 2019-02-12 HIPOSAPP-994 衢州东方-商品数量支持手工录入
 * @author wangyaogang 2019-02-19 HIPOSAPP-1023 修复录入商品数量过大时系统崩溃的问题
 * @author guolina     2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 */
public class SaleItemYsyAdapter extends BaseAdapter {

    public final static int SALE_ITEM_QUANTITY_INCREASED = 0x1012;//增加商品数量
    public final static int SALE_ITEM_QUANTITY_DECREASED = 0x1013;//减少商品数量
    public final static int SALE_ITEM_QUANTITY_MODIFED = 0x1014;//修改商品数量

    private List<BaseSalePluModel> saleItems;   // 商品列表
    private Context context;
    private Handler callerHandler;
    private Boolean isDscbyHand;

    /**
     * 初始化一个 {@code SaleItemsAdapter} 的实例。
     *
     * @param context   Context
     * @param saleItems 商品列表
     * @param handler   Handler
     */
    public SaleItemYsyAdapter(Context context, List<BaseSalePluModel> saleItems, Handler handler) {
        this.context = context;
        this.saleItems = saleItems;
        this.callerHandler = handler;
    }

    public void setData(List<BaseSalePluModel> pluModels) {
        if (!ListUtils.isEmpty(pluModels)) {
            saleItems.clear();
            saleItems.addAll(pluModels);
            notifyDataSetChanged();
        }
    }

    @Override
    public int getCount() {
        return this.saleItems.size();
    }

    @Override
    public BaseSalePluModel getItem(int position) {
        return this.saleItems.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        final BaseSalePluModel item = getItem(position);

        final ViewHolder holder;
        if (convertView == null) {
            LayoutInflater li = (LayoutInflater) this.context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = li.inflate(R.layout.activity_sale_items_adapter, parent, false);

            holder = createViewHolder(convertView);
        } else {
            holder = (ViewHolder) convertView.getTag();
        }

        holder.txtPluName.setText(item.getPluName());
        holder.txtPluCode.setText(item.getPluCode());

        if (GeneralSettings.INSTANCE.isNoShopLimits() && !StringUtils.isEmpty(item.getShopName())) {
            holder.txtShopName.setText(String.format("(%s)", item.getShopName()));
        }

        if (item.isWeightOrEquivalent()) {
            // 称重商品，计重
            holder.txtPluWeight.setText(String.format(Locale.CHINESE, "%.3f %s", Double.parseDouble(item.getQuantity()), item.getUnit()));
            holder.lytPluSpec.setVisibility(View.GONE);
            holder.lytPluUnit.setVisibility(View.GONE);
            holder.lytPluWeight.setVisibility(View.VISIBLE);
            holder.btnIncrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_right));
            holder.btnIncrease.setTextColor(Color.parseColor("#999999"));
        } else if (item.isWeight() && item.isCountQuantity()) {
            // 称重商品，计件
            this.setPluSpec(holder, item.getSpec());
            this.setPluUnit(holder, item.getUnit());
            holder.lytPluWeight.setVisibility(View.GONE);
            holder.btnIncrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_right));
            holder.btnIncrease.setTextColor(Color.parseColor("#999999"));
        } else{
            // 非称重商品，录入的整数件数
            this.setPluSpec(holder, item.getSpec());
            this.setPluUnit(holder, item.getUnit());
            holder.lytPluWeight.setVisibility(View.GONE);
            holder.btnIncrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_right_enable));
            holder.btnIncrease.setTextColor(Color.parseColor("#FFFFFF"));
        }

        holder.imgDscNormal.setVisibility(item.isNormalDsc() ? View.VISIBLE : View.GONE);
        holder.imgDscVip.setVisibility(item.isVipDsc() ? View.VISIBLE : View.GONE);

        updateDecreaseButtonState(holder, item);
        if(isDscbyHand(item)) {
            holder.btnIncrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_right));
            holder.btnIncrease.setTextColor(Color.parseColor("#999999"));
            holder.btnDecrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left));
            holder.btnDecrease.setTextColor(Color.parseColor("#999999"));
        }
        updateTotal(holder, item);
        holder.btnDecrease.setOnClickListener((arg0) -> {
            // 称重或等价于称重（录入的小数）的商品不允许修改数量
            if (item.isLimitDecreaseQuantity()) {
                return;
            }
            if(isDscbyHand(item)){
                Toast.makeText(context, "手工优惠商品不允许改数量", Toast.LENGTH_SHORT).show();
                return;
            }
            ((SaleActivity) context).getSale().decreasePlu(position);
            updateDecreaseButtonState(holder, item);
            updateTotal(holder, item);

            onQuantityChanged(position, SALE_ITEM_QUANTITY_DECREASED);
        });

        holder.btnIncrease.setOnClickListener((arg0) -> {
            // 称重或等价于称重（录入的小数）的商品不允许修改数量
            if (item.isWeightOrEquivalent()) {
                return;
            }
            if(isDscbyHand(item)){
                Toast.makeText(context, "手工优惠商品不允许改数量", Toast.LENGTH_SHORT).show();
                return;
            }
            ((SaleActivity) context).getSale().increasePlu(position);
            updateDecreaseButtonState(holder, item);
            updateTotal(holder, item);

            onQuantityChanged(position, SALE_ITEM_QUANTITY_INCREASED);
        });

        holder.txtCount.setOnClickListener((v) -> {
            // 称重或等价于称重（录入的小数）的商品不允许修改数量
            if (item.isWeightOrEquivalent()) {
                return;
            }
            if(isDscbyHand(item)){
                Toast.makeText(context, "手工优惠商品不允许改数量", Toast.LENGTH_SHORT).show();
                return;
            }
            showDialog(holder, item, position);
        });
        return convertView;
    }

    /**
     * 手工优惠不允许改数量
     *
     * @param item 点击item
     */
    private Boolean isDscbyHand(BaseSalePluModel item) {
        isDscbyHand = false;
        for(SalePluDscModel salePluDscModel: item.getPluDscs()){
            if(salePluDscModel.getDscCode().equals("SGYH000000")) {
                isDscbyHand = true;
            }
        }
        if (isDscbyHand) {

            return true;
        }
        return false;
    }
    /**
     * 展示 修改数量对话框
     *
     * @param holder   ViewHolder复用控件容器
     * @param item     SaleItem 修改商品
     * @param position 点击位置
     */
    private void showDialog(final ViewHolder holder, final BaseSalePluModel item, final int position) {
        View view = LayoutInflater.from(context).inflate(R.layout.dialog_modify_num, null, false);
        final AlertDialog dialog = new AlertDialog.Builder(context).setView(view).create();
        dialog.setCanceledOnTouchOutside(false);
        //dialog 出现时 弹出输入法
        dialog.setOnShowListener((dig) -> {
            SoftInput.openSoftKeyBoard((Activity) context);
        });

        final EditText editCount = view.findViewById(R.id.editCount);
        final Button btnDecrease = view.findViewById(R.id.btnDecrease);
        final Button btnIncrease = view.findViewById(R.id.btnIncrease);
        Button btnCancel = view.findViewById(R.id.btnCancel);
        final Button btnOK = view.findViewById(R.id.btnOK);

        //商品数量
        editCount.setText(holder.txtCount.getText());
        editCount.setSelection(holder.txtCount.getText().length());
        updateDecreaseButtonState2(btnDecrease, Integer.parseInt(editCount.getText().toString().trim()));
        editCount.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                if (s.length() > 0) {
                    btnDecrease.setEnabled(true);
                    btnIncrease.setEnabled(true);
                    btnOK.setEnabled(true);
                    updateDecreaseButtonState2(btnDecrease, Integer.parseInt(s.toString().trim()));
                } else {
                    btnDecrease.setEnabled(false);
                    btnIncrease.setEnabled(false);
                    btnOK.setEnabled(false);
                    CommonTools.toastLong("请输入商品数量");
                }
            }
        });

        //减少
        btnDecrease.setOnClickListener((v) -> {
            int curNum = Integer.parseInt(editCount.getText().toString().trim());
            if (curNum <= 1) {
                return;
            }
            int num = curNum - 1;
            editCount.setText(String.valueOf(num));
            editCount.setSelection(editCount.getText().length());
            updateDecreaseButtonState2(btnDecrease, num);
        });

        //增加
        btnIncrease.setOnClickListener((v) -> {
            int curNum = Integer.parseInt(editCount.getText().toString().trim());
            int num = curNum + 1;
            if (curNum == 999999999) {
                num = curNum;
            }
            editCount.setText(String.valueOf(num));
            editCount.setSelection(editCount.getText().length());
        });
        //确定
        btnOK.setOnClickListener((v) -> {
            int quantity = Integer.parseInt(editCount.getText().toString().trim());
            if (quantity == 0) {
                CommonTools.toastShort("数量不能为0");
                return;
            }

            ((SaleActivity) context).getSale().changeQuantity(position, quantity);
            updateDecreaseButtonState(holder, item);
            updateTotal(holder, item);

            onQuantityChanged(position, SALE_ITEM_QUANTITY_MODIFED);
            SoftInput.closeSoftKeyBoard2((Activity) context, editCount);
            dialog.dismiss();
        });
        //取消
        btnCancel.setOnClickListener((v) -> {
            SoftInput.closeSoftKeyBoard2((Activity) context, editCount);
            dialog.dismiss();
        });
        dialog.show();
    }

    /**
     * 更新 修改商品数量弹框中的减少按钮状态
     *
     * @param btnDecrease 减少按钮控件
     * @param count       当前商品数量
     */
    private void updateDecreaseButtonState2(Button btnDecrease, int count) {
        if (count > 1) {
            btnDecrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left_enable));
            btnDecrease.setTextColor(Color.parseColor("#ffffff"));
        } else {
            btnDecrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left));
            btnDecrease.setTextColor(Color.parseColor("#999999"));
        }
    }

    /**
     * 设置商品规格。
     *
     * @param holder 商品视图的 Holder
     * @param value  商品规格
     */
    private void setPluSpec(ViewHolder holder, String value) {
        holder.txtPluSpec.setText(value);
        holder.lytPluSpec.setVisibility(StringUtils.isWhitespace(value) ? View.GONE : View.VISIBLE);
    }

    /**
     * 设置商品计量单位。
     *
     * @param holder 商品视图的 Holder
     * @param value  商品计量单位
     */
    private void setPluUnit(ViewHolder holder, String value) {
        holder.txtPluUnit.setText(value);
        holder.lytPluUnit.setVisibility(StringUtils.isWhitespace(value) ? View.GONE : View.VISIBLE);
    }

    /**
     * 创建一个新的视图 Holder，并将其缓存在 {@code convertView} 的 Tag 中。
     *
     * @param convertView 视图
     * @return 新创建的视图 Holder
     */
    private ViewHolder createViewHolder(View convertView) {
        ViewHolder holder = new ViewHolder();

        holder.txtPluName = convertView.findViewById(R.id.txtPluName);
        holder.txtPluCode = convertView.findViewById(R.id.txtPluCode);
        holder.txtShopName = convertView.findViewById(R.id.txtShopName);

        holder.imgDscNormal = convertView.findViewById(R.id.imgDscNormal);
        holder.imgDscVip = convertView.findViewById(R.id.imgDscVip);

        holder.txtPluSpec = convertView.findViewById(R.id.txtPluSpec);
        holder.txtPluUnit = convertView.findViewById(R.id.txtPluUnit);
        holder.txtPluWeight = convertView.findViewById(R.id.txtPluWeight);
        holder.lytPluSpec = convertView.findViewById(R.id.lytPluSpec);
        holder.lytPluUnit = convertView.findViewById(R.id.lytPluUnit);
        holder.lytPluWeight = convertView.findViewById(R.id.lytPluWeight);

        holder.txtCount = convertView.findViewById(R.id.txtCount);
        holder.txtTotalDue = convertView.findViewById(R.id.txtTotalDue);
        holder.txtTotal = convertView.findViewById(R.id.txtTotal);
        holder.txtTotal.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG);

        holder.btnDecrease = convertView.findViewById(R.id.btnDecrease);
        holder.btnIncrease = convertView.findViewById(R.id.btnIncrease);

        convertView.setTag(holder);

        return holder;
    }

    /**
     * <p>更新 {@code item} 指定商品的减少购买数量按钮的状态。</p>
     * <p>购买数量大于 1 时，可以减少购买数量。购买数量为 1 时，不允许继续减少购买数量。</p>
     *
     * @param holder 商品视图的 Holder
     * @param item   更新的商品
     */
    private void updateDecreaseButtonState(ViewHolder holder, BaseSalePluModel item) {
        if (!item.isWeightOrEquivalent() && item.count() > 1) {
            holder.btnDecrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left_enable));
            holder.btnDecrease.setTextColor(Color.parseColor("#ffffff"));
        } else {
            holder.btnDecrease.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left));
            holder.btnDecrease.setTextColor(Color.parseColor("#999999"));
        }
    }

    /**
     * 更新 {@code item} 指定商品的数量、合计。
     *
     * @param holder 商品视图的 Holder
     * @param item   更新的商品
     */
    private void updateTotal(ViewHolder holder, BaseSalePluModel item) {
        // 只有当存在优惠时，才显示(优惠前)合计
        if (item.isHasTotalDsc()) {
            holder.txtTotal.setText(item.getTotal());
        } else {
            holder.txtTotal.setText("");
        }

        holder.txtTotalDue.setText(item.getTotalRecv());
        holder.txtCount.setText(String.format(Locale.CHINESE, "%d", item.count()));
    }

    /**
     * 更改购买数量的事件。
     *
     * @param position    商品在列表中的位置
     * @param messageCode 增加/减少购买数量的事件标识
     */
    private void onQuantityChanged(int position, int messageCode) {
        Bundle bundle = new Bundle();
        bundle.putInt("ReturnValue", position);
        Message msgBack = callerHandler.obtainMessage();
        msgBack.obj = position;
        msgBack.what = messageCode;
        msgBack.setData(bundle);
        callerHandler.sendMessage(msgBack);
    }

    /**
     * 视图 Holder。
     */
    private static class ViewHolder {
        TextView txtPluName;        // 商品名称
        TextView txtPluCode;        // 商品编码
        TextView txtShopName;       // 专柜名称

        TextView txtPluUnit;        // 单位
        LinearLayout lytPluUnit;    // 单位布局
        TextView txtPluSpec;        // 规格
        LinearLayout lytPluSpec;    // 规格布局
        TextView txtPluWeight;      // 重量
        LinearLayout lytPluWeight;  // 重量布局

        TextView txtTotal;          // 应收合计(优惠前)
        TextView txtTotalDue;       // 实收合计(优惠后)
        TextView txtCount;          // 购买件数

        ImageView imgDscNormal;     // 普通商品优惠
        ImageView imgDscVip;        // 会员优惠

        Button btnDecrease;         // 减少购买数量
        Button btnIncrease;         // 增加购买数量
    }
}
