/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.ReturnItemViewModel;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hipos.model.SalePluDscModel;

import java.util.List;

/**
 * 退货商品项适配器。
 *
 * @author xieliyan    2017-05-10
 * @author zhangsai    2017-11-15 HIPOSAPP-212
 * @author xieliyan    2018-01-22 HIPOSAPP-290 修复优惠商品退货时程序异常退出的问题
 * @author zhangsai    2018-03-07 HIPOSAPP-345 修复清除退款金额闪退问题
 * @author zhangsai    2018-12-21 HIPOSAPP-699 优化退货逻辑，简化事件处理参数
 * @author zhangsai    2018-12-26 HIPOSAPP-939 UI 商品普通/会员优惠图标
 * @author wangyaogang 2019-06-04 HIPOSAPP-1120 修复【部分退货，如果把商品数量减成0，但是商品处于选择状态，合计退款后，系统会崩溃】的问题
 * @author guolina     2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 */
public class SaleReturnItemsAdapter extends BaseAdapter {
    public final static int SALE_ITEM_QUANTITY_INCREASED = 0x1012;//增加商品数量
    public final static int SALE_ITEM_QUANTITY_DECREASED = 0x1013;//减少商品数量
    public final static int CHECKBOX_SINGLE_ITEM_COUNT = 0x1014;//选择框
    public final static int EDIT_SINGLE_ITEM_TOTAL = 0x1017;//修改实收退款金额

    private List<ReturnItemViewModel> salesItems;
    private Handler callerHandler;
    private LayoutInflater mInflater = null;
    private Context context;
    private Boolean isDscbyHand;
    private boolean isScreenModel = false; // 是否大屏模式

    public SaleReturnItemsAdapter(Context context, List<ReturnItemViewModel> data, Handler handler) {
        mInflater = LayoutInflater.from(context);
        this.context = context;
        this.salesItems = data;
        this.callerHandler = handler;
    }

    public SaleReturnItemsAdapter(Context context, boolean isScreenModel, List<ReturnItemViewModel> data, Handler handler) {
        mInflater = LayoutInflater.from(context);
        this.context = context;
        this.salesItems = data;
        this.callerHandler = handler;
        this.isScreenModel = isScreenModel;
    }

    @Override
    public int getCount() {
        return salesItems.size();
    }

    @Override
    public ReturnItemViewModel getItem(int position) {
        return salesItems.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        final ViewHolder viewHolder;

        final ReturnItemViewModel item = getItem(position);
        if (convertView == null) {
            if (isScreenModel) {
                convertView = mInflater.inflate(R.layout.activity_sale_return_items_wx_adapter, parent, false);
            } else {
                convertView = mInflater.inflate(R.layout.activity_sale_return_items_adapter, parent, false);
            }
            viewHolder = new ViewHolder();
            viewHolder.txtPluName = (TextView) convertView.findViewById(R.id.txtPluName);
            viewHolder.txtPluUnit = (TextView) convertView.findViewById(R.id.txtPluUnit);
            viewHolder.txtPluSpec = (TextView) convertView.findViewById(R.id.txtPluSpec);
            viewHolder.txtTotalPaid = (TextView) convertView.findViewById(R.id.txtTotalPaid);

            viewHolder.imgDscNormal = (ImageView) convertView.findViewById(R.id.imgDscNormal);
            viewHolder.imgDscVip = (ImageView) convertView.findViewById(R.id.imgDscVip);

            viewHolder.btnReturnLess = (Button) convertView.findViewById(R.id.btnReturnLess);
            viewHolder.btnReturnMore = (Button) convertView.findViewById(R.id.btnReturnMore);
            viewHolder.edtTotalRefund = (EditText) convertView.findViewById(R.id.edtTotalRefund);
            viewHolder.txtCount = (TextView) convertView.findViewById(R.id.txtCount);
            viewHolder.lytCheck = (LinearLayout) convertView.findViewById(R.id.lytCheck);
            viewHolder.lytPluItem=convertView.findViewById(R.id.lytPluItem);
            viewHolder.imgCheck = (ImageView) convertView.findViewById(R.id.imgCheck);
            viewHolder.imgCheck.setImageResource(R.drawable.ic_check_checked);
            //数量加法
            viewHolder.btnReturnMore.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    viewHolder.btnReturnLess.setEnabled(item.testReturnLess());
                    if (!item.testReturnMore()) {
                        return;
                    }
                    if(isDscbyHand(position)){
                        Toast.makeText(context, "手工优惠商品不允许改数量", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    item.returnMore();
                    viewHolder.btnReturnMore.setEnabled(item.testReturnMore());

                    Bundle bundle = new Bundle();
                    bundle.putInt("position", position);
                    Message msgBack = callerHandler.obtainMessage();
                    msgBack.what = SALE_ITEM_QUANTITY_INCREASED;
                    msgBack.setData(bundle);
                    callerHandler.sendMessage(msgBack);
                }
            });

            //数量减法
            viewHolder.btnReturnLess.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    viewHolder.btnReturnMore.setEnabled(item.testReturnMore());
                    if (!item.testReturnLess()) {
                        return;
                    }
                    if(isDscbyHand(position)){
                        Toast.makeText(context, "手工优惠商品不允许改数量", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    item.returnLess();
                    viewHolder.btnReturnLess.setEnabled(item.testReturnLess());

                    Bundle bundle = new Bundle();
                    bundle.putInt("position", position);
                    Message msgBack = callerHandler.obtainMessage();
                    msgBack.what = SALE_ITEM_QUANTITY_DECREASED;
                    msgBack.setData(bundle);
                    callerHandler.sendMessage(msgBack);


                    if (BigDecimalUtils.isEquals0(item.count())) {
                        if (item.isChecked()) {
                            item.unCheck();
                            viewHolder.btnReturnMore.setEnabled(item.test());
                            viewHolder.imgCheck.setImageResource(R.drawable.ic_check_normal);
                            Bundle bundle2 = new Bundle();
                            Message msgBack2 = callerHandler.obtainMessage();
                            msgBack.what = CHECKBOX_SINGLE_ITEM_COUNT;
                            msgBack.setData(bundle2);
                            callerHandler.sendMessage(msgBack2);
                        }
                    }

                }
            });

            //复选框
            viewHolder.lytCheck.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {

                    if (salesItems == null || salesItems.size() == 0) {
                        return;
                    }

                    if (item.isChecked()) {
                        item.unCheck();
                        viewHolder.imgCheck.setImageResource(R.drawable.ic_check_normal);
                    } else {
                        item.check();
                        viewHolder.imgCheck.setImageResource(R.drawable.ic_check_checked);
                    }

                    Bundle bundle = new Bundle();
//                    bundle.putInt("position", position);
//                    bundle.putBoolean("checked", item.isChecked());

                    Message msgBack = callerHandler.obtainMessage();

                    msgBack.what = CHECKBOX_SINGLE_ITEM_COUNT;
                    msgBack.setData(bundle);
                    callerHandler.sendMessage(msgBack);
                }

            });

            //修改实收退款金额时
            viewHolder.edtTotalRefund.setInputType(EditorInfo.TYPE_CLASS_PHONE);
            viewHolder.edtTotalRefund.setImeOptions(EditorInfo.IME_ACTION_DONE);
            viewHolder.edtTotalRefund.setOnEditorActionListener(new TextView.OnEditorActionListener() {

                @Override
                public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {

                    double returnTotal = Double.valueOf(viewHolder.edtTotalRefund.getText().toString());
                    if (returnTotal <= 0) {
                        CommonTools.toastShort("请输入正确的金额");
                    } else if (returnTotal > Double.valueOf(item.saleItem().getTotalRecv())) {
                        CommonTools.toastShort("实退金额过大");
                        viewHolder.edtTotalRefund.setText(item.totalRefund());
                    } else {
                        Bundle bundle = new Bundle();
                        bundle.putInt("position", position);
                        bundle.putDouble("returnTotal", returnTotal);
                        Message msgBack = callerHandler.obtainMessage();
                        msgBack.what = EDIT_SINGLE_ITEM_TOTAL;
                        msgBack.setData(bundle);
                        callerHandler.sendMessage(msgBack);
                    }

                    return false;
                }
            });

            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }

        viewHolder.txtPluName.setText(item.pluName());
        viewHolder.txtPluUnit.setText(item.unit());
        viewHolder.txtPluSpec.setText(item.spec());
        viewHolder.txtTotalPaid.setText(item.total());

        viewHolder.imgDscNormal.setVisibility(item.hasNormalDsc() ? View.VISIBLE : View.GONE);
        viewHolder.imgDscVip.setVisibility(item.hasVipDsc() ? View.VISIBLE : View.GONE);

        if (item.isChecked()) {
            viewHolder.imgCheck.setImageResource(R.drawable.ic_check_checked);
        } else {
            viewHolder.imgCheck.setImageResource(R.drawable.ic_check_normal);
        }

        viewHolder.edtTotalRefund.setText(item.totalRefund());
        viewHolder.txtCount.setText(String.valueOf(item.count()));

        viewHolder.btnReturnMore.setEnabled(item.testReturnMore());
        viewHolder.btnReturnLess.setEnabled(item.testReturnLess());

        if (item.testReturnLess() && !isDscbyHand(position)) {
            viewHolder.btnReturnLess.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left_enable));
            viewHolder.btnReturnLess.setTextColor(Color.parseColor("#ffffff"));
        } else {
            viewHolder.btnReturnLess.setBackground(context.getResources().getDrawable(R.drawable.border_radius_left));
            viewHolder.btnReturnLess.setTextColor(Color.parseColor("#999999"));
        }

        if (item.testReturnMore() && !isDscbyHand(position)) {
            viewHolder.btnReturnMore.setBackground(context.getResources().getDrawable(R.drawable.border_radius_right_enable));
            viewHolder.btnReturnMore.setTextColor(Color.parseColor("#ffffff"));
        } else {
            viewHolder.btnReturnMore.setBackground(context.getResources().getDrawable(R.drawable.border_radius_right));
            viewHolder.btnReturnMore.setTextColor(Color.parseColor("#999999"));
        }

        return convertView;
    }
    /**
     * 手工优惠不允许改数量
     *
     * @param position 点击位置
     */
    private Boolean isDscbyHand(int position) {
        isDscbyHand = false;
        for(SalePluDscModel salePluDscModel: getItem(position).saleItem().getPluDscs()){
            if(salePluDscModel.getDscCode().equals("SGYH000000")) {
                isDscbyHand = true;
            }
        }
        if (isDscbyHand) {
            return true;
        }
        return false;
    }
    private class ViewHolder {
        TextView txtTotalPaid;
        TextView txtPluName;
        TextView txtPluUnit;
        TextView txtPluSpec;
        TextView txtCount;
        TextView txtShopName;

        ImageView imgDscNormal;
        ImageView imgDscVip;

        Button btnReturnLess;
        Button btnReturnMore;
        EditText edtTotalRefund;
        ImageView imgCheck;
        LinearLayout lytCheck;
        LinearLayout lytPluItem;
    }

}
