/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.adapter;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hipos.coupon.CouponType;
import com.hisense.hipos.coupon.UsableCoupon;

import java.util.ArrayList;
import java.util.List;

/**
 * 会员券显示Adapter
 *
 * @author xieliyan    2017-12-01
 * @author xieliyan    2017-12-20 HIPOSAPP-248  修复取消用券再次点击时不可用的缺陷
 * @author xieliyan    2018-04-20 HIPOSAPP-409  会员券增加显示赠券名称（批量查询时）
 * @author wangyaogang 2019-07-08 HIPOSAPP-1187 折扣券显示折扣率等信息
 * @author guolina     2020-02-20 HIPOSAPP-1377 批量用券时一次性计算
 */
public class VipCouponListAdapter extends BaseAdapter {

    private LayoutInflater mInflater = null;
    private List<VipCoupon> mCouponData = new ArrayList<>();
    private List<Integer> mListSelected = new ArrayList<>();
    public List<UsableCoupon> mUsableCoupons = new ArrayList<>();
    private double mUseValue = 0.0;
    private double mAvailable = 0.0;

    /**
     * 初始化实例
     *
     * @param context       上下文
     * @param vipCouponList 券列表
     */
    public VipCouponListAdapter(Context context, List<VipCoupon> vipCouponList) {
        mInflater = LayoutInflater.from(context);
        this.mCouponData = vipCouponList;
    }

    /**
     * 初始化布局
     *
     * @param position    item的位置（从0开始）
     * @param convertView 加载的布局
     * @param parent      父布局
     */
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        convertView = mInflater.inflate(R.layout.adapter_vipcoupon_list, parent, false);

        TextView txtSymbol = convertView.findViewById(R.id.txtSymbol);
        TextView txtSymbolDiscount = convertView.findViewById(R.id.txtSymbolDiscount);
        TextView available = convertView.findViewById(R.id.available);
        TextView txtAvailable = convertView.findViewById(R.id.txtAvailable);
        TextView txtType = convertView.findViewById(R.id.txtType);
        TextView txtValue = convertView.findViewById(R.id.txtValue);
        TextView balance = convertView.findViewById(R.id.balance);
        TextView txtBalance = convertView.findViewById(R.id.txtBalance);
        TextView txtNo = convertView.findViewById(R.id.txtNo);
        TextView txtEndDate = convertView.findViewById(R.id.txtEndDate);
        Button chkBox = convertView.findViewById(R.id.chkBox);
        RelativeLayout lytName = convertView.findViewById(R.id.lytName);

        if (mCouponData.get(position).coupon().name() != null && !mCouponData.get(position).coupon().name().equals("")) {
            txtType.setText(mCouponData.get(position).coupon().name());
        } else {
            if (mCouponData.get(position).coupon().type().equals(CouponType.Cash)) {
                txtType.setText(R.string.vip_coupon_type_cash);
            } else {
                txtType.setText(R.string.vip_coupon_type_discount);
            }
        }

        if (mCouponData.get(position).coupon().type().equals(CouponType.Cash)) {
            //现金券 显示左上方￥标志,
            txtSymbol.setVisibility(View.VISIBLE);
            txtSymbolDiscount.setVisibility(View.GONE);
            //现金券显示面额
            txtValue.setText(BigDecimalUtils.transferBig(mCouponData.get(position).coupon().value()));
            //现金券显示可用金额
            txtAvailable.setText(String.valueOf(mCouponData.get(position).coupon().available()));
        } else {
            //折扣券切换背景颜色
            lytName.setBackgroundColor(Color.parseColor("#FF5D74"));
            //折扣券显示右上方折扣字样
            txtSymbol.setVisibility(View.INVISIBLE);
            txtSymbolDiscount.setVisibility(View.VISIBLE);
            //折扣券计算折扣并显示
            double discount = mCouponData.get(position).coupon().value() / 10;
            txtValue.setText(String.valueOf(discount));
            //折扣券不显示余额
            balance.setVisibility(View.INVISIBLE);
            txtBalance.setVisibility(View.INVISIBLE);
            //可用金额,选中前不显示, 选中后显示抵扣金额
            if (mCouponData.get(position).coupon().available() == mCouponData.get(position).coupon().value()) {
                available.setVisibility(View.INVISIBLE);
                txtAvailable.setVisibility(View.INVISIBLE);
            } else {
                txtAvailable.setText(String.valueOf(mCouponData.get(position).coupon().available()));
            }
        }

        if (mCouponData.get(position).coupon().balance() == 0.0) {
            txtBalance.setText(String.valueOf(mCouponData.get(position).coupon().available()));
        } else {
            txtBalance.setText(String.valueOf(mCouponData.get(position).coupon().balance()));
        }
        txtNo.setText(mCouponData.get(position).coupon().no());
        txtEndDate.setText(mCouponData.get(position).coupon().endDate());

        setCouponStatus(chkBox, mCouponData.get(position).isSelected(), mListSelected.indexOf(position) + 1);

        return convertView;
    }

    /**
     * 返回被选中的券
     *
     * @return mListSelected 被选中的券
     */
    public List<Integer> selectedItems() {
        return mListSelected;
    }

    /**
     * 设置券的属性
     *
     * @param status     选中状态
     * @param itemNumber 每个item的序号（从1开始）
     */
    private static void setCouponStatus(Button chkBox, boolean status, int itemNumber) {
        if (status) {
            chkBox.setText(String.valueOf(itemNumber));
            chkBox.setBackgroundResource(R.drawable.button_circle_red);
        } else {
            chkBox.setText("");
            chkBox.setBackgroundResource(R.drawable.button_circle);
        }
    }

    /**
     * 选中券的处理
     *
     * @param position     item的位置（从0开始）
     * @param usableCoupon 可用券
     */
    public void select(int position, UsableCoupon usableCoupon) {
        mListSelected.add(position);
        LoggerUtils.getLogger().debug("券位置" + position + "核销金额" + usableCoupon.useValue() + "可用金额" + usableCoupon.available());
        mUsableCoupons.add(usableCoupon);
        VipCoupon vipCoupon = mCouponData.get(position);
        vipCoupon.select();
        vipCoupon.coupon().available(usableCoupon.available());
        mUseValue=BigDecimalUtils.add(mUseValue,usableCoupon.useValue());
        mAvailable = mAvailable + usableCoupon.available();
        notifyDataSetChanged();
    }
    /**
     * 刷新选中券的处理
     *
     * @param position     item的位置（从0开始）
     * @param usableCoupon 可用券
     */
    public void selectUpdate(int position, UsableCoupon usableCoupon) {
        mListSelected.add(position);
        LoggerUtils.getLogger().debug("刷新券位置" + position + "核销金额" + usableCoupon.useValue() + "可用金额" + usableCoupon.available());
        mUsableCoupons.add(usableCoupon);
        VipCoupon vipCoupon = mCouponData.get(position);
        vipCoupon.select();
        vipCoupon.coupon().available(usableCoupon.available());
        mUseValue = mUseValue + usableCoupon.useValue();
        mAvailable = mAvailable + usableCoupon.available();
        notifyDataSetChanged();
    }
    /**
     * 取消特定位置的券
     *
     * @param position item的位置（从0开始）
     */
    public void unSelect(int position) {
        int index = mListSelected.indexOf(position);
        if (index == -1) {
            return;
        }
        for (int i = mListSelected.size() - 1; i >= index; i--) {
            VipCoupon vipCoupon = mCouponData.get(mListSelected.get(i));
            vipCoupon.unSelect();
            mUseValue=BigDecimalUtils.subtract(mUseValue,mUsableCoupons.get(i).useValue());
            mAvailable = mAvailable - mUsableCoupons.get(i).available();
            mListSelected.remove(i);
            mUsableCoupons.remove(i);
        }
        notifyDataSetChanged();
    }

    /**
     * 取消已选择的所有券
     */
    public void unSelectAll() {
        if (mListSelected.size() > 0) {
            unSelect(mListSelected.get(0));
        }
    }

    /**
     * 返回券是否已被全选
     *
     * @return true 券已被全选；false 券未被全选
     */
    public boolean isAllSelected() {
        return mListSelected.size() == mCouponData.size();
    }

    /**
     * 返回券使用金额
     */
    public double useValue() {
        return this.mUseValue;
    }
    /**
     * 初始化券使用金额
     */
    public void useValue(double useValue) {
        this.mUseValue = useValue;
    }
    /**
     * 返回券可用余额
     */
    public double available() {
        return this.mAvailable;
    }
    /**
     * 更新券
     */
    public void coupons(List<VipCoupon> vipCouponList) {
        this.mCouponData = vipCouponList;
        notifyDataSetChanged();

    }

    @Override
    public int getCount() {
        return mCouponData.size();
    }

    @Override
    public Object getItem(int position) {
        return mCouponData.get(position);
    }

    @Override
    public long getItemId(int position) {
        return 0;
    }
}
