package com.hisense.hiaposcmp.auth;

import android.text.TextUtils;

import java.util.List;
import java.util.Map;

import com.hisense.hiaposcmp.auth.bean.Feature;
import com.hisense.hiaposcmp.auth.bean.Item;
import com.hisense.hiaposcmp.auth.constant.ConfigConstants;
import com.hisense.hiaposcmp.auth.util.AppConfigUtil;
import com.hisense.hiaposcmp.auth.util.EncUtil;

/**
 * 主要配置
 * Created by zhuguangsheng on 2018/7/30.
 */

public class HiAppConfig {
    public static HiAppConfig mInstance;

    public static class ERR_CODE {
        public final static int ALL_OK = 0;
        public final static int CONFIG_FILE_NOT_FOUND = -1;
        public final static int CONFIG_FILE_FORMAT_ERR = -2;
        public final static int CONFIG_FILE_MD5_ERR = -3;
        public final static int UNKNOWN_ERR = -9;
    }

    private HiAppConfig() {

    }

    /**
     * singleton double check lock(DCL)
     *
     * @return
     */
    private static HiAppConfig getInstance() {
        if (mInstance == null) {
            synchronized (HiAppConfig.class) {
                if (mInstance == null) {
                    mInstance = new HiAppConfig();
                }
            }
        }
        return mInstance;
    }

    /**
     * 开始初始化
     *
     * @return 错误代码
     * @throws IllegalStateException
     */
    public static int initRead() {
        getInstance();

        AppConfigUtil.touchDir();

        //校验配置文件合法和完整性,如果不符合则返回错误
        int errCode = AppConfigUtil.readFile2Mem();
        return errCode;
    }

    /**
     * 设置在sdcard下的
     * 默认是hiappconfig目录
     */
    public static void setConfigRootDirInSdcard(String dirName) {
        ConfigConstants.setConfigRootDirInSdcard(dirName);
    }

    /**
     * 设置配置文件名称
     * 扩展名约定为cfg
     * 文件主名约定为产品名称 ＋下划线 + flavor定义的custom渠道名
     */
    public static void setConfigFileName(String fileName) {
        ConfigConstants.setConfigFileName(fileName);
    }

    /**
     * 返回list items
     *
     * @return
     */
    public static List<Item> getListItems() {
        return MemHolder.getListItems();
    }

    /**
     * 返回features
     *
     * @return
     */
    public static List<Feature> getListFeatures() {
        return MemHolder.getListFeatures();
    }

    /**
     * 读取值
     *
     * @param key
     * @param defaultValue
     */
    public static String getValue(String key, String defaultValue) {
        if (TextUtils.isEmpty(key)) {
            return "";
        }

        Item item = getItem(key);
        String encMethod = item.getEnc_method();
        if (!EncUtil.isEncMethodSupported(encMethod)) {
            //不加密
            encMethod = "000";
        }
        String value;
        value = EncUtil.dec(encMethod, item.getValue());

        if (value == null) {
            value = defaultValue;
        }
        return value;
    }

    /**
     * 读取值
     *
     * @param key
     * @param defaultValue
     */
    public static String getValueIfNotEmpty(String key, String defaultValue) {
        if (TextUtils.isEmpty(key)) {
            return "";
        }

        Item item = getItem(key);
        String encMethod = item.getEnc_method();
        if (!EncUtil.isEncMethodSupported(encMethod)) {
            //不加密
            encMethod = "000";
        }
        String value;
        value = EncUtil.dec(encMethod, item.getValue());

        if (value == null || value.isEmpty()) {
            value = defaultValue;
        }
        return value;
    }

    public static Item getItem(String key) {
        if (TextUtils.isEmpty(key)) {
            return null;
        }
        Map<String, Item> map = MemHolder.getMap();
        Item item = map.get(key);
        item = item == null ? new Item() : item;
        return item;
    }
}
