package com.hisense.hiaposcmp.auth.util;

import android.content.res.Resources;
import android.text.TextUtils;
import android.util.Log;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.hisense.hiaposcmp.BuildConfig;
import com.hisense.hiaposcmp.auth.HiAppConfig;
import com.hisense.hiaposcmp.auth.MemHolder;
import com.hisense.hiaposcmp.auth.bean.HiAppConfigBean;
import com.hisense.hiaposcmp.auth.bean.Item;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.hisense.hiaposcmp.auth.constant.ConfigConstants;

/**
 * 工具类
 * Created by zhuguangsheng on 2018/7/30.
 */

public class AppConfigUtil {
    private final static String TAG = AppConfigUtil.class.getSimpleName();
    public static void touchDir(){
        String dir;
        dir = ConfigConstants.getAbsConfigRootDir();
        Log.i(TAG, "dir="+dir);
        File file = new File(dir);
        if(file==null){
            Log.e(TAG, "cannot create dir 1");
            return;
        }

        boolean mkOk;
        if (!file.isDirectory()) {
            try {
                mkOk = file.mkdirs();
                if (!mkOk) {
                    mkOk = file.mkdirs();
                }
            }catch (Exception e){
                Log.e(TAG, "cannot create dir 2 ");
            }
        }
    }


    public static StringBuilder readConfigFile() throws Resources.NotFoundException{

        String absConfigFileName = ConfigConstants.getAbsConfigFileName();
        Log.i(TAG, "absConfigFileName=" + absConfigFileName);
        File file = new File(absConfigFileName);
        if(!file.exists()){
            Log.w(TAG, "config file not EXISTS!");
            return new StringBuilder();
        }

        StringBuilder sb;
        sb = FileUtil.readFileAbsolutePath(file.getAbsolutePath(), "UTF-8");
        if(sb==null){
            return new StringBuilder();
        }else{
            String str = sb.toString();
            if(str.isEmpty()){
                Log.w(TAG, "config file read empty");
            }
            return sb;
        }
    }

    //read file to mem
    public static int readFile2Mem(){
        boolean ret = false;
        //think if valid file
        StringBuilder readSb;
        try {
            readSb = readConfigFile();
        }catch (Exception e){
            return HiAppConfig.ERR_CODE.CONFIG_FILE_NOT_FOUND;
        }
        String readStr = readSb.toString();
        if(readStr.isEmpty()){
            Log.w(TAG, "app config file not valid");
            return HiAppConfig.ERR_CODE.CONFIG_FILE_FORMAT_ERR;
        }

        //parse json string to bean with fastjson(ali)
        HiAppConfigBean bean = JSON.parseObject(readStr, new TypeReference<HiAppConfigBean>(){});
        Log.i(TAG, "body=" + bean.getBody());
        String md5saltsum = bean.getBodySign().getMd5saltsum();
        if(TextUtils.isEmpty(md5saltsum)){
            Log.w(TAG, "app config file md5saltsum not defined");
            return HiAppConfig.ERR_CODE.CONFIG_FILE_MD5_ERR;
        }
        Log.w(TAG, "md5saltsum=" + bean.getBodySign().getMd5saltsum());

        //bodyjson sign check
        JSONObject json = JSONObject.parseObject(readStr, Feature.OrderedField);
        JSONObject bodyJson = json.getJSONObject("body");
        String jsonString = bodyJson.toJSONString();
        Log.i(TAG, "bodyJson toJSONString=" + jsonString);
        String calcMd5saltsum = Md5Util.getMD5String("ilovehisense"+jsonString);
        Log.i(TAG, "calcMd5saltsum=" + calcMd5saltsum);
        if(!md5saltsum.equals(calcMd5saltsum)){
            Log.w(TAG, "md5saltsum not match calcMd5saltsum!");
            if(!BuildConfig.DEBUG) {//临时增加,在开发时，为了加快编写速度，忽略md5错误
                return HiAppConfig.ERR_CODE.CONFIG_FILE_MD5_ERR;
            }
        }

        //保存desc
        if( bean.getBody().getDesc() == null){
            Log.w(TAG, "parse body desc empty");
            return HiAppConfig.ERR_CODE.CONFIG_FILE_FORMAT_ERR;
        }
        MemHolder.setConfigDesc(bean.getBody().getDesc());

        //保存items
        Map<String, Item> map = new HashMap<>();
        List<Item> list = bean.getBody().getItems();
        if(list==null || list.isEmpty()){
            Log.w(TAG, "parse body items empty");
            return HiAppConfig.ERR_CODE.CONFIG_FILE_FORMAT_ERR;
        }

        //保存到list
        MemHolder.setListItems(list);
        //保存到map
        for (Item item :
                list) {
            String name = item.getName();
            if(!TextUtils.isEmpty(name)) {
                map.put(name, item);
            }
        }
        MemHolder.setMap(map);

        //提取features
        //保存items
        //Map<String, Feature> mapFeature = new HashMap<>();
        List<com.hisense.hiaposcmp.auth.bean.Feature> listFeatures = bean.getBody().getFeatures();
        if(list==null || list.isEmpty()){
            Log.w(TAG, "parse body feature empty");
            return HiAppConfig.ERR_CODE.CONFIG_FILE_FORMAT_ERR;
        }
        //保存到list features
        MemHolder.setListFeatures(listFeatures);

        //返回OK
        return HiAppConfig.ERR_CODE.ALL_OK;
    }

    //
}
