package com.hisense.hiaposcmp.bean;

import android.os.Bundle;
import androidx.annotation.Nullable;

import com.hisense.hiaposcmp.constant.EventActions;

/**
 * 基本的EventBus的Bean
 * <p>
 * senderName: 发送者名称 ,通常可以用发送者的TAG来传入，便于调试跟踪发布者
 * actionName: enum类型的action名,参照EventActions类
 * msgDetail: 消息详情字符串
 * extraObject: 对象,可以定义复杂的数据.传入值可以为null
 * bundle: Android Bundle对象 传入值可以为null
 * <p>
 * Created by zhuguangsheng on 2018/5/9
 */
public class BaseEventBean {

    public void log() {
//		StringBuffer buffer = new StringBuffer();
//		buffer.append("Sender:"+senderName);
//		buffer.append(" Action:"+actionName);
//		buffer.append(" Detail:"+msgDetail);
//		HiLog.i("BaseEventBean", buffer.toString());
    }

    /**
     * 只传一个msgDetail (String)
     * 最常用的
     *
     * @param actionName
     * @param msgDetail
     */
    public BaseEventBean(EventActions actionName, String msgDetail) {
        BaseEventBean(this.senderName, actionName, msgDetail, null, null);
    }

    /**
     * 只传一个extraObject
     *
     * @param actionName
     * @param extraObject
     */
    public BaseEventBean(EventActions actionName, Object extraObject) {
        BaseEventBean(this.senderName, actionName, "", null, extraObject);
    }

    /**
     * 只传一个Bundle
     *
     * @param actionName
     * @param bundle
     */
    public BaseEventBean(EventActions actionName, @Nullable Bundle bundle) {
        BaseEventBean(senderName, actionName, "", bundle, null);
    }

    /**
     * 最全的
     *
     * @param senderName
     * @param actionName
     * @param msgDetail
     * @param extraObject
     */
    public void BaseEventBean(String senderName, EventActions actionName, String msgDetail, Bundle bundle, Object extraObject) {
        this.senderName = senderName;
        this.actionName = actionName;
        this.msgDetail = msgDetail;
        this.bundle = bundle;
        this.extraObject = extraObject;
    }

    public BaseEventBean(String senderName, EventActions actionName) {
        this.senderName = senderName;
        this.actionName = actionName;
    }

    public BaseEventBean(String senderName, EventActions actionName, Bundle bundle) {
        this.senderName = senderName;
        this.actionName = actionName;
        this.bundle = bundle;
    }

    public String getSenderName() {
        return senderName;
    }

    public EventActions getActionName() {
        return actionName;
    }

    public String getMsgDetail() {
        return msgDetail;
    }

    public Object getExtraObject() {
        return extraObject;
    }

    public Bundle getBundle() {
        return bundle;
    }

    private static final String DEF_SENDER_NAME = "DefaultSender";

    private String senderName = DEF_SENDER_NAME;
    private EventActions actionName = EventActions.DEFAULT;
    private String msgDetail = "";
    private Bundle bundle = null;
    private Object extraObject = null;

}
