package com.hisense.hiaposcmp.boot;

import android.app.Activity;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.activity.BaseActivity;

import java.lang.ref.WeakReference;
import java.util.Iterator;
import java.util.Stack;

/**
 * Created by Administrator on 2019-07-11.
 */
public class FinishActivityManager extends BaseActivity {

    private FinishActivityManager() {
    }

    private static FinishActivityManager sManager;
    private static Stack<WeakReference<Activity>> mActivityStack;

    public static FinishActivityManager getManager() {
        if (sManager == null) {
            synchronized (FinishActivityManager.class) {
                if (sManager == null) {
                    sManager = new FinishActivityManager();
                }
            }
        }
        return sManager;
    }

    /**
     * 添加Activity到栈
     *
     * @param activity
     */
    public static void addActivity(Activity activity) {
        try {
            if (mActivityStack == null) {
                mActivityStack = new Stack<>();
            }
            mActivityStack.add(new WeakReference<>(activity));
            LoggerUtils.getLogger().info("栈大" + mActivityStack.size());
        } catch (Exception e) {
            LoggerUtils.getLogger().error("添加Activity到栈异常" + e.toString());
        }

    }

    /**
     * 检查弱引用是否释放，若释放，则从栈中清理掉该元素
     */
    public static void checkWeakReference() {
        if (mActivityStack != null) {
            // 使用迭代器进行安全删除
            for (Iterator<WeakReference<Activity>> it = mActivityStack.iterator(); it.hasNext(); ) {
                WeakReference<Activity> activityReference = it.next();
                Activity temp = activityReference.get();
                if (temp == null) {
                    it.remove();
                }
            }
        }
    }

    /**
     * 获取当前Activity（栈中最后一个压入的）
     *
     * @return
     */
    public static Activity currentActivity() {
        checkWeakReference();
        if (mActivityStack != null && !mActivityStack.isEmpty()) {
            return mActivityStack.lastElement().get();
        }
        return null;
    }

    /**
     * 关闭当前Activity（栈中最后一个压入的）
     */
    public static void finishActivity() {
        Activity activity = currentActivity();
        if (activity != null) {
            finishActivity(activity);
        }
    }

    /**
     * 关闭指定的Activity
     *
     * @param activity
     */
    public static void finishActivity(Activity activity) {
        try {
            if (activity != null && mActivityStack != null) {
                // 使用迭代器进行安全删除
                for (Iterator<WeakReference<Activity>> it = mActivityStack.iterator(); it.hasNext(); ) {
                    WeakReference<Activity> activityReference = it.next();
                    Activity temp = activityReference.get();
                    // 清理掉已经释放的activity
                    if (temp == null) {
                        it.remove();
                        continue;
                    }

                    LoggerUtils.getLogger().info("被摧毁的activity" + activity.toString());
                    temp.finish();
                }
                if (mActivityStack.size() > 0) {
                    mActivityStack.clear();
                }
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("关闭指定的Activity异常" + e.toString());
        }

    }

    /**
     * 关闭指定类名的所有Activity
     *
     * @param cls
     */
    public static void finishActivity(Class<?> cls) {
        if (mActivityStack != null) {
            // 使用迭代器进行安全删除
            for (Iterator<WeakReference<Activity>> it = mActivityStack.iterator(); it.hasNext(); ) {
                WeakReference<Activity> activityReference = it.next();
                Activity activity = activityReference.get();
                // 清理掉已经释放的activity
                if (activity == null) {
                    it.remove();
                    continue;
                }
                if (activity.getClass().equals(cls)) {
                    it.remove();
                    activity.finish();
                }
            }
        }
    }

    /**
     * 结束所有Activity
     */
    public static void finishAllActivity() {
        if (mActivityStack != null) {
            for (WeakReference<Activity> activityReference : mActivityStack) {
                Activity activity = activityReference.get();
                if (activity != null) {
                    activity.finish();
                }
            }
            mActivityStack.clear();
        }
    }

    /**
     * 退出应用程序
     */
    public static void exitApp() {
        try {
            finishAllActivity();
            // 退出JVM,释放所占内存资源,0表示正常退出
            System.exit(0);
            // 从系统中kill掉应用程序
            android.os.Process.killProcess(android.os.Process.myPid());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}

