/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.cache;

import com.hisense.webapi.WebApi;

import java.util.Date;
import java.util.HashMap;

/**
 * 按日期更新的本地缓存。本地日期变更时自动清空缓存。
 *
 * @author zhangsai 2018-01-05
 */
public class DailyCache<T> {
    private HashMap<String, T> cache;
    private String today;

    /**
     * 初始化一个 {@code DailyCache} 的实例。
     */
    public DailyCache() {
        this.cache = new HashMap<>();
        this.clear();
    }


    /**
     * 添加一个缓存项。
     *
     * @param key   缓存项的键
     * @param value 缓存值
     */
    public void add(String key, T value) {
        String now = WebApi.formatDate(new Date());
        if (!now.equals(this.today)) {
            this.clear();
        }
        this.cache.put(key, value);
    }

    /**
     * 清空所有缓存。
     */
    public void clear() {
        this.cache.clear();

        // 重置当前日期
        this.today = WebApi.formatDate(new Date());
    }

    /**
     * 返回指定 {@code key} 键值的未失效缓存项。
     *
     * @param key 缓存项的键
     * @return <p>如果缓存过期，则重置缓存。无论缓存中是否存在指定 {@code key} 键值的缓存项，都返回 null。</p>
     * <p>如果存在指定 {@code key} 键值的缓存项，则返回此缓存项。否则，返回 null。</p>
     */
    public T get(String key) {
        String now = WebApi.formatDate(new Date());
        if (now.equals(this.today)) {
            if (this.cache.containsKey(key)) {
                return this.cache.get(key);
            }
        } else {
            this.clear();
        }

        return null;
    }

}
