/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.config;

import android.os.Build;
import android.os.Environment;
import android.text.TextUtils;

import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.Cashier;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.webapi.WebApi;

import org.apache.commons.lang3.StringUtils;

import java.io.File;

/**
 * 配置信息。
 *
 * @author fujiaxin         2017-03-14
 * @author dengyunpeng      2017-07-17  HIPOSAPP-117 删除包含vnd.android.package-archive代码
 * @author zhangsai         2017-10-20  HIPOSAPP-185 WebAPI 初始化
 * @author zhangsai         2017-10-25  HIPOSAPP-192
 * @author xieliyan         2017-11-22  HIPOSAPP-224 支持射频卡2.0：初始化营业员信息
 * @author zhangsai         2017-12-21  HIPOSAPP-250 删除结账日期
 * @author lizhixin         2018-05-11  HIPOSAPP-445 添加检测设备逻辑，是否是海信设备
 * @author dukunbiao        2018-07-13  暂存收银员密码
 * @author dukunbiao(null)  2019-07-22  重构
 * FIXME null 重构
 * @author guolina          2019-12-09  HIAPOSCMP-404 中间件配置中台错误地址时，前台登录报错信息优化
 * @author wangyaogang      2020-01-03  HIAPOSCMP-464 标准版打印功能开发,增加判断316设备的方法
 */
public class Config {

    public static final String APP_NAME = "海信易收银";

    /*制造商名称检测*/
    public static final String MANUFACTURE1 = "Hisense";
    public static final String MANUFACTURE2 = "HICS";

    /*登录设置*/
    public static final String IS_SETTING = "isSetting";                         //是否设置登录设置信息
    public static final String ORG_CODE = "orgCode";                             //组织编码KEY
    public static final String ORG_NAME = "orgName";                             //组织名称KEY
    public static final String SHOP_CODE = "shopCode";                           //专柜号KEY
    public static final String POS_NO = "posNo";                                 //设备编号KEY
    public static final String SERVER_URL = "serverUrl";                         //服务地址KEY
    public static final String SHOP_ID = "shopId";                                 //专柜ID

    public static final String OPERATOR_CODE = "operator_code";                         //是否设置登录设置信息
    public static final String CD_KEY = "cd_key";                             //组织编码KEY
    public static final String SMALL_BAG = "small_bag";                           //专柜号KEY
    public static final String BIG_BAG = "big_bag";                                 //设备编号KEY
    public static final String IS_VPAY = "is_vpay";                          //服务地址KEY
    public static final String PAY_IP = "pay_ip";                            //专柜ID
    public static final String PAY_PORT = "pay_port";
    public static final String PERMISSION_CODE = "permission_code";            //授权码
    public static final String FACE_LIMIT = "face_limit";                      //人脸支付限额
    public static final String UPDATE_URL = "update_url";                      //自动更新URL
    public static final String UPDATE_PORT = "update_port";                    //自动更新端口号
    public static final String IsUseVIP = "isUseVIP";                          //是否开启会员

    public static final String FACE_PID = "face_pid";                    //扫脸支付PID
    public static final String FACE_TOKEN = "face_token";                    //扫脸支付token

    /*工行AIDL版本*/
    public static final String VER_ICBC_AIDL = "V1.0.2";

    /*常用全局配置参数，该值在登录成功后进行赋值*/
    public static String base_url = "";                                          //服务地址
    public static String pos_no = "";                                            //设备编号
    public static String org_code = "";                                           //组织编号
    public static String org_name = "";                                           //组织名称
    public static String shop_code = "";                                         //专柜编码
    public static String shop_id = "";                                             //专柜ID

    public static String user_code = "";                                         //用户编码
    public static String user_id = "";                                           //用户id
    public static String user_name = "";                                         //用户名称
    public static String user_pwd = "";                                          //用户密码

    public static String operator_code = "";
    public static String cd_Key = "";
    public static String permission_code = "";            //授权码
    public static String small_bag = "";
    public static String big_bag = "";

    public static String is_vpay = "";
    public static String pay_ip = "";
    public static String pay_port = "";
    public static String face_limit = "";                   //人脸支付限额
    public static String face_pid = "";                   //人脸支付PID
    public static String face_token = "";                   //人脸支付token

    public static String log_path = "";

    public static String update_url = "";                     //自动更新url
    public static String update_port = "";                    //自动更新端口号

    public static String getBaseUrl() {
        return Config.getUrl(getPre(Config.SERVER_URL));
    }

    public static void init() {
        Config.base_url = Config.getUrl(getPre(Config.SERVER_URL));
        Config.pos_no = getPre(Config.POS_NO);
        Config.shop_code = getPre(Config.SHOP_CODE);
        Config.org_code = getPre(Config.ORG_CODE);
        Config.org_name = getPre(Config.ORG_NAME);
        Config.shop_id = getPre(Config.SHOP_ID);

        Config.user_code = getPre("user_code");
        Config.user_id = getPre("user_id");
        Config.user_name = getPre("user_name");
        Config.user_pwd = getPre("user_pwd");

        Config.operator_code = getPre(Config.OPERATOR_CODE);
        Config.cd_Key = getPre(Config.CD_KEY);
        Config.small_bag = getPre(Config.SMALL_BAG);
        Config.big_bag = getPre(Config.BIG_BAG);

        Config.is_vpay = getPre(Config.IS_VPAY);
        Config.pay_ip = getPre(Config.PAY_IP);
        Config.pay_port = getPre(Config.PAY_PORT);

        Config.permission_code = getPre(Config.PERMISSION_CODE);
        Config.face_limit = getPre(Config.FACE_LIMIT);
        Config.update_url = getPre(Config.UPDATE_URL);
        Config.update_port = getPre(Config.UPDATE_PORT);
        Config.face_pid = getPre(Config.FACE_PID);
        Config.face_token = getPre(Config.FACE_TOKEN);

        Config.log_path = Environment.getExternalStorageDirectory() + "/com.hisense/hiposapp" + File.separator;
        com.hisense.common.Context.config(Config.log_path);

        if (Config.base_url != null && Config.base_url.trim().length() > 0) {
            if (null != App.getContext()) {
                try {
                    WebApi.init(Config.getUrl(Config.base_url) + "/", GeneralSettings.INSTANCE.getAipUrl(), GeneralSettings.INSTANCE.getAipAppId(), GeneralSettings.INSTANCE.getAipKey());
                    if (CommonUtils.isWriteOk()) {
                        LoggerUtils.getLogger().debug("config  getAipAppId:" + GeneralSettings.INSTANCE.getAipAppId() + "  getAipKey:" + GeneralSettings.INSTANCE.getAipKey() + "   getAipUrl:" + GeneralSettings.INSTANCE.getAipUrl());
                    }
                } catch (RuntimeException e) {
                    if (CommonUtils.isWriteOk()) {
                        LoggerUtils.getLogger().debug("中台地址信息错误 WebApi.init e:" + e.getMessage());
                    }
                }


            } else {
                WebApi.init(Config.getUrl(Config.base_url) + "/");
                if (CommonUtils.isWriteOk()) {
                    LoggerUtils.getLogger().debug("App.getContext() = null");
                }
            }

            Context.config(Config.org_code, Config.org_name, Config.pos_no, Config.shop_code);
            Context.cashier(new User(Config.user_id, Config.user_code, Config.user_name));

            Cashier cashier = new Cashier(Config.user_code, Config.user_name);
            com.hisense.crm.webapi.Context.config(Config.org_code, Config.org_name, Config.pos_no, cashier);
        }
    }

    private static void savePre(String name, String value) {
        CommonTools.loginConfig(App.getInstance()).edit().putString(name, value).commit();
    }

    private static String getPre(String name) {
        return CommonTools.loginConfig(App.getInstance()).getString(name, "");
    }

    /**
     * 保存当前参数，防止app重启后空指针
     */
    public static void save() {
        savePre("user_code", user_code);
        savePre("user_id", user_id);
        savePre("user_name", user_name);
        savePre("user_pwd", user_pwd);
        savePre("shop_id", shop_id);
        savePre("operator_code", operator_code);
        savePre("cd_Key", cd_Key);
        savePre("small_bag", small_bag);
        savePre("big_bag", big_bag);
        savePre("is_vpay", is_vpay);
        savePre("pay_ip", pay_ip);
        savePre("pay_port", pay_port);

        savePre("permission_code", permission_code);
        savePre("face_limit", face_limit);
        savePre("update_url", update_url);
        savePre("update_port", update_port);
        savePre("face_pid", face_pid);
        savePre("face_token", face_token);

        savePre("logPath", log_path);

        savePre(Config.ORG_NAME, org_name);

        User user = new User(Config.user_id, Config.user_code, Config.user_name);
        Context.cashier(user);
    }

    /**
     * 返回是否已成功配置服务地址。
     *
     * @return 如果已配置服务地址，则返回 true。
     */
    public static boolean isConfiged() {
        return Config.getUrl(getPre(Config.SERVER_URL)) != null;
    }

    /**
     * 返回标准化的 WebAPI 服务地址。
     *
     * @param url 服务地址
     * @return 标准化后的服务地址
     */
    public static String getUrl(final String url) {
        if (url == null || url.trim().length() == 0) {
            return null;
        }

        String baseUrl = url.toLowerCase().trim();

        if (!baseUrl.startsWith("http://") &&
                !baseUrl.startsWith("https://")) {
            baseUrl = "http://" + baseUrl;
        }

        if (baseUrl.endsWith("/")) {
            baseUrl = baseUrl.substring(0, baseUrl.length() - 1);
        }

        return baseUrl;
    }

    public final static String PAPER_CURT_INTERVAL = "PAPER_CURT_INTERVAL";                  //打印间隔时间
    public final static String REFUND = "REFUND";                  //退货标识
    public final static String PRESALE = "PRESALE";                  //预销售标识
    public final static String PREREFUND = "PREREFUND";                  //预销售退货标识

    /**
     * @return 是否是海信的设备
     */
    public static boolean isHisenseDevice() {
        return TextUtils.equals(Build.MANUFACTURER, MANUFACTURE1)
                || TextUtils.equals(Build.MANUFACTURER, MANUFACTURE2)
                || TextUtils.equals(Build.PRODUCT, MANUFACTURE1)
                || TextUtils.equals(Build.PRODUCT, MANUFACTURE2)
                || TextUtils.equals(Build.MODEL, "HI98");
    }

    /**
     * 是否hk316设备
     *
     * @return
     */
    public static boolean isHk316() {
        return StringUtils.containsIgnoreCase(Build.MODEL, "HK316");
    }

    public static boolean isHt616() {
        return StringUtils.containsIgnoreCase(Build.MODEL, "HT616R");
    }

    public static boolean isHS330R() {
        return StringUtils.containsIgnoreCase(Build.MODEL, "HS330R") || StringUtils.containsIgnoreCase(Build.MODEL, "HK716");
    }

    public static boolean isHK716() {
        return StringUtils.containsIgnoreCase(Build.MODEL, "HK716");
    }
}
