package com.hisense.hiaposcmp.constant;

/**
 * 页面跳转相关常量配置
 *
 * @author dukunbiao(null)  2019-07-24
 */
public class ActivityConstants {

    /*******************************************************************************************
     * 基本规则为：
     * 1、使用16进制4位表示
     * 2、REQUEST CODE 最高位为0
     * 3、RESULT CODE 最高位为1
     * 4、同一请求或返回有多种情况，末尾增加第5位，表示序列号
     ******************************************************************************************/

    /**
     * 转到授权界面
     */
    public static final int REQUEST_REPRINT_TO_PERMISSION = 0x00011;
    public static final int REQUEST_PLU_DELETE_TO_PERMISSION = 0x00012;
    public static final int REQUEST_SALE_CANCEL_TO_PERMISSION = 0x00013;
    public static final int RESULT_PERMISSION_SUCCESS = 0x1001;

    /**
     * 商品扫描界面跳转到商品条码输入界面
     */
    public static final int REQUEST_PLU_SCAN_TO_INPUT = 0x0002;
    public static final int RESULT_PLU_SCAN_TO_INPUT = 0x1002;

    /**
     * 销售界面跳转到商品条码输入界面
     */
    public static final int REQUEST_PLU_SALE_TO_INPUT = 0x0003;

    /**
     * 销售界面跳转到大码商品录入界面
     */
    public static final int REQUEST_SALE_TO_SET_PLU_PRICE = 0x0004;
    public static final int RESULT_SALE_TO_SET_PLU_PRICE = 0x1004;

    /**
     * 销售界面跳转到会员录入界面
     */
    public static final int REQUEST_SALE_TO_VIP_LOGIN = 0x0005;
    public static final int RESULT_SALE_TO_VIP_LOGIN = 0x1005;

    /**
     * 销售跳转到支付界面
     */
    public static final int REQUEST_SALE_TO_PAY = 0x0006;
    public static final int RESULT_SALE_TO_PAY = 0x1006;

    /**
     * 销售跳转到加价换购界面
     */
    public static final int REQUEST_SALE_TO_GIFT = 0x0007;

    /**
     * 加价换购跳转到支付界面
     */
    public static final int REQUEST_GIFT_TO_PAY = 0x0008;
    public static final int RESULT_GIFT_TO_PAY = 0x1008;

    /**
     * 支付界面跳转到扫码引导界面
     */
    public final static int REQUEST_PAY_TO_SCAN = 0x0009;
    public static final int RESULT_PAY_TO_SCAN = 0x1009;

    /**
     * 支付界面跳转到支付完成界面
     */
    public final static int REQUEST_PAY_TO_SUCCESS = 0x0010;
    public final static int RESULT_PAY_TO_SUCCESS = 0x1010;

    /**
     * 跳转到卡支付界面
     */
    public static final int REQUEST_PAY_TO_CARD = 0x0011;
    public static final int RESULT_PAY_TO_CARD_SUCCESS = 0x10111;
    public static final int RESULT_PAY_TO_CARD_CANCEL = 0x10112;

    public static final int RESULT_REFUND_TO_CARD_SUCCESS = 0x0012;

    /**
     * 跳转到卡信息展示界面
     */
    public static final int REQUEST_TO_CARD_PAY_SHOW = 0x0013;
    public static final int RESULT_TO_CARD_PAY_SHOW = 0x0013;

    /**
     * 储值卡余额
     */
    public static final int REQUEST_CARD_BALANCE_QUERY = 0x0014;
    public static final int RESULT_CARD_BALANCE_QUERY_COMMIY = 0x10141;
    public static final int RESULT_CARD_BALANCE_QUERY_CANCEL = 0x10142;

    /**
     * 跳转到积分支付界面
     */
    public static final int REQUEST_PAY_TO_POINTS = 0x0015;
    public static final int RESULT_PAY_TO_POINTS_SUCCESS = 0x1015;

    public static final int RESULT_REFUND_TO_POINTS_SUCCESS = 0x1016;

    /**
     * 跳转到积分信息展示界面
     */
    public static final int REQUEST_TO_POINTS_PAY_SHOW = 0x0017;
    public static final int RESULT_TO_POINTS_PAY_SHOW = 0x0017;

    /**
     * 跳转到卡号校验码对话框界面
     */
    public static final int REQUEST_TO_CARD_SECURITY_INPUT = 0x0018;
    public static final int RESULT_TO_CARD_SECURITY_INPUT = 0x1018;

    /**
     * 跳转到券支付界面
     */
    public static final int REQUEST_PAY_TO_COUPON = 0x0019;
    public static final int RESULT_PAY_TO_COUPON_SUCCESS = 0x10191;
    public static final int RESULT_PAY_TO_COUPON_CANCEL = 0x10192;

    public static final int REQUEST_VIP_COUPON_CALCULATE = 0x0020;
    public static final int RESULT_VIP_COUPON_CALCULATE = 0x1020;

    public static final int REQUEST_COUPON_ADDITION = 0x0021;
    public static final int RESULT_COUPON_ADDITION = 0x1021;

    public static final int REQUEST_REFUND_TO_COUPON = 0x0022;
    public static final int RESULT_REFUND_COUPON_CONFIRM = 0x10221;
    public static final int RESULT_REFUND_COUPON_CANCEL = 0x10222;

    public static final int REQUEST_REFUND_TO_SCRM_COUPON = 0x0023;

    public static final int REQUEST_SCRM_COUPON_ADDITION = 0x0024;

    public static final int REQUEST_SCRM_COUPON_USE = 0x0025;

    /**
     * 其他支付方式
     */
    public static final int REQUEST_OTHER_PAYMETHOD = 0x0026;
    public static final int RESULT_OTHER_PAYMETHOD = 0x1026;

    public static final int REQUEST_OTHER_PAYMETHOD_REFUND = 0x0027;
    public static final int RESULT_OTHER_PAYMETHOD_REFUND = 0x1027;

    /**
     * 支付成功
     */
    public static final int REQUEST_PAY_FINISHED = 0x0028;

    public static final int REQUEST_REFUND_FINISHED = 0x0029;

    /**
     * 银联退款请求
     */
    public static final int REQUEST_UMS_REFUND_CONFIRM = 0x0030;
    public static final int RESULT_UMS_REFUND_COMMIT = 0x10301;
    public static final int RESULT_UMS_REFUND_CANCEL = 0x10302;

    /**
     * 选择支付方式请求
     */
    public static final int RESULT_SELECT_PAY_METHOD = 0x1031;

    /**
     * 米雅支扫码结果
     */
    public static final int RESULT_MIYA_PAY = 0x1099;
    /**
     * 赢商通扫码
     */
    public static final int RESULT_YST_PAY = 0x1098;
    /**
     * 银联钱包扫码，这个码不要动
     */
    public static final int SCANNIN_UPWALLET_CODE = 222;
    /**
     * 标准版扫码支付
     */
    public static final int REQUEST_INTE_PAY_INPUT = 0x0032;
    public static final int RESULT_NTE_PAY_SUCCESS = 0x10321;
    public static final int RESULT_NTE_PAY_CANCEL = 0x10322;
    /**
     * 标准版扫码支付
     */
    public static final int REQUEST_SALE_TO_CHANGE_QUALITY = 0x0034;
    public static final int RESULT_SALE_TO_CHANGE_QUALITY = 0x1034;
}
