/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.constant;

import android.content.Context;
import android.content.SharedPreferences;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.IcCardVersion;
import com.hisense.crm.vip.AuthMethod;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.device.DeviceType;
import com.hisense.hiaposcmp.constant.device.ScannerType;
import com.hisense.hiaposcmp.constant.print.ReceiptPrintType;
import com.hisense.hiaposcmp.service.payment.AggregatePaymentChannel;
import com.hisense.hiaposcmp.service.payment.PaymentChannel;
import com.hisense.hipos.sale.CurrencyPrecision;
import com.hisense.hipos.template.models.PrintTemplateType;

import java.util.Locale;

/**
 * 通用设置
 *
 * @author guanguangjin 2017-05-02
 * @author xieliyan     2017-08-30 HIPOSAPP-132 将getIns方法名修改为getInstance
 * @author xieliyan     2017-09-01 HIPOSAPP-133 是否启用发票打印
 * @author xieliyan     2017-09-25 HIPOSAPP-159
 * @author zhangsai     2017-10-24
 * @author xieliyan     2017-10-31 HIPOSAPP-201 使用枚举实现单例模式（适用于多线程
 * @author xieliyan     2018-01-03 HIPOSAPP-262 支持建行收单（选择第三方通道）
 * @author xieliyan     2018-01-04 HIPOSAPP-265 储值卡查询时增加密码
 * @author xieliyan     2018-04-12 HIPOSAPP-357 设置中增加"会员打印赠券"开关
 * @author guolina      2019-10-21 HIPOSAPP-1317 利客来未上传流水自动上传
 * @author wangyaogang  2019-11-27 HIAPOSCMP-284 海信支付打印签购单
 * @author xieliyan    2019-12-25 HIAPOSCMP-444 增加加密授权配置url和productkey存储
 * @author xieliyan     2020-01-02 HIAPOSCMP-466 商品查询异常时设置提示框倒计时（uniform系列）
 * @author guolina      2020-03-09 HIAPOSCMP-566 微信自助允许录入商品价格需求
 * @author guolina      2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 * @author guolina      2020-04-27 HIAPOSCMP-702 慧收银APP需支持第二件半价促销方案商品合行
 * @author liuyazeng    2019-09-25 HIPOSAPP-1417 会员手机号码登录、会员扫码登录可配置
 */
public enum GeneralSettings {

    INSTANCE;

    public static class Keys {
        public static final String AGGREGATE_PAYMENT_CHANNEL = "aggregatePayChannel";
        public static final String DEVICE_TYPE = "deviceType";
        public static final String IC_CARD_VERSION = "cardVersion";
        public static final String IS_ALLOW_TOTAL_DISCOUNT = "totalReductionCalculate";
        public static final String IS_PRE_SALE_ENABLED = "isPreSales";
        public static final String IS_PRINT_COUPON = "coupon";
        public static final String IS_PRINT_COUPON_ON_SALE_TO_VIP = "vipPrintCoupon";
        public static final String IS_PRINT_GIFT_COUPON = "giftCoupon";
        public static final String IS_PRINT_INVOICE = "InvoicePrint";
        public static final String IS_PRE_SALE_PRINT_INVOICE = "preSalesInvoicePrint";
        public static final String IS_PRINT_RECEIPT = "localVoucherPrinter";
        public static final String IS_REAL_TIME_POINTS_ENABLED = "isRealTimePoint";
        public static final String IS_SALE_CASHIER_ENABLED = "isCashier";
        public static final String IS_SALE_COUPON_RETURN_ENABLED = "returnCoupon";
        public static final String IS_SALE_DISABLED = "isSale";
        public static final String IS_SALE_NO_SHOP_LIMITS = "isAllPay";
        public static final String IS_SALE_REBATE_COUPON = "couponCalculate";
        public static final String IS_SALE_REBATE_GIFT_COUPON = "giftCouponCalculate";
        public static final String IS_SALE_RETURN_ENABLED = "isReturnOfGoods";
        public static final String IS_VIP_REGISTER_ENABLED = "isVipRegister";
        public static final String IS_VIP_POINTS_REGRESS_ENABLED = "isReturnOfPoint";
        public static final String LOG_FILE_EXPIRED_DAYS = "logFileExpiredDays";
        public static final String PAYMENT_CHANNEL = "paymentChannel";
        public static final String PREPAID_CARD_QUERY_BY_PASSWORD = "prepaidCard_queryInputPwd";
        public static final String PREPAID_CARD_QUERY_BY_CHECKING_FACE_NO = "prepaidCard_queryInputCheckBit";
        public static final String PREPAID_CARD_QUERY_BY_CHECKING_VERIFY_CODE = "prepaidCard_queryInputVerifyCode";
        public static final String PRINTER_TEXT_SIZE = "printerTextSize";
        public static final String RETURN_SALE_DAYS = "returnSaleDate";
        public static final String SALE_TRAN_TYPE = "tranType";
        public static final String SCANNER_TYPE = "scannerType";
        public static final String TIMEOUT = "timeout";
        public static final String RECEIPT_PRINT_BY_SHOP = "receiptPrintByShop";
        public static final String TOTAL_REDUCTION_WAY = "totalReductionWay";
        public static final String VIP_POINTS_QUERY_BY_PASSWORD = "pointsCard_queryInputPwd";
        public static final String VIP_POINTS_QUERY_BY_CHECKING_FACE_NO = "pointsCard_queryInputCheckBit";
        public static final String VIP_POINTS_QUERY_BY_CHECKING_VERIFY_CODE = "pointsCard_queryInputVerifyCode";
        public static final String TERMINAL_ID = "terminalId";
        public static final String CURRENCY_PRECISION = "currencyPrecision";
        public static final String AUTH_METHOD = "authMethod";
        public static final String E_PREPAID_CARD_QUERY_BY_PASSWORD = "ePrepaidCard_queryInputPwd";
        public static final String E_VIP_POINTS_QUERY_BY_PASSWORD = "ePointsCard_queryInputPwd";
        public static final String RETURN_CARD = "returnCard";
        public static final String HIDE_PAY_NO_WHEN_PRINT = "hidePayNoWhenPrint";
        public static final String PRINT_HXPAY_RECEIPT = "printHXPayReceipt";
        public static final String TIME_OF_UPLOAD = "timeOfUpload";
        public static final String IS_NO_CODE_PRICE = "isNoCodePrice";
        public static final String IS_USE_WX_FACE_PAY = "isUseWxFacePay";
    }

    private volatile SharedPreferences mSharedPreferences;
    private volatile SharedPreferences.Editor mEditor;

    public static double BAG_DEFAULT_NOT_CONFIG_PRICE = 99.00;

    /**
     * 定义静态类
     */
    GeneralSettings() {
        String preferencesName = String.format(Locale.CHINESE, "%s_preferences",
                App.getContext().getApplication().getPackageName());
        mSharedPreferences = App.getInstance().getSharedPreferences(preferencesName, Context.MODE_PRIVATE);
        mEditor = mSharedPreferences.edit();
    }


    /**********************************************************************
     * 促销设置
     **********************************************************************/

    // get 启用满赠券打印
    public boolean isPrintCoupon() {
        return mSharedPreferences.getBoolean(Keys.IS_PRINT_COUPON, false);
    }

    // get 启用礼品券打印
    public boolean isPrintGiftCoupon() {
        return mSharedPreferences.getBoolean(Keys.IS_PRINT_GIFT_COUPON, false);
    }

    // get 启用满赠券计算
    public boolean isCalcCoupon() {
        return mSharedPreferences.getBoolean(Keys.IS_SALE_REBATE_COUPON, false);
    }

    // get 启用满减券计算
    public boolean allowTotalDiscount() {
        return mSharedPreferences.getBoolean(Keys.IS_ALLOW_TOTAL_DISCOUNT, false);
    }

    // 满减优惠记录方式
    public String getTotalDiscountWay() {
        return mSharedPreferences.getString(Keys.TOTAL_REDUCTION_WAY, "支付方式");
    }

    /**
     * 获取设备类型
     *
     * @return deviceType
     */
    public DeviceType getDeviceType() {
        String device;
        try {
            device = App.getContext().getResources().getString(R.string.device_type);
        } catch (Exception e) {
            device = "LANDI";
        }

        String deviceType = mSharedPreferences.getString(Keys.DEVICE_TYPE, device);
        LoggerUtils.getLogger().info("设备类型:" + deviceType);
        return DeviceType.valueOf(deviceType);
    }

    /**
     * 获取扫描器类型。
     */
    public ScannerType getScannerType() {
        String scannerType;
        try {
            scannerType = mSharedPreferences.getString(Keys.SCANNER_TYPE,
                    App.getContext().getResources().getStringArray(R.array.scannerTypeValues)[0]);
        } catch (Exception e) {
            scannerType = "CAMERA";
        }
        return ScannerType.valueOf(scannerType);
    }

    /**
     * 允许非0商品改价
     *
     * @return true 是 ,false 否
     */
    public boolean isChangePrice() {
        return mSharedPreferences.getBoolean("isChangePrice", false);
    }

    /**
     * 允许0价格商品销售
     */
    public boolean isZeroPriceSale() {
        return mSharedPreferences.getBoolean("isZeroPriceSale", false);
    }

    /**
     * 最大优惠20%
     */
    public boolean isMaximumDiscount() {
        return mSharedPreferences.getBoolean("isMaximumDiscount", false);
    }

    /**
     * 隐藏扫码登录
     *
     * @return true 隐藏 ,false 显示
     */
    public boolean isScanVisibility() {
        return mSharedPreferences.getBoolean("isScanVisibility", false);
    }

    /**
     * 禁止手机号登录会员
     *
     * @return true 禁止 ,false 允许
     */
    public boolean isMobileLogin() {
        return mSharedPreferences.getBoolean("isMobileLogin", false);
    }

    /**
     * 允许商品合行/大码、称重、数量为小数的商品不支持合行
     *
     * @return true 是 ,false 否
     */
    public boolean isPluMerge() {
        return mSharedPreferences.getBoolean("isPluMerge", false);
    }

    /**
     * 获取设备终端号
     */
    public String getTerminalId() {
        return mSharedPreferences.getString(Keys.TERMINAL_ID, "");
    }

    /**********************************************************************
     * 打印设置
     **********************************************************************/

    /**
     * 返回是否发票打印
     *
     * @return true or false
     */
    public boolean isPrintInvoice() {
        return mSharedPreferences.getBoolean(Keys.IS_PRINT_INVOICE, false);
    }

    /**
     * 返回预销售是否发票打印
     *
     * @return true or false
     */
    public boolean isPreSalePrintInvoice() {
        return mSharedPreferences.getBoolean(Keys.IS_PRE_SALE_PRINT_INVOICE, false);
    }

    /**
     * 获取认证方式
     *
     * @return 认证方式
     */
    public AuthMethod getAuthMethod() {
        int authMethod = Integer.parseInt(mSharedPreferences.getString(Keys.AUTH_METHOD, "1"));
        return AuthMethod.valueOf(authMethod);
    }

    //流水自动上传时间间隔
    public int getTimeOfUpload() {
        String timeOfUpload = mSharedPreferences.getString(Keys.TIME_OF_UPLOAD, "0");
        return StringUtils.toInt(timeOfUpload, 0);
    }

    /**
     * 获取计算精度
     *
     * @return
     */
    public CurrencyPrecision getCurrencyPrecision() {
        int currencyPrecision = Integer.parseInt(mSharedPreferences.getString(Keys.CURRENCY_PRECISION, "2"));
        return CurrencyPrecision.valueOf(currencyPrecision);
    }

    /**
     * 返回打印时是否隐藏部分支付号
     *
     * @return true or false
     */
    public boolean hidePayNoWhenPrint() {
        return mSharedPreferences.getBoolean(Keys.HIDE_PAY_NO_WHEN_PRINT, true);
    }

    /**
     * 打印海信Pay签购单
     *
     * @return true or false
     */
    public boolean printHXPayReceipt() {
        return mSharedPreferences.getBoolean(Keys.PRINT_HXPAY_RECEIPT, false);
    }

    // get 启用小票打印机
    public boolean isPrintReceipt() {
        return mSharedPreferences.getBoolean(Keys.IS_PRINT_RECEIPT, false);
    }

    // get 打印字体大小设置
    public String getPrinterTextSize() {
        return mSharedPreferences.getString(Keys.PRINTER_TEXT_SIZE, "small");
    }

    // get 专柜联打印联数
    public ReceiptPrintType getReceiptPrintType() {
        String receiptPrintByShop;
        try {
            receiptPrintByShop = mSharedPreferences.getString(Keys.RECEIPT_PRINT_BY_SHOP, App.getAppContext().getResources().getString(R.string.receipt_print_by_shop));
        } catch (Exception e) {
            receiptPrintByShop = "ByShop";
        }
        return ReceiptPrintType.valueOf(receiptPrintByShop);
    }

    /**********************************************************************
     * 储值卡设置
     **********************************************************************/

    // get 查询时输入卡面后几位
    public boolean isPrepaidCardQueryByCheckingFaceNo() {
        return mSharedPreferences.getBoolean(Keys.PREPAID_CARD_QUERY_BY_CHECKING_FACE_NO, false);
    }

    // get 查询时输入校验码
    public boolean isPrepaidCardQueryByCheckingVerifyCode() {
        return mSharedPreferences.getBoolean(Keys.PREPAID_CARD_QUERY_BY_CHECKING_VERIFY_CODE, false);
    }

    public boolean isPrepaidCardQueryByPassword() {
        return mSharedPreferences.getBoolean(Keys.PREPAID_CARD_QUERY_BY_PASSWORD, false);
    }

    //付款码支付是否需要密码
    public boolean isEPrepaidCardQueryByPassword() {
        return mSharedPreferences.getBoolean(Keys.E_PREPAID_CARD_QUERY_BY_PASSWORD, false);
    }

    //是否限制原卡退货
    public boolean isReturnCard() {
        return mSharedPreferences.getBoolean(Keys.RETURN_CARD, false);
    }

    /**********************************************************************
     * 积分卡设置
     **********************************************************************/
    // get 查询时输入卡面后几位
    public boolean isVipPointsCardQueryByCheckingFaceNo() {
        return mSharedPreferences.getBoolean(Keys.VIP_POINTS_QUERY_BY_CHECKING_FACE_NO, false);
    }

    // get 查询时输入校验码
    public boolean isVipPointsCardQueryByCheckingVerifyCode() {
        return mSharedPreferences.getBoolean(Keys.VIP_POINTS_QUERY_BY_CHECKING_VERIFY_CODE, false);
    }

    public boolean isVipPointsCardQueryByPassword() {
        return mSharedPreferences.getBoolean(Keys.VIP_POINTS_QUERY_BY_PASSWORD, false);
    }

    //付款码支付是否需要密码
    public boolean isEVipPointsCardQueryByPassword() {
        return mSharedPreferences.getBoolean(Keys.E_VIP_POINTS_QUERY_BY_PASSWORD, false);
    }

    /**********************************************************************
     * 交易设置
     **********************************************************************/

    // get 是否允许退货
    public boolean isSaleReturnEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_SALE_RETURN_ENABLED, true);
    }

    // get 是否允许预销售
    public boolean isPreSaleEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_PRE_SALE_ENABLED, false);
    }

    // get 是否允许全场收银
    //JIRA:HIPOSAPP-113
    public boolean isNoShopLimits() {
        return mSharedPreferences.getBoolean(Keys.IS_SALE_NO_SHOP_LIMITS, true);
    }

    // get 是否允许输入营业员
    public boolean isCashierEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_SALE_CASHIER_ENABLED, false);
    }

    // get 是否启用实时积分
    public boolean isRealTimePointEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_REAL_TIME_POINTS_ENABLED, false);
    }

    // 是否禁止销售
    public boolean isSaleEnabled() {
        return !mSharedPreferences.getBoolean(Keys.IS_SALE_DISABLED, false);
    }

    //网络连接超时
    public int getTimeout() {
        String timeout = mSharedPreferences.getString(Keys.TIMEOUT, "60");
        return Integer.valueOf(timeout);
    }

    /**********************************************************************
     * 支付设置
     **********************************************************************/
    /**
     * 获取聚合支付通道名称
     *
     * @return AggregatePaymentChannel 聚合支付通道名称
     */
    public String getAggregatePaymentChannel() {
        return mSharedPreferences.getString(Keys.AGGREGATE_PAYMENT_CHANNEL, AggregatePaymentChannel.CHANNEL);
    }

    /**
     * 是否启用微信刷脸支付
     * @return 默认true
     */
    public boolean isUseWxFacePay() {
        return mSharedPreferences.getBoolean(Keys.IS_USE_WX_FACE_PAY, true);
    }

    /**********************************************************************
     * 射频卡设置
     **********************************************************************/
    public IcCardVersion getIcCardVersion() {
        int cardVersion = Integer.parseInt(mSharedPreferences.getString(Keys.IC_CARD_VERSION, "2"));
        return IcCardVersion.valueOf(cardVersion);
    }

    /**********************************************************************
     * 退货流水日期设置
     **********************************************************************/
    public int getReturnSaleDays() {
        return Integer.valueOf(mSharedPreferences.getString(Keys.RETURN_SALE_DAYS, "10"));
    }

    // get 是否启用积分原路退回
    public boolean isPointsRegressEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_VIP_POINTS_REGRESS_ENABLED, false);
    }

    // get 启用赠券原路退回
    public boolean isSaleCouponReturnEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_SALE_COUPON_RETURN_ENABLED, false);
    }

    // 提交更改
    private void commit() {
        mEditor.commit();
    }

    /**
     * 获取第三方调用名称
     *
     * @return PaymentChannel 第三方名称
     */
    public PaymentChannel getPaymentChannel() {
        String paymentChannel;
        try {
            paymentChannel = mSharedPreferences.getString(Keys.PAYMENT_CHANNEL, App.getContext().getResources().getStringArray(R.array.paymentChannelValue)[0]);
        } catch (Exception e) {
            paymentChannel = "CCB";
        }

        return PaymentChannel.valueOf(paymentChannel);
    }

    /**
     * 返回是否会有注册
     *
     * @return true 会员注册，false 不进行会员注册
     */
    public boolean isVipRegisterEnabled() {
        return mSharedPreferences.getBoolean(Keys.IS_VIP_REGISTER_ENABLED, false);
    }

    /**
     * 存储交易类型
     *
     * @param tranType 交易类型
     */
    public void setTranType(String tranType) {
        mEditor.putString(Keys.SALE_TRAN_TYPE, tranType);
        this.commit();
    }

    /**
     * 返回交易类型值
     *
     * @return setTranType 交易类型
     */
    public String getTranType() {
        return mSharedPreferences.getString(Keys.SALE_TRAN_TYPE, "");
    }

    /**
     * 日结后是否结算
     *
     * @return true 是 ,false 否
     */
    public boolean isSettleAfterAccount() {
        return mSharedPreferences.getBoolean("isSettleAfterAccount", false);
    }

    /**********************************************************************
     * 促销设置
     **********************************************************************/

    // set 启用满赠券打印
    public void setCoupon(boolean value) {
        mEditor.putBoolean("coupon", value);
        this.commit();
    }

    // get 启用会员打印赠券
    public boolean isPrintVipCoupon() {
        return mSharedPreferences.getBoolean("vipPrintCoupon", false);
    }

    /**********************************************************************
     * 打印设置
     **********************************************************************/

    // get 启用小票打印机
    public boolean getLocalVoucherPrinter() {
        return mSharedPreferences.getBoolean("localVoucherPrinter", true);
    }

    // get 打印机波特率设置
    public int getPrinterRate() {
        String printerRate = mSharedPreferences.getString("printerRate", "115200");
        LoggerUtils.getLogger().info("设备打印波特率:" + printerRate);
        return Integer.valueOf(printerRate);
    }

    // get 打印机端口设置
    public int getPrinterPort() {
        String printerPort = mSharedPreferences.getString("printerPort", "1");
        LoggerUtils.getLogger().info("设备打印端口:" + printerPort);
        return Integer.valueOf(printerPort);
    }

    // get 打印完成走纸行数设置
    public int getPrinterFreeLine() {
        int printFreeLine = 0;
        String printFree = mSharedPreferences.getString("printFreeLine", "0");
        if (StringUtils.isEmpty(printFree)) {
            return printFreeLine;
        }
        printFreeLine = Integer.valueOf(printFree);
        return printFreeLine;
    }

    /**********************************************************************
     * 储值卡设置
     **********************************************************************/
    public void setPrepaidCard_queryInputCheckBit(boolean value) {
        mEditor.putBoolean("prepaidCard_queryInputCheckBit", value);
    }

    // get 查询时输入校验位
    public boolean getPrepaidCard_queryInputCheckBit() {
        return mSharedPreferences.getBoolean("prepaidCard_queryInputCheckBit", false);
    }

    public boolean getPrepaidCard_queryInputPwd() {
        return mSharedPreferences.getBoolean("prepaidCard_queryInputPwd", false);
    }

    public void setPrepaidCard_queryInputPwd(boolean value) {
        mEditor.putBoolean("prepaidCard_queryInputPwd", value);
    }


    /**********************************************************************
     * 积分卡设置
     **********************************************************************/
    public void setPointsCard_queryInputCheckBit(boolean value) {
        mEditor.putBoolean("pointsCard_queryInputCheckBit", value);
    }

    // get 查询时输入校验位
    public boolean getPointsCard_queryInputCheckBit() {
        return mSharedPreferences.getBoolean("pointsCard_queryInputCheckBit", false);
    }

    public boolean getPointsCard_queryInputPwd() {
        return mSharedPreferences.getBoolean("pointsCard_queryInputPwd", false);
    }

    public void setPointsCard_queryInputPwd(boolean value) {
        mEditor.putBoolean("pointsCard_queryInputPwd", value);
    }

    /**********************************************************************
     * 交易设置
     **********************************************************************/

    // get 行清是否需要授权
    public boolean isClearCheckPm() {
        return mSharedPreferences.getBoolean("isClearCheckPm", false);
    }

    // get 删除商品是否需要授权
    public boolean isDeleteCheckPm() {
        return mSharedPreferences.getBoolean("isDeleteCheckPm", false);
    }

    // get 减少商品是否需要授权
    public boolean isDecreaseCheckPm() {
        return mSharedPreferences.getBoolean("isDecreaseCheckPm", false);
    }

    // get 是否使用VIP认证
    public boolean isUseVip() {
        return mSharedPreferences.getBoolean("isUseVIP", true);
    }

    // get 是否允许录入大码商品价格
    public boolean isNoCodePrice() {
        return mSharedPreferences.getBoolean("isNoCodePrice", false);
    }

    // get 是否使用核销功能
    public boolean isUse() {
        return mSharedPreferences.getBoolean("isUse", false);
    }

    // 是否禁止销售
    public boolean isSale() {
        return mSharedPreferences.getBoolean("isSale", false);
    }

    /**********************************************************************
     * 支付设置
     **********************************************************************/

    public String getBigBagCode() {
        String bigBagCode = mSharedPreferences.getString("bigBag", "");
        return bigBagCode;
    }

    public void setBigBagCode(String bigBagCode) {
        mEditor.putString("bigBag", bigBagCode);
        this.commit();
    }

    public void setConfigVersion(String configVersion) {
        mEditor.putString("configVersion", configVersion);
        this.commit();
    }

    public String getConfigVersion() {
        String strConfigVersion = mSharedPreferences.getString("configVersion", "0");
        return strConfigVersion;
    }

    public void setAipUrl(String aipUrl) {
        mEditor.putString("aipurl", aipUrl);
        this.commit();
    }

    public String getAipUrl() {
        String strAipUrl = mSharedPreferences.getString("aipurl", "");
        return strAipUrl;
    }

    public void setAipKey(String aipKey) {
        mEditor.putString("aipkey", aipKey);
        this.commit();
    }

    public String getAipKey() {
        String strAipKey = mSharedPreferences.getString("aipkey", "");
        return strAipKey;
    }

    public void setAipAppId(String aipAppId) {
        mEditor.putString("aipappid", aipAppId);
        this.commit();
    }

    public String getAipAppId() {
        String strAipAppId = mSharedPreferences.getString("aipappid", "");
        return strAipAppId;
    }


    public void setBigBagPrice(double price) {
        mEditor.putFloat("bigBagPrice", (float) price);
        this.commit();
    }

    public double getBigBagPrice() {
        String strBigBagPrice = mSharedPreferences.getString("bigBagPrice", "0.00");
        float bigBagPrice = Float.valueOf(strBigBagPrice);
        return bigBagPrice;
    }

    public String getMiddleBagCode() {
        String middleBagCode = mSharedPreferences.getString("middleBag", "");
        return middleBagCode;
    }

    public void setMiddleBagCode(String middleBagCode) {
        mEditor.putString("middleBag", middleBagCode);
        this.commit();
    }

    public void setMiddleBagPrice(double price) {
        mEditor.putFloat("middleBagPrice", (float) price);
        this.commit();
    }

    public double getMiddleBagPrice() {
        String strMiddleBagPrice = mSharedPreferences.getString("middleBagPrice", "0.00");
        float middleBagPrice = Float.valueOf(strMiddleBagPrice);
        return middleBagPrice;
    }

    public String getSmallBagCode() {
        String smallBagCode = mSharedPreferences.getString("smallBag", "");
        return smallBagCode;
    }

    public void setSmallBagCode(String smallBagCode) {
        mEditor.putString("smallBag", smallBagCode);
        this.commit();
    }

    public void setSmallBagPrice(double price) {
        mEditor.putFloat("smallBagPrice", (float) price);
        this.commit();
    }

    public double getSmallBagPrice() {
        String strSmallBagPrice = mSharedPreferences.getString("smallBagPrice", "0.00");
        float smallBagPrice = Float.valueOf(strSmallBagPrice);
        return smallBagPrice;
    }

    public String getPayAntChannel() {
        String payAntChannel = mSharedPreferences.getString("PayAntChannel", "hiamw");
        return payAntChannel;
    }

    public String getWXFaceMchid() {
        String wxFaceMchid = mSharedPreferences.getString("WXFaceMchid", "");
        return wxFaceMchid;
    }

    public int getAliFacePayLimit() {
        String value = mSharedPreferences.getString("AliFacePayLimit", "2000");

        return StringUtils.toInt(value, 2000);
    }

    public int getTemplateVersion(PrintTemplateType type) {
        if (type == PrintTemplateType.Receipt) {
            return Integer.valueOf(mSharedPreferences.getString("vouVersion", "1"));
        } else {
            return Integer.valueOf(mSharedPreferences.getString("couVersion", "1"));
        }
    }

    public void setTemplateVersion(PrintTemplateType type, int version) {
        String sVersion = Integer.valueOf(version).toString();
        if (type == PrintTemplateType.Receipt) {
            mEditor.putString("vouVersion", sVersion);
        } else {
            mEditor.putString("couVersion", sVersion);
        }
        this.commit();
    }

    /**********************************************************************
     * 管理设置
     **********************************************************************/
    public String GetPermissionCode() {
        return mSharedPreferences.getString("PermissionCode", "666666");
    }

    public void setPermissionCode(String permissionCode) {
        mEditor.putString("PermissionCode", permissionCode);
        this.commit();
    }

    public boolean isUsePermission() {
        return mSharedPreferences.getBoolean("isUsePermission", false);
    }

    //默认的APK OTA升级地址，应配置改掉，以免浪费我们的CDN流量
    final String defaultOtaServerIp = "http://fileserver.hics.hisense.com";
    //默认的APK OTA升级地址，推荐80
    final String defaultOtaServerPort = "80";


    final String defaultShopDoorIp = "191.168.0.186";
    final String defaultShopDoorPort = "5000";

    /**
     * 获取APK升级服务器IP地址
     *
     * @return
     */
    public String getOtaServerIp() {
        String serverIp = mSharedPreferences.getString("ApkUpdateServerIp", defaultOtaServerIp);
        return serverIp;
    }

    public String getJXGGShopArea() {
        String shopArea = mSharedPreferences.getString("JXGGShopArea", "JA");
        return shopArea;
    }

    /**
     * 设置APK升级服务器IP地址
     */
    public void setOtaServerIp(String otaServerIp) {
        String serverIp = defaultOtaServerIp;
        if (otaServerIp != null) {
            serverIp = otaServerIp;
        }
        mEditor.putString("ApkUpdateServerIp", serverIp);
        this.commit();
    }

    /**
     * 获取APK升级服务器IP端口
     *
     * @return
     */
    public String getOtaServerPort() {
        String serverPort = mSharedPreferences.getString("ApkUpdateServerPort", defaultOtaServerPort);
        return serverPort;
    }

    /**
     * 设置APK升级服务器IP端口
     */
    public void setOtaServerPort(String otaServerPort) {
        String serverPort = defaultOtaServerPort;
        if (otaServerPort != null) {
            serverPort = otaServerPort;
        }
        mEditor.putString("ApkUpdateServerPort", serverPort);
        this.commit();
    }

    /**
     * 获取安装序列码
     *
     * @return cdkey 安装序列码
     */
    public String getCdKey() {
        //String cdkey = mSharedPreferences.getString("cdkey", "C3B5W-38S79-7ZTX0-6GDI6-60002");
        String cdkey = mSharedPreferences.getString("cdkey", "");
        return cdkey;
    }

    /*
     设置CdKey
     */
    public void setCdKey(String cdKey) {
        mEditor.putString("cdkey", cdKey);
        this.commit();
    }

    /*
     * 获取门店
     */
    public String getOrgName() {
        String cdkey = mSharedPreferences.getString("orgName", "");
        return cdkey;
    }

    /**
     * 获取操作员号码
     *
     * @return operator 操作员号码
     */
    public String getOperator() {
        //String operator = mSharedPreferences.getString("operator", "0375");
        String operator = mSharedPreferences.getString("operator", "");
        return operator;
    }

    public void setOperator(String operator) {
        mEditor.putString("operator", operator);
        this.commit();
    }

    /**
     * 扫描商品是否过滤无效内容
     *
     * @return
     */
    public boolean isCheckScanCode() {
        return mSharedPreferences.getBoolean("isCheckScanCode", false);
    }

    public boolean isScannerBeep() {
        return mSharedPreferences.getBoolean("isScannerBeep", false);
    }

    public boolean isUseBaleBarCode() {
        return mSharedPreferences.getBoolean("isUseBaleBarCode", false);
    }

    public String getTestInterval() {
        String testInterval = mSharedPreferences.getString("testTime", "1");
        return testInterval;
    }

    public void setIntervention(String voidsIntervention) {
        mEditor.putString("voidsIntervention", voidsIntervention);
        this.commit();

    }

    public String getIntervention() {
        String voidsIntervention = mSharedPreferences.getString("voidsIntervention", "1");
        return voidsIntervention;
    }

    /**
     * 获取开票地址
     *
     * @return
     */
    public String getInvoiceAddress() {
        String invoiceAddress = mSharedPreferences.getString("invoiceAddress", "");
        return invoiceAddress;
    }

    /**
     * @return
     */
    public String getInvoiceKey() {
        String invoiceKey = mSharedPreferences.getString("invoiceKey", "94hdf3#@");
        return invoiceKey;
    }

    /**
     * 获取商户秘钥
     *
     * @return shopKey
     */
    public String getShopKey() {
        String shopKey = mSharedPreferences.getString("shopKey", "");
        return shopKey;
    }

    public boolean isRePrint() {
        return mSharedPreferences.getBoolean("isRePrint", false);
    }

    public boolean isUsePrinter() {
        return mSharedPreferences.getBoolean("isUsePrinter", true);
    }

    public boolean isShowDiscount() {
        return mSharedPreferences.getBoolean("isShowDiscount", false);
    }

    /**
     * 设置人脸支付AppID
     */
    public String getAppId() {
        String appID = mSharedPreferences.getString("appID", "");
        return appID;
    }

    public void setAppID(String appID) {
        mEditor.putString("appID", appID);
        this.commit();
    }

    /**
     * 设置签约商户ID
     *
     * @return
     */
    public String getMerchantId() {
        String merchantId = mSharedPreferences.getString("merchantId", "");
        return merchantId;
    }

    public void setMerchantId(String merchantId) {
        mEditor.putString("merchantId", merchantId);
        this.commit();
    }

    /**
     *
     */
    public String getPartnerId() {
        String partnerId = mSharedPreferences.getString("partnerId", "");
        return partnerId;
    }

    public void setPartnerId(String partnerId) {
        mEditor.putString("partnerId", partnerId);
        this.commit();
    }

    public String getCashierCode() {
        String cashierCode = mSharedPreferences.getString("cashierCode", "");
        return cashierCode;
    }

    public String getCashierPwd() {
        String pwd = mSharedPreferences.getString("cashierPwd", "");
        return pwd;
    }

    public void setQRCode(String qrCode) {
        mEditor.putString("qrCode", qrCode);
        this.commit();
    }

    public String getQRCode() {
        return mSharedPreferences.getString("qrCode", "");
    }

    //清理日志天数设置
    public String getLogFileExpiredDays() {
        return mSharedPreferences.getString(Keys.LOG_FILE_EXPIRED_DAYS, "不清理");
    }

    public void setInvoiceUrl(String invoiceUrl) {
        mEditor.putString("invoiceUrl", invoiceUrl);
        this.commit();
    }

    public String getInvoiceUrl() {
        return mSharedPreferences.getString("invoiceUrl", "");
    }

    //启用打印小票提示框
    public boolean isPrintMessage() {
        return mSharedPreferences.getBoolean("printMessage", true);
    }

    /**
     * 保存授权信息
     *
     * @param authorization 授权信息
     */
    public void saveAuthorization(String authorization) {
        mEditor.putString("authorization", authorization);
        this.commit();
    }

    /**
     * 获取授权信息
     *
     * @return auth 授权信息
     */
    public String getAuthorization() {
        String auth = mSharedPreferences.getString("authorization", "");
        return auth;
    }

    /**
     * 设置激活状态
     *
     * @param isActivate 是否被激活
     */
    public void setIsActivate(boolean isActivate) {
        mEditor.putBoolean("activation", isActivate);
        this.commit();
    }

    /**
     * 获取激活状态
     *
     * @return 是否被激活
     */
    public boolean isActivate() {
        return mSharedPreferences.getBoolean("activation", false);
    }

    /**
     * 设置加密授权url
     *
     * @param payAuthUrl 加密授权url
     */
    public void setPayAuthUrl(String payAuthUrl) {
        mEditor.putString("payAuthUrl", payAuthUrl);
        this.commit();
    }

    /**
     * 获取加密授权url
     *
     * @return payAuthUrl 加密授权url
     */
    public String getPayAuthUrl() {
        String payAuthUrl = mSharedPreferences.getString("payAuthUrl", "");
        return payAuthUrl;
    }

    /**
     * 设置加密授权key
     *
     * @param payAuthKey 加密授权key
     */
    public void setPayAuthKey(String payAuthKey) {
        mEditor.putString("payAuthKey", payAuthKey);
        this.commit();
    }

    /**
     * 获取加密授权Key
     *
     * @return payAuthKey 加密授权Key
     */
    public String getPayAuthKey() {
        String payAuthKey = mSharedPreferences.getString("payAuthKey", "");
        return payAuthKey;
    }

    /**
     * 获取商品提示框显示时间
     *
     * @return 提示框显示时间
     */
    public String getPluDialogTime() {
        String testInterval = mSharedPreferences.getString("pluDialogTime", "5000");
        return testInterval;
    }

    /**********************************************************************
     * 归集接口设置
     **********************************************************************/
    /**
     * 是否开启归集接口
     * @return 默认false
     */
    public boolean isOpenRpt() {
        return mSharedPreferences.getBoolean("isOpenRpt", false);
    }

    public boolean isUseGiftCard() {
        return mSharedPreferences.getBoolean("isUseGiftCard", false);
    }
}
