/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.constant.device;

/**
 * 设备类型
 *
 * @author xieliyan 2018-01-24
 */
public enum DeviceType {

    /**
     * 设备类型为联迪A8
     */
    LANDI("LANDI"),
    /**
     * 设备类型为HI98
     */
    HI98("HI98"),
    /**
     * 设备类型为艾体威尔A90
     */
    AISINO("AISINO"),
    /**
     * 设备类型为HS650
     */
    HS650("HS650"),
    /**
     * 设备类型为HS550
     */
    HS550("HS550"),
    /**
     * 设备类型为HK316
     */
    HK316("HK316"),
    /**
     * 设备类型为商米T2机器内置打印机
     */
    SUNMIT2("SUNMIT2");

    private String value;

    /**
     * 初始化一个 DeviceType 枚举。
     *
     * @param value 设备类型。
     */
    DeviceType(String value) {
        this.value = value;
    }

    /**
     * 返回设备的名称。
     *
     * @return 设备的名称。
     */
    public String value() {
        return this.value;
    }

    /**
     * 是否选择联迪A8
     *
     * @return true：联迪A8；false：海信HI98
     */
    public boolean isLANDI() {
        return this.equals(LANDI);
    }

    /**
     * 是否选择海信HI98
     *
     * @return true：海信HI98；false：联迪A8
     */
    public boolean isHI98() {
        return this.equals(HI98);
    }

    /**
     * 是否选择艾体威尔A90
     *
     * @return true：艾体威尔A90；false：联迪A8
     */
    public boolean isAISINO() {
        return this.equals(AISINO);
    }

    /**
     * 是否选择HS650
     *
     * @return true：HS650
     */
    public boolean isHS650() {
        return this.equals(DeviceType.HS650);
    }

    /**
     * 是否选择HS550
     *
     * @return true：HS550
     */
    public boolean isHS550() {
        return this.equals(DeviceType.HS550);
    }

    /**
     * 是否选择HS650 or HS550
     *
     * @return true：HS650 or HS550
     */
    public boolean isHSX() {
        return (this.equals(DeviceType.HS650) || this.equals(DeviceType.HS550));
    }

    /**
     * 是否选择HK316
     *
     * @return true：HK316
     */
    public boolean isHK316() {
        return this.equals(DeviceType.HK316);
    }
    /**
     * 是否选择商米T2
     *
     * @return true：HS650 or HS550
     */
    public boolean isSunmiT2() {
        return (this.equals(DeviceType.SUNMIT2));
    }
}
