package com.hisense.hiaposcmp.dialog;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.CountDownTimer;

import androidx.annotation.ColorInt;

import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;

/**
 * Created by wangwenyue on 2018/7/6.
 * 微信家乐福通用提示对话框
 */
public class AlertWxDialog extends Dialog {

    private Button yes;//确定按钮
    private Button no;//取消按钮
    private TextView titleTv;//消息标题文本
    private TextView messageTv;//消息提示文本
    private String titleStr;//从外界设置的title文本
    private String messageStr;//从外界设置的消息文本
    private String yesStr, noStr;   //确定文本和取消文本的显示内容

    private AlertWxDialog.onNoOnclickListener noOnclickListener;//取消按钮被点击了的监听器
    private AlertWxDialog.onYesOnclickListener yesOnclickListener;//确定按钮被点击了的监听器

    boolean hasNoButton = true;

    private CountDownTimer timer;
    private boolean isClickYesButton;

    private boolean exit=false;//倒计时后退出

    /**
     * 设置取消按钮的显示内容和监听
     *
     * @param str
     * @param onNoOnclickListener
     */
    public void setNoOnclickListener(String str, AlertWxDialog.onNoOnclickListener onNoOnclickListener) {
        if (str != null) {
            noStr = str;
        }
        if (hasNoButton) {
            this.noOnclickListener = onNoOnclickListener;
        }
    }

    /**
     * 设置确定按钮的显示内容和监听
     *
     * @param str
     * @param onYesOnclickListener
     */
    public void setYesOnclickListener(String str, AlertWxDialog.onYesOnclickListener onYesOnclickListener) {
        if (str != null) {
            yesStr = str;
        }
        this.yesOnclickListener = onYesOnclickListener;
    }

    public AlertWxDialog(Context context, boolean noButton) {
        super(context, R.style.SSMyDialog);
        hasNoButton = noButton;
        isClickYesButton = true;
    }

    public AlertWxDialog(Context context, boolean noButton, boolean isClick) {
        super(context, R.style.SSMyDialog);
        hasNoButton = noButton;
        isClickYesButton = isClick;
    }

    public AlertWxDialog(Context context, boolean noButton, boolean yes, boolean isExit) {
        super(context, R.style.SSMyDialog);
        hasNoButton = noButton;
        isClickYesButton = yes;
        exit = isExit;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //按空白处不能取消动画
        setCanceledOnTouchOutside(false);

        if (hasNoButton) {
            setContentView(R.layout.dialog_wechat_doublebtn);
        } else {
            setContentView(R.layout.dialog_wechat_singlebtn);
        }
        //初始化界面控件
        initView();
        //初始化界面数据
        initData();
        //初始化界面控件的事件
        initEvent();

        initTimer();
    }

    /**
     * 初始化界面的确定和取消监听器
     */
    private void initEvent() {
        //设置确定按钮被点击后，向外界提供监听
        yes.setOnClickListener((v) -> {
            if (yesOnclickListener != null) {
                yesOnclickListener.onYesClick();
            }
        });
        if (hasNoButton) {
            //设置取消按钮被点击后，向外界提供监听
            no.setOnClickListener((v) -> {
                if (noOnclickListener != null) {
                    noOnclickListener.onNoClick();
                }
            });
        }
    }

    /**
     * 初始化界面控件的显示数据
     */
    private void initData() {
        //如果用户自定了title和message
        if (titleStr != null) {
            titleTv.setText(titleStr);
        }
        if (messageStr != null) {
            messageTv.setText(messageStr);
            messageTv.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
        }
        //如果设置按钮的文字
        if (yesStr != null) {
            yes.setText(yesStr);
        }
        if (hasNoButton) {
            if (noStr != null) {
                no.setText(noStr);
            }
        }
    }

    /**
     * 初始化界面控件
     */
    private void initView() {
        yes = (Button) findViewById(R.id.yes);
        yes.setFocusable(false);
        titleTv = (TextView) findViewById(R.id.title);
        messageTv = (TextView) findViewById(R.id.message);
        if (hasNoButton) {
            no = (Button) findViewById(R.id.no);
            no.setFocusable(false);
        }
    }

    /**
     * 设置计时器
     */
    private void initTimer() {

        long millsInFuture = 20000;
        if (exit) {
            if(!StringUtils.isEmpty(GeneralSettings.INSTANCE.getPluDialogTime())){
                millsInFuture = Integer.parseInt(GeneralSettings.INSTANCE.getPluDialogTime());
            }

            LoggerUtils.getLogger().debug("设置提示框显示时间"+GeneralSettings.INSTANCE.getPluDialogTime());
        }
        timer = new CountDownTimer(millsInFuture, 1000) {

            @Override
            public void onTick(long millisUntilFinished) {
                if (exit) {
                    yes.setText("确定" + String.format("(%ds)", millisUntilFinished / 1000));
                }
            }

            @Override
            public void onFinish() {

                if (yesOnclickListener != null) {
                    if (isClickYesButton) {
                        yesOnclickListener.onYesClick();
                    } else {
                        noOnclickListener.onNoClick();
                    }
                }

                dismiss();
            }
        }.start();
    }


    @Override
    public void dismiss() {
        try {
            super.dismiss();
            if (timer != null) {
                timer.cancel();
            }
        } catch (Exception e) {
        }
    }

    @Override
    public void onStop() {
        try {
            super.onStop();
            if (timer != null) {
                timer.cancel();
            }
        } catch (Exception e) {
        }
    }

    /**
     * 从外界Activity为Dialog设置标题
     * 没有no按钮的Dialog不需要设置title
     *
     * @param title
     */
    public void setTitle(String title) {
        titleStr = title;
    }

    /**
     * 从外界Activity为Dialog设置dialog的message
     *
     * @param message
     */
    public void setMessage(String message) {
        messageStr = message;
    }


    /**
     * 修改确认按键文字颜色
     *
     * @param resId
     */
    public void setYesButtonColor(@ColorInt int resId) {
        yes.setTextColor(resId);
    }


    /**
     * 设置确定按钮和取消被点击的接口
     */
    public interface onYesOnclickListener {
        public void onYesClick();
    }

    public interface onNoOnclickListener {
        public void onNoClick();
    }


}
