package com.hisense.hiaposcmp.dialog;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.activity.ali.WelcomeAliActivity;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;

/**
 * 界面自动退出对话框
 */

public class AutoExitAliDialog extends Dialog {

    private ImageView yes;//确定按钮
    //private Button no;//取消按钮
    private TextView titleTv;//消息标题文本
    private TextView messageTv;//消息提示文本
    private String titleStr;//从外界设置的title文本
    private String messageStr;//从外界设置的消息文本
    private TextView tvCountDown;

    private Context mContext;
    private Activity mActivity;

    //确定文本和取消文本的显示内容
    private String yesStr, noStr;

    private AutoExitAliDialog.onYesOnclickListener yesOnclickListener;//确定按钮被点击了的监听器

    boolean hasNoButton = false;

    /**
     * 设置确定按钮的显示内容和监听
     *
     * @param str
     * @param onYesOnclickListener
     */
    public void setYesOnclickListener(String str, AutoExitAliDialog.onYesOnclickListener onYesOnclickListener) {
        if (str != null) {
            yesStr = str;
        }
        this.yesOnclickListener = onYesOnclickListener;
    }

    public AutoExitAliDialog(Context context) {
        super(context, R.style.SSMyDialog);
        mContext = context;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //按空白处不能取消动画
        setCanceledOnTouchOutside(false);
        setContentView(R.layout.layout_dialog_auto_back);

        //初始化界面控件
        initView();
        //初始化界面数据
        initData();
        //初始化界面控件的事件
        initEvent();
    }


    @Override
    protected void onStart() {
        super.onStart();
        //beginCountDown();
    }

    /**
     * 初始化界面的确定和取消监听器
     */
    private void initEvent() {
        //设置确定按钮被点击后，向外界提供监听
        yes.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (yesOnclickListener != null) {
                    yesOnclickListener.onYesClick();
                }
            }
        });
    }

    /**
     * 初始化界面控件的显示数据
     */
    private void initData() {
        //如果用户自定了title和message
        if (titleStr != null) {
            titleTv.setText(titleStr);
        }
        if (messageStr != null) {
            messageTv.setText(messageStr);
            messageTv.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
        }
    }

    /**
     * 初始化界面控件
     */
    private void initView() {
        yes = findViewById(R.id.ivClose);
        titleTv = findViewById(R.id.tvHintMainHead);
        messageTv = findViewById(R.id.tvHintSubHead);
        tvCountDown = findViewById(R.id.tvCountDown);
    }

    private CountDownTimer timer = new CountDownTimer(30000, 1000) {

        @Override
        public void onTick(long millisUntilFinished) {
            tvCountDown.setText(String.format("%ds......", millisUntilFinished / 1000));
        }

        @Override
        public void onFinish() {
            if (!(mContext instanceof WelcomeAliActivity)) {
               // SaleModel.getInstance().reset();
                MainSaleModel.getInstance().reset();
                if (CommonUtils.isAliStyle() ){
                    CommonTools.gotoWelcomeAntActivity((BaseActivity) mContext);
                } else if( CommonUtils.isAliLandStyle()){
                    CommonTools.gotoWelcomeAntLandActivity((BaseActivity) mContext);
                } else {
                    CommonTools.gotoWelcomeAntActivity((BaseActivity) mContext);
                }
            }
            dismiss();
        }
    }.start();


    @Override
    public void dismiss() {
        super.dismiss();
        timer.cancel();
    }

    @Override
    public void onStop() {
        super.onStop();
        timer.cancel();
    }

    /**
     * 从外界Activity为Dialog设置标题
     *
     * @param title
     */
    public void setTitle(String title) {
        titleStr = title;
    }

    /**
     * 从外界Activity为Dialog设置dialog的message
     *
     * @param message
     */
    public void setMessage(String message) {
        messageStr = message;
    }

    /**
     * 设置确定按钮和取消被点击的接口
     */
    public interface onYesOnclickListener {
        public void onYesClick();
    }
}
