package com.hisense.hiaposcmp.dialog;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;

/**
 * Created by wangwenyue on 2018/7/6.
 * 微信自动退出对话框
 */
public class AutoExitWxDialog extends Dialog {

    private Button yes;//确定按钮
    private Button no;//取消按钮
    private TextView titleTv;//消息标题文本
    private TextView messageTv;//消息提示文本
    private String titleStr;//从外界设置的title文本
    private String messageStr;//从外界设置的消息文本

    private Context mContext;
    private Activity mActivity;

    private CountDownTimer timer;

    //确定文本和取消文本的显示内容
    private String yesStr, noStr;

    private AutoExitWxDialog.onYesOnclickListener yesOnclickListener;//确定按钮被点击了的监听器
    private AutoExitWxDialog.onNoOnclickListener noOnclickListener;//取消按钮被点击了的监听器


    /**
     * 设置确定按钮的显示内容和监听
     *
     * @param onYesOnclickListener
     */
    public void setYesOnclickListener(AutoExitWxDialog.onYesOnclickListener onYesOnclickListener) {

        this.yesOnclickListener = onYesOnclickListener;
    }

    /**
     * 设置取消按钮的显示内容和监听
     * r
     *
     * @param onNoOnclickListener
     */
    public void setNoOnclickListener(AutoExitWxDialog.onNoOnclickListener onNoOnclickListener) {
        this.noOnclickListener = onNoOnclickListener;

    }

    public AutoExitWxDialog(Context context) {
        super(context, R.style.SSMyDialog);
        mContext = context;
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setCanceledOnTouchOutside(false);  //按空白处不能取消动画
        setContentView(R.layout.dialog_wechat_autoexit);
        initView();    //初始化界面控件
        initData();    //初始化界面数据
        initEvent();    //初始化界面控件的事件
        initTimer();
    }


    @Override
    protected void onStart() {
        super.onStart();
        //beginCountDown();
    }

    /**
     * 初始化界面的确定和取消监听器
     */
    private void initEvent() {
        //设置确定按钮被点击后，向外界提供监听
        yes.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (yesOnclickListener != null) {
                    yesOnclickListener.onYesClick();
                }
            }
        });
        //设置取消按钮被点击后，向外界提供监听
        no.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (noOnclickListener != null) {
                    noOnclickListener.onNoClick();
                }
            }
        });
    }

    /**
     * 初始化界面控件的显示数据
     */
    private void initData() {
        //如果用户自定了title和message
        if (titleStr != null) {
            titleTv.setText(titleStr);
        }
        if (messageStr != null) {
            messageTv.setText(messageStr);
            messageTv.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
        }
    }

    /**
     * 初始化界面控件
     */
    private void initView() {
        yes = findViewById(R.id.yes);
        no = findViewById(R.id.no);
        titleTv = findViewById(R.id.tvHintMainHead);
        messageTv = findViewById(R.id.tvHintSubHead);
    }

    private void initTimer() {
        try {
            timer = new CountDownTimer(20000, 1000) {

                @Override
                public void onTick(long millisUntilFinished) {
                    no.setText("退出" + String.format("(%ds)", millisUntilFinished / 1000));
                }

                @Override
                public void onFinish() {
                    noOnclickListener.onNoClick();
                }
            }.start();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("自动退出异常" + e.toString());
        }

    }

    @Override
    public void dismiss() {
        super.dismiss();
        timer.cancel();
    }

    @Override
    public void onStop() {
        super.onStop();
        timer.cancel();
    }

    /**
     * 从外界Activity为Dialog设置标题
     *
     * @param title
     */
    public void setTitle(String title) {
        titleStr = title;
    }

    /**
     * 从外界Activity为Dialog设置dialog的message
     *
     * @param message
     */
    public void setMessage(String message) {
        messageStr = message;
    }

    /**
     * 设置确定按钮和取消被点击的接口
     */
    public interface onYesOnclickListener {
        public void onYesClick();
    }

    public interface onNoOnclickListener {
        public void onNoClick();
    }
}
