package com.hisense.hiaposcmp.dialog;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.CountDownTimer;
import androidx.annotation.ColorInt;
import androidx.annotation.DrawableRes;
import androidx.annotation.NonNull;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;

/**
 * 提示弹出框
 *
 * @author lizhixin 2018-05-29 HIPOSAPP-466 统一提示弹出框
 * modify by wangwenyue
 * <p>
 * example for fail：
 * commHintDialog.show();
 * commHintDialog.setBasicSytle(commHintDialog.STYLE_FAILURE);
 * commHintDialog.setTextMainHead(title);
 * commHintDialog.setReason(reason);
 * commHintDialog.setOnDismissListener(new DialogInterface.OnDismissListener(){
 * @Override public void onDismiss(DialogInterface dialog) {
 * To do
 * }
 * });
 * <p>
 * example for success:
 * commHintDialog.show();
 * commHintDialog.setBasicSytle(commHintDialog.STYLE_SUCCESS);
 * commHintDialog.setTextMainHead(title);
 * commHintDialog.setOnDismissListener(new DialogInterface.OnDismissListener(){
 * @Override public void onDismiss(DialogInterface dialog) {
 * To do
 * }
 * });
 */
public class HintAliDialog extends Dialog implements View.OnClickListener {

    private TextView tvHintMainHead, tvHintSubHead, tvCountDown;
    private ImageView ivClose, ivLogo;
    private View rootView;
    private Context mContext;

    private static  int MILLIS_IN_FUTURE = 5000;//5s
    private static final int COUNT_DOWN_INTERVAL = 1000;//1s
    /**
     * 这里的设计思路为：通过简单的 1/2 来自动设置两套样式，因为当前项目中只有两种样式，可以省掉诸多的样式set
     */
    public static final int STYLE_SUCCESS = 1;//样式：成功
    public static final int STYLE_FAILURE = 2;//样式：失败

    public static final int VIEW_VISIBLE = 3; //
    public static final int VIEW_GONE = 4;



    public HintAliDialog(@NonNull Context context) {
        this(context, -1);
        mContext = context;
    }

    public HintAliDialog(@NonNull Context context, int themeResId) {
        super(context, R.style.dialog);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //设置基本样式
        rootView = View.inflate(mContext, R.layout.layout_dialog_comm_hint, null);
        setContentView(rootView);
        setCanceledOnTouchOutside(true);
        //初始化控件
        tvHintMainHead = (TextView) findViewById(R.id.tvHintMainHead);
        tvHintSubHead = (TextView) findViewById(R.id.tvHintSubHead);
        tvCountDown = (TextView) findViewById(R.id.tvCountDown);
        ivClose = (ImageView) findViewById(R.id.ivClose);
        ivLogo = (ImageView) findViewById(R.id.ivLogo);
        //设置监听事件
        ivClose.setOnClickListener(this);
        //开始倒计时
        beginCountDown();
    }

    /**
     * 选择一套样式，如成功/失败
     *
     * @param styleTag 样式代号
     */
    public void setBasicSytle(int styleTag) {
        switch (styleTag) {
            case STYLE_SUCCESS:
                setSuccessStyle();
                break;
            case STYLE_FAILURE:
                setFailueStyle();
                break;
            default:
                break;
        }
    }

    /**
     * 失败的一套样式
     */
    private void setFailueStyle() {
        setImgLogo(R.drawable.ic_failure);
        tvHintSubHead.setVisibility(View.VISIBLE);
    }

    /**
     * 成功的一套样式
     */
    private void setSuccessStyle() {
        setImgLogo(R.drawable.ic_success_large);
        tvHintSubHead.setVisibility(View.INVISIBLE);
    }

    /**
     * 设置主标题文本
     *
     * @param txtMainHead txtMainHead
     */
    public void setTextMainHead(String txtMainHead) {
        tvHintMainHead.setText(txtMainHead);
    }

    /**
     * 设置主标题时候可见
     *
     * @param exist
     */
    public void setTextMainHeadView(int exist) {
        if (exist == 3) {
            tvHintMainHead.setVisibility(View.VISIBLE);
        } else {
            tvHintMainHead.setVisibility(View.GONE);
        }
    }


    /**
     * 设置副标题文本
     *
     * @param txtSubHead txtSubHead
     */
    public void setReason(String txtSubHead) {
        tvHintSubHead.setText(txtSubHead);
        tvHintSubHead.setVisibility(TextUtils.isEmpty(txtSubHead) ? View.GONE : View.VISIBLE);
    }

    /**
     * 设置副标题是否可见
     *
     * @param exist
     */
    public void setReasonView(int exist) {
        if (exist == 3) {
            tvHintSubHead.setVisibility(View.VISIBLE);
        } else {
            tvHintSubHead.setVisibility(View.GONE);
        }
    }

    /**
     * 设置主标题文本颜色
     *
     * @param resId resId
     */
    public void setTextMainHeadColor(@ColorInt int resId) {
        tvHintMainHead.setTextColor(resId);
    }

    /**
     * 设置副标题文本颜色
     *
     * @param resId resId
     */
    public void setTextSubHeadColor(@ColorInt int resId) {
        tvHintSubHead.setTextColor(resId);
    }

    /**
     * 设置倒计时文本颜色
     *
     * @param resId resId
     */
    public void setTextCountDownColor(@ColorInt int resId) {
        tvCountDown.setTextColor(resId);
    }

    /**
     * 设置背景颜色
     *
     * @param resId colorId
     */
    public void setBackgroundColor(@ColorInt int resId) {
        rootView.setBackgroundColor(resId);
    }

    /**
     * 设置背景
     *
     * @param resId colorId
     */
    public void setBackground(@ColorInt int resId) {
        rootView.setBackgroundResource(R.drawable.shape_ant_dialog);
    }


    /**
     * 设置关闭按钮样式
     *
     * @param resId resId
     */
    public void setImgClose(@DrawableRes int resId) {
        ivClose.setImageResource(resId);
    }

    /**
     * 设置logo样式
     *
     * @param resId resId
     */
    public void setImgLogo(@DrawableRes int resId) {
        ivLogo.setImageResource(resId);
    }

    /**
     * 开始倒计时
     */
    private void beginCountDown() {
        if (!StringUtils.isEmpty(GeneralSettings.INSTANCE.getPluDialogTime())) {
            MILLIS_IN_FUTURE = Integer.parseInt(GeneralSettings.INSTANCE.getPluDialogTime());
        }

        new CountDownTimer(MILLIS_IN_FUTURE, COUNT_DOWN_INTERVAL) {

            @Override
            public void onTick(long millisUntilFinished) {
                //重置倒计时文本
                tvCountDown.setText(String.format("%ds......", millisUntilFinished / 1000));
            }

            @Override
            public void onFinish() {
                //倒计时结束关闭对话框
                dismiss();
            }
        }.start();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.ivClose:
                dismiss();
                break;
            default:
                break;
        }
    }


}
