package com.hisense.hiaposcmp.hs330r;

import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.RemoteException;

import com.hisense.common.util.LoggerUtils;
import com.histonepos.npsdk.bind.Const;
import com.histonepos.npsdk.bind.PrinterConsts;
import com.histonepos.npsdk.printer.IPrinterService;

/**
 * @author gaozhen
 * @date 2019-06-20
 */
public class PrinterSmartHelper {
    private static final String TAG = "PrinterSmartHelper";
    private static IPrinterService printer = null;
    private volatile static PrinterSmartHelper printerSmartHelper;
    /****打印机型号(58/80)***/
    private static String prnWidth = PrinterConsts.PRT_CFG_VALUE_WIDTH_MODE_80;
    /****打印完成走纸行数***/
    private static int feedLine = 4;

    /**
     * 私有化PrinterSmartHelper
     */
    private PrinterSmartHelper() {
    }

    public static PrinterSmartHelper getInstance() {
        if (printerSmartHelper == null) {
            synchronized (PrinterSmartHelper.class) {
                if (printerSmartHelper == null) {
                    printerSmartHelper = new PrinterSmartHelper();
                }
            }
        }
        return printerSmartHelper;
    }

    public boolean initPrinter() {
        int result = 0;
        try {
            printer = getPrinter();
            if (printer == null) {
                result = 1;
            }
            if (!setPrintConfig()) {
                result = 1;
            }
        } catch (Exception e) {
            result = 1;
            e.printStackTrace();
            LoggerUtils.getLogger().debug("----getPrinters exception----" + e.toString());
        }
        return result == 0;
    }

    private IPrinterService getPrinter() {
        try {
            return IPrinterService.Stub.asInterface(GlobalDefines.sm.getService(Const.PRINTER));
        } catch (RemoteException e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("----getPrinters RemoteException----" + e.toString());
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("----getPrinters exception----" + e.toString());
        }
        return null;
    }

    /**
     * 打印机初始化流程修改，首先调用setPrintConfig进行打印机配置
     */
    private boolean setPrintConfig() {
        boolean status = true;
        try {
            // tell sdk the widthmode(58mm or 80mm) of printer in use.
            Bundle bundle = new Bundle();
            getPrnWidthBySetting();
            bundle.putString(PrinterConsts.PRT_CFG_KEY_WIDTH_MODE, prnWidth);
            printer.setConfig(bundle);
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            status = false;
            LoggerUtils.getLogger().debug("-------------initPrinter exception----------" + e.toString());
            e.printStackTrace();
        }
        return status;
    }

    public int printStatus() {
        int result = 0;
        try {
            result = printer.checkStatus();
        } catch (RemoteException e) {
            result = PrinterConsts.STATUS_ERR_UNKOWN;
            LoggerUtils.getLogger().debug("-------------printStatus exception----------" + e.toString());
        } catch (NullPointerException e) {
            result = PrinterConsts.STATUS_ERR_UNKOWN;
            LoggerUtils.getLogger().debug("-------------NullPointerException exception----------" + e.toString());
        } catch (Exception e) {
            result = PrinterConsts.STATUS_ERR_UNKOWN;
            LoggerUtils.getLogger().debug("-------------Exception exception----------" + e.toString());
        }
        return result;
    }

    private static void getPrnWidthBySetting() {
        //prnWidth = PrinterConsts.PRT_CFG_VALUE_WIDTH_MODE_80;
        prnWidth = PrinterConsts.PRT_CFG_VALUE_WIDTH_MODE_58;
    }

    public void beginPrintTrans() {
        try {
            printer.beginTranscation();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    public void startPrint() {
        int transId = 0;
        try {
            transId = printer.endTranscation();
            printer.commit(transId);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    public boolean printImage(Bitmap bitmap) {
        int result = 0;
        boolean status = true;
        try {
            printer.addImage(bitmap);
        } catch (RemoteException e) {
            e.printStackTrace();
            result = 1;
        }
        return result == 0;
    }

    public boolean printImageV2(Bitmap bitmap) {
        int result = 0;
        try {
            printer.addImage(bitmap);
            printer.addFeedLine(feedLine);
            printer.addCut();
        } catch (RemoteException e) {
            e.printStackTrace();
            result = 1;
        }
        return result == 0;
    }

    public boolean printTextStr(String text, int pattern) {
        int result = 0;
        try {
            printer.addText(text + "\n", pattern);
        } catch (RemoteException e) {
            result = 1;
        }
        return result == 0;
    }

    public boolean printTextStrV2(String text, int pattern) {
        int result = 0;
        try {
            printer.addText(text + "\n", pattern);
            printer.addFeedLine(feedLine);
            printer.addCut();
        } catch (RemoteException e) {
            result = 1;
        }
        return result == 0;
    }

    public boolean printBar(String barcode) {
        int result = 0;
        try {
            printer.addBarcode(barcode);
        } catch (RemoteException e) {
            e.printStackTrace();
            result = 1;
        }
        return result == 0;
    }

    public boolean printBarV2(String barcode) {
        int result = 0;
        try {
            printer.addBarcode(barcode);
            printer.addFeedLine(feedLine);
            printer.addCut();
        } catch (RemoteException e) {
            e.printStackTrace();
            result = 1;
        }
        return result == 0;
    }

    public boolean printQR(String barcode) {
        int result = 0;
        try {
            printer.addQRCode(barcode, 12);
        } catch (RemoteException e) {
            e.printStackTrace();
            result = 1;
        }
        return result == 0;
    }

    public boolean printQRV2(String barcode) {
        int result = 0;
        try {
            printer.addQRCode(barcode, 12);
            printer.addFeedLine(feedLine);
            printer.addCut();
        } catch (RemoteException e) {
            result = 1;
            LoggerUtils.getLogger().debug("-------------hs650 printQRV2 exception----------" + e.toString());
        }
        return result == 0;
    }

    public void feedPaper(int line) {
        try {
            printer.addFeedLine(line);
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 feedPaper exception----------" + e.toString());
        }
    }

    public void feedPaper() {
        try {
            printer.addFeedLine(feedLine);
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 feedPaper feedLine exception----------" + e.toString());
        }
    }

    public void feedPaperV2() {
        try {
            printer.addFeedLine(feedLine);
            printer.addCut();
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 feedPaperV2 exception----------" + e.toString());
        }
    }

    public void cutPaper() {
        try {
            printer.addCut();
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 cutPaper exception----------" + e.toString());
        }
    }
}