/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.layout;

import android.app.Activity;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;

import com.hisense.common.util.LoggerUtils;

import java.lang.reflect.Method;

/**
 * 软键盘输入。
 *
 * @author zhangsai 2018-01-18 HIPOSAPP-286
 */
public class SoftInput {

    private SoftInput() {
        // NO CODE HERE.
    }

    /**
     * 禁用 {@code editText} 指定编辑器的系统软键盘输入。
     *
     * @param editText 文本编辑器
     */
    public static void disable(EditText editText) {
        disableSoftInputMethod(editText, "setShowSoftInputOnFocus");
        disableSoftInputMethod(editText, "setSoftInputShownOnFocus");
    }

    private static void disableSoftInputMethod(EditText editText, String methodName) {
        try {
            Method method = EditText.class.getMethod(methodName, boolean.class);
            method.setAccessible(true);
            method.invoke(editText, false);
        } catch (Exception e) {
        }
    }

    /**
     * 关闭系统软键盘
     *
     * @param context 当前Activity
     */
    public static void closeSoftKeyBoard(Activity context) {
        View view = context.getWindow().peekDecorView();
        if (view != null) {
            InputMethodManager imm = (InputMethodManager) context.getSystemService(Activity.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

    /**
     * 关闭系统软键盘
     *
     * @param context 当前Activity
     */
    public static void closeSoftKeyBoard2(Activity context, EditText edit) {
        InputMethodManager imm = (InputMethodManager) context.getSystemService(Activity.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(edit.getWindowToken(), 0);
    }

    /**
     * 打开系统软键盘
     *
     * @param context
     */
    public static void openSoftKeyBoard(Activity context) {
        InputMethodManager imm = (InputMethodManager) context.getSystemService(Activity.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_NOT_ALWAYS);
    }
}
