/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.layout;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

/**
 * 用于控制控件可见性的监视器。
 *
 * @author xieliyan 2018-02-28 HIPOSAPP-308 销售页面商品搜索框增加扫码按钮
 */
public class SwitchViewWatcher implements TextWatcher {

    private View defaultView;
    private View hiddenView;
    private int visibility;
    private int hiddenViewVisibility = View.VISIBLE;

    /**
     * 初始化一个 {@code SwitchViewWatcher} 的实例。
     *
     * @param defaultView 默认view
     */
    public SwitchViewWatcher(View defaultView) {
        this.defaultView = defaultView;
        this.changeVisibility(null);
    }

    /**
     * 初始化一个 {@code SwitchViewWatcher} 的实例。
     *
     * @param defaultView 默认view
     * @param hiddenView  隐藏view
     */
    public SwitchViewWatcher(View defaultView, View hiddenView) {
        this.defaultView = defaultView;
        this.hiddenView = hiddenView;
        this.switchView(null);
    }

    /**
     * 初始化一个 {@code SwitchViewWatcher} 的实例。
     *
     * @param defaultView 默认view
     * @param hiddenView  隐藏view
     * @param visibility
     */
    public SwitchViewWatcher(View defaultView, View hiddenView, int visibility) {
        this.defaultView = defaultView;
        this.hiddenView = hiddenView;
        hiddenViewVisibility = visibility;
        this.switchView(null);
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {
        if (hiddenView == null) {
            this.changeVisibility(s);
        } else {
            this.switchView(s);
        }
    }

    @Override
    public void afterTextChanged(Editable s) {

    }

    /**
     * 根据字符串更新按钮图标。字符串长度大于 0 时显示清除按钮，其它情况下扫一扫按钮。
     *
     * @param s 用于判断的字符串
     */
    private void switchView(CharSequence s) {
        if (s != null && s.length() > 0) {
            defaultView.setVisibility(View.VISIBLE);
            hiddenView.setVisibility(View.GONE);
        } else {
            defaultView.setVisibility(View.GONE);
            if (View.VISIBLE == hiddenViewVisibility) {
                hiddenView.setVisibility(View.VISIBLE);
            }
        }
    }

    /**
     * 根据字符串判断是否显示清除按钮。字符串长度大于 0 时显示清除按钮，其它情况下显示。
     *
     * @param s 用于判断的字符串
     */
    private void changeVisibility(CharSequence s) {
        visibility = (s != null && s.length() > 0) ? View.VISIBLE : View.GONE;
        defaultView.setVisibility(visibility);
    }

    /**
     * 更新控件可见性
     */
    public void update() {
        defaultView.setVisibility(visibility);
    }
}
