package com.hisense.hiaposcmp.popup;

import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.os.Handler;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.PopupWindow;

import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.adapter.BagPopupAdapter;
import com.hisense.hiaposcmp.bean.BagBean;

import java.util.List;

/**
 * 弹出窗口
 * Created by zhuguangsheng on 2018/7/2.
 */
public class BagPopupWindow extends PopupWindow {
    private int popupWidth;
    private int popupHeight;
    private ListView listView;
    private View parentView;
    BagPopupAdapter popListViewAdapter;
    private ItemClickCallback mItemClickCallback;

    public interface ItemClickCallback {
        void clickBarcode(String barcode);
    }

    public BagPopupWindow(Context context, List<BagBean> bagBeans, ItemClickCallback itemClickCallback) {
        super(context);
        initView(context);
        //setPopConfig();
        initData(context, bagBeans);
        setPopConfig();
        mItemClickCallback = itemClickCallback;
    }

    private void initData(Context context, List<BagBean> bagBeans) {
        //给ListView添加数据
        popListViewAdapter = new BagPopupAdapter(context, bagBeans, this);
        listView.setAdapter(popListViewAdapter);
    }

    private void initView(Context context) {
        parentView = View.inflate(context, R.layout.popup_bags, null);
        listView = parentView.findViewById(R.id.lv_bags);
        setContentView(parentView);

        listView.setOnItemClickListener((parent, view, position, id) -> {
            try {
                String barcode = popListViewAdapter.getItem(position).getBarCode();
                //EventBus.getDefault().post(new BaseEventBean(EventActions.SCAN_GOT_BARCODE, barcode));
                dismissMe();
                mItemClickCallback.clickBarcode(barcode);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void setPopConfig() {
        this.setWidth(200);
        this.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);
        // 设置弹出窗体可点击
        this.setFocusable(true);
        ColorDrawable dw = new ColorDrawable(0xEEFFFFFF);
        this.setBackgroundDrawable(dw);
        //设置外部触摸会关闭窗口
        this.setOutsideTouchable(true);

        /**
         * 获取自身的长宽高
         */
        new Handler().postDelayed(() -> {
            parentView.measure(View.MeasureSpec.UNSPECIFIED, View.MeasureSpec.UNSPECIFIED);
            popupHeight = parentView.getMeasuredHeight();
            popupWidth = parentView.getMeasuredWidth();
        }, 800);
    }

    public ListView getListView() {
        return listView;
    }

    /**
     * 设置显示在v上方(以v的左边距为开始位置)
     *
     * @param v
     */
    public void showUp(View v) {
        //获取需要在其上方显示的控件的位置信息
        int[] location = new int[2];
        v.getLocationOnScreen(location);
        //在控件上方显示
        //showAtLocation(v, Gravity.NO_GRAVITY, (location[0]) - popupWidth / 2, location[1] - popupHeight);
        if (popListViewAdapter.getCount() > 2) {
            popupHeight = 250;
        } else if (popListViewAdapter.getCount() == 2) {
            popupHeight = 180;
        } else {
            popupHeight = 100;
        }

        showAtLocation(v, Gravity.NO_GRAVITY, (location[0]) - popupWidth, location[1] - popupHeight);
        //showAsDropDown(v, 20, -1*popupHeight);
    }

    public void dismissMe() {
        this.dismiss();
    }
}