/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.pos.icbc;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;

import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.pos.payment.icbc.bean.StOrgItem;
import com.hisense.pos.payment.icbc.bean.StOrgItemTransItem;
import com.hisense.pos.payment.icbc.bean.StQRCodeItem;
import com.hisense.pos.payment.icbc.bean.StResultOuter;
import com.icbc.smartpos.transservice.aidl.ITransService;
import com.icbc.smartpos.transservice.aidl.TransHandler;

import org.apache.commons.httpclient.util.DateUtil;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Locale;

/**
 * 工行收单业务处理类
 *
 * @author lizhixin 2018-06-14 HIPOSAPP-520 对接工行收单通道
 * @author lizhixin 2018-07-12 HIPOSAPP-533 对接工行交易汇总并支持打印
 */
public class ICBCHelper {

    /**
     * 消费相关
     */
    public static final String TRANS_TYPE_MULTI_PURCHASE = "MULTI_PURCHASE";//收银
    public static final String TRANS_LIST_PURCHASE = "PURCHASE";//银行卡消费
    public static final String TRANS_LIST_QRPURCHASE = "QRPURCHASE";//二维码消费，微信支付宝银联app二维码
    public static final String TRANS_LIST_INTEGRAL_PURCHASE = "INTEGRAL_PURCHASE";//积分消费
    public static final String TRANS_LIST_INSTALLMENT = "INSTALLMENT";//分期付款
    /**
     * 撤销、退货、查询、结账
     */
    public static final String TRANS_TYPE_INQUERY = "INQUERY";//余额查询
    public static final String TRANS_TYPE_INTEGRAL_QUERY = "INTEGRAL_QUERY";//积分查询
    public static final String TRANS_TYPE_POS_VOID = "POS_VOID";//当日撤销 银行卡
    public static final String TRANS_TYPE_REFUND = "REFUND";//隔日退货
    public static final String TRANS_TYPE_INTEGRAL_REFUND = "INTEGRAL_REFUND";//积分退货 银行卡
    public static final String TRANS_TYPE_QRREFUND = "QRREFUND";//二维码退货
    public static final String TRANS_TYPE_PREAUTH = "PREAUTH";//预授权
    public static final String TRANS_TYPE_QUERY_TRANS_REC = "QUERY_TRANS_REC";//查本机交易记录
    public static final String TRANS_TYPE_PRINT = "START_PRINT";//打印
    public static final String TRANS_TYPE_REPRINT = "REPRINT";//重打印签购单
    public static final String TRANS_TYPE_SETTLEMENT = "SETTLEMENT";//结账
    public static final String TRANS_TYPE_STATISTICS = "STATISTICS";//交易汇总

    private static ICBCHelper icbcHelper;
    private Context mContext;//应用程序环境

    /**
     * 工行收单服务
     */
    private ITransService itransService;
    private ServiceConnection connection = new ServiceConnection() {
        @Override
        public void onServiceDisconnected(ComponentName name) {
            //CommonTools.toastShort("工行收单服务中断");
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            itransService = ITransService.Stub.asInterface(service);
            //CommonTools.toastShort("工行收单服务绑定成功");
        }
    };

    private ICBCHelper() {
    }

    public static ICBCHelper getInstance() {
        if (icbcHelper == null) {
            synchronized (ICBCHelper.class) {
                if (icbcHelper == null) {
                    icbcHelper = new ICBCHelper();
                }
            }
        }
        return icbcHelper;
    }

    public ICBCHelper init(Context context) {
        mContext = context;
        return icbcHelper;
    }

    /**
     * 银行卡消费
     */
    public void purchase(String purchaseType, long amount, TransHandler handler) {

        Bundle ctrlData = getCtrlData();

        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", amount);
//        transData.putLong("INS_TIMES", 10);//分期付款期数，可以在工行收单应用内设置

        if (purchaseType != null) {
            //如果有消费类型则放入
            ArrayList<String> f = new ArrayList<>();
            f.add(purchaseType);
            transData.putStringArrayList("TRANS_LIST", f);
        }

        //打印第二联将持卡人签名改为“右对齐大字体”屏蔽下面三行第一联和第二联相同
        /*ArrayList<String> g = new ArrayList<>();
        g.add("{\"type\":1,\"text\":[{\"align\":2,\"font\":2,\"data\":\"右对齐大字体\"}]}");
        transData.putStringArrayList("ADDITIONAL_PRINT", g);*/

        LoggerUtils.getLogger().debug("工行消费传递参数：");
        LoggerUtils.getLogger().debug(ctrlData.toString());
        LoggerUtils.getLogger().debug(transData.toString());

        //调用消费服务
        trans(TRANS_TYPE_MULTI_PURCHASE, ctrlData, transData, handler);
    }

    /**
     * 当日撤销
     *
     * @param refNo   原交易返回的REF_NO
     * @param handler
     */
    public void cancel(String refNo, TransHandler handler) {

        Bundle ctrlData = getCtrlData();

        Bundle transData = new Bundle();
        transData.putString("OLD_REF_NO", refNo);
        //打印第二联将持卡人签名改为条形码，（条形码内容为1234567）屏蔽下面三行第一联和第二联相同
        /*ArrayList<String> g = new ArrayList<>();
        g.add("{\"type\":3,\"barcode\":{\"width\":280,\"height\":60,\"data\":\"1234567\"}}");
        transData.putStringArrayList("ADDITIONAL_PRINT", g);*/

        LoggerUtils.getLogger().debug("工行当日撤销传递参数：");
        LoggerUtils.getLogger().debug(ctrlData.toString());
        LoggerUtils.getLogger().debug(transData.toString());

        trans(TRANS_TYPE_POS_VOID, ctrlData, transData, handler);
    }

    /**
     * 二维码退货
     *
     * @param amount      金额
     * @param qrcodeOrder 原 交 易 返 回 的 QRCODE_ORDER
     * @param lastCardno  卡号后四位。该字段不传则要求在工银收单内输入，对于微信支付宝等不存在该字段信息的请输入""
     */
    public void scanRefund(long amount, String qrcodeOrder, String lastCardno, TransHandler handler) {

        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", amount);
        transData.putString("OLD_QRCODE_ORDER", qrcodeOrder);
        transData.putString("LAST_OF_CARDNO", lastCardno);

        LoggerUtils.getLogger().debug("工行二维码退货传递参数：");
        LoggerUtils.getLogger().debug(transData.toString());

        trans(TRANS_TYPE_QRREFUND, getCtrlData(), transData, handler);
    }

    /**
     * 使用工行收单系统隔日退货
     *
     * @param amount    金额
     * @param refNo     原交易的检索参考号
     * @param termNo    原交易的终端号
     * @param transDate 原交易的交易日期，格式：yyyyMMdd
     */
    public void refund(long amount, String refNo, String termNo, String transDate, TransHandler handler) {
        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", amount);
        transData.putString("OLD_REF_NO,", refNo);
        transData.putString("OLD_TERM_NO,", termNo);
        transData.putString("OLD_TRANS_DATE,", transDate);
        trans(TRANS_TYPE_REFUND, getCtrlData(), transData, handler);

        LoggerUtils.getLogger().debug("工行隔日退货传递参数：");
        LoggerUtils.getLogger().debug(transData.toString());

    }

    /**
     * 使用工行收单系统银行卡积分退货
     *
     * @param amount    金额
     * @param refNo     原交易的检索参考号
     * @param termNo    原交易的终端号
     * @param transDate 原交易的交易日期，格式：yyyyMMdd
     */
    public void cancelPoint(long amount, String refNo, String termNo, String transDate, TransHandler handler) {
        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", amount);
        transData.putString("OLD_REF_NO,", refNo);
        transData.putString("OLD_TERM_NO,", termNo);
        transData.putString("OLD_TRANS_DATE,", transDate);
        trans(TRANS_TYPE_INTEGRAL_REFUND, getCtrlData(), transData, handler);

        LoggerUtils.getLogger().debug("工行银行卡积分退货传递参数：");
        LoggerUtils.getLogger().debug(transData.toString());

    }

    /**
     * 打印 以图片格式
     *
     * @param path    图片路径
     * @param width   图片宽
     * @param height  图片高
     * @param handler
     */
    public void print(String path, int width, int height, TransHandler handler) {

        try {
            InputStream in = new FileInputStream(new File(path));
            byte[] buffer = new byte[in.available()];
            in.read(buffer);
            in.close();
            JSONArray jarry = new JSONArray();
            for (int i = 0; i < buffer.length; i++) {
                jarry.put((int) buffer[i]);
            }
            JSONObject jimage = new JSONObject();
            JSONObject jobject = new JSONObject();
            jimage.put("offset", 0);
            jimage.put("width", width);
            jimage.put("height", height);
            jimage.put("data", jarry);
            jobject.put("type", 2);
            jobject.put("image", jimage);
            String imageString = jobject.toString();

            ArrayList<String> printData = new ArrayList<>();
            printData.add(imageString);

            Bundle transData = new Bundle();
            transData.putStringArrayList("PRINT_DATA", printData);

            LoggerUtils.getLogger().debug("工行打印接口传递参数：图片宽高 " + width + " " + height);

            trans(TRANS_TYPE_PRINT, getCtrlData(), transData, handler);
        } catch (JSONException e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug(e.getMessage());
        } catch (IOException e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug(e.getMessage());
        }
    }

    /**
     * 打印 以文本格式 暂时用于交易汇总的打印
     *
     * @param resultOuter 交易汇总接口返回的数据对象
     * @param custName    商户名称
     * @param custNo      商户编号
     * @param termNo      终端号
     * @param handler     handler
     */
    public void print(StResultOuter resultOuter, String custName, String custNo, String termNo, TransHandler handler) {

        try {
            Bundle transData = new Bundle();
            transData.putStringArrayList("PRINT_DATA", createPrintStrList(resultOuter, custName, custNo, termNo));

            LoggerUtils.getLogger().debug("工行打印交易汇总接口传递参数： " + custNo + " " + termNo);

            trans(TRANS_TYPE_PRINT, getCtrlData(), transData, handler);

        } catch (JSONException e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug(e.getMessage());
        }
    }

    /**
     * 重打印签购单 无参数时一定打印末笔交易的签购单
     *
     * @param refNo   交易检索参考号
     * @param handler handler
     */
    public void rePrint(String refNo, TransHandler handler) {
        Bundle transData = new Bundle();
//        transData.putString("OLD_REF_NO,", refNo);
        trans(TRANS_TYPE_REPRINT, getCtrlData(), transData, handler);

        LoggerUtils.getLogger().debug("工行重打印签购单传递参数：");
        LoggerUtils.getLogger().debug(transData.toString());

    }

    /**
     * 交易汇总 默认汇总当天数据
     *
     * @param batchNum 批次号   可选
     *                 三个都不传则汇总本应用的所有交易数据
     * @param handler  handler
     */
    public void statistics(String batchNum, TransHandler handler) {
        Bundle transData = new Bundle();

        transData.putString("START_DATE", DateUtils.formatDateToday());
        transData.putString("END_DATE", DateUtils.formatDateToday());
        transData.putString("BATCH_NUM,", batchNum);
        trans(TRANS_TYPE_STATISTICS, getCtrlData(), transData, handler);

        LoggerUtils.getLogger().debug("工行交易汇总传递参数：");
        LoggerUtils.getLogger().debug(transData.toString());
    }

    /**
     * 结账
     *
     * @param handler handler
     */
    public void settlement(TransHandler handler) {
        trans(TRANS_TYPE_SETTLEMENT, getCtrlData(), new Bundle(), handler);
    }

    /**
     * 余额查询
     *
     * @param handler handler
     */
    public void inQuery(TransHandler handler) {
        trans(TRANS_TYPE_INQUERY, getCtrlData(), new Bundle(), handler);
    }

    /**
     * 预授权
     */
    public void preauth(TransHandler handler) {
        String transType = "PREAUTH";

        Bundle ctrlData = new Bundle();
        ctrlData.putString("APP_NAME", "demo");
        ctrlData.putString("AIDL_VER", "V0.0.11");

        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", 10L);

        ArrayList<String> g = new ArrayList<>();
        g.add("{\"type\":4,\"qrcode\":{\"length\":280,\"offset\":52,\"data\":\"12345678\"}}");
        transData.putStringArrayList("ADDITIONAL_PRINT", g);

        trans(transType, ctrlData, transData, handler);

    }

    /**
     * 调用工行服务
     *
     * @param type      服务类型
     * @param ctrlData  控制参数
     * @param transData 交易参数
     * @param handler   回调函数
     */
    private void trans(String type, Bundle ctrlData, Bundle transData, TransHandler handler) {
        try {
            if (itransService != null) {
                itransService.startTrans(type, ctrlData, transData, handler);
            } else {
                CommonTools.toastShort("未发现指定服务，请检查工行paybank是否安装");
            }
        } catch (Exception e) {
            e.printStackTrace();
            CommonTools.toastShort("工行服务调用异常：" + e.getLocalizedMessage());
            LoggerUtils.getLogger().debug("工行服务调用异常：" + e.getLocalizedMessage());
        }
    }

    private Bundle getCtrlData() {
        Bundle ctrlData = new Bundle();
        ctrlData.putString("APP_NAME", Config.APP_NAME);
        ctrlData.putString("AIDL_VER", Config.VER_ICBC_AIDL);
        return ctrlData;
    }

    /**
     * 绑定工行收单服务
     */
    public void bindICBCService() {
        Intent intent = new Intent();
        intent.setAction("com.icbc.smartpos.transservice.TransService");
        intent.setPackage("com.icbc.smartpos.bankpay");
        mContext.bindService(intent, connection, Context.BIND_AUTO_CREATE);
    }

    /**
     * 解绑工行收单服务
     */
    public void unBindICBCService() {
        mContext.unbindService(connection);
    }

    /**
     * 根据交易汇总接口返回的数据组装打印文本集合
     *
     * @param resultOuter 交易汇总接口返回的数据
     * @param custName    商户名称
     * @param custNo      商户编号
     * @param termNo      终端号
     * @return ArrayList<String>
     * @throws JSONException JSON解析错误
     */
    private ArrayList<String> createPrintStrList(StResultOuter resultOuter, String custName, String custNo, String termNo) throws JSONException {
        ArrayList<String> printData = new ArrayList<>();
        if (resultOuter != null) {

            //开头空行
            printData.add("{\"type\":1,\"text\":[{\"data\":\"\"}]}");

            //大标题 大字号 居中
            printData.add("{\"type\":1,\"text\":[{\"font\":2,\"align\":1,\"data\":\"交易汇总\"}]}");

            //小标题
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"商户名称 %s\"}]}", custName));
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"商户编号 %s\"}]}", custNo));
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"终端编号 %s\"}]}", termNo));
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"打印时间 %s\"}]}", DateUtils.formatDateNow()));
            printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
            printData.add("{\"type\":1,\"text\":[{\"data\":\"内卡        笔数        金额\"}]}");

            if (resultOuter.org != null && resultOuter.org.size() > 0) {
                // 银行卡交易数据
                for (StOrgItem orgItem : resultOuter.org) {
                    if ("CUP".equals(orgItem.org_name)) {
                        // 只统计中国银联的
                        int totalCnt = 0;//合计笔数
                        if (orgItem.trans != null) {
                            for (StOrgItemTransItem transItem : orgItem.trans) {

                                if ("PURCHASE".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"消费        %d         %s\"}]}", transItem.trans_cnt, StringUtils.changeF2Y(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("POS_VOID".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"撤销        %d         %s\"}]}", transItem.trans_cnt, StringUtils.changeF2Y(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("REFUND".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"退货        %d         %s\"}]}", transItem.trans_cnt, StringUtils.changeF2Y(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("INTEGRAL_PURCHASE".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"积分        %d         %s\"}]}", transItem.trans_cnt, StringUtils.changeF2Y(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("INSTALLMENT".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"分期        %d         %s\"}]}", transItem.trans_cnt, StringUtils.changeF2Y(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("INTEGRAL_REFUND".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"积分退货    %d         %s\"}]}", transItem.trans_cnt, StringUtils.changeF2Y(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                }
                            }
                        }

                        //总计
                        printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
                        printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"合计        %d         %s\"}]}", totalCnt, StringUtils.changeF2Y(orgItem.org_amt)));
                        printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
                    }
                }
            }

            if (resultOuter.qrcode != null && resultOuter.qrcode.size() > 0) {
                // 二维码交易数据
                StQRCodeItem qrCodeItem = resultOuter.qrcode.get(0);

                printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"二维码消费   %d         %s    \"}]}", qrCodeItem.qr_cnt, StringUtils.changeF2Y(qrCodeItem.qr_amt)));
                printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"二维码退货   %d         %s    \"}]}", qrCodeItem.qr_refund_cnt, StringUtils.changeF2Y(qrCodeItem.qr_refund_amt)));
                printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
            }

            //结尾空行
            printData.add("{\"type\":1,\"text\":[{\"data\":\"----------x----------x----------\"}]}");
            printData.add("{\"type\":1,\"text\":[{\"data\":\"\"}]}");
            printData.add("{\"type\":1,\"text\":[{\"data\":\"\"}]}");
        }

        return printData;
    }

}
