/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.pos.payment;

import android.app.Activity;

import com.hisense.hipos.payment.PayMethod;
import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.ums.UmsAlipayProvider;
import com.hisense.pos.payment.ums.UmsCouponProvider;
import com.hisense.pos.payment.ums.UmsMobilePayProvider;
import com.hisense.pos.payment.ums.UmsPrepayProvider;
import com.hisense.pos.payment.ums.UmsUnionPayProvider;
import com.hisense.pos.payment.ums.UmsVipCard1Provider;
import com.hisense.pos.payment.ums.UmsVipCard2Provider;
import com.hisense.pos.payment.ums.UmsWalletProvider;
import com.hisense.pos.payment.ums.UmsWechatPayProvider;

/**
 * 银联商务支付工厂类。
 *
 * @author zhangsai 2018-11-27 HIPOSAPP-888
 * @author guolina  2020-03-10 HIAPOSCMP-577  邹平圣豪-易收银的微信、支付宝两个支付方式合并为一个
 */
public final class UmsPaymentFactory {

    /**
     * 如果支持 {@code payMethodRemark} 表示的支付方式，则返回一个新的 {@code IPaymentProvider} 实例；否则，
     * 返回 null。
     *
     * @param activity        Activity
     * @param payMethodRemark 支付方式备注
     * @return {@code IPaymentProvider} 实例
     */
    public static IPaymentProvider createProvider(Activity activity, String payMethodRemark) {
        IPaymentProvider paymentProvider = null;

        if (PayMethod.isUnionPay(payMethodRemark)) {
            paymentProvider = new UmsUnionPayProvider(activity);
        } else if (PayMethod.isMobilePay(payMethodRemark)) {
            paymentProvider = new UmsMobilePayProvider(activity);
        }else if (PayMethod.isAliPay(payMethodRemark)) {
            paymentProvider = new UmsAlipayProvider(activity);
        } else if (PayMethod.isWxPay(payMethodRemark)) {
            paymentProvider = new UmsWechatPayProvider(activity);
        } else if (PayMethod.isUnionPrepayCard(payMethodRemark)) {
            paymentProvider = new UmsPrepayProvider(activity);
        } else if (PayMethod.isUnionCoupon(payMethodRemark)) {
            paymentProvider = new UmsCouponProvider(activity);
        } else if (PayMethod.isUnionWallet(payMethodRemark)) {
            paymentProvider = new UmsWalletProvider(activity);
        } else if (PayMethod.isUnionVipCard1(payMethodRemark)) {
            paymentProvider = new UmsVipCard1Provider(activity);
        } else if (PayMethod.isUnionVipCard2(payMethodRemark)) {
            paymentProvider = new UmsVipCard2Provider(activity);
        }

        return paymentProvider;
    }
}
