package com.hisense.hiaposcmp.pos.scanner;

import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.widget.RelativeLayout;

import androidx.annotation.Nullable;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.BaseMvpActivity;
import com.hisense.hiaposcmp.anno.ActivityLayoutInject;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.presenter.pay.PayScanPresenter;
import com.hisense.hiaposcmp.tool.ScanKeyEventHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.pay.IPayScanView;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * 标准版扫码页
 *
 * @author wangyaogang 2020-02-14
 */
@ActivityLayoutInject(R.layout.activity_std_scan)
public class ScanStdActivity extends BaseMvpActivity<IPayScanView, PayScanPresenter> implements IPayScanView, ScanKeyEventHelper.OnScanSuccessListener {


    @BindView(R.id.rl_cancel)
    RelativeLayout rlCancel;

    private ScanKeyEventHelper mScanKeyEventHelper;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    protected PayScanPresenter createPresenter() {
        return new PayScanPresenter();
    }


    @OnClick(R.id.imvAliScanPayQuit)
    public void onClickQuit() {
        finish();
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        if (event.getKeyCode() == KeyEvent.KEYCODE_ENTER && isFastClick()) {
            //屏蔽连扫的第二个条码
            LoggerUtils.getLogger().info("屏蔽条码:" + event.getKeyCode());
        } else {
            mScanKeyEventHelper.analysisKeyEvent(event);
        }

        return super.dispatchKeyEvent(event);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mScanKeyEventHelper.onDestroy();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mScanKeyEventHelper = new ScanKeyEventHelper(this);
    }

    protected void onConfirm(String barcode) {
        Intent intent = new Intent();
        intent.putExtra("barcode", barcode);
        ScanStdActivity.this.setResult(ActivityConstants.RESULT_PAY_TO_SCAN, intent);
        ScanStdActivity.this.finish();
        finish();
    }

    @Override
    public void onScanSuccess(String result) {
        CommonUtils.startBeepBgm(this, result);

        LoggerUtils.getLogger().debug("ScanStdActivity_onScanSuccess:" + result);
        Intent resultIntent = new Intent();
        Bundle bundle = new Bundle();
        bundle.putString(CameraScanner.SCAN_RESULT_KEY, result);
        resultIntent.putExtras(bundle);
        this.setResult(RESULT_OK, resultIntent);
        finish();
    }

    @OnClick(R.id.rl_cancel)
    public void onViewClicked() {
        finish();
    }
}
