package com.hisense.hiaposcmp.presenter;

import com.hisense.common.boot.SingleCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.service.miya.MiYaService;
import com.hisense.hiaposcmp.service.sale.SaleRepository;
import com.hisense.hiaposcmp.service.yst.YstService;
import com.hisense.hiaposcmp.view.IView.IBaseView;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.yst.YstConfig;
import com.hisense.webapi.Callback;

import java.lang.ref.Reference;
import java.lang.ref.WeakReference;

/**
 * @author zhuguangsheng on 2018/5/25.
 * @author dukunbiao(null)  2019-07-22  重构
 */
public class BasePresenter<V extends IBaseView> {

    private Reference<V> mViewRef;
    protected V mMvpView;

    protected YstService mYstService;
    protected MiYaService mMiYaService;
    protected SaleRepository mSaleRepository;
    protected boolean isEcard = false;

    public MainSaleModel getMainSaleModel() {
        return MainSaleModel.getInstance();
    }

    public SaleModel getSale() {
        if (MainSaleModel.getInstance().getSaleModel() == null) {
            return new SaleModel();
        }

        return MainSaleModel.getInstance().getSaleModel();
    }

    public ReturnSaleModel getReturnSale() {
        if (MainSaleModel.getInstance().getReturnSaleModel() == null) {
            return new ReturnSaleModel();
        }
        return MainSaleModel.getInstance().getReturnSaleModel();
    }


    public void initYstConfig() {
        // 赢商通配置
        com.hisense.hipos.webapi.yst.YstService.getConfig(new Callback<YstConfig>() {
            @Override
            public void onSuccess(YstConfig config) {
                mYstService = new YstService(Config.org_code, config);
            }

            @Override
            public void onError(Throwable throwable) {
                throwable.printStackTrace();
            }
        });
    }

    public void initMiYaConfig() {
        mMiYaService = new MiYaService(Config.org_code);
    }

    public void initSaleRepository() {
        mSaleRepository = new SaleRepository(mMvpView.getContext());
    }

    /**
     * 查询交易凭证号traceNo
     */
    public String getTraceNo(String saleNo, int serialNo) {
        return mSaleRepository.getTraceNoBySerialNo(saleNo, serialNo);
    }

    /**
     * 查询终端号termNo
     */
    public String getTermNo(String saleNo, int serialNo) {
        return mSaleRepository.getTermNoBySerialNo(saleNo, serialNo);
    }

    public String getTradeAndMchId(String saleNo, int serialNo) {
        return mSaleRepository.getTradeNoAndMchIdBySerialNo(saleNo, serialNo);
    }

    /**
     * 查询商户订单号outTradeNo
     */
    public String getOutTradeNo(String saleNo, int serialNo) {
        String tradeNoAndMchId = getTradeAndMchId(saleNo, serialNo);
        if (!StringUtils.isEmpty(tradeNoAndMchId)) {
            return tradeNoAndMchId.substring(0, tradeNoAndMchId.indexOf(","));
        }
        return "";
    }

    /**
     * 查询商户号mchId
     */
    public String getMchId(String saleNo, int serialNo) {
        String tradeNoAndMchId = getTradeAndMchId(saleNo, serialNo);
        if (!StringUtils.isEmpty(tradeNoAndMchId)) {
            return tradeNoAndMchId.substring(tradeNoAndMchId.indexOf(",") + 1);
        }
        return "";
    }

    /**
     * 建立关联
     */
    public void attachView(V view) {
        mViewRef = new WeakReference<>(view);
        if (isViewAttached()) {
            mMvpView = getView();
        } else {
            throw new IllegalStateException("view not attached");
        }
    }

    /**
     * 解除关联
     */
    public void detachView() {
        if (mViewRef != null) {
            mViewRef.clear();
            mViewRef = null;
            mMvpView = null;
        }
    }

    /**
     * 获取View
     *
     * @return View
     */
    public V getView() {
        return mViewRef.get();
    }

    /**
     * UI展示相关的操作需要判断一下 Activity 是否已经 finish.
     *
     * @return boolean
     */
    public boolean isViewAttached() {
        return mViewRef != null && mViewRef.get() != null;
    }

    public void hideLoadingDialog() {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.hideLoadingDialog();
    }

    public void showLoadingDialog() {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showLoadingDialog();
    }

    public void showHintDialog(String msg) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintDialog(msg);
    }

    public void showHintDialog(String title, String msg) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintDialog(title, msg);
    }

    public void showHintDialog(String msg, SingleCallback callback) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintDialog(msg, callback);
    }

    public void showHintDialog(String title, String msg, boolean isExit) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintDialog(title, msg, isExit);
    }

    public void showHintConfirm(String msg, UniCallback callback) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintConfirm(msg, callback);
    }

    public void showHintConfirm(String title, String msg, UniCallback callback) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintConfirm(title, msg, callback);
    }

    public void showHintConfirm(String title, String msg, String yesBtn, String noBtn, UniCallback callback) {
        if (!isViewAttached()) {
            return;
        }

        mMvpView.showHintConfirm(title, msg, yesBtn, noBtn, callback);
    }
}
