package com.hisense.hiaposcmp.presenter;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.util.MidConfigUtils;
import com.hisense.hiaposcmp.view.IView.IBootUpView;
import com.hisense.hipos.webapi.config.ConfigResultData;
import com.hisense.hipos.webapi.config.ConfigService;
import com.hisense.webapi.Callback;
import com.hisense.webapi.WebApi;

/**
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-22  重构
 * @author xieliyan         2019-12-24  HIAPOSCMP-446  启动页进度条加载时间
 * @author guolina          2020-03-17  HIAPOSCMP-594  修复启动页无响应时点击返回因view为空导致的闪退
 */
public class BootUpPresenter extends BasePresenter<IBootUpView> {

    public int fetchMidConfig() {
        long beforeCall = System.currentTimeMillis();
        try {
            String baseUrl = Config.getBaseUrl();

            if (StringUtils.isEmpty(baseUrl)) {
                return -1;
            }

            LoggerUtils.getLogger().info("获取配置信息url:" + baseUrl);
            WebApi.init(baseUrl + "/");
        } catch (Exception e) {
            LoggerUtils.getLogger().info("服务地址配置无效:" + e.toString());
            showHintDialog("服务地址配置无效:" + e.getMessage());
            return -1;
        }

        ConfigService.configDownload(Config.org_code, Config.pos_no, GeneralSettings.INSTANCE.getConfigVersion(),
                new Callback<ConfigResultData>() {
                    @Override
                    public void onSuccess(ConfigResultData configResultData) {
                        MidConfigUtils.saveMidConfigParams(configResultData);
                        startCountTimerUtil(beforeCall);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        LoggerUtils.getLogger().error("获取配置信息失败:" + throwable.toString());
                        startCountTimerUtil(beforeCall);
                        showHintDialog("获取配置信息失败:" + throwable.getMessage(), (msg) ->
                                mMvpView.configDownloadFail()
                        );
                    }
                });

        return 0;
    }

    /**
     * 设置时间启动器
     *
     * @param beforeCall 开始时间
     */
    private void startCountTimerUtil(long beforeCall) {
        long afterCall = System.currentTimeMillis();
        long millsInFuture = Integer.parseInt(BigDecimalUtils.subtract(String.valueOf(afterCall), String.valueOf(beforeCall)));
        if (!isViewAttached()) {
            return;
        }
        mMvpView.countTimerUtil().setMillisInFuture(millsInFuture);
        mMvpView.countTimerUtil().start();
    }
}
