package com.hisense.hiaposcmp.presenter;

import android.app.Activity;
import android.content.SharedPreferences;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.CommonConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IConfigView;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.webapi.Callback;
import com.hisense.webapi.WebApi;

import java.io.DataOutputStream;
import java.util.concurrent.TimeUnit;

/**
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-22  重构
 */
public class ConfigPresenter extends BasePresenter<IConfigView> {

    public void checkAndSave(String url) {
        if (mMvpView.performQuitBack(url)) {
            return;
        }

        if (StringUtils.isEmpty(url)) {
            showHintDialog("服务地址不能为空");
            return;
        }

        String orgCode = CommonTools.loginConfig(App.getContext()).getString(Config.ORG_CODE, "");
        String shopCode = CommonTools.loginConfig(App.getContext()).getString(Config.SHOP_CODE, "");
        String posNo = CommonTools.loginConfig(App.getContext()).getString(Config.POS_NO, "");

        serviceOnline(orgCode, shopCode, posNo, url);
    }

    public void checkAndSave(String orgCode, String shopCode, String posNo, String url) {
        if (mMvpView.performQuitBack(orgCode)) {
            return;
        }

        if (StringUtils.isEmpty(orgCode)) {
            showHintDialog("商户编号不能为空");
            return;
        }
        if (StringUtils.isEmpty(posNo)) {
            showHintDialog("收款机号不能为空");
            return;
        }
        if (StringUtils.isEmpty(url)) {
            showHintDialog("服务地址不能为空");
            return;
        }

        LoggerUtils.getLogger().info("门店编号:" + orgCode + ",专柜号:" + shopCode + ",pos机号:" + posNo);
        serviceOnline(orgCode, shopCode, posNo, url);
    }

    private void serviceOnline(String orgCode, String shopCode, String posNo, String url) {
        showLoadingDialog();

        try {
            String baseUrl = Config.getUrl(url);
            WebApi.init(baseUrl + "/", GeneralSettings.INSTANCE.getTimeout(), TimeUnit.SECONDS);
        } catch (Exception e) {
            hideLoadingDialog();
            LoggerUtils.getLogger().info("服务地址配置无效:" + e.toString());
            showHintDialog("服务地址配置无效:" + e.getMessage());
            return;
        }

        Service.online(new Callback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData o) {
                hideLoadingDialog();
                if (saveConfig(orgCode, shopCode, posNo, url)) {
                    Config.init();
                    mMvpView.serviceOnlineSuccess();
                }
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                CommonTools.toastShort("无法连接服务。请检查网络状态或修改服务地址");
            }
        });
    }

    /**
     * 保存登录设置。
     *
     * @return 如果成功写入配置文件则返回 true；否则返回 false。
     */
    private boolean saveConfig(String orgCode, String shopCode, String posNo, String url) {
        try {
            SharedPreferences sharedPreferences = App.getAppContext().getSharedPreferences(CommonConstants.LOGIN_CONFIG,
                    Activity.MODE_PRIVATE);
            //实例化SharedPreferences.Editor对象
            SharedPreferences.Editor editor = sharedPreferences.edit();

            //用putString的方法保存数据
            if (!orgCode.equals(Config.org_code)) {
                Context.orgName("");
            }
            if (!Config.org_code.equals(orgCode) || !Config.pos_no.equals(posNo)) {
                SaleNoService.abandon();//若修改了组织号或者收款机号后，废弃最新流水号重新获取
            }
            //用putString的方法保存数据
            editor.putString(Config.ORG_CODE, orgCode);
            editor.putString(Config.ORG_NAME, Context.orgName());
            editor.putString(Config.SHOP_CODE, shopCode);
            editor.putString(Config.POS_NO, posNo);
            editor.putString(Config.SERVER_URL, url);
            editor.putBoolean(Config.IS_SETTING, true);

            //提交当前数据
            Config.org_code = orgCode;
            Config.org_name = Context.orgName();
            Config.pos_no = posNo;
            Config.base_url = url;
            Config.shop_code = shopCode;
            return editor.commit();
        } catch (Exception ex) {
            return false;
        }
    }

    /**
     * 设备是否有root权限
     */
    public synchronized boolean isRootRight() {
        Process process = null;
        DataOutputStream os = null;
        try {
            process = Runtime.getRuntime().exec("su");
            os = new DataOutputStream(process.getOutputStream());
            os.writeBytes("exit\n");
            os.flush();
            int exitValue = process.waitFor();
            if (exitValue == 0) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().info("Unexpected error - " + e.getMessage());
            return false;
        } finally {
            try {
                if (os != null) {
                    os.close();
                }
                process.destroy();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}
