package com.hisense.hiaposcmp.presenter;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.IcCardType;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.service.sale.SaleAutoUploadService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IMainView;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.auth.AuthService;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.webapi.Callback;

/**
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-22  重构
 * @author guolina          2019-10-24  重构日结
 */
public class MainPresenter extends BasePresenter<IMainView> {

    private SaleAutoUploadService saleAutoUploadService;
    private SaleApplicationService saleApplicationService;

    public void serviceOnline() {
        showLoadingDialog();
        Service.online(new Callback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData service) {
                hideLoadingDialog();
                mMvpView.serviceOnlineSuccess();
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                CommonTools.toastShort(throwable.getMessage());
            }
        });
    }

    public void queryPrivileges(String userId) {
        showLoadingDialog();
        AuthService.queryPrivileges(userId, new Callback<User>() {
            @Override
            public void onSuccess(User user) {
                if (user == null) {
                    LoggerUtils.getLogger().error("获取用户权限失败，返回用户权限信息为空");
                    showHintDialog("获取用户权限失败，返回用户权限信息为空");
                    return;
                }

                mMvpView.queryPrivilegesSuccess(user);
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("获取用户权限失败:" + throwable.toString());
                showHintDialog("获取用户权限失败:" + throwable.getMessage());
                mMvpView.queryPrivilegesFail();
                hideLoadingDialog();
            }
        });
    }

    public void account() {
        showLoadingDialog();
        SaleService.getAccountingDate(new Callback<String>() {
            @Override
            public void onSuccess(String accDate) {
                hideLoadingDialog();
                mMvpView.getAccountingDateSuccess(accDate);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                showHintDialog(throwable.getMessage());
            }
        });
    }


    /**
     * 确认日结。
     */
    public void confirmAccounting(String accDate) {
        showHintConfirm(null, "当前结账日期为" + accDate + "。是否日结？", "是", "否", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                showLoadingDialog();
                SaleService.accounting(new Callback<String>() {
                    @Override
                    public void onSuccess(String s) {
                        hideLoadingDialog();
                        showHintDialog("日结完成");
                        mMvpView.accountingSuccess(s);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        hideLoadingDialog();
                        showHintDialog(throwable.getMessage());
                    }
                });
            }

            @Override
            public void onError(String err) {

            }
        });
    }

    public void getIcCardPassword() {
        VipService.getIcCardPassword(IcCardType.Mifare1, GeneralSettings.INSTANCE.getIcCardVersion(), Context.cashier(),
                new Callback<String>() {
                    @Override
                    public void onSuccess(String password) {
                        if (mMvpView != null) {
                            mMvpView.getIcCardPasswordSuccess(password);
                        }

                        LoggerUtils.getLogger().debug("获取射频卡密码成功");
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        LoggerUtils.getLogger().debug("获取射频卡密码异常: " + throwable.getMessage());
                    }
                });
    }

    public Boolean uploadLocalData() {
        if (saleApplicationService == null) {
            saleApplicationService = new SaleApplicationService(mMvpView.getContext());
        }
        return saleApplicationService.uploadLocalData();
    }

    public void startSaleUploadAuto() {
        if (saleAutoUploadService == null) {
            saleAutoUploadService = new SaleAutoUploadService(mMvpView.getContext());
        }
        saleAutoUploadService.startSaleUpload();
    }
}
