package com.hisense.hiaposcmp.presenter;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IPaymentChannelView;
import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.hisense.pos.payment.icbc.IcbcPaymentServiceProvider;

/**
 * @author guolina  2019-10-23  重构
 */
public class PaymentChannelPresenter extends BasePresenter<IPaymentChannelView> {

    /**
     * 结算。
     */
    public void settlement(IPaymentServiceProvider paymentServiceProvider, Activity activity) {
        try {
            paymentServiceProvider.settle(new PaymentHandler() {
                @Override
                public void onFinish(final Object... args) {
                    handleIcbcSettlement(paymentServiceProvider, activity, args);
                }
            });
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
            LoggerUtils.getLogger().error(ex.getMessage(), ex);
        }
    }

    private void handleIcbcSettlement(IPaymentServiceProvider paymentServiceProvider, Activity activity, final Object... args) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (args != null && args.length > 0) {
                    try {
                        final Intent intent = new Intent();
                        intent.putExtra("baseResult", (Bundle) args[0]);
                        paymentServiceProvider.handleResult(intent);
                        CommonTools.toastShort("结算完成");
                    } catch (Exception ex) {
                        showHintDialog("结算失败: " + ex.getMessage());
                    }
                }
            }
        });
    }

    /**
     * 打印交易汇总。
     */
    public void printTransSummary(IPaymentServiceProvider paymentServiceProvider) {
        try {
            paymentServiceProvider.printTransSummary(new PaymentHandler() {
                @Override
                public void onFinish(Object... args) {
                    handleIcbcTransSummary(paymentServiceProvider, args);
                }
            });
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
            LoggerUtils.getLogger().error(ex.getMessage(), ex);
        }
    }

    private void handleIcbcTransSummary(IPaymentServiceProvider paymentServiceProvider, final Object... args) {
        final IcbcPaymentServiceProvider provider = (IcbcPaymentServiceProvider) paymentServiceProvider;

        try {
            Bundle baseResult = (Bundle) args[0];
            Bundle transResult = (Bundle) args[1];

            provider.printTransSummary(baseResult, transResult, new PaymentHandler() {
                @Override
                public void onFinish(final Object... args) {
                    try {
                        provider.handleBaseResult((Bundle) args[0]);
                    } catch (PaymentProviderException ex) {
                        showHintDialog(ex.getMessage());
                    }
                }
            });
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
        }
    }

    /**
     * 查询余额。
     */
    public void queryBalance(IPaymentServiceProvider paymentServiceProvider, Activity activity) {

        try {
            paymentServiceProvider.queryBalance(new PaymentHandler() {
                @Override
                public void onFinish(final Object... args) {
                    handleIcbcTransResult(paymentServiceProvider, activity, args);
                }
            });
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
            LoggerUtils.getLogger().error(ex.getMessage(), ex);
        }
    }

    private void handleIcbcTransResult(IPaymentServiceProvider paymentServiceProvider, Activity activity, final Object... args) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (args != null && args.length > 0) {
                    try {
                        final Intent intent = new Intent();
                        intent.putExtra("baseResult", (Bundle) args[0]);
                        paymentServiceProvider.handleResult(intent);
                    } catch (Exception ex) {
                        showHintDialog(ex.getMessage());
                    }
                }
            }
        });
    }
}
