package com.hisense.hiaposcmp.presenter;

import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.IPosManageView;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.webapi.sale.SaleSubmitResultData;
import com.hisense.webapi.Callback;

import java.util.List;

/**
 * 修改该类时需同步修改SaleService类中对应方法
 *
 * @author dukunbiao(null)  2019-07-22  重构
 */
public class PosManagePresenter extends BasePresenter<IPosManageView> {

    private SaleApplicationService saleApplicationService;

    public void reSubmit() {
        saleApplicationService = new SaleApplicationService(mMvpView.getContext());

        List<BaseSaleModel> notUploadSales = saleApplicationService.getNotUploadSales();
        if (ListUtils.isEmpty(notUploadSales)) {
            showHintDialog("流水已全部上传");
            return;
        }
        for (BaseSaleModel notUploadSale : notUploadSales) {
            try {
                showLoadingDialog();
                String saleNo = notUploadSale.getSaleNo();
                LoggerUtils.getLogger().debug("正在重传流水,流水号:"+saleNo);
                CommonTools.toastShort("正在上传流水：" + saleNo + "，请稍后");
                saleApplicationService.upload(notUploadSale, new Callback<SaleSubmitResultData>() {
                    @Override
                    public void onSuccess(SaleSubmitResultData submitResultData) {
                        hideLoadingDialog();
                        LoggerUtils.getLogger().debug("流水重传成功:" + saleNo);
                        CommonTools.toastShort("流水重传成功:" + saleNo);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        hideLoadingDialog();
                        LoggerUtils.getLogger().debug("流水重传失败:" + throwable.toString());
                        showHintDialog("流水重传失败:" + throwable.getMessage());
                    }
                });
            } catch (Exception e) {
                LoggerUtils.getLogger().debug("获取重传流水信息异常:" + e.toString());
                showHintDialog("获取重传流水信息异常:" + e.getMessage());
                break;
            }
        }

    }

}
