package com.hisense.hiaposcmp.presenter;

import android.text.TextUtils;
import android.widget.SimpleAdapter;

import androidx.recyclerview.widget.SortedList;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.report.ReportConstants;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.service.payment.PayMethodService;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.view.IView.IReportView;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.SaleReportPayment;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.report.ShopSaleReportItem;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.report.DepReportResultData;
import com.hisense.hipos.webapi.report.DepReportResultItem;
import com.hisense.hipos.webapi.report.PluReportResultData;
import com.hisense.hipos.webapi.report.PluReportResultItem;
import com.hisense.hipos.webapi.report.ReportService;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.webapi.Callback;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * 报表查询。
 *
 * @author xieliyan    2017-08-18
 * @author xieliyan    2017-09-01 选择不同的报表传入不同的报表类型
 * @author zhangsai    2018-12-28 HIPOSAPP-946 消息参数由 reportType 改为 tranType
 * @author wangyaogang 2019-01-16 HIPOSAPP-936 功能权限由CMP5控制下发,尤其是【查询】和【报表】中的功能权限
 * @author xieliyan    2020-04-14 HIAPOSCMP-678 预销售报表，总交易报表小票标题打印区分
 * @author liuyazeng   2020-07-17 HIAPOSCMP-846	总交易报表打印添加组织号、款台号
 */
public class ReportPresenter extends BasePresenter<IReportView> {

    private SaleReport mSaleReport;
    private ShopSaleReport mShopReport;

    private DepReportResultData mDepReport;
    private PluReportResultData mPluReport;
    private String beginTime = null;


    /**
     * 月份和日 不足两位数时补0
     */
    public String appendZero(String var) {

        if (var.length() == 1) {
            return '0' + var;
        } else {
            return var;
        }
    }

    /**
     * 收银员报表结账日期
     */
    public void accountingDateCashier() {
        showLoadingDialog();
        SaleService.getAccountingDate(new Callback<String>() {
            @Override
            public void onSuccess(String accDate) {
                getCashierReport(accDate);

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().info("无法获取当前结账日期" + throwable.getMessage());
                showHintDialog("无法获取当前结账日期");
            }
        });
    }

    /**
     * 销售报表结账日期
     *
     * @param mTranType 交易类型
     */
    public void accountingDateSale(String title, TranType mTranType) {
        showLoadingDialog();
        SaleService.getAccountingDate(new Callback<String>() {
            @Override
            public void onSuccess(String accDate) {
                if (title.equals(ReportConstants.REPORT_SALE) || title.equals(ReportConstants.REPORT_PRESALE)) {
                    getSaleReport(accDate, mTranType);
                } else if (title.equals(ReportConstants.REPORT_CASHIER)) {
                    getCashierReport(accDate);
                } else if (title.equals(ReportConstants.REPORT_SHOP)) {
                    getShopReport(accDate);
                } else if (title.equals(ReportConstants.REPORT_DEPART)) {
                    getDepReport(accDate, GeneralSettings.INSTANCE.isNoShopLimits() ? "*" : Context.shopCode());
                } else if (title.equals(ReportConstants.REPORT_PLU)) {
                    getPluReport(accDate, GeneralSettings.INSTANCE.isNoShopLimits() ? "*" : Context.shopCode());
                }
                mMvpView.accountingDateSaleSuccess(accDate);

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().info("无法获取当前结账日期" + throwable.getMessage());
                showHintDialog("无法获取当前结账日期");
            }
        });
    }

    /**
     * 查询收银员报表。
     *
     * @param accDate 结账日期
     */
    public void getCashierReport(String accDate) {
        ReportService.cashier(Context.cashier().code(), accDate, new Callback<SaleReport>() {
            @Override
            public void onSuccess(SaleReport report) {
                hideLoadingDialog();
                mMvpView.showCashierData(true);
                mSaleReport = report;
                mMvpView.showSaleReport(report);


            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showCashierData(false);
                mSaleReport = null;
                mMvpView.showSaleReport(null);
                showHintDialog(throwable.getMessage());
            }
        });
    }

    /**
     * 查询销售报表。
     *
     * @param accDate   结账日期
     * @param mTranType 交易类型
     */
    public void getSaleReport(String accDate, TranType mTranType) {

        ReportService.sale(accDate, mTranType, new Callback<SaleReport>() {
            @Override
            public void onSuccess(SaleReport report) {
                hideLoadingDialog();
                mMvpView.showCashierData(true);
                mSaleReport = report;
                mMvpView.showSaleReport(report);

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showCashierData(false);
                mSaleReport = null;
                mMvpView.showSaleReport(null);
                showHintDialog(throwable.getMessage());
            }
        });
    }

    /**
     * 查询报表。
     *
     * @param accDate 结账日期
     */
    public void getShopReport(String accDate) {
        ReportService.shop(accDate, new Callback<ShopSaleReport>() {
            @Override
            public void onSuccess(ShopSaleReport report) {
                hideLoadingDialog();
                mMvpView.showCashierData(true);
                mMvpView.showShopSaleReport(report);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showCashierData(false);
                mMvpView.showShopSaleReport(null);
                showHintDialog(throwable.getMessage());
            }
        });
    }

    /**
     * 查询部门报表。
     *
     * @param beginTime 查询时间
     */
    public void getDepReport(String beginTime, String shopCode) {
        if (TextUtils.isEmpty(beginTime)) {
            showHintDialog(mMvpView.getActivity().getString(R.string.select_begin_time));
            return;
        }

        showLoadingDialog();
        ReportService.dep(beginTime, shopCode, new Callback<DepReportResultData>() {
            @Override
            public void onSuccess(DepReportResultData report) {
                hideLoadingDialog();
                mMvpView.showCashierData(true);
                mMvpView.showDepReport(report);

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showCashierData(false);
                mMvpView.showDepReport(null);
                showHintDialog(throwable.getMessage());

            }
        });
    }

    /**
     * 查询商品报表。
     *
     * @param beginTime 查询时间
     */
    public void getPluReport(String beginTime, String shopCode) {
        if (TextUtils.isEmpty(beginTime)) {

            showHintDialog(mMvpView.getActivity().getString(R.string.select_begin_time));


            return;
        }

        showLoadingDialog();
        ReportService.plu(beginTime, shopCode, new Callback<PluReportResultData>() {
            @Override
            public void onSuccess(PluReportResultData report) {
                hideLoadingDialog();
                mMvpView.showCashierData(true);
                mMvpView.showPluReport(report);

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showCashierData(false);
                showHintDialog(throwable.getMessage());

            }
        });
    }

    /**
     * 报表数据。
     *
     * @return mSaleReport 报表
     */
    public SaleReport cashierReport() {
        return mSaleReport;
    }

    /**
     * 保存报表数据
     *
     * @param report 报表
     */
    public void setCashierReport(SaleReport report) {
        mSaleReport = report;

    }

    /**
     * 专柜报表数据。
     *
     * @return mSaleReport 报表
     */
    public ShopSaleReport shopReport() {
        return mShopReport;
    }

    /**
     * 保存专柜报表数据
     *
     * @param report 报表
     */
    public void setShopSaleReport(ShopSaleReport report) {
        mShopReport = report;

    }

    /**
     * 保存部门报表数据
     *
     * @param report 报表
     */
    public void setDepShopSaleReport(DepReportResultData report) {
        mDepReport = report;

    }

    /**
     * 保存商品销售报表数据
     *
     * @param report 报表
     */
    public void setPluSaleReport(PluReportResultData report) {
        mPluReport = report;

    }

    /**
     * 返回报表打印数据。
     *
     * @param title  报表名称
     * @param report 销售报表
     */
    private ArrayList<String> getSaleReportPrintContent(String title, SaleReport report) {
        ArrayList<String> printList = new ArrayList<>();

        printList.add("cmd_centerStart");
        if (title.equals(ReportConstants.REPORT_SALE) && mMvpView.tranType().isPreSale()) {
            title = ReportConstants.REPORT_PRESALE;
        }
        printList.add(title);
        printList.add("cmd_centerEnd");
        if (null != beginTime) {
            printList.add("交易日期：" + beginTime);
        }
        printList.add("门店编号：" + (TextUtils.isEmpty(Config.org_code) ? "未设置" : Config.org_code));
        printList.add("收款机号：" + (TextUtils.isEmpty(Config.pos_no) ? "未设置" : Config.pos_no));
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
        printList.add("打印时间：" + sdf.format(new Date()));
        printList.add("--------------------------------------------");
        if (title.equals(ReportConstants.REPORT_CASHIER)) {
            printList.add("收银员编码：" + Context.cashier().code());
            printList.add("收银员姓名：" + Context.cashier().name());
            printList.add("--------------------------------------------");
        }

        printList.add("支付方式： 实收金额");
        for (SaleReportPayment payment : report.payments()) {
            printList.add(String.format(Locale.CHINESE, "%s: %.2f   退： %.2f",
                    payment.name(), payment.totalPaid(), payment.totalReturned()));
        }
        printList.add("--------------------------------------------");
        printList.add(String.format(Locale.CHINESE, "合计应收： %.2f", report.total()));
        printList.add(String.format(Locale.CHINESE, "合计实收： %.2f     退： %.2f", report.totalReceived(), report.totalReturned()));
        printList.add(String.format(Locale.CHINESE, "合计优惠金额： %.2f", report.totalDiscount()));
        printList.add(String.format(Locale.CHINESE, "合计交易笔数： %d   退： %d", report.totalTrans(), report.totalTransReturned()));
        printList.add("--------------------------------------------");
        printList.add("cmd_paperCut");//判断打印

        return printList;
    }

    /**
     * 返回报表打印数据。
     *
     * @param depReport 销售报表
     */
    private ArrayList<String> getPrintDepContent(String title, DepReportResultData depReport, PluReportResultData pluReport, String beginTime) {
        ArrayList<String> printList = new ArrayList<>();

        printList.add("cmd_centerStart");
        printList.add(title);
        printList.add("cmd_centerEnd");

//        printList.add("百货专柜对账单-专柜联");
        printList.add("--------------------------------------------");
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss",
                Locale.getDefault());
        printList.add("部门：" + Context.shopCode());
        printList.add("汇总日期：" + beginTime);
        printList.add("打印时间：" + format.format(new Date()));
        printList.add("--------------------------------------------");
        if (title.equals(ReportConstants.REPORT_DEPART)) {
            if (depReport == null) {
                return null;
            }

            printList.add("部门名称      部门编码      金额");
            for (DepReportResultItem item : depReport.deps) {
                printList.add(item.depName + "\n");
                printList.add(item.depCode + "              " + String.format(Locale.CHINA, "%.2f", Double.parseDouble(item.totalRecv)));
            }
            printList.add("--------------------------------------------");
            printList.add(String.format(Locale.CHINESE, "合计： %.2f", Double.parseDouble(depReport.totalRecv)));
        } else {
            if (pluReport == null) {
                return null;
            }
            printList.add("商品名称      商品编码      金额");
            for (PluReportResultItem item : pluReport.plus) {
                printList.add(item.pluName + "  ×  " + item.count + "\n");
                printList.add(item.pluCode + "              " + String.format(Locale.CHINA, "%.2f", Double.parseDouble(item.totalRecv)));
            }
            printList.add("--------------------------------------------");
            printList.add(String.format(Locale.CHINESE, "合计： %.2f", Double.parseDouble(pluReport.totalRecv)));
        }

        printList.add("--------------------------------------------");
        printList.add("部门人员（手动签字）： ");
        printList.add("收银员（手动签字）： ");
        printList.add("--------------------------------------------");
        printList.add("cmd_paperCut");//判断打印

        return printList;
    }

    /**
     * 返回报表打印数据。
     *
     * @param report 销售报表
     */
    private ArrayList<String> getPrintShopContent(ShopSaleReport report) {
        ArrayList<String> printList = new ArrayList<>();

        printList.add("cmd_centerStart");
        printList.add("专柜报表");
        printList.add("cmd_centerEnd");

        printList.add("百货专柜对账单-专柜联");
        printList.add("--------------------------------------------");
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss",
                Locale.getDefault());
        printList.add("柜组：" + Context.shopCode());
        printList.add("打印时间：" + format.format(new Date()));
        printList.add("--------------------------------------------");

        printList.add("商品编码");
        for (ShopSaleReportItem item : report.items()) {
            printList.add(String.format(Locale.CHINESE, "%s： %.2f",
                    item.plu().getPluCode(), item.totalRecv()));
        }
        printList.add("--------------------------------------------");
        printList.add(String.format(Locale.CHINESE, "合计： %.2f", report.total()));
        printList.add("--------------------------------------------");
        printList.add("柜组人员（手动签字）： ");
        printList.add("收银员（手动签字）： ");
        printList.add("--------------------------------------------");
        printList.add("cmd_paperCut");//判断打印

        return printList;
    }

    /**
     * 打印报表。
     *
     * @param title 报表名称
     */
    public void printSalePort(String title, String txtBeginTime) {
        if (null != beginTime) {
            beginTime = txtBeginTime;
        }
        printSalePort(title);
    }

    /**
     * 打印报表。
     *
     * @param title 报表名称
     */
    public void printSalePort(String title) {
        if (mSaleReport == null) {
            return;
        }
        ArrayList<String> printList = getSaleReportPrintContent(title, mSaleReport);

        PrinterHelper.getInstance().printWithStringList(printList);

    }

    /**
     * 打印报表。
     *
     * @param beginTime 查询开始时间
     */
    public void printDepPort(String title, String beginTime) {

        ArrayList<String> printList = getPrintDepContent(title, mDepReport, mPluReport, beginTime);
        if (printList == null) {
            showHintDialog("未查询到打印信息");
            return;
        }

        PrinterHelper.getInstance().printWithStringList(printList);

    }

    public void printShopReport() {
        if (mShopReport == null) {
            return;
        }

        ArrayList<String> printList = getPrintShopContent(mShopReport);

        PrinterHelper.getInstance().printWithStringList(printList);
    }


    /**
     * 报表适配器
     *
     * @return adapter
     */
    public SimpleAdapter saleReportadapter() {
        ArrayList<Map<String, Object>> payments = new ArrayList<>();

        if (mSaleReport != null) {
            for (SaleReportPayment payment : mSaleReport.payments()) {
                HashMap<String, Object> map = new HashMap<>();
                map.put(ReportConstants.PAY_METHOD_ICON, PayMethodService.getIconByCode(payment.code()));
                map.put(ReportConstants.PAY_METHOD_NAME, payment.name());
                map.put(ReportConstants.TOTAL_PAID, String.format(Locale.CHINESE, "%.2f", payment.totalPaid()));
                map.put(ReportConstants.TOTAL_RETURN, String.format(Locale.CHINESE, "%.2f", payment.totalReturned()));
                payments.add(map);
            }
        } else {
            payments.clear();
        }
        if (CommonUtils.isStdStyle()) {

            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(), payments, R.layout.adapter_report_pay_item_std,
                    new String[]{ReportConstants.PAY_METHOD_ICON, ReportConstants.PAY_METHOD_NAME,
                            ReportConstants.TOTAL_PAID, ReportConstants.TOTAL_RETURN},
                    new int[]{R.id.imgPayMethod, R.id.txtPayName, R.id.txtTotalPaid, R.id.txtTotalReturned});
            return adapter;
        } else {
            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(), payments, R.layout.adapter_report_pay_item,
                    new String[]{ReportConstants.PAY_METHOD_ICON, ReportConstants.PAY_METHOD_NAME,
                            ReportConstants.TOTAL_PAID, ReportConstants.TOTAL_RETURN},
                    new int[]{R.id.imgPayMethod, R.id.txtPayName, R.id.txtTotalPaid, R.id.txtTotalReturned});
            return adapter;
        }

    }

    /**
     * 专柜报表适配器
     *
     * @return adapter
     */
    public SimpleAdapter shopReportAdapter() {

        SortedList<ShopSaleReportItem> sortedList = new SortedList<>(ShopSaleReportItem.class,
                new SortedList.Callback<ShopSaleReportItem>() {
                    @Override
                    public int compare(ShopSaleReportItem o1, ShopSaleReportItem o2) {
                        return o1.plu().getPluCode().compareTo(o2.plu().getPluCode());
                    }

                    @Override
                    public void onChanged(int position, int count) {

                    }

                    @Override
                    public boolean areContentsTheSame(ShopSaleReportItem oldItem, ShopSaleReportItem newItem) {
                        return false;
                    }

                    @Override
                    public boolean areItemsTheSame(ShopSaleReportItem item1, ShopSaleReportItem item2) {
                        return false;
                    }

                    @Override
                    public void onInserted(int position, int count) {

                    }

                    @Override
                    public void onRemoved(int position, int count) {

                    }

                    @Override
                    public void onMoved(int fromPosition, int toPosition) {

                    }
                });
        List<HashMap<String, Object>> reportData = new ArrayList<>();
        if (mShopReport != null) {
            sortedList.addAll(mShopReport.items());
            for (int i = 0; i < sortedList.size(); i++) {
                ShopSaleReportItem item = sortedList.get(i);
                HashMap<String, Object> map = new HashMap<>();
                map.put(ReportConstants.PLU_CODE, item.plu().getPluCode());
                map.put(ReportConstants.PLU_NAME, item.plu().getPluName());
                map.put(ReportConstants.TOTAL, String.format(Locale.CHINESE, "%.2f", item.totalRecv()));
                reportData.add(map);
            }
        } else {
            reportData.clear();
        }

        if (CommonUtils.isStdStyle()) {
            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(),
                    reportData, R.layout.adapter_report_sale_item_std,
                    new String[]{ReportConstants.PLU_CODE, ReportConstants.TOTAL},
                    new int[]{R.id.txtPluCode, R.id.txtTotal});

            return adapter;
        } else {
            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(),
                    reportData, R.layout.adapter_report_sale_item,
                    new String[]{ReportConstants.PLU_CODE, ReportConstants.PLU_NAME, ReportConstants.TOTAL},
                    new int[]{R.id.txtPluCode, R.id.txtPluName, R.id.txtTotal});

            return adapter;
        }

    }


    /**
     * 部门报表适配器
     *
     * @return adapter
     */
    public SimpleAdapter departAdapter() {
        SortedList<DepReportResultItem> sortedList = new SortedList<>(DepReportResultItem.class,
                new SortedList.Callback<DepReportResultItem>() {
                    @Override
                    public int compare(DepReportResultItem o1, DepReportResultItem o2) {
                        return o1.depCode.compareTo(o2.depCode);
                    }

                    @Override
                    public void onChanged(int position, int count) {

                    }

                    @Override
                    public boolean areContentsTheSame(DepReportResultItem oldItem, DepReportResultItem newItem) {
                        return false;
                    }

                    @Override
                    public boolean areItemsTheSame(DepReportResultItem item1, DepReportResultItem item2) {
                        return false;
                    }

                    @Override
                    public void onInserted(int position, int count) {

                    }

                    @Override
                    public void onRemoved(int position, int count) {

                    }

                    @Override
                    public void onMoved(int fromPosition, int toPosition) {

                    }
                });

        List<HashMap<String, Object>> reportData = new ArrayList<>();
        if (mDepReport != null) {
            sortedList.addAll(mDepReport.deps);
            for (int i = 0; i < sortedList.size(); i++) {
                DepReportResultItem item = sortedList.get(i);
                HashMap<String, Object> map = new HashMap<>();
                map.put(ReportConstants.PLU_CODE, item.depCode);
                map.put(ReportConstants.PLU_NAME, item.depName);
                map.put(ReportConstants.TOTAL, String.format(Locale.CHINA, "%.2f", Double.parseDouble(item.totalRecv)));
                reportData.add(map);
            }
        } else {
            reportData.clear();
        }


        if (CommonUtils.isStdStyle()) {

            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(),
                    reportData, R.layout.adapter_report_dep_item_std,
                    new String[]{ReportConstants.PLU_CODE, ReportConstants.PLU_NAME, ReportConstants.TOTAL},
                    new int[]{R.id.txtPluCode, R.id.txtPluName, R.id.txtTotal});

            return adapter;
        } else {
            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(),
                    reportData, R.layout.adapter_report_sale_item,
                    new String[]{ReportConstants.PLU_CODE, ReportConstants.PLU_NAME, ReportConstants.TOTAL},
                    new int[]{R.id.txtPluCode, R.id.txtPluName, R.id.txtTotal});

            return adapter;
        }

    }

    public SimpleAdapter pluAdapter() {
        SortedList<PluReportResultItem> sortedList = new SortedList<>(PluReportResultItem.class,
                new SortedList.Callback<PluReportResultItem>() {
                    @Override
                    public int compare(PluReportResultItem o1, PluReportResultItem o2) {
                        return o1.pluCode.compareTo(o2.pluCode);
                    }

                    @Override
                    public void onChanged(int position, int count) {

                    }

                    @Override
                    public boolean areContentsTheSame(PluReportResultItem oldItem, PluReportResultItem newItem) {
                        return false;
                    }

                    @Override
                    public boolean areItemsTheSame(PluReportResultItem item1, PluReportResultItem item2) {
                        return false;
                    }

                    @Override
                    public void onInserted(int position, int count) {

                    }

                    @Override
                    public void onRemoved(int position, int count) {

                    }

                    @Override
                    public void onMoved(int fromPosition, int toPosition) {

                    }
                });
        List<HashMap<String, Object>> reportData = new ArrayList<>();
        if (mPluReport != null) {
            sortedList.addAll(mPluReport.plus);
            for (int i = 0; i < sortedList.size(); i++) {
                PluReportResultItem item = sortedList.get(i);
                HashMap<String, Object> map = new HashMap<>(sortedList.size());
                map.put(ReportConstants.PLU_CODE, item.pluCode);
                map.put(ReportConstants.PLU_NAME, item.pluName + "  ×  " + item.count);
                map.put(ReportConstants.TOTAL, String.format(Locale.CHINA, "%.2f", Double.parseDouble(item.totalRecv)));
                reportData.add(map);
            }
        } else {
            reportData.clear();
        }

        if (CommonUtils.isStdStyle()) {
            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(),
                    reportData, R.layout.adapter_report_dep_item_std,
                    new String[]{ReportConstants.PLU_CODE, ReportConstants.PLU_NAME, ReportConstants.TOTAL},
                    new int[]{R.id.txtPluCode, R.id.txtPluName, R.id.txtTotal});

            return adapter;
        } else {

            SimpleAdapter adapter = new SimpleAdapter(mMvpView.getActivity(),
                    reportData, R.layout.adapter_report_sale_item,
                    new String[]{ReportConstants.PLU_CODE, ReportConstants.PLU_NAME, ReportConstants.TOTAL},
                    new int[]{R.id.txtPluCode, R.id.txtPluName, R.id.txtTotal});

            return adapter;
        }

    }

}
