package com.hisense.hiaposcmp.presenter;

import com.hisense.common.Context;
import com.hisense.common.boot.GlobalCallback;
import com.hisense.common.util.CompressUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.hiaposcmp.auth.util.FileUtil;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.tool.ZipHelper;
import com.hisense.hiaposcmp.util.DeviceInfoUtils;
import com.hisense.hiaposcmp.view.IView.IUpLogView;
import com.lidroid.xutils.HttpUtils;
import com.lidroid.xutils.exception.HttpException;
import com.lidroid.xutils.http.RequestParams;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;
import com.lidroid.xutils.http.client.HttpRequest;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author dukunbiao(null)  2019-07-25  重构
 */
public class UpLogPresenter extends BasePresenter<IUpLogView> {

    private HttpUtils mHttpUtils;
    private List<File> totalLogs = new ArrayList<>();
    private List<File> dayLogs = new ArrayList<>();

    public void initHttpUtils() {
        mHttpUtils = new HttpUtils(15 * 1000);
        //设置缓存大小
        int size = (int) (Runtime.getRuntime()
                .totalMemory() / 8);
        mHttpUtils.configHttpCacheSize(size);
        //设置编码格式
        mHttpUtils.configResponseTextCharset("utf-8");
        //设置重连次数,如果链接服务失败,再次尝试
        mHttpUtils.configRequestRetryCount(3);
        //设置多线程迸发(线程池)
        mHttpUtils.configRequestThreadPoolSize(4);
        //设置缓存数据失效的时间
        mHttpUtils.configDefaultHttpCacheExpiry(1000 * 60 * 30);
    }

    /**
     * 开始上传日志流程
     */
    public void startUploadRoutine(String date) {
        if (StringUtils.isEmpty(date)) {
            showHintDialog("上传失败", "清先选择日期");
            return;
        }

        Date beginDate = new Date();
        try {
            beginDate = DateUtils.dateParse(date, "yyyy-MM-dd");
        } catch (Exception e) {

        }

        Date today = new Date();
        int diff = differentDays(beginDate, today);
        String destFileName;
        if (totalLogs.size() > 0) {
            totalLogs.clear();
        }

        for (int i = 0; i <= diff; i++) {
            Date dateToday = DateUtils.dayAddAndSub(today, -1 * i);
            String dateStr = DateUtils.dateFormat(dateToday, "yyyy-MM-dd");

            //查找指定文件名的文件(Tradelog+日期开头的)
            dayLogs = FileUtil.findFilesWithPrefix(Context.logPath()+"tradelog/", LoggerUtils.PRE_LOG_NAME + dateStr);
            totalLogs.addAll(dayLogs);
        }
        destFileName = DeviceInfoUtils.getModel()
                + "_" + System.currentTimeMillis()
                + "_log_"
                + date
                + "_manual"
                + ".zip";
        //压缩
        if (ZipHelper.zipFiles(totalLogs, destFileName)) {
            upload(Context.logPath() + "TRADELOGZIP/" + destFileName);
        } else {
            LoggerUtils.getLogger().error("日志压缩失败,没有找到文件");
            showHintDialog("日志上传失败", "该日期下没有日志");
        }
    }

    /**
     * date2比date1多的天数
     *
     * @param date1
     * @param date2
     * @return
     */
    private static int differentDays(Date date1, Date date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);

        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        int day1 = cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        //同一年
        if (year1 != year2) {
            int timeDistance = 0;
            for (int i = year1; i < year2; i++) {
                //闰年
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0) {
                    timeDistance += 366;
                } else {
                    //不是闰年
                    timeDistance += 365;
                }
            }
            return timeDistance + (day2 - day1);
        } else {
            //不同年
            LoggerUtils.getLogger().info("判断day2 - day1 : " + (day2 - day1));
            return day2 - day1;
        }
    }

    private void upload(String fileName) {
        showLoadingDialog();
        upload(fileName, new GlobalCallback<ResponseInfo<String>>() {
            @Override
            public void onSuccess(ResponseInfo<String> responseInfo) {
                if (responseInfo != null) {
                    try {
                        JSONObject object = new JSONObject(responseInfo.result);
                        int code = 1;
                        if (object.has("code")) {
                            code = object.getInt("code");
                        }
                        if (code == 1) {
                            for (int i = 0; i < totalLogs.size(); i++) {
                                CompressUtils.deleteAll(totalLogs.get(i).getAbsolutePath());
                            }
                            mMvpView.upLogResult("上传成功");
                        } else {
                            mMvpView.upLogResult("上传失败," + responseInfo.result);
                            CompressUtils.deleteAll(fileName);
                        }

                        hideLoadingDialog();
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
            }

            @Override
            public void onError(String message) {
                hideLoadingDialog();
                mMvpView.upLogResult(message);
                CompressUtils.deleteAll(fileName);
            }
        });
    }

    private void upload(String fileName, GlobalCallback<ResponseInfo<String>> callback) {
        ThreadFactoryUtils.getExecutorService("log-%d").execute(() -> {
            String url = Config.base_url + "/api/log/upload";
            RequestParams params = new RequestParams();
            params.addBodyParameter("orgCode", Config.org_code);
            params.addBodyParameter("posNo", Config.pos_no);
            params.addBodyParameter("file", new File(fileName));

            mHttpUtils.send(HttpRequest.HttpMethod.POST, url, params, new RequestCallBack<String>() {
                @Override
                public void onSuccess(ResponseInfo<String> responseInfo) {
                    callback.onSuccess(responseInfo);
                }

                @Override
                public void onFailure(HttpException error, String msg) {
                    LoggerUtils.getLogger().error("日志上传失败:" + error.toString() + "," + msg);
                    callback.onError("上传失败," + msg);
                }
            });
        });
    }
}
