package com.hisense.hiaposcmp.presenter;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.VerifyType;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.webapi.points.PointsService;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.activity.StartActivityService;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.view.IView.IVipLoginView;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.webapi.Callback;

import java.util.Locale;

/**
 * 会员登陆
 *
 * @author wangwenyue
 * @author liaoyuping 富基会员（济南银座）
 * @author xieliyan   crm会员 2018-11-07
 * @author dukunbiao(null)  2019-07-24  重构
 */
public class VipLoginPresenter extends BasePresenter<IVipLoginView> {

    private PayMethod mSelectedPayMethod;
    private Vip mVip;

    public void initSelectedPayMethod() {
        mSelectedPayMethod = MainSaleModel.getInstance().getSaleModel().getPayMethod();
    }

    public void vipQuery(String vipNo) {
        if (StringUtils.isEmpty(vipNo)) {
            if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
                mMvpView.vipLoginFail("请扫描会员码");
            } else {
                mMvpView.vipLoginFail("请输入手机号或会员卡号");
            }

            return;
        }

        CardNo cardNo;
        if (GeneralSettings.INSTANCE.getAuthMethod().isScan()) {
            cardNo = CardNo.PayCode(vipNo);
        } else {
            cardNo = CardNo.MobileOrDefault(vipNo);
        }

        vipQuery(cardNo);
    }

    public void vipQuery(CardNo cardNo) {
        if (mMvpView == null) {
            return;
        }
        showLoadingDialog();

        String saleNo = MainSaleModel.getInstance().getSaleModel() == null ? "" : MainSaleModel.getInstance().getSaleModel().getSaleNo();
        VipService.vipQuery(cardNo, null, saleNo, new Callback<Vip>() {
            @Override
            public void onSuccess(Vip vip) {
                hideLoadingDialog();
                if (vip == null) {
                    mMvpView.vipLoginFail("会员验证失败");
                    return;
                }

                vip.setIseCard(cardNo.getType().getTypeCode() == CardNoType.PayCode.getTypeCode());
                mMvpView.vipLoginSuccess(vip);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();

                String errMsg = throwable.getMessage();
                if (errMsg != null && errMsg.length() == 2) {
                    errMsg = String.format(Locale.CHINESE, "此卡已%s", errMsg);
                }

                LoggerUtils.getLogger().error("会员验证异常:" + throwable.toString());
                mMvpView.vipLoginFail(errMsg);
            }
        });
    }

    public void queryVerifyType(Vip vip) {
        showLoadingDialog();
        String cardFaceNo = vip.card().cardFaceNo();
        VipService.queryVerifyType(CardNo.FaceNo(cardFaceNo), mSelectedPayMethod.remark(), new Callback<VerifyType>() {
            @Override
            public void onSuccess(VerifyType verifyType) {
                mVip = vip;
                //都不需要验证
                if (!verifyType.needVerify()) {
                    mMvpView.verifyTypeSuccess(vip);
                } else {
                    StartActivityService.startCardSecurityInputActivity(mMvpView.getActivity(), false, verifyType);
                }
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                if (isViewAttached()) {
                    hideLoadingDialog();
                    mMvpView.verifyTypeFail("查询会员验证方式失败:" + throwable.getMessage());
                }
                LoggerUtils.getLogger().error("VipCouponShow查询会员验证方式:" + throwable.toString());

            }
        });
    }

    /**
     * 功能为验证密码准确性
     * 会员查询接口仅可确认有无该会员,并不校验密码,此处使用积分查询接口校验支付密码的准确性
     *
     * @param cardSecurity 校验方式
     */
    public void verifyVip(CardSecurity cardSecurity, CardNo cardNo) {
        showLoadingDialog();
        PointsService.query(cardNo, cardSecurity, cardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword(), new Callback<VipCard>() {
            @Override
            public void onSuccess(VipCard vipCard) {
                mMvpView.verifyTypeSuccess(mVip);
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable t) {
                LoggerUtils.getLogger().error("VipCouponShow验证密码准确性失败:" + t.toString());
                showHintDialog("验证密码准确性失败:" + t.getMessage());
                hideLoadingDialog();
            }
        });
    }
}
