package com.hisense.hiaposcmp.presenter;

import android.os.CountDownTimer;
import android.os.Handler;

import com.hisense.common.boot.GlobalCallback;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.pos.print.PrintProvider;
import com.hisense.hiaposcmp.print.Invoice.EInvoice2;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.print.printer.Printer;
import com.hisense.hiaposcmp.print.printer.impl.LocalPOSPrinter;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.tool.SystemConfig;
import com.hisense.hiaposcmp.view.IView.IWelcomeView;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.sale.SaleBeginResultData;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.pos.hs6500.HS650PrintProvider;
import com.hisense.pos.print.IPrintProvider;
import com.hisense.webapi.Callback;
import com.hisense.webapi.WebApi;

import java.util.Date;
import java.util.List;

/**
 * @author wangwenyue on 2018/7/4.
 * @author dukunbiao(null)  2019-07-22  重构
 */
public class WelcomePresenter extends BasePresenter<IWelcomeView> {

    private CountDownTimer timer;

    public void createNewSale(GlobalCallback<String> callback) {
        //View是否绑定 如果没有绑定，就不执行网络请求
        if (!isViewAttached()) {
            return;
        }

        showLoadingDialog();
        LoggerUtils.getLogger().info("saleLastAccountingDate");

        SaleService.getAccountingDate(new Callback<String>() {
            @Override
            public void onSuccess(String s) {
                saleBegin(s, callback);
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("获取结账日期异常:" + throwable.toString());
                callback.onError("获取结账日期异常:" + throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    public void saleBegin(String accDate, GlobalCallback<String> callback) {
        //View是否绑定 如果没有绑定，就不执行网络请求
        if (!isViewAttached()) {
            return;
        }

        showLoadingDialog();
        LoggerUtils.getLogger().info("saleBegin");
        SaleService.saleBegin(accDate, GeneralSettings.INSTANCE.getCdKey(),
                GeneralSettings.INSTANCE.getOperator(), SystemConfig.getMac(), new Callback<SaleBeginResultData>() {
                    @Override
                    public void onSuccess(SaleBeginResultData resultData) {
                        SaleModel saleModel = new SaleModel();
                        User user = new User(Config.user_id, Config.user_code, Config.user_name);
                        saleModel.setCashier(user);
                        saleModel.setPosNo(Config.pos_no);
                        saleModel.setAccDate(accDate);
                        saleModel.setSaleNo(resultData.saleNo);
                        saleModel.setInvoiceNo(resultData.invoiceNo);
                        saleModel.setTranType(TranType.Sale);
                        saleModel.setSaleDate(WebApi.formatDate(new Date()));
                        saleModel.setSaleTime(DateUtils.dateFormat());
                        saleModel.setUdp1(user.getUserName());
                        saleModel.setUdp4(EInvoice2.random());
                        MainSaleModel.getInstance().saleModel(saleModel);
                        callback.onSuccess("");
                        if(isViewAttached()){
                            hideLoadingDialog();
                        }
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        LoggerUtils.getLogger().error("获取流水号异常:" + throwable.toString());
                        callback.onError("获取流水号异常:" + throwable.getMessage());
                        if(isViewAttached()){
                            hideLoadingDialog();
                        }

                    }
                });
    }

    public void initTimer() {
        long interval = Integer.parseInt(StringUtils.isEmpty(GeneralSettings.INSTANCE.getTestInterval()) ? "30" : GeneralSettings.INSTANCE.getTestInterval());

        timer = new CountDownTimer(60000 * interval, 60000) {

            @Override
            public void onTick(long millisUntilFinished) {
            }

            @Override
            public void onFinish() {
                if (!isViewAttached()) {
                    LoggerUtils.getLogger().debug("欢迎界面被finish");
                    return;
                }
                mMvpView.startSaleService();
            }
        };
    }

    public void timeStart() {
        new Handler(mMvpView.getContext().getMainLooper()).post(() -> timer.start());
    }

    public void timeCancel() {
        timer.cancel();
    }

    public void printTicketAgain() {
        List<String> printList = LoadSDCard.loadTxtFile(SDCardConstants.PRINTER_BILL_PATH, 1);

        if (printList != null && printList.size() > 0) {
            printList.add(0, "cmd_centerEnd");
            printList.add(0, "cmd_feedLine");
            printList.add(0, "******************重打小票******************");
            printList.add(0, "cmd_feedLine");
            printList.add(0, "cmd_centerStart");

            ThreadFactoryUtils.getExecutorService("print-%d").execute(() -> {
                try {
                    //操作文件流后延时使用打印方式，否则出现打印少行的情况
                    Thread.sleep(800);
                    IPrintProvider printProvider = PrintProvider.createPrintProvider();
                    Printer.print(printProvider, printList, null);

                    Thread.sleep(1200);
                    mMvpView.checkPrintStatus();
                } catch (Exception e) {
                    showHintDialog("重打小票异常", "请检查打印机类型");
                    LoggerUtils.getLogger().error("重打小票异常:" + e.toString());
                }
            });
//            // 2020-10-17 国美大屏重打小票功能
//            LocalPOSPrinter.isPrint = true;
//            PrinterHelper.getInstance().printWithStringList(printList);
        } else {
            showHintDialog("未找到上笔流水小票", "请联系收银员");
        }
    }

    public String getPrintStatus() {
        try {
            HS650PrintProvider hs650PrintProvider = new HS650PrintProvider();
            if (hs650PrintProvider.initPrinter(GeneralSettings.INSTANCE.getPrinterRate(), GeneralSettings.INSTANCE.getPrinterPort(),
                    GeneralSettings.INSTANCE.getDeviceType().isHS650() ? 0 : 1, GeneralSettings.INSTANCE.getPrinterFreeLine())) {
                if (hs650PrintProvider.getStatus() == 1) {
                    LoggerUtils.getLogger().error("打印机缺纸");
                    return "打印机缺纸";
                }
                hs650PrintProvider.closePrinter();
            } else {
                LoggerUtils.getLogger().error("打印机初始化失败");
                return "打印机初始化失败";
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("打印机获取缺纸状态异常" + e.toString());
            return "打印机获取缺纸状态异常";
        }

        return "";
    }
}
