package com.hisense.hiaposcmp.presenter.auth;

import android.widget.EditText;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.view.IView.IChangePasswordView;
import com.hisense.hipos.webapi.auth.AuthService;
import com.hisense.webapi.Callback;

import java.lang.reflect.Method;

/**
 * @author guolina  2019-10-23  重构
 */
public class ChangePasswordPresenter extends BasePresenter<IChangePasswordView> {

    /**
     * 修改密码。
     *
     * @param oldPwd     旧密码
     * @param newPwd     新密码
     * @param confirmPwd 确认新密码
     */
    public void changePassword(String oldPwd, String newPwd, String confirmPwd) {
        if (oldPwd.length() == 0) {
            showHintDialog(App.getContext().getString(R.string.msg_no_password_old));
            return;
        }

        if (newPwd.length() == 0) {
            showHintDialog(App.getContext().getString(R.string.msg_no_password_new));
            return;
        }

        if (confirmPwd.length() == 0) {
            showHintDialog(App.getContext().getString(R.string.msg_no_password_confirm));
            return;
        }

        if (!newPwd.equals(confirmPwd)) {
            showHintDialog(App.getContext().getString(R.string.msg_new_password_not_equals_confirm));
            return;
        }

       showLoadingDialog();
        String userCode = Config.user_code;
        AuthService.changePassword(userCode, oldPwd, newPwd, new Callback<Object>() {
            @Override
            public void onSuccess(Object o) {
                hideLoadingDialog();
                mMvpView.changePasswordSuccess();

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().error("修改密码失败:" + throwable.toString());
                showHintDialog("修改密码失败:" + throwable.getMessage());
            }
        });
    }


    /**
     * 禁止EditText弹出软件盘，光标依然正常显示。
     *
     * @param editText 编辑文本
     */
    public void disableShowSoftInput(EditText editText) {
        Class<EditText> cls = EditText.class;
        Method method;
        try {
            method = cls.getMethod("setShowSoftInputOnFocus", boolean.class);
            method.setAccessible(true);
            method.invoke(editText, false);
        } catch (Exception e) {

        }

        try {
            method = cls.getMethod("setSoftInputShownOnFocus", boolean.class);
            method.setAccessible(true);
            method.invoke(editText, false);
        } catch (Exception e) {
        }
    }
}
