package com.hisense.hiaposcmp.presenter.auth;

import android.app.ProgressDialog;
import android.os.Environment;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.ProductKey;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.model.AuthorizationInfo;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.model.FeatureInfo;
import com.hisense.hiaposcmp.auth.algorithm.Des;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.pay.WxConstants;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.print.Invoice.EInvoice2;
import com.hisense.hiaposcmp.service.app.UpdateService;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.SystemConfig;
import com.hisense.hiaposcmp.util.FileUtils;
import com.hisense.hiaposcmp.view.IView.ILoginView;
import com.hisense.hipos.invoice.models.EInvoiceConfig;
import com.hisense.hipos.sale.models.ShopInfo;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.app.AppService;
import com.hisense.hipos.webapi.app.CheckUpdateResultData;
import com.hisense.hipos.webapi.auth.AuthService;
import com.hisense.hipos.webapi.auth.LoginResultData;
import com.hisense.hipos.webapi.shop.ShopService;
import com.hisense.hipos.webapi.invoice.InvoiceService;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.webapi.Callback;
import com.ums.AppHelper;

import org.json.JSONObject;

import java.io.File;
import java.util.Date;

/**
 * @author dukunbiao(null)  2019-08-02  重构
 * @author guolina          2019-12-13  HIAPOSCMP-426  防止登陆结果返回前loading过早消失
 * @author wangyaogang      2020-02-18  HIAPOSCMP-525  标点符号修改
 * @author xiliyan          2020-03-13                 授权逻辑调整
 */
public class LoginPresenter extends BasePresenter<ILoginView> {

    private SaleApplicationService saleApplicationService;

    public void getInvoiceUrl() {
        showLoadingDialog();

        InvoiceService.getConfig(new Callback<EInvoiceConfig>() {
            @Override
            public void onSuccess(EInvoiceConfig eInvoiceConfig) {
                EInvoice2.setBaseUrl(eInvoiceConfig.baseUrl());
                GeneralSettings.INSTANCE.setInvoiceUrl(eInvoiceConfig.baseUrl());
                CommonTools.toastLong("电子发票服务地址获取成功");
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("获取电子发票服务地址失败:" + throwable.toString());
                showHintDialog("获取电子发票服务地址失败:" + throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    public void checkApiVersion() {
        showLoadingDialog();
        Service.online(new Callback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData service) {
                if (checkApiVersion(service.getApiVersion())) {
                    if (!isViewAttached()) {
                        LoggerUtils.getLogger().debug("登陆界面被finish");
                        return;
                    }
                    mMvpView.checkApiVersionSuccess();


                } else {
                    hideLoadingDialog();
                }
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("服务状态监测失败:" + throwable.toString());
                showHintDialog("服务状态监测失败:" + throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    private boolean checkApiVersion(String apiVersionValue) {
        int apiVersion = Integer.parseInt(apiVersionValue.substring(apiVersionValue.lastIndexOf(".") + 1));
        if (apiVersion >= 56) {
            return true;
        } else {
            showHintDialog("请更新POS中间件，版本号最低为 5.3.7.12.56");
            return false;
        }
    }

    public void appCheckUpdate(String applicationId, String version) {
        showLoadingDialog();
        AppService.checkUpdate(applicationId, version, new Callback<CheckUpdateResultData>() {
            @Override
            public void onSuccess(CheckUpdateResultData checkUpdateResultData) {
                hideLoadingDialog();
                if (checkUpdateResultData == null) {
                    return;
                }

                showHintConfirm(String.format("检查APP更新成功，当前版本：%s，最新版本：%s，是否更新？", version, checkUpdateResultData.getApkVersion()), new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        apkDownload(checkUpdateResultData.getApkFile());
                    }

                    @Override
                    public void onError(String err) {

                    }
                });
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().error("检查APP更新失败：" + throwable.toString());
                showHintDialog("检查APP更新失败：" + throwable.getMessage());
            }
        });
    }

    private void apkDownload(final String fileName) {
        final ProgressDialog dialog = new ProgressDialog(mMvpView.getContext());
        dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setCancelable(true);
        dialog.setTitle("正在下载");
        dialog.setMessage("请稍后...");
        dialog.setProgress(0);
        dialog.setMax(100);
        dialog.show();

        UpdateService.download(fileName, new UpdateService.UpdateCallback() {
            @Override
            public void onSuccess() {
                dialog.dismiss();

                if (!Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
                    return;
                }

                File file = new File(UpdateService.DOWNLOAD_PATH + fileName);
                try {
                    LoggerUtils.getLogger().info("APP自动更新安装文件目录：" + file);
                    mMvpView.installApk(file);
                } catch (Exception e) {
                    LoggerUtils.getLogger().info("APP自动更新获取打开方式错误" + e.toString());
                    showHintDialog("APP自动更新获取打开方式错误" + e.getMessage());
                }
            }

            @Override
            public void onProgress(int progress) {
                dialog.setProgress(progress);
            }

            @Override
            public void onFailure() {
                dialog.dismiss();
            }
        });
    }

    public void login(String userCode, String password) {
        if (!isViewAttached()) {
            LoggerUtils.getLogger().debug("登陆界面异常退出");
            return;
        }
        if (mMvpView.performQuitBack(userCode)) {
            return;
        }

        //针对厦门国贸美岁，如果是海信设备允许登陆
        if ("com.hisense.hipos.xmums".equals(mMvpView.getActivity().getPackageName())) {
            String info = AppHelper.getBaseSysInfo(mMvpView.getContext());
            try {
                JSONObject jsonInfo = new JSONObject(info);
                String vendor = jsonInfo.getString("Vendor").toLowerCase();
                if (!"hisense".equals(vendor)) {
                    hideLoadingDialog();
                    showHintDialog("请使用海信智能POS");
                    return;
                }
            } catch (Exception e) {
                showHintDialog("获取设备信息异常:" + e.toString());
                hideLoadingDialog();
                return;
            }
        }

        String uuid = SystemConfig.getBluetoothAddress();
        if (StringUtils.isEmpty(uuid)) {
            hideLoadingDialog();
            CommonTools.toastShort("无法获取设备ID");
            return;
        }

        if (StringUtils.isEmpty(userCode) || StringUtils.isEmpty(password)) {
            hideLoadingDialog();
            CommonTools.toastShort("用户名或密码不能为空");
            return;
        }
        // TODO 国美大屏暂时取消检查
        if ("wxpay".equals(Custom.current().getName()) && WxConstants.reInit()) {
            hideLoadingDialog();
            showHintDialog("微信初始化失败，请检查中间件配置是否正确，然后重启应用");
            return;
        }

        showLoadingDialog();
        AuthService.login(userCode, password, uuid, new Callback<LoginResultData>() {
            @Override
            public void onSuccess(LoginResultData loginResult) {
                LoggerUtils.getLogger().debug("登录接口回调成功");
                if (mMvpView != null) {
                    mMvpView.loginSuccess(loginResult);
                }
            }

            @Override
            public void onError(final Throwable t) {
                LoggerUtils.getLogger().error("登录失败:" + t.toString());
                showHintDialog("登录失败:" + t.getMessage());
                hideLoadingDialog();
            }
        });
    }

    public void shopList() {
        showLoadingDialog();
        ShopService.shopList(new Callback<ShopInfo>() {
            @Override
            public void onSuccess(ShopInfo shopInfo) {
                getMainSaleModel().shopInfo(shopInfo);

                Context.address(shopInfo.getAddress());
                Context.telephone(shopInfo.getTelephone());
                //传流水需要orgcode,获取到以后再去重传
                reSubmit();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("获取门店信息异常：" + throwable.toString());
                showHintDialog("获取门店信息异常：" + throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    public void queryPrivileges(LoginResultData loginResult) {
        showLoadingDialog();
        AuthService.queryPrivileges(loginResult.getUserId(), new Callback<User>() {
            @Override
            public void onSuccess(User user) {
                hideLoadingDialog();
                if (!isViewAttached()) {
                    LoggerUtils.getLogger().debug("登陆界面异常退出");
                    return;
                }
                mMvpView.queryPrivilegesSuccess(loginResult, user);
            }

            @Override
            public void onError(Throwable throwable) {
                showHintDialog(throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    private void reSubmit() {

        if (saleApplicationService == null) {
            if (!isViewAttached()) {
                return;
            }
            saleApplicationService = new SaleApplicationService(mMvpView.getContext());
        }

        showLoadingDialog();
        saleApplicationService.reSubmit(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                CommonTools.toastShort(msg);
                hideLoadingDialog();
            }

            @Override
            public void onError(String err) {
                CommonTools.toastShort(err);
                hideLoadingDialog();
            }
        });
    }

    /**
     * 预登陆：检测是否能够登陆
     */
    public void preLogin() {
        showLoadingDialog();
        ProductKey productKey = new ProductKey();
        String info = productKey.authInfo();
        //一台设备只能被激活一次，已经被激活的设备卸载程序重新安装，锁仍然存在，所以要先通过调用getinfo接口获取锁信息
        //若锁存在，直接进行逻辑判断，若锁不存在，调用激活方法
        //如果已被激活，检测支付方式信息是否被授权
        if (null != info) {
            getAuthInfo(productKey);
            LoggerUtils.getLogger().debug("preLogin:getAuthInfo");
        } else {
            //如果从未被激活过，首先调用激活方法，激活成功后再去检测支付方式是否被授权
            activate(productKey, true);
            LoggerUtils.getLogger().debug("preLogin:activate");
        }
    }

    /**
     * 在线激活
     */
    private void activate(ProductKey productKey, boolean isFirst) {
        productKey.activate(isFirst, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                GeneralSettings.INSTANCE.setIsActivate(true);
                getAuthInfo(productKey);
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                GeneralSettings.INSTANCE.setIsActivate(false);

                String filePth = Environment.getExternalStorageDirectory().getPath() + "/android/data/installed.txt";
                String dateStr = FileUtils.readFileContent(filePth);
                if (null == dateStr) {
                    showHintDialog("产品激活/升级失败，试用期已过，请联系开发商");
                    return;
                }
                Date installDate;
                try {
                    installDate = DateUtils.dateParse(Des.decrypt2Base64("ilovehisense", dateStr));
                } catch (Exception e) {
                    LoggerUtils.getLogger().debug("日期格式化失败" + e.toString());
                    showHintDialog("产品激活/升级失败，试用期已过，请联系开发商");
                    return;

                }
                Date expireDate = DateUtils.dayAddAndSub(installDate, 60);
                String expireStr = DateUtils.dateFormat(expireDate);
                if (DateUtils.compareTime(installDate, 60)) {
                    showHintConfirm("产品激活/升级失败，试用期至" + expireStr + "，请尽快联系开发商", new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            checkApiVersion();
                        }

                        @Override
                        public void onError(String err) {

                        }
                    });

                } else {
                    showHintDialog("产品激活/升级失败，试用期已过，请联系开发商");
                }
            }
        });
    }


    /**
     * 在线获取授权信息并校验有效性
     */
    private void getAuthInfo(ProductKey productKey) {
        productKey.getPayAuthInfo(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                authInfo(productKey);
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                showHintDialog("授权信息获取异常，请联系开发商");
            }
        });
    }


    /**
     * 检验本地授权信息是否过期
     */
    private void authInfo(ProductKey productKey) {
        hideLoadingDialog();
        AuthorizationInfo authorizationInfo = com.alibaba.fastjson.JSONObject.parseObject(GeneralSettings.INSTANCE.getAuthorization(), AuthorizationInfo.class);
        for (FeatureInfo featureInfo : authorizationInfo.getFeature()) {

            if (featureInfo.getLicenseType().equals("expiration") &&
                    BigDecimalUtils.isLess(featureInfo.getExpDate(), String.valueOf(System.currentTimeMillis() / 1000))) {

                showHintConfirm("存在支付方式未被授权，点击确定继续使用，点击取消进行升级", new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        checkApiVersion();
                    }

                    @Override
                    public void onError(String err) {
                        activate(productKey, false);
                        LoggerUtils.getLogger().debug("存在支付方式未被授权，暂不使用");
                    }
                });
                return;
            }
        }

        checkApiVersion();
    }
}
