package com.hisense.hiaposcmp.presenter.pay;

import android.content.Intent;
import android.os.Bundle;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.Card;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardPayResult;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.vip.VerifyType;
import com.hisense.crm.webapi.card.CardService;
import com.hisense.crm.webapi.trans.TransService;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.RefundViewModel;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.view.IView.ICardPayView;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.webapi.Callback;
import com.hisense.webapi.RetryCallback;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;

/**
 * 卡支付
 *
 * @author dukunbiao(null)  2019-07-25  重构
 */
public class CardPayPresenter extends BasePresenter<ICardPayView> {

    private PayMethod mSelectedPayMethod;
    private ArrayList<RefundViewModel> listPayments;
    private CardNo mCardNo;
    private Card mCard;
    private CardSecurity mCardSecurity;
    private String mCardFaceNo;
    private String fSsTotal;

    public void initSelectedPayMethod(Intent intent) {
        Bundle bundle = intent.getExtras();
        if (bundle == null) {
            mSelectedPayMethod = getSale().getPayMethod();
        } else {
            mSelectedPayMethod = (PayMethod) bundle.getSerializable("payMethod");
            if (bundle.containsKey("payments")) {
                listPayments = (ArrayList) bundle.getSerializable("payments");
            }
            if (bundle.containsKey("fSsTotal")) {
                fSsTotal = bundle.getString("fSsTotal");
            }
        }
    }

    public void queryVerifyType(CardNo cardNo) {
        boolean iseCard = cardNo.getType() != null && cardNo.getType().getTypeCode() == CardNoType.PayCode.getTypeCode();
        mCardNo = cardNo;
        //先看是不是电子卡, 电子卡看开关, 其他请求接口查看验证类型
        if (iseCard) {
            if (GeneralSettings.INSTANCE.isEPrepaidCardQueryByPassword()) {
                mMvpView.needVerifyType(true, null);
            } else {
                cardQuery(cardNo, null);
            }
            return;
        }

        showLoadingDialog();
        VipService.queryVerifyType(cardNo, mSelectedPayMethod.remark(), new Callback<VerifyType>() {
            @Override
            public void onSuccess(VerifyType verifyType) {
                hideLoadingDialog();
                //都不需要验证
                if (!verifyType.needVerify()) {
                    cardQuery(cardNo, null);
                } else {
                    mMvpView.needVerifyType(false, verifyType);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                mMvpView.queryVerifyTypeFail(throwable);
                finishCardPay("查询验证方式错误", throwable.getMessage());
            }
        });
    }

    public void cardQuery(String vipNo) {
        if (StringUtils.isEmpty(vipNo)) {
            showHintDialog("动态码为空");
            return;
        }

        cardQuery(CardNo.PayCode(vipNo), null);
    }

    public void cardQuery(CardSecurity cardSecurity) {
        cardQuery(mCardNo, cardSecurity);
    }

    private void cardQuery(CardNo cardNo, CardSecurity cardSecurity) {
        mCardSecurity = cardSecurity;

        showLoadingDialog();
        CardService.query(cardNo, cardSecurity, cardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEPrepaidCardQueryByPassword(), new Callback<Card>() {
            @Override
            public void onSuccess(final Card card) {
                mCard = card;
                hideLoadingDialog();

                if (cardNo.getType() != null && cardNo.getType().getTypeCode() == CardNoType.PayCode.getTypeCode()) {
                    mCardNo = CardNo.PayCode(card.vipCardNo());
                } else {
                    mCardNo = CardNo.VipCode(card.vipCardNo());
                }
                mCardFaceNo = card.cardFaceNo();
                //储值卡退款
                if (listPayments != null) {
                    String maxRefund = getMaxRefund(mSelectedPayMethod);
                    boolean returnCard = GeneralSettings.INSTANCE.isReturnCard();
                    String suggestAmount = getReturnSale().getSuggestAmount(mSelectedPayMethod, mCardFaceNo);
                    maxRefund = returnCard ? suggestAmount : maxRefund;
                    mMvpView.cardQueryRefund(card, fSsTotal, maxRefund);
                    return;

                }
                //支付时若金额为0,则提示,不继续执行,退款不用,所以放在退款判断之后
                if (card.usableTotal() == 0) {
                    LoggerUtils.getLogger().debug("CardService此卡可用金额为0");
                    showHintDialog("此卡可用金额为0");
                    return;
                }
                //易收银需按照CRM返回限制储值卡使用范围
                if (!CommonUtils.isUniformStyle() && card.useLimit()) {
                    useLimit(card);
                } else {
                    mMvpView.cardQuerySuccess(card);
                }
            }

            @Override
            public void onError(Throwable t) {
                finishCardPay("储值卡查询错误", t.getMessage());
            }
        });
    }

    private void useLimit(Card card) {
        LoggerUtils.getLogger().debug("需要查询限制金额");
        showLoadingDialog();
        CardService.useLimit(CardNo.VipCode(card.vipCardNo()), getSale(), card, new Callback<Card>() {
            @Override
            public void onSuccess(Card card) {
                hideLoadingDialog();
                LoggerUtils.getLogger().debug("限制金额为:" + card.usableTotal());
                if (card.usableTotal() <= 0) {
                    showHintDialog("此卡在当前商品中可用金额为0");
                } else {
                    mMvpView.cardQuerySuccess(card);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                finishCardPay("查询限制金额错误", throwable.getMessage());
            }
        });
    }

    public void cardPay(Intent data) {
        Bundle bundle = data.getExtras();

        String cardWfTotal = bundle.getString("cardYfTotal");

        if (BigDecimalUtils.isLessAndEquals(getSale().getTotalUnPaid(), "0.001")) {
            showHintDialog("剩余应付金额为0，不能再进行支付");
            return;
        }

        try {
            cardWfTotal = BigDecimalUtils.formatScale(cardWfTotal);
        } catch (Exception e) {
            showHintDialog("未输入支付金额，或金额不合法");
            return;
        }

        if (BigDecimalUtils.isGreater(String.valueOf(cardWfTotal), getSale().getTotalUnPaid())) {
            showHintDialog("输入金额过大，请确保支付金额不大于应付金额，谢谢");
            LoggerUtils.getLogger().debug("支付金额过大，本次支付金额:" + cardWfTotal + ", 未付合计:" + (getSale().getTotalUnPaid()));
            return;
        }

        // 储值卡支付前，保存流水号。避免储值卡支付撤销后，导致流水号重复。
        SaleNoService.save(App.getContext(), Config.org_code, Config.pos_no, getSale().getSaleNo());

        String outTradeNo = CommonUtils.genOutTradeNo(Config.org_code, getSale().getSaleNo());
        SalePayModel payModel = new SalePayModel(mSelectedPayMethod, String.valueOf(cardWfTotal), outTradeNo, getSale().getSaleNo(), getSale().getPays().size() + 1);

        cardPay(mCard, mCardNo, payModel);
    }

    public void cardPay(Card card) {
        //前台取返回信息中 vipCardNo 参数，对应于 储值卡支付接口 cardNo参数，同时储值卡支付接口请求参数 cardNoType 传参为 7-付款码
        String totalPaid = BigDecimalUtils.isGreater(String.valueOf(card.usableTotal()), getSale().getTotalUnPaid()) ? getSale().getTotalUnPaid() : String.valueOf(card.usableTotal());
        String outTradeNo = CommonUtils.genOutTradeNo(Config.org_code, getSale().getSaleNo());
        SalePayModel payModel = new SalePayModel(mSelectedPayMethod, totalPaid, outTradeNo, CommonUtils.generateCardSaleNo(), getSale().getPays().size() + 1);

        cardPay(card, CardNo.PayCode(card.vipCardNo()), payModel);
    }

    private void cardPay(Card card, CardNo cardNo, SalePayModel payModel) {
        showLoadingDialog();
        CardService.pay(cardNo, mCardSecurity, payModel, cardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEPrepaidCardQueryByPassword(), new Callback<CardPayResult>() {

            @Override
            public void onSuccess(CardPayResult cardPayResult) {
                hideLoadingDialog();
                if (card.useLimit()) {
                    LoggerUtils.getLogger().debug("pay_成功_已使用");
                    getSale().cardShare(Double.parseDouble(payModel.getTotalPaid()));
                }
                cardPaySuccess(card, cardNo, payModel, cardPayResult);
            }

            @Override
            public void onError(Throwable throwable) {
                cardPayError(throwable, card, cardNo, payModel);
            }
        });
    }

    private void cardPaySuccess(Card card, CardNo cardNo, SalePayModel payModel, CardPayResult cardPayResult) {
        payModel.setBuyerLogonId(card.cardFaceNo());
        payModel.setOutTradeNo(cardPayResult.outTradeNo());
        payModel.setTradeNo(cardPayResult.refNo());
        payModel.setPayTime(cardPayResult.tranTime());
        payModel.setTotalDscSelf("0.00");
        payModel.setTotalDscChannel("0.00");
        payModel.setBalance(String.valueOf(cardPayResult.balance()));
        payModel.setCardSecurity(mCardSecurity);
        payModel.setCardNo(cardNo);
        getSale().addPay(payModel);

        mMvpView.cardPaySuccess();
    }

    private void cardPayError(Throwable throwable, Card card, CardNo cardNo, SalePayModel payModel) {
        if (card.useLimit()) {
            LoggerUtils.getLogger().debug("pay_失败_未使用");
        }
        LoggerUtils.getLogger().error("储值卡支付异常:" + throwable.toString());

        hideLoadingDialog();
        Throwable cause = throwable.getCause();
        if (NetUtils.isNetException(cause)) {
            showLoadingDialog();
            mMvpView.isNotShowExitDialog(true);
            Service.online(100, 3000, new RetryCallback<OnlineResultData>() {
                @Override
                public void onSuccess(OnlineResultData service) {
                    mMvpView.isNotShowExitDialog(false);
                    LoggerUtils.getLogger().info("储值卡支付结果查询");

                    transCheck(card, cardNo, payModel);
                }

                @Override
                public void onProgress(int i, final String s) {
                    mMvpView.showProgressDialogNoHide(s);
                    LoggerUtils.getLogger().debug(s);
                }

                @Override
                public void onError(Throwable throwable) {
                    mMvpView.hideProgressDialog();
                    mMvpView.isNotShowExitDialog(false);
                    finishCardPay("查询支付状态错误", throwable.getMessage());
                }
            });
        } else {
            finishCardPay("储值卡支付失败", throwable.getMessage());
        }
    }

    private void transCheck(Card card, CardNo cardNo, SalePayModel payModel) {
        showLoadingDialog();
        TransService.check(cardNo, payModel, new Callback<CardPayResult>() {
            @Override
            public void onSuccess(CardPayResult cardPayResult) {
                hideLoadingDialog();
                if (card.useLimit()) {
                    getSale().cardShare(Double.valueOf(payModel.getTotalPaid()));
                    LoggerUtils.getLogger().debug("query_成功_已使用");
                }
                cardPayResult.balance(BigDecimalUtils.subtract(card.usableTotal(), Double.parseDouble(payModel.getTotalPaid())));

                cardPaySuccess(card, cardNo, payModel, cardPayResult);
            }

            @Override
            public void onError(Throwable throwable) {
                if (card.useLimit()) {
                    LoggerUtils.getLogger().debug("query_失败_未使用");
                }

                int code = ((WebApiException) throwable).code();
                String tip;
                if (code == 404) {
                    //重新发起支付
                    cardPay(card, cardNo, payModel);
                    return;
                } else if (code == 2) {
                    tip = "支付失败,请重新支付";
                } else {
                    tip = "储值卡查询支付结果:" + throwable.getMessage();
                }

                finishCardPay("错误", tip);
            }
        });
    }

    private void finishCardPay(String title, String msg) {
        hideLoadingDialog();
        LoggerUtils.getLogger().error(title + ":" + msg);
        showHintDialog(title, msg);

//        if (BaseMvpActivity.isUniformStyle()) {
//            showHintDialog(title, msg);
//        } else {
//            showHintDialog(title, msg, (m) -> mMvpView.getActivity().finish());
//        }
    }

    public void cardRefund(Intent data) {
        Bundle bundle = data.getExtras();
        final String amount = bundle.getString("cardYfTotal");
        mMvpView.showProgressDialog("正在处理储值卡退款, 请稍后");

        SalePayModel salePayModel = new SalePayModel();
        salePayModel.setSerialNo(getReturnSale().getPays().size());
        salePayModel.setPayCode(mSelectedPayMethod.code());
        salePayModel.setPayName(mSelectedPayMethod.name());
        salePayModel.setRemark(mSelectedPayMethod.remark());
        salePayModel.setPayType(mSelectedPayMethod.payType());
        salePayModel.setTotalPaid(amount);
        salePayModel.setSaleNoTemp(getReturnSale().getSaleNo());
        salePayModel.setPayTime(getReturnSale().getSaleTime());

        String outTradeNo = null;
        CardService.returns(mCardNo, mCardSecurity, getReturnSale().oldSaleNo(), salePayModel, outTradeNo, mCardNo.getTypeCode() != 7 ||
                GeneralSettings.INSTANCE.isEPrepaidCardQueryByPassword(), new Callback<CardPayResult>() {
            @Override
            public void onSuccess(CardPayResult result) {
                mMvpView.hideProgressDialog();
                SalePayModel salePayModel = new SalePayModel();
                salePayModel.setPayCode(mSelectedPayMethod.code());
                salePayModel.setPayName(mSelectedPayMethod.name());
                salePayModel.setPayType(mSelectedPayMethod.payType());
                salePayModel.setRemark(mSelectedPayMethod.remark());
                salePayModel.setBuyerLogonId(mCardFaceNo);
                salePayModel.setTradeNo(result.refNo());
                salePayModel.setOutTradeNo(result.refNo());
                salePayModel.setTotalPaid(amount);
                salePayModel.setTotalRecv(amount);
                salePayModel.setSaleNoTemp(getReturnSale().getSaleNo());
                salePayModel.setBalance(String.valueOf(result.balance()));
                getReturnSale().getPays().add(salePayModel);
                mMvpView.cardRefundSuccess();
            }

            @Override
            public void onError(Throwable throwable) {
                mMvpView.hideProgressDialog();
                showHintDialog("储值卡退款失败：" + throwable.getMessage() + getReturnSale().getSaleNo());
            }
        });
    }

    /**
     * 返回指定支付方式的最大可退金额。
     *
     * @param payMethod 当前选择的支付方式
     * @return 指定支付方式的最大可退金额
     */
    public String getMaxRefund(PayMethod payMethod) {
        String maxRefund = "0";

        if (payMethod == null) {
            return maxRefund;
        }

        if (payMethod.isCard()) {
            // 银行卡、储值卡、现金可退至储值卡账户
            maxRefund = getMaxRefundByRemark(PayMethod.CARD);
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.UNION_PAY));
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.CASH));
            maxRefund = BigDecimalUtils.subtract(maxRefund, getOverRefundByRemark(PayMethod.VIP_POINTS));
        }

        return maxRefund;
    }

    /**
     * 返回指定支付方式的合计可退金额。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的最大可退金额
     */
    private String getMaxRefundByRemark(String payMethod) {
        return BigDecimalUtils.subtract(getTotalReceivedByRemark(payMethod), getTotalRefundByRemark(payMethod));
    }

    /**
     * 返回指定支付方式的合计支付金额。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的合计支付金额
     */
    private String getTotalReceivedByRemark(String payMethod) {
        String total = "0";

        if (payMethod != null) {
            for (int i = 0; i < listPayments.size(); i++) {
                if (listPayments.get(i).payment().getRemark().equals(payMethod)) {
                    total = BigDecimalUtils.add(total, listPayments.get(i).totalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 返回指定支付方式的合计退款金额。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的合计退款金额
     */
    private String getTotalRefundByRemark(String payMethod) {
        String total = "0";

        if (payMethod != null) {
            for (SalePayModel payment : getReturnSale().getPays()) {
                if (payMethod.equals(payment.getRemark())) {
                    total = BigDecimalUtils.add(total, payment.getTotalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 返回指定支付方式(储值卡/积分)的溢退金额。例如：支付金额为150，退款200，则溢退金额为50。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的溢退金额
     */
    private String getOverRefundByRemark(String payMethod) {
        // 根据 HIPOSAPP-509 制定的规则，只有储值卡/积分两种支付方式允许将消费时其它支付方式的支付金额退回
        // 到储值卡/积分账户。
        if (!PayMethod.isCard(payMethod) && !PayMethod.isVipPoints(payMethod) && !PayMethod.isScrmVipPoints(payMethod)) {
            return "0";
        }

        String received = getTotalReceivedByRemark(payMethod);
        String refund = getTotalRefundByRemark(payMethod);
        return BigDecimalUtils.isGreater(refund, received) ? BigDecimalUtils.subtract(refund, received) : "0";
    }
}
