package com.hisense.hiaposcmp.presenter.pay;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.adapter.VipCoupon;
import com.hisense.hiaposcmp.adapter.VipCouponListAdapter;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.view.IView.ICouponView;
import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.ICoupon;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.coupon.UsableCouponList;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.coupon.CouponService;
import com.hisense.webapi.Callback;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

/**
 * @author xly on 2019-01-04.
 * @author dukunbiao(null)  2019-07-25  重构
 */
public class CouponPresenter extends BasePresenter<ICouponView> {

    private PayMethod mSelectedPayMethod;
    private String vipCardNo;

    public void initData(Intent intent) {
        initSelectedPayMethod(intent);

        vipCardNo = intent.getStringExtra("vipCardNo");
    }

    public void initSelectedPayMethod(Intent intent) {
        Bundle bundle = intent.getExtras();
        if (bundle == null || !bundle.containsKey("payMethod")) {
            mSelectedPayMethod = getSale().getPayMethod();
            return;
        }

        mSelectedPayMethod = (PayMethod) bundle.getSerializable("payMethod");
        if (mSelectedPayMethod == null) {
            mSelectedPayMethod = getSale().getPayMethod();
        }
    }

    public void couponCalc(String couponCode, String checkCode) {
        if (StringUtils.isEmpty(couponCode)) {
           showHintDialog("券号不能为空");
            return;
        }

        Coupon coupon = new Coupon.Builder().coupon(couponCode, checkCode).build();
        couponCalc(coupon, -1);
    }

    public void couponCalc(Coupon coupon, int position) {
        final String msgTip;
        if (position >= 0) {
            msgTip = ",位置:" + position;
        } else {
            msgTip = "";
        }

        User user = new User(Config.user_code);
        String mCardFaceNo = null;
        if (getSale().getVip() != null) {
            mCardFaceNo = getSale().getVip().card() == null ? null : getSale().getVip().card().cardFaceNo();
        }

        if (getSale().getUsableCoupons().items() != null && getSale().getUsableCoupons().items().size() > 0) {
            for (UsableCoupon usableCoupon : getSale().getUsableCoupons().items()) {
                if (usableCoupon.coupon().no().equals(coupon.no())) {
                   showHintDialog("请勿重复添加此券");
                    return;
                }
                if (usableCoupon.coupon().type() != coupon.type()) {
                   showHintDialog("请勿同时使用金额券和折扣券");
                    return;
                }
            }
        }

        try {
           showLoadingDialog();
            CouponService.calc(coupon, mCardFaceNo, user, new Callback<UsableCoupon>() {
                @Override
                public void onSuccess(UsableCoupon usableCoupon) {
                   hideLoadingDialog();
                    if (usableCoupon == null) {
                       showHintDialog(coupon.getQNo() + "券不可用" + msgTip);
                        LoggerUtils.getLogger().error(coupon.getQNo() + "券不可用" + msgTip);
                        return;
                    }

                    if (getSale().getUsableCoupons() == null) {
                        getSale().setUsableCoupons(new UsableCouponList());
                    }

                    if (usableCoupon.coupon().available() == 0.0) {
                        LoggerUtils.getLogger().debug("此券可核销金额为" + usableCoupon.useValue() + msgTip);
                       showHintDialog("此券可核销金额为" + usableCoupon.useValue() + msgTip);
                        return;
                    }

                    try {
                        getSale().getUsableCoupons().use(usableCoupon);
                    } catch (Exception e) {
                        LoggerUtils.getLogger().error("券计算异常:" + e.toString());
                       showHintDialog("此券不可用:" + e.getMessage());
                        return;
                    }

                    try {
                        if (usableCoupon.useValue() > 0) {
                            mMvpView.couponCalcSuccess(position, usableCoupon);
                        } else {
                            getSale().getUsableCoupons().remove(usableCoupon);
                           showHintDialog("可核销金额为" + usableCoupon.useValue());
                            LoggerUtils.getLogger().debug("券不可用,可核销金额为" + usableCoupon.useValue());
                        }
                    } catch (Exception e) {
                        LoggerUtils.getLogger().error("券移除异常:" + e.toString());
                       showHintDialog("券移除异常:" + e.getMessage());
                    }
                }

                @Override
                public void onError(Throwable throwable) {
                    LoggerUtils.getLogger().error("券计算异常:" + throwable.toString());
                    if (isViewAttached()) {
                       showHintDialog("此券不可用:" + throwable.getMessage());
                       hideLoadingDialog();
                    }
                }
            });
        } catch (Exception e) {
            if (isViewAttached()) {
               showHintDialog("此券不可用:" + e.getMessage());
               hideLoadingDialog();
            }
            LoggerUtils.getLogger().error("券计算异常:" + e.toString());
        }
    }

    public void couponUse1(UsableCoupon usableCoupon) {
        couponUse1(Arrays.asList(usableCoupon), usableCoupon.useValue());
    }

    public void couponUse1(double useValue) {
        couponUse1(getSale().getUsableCoupons().items(), useValue);
    }



    public void offlineConsumeCerti(String qNo) {
        if (StringUtils.isEmpty(qNo)) {
           showHintDialog("券号不能为空");
            return;
        }

        if (StringUtils.isEmpty(vipCardNo)) {
           showHintDialog("会员号为空，请重试");
            return;
        }

       showLoadingDialog();
        com.hisense.crm.webapi.coupon.CouponService.scrmCouponUse(qNo, vipCardNo, new Callback<UsableCoupon>() {
            @Override
            public void onSuccess(UsableCoupon usableCoupon) {
                if (usableCoupon == null) {
                   showHintDialog("已超出用券范围");
                    return;
                }
                if (usableCoupon.coupon().available() == 0.0) {
                   showHintDialog("券核销金额为0");
                    return;
                }

                couponPaySuccess(String.valueOf(usableCoupon.coupon().available()), usableCoupon.coupon().no(), CardNo.VipCode(vipCardNo));
                getSale().getUsableCouponScrm().add(usableCoupon);

                mMvpView.couponUse1Success(Arrays.asList(usableCoupon), null);
               hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("SCRM券核销异常:" + throwable.toString());
               showHintDialog("SCRM券核销异常:" + throwable.getMessage());
               hideLoadingDialog();
            }
        });
    }

    public void couponUse1(List<UsableCoupon> usableCoupons, double useValue) {
        if (ListUtils.isEmpty(getSale().getUsableCoupons().items())) {
           showHintDialog("未选择券");
            return;
        }

       showLoadingDialog();
        CouponService.use1(usableCoupons, new Callback<Collection<ICoupon>>() {
            @Override
            public void onSuccess(Collection<ICoupon> coupons) {
                Collection<Coupon> couponsTmp = new ArrayList<>();
                for (ICoupon coupon : coupons) {
                    couponsTmp.add((Coupon) coupon);
                }

                String useTotal = String.valueOf(useValue);
                if (BigDecimalUtils.isGreater(useTotal, getSale().getTotalUnPaid())) {
                    useTotal = getSale().getTotalUnPaid();
                }

                for (UsableCoupon usableCoupon : usableCoupons) {
                    if (BigDecimalUtils.isGreater(useTotal, String.valueOf(usableCoupon.useValue()))) {
                        couponPaySuccess(String.valueOf(usableCoupon.useValue()), usableCoupon.coupon().no(), null);

                        useTotal = BigDecimalUtils.subtract(useTotal, String.valueOf(usableCoupon.useValue()));
                    } else {
                        couponPaySuccess(useTotal, usableCoupons.get(usableCoupons.size() - 1).coupon().no(), null);
                        break;
                    }
                }

                mMvpView.couponUse1Success(usableCoupons, couponsTmp);
               hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("券核销异常:" + throwable.toString());
               showHintDialog("券核销异常:" + throwable.getMessage());
               hideLoadingDialog();
            }
        });
    }

    public void queryVipCoupons(String cardFaceNo) {
       showLoadingDialog();
        com.hisense.hipos.webapi.vip.VipService.getCoupons(cardFaceNo, new Callback<List<Coupon>>() {
            @Override
            public void onSuccess(List<Coupon> coupons) {
               hideLoadingDialog();
                mMvpView.queryVipCouponsSuccess(coupons);
            }

            @Override
            public void onError(Throwable throwable) {
               hideLoadingDialog();
                LoggerUtils.getLogger().error("查询券异常:" + throwable.toString());
               showHintDialog("查询券异常:" + throwable.getMessage());
            }
        });
    }

    private void couponPaySuccess(String totalPaid, String payNo, CardNo cardNo) {
        SalePayModel payModel = new SalePayModel();
        payModel.setPayCode(mSelectedPayMethod.code());
        payModel.setPayName(mSelectedPayMethod.name());
        payModel.setRemark(mSelectedPayMethod.remark());
        payModel.setPayType(mSelectedPayMethod.payType());
        payModel.setBuyerLogonId(payNo);
        payModel.setOutTradeNo("");
        payModel.setTradeNo("");
        payModel.setPayTime(DateUtils.dateFormat());
        payModel.setTotal(totalPaid);
        payModel.setTotalRecv(totalPaid);
        payModel.setTotalPaid(totalPaid);
        payModel.setTotalDscSelf("0.00");
        payModel.setTotalDscChannel("0.00");
        payModel.setCardNo(cardNo);
        getSale().addPay(payModel);

        LoggerUtils.getLogger().debug(payModel.getPayName() + payModel.getRemark() + "付款金额：" + payModel.getTotalRecv() +
                ",已付金额：" + getSale().getTotalPaid() +
                ",未付金额：" + getSale().getTotalUnPaid());
    }

    /**
     * 调用券计算接口
     */
    public void calCoupon(VipCouponListAdapter vipCouponListAdapter, List<VipCoupon> vipCoupons) {
        User user = new User(Config.user_code);
        final List<Coupon> coupons = new ArrayList<Coupon>();
        for (int i = 0; i < getSale().getUsableCoupons().items().size(); i++) {

            for (int j = i+1; j < getSale().getUsableCoupons().items().size(); j++) {
                if (getSale().getUsableCoupons().items(j).coupon().no().equals(getSale().getUsableCoupons().items(i).coupon().no())) {
                    showHintDialog("请勿重复添加此券");
                    return;
                }
                if (getSale().getUsableCoupons().items(j).coupon().type() != getSale().getUsableCoupons().items(i).coupon().type()) {
                    showHintDialog("请勿同时使用金额券和折扣券");
                    return;
                }

            }
            coupons.add(getSale().getUsableCoupons().items(i).coupon());
        }
        String vipCardNo = getSale().getVip() == null ? null : getSale().getVip().getCard().vipCardNo();
        showLoadingDialog();
        CouponService.calcMulti(coupons, vipCardNo, user, new Callback<List<UsableCoupon>>() {
            @Override
            public void onSuccess(final List<UsableCoupon> usableCoupons) {
                hideLoadingDialog();
                double available = vipCouponListAdapter.available();
                getSale().getUsableCoupons().cancelAll();
                vipCouponListAdapter.unSelectAll();
                vipCouponListAdapter.useValue(0);
                for (UsableCoupon usableCoupon : usableCoupons) {
                    getSale().getUsableCoupons().use(usableCoupon);
                    for (int i = 0; i < vipCoupons.size(); i++) {
                        if (vipCoupons.get(i).coupon().getQNo().equals(usableCoupon.coupon().getQNo())) {
                            vipCouponListAdapter.selectUpdate(i, usableCoupon);
                        }
                    }
                    mMvpView.calcCouponsSuccess();
                }
                if (coupons.size() > usableCoupons.size() || available > vipCouponListAdapter.useValue()) {
                    LoggerUtils.getLogger().debug("已选择" + coupons.size() + "张券，余额" + available + "，本笔交易可用" + usableCoupons.size() + "张，可用金额" + vipCouponListAdapter.useValue() + "，是否继续核销？");

                    new AlertDialog.Builder(App.getContext()).setTitle("提示").setMessage("已选择" + coupons.size() + "张券，余额" + available + "，本笔交易可用" + usableCoupons.size() + "张，可用金额" + vipCouponListAdapter.useValue() + "，是否继续核销？")
                            .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    useCoupon(usableCoupons,vipCouponListAdapter.useValue());
                                }
                            })
                            .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                }
                            }).show();
                } else {
                    useCoupon(usableCoupons,vipCouponListAdapter.useValue());
                }
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                CommonTools.showUiMessage(App.getContext(), "券计算异常:" + throwable.getMessage());
                LoggerUtils.getLogger().debug("券计算异常:" + throwable.getMessage());
            }
        });
    }
    /**
     * 券核销
     */
    private void useCoupon(List<UsableCoupon> usableCoupons, double useValue) {
       showLoadingDialog();

        CouponService.use1(usableCoupons, new Callback<Collection<ICoupon>>() {
            @Override
            public void onSuccess(Collection<ICoupon> coupons) {
                Collection<Coupon> couponsTmp = new ArrayList<>();
                for (ICoupon coupon : coupons) {
                    couponsTmp.add((Coupon) coupon);
                }

                String useTotal = String.valueOf(useValue);
                if (BigDecimalUtils.isGreater(useTotal, getSale().getTotalUnPaid())) {
                    useTotal = getSale().getTotalUnPaid();
                }

                for (UsableCoupon usableCoupon : usableCoupons) {
                    if (BigDecimalUtils.isGreater(useTotal, String.valueOf(usableCoupon.useValue()))) {
                        couponPaySuccess(String.valueOf(usableCoupon.useValue()), usableCoupon.coupon().no(), null);

                        useTotal = BigDecimalUtils.subtract(useTotal, String.valueOf(usableCoupon.useValue()));
                    } else {
                        couponPaySuccess(useTotal, usableCoupons.get(usableCoupons.size() - 1).coupon().no(), null);
                        break;
                    }
                }

                mMvpView.couponUse1Success(usableCoupons, couponsTmp);
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                CommonTools.showUiMessage(App.getContext(), "券核销异常" + throwable.getMessage());
            }
        });
    }
    }
