package com.hisense.hiaposcmp.presenter.pay;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.widget.GridView;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.alipay.zoloz.smile2pay.service.Zoloz;
import com.hisense.common.boot.GlobalCallback;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.JSONUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.common.util.XmlUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.webapi.card.CardService;
import com.hisense.crm.webapi.points.PointsService;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.activity.wx.PayCustomActivity;
import com.hisense.hiaposcmp.adapter.PaymethodCustomAdapter;
import com.hisense.hiaposcmp.adapter.PaymethodCustomOtherAdapter;
import com.hisense.hiaposcmp.adapter.SaleProAdapter;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.FeatureConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SettingConstants;
import com.hisense.hiaposcmp.constant.pay.AliConstants;
import com.hisense.hiaposcmp.constant.pay.PayConstants;
import com.hisense.hiaposcmp.constant.pay.WxConstants;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.service.ali.AliService;
import com.hisense.hiaposcmp.service.app.AuthService;
import com.hisense.hiaposcmp.service.payment.AggregatePaymentChannel;
import com.hisense.hiaposcmp.service.payment.PayMethodService;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.service.sale.SaleRepository;
import com.hisense.hiaposcmp.service.wx.WxService;
import com.hisense.hiaposcmp.service.wx.dto.WxDto;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.SystemConfig;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.view.IView.pay.IPayView;
import com.hisense.hipos.coupon.HxPayCoupon;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.pay.PayUi;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.sale.CalcType;
import com.hisense.hipos.webapi.coupon.CouponService;
import com.hisense.hipos.webapi.pay.CancelResultData;
import com.hisense.hipos.webapi.pay.CouponData;
import com.hisense.hipos.webapi.pay.PayData;
import com.hisense.hipos.webapi.pay.PayResultData;
import com.hisense.hipos.webapi.pay.PayService;
import com.hisense.hipos.webapi.sale.CompletePluDscData;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.hipos.webapi.sale.SaleSubmitResultData;
import com.hisense.hipos.webapi.sale.TotalReductionModel;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.pos.payment.PayResult;
import com.hisense.webapi.Callback;
import com.hisense.webapi.RetryCallback;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

/**
 * 微信支付的presenter
 *
 * @author liaoyuping       2018-07-03
 * @author dukunbiao(null)  2018-07-16  增加微信支付业务
 * @author dukunbiao(null)  2019-07-24  重构
 * @author wangyaogang      2019-12-27 HIAPOSCMP-455 标准版支付功能开发,增加特定的支付和撤销方法
 * @author guolina          2020-01-13 HIAPOSCMP-495 易收银调整整单促销接口
 * @author guolina          2020-03-13 HIAPOSCMP-577 邹平圣豪-易收银的微信、支付宝两个支付方式是否可以合并为一个，加授权
 * @author wangyaogang      2020-04-10 HIAPOSCMP-660 会员限量促销，流水上传前调用交易完成接口更新数量
 * @author xieliyan         2020-04-21 修复支付授权金额传入错误问题
 * @author xieliyan         2020-04-23 HIAPOSCMP-707 修复在支付界面按home键闪退问题
 * @author guolina          2020-05-18 HIAPOSCMP-702 吉麦隆淘鲜达营销方案对接涉及慧收银调整增加支付明细和赠券打印
 */
public class PayPresenter extends BasePresenter<IPayView> {

    private SaleProAdapter mProAdapter;
    private static Looper mLooper = null;

    private PayMethod mSelectedPayMethod;
    private String mTotalPaid;
    private String expireDateStr;
    private boolean checkQuery = false;

    private Zoloz zoloz;
    private AuthService authService;


    /**
     * 常用支付方式适配器
     */
    private PaymethodCustomAdapter mPayMethodCustomAdapterOften;
    /**
     * 其他支付方式适配器
     */
    private PaymethodCustomOtherAdapter mPayMethodCustomAdapterOther;
    private SaleApplicationService saleApplicationService;


    public void assignAdapter(SaleProAdapter saleProAdapter) {
        this.mProAdapter = saleProAdapter;
    }

    /**
     * 更新界面钱的显示
     */
    public synchronized void updateTotal() {
        if (getSale() == null) {
            LoggerUtils.getLogger().error("getSale() is null");
            return;
        }

        LoggerUtils.getLogger().info("应收:" + getSale().getTotalRecv()
                + ", 已付" + getSale().getTotalPaid()
                + ", 未付:" + getSale().getTotalUnPaid()
        );

        mMvpView.updateTotal();
    }

    /**
     * 标准版 支付,统一点击支付方式即支付
     *
     * @param payMethod 支付方式
     */
    public void payStd(PayMethod payMethod) {
        if (!NetUtils.isNetworkAvailable()) {
            showHintDialog("网络错误", "请检查网络");
            return;
        }
        if (payMethod == null) {
            showHintDialog("未配置该支付方式，请联系管理员");
            return;
        }
        if (getSale().hasAllPaid() && !GeneralSettings.INSTANCE.isZeroPriceSale()) {
            if (BigDecimalUtils.isEquals(getSale().getTotalPaid(), "0")) {
                showHintDialog("未付金额错误，请返回销售界面重新操作");
            } else {
                showHintDialog("未付金额为0，不允许继续支付");
            }

            return;
        }

        showLoadingDialog();
        mSelectedPayMethod = payMethod;
        mTotalPaid = getSale().getTotalUnPaid();
        getSale().setPayMethod(payMethod);
        LoggerUtils.getLogger().info("支付方式:" + mSelectedPayMethod.name() + "，支付金额:" + getSale().getTotalUnPaid());
        if (mSelectedPayMethod.isTotalReduction()) {
            hideLoadingDialog();
            showHintDialog("当前模式满减仅可作为商品优惠计算");
           /* //设置中是否开启满减券开关
            if (!GeneralSettings.INSTANCE.allowTotalDiscount()) {
                showHintDialog("未启用满减优惠。如需使用请在设置中开启");
                return;
            }

            if (!"支付方式".equals(GeneralSettings.INSTANCE.getTotalDiscountWay())) {
                showHintDialog("当前满减记录方式为商品优惠，如需记录为支付方式请在设置中修改");
                return;
            }

            if (!GeneralSettings.INSTANCE.isNoShopLimits() && !getSale().allowTotalDsc()) {
                showHintDialog("后台设置不允许满减");
                return;
            }

            if (BigDecimalUtils.isGreater(getSale().totalReduction(), "0")) {
                showHintDialog("当前交易已满减");
                return;
            }

            saleTotalReduction();*/
        } else {
            if (!gotoPayMethod()) {
                mMvpView.gotoPayChannel(mTotalPaid);
            }
        }
    }

    /**
     * 更新 待支付金额
     *
     * @param totalPaid 金额
     */
    public void updatePayAmount(String totalPaid) {
        this.mTotalPaid = totalPaid;
    }

    public void pay(PayMethod payMethod) {
        if (!NetUtils.isNetworkAvailable()) {
            showHintDialog("网络错误", "请检查网络");
            mMvpView.setIsClickPayFalse();
            return;
        }
        if (payMethod == null) {
            showHintDialog("未配置该支付方式，请联系管理员");
            mMvpView.setIsClickPayFalse();
            return;
        }
        if (getSale().hasAllPaid() && !GeneralSettings.INSTANCE.isZeroPriceSale()) {
            if (BigDecimalUtils.isEquals(getSale().getTotalPaid(), "0")) {
                mMvpView.setIsClickPayFalse();
                showHintDialog("未付金额错误，请返回销售界面重新操作");
            } else {
                mMvpView.setIsClickPayFalse();
                showHintDialog("未付金额为0，不允许继续支付");
            }

            return;
        }

        showLoadingDialog();
        mSelectedPayMethod = payMethod;
        mTotalPaid = getSale().getTotalUnPaid();
        getSale().setPayMethod(payMethod);
        LoggerUtils.getLogger().info("支付方式:" + mSelectedPayMethod.name() + "，支付金额:" + getSale().getTotalUnPaid());
        if (mSelectedPayMethod.isAliFacePay()) {
            if (BigDecimalUtils.isGreater(getSale().getTotalUnPaid(), String.valueOf(GeneralSettings.INSTANCE.getAliFacePayLimit()))) {
                return;
            }

            getMetaInfo(mSelectedPayMethod);
        } else if (mSelectedPayMethod.isWxFacePay()) {
            if (WxDto.getWxDto().isInit()) {
                if (WxConstants.reInit()) {
                    LoggerUtils.getLogger().debug("支付界面，微信相关参数仍未空");

                    try {
                        AliConstants.init();
                        WxConstants.init();
                        FeatureConstants.initFeatureConstants();
                        SettingConstants.initSettingConstants();
                    } catch (Exception e) {
                        LoggerUtils.getLogger().debug("支付界面微信初始化失败:" + e.toString());
                    }
                }

                //微信刷脸初始化成功才去调用刷脸
                if (!StringUtils.isEmpty(WxDto.getWxDto().getAuthinfo())) {
                    LoggerUtils.getLogger().info("Authinfo 已存在,不需要重复请求;当前Authinfo:" + WxDto.getWxDto().getAuthinfo());
                    getWxPayFaceCode(WxDto.getWxDto().getAuthinfo());
                } else {
                    getWxPayFaceRawData();
                }
            } else {
                mMvpView.setIsClickPayFalse();
                hideLoadingDialog();
                showHintDialog("等待刷脸初始化", "人脸初始化未完成，请稍候再试");
            }
        } else if (mSelectedPayMethod.isTotalReduction()) {
            hideLoadingDialog();
            //设置中是否开启满减券开关
            if (!GeneralSettings.INSTANCE.allowTotalDiscount()) {
                mMvpView.setIsClickPayFalse();
                showHintDialog("未启用满减优惠。如需使用请在设置中开启");
                return;
            }

            if (!"支付方式".equals(GeneralSettings.INSTANCE.getTotalDiscountWay())) {
                mMvpView.setIsClickPayFalse();
                showHintDialog("当前满减记录方式为商品优惠，如需记录为支付方式请在设置中修改");
                return;
            }

            if (!GeneralSettings.INSTANCE.isNoShopLimits() && !getSale().allowTotalDsc()) {
                mMvpView.setIsClickPayFalse();
                showHintDialog("后台设置不允许满减");
                return;
            }

            if (BigDecimalUtils.isGreater(getSale().totalReduction(), "0")) {
                mMvpView.setIsClickPayFalse();
                showHintDialog("当前交易已满减");
                return;
            }
            if ("支付方式".equals(GeneralSettings.INSTANCE.getTotalDiscountWay())) {
                saleTotalReduction();
            }
        } else if (mSelectedPayMethod.isMobilePay()) {
            hideLoadingDialog();
            mMvpView.gotoScanActivity();
        } else {
            if (!gotoPayMethod()) {
                mMvpView.gotoScanActivity();
            }
        }
    }

    private boolean gotoPayMethod() {
        boolean bool = true;
        if (mSelectedPayMethod.isCardType()) {
            mMvpView.gotoCardPayActivity();
        } else if (mSelectedPayMethod.isPointsType() || mSelectedPayMethod.isRemainType()) {
            mMvpView.gotoPointsPayActivity();
        } else if (mSelectedPayMethod.isVipCoupon()) {
            mMvpView.gotoCouponUseActivity();
        } else if (mSelectedPayMethod.isScrmCoupon()) {
            mMvpView.gotoScrmCouponActivity();
        } else if (mSelectedPayMethod.isOther()) {
            mMvpView.gotoOtherPayActivity();
        } else if (mSelectedPayMethod.isCardByPayType()) {
            mMvpView.gotoCardPayActivity();
        } else if (mSelectedPayMethod.isCouponByPayType()) {
            mMvpView.gotoScrmCouponActivity();
        } else {
            bool = false;
        }

        hideLoadingDialog();
        return bool;
    }

    /**
     * 使用选择的支付方式支付。
     */
    public void pay(String totalPaid) {
        if (getSale().hasAllPaid() && !GeneralSettings.INSTANCE.isZeroPriceSale()) {
            if (BigDecimalUtils.isEquals(getSale().getTotalPaid(), "0")) {
                showHintDialog("未付金额错误，请返回销售界面重新操作");
            } else {
                showHintDialog("未付金额为0，不允许继续支付");
            }
            return;
        }

        if (mSelectedPayMethod == null) {
            showHintDialog("支付", "请重新选择支付方式");
            return;
        }

        String mTotalPaid;
        try {
            mTotalPaid = BigDecimalUtils.formatScale(totalPaid);
        } catch (Exception e) {
            showHintDialog("未输入支付金额，或金额不合法");
            return;
        }

        if (BigDecimalUtils.isGreater(mTotalPaid, getSale().getTotalUnPaid())) {
            showHintDialog("输入金额过大，请确保支付金额不大于应付金额，谢谢");
            LoggerUtils.getLogger().info("支付金额过大，本次paytotal:" + mTotalPaid + ", 已付合计:" + (getSale().getTotalPaid()));
            return;
        }

        LoggerUtils.getLogger().info("支付方式:" + mSelectedPayMethod.name() + "，支付金额:" + mTotalPaid);

        this.mTotalPaid = mTotalPaid;
        if (mSelectedPayMethod.isTotalReduction()) {
            // 满减不作处理
            // 判断支付方式是否为满减，必须在支付方式是否为现金前。因为，海信广场设置的满减remark为$cash。
        } else if (mSelectedPayMethod.isCash()) {
            cashPay(mTotalPaid);
        } else {
            if (!gotoPayMethod()) {
                if (AggregatePaymentChannel.isCHANNEL() &&
                        (mSelectedPayMethod.isMobilePay()
                                || mSelectedPayMethod.isAliPay()
                                || mSelectedPayMethod.isWxPay()
                                || mSelectedPayMethod.isUnionPay()
                                || mSelectedPayMethod.isUnionCloudPay()
                                || mSelectedPayMethod.isUnionVipCard1()
                                || mSelectedPayMethod.isUnionVipCard2())) {
                    prePay(mTotalPaid);
                } else {
                    mMvpView.gotoPayChannel(mTotalPaid);
                }

            }
        }
    }

    public void cashPay(String totalPaid) {
        SalePayModel payModel = new SalePayModel();
        payModel.setPayCode(mSelectedPayMethod.code());
        payModel.setPayName(mSelectedPayMethod.name());
        payModel.setRemark(mSelectedPayMethod.remark());
        payModel.setPayType(mSelectedPayMethod.payType());
        payModel.setBuyerLogonId("");
        payModel.setOutTradeNo("");
        payModel.setTradeNo("");
        payModel.setPayTime(DateUtils.dateFormat());
        payModel.setTotal(totalPaid);
        payModel.setTotalRecv(totalPaid);
        payModel.setTotalPaid(totalPaid);
        payModel.setTotalDscSelf("0.00");
        payModel.setTotalDscChannel("0.00");
        getSale().addPay(payModel);

        paySuccess(payModel);
    }

    /**
     * 获取人脸验证信息，获取到验证信息后，才可调用获取人脸识别
     */
    private void getWxPayFaceRawData() {
        if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
            LoggerUtils.getLogger().debug("微信刷脸支付开关未打开");
            return;
        }
        WxService.getWxPayFaceRawData(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                getWxPayFaceAuthInfo(msg);
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                LoggerUtils.getLogger().debug("获取人脸验证信息失败，清空Authinfo;让下次人脸支付重新请求Authinfo,失败原因:" + err);
                WxDto.getWxDto().setAuthinfo("");
                WxService.releaseWxPayFace(mMvpView.getActivity().getApplicationContext());
                WxService.initWxPayFace(mMvpView.getActivity().getApplicationContext());

                showHintDialog("获取人脸验证信息失败", err);
            }
        });
    }

    /**
     * 获取人脸SDK调用凭证
     *
     * @param rawdata
     */
    private void getWxPayFaceAuthInfo(String rawdata) {
        if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
            LoggerUtils.getLogger().debug("微信刷脸支付开关未打开");
            return;
        }
        WxService.getWxPayFaceAuthInfo(rawdata, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                WxDto.getWxDto().setAuthinfo(msg);
                getWxPayFaceCode(msg);
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                showHintDialog("获取人脸SDK调用凭证失败", err);
            }
        });
    }

    /**
     * 启动人脸APP主界面入口，开启人脸识别
     *
     * @param authinfo
     */
    public void getWxPayFaceCode(final String authinfo) {
        if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
            LoggerUtils.getLogger().debug("微信刷脸支付开关未打开");
            return;
        }
        String telephone = "";
        if (getSale().getVip() != null && !StringUtils.isEmpty(getSale().getVip().card().cardFaceNo())) {
            //判断是否为手机号
            CardNo cardNo = CardNo.MobileOrDefault(getSale().getVip().card().cardFaceNo());
            if (CardNoType.Mobile == cardNo.getType()) {
                telephone = getSale().getVip().card().cardFaceNo();
            }
        }

        WxService.getWxPayFaceCode(authinfo, getSale().getTotalUnPaid(), "", telephone, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                JSONObject jsonObject = JSONObject.parseObject(msg);
                String faceCode = jsonObject.getString(WxConstants.FACE_CODE);
                String openid = jsonObject.getString(WxConstants.OPENID);

                LoggerUtils.getLogger().debug("人脸faceCode:" + faceCode);

                //hideLoadingDialog();
                payPay(faceCode, openid, null, WxConstants.wx_pay_store_name());
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                if (err.contains(WxConstants.RETURN_CODE_USER_CANCEL)) {
                    return;
                }

                showHintDialog("人脸识别失败", err);
            }
        });
    }

    /**
     * 更新支付结果
     *
     * @param authinfo
     * @param payresult
     */
    private void updateWxPayFacePayResult(String authinfo, final String payresult, final UniCallback callBack) {
        if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
            LoggerUtils.getLogger().debug("微信刷脸支付开关未打开");
            return;
        }
        WxService.updateWxPayFacePayResult(authinfo, payresult, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                LoggerUtils.getLogger().info("通知微信界面更新成功:" + msg);
                callBack.onSuccess(msg);
            }

            @Override
            public void onError(String err) {
                LoggerUtils.getLogger().info("通知微信界面更新失败:" + err);
                hideLoadingDialog();
                callBack.onError(err);
            }
        });
    }

    public void paySuccess() {
        paySuccess(getSale().getLastPay());
    }

    public void paySuccess(SalePayModel payModel) {
        LoggerUtils.getLogger().debug(payModel.getPayName() + payModel.getRemark() + "付款金额：" + payModel.getTotalRecv() +
                ",已付金额：" + getSale().getTotalPaid() +
                ",未付金额：" + getSale().getTotalUnPaid());

        updateTotal();
        mMvpView.paySuccess(payModel);

        if (BigDecimalUtils.isGreater(getSale().getTotalUnPaid(), "0.001")) {
            return;
        }

        if (mSelectedPayMethod.isWxFacePay()) {
            if (!GeneralSettings.INSTANCE.isUseWxFacePay()) {
                LoggerUtils.getLogger().debug("微信刷脸支付开关未打开");
                return;
            }
            //调用微信上传交易,不关心是否成功
            WxService.reportOrder(WxDto.getWxDto().getOutTradeNo(), new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    LoggerUtils.getLogger().info("调用微信上传交易成功:" + msg);
                }

                @Override
                public void onError(String err) {
                    LoggerUtils.getLogger().error("调用微信上传交易失败:" + err);
                }
            });

            if (mMvpView.getActivity() == null) {
                LoggerUtils.getLogger().error("调用微信上传交易失败,获取 PayActivity 失败");
                return;
            }
        }

        Bundle bundle = new Bundle();
        bundle.putSerializable("payModel", payModel);
        mMvpView.afterPaySuccess(bundle);
    }

    public void payPay(String authCode) {
        payPay(authCode, null, null, null);
    }

    public void payPay(String authCode, String openid, String terminalParams, String subject) {
        if ((mSelectedPayMethod.isAliPay() || mSelectedPayMethod.isWxPay())
                && !CommonUtils.authCodeType(authCode).equalsIgnoreCase(mSelectedPayMethod.remark())) {
            showHintDialog("请重新扫" + mSelectedPayMethod.name() + "付款码");
            return;
        }

        String scene;
        if (mSelectedPayMethod.isHsyType()) {
            scene = AliConstants.ALI_PAY_SECURITY_CODE;
        } else {
            //这里如果使用人脸支付，showloading有问题
            showLoadingDialog();
            scene = AliConstants.ALI_PAY_BAR_CODE;
        }

        String outTradeNo;
        if (mSelectedPayMethod.isWxFacePay()) {
            outTradeNo = WxDto.getWxDto().getOutTradeNo();
        } else if (mSelectedPayMethod.isYdt()) {
            outTradeNo = CommonUtils.ydtOutTradeNo(getSale().getSaleNo());
        } else if (mSelectedPayMethod.isWxPay() || mSelectedPayMethod.isAliPay() || mSelectedPayMethod.isAliFacePay() || mSelectedPayMethod.isMobilePay() || mSelectedPayMethod.isHsCloudPay()) {
            outTradeNo = WxService.getOutTradeNo(0);
        } else {
            outTradeNo = CommonUtils.genOutTradeNo(Config.org_code, getSale().getSaleNo());
        }

        String saleTime = DateUtils.dateFormat();
        try {
            PayService.payPay(authCode, openid, outTradeNo, mTotalPaid, mSelectedPayMethod.payRemark(), saleTime, scene,
                    subject, terminalParams, GeneralSettings.INSTANCE.getTerminalId(), new Callback<PayResultData>() {
                        @Override
                        public void onSuccess(PayResultData payResultData) {
                            try {
                                if (PayMethod.isWxFacePay(mSelectedPayMethod.payRemark()) && BigDecimalUtils.isLess(PayCustomActivity.WxFacePayVersion.substring(0, 4), "2.13")) {
                                    //告知微信关闭界面
                                    updateWxPayFacePayResult(WxDto.getWxDto().getAuthinfo(), WxConstants.PAYRESULT_SUCCESS, new UniCallback() {
                                        @Override
                                        public void onSuccess(String msg) {
                                            LoggerUtils.getLogger().info("支付成功，通知微信关闭界面，继续执行");
                                            handleResult(payResultData);
                                            hideLoadingDialog();
                                        }

                                        @Override
                                        public void onError(String err) {
                                            handleResult(payResultData);
                                            hideLoadingDialog();
                                            showHintDialog("支付成功，通知微信关闭界面失败", err);
                                        }
                                    });
                                } else {
                                    LoggerUtils.getLogger().info("微信人脸版本前三位" + PayCustomActivity.WxFacePayVersion.substring(0, 4));
                                    handleResult(payResultData);
                                    hideLoadingDialog();
                                }
                            } catch (Exception e) {
                                LoggerUtils.getLogger().info("异常,继续走" + e.toString());
                                handleResult(payResultData);
                                hideLoadingDialog();
                            }
                        }

                        @Override
                        public void onError(Throwable throwable) {
                            payPayError(mSelectedPayMethod.payRemark(), outTradeNo, scene, saleTime, throwable);
                        }
                    });
        } catch (Exception e) {
            showHintDialog("调用下单支付接口异常" + e.getMessage());
            hideLoadingDialog();
            LoggerUtils.getLogger().debug("调用下单支付接口异常" + e.toString());
        }
    }

    private void payPayError(String payRemark, String outTradeNo, String scene, String saleTime, Throwable throwable) {
        hideLoadingDialog();
        LoggerUtils.getLogger().error("下单支付接口调用失败:" + throwable.toString());
        CommonTools.toastShort("下单支付接口调用失败:" + throwable.toString());

        if (NetUtils.isNeedQuery(((WebApiException) throwable).code())) {
            showHintConfirm("订单状态未知，查询请点击【确定】。" + throwable.getMessage(), new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    LoggerUtils.getLogger().error("下单支付接口调用失败,点击确定" + msg);
                    payQuery(outTradeNo, scene, saleTime);
                }

                @Override
                public void onError(String err) {
                    LoggerUtils.getLogger().error("下单支付接口调用失败:" + err);
                }
            });

            return;
        }

        if (NetUtils.isNetException(throwable.getCause()) || throwable.getMessage().contains(PayConstants.FAILED_TO_CONNECT_TO_IP)) {
            showLoadingDialog();
            if (!isViewAttached()) {
                LoggerUtils.getLogger().debug("支付界面被finish");
                return;
            }
            mMvpView.isNotShowExitDialog(true);
            // 无法连接服务时，自动重试。当前定义重试次数为100，间隔时间3秒，共等待5分钟。
            Service.online(100, 3000, new RetryCallback<OnlineResultData>() {
                @Override
                public void onSuccess(OnlineResultData service) {
                    hideLoadingDialog();
                    mMvpView.hideProgressDialog();
                    mMvpView.isNotShowExitDialog(false);
                    payQuery(outTradeNo, scene, saleTime);
                    LoggerUtils.getLogger().debug("下单支付接口调用失败，继续轮询");
                }

                @Override
                public void onProgress(int i, final String s) {
                    mMvpView.showProgressDialogNoHide(s);
                    LoggerUtils.getLogger().info(s);
                }

                @Override
                public void onError(Throwable throwable) {
                    hideLoadingDialog();
                    mMvpView.hideProgressDialog();
                    mMvpView.isNotShowExitDialog(false);
                    showHintDialog("无法查询支付状态,请联系管理员处理");
                }
            });

            return;
        }

        if (PayMethod.isWxFacePay(payRemark)) {
            //告知微信关闭界面
            updateWxPayFacePayResult(WxDto.getWxDto().getAuthinfo(), WxConstants.PAYRESULT_ERROR, new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    LoggerUtils.getLogger().info("支付失败，告知微信关闭界面，继续执行");
                }

                @Override
                public void onError(String err) {
                    LoggerUtils.getLogger().error("支付失败，告知微信关闭界面失败");
                }
            });
        }

        showHintDialog("支付失败", throwable.getMessage());
    }

    public void payMiYaAll(String authCode) {
        payMiYa(authCode, getSale().getTotalUnPaid());
    }

    public void payMiYa(String authCode) {
        payMiYa(authCode, mTotalPaid);
    }

    public void payMiYa(String authCode, String totalPaid) {
        if (!mMiYaService.isInitialized()) {
            showHintDialog("米雅配置错误,请确认配置参数");
            return;
        }

        mSelectedPayMethod = getSale().getPayMethod();
        mTotalPaid = totalPaid;

        mMiYaService.pay(mainHandler, getSale().getSaleNo(),
                BigDecimalUtils.transferBig(BigDecimalUtils.multiply(totalPaid, "100")), authCode);

        showLoadingDialog();
    }

    public void payYst(String authCode) {
        payYst(authCode, mTotalPaid);
    }

    public void payYst(String authCode, String totalPaid) {
        if (mYstService == null || !mYstService.isInitialized()) {
            showHintDialog("赢商通配置错误,请确认配置参数");
            return;
        }

        mSelectedPayMethod = getSale().getPayMethod();
        mTotalPaid = totalPaid;

        mYstService.pay(mainHandler, getSale().getSaleNo(),
                BigDecimalUtils.transferBig(BigDecimalUtils.multiply(totalPaid, "100")), authCode);

        showLoadingDialog();
    }

    private void payQuery(String outTradeNo, String scene, String saleTime) {
        try {
            showLoadingDialog();
            PayService.payQuery(outTradeNo, mSelectedPayMethod.remark(), scene, saleTime, GeneralSettings.INSTANCE.getTerminalId(), new Callback<PayResultData>() {
                @Override
                public void onSuccess(PayResultData payResultData) {
                    handleResult(payResultData);
                    checkQuery = true;
                }

                @Override
                public void onError(final Throwable throwable) {
                    hideLoadingDialog();
                    checkQuery = false;
                    payPayError(mSelectedPayMethod.payRemark(), outTradeNo, scene, saleTime, throwable);
                    LoggerUtils.getLogger().debug("轮询接口调用失败" + throwable.toString());
                }
            });
        } catch (Exception e) {
            checkQuery = false;
            hideLoadingDialog();
            LoggerUtils.getLogger().debug("调用下单支付查询接口出现异常" + e.toString());
        }
    }

    private void handleResult(PayResultData payResultData) {

        if (null != payResultData.coupon) {
            for (CouponData couponData1 : payResultData.coupon) {
                HxPayCoupon hxPayCoupon = new HxPayCoupon();
                hxPayCoupon.type = couponData1.type;
                hxPayCoupon.value = couponData1.value;
                hxPayCoupon.tag = couponData1.tag;
                hxPayCoupon.align = couponData1.align;
                hxPayCoupon.outTradeNo = couponData1.outTradeNo;

                getSale().addHxPayCoupon(hxPayCoupon);
            }
        }

        if (null != payResultData.pay) {
            for (PayData payData : payResultData.pay) {
                PayMethod method;
                method = new PayMethod(payData.payCode, payData.payName);
                method.remark(mSelectedPayMethod.remark());
                method.payType(mSelectedPayMethod.payType());
                LoggerUtils.getLogger().debug("返回code使用返回结果");
                creatPayModelPrint(method, payResultData, payData.totalPaid);
            }
        }
        PayMethod method2;
        if (StringUtils.isEmpty(payResultData.payCode)) {
            method2 = mSelectedPayMethod;
            LoggerUtils.getLogger().debug("未返回code取原来");
        } else {
            method2 = new PayMethod(payResultData.payCode, payResultData.payName);
            method2.remark(mSelectedPayMethod.remark());
            method2.payType(mSelectedPayMethod.payType());
            LoggerUtils.getLogger().debug("返回code使用返回结果");
        }
        creatPayModel(method2, payResultData, payResultData.getTotal());
    }


    /**
     * 生成payModel
     *
     * @param payResultData
     */
    private void creatPayModel(PayMethod method, PayResultData payResultData, String totalPaid) {
        SalePayModel payModel = new SalePayModel();
        payModel.setPayCode(method.code());
        payModel.setPayName(method.name());
        payModel.setRemark(method.remark());
        payModel.setPayType(method.payType());
//        payModel.setBuyerLogonId(StringUtils.isEmpty(payResultData.getBuyerLogonId()) ? payResultData.getOutTradeNo() : payResultData.getBuyerLogonId());
        //微信一般为openid, 容易认为是乱码,暂统一使用outtradeNo
        payModel.setBuyerLogonId(payResultData.getOutTradeNo());
        payModel.setOutTradeNo(payResultData.getOutTradeNo());
        payModel.setTradeNo(payResultData.getTradeNo());
        payModel.setPayTime(payResultData.getSaleTime());
        payModel.setTotal(payResultData.getTotal());
        payModel.setTotalRecv(payResultData.getTotal());
        payModel.setTotalPaid(totalPaid);
        payModel.setTotalDscSelf(payResultData.getTotalDscSelf());
        payModel.setTotalDscChannel(payResultData.getTotalDscChannel());
        getSale().addPay(payModel);

        // 弹框结束会尝试上传流水,所以打印完再调起弹框
        printReceipt(payModel, mSelectedPayMethod.isYdt(), new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                LoggerUtils.getLogger().debug("海信支付-支付-签购单");
                paySuccess(payModel);
            }

            @Override
            public void onError(String err) {
                LoggerUtils.getLogger().debug("海信支付-支付-签购单");
                paySuccess(payModel);
            }
        });
    }

    /**
     * 生成payModelPrint
     *
     * @param payResultData
     */
    private void creatPayModelPrint(PayMethod method, PayResultData payResultData, String totalPaid) {
        SalePayModel payModel = new SalePayModel();
        payModel.setPayCode(method.code());
        payModel.setPayName(method.name());
        payModel.setRemark(method.remark());
        payModel.setPayType(method.payType());
//        payModel.setBuyerLogonId(StringUtils.isEmpty(payResultData.getBuyerLogonId()) ? payResultData.getOutTradeNo() : payResultData.getBuyerLogonId());
        //微信一般为openid, 容易认为是乱码,暂统一使用outtradeNo
        payModel.setBuyerLogonId(payResultData.getOutTradeNo());
        payModel.setOutTradeNo(payResultData.getOutTradeNo());
        payModel.setTradeNo(payResultData.getTradeNo());
        payModel.setPayTime(payResultData.getSaleTime());
        payModel.setTotal(payResultData.getTotal());
        payModel.setTotalRecv(payResultData.getTotal());
        payModel.setTotalPaid(totalPaid);
        payModel.setTotalDscSelf(payResultData.getTotalDscSelf());
        payModel.setTotalDscChannel(payResultData.getTotalDscChannel());
        getSale().addPayPrint(payModel);
    }

    /**
     * 打印海信支付签购单
     *
     * @param payment
     * @param callback
     */
    private void printReceipt(final SalePayModel payment, boolean isYdt, final UniCallback callback) {
        if (isYdt || !GeneralSettings.INSTANCE.printHXPayReceipt() || payment == null || payment.getPayCode() == null) {
            LoggerUtils.getLogger().debug("海信支付原始数据为空或者选项关闭或者一点通,不打印签购单");
            hideAndCallback(callback);
            return;
        }

        try {
            //打印阻塞了主线程,不放在线程内的话,loading框无法正常运行
            new Thread(new Runnable() {
                @Override
                public void run() {
                    LoggerUtils.getLogger().debug("海信支付打印签购单");
                    PrinterHelper.getInstance().printWithStringList(PayService.getPrintContent(getSale(), payment, null), new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            hideAndCallback(callback);
                        }

                        @Override
                        public void onError(String err) {
                            hideAndCallback(callback);
                        }
                    });
                }
            }).start();

        } catch (Exception e) {
            LoggerUtils.getLogger().debug("海信支付打印签购单异常:" + e.toString());
            hideAndCallback(callback);
        }
    }

    /**
     * 隐藏loading框,实际只有onSuccess回调,所以统一处理
     *
     * @param callback 回调
     */
    private void hideAndCallback(final UniCallback callback) {
        if (!isViewAttached()) {
            LoggerUtils.getLogger().debug("支付界面已被关闭，人脸识别返回时被回收");
            return;
        }
        mMvpView.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                hideLoadingDialog();
                callback.onSuccess("");
            }
        });
    }

    public void handleResult(PayResult result) {
        SalePayModel payModel = new SalePayModel();
        String payNo = result.cardNo();

        if (mSelectedPayMethod.isMobilePay() || mSelectedPayMethod.isUccbScan()) {
            if (null == result.payType() || result.payType().equals("")) {
                payNo = result.cardNo();
            } else if (result.payType().indexOf("微信") != -1 || result.payType().indexOf("WEPAY") != -1) {
                payNo = result.cardNo() + "\n(01-微信)";
            } else if (result.payType().indexOf("支付宝") != -1 || result.payType().indexOf("ALIPAY") != -1) {
                payNo = result.cardNo() + "\n(02-支付宝)";
            } else if (result.payType().indexOf("云闪付") != -1 || result.payType().indexOf("UNIONPAY") != -1) {
                payNo = result.cardNo() + "\n(03-云闪付)";
            } else {
                payNo = result.cardNo() + "\n(" + result.payType() + ")";
            }
        }
        LoggerUtils.getLogger().debug("payNo:" + payNo);
        payModel.setPayCode(mSelectedPayMethod.code());
        payModel.setPayName(mSelectedPayMethod.name());
        payModel.setRemark(mSelectedPayMethod.remark());
        payModel.setPayType(mSelectedPayMethod.payType());
        payModel.setBuyerLogonId(payNo);
        payModel.setOutTradeNo(result.traceNo());
        payModel.setTradeNo(result.refNo());
        payModel.setMchId(result.mchId());
        payModel.setPayTime(DateUtils.dateFormat());
        payModel.setTotal(result.amount());
        payModel.setTotalRecv(result.amount());
        payModel.setTotalPaid(result.amount());
        payModel.setTotalDscSelf("0");
        payModel.setTotalDscChannel("0");
        payModel.setUdp2(result.termNo());
        payModel.setUdp3(result.outTradeNo() + "," + result.mchId());
        payModel.setUdp4(result.refNo());
        payModel.setUdp5(result.traceNo());
        getSale().addPay(payModel);

        LoggerUtils.getLogger().debug("银联商务cardNo：" + payNo);
        LoggerUtils.getLogger().debug("银联商务refNo：" + result.refNo());
        LoggerUtils.getLogger().debug("银联商务traceNo：" + result.traceNo());
        LoggerUtils.getLogger().debug("银联商务/建行outTradeNo：" + result.outTradeNo());
        LoggerUtils.getLogger().debug("银联商务/建行amount：" + result.amount());

        paySuccess(payModel);
    }

    /**
     * 获取支付方式
     *
     * @param gridViewOften 常用列表
     * @param gridViewOther 其他列表
     */
    public void getPayMethod(final GridView gridViewOften, final GridView gridViewOther) {
        PayMethodService.createWxLayout(mMvpView.getActivity(), new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                updatePayUi(payMethods);

                if (getPayUiOften().size() == 1) {
                    gridViewOften.setNumColumns(getPayUiOften().size());
                } else if (getPayUiOften().size() % 3 == 0) {
                    gridViewOften.setNumColumns(3);
                } else {
                    gridViewOften.setNumColumns(2);
                }

                if (getPayUiOther().size() <= 5) {
                    gridViewOther.setNumColumns(1);
                } else {
                    gridViewOther.setNumColumns(2);
                }

                mPayMethodCustomAdapterOften = new PaymethodCustomAdapter(getPayUiOften());
                gridViewOften.setAdapter(mPayMethodCustomAdapterOften);
                mPayMethodCustomAdapterOther = new PaymethodCustomOtherAdapter(getPayUiOther());
                gridViewOther.setAdapter(mPayMethodCustomAdapterOther);
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().debug("获取支付方式异常" + throwable.toString());
            }
        });
    }

    public void getPayMethod(GridView gridViewOften) {
        PayMethodService.createWxLayout(mMvpView.getActivity(), new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                updatePayAliUi(payMethods);

                if (getPayUiOften().size() == 1) {
                    gridViewOften.setNumColumns(getPayUiOften().size());
                } else {
                    gridViewOften.setNumColumns(2);
                }
                mPayMethodCustomAdapterOften = new PaymethodCustomAdapter(getPayUiOften());
                gridViewOften.setAdapter(mPayMethodCustomAdapterOften);
                mPayMethodCustomAdapterOther = new PaymethodCustomOtherAdapter(getPayUiOther());
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().debug("获取支付方式异常" + throwable.toString());
            }
        });
    }

    /**
     * 初始化支付方式列表
     */
    public void initPayMethodList() {
        getSale().initPayMethodList();
    }

    /**
     * 获取常用支付方式
     */
    public List<PayUi> getPayUiOften() {
        return getSale().getPayUiOften();
    }

    /**
     * 获取其他支付方式
     */
    public List<PayUi> getPayUiOther() {
        return getSale().getPayUiOther();
    }

    private void updatePayAliUi(List<PayMethod> payMethods) {
        if (getPayUiOften().size() > 0) {
            getPayUiOften().clear();
        }
        for (int i = 0; i < payMethods.size(); i++) {
            PayMethod payMethod = payMethods.get(i);
            if (payMethod.isWxFacePay() || payMethod.isAllOrChannelType() || payMethod.isHsCloudPay() || payMethod.isAliFacePay() || payMethod.isAliPay()) {
                updatePayUi(getPayUiOften(), payMethod, PayMethodService.getWxPayMethodIcon(payMethod));
            }
        }
    }

    private void updatePayUi(List<PayMethod> payMethods) {
        if (getPayUiOften().size() > 0) {
            getPayUiOften().clear();
        }
        if (getPayUiOther().size() > 0) {
            getPayUiOther().clear();
        }

        for (int i = 0; i < payMethods.size(); i++) {
            PayMethod payMethod = payMethods.get(i);

            if (payMethod.isWxFacePay() || payMethod.isAllOrChannelType() || payMethod.isHsCloudPay()) {
                updatePayUi(getPayUiOften(), payMethod, PayMethodService.getWxPayMethodIcon(payMethod));
            } else if (payMethod.isVipCoupon() || payMethod.isCard()) {
                updatePayUi(getPayUiOther(), payMethod, PayMethodService.getWxPayMethodIcon(payMethod));
            }
        }
    }

    private void updatePayUi(List<PayUi> payUiList, PayMethod payMethod, int drawableId) {
        PayUi payUi = new PayUi();
        payUi.setPayName(payMethod.name());
        payUi.setPayDrawable(drawableId);
        payUi.setPayRemark(payMethod.remark());
        payUi.setPayCode(payMethod.code());
        payUi.setPayType(payMethod.payType());
        payUiList.add(payUi);
    }

    private void addPay(String totalPaid, String payNo) {
        SalePayModel payModel = new SalePayModel();
        payModel.setPayCode(mSelectedPayMethod.code());
        payModel.setPayName(mSelectedPayMethod.name());
        payModel.setRemark(mSelectedPayMethod.remark());
        payModel.setPayType(mSelectedPayMethod.payType());
        payModel.setBuyerLogonId(payNo);
        payModel.setOutTradeNo("");
        payModel.setTradeNo("");
        payModel.setPayTime(DateUtils.dateFormat());
        payModel.setTotal(totalPaid);
        payModel.setTotalRecv(totalPaid);
        payModel.setTotalPaid(totalPaid);
        payModel.setTotalDscSelf("0.00");
        payModel.setTotalDscChannel("0.00");
        getSale().addPay(payModel);

        paySuccess(payModel);
    }

    private List<SalePluDscModel> pluDscs() {
        for (BaseSalePluModel pluModel : getSale().getPlus()) {
            for (int i = 0; i < pluModel.pluDscSize(); i++) {
                SalePluDscModel pluDscModel = pluModel.getPluDsc(i);
                boolean isAdd = false;

                for (int j = 0; j < getSale().getPluDscs().size(); j++) {
                    SalePluDscModel pluDscModelTemp = getSale().getPluDscs().get(j);

                    if (pluDscModel.getDscProType().equals(pluDscModelTemp.getDscProType())) {
                        pluDscModelTemp.setTotalDsc(BigDecimalUtils.add(pluDscModelTemp.getTotalDsc(), pluDscModel.getTotalDsc()));
                        isAdd = true;
                        break;
                    }
                }

                if (!isAdd) {
                    getSale().getPluDscs().add(pluDscModel);
                }
            }
        }

        return getSale().getPluDscs();
    }

    public void updateDscUi() {
        if (mProAdapter != null) {
            mProAdapter.setData(pluDscs());
            mProAdapter.notifyDataSetChanged();
        }
    }

    //是否单笔撤销
    private volatile boolean isSingleCancel;

    public void cancelStdSinglePay(int index) {
        isSingleCancel = true;
        SalePayModel payModel = getSale().getPay(index);
        if (PayMethod.isAllOrChannelType(payModel.getRemark()) || PayMethod.isUccbScan(payModel.getRemark())) {
            if (AggregatePaymentChannel.isYST() || AggregatePaymentChannel.isMIYA()
                    || (GeneralSettings.INSTANCE.getPaymentChannel().isCCB() && AggregatePaymentChannel.isCHANNEL())) {
                showHintDialog("暂不支持当前支付通道的撤销操作");
                return;
            }
        }

        if (PayMethod.isYdt(payModel.getRemark())) {
            showHintDialog(String.format("暂不支持%s支付撤销", payModel.getPayName()));
            return;
        }

        String remark = payModel.getRemark();
        if (PayMethod.isCardType(remark)
                || PayMethod.isCardByPayType(payModel.getPayType())
                || PayMethod.isPointsType(remark)
                || PayMethod.isRemainType(remark)) {
            LoggerUtils.getLogger().debug("不支持单笔撤销:" + payModel);
            showHintDialog("该支付方式不支持单笔撤销,请点击右下角【撤销支付】撤销整笔订单");
            return;
        }
        showHintConfirm("是否撤销？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                cancelPayConfirm(index);
            }

            @Override
            public void onError(String err) {
            }
        });
    }

    /**
     * 撤销支付。撤销时按支付顺序倒序依次撤销。
     * <p>
     * 1、默认撤销最后一笔支付。
     * 2、对于会员券支付，由于支付时使用批量核销，显示时逐条显示，因此会员券撤销后，需批量删除显示项。
     * 3、由于银联撤销使用onActivityResult的异步方式，为保障撤销逻辑一致性，每种支付方式撤销成功后，必须退出本方法，
     * 不允许循环调用。然后通过其它方法再次调用本方法。
     */
    public void cancelPay() {
        isSingleCancel = false;
        if (getSale().hasAllPaid()) {
            return;
        }

        if (getSale().paySize() == 0) {
            showHintDialog("未支付，不允许撤销");
            return;
        }

        for (SalePayModel payModel : getSale().getPays()) {
            if (PayMethod.isAllOrChannelType(payModel.getRemark()) || PayMethod.isUccbScan(payModel.getRemark())) {
                if (AggregatePaymentChannel.isYST() || AggregatePaymentChannel.isMIYA()
                        || (GeneralSettings.INSTANCE.getPaymentChannel().isCCB() && AggregatePaymentChannel.isCHANNEL())) {
                    showHintDialog("暂不支持当前支付通道的撤销操作");
                    return;
                }
            }

            if (PayMethod.isYdt(payModel.getRemark())) {
                showHintDialog(String.format("暂不支持%s支付撤销", payModel.getPayName()));
                return;
            }
        }

        showHintConfirm("已部分支付，是否撤销？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                cancelPayConfirm(getSale().paySize() - 1);
            }

            @Override
            public void onError(String err) {
            }
        });
    }

    public void cancelPayConfirm(int index) {
        LoggerUtils.getLogger().debug("剩余支付撤销：" + getSale().paySize());

        SalePayModel payModel = getSale().getPay(index);
        LoggerUtils.getLogger().debug("cancelPayConfirm:" + payModel);
        String remark = payModel.getRemark();
        showLoadingDialog();
        if (PayMethod.isCardType(remark)) {
            cardRevoke(payModel);
        } else if (PayMethod.isPointsType(remark) || PayMethod.isRemainType(remark)) {
            pointsRevoke(payModel);
        } else if (PayMethod.isVipCoupon(remark)) {
            vipCouponCancel(payModel);
        } else if (PayMethod.isScrmCoupon(remark)) {
            scrmCouponCancel(payModel);
        } else if (PayMethod.isAllOrChannelType(remark)) {
            if (CommonUtils.isStdStyle()) {
                LoggerUtils.getLogger().debug("标准版默认使用海信支付撤销");
                payCancel(payModel);
                return;
            }
            if (AggregatePaymentChannel.isCHANNEL()) {
                mMvpView.gotoCancelPayChannel(payModel);
            } else if (AggregatePaymentChannel.isHXPAY()) {
//                if (PayMethod.isAliPay(remark) && CommonUtils.isTimesSquare(App.getContext())) {
//                    LoggerUtils.getLogger().debug("温州时代-海信Pay-支付宝走收单行渠道-cancel");
//                    mMvpView.gotoCancelPayChannel(payModel);
//                    return;
//                }

                payCancel(payModel);
            }
        } else if (PayMethod.isUnionType(remark) || PayMethod.isUnionCardType(remark) ||
                PayMethod.isUnionWallet(remark) || PayMethod.isUnionCoupon(remark) ||
                PayMethod.isUnionPoints(remark) || PayMethod.isUnionInstallment(remark) ||
                PayMethod.isUccbScan(remark) || PayMethod.isUccbCard(remark)) {
            mMvpView.gotoCancelPayChannel(payModel);
        } else if (PayMethod.isCash(remark) || PayMethod.isOther(remark) || "99".equals(payModel.getPayCode())) {
            cancelPaySuccess(payModel);
        } else if (PayMethod.isCardByPayType(payModel.getPayType())) {
            cardRevoke(payModel);
        } else if (PayMethod.isCouponByPayType(payModel.getPayType())) {
            scrmCouponCancel(payModel);
        } else if (PayMethod.isAllInPayType(remark) || PayMethod.isAllInPayByPayType(payModel.getPayType())) {
            payCancel(payModel);
        } else {
            showHintDialog(String.format("暂不支持%s撤销", payModel.getPayName()));
        }
    }

    private void cardRevoke(SalePayModel payModel) {
        CardService.revoke(payModel, payModel.getCardNo().getTypeCode() != 7 || GeneralSettings.INSTANCE.isEPrepaidCardQueryByPassword(), new Callback<String>() {
            @Override
            public void onSuccess(String s) {
                cancelPaySuccess(payModel);
            }

            @Override
            public void onError(Throwable throwable) {
                cancelPayError(payModel, throwable);
            }
        });
    }

    private void pointsRevoke(SalePayModel payModel) {
        PointsService.revoke(payModel, payModel.getCardNo().getTypeCode() != 7 || GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword(), new Callback<String>() {
            @Override
            public void onSuccess(String s) {
                cancelPaySuccess(payModel);
            }

            @Override
            public void onError(Throwable throwable) {
                cancelPayError(payModel, throwable);
            }
        });
    }

    private void vipCouponCancel(SalePayModel payModel) {
        CouponService.cancel(getSale().getUsableCoupons().items(), new Callback<Object>() {
            @Override
            public void onSuccess(Object o) {
                cancelPaySuccess(payModel);

                try {
                    getSale().getUsableCoupons().cancelAll();
                } catch (Exception e) {
                    showHintDialog(e.toString());
                }
            }

            @Override
            public void onError(Throwable throwable) {
                cancelPayError(payModel, throwable);
            }
        });
    }

    private void scrmCouponCancel(SalePayModel payModel) {
        com.hisense.crm.webapi.coupon.CouponService.scrmCouponCancel(getSale().getUsableCouponScrm(), getSale(), payModel, new Callback<Object>() {
            @Override
            public void onSuccess(Object o) {
                getSale().getUsableCouponScrm().clear();
                cancelPaySuccess(payModel);
            }

            @Override
            public void onError(Throwable throwable) {
                cancelPayError(payModel, throwable);
            }
        });
    }

    private void payCancel(SalePayModel payModel) {
        PayService.cancel(payModel, getSale().getSaleNo(), GeneralSettings.INSTANCE.getTerminalId(), new Callback<CancelResultData>() {
                    @Override
                    public void onSuccess(CancelResultData cancelResultData) {
                        LoggerUtils.getLogger().debug("直连取消成功:" + cancelResultData);

                        SalePayModel tempPayModel = new SalePayModel();
                        tempPayModel.setPayCode(payModel.getPayCode());
                        tempPayModel.setPayName(payModel.getPayName());
                        tempPayModel.setRemark(payModel.getRemark());
                        tempPayModel.setBuyerLogonId(payModel.getOutTradeNo());
                        tempPayModel.setOutTradeNo(payModel.getOutTradeNo());
                        tempPayModel.setTradeNo(payModel.getTradeNo());
                        tempPayModel.setTotal(payModel.getTotal());
                        double totalRecv = -Double.parseDouble(payModel.getTotalRecv());
                        tempPayModel.setTotalRecv("" + totalRecv);
                        tempPayModel.setTotalPaid("" + totalRecv);

                        printReceipt(tempPayModel, false, new UniCallback() {
                            @Override
                            public void onSuccess(String msg) {
                                LoggerUtils.getLogger().debug("海信支付-撤销-签购单");
                                cancelPaySuccess(payModel);
                            }

                            @Override
                            public void onError(String err) {
                                LoggerUtils.getLogger().debug("海信支付-撤销-签购单");
                                cancelPaySuccess(payModel);
                            }
                        });

                    }

                    @Override
                    public void onError(Throwable throwable) {
                        cancelPayError(payModel, throwable);
                    }
                }
        );
    }

    public void cancelPaySuccess(SalePayModel payModel) {
        LoggerUtils.getLogger().info(String.format("%s支付撤销成功,撤销金额:%s", payModel.getPayName(), payModel.getTotalRecv()));
//        getSale().removeLastPay();
        getSale().removePay(payModel.getSerialNo() - 1);
        mMvpView.cancelPaySuccess(payModel);
        updateTotal();
        hideLoadingDialog();

        if (!isSingleCancel) {
            LoggerUtils.getLogger().info("非单笔撤销");
            if (getSale().paySize() > 0) {
                cancelPay();
            } else {
                cancelPayFinish();
            }
        } else {
            if (getSale().paySize() == 0) {
                LoggerUtils.getLogger().info("单笔撤销,已经是最后一笔,关闭当前页");
                cancelPayFinish();
            }
        }
    }

    public void cancelPayFinish() {
        getSale().removeHjDsc();
        if (!isViewAttached()) {
            LoggerUtils.getLogger().debug("支付界面异常退出");
            return;
        }
        mMvpView.cancelPayFinish();
    }

    private void cancelPayError(SalePayModel payModel, Throwable throwable) {
        hideLoadingDialog();
        LoggerUtils.getLogger().info(String.format("%s支付撤销失败:%s", payModel.getPayName(), throwable.toString()));

        String remark = payModel.getRemark();
        if (PayMethod.isCard(remark) || PayMethod.isVipPoints(remark) ||
                PayMethod.isScrmVipPoints(remark) || PayMethod.isScrmRemain(remark)) {
            showHintConfirm(payModel.getPayName() + "支付撤销失败撤销失败", throwable.getMessage(), "确定", "重试", new UniCallback() {
                @Override
                public void onSuccess(String msg) {

                }

                @Override
                public void onError(String err) {
                    cancelPayConfirm(getSale().paySize() - 1);
                }
            });
        }

        showHintDialog(String.format("%s支付撤销失败:%s", payModel.getPayName(), throwable.getMessage()));
        mMvpView.cancelPayError();
    }

    public void quit() {
        CommonUtils.loopQuit(mLooper);
    }

    /**
     * 初始化人脸识别服务
     */
    public void initZoloz() {
        zoloz = Zoloz.getInstance(mMvpView.getContext());
        zoloz.zolozInstall(mockInfo());
    }

    /**
     * mock数据
     *
     * @return Map
     */
    private Map mockInfo() {
        Map<String, String> merchantInfo = new HashMap<>();
        merchantInfo.put("partnerId", AliConstants.ali_pay_pid());    // 必填项，ISV的pid。对于自用型商户，填写签约商户的pid，和merchantId保持一致
        merchantInfo.put("appId", AliConstants.ali_pay_app_id());  //必填项，支付宝分配给开发者的应用ID，和当面付请求的appid保持一致。
        merchantInfo.put("deviceNum", AliConstants.ali_pay_device_num());     // 必填项，商户机具终端编号，和当面付请求的terminal_id 保持一致
        merchantInfo.put("deviceMac", SystemConfig.getBluetoothAddress());// 可选，设备Mac 地址，例如："CC:B8:A8:0F:63:3C"
        merchantInfo.put("storeCode", Config.org_code); //可选，商户门店编号，和当面付请求的store_id保持一致，如果当面付请求中传入了store_id，这里也需要传入

        if (!StringUtils.isEmpty(AliConstants.ali_pay_third_pid())) {
            merchantInfo.put("merchantId", AliConstants.ali_pay_third_pid());   // 必填项，签约商户的pid。来自蚂蚁开放平台，示例："2088011211821038"
        } else {
            merchantInfo.put("merchantId", AliConstants.ali_pay_pid());
        }
        LoggerUtils.getLogger().debug("GetMetaInfo params:" + merchantInfo.toString());
        getFacePayID(merchantInfo);
        return merchantInfo;
    }

    /**
     * 人脸识别 - 获取metaInfo 及 zimId
     */
    private void getMetaInfo(PayMethod payMethod) {
        zoloz.zolozGetMetaInfo(mockInfo(), (smileToPayResponse) -> {
            LoggerUtils.getLogger().info("zolozGetMetaInfo response:" + smileToPayResponse);
            if (smileToPayResponse == null) {
                hideLoadingDialog();
                showHintDialog("获取设备信息失败，返回信息为空");
                mMvpView.setIsClickPayFalse();
                return;
            }

            String code = (String) smileToPayResponse.get("code");
            String metaInfo = (String) smileToPayResponse.get("metainfo");

            //获取metainfo成功
            if (PayConstants.CODE_SUCCESS.equals(code) && metaInfo != null) {
                //起一个异步线程发起网络请求
                smilePay(metaInfo);
            } else {
                reStartZolozService();

                hideLoadingDialog();
                mMvpView.setIsClickPayFalse();
                showHintDialog("请注意", "人脸支付异常;请重新尝试支付,或选择其他支付方式");
                LoggerUtils.getLogger().debug("支付宝人脸支付：" + "code is : " + code + ",metal info is:" + metaInfo);
            }
        });
    }

    private void smilePay(String metaInfo) {
        AliService.smilePay(metaInfo, new GlobalCallback<String>() {
            @Override
            public void onSuccess(String s) {
                try {
                    LoggerUtils.getLogger().info("支付宝人脸返回参数:" + s);
                    JSONObject responseAliPay = JSONObject.parseObject(s, Feature.OrderedField);
                    if (responseAliPay != null && "10000".equals(responseAliPay.getString("code"))) {
                        String result = responseAliPay.getString("result");
                        JSONObject resultJson = JSON.parseObject(result);
                        String zimId = resultJson.getString("zimId");
                        String zimInitClientData = resultJson.getString("zimInitClientData");
                        //人脸调用
                        smile(zimId, zimInitClientData);
                    } else {
                        mMvpView.setIsClickPayFalse();
                        hideLoadingDialog();
                        showHintDialog("请注意", JSONUtils.getJSONString(responseAliPay, "sub_msg"));
                    }
                } catch (Exception e) {
                    mMvpView.setIsClickPayFalse();
                    hideLoadingDialog();
                    LoggerUtils.getLogger().error("支付宝人脸返回异常:" + e.toString());
                    showHintDialog("支付宝人脸返回异常:" + e.getMessage());
                }
            }

            @Override
            public void onError(String message) {
                hideLoadingDialog();
                LoggerUtils.getLogger().debug("请求支付宝人脸异常:" + message);
                mMvpView.afterPayFailure(message);
            }
        });
    }

    private void reStartZolozService() {
        Intent intent = new Intent();
        intent.setPackage("com.alipay.zoloz.smile");
        intent.setAction("com.alipay.zoloz.stop.smiletopay");
        getView().getActivity().sendBroadcast(intent);

        //初始化人脸服务
        initZoloz();
    }

    /**
     * 使用zimId调起人脸识别页面
     *
     * @param zimId    zimId
     * @param protocal protocal
     */
    private void smile(String zimId, String protocal) {
        Map<String, String> params = new HashMap<>();
        params.put("zim.init.resp", protocal);

        zoloz.zolozVerify(zimId, params, (smileToPayResponse) -> {
            LoggerUtils.getLogger().debug("smileToPayResponse" + smileToPayResponse);

            if (smileToPayResponse == null) {
                showHintDialog("请注意", "人脸识别返回信息为空");
                return;
            }

            String code = (String) smileToPayResponse.get("code");
            String fToken = (String) smileToPayResponse.get("ftoken");
            String subCode = (String) smileToPayResponse.get("subCode");
            String msg = (String) smileToPayResponse.get("msg");

            //刷脸成功
            if (PayConstants.CODE_SUCCESS.equalsIgnoreCase(code) && fToken != null) {
                String signed = signed(fToken);
                if (signed == null) {
                    LoggerUtils.getLogger().debug("刷脸成功后调用支付,机具签名失败，返回为null");
                    showHintDialog("机具签名失败，返回为null");
                    return;
                }

                payPay(fToken, null, signed, AliConstants.ali_pay_subject());
            } else if (PayConstants.CODE_EXIT.equalsIgnoreCase(code)) {
                showHintDialog("请注意", "已退出刷脸支付");
                hideLoadingDialog();
            } else if (PayConstants.CODE_TIMEOUT.equalsIgnoreCase(code)) {
                showHintDialog("请注意", "操作超时");
                hideLoadingDialog();
            } else if (PayConstants.CODE_OTHER_PAY.equalsIgnoreCase(code)) {
                showHintDialog("请注意", "已退出刷脸支付");
                hideLoadingDialog();
            } else {
                showHintDialog("请注意", "支付失败:" + subCode);
                hideLoadingDialog();
            }
        });
    }

    /**
     * 如果调用人脸支付初始化接口失败，
     * 没有获取到APPID等参数，则从sp文件读取
     */
    private void getFacePayID(Map<String, String> merchantInfo) {
        String appId = merchantInfo.get("appId");
        String partnerId = merchantInfo.get("partnerId");
        String merchantId = merchantInfo.get("merchantId");

        try {
            if ("".equals(appId)) {
                merchantInfo.put("appId", GeneralSettings.INSTANCE.getAppId());
            }
            if ("".equals(partnerId)) {
                merchantInfo.put("partnerId", GeneralSettings.INSTANCE.getPartnerId());
            }
            if ("".equals(merchantId)) {
                merchantInfo.put("merchantId", GeneralSettings.INSTANCE.getMerchantId());
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("人脸支付参数配置失败");
        }
    }

    public String signed(String scanCode) {
        int[] result = new int[1];
        String signedScanCode = App.mXDeviceManager.sign(scanCode.getBytes(), result);
        LoggerUtils.getLogger().debug("机具管控参数：" + signedScanCode + "；result：" + result[0]);
        return signedScanCode;
    }

    public void print() {
        try {
            PrinterHelper.getInstance().print(getMainSaleModel().getSaleModel());
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印出现异常" + e.toString());
        }
    }

    public void submit() {
        showLoadingDialog();

        //保存本地流水
        //uSaveData();

        saleComplete();

        Service.online(200, 3000, new RetryCallback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData service) {
                saleSubmit();
            }

            @Override
            public void onProgress(int i, final String s) {
                // 报告服务重试状态。由于重试在新线程中运行，显示状态时需同步到UI线程。
                mMvpView.showProgressDialogNoHide(s);
                LoggerUtils.getLogger().info(s);
            }

            @Override
            public void onError(final Throwable throwable) {
                // 达到最大重试次数后仍然无法连接服务。由于重试在新线程中运行，显示状态时需同步到UI线程。
                mMvpView.hideProgressDialog();
                showHintDialog("无法上传流水：" + throwable.getMessage());
                LoggerUtils.getLogger().error("无法上传流水：" + throwable.toString());

            }
        });
    }

    public void saleSubmit(){
        saleApplicationService.upload(getMainSaleModel().getSaleModel(), new Callback<SaleSubmitResultData>() {
            @Override
            public void onSuccess(SaleSubmitResultData submitResultData) {
                if (!isViewAttached()) {
                    LoggerUtils.getLogger().debug("支付界面被finish");
                    return;
                }
                mMvpView.whenPayDone();
                LoggerUtils.getLogger().info("流水上传成功:" + getSale().getSaleNo());
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                showHintConfirm("流水上传", "流水上传失败:" + throwable.getMessage() + ",请确认流水是否已上传，若未上传请重试", new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        showLoadingDialog();
                        saleSubmit();
                    }

                    @Override
                    public void onError(String err) {
                        mMvpView.whenPayDone();
                        LoggerUtils.getLogger().error(String.format(Locale.CHINESE, "流水[%s]上传失败: %s",
                                getSale().getSaleNo(), throwable.getMessage()));
                        hideLoadingDialog();
                    }
                });


            }
        });
    }

    /**
     * 交易完成，用于更新限量次数
     */
    private void saleComplete() {
        try {
            LoggerUtils.getLogger().debug("进入完成接口逻辑");
            List<CompletePluDscData> pluDsc = new ArrayList<>();
            for (BaseSalePluModel pluModel : getMainSaleModel().getSaleModel().getPlus()) {
                for (int i = 0; i < pluModel.pluDscSize(); i++) {
                    SalePluDscModel pluDscModel = pluModel.getPluDsc(i);

                    CompletePluDscData pluDscData = new CompletePluDscData();
                    pluDscData.setDscCode(pluDscModel.getDscCode());
                    pluDscData.setDscProType(pluDscModel.getDscProType());
                    pluDscData.setDscNum(pluDscModel.getDscNum());
                    pluDscData.setPluCode(pluModel.getPluCode());
                    pluDscData.setGlobalType(pluDscModel.getGlobalType());
                    pluDsc.add(pluDscData);
                }
            }
            if (!ListUtils.isEmpty(pluDsc)) {
                LoggerUtils.getLogger().debug("有优惠，调用完成接口");
                String vipCardNo = getMainSaleModel().getSaleModel().getVip() == null
                        ? null
                        : getMainSaleModel().getSaleModel().getVip().card().vipCardNo();

                SaleService.saleComplete(getMainSaleModel().getSaleModel().getSaleNo(),
                        vipCardNo,
                        getMainSaleModel().getSaleModel().getSaleDate(),
                        pluDsc,
                        new Callback<Object>() {
                            @Override
                            public void onSuccess(Object o) {
                                LoggerUtils.getLogger().debug("完成接口成功");
                            }

                            @Override
                            public void onError(Throwable throwable) {
                                LoggerUtils.getLogger().debug("完成接口失败：" + throwable.getMessage());
                            }
                        });
            } else {
                LoggerUtils.getLogger().debug("无优惠，不调用完成接口");
            }
        } catch (Exception e) {
            NetUtils.printDetail("交易完成逻辑:", e);
        }
    }

    /**
     * 交易完成后保存交易数据
     */
    private void uSaveData() {
        SaleNoService.save(mMvpView.getContext(), Config.org_code, Config.pos_no, getSale().getSaleNo());

        try {
            if (getSale().getVip() == null || getSale().getVip().card() == null || getSale().getVip().card().vipCardNo() == null) {
                VipCard card = new VipCard.Builder("", null)
                        .cardType("", "")
                        .expireDate("")
                        .status("")
                        .vipCardNo("")
                        .build();

                getSale().addVip(card);
            }
            new SaleRepository(mMvpView.getContext()).save(getMainSaleModel().getSaleModel());
            LoggerUtils.getLogger().error("保存数据成功");

        } catch (Exception e) {
            LoggerUtils.getLogger().error("保存数据出错:" + e.toString());
            LoggerUtils.getLogger().error("出错流水数据:" + JSONObject.toJSONString(getSale()));
        }
    }

    public void initSaleService() {
        saleApplicationService = new SaleApplicationService(mMvpView.getContext());
    }


    /**
     * 返回当前选择的支付方式
     */
    public PayMethod selectedPayMethod() {
        return mSelectedPayMethod;
    }

    public void saleTotalReduction() {
        showLoadingDialog();
        SaleService.saleTotalReduction(CalcType.Total, getSale().getPlus(), new Callback<TotalReductionModel>() {
            @Override
            public void onSuccess(TotalReductionModel totalReductionModel) {

                if (totalReductionModel != null) {
                    String totalDscMj = getSale().updatePluDscNoMj(totalReductionModel.getPluDscs());

                    if (BigDecimalUtils.isEquals0(totalDscMj)) {
                        showHintDialog("订单商品优惠金额为0");
                    } else {
                        addPay(totalDscMj, "");
                    }
                } else {
                    showHintDialog("订单商品不满足优惠条件");
                }

                mMvpView.updateTotal();
                abandonFenDsc();
                hideLoadingDialog();
            }

            @Override
            public void onError(final Throwable throwable) {
                hideLoadingDialog();

                showHintDialog("整单促销计算失败：" + throwable.getMessage());
                LoggerUtils.getLogger().debug("整单促销计算失败：" + throwable.toString());
            }
        });
    }

    public void abandonFenDsc() {
        getSale().addAbandonFenDsc(GeneralSettings.INSTANCE.getCurrencyPrecision().value());
        updateTotal();
    }

    /**
     * 结果响应
     */
    private Handler mainHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            hideLoadingDialog();

            if (msg.what == ActivityConstants.RESULT_YST_PAY) {
                String status;
                String sMessage;

                if (msg.obj instanceof String) {
                    showHintDialog("提示", String.valueOf(msg.obj));
                    return;
                }

                String zfno = "";
                String sResponse;
                try {
                    sResponse = String.valueOf(msg.obj);
                    LoggerUtils.getLogger().debug("赢商通支付结果:" + sResponse);
                    //status = org.apache.commons.lang3.StringUtils.substringBetween(sResponse, "<status>", "</status>");
                    //sMessage = org.apache.commons.lang3.StringUtils.substringBetween(sResponse, "<msg>", "</msg>");

                    status = XmlUtils.getXmlString(sResponse, "status");
                    sMessage = XmlUtils.getXmlString(sResponse, "msg");
                    if ("".equals(status)) {
                        status = "F";
                    }
                } catch (Exception ex) {
                    showHintDialog("提示", "response 异常 ......" + ex.getMessage());
                    return;
                }

                if ("F".equals(status)) {
                    showHintDialog("失败", sMessage);
                    return;
                }
                if ("S".equals(status)) {
                    //zfno = org.apache.commons.lang3.StringUtils.substringBetween(sResponse, "<trans_seq>", "</trans_seq>");
                    try {
                        zfno = XmlUtils.getXmlString(sResponse, "trans_seq");
                    } catch (Exception e) {

                    }

                    SalePayModel payModel = new SalePayModel();
                    payModel.setPayCode(mSelectedPayMethod.code());
                    payModel.setPayName(mSelectedPayMethod.name());
                    payModel.setRemark(mSelectedPayMethod.remark());
                    payModel.setPayType(mSelectedPayMethod.payType());
                    payModel.setBuyerLogonId(zfno);
                    payModel.setOutTradeNo("");
                    payModel.setTradeNo("");
                    payModel.setPayTime(DateUtils.dateFormat());
                    payModel.setTotal(mTotalPaid);
                    payModel.setTotalRecv(mTotalPaid);
                    payModel.setTotalPaid(mTotalPaid);
                    payModel.setTotalDscSelf("0");
                    payModel.setTotalDscChannel("0");
                    getSale().addPay(payModel);

                    paySuccess(payModel);
                    return;
                }
            }

            if (msg.what == ActivityConstants.RESULT_MIYA_PAY) {
                String status;
                String sMessage;
                String statusMessage;
                String out_trade_no = "";
                String trade_no = "";

                if (msg.obj instanceof String) {
                    showHintDialog(String.valueOf(msg.obj));
                    return;
                }

                String sResponse;
                String[] ary;
                try {
                    sResponse = String.valueOf(msg.obj);
                    LoggerUtils.getLogger().info("米雅支付结果:" + sResponse);
                    ary = sResponse.split(",");
                    status = "F";
                    statusMessage = "PAYFAIL";
                    sMessage = "支付失败";
                    for (String item : ary) {
                        if (item.indexOf("RETCODE") > 0) {
                            status = item.substring(item.indexOf("RETCODE=") + 8);
                        }
                        if (item.indexOf("F1=") > 0) {
                            sMessage = item.substring(item.indexOf("F1=") + 3);
                        }
                        if (item.indexOf("RETMSG=") > 0) {
                            statusMessage = item.substring(item.indexOf("RETMSG=") + 7, item.length() - 1);
                        }
                    }

                    if ("00".equals(status) && ("PAYSUCCESS".equals(statusMessage) || "REFUNDSUCCESS".equals(statusMessage))) {
                        status = "S";
                    } else {
                        status = "F";
                    }
                    if ("".equals(status)) {
                        status = "F";
                    }
                } catch (Exception ex) {
                    showHintDialog("提示", "response 异常");
                    return;
                }

                if ("F".equals(status)) {
                    showHintDialog("提示", sMessage);
                    return;
                }
                if ("S".equals(status)) {
                    for (String item : ary) {
                        if (item.indexOf("OUT_TRADE_NO") > 0) {
                            out_trade_no = item.substring(item.indexOf("OUT_TRADE_NO=") + 13, item.length() - 1);
                        }
                        if (item.indexOf("TRADE_NO") > 0) {
                            trade_no = item.substring(item.indexOf("TRADE_NO=") + 9, item.length() - 1);
                        }
                    }

                    SalePayModel payModel = new SalePayModel();
                    payModel.setPayCode(mSelectedPayMethod.code());
                    payModel.setPayName(mSelectedPayMethod.name());
                    payModel.setRemark(mSelectedPayMethod.remark());
                    payModel.setPayType(mSelectedPayMethod.payType());
                    payModel.setBuyerLogonId("");
                    payModel.setOutTradeNo(out_trade_no);
                    payModel.setTradeNo(trade_no);
                    payModel.setPayTime(DateUtils.dateFormat());
                    payModel.setTotal(getSale().getTotalUnPaid());
                    payModel.setTotalRecv(getSale().getTotalUnPaid());
                    payModel.setTotalPaid(getSale().getTotalUnPaid());
                    payModel.setTotalDscSelf("0");
                    payModel.setTotalDscChannel("0");
                    getSale().addPay(payModel);

                    paySuccess(payModel);
                }
            }
        }
    };

    /**
     * 预支付：检测是否能够支付
     */
    public void prePay(String totalPaid) {
        authService = new AuthService(mMvpView, totalPaid, mSelectedPayMethod);

        authService.prePay();

    }

}
