package com.hisense.hiaposcmp.presenter.pay;

import android.content.Intent;
import android.os.Bundle;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardPayResult;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.VerifyType;
import com.hisense.crm.webapi.points.PointsService;
import com.hisense.crm.webapi.trans.TransService;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.RefundViewModel;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.view.IView.pay.IPointsPayView;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.webapi.Callback;
import com.hisense.webapi.RetryCallback;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;


/**
 * 积分支付
 *
 * @author dukunbiao(null)  2019-07-25  重构
 */
public class PointsPayPresenter extends BasePresenter<IPointsPayView> {

    private PayMethod mSelectedPayMethod;
    private CardNo mCardNo;
    private VipCard mVipCard;
    private CardSecurity mCardSecurity;
    private ArrayList<RefundViewModel> listPayments;
    private String fSsTotal;
    private SalePayModel mOldSalePayment;
    private String mCardFaceNo;

    public void initSelectedPayMethod(Intent intent) {
        Bundle bundle = intent.getExtras();
        if (bundle == null) {
            mSelectedPayMethod = getSale().getPayMethod();
        } else {
            mSelectedPayMethod = (PayMethod) bundle.getSerializable("payMethod");
            if (bundle.containsKey("payments")) {
                listPayments = (ArrayList) bundle.getSerializable("payments");
            }
            if (bundle.containsKey("fSsTotal")) {
                fSsTotal = bundle.getString("fSsTotal");
            }
        }
    }

    public PayMethod getPayMethod() {
        return mSelectedPayMethod;
    }

    public void queryVerifyType(CardNo cardNo) {
        mCardNo = cardNo;
        showLoadingDialog();
        boolean iseCard = cardNo.getType() != null && cardNo.getType().getTypeCode() == CardNoType.PayCode.getTypeCode();
        if (iseCard) {
            if (GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword()) {
                mMvpView.needVerifyType(true, null);
            } else {
                pointsQuery(cardNo, null);
            }
            return;
        }

        VipService.queryVerifyType(cardNo, mSelectedPayMethod.remark(), new Callback<VerifyType>() {
            @Override
            public void onSuccess(VerifyType verifyType) {
              hideLoadingDialog();
                //都不需要验证
                if (!verifyType.needVerify()) {
                    pointsQuery(cardNo, null);
                } else {
                    mMvpView.needVerifyType(false, verifyType);
                }
            }

            @Override
            public void onError(Throwable throwable) {
              hideLoadingDialog();
                mMvpView.pointsQueryError();
                LoggerUtils.getLogger().debug("手输查询会员验证方式:" + throwable.toString());
                showHintDialog(throwable.getMessage());
            }
        });
    }

    public void pointsQuery(CardSecurity cardSecurity) {
        pointsQuery(mCardNo, cardSecurity);
    }

    private void pointsQuery(CardNo cardNo, CardSecurity cardSecurity) {
        mCardSecurity = cardSecurity;

        showLoadingDialog();
        PointsService.query(cardNo, cardSecurity, cardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword(), new Callback<VipCard>() {
            @Override
            public void onSuccess(VipCard card) {
                mVipCard = card;
              hideLoadingDialog();

                if (cardNo.getType() != null && cardNo.getType().getTypeCode() == CardNoType.PayCode.getTypeCode()) {
                    mCardNo = CardNo.PayCode(card.vipCardNo());
                } else {
                    mCardNo = CardNo.VipCode(card.vipCardNo());
                }
                mCardFaceNo = card.cardFaceNo();
                //积分卡退款
                if (listPayments != null) {
                    String maxRefund = getMaxRefund(mSelectedPayMethod);
                    try {
                        mOldSalePayment = getReturnSale().refundSuggest(mSelectedPayMethod, 0.0);
                    } catch (Exception e) {
                        showHintDialog(e.getMessage());
                        return;
                    }

                    mMvpView.pointsQueryRefund(card, fSsTotal, maxRefund);
                    return;
                }
                //scrm余额查看usableTotal, scrm积分 和crm积分查看usableBalance,  若为0,则提示不允许继续支付,
                // 放在退款判断之后
                if ((mSelectedPayMethod.isScrmRemain() && card.usableTotal() == 0)
                        || ((mSelectedPayMethod.isVipPoints() || mSelectedPayMethod.isScrmVipPoints()) && card.usableBalance() == 0)) {
                    LoggerUtils.getLogger().debug("PointsService此卡可用金额为0");
                    showHintDialog("此卡可用金额为0");
                    return;
                }
                //同一张scrm积分卡只允许使用一次
                if (mSelectedPayMethod.isScrmVipPoints()) {
                    for (SalePayModel salePayModel : getSale().getPays()) {
                        if (salePayModel.getBuyerLogonId().equals(card.cardFaceNo()) && PayMethod.isScrmVipPoints(salePayModel.getRemark())) {
                            showHintDialog("警告", "本次交易已使用该卡支付");
                            return;
                        }
                    }
                }

                //同一张scrm余额只允许使用一次
                if (mSelectedPayMethod.isScrmRemain()) {
                    for (SalePayModel salePayModel : getSale().getPays()) {
                        if (salePayModel.getBuyerLogonId().equals(card.cardFaceNo()) && PayMethod.isScrmRemain(salePayModel.getRemark())) {
                            showHintDialog("警告", "本次交易已使用余额支付");
                            return;
                        }
                    }
                }

                mMvpView.pointsQuerySuccess(card);
            }

            @Override
            public void onError(Throwable t) {
              hideLoadingDialog();
                mMvpView.pointsQueryError();
                LoggerUtils.getLogger().debug("查询余额出错：" + t.toString());
                showHintDialog("查询余额出错", t.getMessage());
            }
        });
    }

    public void pointsPay(Intent data) {
        Bundle bundle = data.getExtras();
        String cardWfTotal = bundle.getString("cardYfTotal");

        if (BigDecimalUtils.isLessAndEquals(getSale().getTotalUnPaid(), "0.001")) {
            showHintDialog("剩余应付金额为0，不能再进行支付");
            return;
        }

        try {
            cardWfTotal = BigDecimalUtils.formatScale(cardWfTotal);
        } catch (Exception e) {
            showHintDialog("未输入支付金额，或金额不合法");
            return;
        }

        if (BigDecimalUtils.isGreater(String.valueOf(cardWfTotal), getSale().getTotalUnPaid())) {
            showHintDialog("输入金额过大，请确保支付金额不大于应付金额，谢谢");
            LoggerUtils.getLogger().debug("支付金额过大，本次支付金额:" + cardWfTotal + ", 未付合计:" + (getSale().getTotalUnPaid()));
            return;
        }

        SaleNoService.save(App.getContext(), Config.org_code, Config.pos_no, getSale().getSaleNo());

        String outTradeNo = CommonUtils.genOutTradeNo(Config.org_code, getSale().getSaleNo());
        SalePayModel payModel = new SalePayModel(mSelectedPayMethod, String.valueOf(cardWfTotal), outTradeNo, getSale().getSaleNo(), getSale().getPays().size() + 1);

        pointsPay(payModel);
    }

    private void pointsPay(SalePayModel payModel) {
        showLoadingDialog();
        PointsService.pay(mCardNo, mCardSecurity, payModel, mCardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword(), new Callback<CardPayResult>() {
            @Override
            public void onSuccess(CardPayResult cardPayResult) {
              hideLoadingDialog();

                pointsPaySuccess(payModel, cardPayResult);
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("积分支付异常:" + throwable.toString());
              hideLoadingDialog();

                pointsPayError(throwable, payModel);
            }
        });
    }

    public void pointsRefund(Intent data) {
        Bundle bundle = data.getExtras();
        final String amount = bundle.getString("cardYfTotal");
        mMvpView.showProgressDialog("正在处理积分/余额退款,请稍后");

        SalePayModel salePayModel = new SalePayModel();
        salePayModel.setSerialNo(getReturnSale().getPays().size());
        salePayModel.setPayCode(mSelectedPayMethod.code());
        salePayModel.setPayName(mSelectedPayMethod.name());
        salePayModel.setTotalPaid(amount);
        salePayModel.setSaleNoTemp(getReturnSale().getSaleNo());
        salePayModel.setPayTime(getReturnSale().getSaleTime());
        salePayModel.setRemark(mSelectedPayMethod.remark());

        int serialNo = mOldSalePayment.getSerialNo();
        String outTradeNo = getTraceNo(getReturnSale().oldSaleNo(), serialNo);
        mOldSalePayment = null;
        PointsService.returns(mCardNo, mCardSecurity, getReturnSale().oldSaleNo(), salePayModel, outTradeNo, mCardNo.getTypeCode() != 7 ||
                GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword(), new Callback<CardPayResult>() {
            @Override
            public void onSuccess(CardPayResult result) {
                mMvpView.hideProgressDialog();

                SalePayModel salePayModel = new SalePayModel();
                salePayModel.setPayCode(mSelectedPayMethod.code());
                salePayModel.setPayName(mSelectedPayMethod.name());
                salePayModel.setPayType(mSelectedPayMethod.payType());
                salePayModel.setRemark(mSelectedPayMethod.remark());
                salePayModel.setBuyerLogonId(mCardFaceNo);
                salePayModel.setTradeNo(result.refNo());
                salePayModel.setOutTradeNo(result.refNo());
                salePayModel.setTotalPaid(amount);
                salePayModel.setTotalRecv(amount);
                salePayModel.setSaleNoTemp(getReturnSale().getSaleNo());
                salePayModel.setBalance(String.valueOf(result.balance()));
                getReturnSale().getPays().add(salePayModel);
                mMvpView.pointsRefundSuccess();
            }

            @Override
            public void onError(Throwable throwable) {
                mMvpView.hideProgressDialog();
                showHintDialog("积分/余额退款失败：" + throwable.getMessage() + getReturnSale().getSaleNo());
            }
        });

    }

    private void pointsPaySuccess(SalePayModel payModel, CardPayResult cardPayResult) {
        payModel.setBuyerLogonId(mVipCard.cardFaceNo());
        payModel.setOutTradeNo(cardPayResult.outTradeNo());
        payModel.setTradeNo(cardPayResult.refNo());
        payModel.setPayTime(cardPayResult.tranTime());
        payModel.setTotalDscSelf("0.00");
        payModel.setTotalDscChannel("0.00");
        payModel.setBalance(String.valueOf(cardPayResult.balance()));
        payModel.setCardSecurity(mCardSecurity);
        payModel.setCardNo(mCardNo);
        getSale().addPay(payModel);

        mMvpView.pointsPaySuccess();
    }

    private void pointsPayError(Throwable throwable, SalePayModel payModel) {
        Throwable cause = throwable.getCause();
        if (NetUtils.isNetException(cause)) {
            showLoadingDialog();
            mMvpView.isNotShowExitDialog(true);
            Service.online(100, 3000, new RetryCallback<OnlineResultData>() {
                @Override
                public void onSuccess(OnlineResultData service) {
                    LoggerUtils.getLogger().debug("积分/余额支付结果查询");
                    mMvpView.isNotShowExitDialog(false);

                    transCheck(payModel);
                }

                @Override
                public void onProgress(int i, final String s) {
                    mMvpView.showProgressDialogNoHide(s);
                    LoggerUtils.getLogger().info(s);
                }

                @Override
                public void onError(Throwable throwable) {
                  hideLoadingDialog();
                    mMvpView.hideProgressDialog();
                    mMvpView.isNotShowExitDialog(false);
                    showHintDialog("无法查询支付状态,请联系管理员处理");
                    LoggerUtils.getLogger().debug("无法查询积分/余额支付状态：" + throwable.getMessage());
                }
            });
        } else {
            LoggerUtils.getLogger().debug("储值卡支付失败：" + throwable.toString());
            showHintDialog("积分/余额支付失败", throwable.getMessage());
        }
    }

    private void transCheck(SalePayModel payModel) {
        TransService.check(mCardNo, payModel, new Callback<CardPayResult>() {
            @Override
            public void onSuccess(CardPayResult cardPayResult) {
              hideLoadingDialog();

                cardPayResult.balance(BigDecimalUtils.subtract(mVipCard.usableTotal(), Double.parseDouble(payModel.getTotalPaid())));

                pointsPaySuccess(payModel, cardPayResult);
            }

            @Override
            public void onError(Throwable throwable) {
              hideLoadingDialog();
                int code = ((WebApiException) throwable).code();
                String tip;
                if (code == 404) {
                    //重新发起支付
                    pointsPay(payModel);
                    return;
                } else if (code == 2) {
                    tip = "支付失败,请重新支付";
                } else {
                    tip = throwable.getMessage();
                }
                LoggerUtils.getLogger().debug("积分/余额查询支付结果:" + throwable.toString());
                showHintDialog(tip);
            }
        });
    }

    /**
     * 返回指定支付方式的最大可退金额。
     *
     * @param payMethod 当前选择的支付方式
     * @return 指定支付方式的最大可退金额
     */
    public String getMaxRefund(PayMethod payMethod) {
        String maxRefund = "0";

        if (payMethod == null) {
            return maxRefund;
        }
        if (payMethod.isCard()) {
            // 银行卡、储值卡、现金可退至储值卡账户
            maxRefund = getMaxRefundByRemark(PayMethod.CARD);
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.UNION_PAY));
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.CASH));
            maxRefund = BigDecimalUtils.subtract(maxRefund, getOverRefundByRemark(PayMethod.VIP_POINTS));
        } else if (payMethod.isVipPoints()) {
            // 银行卡、积分、现金可退至积分账户
            maxRefund = getMaxRefundByRemark(PayMethod.VIP_POINTS);
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.UNION_PAY));
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.CASH));
            maxRefund = BigDecimalUtils.subtract(maxRefund, getOverRefundByRemark(PayMethod.CARD));
        } else {
            maxRefund = getMaxRefundByRemark(payMethod.remark());
        }
        return maxRefund;
    }

    /**
     * 返回指定支付方式的合计可退金额。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的最大可退金额
     */
    private String getMaxRefundByRemark(String payMethod) {
        return BigDecimalUtils.subtract(getTotalReceivedByRemark(payMethod), getTotalRefundByRemark(payMethod));
    }

    /**
     * 返回指定支付方式的合计支付金额。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的合计支付金额
     */
    private String getTotalReceivedByRemark(String payMethod) {
        String total = "0";

        if (payMethod != null) {
            for (int i = 0; i < listPayments.size(); i++) {
                if (listPayments.get(i).payment().getRemark().equals(payMethod)) {
                    total = BigDecimalUtils.add(total, listPayments.get(i).totalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 返回指定支付方式的合计退款金额。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的合计退款金额
     */
    private String getTotalRefundByRemark(String payMethod) {
        String total = "0";

        if (payMethod != null) {
            for (SalePayModel payment : getReturnSale().getPays()) {
                if (payMethod.equals(payment.getRemark())) {
                    total = BigDecimalUtils.add(total, payment.getTotalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 返回指定支付方式(储值卡/积分)的溢退金额。例如：支付金额为150，退款200，则溢退金额为50。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的溢退金额
     */
    private String getOverRefundByRemark(String payMethod) {
        // 根据 HIPOSAPP-509 制定的规则，只有储值卡/积分两种支付方式允许将消费时其它支付方式的支付金额退回
        // 到储值卡/积分账户。
        if (!PayMethod.isCard(payMethod) && !PayMethod.isVipPoints(payMethod) && !PayMethod.isScrmVipPoints(payMethod)) {
            return "0";
        }

        String received = getTotalReceivedByRemark(payMethod);
        String refund = getTotalRefundByRemark(payMethod);
        return BigDecimalUtils.isGreater(refund, received) ? BigDecimalUtils.subtract(refund, received) : "0";
    }
}
