package com.hisense.hiaposcmp.presenter.query;

import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.card.Card;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.VerifyType;
import com.hisense.crm.webapi.card.CardService;
import com.hisense.crm.webapi.points.PointsService;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.view.IView.query.ICardAndPointsQueryView;
import com.hisense.webapi.Callback;

/**
 * 查询模块下 储值卡,积分查询
 *
 * @author wangyaogang 2019-10-12
 */
public class CardAndPointsQueryPresenter extends BasePresenter<ICardAndPointsQueryView> {

    /**
     * 查询验证方式
     *
     * @param cardNo
     * @param payMethod
     */
    public void verify(CardNo cardNo, String payMethod) {
        showLoadingDialog();
        VipService.queryVerifyType(cardNo, payMethod, new Callback<VerifyType>() {
            @Override
            public void onSuccess(VerifyType verifyType) {
                hideLoadingDialog();
                mMvpView.verifySuccess(verifyType);
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().debug("查询会员验证方式:" + throwable.getMessage());
                hideLoadingDialog();
                mMvpView.showErrorMsg(throwable.getMessage());
            }
        });
    }

    /**
     * 查询卡信息
     *
     * @param cardNo
     * @param cardSecurity
     */
    public void queryCard(CardNo cardNo, CardSecurity cardSecurity) {
        showLoadingDialog();
        CardService.query(cardNo, cardSecurity, cardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEPrepaidCardQueryByPassword(), new Callback<Card>() {
            @Override
            public void onSuccess(Card card) {
                hideLoadingDialog();
                mMvpView.queryCardSuccess(card);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showErrorMsg(throwable.getMessage());
                LoggerUtils.getLogger().debug("查询储值卡:" + throwable.getMessage());
            }
        });
    }

    /**
     * 查询积分
     *
     * @param cardNo
     * @param cardSecurity
     */
    public void queryPoints(CardNo cardNo, CardSecurity cardSecurity) {
        showLoadingDialog();
        PointsService.query(cardNo, cardSecurity, cardNo.getTypeCode() != 7 || GeneralSettings.INSTANCE.isEVipPointsCardQueryByPassword(), new Callback<VipCard>() {
            @Override
            public void onSuccess(VipCard vipCard) {
                hideLoadingDialog();
                mMvpView.queryPointsSuccess(vipCard);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                mMvpView.showErrorMsg(throwable.getMessage());
                LoggerUtils.getLogger().debug("查询积分:" + throwable.getMessage());
            }
        });
    }
}
