package com.hisense.hiaposcmp.presenter.query;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.print.Invoice.EInvoice2;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.view.IView.query.ISaleQueryView;
import com.hisense.hipos.invoice.models.EInvoiceConfig;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.invoice.InvoiceService;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.webapi.Callback;

import org.apache.commons.lang3.StringUtils;

/**
 * 查询模块下 储值卡,积分查询
 *
 * @author wangyaogang 2019-10-12
 * @author guolina     2020-05-26   HIPOSAPP-1408 重打小票限制为本收款台需求
 */
public class SaleQueryPresenter extends BasePresenter<ISaleQueryView> {

    public void getMaxSaleNo() {
        mMvpView.getMaxSaleNoSuccess(SaleNoService.getMax(mMvpView.getContext(), Config.org_code, Config.pos_no));
    }

    public void getInvoiceConfig() {
        if (com.hisense.common.util.StringUtils.isEmpty(EInvoice2.getBaseUrl())) {
            InvoiceService.getConfig(new Callback<EInvoiceConfig>() {
                @Override
                public void onSuccess(EInvoiceConfig eInvoiceConfig) {
                    EInvoice2.setBaseUrl(eInvoiceConfig.baseUrl());
                    GeneralSettings.INSTANCE.setInvoiceUrl(eInvoiceConfig.baseUrl());
                }

                @Override
                public void onError(Throwable throwable) {
                    String msg = "电子发票初始化失败：" + throwable.getMessage();
                    LoggerUtils.getLogger().debug(msg);
                    showHintDialog(msg);
                }
            });
        }
    }

    public void querySale(String saleNo) {
        getMainSaleModel().reset();

        if (StringUtils.isBlank(saleNo)) {
            showHintDialog("流水号不能为空");
            return;
        }

        showLoadingDialog();
        int days = GeneralSettings.INSTANCE.getReturnSaleDays();
        String posNo;
        if(GeneralSettings.INSTANCE.isNoShopLimits()) {
            posNo = "*";
        } else {
            posNo = Context.posNo();
        }
        boolean openRpt = GeneralSettings.INSTANCE.isOpenRpt();
        SaleService.query(openRpt, saleNo, days, posNo, new Callback<BaseSaleModel>() {
            @Override
            public void onSuccess(BaseSaleModel sale) {
                hideLoadingDialog();
                if (sale != null) {
                    mMvpView.querySaleSuccess(sale);
                } else {
                    mMvpView.querySaleFail(null);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().debug("查询流水:" + throwable.getMessage());
                hideLoadingDialog();
                mMvpView.querySaleFail(throwable.getMessage());
            }
        });
    }

    public void reprintTrade(BaseSaleModel saleModel) {
        if (EInvoice2.getBaseUrl() == null) {
            showHintDialog("电子发票未初始化");
            return;
        }
        if (saleModel == null) {
            showHintDialog("请先查询流水");
            return;
        }

        showLoadingDialog();
        PrinterHelper.getInstance().reprint(saleModel, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                hideLoadingDialog();
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
            }
        });
    }
}
