package com.hisense.hiaposcmp.presenter.refund;

import android.app.AlertDialog;
import android.database.SQLException;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.crm.vip.RealTimePoint;
import com.hisense.crm.webapi.points.PointsService;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.activity.StartActivityService;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.RefundViewModel;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.SaleRefundItemsAdapter;
import com.hisense.hiaposcmp.adapter.RefundMethodStdAdapter;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.dialog.ListDialog;
import com.hisense.hiaposcmp.pos.payment.UmsPaymentFactory;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.service.payment.AggregatePaymentChannel;
import com.hisense.hiaposcmp.service.payment.PayMethodService;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.view.IView.refund.IRefundView;
import com.hisense.hipos.miya.RefundMIYAResponse;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.webapi.coupon.CouponService;
import com.hisense.hipos.webapi.coupon.CouponUsedData;
import com.hisense.hipos.webapi.pay.PayResultData;
import com.hisense.hipos.webapi.pay.PayService;
import com.hisense.hipos.webapi.sale.SaleSubmitResultData;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PayResult;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.icbc.IcbcPaymentProvider;
import com.hisense.webapi.Callback;
import com.hisense.webapi.RetryCallback;
import com.hisense.webapi.WebApiException;
import com.sprite.ystpay.api.object.res.RefundResponse;

import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import io.reactivex.Observable;
import io.reactivex.ObservableEmitter;
import io.reactivex.ObservableOnSubscribe;
import io.reactivex.Observer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

import static com.hisense.hipos.payment.PayMethod.getAllInPayRemark;

/**
 * 退款 presenter
 *
 * @author xieliyan    2019-10-26
 * @author guolina     2019-11-22 HIAPOSCMP-335   海信支付的退款查询返回成功后，查询中提示框不消失
 * @author guolina     2019-12-05 HIAPOSCMP-388   修复退款时未选择支付方式点击退款确认按钮的闪退
 * @author wangyaogang 2019-11-27 HIAPOSCMP-284 海信支付打印签购单
 */
public class RefundPresenter extends BasePresenter<IRefundView> {

    /**
     * 支付/退款集合
     */
    private ArrayList<RefundViewModel> listPayments;
    /**
     * 支付/退款列表适配器
     */
    private SaleRefundItemsAdapter returnPaymentsAdapter;

    /**
     * 支付/退款列表适配器
     */
    private RefundMethodStdAdapter mRefundMethodStdAdapter;

    /**
     * 当前选择的支付方式
     */
    private PayMethod mSelectedPayMethod;       // 当前选择的支付方式
    /**
     * 当前退款的原始交易
     */
    private SalePayModel mOldSalePayment;
    /**
     * 储值卡多笔相同支付号的原交易
     */
    private Collection<SalePayModel> oldSalePaymentList;
    /**
     * 一笔订单只可以用一次满减
     */
    private Boolean mIsTotalReduction = true;
    /**
     * 退款交易订单号
     */
    private String mRefundOutTradeNo = "";
    /**
     * 支付方式提供器
     */
    private IPaymentProvider mPaymentProvider;
    /**
     * 终端号（银行使用）
     */
    private String termNo = "";
    /**
     * 商户订单号 建行直连使用
     */
    private String outTradeNo = "";
    /**
     * 商户号 建行直连使用
     */
    private String mchId = "";
    /**
     * 用于区分每笔收单行交易
     */
    private volatile String uniqueNo = "";
    /**
     * 积分原路退回是否成功执行。true 表示已执行。
     */
    private boolean mIsVipPointsReturned;
    /**
     * 赠券原路退回是否成功执行。true 表示已执行。
     */
    private boolean mIsCouponsReturned;
    /**
     * 是否上传过流水
     */
    private volatile boolean hasUploaded;
    /**
     * 一点通券
     */
    private String ydtCoupon;

    /**
     * traceNo
     */
    private String traceNo;

    /**
     * 初始化消费用支付方式集合
     */
    public void initPayments() {
        listPayments = new ArrayList<>();
    }

    /**
     * 返回消费用支付方式集合
     *
     * @return listPayments
     */
    public ArrayList<RefundViewModel> getListPayments() {
        if (listPayments == null) {
            listPayments = new ArrayList<>();
        }
        return listPayments;
    }

    /**
     * 设置消费用支付方式集合
     *
     * @param payments 退款方式
     */
    public void setListPayments(ArrayList<RefundViewModel> payments) {
        this.listPayments = payments;
    }

    /**
     * 返回退款方式
     *
     * @return mOldSalePayment
     */
    public SalePayModel oldSalePayment() {
        return mOldSalePayment;
    }

    /**
     * 设置退款方式
     *
     * @param salePayModel 支付方式
     */
    public void oldSalePayment(SalePayModel salePayModel) {
        this.mOldSalePayment = salePayModel;
    }

    /**
     * 原支付方式列表
     *
     * @return oldSalePaymentList 支付方式
     */
    public Collection<SalePayModel> oldSalePaymentList() {
        return oldSalePaymentList;
    }

    /**
     * 获取适配器
     *
     * @return returnPaymentsAdapter 适配器
     */
    public SaleRefundItemsAdapter getAdapter() {
        return returnPaymentsAdapter;
    }

    public IPaymentProvider paymentProvider() {
        return mPaymentProvider;
    }

    /**
     * 初始化适配器
     *
     * @return returnPaymentsAdapter 适配器
     */
    public SaleRefundItemsAdapter adapter() {
        returnPaymentsAdapter = new SaleRefundItemsAdapter(App.getContext(), listPayments, new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });
        return returnPaymentsAdapter;

    }

    /**
     * 初始化适配器
     *
     * @return returnPaymentsAdapter 适配器
     */
    public RefundMethodStdAdapter stdAdapter() {
        mRefundMethodStdAdapter = new RefundMethodStdAdapter(App.getContext(), listPayments);
        return mRefundMethodStdAdapter;

    }

    /**
     * 初始化适配器
     *
     * @return returnPaymentsAdapter 适配器
     */
    public RefundMethodStdAdapter getStdAdapter() {
        return mRefundMethodStdAdapter;

    }

    /**
     * 退款交易订单号
     *
     * @return mRefundOutTradeNo 交易订单号
     */
    public String refundOutTradeNo() {
        return mRefundOutTradeNo;
    }

    /**
     * 设置退款交易订单号
     *
     * @@param outTradeNo 交易订单号
     */
    public void refundOutTradeNo(String outTradeNo) {
        this.mRefundOutTradeNo = outTradeNo;
    }

    /**
     * 返回终端号
     *
     * @@param termNo 终端号
     */
    public String termNo() {
        return termNo;
    }

    /**
     * 是否上传过流水
     *
     * @return hasUploaded
     */
    public boolean hasUploaded() {
        return hasUploaded;
    }

    /**
     * 是否上传过流水
     *
     * @param hasUploaded
     */
    public void hasUploaded(boolean hasUploaded) {
        this.hasUploaded = hasUploaded;
    }

    /**
     * 获取当前支付方式
     *
     * @return mSelectedPayMethod  当前支付方式
     */
    public PayMethod selectedPayMethod() {
        return mSelectedPayMethod;
    }

    /**
     * 获取支付方式
     */
    public void getPayMethods() {
        // 查询支付方式
        PayMethodService.createCompactLayout(mMvpView.getActivity(), mMvpView.paymentClick(), new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                for (SalePayModel payment : getReturnSale().getSaleModel().getPays()) {
                    for (PayMethod method : payMethods) {
                        if (payment.getPayCode().equals(method.code())
                                || (PayMethod.YI_DIAN_TONG.equalsIgnoreCase(getAllInPayRemark(payment.getRemark())) &&
                                PayMethod.YI_DIAN_TONG.equalsIgnoreCase(getAllInPayRemark(method.remark())))) {
                            listPayments.add(new RefundViewModel(payment));
                            payment.setPayType(method.payType());
                            payment.setRemark(method.remark());

                            break;
                        }
                    }
                }
                mMvpView.setAdapter();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().debug("获取支付方式异常" + throwable.toString());

            }
        });
    }


    /**
     * 清空实时积分信息
     */
    public void clearPoints() {
        getReturnSale().setExpensePoint("0");
        getReturnSale().setParkingPoint("0");
        getReturnSale().setCouponPoint("0");
    }

    /**
     * 选择支付方式操作
     *
     * @param method 当前选择支付方式
     */
    public boolean selectPayMethod(PayMethod method) {
        if (method == null) {
            return false;
        }

        mSelectedPayMethod = method;
        // 切换支付方式后，将建议的原支付交易重置为null
        mOldSalePayment = null;
        oldSalePaymentList = null;

        showLoadingDialog();
        String maxRefund = getMaxRefund(mSelectedPayMethod);
        String refund = BigDecimalUtils.ternaryOperatorLess(maxRefund, getReturnSale().getTotalUnPaid());

        mMvpView.setRefundingText(refund);
        if (StringUtils.isEmpty(mMvpView.refundTextValue())) {
            hideLoadingDialog();
            return false;
        }

        if (BigDecimalUtils.isEquals0(maxRefund)) {
            hideLoadingDialog();
            LoggerUtils.getLogger().debug(String.format("%s可退款金额为零", mSelectedPayMethod.name()));
            showHintDialog(String.format("%s可退款金额为零,请选择其它支付方式", mSelectedPayMethod.name()));
            return false;
        }


        return gotoPayMethod(maxRefund);
    }

    private boolean gotoPayMethod(String maxRefund) {
        boolean bool = true;
        if (mSelectedPayMethod.isCardType()) {
            isEcard = false;
            StartActivityService.gotoCardPayActivity(App.getContext(), mSelectedPayMethod, listPayments, mMvpView.refundTextValue());
        } else if (mSelectedPayMethod.isPointsType() || mSelectedPayMethod.isRemainType()) {
            isEcard = false;
            StartActivityService.gotoPointsPayActivity(App.getContext(), mSelectedPayMethod, listPayments, mMvpView.refundTextValue());
        } else if (mSelectedPayMethod.isVipCoupon()) {
            mMvpView.goToCouponRefund(ActivityConstants.REQUEST_REFUND_TO_COUPON, maxRefund);
        } else if (mSelectedPayMethod.isScrmCoupon()) {
            if (getReturnSale().calcCount() != getReturnSale().getSaleModel().calcCount()) {
                showHintDialog("非整单退货，禁止使用该支付方式");
                hideLoadingDialog();
                return false;
            }

            mMvpView.goToCouponRefund(ActivityConstants.REQUEST_REFUND_TO_SCRM_COUPON, maxRefund);
        } else if (mSelectedPayMethod.isOther()) {
            mMvpView.goToOtherRefund();
        } else if (mSelectedPayMethod.isCardByPayType()) {
            isEcard = false;
            StartActivityService.gotoCardPayActivity(App.getContext(), mSelectedPayMethod, listPayments, mMvpView.refundTextValue());
        } else if (mSelectedPayMethod.isCouponByPayType()) {
            mMvpView.goToCouponRefund(ActivityConstants.REQUEST_REFUND_TO_SCRM_COUPON, maxRefund);
        } else {
            bool = false;
        }

        hideLoadingDialog();
        return bool;
    }


    /**
     * 点击确定退款
     */
    public void refund() {
        if (null == mSelectedPayMethod) {
            hideLoadingDialog();
            LoggerUtils.getLogger().debug("请选择退款方式");
            showHintDialog("请选择退款方式");
            return;
        }
        if (mSelectedPayMethod.isScrmCoupon()) {
            if (getReturnSale().calcCount() != getReturnSale().getSaleModel().calcCount()) {
                showHintDialog("非整单退货，禁止使用该支付方式");
                return;
            }
        }

        if (StringUtils.isEmpty(mMvpView.refundTextValue()) || mMvpView.refundTextValue().equals("0")) {
            showHintDialog("请输入退款金额");
            return;
        }

        showLoadingDialog();
        String refunding = BigDecimalUtils.formatScale(mMvpView.refundTextValue());
        String maxRefund = getMaxRefund(mSelectedPayMethod);

        if (BigDecimalUtils.isEquals0(maxRefund)) {
            hideLoadingDialog();
            LoggerUtils.getLogger().debug(String.format("%s支付可退金额为0", mSelectedPayMethod.name()));
            showHintDialog(String.format("%s支付可退金额为0", mSelectedPayMethod.name()));
            return;
        }

        if (BigDecimalUtils.isGreater(refunding, getReturnSale().getTotalUnPaid())) {
            hideLoadingDialog();
            LoggerUtils.getLogger().debug(String.format("%s支付不允许退款,%s", mSelectedPayMethod.name(), getReturnSale().getTotalUnPaid()));
            showHintDialog(String.format("%s支付不允许退款,%s", mSelectedPayMethod.name(), getReturnSale().getTotalUnPaid()));
            return;
        }

        if (BigDecimalUtils.isGreater(refunding, String.valueOf(maxRefund))) {
            hideLoadingDialog();
            LoggerUtils.getLogger().debug("退款金额不能大于可退金额:" + maxRefund);
            showHintDialog("退款金额不能大于可退金额:" + maxRefund);
            return;
        }

        if (mSelectedPayMethod.isTotalReduction()) {
            if (mIsTotalReduction) {
                mIsTotalReduction = false;
                hideLoadingDialog();
                getReturnSale().refundTotalReduction(refunding);
                mMvpView.refreshAfterRefund();
            } else {
                hideLoadingDialog();
                showHintDialog("该笔订单已使用过" + mSelectedPayMethod.name());
            }
        } else if (mSelectedPayMethod.isCash()) {
            SalePayModel salePayModel = new SalePayModel(mSelectedPayMethod, "", refunding);

            refreshAfterRefund(salePayModel);
        } else {
            if (!gotoPayMethod(refunding)) {
                gotoRefundChannel(refunding);
            }
        }
    }


    public void refundStd(PayMethod method) {
        if (selectPayMethod(method)) {
            return;
        }
        if (mSelectedPayMethod.isScrmCoupon()) {
            if (getReturnSale().calcCount() != getReturnSale().getSaleModel().calcCount()) {
                mMvpView.showHintDialog("非整单退货，禁止使用该支付方式");
                return;
            }
        }

        mMvpView.showLoadingDialog();
        String refunding = BigDecimalUtils.formatScale(mMvpView.refundTextValue());
        String maxRefund = getMaxRefund(mSelectedPayMethod);

        if (BigDecimalUtils.isEquals0(maxRefund)) {
            mMvpView.hideLoadingDialog();
            LoggerUtils.getLogger().debug(String.format("%s支付可退金额为0", mSelectedPayMethod.name()));
            //      mMvpView.showHintDialog(String.format("%s支付可退金额为0", mSelectedPayMethod.name()));
            return;
        }

        if (BigDecimalUtils.isGreater(refunding, getReturnSale().getTotalUnPaid())) {
            mMvpView.hideLoadingDialog();
            LoggerUtils.getLogger().debug(String.format("%s支付不允许退款,%s", mSelectedPayMethod.name(), getReturnSale().getTotalUnPaid()));
            mMvpView.showHintDialog(String.format("%s支付不允许退款,%s", mSelectedPayMethod.name(), getReturnSale().getTotalUnPaid()));
            return;
        }

        if (BigDecimalUtils.isGreater(refunding, String.valueOf(maxRefund))) {
            mMvpView.hideLoadingDialog();
            LoggerUtils.getLogger().debug("退款金额不能大于可退金额:" + maxRefund);
            mMvpView.showHintDialog("退款金额不能大于可退金额:" + maxRefund);
            return;
        }
        if (mSelectedPayMethod.isTotalReduction()) {
            if (mIsTotalReduction) {
                mIsTotalReduction = false;
                mMvpView.hideLoadingDialog();
                getReturnSale().refundTotalReduction(refunding);
                mMvpView.refreshAfterRefund();
            } else {
                mMvpView.hideLoadingDialog();
                mMvpView.showHintDialog("该笔订单已使用过" + mSelectedPayMethod.name());
            }
        } else if (mSelectedPayMethod.isCash()) {
            mMvpView.hideLoadingDialog();
            mMvpView.showHintConfirm(String.format("确认现金退款%s元", refunding), new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    SalePayModel salePayModel = new SalePayModel(mSelectedPayMethod, "", refunding);

                    refreshAfterRefund(salePayModel);
                }

                @Override
                public void onError(String err) {
                    mMvpView.hideLoadingDialog();

                }
            });

        } else {
            if (!gotoPayMethod(refunding)) {
                gotoRefundChannelStd(refunding);
            }
        }

    }

    private void gotoRefundChannelStd(String refunding) {

        if (selectedPayMethod().isAllInPayType()
                || selectedPayMethod().isAllOrChannelType()
                || selectedPayMethod().isAllInPayByPayType()) {
            StartActivityService.gotoIntePayActivity(mMvpView.getActivity(), selectedPayMethod(), refunding);
        } else {
            mMvpView.showHintDialog("暂未支持该支付方式");
        }
    }

    public SalePayModel oldSalePaymentStd(String outTradeNo) {
        // 查找与退款对应的原始支付交易
        try {
            mOldSalePayment = getReturnSale().refundSuggestByTraceNo(mSelectedPayMethod, outTradeNo);
        } catch (Exception e) {
            mMvpView.showHintDialog(e.getMessage());
            return null;
        }
        return mOldSalePayment;

    }

    private void gotoRefundChannel(String refunding) {
        if (mSelectedPayMethod.isAllInPayType()) {
            if (mSelectedPayMethod.isYdt()) {
                payRefundYdt(refunding);
                return;
            }
        }
        // 查找与退款对应的原始支付交易
        try {
            mOldSalePayment = getReturnSale().refundSuggest(mSelectedPayMethod, Double.valueOf(refunding));
        } catch (Exception e) {
            showHintDialog(e.getMessage());
            return;
        }
        // 国美微信支付宝退款
        if (mSelectedPayMethod.isWxPay() || mSelectedPayMethod.isAliPay()) {
            LoggerUtils.getLogger().debug("国美微信支付宝退款");
            String tradeNo = mOldSalePayment.tradeNo;
            String outTradeNo = mOldSalePayment.outTradeNo;
            String traceNo = getTraceNo(getReturnSale().oldSaleNo(), mOldSalePayment.getSerialNo());
            if (null == traceNo) {
                showTraceNoDialog();
            } else {
                refunding = BigDecimalUtils.formatScale(mMvpView.refundTextValue());
                payRefundStd(refunding, traceNo, tradeNo);
            }
            return;
        }
        if (mSelectedPayMethod.isAllInPayType()) {
            LoggerUtils.getLogger().debug("海信全支付退款");
            payRefund(mOldSalePayment);
            return;
        }
        try {
            mPaymentProvider = mMvpView.getPaymentProvider();
        } catch (Exception e) {
            showHintDialog("获取支付提供程序失败：" + e.toString());
            return;
        }

        if (mOldSalePayment == null) {
            showHintDialog("未找到与退款对应的原始支付交易");
            return;
        }
        if (mPaymentProvider != null) {
            int serialNo = mOldSalePayment.getSerialNo();
            String traceNo = getTraceNo(getReturnSale().oldSaleNo(), serialNo);
            termNo = getTermNo(getReturnSale().oldSaleNo(), serialNo);
            //建行直连使用
            outTradeNo = getOutTradeNo(getReturnSale().oldSaleNo(), serialNo);
            mchId = getMchId(getReturnSale().oldSaleNo(), serialNo);
            refundUms(Double.valueOf(refunding), traceNo, mOldSalePayment.getTradeNo());
        } else if (mSelectedPayMethod.isAllOrChannelType()) {
            if (AggregatePaymentChannel.isYST()) {
                refundYst(mSelectedPayMethod);
            } else if (AggregatePaymentChannel.isMIYA()) {
                refundMiya(mSelectedPayMethod);
            } else if (AggregatePaymentChannel.isHXPAY()) {
//                if (mSelectedPayMethod.isAliPay() && CommonUtils.isTimesSquare(App.getContext())) {
//                    LoggerUtils.getLogger().debug("温州时代-海信Pay-支付宝走收单行渠道-refund");
//                    try {
//                        mPaymentProvider = UmsPaymentFactory.createProvider(App.getContext(), mSelectedPayMethod.remark());
//                    } catch (Exception e) {
//                        showHintDialog("获取支付提供程序失败：" + e.toString());
//                        return;
//                    }
//                    int serialNo = mOldSalePayment.getSerialNo();
//                    String traceNo = getTraceNo(getReturnSale().oldSaleNo(), serialNo);
//                    termNo = getTermNo(getReturnSale().oldSaleNo(), serialNo);
//                    refundUms(Double.valueOf(refunding), traceNo, mOldSalePayment.getTradeNo());
//                } else {
                payRefund(mOldSalePayment);
                //  }
            }
        } else if (mSelectedPayMethod.isAllInPayByPayType()) {
            payRefund(mOldSalePayment);
        } else {
            showHintDialog("暂未支持该支付方式");
        }
    }

    private void payRefund(SalePayModel oldSalePayment) {
        String traceNo = getTraceNo(getReturnSale().oldSaleNo(), oldSalePayment.getSerialNo());
        if (null == traceNo) {
            showTraceNoDialog();
        } else {
            payRefund(traceNo);
        }
    }

    private void payRefundYdt(String refunding) {
        LoggerUtils.getLogger().debug("利客来一点通退款:" + refunding);

        /*
         * 选择一点通退货时,先选择退货方式,若是用券,再输入券号,若是其他则直接调起摄像头扫码oldOutTradeNo,
         * 通过oldOutTradeNo 查询原始交易,然后退货.
         */
        final String[] mItems = new String[]{"退   券", "其   他"};
        ListDialog dialog = new ListDialog();
        dialog.setTitle("选择退款方式");
        dialog.setSingleChoiceItems(mItems, -1, (view, position) -> {
            switch (position) {
                // 退券
                case 0:
                    LoggerUtils.getLogger().debug("一点通:券");
                    showCouponDialog();
                    break;
                // 其他
                case 1:
                    LoggerUtils.getLogger().debug("一点通:其他");
                    mMvpView.openCamera();
                    break;
                // 默认
                default:
                    break;
            }
        });
        dialog.show();
    }

    /**
     * 调用实现 {@code IPaymentProvider} 接口的退货。
     * <p>
     * <p>如果 {@code IPaymentProvider} 的实例支持当日撤销，则弹出对话框让用户选择当日撤销/隔日退货。否则，直接
     * 调用退货。</p>
     *
     * @param amount  退款金额
     * @param traceNo 交易凭证号
     * @param refNo   交易参考号
     */
    private void refundUms(double amount, String traceNo, String refNo) {
        if (mPaymentProvider.supportCancellation()) {
            // 对于支持撤销的交易，需用户确认调用当日撤销or隔日退货
            mMvpView.goToUnionRefundConfirm(amount, traceNo, refNo);
        } else {
            //退货总金额
            String maxRefund = getMaxRefund(mSelectedPayMethod);
            // 对于不支持撤销的交易，直接调用退货
            HashMap<String, String> params = new HashMap<>();
            params.put("PAY_NO", mOldSalePayment.getBuyerLogonId());
            params.put("TERM_NO", ""); // 用于工行收单
            //用于建行直连支付
            params.put("refund_fee", String.valueOf(amount));//退货金额
            params.put("mch_id", mchId);
            // params.put("transaction_id",params.get("transaction_id"));
            params.put("out_trade_no", outTradeNo);
            params.put("amount", String.valueOf(maxRefund));

            refund(amount, null, refNo, params);
        }
    }

    /**
     * 赢商通退货
     */
    private void refundYst(PayMethod payMethod) {
        final String payType = payMethod.isWxPay() ? "微信" : "支付宝";
        if (!mYstService.isInitialized()) {

            showHintDialog("赢商通配置错误,请确认配置参数");
            return;
        }

        String payNo = "";
        double payed = 0;

        for (RefundViewModel map : listPayments) {
            if (map.payment().getRemark().compareToIgnoreCase(payMethod.remark()) == 0) {

                if (BigDecimalUtils.isEquals(String.valueOf(map.totalPaid()), String.valueOf(map.totalRefund()))) {
                    continue;//已经退款完成，不能继续退款
                }

                payNo = map.payNo();
                payed = Math.abs(Double.parseDouble(map.totalPaid()));

                payed = Double.valueOf(BigDecimalUtils.ternaryOperatorGreater(String.valueOf(payed), mMvpView.refundTextValue()));

                break;
            }
        }
        if (StringUtils.isEmpty(payNo)) {
            showHintDialog("不能" + payType + "退款，请选用其他的方式");
            return;
        }

        final String tranSeq = payNo;
        final double finalPayed = payed;
        Observable.create(new ObservableOnSubscribe<RefundResponse>() {
            @Override
            public void subscribe(ObservableEmitter<RefundResponse> e) {
                RefundResponse response = mYstService.refund(getReturnSale().oldSaleNo(),
                        mMvpView.refundTextValue(), tranSeq);
                e.onNext(response);
                e.onComplete();
            }
        })
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<RefundResponse>() {
                    @Override
                    public void onSubscribe(Disposable d) {
                    }

                    @Override
                    public void onNext(RefundResponse refundResponse) {
                        if ("S".equals(refundResponse.getStatus())) {//退款成功
                            SalePayModel salePayModel = new SalePayModel(mSelectedPayMethod,
                                    refundResponse.getRefundSequence(), String.valueOf(finalPayed));

                            refreshAfterRefund(salePayModel);
                        } else {
                            showHintDialog(payType + "退款失败" + refundResponse.getMessage());
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        e.printStackTrace();
                        showHintDialog(payType + "退款失败");
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    /**
     * 米雅支付退货
     */
    private void refundMiya(PayMethod payMethod) {

        final String payType = payMethod.isWxPay() ? "微信" : "支付宝";
        if (!mMiYaService.isInitialized()) {
            showHintDialog("米雅配置错误,请确认配置参数");
            return;
        }
        String payed = getMaxRefund(payMethod);
        final String tranSeq = mOldSalePayment.getBuyerLogonId();
        final String finalPayed = payed;
        Observable.create(new ObservableOnSubscribe<RefundMIYAResponse>() {
            @Override
            public void subscribe(ObservableEmitter<RefundMIYAResponse> e) throws Exception {
                RefundMIYAResponse response = mMiYaService.refund(getReturnSale().oldSaleNo(), mMvpView.refundTextValue(), tranSeq);
                e.onNext(response);
                e.onComplete();
            }
        })
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<RefundMIYAResponse>() {
                    @Override
                    public void onSubscribe(Disposable d) {

                    }

                    @Override
                    public void onNext(RefundMIYAResponse refundResponse) {
                        if ("00".equals(refundResponse.getStatus())
                                && "REFUNDSUCCESS".equals(refundResponse.getResponseMessage())) {//退款成功
                            SalePayModel salePayModel = new SalePayModel(mSelectedPayMethod,
                                    refundResponse.RefundSequence, finalPayed);
                            refreshAfterRefund(salePayModel);

                        } else {
                            showHintDialog(payType + "退款失败" + refundResponse.getResponseMessage());
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        e.printStackTrace();
                        showHintDialog(payType + "退款失败");
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    /**
     * 调用 {@code IPaymentProvider} 实例的当日撤销/隔日退货接口。
     * <p>
     * <p>如果 {@param traceNo} 不为空，则调用当日撤销接口；否则，调用隔日退货接口。</p>
     *
     * @param amount  退款金额
     * @param traceNo 交易凭证号
     * @param refNo   交易参考号
     * @param params  扩展参数
     */
    public void refund(double amount, String traceNo, String refNo, Map<String, String> params) {
        try {
            Date saleDate = DateUtils.dateParse(getReturnSale().getSaleModel().getSaleTime());

            if (!StringUtils.isEmpty(traceNo)) {
                // 当日撤销
                uniqueNo = PayRequest.generateUniqueNo(com.hisense.hipos.webapi.Context.posNo());
                PayRequest payRequest = new PayRequest(amount, uniqueNo);
                mPaymentProvider.cancel(payRequest, traceNo, params, new PaymentHandler() {
                    @Override
                    public void onFinish(Object... args) {
                        handleIcbcRefundResult(args);
                    }
                });
            } else {
                // 隔日退货
                uniqueNo = PayRequest.generateUniqueNo(com.hisense.hipos.webapi.Context.posNo());
                PayRequest payRequest = new PayRequest(amount, uniqueNo);
                mPaymentProvider.refund(payRequest, refNo, saleDate, params, new PaymentHandler() {
                    @Override
                    public void onFinish(Object... args) {
                        handleIcbcRefundResult(args);
                    }
                });
            }
        } catch (Exception ex) {
            showHintDialog(ex.getMessage());
            LoggerUtils.getLogger().error(ex.getMessage(), ex);
        }
    }

    /**
     * 工行调用成功后处理
     */
    private void handleIcbcRefundResult(Object... args) {
        if (args == null || args.length != 3) {
            return;
        }

        try {
            IcbcPaymentProvider provider = (IcbcPaymentProvider) mPaymentProvider;
            PayResult result = provider.handleResult(args);
            SalePayModel salePayModel = new SalePayModel(mSelectedPayMethod, result.cardNo(),
                    String.valueOf(result.amount()), result.traceNo(), result.refNo(), "0");

            refreshAfterRefund(salePayModel);
        } catch (Exception ex) {
            LoggerUtils.getLogger().error(ex.getMessage(), ex);
            showHintDialog(ex.getMessage());
        }
    }

    /**
     * traceNo
     */
    private void showTraceNoDialog() {
        AlertDialog traceNoDialog; //traceNo录入框

        View view = LayoutInflater.from(mMvpView.getContext()).inflate(R.layout.dialog_input_traceno, null, false);
        traceNoDialog = new AlertDialog.Builder(mMvpView.getContext()).setView(view).create();
        traceNoDialog.setCanceledOnTouchOutside(false);

        final EditText ed_time = view.findViewById(R.id.ed_time);

        final Button btn_cancel = view.findViewById(R.id.btn_cancel);
        final Button btn_determine = view.findViewById(R.id.btn_determine);

        btn_determine.setOnClickListener((v) -> {
            traceNo = ed_time.getText().toString().trim(); //
            if (TextUtils.isEmpty(traceNo)) {
                showHintDialog("请输入支付订单号");
            } else {
                LoggerUtils.getLogger().debug("traceNo:" + traceNo);
                traceNoDialog.dismiss();
                payRefund(traceNo);

            }
        });

        btn_cancel.setOnClickListener((v) -> {
            traceNo = "";
            traceNoDialog.dismiss();
        });

        traceNoDialog.show();
    }

    /**
     * 一点通
     */
    private void showCouponDialog() {
        AlertDialog couponDialog; //券号录入框

        View view = LayoutInflater.from(mMvpView.getContext()).inflate(R.layout.dialog_input_coupon_no, null, false);
        couponDialog = new AlertDialog.Builder(mMvpView.getContext()).setView(view).create();
        couponDialog.setCanceledOnTouchOutside(false);

        final EditText ed_time = view.findViewById(R.id.ed_time);

        final Button btn_cancel = view.findViewById(R.id.btn_cancel);
        final Button btn_determine = view.findViewById(R.id.btn_determine);

        btn_determine.setOnClickListener((v) -> {
            ydtCoupon = ed_time.getText().toString().trim(); //一点通退券券号
            if (TextUtils.isEmpty(ydtCoupon)) {
                showHintDialog("请输入券号");
            } else {
                LoggerUtils.getLogger().debug("一点通:退券:" + ydtCoupon);
                //打开摄像头,dismiss
                couponDialog.dismiss();
                mMvpView.openCamera();
            }
        });

        btn_cancel.setOnClickListener((v) -> {
            ydtCoupon = "";
            couponDialog.dismiss();
        });

        couponDialog.show();
    }

    /**
     * 海信支付退款
     *
     * @param traceNo 原始交易号(一点通通过扫码获得,海信pay查询获得)
     */
    public void payRefund(final String traceNo) {
        String refunding = BigDecimalUtils.formatScale(mMvpView.refundTextValue());

        payRefundStd(traceNo, refunding);

    }

    public void payRefundStd(String refunding, final String traceNo, String tradeNo) {
        final String totalRefund = refunding;
        mRefundOutTradeNo = PayService.newOutTradeNo(traceNo, getReturnSale());
        PayService.refund("", tradeNo, traceNo, getReturnSale(), mSelectedPayMethod.payRemark(),
                mRefundOutTradeNo, mOldSalePayment.getTotalPaid(), refunding, GeneralSettings.INSTANCE.getTerminalId(),
                new Callback<PayResultData>() {
                    @Override
                    public void onSuccess(PayResultData result) {
                        handleResultRefund(result, false);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        payRefundError(throwable, traceNo, mOldSalePayment.getTotalPaid(), totalRefund, false);
                    }
                });
    }

    /**
     * 海信支付退款
     *
     * @param traceNo 原始交易号(一点通通过扫码获得,海信pay查询获得)
     */
    public void payRefundStd(final String traceNo, String refunding) {
        boolean isYdt = mSelectedPayMethod.isYdt();
        if (isYdt) {
            try {
                mOldSalePayment = getReturnSale().refundSuggest(traceNo);
                refunding = String.valueOf(mOldSalePayment.totalUnRefund());
                mSelectedPayMethod = new PayMethod(mOldSalePayment.getPayCode(), mOldSalePayment.getPayName(),
                        mOldSalePayment.getRemark(), mOldSalePayment.getPayType());
            } catch (Exception e) {
                LoggerUtils.getLogger().debug("一点通退款查询原交易:" + e.toString());
                showHintDialog(e.getMessage());
                return;
            }
        }
        LoggerUtils.getLogger().debug("海信Pay退款金额:" + refunding);

        // 每次都重新生成新的商户订单号  20200429
        // 每笔退款仅对应一个商户订单号，退款成功后再次退款时才生成另外一个新的商户订单号。
        // 此处使用 mRefundOutTradeNo 全局变量保存商户订单号：
        // 1、每次退款时通过检查 mRefundOutTradeNo 是否为 null 或空字符串判断是否生成一个新的商户订单号。
        // 2、退款成功后将 mRefundOutTradeNo 设置未 null，再次退款时生成一个新的商户订单号。
        // 3、退款失败时不需要处理，再次退款时仍使用同一个 mRefundOutTradeNo。
        if (isYdt) {
            mRefundOutTradeNo = CommonUtils.ydtOutTradeNo(getSale().getSaleNo());
        } else {
            mRefundOutTradeNo = PayService.newOutTradeNo(traceNo, getReturnSale());
        }
        showLoadingDialog();
        final String totalRefund = refunding;

        PayService.refund(isYdt ? ydtCoupon : "", traceNo, getReturnSale(), mSelectedPayMethod.payRemark(),
                mRefundOutTradeNo, mOldSalePayment.getTotalPaid(), refunding, GeneralSettings.INSTANCE.getTerminalId(),
                new Callback<PayResultData>() {
                    @Override
                    public void onSuccess(PayResultData result) {
                        handleResultRefund(result, isYdt);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        payRefundError(throwable, traceNo, mOldSalePayment.getTotalPaid(), totalRefund, isYdt);
                    }
                });
    }

    private void payRefundError(Throwable throwable, String traceNo, String total, String totalRefund, boolean isYdt) {
        hideLoadingDialog();
        Throwable cause = throwable.getCause();
        LoggerUtils.getLogger().debug("直连退款失败：" + cause);
        if (NetUtils.isNetException(cause)) {
            LoggerUtils.getLogger().debug("直连退款失败NetException");
            mMvpView.showProgressDialogNoHide("网络异常,正在重试...");
            // 无法连接服务时，自动重试。当前定义重试次数为100，间隔时间3秒，共等待5分钟。
            Service.online(100, 3000, new RetryCallback<OnlineResultData>() {
                @Override
                public void onSuccess(OnlineResultData service) {
                    mMvpView.hideProgressDialog();

                    LoggerUtils.getLogger().debug("直连退款准备查询");
                    payRefundQuery(traceNo, total, totalRefund, isYdt);
                }

                @Override
                public void onProgress(int i, final String s) {
                    // 报告服务重试状态。由于重试在新线程中运行，显示状态时需同步到UI线程。
                    mMvpView.showProgressDialogNoHide(s);
                    LoggerUtils.getLogger().debug(s);
                }

                @Override
                public void onError(Throwable throwable) {
                    // 达到最大重试次数后仍然无法连接服务。由于重试在新线程中运行，显示状态时需同步到UI线程。
                    hideLoadingDialog();
                    mMvpView.hideProgressDialog();
                    showHintDialog("无法查询退款状态,请联系管理员处理");
                    LoggerUtils.getLogger().debug("无法查询退款状态：" + throwable.getMessage());
                }
            });
        } else {
            int code = ((WebApiException) throwable).code();
            if (NetUtils.isNeedQuery(code)) {
                //订单进行中需要继续查询
                LoggerUtils.getLogger().debug("直连退款失败NeedQuery");
                payRefundQuery(traceNo, total, totalRefund, isYdt);
            } else {
                LoggerUtils.getLogger().debug("直连退款失败:" + throwable.getMessage());
                showHintDialog(throwable.getMessage() + getReturnSale().getSaleNo());
            }
        }
    }

    /**
     * 海信支付查询
     *
     * @param traceNo 原始交易号(一点通通过扫码获得,海信pay查询获得)
     */
    private void payRefundQuery(String traceNo, String total, String totalRefund, final boolean isYdt) {
        mMvpView.showProgressDialog("查询中...");
        PayService.refundQuery(traceNo, getReturnSale(), mSelectedPayMethod.remark(), mRefundOutTradeNo,
                GeneralSettings.INSTANCE.getTerminalId(), total, totalRefund, new Callback<PayResultData>() {
                    @Override
                    public void onSuccess(PayResultData result) {
                        mMvpView.hideProgressDialog();
                        LoggerUtils.getLogger().debug("直连退款查询成功:");
                        handleResultRefund(result, isYdt);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        mMvpView.hideProgressDialog();
                        LoggerUtils.getLogger().debug("直连退款查询失败:" + throwable.getMessage());
                        showHintDialog("查询退款结果失败:" + throwable.getMessage());
                    }
                });
    }

    /**
     * 处理海信pay退款结果
     *
     * @param result 返回数据
     */
    private void handleResultRefund(PayResultData result, boolean isYdt) {
        hideLoadingDialog();

        // 退款成功后重置为 null，避免多次提交不同的退款申请
        mRefundOutTradeNo = null;
        if (result.totalDscSelf == null || result.totalDscSelf.equals("")){
            result.totalDscSelf = "0.00";
        }
        if (result.totalDscChannel == null || result.totalDscChannel.equals("")){
            result.totalDscChannel = "0.00";
        }
        if (!BigDecimalUtils.isEquals0(result.totalDscSelf) || !BigDecimalUtils.isEquals0(result.totalDscChannel)) {
            LoggerUtils.getLogger().debug("payTotalDscMap-商户优惠金额：" + result.totalDscSelf + "，平台优惠金额:" + result.totalDscChannel);
        }

        LoggerUtils.getLogger().debug("直连退款成功:" + result.toString());
        //此类支付，支付号传入订单号outTradeNo
        String traceNo = result.tradeNo;
        SalePayModel salePayModel = new SalePayModel(mSelectedPayMethod, result.outTradeNo,
                result.totalRefund, traceNo, traceNo, "0");
        salePayModel.setTotalDscSelf(result.getTotalDscSelf());
        salePayModel.setTotalDscChannel(result.getTotalDscChannel());
        showLoadingDialog();
        //防止打印主控占用问题,必须在签购单打印完成之后才能做其他操作,
        printReceipt(salePayModel, isYdt, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                hideLoadingDialog();
                LoggerUtils.getLogger().debug("海信支付-退款-签购单");
                refreshAfterRefund(salePayModel);
            }

            @Override
            public void onError(String err) {
                hideLoadingDialog();
                LoggerUtils.getLogger().debug("海信支付-退款-签购单");
                refreshAfterRefund(salePayModel);
            }
        });
    }

    /**
     * 打印海信支付签购单
     *
     * @param payment  支付信息
     * @param callback 结果回调
     */
    private void printReceipt(final SalePayModel payment, boolean isYdt, final UniCallback callback) {
        if (isYdt || !GeneralSettings.INSTANCE.printHXPayReceipt() || payment == null || payment.getPayCode() == null) {
            LoggerUtils.getLogger().debug("海信支付原始数据为空或者选项关闭或者一点通,不打印签购单");
            hideAndCallback(callback);
            return;
        }

        try {
            LoggerUtils.getLogger().debug("海信支付打印签购单");
            new Thread(new Runnable() {
                @Override
                public void run() {
                    PrinterHelper.getInstance().printWithStringList(PayService.getPrintContent(getReturnSale(), payment, null), new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            hideAndCallback(callback);
                        }

                        @Override
                        public void onError(String err) {
                            hideAndCallback(callback);
                        }
                    });
                }
            }).start();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("海信支付打印签购单异常:" + e.toString());
            hideAndCallback(callback);
        }
    }

    /**
     * 隐藏loading框,实际只有onSuccess回调,所以统一处理
     *
     * @param callback 回调
     */
    private void hideAndCallback(final UniCallback callback) {
        mMvpView.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                hideLoadingDialog();
                callback.onSuccess("");
            }
        });
    }

    /**
     * 退款后刷新页面。
     *
     * @param refund 退款交易
     */
    public void refreshAfterRefund(SalePayModel refund) {
        refreshAfterRefund(refund, null);
    }

    /**
     * 初始化退货界面
     *
     * @param refund 支付信息
     * @param params 扩展参数
     */
    public void refreshAfterRefund(SalePayModel refund, Map<String, String> params) {
        hideLoadingDialog();

        if (refund == null) {
            LoggerUtils.getLogger().debug("refund == null 刷新");
            mMvpView.refreshTotal();
            return;
        }

        LoggerUtils.getLogger().debug(String.format(Locale.CHINESE, "应退:%s 本次退款:%s 已退:%s",
                getReturnSale().getTotal(), refund.getTotalPaid(), getReturnSale().getTotalPaid()));
        SalePayModel salePayment;
        if (PayMethod.isCard(refund.getRemark()) && GeneralSettings.INSTANCE.isReturnCard()) {
            try {
                oldSalePaymentList = getReturnSale().refundSuggest(mSelectedPayMethod, refund.getBuyerLogonId());
            } catch (Exception e) {
                LoggerUtils.getLogger().debug("获取原支付列表失败");
            }

            LoggerUtils.getLogger().debug("储值卡原卡退款刷新");
            salePayment = getReturnSale().refund(refund, oldSalePaymentList);
        } else {
            LoggerUtils.getLogger().debug("通用退款刷新");
            salePayment = getReturnSale().refund(refund, mOldSalePayment);
        }
        if (params != null) {
            salePayment.setUdp2(params.get("TERM_NO"));

        }
        List<SalePayModel> salePayments = getReturnSale().getSaleModel().getPays();
        if (BigDecimalUtils.isLess(String.valueOf(listPayments.size()), String.valueOf(salePayments.size()))) {
            // 原交易流水没有对应的支付方式，如：原交易使用现金支付，退货时选择退款到积分卡，此时，在支付列表中新增一个退款对应的支付方式

            for (int i = listPayments.size(); i < salePayments.size(); i++) {
                listPayments.add(new RefundViewModel(salePayments.get(i)));
            }
        }


        mMvpView.refreshAfterRefund();
    }

    /**
     * 上传退货流水。
     */
    public void uploadSale() {
        if (getReturnSale().isVip()) {
            returnVipPoints();
        } else if (getReturnSale().isReturnAll()) {
            returnCoupons();
        } else {
            readyUploading();
        }
    }

    /**
     * 积分原路退回。
     */
    private void returnVipPoints() {
        if (!GeneralSettings.INSTANCE.isPointsRegressEnabled() || !getReturnSale().isVip() || mIsVipPointsReturned) {
            returnCoupons();
            return;
        }

        showLoadingDialog();
        PointsService.regressPredict(getReturnSale().oldSaleNo(), getReturnSale(), new Callback<String>() {
            @Override
            public void onSuccess(String confirmId) {
                LoggerUtils.getLogger().debug("refund积分原路退回预判成功");
                PointsService.regressConfrim(confirmId, getReturnSale().getSaleNo(), getReturnSale().getVip().getCard().vipCardNo(), new Callback<RealTimePoint>() {
                    @Override
                    public void onSuccess(RealTimePoint point) {
                        mIsVipPointsReturned = true;
                        hideLoadingDialog();
                        LoggerUtils.getLogger().debug("onSuccess: 退货积分：" + point.expense() + "  停车积分：" + point.parking() + "  赠券积分：" + point.coupon());
                        getReturnSale().setExpensePoint(point.expense());
                        getReturnSale().setParkingPoint(point.parking());
                        getReturnSale().setCouponPoint(point.coupon());
                        returnCoupons();
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        hideLoadingDialog();
                        LoggerUtils.getLogger().error("refund积分原路退回确认失败：" + throwable.getMessage());
                        continueRefundAfterPoints("积分原路退回确认失败: " + throwable.getMessage());
                    }
                });
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().error("refund积分原路退回预判失败：" + throwable.getMessage());
                if (getReturnSale().hasPassPoints) {
                    returnCoupons();
                } else {
                    continueRefundAfterPoints("积分原路退回预判失败: " + throwable.getMessage());
                }
            }
        });
    }

    /**
     * 实时积分预判或者确认失败后,提示用户是否继续退货
     *
     * @param msg 提示信息
     */
    private void continueRefundAfterPoints(String msg) {

        showHintConfirm(msg + "，是否继续？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                returnCoupons();
            }

            @Override
            public void onError(String err) {

                hasUploaded = false;
            }
        });

    }

    /**
     * 原交易赠券原路退回。
     */
    private void returnCoupons() {
        if (!GeneralSettings.INSTANCE.isSaleCouponReturnEnabled() ||
                !getReturnSale().isReturnAll() || mIsCouponsReturned) {
            readyUploading();
            return;
        }

        showLoadingDialog();
        CouponService.couponReturnPredict(getReturnSale().oldSaleNo(), new Callback<CouponUsedData>() {
            @Override
            public void onSuccess(CouponUsedData couponUsedData) {
                LoggerUtils.getLogger().debug("refund赠券原路退回预判成功");
                CouponService.couponReturnConfirm(getReturnSale().oldSaleNo(), new Callback<String>() {
                    @Override
                    public void onSuccess(String s) {
                        mIsCouponsReturned = true;
                        hideLoadingDialog();
                        LoggerUtils.getLogger().debug("赠券原路退回确认成功");
                        readyUploading();
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        hideLoadingDialog();
                        LoggerUtils.getLogger().error("refund赠券原路退回确认失败：" + throwable.getMessage());
                        continueRefundAfterCoupon("赠券原路退回确认失败：" + throwable.getMessage());
                    }
                });
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().error("refund赠券原路退回预判失败：" + throwable.getMessage());
                if (getReturnSale().hasPassCoupons) {
                    readyUploading();
                } else {
                    continueRefundAfterCoupon("赠券原路退回预判失败：" + throwable.getMessage());
                }
            }
        });
    }

    /**
     * 赠券退回预判或者确认失败后,提示用户是否继续退货
     *
     * @param msg 提示信息
     */
    private void continueRefundAfterCoupon(String msg) {
        showHintConfirm(msg + "，是否继续？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                readyUploading();
            }

            @Override
            public void onError(String err) {
                hasUploaded = false;
            }
        });

    }

    /**
     * 准备上传流水。
     */
    private void readyUploading() {

        // 保存本地退货流水
        try {
            SaleNoService.save(App.getContext(), Config.org_code, Config.pos_no, getReturnSale().getSaleNo());
            mSaleRepository.save(getReturnSale());
        } catch (SQLException e) {
            e.printStackTrace();
            //应该有更好的处理方式,比如已经保存过本地不再执行这两行
            LoggerUtils.getLogger().debug("此处应只为已保存过本地流水,重新上传时主键冲突：" + e.getMessage());
        }

        // 打印退货小票
        PrinterHelper.getInstance().print(getReturnSale());
        mMvpView.showProgressDialog("数据上传,请稍后");


        // 检查服务是否在线。服务在线时计算赠券、上传流水并打印小票。
        // 无法连接服务时，自动重试。当前定义重试次数为100，间隔时间3秒，共等待5分钟。
        Service.online(100, 3000, new RetryCallback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData service) {
                returns(getReturnSale());
            }

            @Override
            public void onProgress(int i, final String s) {
                mMvpView.showProgressDialogNoHide(s);

                // 报告服务重试状态。由于重试在新线程中运行，显示状态时需同步到UI线程。
                LoggerUtils.getLogger().debug(s);
            }

            @Override
            public void onError(Throwable throwable) {
                hasUploaded = false;
                // 达到最大重试次数后仍然无法连接服务。由于重试在新线程中运行，显示状态时需同步到UI线程。

                mMvpView.hideProgressDialog();
                showHintDialog("无法上传流水。请联系管理员处理");

                LoggerUtils.getLogger().debug("无法上传流水：" + throwable.getMessage());
            }
        });
    }

    /**
     * 上传退货流水。
     *
     * @param returnSale 退货流水
     */
    private void returns(final ReturnSaleModel returnSale) {

        //原cmp后台，crm会员trantype只有1和9两个值，不区分是否整单退货
        //现scrm要求退货时1为非整单退，2为整单退,预销售退货时 5位非整单退，6为整单退

        if (returnSale.getTranType().isSale()) {
            if (getReturnSale().calcCount() != getReturnSale().getSaleModel().calcCount()) {
                returnSale.setTranType(TranType.Sale);
            } else {
                returnSale.setTranType(TranType.SCRMReturn);
            }
        } else {
            if (getReturnSale().calcCount() != getReturnSale().getSaleModel().calcCount()) {
                returnSale.setTranType(TranType.PreSale);
            } else {
                returnSale.setTranType(TranType.SCRMPreReturn);
            }
        }

        new SaleApplicationService(mMvpView.getContext()).upload(returnSale, new Callback<SaleSubmitResultData>() {
            @Override
            public void onSuccess(SaleSubmitResultData o) {
                hasUploaded = true;
                if (null != mMvpView) {
                    mMvpView.hideProgressDialog();
                }

                LoggerUtils.getLogger().debug("退货流水上传成功" + returnSale.getSaleNo());
                showHintDialog("退货流水上传成功：" + returnSale.getSaleNo(), (m) -> mMvpView.goToMainActivity());


            }

            @Override
            public void onError(Throwable throwable) {
                hasUploaded = false;
                if (null != mMvpView) {
                    mMvpView.hideProgressDialog();
                }
                showHintConfirm("退货流水上传失败：" + throwable.getMessage() + " 请确认流水是否已上传，若未上传请重试", new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        mMvpView.goToMainActivity();
                    }

                    @Override
                    public void onError(String err) {
                        returns(returnSale);
                    }
                });

                LoggerUtils.getLogger().debug("退货流水上传失败:" + throwable.getMessage());
            }
        });
    }

    /**
     * 返回指定支付方式的最大可退金额。
     *
     * @param payMethod 当前选择的支付方式
     * @return 指定支付方式的最大可退金额
     */
    public String getMaxRefund(PayMethod payMethod) {
        String maxRefund = "0";

        if (payMethod == null) {
            return maxRefund;
        }

        if (payMethod.isCardType()) {
            // 银行卡、储值卡、现金可退至储值卡账户
            maxRefund = getMaxRefundByRemark(PayMethod.CARD);
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.UNION_PAY));
            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.CASH));
            maxRefund = BigDecimalUtils.subtract(maxRefund, getOverRefundByRemark(PayMethod.VIP_POINTS));
        }
//        else if (payMethod.isPointsType()) {
//            // 银行卡、积分、现金可退至积分账户
//            maxRefund = getMaxRefundByRemark(PayMethod.VIP_POINTS);
//            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.UNION_PAY));
//            maxRefund = BigDecimalUtils.add(maxRefund, getMaxRefundByRemark(PayMethod.CASH));
//            maxRefund = BigDecimalUtils.subtract(maxRefund, getOverRefundByRemark(PayMethod.CARD));
//        }
        else if (payMethod.isCash()) {
            maxRefund = getMaxRefund(payMethod, PayMethod.UNION_PAY);
        } else if (payMethod.isUnionType() || payMethod.isUccbCard()) {
            maxRefund = getMaxRefund(payMethod, PayMethod.CASH);
        } else {
            maxRefund = getMaxRefundByRemark(payMethod.remark());
        }

        return maxRefund;
    }

    /**
     * 剩余可退金额。
     */
    private String getMaxRefundByRemark(String payMethod) {
        return BigDecimalUtils.subtract(getTotalReceivedByRemark(payMethod), getTotalRefundByRemark(payMethod));
    }

    /**
     * 支付金额
     */
    private String getTotalReceivedByRemark(String payMethod) {
        String total = "0";

        if (payMethod != null) {
            for (int i = 0; i < listPayments.size(); i++) {
                if (listPayments.get(i).payment().getRemark().equals(payMethod)) {
                    total = BigDecimalUtils.add(total, listPayments.get(i).totalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 已退款金额
     */
    private String getTotalRefundByRemark(String payMethod) {
        String total = "0";

        if (payMethod != null) {
            for (SalePayModel payment : getReturnSale().getPays()) {
                if (payMethod.equals(payment.getRemark())) {
                    total = BigDecimalUtils.add(total, payment.getTotalPaid());
                }
            }
        }

        return total;
    }

    /**
     * 返回指定支付方式(储值卡/积分)的溢退金额。例如：支付金额为150，退款200，则溢退金额为50。
     *
     * @param payMethod 支付方式的备注
     * @return 指定支付方式的溢退金额
     */
    private String getOverRefundByRemark(String payMethod) {
        // 根据 HIPOSAPP-509 制定的规则，只有储值卡/积分两种支付方式允许将消费时其它支付方式的支付金额退回
        // 到储值卡/积分账户。
        if (!PayMethod.isCardType(payMethod) && !PayMethod.isPointsType(payMethod)) {
            return "0";
        }

        String received = getTotalReceivedByRemark(payMethod);
        String refund = getTotalRefundByRemark(payMethod);
        return BigDecimalUtils.isGreater(refund, received) ? BigDecimalUtils.subtract(refund, received) : "0";
    }

    /**
     * 返回指定支付方式（现金/银行卡）的最大可退金额。
     *
     * @param payMethod          指定支付方式的备注
     * @param exclusivePayMethod 互斥支付方式的备注
     * @return 指定支付方式的最大可退金额
     */
    private String getMaxRefund(PayMethod payMethod, String exclusivePayMethod) {
        String maxRefund = getMaxRefundByRemark(payMethod.remark());

        if (!payMethod.isCashType() && !payMethod.isUnionType() && !payMethod.isUccbCard()) {
            return maxRefund;
        }

        // 计算合计溢退金额
        String totalOverRefund = BigDecimalUtils.add(getOverRefundByRemark(PayMethod.CARD), getOverRefundByRemark(PayMethod.VIP_POINTS));
        if (BigDecimalUtils.isGreater(totalOverRefund, "0")) {
            // 如果合计溢退金额大于零，表示已将部分或全部指定支付方式的付款退款到储值卡/积分账户，那么，从溢退金额中
            // 减去互斥支付方式的退款，剩余金额即为指定支付方式的已退金额。
            // 可退金额=支付金额-已退金额-（合计溢退金额-互斥支付方式的退款）
            String leftOverRefund = BigDecimalUtils.subtract(totalOverRefund, getMaxRefundByRemark(exclusivePayMethod));
            if (BigDecimalUtils.isGreaterAndEquals(maxRefund, leftOverRefund)) {
                maxRefund = BigDecimalUtils.subtract(maxRefund, leftOverRefund);
                // HIAPOSCMP-376 当可退款金额小于等于0时，可退款金额=银行卡+现金可退款金额-合计溢退金额
                if (BigDecimalUtils.isLessAndEquals0(maxRefund)) {
                    maxRefund = BigDecimalUtils.add(getMaxRefundByRemark(payMethod.remark()), getMaxRefundByRemark(exclusivePayMethod));
                    maxRefund = BigDecimalUtils.subtract(maxRefund, totalOverRefund);
                }
            } else {
                String totalPaid = BigDecimalUtils.add(getTotalReceivedByRemark(PayMethod.CASH), getTotalReceivedByRemark(PayMethod.UNION_PAY));
                if (BigDecimalUtils.isGreater(BigDecimalUtils.add(maxRefund, leftOverRefund), totalPaid)) {
                    maxRefund = "0";
                }
            }
        } else {
            // 如果合计溢退金额为零，表示未将指定支付方式的付款退款到储值卡/积分账户，那么，
            // 可退金额=原销售支付金额-已退金额
            // NO CODE HERE.
        }

        return maxRefund;
    }


}
