package com.hisense.hiaposcmp.presenter.refund;

import android.database.SQLException;
import android.os.Bundle;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.Cashier;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.webapi.points.PointsService;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.activity.ysy.refund.viewModel.ReturnItemViewModel;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.print.Invoice.EInvoice2;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.service.sale.SaleRepository;
import com.hisense.hiaposcmp.service.sale.SaleState;
import com.hisense.hiaposcmp.view.IView.refund.ISaleReturnView;
import com.hisense.hipos.invoice.models.EInvoiceConfig;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.ReturnSalePluModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.coupon.CouponService;
import com.hisense.hipos.webapi.coupon.CouponUsedData;
import com.hisense.hipos.webapi.invoice.InvoiceService;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.pos.scanner.ScannerCallback;
import com.hisense.webapi.Callback;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class SaleReturnPresenter extends BasePresenter<ISaleReturnView> {

    private List<ReturnItemViewModel> listSalesItemData;
    private ReturnSaleModel returnSaleModel = new ReturnSaleModel();

    public ReturnSaleModel getReturnSaleModel() {
        return returnSaleModel;
    }

    public void setReturnSaleModel(ReturnSaleModel returnSaleModel) {
        returnSaleModel = returnSaleModel;
    }

    public List<ReturnItemViewModel> saleItem() {
        if (listSalesItemData == null) {
            listSalesItemData = new ArrayList<>();
        }

        return listSalesItemData;
    }

    public List<ReturnItemViewModel> getListSalesItemData() {
        return listSalesItemData;
    }

    public void saveAuthNo() {
        Bundle bundle = mMvpView.getActivity().getIntent().getExtras();
        if (bundle != null) {
            returnSaleModel.setAuthNo(bundle.getString("authNo"));
        }

    }

    /**
     * 使用摄像头或激光扫描器扫描流水号。
     */
    public void scanForSaleNo() {
        mMvpView.iScanner().scan(new ScannerCallback() {
            @Override
            public void onSuccess(final String s) {
                mMvpView.updateSaleNo(s);
                querySale(s);
            }

            @Override
            public void onError(final Throwable t) {
                showHintDialog(t.getMessage());
            }
        });
    }

    /**
     * 查询流水。
     */
    public void querySale(String saleNo) {

        if (StringUtils.isEmpty(saleNo)) {
            return;
        }


        if (returnSaleModel != null && saleNo.equals(returnSaleModel.oldSaleNo())) {
            return;
        }
        getMainSaleModel().reset();

        showLoadingDialog();

        saveAuthNo();
        SaleRepository saleRepository = new SaleRepository(App.getContext());
        try {
            SaleState saleState = saleRepository.getState(saleNo);

            if (!saleState.equals(SaleState.RETURN)) {
                querySaleBySaleNo(saleNo);
            } else {
                showHintConfirm("此流水已进行退货交易，是否继续？", new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        querySaleBySaleNo(saleNo);
                    }

                    @Override
                    public void onError(String err) {
                        hideLoadingDialog();
                    }
                });

            }
        } catch (SQLException e) {
            LoggerUtils.getLogger().error("SQL异常：" + e.getMessage());
        }

    }


    /**
     * 使用指定的 {@code saleNo} 查询流水。
     *
     * @param saleNo 交易流水号
     */
    private void querySaleBySaleNo(String saleNo) {
        int days = GeneralSettings.INSTANCE.getReturnSaleDays();
        boolean openRpt = GeneralSettings.INSTANCE.isOpenRpt();
        SaleService.query(openRpt, saleNo, days, "*", new Callback<BaseSaleModel>() {
            @Override
            public void onSuccess(BaseSaleModel baseSaleModel) {

                if (baseSaleModel instanceof ReturnSaleModel) {
                    hideLoadingDialog();
                    showHintDialog("退货流水不允许退货");
                    return;
                }
                getMainSaleModel().saleModel((SaleModel) baseSaleModel);

                initNewSale();
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                showHintDialog(throwable.getMessage());
                LoggerUtils.getLogger().info("流水查询失败" + throwable.getMessage());
            }
        });

    }

    /**
     * 初始化退货流水。
     */
    private void initNewSale() {
        if (getSale().isVip()) {
            queryVip();
        } else {
            initNewSale(null);
        }
    }

    /**
     * @param vip 原流水会员
     */
    private void initNewSale(final Vip vip) {

        initNewSale(vip, new Callback<ReturnSaleModel>() {
            @Override
            public void onSuccess(ReturnSaleModel returnSale) {
                hideLoadingDialog();
                mMvpView.updateSaleNo(getSale().getSaleNo());
                listSalesItemData.clear();

                for (BaseSalePluModel baseSalePluModel : returnSale.getPlus()) {

                    ReturnItemViewModel salesItem = new ReturnItemViewModel((ReturnSalePluModel) baseSalePluModel);
                    listSalesItemData.add(salesItem);
                }
                mMvpView.adapter();
                mMvpView.updateTotal(returnSale.calcQuantity(), returnSale.calcQuantity(), getSale().getTotalRecv(), returnSale.getTotalRecv());
                mMvpView.updateVip(vip);

            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                showHintDialog("交易初始化失败");
            }
        });
    }

    /**
     * 初始化退货流水。
     *
     * @param vip      原流水会员
     * @param callback 退货流水初始化后的回调方法
     */
    private void initNewSale(final Vip vip, final Callback<ReturnSaleModel> callback) {

        /*
         * TODO：某些情况下，cashier对象仍会出现为null的情况。目前没有找到原因，此处强制检查cashier是否为null。
         * 如果为null则使用登录信息重新初始化cashier对象。
         */
        if (Context.cashier() == null) {
            Context.cashier(new User(Config.user_id, Config.user_code, Config.user_name));
            com.hisense.crm.webapi.Context.cashier(new Cashier(Config.user_code, Config.user_name));
        }
        if (Context.cashier() == null) {
            showHintDialog("交易初始化失败。请退出重新打开应用");
            if (callback != null) {
                callback.onError(null);
            }
            return;
        }
        //获取结账日期
        SaleService.getAccountingDate(new Callback<String>() {
            @Override
            public void onSuccess(final String accDate) {

                SaleNoService.next(App.getContext(), accDate, new Callback<String>() {
                    @Override
                    public void onSuccess(String saleNo) {
                        beginSale(vip, saleNo, accDate, callback);
                    }

                    @Override
                    public void onError(Throwable throwable) {
                        showHintDialog("从中间件获取新流水号失败：" + throwable.getMessage());
                        LoggerUtils.getLogger().error("从中间件获取新流水号失败：" + throwable.getMessage());

                    }
                });


            }

            @Override
            public void onError(Throwable t) {

                showHintDialog("结账日期获取失败：" + t.getMessage());
                LoggerUtils.getLogger().error("结账日期获取失败：" + t.getMessage());

                if (callback != null) {
                    callback.onError(t);
                }
            }
        });
    }

    /**
     * 开始新交易。
     *
     * @param vip      会员
     * @param saleNo   流水号
     * @param accDate  结账日期
     * @param callback 回调方法
     */
    private void beginSale(Vip vip, String saleNo, String accDate, final Callback<ReturnSaleModel> callback) {
        returnSaleModel.clearPlu();
        returnSaleModel.clearPluDsc();
        returnSaleModel.setSaleNo(saleNo);
        returnSaleModel.setPosNo(Config.pos_no);
        returnSaleModel.setCashier(Context.cashier());
        returnSaleModel.setAccDate(accDate);
        returnSaleModel.setVip(vip);
        returnSaleModel.setSaleTime(DateUtils.dateFormat());
        returnSaleModel.setTranType(getSale().getTranType());


        for (BaseSalePluModel pluModel : getSale().getPlus()) {
            ReturnSalePluModel returnSalePluModel = returnSaleModel.add(pluModel);
            returnSaleModel.addPlu(returnSalePluModel);
        }
        returnSaleModel.setUdp1(Context.cashier().getUserName());
        returnSaleModel.setUdp2(getSale().getSaleNo());
        returnSaleModel.setUdp4(getSale().getUdp4());
        returnSaleModel.setSaleModel(getSale());


        if (getSale().getTranType() == TranType.Sale) {
            LoggerUtils.getLogger().info(String.format(Locale.CHINESE, "退货[%s]。收银员：%s，结账日期：%s",
                    returnSaleModel.getSaleNo(), Context.cashier().code(), accDate));

        } else {
            LoggerUtils.getLogger().info(String.format(Locale.CHINESE, "预售退货[%s]。收银员：%s，结账日期：%s",
                    returnSaleModel.getSaleNo(), Context.cashier().code(), accDate));
        }

        checkEInvoice(callback);

    }

    /**
     * 查询原流水会员。
     */
    private void queryVip() {


        CardNo cardNo = CardNo.cardNo(getSale().getVip().getCard().cardFaceNo(), getSale().getVip().getCard().cardNoType());

        VipService.vipQuery(cardNo, null, getSale().getSaleNo(), new Callback<Vip>() {
            @Override
            public void onSuccess(Vip vip) {
                initNewSale(vip);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                //mLoading.dismiss();
                LoggerUtils.getLogger().error("会员查询失败：" + throwable.getMessage());
                showHintDialog("会员查询失败:" + throwable.getMessage());
            }
        });

    }


    /**
     * 开始退款。
     */
    public void refund() {
        //退货商品为0时校验
        if (returnSaleModel == null || StringUtils.isEmpty(returnSaleModel.getTotalRecv()) || BigDecimalUtils.isLessAndEquals(returnSaleModel.getTotalRecv(), "0")) {
            showHintDialog("请选择需要退货的商品");
            return;
        }

        if (returnSaleModel.isVip()) {
            predictVipPoints();
        } else if (returnSaleModel.isReturnAll()) {
            predictCoupons();
        } else {
            gotoRefund();
        }
    }

    /**
     * 积分原路退回预判。
     */
    private void predictVipPoints() {
        if (!GeneralSettings.INSTANCE.isPointsRegressEnabled() || !returnSaleModel.isVip()) {
            predictCoupons();
            return;
        }

        showLoadingDialog();
        PointsService.regressPredict(returnSaleModel.oldSaleNo(), returnSaleModel, new Callback<String>() {
            @Override
            public void onSuccess(String s) {
                hideLoadingDialog();
                predictCoupons();
                LoggerUtils.getLogger().debug("积分原路退回预判成功");
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().error("积分原路退回预判失败：" + throwable.getMessage());
                continueRefundAfterPoints("积分原路退回预判失败: " + throwable.getMessage());
            }
        });
    }

    /**
     * 原交易赠券原路退回预判。
     */
    private void predictCoupons() {
        if (!GeneralSettings.INSTANCE.isSaleCouponReturnEnabled() ||
                !returnSaleModel.isReturnAll()) {
            gotoRefund();
            return;
        }

        showLoadingDialog();
        CouponService.couponReturnPredict(returnSaleModel.oldSaleNo(), new Callback<CouponUsedData>() {
            @Override
            public void onSuccess(CouponUsedData couponUsedData) {
                hideLoadingDialog();
                LoggerUtils.getLogger().debug("赠券原路退回预判成功");
                if (!couponUsedData.rebate().usedTotal.equals("0") || couponUsedData.rebate().totalCount > 0) {

                    showHintConfirm("原交易共赠券" + couponUsedData.rebate().totalCount + "张，已使用" + couponUsedData.rebate().usedTotal + "元，是否继续退货？", new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            gotoRefund();
                        }

                        @Override
                        public void onError(String err) {

                        }
                    });

                } else {
                    gotoRefund();
                }
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                LoggerUtils.getLogger().error("赠券原路退回预判失败：" + throwable.getMessage());
                continueRefundAfterCoupon("赠券原路退回预判失败：" + throwable.getMessage());
            }
        });
    }

    /**
     * 赠券退回预判失败后,提示用户是否继续退货
     *
     * @param msg 提示信息
     */
    private void continueRefundAfterCoupon(String msg) {

        showHintConfirm(msg + "，是否继续退货？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                returnSaleModel.hasPassCoupons = true;
                gotoRefund();
            }

            @Override
            public void onError(String err) {
                returnSaleModel.hasPassCoupons = false;
            }
        });
    }

    /**
     * 实时积分预判失败后,提示用户是否继续退货
     *
     * @param msg 提示信息
     */
    private void continueRefundAfterPoints(String msg) {

        showHintConfirm(msg + "，是否继续退货？", new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                returnSaleModel.hasPassPoints = true;
                predictCoupons();
            }

            @Override
            public void onError(String err) {
                returnSaleModel.hasPassPoints = false;
            }
        });

    }

    /**
     * 检查电子发票配置是否初始化。如果未初始化，则从后台服务端获取电子发票配置。
     *
     * @param callback 检查后的回调方法
     */
    private void checkEInvoice(Callback<ReturnSaleModel> callback) {
        if (!StringUtils.isEmpty(EInvoice2.getBaseUrl())) {
            callback.onSuccess(returnSaleModel);

        } else {
            InvoiceService.getConfig(new Callback<EInvoiceConfig>() {
                @Override
                public void onSuccess(EInvoiceConfig eInvoiceConfig) {
                    EInvoice2.setBaseUrl(eInvoiceConfig.baseUrl());
                    GeneralSettings.INSTANCE.setInvoiceUrl(eInvoiceConfig.baseUrl());

                    callback.onSuccess(returnSaleModel);
                }

                @Override
                public void onError(Throwable throwable) {
                    LoggerUtils.getLogger().error("获取电子发票服务地址失败:" + throwable.toString());
                    callback.onError(throwable);
                }
            });
        }
    }


    /**
     * 转到退款界面。
     */
    private void gotoRefund() {
        if (GeneralSettings.INSTANCE.isPrintInvoice()) {
            if (!StringUtils.isEmpty(EInvoice2.getBaseUrl())) {
                returnSaleModel.setUdp4(EInvoice2.random());
            } else {
                showHintDialog("未获取电子发票配置");
                //电子发票为空
                LoggerUtils.getLogger().debug("销售页面-支付完成-保存流水-电子发票为空......");
                return;
            }
        } else {
            returnSaleModel.setUdp4("");
        }
        getMainSaleModel().returnSaleModel(returnSaleModel);
        mMvpView.gotoRefundActivity();

    }

    /**
     * 添加选择的退货商品。
     */
    public void addCheckedItems() {
        if (returnSaleModel == null || getListSalesItemData() == null) {
            return;
        }
        // 清除已添加的退货商品，将选中的商品添加到退货流水
        returnSaleModel.clearPlu();

        for (ReturnItemViewModel item : getListSalesItemData()) {
            if (item.isChecked()) {
                ReturnSalePluModel returnSalePluModel = item.returnSaleItem();
                if (BigDecimalUtils.isGreater0(item.count()) && returnSalePluModel != null) {
                    returnSaleModel.updatePlu(returnSalePluModel);
                    item.returnSaleItem(returnSalePluModel, returnSalePluModel.getQuantity());
                } else if (returnSalePluModel == null) {
                    returnSaleModel.addPlu(returnSaleModel.add(item.saleItem()));

                    item.returnSaleItem((ReturnSalePluModel) returnSaleModel.getPlu(returnSaleModel.queryIndex(item.saleItem())));
                }
            }
        }
    }
}

