package com.hisense.hiaposcmp.presenter.sale;

import android.util.Log;

import com.alibaba.fastjson.JSONObject;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.Cashier;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.RealTimePoint;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.vip.VipService;
import com.hisense.hiaposcmp.activity.ysy.sale.SaleActivity;
import com.hisense.hiaposcmp.auth.util.Md5Util;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.print.Invoice.EInvoice2;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.service.sale.SaleApplicationService;
import com.hisense.hiaposcmp.service.sale.SaleNoService;
import com.hisense.hiaposcmp.service.sale.SaleRepository;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.NetUtils;
import com.hisense.hiaposcmp.view.IView.sale.IGiftView;
import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.GiftCoupon;
import com.hisense.hipos.coupon.ICoupon;
import com.hisense.hipos.invoice.models.EInvoiceConfig;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SaleGiftModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.plu.PluType;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.invoice.InvoiceService;
import com.hisense.hipos.webapi.plu.PluQueryResultData;
import com.hisense.hipos.webapi.sale.CertiDtlQueryResultData;
import com.hisense.hipos.webapi.sale.CompletePluDscData;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.hipos.webapi.sale.SaleSubmitResultData;
import com.hisense.hipos.webapi.plu.PluQueryResultData;
import com.hisense.hipos.webapi.service.OnlineResultData;
import com.hisense.hipos.webapi.service.Service;
import com.hisense.webapi.Callback;
import com.hisense.webapi.RetryCallback;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.CountDownLatch;

/**
 * 礼品券兑换presenter
 *
 * @author yuminghao  on  2020/09/28.
 */
public class GiftPresenter extends BasePresenter<IGiftView> {

    public void createNewSale(Vip vip, TranType tranType, UniCallback callback) {
        //View是否绑定 如果没有绑定，就不执行网络请求
        if (!isViewAttached()) {
            CommonTools.showUiDialogNoView();
            return;
        }

        /*
         * TODO：某些情况下，cashier对象仍会出现为null的情况。目前没有找到原因，此处强制检查cashier是否为null。
         * 如果为null则使用登录信息重新初始化cashier对象。
         */
        if (com.hisense.hipos.webapi.Context.cashier() == null) {
            com.hisense.hipos.webapi.Context.cashier(new User(Config.user_id, Config.user_code, Config.user_name));
            com.hisense.crm.webapi.Context.cashier(new Cashier(Config.user_code, Config.user_name));
        }
        if (com.hisense.hipos.webapi.Context.cashier() == null) {
            hideLoadingDialog();
            callback.onError("交易初始化失败。请退出重新打开应用");
            return;
        }

        showLoadingDialog();
        SaleService.getAccountingDate(new Callback<String>() {
            @Override
            public void onSuccess(String accDate) {
                hideLoadingDialog();
                createNewSale(accDate, vip, tranType, callback);
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("获取结账日期异常:" + throwable.toString());
                callback.onError("获取结账日期异常:" + throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    private void createNewSale(String accDate, Vip vip, TranType tranType, UniCallback callback) {
        showLoadingDialog();
        SaleNoService.next(mMvpView.getContext(), accDate, new Callback<String>() {
            @Override
            public void onSuccess(String saleNo) {
                saleBegin(tranType, accDate, saleNo, vip, callback);
                hideLoadingDialog();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().debug("获取流水号失败：" + throwable.toString());
                callback.onError("获取流水号失败：" + throwable.getMessage());
                hideLoadingDialog();
            }
        });
    }

    private void saleBegin(TranType tranType, String accDate, String saleNo, Vip vip, UniCallback callback) {
        getMainSaleModel().reset();

        SaleModel saleModel = new SaleModel();
        User user = new User(Config.user_id, Config.user_code, Config.user_name);
        saleModel.setCashier(user);
        saleModel.setPosNo(Config.pos_no);
        saleModel.setAccDate(accDate);
        saleModel.setSaleNo(saleNo);
        saleModel.setTranType(tranType);
        saleModel.setSaleDate(WebApi.formatDate(new Date()));
        saleModel.setSaleTime(DateUtils.dateFormat());
        saleModel.setUdp1(user.getUserName());
        saleModel.setUdp4(EInvoice2.random());
        saleModel.setVip(vip);
        getMainSaleModel().saleModel(saleModel);

        if (tranType == TranType.Sale) {
            LoggerUtils.getLogger().info(String.format(Locale.CHINESE, "开始交易[%s]。收银员：%s，结账日期：%s",
                    saleModel.getSaleNo(), saleModel.getCashier().code(), accDate));
        } else {
            LoggerUtils.getLogger().info(String.format(Locale.CHINESE, "【预】开始交易[%s]。收银员：%s，结账日期：%s",
                    saleModel.getSaleNo(), saleModel.getCashier().code(), accDate));
        }

        checkEInvoice(callback);
    }

    /**
     * 检查电子发票配置是否初始化。如果未初始化，则从后台服务端获取电子发票配置。
     *
     * @param callback 检查后的回调方法
     */
    private void checkEInvoice(UniCallback callback) {
        if (!StringUtils.isEmpty(EInvoice2.getBaseUrl())) {
            callback.onSuccess("");
        } else {
            InvoiceService.getConfig(new Callback<EInvoiceConfig>() {
                @Override
                public void onSuccess(EInvoiceConfig eInvoiceConfig) {
                    EInvoice2.setBaseUrl(eInvoiceConfig.baseUrl());
                    GeneralSettings.INSTANCE.setInvoiceUrl(eInvoiceConfig.baseUrl());

                    callback.onSuccess("");
                }

                @Override
                public void onError(Throwable throwable) {
                    LoggerUtils.getLogger().error("获取电子发票服务地址失败:" + throwable.toString());
                    callback.onError("获取电子发票服务地址失败:" + throwable.getMessage());
                }
            });
        }
    }

    public void giftQueryWithCode(String certiCode) {
        String newCertiCode = "[\"" + certiCode + "\"]";
        String orgCode = Context.orgCode();
        String posNo = Context.posNo();
        String saleNo = getMainSaleModel().getSaleModel().getSaleNo();

        StringBuilder sb = new StringBuilder();
        sb.append("orgCode=");
        sb.append(orgCode);
        sb.append("&certiCode=");
        sb.append(newCertiCode);
        sb.append("&key=!!1890&MoydjX*%!YI090132!#23FWEjkuiyieyI3");
        Log.d("arvin debug", sb.toString());
        String sign = Md5Util.getMD5String(sb.toString());
        showLoadingDialog();
        SaleService.giftQuery(orgCode, newCertiCode, posNo, saleNo, sign, new Callback<CertiDtlQueryResultData>() {
            @Override
            public void onSuccess(CertiDtlQueryResultData certiDtlQueryResultData) {
                hideLoadingDialog();
                // 添加商品
                getSale().clearPlu();
                for (PluQueryResultData pluQueryResultData : certiDtlQueryResultData.productData) {
                    try {
                        BaseSalePluModel pluModel = PluQueryResultData.handleResult(pluQueryResultData);
                        // 券核销记录方式：0-记支付方式；1-按优惠分摊到商品明细
                        if (certiDtlQueryResultData.apportionType.equals("0")) {
                            pluModel.setTagPrice(pluModel.getPrice());
                            pluModel.setTotalDsc("0.00");
                            pluModel.setTotalRecv(pluModel.getTotal());
                        } else {
                            pluModel.setTagPrice(pluModel.getPrice());
                            pluModel.setPrice("0.00");
                            pluModel.setTotalDsc(pluModel.getTotal());
                            pluModel.setTotalRecv("0.00");
                            SalePluDscModel salePluDscModel = new SalePluDscModel();
                            salePluDscModel.setDscType("1");
                            salePluDscModel.setDscCode(certiDtlQueryResultData.lpDscCode);
                            salePluDscModel.setDscProType(certiDtlQueryResultData.lpDscCode);
                            salePluDscModel.setDscProName(certiDtlQueryResultData.lpDscName);
                            salePluDscModel.setDscNum(pluModel.getQuantity());
                            salePluDscModel.setDscPrice("0.00");
                            salePluDscModel.setDscTotal("0.00");
                            salePluDscModel.setPluId(pluModel.getPluId());
                            salePluDscModel.setPluCode(pluModel.getPluCode());
                            salePluDscModel.setPageNo(1);
                            salePluDscModel.setItemNo(1);
                            salePluDscModel.setSerialNo(1);
                            salePluDscModel.setTotalDsc(pluModel.getTotalDsc());
                            salePluDscModel.setGlobalType("2222");
                            List<SalePluDscModel> salePluDscModels = new ArrayList<>();
                            salePluDscModels.add(salePluDscModel);
                            pluModel.setPluDscs(salePluDscModels);
                        }
                        getSale().addPlu(pluModel);
                    } catch (Exception e) {
                        LoggerUtils.getLogger().error("礼品券商品转换失败：" + e.toString());
                    }
                }
                // 添加vip信息
                if (certiDtlQueryResultData.vipInfoId != null) {
                    VipCard card = new VipCard.Builder(certiDtlQueryResultData.vipCode, certiDtlQueryResultData.vipName)
                            .cardType("", "")
                            .expireDate("")
                            .status("")
                            .vipCardNo(certiDtlQueryResultData.vipInfoId)
                            .build();
                    getSale().addVip(card);
                }
                mMvpView.onQuerySuccess(certiDtlQueryResultData);
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                getSale().clearPlu();
                mMvpView.onQueryFail();
                Log.d("arvin debug", "onError: " + throwable.toString());
                showHintDialog(throwable.toString());
            }
        });
    }

    public void consumeCerti(String certiCode) {
        String newCertiCode = "[\"" + certiCode + "\"]";
        String orgCode = Context.orgCode();
        String saleNo = getMainSaleModel().getSaleModel().getSaleNo();
        String bizBillNo = "123456";
        String origOrderAmount = "0";
        String consumeAmount = "0";
        String vipInfoId = getMainSaleModel().getSaleModel().getVip().getCard().vipCardNo();

        StringBuilder sb = new StringBuilder();
        sb.append("orgCode=");
        sb.append(orgCode);
        sb.append("&certiCode=");
        sb.append(newCertiCode);
        sb.append("&saleNo=");
        sb.append(saleNo);

        sb.append("&key=!!1890&MoydjX*%!YI090132!#23FWEjkuiyieyI3");
        Log.d("arvin debug", sb.toString());
        String sign = Md5Util.getMD5String(sb.toString());
        showLoadingDialog();
        SaleService.consumeCerti(orgCode, newCertiCode, saleNo, bizBillNo, origOrderAmount, consumeAmount, vipInfoId, sign, new Callback<Object>() {
            @Override
            public void onSuccess(Object object) {
                hideLoadingDialog();
                mMvpView.consumeSuccess();
            }

            @Override
            public void onError(Throwable throwable) {
                hideLoadingDialog();
                showHintDialog(throwable.toString());
            }
        });
    }

    private SaleApplicationService saleApplicationService;
    private CountDownLatch printerLatch;                    //同步多个任务，都执行完之后再进行其他任务
    private int hasCoupons;                                 //券是否操作完成，默认-1；成功1；失败0
    private int hasPoints;                                  //积分是否操作完成，默认-1；成功1；失败0
    private int hasUploaded;                                //流水是否上传，默认-1；成功1；失败0

    public void saleSubmit() {
        showLoadingDialog();
        //保存本地流水
        uSaveData();
        Service.online(100, 3000, new RetryCallback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData service) {
                printerLatch = new CountDownLatch(3);
                mMvpView.hideProgressDialog();
                hasCoupons = -1;
                hasPoints = -1;
                hasUploaded = -1;
                print();
                saleComplete();
                calcCoupons();
                realTimePoint();
                submit();
            }

            @Override
            public void onProgress(int i, final String s) {
                // 报告服务重试状态。由于重试在新线程中运行，显示状态时需同步到UI线程。
                mMvpView.showProgressDialogNoHide(s);
                LoggerUtils.getLogger().info(s);
            }

            @Override
            public void onError(final Throwable throwable) {
                mMvpView.onlineFail(throwable.getMessage());
                // 达到最大重试次数后仍然无法连接服务。由于重试在新线程中运行，显示状态时需同步到UI线程。
                mMvpView.hideProgressDialog();
                showHintDialog("无法上传流水：" + throwable.getMessage());
                LoggerUtils.getLogger().error("无法上传流水：" + throwable.toString());

                hasCoupons = 0;
                hasPoints = 0;
                hasUploaded = 0;
                resetSaleModelForYsyUpSale();
            }
        });
    }

    /**
     * 交易完成后保存交易数据
     */
    private void uSaveData() {
        hideLoadingDialog();
        LoggerUtils.getLogger().error("交易完成后开始保存流水号");
        SaleNoService.save(mMvpView.getContext(), Config.org_code, Config.pos_no, getSale().getSaleNo());
        LoggerUtils.getLogger().error("交易完成后保存流水号成功");
        try {
            new SaleRepository(mMvpView.getContext()).save(getMainSaleModel().getSaleModel());
            LoggerUtils.getLogger().error("保存数据成功");
        } catch (Exception e) {
            LoggerUtils.getLogger().error("保存数据出错:" + e.toString());
            LoggerUtils.getLogger().error("出错流水数据:" + JSONObject.toJSONString(getSale()));
        }
    }

    /**
     * 交易完成，用于更新限量次数
     */
    private void saleComplete() {
        try {
            LoggerUtils.getLogger().debug("进入完成接口逻辑");
            List<CompletePluDscData> pluDsc = new ArrayList<>();
            List<BaseSalePluModel> baseSalePluModels = getMainSaleModel().getSaleModel().getPlus();
            for (BaseSalePluModel pluModel : baseSalePluModels) {
                for (int i = 0; i < pluModel.pluDscSize(); i++) {
                    SalePluDscModel pluDscModel = pluModel.getPluDsc(i);

                    CompletePluDscData pluDscData = new CompletePluDscData();
                    pluDscData.setDscCode(pluDscModel.getDscCode());
                    pluDscData.setDscProType(pluDscModel.getDscProType());
                    pluDscData.setDscNum(pluDscModel.getDscNum());
                    pluDscData.setPluCode(pluModel.getPluCode());
                    pluDscData.setGlobalType(pluDscModel.getGlobalType());
                    pluDsc.add(pluDscData);
                }
            }
            if (!ListUtils.isEmpty(pluDsc)) {
                LoggerUtils.getLogger().debug("有优惠，调用完成接口");
                String vipCardNo = getMainSaleModel().getSaleModel().getVip() == null
                        ? null
                        : getMainSaleModel().getSaleModel().getVip().card().vipCardNo();

                SaleService.saleComplete(getMainSaleModel().getSaleModel().getSaleNo(),
                        vipCardNo,
                        getMainSaleModel().getSaleModel().getSaleDate(),
                        pluDsc,
                        new Callback<Object>() {
                            @Override
                            public void onSuccess(Object o) {
                                LoggerUtils.getLogger().debug("完成接口成功");
                            }

                            @Override
                            public void onError(Throwable throwable) {
                                LoggerUtils.getLogger().debug("完成接口失败：" + throwable.getMessage());
                            }
                        });
            } else {
                LoggerUtils.getLogger().debug("无优惠，不调用完成接口");
            }
        } catch (Exception e) {
            NetUtils.printDetail("交易完成逻辑:", e);
        }
    }

    public void print() {
        try {
            PrinterHelper.getInstance().print(getSale(), printerLatch);

            LoggerUtils.getLogger().debug("##########打印锁初始化############");
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印出现异常" + e.toString());
        }
    }

    private void calcCoupons() {
        if (!GeneralSettings.INSTANCE.isCalcCoupon()) {
            hasCoupons = 1;
            printerLatch.countDown();
            resetSaleModelForYsyUpSale();
            return;
        }

        LoggerUtils.getLogger().debug("启用赠券");
        SaleService.rebate(new Callback<Collection<ICoupon>>() {
            @Override
            public void onSuccess(Collection<ICoupon> coupons) {
                LoggerUtils.getLogger().debug("赠券接口调用成功");
                Collection<Coupon> couponsTmp = new ArrayList<>();
                Collection<GiftCoupon> giftCouponsTmp = new ArrayList<>();
                for (ICoupon coupon : coupons) {
                    if (coupon instanceof Coupon) {
                        couponsTmp.add((Coupon) coupon);
                    } else if (coupon instanceof GiftCoupon) {
                        giftCouponsTmp.add((GiftCoupon) coupon);
                    }
                }

                SaleActivity.couponMap.put(getSale().getSaleNo(), couponsTmp);
                SaleActivity.giftCouponMap.put(getSale().getSaleNo(), giftCouponsTmp);

                hasCoupons = 1;
                printerLatch.countDown();
                resetSaleModelForYsyUpSale();
            }

            @Override
            public void onError(Throwable t) {
                LoggerUtils.getLogger().error("赠券计算失败：" + t.toString());
                showHintDialog("赠券计算失败：" + t.getMessage());
                hasCoupons = 0;
                printerLatch.countDown();
                resetSaleModelForYsyUpSale();
            }
        });
    }

    public void resetSaleModelForYsyUpSale() {
        if (hasCoupons != -1 && hasPoints != -1 && hasUploaded != -1) {
            LoggerUtils.getLogger().debug("券/积分/流水接口全部完成(可能失败或成功),重置:" + hasCoupons + ";" + hasPoints + ";" + hasUploaded);

            resetSaleModel();

            if (hasUploaded == 1) {
                mMvpView.submitSuccess();
            }
            if (hasUploaded == 0) {
                //获取新的流水号
                SaleNoService.abandon();
                mMvpView.submitFail("流水上传失败，请点击右上角上传流水重试");
            }

            hasCoupons = -1;
            hasPoints = -1;
            hasUploaded = -1;
        } else {
            LoggerUtils.getLogger().debug("券/积分/流水接口未全部完成,暂不重置:" + hasCoupons + ";" + hasPoints + ";" + hasUploaded);
        }
    }

    public void resetSaleModel() {
        getMainSaleModel().reset();
    }

    private void realTimePoint() {
        if (!(getSale().isVip() && GeneralSettings.INSTANCE.isRealTimePointEnabled())) {
            getSale().setExpensePoint("0");
            getSale().setParkingPoint("0");
            getSale().setCouponPoint("0");
            hasPoints = 1;
            printerLatch.countDown();
            resetSaleModelForYsyUpSale();
            return;
        }

        VipService.realTimePoint(new Callback<RealTimePoint>() {
            @Override
            public void onSuccess(RealTimePoint point) {
                LoggerUtils.getLogger().debug("消费积分：" + point.expense() + "  停车积分：" + point.parking() + "  赠券积分：" + point.coupon());
                getSale().setExpensePoint(point.expense());
                getSale().setParkingPoint(point.parking());
                getSale().setCouponPoint(point.coupon());

                PrinterHelper.getInstance().refreshSaleModel(getSale());
                hasPoints = 1;
                printerLatch.countDown();
                resetSaleModelForYsyUpSale();
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("积分实时积分计算失败:" + throwable.toString());
                showHintDialog("积分实时积分计算失败:" + throwable.getMessage());
                hasPoints = 0;
                printerLatch.countDown();
                resetSaleModelForYsyUpSale();
            }
        });
    }

    private void submit() {
        if (saleApplicationService == null) {
            saleApplicationService = new SaleApplicationService(mMvpView.getContext());
        }

        saleApplicationService.upload(getMainSaleModel().getSaleModel(), new Callback<SaleSubmitResultData>() {
            @Override
            public void onSuccess(SaleSubmitResultData submitResultData) {
                if (submitResultData != null) {
                    Collection<Coupon> couponsTmp = new ArrayList<>();
                    Collection<GiftCoupon> giftCouponsTmp = new ArrayList<>();
                    if (submitResultData.iCoupons != null) {
                        for (ICoupon coupon : submitResultData.iCoupons) {
                            if (coupon instanceof Coupon) {
                                couponsTmp.add((Coupon) coupon);
                            } else if (coupon instanceof GiftCoupon) {
                                giftCouponsTmp.add((GiftCoupon) coupon);
                            }

                        }
                    }
                    SaleActivity.couponMap.put(getSale().getSaleNo(), couponsTmp);
                    SaleActivity.giftCouponMap.put(getSale().getSaleNo(), giftCouponsTmp);

                    getSale().setExpensePoint(submitResultData.tradePoints);
                    getSale().setParkingPoint("0");
                    getSale().setCouponPoint("0");
                    getSale().setVipPoints(submitResultData.totalPoints);
                }

                PrinterHelper.getInstance().refreshSaleModel(getSale());
                showHintDialog("上传成功，流水：" + getSale().getSaleNo());
                LoggerUtils.getLogger().debug("流水上传成功:" + getSale().getSaleNo());

                hasUploaded = 1;
                printerLatch.countDown();
                resetSaleModelForYsyUpSale();
            }

            @Override
            public void onError(Throwable throwable) {
                showHintConfirm("流水上传", "流水上传失败:" + throwable.getMessage() + ",请确认流水是否已上传，若未上传请重试", new UniCallback() {
                    @Override
                    public void onSuccess(String msg) {
                        showLoadingDialog();
                        submit();
                    }

                    @Override
                    public void onError(String err) {
                        LoggerUtils.getLogger().error(String.format(Locale.CHINESE, "流水[%s]上传失败: %s",
                                getSale().getSaleNo(), throwable.getMessage()));

                        hasUploaded = 0;
                        printerLatch.countDown();
                        resetSaleModelForYsyUpSale();
                    }
                });
            }
        });
    }

}
