package com.hisense.hiaposcmp.presenter.sale;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.adapter.GiftItemAdapter;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.view.IView.sale.IProGiftView;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.plu.PluGiftAction;
import com.hisense.hipos.webapi.plu.PluService;
import com.hisense.webapi.Callback;

/**
 * 加价换购实现
 *
 * @author xieliyan         2018-11-24
 * @author dukunbiao(null)  2019-07-24  重构
 */
public class ProGiftPresenter extends BasePresenter<IProGiftView> {

    private GiftItemAdapter mAdapter;

    public void assignAdapter(GiftItemAdapter giftItemWxAdapter) {
        this.mAdapter = giftItemWxAdapter;
    }

    public void clearAdapter() {
        mAdapter.clear();
    }

    public void setAdapter() {
        mAdapter.setData(getSale().getPluGiftActions());
    }

    public void updateAllTotal() {
        mMvpView.updateAllTotal(getSale().calcCount(), getSale().getTotalDsc(), getSale().getTotalRecv());
    }

    public void increase(int position) {
        queryPluGift(position);
    }

    public void decrease(int position) {
        String pluCode = getSale().getPluGiftActions().get(position).pluCode();
        int pluItemNo = getSale().queryPluItemNo(pluCode);
        getSale().decreasePlu(pluItemNo - 1);
        int count = getSale().getPlu(pluItemNo - 1).count();
        mAdapter.changeQuantity(position, count);
        mAdapter.removeSelected(position);
        updateAllTotal();
    }

    public void delete(int countSale) {
        for (int i = getSale().getPlus().size() - 1; i >= countSale; i--) {
            getSale().removePlu(i);
        }
        getSale().clearPluGift();
    }

    public int countSale() {
        return getSale().getPlus().size();
    }

    /**
     * 查询换购商品。
     *
     * @param position 换购商品的位置（从0开始）
     */
    private void queryPluGift(int position) {
        PluGiftAction pluGiftAction = getSale().getPluGiftActions().get(position);
        if (BigDecimalUtils.isGreater(pluGiftAction.quantity(), "0")) {
            addSaleItem(null, position);
            return;
        }

        String pluCode = getSale().getPluGiftActions().get(position).pluCode();
        PluService.pluQuery(pluCode, "", new Callback<BaseSalePluModel>() {
            @Override
            public void onSuccess(BaseSalePluModel pluModel) {
                for (PluGiftAction pluGiftAction : getSale().getPluGiftActions()) {
                    if (pluGiftAction.pluCode().equals(pluCode)) {
                        pluModel.clearPluDsc();
                        SalePluDscModel pluDscModel = new SalePluDscModel();
                        pluDscModel.setDscCode(pluGiftAction.getDscCode());
                        pluDscModel.setDscType(pluGiftAction.getDscType());
                        pluDscModel.setDscProType(pluGiftAction.getDscProType());
                        pluDscModel.setDscProName(pluGiftAction.getDscProName());
                        pluDscModel.setDscPrice(pluGiftAction.dscPrice());
                        pluDscModel.setPageNo(pluModel.getPageNo());
                        pluDscModel.setItemNo(pluModel.getItemNo());
                        pluDscModel.setTotalDsc(BigDecimalUtils.subtract(pluModel.getTagPrice(), pluGiftAction.dscPrice()));

                        pluModel.addPluDsc(pluDscModel);

                        break;
                    }
                }
                addSaleItem(pluModel, position);
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("加价换购商品查询失败:" + throwable.toString());
                showHintDialog("没找到该商品", "请确认条码扫描正确");
            }
        });
    }

    /**
     * 添加销售商品。
     *
     * @param position 换购商品的位置（从0开始）
     */
    private void addSaleItem(BaseSalePluModel pluModel, int position) {
        int count;
        if (pluModel == null) {
            String pluCode = getSale().getPluGiftActions().get(position).pluCode();
            int pluItemNo = getSale().queryPluItemNo(pluCode);
            getSale().increasePlu(pluItemNo - 1);
            count = getSale().getPlu(pluItemNo - 1).count();
        } else {
            getSale().addPlu(pluModel);
            count = 1;
        }

        mAdapter.changeQuantity(position, count);
        mAdapter.setDataListSelected(position, getSale().getPluGiftActions().get(position));
        updateAllTotal();
    }
}
