package com.hisense.hiaposcmp.presenter.sale;

import android.widget.TextView;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.presenter.BasePresenter;
import com.hisense.hiaposcmp.view.IView.sale.ISaleItemView;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.model.SalePluModel;
import com.hisense.hipos.sale.CalcType;
import com.hisense.hipos.webapi.sale.PromotionCalcModel;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.webapi.Callback;

import java.util.Arrays;
import java.util.Locale;

/**
 * @author dukunbiao(null)  2019-07-23  重构
 * @author guolina          2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 */
public class SaleItemPresenter extends BasePresenter<ISaleItemView> {

    public RangeValidator tagPriceValidator;   // 单价验证器
    public RangeValidator priceValidator;      // 售价验证器
    public RangeValidator discountValidator;   // 折扣验证器
    public RangeValidator quantityValidator;   // 购买数量验证器
    public RangeValidator dscTotalValidator;   // 单项优惠验证器
    public RangeValidator totalValidator;      // 小计金额验证器

    public void initRangeValidator(BaseSalePluModel pluModel) {
        tagPriceValidator = new RangeValidator("单价", pluModel.getMinPrice(), pluModel.getMaxPrice());
        priceValidator = new RangeValidator("售价", pluModel.getMinPrice(), pluModel.getMaxPrice());
        discountValidator = new RangeValidator("折扣", "0", "100");
        quantityValidator = new RangeValidator("购买数量", "0");
        dscTotalValidator = new RangeValidator("单项优惠", "0", BigDecimalUtils.multiply(pluModel.getTagPrice(), pluModel.getQuantity()));
        totalValidator = new RangeValidator("小计金额", "0");
    }

    public void checkNoCodePlu(BaseSalePluModel pluModel, TextView edtTagPrice, TextView edtPrice,
                               TextView edtDiscount, TextView edtQuantity, TextView txtTotal, TextView edtSpec) {
        checkNoCodePlu(pluModel, edtTagPrice, edtPrice,
                edtDiscount, edtQuantity, txtTotal, edtSpec, null);
    }

    public void checkNoCodePlu(BaseSalePluModel pluModel, TextView edtTagPrice, TextView edtPrice,
                               TextView edtDiscount, TextView edtQuantity, TextView txtTotal, TextView edtSpec, TextView edtDscTotal) {
        String err = tagPriceValidator.validate(edtTagPrice, true);
        if (err != null) {
            showHintDialog(err);
            return;
        }

        err = priceValidator.validate(edtPrice, true);
        if (err != null) {
            showHintDialog(err);
            return;
        }

        err = discountValidator.validate(edtDiscount, true);
        if (err != null) {
            showHintDialog(err);
            return;
        }

        err = quantityValidator.validate(edtQuantity, true);
        if (err != null) {
            showHintDialog(err);
            return;
        }

        if (null != edtDscTotal) {
            if (StringUtils.isEmpty(edtDscTotal.getText().toString())) {
                showHintDialog("单项优惠不能为空");
                return;
            }
        }


        if (BigDecimalUtils.isEquals(txtTotal.getText().toString(), "0") && !GeneralSettings.INSTANCE.isZeroPriceSale()) {
            showHintDialog("商品金额不能为 0");
            return;
        }

        if (!StringUtils.isEmpty(edtSpec.getText().toString())) {
            pluModel.setSpec(edtSpec.getText().toString().trim());
        }

        updatePluInfo(pluModel, edtTagPrice.getText().toString(), edtPrice.getText().toString(),
                edtQuantity.getText().toString(), txtTotal.getText().toString(), edtDiscount.getText().toString(), null == edtDscTotal ? "0.00" : edtDscTotal.getText().toString());
    }

    private void updatePluInfo(BaseSalePluModel pluModel, String tagPrice, String price, String quantity, String totalRecv, String discount, String dscTotal) {
        pluModel.setTagPrice(tagPrice);
        pluModel.setPrice(price);
        if (GeneralSettings.INSTANCE.isChangePrice()) {
            pluModel.setTotalRecv(totalRecv);
        }
        pluModel.setTotal(BigDecimalUtils.multiply(tagPrice, quantity));

        pluModel.setQuantity(BigDecimalUtils.decimalFormat(quantity, "0.000"));

        // 设置优惠（只在存在折扣时，即折扣不为100时，设置优惠）
        if (BigDecimalUtils.isNotEquals(discount, "100")) {
            SalePluDscModel pluDscModel = new SalePluDscModel();
            pluDscModel.setDscType("1");
            pluDscModel.setDscCode("SGYH000000");
            pluDscModel.setDscProType("18");
            pluDscModel.setDscProName("手工优惠");
            pluDscModel.setDscNum(quantity);
            pluDscModel.setDscPrice(price);
            pluDscModel.setPluId(pluModel.getPluId());

            pluModel.clearPluDsc();
            pluModel.addPluDsc(pluDscModel);
        } else if (GeneralSettings.INSTANCE.isChangePrice()) {
            SalePluDscModel pluDscModel = new SalePluDscModel();
            pluDscModel.setDscType("1");
            pluDscModel.setDscCode("SGYH000000");
            pluDscModel.setDscProType("18");
            pluDscModel.setDscProName("手工优惠");
            pluDscModel.setDscNum(quantity);
            pluDscModel.setDscTotal(totalRecv);
            pluDscModel.setPluId(pluModel.getPluId());
            pluDscModel.setTotalDsc(dscTotal);

            pluModel.clearPluDsc();
            pluModel.addPluDsc(pluDscModel);
        }

        if (pluModel.isWeight()) {
            pluModel.updateCalcPluDscByNum();
        }

        if (GeneralSettings.INSTANCE.isChangePrice()) {
            pluModel.calcTotalDscbyTotal(totalRecv);

            showLoadingDialog();
            SaleService.promotionCalc(CalcType.Enter, Arrays.asList(pluModel), GeneralSettings.INSTANCE.getTotalDiscountWay(), new Callback<PromotionCalcModel>() {
                @Override
                public void onSuccess(PromotionCalcModel promotionCalcModel) {
                    hideLoadingDialog();
                    pluModel.clearPluDsc();
                    if (null == promotionCalcModel) {
                        showHintDialog("促销返回信息为空");
                        return;
                    }
                    for (SalePluModel salePluModel : promotionCalcModel.getPlu()) {
                        if (!ListUtils.isEmpty(salePluModel.getPluDscs())) {
                            for (SalePluDscModel salePluDscModel : salePluModel.getPluDscs()) {
                                pluModel.addPluDsc(salePluDscModel);
                            }

                        }
                    }

                    mMvpView.checkNoCodePluSuccess(pluModel);
                }

                @Override
                public void onError(final Throwable throwable) {
                    if (mMvpView != null) {
                        hideLoadingDialog();
                    }
                    hideLoadingDialog();
                    //FIXME null 改成中间件返回成功，优惠列表为空
                    if (!StringUtils.isEmpty(throwable.getMessage()) && throwable.getMessage().contains("无促销")) {
                        mMvpView.checkNoCodePluSuccess(pluModel);
                        return;
                    }

                    showHintDialog(throwable.getMessage());
                }
            });
        } else {
            pluModel.calcTotalDsc();
            mMvpView.checkNoCodePluSuccess(pluModel);
        }


    }

    /**
     * 格式化价格。保留一位小数。
     *
     * @param value 要格式化的价格
     * @return 最多保留一位小数的价格字符串
     */
    private String formatPrice(String value) {
        return BigDecimalUtils.decimalFormat(BigDecimalUtils.formatScale(value, 1), 2);
    }

    public class RangeValidator {
        private String name;         // 验证项的名称
        private String minValue;     // 区间最小值
        private String maxValue;     // 区间最大值
        private String minValueText; // 区间最小值的名称
        private String maxValueText; // 区间最大值的名称

        /**
         * 初始化一个 {@code RangeValidator} 的实例。
         *
         * @param name     验证项的名称
         * @param minValue 区间最小值
         */
        public RangeValidator(String name, String minValue) {
            this(name, minValue, "0");
        }

        /**
         * 初始化一个 {@code RangeValidator} 的实例。
         *
         * @param name     验证项的名称
         * @param minValue 区间最小值
         * @param maxValue 区间最大值
         */
        public RangeValidator(String name, String minValue, String maxValue) {
            this.name = name;

            if (BigDecimalUtils.isGreater(maxValue, "0")) {
                this.maxValue = maxValue;
            } else {
                this.maxValue = String.valueOf(Double.MAX_VALUE);
            }

            if (BigDecimalUtils.isGreaterAndEquals(minValue, "0")) {
                this.minValue = minValue;
            } else {
                this.minValue = "0";
            }

            this.maxValueText = BigDecimalUtils.decimalFormat(this.maxValue, 2);
            this.minValueText = BigDecimalUtils.decimalFormat(this.minValue, 2);
        }

        /**
         * 验证指定的 {@code TextView} 文本。
         *
         * @param textView        待验证的 {@code TextView}
         * @param checkWhitespace 是否验证空字符串
         * @return 如果验证通过则返回 null。否则，返回验证失败的原因。
         */
        public String validate(TextView textView, boolean checkWhitespace) {
            return validate(textView.getText(), checkWhitespace);
        }

        /**
         * 验证指定的 {@code CharSequence} 字符串序列。
         *
         * @param s 待验证的 {@code CharSequence} 字符串序列
         * @return 如果验证通过则返回 null。否则，返回验证失败的原因。
         */
        public String validate(CharSequence s) {
            return validate(s, false);
        }

        /**
         * 验证指定的 {@code CharSequence} 字符串序列。
         *
         * @param s               待验证的 {@code CharSequence} 字符串序列
         * @param checkWhitespace 是否验证空字符串
         * @return 如果验证通过则返回 null。否则，返回验证失败的原因。
         */
        public String validate(CharSequence s, boolean checkWhitespace) {
            if (checkWhitespace && s.length() == 0) {
                return String.format(Locale.CHINESE, "未输入%s", this.name);
            }

            try {
                if (BigDecimalUtils.isGreater(s.toString(), this.maxValue)) {
                    return String.format(Locale.CHINESE, "%s不能大于 %s", this.name, this.maxValueText);
                } else if (BigDecimalUtils.isLess(s.toString(), this.minValue)) {
                    return String.format(Locale.CHINESE, "%s不能小于 %s", this.name, this.minValueText);
                } else if (BigDecimalUtils.isEquals(s.toString(), "0") && !GeneralSettings.INSTANCE.isZeroPriceSale()) {
                    return String.format(Locale.CHINESE, "%s不能为 0", this.name);
                }
            } catch (Exception e) {
                return String.format(Locale.CHINESE, "%s不是有效的数字", this.name);
            }

            return null;
        }
    }
}
