/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print;

import android.app.Activity;

/*******************************************************************************
 * 打印服务接口。
 *
 * @author guanguangjin 2017-02-04
 * @author dengyunpeng  2017-07-17 HIPOSAPP-118 将打印模板保存在本地SQLite数据库
 * @author wangyaogang  2019-06-24 HIPOSAPP-1090 增加礼品券类型
 ******************************************************************************/

public interface IPrintService {

    /**
     * 打印类型枚举
     */
    enum PrtPatternType {
        //   INVOICE("invoice_prn_pattern.txt"),
        VOUCHER("voucher_prn_pattern.txt"),
        GIFTCOUPON("giftcoupon_prn_pattern.txt"),
        COUPON("coupon_prn_pattern.txt");

        private String filePath = null;

        PrtPatternType(String filePath) {
            this.filePath = filePath;
        }

        public String getFilePath() {
            return this.filePath;
        }
    }

    /**
     * 将本地的打印模板文件读入到内存中，然后缓存起来
     *
     * @param type 模板类型
     */
    void loadingPrtPatternInMem(Activity context, PrtPatternType type);

    /**
     * 更新缓存中的打印模板信息，并将其保存在本地文件中（SDCard）
     * 其中，有re-try机制，如果保存打印模板失败的话，会重试3次，三次都失败的话会再日志中记录
     *
     * @param content 模板内容
     * @param type    模板类型
     */
    void refreshMemForPattern(Activity context, String content, PrtPatternType type);

    /**
     * 保存打印模板信息到SQLite
     *
     * @param content 模板内容
     * @param type    模板类型
     * @return true 成功，false 失败
     * 修改: dengyunpeng 2017-7-17 HIPOSAPP-118 将打印模板保存在本地SQLite数据库
     */
    boolean savePatternInSQLite(Activity context, String content, PrtPatternType type);

    /**
     * 查询缓存中对应的打印模板
     *
     * @param type 模板类型
     * @return 模板内容
     */
    String getTemplateFromCached(PrtPatternType type);

    /**
     * 设置缓存中的打印模板
     *
     * @param type    模板类型
     * @param content 模板内容
     */
    void setTemplateInCached(PrtPatternType type, String content);
}
