/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.Invoice;

import com.hisense.common.util.DateUtils;
import com.hisense.common.util.DesUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.Random;

/**
 * 打印发票设置
 *
 * @author xieliyan     2017-09-15 HIPOSAPP-133
 */
public class EInvoice {

    // 发票服务地址
    private static String url = "";
    private static String checkCode = "";
    private static String encryptType = "";
    private static String key = "";

    private EInvoice() {
        // NO CODE HERE.
        // 避免类被初始化.
    }

    /**
     * 获取服务地址。
     */
    public static String serviceUrl() {
        return url;
    }

    /**
     * 设置服务地址。
     *
     * @param value 服务地址
     */
    public static void serviceUrl(String value) {
        EInvoice.url = value;
    }

    public static String getEncryptType() {
        return encryptType;
    }

    public static void setEncryptType(String encryptType) {
        EInvoice.encryptType = encryptType;
    }

    public static String getKey() {
        return key;
    }

    public static void setKey(String key) {
        EInvoice.key = key;
    }

    /**
     * 设置服务地址。
     */
    public static String checkCode() {
        String invoiceCode = url;
        if (invoiceCode == null) {
            return null;
        }
        if (invoiceCode.contains("[random:")) {
            String length = invoiceCode.substring((invoiceCode.indexOf("[random:") + 8), invoiceCode.indexOf("[random:") + 8 + invoiceCode.substring(invoiceCode.indexOf("[random:") + 8).indexOf("]"));

            checkCode = newCheckCode(Integer.parseInt(length));
        } else {
            checkCode = newCheckCode(4);
        }

        return checkCode;
    }

    /**
     * 二维码内容
     *
     * @return invoiceConfig 返回组装后的二维码内容
     */
    public static String eInvoiceCode() {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();

        if (StringUtils.isEmpty(url)) {
            url = GeneralSettings.INSTANCE.getInvoiceUrl();
        }
        if (StringUtils.isEmpty(GeneralSettings.INSTANCE.getInvoiceUrl())) {
            url = "";
        }
        String invoiceCode = url;
        LoggerUtils.getLogger().debug("组装发票二维码，二维码模板：" + invoiceCode);
        if (invoiceCode.contains("[orgCode]")) {
            invoiceCode = invoiceCode.replace("[orgCode]", Config.org_code);

        }
        if (invoiceCode.contains("[date:")) {
            String formatDate = invoiceCode.substring(invoiceCode.indexOf("[date:") + 6, invoiceCode.indexOf("[date:") + 6 + invoiceCode.substring(invoiceCode.indexOf("[date:") + 6).indexOf("]"));
            invoiceCode = invoiceCode.replace(("[date:" + formatDate + "]"), DateUtils.dateFormat(new Date(), formatDate));
        }
        if (invoiceCode.contains("[posNo]")) {
            invoiceCode = invoiceCode.replace("[posNo]", Config.pos_no);
        }
        if (invoiceCode.contains("[random:")) {
            String length = invoiceCode.substring((invoiceCode.indexOf("[random:") + 8), invoiceCode.indexOf("[random:") + 8 + invoiceCode.substring(invoiceCode.indexOf("[random:") + 8).indexOf("]"));
            String newCheckCode = newCheckCode(Integer.parseInt(length));

            invoiceCode = invoiceCode.replace(("[random:" + length + "]"), checkCode == null ? "" : checkCode);
        }
        if (invoiceCode.contains("[accDate:")) {
            String formatDate = analysisRegex(invoiceCode, "accDate");
            String formatValue = "yyyy-MM-dd";
            try {
                invoiceCode = invoiceCode.replace(("[accDate:" + formatDate + "]"), DateUtils.dateFormat(saleModel.getAccDate(), formatValue, formatDate));
            } catch (Exception e) {
                invoiceCode = "";
            }
        }

        if (invoiceCode.contains("[saleNo]")) {
            invoiceCode = invoiceCode.replace("[saleNo]", saleModel.getSaleNo());
            LoggerUtils.getLogger().debug("流水号saleNo" + saleModel.getSaleNo());
        }
        if (invoiceCode.contains("[timestampMillis:")) {
            String day = analysisRegex(invoiceCode, "timestampMillis");
            long currentTimeMillis = System.currentTimeMillis() + StringUtils.toInt(day, 0) * 24 * 60 * 60 * 1000;
            invoiceCode = invoiceCode.replace(("[timestampMillis:" + day + "]"), String.valueOf(currentTimeMillis));
        }
        if (invoiceCode.contains("[totalRecv:")) {
            String numberScale = analysisRegex(invoiceCode, "totalRecv");
            int round = Integer.parseInt(numberScale.substring(0, 1));
            BigDecimal total = new BigDecimal(saleModel.getTotalRecv()).setScale(round, RoundingMode.HALF_UP);
            invoiceCode = invoiceCode.replace(("[totalRecv:" + numberScale + "]"), total.toString());
        }

        if (invoiceCode.contains("{ENCRYPT:")) {
            String encrypt = analysisEncrypt(invoiceCode, "ENCRYPT");
            encryptType = encrypt.substring(0, encrypt.indexOf(":"));
            key = encrypt.replace(encryptType + ":", "");
            invoiceCode = invoiceCode.replace("{ENCRYPT:" + encrypt + "}", "");
            int beginIndexEncrypt = invoiceCode.indexOf("{") + 1;
            int endIndexEncrypt = beginIndexEncrypt + invoiceCode.substring(beginIndexEncrypt).indexOf("}");
            String contents = invoiceCode.substring(beginIndexEncrypt, endIndexEncrypt);
            String encryptedContents = "";
            try {
                encryptedContents = DesUtils.encryptDES(contents, key);
            } catch (Exception e) {
                LoggerUtils.getLogger().debug("发票信息加密异常");
            }

            invoiceCode = invoiceCode.replace("{" + contents + "}", encryptedContents);

        }

        LoggerUtils.getLogger().debug("组装发票二维码完成：" + invoiceCode);
        return invoiceCode;
    }

    /**
     * 返回随机产生的提取码(0000-9999)。
     *
     * @return 包含 4 位数字的字符串
     */
    public static String newCheckCode() {
        Random random = new Random();
        String fourRandom = random.nextInt(10000) + "";
        int randLength = fourRandom.length();
        if (randLength < 4) {
            for (int i = 1; i <= 4 - randLength; i++) {
                fourRandom = "0" + fourRandom;
            }
        }
        return fourRandom;
    }

    /**
     * 返回随机产生的提取码(0000-9999)。
     *
     * @param length 随机数个数
     * @return 包含 4 位数字的字符串
     */
    public static String newCheckCode(int length) {
        Random random = new Random();
        String fourRandom = random.nextInt(10000) + "";
        int randLength = fourRandom.length();
        if (randLength < length) {
            for (int i = 1; i <= length - randLength; i++) {
                fourRandom = "0" + fourRandom;
            }
        }
        return fourRandom;
    }

    private static String analysisRegex(String value, String regex) {
        return value.substring((value.indexOf("[" + regex + ":") + regex.length() + 2),
                value.indexOf("[" + regex + ":") + regex.length() + 2 +
                        value.substring(value.indexOf("[" + regex + ":") + regex.length() + 2).indexOf("]"));
    }

    private static String analysisEncrypt(String value, String regex) {
        return value.substring((value.indexOf("{" + regex + ":") + regex.length() + 2),
                value.indexOf("{" + regex + ":") + regex.length() + 2 +
                        value.substring(value.indexOf("{" + regex + ":") + regex.length() + 2).indexOf("}"));
    }

}
