/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Environment;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.common.BitMatrix;
import com.hisense.common.util.LoggerUtils;
import com.ums.AppHelper;

import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.util.List;

/*******************************************************************************
 * 功能说明：打印机工具类。
 *           封装了所有对打印机进行直接调用的逻辑。
 * 版权说明：青岛海信智能商用系统有限公司  版权所有
 * 创建：guanguangjin 2017-02-04
 * 修改： 
 * @author guolina 2018-08-08 HIPOSAPP-704 修复生成二维码图片时抛出异常,导致的APK闪退
 ******************************************************************************/

public class PrintTools {

    private static PrintTools printTools = null;

    /**
     * 通用打印方法
     * 根据打印票据类型和打印机的设置情况选择打印机
     *
     * @param printList 打印字符串列表
     * @param type      打印类型
     * @param context   当前活动的上下文对象
     */
    public synchronized static void print(List<String> printList, IPrintService.PrtPatternType type,
                                          Activity context) {

        // 单例模式
        if (printTools == null) {
            printTools = new PrintTools();
        }

    }


    /**
     * 调用本地打印机
     * 将打印的内容生成图片，最后打印图片。
     *
     * @param printList 待打印的字符串列表
     * @param context   当前活动的上下文
     */
    private void printByLocal(List<String> printList, Activity context) {

        int lineHeight = 23;    // 小：23， 中：26， 大：29
        int textSize = 22;      // 小：22， 中：25， 大：28
        int picWidth = 400;     // 生成打印图像的宽度（px）

        // TODO 高度的处理要考虑指令的行数对图像整体高度的影响，如果由很多指令图像底部就会由大量的
        // TODO 留白
        Bitmap picture = Bitmap.createBitmap(picWidth, lineHeight * printList.size(), Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(picture);
        canvas.drawColor(Color.WHITE);
        Paint paint = new Paint();
        paint.setColor(Color.BLACK);
        paint.setTextSize(textSize);
        paint.setTextAlign(Paint.Align.LEFT);

        int baseLineX = 0, baseLineY = 25;
        String line = null;
        for (int i = 0; i < printList.size(); i++) {
            line = printList.get(i);

            /**
             * 匹配自定义的指令
             * 当匹配到自定义的指令时，根据不同的指令来设置下次绘图的样式。
             */
            if (StringUtils.equalsIgnoreCase("cmd_centerStart", line)) {    // 文字居中开始
                paint.setTextAlign(Paint.Align.CENTER);
                baseLineX = picture.getWidth() / 2;
            } else if (StringUtils.equalsIgnoreCase("cmd_centerStart", line)) {    // 文字居中结束
                paint.setTextAlign(Paint.Align.LEFT);
                baseLineX = 0;
            } else {
                canvas.drawText(line, baseLineX, baseLineY, paint);
                baseLineY += lineHeight;
            }
        }
        canvas.save();
        canvas.restore();

        String path = Environment.getExternalStorageDirectory() + "/printPicture.png";
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(new File(path));
        } catch (Exception e) {
            e.printStackTrace();
        }

        picture.compress(Bitmap.CompressFormat.PNG, 100, out);
        AppHelper.callPrint(context, path);
    }


    /**
     * 返回条形码位图
     *
     * @param content 条形码信息
     * @param pWidth  条形码宽度
     * @param pHeight 条形码高度
     * @return 位图
     * @author xieliyan 2017-08-31 HIPOSAPP-133 重构生成Bitmap代码
     */
    public static Bitmap CreateBarCode(String content, int pWidth, int pHeight) {
        // 生成一维条码,编码时指定大小,不要生成了图片以后再进行缩放,这样会模糊导致识别失败
        try {
            BitMatrix matrix = new MultiFormatWriter().encode(content, BarcodeFormat.CODE_128, pWidth, pHeight);
            return createBitmap(matrix);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("CreateBarCode:" + e.toString());
            return null;
        }

    }


    /**
     * 返回二维码位图
     *
     * @param content 二维码信息
     * @param pWidth  二维码宽度
     * @param pHeight 二维码高度
     * @return 位图
     * @author xieliyan 2017-08-31 HIPOSAPP-133 生成二维码位图
     */
    public static Bitmap CreateQRCode(String content, int pWidth, int pHeight) {
        try {
            BitMatrix matrix = new MultiFormatWriter().encode(content, BarcodeFormat.QR_CODE, pWidth, pHeight);
            return createBitmap(matrix);
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("CreateQRCode:" + e.toString());
            return null;
        }


    }

    /**
     * 返回位图
     *
     * @param matrix 位阵
     * @return 位图
     * @author xieliyan 2017-08-31 HIPOSAPP-133 创建位图对象
     */
    public static Bitmap createBitmap(BitMatrix matrix) {
        // 开始利用二维码数据创建Bitmap图片，分别设为黑白两色
        int width = matrix.getWidth();
        int height = matrix.getHeight();
        int[] pixels = new int[width * height];
        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                if (matrix.get(x, y)) {
                    pixels[y * width + x] = 0xff000000;
                }
            }
        }
        // 创建一张bitmap图片，采用最高的效果显示
        Bitmap bitmap = Bitmap.createBitmap(width, height,
                Bitmap.Config.ARGB_8888);
        // 将上面的二维码颜色数组传入，生成图片颜色
        bitmap.setPixels(pixels, 0, width, 0, 0, width, height);
        return bitmap;

    }
}
