/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print;


import com.hisense.hiaposcmp.print.anno.PrtPatternAnno;
import com.hisense.hiaposcmp.print.bean.BasePrintBean;

import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/*******************************************************************************
 * 功能说明：打印模板引擎 TODO 尚未详细理解
 * 用来操作存储在本地文件中的打印模板。
 * 版权说明：青岛海信智能商用系统有限公司  版权所有
 * 创建：guanguangjin 2017-02-04
 * 修改：
 ******************************************************************************/

public class TemplateEngine implements IContentEngine {

    /**
     * 生成待打印的数据
     *
     * @param templateContent 打印模板数据，没有模板传null
     * @param bean            打印单元对象
     * @param type            打印类型
     * @return 待打印的字符串列表
     */
    @Override
    public List<String> getContentNeedPrint(String templateContent, BasePrintBean bean, IPrintService.PrtPatternType type) {

        // sb用来存储替换过的内容，它会把多次处理过的字符串按源字符串序 存储起来。
        StringBuffer sb = new StringBuffer();

        // 解析Bean对象的注解，组织成map
        Map<String, ?> data = parseFromBean(bean);

        try {
            Pattern pattern = Pattern.compile("\\[(.+?)\\]", Pattern.DOTALL);
            Matcher matcher = pattern.matcher(templateContent);
            String value = null;

            while (matcher.find()) {
                String name = matcher.group();      // 键名
                String key = StringUtils.lowerCase(StringUtils.trim(StringUtils.substringBetween(
                        name, "[", "]")));

                // if key start with cmd then means that current line is special
                // command
                if (StringUtils.startsWithIgnoreCase(key, "cmd")) {
                    value = parseCmd(key, data);
                } else {
                    value = StringUtils.trimToEmpty((String) data.get(key));    // 键值
                }

                matcher.appendReplacement(sb, value == null ? "" : value);
            }
            matcher.appendTail(sb);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return contentToStrList(sb.toString());
    }

    private List<Map<String, Object>> parseList(List<Object> list) {

        if (list == null || list.size() == 0) {
            return null;
        }

        List<Map<String, Object>> resultList = new ArrayList<Map<String, Object>>();

        Map<String, Object> map = null;
        for (Object obj : list) {

            map = parseFromBean(obj);
            resultList.add(map);
        }
        return resultList;
    }

    private Map<String, Object> parseFromBean(Object obj) {
        Map<String, Object> map = new HashMap<String, Object>();

        Method[] methArr = obj.getClass().getDeclaredMethods();
        PrtPatternAnno anno = null;
        for (Method method : methArr) {

            if (StringUtils.startsWithIgnoreCase(method.getName(), "get")
                    || StringUtils.startsWithIgnoreCase(method.getName(), "is")) {

                if (method == null
                        || method.getAnnotation(PrtPatternAnno.class) == null) {
                    continue;
                }

                if (method.getReturnType() == List.class) {
                    try {
                        @SuppressWarnings("unchecked")
                        List<Map<String, Object>> listMap = parseList((List<Object>) method
                                .invoke(obj));
                        anno = method.getAnnotation(PrtPatternAnno.class);
                        map.put(StringUtils.lowerCase(anno.key()), listMap);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    continue;
                }

                try {
                    anno = method.getAnnotation(PrtPatternAnno.class);
                    Object resultFromInvoke = method.invoke(obj);
                    if (resultFromInvoke instanceof String) {
                        map.put(StringUtils.lowerCase(anno.key()), StringUtils
                                .trimToEmpty((String) resultFromInvoke));

                        // support alias for enhancing compatible
                        if (StringUtils.isNotBlank(anno.alias())) {
                            map.put(StringUtils.lowerCase(anno.alias()),
                                    StringUtils.trimToEmpty((String) resultFromInvoke));
                        }
                    } else if (resultFromInvoke instanceof Boolean) {
                        map.put(StringUtils.lowerCase(anno.key()), (Boolean) resultFromInvoke);
                    } else {
                        map.put(StringUtils.lowerCase(anno.key()), resultFromInvoke);
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return map;
    }

    private String parseCmd(String key, Map<String, ?> dataMap) {
        // re-print
        if (StringUtils.startsWithIgnoreCase(key, PatternCmd.CMD_RE_PRINT.getCommad())) {
            String mapKey = StringUtils.substringBetween(key, ":", "(");
            Object valueFromIs = dataMap.get(mapKey);
            boolean isVar = false;
            if (valueFromIs == null) {
                return null;
            }
            if (valueFromIs instanceof Boolean) {
                isVar = (Boolean) valueFromIs;
            } else {
                isVar = BooleanUtils.toBoolean(valueFromIs.toString());
            }
            String content = null;
            if (isVar) {
                content = StringUtils.substringBetween(key, "(", ")");
            }
            return content;
        }

        if (StringUtils.startsWithIgnoreCase(key, PatternCmd.CMD_LOOP.getCommad())) {
            String mapKey = StringUtils.substringBetween(key, ":", "{");

            if (StringUtils.isBlank(mapKey)) {
                return null;
            }

            Object mapInList = dataMap.get(mapKey);
            if (mapInList instanceof List<?>) {
                @SuppressWarnings("unchecked")
                List<Map<String, String>> subListMap = (List<Map<String, String>>) mapInList;
                String subFormat = StringUtils.substringBetween(key, "{", "}");

                if (StringUtils.isBlank(subFormat)) {
                    return null;
                }

                StringBuffer sb = new StringBuffer();
                for (Map<String, String> subMap : subListMap) {
                    try {
                        Pattern pattern = Pattern.compile("\\((.+?)\\)");
                        Matcher matcher = pattern.matcher(subFormat);
                        String value = null;
                        while (matcher.find()) {
                            String name = matcher.group();  // 键名
                            String subKey = StringUtils.trim(StringUtils
                                    .substringBetween(name, "(", ")"));
                            value = StringUtils.trimToEmpty((String) subMap
                                    .get(subKey));          // 键值
                            matcher.appendReplacement(sb, value);
                        }
                        matcher.appendTail(sb);
                        sb.append("\r\n");
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                return sb.toString();
            }
        }
        return null;
    }

    private List<String> contentToStrList(String content) {

        // 按照 换行 分割
        List<String> resultList = Arrays.asList(StringUtils.split(content,
                "\r\n"));
        return resultList;
    }

    public static enum PatternCmd {
        CMD_RE_PRINT("cmdBool"), CMD_LOOP("cmdLoop");

        private String command = null;

        PatternCmd(String command) {
            this.command = command;
        }

        public String getCommad() {
            return command;
        }
    }
}
