/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.bean;

import com.hisense.hiaposcmp.print.anno.PrtPatternAnno;

import org.apache.commons.lang3.math.NumberUtils;

import java.util.List;

/**
 * 赠券信息Bean，用作打印单元的一部分。
 *
 * @author guanguangjin 2017-02-04
 */
public class CouponUnitBean extends BasePrintBean {

    private String saleNo;              // 流水号
    private String couponName;          // 赠券名称
    private String couponNo;            // 赠券号
    private String couponPrice;         // 赠券金额
    private String checkCode;           // 校验码
    private boolean printable;          // 是否打印
    private String expDateBegin;        // 有效期 开始日期
    private String expDateEnd;          // 有效期 结束日期
    private String couponText;          // 赠券说明

    /**
     * 赠券信息汇总
     *
     * @param couponList 赠券列表
     * @return 赠券汇总信息
     * @author guanguangjin 2017-02-04
     */
    public static String getTotalInfo(List<CouponUnitBean> couponList) {

        if (couponList == null || couponList.size() == 0) {
            return "No Coupon";
        }

        float priceTotal = 0F;
        int count = 0;
        for (CouponUnitBean coupon : couponList) {
            count++;
            priceTotal += NumberUtils.toFloat(coupon.getCouponPrice());
        }

        return "共发放了" + count + " 张券，合计金额 " + String.format("%.2f", priceTotal) + " 元";
    }

    /*************************** getter and setter ****************************/

    @PrtPatternAnno(key = "saleNo")
    public String getSaleNo() {
        return saleNo;
    }

    public void setSaleNo(String saleNo) {
        this.saleNo = saleNo;
    }

    @PrtPatternAnno(key = "couponName")
    public String getCouponName() {
        return couponName;
    }

    public void setCouponName(String couponName) {
        this.couponName = couponName;
    }

    @PrtPatternAnno(key = "couponNo")
    public String getCouponNo() {
        return couponNo;
    }

    public void setCouponNo(String couponNo) {
        this.couponNo = couponNo;
    }

    @PrtPatternAnno(key = "couponPrice")
    public String getCouponPrice() {
        return couponPrice;
    }

    public void setCouponPrice(String couponPrice) {
        this.couponPrice = couponPrice;
    }

    @PrtPatternAnno(key = "couponCheckCode")
    public String getCheckCode() {
        return checkCode;
    }

    public void setCheckCode(String checkCode) {
        this.checkCode = checkCode;
    }

    public boolean isPrintable() {
        return printable;
    }

    public void setPrintable(boolean printable) {
        this.printable = printable;
    }

    @PrtPatternAnno(key = "couponExpDateBegin")
    public String getExpDateBegin() {
        return expDateBegin;
    }

    public void setExpDateBegin(String expDateBegin) {
        this.expDateBegin = expDateBegin;
    }

    @PrtPatternAnno(key = "couponExpDateEnd")
    public String getExpDateEnd() {
        return expDateEnd;
    }

    public void setExpDateEnd(String expDateEnd) {
        this.expDateEnd = expDateEnd;
    }

    @PrtPatternAnno(key = "couponText")
    public String getCouponText() {
        return couponText;
    }

    public void setCouponText(String couponText) {
        this.couponText = couponText;
    }
}
