/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.bean;

import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.print.anno.PrtPatternAnno;

import java.util.List;

/**
 * 打印单元Bean。包含打印信息、打印机配置、打印模板
 *
 * @author guanguangjin 2017-02-04
 * @author guolina      2018-08-09 HIPOSAPP-446 增加实时积分打印参数
 * @author guolina      2018-11-14 HIPOSAPP-823 小票增加礼品券打印
 * @author guolina      2018-12-11 HIPOSAPP-850 小票增加门店名称打印
 * @author guolina      2018-12-27 HIPOSAPP-839 小票增加促销内容打印，支持聚合支付优惠
 * @author guolina      2019-03-21 HIPOSAPP-1056 万科小票打印字段完善
 * @author wangyaogang  2019-05-31 HIPOSAPP-1114 温州时代-小票增加"本笔交易赠送金额券：总额"
 * @author wangyaogang  2019-06-13 HIPOSAPP-1134 增加剩余可用券remainedCouponList
 * @author wangyaogang  2019-08-14 HIPOSAPP-1219 易收银销售小票-打印累计数量
 * @author guolina      2020-02-21 HIAPOSCMP-541 慧收银支持会员积分打印
 * @author guolina       2020-05-18 HIAPOSCMP-702 吉麦隆淘鲜达营销方案对接涉及慧收银调整增加支付明细和赠券打印

 */
public class SalePrintBean extends BasePrintBean {

    private boolean isRePrint;                  // 是否重打
    private boolean invoiceExchangeable;        // 是否可兑换发票
    private String saleNo;                      //流水号
    private String casherCode;                  // 收款员号
    private String shopCode;                    // 专柜号
    private String orgName;                     // 门店名称
    private String shopAddress;                 // 门店地址
    private String shopTelephone;               // 门店电话
    private String saleDate;                    // 销售日期
    private double totalCharge;                 //实收金额
    private double grossCharge;                 //合计金额（发票）
    private String grossChargeChinese;          // 合计金额中文大写
    private double discount;                    //优惠金额
    private double change;                      //找零
    private String cardFaceNo;                  // 会员卡面号
    private String cardType;                    // 会员卡类型
    private String expensePoint;                // 本次消费积分
    private String parkingPoint;                // 本次停车积分
    private String lastUsablePoint;             // 上次可用积分,除scrm时,与vipPoints取值相同
    private String couponPoint;                 // 本次赠券积分
    private String vipPoints;                  // 当前累计积分
    private String sumPoints;                  // 慧收银会员积分
    private String totalDscSelf;                // 直连支付商户优惠金额
    private String totalDscChannel;             // 直连支付平台优惠金额
    private int invPluNumPerPage;               // 每张发票的商品数
    private boolean isDisForLastInvPage;        // 发票的最后一页是否要进行分页
    private int invoiceMaxLine;                 // 每张发票的打印行数
    private int invoiceConsume;                 // 本次交易使用的发票数量
    private String invoiceTransCode;               // 发票交易码
    private String invoiceRandom;                  // 发票提取码
    private String tradeMode;                   // 交易模式
    private String shMode;                      // 收货方式
    private String shrName;                     // 收货人
    private String shrTel;                      // 电话
    private String shrAddr;                     // 收货地址
    private String posNo;                       //机器号
    private String orgCode;                     //门店号
    private String totalCount;                    //总件数 ,等于 pluTotalCount
    private double total;                         //总合计金额 ,等于 originalCharge
    private double totalDsc;                      //总优惠合计 ,等于discount ,但在易收银里discount赋值逻辑不同,皆保留
    private String couponTotalValue;              //赠券总金额
    private boolean isVip;                      //是否会员
    private String currPoints;                  //当前积分  等于 vipPoints ?
    private String invoiceNo;                   //小票号
    private String invoiceQRInfo;               //小票二维码信息
    private String invoiceInfo;                 //小票二维码开票信息
    private String payNo;                       //支付号、商户订单号

    private List<String> shopCodeList;            //专柜
    private List<SalePluPrintBean> pluList;              // 商品列表
    private List<HxPayCouponPrintBean> hxPayCouponList;  // 石基淘鲜达返回券
    private List<SalePluDscPrintBean> pluDscList;        // 商品促销列表
    private List<GiftCouponPrintBean> giftCouponList;// 礼品券列表
    private List<PayMethodPrintBean> payMethodList;  // 支付方式列表
    private List<CouponUnitBean> couponList;    // 满赠券列表
    private List<CouponUnitBean> remainedCouponList;    // 核销过后赠券列表
    private List<CouponPrintBean> couponPrintTiceketList;//满赠券打印小票列表

    /*************************** getter and setter ****************************/

    @PrtPatternAnno(key = "couponTotalValue")
    public String getCouponTotalValue() {
        return couponTotalValue;
    }

    public void setCouponTotalValue(String couponTotalValue) {
        this.couponTotalValue = couponTotalValue;
    }

    @PrtPatternAnno(key = "isRePrint")
    public boolean isRePrint() {
        return isRePrint;
    }

    public void setRePrint(boolean rePrint) {
        isRePrint = rePrint;
    }

    public boolean isInvoiceExchangeable() {
        return invoiceExchangeable;
    }

    public void setInvoiceExchangeable(boolean invoiceExchangeable) {
        this.invoiceExchangeable = invoiceExchangeable;
    }

    @PrtPatternAnno(key = "posNo")
    public String getPosNo() {
        return posNo;
    }

    public void setPosNo(String posNo) {
        this.posNo = posNo;
    }

    @PrtPatternAnno(key = "orgCode")
    public String getOrgCode() {
        return orgCode;
    }

    public void setOrgCode(String orgCode) {
        this.orgCode = orgCode;
    }

    @PrtPatternAnno(key = "orgName")
    public String getOrgName() {
        return orgName;
    }

    public void setOrgName(String orgName) {
        this.orgName = orgName;
    }

    @PrtPatternAnno(key = "shopAddress")
    public String getShopAddress() {
        return shopAddress;
    }

    public void setShopAddress(String shopAddress) {
        this.shopAddress = shopAddress;
    }

    @PrtPatternAnno(key = "shopTelephone")
    public String getShopTelephone() {
        return shopTelephone;
    }

    public void setShopTelephone(String shopTelephone) {
        this.shopTelephone = shopTelephone;
    }

    @PrtPatternAnno(key = "cashCode", alias = "clsCode")
    public String getCasherCode() {
        return casherCode;
    }

    public void setCasherCode(String casherCode) {
        this.casherCode = casherCode;
    }

    @PrtPatternAnno(key = "shopCode", alias = "packCode")
    public String getShopCode() {
        return shopCode;
    }

    public void setShopCode(String shopCode) {
        this.shopCode = shopCode;
    }

    @PrtPatternAnno(key = "saleDate", alias = "time")
    public String getSaleDate() {
        return saleDate;
    }

    public void setSaleDate(String saleDate) {
        this.saleDate = saleDate;
    }

    @PrtPatternAnno(key = "grossChargeChinese")
    public String getGrossChargeChinese() {
        return grossChargeChinese;
    }

    public void setGrossChargeChinese(String grossChargeChinese) {
        this.grossChargeChinese = grossChargeChinese;
    }

    @PrtPatternAnno(key = "cardFaceNo", alias = "vipcode")
    public String getCardFaceNo() {
        return cardFaceNo;
    }

    public void setCardFaceNo(String cardFaceNo) {
        this.cardFaceNo = cardFaceNo;
    }

    @PrtPatternAnno(key = "cardType")
    public String getCardType() {
        return cardType;
    }

    public void setCardType(String cardType) {
        this.cardType = cardType;
    }

    @PrtPatternAnno(key = "lastUsablePoint")
    public String getLastUsablePoint() {
        return lastUsablePoint == null ? "0" : lastUsablePoint;
    }

    public void setLastUsablePoint(String lastUsablePoint) {
        this.lastUsablePoint = lastUsablePoint;
    }

    @PrtPatternAnno(key = "expensePoint")
    public String getExpensePoint() {
        return expensePoint == null ? "0" : expensePoint;
    }

    public void setExpensePoint(String expensePoint) {
        this.expensePoint = expensePoint;
    }

    @PrtPatternAnno(key = "parkingPoint")
    public String getParkingPoint() {
        return parkingPoint == null ? "0" : parkingPoint;
    }

    public void setParkingPoint(String parkingPoint) {
        this.parkingPoint = parkingPoint;
    }

    @PrtPatternAnno(key = "vipPoints")
    public String getVipPoints() {
        return vipPoints == null ? "0" : vipPoints;
    }

    public void setVipPoints(String vipPoints) {
        this.vipPoints = vipPoints;
    }

    @PrtPatternAnno(key = "sumPoints")
    public String getSumPoints() {
        return sumPoints == null ? "0" : sumPoints;
    }

    public void setSumPoints(String sumPoints) {
        this.sumPoints = sumPoints;
    }

    @PrtPatternAnno(key = "couponPoint")
    public String getCouponPoint() {
        return couponPoint == null ? "0" : couponPoint;
    }

    public void setCouponPoint(String couponPoint) {
        this.couponPoint = couponPoint;
    }

    @PrtPatternAnno(key = "totalDscSelf")
    public String getTotalDscSelf() {
        return totalDscSelf;
    }

    public void setTotalDscSelf(String totalDscSelf) {
        this.totalDscSelf = totalDscSelf;
    }

    @PrtPatternAnno(key = "totalDscChannel")
    public String getTotalDscChannel() {
        return totalDscChannel;
    }

    public void setTotalDscChannel(String totalDscChannel) {
        this.totalDscChannel = totalDscChannel;
    }

    public int getInvPluNumPerPage() {
        return invPluNumPerPage;
    }

    public void setInvPluNumPerPage(int invPluNumPerPage) {
        this.invPluNumPerPage = invPluNumPerPage;
    }

    public boolean isDisForLastInvPage() {
        return isDisForLastInvPage;
    }

    public void setDisForLastInvPage(boolean disForLastInvPage) {
        isDisForLastInvPage = disForLastInvPage;
    }

    public int getInvoiceMaxLine() {
        return invoiceMaxLine;
    }

    public void setInvoiceMaxLine(int invoiceMaxLine) {
        this.invoiceMaxLine = invoiceMaxLine;
    }

    public void setInvoiceConsume(int invoiceConsume) {
        this.invoiceConsume = invoiceConsume;
    }

    @PrtPatternAnno(key = "invoiceTransCode")
    public String getInvoiceTransCode() {
        return invoiceTransCode;
    }

    public void setInvoiceTransCode(String invoiceTransCode) {
        this.invoiceTransCode = invoiceTransCode;
    }

    @PrtPatternAnno(key = "invoiceRandom")
    public String getInvoiceRandom() {
        return invoiceRandom;
    }

    public void setInvoiceRandom(String invoiceRandom) {
        this.invoiceRandom = invoiceRandom;
    }

    @PrtPatternAnno(key = "tradeMode")
    public String getTradeMode() {
        return tradeMode;
    }

    public void setTradeMode(String tradeMode) {
        this.tradeMode = tradeMode;
    }

    @PrtPatternAnno(key = "shMode")
    public String getShMode() {
        return shMode;
    }

    public void setShMode(String shMode) {
        this.shMode = shMode;
    }

    @PrtPatternAnno(key = "shrName")
    public String getShrName() {
        return shrName;
    }

    public void setShrName(String shrName) {
        this.shrName = shrName;
    }

    @PrtPatternAnno(key = "shrTel")
    public String getShrTel() {
        return shrTel;
    }

    public void setShrTel(String shrTel) {
        this.shrTel = shrTel;
    }

    @PrtPatternAnno(key = "shrAddr")
    public String getShrAddr() {
        return shrAddr;
    }

    public void setShrAddr(String shrAddr) {
        this.shrAddr = shrAddr;
    }

    @PrtPatternAnno(key = "pluList")
    public List<SalePluPrintBean> getPluList() {
        return pluList;
    }

    public void setPluList(List<SalePluPrintBean> pluList) {
        this.pluList = pluList;
    }

    @PrtPatternAnno(key = "hxPayCouponList")
    public List<HxPayCouponPrintBean> getHxPayCouponList() {
        return hxPayCouponList;
    }
    public void setHxPayCouponList(List<HxPayCouponPrintBean> hxPayCouponList) {
        this.hxPayCouponList = hxPayCouponList;
    }

    @PrtPatternAnno(key = "pluDscList")
    public List<SalePluDscPrintBean> getPluDscList() {
        return pluDscList;
    }

    public void setPluDscList(List<SalePluDscPrintBean> pluDscList) {
        this.pluDscList = pluDscList;
    }

    @PrtPatternAnno(key = "giftCouponList")
    public List<GiftCouponPrintBean> getGiftCouponList() {
        return giftCouponList;
    }

    public void setGiftCouponList(List<GiftCouponPrintBean> giftCouponList) {
        this.giftCouponList = giftCouponList;
    }

    @PrtPatternAnno(key = "payMethodList")
    public List<PayMethodPrintBean> getPayMethodList() {
        return payMethodList;
    }

    public void setPayMethodList(List<PayMethodPrintBean> payMethodList) {
        this.payMethodList = payMethodList;
    }

    @PrtPatternAnno(key = "couponList")
    public List<CouponUnitBean> getCouponList() {
        return couponList;
    }

    public void setCouponList(List<CouponUnitBean> couponList) {
        this.couponList = couponList;
    }

    public List<CouponUnitBean> getRemainedCouponList() {
        return remainedCouponList;
    }

    public void setRemainedCouponList(List<CouponUnitBean> remainedCouponList) {
        this.remainedCouponList = remainedCouponList;
    }

    @PrtPatternAnno(key = "couponPrintList")
    public List<CouponPrintBean> getCouponPrintTiceketList() {
        return couponPrintTiceketList;
    }

    public void setCouponPrintTiceketList(List<CouponPrintBean> couponPrintTiceketList) {
        this.couponPrintTiceketList = couponPrintTiceketList;
    }

    @PrtPatternAnno(key = "shopCodeList")
    public List<String> getShopCodeList() {
        return shopCodeList;
    }

    public void setShopCodeList(List<String> shopCodeList) {
        this.shopCodeList = shopCodeList;
    }

    @PrtPatternAnno(key = "totalCount")
    public String getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(String totalCount) {
        this.totalCount = totalCount;
    }

    @PrtPatternAnno(key = "total")
    public String getTotal() {
        return StringUtils.doubleFormat(total);
    }

    public void setTotal(double total) {
        this.total = total;
    }

    @PrtPatternAnno(key = "totalDsc")
    public String getTotalDsc() {
        return StringUtils.doubleFormat(totalDsc);
    }

    public void setTotalDsc(double totalDsc) {
        this.totalDsc = totalDsc;
    }

    @PrtPatternAnno(key = "saleNo")
    public String getSaleNo() {
        return saleNo;
    }

    public void setSaleNo(String saleNo) {
        if (saleNo.contains("#") && saleNo.contains("%")) {
            this.saleNo = saleNo.substring(saleNo.indexOf("#") + 1, saleNo.indexOf("%"));
        } else {
            this.saleNo = saleNo;
        }
    }

    @PrtPatternAnno(key = "invoiceNo")
    public String getInvoiceNo() {
        return invoiceNo;
    }

    public void setInvoiceNo(String invoiceNo) {
        this.invoiceNo = invoiceNo;
    }

    @PrtPatternAnno(key = "invoiceQRInfo")
    public String getInvoiceQRInfo() {
        return invoiceQRInfo;
    }

    public void setInvoiceQRInfo(String invoiceQRInfo) {
        this.invoiceQRInfo = invoiceQRInfo;
    }

    @PrtPatternAnno(key = "invoiceInfo")
    public String getInvoiceInfo() {
        return invoiceInfo;
    }

    public void setInvoiceInfo(String invoiceInfo) {
        this.invoiceInfo = invoiceInfo;
    }

    @PrtPatternAnno(key = "payNo")
    public String getPayNo() {
        return payNo;
    }

    public void setPayNo(String payNo) {
        this.payNo = payNo;
    }

    @PrtPatternAnno(key = "totalCharge", alias = "SsTotal")
    public String getTotalCharge() {
        return StringUtils.doubleFormat(totalCharge);
    }

    public void setTotalCharge(double totalCharge) {
        this.totalCharge = totalCharge;
    }

    @PrtPatternAnno(key = "grossCharge")
    public String getGrossCharge() {
        return StringUtils.doubleFormat(grossCharge);
    }

    public void setGrossCharge(double grossCharge) {
        this.grossCharge = grossCharge;
    }

    @PrtPatternAnno(key = "discount")
    public String getDiscount() {
        return StringUtils.doubleFormat(discount);
    }

    public void setDiscount(double discount) {
        this.discount = discount;
    }

    public void setDiscount(String discount) {
        this.discount = Double.valueOf(discount);
    }

    @PrtPatternAnno(key = "change", alias = "zlTotal")
    public String getChange() {
        return StringUtils.doubleFormat(change);
    }

    public void setChange(double change) {
        this.change = change;
    }

    @PrtPatternAnno(key = "currPoints")
    public String getCurrPoints() {
        return currPoints;
    }

    public void setCurrPoints(String currPoints) {
        this.currPoints = currPoints;
    }

    @PrtPatternAnno(key = "isVip")
    public boolean getIsVip() {
        return isVip;
    }

    public void setIsVip(boolean isVip) {
        this.isVip = isVip;
    }
}
