/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.engine.impl;

import android.annotation.SuppressLint;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.print.bean.CouponUnitBean;
import com.hisense.hiaposcmp.print.bean.GiftCouponPrintBean;
import com.hisense.hiaposcmp.print.bean.PayMethodPrintBean;
import com.hisense.hiaposcmp.print.bean.SalePluPrintBean;
import com.hisense.hiaposcmp.print.bean.SalePrintBean;
import com.hisense.hiaposcmp.print.engine.ITemplateEngine;
import com.hisense.hipos.template.models.PrintTemplateType;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 默认模板引擎
 *
 * @author guanguangjin 2017-02-04
 * @author guolina      2019-04-01 HIPOSAPP-1010 支持前台日志上传CMP5
 * @author wangyaogang  2019-06-24 HIPOSAPP-1090 默认模板增加礼品券
 * @author wangyaogang  2019-11-15  HIPOSAPP-1343 打印换行问题,多行时使用StaticLayout
 */
public class DefaultTemplateEngine implements ITemplateEngine {

    private int HEADER_LINE = 0;

    /**
     * 生成待打印的数据
     *
     * @param template 打印模板数据，没有模板传null
     * @param bean     打印单元对象
     * @param type     打印类型
     * @return 待打印的字符串列表
     */
    @Override
    public List<String> getPrintContent(String template, SalePrintBean bean,
                                        PrintTemplateType type) {
        List<String> resultList = null;

        switch (type) {
            case Receipt:
                resultList = this.invokeForVoucher(bean);
                break;
            case Coupon:
                resultList = this.invokeForCoupon(bean);
                break;
            case GiftCoupon:
                resultList = this.invokeForGiftCoupon(bean);
                break;
            default:
                LoggerUtils.getLogger().debug("BuiltInContentEngine.getContentNeedPrint  WRONG TYPE... ");
                break;
        }

        return resultList;
    }

    /**
     * 生成打印发票所需的字符串列表
     *
     * @param bean 打印单元Bena
     * @return 打印字符串列表
     */
    private List<String> invokeForInvoice(SalePrintBean bean) {

        if (bean.getPluList() == null || bean.getPluList().size() == 0) {
            return null;
        }

        List<String> resultList = new ArrayList<String>();
        double countTmp = 0D;                                   // 小计
        int invCount = 0;                                       // 本次交易所需发票数量

        int pageSize = bean.getInvPluNumPerPage();              // 每张发票包含的商品数量
        int maxLineInPage = bean.getInvoiceMaxLine();           // 每张发票打印的最大行数

        // 生成发票头部
        List<String> headerList = this.generateHeaderForInvoice(bean);
        resultList.addAll(headerList);

        // 生成发票尾部
        List<String> tailList = this.generateTailForInvoice(bean);
        int tailSize = tailList.size();

        List<SalePluPrintBean> pluList = bean.getPluList();
        SalePluPrintBean pluBean = null;
        for (int i = 0; i < pluList.size(); i++) {
            pluBean = pluList.get(i);

            resultList.add("  " + String.valueOf(pluBean.getPluCode()) + "  " + pluBean.getPluName());
            resultList.add("        " + pluBean.getPluCount() + "  " + pluBean.getPluPrice()
                    + "  " + pluBean.getPluPriceAct());

            // 计算小计
            countTmp += NumberUtils.toDouble(pluBean.getPluPriceAct());

            // 如果遇到每页的结束或者商品列表遍历完毕，则添加小计，小计清零，发票增1。
            if ((i != 0 && (i + 1) % pageSize == 0) || i == pluList.size() - 1) {

                resultList.add("小计：" + String.format("%.2f", countTmp)
                        + "  流水号：" + bean.getSaleNo() + "\n");

                // 小计清零
                countTmp = 0D;

                // 所需发票数量加一
                invCount++;

                // 如果商品遍历完毕，判断是否要添加发票尾，否则填充剩余行数，切纸。
                if (i == pluList.size() - 1) {

                    // 如果发票尾需要分页
                    if ((tailSize + resultList.size()) > invCount * maxLineInPage) {

                        // 结束最后一张商品页，另起一张打印结尾页
                        if (!dealPageEnd(resultList, invCount, maxLineInPage)) {
                            LoggerUtils.getLogger().debug("此处设置的发票商品个数和发票每页行数不合适，商品个数过大，导致发票行数超过发票每页行数。。。");
                            return null;
                        }
                        resultList.add("cmd_paperCut");
                        resultList.addAll(headerList);

                        invCount++;
                    }

                    // 将结尾部分单独添加到一页中，如果结尾部分超长，一页打印不开，则继续分页
                    invCount += addAllInvTailInPage(resultList, tailList, invCount, maxLineInPage);

                    // 如果结尾部分超长，继续分页
                    if (!dealPageEnd(resultList, invCount, maxLineInPage)) {
                        LoggerUtils.getLogger().debug("此处设置的发票商品个数和发票每页行数不合适，商品个数过大，导致发票行数超过发票每页行数。。。");
                        return null;
                    }
                    resultList.add("cmd_paperCut");
                    break;
                }

                // 如果结尾部分超长，继续分页
                if (!dealPageEnd(resultList, invCount, maxLineInPage)) {
                    LoggerUtils.getLogger().debug("此处设置的发票商品个数和发票每页行数不合适，商品个数过大，导致发票行数超过发票每页行数。。。");
                    return null;
                }
                resultList.add("cmd_paperCut");

                // 如果不是最后一页，添加发票头
                if (i != pluList.size() - 1) {
                    resultList.addAll(headerList);
                }
            }
        }
        bean.setInvoiceConsume(invCount);
        return resultList;
    }

    /**
     * 生成发票头
     * TODO 发票头
     *
     * @param bean 打印单元
     * @return 发票头字符串列表
     */
    private List<String> generateHeaderForInvoice(SalePrintBean bean) {

        StringBuilder sb = new StringBuilder();
        sb.append("       Temp 青岛海信东海商贸有限公司\n");
//        sb.append("              " + LoginActivity.userinfo.ORGNAME + "\n");
        if (bean.isRePrint()) {
            sb.append("发票重打\n");
        }
        sb.append("  " + bean.getSaleDate() + "     " + bean.getCasherCode() + "\n");
        sb.append("              个人\n");
        sb.append(" \n");        // 此处注意，\n之间要有空格，否则转换数组的时候会丢失换行
        sb.append(" \n");        // 此处为了计算总行数
        sb.append(" \n");

        String[] header = StringUtils.split(sb.toString(), "\n");
        HEADER_LINE = header == null ? 0 : header.length;

        return Arrays.asList(header);
    }

    /**
     * 生成发票尾
     *
     * @param bean 打印数据单元对象
     * @return 发票尾字符串列表
     */
    @SuppressLint("SimpleDateFormat")
    private List<String> generateTailForInvoice(SalePrintBean bean) {

        List<String> resultList = new ArrayList<String>();

        //增加会员信息
        if (StringUtils.isNotBlank(bean.getCardFaceNo())) {
            resultList.add("会员卡号:" + bean.getCardFaceNo());
        }

        if (StringUtils.isNotBlank(bean.getDiscount())
                && NumberUtils.toDouble(bean.getDiscount()) != 0d) {
            resultList.add("储值卡优惠：" + bean.getDiscount());
        }

        //此处修改为实收金额trade.get_YsTotal()
        resultList.add("合计：" + bean.getGrossCharge() + "  " + bean.getGrossChargeChinese());

        //增加支付方式列表
        if (null != bean.getPayMethodList() && bean.getPayMethodList().size() != 0) {
            for (int j = 0; j < bean.getPayMethodList().size(); j++) {

                PayMethodPrintBean payBean = bean.getPayMethodList().get(j);
                LoggerUtils.getLogger().debug("---PayMethod: SaleNo:" + bean.getSaleNo()
                        + ",PayName:" + payBean.getPayName() + ",PayCount:" + payBean.getPayCount()
                        + ", PayNo:" + payBean.getPayNo());

                if (StringUtils.isEmpty(payBean.getPayNo())) {
                    resultList.add(payBean.getPayName()
                            + ":" + payBean.getPayCount());
                } else {
                    resultList.add(payBean.getPayName()
                            + ":" + payBean.getPayCount()
                            + "(" + payBean.getPayNo() + ")");
                }
            }
        }

        //找零
        resultList.add("找零: " + bean.getChange());

        // 增加 满赠券信息
        if (bean.getCouponList() != null
                && bean.getCouponList().size() != 0) {
            List<CouponUnitBean> couponList = bean.getCouponList();
            resultList.add(" \n");
            resultList.add("赠送信息");
            resultList.add(CouponUnitBean.getTotalInfo(couponList));

            for (CouponUnitBean coupon : couponList) {
                resultList.add("赠券名称： " + coupon.getCouponName());
                resultList.add("赠券号： " + coupon.getCouponNo()
                        + " 赠券金额：" + coupon.getCouponPrice());
            }
        }

        // 结尾空行
        resultList.add(" \n");
        resultList.add(" \n");
        resultList.add(" \n");
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        resultList.add("打印时间：" + format.format(new Date()));
        resultList.add("=========================");

        return resultList;
    }

    /**
     * 生成打印小票所需要的字符串列表
     *
     * @param bean 打印数据单元
     * @return 字符串列表
     */
    private List<String> invokeForVoucher(SalePrintBean bean) {

        List<String> tempList = new ArrayList<String>();
//        tempList.add("       "+ LoginActivity.userinfo.ORGNAME +"零售商品销售凭证"); TODO 用户信息

        if (bean.isRePrint()) {
            tempList.add("小票重打");
        }


        tempList.add("流水号:" + bean.getSaleNo() + "  收款员:" + bean.getCasherCode());
        tempList.add("专柜:" + bean.getShopCode() + "  日期:" + bean.getSaleDate());

        //添加商品名称
        tempList.add("  商品编码    品名   数量   实收金额");
        if (null != bean.getPluList() && bean.getPluList().size() != 0) {

            for (int i = 0; i < bean.getPluList().size(); i++) {
                SalePluPrintBean pluBean = bean.getPluList().get(i);

                String str = pluBean.getPluCode() + "  " +pluBean.getPluName() + "  "
                        + pluBean.getPluCount() + " " + pluBean.getPluPriceAct();

                tempList.add(str);
            }
        }

        tempList.add("--------------------------------------");
        tempList.add("合计：                         " + bean.getTotalCharge());
        tempList.add("找零：" + bean.getChange());

        if (StringUtils.isNotBlank(bean.getCardFaceNo())) {
            tempList.add("  会员卡号:" + bean.getCardFaceNo());
            //由于无法显示本币积分，所以将会员积分信息暂时 屏蔽
            //resultList.add("  积分："+tmpTrade.vipInfo.JFLIST);
        }

        //增加支付方式列表
        if (null != bean.getPayMethodList() && bean.getPayMethodList().size() != 0) {
            for (int i = 0; i < bean.getPayMethodList().size(); i++) {

                PayMethodPrintBean payBean = bean.getPayMethodList().get(i);
                if (StringUtils.isEmpty(payBean.getPayNo())) {
                    tempList.add(payBean.getPayName()
                            + ":" + payBean.getPayCount());
                } else {
                    tempList.add(payBean.getPayName()
                            + ":" + payBean.getPayCount()
                            + "(" + payBean.getPayNo() + ")");
                }
            }
        }
        tempList.add("\n");

        // 如果是电商模式    空为传统模式，非空为电商模式
        if (StringUtils.isNotBlank(bean.getTradeMode())) {
            tempList.add(bean.getTradeMode() + ":");
            tempList.add("收货方式: " + bean.getShMode());
            tempList.add("收货人: " + bean.getShrName());
            tempList.add("电话: " + bean.getShrTel());
            tempList.add("收货地址: " + bean.getShrAddr());
        }
        tempList.add("\n\n");

        List<String> resultList = new ArrayList<String>();
        for (int i = 0; i < 3; i++) {
            switch (i) {
                case 0:
                    resultList.add("     第一联       顾客留存");
                    break;
                case 1:
                    resultList.add("     第二联       专柜留存");
                    break;
                case 2:
                    resultList.add("     第三联       商场留存");
                    break;
            }
            resultList.add("\n");
            resultList.addAll(tempList);
            resultList.add("\n\n\n");
            resultList.add("cmd_paperCut");
        }

        return resultList;
    }

    /**
     * 生成打印满赠券所需要的字符串列表
     *
     * @param bean 打印数据单元
     * @return 满赠券打印字符串列表
     */
    private List<String> invokeForCoupon(SalePrintBean bean) {

        List<String> resultList = new ArrayList<String>();

        // 增加 满赠券信息
        if (bean.getCouponList() != null
                && bean.getCouponList().size() != 0 && !bean.isRePrint()) {
            List<CouponUnitBean> couponList = bean.getCouponList();

            for (CouponUnitBean coupon : couponList) {

                if (!coupon.isPrintable()) {
                    LoggerUtils.getLogger().debug("BuiltInContentEngine.invokeForCoupon  coupon[" + coupon.getCouponNo() + "] should NOT be print... ");
                    continue;
                }

                resultList.add("cmd_centerStart");
                resultList.add("cmd_bold_2_start");
//                resultList.add(LoginActivity.userinfo.ORGNAME); TODO 登陆信息
//                resultList.add("┌────────┐");
//                resultList.add("│   JC V代金券   │");
//                resultList.add("└────────┘");
                resultList.add("┌──────────┐");
                resultList.add("│        百货券      │");
                resultList.add("└──────────┘");
                resultList.add("cmd_bold_2_end");
                resultList.add("cmd_centerEnd");
                resultList.add("cmd_bold_3_start");
                resultList.add("面值:  " + coupon.getCouponPrice() + " 元");    // 面值
                resultList.add("\n");
                resultList.add("cmd_bold_3_end");
                resultList.add("有效时间:  " + coupon.getExpDateBegin()
                        + " 至 " + coupon.getExpDateEnd());                        // 有效期
                resultList.add("赠券说明: \n" + coupon.getCouponText());        // 赠券说明
                resultList.add("\n");
                resultList.add("赠券号:");                                        // 赠券号
                resultList.add("BarCodeFlag");                                    //  BarCode
                resultList.add(coupon.getCouponNo());
                resultList.add("校验码:" + coupon.getCheckCode());                // 校验码

                resultList.add("\n流水号:" + bean.getSaleNo());                    // 流水号

                resultList.add("BarCode2Flag");                                    // 二维码测试
                resultList.add("      IOS版                  安卓版");
                resultList.add("APP|服务预约:无障碍出车·品牌独家优惠·最炫单品全搜罗");
                resultList.add("\n\n\n");
                resultList.add("cmd_paperCut");
            }
        }
        return resultList;
    }

    /**
     * 生成礼品券所需要的字符串列表
     *
     * @param bean 打印数据单元
     * @return 礼品券打印字符串列表
     */
    private List<String> invokeForGiftCoupon(SalePrintBean bean) {

        List<String> resultList = new ArrayList<>();

        // 增加礼品券信息
        if (bean.getGiftCouponList() != null
                && bean.getGiftCouponList().size() != 0 && !bean.isRePrint()) {
            List<GiftCouponPrintBean> couponList = bean.getGiftCouponList();

            for (GiftCouponPrintBean coupon : couponList) {

                if (!coupon.isGiftPrintable()) {
                    LoggerUtils.getLogger().debug("BuiltInContentEngine.invokeForGiftCoupon  coupon[" + coupon.getGiftCouponNo() + "] should NOT be print... ");
                    continue;
                }

                resultList.add("cmd_centerStart");
                resultList.add("cmd_bold_2_start");
                resultList.add("┌──────────┐");
                resultList.add("│        礼品券      │");
                resultList.add("└──────────┘");
                resultList.add("cmd_bold_2_end");
                resultList.add("cmd_centerEnd");
                resultList.add("\n");
                resultList.add("有效时间:  " + coupon.getGiftBeginDate()
                        + " 至 " + coupon.getGiftEndDate());                        // 有效期
                resultList.add("赠券说明: \n" + coupon.getGiftRemark());        // 赠券说明
                resultList.add("\n");
                resultList.add("赠券号:");                                        // 赠券号
                resultList.add(coupon.getGiftName() + "   " + coupon.getGiftCouponCount());                                        // 赠券号
                resultList.add("BarCodeFlag");                                    //  BarCode
                resultList.add(coupon.getGiftCouponNo());
                resultList.add("\n流水号:" + bean.getSaleNo());                    // 流水号
                resultList.add("\n\n\n");
                resultList.add("cmd_paperCut");
            }
        }
        return resultList;
    }

    /**
     * 切纸前，根据MaxLine对发票长度进行填充，返回Null表示当前页行数超过最大行数
     *
     * @param printList 打印字符串列表
     * @param pageNum   每张发票最大商品数
     * @param maxLine   每张发票打印行数
     * @return
     */
    private boolean dealPageEnd(List<String> printList, int pageNum, int maxLine) {
        int totalLine = printList.size();
        if (totalLine > pageNum * maxLine + pageNum) {
            LoggerUtils.getLogger().debug("发票打印长度超长【totalLine(" + totalLine +
                    ") > pageNum(" + pageNum + ") * pageSize(" + maxLine + ")】");
            return false;
        }

        if (totalLine == pageNum * maxLine + pageNum) {
            return true;
        }

        for (int start = (totalLine - totalLine / maxLine) % maxLine;
             start < maxLine; start++) {
            printList.add("cmd_lineBreak");
        }
        return true;
    }

    /**
     * 如果是发票最后一页，再次判断是否最后一页任然超长
     *
     * @param printList 打印字符串列表
     * @param tailList  发票尾部字符串列表
     * @param pageNum   每张发票最大商品数
     * @param maxLine   每张发票最大打印行数
     * @return
     */
    private int addAllInvTailInPage(List<String> printList, List<String> tailList,
                                    int pageNum, int maxLine) {

        int occurPageCount = 0;
        int mainCurSize = printList.size();
        int tailSize = tailList.size();

        if ((mainCurSize + tailSize) <= pageNum * maxLine + pageNum) {
            printList.addAll(tailList);
            return occurPageCount;
        }

        for (String tailStr : tailList) {
            printList.add(tailStr);
            mainCurSize = printList.size();
            if ((mainCurSize - mainCurSize / maxLine + 1) % maxLine == 0) {
                printList.add("cmd_paperCut");
                occurPageCount++;
            }
        }
        return occurPageCount;
    }
}
