/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.engine.impl;


import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.print.TemplateEngine;
import com.hisense.hiaposcmp.print.anno.PrtPatternAnno;
import com.hisense.hiaposcmp.print.bean.SalePrintBean;
import com.hisense.hiaposcmp.print.engine.ITemplateEngine;
import com.hisense.hipos.template.models.PrintTemplateType;

import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 外置模板引擎
 *
 * @author guanguangjin 2017-02-04
 * @author guolina      2018-12-27  HIPOSAPP-839 小票增加促销内容打印
 * @author wangyaogang  2019-11-15  HIPOSAPP-1343 打印换行问题,多行时使用StaticLayout
 */
public class ExternalTemplateEngine implements ITemplateEngine {

    /**
     * 生成待打印的数据
     *
     * @param templateContent 打印模板数据，没有模板传null
     * @param bean            打印单元对象
     * @param type            打印类型
     * @return 待打印的字符串列表
     */
    @Override
    public List<String> getPrintContent(String templateContent, SalePrintBean bean, PrintTemplateType type) {
        // sb用来存储替换过的内容，它会把多次处理过的字符串按源字符串序 存储起来。
        StringBuffer sb = new StringBuffer();

        // 解析Bean对象的注解，组织成map
        Map<String, ?> data = parseFromBean(bean);

        try {
            Pattern pattern = Pattern.compile("\\[(.+?)\\]", Pattern.DOTALL);
            Matcher matcher = pattern.matcher(templateContent);
            String value = null;

            while (matcher.find()) {
                String name = matcher.group();      // 键名
                String key = StringUtils.lowerCase(StringUtils.trim(StringUtils.substringBetween(name, "[", "]")));

                // if key start with cmd then means that current line is special
                // command
                if (StringUtils.startsWithIgnoreCase(key, "cmd")) {
                    value = parseCmd(key, data);
                } else {
                    value = StringUtils.trimToEmpty((String) data.get(key));    // 键值
                }

                matcher.appendReplacement(sb, value == null ? "" : value);
            }

            matcher.appendTail(sb);
        } catch (Exception e) {
            LoggerUtils.getLogger().error("组装打印内容异常：" + e.toString());
        }

        return contentToStrList(sb.toString());
    }

    private Map<String, Object> parseFromBean(Object obj) {
        Map<String, Object> map = new HashMap<String, Object>();

        Method[] methArr = obj.getClass().getDeclaredMethods();
        PrtPatternAnno anno = null;
        for (Method method : methArr) {
            if (StringUtils.startsWithIgnoreCase(method.getName(), "get") || StringUtils.startsWithIgnoreCase(method.getName(), "is")) {
                if (method == null || method.getAnnotation(PrtPatternAnno.class) == null) {
                    continue;
                }

                if (method.getReturnType() == List.class) {
                    try {
                        @SuppressWarnings("unchecked")
                        List<Map<String, Object>> listMap = parseList((List<Object>) method.invoke(obj));
                        if (listMap == null) {
                            continue;
                        }

                        anno = method.getAnnotation(PrtPatternAnno.class);
                        map.put(StringUtils.lowerCase(anno.key()), listMap);
                    } catch (Exception e) {
                        LoggerUtils.getLogger().error("组装打印内容 parseFromBean1 异常：" + e.toString());
                    }
                    continue;
                }

                try {
                    anno = method.getAnnotation(PrtPatternAnno.class);
                    Object resultFromInvoke = method.invoke(obj);
                    if (resultFromInvoke instanceof String) {
                        map.put(StringUtils.lowerCase(anno.key()), StringUtils
                                .trimToEmpty((String) resultFromInvoke));

                        // support alias for enhancing compatible
                        if (StringUtils.isNotBlank(anno.alias())) {
                            map.put(StringUtils.lowerCase(anno.alias()),
                                    StringUtils.trimToEmpty((String) resultFromInvoke));
                        }
                    } else if (resultFromInvoke instanceof Boolean) {
                        map.put(StringUtils.lowerCase(anno.key()), (Boolean) resultFromInvoke);
                    } else {
                        map.put(StringUtils.lowerCase(anno.key()), resultFromInvoke);
                    }
                } catch (Exception e) {
                    LoggerUtils.getLogger().error("组装打印内容 parseFromBean2 异常：" + e.toString());
                }
            }
        }

        return map;
    }

    private List<Map<String, Object>> parseList(List<Object> list) {
        if (ListUtils.isEmpty(list)) {
            return null;
        }

        List<Map<String, Object>> resultList = new ArrayList<>();
        Map<String, Object> map;
        for (Object obj : list) {
            map = parseFromBean(obj);
            resultList.add(map);
        }

        return resultList;
    }

    private String parseCmd(String key, Map<String, ?> data) {
        // re-print
        if (StringUtils.startsWithIgnoreCase(key, TemplateEngine.PatternCmd.CMD_RE_PRINT.getCommad())) {
            String mapKey = StringUtils.substringBetween(key, ":", "(");
            Object valueFromIs = data.get(mapKey);
            boolean isVar = false;
            if (valueFromIs == null) {
                return null;
            }
            if (valueFromIs instanceof Boolean) {
                isVar = (Boolean) valueFromIs;
            } else {
                isVar = BooleanUtils.toBoolean(valueFromIs.toString());
            }

            String content = null;
            if (isVar) {
                content = StringUtils.substringBetween(key, "(", ")");
            }
            return content;
        }

        if (StringUtils.startsWithIgnoreCase(key, TemplateEngine.PatternCmd.CMD_LOOP.getCommad())) {
            String mapKey = StringUtils.substringBetween(key, ":", "{");
            String mapKeySecond = StringUtils.substringBetween(key, "cmdloopdsc:", "{");
            String subFormatSecond = StringUtils.substringBetween(key, "pludsclist{", "}");

            if (StringUtils.isBlank(mapKey)) {
                return null;
            }

            Object mapInList = data.get(mapKey);
            Object mapInListDsc = null;
            if (StringUtils.isBlank(mapKeySecond)) {
                LoggerUtils.getLogger().debug("无需展示优惠列表");
            } else {
                mapInListDsc = data.get(mapKeySecond);
            }
            if (mapInList instanceof List<?>) {
                @SuppressWarnings("unchecked")
                List<Map<String, String>> subListMap = (List<Map<String, String>>) mapInList;
                List<Map<String, String>> subListMapDsc = new ArrayList<>();
                if (mapInListDsc != null && mapInListDsc instanceof List<?>) {
                    subListMapDsc = (List<Map<String, String>>) mapInListDsc;
                }

                String subFormat = StringUtils.substringBetween(key, "{", "}");
                if (StringUtils.isBlank(subFormat)) {
                    return null;
                }

                StringBuffer sb = new StringBuffer();
                List<Map<String, String>> dscCodeList = new ArrayList<>();
                for (int i = 0; i < subListMap.size(); i++) {
                    String pluDscName = "";
                    Map<String, String> subMap = subListMap.get(i);
                    listFormat(subMap, subFormat, sb);

                    //遍历每个商品的优惠
                    for (int j = 0; j < subListMapDsc.size(); j ++) {
                        Map<String, String> subMapDsc = subListMapDsc.get(j);
                        String dscPluCode = subMapDsc.get("pludsccode");
                        String pluCode = subMap.get("plucode");
                        LoggerUtils.getLogger().debug("优惠名称：" + subMapDsc.get("pludscname"));
                        //相同优惠名称的只允许遍历一次
                        if (dscPluCode.equals(pluCode) && !(subMapDsc.get("pludscname") == null ? "" : subMapDsc.get("pludscname")).equals(pluDscName) && !dscCodeList.contains(subMapDsc)) {
                            dscCodeList.add(subMapDsc);
                            listFormat(subMapDsc, subFormatSecond, sb);

                            pluDscName = subMapDsc.get("pludscname");
                        }
                    }
                }

                return sb.toString();
            }
        }

        //会员
        if (StringUtils.startsWithIgnoreCase(key, PatternCmd.CMD_VIP.getCommad())) {
            if (!data.containsKey("isvip")) {
                return null;
            }

            Object valueFromIs = data.get("isvip");
            if (!(boolean) valueFromIs) {
                return null;
            }

            StringBuffer sb = new StringBuffer();
            String subFormat = StringUtils.substringAfter(key, PatternCmd.CMD_VIP.getCommad().toLowerCase() + ":");

            Pattern pattern = Pattern.compile("\\((.+?)\\)");
            Matcher matcher = pattern.matcher(subFormat);
            String value;
            while (matcher.find()) {
                String name = matcher.group();
                String subKey = StringUtils.trim(StringUtils.substringBetween(name, "(", ")"));
                value = StringUtils.trimToEmpty((String) data.get(subKey));
                matcher.appendReplacement(sb, value);
            }
            matcher.appendTail(sb);
            sb.append("\r\n");

            return sb.toString();
        }

        return null;
    }

    /**
     * 循环列表
     *
     * @param subMap    待处理map
     * @param subFormat 待打印字符串
     * @param sb        处理后字符串
     */
    private void listFormat(Map<String, String> subMap, String subFormat, StringBuffer sb) {
        try {
            Pattern pattern = Pattern.compile("\\((.+?)\\)");
            Matcher matcher = pattern.matcher(subFormat);
            String value = null;
            while (matcher.find()) {
                String name = matcher.group();  // 键名
                String subKey = StringUtils.trim(StringUtils.substringBetween(name, "(", ")"));
                value = StringUtils.trimToEmpty((String) subMap.get(subKey));          // 键值
                matcher.appendReplacement(sb, value);
            }
            matcher.appendTail(sb);
            sb.append("\r\n");


            //如果模板中项目金额配置的为pluPrice(原价),当有折扣的时候另起一行显示促销折扣
            if (subMap.containsKey("pludsc") && Double.parseDouble(subMap.get("pludsc")) != 0
                    && (subFormat.contains("pluprice)") || subFormat.contains("pluprice "))) {
                sb.append("ﾤ         促销折扣                 -" + subMap.get("pludsc"));
                sb.append("\r\n");
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("组装打印内容异常 listFormat：" + e.toString());
        }
    }

    private List<String> contentToStrList(String content) {
        return new ArrayList<>(Arrays.asList(StringUtils.split(content, "\r\n")));
    }

    public enum PatternCmd {

        CMD_RE_PRINT("cmdBool"),
        CMD_LOOP("cmdLoop"),
        CMD_VIP("cmdVip");

        private String command;

        PatternCmd(String command) {
            this.command = command;
        }

        public String getCommad() {
            return command;
        }
    }
}
