/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.print.printer;

import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.ConditionVariable;
import android.os.RemoteException;
import android.os.SystemClock;
import android.view.WindowManager;

import androidx.appcompat.app.AlertDialog;

import com.boc.aidl.printer.AidlPrinter;
import com.boc.aidl.printer.AidlPrinterListener;
import com.ccb.deviceservice.aidl.printer.Constant;
import com.ccb.deviceservice.aidl.printer.IPrinter;
import com.ccb.deviceservice.aidl.printer.OnPrintListener;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.dialog.HintAliDialog;
import com.hisense.hiaposcmp.hs330r.PrinterSmartHelper;
import com.hisense.hiaposcmp.pos.ccb.ImageTransformer;
import com.hisense.hiaposcmp.pos.print.PrintProvider;
import com.hisense.hiaposcmp.print.PrintTools;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.pos.hs6500.HS650PrintProvider;
import com.hisense.pos.print.IPrintProvider;
import com.hisense.pos.print.boc.BOCDeviceHelper;
import com.hisense.pos.print.ccb.CCBDeviceHelper;
import com.hisense.pos.print.hi98.HI98PrintProvider;
import com.hisense.pos.print.hk316.HI316DeviceHelper;
import com.hisense.pos.spiprinter.SpiPrinter;
import com.hisense.pos.sunmit2.printer.SunmiPrintHelper;
import com.hisense.pos.sunmit2.printer.SunmiPrintProvider;
import com.histonepos.npsdk.bind.PrinterConsts;
import com.histonepos.npsdk.printer.IPrinterService;

import org.apache.commons.lang3.StringUtils;

import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

import static com.hisense.hiaposcmp.tool.LoadSDCard.getBitmap;


/**
 * 打印（调用HI98/联迪A8 驱动）
 *
 * @author xieliyan  2017-09-22 HIPOSAPP-159
 * @author xieliyan  2018-01-18 HIPOSAPP-285 支持调用联迪驱动打印
 * @author dukunbiao(null) 2018-05-11 支持HS650打印
 * @author wangyaogang 2018-11-10 HIPOSAPP-817 支持中行打印
 * @author wangyaogang 2018-12-11 HIPOSAPP-872 优化报表打印后返回主菜单速度
 * @author wangyaogang 2018-12-19 HIPOSAPP-897 更新中行打印接口,修复主控占用问题
 * @author wangyaogang 2019-05-27 HIPOSAPP-1140 调整中行版打印服务逻辑,调用收单行渠道的操作前解绑打印服务,返回结果后重新绑定服务
 * @author wangyaogang 2019-08-01 HIPOSAPP-1236 中行版打印逻辑调整,走纸移入打印完成的回调里
 * @author wangyaogang 2019-11-20 HIAPOSCMP-320 调整打印文本的逻辑,命令只是修改打印属性,其他根据现有属性去打印
 * @author wangyaogang 2019-11-28 HIAPOSCMP-357 【慧收银】小票流水号重复(生成barcode自带内容)，公众号及发票地址二维码未打印(无发票开关)
 * @author wangyaogang 2020-01-03 HIAPOSCMP-464 标准版打印功能开发,316设备
 * @author guolina     2020-05-18 HIAPOSCMP-702 吉麦隆淘鲜达营销方案对接涉及慧收银调整增加支付明细和赠券打印
 */
public class Printer {
    private static ConditionVariable conditionVariable = new ConditionVariable();

    /**
     * 打印（调用HI98驱动）
     *
     * @param drawable 图片
     * @return true 打印成功，false 打印失败
     */
    public static boolean printBitmap(Bitmap drawable) {
        boolean result = false;
        SpiPrinter printer = new SpiPrinter();
        try {
            printer.Printer_init();
            int ret = printer.Printer_Image(drawable, 0);
            switch (ret) {
                case SpiPrinter.PRINTER_OK:
                    result = true;
                    break;
                case SpiPrinter.PRINTER_ERROR:
                    showDialog("打印失败,请重试", drawable, null);
                    break;
                case SpiPrinter.PRINTER_ERROR_NOPAPER:
                    showDialog("打印机缺纸,请添加打印纸后重试", drawable, null);

                    break;
                case SpiPrinter.PRINTER_ERROR_HOT:
                    showDialog("打印机过热,请稍后重试", drawable, null);
                    break;
                case SpiPrinter.PRINTER_ERROR_BUSY:
                    showDialog("打印机繁忙,请稍后重试", drawable, null);
                    break;
                default:
                    CommonTools.toastShort(App.getContext(), "打印异常");
                    break;
            }
            //printer.Printer_Start(); 打印图片不需要调用这行
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("打印初始化失败" + e.getMessage());
        }
        return result;
    }

    /**
     * 打印（调用联迪A8驱动）
     *
     * @param imageData 图片的字节数据
     */

    public static void printBitmap(byte[] imageData) {
        CCBDeviceHelper ccbDeviceHelper = CCBDeviceHelper.getInstance();
        ccbDeviceHelper.lockService();
        IPrinter printer = ccbDeviceHelper.getPrinter();
        int getStatueRet;
        try {
            getStatueRet = printer.getStatus();
            LoggerUtils.getLogger().debug("联迪建行打印机状态:" + getStatueRet);
            switch (getStatueRet) {
                case Constant.ErrorCode.ERROR_PAPERENDED:
                case Constant.ErrorCode.ERROR_PAPERENDING:
                    showDialog("打印机缺纸,请添加打印纸后重试", null, imageData);
                    break;
                case Constant.ErrorCode.ERROR_PAPERJAM:
                    showDialog("卡纸", null, imageData);
                    break;
                case Constant.ErrorCode.ERROR_BUSY:
                    showDialog("打印机处于忙碌状态", null, imageData);
                    break;
                case 0:
                    Bundle format = new Bundle();
                    format.putInt("offset", 0);
                    if (imageData != null) {
                        printer.addImage(format, imageData);
                        printer.feedLine(2);
                        printer.startPrint(new OnPrintListener.Stub() {
                            @Override
                            public void onFinish() throws RemoteException {
                                conditionVariable.open();
                                LoggerUtils.getLogger().debug("联迪建行打印finish");
                            }

                            @Override
                            public void onError(int i) throws RemoteException {
                                conditionVariable.open();
                                LoggerUtils.getLogger().debug("联迪建行打印中状态异常:" + i);
                                String tip;
                                switch (i) {
                                    case Constant.ErrorCode.ERROR_PAPERENDED:
                                    case Constant.ErrorCode.ERROR_PAPERENDING:
                                        tip = "打印机缺纸,请添加打印纸后重试";
                                        break;
                                    case Constant.ErrorCode.ERROR_PAPERJAM:
                                        tip = "卡纸";
                                        break;
                                    case Constant.ErrorCode.ERROR_BUSY:
                                        tip = "打印机处于忙碌状态";
                                        break;
                                    default:
                                        tip = "打印机异常";
                                        break;
                                }
                                showDialog(tip, null, imageData);
                            }
                        });
                        conditionVariable.block(30000);//此方法不调用会引起打印数据不清除
                        conditionVariable.close();
                    }
                    break;
                default:
                    showDialog("打印机异常", null, imageData);
                    break;
            }

            ccbDeviceHelper.unlockService();

        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常_printBitmap" + e.getMessage());
        }
    }

    /**
     * 打印（中行版本）
     *
     * @param bitmap 打印内容
     */
    public static void printBitmapWithBOC(Bitmap bitmap) {
        final AidlPrinter newPrinter = BOCDeviceHelper.getInstance().getNewPrinter();
        try {
            int getStatueRet = newPrinter.getStatus();
            switch (getStatueRet) {
                case 0x00:

                    newPrinter.printBitMap(0, bitmap, 30, new AidlPrinterListener.Stub() {
                        @Override
                        public void onError(int code, String detail) throws RemoteException {
                            LoggerUtils.getLogger().debug("boc打印异常" + code + detail);
                            CommonTools.toastShort(App.getContext(), detail);
                        }

                        @Override
                        public void onFinish() throws RemoteException {
                            // 走紙
                            newPrinter.paperSkip(2);
                            LoggerUtils.getLogger().debug("boc打印完成");
                        }
                    });
                    break;
                case 0x02:
                    showDialog("打印机缺纸,请添加打印纸后重试", bitmap, null);
                    break;
                case 0x06:
                    showDialog("卡纸", bitmap, null);
                    break;
                case 0x04:
                    showDialog("打印机处于忙碌状态", bitmap, null);
                    break;
                default:
                    CommonTools.toastShort(App.getContext(), "打印异常");
                    break;
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常_printBitmapWithBOC " + e.toString());
        }
    }

    /**
     * hk316 打印方法
     *
     * @param bitmap
     */
    public static void printBitmapWithHk316(Bitmap bitmap) {
        final IPrinterService printer = HI316DeviceHelper.getInstance().getPrinter();
        try {
            if (printer == null) {
                showDialog("未获取到打印机,请检查配置");
                return;
            }
            int status = printer.checkStatus();
            SystemClock.sleep(150);
            status = printer.checkStatus();
            String msg;
            switch (status) {
                case PrinterConsts.STATUS_OK:
                case PrinterConsts.STATUS_OK_NEAR_PAPER:
                    msg = "打印机有纸, 可用";
                    printer.beginTranscation();
                    printer.addImage(bitmap);
                    printer.addFeedLine(3);
                    printer.addCut();
                    int transId = printer.endTranscation();
                    printer.commit(transId);
                    break;
                /*case PrinterConsts.STATUS_OK_NEAR_PAPER:
                    msg = "打印机纸将尽";
                    break;*/
                case PrinterConsts.STATUS_ERR_PAPER:
                    msg = "打印机缺纸";
                    showDialogHk316(msg, bitmap);
                    break;
                case PrinterConsts.STATUS_ERR_COVER:
                    msg = "打印机纸仓未关";
                    showDialogHk316(msg, bitmap);
                    break;
                case PrinterConsts.STATUS_ERR_OFFLINE:
                    msg = "打印机离线";
                    showDialogHk316(msg, bitmap);
                    break;
                case PrinterConsts.STATUS_ERR_UNKOWN:
                    msg = "打印机未知错误";
                    showDialogHk316(msg, bitmap);
                    break;
                case PrinterConsts.STATUS_ERR_BUSY:
                    msg = "打印机忙碌";
                    showDialogHk316(msg, bitmap);
                    break;
                default:
                    CommonTools.toastShort(App.getContext(), "打印异常");
                    break;
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常_printBitmapWithBOC " + e.toString());
        }
    }

    /**
     * hk316打印机 重试提示框
     *
     * @param msg
     * @param bitmap
     */
    private static void showDialogHk316(final String msg, final Bitmap bitmap) {
        AndroidSchedulers.mainThread().scheduleDirect(new Runnable() {
            @Override
            public void run() {
                AlertDialog.Builder builder = new AlertDialog.Builder(App.getInstance(), R.style.Base_Theme_AppCompat_Light_Dialog_Alert);
                AlertDialog dialog = builder.setMessage(msg)
                        .setPositiveButton("重试", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                Schedulers.io().scheduleDirect(new Runnable() {
                                    @Override
                                    public void run() {
                                        if (bitmap != null) {
                                            printBitmapWithHk316(bitmap);
                                        }
                                    }
                                });
                            }
                        })
                        .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                LoggerUtils.getLogger().debug("hk316打印取消");
                            }
                        })
                        .create();
                dialog.getWindow().setType(WindowManager.LayoutParams.TYPE_SYSTEM_ALERT);
                dialog.setCanceledOnTouchOutside(false);//点击屏幕不消失
                if (!dialog.isShowing()) {//此时提示框未显示
                    dialog.show();
                }
            }
        });
    }

    /**
     * 对话框
     *
     * @param msg 提示 drawable 打印的图片
     * @author wangyaogang  2018-11-15 HIPOSAPP-817 打印机异常,重试时 调用中行打印方法
     */
    private static void showDialog(final String msg, final Bitmap drawable, final byte[] imageData) {
        AndroidSchedulers.mainThread().scheduleDirect(new Runnable() {
            @Override
            public void run() {
                AlertDialog.Builder builder = new AlertDialog.Builder(App.getInstance(), R.style.Base_Theme_AppCompat_Light_Dialog_Alert);
                AlertDialog dialog = builder.setMessage(msg)
                        .setPositiveButton("重试", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                Schedulers.io().scheduleDirect(new Runnable() {
                                    @Override
                                    public void run() {
                                        if (drawable != null) {
                                            if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
                                                printBitmapWithBOC(drawable);
                                            } else {
                                                printBitmap(drawable);
                                            }
                                        } else if (imageData != null) {
                                            printBitmap(imageData);
                                        }
                                    }
                                });
                            }
                        })
                        .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                LoggerUtils.getLogger().debug("打印取消");
                            }
                        })
                        .create();
                dialog.getWindow().setType(WindowManager.LayoutParams.TYPE_SYSTEM_ALERT);
                dialog.setCanceledOnTouchOutside(false);//点击屏幕不消失
                if (!dialog.isShowing()) {//此时提示框未显示
                    dialog.show();
                }
            }
        });
    }


    /**
     * 文本打印（工行、建行、hs6500）
     *
     * @param printProvider 设备打印服务
     * @param printList     打印内容
     * @param invoiceUrl    二维码地址
     */
    public static void print(IPrintProvider printProvider, List<String> printList, String invoiceUrl) {
        boolean isInit = printProvider.initPrinter(GeneralSettings.INSTANCE.getPrinterRate(), GeneralSettings.INSTANCE.getPrinterPort(),
                GeneralSettings.INSTANCE.getDeviceType().isHS650() ? 0 : 1, GeneralSettings.INSTANCE.getPrinterFreeLine());
        if (!isInit) {
            LoggerUtils.getLogger().debug("打印机配置错误");
            showDialogCCb("打印机配置错误，请联系管理员");
            return;
        }
        int getStatueRet;
        try {
            getStatueRet = printProvider.getStatus();
            switch (getStatueRet) {
                case Constant.ErrorCode.ERROR_PAPERENDED:
                case SpiPrinter.PRINTER_ERROR_NOPAPER:
                case 0x02:
                case 1:
                    LoggerUtils.getLogger().debug("打印机缺纸,请添加打印纸后重试");
                    showDialogCCb("打印机缺纸,请添加打印纸后重试");
                    break;
                case Constant.ErrorCode.ERROR_PAPERJAM:
                case SpiPrinter.PRINTER_ERROR:
                case 0x06:
                    LoggerUtils.getLogger().debug("卡纸");
                    showDialogCCb("卡纸");
                    break;
                case Constant.ErrorCode.ERROR_BUSY:
                case SpiPrinter.PRINTER_ERROR_BUSY:
                case 0x04:
                    LoggerUtils.getLogger().debug("打印机处于忙碌状态");
                    showDialogCCb("打印机处于忙碌状态");
                    break;
                case 0:
                case -2://hs6500纸将用尽
                    LoggerUtils.getLogger().debug("正常进入打印状态");
                    String line;
                    int align = 0;
                    int font = 1;
                    for (int i = 0; i < printList.size(); i++) {
                        line = printList.get(i).trim().replace("\uFEFF", "");   //处理BOM

                        if (StringUtils.equalsIgnoreCase("BarCodeFlag", line)) {
                            line = printList.get(i + 2).trim().replace("\uFEFF", "");
                            if (!StringUtils.isEmpty(line)) {
                                if (printProvider instanceof HI98PrintProvider) {
                                    Bitmap barCode = PrintTools.CreateBarCode(line, 380, 80);
                                    printProvider.addImage(null, barCode, 0, 0, 0);
                                } else {
                                    printProvider.addBarCode(line);
                                }
                                if (CommonUtils.isUniformStyle()) {
                                    printList.set(i + 2, "cmd_ignore");
                                }
                            }
                        } else if (StringUtils.equalsIgnoreCase("hxPayCouponList", line)){
                            line = printList.get(i + 1).trim().replace("\uFEFF", "");//处理BOM
                        } else if (StringUtils.equalsIgnoreCase("QRCodeFlag", line)) {
                            i++;
                            line = printList.get(i).trim().replace("\uFEFF", "");
                            String QRCodeData;
                            if (StringUtils.equalsIgnoreCase("eInvoiceQRCode", line)) {
                                QRCodeData = (invoiceUrl == null ? "" : invoiceUrl);
                            } else if (StringUtils.equalsIgnoreCase("couponQRCode", line)) {
                                i++;
                                line = printList.get(i).trim().replace("\uFEFF", "");
                                QRCodeData = printList.get(i);
                                
                            } else {
                                QRCodeData = printList.get(i);
                            }
                            LoggerUtils.getLogger().debug("打印二维码地址:" + line);
                            if (!StringUtils.isEmpty(QRCodeData)) {

                                if (printProvider instanceof HI98PrintProvider) {
                                    Bitmap qrCode = PrintTools.CreateQRCode(line, 300, 300);
                                    printProvider.addImage(null, qrCode, 0, 0, 0);
                                } else {
                                    printProvider.addQrCode(QRCodeData, 60, 300);
                                }
                            }

                        } else if (line.length() > 14 && StringUtils.equalsIgnoreCase("cmd_print_image", line.substring(0, 15))) {
                            Bitmap bitmap = getBitmap(line);
                            if (bitmap != null) {
                                printProvider.addImage(ImageTransformer.convert1BitBmp(bitmap).toByteArray(), bitmap, 60, bitmap.getWidth(), bitmap.getHeight());
                            }

                        } else {
                            if (StringUtils.equalsIgnoreCase("cmd_centerStart", line)) {// 文字居中
                                align = 1;
                                LoggerUtils.getLogger().debug("打印文字居中" + line);
                            } else if (StringUtils.equalsIgnoreCase("cmd_centerEnd", line)) {
                                LoggerUtils.getLogger().debug("打印文字" + line);
                                align = 0;
                            } else if (StringUtils.equalsIgnoreCase("cmd_textSizeLarge", line)) {
                                LoggerUtils.getLogger().debug("打印文字" + line);
                                font = 2;
                            } else if (StringUtils.equalsIgnoreCase("cmd_textSizeMiddle", line)) {
                                LoggerUtils.getLogger().debug("打印文字" + line);
                                font = 1;
                            } else if (StringUtils.equalsIgnoreCase("cmd_textSizeEnd", line)) {
                                LoggerUtils.getLogger().debug("打印文字" + line);
                                font = 1;
                            } else if (StringUtils.equalsIgnoreCase("cmd_paperCut", line)) {
                                LoggerUtils.getLogger().debug("切纸");
                                printProvider.cutPaper();

                            } else if (StringUtils.equalsIgnoreCase("cmd_feedLine", line)) {            // 屏蔽掉易收银暂不支持的指令
                                //暂不处理
                            } else if (StringUtils.equalsIgnoreCase("cmd_ignore", line)) {            // 屏蔽掉易收银暂不支持的指令
                                //暂不处理
                            } else {
                                printProvider.addText(line, align, font);
                            }

                        }
                    }

                    printProvider.feedLine(4);
                    printProvider.startPrint();

                    break;
                default:
                    showDialogCCb("打印异常");
                    LoggerUtils.getLogger().debug("打印异常" + getStatueRet);
                    break;
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常_printBitmap" + e.getMessage());
        }

    }

    /**
     * 打印条码
     *
     * @param barcode
     * @param mode    0-打印二维码， 其他-打印条码
     * @return
     */
    public static boolean printBarHs650(String barcode, int mode) {

        boolean result = false;
        try {
            HS650PrintProvider hs650PrintProvider = new HS650PrintProvider();
            if (hs650PrintProvider.initPrinter(GeneralSettings.INSTANCE.getPrinterRate(), GeneralSettings.INSTANCE.getPrinterPort(),
                    GeneralSettings.INSTANCE.getDeviceType().isHS650() ? 0 : 1, GeneralSettings.INSTANCE.getPrinterFreeLine())) {
                int status = hs650PrintProvider.getStatus();
                if (status == 1) {
                    LoggerUtils.getLogger().debug("打印机缺纸");
                    HintAliDialog commHintDialog = new HintAliDialog(App.getInstance());
                    commHintDialog.show();
                    commHintDialog.setBasicSytle(HintAliDialog.STYLE_FAILURE);
                    commHintDialog.setTextMainHead("打印机缺纸");
                    commHintDialog.setReason("请联系管理员");
                } else {

                    String currBarcode = barcode.trim().replace("\uFEFF", "");
                    if (mode == 0) {
                        hs650PrintProvider.feedLine(6);
                        hs650PrintProvider.addQrCode(currBarcode, 0, 0);
                        hs650PrintProvider.feedLine(6);
                        hs650PrintProvider.cutPaper();
                        LoggerUtils.getLogger().debug("打印授权码切纸");
                    } else {
                        hs650PrintProvider.addBarCode(currBarcode);
                        hs650PrintProvider.feedLine(6);
                        hs650PrintProvider.cutPaper();
                        LoggerUtils.getLogger().debug("打印条形码切纸");
                    }
                    hs650PrintProvider.closePrinter();
                }
            } else {
                showDialog("打印机配置错误，请联系管理员");
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常---------------------" + e.getMessage());
        }
        return result;
    }

    /**
     * 打印条码 商米T2机器内置打印机
     *
     * @param barcode
     * @param mode    0-打印二维码， 其他-打印条码
     * @return
     */
    public static boolean printBarSunmiT2(String barcode, int mode) {

        boolean result = false;
        try {
            SunmiPrintProvider printProvider = new SunmiPrintProvider();
            if (printProvider.initPrinter(0,0,0,0)) {
                int status = printProvider.getStatus();
                if (status == 1) {
                    LoggerUtils.getLogger().debug("打印机缺纸");
                    HintAliDialog commHintDialog = new HintAliDialog(App.getInstance());
                    commHintDialog.show();
                    commHintDialog.setBasicSytle(HintAliDialog.STYLE_FAILURE);
                    commHintDialog.setTextMainHead("打印机缺纸");
                    commHintDialog.setReason("请联系管理员");
                } else {

                    String currBarcode = barcode.trim().replace("\uFEFF", "");
                    if (mode == 0) {
                        printProvider.feedLine(6);
                        printProvider.addQrCode(currBarcode, 0, 0);
                        printProvider.feedLine(6);
                        printProvider.cutPaper();
                        LoggerUtils.getLogger().debug("打印授权码切纸");
                    } else {
                        printProvider.addBarCode(currBarcode);
                        printProvider.feedLine(6);
                        printProvider.cutPaper();
                        LoggerUtils.getLogger().debug("打印条形码切纸");
                    }
                    printProvider.closePrinter();
                }
            } else {
                showDialog("打印机配置错误，请联系管理员");
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常---------------------" + e.getMessage());
        }
        return result;
    }


    /**
     * 对话框（建行、工行打印异常时提示框）
     *
     * @param msg 提示 drawable 打印的图片
     */
    private static void showDialogCCb(String msg) {
        List<String> printList = LoadSDCard.loadTxtFile(SDCardConstants.PRINTER_BILL_PATH, 1);
        LoggerUtils.getLogger().info("打印重试字符串" + printList.toString());
        AndroidSchedulers.mainThread().scheduleDirect(new Runnable() {
            @Override
            public void run() {
                AlertDialog.Builder builder = new AlertDialog.Builder(App.getInstance(), R.style.Base_Theme_AppCompat_Light_Dialog_Alert);
                AlertDialog dialog = builder.setMessage(msg)
                        .setPositiveButton("重试", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                Schedulers.io().scheduleDirect(new Runnable() {
                                    @Override
                                    public void run() {
                                        try {
                                            if (printList.size() > 0) {
                                                LoggerUtils.getLogger().info("打印重试");
                                                IPrintProvider printProvider = PrintProvider.createPrintProvider();
                                                //todo
                                                print(printProvider, printList, null);
                                            }
                                            LoggerUtils.getLogger().info("打印重试printlist" + printList);
                                        } catch (Exception e) {
                                            LoggerUtils.getLogger().error(e.toString());
                                        }

                                    }
                                });
                            }
                        })
                        .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                LoggerUtils.getLogger().debug("打印取消");
                            }
                        })
                        .create();
                dialog.getWindow().setType(WindowManager.LayoutParams.TYPE_SYSTEM_ALERT);
                dialog.setCanceledOnTouchOutside(false);//点击屏幕不消失
                if (!dialog.isShowing()) {//此时提示框未显示
                    dialog.show();
                }
            }
        });
    }


    /**
     * 对话框
     */
    private static void showDialog(String msg) {

        AndroidSchedulers.mainThread().scheduleDirect(() -> {
            AlertDialog.Builder builder = new AlertDialog.Builder(App.getInstance(), R.style.Base_Theme_AppCompat_Light_Dialog_Alert);
            AlertDialog dialog = builder.setMessage(msg)
                    .setPositiveButton("确定", (dialogInterface, i) -> {

                    })
                    .create();
            dialog.getWindow().setType(WindowManager.LayoutParams.TYPE_SYSTEM_ALERT);
            dialog.setCanceledOnTouchOutside(false);//点击屏幕不消失
            if (!dialog.isShowing()) {//此时提示框未显示
                dialog.show();
            }
        });
    }
    /**
     * HS330R打印
     *
     * @param printList
     */
    public static void printHS330R(List<String> printList) {
        try {
            PrinterSmartHelper printerSmartHelper = PrinterSmartHelper.getInstance();
            printerSmartHelper.initPrinter();
            int status = printerSmartHelper.printStatus();
            if (status == PrinterConsts.STATUS_ERR_PAPER) {
                LoggerUtils.getLogger().debug("打印机缺纸");
                showDialog("打印机缺纸,请联系管理员");
            } else if (status == PrinterConsts.STATUS_ERR_OFFLINE) {
                LoggerUtils.getLogger().debug("打印机离线");
                showDialog("打印机离线,请联系管理员");
            } else {
                String line;
                int pattern = 0;
                printerSmartHelper.beginPrintTrans();
                for (int i = 0; i < printList.size(); i++) {
                    //处理BOM
                    line = printList.get(i).trim().replace("\uFEFF", "");
                    if (StringUtils.equalsIgnoreCase("cmd_centerStart", line)) {
                        // 文字居中
                        pattern = 1;
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("cmd_centerEnd", line)) {
                        // 撤销文字居中
                        pattern = 0;
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("cmd_rightStart", line)) {
                        // 撤销文字居右
                        pattern = 2;
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("BarCodeFlag", line)) {
                        // 打印条形码
                        i++;
                        line = printList.get(i).trim().replace("\uFEFF", "");
                        if (StringUtils.equalsIgnoreCase("cmd_centerStart", line)) {
                            i++;
                            line = printList.get(i).trim().replace("\uFEFF", "");
                        }
                        printerSmartHelper.printBar(line);
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("QRCodeFlag", line)) {
                        // 打印二维码
                        i++;
                        line = printList.get(i).trim().replace("\uFEFF", "");
                        printerSmartHelper.printTextStr("", pattern);
                        printerSmartHelper.printQR(line);
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("cmd_feedLine", line)) {
                        //走纸
                        printerSmartHelper.feedPaper(1);
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("cmd_paperCut", line)) {
                        // 裁纸
                        printerSmartHelper.feedPaper(2);
                        printerSmartHelper.cutPaper();
                        continue;
                    } else if (StringUtils.equalsIgnoreCase("TitleFlagCitySuper", line)) {
                        Bitmap logo = BitmapFactory.decodeResource(App.getAppContext().getResources(), R.mipmap.ic_launcher);
                        printerSmartHelper.printImage(logo);
                        continue;
                    }
                    printerSmartHelper.printTextStr(line, pattern);
                }
                printerSmartHelper.startPrint();
                LoggerUtils.getLogger().debug("执行打印过程");
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常---------------------" + e.getMessage());
        }
    }
}
