/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.printer.impl;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Environment;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.text.TextUtils;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.hs330r.HS330RPrintProvider;
import com.hisense.hiaposcmp.pos.ccb.ImageTransformer;
import com.hisense.hiaposcmp.pos.print.PrintProvider;
import com.hisense.hiaposcmp.print.IPrintService;
import com.hisense.hiaposcmp.print.PrintTools;
import com.hisense.hiaposcmp.print.PrinterHelper;
import com.hisense.hiaposcmp.print.printer.IPOSPrinter;
import com.hisense.hiaposcmp.print.printer.Printer;
import com.hisense.hiaposcmp.tool.LoadSDCard;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.SpUtils;
import com.hisense.pos.hs6500.HS650PrintProvider;
import com.hisense.pos.print.IPrintProvider;
import com.hisense.pos.sunmit2.printer.SunmiPrintProvider;
import com.ums.AppHelper;

import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.TimeUnit;

import static com.hisense.hiaposcmp.tool.LoadSDCard.getBitmap;


/*******************************************************************************
 * 本地打印机
 *
 * @author guanguangjin 2017-02-20
 * @author dengyunpeng  2017-07-17 HIPOSAPP-117 删除包含vnd.android.package-archive代码
 * @author xieliyan     2017-09-18 优化编码规范
 * @author xieliyan     2017-10-30 HIPOSAPP-181 赠券说明格式调整
 * @author xieliyan     2017-10-31 HIPOSAPP-200 电子发票二维码格式调整；调整二维码位置
 * @author xieliyan     2017-10-31 HIPOSAPP-201 使用枚举实现单例模式:更改调用方法
 * @author xieliyan     2018-01-18 HIPOSAPP-285 支持调用联迪驱动打印
 * @author xieliyan     2018-03-29 HIPOSAPP-368 修复跨专柜打印小票，交易类型重复显示的问题
 * @author xieliyan     2018-03-26 HIPOSAPP-362 打印小票增加图片打印功能
 * @author lizhixin     2018-07-30 HIPOSAPP-700 工行打印小票功能支持回调与等待，兼容长短时间差
 * @author lizhixin     2018-08-07 HIPOSAPP-712 支持艾体威尔设备类型
 * @author guolina      2018-08-08 HIPOSAPP-704 修复生成二维码图片时抛出异常,导致APK闪退
 * @author wangyaogang  2018-11-10 HIPOSAPP-817 支持中行打印
 * @author wangyaogang  2018-12-19 HIPOSAPP-897 更新中行打印接口,修复主控占用问题
 * @author guolina      2019-03-21 HIPOSAPP-1056 万科小票打印字段完善
 * @author wangyaogang  2019-06-24 HIPOSAPP-1090 增加礼品券类型
 * @author wangyaogang  2019-06-25 HIPOSAPP-1158 根据配置决定是否打印预销售电子发票二维码
 * @author wangyaogang  2019-11-15 HIPOSAPP-1343 打印换行问题,多行时使用StaticLayout
 * @author wangyaogang  2019-11-28 HIAPOSCMP-357 【慧收银】小票流水号重复(生成barcode自带内容)，公众号及发票地址二维码未打印(无发票开关)
 * @author wangyaogang  2019-11-27 HIAPOSCMP-284 增加右对齐命令
 * @author wangyaogang  2020-01-03 HIAPOSCMP-464 标准版打印功能开发
 * @author guolina      2020-05-18 HIAPOSCMP-702 吉麦隆淘鲜达营销方案对接涉及慧收银调整增加支付明细和赠券打印
 ******************************************************************************/
public class LocalPOSPrinter implements IPOSPrinter {

    public static boolean isPrint = false;
    public static String name = "LOCAL_PRINTER";
    private Object mLock = new Object();

    private int printerTextSize;
    private int paperCutInterval = 5; // 打印间隔，用于撕小票
    private Context context;

    private static Bitmap mPicture;

    private IPrintService.PrtPatternType prtPatternType;

    /**
     * @param printList 打印字符串列表
     * @param context   当前活动对象
     *                  原来的接口定义为List了而不是用自定义对象，T_T ... 退款不能继续扩展，只能将退货标识添加list的首位置，根据退货标识添加退货header
     */
    @Override
    public synchronized void print(List<String> printList, Activity context, IPrintService.PrtPatternType type, String invoiceUrl) {
        if (printList.isEmpty()) {
            return;
        }
        prtPatternType = type;
        this.context = App.getAppContext();
        boolean refund;
        boolean preSales;
        boolean preRefund;

        getConfig();
        List<String> temp = new ArrayList<>();
        refund = printList.get(0).equals(Config.REFUND);
        preSales = printList.get(0).equals(Config.PRESALE);
        preRefund = printList.get(0).equals(Config.PREREFUND);

        if (refund || preSales || preRefund) {
            printList.remove(0);
        }

        //如果打印的printlist中的第一个数为数字，则为打印小票，否则，为打印满赠券
        if (com.hisense.common.util.StringUtils.isNumeric(printList.get(0))) {
            int count = Integer.parseInt(printList.get(0));
            printList.remove(0);

            String cmdShop = "";
            int lineTemp = 0;
            for (String line : printList) {//根据 cmd_paperCut 延时。将原有长图片分割为小图片多次打印
                line = line.trim();
                lineTemp++;
                temp.add(line);

                if (lineTemp > 2) {
                    cmdShop = printList.get(lineTemp - 2);
                }

                if (GeneralSettings.INSTANCE.isNoShopLimits()) {
                    if (StringUtils.equalsIgnoreCase("cmd_shopCode", line)) {
                        printList.set(lineTemp, "收款员:" + Config.user_code);
                        temp.remove(temp.size() - 1);
                    }
                } else {
                    if (StringUtils.equalsIgnoreCase("cmd_shopCode", line)) {
                        temp.remove(temp.size() - 1);
                    }
                }

                if (StringUtils.equalsIgnoreCase("cmd_paperCut", line)) {
                    // cmd_shop 用于控制跨专柜的商品打印的联数
                    if (StringUtils.equalsIgnoreCase("cmd_shop", cmdShop)) {
                        for (int i = 0; i < count; i++) {
                            temp.remove(temp.size() - 2);
                            printWithInterval(temp, paperCutInterval, refund, preSales, preRefund, invoiceUrl);
                            //多个专柜情况下，交易类型只添加一遍
                            refund = false;
                            preSales = false;
                            preRefund = false;
                        }
                        temp.clear();
                    } else {
                        printWithInterval(temp, paperCutInterval, refund, preSales, preRefund, invoiceUrl);
                        temp.clear();
                    }
                }
            }
        } else {
            for (String line : printList) {//根据 cmd_paperCut 延时。将原有长图片分割为小图片多次打印
                temp.add(line);
                if (StringUtils.equalsIgnoreCase("cmd_paperCut", line)) {
                    printWithInterval(temp, paperCutInterval, refund, preSales, preRefund, invoiceUrl);
                    temp.clear();
                }
            }
        }
    }

    /**
     * 打印
     *
     * @param printList  打印内容
     * @param interval   打印间隔
     * @param isRefund   交易类型是否为退货
     * @param isPreSales 交易类型是否为预销售
     * @param preRefund  交易类型是否为预销售退货
     * @author guanguagnjin 2017-03-02
     * @author xieliyan     2017-09-01  HIPOSAPP-133 判断是否启用打印发票，若未启用，将打印二维码命令去除
     * @author xieliyan     2017-09-15  HIPOSAPP-133 判断是否启用打印发票，若未启用，将打印二维码命令去除,采用倒序查询方式
     * @author xieliyan     2017-09-15  HIPOSAPP-159 判断设备类型，调用不同打印方式
     * @author guolina      2018-07-06  HIPOSAPP-650 解决第三联无头部“退货小票”，优化跨专柜打印小票，交易类型重复显示的问题
     */
    private void printWithInterval(List<String> printList, int interval, boolean isRefund, boolean isPreSales, boolean preRefund, String invoiceUrl) {
        if (preRefund) {
            if (!(printList.contains("预销售退款小票"))) {
                addRefundHeader("预销售退款小票", printList);
            }
        } else if (isRefund) {
            if (!(printList.contains("退款小票"))) {
                addRefundHeader("退款小票", printList);
            }
        } else if (isPreSales) {
            if (!(printList.contains("预销售"))) {
                addRefundHeader("预销售", printList);
            }
        }
        //易收银有销售和预销售发票打印开关, 慧收银默认都打印
        if (!CommonUtils.isUniformStyle()) {
            if (!GeneralSettings.INSTANCE.isPrintInvoice()
                    || (isPreSales && !GeneralSettings.INSTANCE.isPreSalePrintInvoice())
                    || (preRefund && !GeneralSettings.INSTANCE.isPreSalePrintInvoice())) {

                for (int i = printList.size() - 1; i >= 0; i--) {
                    if (printList.get(i).equals("eInvoiceQRCode")) {
                        printList.remove(i);
                    }
                    if (printList.get(i).equals("QRCodeFlag") && !printList.get(i + 1).equals("couponQRCode")) {
                        printList.remove(i);
                    }
                }

            }
        }

        try {
            String picPath = toPicture(printList, invoiceUrl);
            if (Config.isHk316() || Config.isHt616()) {
                if (isPrint) {
                    LoggerUtils.getLogger().debug("准备hk316打印");
                    Printer.printBitmapWithHk316(mPicture);
                }
                LoadSDCard.saveTxtFile(SDCardConstants.PRINTER_BILL_PATH, printList, SDCardConstants.SAVE_TXT_FILE_TAG);
            } else if (GeneralSettings.INSTANCE.getDeviceType().isHI98()) {
                //HI98使用这种方式打印二维码是黑色的，暂时采用原图片形式
//                IPrintProvider printProvider = PrintProvider.createPrintProvider();
//                Printer.print(printProvider, printList, invoiceUrl);
                Printer.printBitmap(mPicture);
            }
            //hs650系列、工行、建行通道均使用打印文本格式
            else if (GeneralSettings.INSTANCE.getPaymentChannel().isICBC()
                    || GeneralSettings.INSTANCE.getDeviceType().isHSX()
                    || GeneralSettings.INSTANCE.getDeviceType().isSunmiT2()
                    || Config.isHS330R()) {
                LoggerUtils.getLogger().info("HS330R:" + Config.isHS330R());
                for (int i = 0; i < printList.size(); i++) {
                    if (StringUtils.equalsIgnoreCase("QRCodeFlag", printList.get(i).trim())) {
                        String QRCodeData;
                        if (StringUtils.equalsIgnoreCase("eInvoiceQRCode", printList.get(i + 1).trim().replace("\uFEFF", ""))) {
                            QRCodeData = (invoiceUrl == null ? "" : invoiceUrl);
                            printList.set(i + 1, QRCodeData);
                        } else if (StringUtils.equalsIgnoreCase("couponQRCode", printList.get(i + 1).trim().replace("\uFEFF", ""))) {

                            QRCodeData = printList.get(i + 2).trim().replace("\uFEFF", "");
                            //printList.set(i + 1, QRCodeData);

                        }
                    }
                }
                LoggerUtils.getLogger().info("打印字符串保存:" + printList.toString());
                //保存成文本文件，用于缺纸重试
                LoadSDCard.saveTxtFile(SDCardConstants.PRINTER_BILL_PATH, printList, SDCardConstants.SAVE_TXT_FILE_TAG);
                //工行、建行打印小票
                IPrintProvider printProvider = PrintProvider.createPrintProvider();
                if ((printProvider instanceof HS650PrintProvider) || (printProvider instanceof SunmiPrintProvider)) {
                    if (isPrint) {
//                        printList=Arrays.asList("[cmd_centerStart, 欢迎光临亿家汇好超市, 2020-01-06 18:18:22, ----------------------------------------------, cmd_centerEnd, 编码/售价  品名/优惠价 数量    优惠额      合计, 33260002     2号购物袋, 0.20       0.20     1.000     0.00      0.20, ----------------------------------------------, 总数量：1, 总应收：0.20, 总实收:0.20, 总优惠：0.00, 会员号:, ----------------------------------------------, 支付方式   支付金额     支付号       商户优惠, 微信人脸    0.20  20200106181836054498, ----------------------------------------------, cmd_centerStart, 单据号：202001061818191246, 门店：05 款台号：0032 上机号:009503, **********亿家汇好用心为您服务!***********, 客服电话:0991-3112111, 监督热线:0996-2048923, 请妥善保管好小票,商品如有质量问题 ,, 七日内凭此票进行退换货,30日内凭此票, 开具发票.欢迎您办理会员卡,优惠更多!, cmd_centerEnd, cmd_paperCut]");
//                        LoggerUtils.getLogger().info("测试字符串保存:" + printList.toString());
                        Printer.print(printProvider, printList, invoiceUrl);
                    }
                    return;
                }
                if (printProvider instanceof HS330RPrintProvider) {
                    Printer.printHS330R(printList);
                    return;
                }
                Printer.print(printProvider, printList, invoiceUrl);
            } else if (GeneralSettings.INSTANCE.getDeviceType().isLANDI()) {

                if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS()) {
                    LoggerUtils.getLogger().debug("----调用银联打印接口----");
                    AppHelper.callSyncPrint(context, picPath);
                } else if (GeneralSettings.INSTANCE.getPaymentChannel().isBOC()) {
                    LoggerUtils.getLogger().debug("准备boc打印");
                    Printer.printBitmapWithBOC(mPicture);
                } else if (GeneralSettings.INSTANCE.getPaymentChannel().isCCB()) {
                    Printer.printBitmap(ImageTransformer.convert1BitBmp(mPicture).toByteArray());
                }
            }

            TimeUnit.SECONDS.sleep(interval);

        } catch (Exception e) {
            LoggerUtils.getLogger().debug("打印异常：" + e.toString());
        }
    }


    private void addRefundHeader(String header, List<String> printList) {
        printList.add(0, "cmd_centerStart");
        printList.add(1, header);
        printList.add(2, "cmd_centerEnd");
    }

    /**
     * 获取打印机配置信息
     */
    private void getConfig() {
        String interval = (String) SpUtils.get(Config.PAPER_CURT_INTERVAL, "5");
        paperCutInterval = Integer.parseInt(interval);

        String textSize = GeneralSettings.INSTANCE.getPrinterTextSize();
        if ("small".equals(textSize)) {
            printerTextSize = 22;
        } else if ("middle".equals(textSize)) {
            printerTextSize = 25;
        } else if ("big".equals(textSize)) {
            printerTextSize = 28;
        }
    }

    /**
     * 将待打印的内容转换为图片并返回图片路径
     *
     * @param printList 待打印字符串列表
     * @param httpAdr   http地址
     * @return 图片路径
     * @author caokai   2017-02-01
     * @author xieliyan 2017-08-31   HIPOSAPP-133 二维码打印
     * @author xieliyan 2017-10-30   HIPOSAPP-181 赠券说明格式调整
     * @author xieliyan 2017-10-31   HIPOSAPP-200 电子发票二维码格式调整；调整二维码位置
     */
    private String toPicture(List<String> printList, String httpAdr) {
        LoggerUtils.getLogger().debug("begin to print,convert to picture");

        int textSize = printerTextSize;      // 小：22， 中：25， 大：28
        int lineHeight = textSize + 1;          // 小：23， 中：26， 大：29
        int cmdLineNum = 0;                    // 打印内容中命令的行数
        int picWidth = 380;                    // 生成打印图像的宽度（px）
        if (Config.isHk316()) {
            picWidth = 520;                    // 生成打印图像的宽度（px）
        }
        int picHeight;                         // 生成打印图像的高度（px）
        int baseLineX = 0;                     // 图像的水平方向绘图基线
        int baseLineY = 25;                    // 图像的竖直方向上绘图基线
        String couponNo;
        String picPath;                 // 生成图像的本地存储路径

        int manyLines = 0;//需要换行的数目
        TextPaint paint = new TextPaint();
        paint.setColor(Color.BLACK);
        paint.setStyle(Paint.Style.FILL);
        paint.setTextSize(textSize);
        paint.setTextAlign(Paint.Align.LEFT);
        /**
         * 统计打印字符串列表中命令的个数
         */
        for (int i = 0; i < printList.size(); i++) {
            boolean hasFound = false;
            for (int j = 0; j < printCMD.length; j++) {
                if (StringUtils.equalsIgnoreCase(printList.get(i), printCMD[j]) || (printList.get(i).length() > 14 && StringUtils.equalsIgnoreCase(printList.get(i).substring(0, 15), "cmd_print_image"))) {
                    hasFound = true;
                    if (StringUtils.equalsIgnoreCase(printList.get(i), "cmd_paperCut")) {
                        cmdLineNum = cmdLineNum - 2;
                    } else if (StringUtils.equalsIgnoreCase(printList.get(i), "BarCodeFlag")) {
                        cmdLineNum = cmdLineNum - 4;
                    } else if (StringUtils.equalsIgnoreCase(printList.get(i), "eInvoiceQRCode")) {
                        cmdLineNum = cmdLineNum - 14;
                    } else if (StringUtils.equalsIgnoreCase(printList.get(i), "couponQRCode")) {
                        cmdLineNum = cmdLineNum - 14;
                    } else if (StringUtils.equalsIgnoreCase(printList.get(i), "QRCodeFlag")) {
                        cmdLineNum = cmdLineNum - 14;
                    } else if (printList.get(i).length() > 14 && StringUtils.equalsIgnoreCase(printList.get(i).substring(0, 15), "cmd_print_image")) {
                        if (getBitmap(printList.get(i)) != null) {
                            cmdLineNum = cmdLineNum - getBitmap(printList.get(i)).getHeight() / 20;
                        }
                    } else {
                        //一般的命令占一行,在size() 中
                        cmdLineNum++;
                    }
                    break;
                }
            }
            //如果不是命令,且不是分割线,内容需要多行展示,重新计算总行数,因为size()中包含,所以需要再减1
            if (!hasFound) {
                String line = printList.get(i);
                int w = PrinterHelper.getTextWidth(paint, line);
                if (!com.hisense.common.util.StringUtils.isCutLine(line) && w > picWidth) {
                    int ceil = (int) Math.ceil(1.0 * w / picWidth);
                    manyLines += ceil - 1;
                }
            }
        }
        picHeight = lineHeight * (printList.size() - cmdLineNum + manyLines);

        /*
         * 创建位图对象，画布对象，画笔对象并设置它们
         */

        mPicture = Bitmap.createBitmap(picWidth, picHeight,
                Bitmap.Config.ARGB_8888);


        Canvas canvas = new Canvas(mPicture);
        canvas.drawColor(Color.WHITE);

        /*
         * 遍历打印字符串列表，并解析遇到的打印指令来控制画笔绘图
         */
        String line;

        int lineSize = printList.size();

        for (int i = 0; i < lineSize; i++) {
            line = printList.get(i).trim().replace("\uFEFF", "");//处理BOM

            /**
             * 匹配自定义的指令
             * 当匹配到自定义的指令时，根据不同的指令来设置下次绘图的样式。
             * 注意：有部分指令目前还不支持，为了避免将指令打印出来，所以暂时留空
             //             */

            if (StringUtils.equalsIgnoreCase("cmd_centerStart", line)) {            // 文字居中
                paint.setTextAlign(Paint.Align.CENTER);
                baseLineX = mPicture.getWidth() / 2;
            } else if (StringUtils.equalsIgnoreCase("cmd_centerEnd", line)) {           // 撤销文字居中
                paint.setTextAlign(Paint.Align.LEFT);
                baseLineX = 0;
            } else if (StringUtils.equalsIgnoreCase("cmd_alignRightStart", line)) {           // 文字居右
                paint.setTextAlign(Paint.Align.RIGHT);
                baseLineX = mPicture.getWidth();
            } else if (StringUtils.equalsIgnoreCase("cmd_alignRightEnd", line)) {           // 撤销文字居右
                paint.setTextAlign(Paint.Align.LEFT);
                baseLineX = 0;
            } else if (StringUtils.equalsIgnoreCase("cmd_paperCut", line)) {            // 裁纸
                baseLineY += lineHeight * 2;
            } else if (StringUtils.equalsIgnoreCase("cmd_feedLine", line)) {            // 屏蔽掉易收银暂不支持的指令

            } else if (StringUtils.equalsIgnoreCase("BarCodeFlag", line)) {            // 在后两行打印条形码
                couponNo = printList.get(i + 2);
                LoggerUtils.getLogger().debug("打印BarCodeFlag:" + couponNo);
                Bitmap barCode = PrintTools.CreateBarCode(couponNo, picWidth, lineHeight * 3);
                if (barCode != null) {
                    canvas.drawBitmap(barCode, 10, baseLineY, paint);
                    baseLineY += lineHeight * 4;
                } else {
                    LoggerUtils.getLogger().debug("barCode为null,不打印");
                }

            } else if (StringUtils.equalsIgnoreCase("cmd_textSizeLarge", line)) {          // 新行
                paint.setTextSize(26);
            } else if (StringUtils.equalsIgnoreCase("cmd_textSizeMiddle", line)) {          // 新行
                paint.setTextSize(22);
            } else if (StringUtils.equalsIgnoreCase("cmd_textSizeEnd", line)) {          // 新行
                paint.setTextSize(printerTextSize);
            } else if (StringUtils.equalsIgnoreCase("hxPayCouponList", line)) {
                line = printList.get(i + 1).trim().replace("\uFEFF", "");//处理BOM
            } else if (StringUtils.equalsIgnoreCase("QRCodeFlag", line)) {
                line = printList.get(i + 1).trim().replace("\uFEFF", "");//处理BOM
                i++;

                String QRCodeData;
                if (StringUtils.equalsIgnoreCase("eInvoiceQRCode", line)) {
                    QRCodeData = (httpAdr == null ? "" : httpAdr);
                } else if (StringUtils.equalsIgnoreCase("couponQRCode", line)) {
                    QRCodeData = printList.get(i + 1).trim().replace("\uFEFF", "");

                    i++;
                } else {
                    QRCodeData = printList.get(i);
                }

                if (!TextUtils.isEmpty(QRCodeData)) {
                    //当成功获取到二维码内容时才画上去二维码图
                    LoggerUtils.getLogger().debug("二维码信息:" + QRCodeData);

                    int QRCodeLength = 300;
                    Bitmap QRCode = PrintTools.CreateQRCode(QRCodeData, QRCodeLength, QRCodeLength);
                    canvas.translate(50, 0);
                    canvas.drawBitmap(QRCode, 0, baseLineY, paint);
                    canvas.translate(-50, 0);
                    baseLineY += QRCodeLength;
                } else {
                    LoggerUtils.getLogger().debug("二维码信息为空，所以打印小票时不能画上二维码");
                }
            } else if (line.length() > 14 && StringUtils.equalsIgnoreCase("cmd_print_image", line.substring(0, 15))) {
                Bitmap bitmap = getBitmap(line);
                if (bitmap != null) {
                    int width = bitmap.getWidth();
                    int left = (picWidth - width) / 2;
                    canvas.drawBitmap(bitmap, left, baseLineY, paint);
                    baseLineY += bitmap.getHeight() * 1.5;
                }

            } else {
                //分割线或者单行内容,可以直接drawText
                int w = PrinterHelper.getTextWidth(paint, line);
                if (com.hisense.common.util.StringUtils.isCutLine(line) || w <= picWidth) {
                    canvas.drawText(line, baseLineX, baseLineY, paint);
                    baseLineY += lineHeight;
                } else {
                    /**
                     * 多行内容则使用StaticLayout,需要先平移canvas到下次画的位置,画完之后必须将
                     * canvas复原,即反向平移,因为和drawText不同,增加的高度行数额外加1
                     */
                    baseLineY -= lineHeight;
                    StaticLayout layout = new StaticLayout(line, paint, canvas.getWidth(), Layout.Alignment.ALIGN_NORMAL, 0.9F, 0.0F, false);
                    canvas.translate(baseLineX, baseLineY);
                    layout.draw(canvas);

                    canvas.translate(-baseLineX, -baseLineY);
                    baseLineY += (layout.getLineCount() + 1) * lineHeight;
                }
            }
        }


        canvas.save();
        canvas.restore();

        String name = "/printPicture" + Math.random();
        picPath = Environment.getExternalStorageDirectory() + name + ".png";
        FileOutputStream out = null;
        try {
            out = new FileOutputStream(new File(picPath));
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("生成图片文件:" + e.toString());
        }

        mPicture.compress(Bitmap.CompressFormat.PNG, 100, out);

        delayDelete(picPath, 1000 * 60);

        return picPath;
    }


    /**
     * 延迟删除打印图片
     *
     * @param path  图片路径
     * @param delay 延迟时间
     */
    private void delayDelete(final String path, int delay) {
        final Timer timer = new Timer();
        timer.schedule(new TimerTask() {

            @Override
            public void run() {
                File file = new File(path);
                if (file.exists() && file.isFile()) {
                    if (file.delete()) {
                        LoggerUtils.getLogger().debug("删除打印图片文件成功");
                    } else {
                        LoggerUtils.getLogger().debug("删除打印图片文件失败");
                    }
                } else {
                    LoggerUtils.getLogger().debug("待删除的打印图片文件不存在");
                }
                timer.cancel();
            }
        }, delay);
    }


}
