/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.print.repository;

import android.content.Context;
import android.database.Cursor;

import com.hisense.database.sqlite.QueryCallback;
import com.hisense.database.sqlite.SQLiteRepository;
import com.hisense.hiaposcmp.db.PadPosDatabase;
import com.hisense.hipos.template.models.PrintImage;

import java.util.ArrayList;
import java.util.List;

/**
 * 打印图片资源库
 *
 * @author xieliyan 2018-03-26
 * @author zhangsai 2019-06-17 HIPOSAPP-1174 使用 PadPosDatabase 初始化 SQLite 数据库
 */
public class PrintImageRepository extends SQLiteRepository {

    /**
     * 初始化一个 {@code PrintImageRepository} 的实例。
     *
     * @param context Context
     */
    public PrintImageRepository(Context context) {
        super(new PadPosDatabase(context));
    }

    /**
     * 保存图片信息。
     *
     * @param name      图片名称
     * @param timeStamp 图片时间戳
     */
    public void save(String name, String timeStamp) {
        String sql = "REPLACE INTO tPrintImage(Name,TimeStamp,Value) VALUES (?,?,?) ";
        this.execute(sql, name, timeStamp);
    }

    /**
     * 删除图片信息。
     *
     * @param name 图片名称
     */
    public void delete(String name) {
        String sql = "DELETE FROM tPrintImage WHERE Name= ?";
        this.execute(sql, name);
    }

    /**
     * 查询图片信息。
     */
    public List<PrintImage> select() {
        final List<PrintImage> images = new ArrayList<>();
        try {
            String sql = "SELECT Name,TimeStamp FROM tPrintImage";
            this.executeCursor(sql, new QueryCallback() {
                @Override
                public void onSuccess(Cursor cursor) {
                    if (cursor != null) {

                        do {
                            String name = cursor.getString(0);
                            String timeStamp = cursor.getString(1);
                            PrintImage image = new PrintImage(name, Integer.parseInt(timeStamp));
                            images.add(image);
                        } while (cursor.moveToNext());
                    }
                }
            });
        } catch (Exception e) {

        } finally {

        }
        return images;
    }

}
