/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.repository;

import android.content.Context;

import com.hisense.database.sqlite.SQLiteRepository;
import com.hisense.hiaposcmp.db.PadPosDatabase;
import com.hisense.hipos.template.models.PrintTemplate;
import com.hisense.hipos.template.models.PrintTemplateType;

/**
 * 打印模板资源库。
 *
 * @author zhangsai 2018-07-01
 * @author zhangsai 2019-06-17 HIPOSAPP-1174 使用 PadPosDatabase 初始化 SQLite 数据库
 */
public class PrintTemplateRepository extends SQLiteRepository {

    /**
     * 初始化一个 {@code PrintTemplateRepository} 的实例。
     *
     * @param context Context
     */
    public PrintTemplateRepository(Context context) {
        super(new PadPosDatabase(context));
    }


    /**
     * 返回指定 {@code type} 模板类型的打印模板。
     *
     * @param type 模板类型
     * @return 打印模板
     */
    public String getTemplate(PrintTemplateType type) {
        String sql = "SELECT Content FROM tPrnTemplate WHERE Type = ? ";
        return this.executeScalar(sql, String.valueOf(type.value()));
    }

    /**
     * 返回指定 {@code type} 模板类型的本地版本号。
     *
     * @param type 模板类型
     * @return 模板的本地版本号
     */
    public int getVersion(PrintTemplateType type) {
        String sql = "SELECT Version FROM tPrnTemplate WHERE Type = ? ";
        String value = this.executeScalar(sql, String.valueOf(type.value()));
        return this.getInteger(value, 0);
    }

    /**
     * 保存打印模板。
     *
     * @param template 打印模板
     */
    public void save(PrintTemplate template) {
        String sql = "REPLACE INTO tPrnTemplate (Type, Content, Version) VALUES (?, ?, ?)";
        this.execute(sql, String.valueOf(template.templateType().value()),
                template.content(),
                String.valueOf(template.version()));
    }
}
