/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.print.services;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.widget.Toast;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.print.repository.PrintImageRepository;
import com.hisense.hipos.template.models.PrintImage;
import com.hisense.hipos.webapi.images.ImageService;
import com.hisense.webapi.WebApiException;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;

/**
 * 打印图片更新
 *
 * @author xieliyan 2018-03-26 HIPOSAPP-362 打印小票增加图片打印功能
 */

public class PrintImageHelper {

    /**
     * 检查图片是否存在更新。
     *
     * @param activity 当前activity
     */
    public static void checkUpdate(final Activity activity) {

        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    List<PrintImage> requestImages = new PrintImageRepository(activity).select();
                    List<PrintImage> resultImages = ImageService.checkUpdate(requestImages);

                    if (resultImages == null) {
                        return;
                    }
                    for (PrintImage image : resultImages) {
                        String servicePath = "print-images" + "/" + image.name();

                        String localName = image.name().substring(0, image.name().indexOf("."));
                        String localPath = Environment.getExternalStorageDirectory() + "/" + localName + ".bmp";
                        new PrintImageRepository(activity).delete(image.name());
                        File file = new File(localPath);
                        if (file.exists()) {
                            file.delete();
                        }
                        if (image.state() == 1) {
                            new PrintImageRepository(activity).save(image.name(), String.valueOf(image.timeStamp()));
                            String mUrl = Config.base_url + "/" + servicePath;
                            localName = image.name().substring(0, image.name().indexOf("."));
                            localPath = Environment.getExternalStorageDirectory() + "/" + localName + ".bmp";
                            Bitmap bitmap = downloadImages(mUrl);

                            FileOutputStream out;
                            try {
                                out = new FileOutputStream(new File(localPath));
                                bitmap.compress(Bitmap.CompressFormat.JPEG, 100, out);
                                out.flush();
                                out.close();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }
                    }

                } catch (final WebApiException ex) {
                    activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(App.getContext(), ex.getMessage(), Toast.LENGTH_SHORT).show();
                        }
                    });
                }
            }
        }).start();
    }

    /**
     * 下载服务器对应文件夹的图片
     *
     * @param mUrl 服务地址
     */
    public static Bitmap downloadImages(String mUrl) {
        Bitmap bitmap = null;
        try {
            URL url = new URL(mUrl);
            HttpURLConnection httpURLConnection = (HttpURLConnection) url.openConnection();
            httpURLConnection.setDoInput(true);
            httpURLConnection.connect();
            InputStream inputStream = httpURLConnection.getInputStream();
            bitmap = BitmapFactory.decodeStream(inputStream);
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("下载图片异常" + e.toString());
        } finally {

        }
        return bitmap;
    }
}
