/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.print.services;

import android.app.Activity;
import android.util.Log;
import android.widget.Toast;

import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.print.repository.PrintTemplateRepository;
import com.hisense.hipos.template.models.PrintTemplate;
import com.hisense.hipos.template.models.PrintTemplateType;
import com.hisense.hipos.webapi.template.TemplateService;
import com.hisense.webapi.Callback;

import java.util.Locale;

/**
 * 打印模板工具类。
 *
 * @author zhangsai    2017-10-23 HIPOSAPP-189
 * @author xieliyan    2017-10-31 HIPOSAPP-201  使用枚举实现单例模式:更改调用方法
 * @author zhangsai    2018-07-01 HIPOSAPP-612  统一使用 SQLite 保存模板及其版本号
 * @author zhangsai    2018-07-18 HIPOSAPP-675  仅在模板需要更新时打印日志
 * @author guolina     2019-04-03 HIPOSAPP-1073 修复新安装应用打印日志时闪退
 * @author zhangsai    2019-06-17 HIPOSAPP-1174 在一个线程中更新打印模板，避免同时启动多个线程导致多次调用 SQLite 数据库 onUpgrade() 升级
 * @author wangyaogang 2019-06-24 HIPOSAPP-1090 增加检查礼品券模板更新
 */
public class PrintTemplateService {

    /**
     * 检查模板是否存在更新。
     *
     * @param activity Activity
     */
    public static void checkUpdate(final Activity activity) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                PrintTemplateRepository repository = new PrintTemplateRepository(activity);
                checkUpdate(activity, repository, PrintTemplateType.Receipt);
                checkUpdate(activity, repository, PrintTemplateType.Coupon);
                checkUpdate(activity, repository, PrintTemplateType.GiftCoupon);
            }
        }).start();
    }

    /**
     * 检查指定 {@code type} 类型的模板是否存在更新。
     *
     * @param activity Activity
     * @param type     打印模板类型
     */
    public static void checkUpdate(final Activity activity, final PrintTemplateType type) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                PrintTemplateRepository repository = new PrintTemplateRepository(activity);
                checkUpdate(activity, repository, type);
            }
        }).start();
    }

    /**
     * 检查指定 {@code type} 类型的模板是否存在更新。
     *
     * @param activity   Activity
     * @param repository 打印模板资源库
     * @param type       打印模板类型
     */
    private static void checkUpdate(final Activity activity, final PrintTemplateRepository repository,
                                    final PrintTemplateType type) {
        checkUpdate(activity, repository, type, new Callback<String>() {
            @Override
            public void onSuccess(final String s) {
                activity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(App.getContext(), s, Toast.LENGTH_SHORT).show();
                    }
                });
            }

            @Override
            public void onError(final Throwable throwable) {
                activity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(App.getContext(), throwable.getMessage(), Toast.LENGTH_SHORT).show();
                    }
                });
            }
        });
    }

    /**
     * 检查指定 {@code type} 类型的模板是否存在更新。
     *
     * @param activity   Activity
     * @param repository 打印模板资源库
     * @param type       打印模板类型
     * @param callback   检查模板更新的回调方法
     */
    private static void checkUpdate(final Activity activity, PrintTemplateRepository repository,
                                    final PrintTemplateType type, Callback<String> callback) {
        try {
            int localVersion = repository.getVersion(type);

            String templateName = activity.getString(R.string.coupon);
            if (type == PrintTemplateType.Receipt) {
                templateName = activity.getString(R.string.receipt);
            } else if (type == PrintTemplateType.GiftCoupon) {
                templateName = "礼品券";
            }

            PrintTemplate template = TemplateService.checkUpdate(type, localVersion);
            if (template != null) {
                Log.d("PrintTemplateService", String.format(Locale.CHINESE,
                        "更新%s打印模板，Local Version: %d, Server Version: %d",
                        templateName, localVersion, template.version()));

                repository.save(template);

                final String message = String.format(Locale.CHINESE, "%s模板更新完成", templateName);
                callback.onSuccess(message);
            }
        } catch (Exception ex) {
            callback.onError(ex);
        }
    }
}
