/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.service.ali;

import android.os.Environment;

import com.alibaba.fastjson.JSONObject;
import com.hisense.common.boot.GlobalCallback;
import com.hisense.common.common.utils.Signature;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.common.util.WebUtils;
import com.hisense.hiaposcmp.constant.pay.AliConstants;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.ConnectException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 支付宝直连
 *
 * @author xieliyan 2018-04-28
 * @author lizhixin 2018-04-28 HIPOSAPP-426 支持支付宝人脸识别
 */
public class AliService {

    /**
     * 阿里人脸初始化
     *
     * @param callback payHandler
     * @param metaInfo metaInfo
     */
    public static void smilePay(String metaInfo, GlobalCallback<String> callback) {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA);//设置日期格式
        String date = df.format(new Date());// new Date()为获取当前系统时间，也可使用当前时间戳

        final Map<String, String> requestAliParams = new HashMap<>();
        requestAliParams.put("app_id", AliConstants.ali_pay_app_id());
        requestAliParams.put("method", AliConstants.ALI_FACE_PAY_INITIALIZE);
        requestAliParams.put("format", AliConstants.ALI_PAY_FORMAT);
        requestAliParams.put("charset", AliConstants.ALI_PAY_CHARSET);
        requestAliParams.put("sign_type", AliConstants.ALI_PAY_SIGN_TYPE);
        requestAliParams.put("timestamp", date);
        requestAliParams.put("version", AliConstants.ALI_PAY_VERSION);

        if (!StringUtils.isEmpty(AliConstants.ali_pay_third_token())) {
            requestAliParams.put("app_auth_token", AliConstants.ali_pay_third_token());
        }

        JSONObject biz = new JSONObject();
        biz.put("zimmetainfo", metaInfo);
        requestAliParams.put("biz_content", biz.toString());

        String sign = Signature.rsa256Sign(requestAliParams, AliConstants.ali_pay_app_private_key(), AliConstants.ALI_PAY_CHARSET);
        if (null == sign) {
            callback.onError("初始化人脸签名为空，请检查私钥配置");
        }

        requestAliParams.put("sign", sign);

        LoggerUtils.getLogger().debug("支付宝人脸支付请求参数");
        writeLog("支付宝人脸支付请求参数： " + requestAliParams.toString());

        ThreadFactoryUtils.getExecutorService("pay-%d").execute(() -> {
            String strResponseParams;
            try {
                strResponseParams = doPost(requestAliParams, AliConstants.ALI_PAY_URL);
                LoggerUtils.getLogger().debug("支付宝人脸支付返回参数");
                writeLog("AlipayService before verify request : " + strResponseParams);

                if (strResponseParams == null) {
                    callback.onError("支付宝人脸支付返回参数为空");
                    return;
                }

                callback.onSuccess(new org.json.JSONObject(strResponseParams).getString("zoloz_authentication_customer_smilepay_initialize_response"));
            } catch (Exception e) {
                LoggerUtils.getLogger().error(e.getMessage());
                callback.onError(e.getMessage());
            }
        });
    }

    /**
     * 请求阿里服务
     *
     * @param requestParams 请求参数
     * @param url           请求地址
     * @return strResponseParams 返回参数
     */
    private static String doPost(Map<String, String> requestParams, String url) throws Exception {
        int connectTimeout = 1000 * 60; // 连接超时时间
        int readTimeout = 1000 * 180;   // 读取数据超时时间

        String strResponseParams;
        try {
            strResponseParams = WebUtils.doPost(url, requestParams, connectTimeout, readTimeout);
        } catch (ConnectException e) {
            String errorMessage = "请求第三方服务超时：" + e.getMessage();
            LoggerUtils.getLogger().debug(errorMessage);
            throw new ConnectException(errorMessage);
        }
        return strResponseParams;
    }

    private static void writeLog(String logs) {
        Date date = new Date();
        if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
            String path = "/sdcard/crash/";
            String fileName = "zoloz-" + date.toString() + "-" + date.getTime() + ".log";
            File dir = new File(path);
            if (!dir.exists()) {
                dir.mkdirs();
            }
            FileOutputStream fos = null;
            try {
                fos = new FileOutputStream(path + fileName);
                fos.write(logs.getBytes());
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    fos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
