package com.hisense.hiaposcmp.service.app;

import android.os.Environment;

import com.alibaba.fastjson.JSONObject;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.ProductKey;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.model.AuthorizationInfo;
import com.hisense.hiaposcmp.activity.common.authorization.ldk_rus.model.FeatureInfo;
import com.hisense.hiaposcmp.auth.algorithm.Des;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.service.payment.AggregatePaymentChannel;
import com.hisense.hiaposcmp.util.FileUtils;
import com.hisense.hiaposcmp.view.IView.pay.IPayView;
import com.hisense.hipos.payment.PayMethod;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

/**
 * 支付授权服务
 *
 * @author guolina  2020-03-18
 */
public class AuthService {

    private IPayView mMvpView;
    private String mTotalPaid;
    private String expireDateStr;
    private PayMethod mSelectedPayMethod;

    public AuthService(IPayView mvpView, String totalPaid, PayMethod selectedPayMethod) {
        mMvpView = mvpView;
        mTotalPaid = totalPaid;
        mSelectedPayMethod = selectedPayMethod;
    }

    /**
     * 预支付：检测是否能够支付
     */
    public void prePay() {
        mMvpView.showLoadingDialog();
        ProductKey productKey = new ProductKey();

        String info = productKey.authInfo();
        //一台设备只能被激活一次，已经被激活的设备卸载程序重新安装，锁仍然存在，所以要先通过调用getinfo接口获取锁信息
        //若锁存在，直接进行逻辑判断，若锁不存在，调用激活方法
        //如果已被激活，检测支付方式信息是否被授权
        if (null != info) {
            getAuthInfo(productKey);
            LoggerUtils.getLogger().debug("prePay:getAuthInfo");
        } else {
            //如果从未被激活过，首先调用激活方法，激活成功后再去检测支付方式是否被授权
            activate(productKey, true);
            LoggerUtils.getLogger().debug("prePay:activate");
        }
    }

    /**
     * 在线激活
     */
    private void activate(ProductKey productKey, boolean isFirst) {
        productKey.activate(isFirst, new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                GeneralSettings.INSTANCE.setIsActivate(true);
                getAuthInfo(productKey);
            }

            @Override
            public void onError(String err) {
                mMvpView.hideLoadingDialog();
                GeneralSettings.INSTANCE.setIsActivate(false);

                String filePth = Environment.getExternalStorageDirectory().getPath() + "/android/data/installed.txt";
                String dateStr = FileUtils.readFileContent(filePth);
                if (null == dateStr) {
                    mMvpView.showHintDialog("产品激活/升级失败，试用期已过，请联系开发商");
                    return;
                }
                Date installDate;
                try {
                    installDate = DateUtils.dateParse(Des.decrypt2Base64("ilovehisense", dateStr));
                } catch (Exception e) {
                    LoggerUtils.getLogger().debug("日期格式化失败" + e.toString());
                    mMvpView.showHintDialog("产品激活/升级失败，试用期已过，请联系开发商");
                    return;

                }
                Date expireDate = DateUtils.dayAddAndSub(installDate, 60);
                String expireStr = DateUtils.dateFormat(expireDate);
                if (DateUtils.compareTime(installDate, 60)) {
                    mMvpView.showHintConfirm("产品激活/升级失败，试用期至" + expireStr + "，请尽快联系开发商", new UniCallback() {
                        @Override
                        public void onSuccess(String msg) {
                            mMvpView.gotoPayChannel(mTotalPaid);
                        }

                        @Override
                        public void onError(String err) {
                            return;
                        }
                    });

                } else {
                    mMvpView.showHintDialog("产品激活/升级失败，试用期已过，请联系开发商");
                    return;
                }
            }
        });
    }

    /**
     * 在线获取授权信息并校验有效性
     */
    private void getAuthInfo(ProductKey productKey) {
        productKey.getPayAuthInfo(new UniCallback() {
            @Override
            public void onSuccess(String msg) {
                authInfo(productKey);
            }

            @Override
            public void onError(String err) {
                mMvpView.hideLoadingDialog();
                mMvpView.showHintDialog("授权信息获取异常，请联系开发商");
            }
        });
    }

    /**
     * 检验是否在试用期
     */
    private boolean isInExpireDate() {
        String filePth = Environment.getExternalStorageDirectory().getPath() + "/android/data/installed.txt";
        String dateStr = FileUtils.readFileContent(filePth);
        if (null == dateStr) {
            return false;
        }
        Date installDate;
        try {
            installDate = DateUtils.dateParse(Des.decrypt2Base64("ilovehisense", dateStr));
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("日期格式化失败" + e.toString());
            return false;

        }
        Date expireDate = DateUtils.dayAddAndSub(installDate, 60);
        expireDateStr = DateUtils.dateFormat(expireDate);
        if (DateUtils.compareTime(installDate, 60)) {
            LoggerUtils.getLogger().debug("试用期至：" + expireDateStr);
            return true;

        } else {
            return false;
        }
    }

    /**
     * 试用期对话框
     */
    private void expireDialog(ProductKey productKey) {
        if (isInExpireDate()) {
            mMvpView.showHintConfirm("支付方式未被授权，试用期至" + expireDateStr + "，点击确定继续使用，点击取消进行升级", new UniCallback() {
                @Override
                public void onSuccess(String msg) {
                    mMvpView.gotoPayChannel(mTotalPaid);
                    LoggerUtils.getLogger().debug(mSelectedPayMethod.remark() + "支付方式未被授权，继续使用，试用期至" + expireDateStr);
                }

                @Override
                public void onError(String err) {
                    activate(productKey, false);
                    LoggerUtils.getLogger().debug(mSelectedPayMethod.remark() + "支付方式未被授权，暂不使用,进行升级");
                }
            });
        } else {
            mMvpView.showHintDialog("支付方式未授权,且不在试用期内");
            LoggerUtils.getLogger().debug(mSelectedPayMethod.remark() + "支付方式未授权,且不在试用期内");
            return;
        }
    }

    /**
     * 检验本地授权信息是否过期
     */
    private void authInfo(ProductKey productKey) {
        mMvpView.hideLoadingDialog();
        AuthorizationInfo authorizationInfo = JSONObject.parseObject(GeneralSettings.INSTANCE.getAuthorization(), AuthorizationInfo.class);
        if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS() && AggregatePaymentChannel.isCHANNEL()) {
            String featureIdPre;
            try {
                for (int i = 0; i < authorizationInfo.feature.size(); i++) {
                    featureIdPre = authorizationInfo.feature.get(i).featureId.substring(0, 4);
                    if (!featureIdPre.equals("6070")) {
                        authorizationInfo.feature.remove(i);
                        i--;
                    }
                }
            } catch (Exception e) {
                mMvpView.showHintDialog("授权码格式错误");
                return;
            }

            if (authorizationInfo.feature.size() > 0) {
                ArrayList<String> arrlist = new ArrayList<>();
                HashMap<String, String> map = new HashMap<>(authorizationInfo.getFeature().size() * 2);

                String featureIdUms = "60700";
                String featureIdAli = "60701";
                String featureIdWx = "60702";
                String featureIdUnionCloudPay = "60703";
                String featureIdUnionPay = "60707";
                String featureIdUnionVipCard1 = "60708";
                String featureIdUnionVipCard2 = "60709";
                for (FeatureInfo featureInfo : authorizationInfo.getFeature()) {
                    arrlist.add(featureInfo.featureId);
                    if (featureInfo.featureId.equals(featureIdAli)) {
                        map.put("aliExpDate", featureInfo.expDate);
                        map.put("aliLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdWx)) {
                        map.put("wxExpDate", featureInfo.expDate);
                        map.put("wxLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdUms)) {
                        map.put("umsExpDate", featureInfo.expDate);
                        map.put("umsLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdUnionCloudPay)) {
                        map.put("UnionCloudPayExpDate", featureInfo.expDate);
                        map.put("UnionCloudPayLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdUnionPay)) {
                        map.put("UnionPayExpDate", featureInfo.expDate);
                        map.put("UnionPayLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdUnionVipCard1)) {
                        map.put("UnionVipCard1ExpDate", featureInfo.expDate);
                        map.put("UnionVipCard1LicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdUnionVipCard2)) {
                        map.put("UnionVipCard2ExpDate", featureInfo.expDate);
                        map.put("UnionVipCard2LicenseType", featureInfo.licenseType);
                    }
                }
                if (arrlist.contains(featureIdUms)) {
                    overdue00(map.get("umsLicenseType"), map.get("umsExpDate"), productKey);
                } else if (mSelectedPayMethod.isAliPay()) {
                    overdue(arrlist, featureIdAli, map.get("aliLicenseType"), map.get("aliExpDate"), productKey);
                } else if (mSelectedPayMethod.isWxPay()) {
                    overdue(arrlist, featureIdWx, map.get("WxLicenseType"), map.get("WxExpDate"), productKey);
                } else if (mSelectedPayMethod.isMobilePay()) {

                    overdueMobile(arrlist, featureIdAli, featureIdWx, map.get("aliLicenseType"), map.get("WxLicenseType"), map.get("aliExpDate"), map.get("WxExpDate"), productKey);


                } else if (mSelectedPayMethod.isUnionCloudPay()) {
                    overdue(arrlist, featureIdUnionCloudPay, map.get("UnionCloudPayLicenseType"), map.get("UnionCloudPayExpDate"), productKey);
                } else if (mSelectedPayMethod.isUnionPay()) {
                    overdue(arrlist, featureIdUnionPay, map.get("UnionPayLicenseType"), map.get("UnionPayExpDate"), productKey);
                } else if (mSelectedPayMethod.isUnionVipCard1()) {
                    overdue(arrlist, featureIdUnionVipCard1, map.get("UnionVipCard1LicenseType"), map.get("UnionVipCard1ExpDate"), productKey);
                } else if (mSelectedPayMethod.isUnionVipCard2()) {
                    overdue(arrlist, featureIdUnionVipCard2, map.get("UnionVipCard2LicenseType"), map.get("UnionVipCard2ExpDate"), productKey);
                }
            } else {
                expireDialog(productKey);
            }

        } else if (GeneralSettings.INSTANCE.getPaymentChannel().isCCB()) {
            String featureIdPre;
            try {
                for (int i = 0; i < authorizationInfo.feature.size(); i++) {
                    featureIdPre = authorizationInfo.feature.get(i).featureId.substring(0, 4);
                    if (!featureIdPre.equals("6050")) {
                        authorizationInfo.feature.remove(i);
                        i--;
                    }
                }

            } catch (Exception e) {
                mMvpView.showHintDialog("授权码格式错误");
                return;
            }

            if (authorizationInfo.feature.size() > 0) {
                ArrayList<String> arrlist = new ArrayList<>();
                HashMap<String, String> map = new HashMap<>(authorizationInfo.getFeature().size() * 2);

                String featureIdUms = "60500";
                String featureIdUnionPay = "60501";
                String featureIdWx = "60502";//在建行用60502代表微信、支付宝、聚合支付


                for (FeatureInfo featureInfo : authorizationInfo.getFeature()) {
                    arrlist.add(featureInfo.featureId);
                    if (featureInfo.featureId.equals(featureIdUnionPay)) {
                        map.put("UnionPayExpDate", featureInfo.expDate);
                        map.put("UnionPayLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdWx)) {
                        map.put("wxExpDate", featureInfo.expDate);
                        map.put("wxLicenseType", featureInfo.licenseType);
                    } else if (featureInfo.featureId.equals(featureIdUms)) {
                        map.put("umsExpDate", featureInfo.expDate);
                        map.put("umsLicenseType", featureInfo.licenseType);
                    }
                }
                if (arrlist.contains(featureIdUms)) {
                    overdue00(map.get("umsLicenseType"), map.get("umsExpDate"), productKey);
                } else if (mSelectedPayMethod.isWxPay() || mSelectedPayMethod.isAliPay() || mSelectedPayMethod.isMobilePay()) {
                    overdue(arrlist, featureIdWx, map.get("WxLicenseType"), map.get("WxExpDate"), productKey);
                } else if (mSelectedPayMethod.isUnionPay()) {
                    overdue(arrlist, featureIdUnionPay, map.get("UnionPayLicenseType"), map.get("UnionPayExpDate"), productKey);
                }
            } else {
                expireDialog(productKey);
            }

        } else {
            mMvpView.gotoPayChannel(mTotalPaid);

        }

    }

    /**
     * 判断总权限是否过期
     */
    private void overdue00(String licenseType, String expDateObj, ProductKey productKey) {
        String expDate = expDateObj == null ? "" : expDateObj;
        //未过期
        if (!("expiration".equals(licenseType) && BigDecimalUtils.isLess(expDate, String.valueOf(System.currentTimeMillis() / 1000)))) {
            mMvpView.gotoPayChannel(mTotalPaid);
            return;
        }
        expireDialog(productKey);

    }

    /**
     * 判断是否过期
     */
    private void overdue(ArrayList<String> arrlist, String featureId, String licenseType, String expDateObj, ProductKey productKey) {
        String expDate = expDateObj == null ? "" : expDateObj;
        if (arrlist.contains(featureId)) {
            //未过期
            if (!("expiration".equals(licenseType) && BigDecimalUtils.isLess(expDate, String.valueOf(System.currentTimeMillis() / 1000)))) {
                mMvpView.gotoPayChannel(mTotalPaid);
                return;
            }
        }
        expireDialog(productKey);
    }

    /**
     * 聚合判断是否过期
     */
    private void overdueMobile(ArrayList<String> arrlist, String featureId1, String featureId2, String licenseType1, String licenseType2, String expDateObj1, String expDateObj2, ProductKey productKey) {
        String expDate1 = expDateObj1 == null ? "" : expDateObj1;
        String expDate2 = expDateObj2 == null ? "" : expDateObj2;
        if (arrlist.contains(featureId1) && arrlist.contains(featureId2)) {
            String mobileExpDate = BigDecimalUtils.isLess(expDate1, expDate2) ? expDate1 : expDate2;
            String mobileLicenseType = ("expiration".equals(licenseType1) || "expiration".equals(licenseType2)) ? "expiration" : "";

            //未过期
            if (!("expiration".equals(mobileLicenseType) && BigDecimalUtils.isLess(mobileExpDate, String.valueOf(System.currentTimeMillis() / 1000)))) {
                mMvpView.gotoPayChannel(mTotalPaid);
                return;
            }
        }
        expireDialog(productKey);
    }
}
