package com.hisense.hiaposcmp.service.bgm;

import android.app.Service;
import android.content.Intent;
import android.media.MediaPlayer;
import android.os.IBinder;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.tool.LoadSDCard;

/**
 * 播放背景音乐
 *
 * @author dukunbiao(null) 2018-05-22
 */

public class BGMService extends Service {

    private MediaPlayer mediaPlayer;
    private static int interval = Integer.MAX_VALUE;

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        String bgmTag = intent.getStringExtra(SDCardConstants.BGM_TAG);
        String bgmPath = "";

        switch (bgmTag) {
            case SDCardConstants.BGM_SCAN_PLU:
                bgmPath = SDCardConstants.SSPOS_SCAN_PLU_PATH;
                interval = 20000;
                break;
            case SDCardConstants.BGM_ALI_PAY:
                bgmPath = SDCardConstants.SSPOS_ALI_PAY_PATH;
                interval = Integer.MAX_VALUE;
                break;
        }

        mediaPlayer = LoadSDCard.loadMediaPlayer(bgmPath);

        if (mediaPlayer == null) {
            return super.onStartCommand(intent, flags, startId);
        }

        mediaPlayer.start();

        mediaPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {

            @Override
            public void onCompletion(MediaPlayer arg0) {
                if (interval == Integer.MAX_VALUE) {
                    return;
                }

                long beginTime = System.currentTimeMillis();
                long endTime = System.currentTimeMillis();
                while (endTime - beginTime < interval) {
                    endTime = System.currentTimeMillis();
                }
                mediaPlayer.start();
            }
        });

        mediaPlayer.setOnErrorListener((mediaPlayer, what, extra) -> {
            try {
                mediaPlayer.release();
            } catch (Exception e) {
                LoggerUtils.getLogger().error("-------------BGMService 异常----------" + e.toString());
            }

            return false;
        });

        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onCreate() {
        super.onCreate();
    }

    @Override
    public void onDestroy() {
        if (mediaPlayer != null) {
            mediaPlayer.stop();
            mediaPlayer.release();
            mediaPlayer = null;
        }

        super.onDestroy();
    }
}
