/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.log;

import com.hisense.common.Context;
import com.hisense.common.util.LoggerUtils;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * 删除设备n天前的日志文件
 *
 * @author xieliyan
 */
public class LogFileHandler {
    private static String pathCrashPart = Context.logPath() + "crash/";
    private static String pathTradeLogPart = Context.logPath() + "tradelog/";
    private static String pathTradelogZipPart = Context.logPath() + "TRADELOGZIP/";

    public static void deleteExpired(int expiredDays) {
        File fileCrash = new File(pathCrashPart);
        File[] filesCrash = fileCrash.listFiles();// 读取
        deleteCrashFiles(filesCrash, expiredDays);


        File fileTradeLog = new File(pathTradeLogPart);
        File[] filesTradeLog = fileTradeLog.listFiles();// 读取
        deleteTradeLogFiles(filesTradeLog, expiredDays);


        File fileTradeLogZip = new File(pathTradelogZipPart);
        File[] filesTradeLogZip = fileTradeLogZip.listFiles();// 读取
        deleteLogZip(filesTradeLogZip, expiredDays);
    }

    /**
     * 获取今天与指定日期之间的间隔天数
     *
     * @return 间隔天数
     */
    public static int getLastDays(Date logDate) {
        Date curDate = new Date();
        int timeVar = 1000 * 60 * 60 * 24;

        return (int) ((curDate.getTime() - logDate.getTime()) / timeVar);
    }


    /**
     * 清理crash日志
     */
    private static void deleteCrashFiles(File[] files, int maxLastDays) {
        if (files != null) {// 先判断目录是否为空，否则会报空指针
            for (File file : files) {
                if (file.isDirectory()) {
                    deleteCrashFiles(file.listFiles(), maxLastDays);
                    LoggerUtils.getLogger().debug("若是文件目录。继续读2" + file.getName() + file.getPath());

                } else {
                    String fileName = file.getName();
                    if (fileName.endsWith(".log") && fileName.startsWith("crash")) {
                        try {
                            String s = fileName.substring(0, fileName.lastIndexOf("."));
                            //LoggerUtils.getLogger().debug("LogFileHandler" + " s = " + s);
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINESE);
                            String date = s.substring(6, 16);
                            //LoggerUtils.getLogger().debug("LogFileHandler" + " date = " + date);
                            Date date1 = format.parse(date.trim());
                            if (getLastDays(date1) > maxLastDays) {
                                file.delete();
                            }
                        } catch (ParseException e) {
                            e.printStackTrace();
                            LoggerUtils.getLogger().debug("LogFileHandler" + e.toString());
                        }


                    }
                }
            }
        } else {

            LoggerUtils.getLogger().debug("LogFileHandler" + " files 目录为空 ");
        }
    }

    /**
     * 清理tradelog日志
     */
    private static void deleteTradeLogFiles(File[] files, int maxLastDays) {
        if (files != null) {// 先判断目录是否为空，否则会报空指针
            for (File file : files) {
                if (file.isDirectory()) {

                    deleteTradeLogFiles(file.listFiles(), maxLastDays);
                    LoggerUtils.getLogger().debug("若是文件目录。继续读2" + file.getName() + file.getPath());
                } else {
                    String fileName = file.getName();
                    try {
                        String s = fileName.substring(0, fileName.indexOf("."));
                        LoggerUtils.getLogger().debug("LogFileHandler" + " s = " + s);
                        if (s.length() < 19) {
                            return;
                        }
                        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINESE);
                        String date = s.substring(9, 19);
                        LoggerUtils.getLogger().debug("LogFileHandler" + " date = " + date);
                        Date date1 = format.parse(date.trim());
                        if (getLastDays(date1) > maxLastDays) {
                            file.delete();
                        }

                    } catch (ParseException e) {
                        e.printStackTrace();
                        LoggerUtils.getLogger().debug("LogFileHandler" + e.toString());
                    }
                }
            }

        } else {
            LoggerUtils.getLogger().debug("LogFileHandler" + " files 目录为空 ");
        }
    }

    /**
     * 清理crash日志
     */
    private static void deleteLogZip(File[] files, int maxLastDays) {

        if (files != null) {// 先判断目录是否为空，否则会报空指针
            for (File file : files) {
                if (file.isDirectory()) {
                    deleteLogZip(file.listFiles(), maxLastDays);
                    LoggerUtils.getLogger().debug("若是文件目录。继续读2" + file.getName() + file.getPath());

                } else {
                    String fileName = file.getName();
                    if (fileName.endsWith(".zip")) {
                        try {
                            String s1 = fileName.substring(fileName.indexOf("_") + 1, fileName.lastIndexOf("_"));
                            String date = s1.substring(s1.lastIndexOf("_") + 1);
                            LoggerUtils.getLogger().debug("LogFileHandler" + " date = " + date);
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINESE);
                            Date date1 = format.parse(date.trim());
                            if (getLastDays(date1) > maxLastDays) {
                                file.delete();
                            }
                        } catch (ParseException e) {
                            e.printStackTrace();
                            LoggerUtils.getLogger().debug("LogFileHandler" + e.toString());
                        }


                    }
                }
            }
        } else {

            LoggerUtils.getLogger().debug("LogFileHandler" + " files 目录为空 ");
        }
    }

}

