package com.hisense.hiaposcmp.service.log;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;

import androidx.annotation.Nullable;

import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.hiaposcmp.auth.HiAppConfig;
import com.hisense.hiaposcmp.auth.util.FileUtil;
import com.hisense.hiaposcmp.bean.BaseEventBean;
import com.hisense.hiaposcmp.tool.ZipHelper;
import com.hisense.hiaposcmp.util.DeviceInfoUtils;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by zhuguangsheng on 2018/9/29.
 * 操作日志上传的Service
 */
public class LogUploadService extends Service {

    private final static String DEF_FTP_SERVER_IP = "115.28.65.102";
    private final static int DEF_FTP_SERVER_PORT = 21;
    private final static String DEF_FTP_USERNAME = "tom";
    private static String DEF_FTP_PASSWORD = "123";       //临时使用，后续会改密码。应该在配置文件中以加密方式指定

    private final static int UPLOAD_LOG_DAYS = 3; //建议1~3,不能太多，以免让服务器崩溃

    class FtpParams {
        String serverIp = "";
        int port = DEF_FTP_SERVER_PORT;
        String userName = "";
        String password = "";
    }

    FtpParams mFtpParams = new FtpParams();

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        LoggerUtils.getLogger().info("LogUploadService onCreate...");

        EventBus.getDefault().register(this);

        readFtpParamsFromAppConfig();

        startUploadRoutine();
    }

    @Override
    public void onDestroy() {
        LoggerUtils.getLogger().info("LogUploadService onDestroy...");
        EventBus.getDefault().unregister(this);
        super.onDestroy();
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(Object event) {
        if (event instanceof BaseEventBean) {

        }
    }

    /**
     * 从配置文件中读取
     * 如果配置文件中没有，就使用本文定义的默认参数
     */
    private void readFtpParamsFromAppConfig() {
        int mErrCode = HiAppConfig.initRead();

        mFtpParams.serverIp = HiAppConfig.getValueIfNotEmpty("ftpserverip", DEF_FTP_SERVER_IP);
        String portStr;
        try {
            portStr = HiAppConfig.getValueIfNotEmpty("ftpport", String.valueOf(DEF_FTP_SERVER_PORT));
            mFtpParams.port = Integer.valueOf(portStr).intValue();
        } catch (Exception e) {
            e.printStackTrace();
            mFtpParams.port = DEF_FTP_SERVER_PORT;
        }
        mFtpParams.userName = HiAppConfig.getValueIfNotEmpty("ftpusername", DEF_FTP_USERNAME);
        mFtpParams.password = HiAppConfig.getValueIfNotEmpty("ftppassword", DEF_FTP_PASSWORD);
    }

    /**
     * 开始上传日志流程
     */
    private void startUploadRoutine() {
        ThreadFactoryUtils.getExecutorService("log-%d").execute(() -> {
            try {
                Thread.sleep(1000);
                findFileZipAndUpload();
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void findFileZipAndUpload() {
        SimpleDateFormat sDateFormat = new SimpleDateFormat("yyyyMMdd");
        Date today = new java.util.Date();

        for (int i = 0; i < UPLOAD_LOG_DAYS; i++) {
            Date date = DateUtils.dayAddAndSub(today, -1 * i);
            String dateStr = sDateFormat.format(date);
            String destFileName = DeviceInfoUtils.getModel()
                    + "_" + DeviceInfoUtils.getSystemBuildSerial()
                    + "_log_"
                    + dateStr
                    + ".zip";

            //查找指定文件名的文件(Tradelog+日期开头的)
            List<File> list = FileUtil.findFilesWithPrefix("/HIAPOS/",
                    LoggerUtils.PRE_LOG_NAME + dateStr);
            LoggerUtils.getLogger().info("list=" + list.toString());
            if (list == null && list.size() == 0) {
                continue;
            }

            //压缩
            boolean b = ZipHelper.zipFiles(list, destFileName);
            if (!b) {
                continue;
            }
        }
    }
}
