package com.hisense.hiaposcmp.service.log;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;

import androidx.annotation.Nullable;

import com.hisense.common.Context;
import com.hisense.common.util.CompressUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.hiaposcmp.auth.util.FileUtil;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.tool.ZipHelper;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hiaposcmp.util.DeviceInfoUtils;
import com.lidroid.xutils.HttpUtils;
import com.lidroid.xutils.exception.HttpException;
import com.lidroid.xutils.http.RequestParams;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;
import com.lidroid.xutils.http.client.HttpRequest;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;


/**
 * 设备日志上传(自动）
 *
 * @author xieliyan 2019-06-25
 * @author xieliyan 2020-03-31 HIAPOSCMP-633 修复日志不能自动上传的问题
 */
public class UploadLogService extends Service {

    private List<File> list = new ArrayList<>();
    private String dateStr;
    private HttpUtils mHttpUtils;

    public UploadLogService() {
        super();
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        if (CommonUtils.isWriteOk()) {
            initHttpUtils();
            startUploadRoutine();
            LoggerUtils.getLogger().debug("开始上传设备日志，UploadLogService onCreate");
        }
    }

    /**
     * 开始上传日志流程
     */
    private void startUploadRoutine() {
        ThreadFactoryUtils.getExecutorService("log-%d").execute(() -> {
            try {
                Thread.sleep(1000);
                findFileZipAndUpload();
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void findFileZipAndUpload() {
        SimpleDateFormat sDateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.CHINESE);
        Date today = new java.util.Date();

        Date date = DateUtils.dayAddAndSub(today, -1 * 1);
        dateStr = sDateFormat.format(date);
        String destFileName = DeviceInfoUtils.getModel()
                + "_" + System.currentTimeMillis()
                + "_log_"
                + dateStr
                + "_automatic"
                + ".zip";

        //查找指定文件名的文件(Tradelog+日期开头的)
        list = FileUtil.findFilesWithPrefix(Context.logPath()+"tradelog",
                LoggerUtils.PRE_LOG_NAME + dateStr);
        LoggerUtils.getLogger().debug("list=" + list.toString());

        //压缩
        boolean b = ZipHelper.zipFiles(list, destFileName);
        if (b) {
            upload(Config.log_path + "TRADELOGZIP/" + destFileName);
        } else {
            LoggerUtils.getLogger().debug("日志压缩失败");
        }

    }

    /**
     * 初始化xutils
     */
    private void initHttpUtils() {
        mHttpUtils = new HttpUtils(15 * 1000);//5000-->联网超时时间-->可以无参
        //设置缓存大小
        int size = (int) (Runtime.getRuntime()
                .totalMemory() / 8);
        mHttpUtils.configHttpCacheSize(size);
        //设置编码格式
        mHttpUtils.configResponseTextCharset("utf-8");
        //设置重连次数,如果链接服务失败,再次尝试
        mHttpUtils.configRequestRetryCount(3);
        //设置多线程迸发(线程池)
        mHttpUtils.configRequestThreadPoolSize(4);
        //设置缓存数据失效的时间
        mHttpUtils.configDefaultHttpCacheExpiry(1000 * 60 * 30);
    }

    private void upload(final String s) {

        String url = Config.base_url + "/api/log/upload";
        RequestParams params = new RequestParams();
        params.addBodyParameter("orgCode", Config.org_code);
        params.addBodyParameter("posNo", Config.pos_no);
        params.addBodyParameter("file", new File(s));
        mHttpUtils.send(HttpRequest.HttpMethod.POST, url, params, new RequestCallBack<String>() {
            @Override
            public void onSuccess(ResponseInfo<String> responseInfo) {

                if (responseInfo != null) {
                    try {
                        JSONObject object = new JSONObject(responseInfo.result);
                        int code = 1;
                        if (object.has("code")) {
                            code = object.getInt("code");
                        }
                        if (code == 1) {
                            LoggerUtils.getLogger().debug("日志成功上传到中间件服务器");
                            //成功上传后暂时不删除源文件
                       //     FileUtil.deleteFilesAtPath(Config.log_path + "tradelog", LoggerUtils.PRE_LOG_NAME + dateStr);
                        } else {
                            LoggerUtils.getLogger().debug("日志未成功上传到中间件服务器" + responseInfo.result);
                            CompressUtils.deleteAll(s);

                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }

            }

            @Override
            public void onFailure(HttpException error, String msg) {
                LoggerUtils.getLogger().debug("日志未成功上传到中间件服务器" + msg);
                CompressUtils.deleteAll(s);
            }
        });
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }
}
