/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.service.miya;

import android.os.Environment;
import android.os.Handler;
import android.os.Message;

import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hipos.miya.RefundMIYAResponse;
import com.miya.TcpSend;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Random;
import java.util.StringTokenizer;

/**
 * 米雅服务。
 *
 * @author caokai   2018-03-28              使用 MIYAConfig 初始化服务
 * @author xieliyan 2018-04-18 HIPOSAPP-408 优化米雅服务代码
 */
public class MiYaService {

    private static boolean IS_INIT = false;
    private String mOrgCode;
    private Map mMapConfigMiYa;
    private final String mConfigFilePath = Environment.getExternalStorageDirectory() + "/";
    public static String outTradeNo = "";

    public MiYaService(String orgCode) {
        this.mOrgCode = orgCode;
    }

    /**
     * 生成 赢商通 的请求流水号。
     */
    private static String generReqSeq(final String orgCode, final String saleNo) {
        // 业务流水号 + 3位随机数 + orgCode
        String str = "";
        switch (Custom.current().getName()) {
            case "hs5510yonghui":
                //str=Config.org_code + Config.pos_no + DateUtils.dateFormat(new Date(), "yyyyMMddHHmmss");
                str = saleNo + DateUtils.dateFormat(new Date(), "HHmmss");
                if (str.length() > 64) {
                    str = str.substring(0, 64);
                }
                break;
            default:
                Random rand = new Random();
                int randInt = rand.nextInt(1000);
                str = saleNo + String.format(Locale.CHINESE, "%03d", randInt) + orgCode;
                break;
        }
        return str;
    }

    /**
     * 扫码支付。
     *
     * @param payHandler 处理程序
     * @param saleNo     流水号
     * @param ssTotal    实收金额
     * @param authCode   条码
     */
    public void pay(final Handler payHandler, final String saleNo,
                    final String ssTotal, final String authCode) {
        if (!IS_INIT) {
            if (!isInitialized()) {
                Message msg = new Message();
                msg.what = ActivityConstants.RESULT_MIYA_PAY;
                msg.obj = "米雅服务配置错误,请确认配置文件";
                payHandler.sendMessage(msg);
                return;
            }
        }

        new Thread(() -> {
            int money = new BigDecimal(ssTotal).intValue();
            String requestSeq = generReqSeq(mOrgCode, saleNo);
            outTradeNo = requestSeq;
            LoggerUtils.getLogger().debug("----YST payment calling, ssTotal:" + ssTotal + "(" + money + "), requestSeq:" + requestSeq);
            Map<String, Object> map = new HashMap<>();
            map.put("VERSION", "1.5");
            map.put("TRCODE", "1001");
            map.put("NUMID", mMapConfigMiYa.get("marketid"));
            map.put("USERID", mMapConfigMiYa.get("userid"));
            map.put("OPERATOR_ID", mMapConfigMiYa.get("operator_id"));
            map.put("PAYMENTPLATFORM", "A");
            map.put("SERVEICETYPE", "A");
            map.put("OUT_TRADE_NO", requestSeq);
            map.put("TOTAL_FEE", money);
            map.put("PAYMENT_TYPE", "");
            map.put("F1", authCode);
            map.put("path", mConfigFilePath);
            LoggerUtils.getLogger().debug("米雅支付请求参数" + map.toString());
            try {
                Map mapRequest = TcpSend.sendMiya(map, null);
                Message msg = new Message();
                msg.what = ActivityConstants.RESULT_MIYA_PAY;
                msg.obj = mapRequest;
                payHandler.sendMessage(msg);
            } catch (Exception e) {
                LoggerUtils.getLogger().debug("米雅支付请求异常" + e.getMessage());
            }
        }).start();
    }

    /**
     * 退款。
     *
     * @param saleNo  流水号
     * @param ssTotal 退款金额
     * @param tranSeq 米雅支付流水号
     * @return 退款处理结果
     */
    public RefundMIYAResponse refund(final String saleNo, final String ssTotal, final String tranSeq) {
        String requestSeq = generReqSeq(mOrgCode, saleNo);
        RefundMIYAResponse refundResponse = new RefundMIYAResponse();
        Map<String, Object> map = new HashMap<>();
        map.put("VERSION", "1.5");
        map.put("TRCODE", "1002");
        map.put("NUMID", mMapConfigMiYa.get("marketid"));
        map.put("USERID", mMapConfigMiYa.get("userid"));
        map.put("OPERATOR_ID", mMapConfigMiYa.get("operator_id"));
        map.put("PAYMENTPLATFORM", "A");
        map.put("SERVEICETYPE", "C");
        map.put("OUT_TRADE_NO", tranSeq);
        map.put("OUT_REQUEST_NO", requestSeq);
        map.put("TOTAL_FEE", new BigDecimal(ssTotal).setScale(2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)));
        map.put("PAYMENT_TYPE", "");
        map.put("path", mConfigFilePath);
        try {
            Map mapRequest = TcpSend.sendMiya(map, null);
            refundResponse.setStatus(mapRequest.get("RETCODE").toString());
            refundResponse.setResponseMessage(mapRequest.get("RETMSG").toString());
            refundResponse.setRefundSequence(mapRequest.get("OUT_REQUEST_NO").toString());
            refundResponse.setRequestSequence(mapRequest.get("OUT_TRADE_NO").toString());
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("米雅退货请求异常" + e.getMessage());
        }
        return refundResponse;
    }

    /**
     * 撤销。
     *
     * @param ssTotal 撤销金额
     * @param tradeNo 米雅支付流水号
     * @return 撤销处理结果
     */
    public RefundMIYAResponse cancel(final String ssTotal, final String tradeNo) {
        RefundMIYAResponse refundResponse = new RefundMIYAResponse();
        Map<String, Object> map = new HashMap<>();
        map.put("VERSION", "1.5");
        map.put("TRCODE", "1003");
        map.put("NUMID", mMapConfigMiYa.get("marketid"));
        map.put("USERID", mMapConfigMiYa.get("userid"));
        map.put("OPERATOR_ID", mMapConfigMiYa.get("operator_id"));
        map.put("PAYMENTPLATFORM", "A");
        map.put("SERVEICETYPE", "E");
        map.put("OUT_TRADE_NO", tradeNo);
        map.put("TOTAL_FEE", new BigDecimal(ssTotal).setScale(2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)));
        map.put("PAYMENT_TYPE", "");
        map.put("path", mConfigFilePath);
        try {
            Map mapRequest = TcpSend.sendMiya(map, null);
            refundResponse.setStatus(mapRequest.get("RETCODE").toString());
            refundResponse.setResponseMessage(mapRequest.get("RETMSG").toString());
            refundResponse.setRefundSequence(mapRequest.get("OUT_REQUEST_NO").toString());
            refundResponse.setRequestSequence(mapRequest.get("OUT_TRADE_NO").toString());
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("米雅支付撤销异常" + e.getMessage());
        }
        return refundResponse;
    }

    /**
     * 查询。
     *
     * @param ssTotal
     * @param tradeNo 米雅支付流水号
     * @return 查询处理结果
     */
    public RefundMIYAResponse query(final String ssTotal, final String tradeNo) {
        RefundMIYAResponse refundResponse = new RefundMIYAResponse();
        Map<String, Object> map = new HashMap<>();
        map.put("VERSION", "1.5");
        map.put("TRCODE", "1004");
        map.put("NUMID", mMapConfigMiYa.get("marketid"));
        map.put("USERID", mMapConfigMiYa.get("userid"));
        map.put("OPERATOR_ID", mMapConfigMiYa.get("operator_id"));
        map.put("PAYMENTPLATFORM", "A");
        map.put("SERVEICETYPE", "B");
        map.put("OUT_TRADE_NO", tradeNo);
        map.put("TOTAL_FEE", new BigDecimal(ssTotal).setScale(2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)));
        map.put("PAYMENT_TYPE", "");
        map.put("path", mConfigFilePath);
        try {
            Map mapRequest = TcpSend.sendMiya(map, null);
            refundResponse.setStatus(mapRequest.get("RETCODE").toString());
            refundResponse.setResponseMessage(mapRequest.get("RETMSG").toString());
            refundResponse.setRefundSequence(mapRequest.get("OUT_REQUEST_NO").toString());
            refundResponse.setRequestSequence(mapRequest.get("OUT_TRADE_NO").toString());
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("米雅支付查询异常" + e.getMessage());
        }
        return refundResponse;
    }

    /**
     * 初始化米雅配置
     */
    public boolean isInitialized() {
        IS_INIT = true;
        mMapConfigMiYa = readKeyValueTxtToMap(mConfigFilePath + "miyaapay/config/miyaConfig.txt");
        LoggerUtils.getLogger().debug("------initMIYAConfig (filePath:" + mConfigFilePath + ",NUMID:" + mMapConfigMiYa.get("marketid") + ",USERID:" + mMapConfigMiYa.get("userid") + ",OPERATOR_ID:" + mMapConfigMiYa.get("operator_id") + ")-----");
        return true;
    }

    /**
     * 将txt配置文件转换为Map对象
     *
     * @param filePath 文本文件存放地址
     */
    private Map readKeyValueTxtToMap(String filePath) {
        //循环直至返回map

        final HashMap<String, String> keyValueMap = new HashMap<>();//保存读取数据keyValueMap
        //每一个循环读取一组key=value
        try {
            final BufferedInputStream open = new BufferedInputStream(new FileInputStream(filePath));
            final byte[] readArray = new byte[open.available()];
            open.read(readArray);
            open.close();
            final StringTokenizer allLine = new StringTokenizer(new String(readArray, "GB2312"), "\r\n");//以"\r\n"作为key=value的分解标志
            while (allLine.hasMoreTokens()) {
                final StringTokenizer oneLine = new StringTokenizer(allLine.nextToken(), "=");//以"="作为分解标志
                final String leftKey = oneLine.nextToken();//读取第一个字符串key
                //如果包括特殊字符#,则代表为注释行，无需处理
                if (!(leftKey.replaceAll("[\u4e00-\u9fa5]*[a-z]*[A-Z]*\\d*-*_*\\s*", "").length() == 0)) {
                    continue;
                }
                if (!oneLine.hasMoreTokens()) {
                    break;
                }
                final String rightValue = oneLine.nextToken();//读取第二个字符串value
                keyValueMap.put(leftKey, rightValue);
            }
            return keyValueMap;
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("获取参数设置" + e.toString());
            return keyValueMap;
        }
    }

}
