package com.hisense.hiaposcmp.service.payment;

import androidx.annotation.StringDef;

import com.hisense.hiaposcmp.constant.GeneralSettings;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * 支付通道注解
 *
 * @author xieliyan 2018-04-19 HIPOSAPP-408
 */

public final class AggregatePaymentChannel {

    /**
     * 与支付通道相同
     */
    public static final String CHANNEL = "CHANNELEnable";
    /**
     * 赢商通
     */
    public static final String YST = "YSTEnable";
    /**
     * 米雅
     */
    public static final String MIYA = "MIYAEnable";
    /**
     * 直连支付方式
     */
    public static final String HXPAY = "HXEnable";

    @StringDef({CHANNEL, YST, MIYA, HXPAY})
    @Retention(RetentionPolicy.SOURCE)
    public @interface Aggregate {
    }

    /**
     * 返回是否选择与支付通道相同
     *
     * @param value 选择的支付通道
     * @return true: 选择与支付通道相同，false:不同
     */
    public static boolean isCHANNEL(String value) {
        return value.equals(CHANNEL);
    }

    public static boolean isCHANNEL() {
        return CHANNEL.equals(GeneralSettings.INSTANCE.getAggregatePaymentChannel());
    }

    /**
     * 返回是否选择赢商通
     *
     * @param value 选择的支付通道
     * @return true: 选择赢商通，false:未选择赢商通
     */
    public static boolean isYST(String value) {
        return value.equals(YST);
    }

    public static boolean isYST() {
        return YST.equals(GeneralSettings.INSTANCE.getAggregatePaymentChannel());
    }

    /**
     * 返回是否选择米雅支付
     *
     * @param value 选择的支付通道
     * @return true: 选择米雅支付，false:未选择米亚支付
     */
    public static boolean isMIYA(String value) {
        return value.equals(MIYA);
    }

    public static boolean isMIYA() {
        return MIYA.equals(GeneralSettings.INSTANCE.getAggregatePaymentChannel());
    }

    /**
     * 返回是否选择直连
     *
     * @param value 选择的支付通道
     * @return true: 选择直连支付，false:未选择直连支付
     */
    public static boolean isHXPAY(String value) {
        return value.equals(HXPAY);
    }

    public static boolean isHXPAY() {
        return HXPAY.equals(GeneralSettings.INSTANCE.getAggregatePaymentChannel());
    }
}
