/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.service.payment;

import android.app.Activity;
import android.content.Context;
import android.view.View;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.AppContext;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.webapi.payment.PaymentService;
import com.hisense.webapi.Callback;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * 支付方式服务。
 *
 * @author zhangsai        2018-01-12 HIPOSAPP-272 支付方式按使用频次排序
 * @author zhangsai        2018-07-16 HIPOSAPP-668 加载支付方式时初始化图标
 * @author wangyaogang     2018-12-27 HIPOSAPP-876 中行银联二维码支付对接,增加云闪付支付方式
 * @author guolina         2019-03-14 HIPOSAPP-1052 支持其他支付方式
 * @author xieliyan        2019-08-07 HIPOSAPP-1226  增加SCRM积分支付
 * @author xieliyan        2019-08-07 HIPOSAPP-1258  增加SCRM余额支付
 * @author xieliyan        2019-08-07 	HIPOSAPP-1238  增加SCRM券支付
 * @author wangyaogang     2019-10-08 HIPOSAPP-1289 对接利客来一点通支付
 * @author dukunbiao(null) 2018-05-15 生成自助自助方式
 * @author wangyaogang     2019-12-27 HIAPOSCMP-455 标准版支付功能开发,增加标准版图标
 * @author guolina         2020-03-13 HIAPOSCMP-587/588 聚合支付银联商务版换成云闪付图标
 */
public class PayMethodService {

    private static Map<String, Integer> payMethodIcons;

    private PayMethodService() {
        // NO CODE HERE. 避免初始化
    }

    /**
     * 创建支付方式精简布局。
     *
     * @param activity 布局所在的 Activity 实例
     * @param listener 支付方式点击的处理程序
     */
    public static void createCompactLayout(final Activity activity,
                                           final View.OnClickListener listener,
                                           final Callback<List<PayMethod>> callback) {
        // 查询支付方式
        PayMethodService.getAll(activity, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                PaymentHelper.createCompactLayout(activity, payMethods, listener);
                if (callback != null) {
                    callback.onSuccess(payMethods);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                CommonTools.toastShort(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
            }
        });
    }

    public static void createCompactLayout(final Activity activity, final View.OnClickListener listener) {
        createCompactLayout(activity, listener, null);
    }

    public static void createFullLayout(final Activity activity, final View.OnClickListener listener) {
        PayMethodService.getAll(activity, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                PaymentHelper.createFullLayout(activity, payMethods, listener);
            }

            @Override
            public void onError(Throwable throwable) {
                CommonTools.toastShort(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
            }
        });
    }

    /**
     * 蚂蚁金服支付方式布局。
     *
     * @param activity 布局所在的 Activity 实例
     * @param listener 支付方式点击的处理程序
     */
    public static void createAliLayout(final Activity activity, final View.OnClickListener listener) {

        PayMethodService.getAll(activity, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                PaymentHelper.createAliLayout(activity, payMethods, listener);
            }

            @Override
            public void onError(Throwable throwable) {
                CommonTools.toastShort(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
            }
        });
    }

    /**
     * 蚂蚁金服支付方式布局。
     *
     * @param activity 布局所在的 Activity 实例
     */
    public static void createAliLayout(final Activity activity, final Callback<List<PayMethod>> callback) {

        PayMethodService.getAll(activity, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                if (callback != null) {
                    callback.onSuccess(payMethods);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                CommonTools.toastShort(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
            }
        });
    }

    /**
     * 微信支付方式布局。
     *
     * @param activity 布局所在的 Activity 实例
     */
    public static void createWxLayout(final Activity activity, final Callback<List<PayMethod>> callback) {
        PayMethodService.getAll(activity, new Callback<List<PayMethod>>() {
            @Override
            public void onSuccess(List<PayMethod> payMethods) {
                if (callback != null) {
                    callback.onSuccess(payMethods);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                CommonTools.toastShort(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
                callback.onError(throwable);
            }
        });
    }

    /**
     * 返回支付方式。
     *
     * @param context  Context
     * @param callback 返回查询到的支付方式的回调方法
     */
    public static void getAll(final Context context, final Callback<List<PayMethod>> callback) {
        // 从 AppContext 中获取缓存
        List<PayMethod> payMethods = AppContext.payMethods();
        if (ListUtils.isEmpty(payMethods)) {
            // 首先通过 WebAPI 服务从后台加载支付方式
            PaymentService.getPayMethods(new Callback<List<PayMethod>>() {
                @Override
                public void onSuccess(List<PayMethod> payMethods) {
                    try {
                        if (!CommonUtils.isUniformStyle()) {
                            Iterator<PayMethod> iterator = payMethods.iterator();
                            while (iterator.hasNext()) {
                                PayMethod payMethod = iterator.next();

                                if (payMethod.isHsyType() || payMethod.isAbandon()) {
                                    iterator.remove();
                                }
                            }
                        }
                    } catch (Exception e) {
                        LoggerUtils.getLogger().debug(String.format("移除支付方式时发生错误: %s", e.getMessage()));
                    }

                    // 对后台返回的支付方式进行排序
                    List<PayMethod> sorted = new PayMethodRepository(context).sortByFrequency(payMethods);

                    // 缓存
                    AppContext.payMethods(sorted);

                    // 初始化支付方式图标
                    initPayMethodIcon(sorted);

                    if (callback != null) {
                        callback.onSuccess(sorted);
                    }
                }

                @Override
                public void onError(Throwable throwable) {
                    LoggerUtils.getLogger().debug(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
                    CommonTools.toastShort(String.format("查询支付方式时发生错误: %s", throwable.getMessage()));
                }
            });
        } else {
            if (callback != null) {
                // 如果有本地缓存则直接返回缓存数据
                callback.onSuccess(payMethods);
            }
        }
    }

    public static void updateFrequency(final Context context, final PayMethod method) {
        new PayMethodRepository(context).updateFrequency(method);
    }

    /**
     * 选中选择的支付方式，并重新布局。
     *
     * @param activity 布局所在的 Activity 实例
     * @param method   选择的支付方式
     * @param listener 支付方式点击的处理程序
     */
    public static void selectPayMethod(Activity activity, PayMethod method, View.OnClickListener listener) {
        PaymentHelper.selectPayMethod(activity, method, listener);
    }

    /**
     * 根据编码获取支付图标的资源 ID。
     *
     * @param code 支付方式编码
     * @return 支付图标的资源 ID
     */
    public static Integer getIconByCode(String code) {
        if (payMethodIcons != null && payMethodIcons.containsKey(code)) {
            return payMethodIcons.get(code);
        }
        return R.drawable.ic_logo_gray;
    }

    /**
     * 根据编码获取支付图标的资源 ID。
     *
     * @return 支付图标的资源 ID
     */
    public static Integer getIconByCode(PayMethod payMethod) {
        if (payMethodIcons != null && payMethodIcons.containsKey(payMethod.code())) {
            return payMethodIcons.get(payMethod.code());
        }

        return PayMethodService.getPayMethodIcon(payMethod);
    }

    private static void initPayMethodIcon(List<PayMethod> payMethods) {
        if (payMethods == null) {
            payMethodIcons = new HashMap<>();
            return;
        }

        Map<String, Integer> map = new HashMap<>(payMethods.size());
        for (PayMethod method : payMethods) {
            int icon = getPayMethodIcon(method);

            if (!map.containsKey(method.code())) {
                map.put(method.code(), icon);
            }
        }

        payMethodIcons = map;
    }

    private static int getPayMethodIcon(PayMethod payMethod) {
        if (CommonUtils.isStdStyle()) {
            return getStdPayMethodIcon(payMethod);
        }
        int icon;
        if (payMethod.isCashType()) {
            icon = R.drawable.ic_cash;
        } else if (payMethod.isCardType()) {
            icon = R.drawable.ic_value_card;
        } else if (payMethod.isPointsType()) {
            icon = R.drawable.ic_vip_points;
        } else if (payMethod.isRemainType() || payMethod.isUnionWallet()) {
            icon = R.drawable.ic_card_balance;
        } else if (payMethod.isCouponType() || payMethod.isUnionCoupon()) {
            icon = R.drawable.ic_crm_coupon;
        } else if (payMethod.isAliPay()) {
            icon = R.drawable.ic_alipay;
        } else if (payMethod.isWxPay()) {
            icon = R.drawable.ic_wechat_pay;
        } else if (payMethod.isMobilePay()) {
            if (GeneralSettings.INSTANCE.getPaymentChannel().isUMS()) {
                icon = R.drawable.ic_union_cloud;
            } else {
                icon = R.drawable.qrpay1;
            }

        } else if (payMethod.isUnionType() || payMethod.isUccbType()) {
            icon = R.drawable.ic_union_pay;
        } else if (payMethod.isUnionCardType()) {
            icon = R.drawable.ic_card_pink;
        } else if (payMethod.isIcbcPointPay()) {
            icon = R.drawable.ic_points_pay;
        } else if (payMethod.isIcbcInstallmentPay()) {
            icon = R.drawable.ic_installment;
        } else if (payMethod.isAllInPayType() || payMethod.isAllInPayByPayType()) {
            icon = R.drawable.qrpay1;
        } else if (payMethod.isCashByPayType()) {
            payMethod.asCash();
            icon = R.drawable.ic_cash;
        } else if (payMethod.isCardByPayType()) {
            icon = R.drawable.ic_card_green;
        } else if (payMethod.isCouponByPayType()) {
            icon = R.drawable.ic_crm_coupon;
        } else {
            payMethod.asCash();
            icon = R.drawable.ic_cash;
        }

        return icon;
    }

    private static int getStdPayMethodIcon(PayMethod payMethod) {
        int icon;
        if (payMethod.isCashType()) {
            icon = R.drawable.ic_std_cash;
        } else if (payMethod.isCardType()) {
            icon = R.drawable.ic_std_card;
        } else if (payMethod.isPointsType()) {
            icon = R.drawable.ic_vip_points;
        } else if (payMethod.isRemainType() || payMethod.isUnionWallet()) {
            icon = R.drawable.ic_card_balance;
        } else if (payMethod.isCouponType() || payMethod.isUnionCoupon()) {
            icon = R.drawable.ic_crm_coupon;
        } else if (payMethod.isAliPay()) {
            icon = R.drawable.ic_std_alipay;
        } else if (payMethod.isWxPay()) {
            icon = R.drawable.ic_std_wechat;
        } else if (payMethod.isMobilePay()) {
            icon = R.drawable.qrpay1;
        } else if (payMethod.isUnionCloudPay()) {
            icon = R.drawable.ic_std_union_cloud;
        } else if (payMethod.isUnionType() || payMethod.isUccbType()) {
            icon = R.drawable.ic_union_pay;
        } else if (payMethod.isUnionCardType()) {
            icon = R.drawable.ic_card_pink;
        } else if (payMethod.isIcbcPointPay()) {
            icon = R.drawable.ic_points_pay;
        } else if (payMethod.isIcbcInstallmentPay()) {
            icon = R.drawable.ic_installment;
        } else if (payMethod.isAllInPayType() || payMethod.isAllInPayByPayType()) {
            icon = R.drawable.qrpay1;
        } else if (payMethod.isCashByPayType()) {
            payMethod.asCash();
            icon = R.drawable.ic_std_cash;
        } else if (payMethod.isCardByPayType()) {
            icon = R.drawable.ic_card_green;
        } else if (payMethod.isCouponByPayType()) {
            icon = R.drawable.ic_crm_coupon;
        } else {
            payMethod.asCash();
            icon = R.drawable.ic_std_cash;
        }

        return icon;
    }

    public static int getAliPayMethodIcon(PayMethod payMethod) {
        int icon;
        if (payMethod.isAliFacePay()) {
            if (isAliFacePayLimit()) {
                icon = R.drawable.facepay_ban;
            } else {
                icon = R.drawable.facepay;
            }
        } else if(payMethod.isAliPay()){
            icon = R.drawable.alipay;
        }else {
            icon=R.drawable.ic_qrpay;
        }

        return icon;
    }

    public static boolean isAliFacePayLimit() {
        return BigDecimalUtils.isGreater(MainSaleModel.getInstance().getSaleModel().getTotalUnPaid(), String.valueOf(GeneralSettings.INSTANCE.getAliFacePayLimit()));
    }

    public static int getWxPayMethodIcon(PayMethod payMethod) {
        int icon;
        if (payMethod.isWxFacePay()) {
            icon = R.drawable.ic_wx_facepay;
        } else if (payMethod.isWxPay()) {
            icon = R.drawable.ic_wx_pay_icon;
        } else if (payMethod.isAliPay()) {
            icon = R.drawable.ic_ali_lay_icon;
        } else if (payMethod.isAllOrChannelType()) {
            icon = R.drawable.qrpay1;
        } else if (payMethod.isVipCoupon()) {
            icon = R.drawable.ic_crm_coupon;
        } else if (payMethod.isCard()) {
            icon = R.drawable.ic_card_pay;
        } else {
            icon = R.drawable.qrpay1;
        }

        return icon;
    }
}
