/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.payment;

/**
 * 支付通道
 *
 * @author xieliyan    HIPOSAPP-262 支持建行收单
 * @author wangyaogang HIPOSAPP-813 支持中行收单
 * @author lizhixin    2018-06-12 HIPOSAPP-520 加入工行收单支付通道
 */
public enum PaymentChannel {

    /**
     * 银联商务
     */
    UMS("UMS"),
    /**
     * 建行收单
     */
    CCB("CCB"),
    /**
     * 工行
     */
    ICBC("ICBC"),
    /**
     * 中行收单
     */
    BOC("BOC");

    private String value;

    /**
     * 初始化一个 PaymentChannel 枚举。
     *
     * @param value 支付通道的代码值。
     */
    PaymentChannel(String value) {
        this.value = value;
    }

    /**
     * 返回调用支付通道的代码值。
     *
     * @return 支付通道的代码值。
     */
    public String value() {
        return this.value;
    }

    /**
     * 是否为银联商务通道
     *
     * @return true：是银联商务通道；false：其它
     */
    public boolean isUMS() {
        return this.equals(PaymentChannel.UMS);
    }

    /**
     * 是否为建行收单通道
     *
     * @return true：是建行收单通道；false：其它
     */
    public boolean isCCB() {
        return this.equals(PaymentChannel.CCB);
    }

    /**
     * 是否为工行通道
     *
     * @return true：是建行收单通道；false：其它
     */
    public boolean isICBC() {
        return this.equals(PaymentChannel.ICBC);
    }

    /**
     * 是否为中行收单通道
     *
     * @return true：是中行收单通道；false：其它
     */
    public boolean isBOC() {
        return this.equals(PaymentChannel.BOC);
    }
}
