/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.service.sale;

import android.content.Context;

import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.sale.models.CmpSaleNoProvider;
import com.hisense.hipos.sale.models.ISaleNoProvider;
import com.hisense.hipos.webapi.sale.SaleBeginResultData;
import com.hisense.hipos.webapi.sale.SaleService;
import com.hisense.webapi.Callback;

/**
 * 流水号服务。
 *
 * @author zhangsai    2018-01-10
 * @author zhangsai    2018-05-04 HIPOSAPP-433  根据产品代码初始化流水号提供程序
 * @author zhangsai    2018-06-11 HIPOSAPP-508  优化流水号取值算法，避免流水号重复
 * @author guolina     2018-12-25 HIPOSAPP-850  获取新流水号兼容POS中间件,规则可以在中间件设置
 * @author guolina     2019-03-04 HIPOSAPP-1028 若修改了组织号或者收款机号后，废弃最新流水号重新获取
 * @author guolina     2019-04-16 HIPOSAPP-1064 获取流水号时，如果本地数据库已存在该流水号则获取新的流水号
 * @author wangyaogang 2019-05-24 HIPOSAPP-1127 优化获取WebAPI 服务实例的方法
 */
public class SaleNoService {
    private static String latestSaleNo = null; //最新流水号

    /**
     * 返回当前的最大流水号。
     *
     * @param context Context
     * @param orgCode 组织编码
     * @param posNo   收款机号
     * @return 当前的最大流水号
     */
    public static String getMax(Context context, String orgCode, String posNo) {
        return new SaleRepository(context).getMaxSaleNo(orgCode, posNo);
    }

    /**
     * 返回一个新的流水号。
     *
     * @param context  Context
     * @param accDate  结账日期
     * @param callback 回调方法
     */
    public static void next(Context context, String accDate, final Callback<String> callback) {
        if (latestSaleNo == null) {
            SaleService.saleBegin(accDate, null, null, null, new Callback<SaleBeginResultData>() {
                @Override
                public void onSuccess(SaleBeginResultData data) {
                    latestSaleNo = data.saleNo;
                    if (callback != null) {
                        callback.onSuccess(latestSaleNo);
                    }
                }

                @Override
                public void onError(Throwable throwable) {
                    if (callback != null) {
                        callback.onError(throwable);
                    }
                }
            });
        } else {
            BaseSaleModel sale = new SaleRepository(context).get(latestSaleNo);
            if (sale == null) {
                if (callback != null) {
                    callback.onSuccess(latestSaleNo);
                }
            } else {
                abandon();
                next(context, accDate, callback);
            }
        }
    }

    /**
     * 保存流水号。
     *
     * @param context Context
     * @param orgCode 组织编码
     * @param posNo   收款机号
     * @param saleNo  流水号
     */
    public static void save(Context context, final String orgCode, final String posNo, final String saleNo) {
        ISaleNoProvider iSaleNoProvider = getProvider(context);
        iSaleNoProvider.save(orgCode, posNo, saleNo);

        latestSaleNo = null;
    }

    /**
     * 废弃当前最新流水号。
     */
    public static void abandon() {
        if (latestSaleNo != null) {
            latestSaleNo = null;
        }
    }

    /**
     * 返回一个新的 {@code ISaleNoProvider} 实例。
     *
     * @param context Context
     * @return {@code ISaleNoProvider} 实例
     */
    private static ISaleNoProvider getProvider(Context context) {
        return new CmpSaleNoProvider(new SerialNoRepository(context));
    }
}
