/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.sale;

import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.text.TextUtils;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.crm.card.VipCard;
import com.hisense.database.sqlite.QueryCallback;
import com.hisense.database.sqlite.SQLiteRepository;
import com.hisense.database.sqlite.TransactionCallback;
import com.hisense.hiaposcmp.db.PadPosDatabase;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.model.SalePluModel;
import com.hisense.hipos.plu.PluType;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.User;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 销售流水资源库。
 *
 * @author zhangsai    2018-01-11
 * @author zhangsai    2018-12-22 HIPOSAPP-931  新增流水保存/读取；流水状态更新；读取交易凭证号
 * @author zhangsai    2019-01-07 HIPOSAPP-956  支持根据商品分摊计算商品实际发生价
 * @author zhangsai    2019-01-08 HIPOSAPP-966  除商品优惠外，其它促销计入分摊
 * @author guolina     2019-04-10 HIPOSAPP-1065 支持清理本地三个月前已上传流水信息
 * @author wangyaogang 2019-05-13 HIPOSAPP-1087 控制每台机器收款的小数位
 * @author wangyaogang 2019-05-15 HIPOSAPP-1087 优化[控制每台机器收款的小数位]逻辑,去掉isNeedRound
 * @author wangyaogang 2019-06-05 HIPOSAPP-1154 易收银销售小票-打印商品"货号"字段，tSalSalePlu表增加udp1到5字段
 * @author zhangsai    2019-06-13 HIPOSAPP-1169 Sale.payments() 属性返回类型改为 ISalePayment
 * @author wangyaogang 2019-06-14 HIPOSAPP-1098 保存本地流水时增加AuthNo字段
 * @author zhangsai    2019-06-17 HIPOSAPP-1174 使用 PadPosDatabase 初始化 SQLite 数据库
 * @author wangyaogang 2019-07-29 HIPOSAPP-1224 修复重新上传流水时,payNo记录不正确的问题
 * @author wangyaogang 2019-08-15 HIPOSAPP-1276 修改重传流水时，流水类型的判断方式
 * @author wangyaogang 2019-08-22 HIPOSAPP-1286 读取本地流水时获取vipCardNo
 * @author wangyaogang 2019-08-30 HIPOSAPP-1284 保存和读取本地支付表时增加Remark字段
 * @author xieliyan    2019-10-18 HIPOSAPP-1328 新增建行直连通道
 * @author guolina     2019-10-21 HIPOSAPP-1317 利客来未上传流水自动上传
 * @author wangyaogang 2020-03-23 HIPOSAPP-1392 调整删除过期流水逻辑（改为删除销售日期三个月之前流水，不管是否成功
 */
public class SaleRepository extends SQLiteRepository {

    /**
     * 初始化一个 {@code SaleRepository} 的实例。
     *
     * @param context Context
     */
    public SaleRepository(Context context) {
        super(new PadPosDatabase(context));
    }

    @Override
    protected void executeCursor(String sql, QueryCallback callback) throws SQLException {
        super.executeCursor(sql, callback);
    }

    @Override
    protected void executeCursor(String sql, String[] args, QueryCallback callback) throws SQLException {
        super.executeCursor(sql, args, callback);
    }

    @Override
    protected String executeScalar(String sql, String... args) throws SQLException {
        return super.executeScalar(sql, args);
    }

    /**
     * 读取流水。
     *
     * @param saleNo 流水号
     * @return 实现 {@code BaseSaleModel} 接口的流水实例。
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public BaseSaleModel get(String saleNo) throws SQLException {
        return getSale(saleNo);
    }

    /**
     * 读取销售/退货流水。
     *
     * @param saleNo 流水号
     * @return 实现 {@code BaseSaleModel} 接口的流水实例。
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private BaseSaleModel getSale(String saleNo) throws SQLException {
        final ArrayList<BaseSaleModel> result = new ArrayList<>();

        String sql = "SELECT * FROM tSalSale WHERE SaleNo = ?";
        this.executeCursor(sql, new String[]{saleNo}, new QueryCallback() {
            @Override
            public void onSuccess(Cursor cursor) {
                String saleNo, returnSaleNo = "";
                String saleTotal = getString(cursor, "Total");
                boolean isReturn = false;

                String udp2 = getString(cursor, "udp2");
                if (TextUtils.isEmpty(udp2)) {
                    saleNo = getString(cursor, "SaleNo");
                } else {
                    returnSaleNo = getString(cursor, "SaleNo");
                    saleNo = getString(cursor, "udp2");
                    isReturn = true;
                }

                String cardNo = getString(cursor, "CardNo");
                String vipCardNo = getString(cursor, "vipCardNo");

                VipCard card = null;
                if (cardNo != null && cardNo.length() > 0) {
                    card = new VipCard.Builder(cardNo, "")
                            .cardType(getString(cursor, "CardType"), null)
                            .vipCardNo(vipCardNo)
                            .build();
                }

                User cashier = new User(
                        getString(cursor, "CshID"),
                        getString(cursor, "CshCode"), null);

                SaleModel saleModel = new SaleModel();
                saleModel.setSaleNo(saleNo);
                saleModel.setOrgCode(getString(cursor, "OrgCode"));
                saleModel.setPosNo(getString(cursor, "PosNo"));
                saleModel.setCashier(cashier);
                saleModel.setAccDate(getString(cursor, "AccDate"));
                saleModel.setTranType(TranType.valueOf(Integer.valueOf(getString(cursor, "TranType"))));
                saleModel.addVip(card);
                saleModel.setSaleTime(getString(cursor, "SaleDate"));
                saleModel.setUdp1(getString(cursor, "udp1"));
                saleModel.setUdp2(getString(cursor, "udp2"));
                saleModel.setUdp3(getString(cursor, "udp3"));
                saleModel.setUdp4(getString(cursor, "udp4"));
                saleModel.setUdp5(getString(cursor, "udp5"));

                Map<Integer, Double> itemTotalRecv = getSaleItems(saleNo, saleModel);
                getSaleItemDiscounts(saleNo, saleModel);
                getSalePayments(saleNo, saleModel);
                String authNo = getString(cursor, "AuthNo");

                if (!isReturn) {// 普通流水
                    result.add(saleModel);
                } else {        // 退货流水
                    ReturnSaleModel returnSaleModel = new ReturnSaleModel();
                    returnSaleModel.setSaleNo(returnSaleNo);
                    returnSaleModel.setSaleModel(saleModel);
                    returnSaleModel.setSaleTime(saleModel.getSaleTime());
                    returnSaleModel.setAuthNo(authNo);
                    returnSaleModel.setPlus(saleModel.getPlus());
                    returnSaleModel.setPays(saleModel.getPays());

                    returnSaleModel.setUdp2(saleModel.getSaleNo());
                    returnSaleModel.setUdp4(saleModel.getUdp4());
                    result.add(returnSaleModel);
                }
            }
        });

        if (result.size() > 0) {
            return result.get(0);
        } else {
            return null;
        }
    }

    /**
     * 读取流水商品。
     *
     * @param saleNo    流水号
     * @param saleModel 流水
     * @return 退货商品实退金额的集合
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private Map<Integer, Double> getSaleItems(String saleNo, final SaleModel saleModel) throws SQLException {
        final Map<Integer, Double> itemTotalRecv = new HashMap<>();

        String sql = "SELECT * FROM tSalSalePlu WHERE SaleNo = ?";
        this.executeCursor(sql, new String[]{saleNo}, new QueryCallback() {
            @Override
            public void onSuccess(Cursor cursor) {
                do {
                    String pluCode = getString(cursor, "PluCode");
                    String pluId = getString(cursor, "PluID");
                    String pluName = getString(cursor, "PluName");

                    String barCode = getString(cursor, "BarCode");
                    String brandCode = getString(cursor, "BrandCode");
                    String unit = getString(cursor, "Unit");
                    String spec = getString(cursor, "Spec");

                    String depCode = getString(cursor, "DepCode");
                    String depId = getString(cursor, "DepID");
                    String depName = getString(cursor, "DepName");

                    String shopCode = getString(cursor, "ShopCode");
                    String shopId = "";

                    String pluType = getString(cursor, "PluType");
                    double quantity = getDouble(cursor, "Quantity");
                    String cargoNo = getString(cursor, "udp5");

                    SalePluModel salePluModel = new SalePluModel();
                    salePluModel.setPluCode(pluCode);
                    salePluModel.setPluId(pluId);
                    salePluModel.setPluName(pluName);
                    salePluModel.setBarCode(barCode);
                    salePluModel.setBrandCode(brandCode);
                    salePluModel.setUnit(unit);
                    salePluModel.setSpec(spec);
                    salePluModel.setCargoNo(cargoNo);
                    salePluModel.setClassCode(getString(cursor, "ClassCode"));
                    salePluModel.setDepCode(depCode);
                    salePluModel.setDepId(depId);
                    salePluModel.setDepName(depName);
                    salePluModel.setShopCode(shopCode);
                    salePluModel.setShopId(shopId);
                    salePluModel.setTagPrice(BigDecimalUtils.absToString(getDouble(cursor, "TagPrice")));
                    salePluModel.setPluType(PluType.valueOfDefined(pluType));

                    salePluModel.setClkId(getString(cursor, "ClkID"));
                    salePluModel.setClkCode(getString(cursor, "ClkCode"));
                    salePluModel.setPrice(getString(cursor, "Price"));
                    salePluModel.setQuantity(BigDecimalUtils.absToString(quantity));
                    int itemNo = getInt(cursor, "ItemNo");
                    salePluModel.setItemNo(itemNo);
                    double totalRecv = getDouble(cursor, "TotalRecv");
                    salePluModel.setTotalRecv(totalRecv);
                    salePluModel.setTotal(getString(cursor, "Total"));
                    salePluModel.setTotalDsc(getString(cursor, "TotalDsc"));
                    salePluModel.setTotalRound(getString(cursor, "TotalRound"));

                    saleModel.addPlu(salePluModel);

                    itemTotalRecv.put(itemNo, totalRecv);
                } while (cursor.moveToNext());
            }
        });

        return itemTotalRecv;
    }

    /**
     * 读取流水商品优惠。
     *
     * @param saleNo    流水号
     * @param saleModel 流水
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private void getSaleItemDiscounts(String saleNo, final SaleModel saleModel) throws SQLException {
        String sql = "SELECT * FROM tSalSalePluDsc WHERE SaleNo = ?";
        this.executeCursor(sql, new String[]{saleNo}, new QueryCallback() {
            @Override
            public void onSuccess(Cursor cursor) {
                do {
                    int itemNo = getInt(cursor, "ItemNo");
                    String dscType = getString(cursor, "DscType");
                    String dscCode = getString(cursor, "DscCode");
                    double totalDsc = getDouble(cursor, "DscTotal");
                    String pluId = getString(cursor, "PluID");

                    SalePluDscModel salePluDscModel = new SalePluDscModel();
                    salePluDscModel.setItemNo(itemNo);
                    salePluDscModel.setDscProType(dscType);
                    salePluDscModel.setDscCode(dscCode);
                    salePluDscModel.setTotalDsc(totalDsc);
                    salePluDscModel.setPluId(pluId);

                    List<BaseSalePluModel> plus = saleModel.getPlus();
                    for (int i = 0; i < plus.size(); i++) {
                        SalePluModel model = (SalePluModel) plus.get(i);
                        if (salePluDscModel.getItemNo() == model.getItemNo()) {
                            //添加优惠
                            model.addPluDsc(salePluDscModel);
                            break;
                        }
                    }
                } while (cursor.moveToNext());
            }
        });
    }

    /**
     * 读取流水支付信息。
     *
     * @param saleNo    流水号
     * @param saleModel 流水
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private void getSalePayments(String saleNo, final SaleModel saleModel) throws SQLException {
        String sql = "SELECT * FROM tSalSalePay WHERE SaleNo = ?";
        this.executeCursor(sql, new String[]{saleNo}, new QueryCallback() {
            @Override
            public void onSuccess(Cursor cursor) {
                do {
                    SalePayModel salePayModel = new SalePayModel();
                    salePayModel.setPayCode(getString(cursor, "PayCode"));
                    salePayModel.setPayName(getString(cursor, "PayName"));
                    salePayModel.setPayType(getString(cursor, "UDP1"));
                    salePayModel.setBuyerLogonId(getString(cursor, "PayNo"));
                    salePayModel.setTotalPaid(BigDecimalUtils.absToString(getDouble(cursor, "TotalPaid")));
                    salePayModel.setTotalRecv(BigDecimalUtils.absToString(getDouble(cursor, "TotalRecv")));
                    salePayModel.setRemark(getString(cursor, "Remark"));
                    salePayModel.setUdp1(getString(cursor, "UDP1"));
                    salePayModel.setUdp2(getString(cursor, "UDP2"));
                    salePayModel.setUdp3(getString(cursor, "UDP3"));
                    salePayModel.setUdp4(getString(cursor, "UDP4"));
                    salePayModel.setUdp5(getString(cursor, "UDP5"));

                    saleModel.addPay(salePayModel);
                } while (cursor.moveToNext());
            }
        });
    }


    /**
     * 返回 {@param accDate} 本地未上传流水的流水号。
     *
     * @return 本地未上传流水的流水号集合
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public Collection<String> getNotUploadSaleNo() throws SQLException {
        final ArrayList<String> notUploadSales = new ArrayList<>();
        String sql = "SELECT SaleNo FROM tSalSale " +
                " WHERE (Tag = ? OR Tag = ? OR Tag = ?)  ORDER BY SaleNo ASC";
        this.executeCursor(sql, new String[]{
                String.valueOf(SaleState.DEFAULT.value()),
                String.valueOf(SaleState.VIP_FAILURE.value()),
                String.valueOf(SaleState.FAILURE.value())}, new QueryCallback() {
            @Override
            public void onSuccess(Cursor cursor) {
                do {
                    notUploadSales.add(getString(cursor, "SaleNo"));
                } while (cursor.moveToNext());
            }
        });
        return notUploadSales;
    }

    /**
     * 返回本地90天前已上传流水的流水号。
     * (修改为删除销售日期三个月之前的流水，不管成功与否)
     *
     * @return 本地90天前已上传流水的流水号集合
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public Map<String, String> getUploadedSaleNo() throws SQLException {
        final Map<String, String> UploadSalesMap = new HashMap<>();

        String sql = "SELECT * FROM tSalSale " +
                " WHERE SaleDate < date('now', '-90 day') ORDER BY SaleNo ASC";
        this.executeCursor(sql, new QueryCallback() {
            @Override
            public void onSuccess(Cursor cursor) {
                do {
                    UploadSalesMap.put(getString(cursor, "SaleNo"), getString(cursor, "SaleDate"));
                } while (cursor.moveToNext());
            }
        });
        return UploadSalesMap;
    }

    /**
     * 删除 {@param saleNo} 指定流水号的流水信息。
     *
     * @param saleNo 流水号
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public void delete(String saleNo) throws SQLException {
        this.execute("DELETE FROM tSalSale WHERE SaleNo = ?", saleNo);
        this.execute("DELETE FROM tSalSalePlu WHERE SaleNo = ?", saleNo);
        this.execute("DELETE FROM tSalSalePluDsc WHERE SaleNo = ?", saleNo);
        this.execute("DELETE FROM tSalSalePay WHERE SaleNo = ?", saleNo);

    }


    /**
     * 返回 {@param saleNo} 指定流水的状态。
     *
     * @param saleNo 流水号
     * @return 流水状态
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public SaleState getState(String saleNo) throws SQLException {
        String sql = "SELECT Tag FROM tSalSale WHERE SaleNo = ?";
        return SaleState.valueOf(getInteger(this.executeScalar(sql, saleNo), 0));
    }

    /**
     * 返回 {@param payNo} 指定支付交易的交易凭证号。
     *
     * @param saleNo 流水号
     * @param payNo  支付号
     * @return 交易凭证号
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public String getTraceNoByPayNo(String saleNo, String payNo) throws SQLException {
        String sql = "SELECT UDP5 FROM tSalSalePay WHERE SaleNo = ? AND PayNo=?";
        return this.executeScalar(sql, saleNo, payNo);
    }

    /**
     * 返回 {@param serialNo} 指定支付交易的交易凭证号。
     *
     * @param saleNo   流水号
     * @param serialNo 支付交易序号
     * @return 交易凭证号
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public String getTraceNoBySerialNo(String saleNo, int serialNo) throws SQLException {
        String sql = "SELECT UDP5 FROM tSalSalePay WHERE SaleNo = ? AND serialNo=?";
        return this.executeScalar(sql, saleNo, String.valueOf(serialNo));
    }

    /**
     * 返回 {@param serialNo} 指定支付交易的终端号。
     *
     * @param saleNo   流水号
     * @param serialNo 支付交易序号
     * @return 终端号
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public String getTermNoBySerialNo(String saleNo, int serialNo) throws SQLException {
        String sql = "SELECT UDP2 FROM tSalSalePay WHERE SaleNo = ? AND serialNo=?";
        return this.executeScalar(sql, saleNo, String.valueOf(serialNo));
    }

//    /**
//     * 返回 {@param serialNo} 指定支付交易的商户订单号。
//     *
//     * @param saleNo   流水号
//     * @param serialNo 支付交易序号
//     * @return 商户订单
//     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
//     */
//    public String getOutTradeNoBySerialNo(String saleNo, int serialNo) throws SQLException {
//        String sql = "SELECT UDP2 FROM tSalSalePay WHERE SaleNo = ? AND serialNo=?";
//        return this.executeScalar(sql, saleNo, String.valueOf(serialNo));
//    }

    /**
     * 返回 {@param serialNo} 指定支付交易的商户订单号。
     *
     * @param saleNo   流水号
     * @param serialNo 支付交易序号
     * @return 商户订单
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public String getTradeNoAndMchIdBySerialNo(String saleNo, int serialNo) throws SQLException {
        String sql = "SELECT UDP3 FROM tSalSalePay WHERE SaleNo = ? AND serialNo=?";
        return this.executeScalar(sql, saleNo, String.valueOf(serialNo));
    }

    /**
     * 返回指定收款机的最大流水号。
     *
     * @param orgCode 组织编码
     * @param posNo   收款机号
     * @return 流水号
     */
    public String getMaxSaleNo(String orgCode, String posNo) throws SQLException {
        String sql = "SELECT MAX(SaleNo) FROM tSalSale WHERE OrgCode = ? AND PosNo = ? ";
        return this.executeScalar(sql, orgCode, posNo);
    }


    /**
     * 保存流水。
     *
     * @param saleModel 销售/退货流水
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public void save(final BaseSaleModel saleModel) throws SQLException {
        this.execute(new TransactionCallback() {
            @Override
            public void onTransaction(SQLiteDatabase database) {
                saveSale(database, saleModel);
                for (BaseSalePluModel salePluModel : saleModel.getPlus()) {
                    saveSaleItem(database, saleModel, salePluModel);
                }
                for (SalePayModel salePayModel : saleModel.getPays()) {
                    savePayment(database, saleModel, salePayModel);
                }
            }
        });
    }

    /**
     * 保存流水。
     *
     * @param database  打开的 SQLite 数据库
     * @param saleModel 实现 {@code BaseSaleModel} 接口的销售/退货流水
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private void saveSale(SQLiteDatabase database, BaseSaleModel saleModel) throws SQLException {
        String sql = "insert into tSalSale(OrgCode,SaleNo,PosNo,SaleDate,AccDate,CshID,CshCode," +
                "Total,TotalRecv,TotalPaid,TotalChange,TotalRound,TranType," +
                "CardNo,vipCardNo,CardType,Tag,Remark,udp1,udp2,udp3,udp4,udp5,AuthNo) " +
                "values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        MainSaleModel mainSaleModel = MainSaleModel.getInstance();
        String cardNo = "";
        String vipCardNo = "";
        String cardType = "";
        if (saleModel.isVip() && saleModel.getVip().getCard() != null) {
            cardNo = saleModel.getVip().getCard().cardFaceNo();
            vipCardNo = saleModel.getVip().getCard().vipCardNo();
            cardType = saleModel.getVip().getCard().cardType().code();
        }

        this.execute(database, sql,
                mainSaleModel.getShopInfo().getOrgCode(),
                saleModel.getSaleNo(),
                saleModel.getPosNo(),
                saleModel.getSaleTime(),
                saleModel.getAccDate(),
                saleModel.getCashier().getUserId(),
                saleModel.getCashier().getUserCode(),
                saleModel.getTotal(),//应收合计
                saleModel.getTotalRecv(),//实收合计
                saleModel.getTotalPaid(),//支付合计
                saleModel.getTotalChange(),                  //找零合计
                saleModel.getTotalRound(),                  //抹零合计
                String.valueOf(saleModel.getTranType().value()),                //交易类型，默认"1"
                cardNo,
                vipCardNo,//会员卡号
                cardType,// 会员卡类型
                String.valueOf(SaleState.DEFAULT.value()),                  //流水上传状态
                saleModel.getRemark(),
                saleModel.getUdp1(),
                saleModel.getUdp2(),
                saleModel.getUdp3(),
                saleModel.getUdp4(),
                saleModel.getUdp5(),
                (saleModel instanceof ReturnSaleModel) ? ((ReturnSaleModel) saleModel).getAuthNo() : ""
        );

    }

    /**
     * 保存流水商品。
     *
     * @param database  打开的 SQLite 数据库
     * @param saleModel 实现 {@code BaseSaleModel} 接口的销售/退货流水
     * @param pluModel  流水商品
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private void saveSaleItem(SQLiteDatabase database, BaseSaleModel saleModel, BaseSalePluModel pluModel) throws SQLException {
        String sql = "insert into tSalSalePlu(OrgCode ,ShopCode,SaleNo,TranType,PageNo," +
                "                ItemNo,ClkID,ClkCode,PluID,PluCode,PluName,PluType,BarCode,BrandCode,ClassCode," +
                "                Unit,Spec,DepID,DepCode,DepName,Price,TagPrice,Quantity," +
                "                Total,TotalRecv,TotalDsc,TotalRound,InvNo,Remark,udp1,udp5)" +
                "            values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

        MainSaleModel mainSaleModel = MainSaleModel.getInstance();
        this.execute(database, sql,
                mainSaleModel.getShopInfo().getOrgCode(),
                pluModel.getShopCode(),
                saleModel.getSaleNo(),
                String.valueOf(saleModel.getTranType().value()),
                String.valueOf(pluModel.getPageNo()),                   //pageno
                String.valueOf(pluModel.getItemNo()),
                saleModel.getCashier().getUserId(),    //此处赋值为收银员
                saleModel.getCashier().getUserCode(),
                pluModel.getPluId(),
                pluModel.getPluCode(),
                pluModel.getPluName(),
                /*
                 *  商品类型，
                 *  如果pluType为0： 普通商品，则 packType：0
                 *  如果pluType为1： 商品包，则 packType：2
                 *  packType：1表示 拆包后商品明细
                 */
                String.valueOf(pluModel.getPluType().value()),
                pluModel.getBarCode(),
                pluModel.getBrandCode(),
                pluModel.getClassCode(),
                pluModel.getUnit(),
                pluModel.getSpec(),
                pluModel.getDepId(),              //部门id
                pluModel.getDepCode(),
                pluModel.getDepName(),
                pluModel.getPrice(),     //发生价格
                pluModel.getTagPrice(),         //商品标价
                pluModel.getQuantity(),
                pluModel.getTotal(),     //应收金额
                pluModel.getTotalRecv(),     //实收金额
                pluModel.getTotalDsc(),//优惠金额
                pluModel.getTotalRound(),                         //抹零金额
                "", // 发票号
                "",                       //remark
                pluModel.isWeightAndNotCount() ? "1" : "0",    //是否称重且非计件
                pluModel.getUdp5()     //货号
        );

        for (SalePluDscModel salePluDscModel : pluModel.getPluDscs()) {
            saveSaleItemDiscount(database, saleModel, pluModel, salePluDscModel);
        }
    }

    /**
     * 保存流水商品优惠。
     *
     * @param database    打开的 SQLite 数据库
     * @param saleModel   实现 {@code ISale} 接口的销售/退货流水
     * @param pluModel    流水商品
     * @param pluDscModel 流水商品优惠
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private void saveSaleItemDiscount(SQLiteDatabase database, BaseSaleModel saleModel, BaseSalePluModel pluModel, SalePluDscModel pluDscModel) throws SQLException {
        String sql = "insert into tSalSalePluDsc(OrgCode ,SaleNo,TranType,PageNo," +
                "                ItemNo,PluID,PluCode,PluName,SerialNo,DscType,DscCode,DscTotal,Remark)" +
                "            values(?,?,?,?,?,?,?,?,?,?,?,?,?)";
        MainSaleModel mainSaleModel = MainSaleModel.getInstance();
        this.execute(database, sql,
                mainSaleModel.getShopInfo().getOrgCode(),
                saleModel.getSaleNo(),
                String.valueOf(saleModel.getTranType().value()),
                String.valueOf(pluDscModel.getPageNo()),
                String.valueOf(pluDscModel.getItemNo()),
                pluModel.getPluId(),
                pluModel.getPluCode(),
                pluModel.getPluName(),
                String.valueOf(pluDscModel.getSerialNo()),
                pluDscModel.getDscProType(),
                pluDscModel.getDscCode(),
                pluDscModel.getTotalDsc(),
                "*"
        );
    }

    /**
     * 保存流水支付信息。
     *
     * @param database  打开的 SQLite 数据库
     * @param saleModel 实现 {@code BaseSaleModel} 接口的销售/退货流水
     * @param payModel  支付信息
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    private void savePayment(SQLiteDatabase database, BaseSaleModel saleModel, SalePayModel payModel) throws SQLException {
        String sql = "insert into tSalSalePay(OrgCode ,SaleNo,TranType,SerialNo," +
                "                PayCode,PayName,PayNo,TotalPaid,TotalRecv,UDP1,UDP2,UDP3,UDP4,UDP5,Remark,Tag)" +
                "            values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
        MainSaleModel mainSaleModel = MainSaleModel.getInstance();
        this.execute(database, sql,
                mainSaleModel.getShopInfo().getOrgCode(),
                saleModel.getSaleNo(),
                String.valueOf(saleModel.getTranType().value()),
                String.valueOf(payModel.getSerialNo()),   //payment.getPayId()
                payModel.getPayCode(),
                payModel.getPayName(),
                payModel.getBuyerLogonId(),
                payModel.getTotalPaid(),
                payModel.getTotalRecv(),
                payModel.getPayType(),
                payModel.getUdp2(),
                payModel.getUdp3(),
                payModel.getTradeNo(),  //upd4，记录银联交易参考号
                payModel.getOutTradeNo(),      //upd5，记录银联交易凭证号 只记录在本地
                payModel.getRemark(),
                "0"                     //Tag,记录退货状态
        );
    }


    /**
     * 更新流水状态。
     *
     * @param saleNo 流水号
     * @param state  流水状态
     * @throws SQLException 如果 SQL 执行失败或无法打开数据库
     */
    public void updateState(String saleNo, SaleState state) throws SQLException {
        String sql = "UPDATE tSalSale SET Tag = ? WHERE SaleNo = ?";
        this.execute(sql, String.valueOf(state.value()), saleNo);
    }
}
