/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.sale;

/**
 * 流水状态
 *
 * @author xileiyan 2017-09-13
 * @author xileiyan 2017-10-16
 * @author zhangsai 2018-12-25 HIPOSAPP-931
 */
public enum SaleState {

    /**
     * 默认状态。
     * <p>流水保存后的默认状态。此时还未上传流水。</p>
     */
    DEFAULT(0),

    /**
     * 流水上传成功。
     * <p>流水上传成功后的状态。</p>
     */
    SUCCESS(1),

    /**
     * 流水上传失败【会员】。
     * <p>业务流水上传成功，但是会员流水上传失败。重传流水时，只需重传会员流水。</p>
     */
    VIP_FAILURE(2),

    /**
     * 流水上传失败。
     * <p>流水上传失败。此时还未上传会员流水。重传流水时，与 DEFAULT 状态时一致。</p>
     */
    FAILURE(3),

    /**
     * 原交易流水发生退货。
     */
    RETURN(4);

    //流水状态值
    private int value;

    /**
     * 初始化一个 {@code SaleState} 的实例。
     *
     * @param value 表示流水状态值
     */
    SaleState(int value) {
        this.value = value;
    }

    /**
     * 返回流水状态的值
     *
     * @return 流水状态对应的值
     */
    public int value() {
        return this.value;
    }

    /**
     * 返回 {@param value} 值对应的流水状态。
     *
     * @param value 流水状态的值
     * @return {@param value} 值对应的流水状态
     */
    public static SaleState valueOf(int value) {
        switch (value) {
            case 1:
                return SaleState.SUCCESS;
            case 2:
                return SaleState.VIP_FAILURE;
            case 3:
                return SaleState.FAILURE;
            case 4:
                return SaleState.RETURN;
            default:
                return SaleState.DEFAULT;
        }
    }
}


