/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hiaposcmp.service.sale;

import android.content.Context;

import com.hisense.database.sqlite.SQLiteRepository;
import com.hisense.hiaposcmp.db.PadPosDatabase;
import com.hisense.hipos.sale.models.ISerialNoRepository;

import java.util.Locale;

/**
 * <p>流水顺序号资源库。顺序号存储在本地 SQLite 数据库的 tSaleNo 表中。</p>
 * <p>每个组织的每台收款机对应一条记录。</p>
 *
 * @author zhangsai 2018-01-10
 * @author zhangsai 2018-07-18 HIPOSAPP-675  改为使用基类的方法操作 SQLite 数据库
 * @author zhangsai 2019-06-17 HIPOSAPP-1174 使用 PadPosDatabase 初始化 SQLite 数据库
 */
public class SerialNoRepository extends SQLiteRepository implements ISerialNoRepository {

    /**
     * 初始化一个 {@code SerialNoRepository} 的实例。
     *
     * @param context Context
     */
    public SerialNoRepository(Context context) {
        super(new PadPosDatabase(context));
    }


    /**
     * 返回指定收款机的当前流水顺序号。
     *
     * @param orgCode 门店编码
     * @param posNo   收款机号
     * @return 指定收款机的当前流水顺序号。-1 表示没有对应的记录。
     */
    @Override
    public int get(String orgCode, String posNo) {
        String sql = "SELECT Value FROM tSaleNo WHERE Name = ? ";
        String key = getKey(orgCode, posNo);
        String value = this.executeScalar(sql, key);

        return this.getInteger(value, -1);
    }

    /**
     * 保存流水顺序号。
     *
     * @param orgCode  组织编码
     * @param posNo    收款机号
     * @param serialNo 流水顺序号
     */
    @Override
    public void save(String orgCode, String posNo, int serialNo) {
        String sql = "REPLACE INTO tSaleNo (Name, Value) VALUES (?, ?) ";
        String key = getKey(orgCode, posNo);
        this.execute(sql, key, String.valueOf(serialNo));
    }

    /**
     * 返回流水号存储在数据表中的键值。
     *
     * @param orgCode 组织编码
     * @param posNo   收款机号
     * @return 键值
     */
    private String getKey(String orgCode, String posNo) {
        return String.format(Locale.CHINESE, "saleNo_%s_%s", orgCode, posNo);
    }
}
