package com.hisense.hiaposcmp.service.wx;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.CountDownTimer;
import android.os.RemoteException;

import com.alibaba.fastjson.JSONObject;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.common.util.WebUtils;
import com.hisense.hiaposcmp.activity.BaseActivity;
import com.hisense.hiaposcmp.activity.ali.WelcomeAliActivity;
import com.hisense.hiaposcmp.activity.wx.PayCustomActivity;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.pay.WxConstants;
import com.hisense.hiaposcmp.tool.CommonTools;
import com.hisense.hiaposcmp.util.CommonUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.MapUtils;
import com.hisense.common.util.SignUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.common.util.XmlUtils;
import com.hisense.hiaposcmp.service.wx.dto.WxDto;
import com.hisense.hipos.model.MainSaleModel;
import com.tencent.wxpayface.IWxPayFaceCallbackAIDL;
import com.tencent.wxpayface.IWxPayfaceCallback;
import com.tencent.wxpayface.WxPayFace;

import java.text.BreakIterator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.PropertyResourceBundle;
import java.util.Random;
import java.util.Set;

/**
 * 微信支付服务类
 *
 * @author dukunbiao(null)  2018-06-29
 */
public class WxService {
    private static boolean isUpdate = true;
    private static int updateCount = 3;

    /*******************************************************************************************

     通过人脸识别，返回微信用户信息(openid, face_code);
     face_code：人脸凭证，用于人脸支付，作为订单的支付凭证;

     调用微信人脸SDK:
     -->1. 程序启动时初始化 initWxpayface-->返回初始化结果
     -->2. 获取数据 getWxpayfaceRawdata-->返回rawdata
     调用商户server:
     -->3. 获取人脸SDK调用凭证-->商户service请求微信支付后台API get_wxpayface_authinfo(rawdata)-->返回authinfo
     调用微信人脸SDK:
     -->4. 进行人脸识别 getWxpayfaceCode(authinfo)-->SDK自调摄像头-->返回人脸识别结果(face_code, openid)
     调用商户server:
     -->4.1. 发起订单人脸支付
     商户service请求微信支付后台API
     -->发起订单支付 facepay(face_code, orderinfo)-->返回支付结果
     支付结果未知（支付中/网络超时），需查询支付结果，直到返回确定的订单状态（成功/失败/订单不存在）/超过轮询时间-->4.2. 查询订单状态 queryorder
     轮询结束仍然没有支付成功-->4.3. 撤销交易易 reverseorder

     调用微信人脸SDK:
     -->5. 更新支付结果 updateWxpayfacePayResult
     -->6. 释放资源 releaseWxpayface

     1. 初始化 initWxpayface, 只需要在程序启动时调用;
     2. 释放资源 releaseWxpayface，只需要在程序退出时调用;


     1. 商户使用SDK接口均封装在WxPayFace 类中，该类以单例例形式存在。调用结果统一由IWxPayfaceCallback 回调。
     2. 参数或回调结果以Map形式传递，方便便扩展，具体字段名称及返回值定义中。

     当遇到付异常，请按如下说明处理
     （1）当交易超时或支付交易失败，商户收银系统必须调用【撤销订单API】，撤销此交易。
     （2）由于银行系统异常、用户余额不足、不支持用户卡种等原因使当前支付交易失败，商户收银系统应该把错误提示明确展示给收银员。
     （3）根据返回的错误码，判断是否需要撤销交易
     ******************************************************************************************/

    /**
     * 1、初始化 initWxpayface
     *
     * @param context
     */
    public static void initWxPayFace(Context context) {
        try {
            LoggerUtils.getLogger().debug("1、微信刷脸支付，初始化initWxPayFace");

            WxPayFace.getInstance().initWxpayface(context, new IWxPayfaceCallback() {
                @Override
                public void response(Map map) throws RemoteException {
                    if (map != null) {
                        String code = (String) map.get("return_code");
                        String msg = (String) map.get("return_msg");
                        if (code == null || !code.equals("SUCCESS")) {
                            new RuntimeException("调用返回非成功信息: " + msg).printStackTrace();
                        } else {
                            WxDto.getWxDto().setInit(true);
                        }
                    }

                    LoggerUtils.getLogger().info("微信刷脸支付，初始化，返回参数：" + map);
                }
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("微信刷脸支付，初始化异常，请检查微信刷脸服务是否正确安装：" + e.toString());
        }
    }
    /**
     * 1、初始化 initWxpayface
     *
     * @param context
     */
    public static void initWxPayFace(Context context, IWxPayfaceCallback callBack) {
        try {
            LoggerUtils.getLogger().debug("1、微信刷脸支付，初始化initWxPayFace");

            WxPayFace.getInstance().initWxpayface(context, new IWxPayfaceCallback() {
                @Override
                public void response(Map map) throws RemoteException {
                    if (map != null) {
                        String code = (String) map.get("return_code");
                        String msg = (String) map.get("return_msg");
                        if (code == null || !code.equals("SUCCESS")) {
                            new RuntimeException("调用返回非成功信息: " + msg).printStackTrace();
                        } else {
                            WxDto.getWxDto().setInit(true);
                        }
                    }

                    LoggerUtils.getLogger().info("微信刷脸支付，初始化，返回参数：" + map);
                    callBack.response(map);
                }
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("微信刷脸支付，初始化异常，请检查微信刷脸服务是否正确安装：" + e.toString());
        }
    }
    /**
     * 2、获取数据 getWxpayfaceRawdata
     * 获取人脸验证信息，获取到验证信息后，才可调用获取人脸识别
     *
     * @return 人脸验证信息rawdata
     */
    public static void getWxPayFaceRawData(final UniCallback callBack) {
        try {
            WxPayFace.getInstance().getWxpayfaceRawdata(new IWxPayfaceCallback() {
                @Override
                public void response(Map map) throws RemoteException {
                    LoggerUtils.getLogger().debug("微信刷脸支付，获取人脸验证信息，返回参数：" + map);

                    if (WxConstants.RETURN_CODE_SUCCESS.equals(map.get(WxConstants.RETURN_CODE))) {
                        callBack.onSuccess(MapUtils.getMapString(map, WxConstants.RAWDATA));
                    } else {
                        callBack.onError(MapUtils.getMapString(map, WxConstants.RETURN_MSG));
                    }
                }
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("微信刷脸支付，获取人脸验证信息异常:" + e.toString());
            callBack.onError("微信刷脸支付，获取人脸验证信息异常:" + e.getMessage());
        }
    }

    /**
     * 3、获取人脸SDK调用凭证 get_wxpayface_authinfo(rawdata)
     *
     * @param rawdata 人脸验证信息
     * @return SDK调用凭证authinfo
     */
    public static void getWxPayFaceAuthInfo(String rawdata, final UniCallback callBack) {
        final String requestContent;
        try {
            Map<String, String> requestParams = setRequestParamsPublic();
            requestParams.put(WxConstants.STORE_ID, WxConstants.wx_pay_store_id());   //是 string(32) 门店编号，由商户定义，各门店唯一。
            requestParams.put(WxConstants.STORE_NAME, WxConstants.wx_pay_store_name());   //是 string(128) 门店名称，由商户定义，可用于展示。
            requestParams.put(WxConstants.DEVICE_ID, WxConstants.wx_pay_device_id());   //是 string(32) 终端设备编号，由商户定义。
            //requestParams.put(WxConstants.ATTACH, "");   //否 string 附加字段。字段格式使用Json
            requestParams.put(WxConstants.RAWDATA, rawdata);   //是 string(2048) 初始化数据。由微信⼈人脸SDK的接⼝口返回
            requestParams.put(WxConstants.SIGN,
                    SignUtils.signCreate(requestParams, WxConstants.wx_pay_md5_key(), WxConstants.CHARSET_UTF8, WxConstants.SIGN_TYPE_MD5));
            requestContent = XmlUtils.mapToXml(requestParams, "xml");
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("微信刷脸支付，组装获取人脸SDK调用凭证请求参数异常：" + e.toString());
            callBack.onError("微信刷脸支付，组装获取人脸SDK调用凭证请求参数异常：" + e.getMessage());
            return;
        }

        ThreadFactoryUtils.getExecutorService("pay-%d").execute(() -> {
                    try {
                        Map<String, String> responseParams;
                        LoggerUtils.getLogger().debug("微信刷脸支付，获取人脸SDK调用凭证，请求参数：" + requestContent);

                        String strResponseParams = WebUtils.doPostV2(requestContent, WxConstants.URL_GET_WXPAYFACE_AUTHINFO);
                        LoggerUtils.getLogger().debug("微信刷脸支付，获取人脸SDK调用凭证，返回参数：" + strResponseParams);

                        if (StringUtils.isEmpty(strResponseParams)) {
                            callBack.onError("微信刷脸支付，获取人脸SDK调用凭证，返回参数为空");
                            return;
                        }

                        responseParams = XmlUtils.xmlToMap(strResponseParams, WxConstants.CHARSET_UTF8);
                        if (MapUtils.isEmpty(responseParams)) {
                            callBack.onError("微信刷脸支付，获取人脸SDK调用凭证，返回参数非法");
                            return;
                        }

                        if (WxConstants.RETURN_CODE_SUCCESS.equals(responseParams.get(WxConstants.RETURN_CODE))) {
                            callBack.onSuccess(MapUtils.getMapString(responseParams, WxConstants.AUTHINFO));
                        } else {
                            callBack.onError(MapUtils.getMapString(responseParams, WxConstants.RETURN_MSG));
                        }
                    } catch (Exception e) {
                        LoggerUtils.getLogger().debug("微信刷脸支付，获取人脸SDK调用凭证异常：" + e.toString());
                        callBack.onError("微信刷脸支付，获取人脸SDK调用凭证异常：" + e.getMessage());
                    }
                }
        );
    }

    /**
     * 4、人脸识别 getWxpayfaceCode(authinfo)
     * 启动人脸APP主界面入口，开启人脸识别
     *
     * @param authinfo SDK调用凭证
     * @param total    总金额
     * @param tradeNo  交易流水号
     * @param phoneNo  数据号
     * @return 人脸识别结果(face_code, openid)
     */
    public static void getWxPayFaceCode(String authinfo, String total, String tradeNo, String phoneNo, final UniCallback callBack) {
        Map<String, String> requestParams = setRequestParamsPublic();
        //移除无用参数
        requestParams.remove(WxConstants.NOW);
        requestParams.remove(WxConstants.VERSION);
        requestParams.remove(WxConstants.SIGN_TYPE);
        requestParams.remove(WxConstants.NONCE_STR);

        requestParams.put(WxConstants.STORE_ID, WxConstants.wx_pay_store_id());   //是 string(32) 门店编号，由商户定义，各门店唯一。
        //否 string 用户手机号
        if (!StringUtils.isEmpty(phoneNo)) {
            requestParams.put(WxConstants.TELEPHONE, phoneNo);
        }
        //requestParams.put(WxConstants.OUT_TRADE_NO, getOutTradeNo(0));    //否 string 商户订单号。FACEPAY 时必填
        requestParams.put(WxConstants.OUT_TRADE_NO, getOutTradeNo(0));    //否 string 商户订单号。FACEPAY 时必填
        requestParams.put(WxConstants.TOTAL_FEE, StringUtils.changeY2F(total));   //否 string 订单金额(数字), 单位分. FACEPAY 时必填
        requestParams.put(WxConstants.FACE_AUTHTYPE, WxConstants.FACE_AUTHTYPE_FACEPAY);   //是 string FACEPAY: 刷脸支付
        requestParams.put(WxConstants.AUTHINFO, authinfo);   //是 string 调用凭证。获取方式:get_wxpayface_authinfo
        requestParams.put(WxConstants.ASKFACRPERMIT, "0");
        requestParams.put(WxConstants.ASKRETPAGE, "0");
        LoggerUtils.getLogger().info("微信刷脸支付，开启人脸识别，请求参数：" + requestParams);
        WxPayFace.getInstance().getWxpayfaceCode(requestParams, new IWxPayfaceCallback() {
            @Override
            public void response(Map map) throws RemoteException {
                LoggerUtils.getLogger().info("微信刷脸支付，开启人脸识别，返回参数：" + map);

                if (WxConstants.RETURN_CODE_SUCCESS.equals(map.get(WxConstants.RETURN_CODE))) {
                    String faceCode = MapUtils.getMapString(map, WxConstants.FACE_CODE);
                    String openid = MapUtils.getMapString(map, WxConstants.OPENID);

                    if (StringUtils.isEmpty(faceCode) || StringUtils.isEmpty(openid)) {
                        callBack.onError("face_code或openid为空");
                        return;
                    }

                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(WxConstants.FACE_CODE, faceCode);
                    jsonObject.put(WxConstants.OPENID, openid);

                    callBack.onSuccess(jsonObject.toString());
                } else {
                    callBack.onError(MapUtils.getMapString(map, WxConstants.RETURN_MSG) + "|" +
                            MapUtils.getMapString(map, WxConstants.RETURN_CODE));
                }
            }
        });
    }

    /**
     * 5. 更新支付结果 updateWxpayfacePayResult
     *
     * @param authinfo
     * @param payresult
     * @param callBack
     */
    public static void updateWxPayFacePayResult(String authinfo, String payresult, final UniCallback callBack) {
        Map<String, String> requestParams = setRequestParamsPublic();
        //移除无用参数
        requestParams.remove(WxConstants.NOW);
        requestParams.remove(WxConstants.VERSION);
        requestParams.remove(WxConstants.SIGN_TYPE);
        requestParams.remove(WxConstants.NONCE_STR);
        requestParams.put(WxConstants.STORE_ID, WxConstants.wx_pay_store_id());   //是 string(32) 门店编号，由商户定义，各门店唯一。
        requestParams.put(WxConstants.AUTHINFO, authinfo);   //是 string 调用凭证。获取方式:get_wxpayface_authinfo
        requestParams.put(WxConstants.PAYRESULT, payresult);    //是 string 支付结果，SUCCESS:支付成功 ERROR:支付失败
        LoggerUtils.getLogger().debug("微信刷脸支付，更新支付结果，请求参数：" + requestParams);
        WxPayFace.getInstance().updateWxpayfacePayResult(requestParams, new IWxPayfaceCallback() {
            @Override
            public void response(Map map) {
                LoggerUtils.getLogger().debug("微信刷脸支付，更新支付结果，返回参数：" + map);
                if (WxConstants.RETURN_CODE_SUCCESS.equals(map.get(WxConstants.RETURN_CODE))) {
                    callBack.onSuccess(WxConstants.RETURN_CODE_SUCCESS);
                } else {
                    callBack.onError(MapUtils.getMapString(map, WxConstants.RETURN_MSG));
                }
            }
        });


//
//            while (isUpdate) {
//                if(updateCount>0){
//                    updateCount--;
//                    try {
//                        new Thread(new Runnable() {
//                            @Override
//                            public void run() {
//                                WxPayFace.getInstance().updateWxpayfacePayResult(requestParams, new IWxPayfaceCallback() {
//                                    @Override
//                                    public void response(Map map) {
//                                        isUpdate = false;
//                                        updateCount=0;
//                                        LoggerUtils.getLogger().debug("微信刷脸支付，更新支付结果，返回参数：" + map);
//                                        if (WxConstants.RETURN_CODE_SUCCESS.equals(map.get(WxConstants.RETURN_CODE))) {
//                                            callBack.onSuccess(WxConstants.RETURN_CODE_SUCCESS);
//                                        } else {
//                                            callBack.onError(MapUtils.getMapString(map, WxConstants.RETURN_MSG));
//                                        }
//                                    }
//                                });
//                                try {
//                                    Thread.sleep(30000);
//                                }catch (Exception e){
//                                    LoggerUtils.getLogger().debug("微信刷脸支付，更新支付结果，异常：" + e.toString());
//                                }
//                            }
//                        }).start();
//
//                    } catch (Exception e) {
//                        isUpdate = false;
//                        updateCount=0;
//                        LoggerUtils.getLogger().debug("微信刷脸支付，更新支付结果，异常：" + e.toString());
//                    }
//                }else {
//                    isUpdate=false;
//                    callBack.onError("微信刷脸支付，更新支付结果，异常,调用90s仍然失败");
//                }
//            }


    }


    /**
     * 6. 释放资源 releaseWxpayface
     *
     * @param context
     */
    public static void releaseWxPayFace(Context context) {
        if (context == null) {
            return;
        }

        try {
            WxPayFace.getInstance().releaseWxpayface(context);
            WxDto.getWxDto().setInit(false);
            LoggerUtils.getLogger().debug("微信刷脸支付，释放资源");
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("微信刷脸支付，释放资源异常" + e.toString());
        }
    }

    /**
     * 交易上报
     *
     * @param callBack
     */
    public static void reportOrder(String tradeNo, final UniCallback callBack) {
        Map<String, String> requestParams = setRequestParamsPublic();
        //移除无用参数
        requestParams.remove(WxConstants.NOW);
        requestParams.remove(WxConstants.VERSION);
        requestParams.remove(WxConstants.SIGN_TYPE);
        requestParams.remove(WxConstants.NONCE_STR);

        requestParams.put(WxConstants.OUT_TRADE_NO, tradeNo);    //否 string 商户订单号。FACEPAY 时必填

        LoggerUtils.getLogger().debug("上报微信支付交易，请求参数：" + requestParams);
        WxPayFace.getInstance().reportOrder(requestParams, new IWxPayfaceCallback() {
            @Override
            public void response(Map map) throws RemoteException {
                LoggerUtils.getLogger().debug("上报微信支付交易，返回参数：" + map);

                if (WxConstants.RETURN_CODE_SUCCESS.equals(map.get(WxConstants.RETURN_CODE))) {
                    callBack.onSuccess(WxConstants.RETURN_CODE_SUCCESS);
                } else {
                    callBack.onError(MapUtils.getMapString(map, WxConstants.RETURN_MSG) + "|" +
                            MapUtils.getMapString(map, WxConstants.RETURN_CODE));
                }
            }
        });
    }

    private static Map<String, String> setRequestParamsPublic() {
        Map<String, String> requestParams = new HashMap<>();
        requestParams.put(WxConstants.APPID, WxConstants.wx_face_pay_appid());         //是 string 公众号
        requestParams.put(WxConstants.MCH_ID, WxConstants.wx_face_pay_mch_id());       //是 string 商户号
        //requestParams.put(WxConstants.SUB_APPID, WxConstants.wx_pay_sub_appid());    //否 string(32) 子商户公众账号ID(服务商模式)
        //requestParams.put(WxConstants.SUB_MCH_ID, WxConstants.wx_pay_sub_mch_id());  //否 string(32) 子商户号(服务商模式)

        if (!StringUtils.isEmpty(WxConstants.wx_pay_sub_mch_id())) {
            requestParams.put(WxConstants.SUB_MCH_ID, WxConstants.wx_pay_sub_mch_id());  //否 string(32) 子商户号(服务商模式)
            LoggerUtils.getLogger().debug("wx_pay_sub_mch_id:" + WxConstants.wx_pay_sub_mch_id());
        }

        if (!StringUtils.isEmpty(WxConstants.wx_pay_sub_appid())) {
            requestParams.put(WxConstants.SUB_APPID, WxConstants.wx_pay_sub_appid());    //否 string(32) 子商户公众账号ID(服务商模式)
        }

        LoggerUtils.getLogger().debug("FaceMchid:" + GeneralSettings.INSTANCE.getWXFaceMchid());
        //正式环境需要传
        if (!StringUtils.isEmpty(GeneralSettings.INSTANCE.getWXFaceMchid())) {
            requestParams.put(WxConstants.SUB_MCH_ID, GeneralSettings.INSTANCE.getWXFaceMchid());  //子商户号(服务商模式)
        }

        requestParams.put(WxConstants.NOW, DateUtils.currentTime());                   //是 int 取当前时间，10位unix时间戳
        requestParams.put(WxConstants.VERSION, WxConstants.VERSION_VALUE);             //是 string 版本号。固定为1
        requestParams.put(WxConstants.SIGN_TYPE, WxConstants.SIGN_TYPE_MD5);           //是 string 签名类型，目前支持HMAC-SHA256和MD5，默认为MD5
        requestParams.put(WxConstants.NONCE_STR, StringUtils.randomUUID(32));        //是 string(32) 随机字符串，不长于32位

        return requestParams;
    }

    public static String getOutTradeNo(int tag) {
        if (tag == 0) {
            WxDto.getWxDto().setOutTradeNo(DateUtils.dateFormat(new Date(), "yyyyMMddHHmmss") + WxConstants.wx_pay_store_id() +
                    String.format(Locale.CHINESE, "%04d", new Random().nextInt(10000)));

        }
        return WxDto.getWxDto().getOutTradeNo();
    }

    /**
     * 设置运营位状态
     *
     * @param bean
     */
    public static void updateWxpayfaceBannerState(final Map bean, final IWxPayFaceCallbackAIDL wxpayfaceCallBack){
        WxPayFace.getInstance().updateWxpayfaceBannerState(bean,new IWxPayfaceCallback(){
            @Override
            public void response(Map map) throws RemoteException {
                wxpayfaceCallBack.response(bean);
            }
        });
    }
}
