/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.service.yst;

import android.os.Handler;
import android.os.Message;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.hiaposcmp.constant.ActivityConstants;
import com.hisense.hipos.yst.YstConfig;
import com.sprite.ystpay.api.YSTPayClient;
import com.sprite.ystpay.api.object.req.RefundRequest;
import com.sprite.ystpay.api.object.req.ScanpayRequest;
import com.sprite.ystpay.api.object.res.RefundResponse;
import com.sprite.ystpay.api.object.res.ScanpayResponse;
import com.sprite.ystpay.config.YSTConfig;

import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Locale;
import java.util.Random;

/**
 * 赢商通服务。
 *
 * @author zhangsai    2017-12-22 使用 YstConfig 初始化服务
 * @author wangyaogang 2019-07-01 HIPOSAPP-1189 赢商通初始化失败时返回false
 */
public class YstService {

    public static boolean hasInit = false;

    private YstConfig config;
    private String orgCode;

    public YstService(String orgCode, YstConfig config) {
        this.config = config;
        this.orgCode = orgCode;
    }

    /**
     * 生成 赢商通 的请求流水号。
     */
    private static String generReqSeq(final String orgCode, final String saleNo) {
        // 业务流水号 + 3位随机数 + orgCode

        Random rand = new Random();
        int randInt = rand.nextInt(1000);
        return saleNo + String.format(Locale.CHINESE, "%03d", randInt) + orgCode;
    }

    /**
     * 扫码支付。
     *
     * @param payHandler 处理程序
     * @param saleNo     流水号
     * @param ssTotal    实收金额
     * @param authCode   条码
     */
    public void pay(final Handler payHandler, final String saleNo,
                    final String ssTotal, final String authCode) {
        if (!hasInit) {
            if (!isInitialized()) {
                Message msg = new Message();
                msg.what = ActivityConstants.RESULT_YST_PAY;
                msg.obj = "赢商通配置错误,请确认配置参数";
                payHandler.sendMessage(msg);
                return;
            }
        }

        ThreadFactoryUtils.getExecutorService("pay-%d").execute(() -> {
            int money = new BigDecimal(ssTotal).intValue();
            String requestSeq = generReqSeq(orgCode, saleNo);
            ScanpayRequest request = new ScanpayRequest();
            request.setAccount(config.account());       // 门店账号
            request.setTotalFee(money);                 // 金额 # 1分钱 (单位分
            request.setRequestSequence(requestSeq);     // 请求流水
            request.setAuthCode(authCode);              // 微信支付条码
            ScanpayResponse response = YSTPayClient.executeUnitivePay(request);
            LoggerUtils.getLogger().debug("----YST payment calling, ssTotal:" + ssTotal + "(" + money + "), requestSeq:" + requestSeq);
            LoggerUtils.getLogger().debug("----YST payment calling, account:" + config.account() + "(" + money + "), requestSeq:" + requestSeq);

            Message msg = new Message();
            msg.what = ActivityConstants.RESULT_YST_PAY;
            msg.obj = response;
            payHandler.sendMessage(msg);
            LoggerUtils.getLogger().debug("----YST payment calling, ssTotal:" + ssTotal + "(" + money + "), requestSeq:" + requestSeq);
        });
    }

    /**
     * 退款。
     *
     * @param saleNo  流水号
     * @param ssTotal 退款金额
     * @param tranSeq 赢商通支付流水号
     * @return 退款处理结果
     */
    public RefundResponse refund(final String saleNo, final String ssTotal, final String tranSeq) {
        String requestSeq = generReqSeq(orgCode, saleNo);
        RefundRequest refundRequest = new RefundRequest();
        refundRequest.setAccount(config.account());
        refundRequest.setReason("退款");
        refundRequest.setRequestSequence(requestSeq);
        refundRequest.setRefundFee(new BigDecimal(ssTotal).setScale(2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)).intValue());
        refundRequest.setTransSequence(tranSeq);
        return YSTPayClient.executeUnitivePay(refundRequest);
    }

    public boolean isInitialized() {
        LoggerUtils.getLogger().debug("------YSTConfig.initYST (hostAddr:" + config.url() + ",key:" + config.key() + ")-----");
        if (config == null || StringUtils.isBlank(config.url()) || StringUtils.isBlank(config.key())) {
            LoggerUtils.getLogger().debug("------YSTConfig url, key is empty! 赢商通配置为空-----");
            return false;
        }
        try {
            YSTConfig.initYST(config.url(), config.key());
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        hasInit = true;
        return true;
    }
}
