/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.tool;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.widget.Toast;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;
import com.hisense.hiaposcmp.activity.ali.WelcomeAliActivity;
import com.hisense.hiaposcmp.activity.aliland.WelcomeAliLandActivity;
import com.hisense.hiaposcmp.activity.wx.WelcomeWxActivity;

/**
 * 常用工具集。
 *
 * @author fujiaxin    2017-01-13
 * @author fujiaxin    2017-02-07 登录应用配置。
 * @author fujiaxin    2017-03-04 UI 线程的 Toast 提示。
 * @author zhenghao    2017-04-07 显示/隐藏对话框、进度对话框。
 * @author gaoweisheng 2017-05-12 Toast 提示。
 */
public class CommonTools {

    private static Toast toast = null;
    private static long mExitTime;

    /**
     * 短时间显示 Toast。
     *
     * @param activity 使用的 Context。
     * @param message  显示的消息。
     */
    public static void toastShort(Activity activity, String message) {
        if (toast == null) {
            toast = Toast.makeText(activity.getApplicationContext(), message, Toast.LENGTH_SHORT);
        } else {
            toast.setText(message);
        }
        toast.show();
    }

    /**
     * 长时间显示 Toast。
     *
     * @param activity 使用的 Context。
     * @param message  显示的消息。
     */
    public static void toastLong(Activity activity, String message) {
        Toast.makeText(activity.getApplicationContext(), message, Toast.LENGTH_LONG).show();
    }

    /**
     * 短时间显示 Toast。
     *
     * @param message 显示的消息。
     */
    public static void toastShort(String message) {
        Toast.makeText(App.getInstance(), message, Toast.LENGTH_SHORT).show();
    }

    /**
     * 长时间显示 Toast。
     *
     * @param message 显示的消息。
     */
    public static void toastLong(String message) {
        Toast.makeText(App.getInstance(), message, Toast.LENGTH_SHORT).show();
    }

    /**
     * 启动新的 Activity。
     *
     * @param from 启动 Activity 的 Activity。
     * @param to   要启动的 Activity。
     */
    public static void goTo(Activity from, Class to) {
        Intent intent = new Intent(from, to);
        from.startActivity(intent);
    }

    /**
     * 启动新的 Activity。
     *
     * @param from    启动 Activity 的 Activity。
     * @param to      要启动的 Activity。
     * @param Request 请求码
     */
    public static void goTo(Activity from, Class to, int Request) {
        Intent intent = new Intent(from, to);
        from.startActivityForResult(intent, Request);
    }

    /**
     * 启动新的 Activity。
     * 并finish自身
     *
     * @param from 启动 Activity 的 Activity。
     * @param to   要启动的 Activity。
     */
    public static void goToWithFinishMe(Activity from, Class to) {
        Intent intent = new Intent(from, to);
        from.startActivity(intent);
        from.finish();
    }

    public static void goTo(Activity from, Class cls, String key, String value) {
        Bundle bundle = new Bundle();
        bundle.putString(key, value);

        Intent intent = new Intent();
        intent.putExtras(bundle);

        intent.setClass(from, cls);
        from.startActivity(intent);
    }

    /**
     * 跳转到WelcomeAntActivity
     *
     * @param from
     */
    public static void gotoWelcomeAntActivity(Activity from) {
        Intent intent = new Intent(from, WelcomeAliActivity.class);
        from.startActivity(intent);
        from.finish();
    }

    /**
     * 跳转到WelcomeAntLandActivity
     *
     * @param from
     */
    public static void gotoWelcomeAntLandActivity(Activity from) {
        Intent intent = new Intent(from, WelcomeAliLandActivity.class);
        from.startActivity(intent);
        from.finish();
    }

    /**
     * 跳转到WelcomeWXActivity
     *
     * @param from
     */
    public static void gotoWelcomeWxActivity(Activity from) {
        Intent intent = new Intent(from, WelcomeWxActivity.class);
        from.startActivity(intent);
        from.finish();
    }

    /**
     * 启动新的 Activity。
     *
     * @param from   启动 Activity 的 Activity。
     * @param to     要启动的 Activity。
     * @param bundle 传递的bundle
     */
    public static void goTo(Activity from, Class to, Bundle bundle) {
        Intent intent = new Intent(from, to);
        intent.putExtras(bundle);
        from.startActivity(intent);
    }

    /**
     * 转到 {@param cls} 指定的 Activity。
     *
     * @param cls Activity 的类型
     */
    public static void gotoActivity(Class cls) {
        gotoActivity(cls, null);
    }

    /**
     * 转到 {@param cls} 指定的 Activity。
     *
     * @param cls   Activity 的类型
     * @param key   消息的键
     * @param value 消息值
     */
    protected static void gotoActivity(Class cls, String key, String value) {
        Bundle bundle = new Bundle();
        bundle.putString(key, value);
        gotoActivity(cls, bundle);
    }


    /**
     * 转到 {@param cls} 指定的 Activity。
     *
     * @param cls   Activity 的类型
     * @param key   消息的键
     * @param value 消息值
     */
    public static void gotoActivity(Class cls, String key, int value) {
        Bundle bundle = new Bundle();
        bundle.putInt(key, value);
        gotoActivity(cls, bundle);
    }

    /**
     * 转到 {@param cls} 指定的 Activity。
     *
     * @param cls    Activity 的类型
     * @param bundle 传递的消息
     */
    protected static void gotoActivity(Class cls, Bundle bundle) {
        Intent intent = new Intent();
        if (bundle != null) {
            intent.putExtras(bundle);
        }
        intent.setClass(App.getContext(), cls);
        App.getContext().startActivity(intent);
    }

    /**
     * 启动新的 Activity 并获取result
     *
     * @param from
     * @param to
     * @param requestCode
     * @param bundle
     */
    public static void goToForResult(Activity from, Class to, int requestCode, Bundle bundle) {
        Intent intent = new Intent(from, to);
        intent.putExtras(bundle);
        from.startActivityForResult(intent, requestCode);
    }

    /**
     * 返回登录的应用配置。
     *
     * @param context 使用的 Context。
     * @return 登录的应用配置。
     */
    public static SharedPreferences loginConfig(Context context) {
        return context.getSharedPreferences("loginConfig", Activity.MODE_PRIVATE);
    }

    /**
     * 在 UI 线程上显示提示消息。
     *
     * @param activity 使用的 Context。
     * @param message  显示的消息。
     */
    public static void showUiMessage(final Activity activity, final String message) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                toastShort(activity, message);
            }
        });
    }

    /**
     * 在 UI 线程上显示进度对话框。
     *
     * @param message  显示的消息。
     * @param activity 显示对话框的活动。
     * @param dialog   显示的对话框。
     */
    public static void showUiProgressDialog(final String message, final Activity activity, final ProgressDialog dialog) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                hideUiProgressDialog(dialog, activity);
                dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                // dialog.setTitle("提示");
                dialog.setMessage(message);
                dialog.setIndeterminate(false);
                dialog.setCancelable(false);
                dialog.show();
            }
        });
    }

    /**
     * 隐藏进度对话框。
     *
     * @param activity 显示对话框的活动。
     * @param dialog   显示的对话框。
     */
    public static void hideUiProgressDialog(final ProgressDialog dialog, Activity activity) {
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (dialog != null) {
                    dialog.dismiss();
                }
            }
        });
    }

    public static void showUiDialogNoView() {
        showUiDialog("View未绑定，取消网络请求，请关闭页面后重试");
    }

    /**
     * 在 UI 线程上显示对话框。
     *
     * @param message 显示的消息。
     */
    public static void showUiDialog(final String message) {
        try {
            App.getContext().runOnUiThread(() -> {
                showDialog("提示", message, (dialog, which) -> {

                });
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("showUiDialog异常：" + e.toString());
        }
    }

    /**
     * 显示对话框。
     *
     * @param title    显示的标题。
     * @param message  显示的消息。
     * @param listener 对话框按钮点击监听器。
     */
    private static void showDialog(String title, String message, DialogInterface.OnClickListener listener) {
        try {
            new AlertDialog.Builder(App.getContext())
                    .setTitle(title)
                    .setMessage(message)
                    .setPositiveButton("确定", listener).show();
        } catch (Exception e) {
            LoggerUtils.getLogger().error("showDialog异常：" + e.toString());
        }
    }

    public static void showConfirmDialog(String title, String message, DialogInterface.OnClickListener positiveClickListener, DialogInterface.OnClickListener negativeClickListener) {
        AlertDialog.Builder builder = new AlertDialog.Builder(App.getContext());
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setPositiveButton("确定", positiveClickListener);
        builder.setNegativeButton("取消", negativeClickListener);
        builder.show();
    }

    public static void showConfirmDialog(String message, DialogInterface.OnClickListener positiveClickListener) {
        AlertDialog.Builder builder = new AlertDialog.Builder(App.getContext());
        builder.setTitle("提示");
        builder.setMessage(message);
        builder.setPositiveButton("确定", positiveClickListener);
        builder.setNegativeButton("取消", null);
        builder.show();
    }
    public static void exit(Activity from,Class toActivity) {
        if ((System.currentTimeMillis() - mExitTime) > 2000) {

            Toast.makeText(from, "再次点击将会退出当前界面", Toast.LENGTH_LONG).show();
            mExitTime = System.currentTimeMillis();
        } else {
            CommonTools.goToWithFinishMe(from, toActivity);
        }
    }
}
