package com.hisense.hiaposcmp.tool;


import android.content.Context;
import android.graphics.Canvas;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.animation.Animation;
import android.view.animation.ScaleAnimation;
import android.widget.FrameLayout;


/**
 * 主界面磁贴效果
 *
 * @author guolina  2019-12-18  新建标准版主界面磁贴点击效果
 */
public class HomeMenuButton extends FrameLayout {
    private boolean isPressed;
    private ScaleAnimation zoomInAnimation;
    private ScaleAnimation zoomOutAnimation;

    public HomeMenuButton(Context context) {
        this(context, null);
    }

    public HomeMenuButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
    }

    private void init() {
        /**
         * 初始化动画
         */
        zoomInAnimation = new ScaleAnimation(1f, 0.95f, 1f, 0.95f, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f);
        zoomInAnimation.setFillAfter(true);
        zoomInAnimation.setDuration(200);
        zoomOutAnimation = new ScaleAnimation(0.95f, 1f, 0.95f, 1f, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f);
        zoomOutAnimation.setFillAfter(true);
        zoomOutAnimation.setDuration(200);

    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
    }

    private void toNormalState() {
        isPressed = false;
        invalidate();
        startAnimation(zoomOutAnimation);
    }

    private boolean pointInView(float localX, float localY, float slop) {
        return localX >= -slop && localY >= -slop && localX < getWidth() + slop
                && localY < getHeight() + slop;
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                isPressed = true;//设置true
                invalidate();//重绘
                this.startAnimation(zoomInAnimation);//执行动画
                break;
            case MotionEvent.ACTION_UP:
                boolean needPerformClick = isPressed;
                toNormalState();//正常
                if (needPerformClick) {
                    performClick();
                }
                break;

            case MotionEvent.ACTION_MOVE:
                final int x = (int) event.getX();
                final int y = (int) event.getY();
                if (!pointInView(x, y, 20)) {
                    toNormalState();
                }
                break;

            case MotionEvent.ACTION_CANCEL:
            case MotionEvent.ACTION_OUTSIDE:
                toNormalState();
                break;
        }

        return true;

    }
}