package com.hisense.hiaposcmp.tool;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.media.MediaPlayer;
import android.os.Environment;
import android.util.Log;
import android.widget.ImageView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.constant.SDCardConstants;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * 加载SD卡资源
 *
 * @author dukunbiao(null) 2018-05-22
 */
public class LoadSDCard {

    public static void saveTxtFile(String path, List<String> value, String tag) {
        FileOutputStream outputStream = null;

        try {
            File file = new File(path);

            if (file.exists()) {
                file.delete();
            }

            outputStream = new FileOutputStream(file);

            for (int i = 0; i < value.size(); i++) {
                outputStream.write(value.get(i).getBytes());
                outputStream.write("\r\n".getBytes());
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("-------------saveTxtFile 异常:" + e.toString());
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.flush();
                    outputStream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static List<String> loadTxtFile(String path, int tag) {
        File file = new File(path);
        if (!file.exists()) {
            return null;
        }

        List<String> listValue = new ArrayList<>();
        String str;
        try {
            InputStream inputStream = new FileInputStream(file);
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            while ((str = bufferedReader.readLine()) != null) {
                listValue.add(str);
            }

            bufferedReader.close();
            inputStreamReader.close();
            inputStream.close();

            return listValue;
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("-------------loadTxtFile 异常,path:" + path + "----------" + e.toString());
        }

        return null;
    }

    public static String loadTxtFile(String path) {
        File file = new File(path);
        if (!file.exists()) {
            LoggerUtils.getLogger().debug("-------------loadTxtFile 路径未配置,需配置路径:" + path + "----------");
            return "";
        }

        StringBuffer stringBuffer = new StringBuffer();
        String str;
        try {
            InputStream inputStream = new FileInputStream(file);
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            while ((str = bufferedReader.readLine()) != null) {
                stringBuffer.append(str);
            }

            LoggerUtils.getLogger().debug("-------------loadTxtFile success,path:" + path + "----------");
            return stringBuffer.toString();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("-------------loadTxtFile 异常,path:" + path + "----------" + e.toString());
        }

        return "";
    }

    public static MediaPlayer loadMediaPlayer(String path) {
        File file = new File(path);
        if (!file.exists()) {
            LoggerUtils.getLogger().debug("-------------loadMediaPlayer 路径未配置,需配置路径:" + path + "----------");
            return null;
        }

        try {
            MediaPlayer mediaPlayer = new MediaPlayer();

            mediaPlayer.setDataSource(file.getPath());
            mediaPlayer.prepare();
            LoggerUtils.getLogger().debug("-------------loadMediaPlayer success,path:" + path + "----------");

            return mediaPlayer;
        } catch (IOException e) {
            LoggerUtils.getLogger().debug("-------------loadMediaPlayer 异常,path:" + path + "----------" + e.toString());
        }

        return null;
    }

    /**
     * 获取sd卡的缓存路径， 一般在卡中sdCard就是这个目录
     *
     * @return SDPath
     */
    public static String getSDPath() {
        File sdDir = null;
        boolean sdCardExist = Environment.getExternalStorageState().equals(
                android.os.Environment.MEDIA_MOUNTED); // 判断sd卡是否存在
        if (sdCardExist) {
            sdDir = Environment.getExternalStorageDirectory();// 获取根目录
        } else {
            Log.e("ERROR", "没有内存卡");
        }
        return sdDir.toString();
    }

    /**
     * 获取缓存文件夹目录 如果不存在创建 否则则创建文件夹
     *
     * @return filePath
     */
    private static String isExistsFilePath() {
        String filePath = getSDPath() + "logo";
        File file = new File(filePath);
        if (!file.exists()) {
            file.mkdirs();
        }
        return filePath;
    }

    /**
     * 获取SDCard文件
     *
     * @return Bitmap
     */
    public static Bitmap getImageFromSDCard(String filepath) {
        File file = new File(filepath);
        if (file.exists()) {
            Bitmap bm = BitmapFactory.decodeFile(filepath);
            return bm;
        }
        return null;
    }


    public static boolean fileExits(String filepath) {
        File file = new File(filepath);

        return file.exists();
    }

    public static void releaseImageViewResouce(ImageView imageView) {
        if (imageView == null) {
            return;
        }
        try {
            Drawable drawable = imageView.getDrawable();
            if (drawable != null && drawable instanceof BitmapDrawable) {
                BitmapDrawable bitmapDrawable = (BitmapDrawable) drawable;
                Bitmap bitmap = bitmapDrawable.getBitmap();
                if (bitmap != null && !bitmap.isRecycled()) {
                    if (LoadSDCard.fileExits(SDCardConstants.SSPOS_LOGO_PATH)) {
                        bitmap.recycle();
                    } else {
                        bitmap = null;
                    }
                    LoggerUtils.getLogger().info("图片被回收bitmap=" + bitmap);
                }
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("回收异常" + e.toString());
        }
    }

    /**
     * 返回存储在sd卡的图片
     *
     * @param line 模板的字符串
     * @return bm  bitmap格式的图片
     */
    public static Bitmap getBitmap(String line) {
        String name = line.substring(line.indexOf(":") + 1);
        String path = Environment.getExternalStorageDirectory() + "/" + name + ".bmp";
        Bitmap bm = null;
        File mFile = new File(path);
        try {
            if (mFile.exists()) {
                bm = BitmapFactory.decodeFile(path);
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("下载图片异常" + e.getMessage());
        }
        return bm;
    }
}
