package com.hisense.hiaposcmp.tool;

import android.os.Handler;
import android.view.InputDevice;
import android.view.KeyEvent;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.constant.GeneralSettings;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by wangwenyue on 2018/5/29.
 * 扫码工具类，自助蚂蚁金服项目使用
 */
public class ScanKeyEventHelper {

    private final static long MESSAGE_DELAY = 500;             //延迟100ms，判断扫码是否完成。
    private StringBuffer mStringBufferResult;                  //扫码内容
    private boolean mCaps;                                     //大小写区分
    private final Handler mHandler;
    private final Runnable mScanningFishedRunnable;
    private OnScanSuccessListener mOnScanSuccessListener;
    private String mDeviceName;
    private List<String> listBarCode = new ArrayList<>();
    private boolean isScanGoods = false;                        //是否用于扫商品码

    public ScanKeyEventHelper(OnScanSuccessListener onScanSuccessListener) {
        this.isScanGoods = false;
        mOnScanSuccessListener = onScanSuccessListener;
        mStringBufferResult = new StringBuffer();
        mHandler = new Handler();
        mScanningFishedRunnable = new Runnable() {
            @Override
            public void run() {
                performScanSuccess();
            }
        };
    }

    public ScanKeyEventHelper(OnScanSuccessListener onScanSuccessListener, boolean isScanGoods) {
        this.isScanGoods = isScanGoods;
        mOnScanSuccessListener = onScanSuccessListener;
        mStringBufferResult = new StringBuffer();
        mHandler = new Handler();
        mScanningFishedRunnable = new Runnable() {
            @Override
            public void run() {
                performScanSuccess();
            }
        };
    }

    /**
     * 返回扫码成功后的结果
     */
    private void performScanSuccess() {
        //支持捆绑码
        if (GeneralSettings.INSTANCE.isUseBaleBarCode() && this.isScanGoods) {
            if (listBarCode == null || listBarCode.isEmpty()) {
                return;
            }
        } else {
            String barcode = mStringBufferResult.toString();
            if (mOnScanSuccessListener != null) {
                LoggerUtils.getLogger().debug("接收到扫码"+barcode);
                mOnScanSuccessListener.onScanSuccess(barcode);
            }
            mStringBufferResult.setLength(0);
        }
    }

    /**
     * 扫码枪事件解析
     *
     * @param event
     */
    public void analysisKeyEvent(KeyEvent event) {
        int keyCode = event.getKeyCode();

        //字母大小写判断
        checkLetterStatus(event);
        if (event.getAction() == KeyEvent.ACTION_DOWN) {
            char aChar = getInputCode(event);

            if (aChar != 0) {

                mStringBufferResult.append(aChar);
            }
            if (keyCode == KeyEvent.KEYCODE_ENTER) {
                //是否支持捆绑码(多个商品码做成一个条码，其中以空格的形式分开)
                if (GeneralSettings.INSTANCE.isUseBaleBarCode() && this.isScanGoods) {
                    listBarCode.add(mStringBufferResult.toString());
                    mStringBufferResult.setLength(0);
                } else {
                    //若为回车键，直接返回
                    mHandler.removeCallbacks(mScanningFishedRunnable);
                    mHandler.post(mScanningFishedRunnable);
                }
            } else {
                //延迟post，若500ms内，有其他事件
                mHandler.removeCallbacks(mScanningFishedRunnable);
                mHandler.postDelayed(mScanningFishedRunnable, MESSAGE_DELAY);
            }
        }
    }

    /**
     * 扫码枪事件解析
     *
     * @param event
     */
    public void analysisKeyEventEnter(KeyEvent event) {
        int keyCode = event.getKeyCode();

        //字母大小写判断
        checkLetterStatus(event);

        if (event.getAction() == KeyEvent.ACTION_DOWN) {

            char aChar = getInputCode(event);

            if (aChar != 0) {
                mStringBufferResult.append(aChar);
            }
            if (keyCode == KeyEvent.KEYCODE_ENTER) {
                //若为回车键，直接返回
//                mHandler.removeCallbacks(mScanningFishedRunnable);
//                mHandler.post(mScanningFishedRunnable);
            } else {
                //延迟post，若500ms内，有其他事件
                mHandler.removeCallbacks(mScanningFishedRunnable);
                mHandler.postDelayed(mScanningFishedRunnable, MESSAGE_DELAY);
            }
        }
    }

    //检查shift键
    private void checkLetterStatus(KeyEvent event) {
        int keyCode = event.getKeyCode();
        if (keyCode == KeyEvent.KEYCODE_SHIFT_RIGHT || keyCode == KeyEvent.KEYCODE_SHIFT_LEFT) {
            if (event.getAction() == KeyEvent.ACTION_DOWN) {
                //按着shift键，表示大写
                mCaps = true;
            } else {
                //松开shift键，表示小写
                mCaps = false;
            }
        }
    }

    //获取扫描内容
    private char getInputCode(KeyEvent event) {

        int keyCode = event.getKeyCode();

        char aChar;

        if (keyCode >= KeyEvent.KEYCODE_A && keyCode <= KeyEvent.KEYCODE_Z) {
            //字母
            aChar = (char) ((mCaps ? 'A' : 'a') + keyCode - KeyEvent.KEYCODE_A);
        } else if (keyCode >= KeyEvent.KEYCODE_0 && keyCode <= KeyEvent.KEYCODE_9) {
            //数字
            aChar = (char) ('0' + keyCode - KeyEvent.KEYCODE_0);
        } else {
            //其他符号
            switch (keyCode) {
                case KeyEvent.KEYCODE_PERIOD:
                    aChar = '.';
                    break;
                case KeyEvent.KEYCODE_MINUS:
                    aChar = mCaps ? '_' : '-';
                    break;
                case KeyEvent.KEYCODE_SLASH:
                    aChar = '/';
                    break;
                case KeyEvent.KEYCODE_BACKSLASH:
                    aChar = mCaps ? '|' : '\\';
                    break;
                case KeyEvent.KEYCODE_LEFT_BRACKET:
                    aChar = '{';
                    break;
                case KeyEvent.KEYCODE_RIGHT_BRACKET:
                    aChar = '}';
                    break;
                case KeyEvent.KEYCODE_COMMA:
                    aChar = ',';
                    break;
                case KeyEvent.KEYCODE_APOSTROPHE:
                    aChar = '"';
                    break;
                case KeyEvent.KEYCODE_SEMICOLON:
                    aChar = mCaps ? ':' : ';';
                    break;
                case KeyEvent.KEYCODE_EQUALS:
                    aChar = '=';
                    break;
                default:
                    aChar = 0;
                    break;
            }
        }
        return aChar;

    }

    public interface OnScanSuccessListener {
        void onScanSuccess(String barcode);
    }

    public void onDestroy() {
        mHandler.removeCallbacks(mScanningFishedRunnable);
        mOnScanSuccessListener = null;
    }

    /**
     * 输入设备是否存在
     *
     * @param deviceName
     * @return
     */
    private boolean isInputDeviceExist(String deviceName) {
        int[] deviceIds = InputDevice.getDeviceIds();

        for (int id : deviceIds) {
            if (InputDevice.getDevice(id).getName().equals(deviceName)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 是否为扫码枪事件(部分机型KeyEvent获取的名字错误)
     *
     * @param event
     * @return
     */
    @Deprecated
    public boolean isScanGunEvent(KeyEvent event) {
        return isInputDeviceExist(event.getDevice().getName());
    }
}
