package com.hisense.hiaposcmp.tool;

import android.text.Editable;
import android.text.TextWatcher;
import android.widget.EditText;
import android.widget.Toast;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.NumberUtils;
import com.hisense.hiaposcmp.App;

import java.util.Locale;

/**
 * 积分卡/储值卡查询更新
 *
 * @author xieliyan    2017-12-21 HIPOSAPP-254
 * @author xieliyan    2018-01-08 HIPOSAPP-266 修复使用积分支付和梦卡支付后，返回到支付界面，提示：海信易收银无响应 的异常
 * @author xieliyan    2018-02-05 HIPOSAPP-318 修复小数无法继续进行支付的问题
 * @author guolina     2018-07-13 HIPOSAPP-667 删除支付/退款金额不能大于未付金额 的提示
 * @author guolina     2018-08-16 HIPOSAPP-735 修复撤销时需付金额刷新错误
 * @author guolina     2019-01-15 HIPOSAPP-975 修复输入金额不合法时自动修正金额错误
 * @author wangyaogang 2019-05-15 HIPOSAPP-1096 修复未输入金额时多次提示的问题
 */

public class UpdateTextButton {

    private static boolean isHandleTextChanged = false; // 控制更改文本时，不要产生重复调用
    private static String tempTotal;

    /**
     * 文本变更
     *
     * @param editText 编辑框
     * @param total    总金额
     */
    public static void setOnTextChange(final EditText editText, final String total) {
        setOnTextChange(editText, total, -1);
    }

    /**
     * 文本变更
     *
     * @param editText    编辑框
     * @param total       总金额
     * @param usableTotal 卡可用余额
     */
    public static void setOnTextChange(final EditText editText, final String total, final double usableTotal) {
        tempTotal = total;//final total只第一次赋值生效，所以加中间变量tempTotal
        editText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                if (isHandleTextChanged) {
                    return;
                }
                if (editText.getText().toString().equals("")) {
                    Toast.makeText(App.getContext(), "请输入交易金额", Toast.LENGTH_SHORT).show();
                    return;
                }
                isHandleTextChanged = true;
                try {
                    if (!NumberUtils.isRealNumber(editText.getText().toString())) {
                        Toast.makeText(App.getContext(), "交易金额不合法", Toast.LENGTH_SHORT).show();
                        updateTotalText(editText, tempTotal, usableTotal);
                        return;
                    }
                    if (BigDecimalUtils.isGreater0(BigDecimalUtils.subtract(editText.getText().toString(), total))) {
                        // updateTotalText(editText, total, usableTotal);
                    }
                } catch (Exception e) {
                    Toast.makeText(App.getContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
                } finally {
                    isHandleTextChanged = false;
                }
            }
        });
    }

    /**
     * 更新编辑框
     *
     * @param editText    编辑框
     * @param total       总金额
     * @param usableTotal 卡可用余额
     */
    public static void updateTotalText(EditText editText, String total, double usableTotal) {
        if (BigDecimalUtils.subtract(usableTotal, -1) == 0) {
            editText.setText(total);
            editText.setSelection(editText.getText().length());
            return;
        }
        if (BigDecimalUtils.isGreaterAndEquals0(BigDecimalUtils.subtract(total, String.valueOf(usableTotal)))) {
            editText.setText(String.format(Locale.CHINESE, "%.2f", usableTotal));
            editText.setSelection(editText.getText().length());
            return;
        }
        editText.setText(total);
        editText.setSelection(editText.getText().length());
    }
}



