package com.hisense.hiaposcmp.util;

import android.Manifest;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.media.MediaPlayer;
import android.os.Looper;

import com.hisense.common.boot.UniCallback;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.common.util.ThreadFactoryUtils;
import com.hisense.crm.Cashier;
import com.hisense.hiaposcmp.R;
import com.hisense.hiaposcmp.boot.Custom;
import com.hisense.hiaposcmp.config.Config;
import com.hisense.hiaposcmp.constant.GeneralSettings;
import com.hisense.hiaposcmp.constant.SDCardConstants;
import com.hisense.hiaposcmp.service.log.LogFileHandler;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.sale.models.ShopInfo;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.shop.ShopService;
import com.hisense.pos.payment.ums.AppId;
import com.hisense.security.MD5;
import com.hisense.webapi.Callback;
import com.qw.soul.permission.SoulPermission;

import java.util.Date;
import java.util.Locale;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CommonUtils {

    public static boolean isWriteOk() {
        return SoulPermission.getInstance().checkSinglePermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != null &&
                SoulPermission.getInstance().checkSinglePermission(Manifest.permission.WRITE_EXTERNAL_STORAGE).isGranted();
    }

    public static boolean isStdStyle() {
        return "standard".equalsIgnoreCase(Custom.current().getName());
    }

    public static boolean isStdStyle(String style) {
        return "standard".equalsIgnoreCase(style);
    }

    public static boolean isWxStyle() {
        return "wxpay".equalsIgnoreCase(Custom.current().getName()) || "hsyuniform".equalsIgnoreCase(Custom.current().getName());
    }

    public static boolean isWxStyle(String style) {
        return "wxpay".equalsIgnoreCase(style) || "hsyuniform".equalsIgnoreCase(style);
    }

    public static boolean isAliStyle() {
        return "hs6500qzdf".equalsIgnoreCase(Custom.current().getName());
    }

    public static boolean isAliLandStyle() {
        return "alilandpay".equalsIgnoreCase(Custom.current().getName());
    }

    public static boolean isAliStyle(String style) {
        return "hs6500qzdf".equalsIgnoreCase(style);
    }

    public static boolean isUniformStyle() {
        return isWxStyle() || isAliStyle() || isAliLandStyle();
    }

    public static boolean isHsyUniformStyle() {
        return "hsyuniform".equalsIgnoreCase(Custom.current().getName());
    }

    /**
     * 包名是否温州时代
     * *
     * * @param context 上下文
     * * @return true
     */
    public static boolean isTimesSquare(Context context) {
        return AppId.TIME_SQUARE.packageName.equals(context.getPackageName());
    }

    /**
     * 包名是否利客来（银商）
     *
     * @param context 上下文
     * @return true  是
     */
    public static boolean isLklUms(Context context) {
        return "com.hisense.hipos.lklums".equals(context.getPackageName());
    }

    public static void reInitSaleModel(UniCallback callback) {
        if (MainSaleModel.getInstance().getShopInfo() != null) {
            callback.onSuccess("");
            return;
        }

        LoggerUtils.getLogger().error("应用闪退crash，重新初始化基础数据");
        //应用闪退后，Context中的对象无法自动恢复。重新初始化对象。
        Config.init();

        com.hisense.hipos.webapi.Context.cashier(new User(Config.user_id, Config.user_code, Config.user_name));
        com.hisense.crm.webapi.Context.cashier(new Cashier(Config.user_code, Config.user_name));
        com.hisense.common.Context.config(Config.log_path);

        //重新初始化ShopInfo
        ShopService.shopList(new Callback<ShopInfo>() {
            @Override
            public void onSuccess(ShopInfo shopInfo) {
                LoggerUtils.getLogger().error("应用闪退crash，重新初始化ShopInfo成功");
                MainSaleModel.getInstance().shopInfo(shopInfo);

                com.hisense.hipos.webapi.Context.address(shopInfo.getAddress());
                com.hisense.hipos.webapi.Context.telephone(shopInfo.getTelephone());

                callback.onSuccess("");
            }

            @Override
            public void onError(Throwable throwable) {
                LoggerUtils.getLogger().error("应用闪退crash，重新初始化ShopInfo失败:" + throwable.toString());

                callback.onError("获取门店信息失败，请重新启动程序或检查后台服务：" + throwable.getMessage());
            }
        });
    }

    public static void cleanLog() {
        try {
            String logs = GeneralSettings.INSTANCE.getLogFileExpiredDays();
            if ("不清理".equals(logs)) {
                return;
            }

            LogFileHandler.deleteExpired(Integer.parseInt(logs));
        } catch (Exception e) {
            LoggerUtils.getLogger().error(GeneralSettings.INSTANCE.getLogFileExpiredDays() + "天前的，过期日志清理失败:" + e.toString());
        }
    }

    public static String genOutTradeNo(final String orgCode, final String saleNo) {
        Random rand = new Random();
        int randInt = rand.nextInt(10000);
        return saleNo + String.format(Locale.CHINESE, "%04d", randInt) + orgCode;
    }

    public static String ydtOutTradeNo(final String saleNo) {
        return saleNo + System.currentTimeMillis() / 1000;
    }

    public static String generateCardSaleNo() {
        return Config.org_code + DateUtils.dateFormat(new Date(), "yyyyMMddHHmmss");
    }

    public static boolean performQuitBack(String value) {
        //退出程序后门,用md5加盐防静态破解,密码05327788521
        //运算过程 pwd:ilovehisense05327788521 md5->a54557cb566c066cbedad874b01899e8
        String salt = "ilovehisense";
        String md5 = MD5.sign(salt + value).toLowerCase();
        return md5.endsWith("1899e8");
    }

    public static void loopQuit(Looper mLooper) {
        LoggerUtils.getLogger().debug("Looper.myLooper()=" + mLooper);
        try {
            if (mLooper != null) {
                mLooper.quitSafely();
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("关闭looper队列异常" + e.toString());
        }
    }

    public static void startApp(String packageName, String className, Context context) {
        Intent intent = new Intent(Intent.ACTION_MAIN);
        // 设置ComponentName参数1:packagename参数2:MainActivity路径
        intent.addCategory(Intent.CATEGORY_LAUNCHER);
        ComponentName cn = new ComponentName(packageName, className);
        intent.setComponent(cn);
        context.startActivity(intent);
    }

    /**
     * 扫码枪扫描数据监测
     * ①过滤数字，只允许0-9的参与判断。
     * ②判断字符串总长度，凡是大于18长度的一律放弃处理。
     * ③以上判断追加开关，并只在扫描商品界面执行
     *
     * @param barcode
     * @return
     */
    public static boolean checkBarcode(String barcode) {
        if (barcode.length() > 18) {
            return true;
        }

        return (!StringUtils.isNumeric(barcode));
    }

    public static int dp2Px(Context context, float dpValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }

    public static String authCodeType(String authCode) {
        if (StringUtils.isEmpty(authCode)) {
            return PayMethod.UNKNOWN;
        }

        String[] authCodeWxs = {"10", "11", "12", "13", "14", "15"};
        String[] authCodeAlis = {"25", "26", "27", "28", "29", "30"};

        for (String authCodeWx : authCodeWxs) {
            if (authCode.startsWith(authCodeWx)) {
                return PayMethod.WXPAY;
            }
        }

        for (String authCodeAli : authCodeAlis) {
            if (authCode.startsWith(authCodeAli)) {
                return PayMethod.ALIPAY;
            }
        }

        return PayMethod.UNKNOWN;
    }

    private static final char[] UNIT = {'亿', '拾', '佰', '仟', '万', '拾', '佰', '仟'};
    private static final char[] CHAINIESFIGURE2 = {'零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖'};

    public static String toChineseChar(String intString) throws NumberFormatException {
        //用来存放转换后的大写数字,因为是StringBuffer类型,所以顺便把没有转换
        //的数字倒序排列一下,省一个变量.
        StringBuffer ChineseCharI = new StringBuffer(intString);
        //倒序的数字,便于同单位合并
        String rintString = ChineseCharI.reverse().toString();
        //清空一下
        ChineseCharI.delete(0, ChineseCharI.length());
        //单位索引
        int unitIndex = 0;
        //数字长度
        int intStringLen = intString.length();
        //一位由字符转换的数字
        int intStringnumber = 0;
        //补0
        boolean addZero = false;
        boolean flagZero = false;
        for (int i = 0; i < intStringLen; i++) {
            //按单位长度循环索引
            unitIndex = i % UNIT.length;

            //异常检查
            if (!Character.isDigit(rintString.charAt(i))) {
                throw new NumberFormatException("数字中含有非法字符");
            }
            intStringnumber = Character.digit(rintString.charAt(i), 10);

            //如果当前位是0,开启补0继续循环直到不是0的位
            if (intStringnumber == 0) {
                addZero = true;
                if (i != 0 && i % 4 == 0) {
                    if (addZero && ChineseCharI.length() != 0) {
                        ChineseCharI.append(CHAINIESFIGURE2[0]);
                        addZero = false;
                    }
                    flagZero = true;
                    ChineseCharI.append(UNIT[unitIndex]);
                }
            } else {
                //若当前位不是第一位且补0开启
                if (addZero && ChineseCharI.length() != 0 && !flagZero) {
                    ChineseCharI.append(CHAINIESFIGURE2[0]);
                }
                flagZero = false;
                //插入单位
                //个位数后面不需 要单位
                if (i > 0) {
                    System.out.println(i);
                    ChineseCharI.append(UNIT[unitIndex]);
                }

                //插入大写数字
                ChineseCharI.append(CHAINIESFIGURE2[intStringnumber]);
                //补0关闭
                addZero = false;
            }
        }

        //异常处理
        if (ChineseCharI.length() == 0) {
            return "零";
        }
        return ChineseCharI.reverse().toString() + "圆整";
    }

    public static int verifyPhoneNo(String code) {
        int type = 1;
        //验证手机号正则表达式
        String phoneCheck = "^1[3|4|5|7|8][0-9]\\d{8}$";
        Pattern pattern = Pattern.compile(phoneCheck);
        Matcher matcher = pattern.matcher(code);
        // 字符串是否与正则表达式相匹配
        boolean rs = matcher.matches();
        if (rs) {
            //匹配的情况下为手机登录
            type = 2;
        } else {
            //不匹配的情况下为会员卡登录
            type = 1;
        }
        return type;
    }

    public static String getScalesErrorInfo(int i) {
        switch (i) {
            case 0x8000:
                return "命令数据不在合法范围内";
            case 0x8100:
                return "重量不稳定";
            case 0x8200:
                return "AD值溢出";
            case 0x8300:
                return "当前在去皮模式";
            case 0x8400:
                return "没有开机归零";
            case 0x8500:
                return "当前在预置去皮模式";
            case 0x8600:
                return "防作弊没有打开，不能设置";
            case 0xFE00:
                return "错误的命令";
            default:
                return "";
        }
    }

    public static void startBeepBgm(final Context context, String barcode) {
        if (StringUtils.isEmpty(barcode)) {
            return;
        }

        if (GeneralSettings.INSTANCE.isScannerBeep()) {
            startBgm(context, SDCardConstants.BGM_UART_SCANNER);
        }
    }

    public static void startBgm(final Context context, final String bgmTag) {
        ThreadFactoryUtils.getExecutorService("bgm-%d").execute(() -> {
            startBgmMediaPlayer(context, bgmTag);
        });
    }

    public static void startBgmMediaPlayer(Context context, String bgmTag) {
        try {
            MediaPlayer mediaPlayer = null;
            int bgmRaw = 0;
            switch (bgmTag) {
                case SDCardConstants.BGM_UART_SCANNER:
                    bgmRaw = R.raw.uart_scanner_beep;
                    break;
                default:
                    break;
            }

            if (bgmRaw != 0) {
                mediaPlayer = MediaPlayer.create(context, bgmRaw);
            }

            if (mediaPlayer == null) {
                LoggerUtils.getLogger().info("BGM mediaPlayer 为空");
                return;
            }

            mediaPlayer.start();

            mediaPlayer.setOnCompletionListener((mp) -> {
                mp.stop();
                mp.release();
            });

            mediaPlayer.setOnErrorListener((mp, what, extra) -> {
                try {
                    mp.stop();
                    mp.release();
                } catch (Exception e) {
                    LoggerUtils.getLogger().error("BGM 播放异常:" + e.toString());
                }

                return false;
            });
        } catch (Exception e) {
            LoggerUtils.getLogger().error("BGM 异常:" + e.toString());
        }
    }


}
