package com.hisense.hiaposcmp.util;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.util.Log;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;

import java.util.UUID;

/**
 * 设备信息工具
 * Created by zhuguangsheng on 2018/5/7.
 */
public class DeviceInfoUtils {

    private static final String TAG = DeviceInfoUtils.class.getSimpleName();

    private static final String LAKALAV10_BUILDMODEL = "AOSP on Intel Platform";

    /**
     * 获取设备唯一id
     *
     * @return
     */
    public static String getDeviceId() {
        if (android.os.Build.MODEL.equals(LAKALAV10_BUILDMODEL)) {
            String addrString = getLocalMacAddressFromWifiInfo();
            if (!addrString.isEmpty()) {
                LoggerUtils.getLogger().error("local mac addrString=" + addrString);
            }
            addrString = "wf" + addrString.replace(":", "").toLowerCase();//wf means "wifi"
            return addrString;
        }
        return getUniquePsuedoID();
    }

    /**
     * 去掉中间减号分隔符的deviceid
     *
     * @return
     */
    public static String getDeviceIdWithoutMinuschar() {
        return getUniquePsuedoID().replace("-", "");
    }

    /**
     * 系统Build.SERIAL, 在某些山寨机上为空字串,或者都是123456789ABCDEF
     *
     * @return
     */
    public static String getSystemBuildSerial() {
        return android.os.Build.SERIAL;
    }

    @Deprecated
    public static String getSystemBuildId() {
        return android.os.Build.ID;
    }

    //获得独一无二的Psuedo ID
    private static String getUniquePsuedoID() {
        String serial = null;

        String m_szDevIDShort = "35" +
                Build.BOARD.length() % 10 + Build.BRAND.length() % 10 +

                Build.CPU_ABI.length() % 10 + Build.DEVICE.length() % 10 +

                Build.DISPLAY.length() % 10 + Build.HOST.length() % 10 +

                Build.ID.length() % 10 + Build.MANUFACTURER.length() % 10 +

                Build.MODEL.length() % 10 + Build.PRODUCT.length() % 10 +

                Build.TAGS.length() % 10 + Build.TYPE.length() % 10 +

                Build.USER.length() % 10; //13 位

        try {
            serial = android.os.Build.class.getField("SERIAL").get(null).toString();
            //API>=9 使用serial号
            return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
        } catch (Exception exception) {
            //serial需要一个初始化
            serial = "serial"; // 随便一个初始化
        }
        //使用硬件信息拼凑出来的15位号码
        return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
    }

    /**
     * 从wifi网卡获取本机mac地址
     *
     * @return
     */
    public static String getLocalMacAddressFromWifiInfo() {
        String localWifiMacAddress = "";
        Context context = App.getInstance().getApplicationContext();
        try {
            WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
            WifiInfo info = wifi.getConnectionInfo();
            String tmpString = info.getMacAddress();
            if (tmpString != null) {
                localWifiMacAddress = tmpString;
            } else {
                Log.e(TAG, "Cannot get wifi mac!!! info.getMacAddress==null!!!");
                localWifiMacAddress = "";
            }
        } catch (Exception e) {
            e.printStackTrace();
            Log.e(TAG, "Cannot get wifi mac!!!");
            localWifiMacAddress = "";
        }

        return localWifiMacAddress;
    }

    /**
     * 获取厂商名
     *
     * @return 厂商名
     */
    public static String getManufacturer() {
        try {
            return android.os.Build.MANUFACTURER;
        } catch (Exception e) {
            return "unknown";
        }
    }

    /**
     * 获取设备型号名
     *
     * @return 设备型号名
     */
    public static String getModel() {
        try {
            return Build.MODEL;
        } catch (Exception e) {
            return "unknown";
        }
    }

    public static PackageInfo getPackageInfo(Context context) {
        PackageInfo pi = null;

        try {
            PackageManager pm = context.getPackageManager();
            pi = pm.getPackageInfo(context.getPackageName(),
                    PackageManager.GET_CONFIGURATIONS);

            return pi;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return pi;
    }
}
