package com.hisense.hiaposcmp.util;


import android.content.Context;
import android.os.Bundle;
import android.os.Environment;


import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.constant.FeatureConstants;
import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.InitListener;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechSynthesizer;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.SynthesizerListener;
import com.iflytek.cloud.util.ResourceUtil;
import com.iflytek.cloud.util.ResourceUtil.RESOURCE_TYPE;

/**
 * 科大讯飞语音合成支持类
 *
 * @author xieliyan  2020-01-02
 */
public class IFlyTtsUtil {

    // 语音合成对象
    private static SpeechSynthesizer mTts;

    // 默认本地发音人
    public static String voicerLocal = "xiaoyan";

    private static Context mContext;

    /**
     * 初始化讯飞语音合成
     */
    public static void initIflyTts(Context context) {
        StringBuffer param = new StringBuffer();
        param.append("appid=" + "5e095aa7");
        param.append(",");
        // 设置使用v5+
        param.append(SpeechConstant.ENGINE_MODE + "=" + SpeechConstant.MODE_MSC);
        param.append(SpeechConstant.FORCE_LOGIN + "=true");
        SpeechUtility.createUtility(context, param.toString());
    }


    public static void init(Context context) {
        switch (FeatureConstants.isStartSpeech){
            case FeatureConstants.IS_ENABLE_FEATURE:
                // 初始化合成对象
                mTts = SpeechSynthesizer.createSynthesizer(context, new InitListener() {
                    @Override
                    public void onInit(int i) {
                        if (i == ErrorCode.SUCCESS) {
                            LoggerUtils.getLogger().debug("初始化成功");
                        }
                    }
                });

                break;
            case FeatureConstants.IS_DISABLE_FEATURE:
                mTts=null;
                break;
        }

        mContext = context;


    }

    public static void start(String text) {
        if(null==mTts){
            LoggerUtils.getLogger().debug("禁止播放语音：" + System.currentTimeMillis());
            return;
        }

        int code = mTts.startSpeaking(text, new SynthesizerListener() {
            @Override
            public void onSpeakBegin() {
               // LoggerUtils.getLogger().debug("开始播放：" + System.currentTimeMillis());
            }

            @Override
            public void onSpeakPaused() {
              //  LoggerUtils.getLogger().debug("暂停播放：" + System.currentTimeMillis());
            }

            @Override
            public void onSpeakResumed() {
              //  LoggerUtils.getLogger().debug("继续播放：" + System.currentTimeMillis());
            }

            @Override
            public void onBufferProgress(int percent, int beginPos, int endPos,
                                         String info) {

              //  LoggerUtils.getLogger().debug("合成进度：" + percent);

            }

            @Override
            public void onSpeakProgress(int percent, int beginPos, int endPos) {
               // LoggerUtils.getLogger().debug("播放进度：" + percent);

            }

            @Override
            public void onCompleted(SpeechError error) {
              //  LoggerUtils.getLogger().debug("播放完成：" + error.toString());

            }

            @Override
            public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {


            }
        });
        if (code == ErrorCode.SUCCESS) {
            LoggerUtils.getLogger().debug("播放成功");
        }
//			/**
//			 * 只保存音频不进行播放接口,调用此接口请注释startSpeaking接口
//			 * text:要合成的文本，uri:需要保存的音频全路径，listener:回调接口
//			*/
//			String path = Environment.getExternalStorageDirectory()+"/tts.pcm";
//			int code = mTts.synthesizeToUri(text, path, mTtsListener);


    }

    /**
     * 合成回调监听。
     */
    private SynthesizerListener mTtsListener = new SynthesizerListener() {

        @Override
        public void onSpeakBegin() {
            LoggerUtils.getLogger().debug("开始播放：" + System.currentTimeMillis());
        }

        @Override
        public void onSpeakPaused() {
            LoggerUtils.getLogger().debug("暂停播放：" + System.currentTimeMillis());
        }

        @Override
        public void onSpeakResumed() {
            LoggerUtils.getLogger().debug("继续播放：" + System.currentTimeMillis());
        }

        @Override
        public void onBufferProgress(int percent, int beginPos, int endPos,
                                     String info) {

            LoggerUtils.getLogger().debug("合成进度：" + percent);

        }

        @Override
        public void onSpeakProgress(int percent, int beginPos, int endPos) {
            LoggerUtils.getLogger().debug("播放进度：" + percent);

        }

        @Override
        public void onCompleted(SpeechError error) {
            LoggerUtils.getLogger().debug("播放完成：" + error.toString());

        }

        @Override
        public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {


        }
    };

    /**
     * 合成参数设置
     */
    public static void setParams() {
        if(null==mTts){
            LoggerUtils.getLogger().debug("禁止播放语音：" + System.currentTimeMillis());
            return;
        }
        // 清空参数
        mTts.setParameter(SpeechConstant.PARAMS, null);
        //设置合成
        //设置使用本地引擎
        mTts.setParameter(SpeechConstant.ENGINE_TYPE, SpeechConstant.TYPE_LOCAL);
        //设置发音人资源路径
        mTts.setParameter(ResourceUtil.TTS_RES_PATH, getResourcePath(mContext));
        //设置发音人
        mTts.setParameter(SpeechConstant.VOICE_NAME, voicerLocal);

        //mTts.setParameter(SpeechConstant.TTS_DATA_NOTIFY,"1");//支持实时音频流抛出，仅在synthesizeToUri条件下支持
        //设置合成语速
        mTts.setParameter(SpeechConstant.SPEED, "50");
        //设置合成音调
        mTts.setParameter(SpeechConstant.PITCH, "50");
        //设置合成音量
        mTts.setParameter(SpeechConstant.VOLUME, "100");
        //设置播放器音频流类型
        mTts.setParameter(SpeechConstant.STREAM_TYPE, "3");
        //	mTts.setParameter(SpeechConstant.STREAM_TYPE, AudioManager.STREAM_MUSIC+"");

        // 设置播放合成音频打断音乐播放，默认为true
        mTts.setParameter(SpeechConstant.KEY_REQUEST_FOCUS, "true");

        // 设置音频保存路径，保存音频格式支持pcm、wav，设置路径为sd卡请注意WRITE_EXTERNAL_STORAGE权限
        mTts.setParameter(SpeechConstant.AUDIO_FORMAT, "wav");

        mTts.setParameter(SpeechConstant.TTS_AUDIO_PATH, Environment.getExternalStorageDirectory() + "/msc/tts.wav");
    }


    /**
     * 获取发音人资源路径
     */
    private static String getResourcePath(Context context) {
        StringBuffer tempBuffer = new StringBuffer();
        String type = "tts";

        //合成通用资源
        tempBuffer.append(ResourceUtil.generateResourcePath(context, RESOURCE_TYPE.assets, type + "/common.jet"));
        tempBuffer.append(";");
        //发音人资源
        tempBuffer.append(ResourceUtil.generateResourcePath(context, RESOURCE_TYPE.assets, type + "/" + voicerLocal + ".jet"));


        return tempBuffer.toString();
    }

    /**
     * 销毁合成对象
     */
    public static void destroy() {
        if (null != mTts) {
            mTts.stopSpeaking();
            // 退出时释放连接
            mTts.destroy();
        }
    }

}
