/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.util;

import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.Drawable;
import android.inputmethodservice.Keyboard;
import android.inputmethodservice.Keyboard.Key;
import android.inputmethodservice.KeyboardView;
import android.inputmethodservice.KeyboardView.OnKeyboardActionListener;
import android.text.Editable;
import android.util.Log;
import android.view.View;
import android.widget.EditText;

import com.hisense.hiaposcmp.R;

import java.util.List;

import androidx.core.content.res.ResourcesCompat;


/**
 * 自定义软键盘。
 *
 * @author zhangsai 2018-01-18 HIPOSAPP-286
 */
public class KeyboardUtil {

    private final int SYMBOL_BOARD = -100;
    private final int ALL_BOARD = -200;
    public boolean isnun = false;// 是否数据键盘
    public boolean isupper = false;// 是否大写
    public boolean isShow = false;
    private Context ctx;
    private Activity act;
    private KeyboardView keyboardView;
    private Keyboard k1;// 字母键盘
    private Keyboard k2;// 数字键盘
    private Keyboard k3;// 符号键盘
    private EditText edit;
    public OnKeyboardActionListener listener = new OnKeyboardActionListener() {
        @Override
        public void swipeUp() {
        }

        @Override
        public void swipeRight() {
        }

        @Override
        public void swipeLeft() {
        }

        @Override
        public void swipeDown() {
        }

        @Override
        public void onText(CharSequence text) {
        }

        @Override
        public void onRelease(int primaryCode) {
        }

        @Override
        public void onPress(int primaryCode) {
        }

        @Override
        public void onKey(int primaryCode, int[] keyCodes) {
            try {
                Editable editable = edit.getText();
                int start = edit.getSelectionStart();
                if (primaryCode == Keyboard.KEYCODE_CANCEL) {// 完成
                    hideKeyboard();
                } else if (primaryCode == Keyboard.KEYCODE_DELETE) {// 回退
                    if (editable != null && editable.length() > 0) {
                        if (start > 0) {
                            editable.delete(start - 1, start);
                        }
                    }
                } else if (primaryCode == Keyboard.KEYCODE_SHIFT) {// 大小写切换
                    changeKey();
                    keyboardView.setKeyboard(k1);

                } else if (primaryCode == Keyboard.KEYCODE_MODE_CHANGE) {// 数字键盘切换
                    if (isnun) {
                        isnun = false;
                        keyboardView.setKeyboard(k1);
                    } else {
                        isnun = true;
                        keyboardView.setKeyboard(k2);
                    }
                } else if (primaryCode == SYMBOL_BOARD) {
                    keyboardView.setKeyboard(k3);
                } else if (primaryCode == ALL_BOARD) {
                    keyboardView.setKeyboard(k1);
                } else {
                    editable.insert(start, Character.toString((char) primaryCode));
                }
            } catch (Exception e) {
                Log.e("KeyBoardUtil", "onKey:" + e.toString());
            }

        }
    };

    public KeyboardUtil(EditText edit) {
        this.act = (Activity) edit.getContext();
        this.edit = edit;
//        k1 = new Keyboard(act, R.xml.qwerty);
        k1 = new Keyboard(act, R.xml.qwert2);
        k2 = new Keyboard(act, R.xml.number);
        k3 = new Keyboard(act, R.xml.symbol);
        keyboardView = (KeyboardView) act.findViewById(R.id.keyBoard);

        keyboardView.setKeyboard(k2);
        // keyboardView.setKeyboard(k1);
//        keyboardView.setKeyboard(k3);
        keyboardView.setEnabled(true);
        keyboardView.setPreviewEnabled(false);
        keyboardView.setOnKeyboardActionListener(listener);
    }

    /**
     * 更改 {@code EditText}。
     *
     * @param edit EditText
     */
    public void change(EditText edit) {
        this.edit = edit;
    }

    /**
     * 键盘大小写切换
     */
    private void changeKey() {
        List<Key> keylist = k1.getKeys();
        k1.getShiftKeyIndex();

        if (isupper) {//大写切换小写
            isupper = false;
            Drawable drw = ResourcesCompat.getDrawable(act.getResources(), R.mipmap.sym_keyboard_shift, null);
            Key shift = keylist.get(k1.getShiftKeyIndex());
            shift.icon = drw;
            for (Key key : keylist) {
                if (key.label != null && isWord(key.label.toString())) {
                    key.label = key.label.toString().toLowerCase();
                    key.codes[0] = key.codes[0] + 32;
                }
            }
        } else {//小写切换大写
            isupper = true;
            Drawable drw = ResourcesCompat.getDrawable(act.getResources(), R.mipmap.shift_down, null);
            Key test = keylist.get(k1.getShiftKeyIndex());
            test.icon = drw;
            for (Key key : keylist) {
                if (key.label != null && isWord(key.label.toString())) {
                    key.label = key.label.toString().toUpperCase();
                    key.codes[0] = key.codes[0] - 32;
                }
            }
        }
    }

    public void showKeyboard() {
        int visibility = keyboardView.getVisibility();
        if (visibility == View.GONE || visibility == View.INVISIBLE) {
            keyboardView.setVisibility(View.VISIBLE);
        }
    }

    public void hideKeyboard() {
        int visibility = keyboardView.getVisibility();
        if (visibility == View.VISIBLE) {
            keyboardView.setVisibility(View.GONE);
        }
    }

    private boolean isWord(String str) {
        String wordStr = "abcdefghijklmnopqrstuvwxyz";
        if (wordStr.indexOf(str.toLowerCase()) > -1) {
            return true;
        }
        return false;
    }

}
