package com.hisense.hiaposcmp.util;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hiaposcmp.App;

import org.json.JSONObject;

import java.io.EOFException;
import java.net.SocketException;
import java.net.SocketTimeoutException;

/**
 * 网络工具类
 * Created by zhuguangsheng on 2018/6/26.
 */
public class NetUtils {

    public static boolean isNetworkAvailable() {
        try {
            ConnectivityManager cm = (ConnectivityManager) App.getAppContext().
                    getSystemService(Context.CONNECTIVITY_SERVICE);
            if (cm != null) {
                //获知网络管理的对象
                NetworkInfo info = cm.getActiveNetworkInfo();
                //判断当前网络是否已经连接
                return (info.getState() == NetworkInfo.State.CONNECTED);
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().error("检测网络状态异常:" + e.toString());
        }

        return false;
    }

    /**
     * 是否网络异常
     * (HttpHostConnectException extends ConnectException extends SocketException)
     *
     * @param cause 异常原因
     * @return true 是网络异常
     */
    public static boolean isNetException(Throwable cause) {
        return cause instanceof SocketTimeoutException
                || cause instanceof EOFException
                || cause instanceof SocketException;
    }

    /**
     * 是否需要查询,此方法针对海信支付
     * 1003  订单状态未知
     * 5000  中间件异常
     *
     * @param code
     * @return
     */
    public static boolean isNeedQuery(int code) {
        return code == 1003 || code == 1004 || code == 5000;
    }

    /**
     * 包名是否温州时代
     *
     * @param context 上下文
     * @return true
     */
    public static boolean isTimesSquare(Context context) {
        return "com.hisense.hipos.timessquare".equals(context.getPackageName());
    }

    /**
     * 打印 extras
     *
     * @param extras
     */
    public static void printExtras(Bundle extras) {
        if (extras == null) {
            LoggerUtils.getLogger().debug("extras == null");
            return;
        }
        JSONObject obj = new JSONObject();
        for (String key : extras.keySet()) {
            Object value = extras.get(key);
            if (value instanceof Bundle) {
                printExtras((Bundle) value);
            } else {
                try {
                    obj.put(key, extras.get(key));
                } catch (Exception e) {
                    LoggerUtils.getLogger().debug("Extra contains unknown class instance for [" + key + "]: ", e);
                }
            }
        }
        LoggerUtils.getLogger().debug("收单行信息：" + obj.toString());
    }

    /**
     * 打印
     *
     * @param key 打印提示信息
     * @param e   异常
     */
    public static void printDetail(String key, Exception e) {
        StringBuffer sb = new StringBuffer();
        sb.append(key + " 异常:" + e + "\n");
        StackTraceElement[] trace = e.getStackTrace();
        if (trace != null) {
            for (StackTraceElement element : trace) {
                sb.append(element + "\n");
            }
        }
        LoggerUtils.getLogger().error(sb.toString());
    }
}
