package com.hisense.hiaposcmp.util;

import android.view.View;

import androidx.appcompat.app.AppCompatActivity;

import com.hisense.common.util.LoggerUtils;

import java.lang.reflect.Method;

/**
 * Created by zhuguangsheng on 2018/5/24.
 */
public class SysBarUtils {

    public static final int DISABLE_EXPAND = 0x00010000;//4.2以上的整形标识
    public static final int DISABLE_EXPAND_LOW = 0x00000001;//4.2以下的整形标识
    public static final int DISABLE_NONE = 0x00000000;//取消StatusBar所有disable属性，即还原到最最原始状态

    public static void unBanStatusBar(AppCompatActivity appCompatActivity) {//利用反射解除状态栏禁止下拉
        if (appCompatActivity == null) {
            LoggerUtils.getLogger().error("ban status bar, context null");
            return;
        }
        Object service = appCompatActivity.getSystemService("statusbar");
        try {
            Class<?> statusBarManager = Class.forName
                    ("android.app.StatusBarManager");
            Method expand = statusBarManager.getMethod("disable", int.class);
            expand.invoke(service, DISABLE_NONE);
        } catch (Exception e) {
//            e.printStackTrace();
        }
    }

    private static void setStatusBarDisable(AppCompatActivity appCompatActivity, int disable_status) {//调用statusBar的disable方法
        if (appCompatActivity == null) {
            LoggerUtils.getLogger().error("set status bar disable, context null");
            return;
        }

        Object service = appCompatActivity.getSystemService("statusbar");
        try {
            Class<?> statusBarManager = Class.forName
                    ("android.app.StatusBarManager");
            Method expand = statusBarManager.getMethod("disable", int.class);
            expand.invoke(service, disable_status);
        } catch (Exception e) {
            unBanStatusBar(appCompatActivity);
//            e.printStackTrace();
        }
    }

    public static void banStatusBar(AppCompatActivity appCompatActivity) {//禁止statusbar下拉，适配了高低版本
        int currentApiVersion = android.os.Build.VERSION.SDK_INT;
        if (currentApiVersion <= 16) {
            setStatusBarDisable(appCompatActivity, DISABLE_EXPAND_LOW |
                    0x00040000 | 0x00080000 | 0x00400000 |
                    0x00200000 | 0x01000000 | 0x00800000 |
                    0x00020000 | 0x02000000);
        } else {
            setStatusBarDisable(appCompatActivity, DISABLE_EXPAND |
                    0x00040000 | 0x00080000 | 0x00400000 |
                    0x00200000 | 0x01000000 | 0x00800000 |
                    0x00020000 | 0x02000000);
        }
    }


    public static void hideSystemUi(AppCompatActivity appCompatActivity) {
        View mDecorView = appCompatActivity.getWindow().getDecorView();

        mDecorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION // hide nav bar
                        | View.SYSTEM_UI_FLAG_FULLSCREEN // hide status bar
                        | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
        );
    }
}
