package com.hisense.hiaposcmp.view;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.inputmethodservice.Keyboard;
import android.inputmethodservice.KeyboardView;
import android.util.AttributeSet;

import com.hisense.hiaposcmp.R;

import java.util.List;

/**
 * Created by wangwenyue on 2018/5/24.
 * 自助蚂蚁金服版通用键盘工具类
 */
public class KeyBoardAliView extends KeyboardView implements android.inputmethodservice.KeyboardView.OnKeyboardActionListener {

    // 用于区分左下角空白的按键
    private static final int KEYCODE_EMPTY = -10;
    //private static final int KEYCODE_DOUBLEZERO=00;

    private int mDeleteBackgroundColor;
    private Rect mDeleteDrawRect;
    private Drawable mDeleteDrawable;

    private KeyBoardAliView.IOnKeyboardListener mOnKeyboardListener;

    public KeyBoardAliView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context, attrs, 0);
    }

    public KeyBoardAliView(Context context, AttributeSet attrs,
                           int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context, attrs, defStyleAttr);
    }

    private void init(Context context, AttributeSet attrs,
                      int defStyleAttr) {
        TypedArray a = context.obtainStyledAttributes(attrs,
                R.styleable.AntKeyboardView, defStyleAttr, 0);
        mDeleteDrawable = a.getDrawable(
                R.styleable.AntKeyboardView_antPkvDeleteDrawable);
        mDeleteBackgroundColor = a.getColor(
                R.styleable.AntKeyboardView_antPkvDeleteBackgroundColor,
                Color.TRANSPARENT);
        a.recycle();

        // 设置软键盘按键的布局
        Keyboard keyboard = new Keyboard(context,
                R.xml.keyboard_number_ant);
        setKeyboard(keyboard);

        setEnabled(true);
        setPreviewEnabled(false);
        setOnKeyboardActionListener(this);
    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        Paint paint = new Paint();

        // 遍历所有的按键
        List<Keyboard.Key> keys = getKeyboard().getKeys();
        for (Keyboard.Key key : keys) {
            if (key.codes[0] == Keyboard.KEYCODE_DONE) {
                paint.setColor(getContext().getResources().getColor(R.color.ant_keyboard_blue));
                paint.setTextSize(46);
                canvas.drawText("确认", key.x + key.width / 2 - 46, key.y + key.height / 2 + 46 / 3, paint);
                //drawKeyBackground(key, canvas, INVISIBLE);
            }
            if (key.codes[0] == Keyboard.KEYCODE_DELETE) {

                paint.setColor(getContext().getResources().getColor(R.color.ant_keyboard_yellow));
                paint.setTextSize(46);
                canvas.drawText("删除", key.x + key.width / 2 - 46, key.y + key.height / 2 + 46 / 3, paint);
                //drawDeleteButton(key, canvas);
            }

            //drawKeyBackground(key, canvas, mDeleteBackgroundColor);
            //drawDeleteButton(key, canvas);

        }
    }

    // 绘制按键的背景
    private void drawKeyBackground(Keyboard.Key key, Canvas canvas,
                                   int color) {
        ColorDrawable drawable = new ColorDrawable(color);
        drawable.setBounds(key.x, key.y,
                key.x + key.width, key.y + key.height);
        drawable.draw(canvas);
    }

    // 绘制删除按键
    private void drawDeleteButton(Keyboard.Key key, Canvas canvas) {
        Paint paint = new Paint();

        if (mDeleteDrawable == null) {
            return;
        }

        // 计算删除图标绘制的坐标
        if (mDeleteDrawRect == null || mDeleteDrawRect.isEmpty()) {
            int intrinsicWidth = mDeleteDrawable.getIntrinsicWidth();
            int intrinsicHeight = mDeleteDrawable.getIntrinsicHeight();
            int drawWidth = intrinsicWidth;
            int drawHeight = intrinsicHeight;

            // 限制图标的大小，防止图标超出按键
            if (drawWidth > key.width) {
                drawWidth = key.width;
                drawHeight = drawWidth * intrinsicHeight / intrinsicWidth;
            }
            if (drawHeight > key.height) {
                drawHeight = key.height;
                drawWidth = drawHeight * intrinsicWidth / intrinsicHeight;
            }

            // 获取删除图标绘制的坐标
            int left = key.x + (key.width - drawWidth) / 2;
            int top = key.y + (key.height - drawHeight) / 2;
            mDeleteDrawRect = new Rect(left, top,
                    left + drawWidth, top + drawHeight);

            paint.setColor(getContext().getResources().getColor(R.color.ant_keyboard_yellow));
            paint.setTextSize(46);
            canvas.drawText("删除", left, top, paint);
        }

        // 绘制删除的图标
       /* if (mDeleteDrawRect != null && !mDeleteDrawRect.isEmpty()) {
            mDeleteDrawable.setBounds(mDeleteDrawRect.left,
                    mDeleteDrawRect.top, mDeleteDrawRect.right,
                    mDeleteDrawRect.bottom);
            mDeleteDrawable.draw(canvas);
        }*/
    }

    @Override
    public void onKey(int primaryCode, int[] keyCodes) {
        if (primaryCode == Keyboard.KEYCODE_DELETE) {
            if (mOnKeyboardListener != null) {
                mOnKeyboardListener.onDeleteKeyEvent();
            }
        } else if (primaryCode == Keyboard.KEYCODE_DONE) {
            if (mOnKeyboardListener != null) {
                mOnKeyboardListener.onDoneKeyEvent();
            }
        } else if (primaryCode != KEYCODE_EMPTY) {   // 点击了非左下角按键的其他按键
            if (mOnKeyboardListener != null) {
                mOnKeyboardListener.onInsertKeyEvent(
                        Character.toString((char) primaryCode));
            }
        }
    }

    @Override
    public void onPress(int primaryCode) {

    }

    @Override
    public void onRelease(int primaryCode) {

    }

    @Override
    public void onText(CharSequence text) {

    }

    @Override
    public void swipeLeft() {

    }

    @Override
    public void swipeRight() {

    }

    @Override
    public void swipeDown() {

    }

    @Override
    public void swipeUp() {

    }

    /**
     * 设置键盘的监听事件。
     *
     * @param listener 监听事件
     */
    public void setIOnKeyboardListener(KeyBoardAliView.IOnKeyboardListener listener) {
        this.mOnKeyboardListener = listener;
    }

    public interface IOnKeyboardListener {

        void onInsertKeyEvent(String text);

        void onDeleteKeyEvent();

        void onDoneKeyEvent();


    }
}
