package com.hisense.hiaposcmp.view;

import android.content.Context;
import android.text.Editable;
import android.util.AttributeSet;
import android.view.View;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hiaposcmp.R;

import androidx.annotation.Nullable;

/**
 * 数字键盘
 *
 * @author dukunbiao(null)  2016-12-19
 */
public class KeyBoardStdView extends LinearLayout implements View.OnClickListener {

    private KeyBoardStdView.OnKeyboardListener mOnKeyboardListener;

    public KeyBoardStdView(Context context) {
        super(context);
        init();
    }

    public KeyBoardStdView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public KeyBoardStdView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
        View inflate = inflate(getContext(), R.layout.std_keyboard, this);

        inflate.findViewById(R.id.tvStdBoardNum1).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum2).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum3).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum4).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum5).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum6).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum7).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum8).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum9).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum0).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNum00).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardNumPoints).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardDel).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardClear).setOnClickListener(this);
        inflate.findViewById(R.id.tvStdBoardConfirm).setOnClickListener(this);
    }

    @Override
    public void onClick(View view) {
        if (view.getId() == R.id.tvStdBoardDel) {
            if (mOnKeyboardListener != null) {
                mOnKeyboardListener.onDeleteKeyEvent();
            }
        } else if (view.getId() == R.id.tvStdBoardClear) {
            if (mOnKeyboardListener != null) {
                mOnKeyboardListener.onClearKeyEvent();
            }
        } else if (view.getId() == R.id.tvStdBoardConfirm) {
            if (mOnKeyboardListener != null) {
                mOnKeyboardListener.onDoneKeyEvent();
            }
        } else {
            if (mOnKeyboardListener != null) {
                try {
                    mOnKeyboardListener.onInsertKeyEvent(((TextView) view).getText().toString());
                } catch (Exception e) {
                    LoggerUtils.getLogger().error("按键错误:" + e.toString());
                }
            }
        }
    }

    public static void addChar(EditText editText, String data) {
        int index = editText.getSelectionStart();
        editText.getText().insert(index, data);
    }

    public static void deleteChar(EditText editText) {
        int index = editText.getSelectionStart();
        Editable editable = editText.getText();

        if (StringUtils.isEmpty(editable.toString())) {
            return;
        }
        if (index == 0) {
            return;
        }
        editable.delete(index - 1, index);
    }

    public static void clearChar(EditText editText) {
        editText.getText().clear();
    }

    public void setOnKeyboardListener(KeyBoardStdView.OnKeyboardListener listener) {
        this.mOnKeyboardListener = listener;
    }

    public interface OnKeyboardListener {

        void onInsertKeyEvent(String text);

        void onDeleteKeyEvent();

        void onClearKeyEvent();

        void onDoneKeyEvent();
    }
}
