/***************************************************************************************************
 * Copyright (c) 2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hiaposcmp.view;

import android.content.Context;
import android.graphics.PixelFormat;
import android.util.Log;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;

import androidx.appcompat.widget.AppCompatTextView;

import com.hisense.hiaposcmp.App;

/**
 * 扫码悬浮窗
 *
 * @author caokai   2017-01-10
 * @author xieliyan 2017-12-08
 */
public class MyFloatView extends AppCompatTextView {

    private float mTouchStartX;
    private float mTouchStartY;
    private float x;
    private float y;
    private float mStartX;
    private float mStartY;
    private View.OnClickListener mClickListener;

    public static WindowManager wm = (WindowManager) App.getContext().getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
    //此wmParams为获取的全局变量，用以保存悬浮窗口的属性
    public static WindowManager.LayoutParams wmParams = App.getInstance().getMywmParams();

    public MyFloatView(Context context) {
        super(context);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        //获取相对屏幕的坐标，即以屏幕左上角为原点
        x = event.getRawX();
        y = event.getRawY(); //25是系统状态栏的高度
        Log.i("currP", "currX" + x + "====currY" + y);
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                //获取相对View的坐标，即以此View左上角为原点
                mTouchStartX = event.getX();
                mTouchStartY = event.getY();
                mStartX = x;
                mStartY = y;
                break;
            case MotionEvent.ACTION_MOVE:
                updateViewPosition();
                break;
            case MotionEvent.ACTION_UP:
                //updateViewPosition();
                if ((x - mStartX) < 2 && (y - mStartY) < 2) {
                    // 设置监听
                    if (mClickListener != null) {
                        mClickListener.onClick(this);
                    }
                }
                mTouchStartX = mTouchStartY = 0;
                break;
            default:
                break;
        }
        return true;
    }

    private void updateViewPosition() {
        //更新浮动窗口位置参数
        wmParams.x = (int) (x - mTouchStartX);
        wmParams.y = (int) (y - mTouchStartY);
        wm.updateViewLayout(this, wmParams);
    }

    @Override
    public void setOnClickListener(View.OnClickListener listener) {
        this.mClickListener = listener;
    }

    /**
     * 设置悬浮窗属性
     *
     * @param myFV        悬浮窗
     * @param FLOW_VIEW_X X轴初始值
     * @param FLOW_VIEW_Y Y轴初始值
     */
    public static void configFloat(MyFloatView myFV, int FLOW_VIEW_X, int FLOW_VIEW_Y) {
        wmParams.type = WindowManager.LayoutParams.TYPE_SYSTEM_ALERT; //设置window type
        wmParams.format = PixelFormat.RGBA_8888; //设置图片格式，效果为背景透明
        //设置Window flag
        wmParams.flags = WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL
                | WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE;
        wmParams.gravity = Gravity.START | Gravity.TOP; //调整悬浮窗口至左上角
        //以屏幕左上角为原点，设置x、y初始值
        wmParams.x = FLOW_VIEW_X;
        wmParams.y = FLOW_VIEW_Y;
        //设置悬浮窗口长宽数据
        wmParams.width = 128;
        wmParams.height = 128;
        //显示myFloatView图像
        wm.addView(myFV, wmParams);
    }
}