package com.hisense.common.util;

import java.math.BigDecimal;
import java.text.DecimalFormat;

/**
 * BigDecimal 工具类
 *
 * @author dukunbiao(null) 2018-09-28
 */
public class BigDecimalUtils {

    private BigDecimalUtils() {
    }

    public static int add(int value1, int value2) {
        return BigDecimal.valueOf(value1).add(BigDecimal.valueOf(value2)).stripTrailingZeros().intValue();
    }

    public static double add(double value1, double value2) {
        return BigDecimal.valueOf(value1).add(BigDecimal.valueOf(value2)).stripTrailingZeros().doubleValue();
    }

    public static String add(String value1, String value2) {
        return new BigDecimal(value1).add(new BigDecimal(value2)).stripTrailingZeros().toPlainString();
    }

    public static int subtract(int value1, int value2) {
        return BigDecimal.valueOf(value1).subtract(BigDecimal.valueOf(value2)).stripTrailingZeros().intValue();
    }

    public static double subtract(double value1, double value2) {
        return BigDecimal.valueOf(value1).subtract(BigDecimal.valueOf(value2)).stripTrailingZeros().doubleValue();
    }

    public static String subtract(String value1, String value2) {
        return new BigDecimal(value1).subtract(new BigDecimal(value2)).stripTrailingZeros().toPlainString();
    }

    public static int multiply(int value1, int value2) {
        return BigDecimal.valueOf(value1).multiply(BigDecimal.valueOf(value2)).stripTrailingZeros().intValue();
    }

    public static double multiply(double value1, double value2) {
        return BigDecimal.valueOf(value1).multiply(BigDecimal.valueOf(value2)).stripTrailingZeros().doubleValue();
    }

    public static String multiply(String value1, String value2) {
        return new BigDecimal(value1).multiply(new BigDecimal(value2)).stripTrailingZeros().toPlainString();
    }

    public static double multiply(double value1, double value2, int scale) {
        return new BigDecimal(value1).multiply(new BigDecimal(value2)).setScale(scale, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().doubleValue();
    }

    public static String multiply(String value1, String value2, int scale) {
        return new BigDecimal(value1).multiply(new BigDecimal(value2)).setScale(scale, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
    }

    public static double multiply(double value1, double value2, int scale, int roundingMode) {
        return new BigDecimal(value1).multiply(new BigDecimal(value2)).setScale(scale, roundingMode).stripTrailingZeros().doubleValue();
    }

    public static String multiply(String value1, String value2, int scale, int roundingMode) {
        return new BigDecimal(value1).multiply(new BigDecimal(value2)).setScale(scale, roundingMode).stripTrailingZeros().toPlainString();
    }

    public static String divide(String value1, String value2) {
        return new BigDecimal(value1).divide(new BigDecimal(value2), BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
    }

    public static double divide(int value1, int value2, int scale) {
        return BigDecimal.valueOf(value1).divide(BigDecimal.valueOf(value2), scale, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().doubleValue();
    }

    public static double divide(double value1, double value2, int scale) {
        return BigDecimal.valueOf(value1).divide(BigDecimal.valueOf(value2), scale, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().doubleValue();
    }

    public static String divide(String value1, String value2, int scale) {
        return new BigDecimal(value1).divide(new BigDecimal(value2), scale, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
    }

    public static double divide(int value1, int value2, int scale, int roundingMode) {
        return BigDecimal.valueOf(value1).divide(BigDecimal.valueOf(value2), scale, roundingMode).stripTrailingZeros().doubleValue();
    }

    public static double divide(double value1, double value2, int scale, int roundingMode) {
        return BigDecimal.valueOf(value1).divide(BigDecimal.valueOf(value2), scale, roundingMode).stripTrailingZeros().doubleValue();
    }

    public static String divide(String value1, String value2, int scale, int roundingMode) {
        return new BigDecimal(value1).divide(new BigDecimal(value2), scale, roundingMode).stripTrailingZeros().toPlainString();
    }

    public static double format(double value) {
        return BigDecimal.valueOf(value).stripTrailingZeros().doubleValue();
    }

    public static String format(String value) {
        return new BigDecimal(value).stripTrailingZeros().toPlainString();
    }

    public static String decimalFormat(double value) {
        return new DecimalFormat("0.00").format(new BigDecimal(value));
    }

    public static String decimalFormat(String value) {
        if (StringUtils.isEmpty(value)) {
            value = "0";
        }

        return new DecimalFormat("0.00").format(new BigDecimal(value));
    }

    public static String decimalFormat(String value, String pattern) {
        if (StringUtils.isEmpty(value)) {
            value = "0";
        }

        return new DecimalFormat(pattern).format(new BigDecimal(value));
    }

    public static String decimalFormat(String value, int pattern) {
        if (StringUtils.isInteger(value)) {
            return format(value);
        } else {
            StringBuilder sb = new StringBuilder();
            sb.append("0.");
            for (int i = 0; i < pattern; i++) {
                sb.append("0");
            }

            return decimalFormat(value, sb.toString());
        }
    }

    public static String formatScale(String value) {
        return new BigDecimal(value).setScale(2, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
    }

    public static String formatScale(String value, int newScale) {
        return new BigDecimal(value).setScale(newScale, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
    }

    /**
     * 将原字符串型数值取绝对值,并返回字符串
     *
     * @param value
     * @return
     */
    public static String absToString(String value) {
        return new BigDecimal(value).abs().toPlainString();
    }

    public static String absToString(double value) {
        return BigDecimal.valueOf(value).abs().toPlainString();
    }

    public static int compareTo(double value1, double value2) {
        return BigDecimal.valueOf(value1).compareTo(BigDecimal.valueOf(value2));
    }

    public static int compareTo(String value1, String value2) {
        return new BigDecimal(value1).compareTo(new BigDecimal(value2));
    }

    public static boolean isEquals(String value1, String value2) {
        return compareTo(value1, value2) == 0;
    }

    public static boolean isNotEquals(String value1, String value2) {
        return compareTo(value1, value2) != 0;
    }

    public static boolean isGreater(String value1, String value2) {
        return compareTo(value1, value2) > 0;
    }

    public static boolean isLess(String value1, String value2) {
        return compareTo(value1, value2) < 0;
    }

    public static boolean isGreaterAndEquals(String value1, String value2) {
        return compareTo(value1, value2) >= 0;
    }

    public static boolean isLessAndEquals(String value1, String value2) {
        return compareTo(value1, value2) <= 0;
    }

    public static boolean isEquals0(String value1) {
        return compareTo(value1, "0") == 0;
    }

    public static boolean isNotEquals0(String value1) {
        return compareTo(value1, "0") != 0;
    }

    public static boolean isGreater0(String value1) {
        return compareTo(value1, "0") > 0;
    }

    public static boolean isLess0(String value1) {
        return compareTo(value1, "0") < 0;
    }

    public static boolean isGreaterAndEquals0(String value1) {
        return compareTo(value1, "0") >= 0;
    }

    public static boolean isLessAndEquals0(String value1) {
        return compareTo(value1, "0") <= 0;
    }

    public static String transferDecimal(String value) {
        DecimalFormat decimalFormat = new DecimalFormat("#.######");
        return decimalFormat.format(value);
    }

    public static String transferBig(double value) {
        DecimalFormat decimalFormat = new DecimalFormat("########.##");
        return decimalFormat.format(value);
    }

    public static String transferBig(String value) {
        DecimalFormat decimalFormat = new DecimalFormat("########.##");
        return decimalFormat.format(value);
    }

    public static String ternaryOperatorLess(String value1, String value2) {
        return isLess(value1, value2) ? value1 : value2;
    }

    public static String ternaryOperatorGreater(String value1, String value2) {
        return isGreater(value1, value2) ? value1 : value2;
    }
}
