package com.hisense.common.util;

import com.hisense.common.constant.UtilConstants;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 * 时间工具类
 *
 * @author dukunbiao(null)  2018-06-02
 */
public class DateUtils {

    private static final SimpleDateFormat SIMPLE_DATE_FORMAT = new SimpleDateFormat(UtilConstants.DATE_FORMAT, Locale.CHINA);
    private static final String[] WEEK_DAYS = {"周日", "周一", "周二", "周三", "周四", "周五", "周六"};

    private DateUtils() {

    }

    public static String dateFormat() {
        return SIMPLE_DATE_FORMAT.format(new Date());
    }

    /**
     * 时间格式化
     *
     * @param date 待格式化时间
     * @return 时间字符串
     */
    public static String dateFormat(Date date) {
        return SIMPLE_DATE_FORMAT.format(date);
    }

    /**
     * 时间格式化
     *
     * @param date   待格式化时间
     * @param format 格式
     * @return 时间字符串
     */
    public static String dateFormat(Date date, String format) {
        return new SimpleDateFormat(format, Locale.CHINA).format(date);
    }

    /**
     * 时间格式化
     *
     * @param format 格式
     * @return 时间字符串
     */
    public static String dateFormat(String format) {
        return new SimpleDateFormat(format, Locale.CHINA).format(new Date());
    }

    /**
     * 时间字符串格式化
     *
     * @param dateValue 待格式化时间字符串
     * @param toFormat  要格式化的格式
     * @return 时间字符串
     */
    public static String dateFormat(String dateValue, String toFormat) throws Exception {
        if (StringUtils.isEmpty(dateValue) || StringUtils.isEmpty(toFormat)) {
            return null;
        }

        Date date = SIMPLE_DATE_FORMAT.parse(dateValue);
        return dateFormat(date, toFormat);
    }

    /**
     * 时间字符串格式化
     *
     * @param dateValue  待格式化时间字符串
     * @param fromFormat 待格式化时间字符串格式
     * @param toFormat   要格式化的格式
     * @return 时间字符串
     */
    public static String dateFormat(String dateValue, String fromFormat, String toFormat) throws Exception {
        if (StringUtils.isEmpty(dateValue) || StringUtils.isEmpty(fromFormat) || StringUtils.isEmpty(toFormat)) {
            return null;
        }

        Date date = SIMPLE_DATE_FORMAT.parse(dateValue);
        return dateFormat(date, toFormat);
    }

    public static String dateFormatWithWeek(String format) {
        if (StringUtils.isEmpty(format)) {
            return "";
        }
        if (format.contains("ww")) {
            try {
                format = format.replace("ww", dateToWeek(dateFormat()));
            } catch (Exception e) {
                format = format.replace("ww", "");
            }
        }

        return new SimpleDateFormat(format, Locale.ENGLISH).format(new Date());
    }

    public static String dateToWeek(String dateValue) throws Exception {
        Date date = dateParse(dateValue, "yyyy-MM-dd");

        int week = 0;
        if (date != null) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            week = cal.get(Calendar.DAY_OF_WEEK) - 1;
            if (week < 0) {
                week = 0;
            }
        }

        return WEEK_DAYS[week];
    }

    public static String formatDateToday() {
        return dateFormat("yyyyMMdd");
    }

    public static String formatDateNow() {
        return dateFormat("yyyyMMdd HH:mm:ss");
    }

    public static Date dateParse(String dateValue) throws Exception {
        return SIMPLE_DATE_FORMAT.parse(dateValue);
    }

    /**
     * 时间字符串格式化
     *
     * @param dateValue  待格式化时间字符串
     * @param fromFormat 待格式化时间字符串格式
     * @return 时间字符串
     */
    public static Date dateParse(String dateValue, String fromFormat) throws Exception {
        if (StringUtils.isEmpty(dateValue) || StringUtils.isEmpty(fromFormat)) {
            return null;
        }

        return new SimpleDateFormat(fromFormat, Locale.CHINA).parse(dateValue);
    }

    public static Date dateParseWhitDefault(String dateValue) {
        try {
            return dateParse(dateValue);
        } catch (Exception ex) {
            return new Date();
        }
    }

    /**
     * 获取Linux时间戳，精确到秒
     *
     * @return Linux时间戳
     */
    public static String currentTime() {
        return String.valueOf(System.currentTimeMillis() / 1000);
    }

    /**
     * 根据指定天数差值，取未来日期(+)或过往日期(-)
     *
     * @return
     */
    public static Date dayAddAndSub(Date srcDate, int day) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(srcDate);

        calendar.add(Calendar.DATE, day);
        Date startDate = calendar.getTime();
        return startDate;
    }

    public static boolean compareTime(Date compareDate,int day) {
        boolean flag = false;
        try {
            Date nowTime = new Date();
            Date compareTime = dayAddAndSub(compareDate,day);
            if (nowTime.before(compareTime)) {
                flag = true;
            }


        } catch (Exception e) {
            e.printStackTrace();
        }
        return flag;
    }

}
