package com.hisense.common.util;

import com.alibaba.fastjson.JSONObject;

import java.util.Map;

/**
 * Map 工具类
 *
 * @author dukunbiao(null) 2018-06-20
 */
public class MapUtils {

    private MapUtils() {

    }

    /**
     * 检查Map是否为空
     *
     * @param map 待检查Map
     * @return true/false
     */
    public static boolean isEmpty(Map map) {
        return (map == null || map.isEmpty());
    }

    /**
     * String型获取Map value值
     *
     * @param map Map
     * @param key key值
     * @return value值
     */
    public static String getMapString(Map map, String key) {
        if (isEmpty(map) || StringUtils.isEmpty(key)) {
            return "";
        }

        if (map.containsKey(key) && map.get(key) != null) {
            return String.valueOf(map.get(key));
        }

        return "";
    }

    /**
     * 按指定格式获取Map中value值
     * key 不存在，value 为空或null --> 返回value = 0.00
     * 返回小数，精确到小数点后两位
     *
     * @param map
     * @param key
     * @return
     */
    public static String getMapStringV2(Map map, String key) {
        double value = 0d;
        try {
            if (map.containsKey(key)) {
                if (StringUtils.isEmpty(map.get(key).toString())) {
                    value = 0d;
                } else {
                    value = Double.valueOf(map.get(key).toString());
                }
            }
        } catch (NumberFormatException e) {
            value = 0d;
        }

        return BigDecimalUtils.decimalFormat(value);
    }

    /**
     * int型获取Map value值
     *
     * @param map Map
     * @param key key值
     * @return value值
     * @throws Exception
     */
    public static int getMapInt(Map map, String key) throws Exception {
        if (isEmpty(map) || StringUtils.isEmpty(key)) {
            return 0;
        }

        if (map.containsKey(key) && map.get(key) != null) {
            return Integer.valueOf(map.get(key).toString());
        }

        return 0;
    }

    /**
     * int型获取Map value值，无法转化则返回默认值
     *
     * @param map          Map
     * @param key          key值
     * @param defaultValue 默认值
     * @return value值
     */
    public static int getMapInt(Map map, String key, int defaultValue) {
        if (isEmpty(map) || StringUtils.isEmpty(key)) {
            return defaultValue;
        }

        try {
            if (map.containsKey(key) && map.get(key) != null) {
                return Integer.valueOf(map.get(key).toString());
            }
        } catch (NumberFormatException e) {
            return defaultValue;
        }

        return defaultValue;
    }

    /**
     * double型获取Map value值
     *
     * @param map Map
     * @param key key值
     * @return value值
     * @throws Exception
     */
    public static double getMapDouble(Map map, String key) throws Exception {
        if (isEmpty(map) || StringUtils.isEmpty(key)) {
            return 0d;
        }

        if (map.containsKey(key) && map.get(key) != null) {
            return Double.valueOf(map.get(key).toString());
        }

        return 0d;
    }

    /**
     * double型获取Map value值，无法转化则返回默认值
     *
     * @param map          Map
     * @param key          key值
     * @param defaultValue 默认值
     * @return value值
     */
    public static double getMapDouble(Map map, String key, double defaultValue) {
        if (isEmpty(map) || StringUtils.isEmpty(key)) {
            return defaultValue;
        }

        try {
            if (map.containsKey(key) && map.get(key) != null) {
                return Double.valueOf(map.get(key).toString());
            }
        } catch (NumberFormatException e) {
            return defaultValue;
        }

        return defaultValue;
    }

    /**
     * Java Bean 转Map
     *
     * @param object 待转化Java Bean
     * @return Map
     */
    public static Map<String, String> javaBeanToMap(Object object) {
        return JSONUtils.JSONToMap(JSONObject.parseObject(JSONObject.toJSONString(object)));
    }

    /**
     * Map转 Java Bean
     *
     * @param map Map
     * @return Java Bean
     */
    public static <T> T mapToJavaBean(Map map, Class<T> clazz) {
        return JSONObject.parseObject(new JSONObject(map).toJSONString(), clazz);
    }
}
