package com.hisense.common.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 数字类型判断
 *
 * @author xieliyan 2018-01-05
 */
public class NumberUtils {

    /**
     * 返回是否为正整数
     *
     * @param original 数值
     * @return true 是正整数 false 不是正整数
     */
    public static boolean isPositiveInteger(String original) {
        return isMatch("^\\+{0,1}[1-9]\\d*", original);
    }

    /**
     * 返回是否为负整数
     *
     * @param original 数值
     * @return true 是负整数 false 不是负整数
     */
    public static boolean isNegativeInteger(String original) {
        return isMatch("^-[1-9]\\d*", original);
    }

    /**
     * 返回是否为整数
     *
     * @param original 数值
     * @return true 是整数 false 不是整数
     */
    public static boolean isWholeNumber(String original) {
        return isMatch("[+-]{0,1}0", original) || isPositiveInteger(original) || isNegativeInteger(original);
    }

    /**
     * 返回是否为正小数
     *
     * @param original 数值
     * @return true 是正小数 false 不是正小数
     */
    public static boolean isPositiveDecimal(String original) {
        return isMatch("\\+{0,1}[0]\\.[1-9]*|\\+{0,1}[1-9]\\d*\\.\\d*", original);
    }

    /**
     * 返回是否为负小数
     *
     * @param original 数值
     * @return true 是负小数 false 不是负小数
     */
    public static boolean isNegativeDecimal(String original) {
        return isMatch("^-[0]\\.[1-9]*|^-[1-9]\\d*\\.\\d*", original);
    }

    /**
     * 返回是否为小数
     *
     * @param original 数值
     * @return true 是小数 false 不是小数
     */
    public static boolean isDecimal(String original) {
        return isMatch("[-+]{0,1}\\d+\\.\\d*|[-+]{0,1}\\d*\\.\\d+", original);
    }

    /**
     * 返回是否为实数
     *
     * @param original 数值
     * @return true 是实数 false 不是实数
     */
    public static boolean isRealNumber(String original) {
        return isWholeNumber(original) || isDecimal(original);
    }

    /**
     * 匹配方法
     *
     * @param original 数值
     * @param regex    匹配值
     * @return true 匹配 false 不匹配
     */
    private static boolean isMatch(String regex, String original) {
        if (original == null || original.trim().equals("")) {
            return false;
        }
        Pattern pattern = Pattern.compile(regex);
        Matcher isNum = pattern.matcher(original);
        return isNum.matches();
    }
}
