/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi;

import com.hisense.crm.Cashier;

/**
 * WebAPI 的执行环境。
 *
 * @author zhangsai 2017-09-02
 * @author xieliyan 2017-11-12
 * @author guolina  2019-01-04 HIPOSAPP-850 登陆界面添加显示门店名称
 */
public class Context {

    // Context 的实例
    private static Context INSTANCE;

    // 组织编码
    private String orgCode;
    // 组织名称
    private String orgName;
    // 收款机号
    private String posNo;
    private Cashier cashier;


    /**
     * 初始化一个 {@code Context} 的实例。
     */
    private Context() {
        // NO CODE HERE. 避免初始化。
    }


    /**
     * 返回配置的组织编码。
     *
     * @return 组织编码
     */
    public static String orgCode() {
        return INSTANCE.orgCode;
    }

    /**
     * 返回配置的组织名称。
     *
     * @return 组织名称
     */
    public static String orgName() {
        return INSTANCE.orgName;
    }

    /**
     * 返回配置的收款机号。
     *
     * @return 收款机号
     */
    public static String posNo() {
        return INSTANCE.posNo;
    }

    /**
     * 返回配置的营业员信息。
     *
     * @return 营业员信息
     */
    public static Cashier cashier() {
        return INSTANCE.cashier;
    }

    /**
     * 设置当前登录的收银员。
     *
     * @param cashier 收银员
     */
    public static void cashier(Cashier cashier) {
        if (INSTANCE != null) {
            INSTANCE.cashier = cashier;
        }
    }


    /**
     * 配置 WebAPI 执行环境。
     *
     * @param orgCode 组织编码
     * @param orgName 组织名称
     * @param posNo   收款机号
     * @param cashier 营业员信息
     */
    public static void config(String orgCode, String orgName, String posNo, Cashier cashier) {
        Context context = new Context();
        context.orgCode = orgCode;
        context.orgName = orgName;
        context.posNo = posNo;
        context.cashier = cashier;
        INSTANCE = context;
    }


}
