/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi;

import com.hisense.security.RSA;
import com.hisense.webapi.WebApi;

/**
 * WebAPI 的 RSA 加密/解密服务。
 *
 * @author zhangsai 2017-09-02
 */
public class RSAService {

    /**
     * 注意：WebAPI 加密/解密使用两对不同的 RSA 公钥/私钥。
     * RSA 公钥用于加密请求数据；RSA 私钥用于解密返回数据。
     */

    // WebAPI 的 RSA 公钥
    private final static String PUBLIC_KEY = "<RSAKeyValue><Modulus>b2kBrWhcWE5rSovNtXDn9bRTXe1atGxGaGGyC80ytseqTrai+GnHqtswypxrqh3Pgigv48qBJDV7g9ZjTaDvnTTgUFFA0GZbri8bxLJBv5XsvwnbJkd9FqYO/8+dUYJhbsPtFWSaXCMVNqFgcmdrxP7XaN4cupfVPCWxd7MYtv8=</Modulus><Exponent>AQAB</Exponent></RSAKeyValue>";
    // WebAPI 的 RSA 私钥
    private final static String PRIVATE_KEY = "<RSAKeyValue><Modulus>ULR6ekEXCMS1gCl9fy6v/K3+Xz0ubpaB/TKexkbFXbLEmtIWgM6E4IqOKqK11py2ANZrhcCJSFVlKpY8IG/KfhyIFsP7g8QTnWIh5R406H691kr80NZSs4n42EV9SQRG/z01b4MLsd0PVceGW1dCdnzedAnUesX8o4dE3qYtnFc=</Modulus><Exponent>AQAB</Exponent><P>lB+jUZ4So+nyC27l+1H/MfddRbCoKsQWedoGdZq50bHtOJLOOTxMskJhsM/7nzEqbX00Nb/d2vgzD/GZiNONkw==</P><Q>i3s6Ec2RrlfefEJ5F3y6rZkUc6h7jkvrtrn+zt7XltSS7K39M2WKEX50hczyV45V42rJN2WqGUHeumB2VMNQrQ==</Q><DP>SSnTbrYIeQRONQ2C9X9L8O9BF/IpLm6Kt4dUl3RkgkpDZSVGTYkWTRpyzHX+0GPllA06462wtL7R4fcxIh90lw==</DP><DQ>NYWnlsVHLZtSKGRpivW1vH+NNYRlRQkk/g0JFeC04Y/YgTGSg3JnNKdNlzo5lBduJcIITFWam+6GxQXW4lfAUQ==</DQ><InverseQ>Ri387F5sa2TKvyLmFfsxHzygKSrx+ZWcEXwqtGdyKo9IA8EWQYXAmRR1zyZNCiq9OnWfPkwoqbSnEqxiEUN8yg==</InverseQ><D>Kr0Ix0lE+33AaFiQfxOoWvWBacoqlqHZwM5XysSbrF0JRLvQ8vFq6MY3N1jnVvmyYIssca5ehRHD+2OH/91zq/D9Z489YaUS55seT74lpLbjAQknVnvSX7zpadx1rXwr2qJDCcpc0Ix8R43kL+oWMGtjn7il5S00Dp2zL9KGEbE=</D></RSAKeyValue>";
    // RSA 加密/解密的实例
    private final static RSA RSA = new RSA(PUBLIC_KEY, PRIVATE_KEY);


    /**
     * 初始化一个 {@code RSAService} 的实例。
     */
    private RSAService() {
        // NO CODE HERE. 避免初始化。
    }


    /**
     * 使用默认公钥加密字符串。
     * @param value 待加密的字符串
     * @return 加密后的 Base64 字符串
     */
    public static String encrypt(final String value) {
        if (WebApi.isAmwAndAip()){
            return value;
        } else {
            return RSA.encrypt(value);
        }
    }

    /**
     * 使用默认私钥解密 Base64 字符串。
     * @param value 待解密的 Base64 字符串
     * @return 解密后的字符串
     */
    public static String decrypt(final String value) {
        return RSA.decrypt(value);
    }

}
