/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.crm.webapi.account;

import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 账户支付的请求数据。
 *
 * @author zhangsai    2018-02-01 HIPOSAPP-313
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 支付接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 支付接口增加校验码cardVerifyCode字段
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 */
public class AccountPayRequestData {
    /**
     * 批次号
     */
    public String batchNo;
    /**
     * 卡号
     */
    public String cardNo;
    /**
     * 卡号类型
     */
    public int cardNoType;
    /**
     * 收银员编码
     */
    public String cshCode;
    /**
     * 收银员名称
     */
    public String cshName;
    /**
     * 是否校验密码	0 - 不校验 1 - 校验 非必传，默认为0 仅cardNoType=7时生效
     */
    public String isVerifyPwd;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 支付密码
     */
    public String password;
    /**
     * 校验位
     */
    public String cardFaceNoCheck;
    /**
     * 校验码
     */
    public String cardVerifyCode;
    /**
     * 支付方式编码
     */
    public String payCode;
    /**
     * 支付方式名称
     */
    public String payName;
    /**
     * 支付时间
     */
    public String payTime;
    /**
     * 收款机号
     */
    public String posNo;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 顺序号
     */
    public int serialNo;
    /**
     * 支付金额
     */
    public String totalPay;
    /**
     * 商户订单号
     */
    public String outTradeNo;
    /**
     * 支付方式remark
     */
    public String payMethod;
    /**
     * CRM类型
     */
    public String crmType;
    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param cardNo       支付卡号
     * @param cardSecurity 校验方式
     * @param isVerifyPwd  是否校验密码
     * @return WebAPI 请求
     */
    public static Request<AccountPayRequestData> request(CardNo cardNo, CardSecurity cardSecurity, SalePayModel payModel, boolean isVerifyPwd) {
        AccountPayRequestData requestData = new AccountPayRequestData();
        requestData.batchNo = RSAService.encrypt("");
        requestData.cardFaceNoCheck = cardSecurity == null ? "" : cardSecurity.cardFaceNoCheck();
        requestData.cardNo = RSAService.encrypt(cardNo.value());
        requestData.cardNoType = cardNo.getTypeCode();
        requestData.cardVerifyCode = cardSecurity == null ? "" : cardSecurity.cardVerifyCode();
        requestData.cshCode = Context.cashier().getCode();
        requestData.cshName = Context.cashier().getName();
        requestData.isVerifyPwd = isVerifyPwd ? "1" : "0";
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.password = RSAService.encrypt((cardSecurity == null) ? "" : cardSecurity.password());
        requestData.payCode = RSAService.encrypt(payModel.getPayCode());
        requestData.payName = payModel.getPayName();
        requestData.payTime = payModel.getPayTime();
        requestData.posNo = RSAService.encrypt(Context.posNo());
        requestData.saleNo = RSAService.encrypt(payModel.getSaleNoTemp());
        requestData.serialNo = payModel.getSerialNo();
        requestData.totalPay = RSAService.encrypt(payModel.getTotalPaid());
        requestData.outTradeNo = payModel.getOutTradeNo();
        requestData.payMethod = payModel.getRemark();
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
