/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.account;

import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 账户查询的请求数据。
 *
 * @author zhangsai    2017-10-09
 * @author zhangsai    2017-12-27 增加创建查询请求的方法
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 查询接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 查询接口增加校验码cardVerifyCode字段
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 */
public final class AccountQueryRequestData {
    /**
     * 卡面号
     */
    public String cardFaceNo;
    /**
     * 卡面号校验码
     */
    public String cardFaceNoCheck;
    /**
     * 卡号
     */
    public String cardNo;
    /**
     * 卡号类型
     */
    public int cardNoType;
    /**
     * 卡片校验码
     */
    public String cardVerifyCode;
    /**
     * 是否校验密码	0 - 不校验 1 - 校验 非必传，默认为0 仅cardNoType=7时生效
     */
    public String isVerifyPwd;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 卡片密码
     */
    public String password;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * CRM类型
     */
    public String crmType;
    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param cardNo       卡号
     * @param cardSecurity 校验方式
     * @param isVerifyPwd  是否校验密码
     * @return WebAPI 请求
     */
    public static Request<AccountQueryRequestData> request(CardNo cardNo, CardSecurity cardSecurity, String saleNo, boolean isVerifyPwd) {
        AccountQueryRequestData requestData = new AccountQueryRequestData();
        requestData.cardFaceNo = "";
        requestData.cardFaceNoCheck = (cardSecurity == null) ? "" : cardSecurity.cardFaceNoCheck();
        requestData.cardNo = RSAService.encrypt(cardNo.value());
        requestData.cardNoType = cardNo.getTypeCode();
        requestData.cardVerifyCode = (cardSecurity == null) ? "" : cardSecurity.cardVerifyCode();
        requestData.isVerifyPwd = isVerifyPwd ? "1" : "0";
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.password = RSAService.encrypt((cardSecurity == null) ? "" : cardSecurity.password());
        requestData.saleNo = RSAService.encrypt(saleNo);
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
