package com.hisense.crm.webapi.account;

import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 账户退款的请求数据。
 *
 * @author zhangsai    2018-07-23 HIPOSAPP-685
 * @author guolina     2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 退货接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 退货接口增加校验码cardVerifyCode字段,
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 */
public class AccountReturnRequestData {
    /**
     * 收银员编码
     */
    public String cshCode;
    /**
     * 收银员名称
     */
    public String cshName;
    /**
     * 原交易流水号
     */
    public String oldSaleNo;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 支付密码
     */
    public String password;
    /**
     * 卡面后几位
     */
    public String cardFaceNoCheck;
    /**
     * 校验码
     */
    public String cardVerifyCode;
    /**
     * 是否校验密码	0 - 不校验 1 - 校验 非必传，默认为0 仅cardNoType=7时生效
     */
    public String isVerifyPwd;
    /**
     * 支付方式编码
     */
    public String payCode;
    /**
     * 支付方式名称
     */
    public String payName;
    /**
     * 支付时间
     */
    public String payTime;
    /**
     * 收款机号
     */
    public String posNo;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 顺序号
     */
    public int serialNo;
    /**
     * 支付金额
     */
    public String totalPay;
    /**
     * 会员号
     */
    public String vipCardNo;
    /**
     * 卡号类型
     */
    public String cardNoType;

    /**
     * 商户订单号
     */
    public String outTradeNo;
    /**
     * 支付方式标记
     */
    public String payMethod;
    /**
     * CRM类型
     */
    public String crmType;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param cardNo       支付卡号
     * @param cardSecurity 校验方式
     * @param payment      支付信息
     * @param isVerifyPwd  是否校验密码
     * @return WebAPI 请求
     */
    public static Request<AccountReturnRequestData> request(CardNo cardNo, CardSecurity cardSecurity,
                                                            String oldSaleNo, SalePayModel payment, String outTradeNo, boolean isVerifyPwd) {
        AccountReturnRequestData requestData = new AccountReturnRequestData();

        requestData.cshCode = Context.cashier().getCode();
        requestData.cshName = Context.cashier().getName();
        requestData.oldSaleNo = RSAService.encrypt(oldSaleNo);
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.password = RSAService.encrypt((cardSecurity == null) ? "" : cardSecurity.password());
        requestData.cardFaceNoCheck = (cardSecurity == null) ? "" : cardSecurity.cardFaceNoCheck();
        requestData.cardVerifyCode = (cardSecurity == null) ? "" : cardSecurity.cardVerifyCode();
        requestData.isVerifyPwd = isVerifyPwd ? "1" : "0";
        requestData.payCode = RSAService.encrypt(payment.getPayCode());
        requestData.payName = payment.getPayName();
        requestData.payTime = payment.getPayTime();
        requestData.posNo = RSAService.encrypt(Context.posNo());
        requestData.saleNo = RSAService.encrypt(payment.getSaleNoTemp());
        requestData.serialNo = payment.getSerialNo();
        requestData.totalPay = RSAService.encrypt(String.valueOf(payment.getTotalPaid()));
        requestData.vipCardNo = RSAService.encrypt(cardNo.value());
        requestData.cardNoType = String.valueOf(cardNo.getTypeCode());
        requestData.outTradeNo = outTradeNo;
        requestData.payMethod = payment.getRemark();
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
