/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.card;

import com.hisense.crm.Cashier;
import com.hisense.crm.card.Card;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardPayResult;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.webapi.account.AccountPayRequestData;
import com.hisense.crm.webapi.account.AccountQueryRequestData;
import com.hisense.crm.webapi.account.AccountReturnRequestData;
import com.hisense.crm.webapi.trans.TransactionResultData;
import com.hisense.crm.webapi.trans.TransactionRevokeRequestData;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.concurrent.TimeUnit;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 储值卡服务。
 *
 * @author zhangsai     2017-10-09
 * @author zhangsai     2017-12-27 queryByPassword() 返回结果增加卡片状态
 * @author zhangsai     2018-01-31 HIPOSAPP-311
 * @author zhangsai     2018-02-01 HIPOSAPP-313 储值卡支付
 * @author zhangsai     2018-02-01 HIPOSAPP-314 储值卡支付撤销
 * @author zhangsai     2018-07-23 HIPOSAPP-685 储值卡退款
 * @author guolina      2018-09-26 HIPOSAPP-770 支持销售/退款后增加储值卡和积分余额打印
 * @author guanguangjin 2019-04-05 HIORP-121 储值卡赠送
 * @author guanguangjin 2019-04-05 HIORP-128 储值卡付款码
 * @author lizhixin     2019-04-09 HIORP-130 购买储值卡
 * @author guolina      2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author wangyaogang  2019-05-23 HIPOSAPP-1124 请求储值卡支付时增加交易查询避免网络问题导致单边账
 * @author wangyaogang  2019-05-24 HIPOSAPP-1123 优化储值卡/积分/海信支付调用支付结果查询接口的逻辑
 * @author wangyaogang  2019-06-05 HIPOSAPP-1151 储值卡支付,撤销和退款时超时时间设置为2分钟
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 储值卡 查询/支付/撤销/退款接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 储值卡查询,支付,撤销，退款增加校验码cardVerifyCode字段
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 储值卡校验参数封装为CardSecurity
 * @author wangyaogang 2019-10-08 HIPOSAPP-1305 储值卡消费可限制品牌品类
 */
public final class CardService {

    private CardService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 支付。
     *
     * @param cardNo       支付卡号
     * @param cardSecurity 校验方式
     * @param isVerifyPwd  是否校验密码
     * @param callback     储值卡支付的回调方法
     */
    public static void pay(CardNo cardNo, CardSecurity cardSecurity, SalePayModel payModel, boolean isVerifyPwd,
                           final com.hisense.webapi.Callback<CardPayResult> callback) {
        Request<AccountPayRequestData> request = AccountPayRequestData.request(cardNo, cardSecurity, payModel, isVerifyPwd);
        ICardService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ICardService.class);
        WebApi.requestAsync(service.pay(request), new Callback<Result<TransactionResultData>>() {
            @Override
            public void onResponse(Call<Result<TransactionResultData>> call, Response<Result<TransactionResultData>> response) {
                CardPayResult cardPayResult = new CardPayResult(
                        response.body().data.refNo,
                        response.body().data.tranTime,
                        response.body().data.balance,
                        request.data.outTradeNo);
                callback.onSuccess(cardPayResult);
            }

            @Override
            public void onFailure(Call<Result<TransactionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },TransactionResultData.class);
    }

    /**
     * 查询储值卡信息。
     *
     * @param cardNo       储值卡卡号
     * @param cardSecurity 校验方方式
     * @param isVerifyPwd  是否校验密码
     * @param callback     储值卡查询的回调方法
     */
    public static void query(CardNo cardNo, CardSecurity cardSecurity, boolean isVerifyPwd,
                             final com.hisense.webapi.Callback<Card> callback) {
        Request<AccountQueryRequestData> request = AccountQueryRequestData.request(cardNo, cardSecurity, null, isVerifyPwd);

        WebApi.requestAsync(WebApi.create(ICardService.class).query(request), new Callback<Result<CardQueryResultData>>() {
            @Override
            public void onResponse(Call<Result<CardQueryResultData>> call, Response<Result<CardQueryResultData>> response) {
                callback.onSuccess(CardQueryResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<CardQueryResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },CardQueryResultData.class);
    }

    /**
     * 查询限制金额
     *
     * @param cardNo   储值卡卡号
     * @param sale     当前交易
     * @param card     查询余额返回的卡信息
     * @param callback 储值卡限制金额查询的回调方法
     */
    public static void useLimit(CardNo cardNo, final BaseSaleModel sale, final Card card,
                                final com.hisense.webapi.Callback<Card> callback) {
        Request<CardUseLimitRequestData> request = CardUseLimitRequestData.request(cardNo, sale);

        WebApi.requestAsync(WebApi.create(ICardService.class).useLimit(request), new Callback<Result<CardUseLimitResultData>>() {
            @Override
            public void onResponse(Call<Result<CardUseLimitResultData>> call, Response<Result<CardUseLimitResultData>> response) {
                callback.onSuccess(CardUseLimitResultData.handleResult(response.body().data, card, sale));
            }

            @Override
            public void onFailure(Call<Result<CardUseLimitResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },CardUseLimitResultData.class);
    }

    /**
     * 支付撤销。
     *
     * @param payModel    支付信息
     * @param isVerifyPwd 是否校验密码
     * @param callback    储值卡支付的回调方法
     */
    public static void revoke(SalePayModel payModel, boolean isVerifyPwd, final com.hisense.webapi.Callback<String> callback) {
        if (payModel.getCardNo() == null || (payModel.getCardNo().getType() != CardNoType.VipCode && payModel.getCardNo().getType() != CardNoType.PayCode)) {
            callback.onError(new Exception("卡号类型错误"));
            return;
        }

        Request<TransactionRevokeRequestData> request = TransactionRevokeRequestData.request(payModel, isVerifyPwd);
        ICardService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ICardService.class);
        WebApi.requestAsync(service.revoke(request), new Callback<Result<TransactionResultData>>() {
            @Override
            public void onResponse(Call<Result<TransactionResultData>> call, Response<Result<TransactionResultData>> response) {
                callback.onSuccess(response.body().data.refNo);
            }

            @Override
            public void onFailure(Call<Result<TransactionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },TransactionResultData.class);
    }

    /**
     * 退款。
     *
     * @param cardNo       支付卡号
     * @param cardSecurity 校验方式
     * @param oldSaleNo    原交易流水号
     * @param payment      支付信息
     * @param isVerifyPwd  是否校验密码
     * @param callback     储值卡支付的回调方法
     */
    public static void returns(CardNo cardNo, CardSecurity cardSecurity, String oldSaleNo, SalePayModel payment, String outTradeNo,
                               boolean isVerifyPwd, final com.hisense.webapi.Callback<CardPayResult> callback) {
        if (cardNo == null || (cardNo.getType() != CardNoType.VipCode && cardNo.getType() != CardNoType.PayCode)) {
            callback.onError(new Exception("卡号类型错误"));
            return;
        }

        Request<AccountReturnRequestData> request = AccountReturnRequestData.request(cardNo, cardSecurity, oldSaleNo, payment, outTradeNo, isVerifyPwd);
        ICardService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ICardService.class);
        WebApi.requestAsync(service.returns(request), new Callback<Result<TransactionResultData>>() {
            @Override
            public void onResponse(Call<Result<TransactionResultData>> call, Response<Result<TransactionResultData>> response) {
                CardPayResult cardPayResult = new CardPayResult(
                        response.body().data.refNo,
                        response.body().data.tranTime,
                        response.body().data.balance);
                callback.onSuccess(cardPayResult);
            }

            @Override
            public void onFailure(Call<Result<TransactionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },TransactionResultData.class);
    }

    /**
     * 查询会员卡交易明细。
     *
     * @param cardNo    支付卡号
     * @param pageNo    页码
     * @param pageRows  每页数据条数
     * @param beginDate 开始日期
     * @param endDate   结束日期
     */
    public static CardTransResultData queryCardTransDetails(CardNo cardNo, String pageNo, String pageRows, String beginDate, String endDate) throws WebApiException {
        Request<CardTransRequestData> request = CardTransRequestData.request(cardNo, pageNo, beginDate, endDate, pageRows);

        CardTransResultData resultData = WebApi.request(WebApi.create(ICardService.class).queryCardTransDetails(request));

        return resultData;
    }

    /**
     * 储值卡充值。
     *
     * @param cardNo       支付卡号
     * @param cashier      收款员信息
     * @param depositTotal 充值金额
     * @param giftTotal    赠送金额
     * @param refNo        外部支付单号
     * @param accDate      结账日期
     */
    public static CardRechargeResultData deposit(CardNo cardNo, Cashier cashier,
                                                 Double depositTotal, Double giftTotal, String refNo, String accDate) throws WebApiException {
        Request<CardRechargeRequestData> request = CardRechargeRequestData.request(
                cardNo, cashier, depositTotal, giftTotal, refNo, accDate);
        CardRechargeResultData resultData = WebApi.request(WebApi.create(ICardService.class).deposit(request));

        return resultData;
    }

    /**
     * 储值卡赠送
     *
     * @param cardFaceNo 卡号
     * @param fromCardNo 赠送人卡号
     * @param toCardNo   受赠人卡号
     */
    public static CardGiftResultData gift(String cardFaceNo, CardNo fromCardNo, CardNo toCardNo)
            throws WebApiException {
        if (cardFaceNo == null) {
            throw new WebApiException("未提供待赠送的卡面号");
        }
        if (fromCardNo == null) {
            throw new WebApiException("赠送人卡号不能为空");
        }
        if (toCardNo == null) {
            throw new WebApiException("受赠人卡号不能为空");
        }
        Request<CardGiftRequestData> request = CardGiftRequestData.request(cardFaceNo, fromCardNo, toCardNo);
        return WebApi.request(WebApi.create(ICardService.class).gift(request));
    }

    /**
     * 储值卡付款码
     *
     * @param cardNo 卡号
     * @return 付款码
     */
    public static CardPayCodeResultData payCode(CardNo cardNo)
            throws WebApiException {
        if (cardNo == null) {
            throw new WebApiException("卡号不能为空");
        }
        Request<CardPayCodeRequestData> request = CardPayCodeRequestData.request(cardNo);
        return WebApi.request(WebApi.create(ICardService.class).payCode(request));
    }

    /**
     * 购买储值卡
     *
     * @param orgCode    组织号
     * @param cardNo     卡号
     * @param cardNoType 1-卡面号； 3-会员卡ID
     * @param billNo     订单号
     * @param saleCount  销售数量
     * @param total      总金额
     * @param dscTotal   折扣金额
     * @param payTotal   实收金额
     * @param items      明细信息
     * @param pay        支付信息
     * @return 购买卡信息
     */
    public static CardBuyResultData buy(String orgCode, String cardNo, int cardNoType, String billNo,
                                        String saleCount, String total, String dscTotal, String payTotal,
                                        java.util.List<CardBuyItemRequestData> items, java.util.List<CardBuyPayRequestData> pay) throws WebApiException {
        if (orgCode == null) {
            throw new WebApiException("组织号不能为空");
        }
        if (cardNo == null) {
            throw new WebApiException("会员卡号不能为空");
        }
        if (billNo == null) {
            throw new WebApiException("订单号不能为空");
        }
        if (saleCount == null) {
            throw new WebApiException("销售数量不能为空");
        }
        if (total == null) {
            throw new WebApiException("总金额不能为空");
        }
        if (dscTotal == null) {
            throw new WebApiException("折扣金额不能为空");
        }
        if (payTotal == null) {
            throw new WebApiException("实收金额不能为空");
        }
        if (items == null) {
            throw new WebApiException("明细信息不能为空");
        }
        if (pay == null) {
            throw new WebApiException("支付信息不能为空");
        }

        Request<CardBuyRequestData> request = CardBuyRequestData.request(orgCode, cardNo, cardNoType, billNo, saleCount, total, dscTotal, payTotal, items, pay);
        return WebApi.request(WebApi.create(ICardService.class).buy(request));
    }
}
