/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.card;

import com.hisense.common.util.DateUtils;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.UseLimitPlu;
import com.hisense.crm.webapi.Context;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 储值卡查询可用金额接口
 *
 * @author wangyaogang 2019-10-08 HIPOSAPP-1305 储值卡查询可用金额接口
 * @author wangyaogang 2019-10-15 HIPOSAPP-1318 储值卡限制品牌/品类消费逻辑调整,不再限制只可使用一次,每次支付完进行分摊,查询时传未支付金额
 */
public class CardUseLimitRequestData {

    /**
     * 卡号
     */
    public String cardNo;

    /**
     * 卡号类型
     */
    public int cardNoType;

    /**
     * 组织编码
     */
    public String orgCode;

    /**
     * 收款机号
     */
    public String posNo;

    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 交易商品
     */
    public List<UseLimitPlu> plu;
    /**
     * 交易时间
     */
    public String saleTime;
    /**
     * 交易类型
     */
    public int tranType;


    public CardUseLimitRequestData() {

    }

    public static Request<CardUseLimitRequestData> request(CardNo cardNo, BaseSaleModel sale) {
        CardUseLimitRequestData requestData = new CardUseLimitRequestData();
        requestData.cardNo = cardNo.value();
        requestData.cardNoType = cardNo.getTypeCode();
        requestData.orgCode = Context.orgCode();
        requestData.posNo = Context.posNo();
        requestData.saleNo = sale.getSaleNo();
        requestData.plu = new ArrayList<>();

        for (BaseSalePluModel salePluModel : sale.getPlus()) {
            UseLimitPlu useLimitPlu = new UseLimitPlu();
            useLimitPlu.brandCode = salePluModel.getBrandCode();
            useLimitPlu.classCode = salePluModel.getClassCode();
            useLimitPlu.depCode = salePluModel.getDepCode();
            useLimitPlu.shopCode = salePluModel.getShopCode();
            useLimitPlu.itemNo = salePluModel.getItemNo();
            useLimitPlu.pageNo = salePluModel.getPageNo();
            useLimitPlu.pluCode = salePluModel.getPluCode();

            if (sale instanceof SaleModel) {
                List<UseLimitPlu> pluList = ((SaleModel) sale).getPluList();
                //如果已分摊列表为空,则直接使用total
                if (pluList == null || pluList.isEmpty()) {
                    useLimitPlu.total = salePluModel.getTotalRecv();
                } else {
                    //否则,查询pluCode,有,相减,否则使用total
                    boolean hasFound = false;
                    for (int i = 0; i < pluList.size(); i++) {
                        UseLimitPlu limitPlu = pluList.get(i);
                        if (useLimitPlu.pluCode.equals(limitPlu.pluCode)) {
                            hasFound = true;
                            useLimitPlu.total = new BigDecimal(salePluModel.getTotalRecv()).subtract(limitPlu.totalDiscount()).toPlainString();
                            break;
                        }
                    }
                    if (!hasFound) {
                        useLimitPlu.total = salePluModel.getTotalRecv();
                    }
                }
            } else {
                useLimitPlu.total = salePluModel.getTotalRecv();
            }
            requestData.plu.add(useLimitPlu);
        }

        requestData.saleTime = DateUtils.dateFormat("yyyyMMdd");
        requestData.tranType = (sale instanceof SaleModel) ? 0 : 1;

        return WebApi.sign(requestData);
    }

}
