/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.card;

import com.hisense.crm.card.Card;
import com.hisense.crm.card.UseLimitPlu;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.SaleModel;

import java.util.List;

/**
 * 储值卡查询的返回结果。
 *
 * @author wangyaogang 2019-10-08 HIPOSAPP-1305 储值卡查询可用金额接口
 * @author wangyaogang 2019-10-15 HIPOSAPP-1318 储值卡限制品牌/品类消费逻辑调整,不再限制只可使用一次,每次支付完进行分摊,查询时传未支付金额
 */
final class CardUseLimitResultData {
    /**
     * 卡号
     */
    public String cardNo;
    /**
     * 会员卡号//
     */
    public String vipCardNo;
    /**
     * 卡面号//
     */
    public String cardFaceNo;
    /**
     * 可用商品待支付金额合计//
     */
    public String total;
    /**
     * 支付参考金额//
     */
    public String totalLimit;

    /**
     * 卡类型编码
     */
    public String cardType;

    /**
     * 有效期
     */
    public String expireDate;
    /**
     * 可用余额
     */
    public String usableTotal;


    /**
     * 商品
     */
    public List<UseLimitPlu> plu;

    /**
     * 处理 WebAPI 接口返回数据。
     *
     * @param resultData 接口返回数据
     * @return 储值卡信息
     */
    public static Card handleResult(CardUseLimitResultData resultData, Card card, BaseSaleModel sale) {
        if (sale instanceof SaleModel) {
            ((SaleModel) sale).setTempPluList(resultData.plu);
        }
        return new Card.Builder(resultData.cardFaceNo, card.cardNo())
                .setCardStatus(card.status().code())
                .setCardType(card.cardType().code(), card.cardType().name())
                .setExpireDate(card.expireDate())
                .setUsableTotal(Double.valueOf(resultData.totalLimit))
                .setVipCardNo(resultData.vipCardNo)
                .setUseLimit(card.useLimitValue())
                .build();
    }
}
