/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.card;

import com.hisense.crm.webapi.account.AccountPayRequestData;
import com.hisense.crm.webapi.account.AccountQueryRequestData;
import com.hisense.crm.webapi.account.AccountReturnRequestData;
import com.hisense.crm.webapi.trans.TransactionResultData;
import com.hisense.crm.webapi.trans.TransactionRevokeRequestData;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * 储值卡服务的 retrofit 接口。
 *
 * @author zhangsai     2017-10-09
 * @author zhangsai     2018-02-01 HIPOSAPP-313 储值卡支付
 * @author zhangsai     2018-02-01 HIPOSAPP-314 储值卡支付撤销
 * @author zhangsai     2018-07-23 HIPOSAPP-685 储值卡退款
 * @author guanguangjin 2019-04-05 HIORP-121 储值卡赠送
 * @author guanguangjin 2019-04-05 HIORP-128 储值卡付款码
 * @author wangyaogang  2019-05-23 HIPOSAPP-1124 请求储值卡支付时增加交易查询避免网络问题导致单边账
 * @author wangyaogang 2019-10-08 HIPOSAPP-1305 储值卡消费可限制品牌品类
 */
interface ICardService {

    /**
     * 储值卡支付的 retrofit 请求。
     *
     * @param request 储值卡支付请求
     * @return 储值卡支付的 retrofit 调用
     */
    @POST("crm/card/pay")
    Call<Result<TransactionResultData>> pay(@Body Request<AccountPayRequestData> request);

    /**
     * 查询储值卡信息的 retrofit 请求。
     *
     * @param request 查询储值卡信息的请求
     * @return 查询储值卡信息的 retrofit 调用
     */
    @POST("crm/card/query")
    Call<Result<CardQueryResultData>> query(@Body Request<AccountQueryRequestData> request);

    /**
     * 储值卡查询可用金额接口
     *
     * @param request 查询金额的请求
     * @return 查询金额的 retrofit 调用
     */
    @POST("crm/card/useLimit")
    Call<Result<CardUseLimitResultData>> useLimit(@Body Request<CardUseLimitRequestData> request);

    /**
     * 储值卡支付退款的 retrofit 请求。
     *
     * @param request 储值卡支付退款请求
     * @return 储值卡支付退款的 retrofit 调用
     */
    @POST("crm/card/return")
    Call<Result<TransactionResultData>> returns(@Body Request<AccountReturnRequestData> request);

    /**
     * 储值卡支付撤销的 retrofit 请求。
     *
     * @param request 储值卡支付撤销请求
     * @return 储值卡支付撤销的 retrofit 调用
     */
    @POST("crm/card/revoke")
    Call<Result<TransactionResultData>> revoke(@Body Request<TransactionRevokeRequestData> request);

    /**
     * 查询会员储值卡交易明细
     *
     * @return 用户信息
     */
    @POST("crm/card/details")
    Call<Result<CardTransResultData>> queryCardTransDetails(@Body Request<CardTransRequestData> request);

    /**
     * 查询会员储值卡交易明细
     *
     * @return 用户信息
     */
    @POST("crm/card/deposit")
    Call<Result<CardRechargeResultData>> deposit(@Body Request<CardRechargeRequestData> request);

    /**
     * 储值卡赠送
     *
     * @param request 赠送请求数据
     * @return 赠送结果
     */
    @POST("crm/card/gift")
    Call<Result<CardGiftResultData>> gift(@Body Request<CardGiftRequestData> request);

    /**
     * 储值卡付款码
     *
     * @param request 请求数据
     * @return 付款码
     */
    @POST("crm/card/paycode")
    Call<Result<CardPayCodeResultData>> payCode(@Body Request<CardPayCodeRequestData> request);

    /**
     * 购买储值卡
     *
     * @param request 请求数据
     * @return 付款码
     */
    @POST("crm/card/purchase")
    Call<Result<CardBuyResultData>> buy(@Body Request<CardBuyRequestData> request);
}
