/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.coupon;

import com.hisense.crm.coupon.Coupon;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * {功能描述}
 *
 * @author zhangsai 2017-10-09
 * @author lizhixin 2019-05-24 优惠券活动查询接口
 * @author lizhixin 2019-05-28 会员优惠券绑定接口
 * @author xieliyan 2019-08-02
 */
public final class CouponService {

    private CouponService() {
        // NO CODE HERE. 避免初始化。
    }

    public static Coupon query(String qNo) throws WebApiException {
        List<String> requestData = new ArrayList<>();
        requestData.add(qNo);

        Request<List<String>> request = WebApi.sign(requestData);

        List<CouponQueryResultData> result = WebApi.request(WebApi.create(ICouponService.class).query(request));
        return new Coupon();
    }

    /**
     * 请求POS中间件优惠券活动查询接口
     *
     * @param promNos 促销单号集合
     * @return List<CouponPromotionResultData>
     * @throws WebApiException
     */
    public static List<CouponPromotionResultData> queryCoupons(List<String> promNos) throws WebApiException {
        if (promNos == null || promNos.isEmpty()) {
            throw new WebApiException("无促销单号，不予请求POS中间件优惠券活动查询接口");
        }
        Request<List<String>> request = WebApi.sign(promNos);
        return WebApi.request(WebApi.create(ICouponService.class).queryCoupons(request));
    }

    /**
     * POS中间件会员优惠券绑定接口
     *
     * @param billNo
     * @param openId
     * @param couponValue
     * @param typeCode
     * @param cardFaceNo
     * @param beginDate
     * @param endDate
     * @param count
     * @return List<String> 用户已领用过的全部优惠券号集合
     * @throws WebApiException
     */
    public static List<String> bindCoupon(String billNo, String openId, String couponValue, String typeCode, String cardFaceNo, String beginDate, String endDate, String count) throws WebApiException {
        if (billNo == null) {
            throw new WebApiException("无促销单号，不予请求POS中间件会员优惠券接口");
        }
        if (openId == null) {
            throw new WebApiException("无openId，不予请求POS中间件会员优惠券接口");
        }
        if (couponValue == null) {
            throw new WebApiException("无券面额，不予请求POS中间件会员优惠券接口");
        }
        if (typeCode == null) {
            throw new WebApiException("无券类型编码，不予请求POS中间件会员优惠券接口");
        }
        if (cardFaceNo == null) {
            throw new WebApiException("无会员卡面号，不予请求POS中间件会员优惠券接口");
        }
        if (beginDate == null) {
            throw new WebApiException("无有效开始日期，不予请求POS中间件会员优惠券接口");
        }
        if (endDate == null) {
            throw new WebApiException("无有效结束日期，不予请求POS中间件会员优惠券接口");
        }
        if (count == null) {
            throw new WebApiException("无领用数量，不予请求POS中间件会员优惠券接口");
        }
        Request<CouponBindRequestData> request = CouponBindRequestData.request(billNo, openId, couponValue, typeCode, cardFaceNo, beginDate, endDate, count);
        return WebApi.request(WebApi.create(ICouponService.class).bindCoupon(request));
    }

    /**
     * 券核销。
     *
     * @param qNo       券号
     * @param vipCardNo 会员卡号
     * @param callback  券核销的回调方法
     */
    public static void scrmCouponUse(final String qNo, String vipCardNo, final com.hisense.webapi.Callback<UsableCoupon> callback) {
        Request<ScrmCouponUseRequestData> request = ScrmCouponUseRequestData.request(qNo, vipCardNo);

        WebApi.requestAsync(WebApi.create(ICouponService.class).certiConsume(request), new Callback<Result<ScrmCouponUseResultData>>() {
            @Override
            public void onResponse(Call<Result<ScrmCouponUseResultData>> call, Response<Result<ScrmCouponUseResultData>> response) {
                callback.onSuccess(ScrmCouponUseResultData.handleResult(qNo, response.body().data));
            }

            @Override
            public void onFailure(Call<Result<ScrmCouponUseResultData>> call, Throwable t) {
                callback.onError(t);

            }
        },ScrmCouponUseResultData.class);
    }

    /**
     * 券撤销。
     *
     * @param usableCoupons 已核销券信息
     * @param callback      券撤销的回调方法
     */
    public static void scrmCouponCancel(final Collection<UsableCoupon> usableCoupons, SaleModel saleModel, SalePayModel payModel,
                                        final com.hisense.webapi.Callback<Object> callback) {
        Request<ScrmCouponCancelRequestData> request = ScrmCouponCancelRequestData.request(usableCoupons, saleModel, payModel);

        WebApi.requestAsync(WebApi.create(ICouponService.class).certiCancel(request), new Callback<Result<Object>>() {
            @Override
            public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                callback.onSuccess(null);
            }

            @Override
            public void onFailure(Call<Result<Object>> call, Throwable t) {
                callback.onError(t);

            }
        },Object.class);
    }
}
