/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.coupon;

import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import java.util.List;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * {功能描述}
 *
 * @author zhangsai 2017-10-09
 * @author lizhixin 2019-05-24 优惠券活动查询接口
 * @author lizhixin 2019-05-28 会员优惠券绑定接口
 * @author xieliyan 2019-08-02
 */
interface ICouponService {
    /**
     * 会员券查询的 retrofit 请求。
     *
     * @param request 会员券查询的请求。
     * @return 检查会员券查询的 retrofit 调用。
     */
    @POST("crm/coupon/query")
    Call<Result<List<CouponQueryResultData>>> query(@Body Request<List<String>> request);

    /**
     * POS中间件优惠券活动查询接口的请求
     *
     * @param request 促销单号集合
     * @return Caller
     */
    @POST("api/coupon/project/coupons")
    Call<Result<List<CouponPromotionResultData>>> queryCoupons(@Body Request<List<String>> request);

    /**
     * POS中间件会员优惠券绑定接口的请求
     *
     * @param request 优惠券对象
     * @return Caller <用户已领用过的全部优惠券号集合>
     */
    @POST("api/coupon/bind")
    Call<Result<List<String>>> bindCoupon(@Body Request<CouponBindRequestData> request);

    /**
     * SCRM会员券核销的 retrofit 请求。
     *
     * @param request SCRM会员券核销的请求。
     * @return 检查SCRM会员券核销的 retrofit 调用。
     */
    @POST("crm/coupon/offlineConsumeCerti")
    Call<Result<ScrmCouponUseResultData>> certiConsume(@Body Request<ScrmCouponUseRequestData> request);

    /**
     * SCRM会员券核销的 retrofit 请求。
     *
     * @param request SCRM会员券核销的请求。
     * @return 检查SCRM会员券核销的 retrofit 调用。
     */
    @POST("crm/coupon/offlineCancelCerti")
    Call<Result<Object>> certiCancel(@Body Request<ScrmCouponCancelRequestData> request);
}
