/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.crm.webapi.coupon;

import com.hisense.crm.webapi.Context;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;

/**
 * SCRM券核销请求数据。
 *
 * @author xieliyan  2019-08-02
 */
public class ScrmCouponUseRequestData {
    private String orgCode;//门店编码
    private String saleNo;//水号
    private List<ScrmCouponUsePluData> plu;//交易商品信息数据集
    private List<ScrmCouponUsePayData> pay;//	交易支付信息数据集
    private String qNo;//券号
    private String vipCardNo;

    public String getOrgCode() {
        return orgCode;
    }

    public void setOrgCode(String orgCode) {
        this.orgCode = orgCode;
    }

    public String getSaleNo() {
        return saleNo;
    }

    public void setSaleNo(String saleNo) {
        this.saleNo = saleNo;
    }

    public List<ScrmCouponUsePluData> getPlu() {
        return plu;
    }

    public void setPlu(List<ScrmCouponUsePluData> plu) {
        this.plu = plu;
    }

    public List<ScrmCouponUsePayData> getPay() {
        return pay;
    }

    public void setPay(List<ScrmCouponUsePayData> pay) {
        this.pay = pay;
    }

    public String getqNo() {
        return qNo;
    }

    public void setqNo(String qNo) {
        this.qNo = qNo;
    }

    public String getVipCardNo() {
        return vipCardNo;
    }

    public void setVipCardNo(String vipCardNo) {
        this.vipCardNo = vipCardNo;
    }

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param qNo       待核销券号
     * @param vipCardNo 会员卡号
     * @return WebAPI 请求。
     */
    public static Request<ScrmCouponUseRequestData> request(String qNo, String vipCardNo) {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();
        ScrmCouponUseRequestData requestData = new ScrmCouponUseRequestData();
        requestData.setqNo(qNo);
        requestData.setOrgCode(Context.orgCode());
        requestData.setSaleNo(saleModel.getSaleNo());
        requestData.setVipCardNo(vipCardNo);
        requestData.plu = new ArrayList<>();
        requestData.pay = new ArrayList<>();
        for (BaseSalePluModel pluModel : saleModel.getPlus()) {
            ScrmCouponUsePluData pluData = new ScrmCouponUsePluData();
            pluData.setPluCode(pluModel.getPluCode());
            pluData.setPluId(pluModel.getPluId());
            pluData.setPluName(pluModel.getPluName());
            pluData.setQuantity(pluModel.getQuantity());
            pluData.setSerialNo(String.valueOf(pluModel.getItemNo()));
            pluData.setTotal(pluModel.getTotal());
            pluData.setTotalRecv(pluModel.getTotalRecv());
            requestData.plu.add(pluData);
        }
        requestData.setPlu(requestData.getPlu());
        for (SalePayModel payModel : saleModel.getPays()) {
            ScrmCouponUsePayData payData = new ScrmCouponUsePayData();
            payData.setPayCode(payModel.getPayCode());
            payData.setPayNo(payModel.getOutTradeNo());
            payData.setPayType(payModel.getPayType());
            payData.setSerialNo(String.valueOf(payModel.getSerialNo()));
            payData.setTotalRecv(payModel.getTotalRecv());
            requestData.pay.add(payData);
        }
        requestData.setPay(requestData.getPay());


        return WebApi.sign(requestData);
    }
}
