/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.points;

import com.hisense.crm.webapi.account.AccountPayRequestData;
import com.hisense.crm.webapi.account.AccountQueryRequestData;
import com.hisense.crm.webapi.account.AccountReturnRequestData;
import com.hisense.crm.webapi.trans.TransactionResultData;
import com.hisense.crm.webapi.trans.TransactionRevokeRequestData;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * 积分服务的 retrofit 接口。
 *
 * @author zhangsai    2017-10-10
 * @author zhangsai    2018-02-02 HIPOSAPP-315 增加积分查询
 * @author zhangsai    2018-02-02 HIPOSAPP-316 增加积分支付
 * @author zhangsai    2018-02-02 HIPOSAPP-317 增加积分支付撤销
 * @author zhangsai    2018-05-22 HIPOSAPP-456 增加积分原路返回预判
 * @author zhangsai    2018-05-22 HIPOSAPP-458 增加积分原路返回确认
 * @author zhangsai    2018-07-23 HIPOSAPP-686 增加积分退款
 * @author wangyaogang 2019-08-06 HIPOSAPP-1229 退货打印实时积分
 */
interface IPointsService {

    /**
     * 积分支付的 retrofit 请求。
     *
     * @param request 积分支付请求
     * @return 积分支付的 retrofit 调用
     */
    @POST("crm/points/pay")
    Call<Result<TransactionResultData>> pay(@Body Request<AccountPayRequestData> request);

    /**
     * 积分查询的 retrofit 请求。
     *
     * @param request 积分查询的请求。
     * @return 积分查询的 retrofit 调用。
     */
    @POST("crm/points/query")
    Call<Result<PointsQueryResultData>> query(@Body Request<AccountQueryRequestData> request);

    /**
     * 积分退款的 retrofit 请求。
     *
     * @param request 积分退款请求
     * @return 积分退款的 retrofit 调用
     */
    @POST("crm/points/return")
    Call<Result<TransactionResultData>> returns(@Body Request<AccountReturnRequestData> request);

    /**
     * 积分支付撤销的 retrofit 请求。
     *
     * @param request 积分支付撤销请求
     * @return 积分支付撤销的 retrofit 调用
     */
    @POST("crm/points/revoke")
    Call<Result<TransactionResultData>> revoke(@Body Request<TransactionRevokeRequestData> request);

    /**
     * 积分原路返回预判的 retrofit 请求。
     *
     * @param request 积分原路返回预判请求
     * @return 积分原路返回预判的 retrofit 调用
     */
    @POST("crm/points/regress/predict")
    Call<Result<RegressResultData>> regressPredict(@Body Request<RegressRequestData> request);

    /**
     * 积分原路返回确认的 retrofit 请求。
     *
     * @param request 积分原路返回确认请求
     * @return 积分原路返回确认的 retrofit 调用
     */
    @POST("crm/points/regress/confirm")
    Call<Result<RegressComfirmResultData>> regressConfirm(@Body Request<RegressConfirmRequestData> request);

    /**
     * 会员积分明细查询 retrofit 请求。
     *
     * @param request 积分明细查询
     * @return 会员积分明细 retrofit 调用
     */
    @POST("crm/points/details")
    Call<Result<PointDetailsResultData>> queryVipPointDetails(@Body Request<PointDetailsRequest> request);

    /**
     * 会员累计积分查询 retrofit 请求。
     *
     * @param request 累计积分查询
     * @return 会员积分明细 retrofit 调用
     */
    @POST("crm/points/accumulate")
    Call<Result<PointAccumulateResultData>> queryVipAccumulatedPoints(@Body Request<PointAccumulateRequestData> request);
}
