/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.points;

import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.CardPayResult;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.RealTimePoint;
import com.hisense.crm.webapi.account.AccountPayRequestData;
import com.hisense.crm.webapi.account.AccountQueryRequestData;
import com.hisense.crm.webapi.account.AccountReturnRequestData;
import com.hisense.crm.webapi.trans.TransactionResultData;
import com.hisense.crm.webapi.trans.TransactionRevokeRequestData;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.concurrent.TimeUnit;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 积分服务。
 *
 * @author zhangsai     2017-10-10
 * @author zhangsai     2018-02-02 HIPOSAPP-315 增加积分查询
 * @author zhangsai     2018-02-02 HIPOSAPP-316 增加积分支付
 * @author zhangsai     2018-02-02 HIPOSAPP-317 增加积分支付撤销
 * @author zhangsai     2018-05-22 HIPOSAPP-456 增加积分原路返回预判
 * @author zhangsai     2018-02-02 HIPOSAPP-631 积分查询产生的异常直接通过onError返回调用方
 * @author zhangsai     2018-05-22 HIPOSAPP-458 增加积分原路返回确认
 * @author zhangsai     2018-07-23 HIPOSAPP-686 增加积分退款
 * @author guolina      2018-09-26 HIPOSAPP-770 支持销售/退款后增加储值卡和积分余额打印
 * @author zhangsai     2018-12-21 HIPOSAPP-699 交易流水改为 ISale 类型
 * @author guolina      2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author wangyaogang  2019-06-05 HIPOSAPP-1151 积分支付,撤销和退款时超时时间设置为2分钟
 * @author wangyaogang  2019-06-10 HIPOSAPP-1135 积分 查询/支付/撤销/退款接口增加isVerifyPwd字段
 * @author wangyaogang  2019-06-19 HIPOSAPP-1171 积分卡查询,支付,撤销，退款增加校验码cardVerifyCode字段
 * @author wangyaogang  2019-07-02 HIPOSAPP-1180 积分卡校验参数封装为CardSecurity
 * @author wangyaogang 2019-08-06 HIPOSAPP-1229 退货打印实时积分
 */
public final class PointsService {

    private PointsService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 支付。
     *
     * @param cardNo       支付卡号
     * @param cardSecurity 校验方式
     * @param payModel     支付信息
     * @param isVerifyPwd  是否校验密码
     * @param callback     积分支付的回调方法
     */
    public static void pay(CardNo cardNo, CardSecurity cardSecurity, SalePayModel payModel,
                           boolean isVerifyPwd, final com.hisense.webapi.Callback<CardPayResult> callback) {
        final Request<AccountPayRequestData> request = AccountPayRequestData.request(cardNo, cardSecurity, payModel, isVerifyPwd);
        IPointsService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPointsService.class);
        WebApi.requestAsync(service.pay(request), new Callback<Result<TransactionResultData>>() {
            @Override
            public void onResponse(Call<Result<TransactionResultData>> call, Response<Result<TransactionResultData>> response) {
                CardPayResult cardPayResult = new CardPayResult(
                        response.body().data.refNo,
                        response.body().data.tranTime,
                        response.body().data.balance,
                        request.data.outTradeNo);
                callback.onSuccess(cardPayResult);
            }

            @Override
            public void onFailure(Call<Result<TransactionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },TransactionResultData.class);
    }

    /**
     * 查询积分信息。
     *
     * @param cardNo       会员卡号
     * @param cardSecurity 校验方式
     * @param isVerifyPwd  是否校验密码
     * @param callback     积分查询的回调方法
     */
    public static void query(CardNo cardNo, CardSecurity cardSecurity, boolean isVerifyPwd,
                             final com.hisense.webapi.Callback<VipCard> callback) {
        Request<AccountQueryRequestData> request = AccountQueryRequestData.request(cardNo, cardSecurity, null, isVerifyPwd);
        IPointsService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPointsService.class);
        WebApi.requestAsync(service.query(request), new Callback<Result<PointsQueryResultData>>() {
            @Override
            public void onResponse(Call<Result<PointsQueryResultData>> call, Response<Result<PointsQueryResultData>> response) {
                if (response.body() == null || response.body().data == null) {
                    callback.onError(new Throwable("没有积分卡记录"));
                }

                callback.onSuccess(PointsQueryResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<PointsQueryResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },PointsQueryResultData.class);
    }

    /**
     * 支付撤销。
     *
     * @param isVerifyPwd 是否校验密码
     * @param callback    积分支付的回调方法
     */
    public static void revoke(SalePayModel payModel, boolean isVerifyPwd, final com.hisense.webapi.Callback<String> callback) {
        if (payModel.getCardNo() == null || (payModel.getCardNo().getType() != CardNoType.VipCode && payModel.getCardNo().getType() != CardNoType.PayCode)) {
            callback.onError(new Exception("卡号类型错误"));
            return;
        }

        Request<TransactionRevokeRequestData> request = TransactionRevokeRequestData.request(payModel, isVerifyPwd);
        IPointsService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPointsService.class);
        WebApi.requestAsync(service.revoke(request), new Callback<Result<TransactionResultData>>() {
            @Override
            public void onResponse(Call<Result<TransactionResultData>> call, Response<Result<TransactionResultData>> response) {
                callback.onSuccess(response.body().data.refNo);
            }

            @Override
            public void onFailure(Call<Result<TransactionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },TransactionResultData.class);
    }


    /**
     * 积分原路返回预判。
     *
     * @param oldSaleNo 原交易流水号
     * @param sale      退货流水
     * @param callback  积分原路返回预判的回调方法
     */
    public static void regressPredict(String oldSaleNo, BaseSaleModel sale,
                                      final com.hisense.webapi.Callback<String> callback) {
        Request<RegressRequestData> request = RegressRequestData.request(oldSaleNo, sale);
        IPointsService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPointsService.class);
        WebApi.requestAsync(service.regressPredict(request), new Callback<Result<RegressResultData>>() {
            @Override
            public void onResponse(Call<Result<RegressResultData>> call, Response<Result<RegressResultData>> response) {
                callback.onSuccess(response.body().data.confirmId);
            }

            @Override
            public void onFailure(Call<Result<RegressResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },RegressResultData.class);
    }

    /**
     * 积分原路返回确认。
     *
     * @param confirmId 积分原路返回确认的唯一标识
     * @param saleNo    退货流水号
     * @param vipCardNo 会员卡号
     * @param callback  积分原路返回确认的回调方法
     */
    public static void regressConfrim(String confirmId, String saleNo, String vipCardNo,
                                      final com.hisense.webapi.Callback<RealTimePoint> callback) {
        Request<RegressConfirmRequestData> request = RegressConfirmRequestData
                .request(confirmId, saleNo, vipCardNo);
        IPointsService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPointsService.class);
        WebApi.requestAsync(service.regressConfirm(request), new Callback<Result<RegressComfirmResultData>>() {
            @Override
            public void onResponse(Call<Result<RegressComfirmResultData>> call, Response<Result<RegressComfirmResultData>> response) {
                RealTimePoint realTimePoint = new RealTimePoint(
                        response.body().data.coupon,
                        response.body().data.expense,
                        response.body().data.parking);
                callback.onSuccess(realTimePoint);
            }

            @Override
            public void onFailure(Call<Result<RegressComfirmResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },RegressComfirmResultData.class);
    }

    /**
     * 退款。
     *
     * @param cardNo       支付卡号
     * @param cardSecurity 校验方式
     * @param oldSaleNo    原交易流水号
     * @param payment      支付信息
     * @param isVerifyPwd  是否校验密码
     * @param callback     储值卡支付的回调方法
     */
    public static void returns(CardNo cardNo, CardSecurity cardSecurity, String oldSaleNo, SalePayModel payment,
                               String outTradeNo, boolean isVerifyPwd, final com.hisense.webapi.Callback<CardPayResult> callback) {
        if (cardNo == null || (cardNo.getType() != CardNoType.VipCode && cardNo.getType() != CardNoType.PayCode)) {
            callback.onError(new Exception("卡号类型错误"));
            return;
        }

        Request<AccountReturnRequestData> request = AccountReturnRequestData.request(cardNo, cardSecurity, oldSaleNo, payment, outTradeNo, isVerifyPwd);
        IPointsService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IPointsService.class);
        WebApi.requestAsync(service.returns(request), new Callback<Result<TransactionResultData>>() {
            @Override
            public void onResponse(Call<Result<TransactionResultData>> call, Response<Result<TransactionResultData>> response) {
                CardPayResult cardPayResult = new CardPayResult(
                        response.body().data.refNo,
                        response.body().data.tranTime,
                        response.body().data.balance);
                callback.onSuccess(cardPayResult);
            }

            @Override
            public void onFailure(Call<Result<TransactionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },TransactionResultData.class);
    }

    /**
     * 会员积分明细查询。
     *
     * @param cardNo   会员卡信息
     * @param pageNo   页码
     * @param pageRows 每页条数
     * @param year     查询年份
     * @param month    查询月份
     */
    public static PointDetailsResultData queryPointDetails(CardNo cardNo, String pageNo, String pageRows, String year, String month) throws WebApiException {
        Request<PointDetailsRequest> request = PointDetailsRequest.request(cardNo, pageNo, pageRows, year, month);

        PointDetailsResultData resultData = WebApi.request(WebApi.create(IPointsService.class).queryVipPointDetails(request));

        return resultData;
    }

    /**
     * 查询会员累计积分。
     *
     * @param cardNo 支付卡号
     */
    public static PointAccumulateResultData queryAccumulatedPoints(CardNo cardNo) throws WebApiException {
        Request<PointAccumulateRequestData> request = PointAccumulateRequestData.request(cardNo);
        PointAccumulateResultData result = WebApi.request(WebApi.create(IPointsService.class).queryVipAccumulatedPoints(request));
        return result;
    }
}
