package com.hisense.crm.webapi.points;

import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 积分原路返回确认的请求数据。
 *
 * @author zhangsai 2018-05-22 HIPOSAPP-458
 */
final class RegressConfirmRequestData {

    /**
     * 积分原路返回确认的唯一标识。
     */
    public String confirmId;

    /**
     * 组织编码。
     */
    public String orgCode;

    /**
     * 退货流水号。
     */
    public String saleNo;

    /**
     * 会员卡号。
     */
    public String vipCardNo;

    /**
     * CRM类型
     */
    public String crmType;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param confirmId 积分原路返回确认的唯一标识
     * @param saleNo    退货流水号
     * @param vipCardNo 会员卡号
     * @return WebAPI 请求。
     */
    public static Request<RegressConfirmRequestData> request(String confirmId, String saleNo, String vipCardNo) {
        RegressConfirmRequestData requestData = new RegressConfirmRequestData();
        requestData.confirmId = RSAService.encrypt(confirmId);
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.saleNo = RSAService.encrypt(saleNo);
        requestData.vipCardNo = RSAService.encrypt(vipCardNo);
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
