package com.hisense.crm.webapi.points;

import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;

/**
 * 积分原路返回预判的请求数据。
 *
 * @author zhangsai    2018-05-22
 * @author zhangsai    2018-12-19 HIPOSAPP-698 交易商品改为 ISaleItem 类型
 * @author zhangsai    2018-12-21 HIPOSAPP-699 交易流水改为 ISale 类型
 * @author wangyaogang 2020-03-24 HIPOSAPP-1393 积分原路返回预判接口调整
 */
final class RegressRequestData {
    /**
     * 原交易流水号。
     */
    public String oldSaleNo;

    /**
     * 组织编码。
     */
    public String orgCode;

    /**
     * 退货商品列表。
     */
    public ArrayList<RegressPlu> plu;

    /**
     * 退货流水号。
     */
    public String saleNo;

    /**
     * 会员卡号。
     */
    public String vipCardNo;

    /**
     * CRM类型
     */
    public String crmType;
    /**
     * 交易类型
     */
    public String tranType;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param oldSaleNo 原交易流水号
     * @param sale      退货流水
     * @return WebAPI 请求。
     */
    public static Request<RegressRequestData> request(String oldSaleNo, BaseSaleModel sale) {
        RegressRequestData requestData = new RegressRequestData();
        requestData.oldSaleNo = RSAService.encrypt(oldSaleNo);
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.saleNo = RSAService.encrypt(sale.getSaleNo());
        requestData.vipCardNo = RSAService.encrypt(sale.getVip().getCard().vipCardNo());
        requestData.tranType = String.valueOf(sale.getTranType().value());

        requestData.plu = new ArrayList<>();

        for (BaseSalePluModel item : sale.getPlus()) {
            requestData.plu.add(new RegressPlu(
                    item.getDepCode(),
                    item.getDepName(),
                    item.getItemNo(),
                    item.getPageNo(),
                    RSAService.encrypt(item.getPluCode()),
                    item.getPluName(),
                    String.valueOf(item.getPrice()),
                    String.valueOf(item.getQuantity()),
                    String.valueOf(item.getTagPrice()),
                    RSAService.encrypt(item.getTotal()),
                    RSAService.encrypt(item.getTotalDsc()),
                    RSAService.encrypt(item.getTotalRecv()),
                    item.getBrandCode(),
                    item.getClassCode()));
        }
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
