/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.crm.webapi.trans;

import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 交易撤销的请求数据。
 *
 * @author zhangsai    2018-02-01 HIPOSAPP-314
 * @author guolina     2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 撤销接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 撤销接口增加校验码cardVerifyCode字段
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity
 */
public class TransactionRevokeRequestData {
    /**
     * 收银员编码
     */
    public String cshCode;
    /**
     * 收银员名称
     */
    public String cshName;
    /**
     * 是否校验密码	0 - 不校验 1 - 校验 非必传，默认为0 仅cardNoType=7时生效
     */
    public String isVerifyPwd;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 支付密码
     */
    public String password;
    /**
     * 卡面后几位
     */
    public String cardFaceNoCheck;
    /**
     * 支付校验码
     */
    public String cardVerifyCode;
    /**
     * 支付方式编码
     */
    public String payCode;
    /**
     * 支付方式名称
     */
    public String payName;
    /**
     * 支付时间
     */
    public String payTime;
    /**
     * 收款机号
     */
    public String posNo;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 顺序号
     */
    public int serialNo;
    /**
     * 支付金额
     */
    public String totalPay;
    /**
     * 会员号
     */
    public String vipCardNo;
    /**
     * 卡号类型
     */
    public String cardNoType;

    /**
     * 业务单据号（SCRM必传储值卡支付时返回的交易参考号）
     */
    public String refNo;
    /**
     * 商户订单号（SCRM必传储值卡支付时返回的商户订单号）
     */
    public String outTradeNo;

    /**
     * 支付方式标记（SCRM必传）
     */
    public String payMethod;
    /**
     * CRM类型
     */
    public String crmType;
    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param isVerifyPwd 是否校验密码
     * @return WebAPI 请求
     */
    public static Request<TransactionRevokeRequestData> request(SalePayModel payModel, boolean isVerifyPwd) {
        TransactionRevokeRequestData requestData = new TransactionRevokeRequestData();

        CardSecurity cardSecurity = payModel.getCardSecurity();
        CardNo cardNo = payModel.getCardNo();
        requestData.cshCode = Context.cashier().getCode();
        requestData.cshName = Context.cashier().getName();
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.password = RSAService.encrypt((cardSecurity == null) ? "" : cardSecurity.password());
        requestData.cardFaceNoCheck = (cardSecurity == null) ? "" : cardSecurity.cardFaceNoCheck();
        requestData.cardVerifyCode = (cardSecurity == null) ? "" : cardSecurity.cardVerifyCode();
        requestData.isVerifyPwd = isVerifyPwd ? "1" : "0";
        requestData.payCode = RSAService.encrypt(payModel.getPayCode());
        requestData.payName = payModel.getPayName();
        requestData.payTime = payModel.getPayTime();
        requestData.posNo = RSAService.encrypt(Context.posNo());
        requestData.saleNo = RSAService.encrypt(payModel.getSaleNoTemp());
        requestData.serialNo = payModel.getSerialNo();
        requestData.totalPay = RSAService.encrypt(String.valueOf(payModel.getTotalPaid()));
        requestData.vipCardNo = RSAService.encrypt(cardNo.value());
        requestData.cardNoType = String.valueOf(cardNo.getTypeCode());
        requestData.refNo = payModel.getTradeNo();
        requestData.outTradeNo = payModel.getOutTradeNo();
        requestData.payMethod = payModel.getRemark();
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
