/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.vip;

import com.hisense.crm.webapi.account.AccountQueryRequestData;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import java.util.List;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * 会员服务的 retrofit 接口。
 *
 * @author zhangsai    2017-10-09
 * @author zhangsai    2018-04-04 HIPOSAPP-381 新增会员注册接口
 * @author guolina     2018-05-21 HIPOSAPP-446 新增实时积分接口
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 增加会员验证方式查询接口
 */
interface IVipService {
    /**
     * 会员查询的 retrofit 请求。
     *
     * @param request 会员查询请求。
     * @return 会员查询的 retrofit 调用。
     */
    @POST("crm/vip/query")
    Call<Result<VipQueryResultData>> query(@Body Request<AccountQueryRequestData> request);

    /**
     * 获取 IC 卡读卡密码的 retrofit 请求。
     *
     * @param request 获取 IC 卡读卡密码的请求。
     * @return 获取 IC 卡读卡密码的 retrofit 调用。
     */
    @POST("crm/vip/icCardPassword")
    Call<Result<IcCardPasswordResultData>> getIcCardPassword(@Body Request<IcCardPasswordRequestData> request);

    /**
     * 会员注册的 retrofit 请求。
     *
     * @param request 会员注册请求。
     * @return 会员注册的 retrofit 调用。
     */
    @POST("crm/vip/register")
    Call<Result<RegisterResultData>> register(@Body Request<RegisterRequestData> request);

    /**
     * 实时积分的 retrofit 请求。
     *
     * @param request 实时积分请求。
     * @return 实时积分的 retrofit 调用。
     */
    @POST("crm/points/reward/realtime")
    Call<Result<RealTimePointResultData>> realTimePoint(@Body Request<RealTimePointRequestData> request);

    /**
     * 更新会员信息
     *
     * @return 用户信息
     */
    @POST("crm/vip/updateInfo")
    Call<Result<Object>> updateVipInfo(@Body Request<VipUpdateInfoData> request);

    /**
     * 查询会员升降级方案
     *
     * @return 用户信息
     */
    @POST("crm/vip/levelProject")
    Call<Result<List<VipUpgradeResultData>>> queryVipUpgradeProject(@Body Request<VipUpgradeRequestData> request);

    /**
     * 查询会员卡验证方式
     *
     * @param request 验证方式请求
     * @return 验证方式
     */
    @POST("crm/vip/verifyType")
    Call<Result<VerifyTypeResultData>> queryVerifyType(@Body Request<VerifyTypeRequestData> request);
}
