/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.vip;

import com.hisense.crm.Cashier;
import com.hisense.crm.card.IcCardType;
import com.hisense.crm.card.IcCardVersion;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;


/**
 * 获取 IC 卡读卡密码的请求数据。
 *
 * @author zhangsai 2017-10-10
 */
final class IcCardPasswordRequestData {
    /**IC 卡类型*/
    public int cardType;
    /**收款员编码*/
    public String cshCode;
    /**收款员姓名*/
    public String cshName;
    /**IC 卡数据格式*/
    public int dataFormat;
    /**Mifare1 卡第一区块的读取数据*/
    public String icNo;
    /**组织编码*/
    public String orgCode;
    /**收款机号*/
    public String posNo;


    /**
     * 返回一个新的 WebAPI 请求。
     * @param cardType      IC 卡类型
     * @param dataFormat    数据格式
     * @param icNo          Mifare1 卡第一区块的读取数据
     * @param cashier       收银员
     * @return WebAPI 请求。
     */
    public static Request<IcCardPasswordRequestData> request(IcCardType cardType, IcCardVersion dataFormat,
                                                             String icNo, Cashier cashier) {
        IcCardPasswordRequestData requestData = new IcCardPasswordRequestData();
        requestData.cardType = cardType.value();
        requestData.cshCode = cashier != null ? cashier.getCode() : "";
        requestData.cshName = cashier != null ? cashier.getName() : "";
        requestData.dataFormat = dataFormat.value();
        requestData.icNo = icNo != null ? icNo : "";
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.posNo = RSAService.encrypt(Context.posNo());

        return WebApi.sign(requestData);
    }
}
