/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.crm.webapi.vip;

import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;

/**
 * 实时积分的请求数据。
 *
 * @author guolina     2018-05-21 HIPOSAPP-446  新增实时积分
 * @author zhangsai    2018-12-19 HIPOSAPP-698  交易商品类型改为 ISaleItem
 * @author zhangsai    2019-06-13 HIPOSAPP-1169 Sale.payments() 返回类型改为 ISalePayment 集合
 * @author wangyaogang 2019-08-22 HIPOSAPP-1286 实时积分计算接口增加cardFaceNo
 */
final class RealTimePointRequestData {

    /**
     * 组织编码 使用RSA加密
     */
    public String orgCode;

    /**
     * 组织名称
     */
    public String orgName;

    /**
     * 支付方式 Array
     */
    public ArrayList<RealTimePointPaymentData> pay;

    /**
     * 交易商品Array
     */
    public ArrayList<RealTimePointPluData> plu;

    /**
     * 收款机号 使用RSA加密
     */
    public String posNo;

    /**
     * 流水号 使用RSA加密
     */
    public String saleNo;

    /**
     * 交易时间
     */
    public String saleTime;

    /**
     * 交易类型
     */
    public int tranType;

    /**
     * 会员卡号
     */
    public String vipCardNo;

    /**
     * 卡面号
     */
    public String cardFaceNo;

    /**
     * CRM类型
     */
    public String crmType;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @return WebAPI 请求。
     */
    public static Request<RealTimePointRequestData> request() {
        MainSaleModel mainSaleModel = MainSaleModel.getInstance();

        SaleModel saleModel = mainSaleModel.getSaleModel();

        RealTimePointRequestData requestData = new RealTimePointRequestData();
        requestData.orgCode = RSAService.encrypt(Context.orgCode());//RSAService.encrypt(mainSaleModel.getShopInfo().getOrgCode());
        requestData.orgName = mainSaleModel.getShopInfo().getOrgName();

        requestData.pay = new ArrayList<>();
        for (SalePayModel payModel : saleModel.getPays()) {
            RealTimePointPaymentData pay = new RealTimePointPaymentData();
            pay.payCode = RSAService.encrypt(payModel.getPayCode());
            pay.total = RSAService.encrypt(String.valueOf(payModel.getTotalRecv()));

            requestData.pay.add(pay);
        }

        requestData.plu = new ArrayList<>();
        for (BaseSalePluModel pluModel : saleModel.getPlus()) {
            RealTimePointPluData pluData = new RealTimePointPluData();
            pluData.brandCode = pluModel.getBrandCode(); //品牌编码
            pluData.classCode = pluModel.getClassCode(); //品类编码
            pluData.depCode = pluModel.getDepCode();
            pluData.depName = pluModel.getDepName();
            pluData.itemNo = pluModel.getItemNo();
            pluData.pageNo = pluModel.getPageNo();
            pluData.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluData.pluName = pluModel.getPluName();
            pluData.price = pluModel.getPrice(); // 发生价格
            pluData.quantity = pluModel.getQuantity(); // 数量
            pluData.tagPrice = pluModel.getTagPrice(); // 商品原价

            pluData.total = RSAService.encrypt(pluModel.getTotal());
            pluData.totalDsc = RSAService.encrypt(pluModel.getTotalDsc());
            pluData.totalRecv = RSAService.encrypt(pluModel.getTotalRecv());

            requestData.plu.add(pluData);
        }

        requestData.posNo = RSAService.encrypt(saleModel.getPosNo());
        requestData.saleNo = RSAService.encrypt(saleModel.getSaleNo());
        requestData.saleTime = saleModel.getSaleTime();
        requestData.tranType = saleModel.getTranType().value();
        requestData.vipCardNo = RSAService.encrypt(saleModel.getVip().card().vipCardNo());
        requestData.cardFaceNo = saleModel.getVip().card().cardFaceNo();
        if (WebApi.isAmwAndAip()) {
            requestData.crmType = "CRM6";
        }
        return WebApi.sign(requestData);
    }
}
