/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.crm.webapi.vip;

import com.hisense.crm.vip.Vip;
import com.hisense.crm.webapi.Context;
import com.hisense.crm.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 会员注册的请求数据。
 *
 * @author zhangsai 2018-04-04 HIPOSAPP-381
 * @author xieliyan 2019-07-29 HIPOSAPP-1227 对接Scrm，增加注册人工号和收款员编码两个参数
 */
final class RegisterRequestData {
    /**
     * 外部订单号
     */
    public String billNo;
    /**
     * 生日。格式YYYY-MM-DD
     */
    public String birthday;
    /**
     * 身份证号
     */
    public String certNo;
    /**
     * 手机号
     */
    public String mobile;
    /**
     * 注册组织编码
     */
    public String orgCode;
    /**
     * 性别。0-男；1-女
     */
    public int sex;
    /**
     * 姓名
     */
    public String vipName;
    /**
     * 收款员编码
     */
    public String cshCode;
    /**
     * 注册人工号
     */
    public String registerNo;


    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param billNo 外部单据号
     * @param vip    会员
     * @param certNo 身份证号
     * @return WebAPI 请求。
     */
    public static Request<RegisterRequestData> request(String billNo, Vip vip, String certNo) {
        RegisterRequestData requestData = new RegisterRequestData();
        requestData.billNo = billNo;
        requestData.birthday = vip.birthday();
        requestData.certNo = RSAService.encrypt(certNo);
        requestData.mobile = vip.mobile();
        requestData.sex = vip.sex().value();
        requestData.orgCode = RSAService.encrypt(Context.orgCode());
        requestData.vipName = vip.name();
        requestData.cshCode= Context.cashier().getCode();
        requestData.registerNo=requestData.cshCode;

        return WebApi.sign(requestData);
    }
}
