/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.vip;

import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.vip.VipPointsProject;
import com.hisense.crm.vip.VipSex;
import com.hisense.crm.vip.VipStatus;

import java.util.List;

/**
 * 会员查询的返回结果。
 *
 * @author zhangsai 2017-10-09
 * @author zhangsai 2017-12-27 HIPOSAPP-258 增加处理会员查询返回结果的方法
 * @author guolina  2018-06-13 HIPOSAPP-396 添加退货时积分原路退回
 */
final class VipQueryResultData {
    /**
     * 卡面号
     */
    public String cardFaceNo;
    /**
     * 卡号
     */
    public String cardNo;
    /**
     * 卡状态
     */
    public String cardStatus;
    /**
     * 卡类型编码
     */
    public String cardType;
    /**
     * 卡类型名称
     */
    public String cardTypeName;
    /**
     * 卡片有效期
     */
    public String expireDate;
    /**
     * 会员积分方案
     */
    public List<VipPointsProjectData> projects;
    /**
     * 会员信息
     */
    public VipData vip;

    public VipCardData card;


    /**
     * 处理 WebAPI 接口返回数据。
     *
     * @param resultData 接口返回数据
     * @return 查询到的会员信息
     */
    public static Vip handleResult(VipQueryResultData resultData) {
        if (resultData != null) {

            VipCard card = new VipCard.Builder(resultData.cardFaceNo, null)
                    .cardType(resultData.cardType, resultData.cardTypeName)
                    .expireDate(resultData.expireDate)
                    .status(resultData.cardStatus)
                    .vipCardNo(resultData.cardNo)
                    .usable(Double.valueOf(resultData.card.sumPoints),Double.valueOf(resultData.card.sumPoints))
                    .build();

            Vip vip = new Vip.Builder(resultData.vip.name)
                    .birthday(resultData.vip.birthday)
                    .mobile(resultData.vip.mobile)
                    .message(resultData.vip.careMsg, resultData.vip.hintMsg)
                    .sex(VipSex.valueOf(resultData.vip.sex))
                    .status(VipStatus.valueOf(resultData.vip.status))
                    .card(card)
                    .build();

            for (VipPointsProjectData project : resultData.projects) {
                vip.addProject(new VipPointsProject.Builder(project.code, project.name)
                        .usable(Double.valueOf(project.usablePoints == null ? "0" : project.usablePoints), Double.valueOf(project.usableBalance == null ? "0" : project.usableBalance))
                        .build());
            }

            return vip;
        }

        return null;
    }
}
