/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.webapi.vip;

import com.hisense.crm.Cashier;
import com.hisense.crm.card.CardNo;
import com.hisense.crm.card.CardSecurity;
import com.hisense.crm.card.IcCardType;
import com.hisense.crm.card.IcCardVersion;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.RealTimePoint;
import com.hisense.crm.vip.VerifyType;
import com.hisense.crm.vip.Vip;
import com.hisense.crm.webapi.RSAService;
import com.hisense.crm.webapi.account.AccountQueryRequestData;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.List;
import java.util.concurrent.TimeUnit;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 会员服务。
 *
 * @author zhangsai    2017-10-09
 * @author zhangsai    2017-12-27 HIPOSAPP-258 增加会员异步查询方法
 * @author zhangsai    2018-01-31 HIPOSAPP-311
 * @author zhangsai    2018-04-04 HIPOSAPP-381 新增会员注册
 * @author guolina     2018-05-21 HIPOSAPP-446 新增实时积分接口
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 查询接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-10 HIPOSAPP-1135 查询接口增加isVerifyPwd字段
 * @author wangyaogang 2019-06-19 HIPOSAPP-1171 查询接口增加校验码cardVerifyCode字段,
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 增加会员验证方式查询接口,校验参数封装为CardSecurity
 * @author xieliyan    2019-08-05 HIPOSAPP-1277 对接SCRM-获取会员卡验证方式:请求参数增加payMethod
 */
public class VipService {

    private VipService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 使用卡号/密码查询会员。
     *
     * @param cardNo   会员卡号
     * @param security 校验方式
     * @param callback 会员查询的回调方法
     */
    public static void vipQuery(CardNo cardNo, CardSecurity security, String saleNo,
                                final com.hisense.webapi.Callback<Vip> callback) {
        Request<AccountQueryRequestData> request = AccountQueryRequestData.request(cardNo, security, saleNo, false);

        WebApi.requestAsync(WebApi.create(IVipService.class).query(request), new Callback<Result<VipQueryResultData>>() {
            @Override
            public void onResponse(Call<Result<VipQueryResultData>> call, Response<Result<VipQueryResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("会员查询返回信息为空"));
                    return;
                }

                callback.onSuccess(VipQueryResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<VipQueryResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },VipQueryResultData.class);
    }

    /**
     * 获取指定类型 IC 卡的读卡密码。
     *
     * @param cardType   IC 卡类型
     * @param dataFormat 数据格式
     * @param cashier    收银员
     * @param callback   获取读卡密码的回调方法
     */
    public static void getIcCardPassword(IcCardType cardType, IcCardVersion dataFormat,
                                         Cashier cashier,
                                         final com.hisense.webapi.Callback<String> callback) {
        getIcCardPassword(cardType, dataFormat, null, cashier, callback);
    }

    /**
     * 获取 Mifare1 2.0 卡实际存储扇区的读卡密码。
     *
     * @param icNo     Mifare1 卡第一区块的读取数据
     * @param cashier  收银员
     * @param callback 获取读卡密码的回调方法
     */
    public static void getMifare1CardPassword(String icNo, Cashier cashier,
                                              final com.hisense.webapi.Callback<String> callback) {
        getIcCardPassword(IcCardType.Mifare1, IcCardVersion.Version2, icNo, cashier, callback);
    }

    /**
     * 获取指定类型 IC 卡的读卡密码。
     *
     * @param cardType   IC 卡类型
     * @param dataFormat 数据格式
     * @param icNo       Mifare1 卡第一区块的读取数据
     * @param cashier    收银员
     * @param callback   获取读卡密码的回调方法
     */
    private static void getIcCardPassword(IcCardType cardType, IcCardVersion dataFormat,
                                          String icNo, Cashier cashier,
                                          final com.hisense.webapi.Callback<String> callback) {
        Request<IcCardPasswordRequestData> request = IcCardPasswordRequestData
                .request(cardType, dataFormat, icNo, cashier);

        WebApi.requestAsync(WebApi.create(IVipService.class).getIcCardPassword(request), new Callback<Result<IcCardPasswordResultData>>() {
            @Override
            public void onResponse(Call<Result<IcCardPasswordResultData>> call, Response<Result<IcCardPasswordResultData>> response) {
                callback.onSuccess(RSAService.decrypt(response.body().data.password));
            }

            @Override
            public void onFailure(Call<Result<IcCardPasswordResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },IcCardPasswordResultData.class);
    }

    /**
     * 注册会员。
     *
     * @param billNo   外部单据号
     * @param vip      会员
     * @param certNo   身份证号
     * @param callback 注册会员的回调方法
     */
    public static void register(final String billNo, final Vip vip, final String certNo,
                                final com.hisense.webapi.Callback<VipCard> callback) {
        Request<RegisterRequestData> request = RegisterRequestData.request(billNo, vip, certNo);
        WebApi.requestAsync(WebApi.create(IVipService.class).register(request), new Callback<Result<RegisterResultData>>() {
            @Override
            public void onResponse(Call<Result<RegisterResultData>> call, Response<Result<RegisterResultData>> response) {
                callback.onSuccess(RegisterResultData.handleResult(vip, response.body().data));
            }

            @Override
            public void onFailure(Call<Result<RegisterResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },RegisterResultData.class);
    }

    /**
     * 实时积分。
     *
     * @param callback 实时积分的回调方法
     */
    public static void realTimePoint(final com.hisense.webapi.Callback<RealTimePoint> callback) {
        Request<RealTimePointRequestData> request = RealTimePointRequestData.request();
        WebApi.requestAsync(WebApi.create(IVipService.class).realTimePoint(request), new Callback<Result<RealTimePointResultData>>() {
            @Override
            public void onResponse(Call<Result<RealTimePointResultData>> call, Response<Result<RealTimePointResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("积分实时积分计算返回参数为空"));
                    return;
                }

                RealTimePoint realTimePoint = new RealTimePoint(
                        response.body().data.coupon,
                        response.body().data.expense,
                        response.body().data.parking);
                callback.onSuccess(realTimePoint);
            }

            @Override
            public void onFailure(Call<Result<RealTimePointResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },RealTimePointResultData.class);
    }

    /**
     * 更新会员信息。
     *
     * @param cardNo
     * @param vip
     */
    public static void updateVipInfo(CardNo cardNo, Vip vip) throws WebApiException {
        Request<VipUpdateInfoData> request = VipUpdateInfoData.request(cardNo, vip);
        WebApi.request(WebApi.create(IVipService.class).updateVipInfo(request));
    }

    /**
     * 查询会员升降级方案。
     *
     * @param orgCode
     */
    public static List<VipUpgradeResultData> queryVipUpgradeProject(String orgCode) throws WebApiException {
        Request<VipUpgradeRequestData> request = VipUpgradeRequestData.request(orgCode);
        List<VipUpgradeResultData> resultData = WebApi.request(WebApi.create(IVipService.class).queryVipUpgradeProject(request));

        return resultData;
    }

    /**
     * 查询会员卡验证方式(付款码不支持)
     *
     * @param cardNo   会员号
     * @param callback 结果回调
     */
    public static void queryVerifyType(CardNo cardNo, String payMethod, final com.hisense.webapi.Callback<VerifyType> callback) {
        Request<VerifyTypeRequestData> request = VerifyTypeRequestData.request(cardNo, payMethod);
        IVipService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(IVipService.class);
        WebApi.requestAsync(service.queryVerifyType(request), new Callback<Result<VerifyTypeResultData>>() {
            @Override
            public void onResponse(Call<Result<VerifyTypeResultData>> call, Response<Result<VerifyTypeResultData>> response) {
                callback.onSuccess(VerifyTypeResultData.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<VerifyTypeResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },VerifyTypeResultData.class);
    }
}
