/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

import java.io.Serializable;

/**
 * 储值卡。
 *
 * @author zhangsai    2017-10-10
 * @author zhangsai    2017-12-27 增加卡片状态
 * @author wangyaogang 2019-10-08 HIPOSAPP-1305 储值卡消费可限制品牌品类
 */
public class Card implements Serializable {
    // 储值卡卡面号
    private String cardFaceNo;
    // 储值卡卡号
    private String cardNo;
    // 卡片状态
    private CardStatus status;
    // 储值卡类型
    private CardType cardType;
    // 储值卡有效期
    private String expireDate;
    // 储值卡可用余额
    private double usableTotal;
    // 会员卡号
    private String vipCardNo;
    /**
     * 使用范围控制,
     * 0 – 不控制
     * 1 – 提货
     * 2 – 限制商品使用范围
     */
    private String useLimit;

    private Card() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回是否控制适用范围
     *
     * @return true 是
     */
    public boolean useLimit() {
        return "2".equals(this.useLimit);
    }

    /**
     * 返回控制数值
     *
     * @return 控制数值
     */
    public String useLimitValue() {
        return this.useLimit;
    }

    /**
     * 返回储值卡卡面号。
     *
     * @return 储值卡卡面号。
     */
    public String cardFaceNo() {
        return this.cardFaceNo;
    }

    /**
     * 返回储值卡卡号。
     *
     * @return 储值卡卡号。
     */
    public String cardNo() {
        return this.cardNo;
    }

    /**
     * 返回储值卡类型。
     *
     * @return 储值卡类型。
     */
    public CardType cardType() {
        return this.cardType;
    }

    /**
     * 返回储值卡有效期。
     *
     * @return 储值卡有效期。
     */
    public String expireDate() {
        return this.expireDate;
    }

    /**
     * 返回卡片状态。
     *
     * @return 卡片状态。
     */
    public CardStatus status() {
        return this.status;
    }

    /**
     * 返回储值卡可用余额。
     *
     * @return 储值卡可用余额。
     */
    public double usableTotal() {
        return this.usableTotal;
    }

    /**
     * 返回会员卡号。
     *
     * @return 会员卡号。
     */
    public String vipCardNo() {
        return this.vipCardNo;
    }

    /**
     * 使用 Builder 模式初始化一个 {@code Card} 的实例。
     */
    public static class Builder {
        // 储值卡卡面号
        private String cardFaceNo;
        // 储值卡卡号
        private String cardNo;
        // 卡片状态
        private CardStatus status;
        // 储值卡类型
        private CardType cardType;
        // 储值卡有效期
        private String expireDate;
        // 储值卡可用余额
        private double usableTotal;
        // 会员卡号
        private String vipCardNo;
        /**
         * 使用范围控制,
         * 0 – 不控制
         * 1 – 提货
         * 2 – 限制商品使用范围
         */
        private String useLimit;

        /**
         * 初始化一个 {@code Card.Builder} 的实例。
         *
         * @param cardFaceNo 卡面号。
         * @param cardNo     卡号。
         */
        public Builder(String cardFaceNo, String cardNo) {
            this.cardFaceNo = cardFaceNo;
            this.cardNo = cardNo;
            this.useLimit = "0";
        }

        /**
         * 设置使用范围控制
         *
         * @param useLimit 使用范围控制
         */
        public Builder setUseLimit(String useLimit) {
            this.useLimit = useLimit;

            return this;
        }

        /**
         * 设置卡片状态。
         *
         * @param status 卡片状态编码
         */
        public Builder setCardStatus(String status) {
            this.status = new CardStatus(status);

            return this;
        }

        /**
         * 设置储值卡类型。
         *
         * @param typeCode 储值卡类型编码。
         * @param typeName 储值卡类型名称。
         */
        public Builder setCardType(String typeCode, String typeName) {
            this.cardType = new CardType(typeCode, typeName);

            return this;
        }

        /**
         * 设置储值卡有效期。
         *
         * @param expireDate 储值卡有效期。
         */
        public Builder setExpireDate(String expireDate) {
            this.expireDate = expireDate;

            return this;
        }

        /**
         * 设置储值卡可用余额。
         *
         * @param usableTotal 储值卡可用余额。
         */
        public Builder setUsableTotal(double usableTotal) {
            this.usableTotal = usableTotal;

            return this;
        }

        /**
         * 设置会员卡号。
         *
         * @param vipCardNo 会员卡号。
         */
        public Builder setVipCardNo(String vipCardNo) {
            this.vipCardNo = vipCardNo;

            return this;
        }

        /**
         * 构建一个 {@code Card} 的实例。
         */
        public Card build() {
            Card card = new Card();
            card.cardFaceNo = this.cardFaceNo;
            card.cardNo = this.cardNo;
            card.status = this.status;
            card.cardType = this.cardType;
            card.expireDate = this.expireDate;
            card.usableTotal = this.usableTotal;
            card.vipCardNo = this.vipCardNo;
            card.useLimit = this.useLimit;

            return card;
        }
    }
}
