/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

/**
 * 卡号类型。
 *
 * @author zhangsai 2017-10-09
 * @author guolina  2019-04-09 HIPOSAPP-1074 支持储值卡/积分动态码支付
 */
public enum CardNoType {
    /**
     * 默认卡内号。
     */
    Default(0),

    /**
     * 卡面号。
     */
    CardFaceNo(1),

    /**
     * 手机号。
     */
    Mobile(2),

    /**
     * 会员卡号。
     */
    VipCode(3),

    /**
     * 动态付款码。
     */
    PayCode(7);

    // 卡号类型编码。
    private final int typeCode;
    /**
     * 初始化一个 {@code CardNoType} 的枚举。
     *
     * @param typeCode 卡号类型编码。
     */
    CardNoType(int typeCode) {
        this.typeCode = typeCode;
    }

    /**
     * 返回 {@code typeCode} 对应的卡号类型。
     *
     * @param typeCode 卡号类型编码
     * @return 卡号类型
     */
    public static CardNoType valueOf(int typeCode) {
        switch (typeCode) {
            case 0:
                return CardNoType.Default;
            case 1:
                return CardNoType.CardFaceNo;
            case 2:
                return CardNoType.Mobile;
            case 3:
                return CardNoType.VipCode;
            case 7:
                return CardNoType.PayCode;
            default:
                return CardNoType.CardFaceNo;
        }
    }

    /**
     * 返回卡号类型编码。
     *
     * @return 卡号类型编码。
     */
    public int getTypeCode() {
        return this.typeCode;
    }
}
