/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

import java.io.Serializable;

/**
 * 储值卡积分卡支付的返回信息。
 *
 * @author guolina  2018-09-26 HIPOSAPP-770 支持销售/退款后增加储值卡和积分余额打印
 */
public final class CardPayResult implements Serializable {
    /**
     * 支付参考号
     */
    private String refNo;
    /**
     * 交易时间
     */
    private String tranTime;
    /**
     * 支付后可用余额
     */
    private double balance;
    /**
     * 商户订单号
     */
    private String outTradeNo;

    /**
     * 初始化一个 {@code CardPayResult} 的实例。
     *
     * @param refNo    支付参考号。
     * @param tranTime 交易时间。
     * @param balance  支付后可用余额。
     */
    public CardPayResult(String refNo, String tranTime, double balance) {
        this.refNo = refNo;
        this.tranTime = tranTime;
        this.balance = balance;
    }

    /**
     * 初始化一个 {@code CardPayResult} 的实例。
     *
     * @param refNo      支付参考号。
     * @param tranTime   交易时间。
     * @param balance    支付后可用余额。
     * @param outTradeNo 商户订单号
     */
    public CardPayResult(String refNo, String tranTime, double balance, String outTradeNo) {
        this.refNo = refNo;
        this.tranTime = tranTime;
        this.balance = balance;
        this.outTradeNo = outTradeNo;
    }

    /**
     * 返回支付参考号。
     *
     * @return 支付参考号。
     */
    public String refNo() {
        return this.refNo;
    }

    /**
     * 返回交易时间。
     *
     * @return 交易时间。
     */
    public String tranTime() {
        return this.tranTime;
    }

    /**
     * 支付后可用余额。
     *
     * @return 支付后可用余额。
     */
    public double balance() {
        return this.balance;
    }

    public double balance(double balance) {
        return this.balance = balance;
    }

    /**
     * 商户订单号。
     *
     * @return 商户订单号。
     */
    public String outTradeNo() {
        return outTradeNo;
    }


}
