/***************************************************************************************************
 * Copyright (c) 2003-2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

import java.io.Serializable;

/**
 * 储值卡/积分卡校验参数
 *
 * @author wangyaogang 2019-07-02 HIPOSAPP-1180 校验参数封装为CardSecurity,密码属性默认为""
 */
public class CardSecurity implements Serializable {
    //校验码
    private String cardVerifyCode;
    //卡面后几位
    private String cardFaceNoCheck;
    //密码
    private String password;

    private CardSecurity() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回校验码
     *
     * @return 校验码
     */
    public String cardVerifyCode() {
        return cardVerifyCode;
    }

    /**
     * 返回卡面后几位
     *
     * @return 卡面后几位
     */
    public String cardFaceNoCheck() {
        return cardFaceNoCheck;
    }

    /**
     * 返回密码
     *
     * @return 密码
     */
    public String password() {
        return password;
    }

    public static class Builder {
        //校验码
        private String cardVerifyCode;
        //卡面后几位
        private String cardFaceNoCheck;
        //密码
        private String password;

        /**
         * 设置校验位
         *
         * @param cardVerifyCode 校验位
         * @return CardSecurity.Builder
         */
        public Builder cardVerifyCode(String cardVerifyCode) {
            this.cardVerifyCode = cardVerifyCode;
            return this;
        }

        /**
         * 设置卡面后几位
         *
         * @param cardFaceNoCheck 卡面后几位
         * @return CardSecurity.Builder
         */
        public Builder cardFaceNoCheck(String cardFaceNoCheck) {
            this.cardFaceNoCheck = cardFaceNoCheck;
            return this;
        }

        /**
         * 设置密码
         *
         * @param password 密码
         * @return CardSecurity.Builder
         */
        public Builder password(String password) {
            this.password = password;
            return this;
        }

        /**
         * 构建一个 {@code CardSecurity} 的实例。
         */
        public CardSecurity build() {
            CardSecurity card = new CardSecurity();
            card.cardVerifyCode = this.cardVerifyCode;
            card.cardFaceNoCheck = this.cardFaceNoCheck;
            card.password = this.password == null ? "" : this.password;

            return card;
        }
    }
}
