/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.crm.card;

import java.io.Serializable;

/**
 * 卡片状态。
 *
 * @author zhangsai 2017-12-27 HIPOSAPP-258
 */
public class CardStatus implements Serializable {
    /**
     * 已注册
     */
    public static final String REGISTERED = "00";
    /**
     * 销售未开通
     */
    public static final String SOLD = "01";
    /**
     * 正常
     */
    public static final String NORMAL = "02";
    /**
     * 挂失
     */
    public static final String LOSS = "03";
    /**
     * 禁用
     */
    public static final String FORBIDDEN = "04";
    /**
     * 过期
     */
    public static final String EXPIRED = "05";
    /**
     * 注销
     */
    public static final String CANCELLED = "06";
    /**
     * 回收
     */
    public static final String RECYCLED = "07";

    // 状态编码
    private String code;

    /**
     * 初始化一个 {@code CardNo} 的实例。
     * @param code 状态编码
     */
    public CardStatus(String code) {
        this.code = code;
    }

    /**
     * 返回卡片状态编码。
     * @return 卡片状态编码
     */
    public String code() {
        return code;
    }

    /**
     * 返回卡片状态是否正常。
     * @return 正常时返回 true；其它返回 false。
     */
    public boolean isNormal() {
        return this.code.equals(CardStatus.NORMAL);
    }

    /**
     * 返回卡片状态是否为注册。
     * @return 注册时返回 true；其它返回 false。
     */
    public boolean isRegistered() {
        return this.code.equals(CardStatus.REGISTERED);
    }

    /**
     * 返回卡片状态是否已销售（但未开通）。
     * @return 已销售时返回 true；其它返回 false。
     */
    public boolean isSold() {
        return this.code.equals(CardStatus.SOLD);
    }

    /**
     * 返回卡片状态名称。
     * @return 卡片状态名称
     */
    public String name() {
        if (this.code.equals(CardStatus.REGISTERED)) {
            return "已注册";
        } else if (this.code.equals(CardStatus.SOLD)) {
            return "未开通";
        } else if (this.code.equals(CardStatus.LOSS)) {
            return "已挂失";
        } else if (this.code.equals(CardStatus.FORBIDDEN)) {
            return "已禁用";
        } else if (this.code.equals(CardStatus.EXPIRED)) {
            return "已过期";
        } else if (this.code.equals(CardStatus.CANCELLED)) {
            return "已注销";
        } else if (this.code.equals(CardStatus.RECYCLED)) {
            return "已回收";
        } else {
            return "正常";
        }
    }
}
