/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

/**
 * IC 卡版本。不同版本 IC 卡的数据格式不同。
 * <p>
 * IC 卡类型   Version1    Version2
 * -------------------------------
 * SLE4442      支持         ---
 * Mifare1      支持         支持
 *
 * @author zhangsai 2017-10-10
 * @author xieliyan 2017-11-27
 */
public enum IcCardVersion {
    /**
     * 1.0
     */
    Version1(0),
    /**
     * 2.0
     */
    Version2(2);

    private int value;

    /**
     * 初始化一个 IcCardVersion 枚举。
     *
     * @param value 指定 IC 卡版本的对应值。
     */
    IcCardVersion(int value) {
        this.value = value;
    }

    /**
     * 返回指定 IC 卡版本的对应值。
     *
     * @return IC 卡版本的对应值。
     */
    public int value() {
        return this.value;
    }

    /**
     * 将 {@code int} 值转换为对应的 {@code IcCardVersion}。异常时默认为2.0。
     *
     * @param value 要转换的值。
     * @return IC 卡版本号。
     */
    public static IcCardVersion valueOf(int value) {
        switch (value) {
            case 0:
                return IcCardVersion.Version1;
            default:
                return IcCardVersion.Version2;
        }
    }
}
