/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.card;

import java.io.Serializable;

/**
 * 会员卡。
 *
 * @author zhangsai    2017-10-10
 * @author zhangsai    2017-12-27 增加卡片状态
 * @author zhangsai    2018-02-02 HIPOSAPP-315 增加 VipCardNo 属性
 * @author wangyaogang 2019-08-15 HIPOSAPP-1278 调整积分查询接口，增加pointsConRatio积分支付换算比例
 */
public class VipCard implements Serializable {
    // 会员卡卡面号
    private String cardFaceNo;
    // 卡片状态
    private CardStatus status;
    // 会员卡类型
    private CardType cardType;
    // 会员卡有效期
    private String expireDate;
    // 可用积分
    private double usablePoints;
    // 积分可抵扣金额
    private double usableBalance;
    // 会员姓名
    private String vipName;
    // 会员卡号
    private String vipCardNo;
    // 可用金额
    private double usableTotal;//(SCRM余额支付）
    //积分支付换算比例
    private String pointsConRatio;
    private CardNoType cardNoType;

    /**
     * 初始化一个 {@code VipCard} 的实例。
     */
    private VipCard() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回会员卡卡面号。
     *
     * @return 会员卡卡面号。
     */
    public String cardFaceNo() {
        return this.cardFaceNo;
    }

    /**
     * 返回会员卡类型。
     *
     * @return 会员卡类型。
     */
    public CardType cardType() {
        return this.cardType;
    }

    /**
     * 返回会员卡有效期。
     *
     * @return 会员卡有效期。
     */
    public String expireDate() {
        return this.expireDate;
    }

    /**
     * 返回卡片状态。
     *
     * @return 卡片状态。
     */
    public CardStatus status() {
        return this.status;
    }

    /**
     * 返回会员卡积分可抵扣金额。
     *
     * @return 会员卡积分可抵扣金额。
     */
    public double usableBalance() {
        return this.usableBalance;
    }

    /**
     * 返回会员卡可用积分。
     *
     * @return 会员卡可用积分。
     */
    public double usablePoints() {
        return this.usablePoints;
    }

    /**
     * 返回会员姓名。
     *
     * @return 会员姓名。
     */
    public String vipName() {
        return this.vipName;
    }

    /**
     * 返回会员卡号。
     *
     * @return 会员卡号。
     */
    public String vipCardNo() {
        return this.vipCardNo;
    }

    /**
     * 返回会员卡号。
     *
     * @return 会员卡号。
     */
    public double usableTotal() {
        return this.usableTotal;
    }

    /**
     * 返回积分支付换算比例。
     *
     * @return 积分支付换算比例。
     */
    public String pointsConRatio() {
        return this.pointsConRatio;
    }

    public CardNoType cardNoType(){
        return this.cardNoType;
    }

    /**
     * 使用 Builder 模式初始化一个 {@code VipCard} 的实例。
     */
    public static class Builder {
        // 会员卡卡面号
        private String cardFaceNo;
        // 卡片状态
        private CardStatus status;
        // 会员卡类型
        private CardType cardType;
        // 会员卡有效期
        private String expireDate;
        // 可用积分
        private double usablePoints;
        // 积分可抵扣金额
        private double usableBalance;
        // 会员姓名
        private String vipName;
        // 会员卡号
        private String vipCardNo;
        // 可用金额
        private double usableTotal;
        //积分支付换算比例
        private String pointsConRatio;

        private CardNoType cardNoType;

        /**
         * 初始化一个 {@code VipCard.Builder} 的实例。
         *
         * @param cardFaceNo 会员卡卡面号。
         * @param vipName    会员姓名。
         */
        public Builder(String cardFaceNo, String vipName) {
            this.cardFaceNo = cardFaceNo;
            this.vipName = vipName;
        }

        /**
         * 设置会员卡类型。
         *
         * @param typeCode 会员卡类型编码。
         * @param typeName 会员卡类型名称。
         */
        public Builder cardType(String typeCode, String typeName) {
            this.cardType = new CardType(typeCode, typeName);

            return this;
        }

        /**
         * 设置会员卡有效期。
         *
         * @param expireDate 会员卡有效期。
         */
        public Builder expireDate(String expireDate) {
            this.expireDate = expireDate;

            return this;
        }

        /**
         * 设置卡片状态。
         *
         * @param value 卡片状态编码
         */
        public Builder status(String value) {
            this.status = new CardStatus(value);

            return this;
        }

        /**
         * 设置会员卡可用积分和积分可抵扣金额。
         *
         * @param usablePoints  会员卡可用积分。
         * @param usableBalance 积分可抵扣金额。
         */
        public Builder usable(double usablePoints, double usableBalance) {
            this.usablePoints = usablePoints;
            this.usableBalance = usableBalance;

            return this;
        }

        /**
         * 设置会员可用余额。
         *
         * @param usableTotal 可用余额。
         */
        public Builder usableTotal(double usableTotal) {
            this.usableTotal = usableTotal;
            return this;
        }

        /**
         * 设置会员卡号。
         *
         * @param vipCardNo 会员卡号。
         */
        public Builder vipCardNo(String vipCardNo) {
            this.vipCardNo = vipCardNo;

            return this;
        }

        /**
         * 设置积分支付换算比例。
         *
         * @param pointsConRatio 积分支付换算比例。
         */
        public Builder pointsConRatio(String pointsConRatio) {
            this.pointsConRatio = pointsConRatio;

            return this;
        }
        /**
         * 设置积分支付换算比例。
         *
         * @param pointsConRatio 积分支付换算比例。
         */
        public Builder cardNoType(CardNoType cardNoType) {
            this.cardNoType = cardNoType;

            return this;
        }

        /**
         * 创建一个 {@code VipCard} 的实例。
         */
        public VipCard build() {
            VipCard card = new VipCard();
            card.cardFaceNo = this.cardFaceNo;
            card.status = this.status;
            card.cardType = this.cardType;
            card.expireDate = this.expireDate;
            card.usablePoints = this.usablePoints;
            card.usableBalance = this.usableBalance;
            card.vipCardNo = this.vipCardNo;
            card.vipName = this.vipName;
            card.usableTotal = this.usableTotal;
            card.pointsConRatio = this.pointsConRatio;
            card.cardNoType=this.cardNoType;
            return card;
        }
    }
}
