/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.vip;

/**
 * 认证方式。
 *
 * @author wangyaogang 2019-05-28 HIPOSAPP-1117 会员认证方式可配置
 */
public enum AuthMethod {
    /**
     * 所有
     */
    All(1),

    /**
     * 扫码
     */
    SCAN(2);

    private int value;

    /**
     * 初始化一个 AuthMethod 枚举。
     *
     * @param value 指定的认证方式
     */
    AuthMethod(int value) {
        this.value = value;
    }

    /**
     * 返回认证方式。
     *
     * @return 认证方式对应值。
     */
    public int value() {
        return this.value;
    }

    /**
     * 将 {@code int} 值转换为对应的 {@code AuthMethod}。默认为角 1。
     *
     * @param value 要转换的值。
     * @return 认证方式。
     */
    public static AuthMethod valueOf(int value) {
        switch (value) {
            case 2:
                return AuthMethod.SCAN;
            case 1:
            default:
                return AuthMethod.All;
        }
    }

    /**
     * 是否按所有
     *
     * @return true：All；false：SCAN
     */
    public boolean isAll() {
        return this == AuthMethod.All;
    }

    /**
     * 是否仅扫码
     *
     * @return false：All；true：SCAN
     */
    public boolean isScan() {
        return this == AuthMethod.SCAN;
    }
}
