/***************************************************************************************************
 * Copyright (c) 2003-2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.vip;

import java.io.Serializable;

/**
 * 会员验证方式。
 *
 * @author wangyaogang 2019-06-26 HIPOSAPP-1180 增加会员验证方式VerifyType
 */
public class VerifyType implements Serializable {
    /**
     * 会员卡号
     */
    private String vipCardNo;
    /**
     * 卡面号
     */
    private String cardFaceNo;
    /**
     * 是否需要消费密码
     * 0-不需要
     * 1-需要
     */
    private boolean isVerifyPassword;
    /**
     * 会员卡面号后几位校验位数
     * 0-不校验
     * 其他值表示需要校验卡的后几位
     */
    private boolean isVerifyFaceNoCheck;
    /**
     * 卡面号验证长度
     */
    private int cardFaceNoCheckLength;
    /**
     * 卡面号校验码位数
     * 0-不校验
     * 其他值表示卡校验码的位数
     */
    private boolean isVerifyCode;
    /**
     * 校验码长度
     */
    private int verifyCodeLength;

    private VerifyType() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 是否进行校验
     *
     * @return true 需要 ,false 不需要
     */
    public boolean needVerify() {
        return isVerifyPassword || isVerifyFaceNoCheck || isVerifyCode;
    }

    /**
     * 获取会员卡号
     *
     * @return 会员卡号
     */
    public String vipCardNo() {
        return vipCardNo;
    }

    /**
     * 获取卡面号
     *
     * @return 卡面号
     */
    public String cardFaceNo() {
        return cardFaceNo;
    }

    /**
     * 校验密码
     *
     * @return
     */

    public boolean isVerifyPassword() {
        return isVerifyPassword;
    }

    /**
     * 校验卡面后几位
     *
     * @return
     */

    public boolean isVerifyFaceNoCheck() {
        return isVerifyFaceNoCheck;
    }

    /**
     * 校验卡面号长度
     *
     * @return
     */
    public int cardFaceNoCheckLength() {
        return cardFaceNoCheckLength;
    }

    /**
     * 校验码
     *
     * @return
     */

    public boolean isVerifyCode() {
        return isVerifyCode;
    }

    /**
     * 校验码长度
     *
     * @return
     */
    public int verifyCodeLength() {
        return verifyCodeLength;
    }


    /**
     * 使用 Builder 模式初始化一个 {@code VipPointsProject} 的实例。
     */
    public static class Builder {
        /**
         * 会员卡号
         */
        private String vipCardNo;
        /**
         * 卡面号
         */
        private String cardFaceNo;
        /**
         * 是否需要消费密码
         * 0-不需要
         * 1-需要
         */
        /**
         * 是否需要消费密码
         * 0-不需要
         * 1-需要
         */
        private boolean isVerifyPassword;
        /**
         * 会员卡面号后几位校验位数
         * 0-不校验
         * 其他值表示需要校验卡的后几位
         */
        private boolean isVerifyFaceNoCheck;
        /**
         * 卡面号验证长度
         */
        private int cardFaceNoCheckLength;
        /**
         * 卡面号校验码位数
         * 0-不校验
         * 其他值表示卡校验码的位数
         */
        private boolean isVerifyCode;
        /**
         * 校验码长度
         */
        private int verifyCodeLength;

        /**
         * 初始化一个 {@code VerifyType.Builder} 的实例。
         *
         * @param isVerifyPassword    是否需要消费密码。
         * @param isVerifyFaceNoCheck 会员卡面号后几位校验位数。
         * @param isVerifyCode        卡面号校验码位数。
         */
        public Builder(boolean isVerifyPassword, boolean isVerifyFaceNoCheck, boolean isVerifyCode) {
            this.isVerifyPassword = isVerifyPassword;
            this.isVerifyFaceNoCheck = isVerifyFaceNoCheck;
            this.isVerifyCode = isVerifyCode;
        }

        /**
         * 设置卡面号校验长度。
         *
         * @param cardFaceNoCheckLength 卡面号校验长度。
         */
        public Builder cardFaceNoCheckLength(int cardFaceNoCheckLength) {
            this.cardFaceNoCheckLength = cardFaceNoCheckLength;
            return this;
        }

        /**
         * 设置校验码长度。
         *
         * @param verifyCodeLength 校验码长度。
         */
        public Builder verifyCodeLength(int verifyCodeLength) {
            this.verifyCodeLength = verifyCodeLength;
            return this;
        }

        /**
         * 设置会员卡号。
         *
         * @param vipCardNo 会员卡号。
         */
        public Builder vipCardNo(String vipCardNo) {
            this.vipCardNo = vipCardNo;
            return this;
        }

        /**
         * 设置卡面号。
         *
         * @param cardFaceNo 卡面号。
         */
        public Builder cardFaceNo(String cardFaceNo) {
            this.cardFaceNo = cardFaceNo;
            return this;
        }

        /**
         * 构建一个 {@code VerifyType} 的实例。
         */
        public VerifyType build() {
            VerifyType card = new VerifyType();
            card.vipCardNo = this.vipCardNo;
            card.cardFaceNo = this.cardFaceNo;
            card.isVerifyPassword = this.isVerifyPassword;
            card.isVerifyFaceNoCheck = this.isVerifyFaceNoCheck;
            card.isVerifyCode = this.isVerifyCode;
            card.cardFaceNoCheckLength = this.cardFaceNoCheckLength;
            card.verifyCodeLength = this.verifyCodeLength;

            return card;
        }
    }
}
