/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.vip;

import com.hisense.crm.card.VipCard;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import lombok.Data;

/**
 * 会员。
 *
 * @author zhangsai 2017-10-12
 */
@Data
public class Vip implements Serializable {
    // 生日
    private String birthday;
    // 会员关怀信息
    private String careMsg;
    // 会员提示信息
    private String hintMsg;
    // 会员绑定的手机号
    private String mobile;
    // 会员姓名
    private String name;
    // 会员性别
    private VipSex sex;
    // 会员状态
    private VipStatus status;

    // 会员卡
    private VipCard card;
    // 积分方案
    private List<VipPointsProject> projects;

    private boolean iseCard;

    /**
     * 初始化一个 {@code Vip} 的实例。
     */
    private Vip() {
        this.projects = new ArrayList<>();
    }


    /**
     * 返回会员姓名。
     */
    public String name() {
        return name;
    }

    /**
     * 返回会员生日日期。格式：YYYY-HH-MM
     */
    public String birthday() {
        return birthday;
    }

    /**
     * 返回会员关怀信息。
     */
    public String careMessage() {
        return careMsg;
    }

    /**
     * 返回会员提示信息。
     */
    public String hintMessage() {
        return hintMsg;
    }

    /**
     * 返回会员绑定的手机号。
     */
    public String mobile() {
        return mobile;
    }

    /**
     * 返回会员性别。
     */
    public VipSex sex() {
        return sex;
    }

    /**
     * 返回会员状态。
     */
    public VipStatus status() {
        return status;
    }

    /**
     * 返回会员卡信息。
     */
    public VipCard card() {
        return card;
    }

    /**
     * 返回会员积分方案。
     */
    public List<VipPointsProject> projects() {
        return Collections.unmodifiableList(projects);
    }

    /**
     * 返回 {@code i} 指定位置的会员积分方案。
     *
     * @param i 积分方案在集合中的索引位置。
     * @return 指定位置的会员积分方案。
     */
    public VipPointsProject projects(int i) {
        return this.projects.get(i);
    }

    /**
     * 添加一个新的会员积分方案。
     *
     * @param project 要添加的会员积分方案。
     */
    public void addProject(VipPointsProject project) {
        if (project != null) {
            this.projects.add(project);
        }
    }


    /**
     * 使用 Builder 模式初始化一个 {@code Vip} 的实例。
     */
    public static class Builder {
        // 生日
        private String birthday;
        // 会员关怀信息
        private String careMsg;
        // 会员提示信息
        private String hintMsg;
        // 会员绑定的手机号
        private String mobile;
        // 会员姓名
        private String name;
        // 会员性别
        private VipSex sex;
        // 会员状态
        private VipStatus status;
        // 会员卡
        private VipCard card;


        /**
         * 初始化一个 {@code VipCard.Builder} 的实例。
         *
         * @param name 会员姓名。
         */
        public Builder(String name) {
            this.name = name;
        }


        /**
         * 设置会员生日。
         *
         * @param birthday 会员生日
         */
        public Builder birthday(String birthday) {
            this.birthday = birthday;
            return this;
        }

        /**
         * 设置会员关怀和提示信息。
         *
         * @param careMsg 会员关怀信息
         * @param hintMsg 会员提示信息
         */
        public Builder message(String careMsg, String hintMsg) {
            this.careMsg = careMsg;
            this.hintMsg = hintMsg;
            return this;
        }

        /**
         * 设置会员绑定的手机号。
         *
         * @param mobile 手机号
         */
        public Builder mobile(String mobile) {
            this.mobile = mobile;
            return this;
        }

        /**
         * 设置会员性别。
         *
         * @param sex 性别
         */
        public Builder sex(VipSex sex) {
            this.sex = sex;
            return this;
        }

        /**
         * 设置会员状态。
         *
         * @param status 会员状态
         */
        public Builder status(VipStatus status) {
            this.status = status;
            return this;
        }

        /**
         * 设置会员卡信息。
         *
         * @param card 会员卡
         */
        public Builder card(VipCard card) {
            this.card = card;
            return this;
        }

        /**
         * 构建一个 {@code Vip} 的实例。
         */
        public Vip build() {
            Vip vip = new Vip();
            vip.name = this.name;
            vip.birthday = this.birthday;
            vip.careMsg = this.careMsg;
            vip.hintMsg = this.hintMsg;
            vip.mobile = this.mobile;
            vip.sex = this.sex;
            vip.status = this.status;
            vip.card = this.card;

            return vip;
        }
    }
}
