/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.vip;

import java.io.Serializable;

/**
 * 会员积分方案。
 *
 * @author zhangsai 2017-10-12
 */
public class VipPointsProject implements Serializable {
    // 积分方案编码
    private String code;
    // 积分方案名称
    private String name;
    // 可用积分
    private double usablePoints;
    // 积分可抵扣金额
    private double usableBalance;

    private VipPointsProject() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回积分方案编码。
     * @return 积分方案编码。
     */
    public String code() {
        return code;
    }

    /**
     * 返回积分方案名称。
     * @return 积分方案名称。
     */
    public String name() {
        return name;
    }

    /**
     * 返回可用积分。
     * @return 可用积分。
     */
    public double usableBalance() {
        return usableBalance;
    }

    /**
     * 返回积分可抵扣金额。
     * @return 积分可抵扣金额。
     */
    public double usablePoints() {
        return usablePoints;
    }

    /**
     * 使用 Builder 模式初始化一个 {@code VipPointsProject} 的实例。
     */
    public static class Builder {
        // 积分方案编码
        private String code;
        // 积分方案名称
        private String name;
        // 可用积分
        private double usablePoints;
        // 积分可抵扣金额
        private double usableBalance;

        /**
         * 初始化一个 {@code VipPointsProject.Builder} 的实例。
         * @param code  积分方案编码。
         * @param name  积分方案名称。
         */
        public Builder(String code, String name) {
            this.code = code;
            this.name = name;
        }

        /**
         * 设置会员卡可用积分和积分可抵扣金额。
         * @param usablePoints  会员卡可用积分。
         * @param usableBalance 积分可抵扣金额。
         */
        public Builder usable(double usablePoints, double usableBalance) {
            this.usablePoints = usablePoints;
            this.usableBalance = usableBalance;

            return this;
        }

        /**
         * 构建一个 {@code VipCard} 的实例。
         */
        public VipPointsProject build() {
            VipPointsProject card = new VipPointsProject();
            card.code = this.code;
            card.name = this.name;
            card.usablePoints = this.usablePoints;
            card.usableBalance = this.usableBalance;

            return card;
        }
    }
}
