/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.crm.vip;

/**
 * 会员状态。
 *
 * @author zhangsai 2017-10-12
 */
public enum VipStatus {
    /**
     * 未知。
     */
    Unknown(-1),

    /**
     * 未审核。
     */
    UnReviewed(0),

    /**
     * 正常。
     */
    Normal(1),

    /**
     * 淘汰。
     */
    Dropped(2);

    // 表示会员状态的值
    private final int value;

    /**
     * 初始化一个 {@code VipStatus} 的实例。
     *
     * @param value 表示会员状态的值
     */
    VipStatus(int value) {
        this.value = value;
    }

    /**
     * 返回会员状态的对应值。
     */
    public int value() {
        return value;
    }


    /**
     * 将 {@code int} 值转换为对应的 {@code VipStatus}。异常值默认返回 VipStatus.Unknown。
     *
     * @param value 要转换的值。
     * @return 会员状态。
     */
    public static VipStatus valueOf(int value) {
        switch (value) {
            case 0:
                return VipStatus.UnReviewed;
            case 1:
                return VipStatus.Normal;
            case 2:
                return VipStatus.Dropped;
            default:
                return VipStatus.Unknown;
        }
    }

    public String trueName() {
        if (this == VipStatus.UnReviewed) {
            return "未审核";
        } else if (this == VipStatus.Normal) {
            return "正常";
        } else if (this == VipStatus.Dropped) {
            return "淘汰";
        } else {
            return "未知";
        }
    }

    /**
     * 返回会员状态是否通过审核。
     *
     * @return 通过审核时返回 true；其它返回 false。
     */
    public boolean isUnReviewed() {
        return valueOf(this.value).equals(UnReviewed);
    }

    /**
     * 返回会员状态是否正常。
     *
     * @return 正常时返回 true；其它返回 false。
     */
    public boolean isNormal() {
        return valueOf(this.value).equals(Normal);
    }

    /**
     * 返回会员状态是否被淘汰。
     *
     * @return 被淘汰时返回 true；其它返回 false。
     */
    public boolean isDropped() {
        return valueOf(this.value).equals(Dropped);
    }

    /**
     * 返回会员状态是否异常。
     *
     * @return 异常时返回 true；其它返回 false。
     */
    public boolean isUnknown() {
        return valueOf(this.value).equals(Unknown);
    }
}
