/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi;

import com.hisense.hipos.user.User;

/**
 * WebAPI 的执行环境。
 *
 * @author zhangsai    2017-09-02
 * @author guolina     2018-12-11 HIPOSAPP-850 增加门店名称
 * @author guolina     2019-03-27 HIPOSAPP-1056 万科小票打印支持地址和电话字段
 * @author wangyaogang 2019-05-14 HIPOSAPP-1093 修复系统崩溃后上传日志引发的再次崩溃
 */
public class Context {
    // Context 的实例
    private static Context INSTANCE;

    // 组织编码
    private String orgCode;
    // 门店名称
    private String orgName;
    // 门店地址
    private String address;
    // 门店电话
    private String telephone;
    // 收款机号
    private String posNo;
    // 专柜编号
    private String shopCode;
    // 收银员
    private User cashier;

    /**
     * 初始化一个 {@code Context} 的实例。
     */
    private Context() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回配置的组织编码。
     *
     * @return 组织编码
     */
    public static String orgCode() {
        if (INSTANCE != null) {
            return INSTANCE.orgCode;
        }
        return "";
    }

    /**
     * 返回配置的门店名称。
     *
     * @return 门店名称
     */
    public static String orgName() {
        if (INSTANCE != null) {
            return INSTANCE.orgName;
        }
        return "";
    }

    /**
     * 返回配置的门店地址。
     *
     * @return 门店地址
     */
    public static String address() {
        if (INSTANCE != null) {
            return INSTANCE.address;
        }
        return "";
    }

    /**
     * 返回配置的门店电话。
     *
     * @return 门店电话
     */
    public static String telephone() {
        if (INSTANCE != null) {
            return INSTANCE.telephone;
        }
        return "";
    }

    /**
     * 设置当前门店名称。
     *
     * @param orgName 门店名称
     */
    public static void orgName(String orgName) {
        if (INSTANCE != null) {
            INSTANCE.orgName = orgName;
        }
    }

    /**
     * 设置当前门店地址。
     *
     * @param address 门店地址
     */
    public static void address(String address) {
        if (INSTANCE != null) {
            INSTANCE.address = address;
        }
    }

    /**
     * 设置当前门店电话。
     *
     * @param telephone 门店电话
     */
    public static void telephone(String telephone) {
        if (INSTANCE != null) {
            INSTANCE.telephone = telephone;
        }
    }

    /**
     * 返回配置的收款机号。
     *
     * @return 收款机号
     */
    public static String posNo() {
        if (INSTANCE != null) {
            return INSTANCE.posNo;
        }
        return "";
    }

    /**
     * 返回配置的专柜编号。
     *
     * @return 专柜编号
     */
    public static String shopCode() {
        if (INSTANCE != null) {
            return INSTANCE.shopCode;
        }
        return "";
    }

    /**
     * 返回当前登录的收银员。
     *
     * @return 当前登录的收银员
     */
    public static User cashier() {
        if (INSTANCE != null) {
            return INSTANCE.cashier;
        }
        return null;
    }

    /**
     * 设置当前登录的收银员。
     *
     * @param cashier 收银员
     */
    public static void cashier(User cashier) {
        if (INSTANCE != null) {
            INSTANCE.cashier = cashier;
        }
    }


    /**
     * 配置 WebAPI 执行环境。
     *
     * @param orgCode  组织编码
     * @param orgName  组织名称
     * @param posNo    收款机号
     * @param shopCode 专柜编号
     */
    public static void config(String orgCode, String orgName, String posNo, String shopCode) {
        if (INSTANCE != null) {
            INSTANCE.orgCode = orgCode;
            INSTANCE.orgName = orgName;
            INSTANCE.posNo = posNo;
            INSTANCE.shopCode = shopCode;
        } else {
            Context context = new Context();
            context.orgCode = orgCode;
            context.orgName = orgName;
            context.posNo = posNo;
            context.shopCode = shopCode;

            INSTANCE = context;
        }
    }

}
