/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi;

import com.hisense.security.RSA;
import com.hisense.webapi.WebApi;

/**
 * WebAPI 的 RSA 加密/解密服务。
 *
 * @author zhangsai 2017-09-02
 */
public class RSAService {

    // WebAPI 的 RSA 公钥
    private final static String PUBLIC_KEY = "<RSAKeyValue><Modulus>rQZa25l8QY6TqsFGhnd+J1ZkDqNj7bEFhxh+jQRSU+U5SZvmMoG8htW+jF/phySJ5xdzyycdPU6JruagfQ+jBLVWWi05dq8uq62+BzP20DJUznZO/hpNYPDi+moxA9SFf2Mc6prr+fqUAof4v8o0NEHM7hWsLdSU34AGMiutKzM=</Modulus><Exponent>AQAB</Exponent></RSAKeyValue>";
    private final static String PRIVATE_KEY = "<RSAKeyValue><Modulus>1A0sfo4a0ureiZIw90CPvFHFvKzSpJM4zbrQbhzsxTKbTI8o12hw3Pw/tIMdNdGJjXZKyIT5GVzXnz29aKnYoXVTSU6TWUw2/ep63vVi+Pyi8GiDjo/tJjLbUEY4VY5d+Nw6GjHwv0Ulx54JzVXEDZUlw0KueR7ECGOOAsEe00M=</Modulus><Exponent>AQAB</Exponent><P>9JaP5WJ2+kdRbdiG3/lVJh5UjlhaCxE/IRe7SFExzfGd3ghob1nhuTLeg+5nKNiKerfiqJsghd3EudfO4dl+0Q==</P><Q>3fH7wQ0mFKNPZVQTwiqvulzn3PVDOCRxci+Q6AqqCiIAisLpv8qH1Fw1B0wCEzvOBUWdpscxWpnMONUNEUu90w==</Q><DP>l0ld3jumbi6Y3oH1Ayq9EWPnV1is8XWMf9KnBBc84C7YQBMhTebhqOUfH6nh4zD1juyY5UN2vXasDWvASpB2UQ==</DP><DQ>tU9TXLgDHeTznVF7+qioedQfzg/GY/zReHiHJihgFHsGelUUY4tuPvs8jsN3BoNaoE/opDAhj2QIw3y4eiKn5Q==</DQ><InverseQ>HB5tk8yk8wCW62xbzfIRZQ5bFAHLl2pUiyyGcUWiL4RN524bjUIuWu7WYuvQEdbpA5pO6jew75NCZBXsfmciWw==</InverseQ><D>QrXARTIcgzUrfFkW+XoAngyVqhUukWLVcHdwgeusmx8gFzZEcXvkYckjm3zSEC/S18OapDmdChsYpHquX0JrqVDz+jVuxJX+M50U3M7Zk79VyDyFH1v0rVbeIGjmOw4blxgQhnDG6sJ+cTknsISaL9wvldxnnULumHbVBjmvZvE=</D></RSAKeyValue>";

    // RSA 加密/解密的实例
    private final static RSA RSA = new RSA(PUBLIC_KEY, PRIVATE_KEY);


    /**
     * 初始化一个 {@code RSAService} 的实例。
     */
    private RSAService() {
        // NO CODE HERE. 避免初始化。
    }


    /**
     * 使用默认公钥加密字符串。
     *
     * @param value 待加密的字符串
     * @return 加密后的 Base64 字符串
     */
    public static String encrypt(final String value) {

        if (WebApi.isAmwAndAip()) {
            return value;
        } else {
            return new RSA(PUBLIC_KEY, PRIVATE_KEY).encrypt(value);
        }
    }


    /**
     * 使用默认私钥解密字符串。
     *
     * @param encrypted 加密的 Base64 字符串
     * @return 解密后的字符串
     */
    public static String decrypt(final String encrypted) {
        return RSA.decrypt(encrypted);
    }

}
