/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.auth;

import com.hisense.hipos.user.User;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 认证服务。
 *
 * @author zhangsai    2017-09-02
 * @author zhangsai    2017-09-07 增加修改登录密码
 * @author zhangsai    2017-10-19 增加收款员登录异步调用
 * @author zhangsai    2017-10-20 增加收款员暂离登录（无需提供收款员编码）
 * @author zhangsai    2017-10-27 增加修改登录密码异步调用
 * @author wangyaogang 2019-01-07 HIPOSAPP-851 用户账号权限控制优化
 */
public class AuthService {

    private AuthService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 收款员登录。
     *
     * @param userCode 收款员编码。
     * @param password 登录密码。
     * @param uuid     设备唯一标识。
     * @param callback 收款员登录回调方法。
     */
    public static void login(final String userCode, final String password, final String uuid,
                             final com.hisense.webapi.Callback<LoginResultData> callback) {
        Request<LoginRequestData> request = LoginRequestData.request(userCode, password, uuid);
        WebApi.requestAsync(WebApi.create(IAuthService.class).login(request), new Callback<Result<LoginResultData>>() {
            @Override
            public void onResponse(Call<Result<LoginResultData>> call, Response<Result<LoginResultData>> response) {
                if (response.body() == null || response.body().data == null) {
                    callback.onError(new Throwable("登录接口返回信息为空"));
                    return;
                }
                if ("null".equals(response.body().data)) {
                    callback.onError(new Throwable("登录接口返回信息:null"));
                    return;
                }
                callback.onSuccess(LoginResultData.handlerResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<LoginResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },LoginResultData.class);
    }

    /**
     * 修改登录密码。
     *
     * @param userCode    收款员编码。
     * @param oldPassword 原登录密码。
     * @param newPassword 新登录密码。
     * @param callback    修改密码回调方法。
     */
    public static void changePassword(final String userCode, final String oldPassword, final String newPassword, final com.hisense.webapi.Callback<Object> callback) {
        Request request = ChangePasswordData.request(userCode, oldPassword, newPassword);

        WebApi.requestAsync(WebApi.create(IAuthService.class).changePassword(request), new Callback<Result<Object>>() {
            @Override
            public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                callback.onSuccess(new Object());
            }

            @Override
            public void onFailure(Call<Result<Object>> call, Throwable t) {
                callback.onError(t);
            }
        },Object.class);
    }

    /**
     * 查询权限列表
     *
     * @param userId   用户id
     * @param callback 权限列表回调
     */
    public static void queryPrivileges(final String userId,
                                       final com.hisense.webapi.Callback<User> callback) {
        Request request = PrivilegeRequestData.request(userId);

        WebApi.requestAsync(WebApi.create(IAuthService.class).queryPrivileges(request), new Callback<Result<PrivilegeResultData>>() {
            @Override
            public void onResponse(Call<Result<PrivilegeResultData>> call, Response<Result<PrivilegeResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("查询权限列表返回信息为空"));
                    return;
                }

                callback.onSuccess(PrivilegeResultData.handlerResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<PrivilegeResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },PrivilegeResultData.class);
    }
}
