/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.auth;

import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import lombok.Data;

/**
 * 收款员登录请求数据。
 *
 * @author zhangsai 2017-09-02
 * @author zhangsai 2017-10-20
 */
@Data
public class LoginRequestData {

    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 登录密码
     */
    public String password;
    /**
     * 收款机号
     */
    public String posNo;
    /**
     * 专柜编码
     */
    public String shopCode;
    /**
     * 专柜编码中台
     */
    public String depCode;
    /**
     * 收款员编码
     */
    public String userCode;
    /**
     * 设备唯一标识
     */
    public String uuid;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param userCode 收款员编码
     * @param password 登录密码
     * @param uuid     设备ID
     * @return WebAPI 请求
     */
    public static Request<LoginRequestData> request(final String userCode, final String password, final String uuid) {
        LoginRequestData data = new LoginRequestData();
        data.orgCode = RSAService.encrypt(Context.orgCode());
        data.posNo = RSAService.encrypt(Context.posNo());
        if (WebApi.isAmwAndAip()) {
            data.depCode = Context.shopCode();
        } else {
            data.shopCode = RSAService.encrypt(Context.shopCode());
        }

        data.userCode = RSAService.encrypt(userCode);
        data.password = RSAService.encrypt(password);
        data.uuid = RSAService.encrypt(uuid);

        return WebApi.sign(data);
    }
}
