/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.coupon;

import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.user.User;
import com.hisense.hipos.webapi.Context;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.LinkedList;
import java.util.List;

/**
 * 赠券计算请求数据。
 *
 * @author zhangsai 2017-11-22
 */
class CouponCalcRequestData {
    /**
     * CRM类型
     */
    public String crmType;
    /**
     * 会员卡面号
     */
    public String cardFaceNo;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 销售商品
     */
    public List<CouponCalcRequestPlu> plu;
    /**
     * 赠券校验码
     */
    public String qCheckCode;
    /**
     * 券号
     */
    public String qNo;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 销售日期
     */
    public String saleTime;
    /**
     * 收银员编码
     */
    public String userCode;
    /**
     * 收银员 ID
     */
    public String userId;
    /**
     * 收银员姓名
     */
    public String userName;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param coupon     赠券
     * @param cardFaceNo 会员卡面号
     * @param user       收银员
     * @return WebAPI 请求
     */
    public static Request<CouponCalcRequestData> request(Coupon coupon, String cardFaceNo, User user) {
        MainSaleModel mainSaleModel = MainSaleModel.getInstance();
        SaleModel saleModel = mainSaleModel.getSaleModel();

        CouponCalcRequestData data = new CouponCalcRequestData();
        if (WebApi.isAmwAndAip()) {
            data.crmType = "CRM6";
        }
        data.cardFaceNo = cardFaceNo;
        data.orgCode = Context.orgCode();
        data.qCheckCode = coupon.checkCode();
        data.qNo = coupon.no();
        data.saleNo = saleModel.getSaleNo();
        data.saleTime = saleModel.getSaleDate();
        data.userCode = user.code();
        data.userId = user.id();
        data.userName = user.name();
        //此处使用LinkedList是保证传入后台数据库的顺序和商品添加顺序一致
        data.plu = new LinkedList<>();
        for (BaseSalePluModel pluModel : saleModel.getPlus()) {
            CouponCalcRequestPlu requestPlu = new CouponCalcRequestPlu();
            requestPlu.pageNo = pluModel.getPageNo();
            requestPlu.pluCode = pluModel.getPluCode();
            requestPlu.pluId = pluModel.getPluId();
            requestPlu.quantity = pluModel.getQuantity();
            requestPlu.serialNo = pluModel.getItemNo();
            requestPlu.total = pluModel.getTotalRecv();
            requestPlu.totalPaid = pluModel.getTotalPaid();
            requestPlu.depCode = pluModel.getDepCode();
            requestPlu.shopCode = pluModel.getShopCode();

            data.plu.add(requestPlu);
        }

        return WebApi.sign(data);
    }
}
