/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.coupon;

import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.CouponType;
import com.hisense.hipos.coupon.SaleItemShare;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;

import java.util.List;

/**
 * 赠券计算返回结果。
 *
 * @author zhangsai 2017-11-22
 * @author guolina  2019-03-25 HIPOSAPP-1048 修复手动添加会员券再次选中时未传校验码导致的用券失败
 */
class CouponCalcResultData {
    /**
     * 限制的支付方式
     */
    public String payLimits;
    /**
     * 商品分摊
     */
    public List<CouponCalcResultPluShare> plu;
    /**
     * 促销赠券编号
     */
    public String proNo;
    /**
     * 可核销金额
     */
    public String qAvailable;
    /**
     * 券类型
     */
    public int qType;
    /**
     * 面额或折扣
     */
    public String qValue;
    /**
     * 使用截止日期
     */
    public String endDate;

    /**
     * 处理 WebAPI 接口返回数据。
     *
     * @param resultData 接口返回数据
     * @param no
     * @param qNo        券号
     * @return 赠券的可用余额及商品分摊
     */
    public static UsableCoupon handleResult(CouponCalcResultData resultData, String qNo, String checkCode) {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();
        if (resultData != null) {
            Coupon coupon = new Coupon.Builder()
                    .coupon(qNo, CouponType.valueOf(resultData.qType))
                    .checkCode(checkCode)
                    .values(Double.valueOf(resultData.qValue))
                    .available(Double.valueOf(resultData.qAvailable))
                    .build();

            UsableCoupon usableCoupon = new UsableCoupon(resultData.proNo, coupon, resultData.payLimits);

            if (resultData.plu != null) {
                for (CouponCalcResultPluShare plu : resultData.plu) {
                    SaleItemShare share = new SaleItemShare.Builder()
                            .item(plu.pageNo, plu.serialNo)
                            .plu(plu.pluCode, plu.pluId)
                            .share(Double.valueOf(plu.qShare))
                            .total(Double.parseDouble(saleModel.getPlu(plu.serialNo - 1).getTotalRecv()),
                                    Double.parseDouble(saleModel.getPlu(plu.serialNo - 1).getTotalPaid()))
                            .build();
                    usableCoupon.addShare(share);
                }
            }

            return usableCoupon;
        }

        return null;
    }
}
