/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.coupon;

import com.hisense.hipos.coupon.SaleItemShare;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;

/**
 * 赠券核销请求数据。
 *
 * @author zhangsai 2017-11-24
 * @author guolina  2019-12-09   HIAPOSCMP-403  券核销组织编码错误
 */
class CouponUse1RequestData {
    /**
     * 待核销赠券列表
     */
    public List<CouponUse1RequestCoupon> coupons;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 流水号
     */
    public String saleNo;


    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param usableCoupons 待核销的赠券集合
     * @return WebAPI 请求
     */
    public static Request<CouponUse1RequestData> request(List<UsableCoupon> usableCoupons) {
        MainSaleModel mainSaleModel=MainSaleModel.getInstance();
        SaleModel saleModel = mainSaleModel.getSaleModel();

        CouponUse1RequestData requestData = new CouponUse1RequestData();
        requestData.orgCode = Context.orgCode();
        requestData.saleNo = saleModel.getSaleNo();

        requestData.coupons = new ArrayList<>();
        for (UsableCoupon usableCoupon : usableCoupons) {
            CouponUse1RequestCoupon requestCoupon = new CouponUse1RequestCoupon();
            requestCoupon.proNo = usableCoupon.proNo();
            requestCoupon.qNo = usableCoupon.coupon().no();
            requestCoupon.qType = usableCoupon.coupon().type().value();
            requestCoupon.qUse = Double.valueOf(usableCoupon.useValue()).toString();

            if (usableCoupon.shares() != null && usableCoupon.shares().size() > 0) {
                requestCoupon.plu = new ArrayList<>();
                for (SaleItemShare itemShare :
                        usableCoupon.shares()) {
                    CouponUse1RequestPluShare pluShare = new CouponUse1RequestPluShare();
                    pluShare.pageNo = itemShare.pageNo();
                    pluShare.pluCode = itemShare.pluCode();
                    pluShare.pluId = itemShare.pluId();
                    pluShare.qShare = Double.valueOf(itemShare.value()).toString();
                    pluShare.qUse = Double.valueOf(itemShare.useValue()).toString();
                    pluShare.serialNo = itemShare.itemNo();
                    pluShare.total = Double.valueOf(itemShare.total()).toString();
                    pluShare.totalPaid = Double.valueOf(itemShare.totalPaid()).toString();

                    requestCoupon.plu.add(pluShare);
                }
            }

            requestData.coupons.add(requestCoupon);
        }

        return WebApi.sign(requestData);
    }
}
