package com.hisense.hipos.webapi.coupon;

import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.ICoupon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * {一段式核销返回结果}
 *
 * @author wangyaogang 2019-06-13 HIPOSAPP-1134 一段式核销接口返回剩余可用券信息
 */
class CouponUse1ResultData {
    /**
     * 核销过后有剩余金额的券
     */
    public List<RemainCoupon> remainedCoupons;

    static Collection<ICoupon> handleResult(CouponUse1ResultData resultData) {
        List<ICoupon> coupons = new ArrayList<>();
        if (resultData.remainedCoupons != null && resultData.remainedCoupons.size() > 0) {
            for (RemainCoupon c : resultData.remainedCoupons) {
                ICoupon coupon = new Coupon.Builder()
                        .coupon(c.no, c.checkCode, c.name)
                        .values(Double.valueOf(c.value))
                        .beginAt(c.beginDate)
                        .endsAt(c.endDate)
                        .isPrint(c.isPrint == 1)
                        .remark(c.remark)
                        .build();
                coupons.add(coupon);
            }
        }
        return coupons;
    }


    class RemainCoupon {
        /**
         * 促销方案名称
         */
        public String name;

        /**
         * 赠券编号
         */
        public String no;

        /**
         * 校验码
         */
        public String checkCode;

        /**
         * 面额。
         */
        public String value;

        /**
         * 用券开始日期 格式：yyyy-MM-dd
         */
        public String beginDate;

        /**
         * 用券结束日期
         */
        public String endDate;

        /**
         * 会员卡号
         */
        public String cardNo;

        /**
         * 会员卡类型
         */
        public String cardType;

        /**
         * 是否打印。0-不打印；1-打印
         */
        public int isPrint;

        /**
         * 促销说明
         */
        public String remark;
    }
}
