/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.coupon;

/**
 * 赠券原路退回返回数据的赠券使用情况。
 *
 * @author guolina 2018-11-23 HIPOSAPP-772 整单退货支持赠券退回
 */
public class CouponUsedData {

    /**
     * 原交易产生送券的使用情况
     */
    private CouponRebate rebate;

    /**
     * 原交易使用赠券情况
     */
    private CouponSale sale;


    /**
     * 初始化一个 {@code CouponUsedData} 的实例。
     *
     * @param rebate 原交易产生送券的使用情况
     * @param sale   原交易使用赠券情况
     */
    private CouponUsedData(CouponRebate rebate, CouponSale sale) {
        this.rebate = rebate;
        this.sale = sale;
    }

    /**
     * 返回原交易产生送券的使用情况。
     *
     * @return 原交易产生送券的使用情况
     */
    public CouponRebate rebate() {
        return this.rebate;
    }

    /**
     * 返回原交易使用赠券情况。
     *
     * @return 原交易使用赠券情况
     */
    public CouponSale sale() {
        return this.sale;
    }


    /**
     * 使用 Builder 模式初始化一个 {@code Sale} 的实例。
     */
    public static class Builder {

        // 原交易产生送券的使用情况
        private CouponRebate rebate;

        //原交易使用赠券情况
        private CouponSale sale;


        /**
         * 初始化一个 {@code CouponUsedData.Builder} 的实例。
         *
         * @param rebate 原交易产生送券的使用情况
         * @param sale   原交易使用赠券情况
         */
        public Builder(CouponRebate rebate, CouponSale sale) {

            this.rebate = rebate;
            this.sale = sale;
        }

        public CouponUsedData build() {
            CouponUsedData couponUsedData = new CouponUsedData(this.rebate, this.sale);
            return couponUsedData;
        }
    }
}
