/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.coupon;

import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import java.util.List;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * 赠券服务的 retrofit 接口。
 *
 * @author zhangsai    2017-10-16
 * @author zhangsai    2017-11-22 增加赠券计算
 * @author xieliyan    2018-04-10 HIPOSAPP-346 增加赠券撤销
 * @author guolina     2018-11-15 HIPOSAPP-772 整单退货支持赠券退回
 * @author guolina     2018-09-10 HIPOSAPP-766 修复赠券核销(coupon/use)返回数据类型与服务不一致的问题
 * @author wangyaogang 2019-06-13 HIPOSAPP-1134 一段式核销接口返回剩余可用券信息
 */
interface ICouponService {

    /**
     * 计算赠券可用金额及分摊。
     *
     * @param request 计算赠券的请求。
     * @return 计算赠券的 retrofit 调用。
     */
    @POST("api/coupon/calc")
    Call<Result<CouponCalcResultData>> calc(@Body Request<CouponCalcRequestData> request);
    /**
     * 多张券计算赠券可用金额及分摊。
     *
     * @param request 计算赠券的请求。
     * @return 计算赠券的 retrofit 调用。
     */
    @POST("api/coupon/calcMulti")
    Call<Result<List<CouponCalcMultiResultData>>> calcMulti(@Body Request<CouponCalcMultiRequestData> request);

    /**
     * 一段式赠券核销。
     *
     * @param request 一段式赠券的核销请求。
     * @return 一段式赠券核销的 retrofit 调用。
     */
    @POST("api/coupon/use1")
    Call<Result<CouponUse1ResultData>> use1(@Body Request<CouponUse1RequestData> request);

    /**
     * 赠券撤销。
     *
     * @param request 赠券的撤销请求。
     * @return 赠券撤销的 retrofit 调用。
     */
    @POST("api/coupon/cancel")
    Call<Result> cancel(@Body Request<CouponUse1RequestData> request);

    /**
     * 赠券原路退回预判。
     *
     * @param request 赠券原路退回预判请求。
     * @return 赠券原路退回预判的 retrofit 调用。
     */
    @POST("api/coupon/return/predict")
    Call<Result<CouponReturnResultData>> couponReturnPredict(@Body Request<CouponReturnRequestData> request);

    /**
     * 赠券原路退回确认。
     *
     * @param request 赠券原路退回确认请求。
     * @return 赠券原路退回确认的 retrofit 调用。
     */
    @POST("api/coupon/return/confirm")
    Call<Result<String>> couponReturnConfirm(@Body Request<CouponReturnRequestData> request);

    /**
     * 订单可用优惠券查询接口。
     *
     * @param request 订单可用优惠券查询请求。
     * @return 赠券原路退回确认的 retrofit 调用。
     */
    @POST("api/vip/coupons")
    Call<Result<List<CouponPluQueryResultData>>> couponPluQuery(@Body Request<CouponPluQueryRequestData> request);

    /**
     * 促销方案查询接口。
     *
     * @param request 促销方案查询请求。
     * @return 促销方案的 retrofit 调用。
     */
    @POST("api/coupon/project")
    Call<Result<List<CouponProjectQueryResultData>>> couponProjectQuery(@Body Request<CouponProjectQueryRequestData> request);
}
