/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.images;

import com.hisense.hipos.template.models.PrintImage;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;
import java.util.List;

/**
 * 打印图片服务。
 *
 * @author xieliyan 2018-03-22
 */

public class ImageService {

    private ImageService() {

    }

    /**
     * 检查本地打印图片是否与服务器中一致。如果不一致，则返回最新打印图片。
     *
     * @param images 打印图片信息
     * @return 服务器最新的图片列表。如果服务器与本地一致，则返回 null。
     */
    public static List<PrintImage> checkUpdate(List<PrintImage> images) throws WebApiException {
        List<PrintImage> imageList = new ArrayList<>();
        List<CheckUpdateRequestData> checkUpdateRequestDataList = new ArrayList<>();
        for (PrintImage image : images) {
            CheckUpdateRequestData requestData = new CheckUpdateRequestData(image.name(), image.timeStamp());
            checkUpdateRequestDataList.add(requestData);
        }

        Request<List<CheckUpdateRequestData>> request = WebApi.sign(checkUpdateRequestDataList);

        List<CheckUpdateResultData> resultData = WebApi.request(WebApi.create(IImageService.class).checkUpdate(request));
        if (resultData != null) {
            for (CheckUpdateResultData checkUpdateResultData : resultData) {
                imageList.add(new PrintImage(checkUpdateResultData.name, checkUpdateResultData.state,checkUpdateResultData.timestamp));
            }
            return imageList;
        }

        return null;
    }
}
