/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.pay;

import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * 直连 支付相关接口
 *
 * @author wangyaogang 2018-11-27
 * @author wangyaogang 2019-08-08 HIPOSAPP-1259 对接海信支付退款结果查询接口
 */
interface IPayService {

    /**
     * 支付接口。
     *
     * @param request 支付请求。
     * @return 支付的 retrofit 调用。
     */
    @POST("api/pay/pay")
    Call<Result<PayResultData>> pay(@Body Request<PayRequestData> request);

    /**
     * 查询接口。
     *
     * @param request 查询请求。
     * @return 查询的 retrofit 调用。
     */
    @POST("api/pay/query")
    Call<Result<PayResultData>> query(@Body Request<PayRequestData> request);

    /**
     * 撤销接口。
     *
     * @param request 撤销请求。
     * @return 撤销的 retrofit 调用。
     */
    @POST("api/pay/cancel")
    Call<Result<CancelResultData>> cancel(@Body Request<PayRequestData> request);

    /**
     * 退款接口。
     *
     * @param request 退款请求。
     * @return 退款的 retrofit 调用。
     */
    @POST("api/pay/refund")
    Call<Result<PayResultData>> refund(@Body Request<PayRequestData> request);

    /**
     * 退款状态查询
     *
     * @param request 上传请求
     * @return 退款的 retrofit 调用
     */
    @POST("api/pay/refundQuery")
    Call<Result<PayResultData>> refundQuery(@Body Request<PayRequestData> request);

    /**
     * 交易支付上传接口。
     *
     * @param request 上传请求。
     * @return 退款的 retrofit 调用。
     */
    @POST("api/pay/uploadOrders")
    Call<Result<Object>> uploadOrders(@Body Request<UploadOrdersRequestData> request);
}
