package com.hisense.hipos.webapi.pay;

import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.HashMap;

/**
 * 付款完成请求参数
 *
 * @author dukunbiao(null) 2018-05-24
 */
public class PayCompleteRequestData {

    public String cshCode;  //收银员编码
    public PayData pay; //支付信息
    public String posNo; //收款机号
    public String saleNo;   //当前流水号

    public String saleTime;//销售时间



    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @return WebAPI 请求
     */
    public static Request<PayCompleteRequestData> requestJNYZ(String saleNo, HashMap<String, String> zfMap, String operator,String saleTime) {
        PayCompleteRequestData data = new PayCompleteRequestData();
        data.cshCode = operator;
        data.saleNo = saleNo;
        data.pay = new PayData();
        data.pay.payCode = zfMap.get("paycode");
        data.pay.payName = zfMap.get("payname");
        data.pay.payNo = zfMap.get("zfno");
        data.pay.totalPaid = zfMap.get("total");
        data.saleTime=saleTime;

        return WebApi.sign(data);
    }


    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @return WebAPI 请求
     */
    public static Request<PayCompleteRequestData> request(String saleNo, HashMap<String, String> zfMap, String operator) {
        PayCompleteRequestData data = new PayCompleteRequestData();
        data.cshCode = operator;
        data.saleNo = saleNo;
        data.pay = new PayData();
        data.pay.payCode = zfMap.get("paycode");
        data.pay.payName = zfMap.get("payname");
        data.pay.payNo = zfMap.get("zfno");
        data.pay.totalPaid = zfMap.get("total");

        return WebApi.sign(data);
    }

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @return WebAPI 请求
     */
    public static Request<PayCompleteRequestData> requestYh(String saleNo, HashMap<String, String> zfMap, String operator) {
        PayCompleteRequestData data = new PayCompleteRequestData();
        data.cshCode = operator;
        data.saleNo = saleNo;
        data.pay = new PayData();
        data.pay.payCode = zfMap.get("paycode");
        data.pay.payName = zfMap.get("payname");
        if(zfMap.get("traceNo")!=null&&zfMap.get("traceNo").length()>0) {
            data.pay.payNo = zfMap.get("traceNo");
        }
        data.pay.totalPaid = zfMap.get("total");
        data.pay.remark=zfMap.get("refereceNo");

        return WebApi.sign(data);
    }

    /**
     * 付款完成接口-针对海信-JAVA中间件
     * 返回一个新的 WebAPI 请求。
     *
     * @return WebAPI 请求
     */
    public static Request<PayCompleteRequestData> requestHisense(String saleNo, HashMap<String, String> zfMap, String operator) {
        PayCompleteRequestData data = new PayCompleteRequestData();
        data.cshCode = operator;
        data.saleNo = RSAService.encrypt(saleNo);
        data.pay = new PayData();
        data.pay.payCode = zfMap.get("paycode");
        data.pay.payName = zfMap.get("payname");
        data.pay.payNo = RSAService.encrypt(zfMap.get("zfno"));
        data.pay.totalPaid = RSAService.encrypt(zfMap.get("total"));

        return WebApi.sign(data);
    }


    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @return WebAPI 请求
     */
    public static Request<PayCompleteRequestData> requestJLF(String saleNo, HashMap<String, String> zfMap, String operator) {
        PayCompleteRequestData data = new PayCompleteRequestData();
        data.cshCode = operator;
        data.posNo = Context.posNo();
        data.saleNo = saleNo;
        data.pay = new PayData();
        data.pay.payCode = zfMap.get("paycode");
        data.pay.payName = zfMap.get("payname");
        data.pay.payNo = zfMap.get("zfno");
        data.pay.totalPaid = zfMap.get("total");

        return WebApi.sign(data);
    }
}
