/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.pay;

import com.hisense.common.util.StringUtils;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;


/**
 * 下单支付请求数据
 *
 * @author wangyaogang 2018-11-27
 * @author zhangsai    2018-12-19 HIPOSAPP-698 交易商品类型改为 ISaleItem
 * @author wangyaogang 2019-03-05 HIPOSAPP-1032 海信Pay配置为石基支付时,支持撤销和退款
 * @author wangyaogang 2019-05-09 HIPOSAPP-1086 支持银联扫码支付
 * @author wangyaogang 2019-05-23 HIPOSAPP-1123 请求海信支付时增加交易查询避免网络问题导致单边账
 * @author wangyaogang 2019-08-08 HIPOSAPP-1259 对接海信支付退款结果查询接口
 */

class PayRequestData {
    /**
     * 支付付款码
     */
    public String authCode;
    /**
     * 用户标识
     */
    public String openid;
    /**
     * 商品pos交易订单号
     */
    public String billNo;
    /**
     * 收款员编码
     */
    public String cshCode;
    /**
     * 可打折金额
     */
    public String dscAmount;
    /**
     * 组织编码
     */
    public String orgCode;

    public String tradeNo;
    /**
     * 商户订单号
     */
    public String outTradeNo;
    /**
     * 支付方式
     */
    public String payMethod;
    /**
     * 交易商品
     */
    public List<PayRequestPlu> plu;
    /**
     * 收款机号
     */
    public String posNo;

    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 销售时间 格式：yyyy-MM-dd HH:mm:ss
     */
    public String saleTime;
    /**
     * 支付场景
     */
    public String scene;
    /**
     * 订单标题
     */
    public String subject;
    /**
     * 订单总金额
     */
    public String total;
    /**
     * 不可打折金额
     */
    public String undscAmount;

    /**
     * 机具管控参数
     * 支付宝支付，机具管控参数
     */
    public String terminalParams;

    /**
     * 退款请求号
     * 使用RSA加密.
     * 一笔交易多次退款，传入不同的退款请求号
     */
    public String refundOutTradeNo;

    /**
     * 退款时间
     */
    public String refundTime;

    /**
     * 退款金额
     * 使用RSA加密.
     * 需要退款的金额，该金额不能大于订单金额,单位为分
     */
    public String totalRefund;

    /**
     * 终端号
     */
    public String terminalId;

    /**
     * 一点通券号
     */
    public String couponNo;

    /**
     * 返回一个新的 WebAPI 请求。(支付）
     *
     * @return WebAPI 请求
     */
    public static Request<PayRequestData> request(String authCode, String openid, String outTradeNo, String totalPaid, String payRemark, String saleTime,
                                                  String scene, String subject, String terminalParams, String terminalId) {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();

        PayRequestData data = new PayRequestData();
        data.authCode = RSAService.encrypt(authCode);
        if (!StringUtils.isEmpty(openid)) {
            data.openid = RSAService.encrypt(openid);
        }
        data.cshCode = Context.cashier().code();
        data.dscAmount = RSAService.encrypt(saleModel.getTotalUnPaid());
        data.orgCode = Context.orgCode();
        //data.orgCode = "AAAA";
        data.outTradeNo = RSAService.encrypt(outTradeNo);
        data.payMethod = payRemark;
        data.plu = new ArrayList<>();
        for (BaseSalePluModel pluModel : saleModel.getPlus()) {
            PayRequestPlu pluSingle = new PayRequestPlu();
            pluSingle.barCode = pluModel.getBarCode();
            pluSingle.classCode = pluModel.getClassCode();
            pluSingle.itemNo = pluModel.getItemNo();
            pluSingle.pageNo = pluModel.getPageNo();
            pluSingle.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluSingle.pluName = pluModel.getPluName();
            pluSingle.price = RSAService.encrypt(String.valueOf(pluModel.getPrice()));
            pluSingle.quantity = RSAService.encrypt(String.valueOf(pluModel.getQuantity()));
            pluSingle.total = RSAService.encrypt(String.valueOf(pluModel.getTotalRecv()));
            data.plu.add(pluSingle);
        }

        data.posNo = Context.posNo();
        data.saleNo = RSAService.encrypt(saleModel.getSaleNo());
        data.saleTime = saleTime;
        data.scene = scene;
        data.subject = subject;
        data.total = RSAService.encrypt(totalPaid);
        data.terminalParams = terminalParams;
        data.undscAmount = RSAService.encrypt("0");
        data.terminalId = terminalId;

        return WebApi.sign(data);
    }

    /**
     * 返回一个新的 WebAPI 请求。（查询）
     *
     * @return WebAPI 请求
     */
    public static Request<PayRequestData> requestQuery(String outTradeNo, String payRemark, String scene, String saleTime, String terminalId) {
        PayRequestData data = new PayRequestData();
        data.cshCode = Context.cashier().code();
        data.orgCode = Context.orgCode();
//        data.orgCode = "AAAA";
        data.outTradeNo = RSAService.encrypt(outTradeNo);
        data.payMethod = payRemark;
        data.posNo = Context.posNo();
        data.saleNo = RSAService.encrypt(MainSaleModel.getInstance().getSaleModel().getSaleNo());
        data.scene = scene;
        data.saleTime = saleTime;
        data.terminalId = terminalId;

        return WebApi.sign(data);
    }

    /**
     * 返回一个新的 WebAPI 请求。（撤销）
     *
     * @return WebAPI 请求
     */
    public static Request<PayRequestData> requestCancel(SalePayModel payModel, String saleNo, String terminalId) {
        PayRequestData data = new PayRequestData();
        data.cshCode = Context.cashier().code();
        data.orgCode = Context.orgCode();
//        data.orgCode = "AAAA";
        data.outTradeNo = RSAService.encrypt(payModel.getOutTradeNo());
        data.payMethod = payModel.getRemark();
        data.posNo = Context.posNo();
        data.saleNo = RSAService.encrypt(saleNo);
        data.saleTime = payModel.getPayTime();
        data.terminalId = terminalId;
        data.total = RSAService.encrypt(payModel.getTotalRecv());

        return WebApi.sign(data);
    }


    /**
     * 返回一个新的 WebAPI 请求。（退款）
     *
     * @return WebAPI 请求
     */
    public static Request<PayRequestData> requestRefund(String couponNo, String outTradeNo, ReturnSaleModel sale,
                                                        String payRemark, String refundOutTradeNo,
                                                        String total, String totalRefund,
                                                        String terminalId) {
        PayRequestData data = new PayRequestData();

        data.cshCode = Context.cashier().code();
        data.couponNo = couponNo;
        data.orgCode = Context.orgCode();
//        data.orgCode = "AAAA";
        data.outTradeNo = RSAService.encrypt(outTradeNo);
        data.payMethod = payRemark;

        data.plu = new ArrayList<>();
        for (BaseSalePluModel pluModel : sale.getPlus()) {
            PayRequestPlu pluSingle = new PayRequestPlu();
            pluSingle.barCode = pluModel.getBarCode();
            pluSingle.classCode = pluModel.getClassCode();
            pluSingle.itemNo = pluModel.getItemNo();
            pluSingle.pageNo = pluModel.getPageNo();
            pluSingle.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluSingle.pluName = pluModel.getPluName();
            pluSingle.price = RSAService.encrypt(String.valueOf(pluModel.getPrice()));
            pluSingle.quantity = RSAService.encrypt(String.valueOf(pluModel.getQuantity()));
            pluSingle.total = RSAService.encrypt(String.valueOf(pluModel.getTotalRecv()));
            data.plu.add(pluSingle);
        }

        data.posNo = Context.posNo();
        data.refundOutTradeNo = RSAService.encrypt(refundOutTradeNo);
        data.refundTime = sale.getSaleTime();
        data.saleNo = RSAService.encrypt(sale.oldSaleNo());
        data.saleTime = sale.getSaleModel().getSaleTime();
        data.terminalId = terminalId;
        data.total = RSAService.encrypt(total);
        data.totalRefund = RSAService.encrypt(totalRefund);

        return WebApi.sign(data);
    }

    /**
     * 返回一个新的 WebAPI 请求。（退款）
     *
     * @return WebAPI 请求
     */
    public static Request<PayRequestData> requestRefund(String couponNo, String tradeNo, String outTradeNo, ReturnSaleModel sale,
                                                        String payRemark, String refundOutTradeNo,
                                                        String total, String totalRefund,
                                                        String terminalId) {
        PayRequestData data = new PayRequestData();

        data.cshCode = Context.cashier().code();
        data.couponNo = couponNo;
        data.orgCode = Context.orgCode();
//        data.orgCode = "AAAA";
        data.tradeNo = tradeNo;
        data.outTradeNo = RSAService.encrypt(outTradeNo);
        data.payMethod = payRemark;

        data.plu = new ArrayList<>();
        for (BaseSalePluModel pluModel : sale.getPlus()) {
            PayRequestPlu pluSingle = new PayRequestPlu();
            pluSingle.barCode = pluModel.getBarCode();
            pluSingle.classCode = pluModel.getClassCode();
            pluSingle.itemNo = pluModel.getItemNo();
            pluSingle.pageNo = pluModel.getPageNo();
            pluSingle.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluSingle.pluName = pluModel.getPluName();
            pluSingle.price = RSAService.encrypt(String.valueOf(pluModel.getPrice()));
            pluSingle.quantity = RSAService.encrypt(String.valueOf(pluModel.getQuantity()));
            pluSingle.total = RSAService.encrypt(String.valueOf(pluModel.getTotalRecv()));
            data.plu.add(pluSingle);
        }

        data.posNo = Context.posNo();
        data.refundOutTradeNo = RSAService.encrypt(refundOutTradeNo);
        data.refundTime = sale.getSaleTime();
        data.saleNo = RSAService.encrypt(sale.oldSaleNo());
        data.saleTime = sale.getSaleModel().getSaleTime();
        data.terminalId = terminalId;
        data.total = RSAService.encrypt(total);
        data.totalRefund = RSAService.encrypt(totalRefund);

        return WebApi.sign(data);
    }

    public static Request<PayRequestData> requestRefundQuery(String outTradeNo, ReturnSaleModel sale,
                                                             String payRemark, String refundOutTradeNo,
                                                             String terminalId, String total, String totalRefund) {
        PayRequestData data = new PayRequestData();

        data.cshCode = Context.cashier().code();
        data.orgCode = Context.orgCode();
//        data.orgCode ="AAAA";
        data.outTradeNo = RSAService.encrypt(outTradeNo);
        data.refundOutTradeNo = RSAService.encrypt(refundOutTradeNo);
        data.payMethod = payRemark;
        data.posNo = Context.posNo();
        data.saleNo = RSAService.encrypt(sale.oldSaleNo());
        data.saleTime = sale.getSaleModel().getSaleTime();
        data.refundTime = sale.getSaleTime();
        data.terminalId = terminalId;
        data.total = RSAService.encrypt(total);
        data.totalRefund = RSAService.encrypt(totalRefund);
        return WebApi.sign(data);
    }
}
