/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.pay;


import java.util.ArrayList;

import lombok.Data;

/**
 * 支付返回结果
 *
 * @author wangyaogang 2018-11-27
 * @author guolina     2018-09-26 HIPOSAPP-839 打印支持聚合支付优惠
 * @author guolina     2020-05-18 HIAPOSCMP-702 吉麦隆淘鲜达营销方案对接涉及慧收银调整增加支付明细和赠券打印
 */
@Data
public class PayResultData {
    /**
     * 卖家支付账号
     */
    public String buyerLogonId;
    /**
     * 卖家支付账号id
     */
    public String buyerUserId;
    /**
     * 商户订单号
     */
    public String outTradeNo;
    /**
     * 支付方式编码
     */
    public String payCode;
    /**
     * 支付方式名称
     */
    public String payName;
    /**
     * 销售日期
     */
    public String saleTime;
    /**
     * 订单总金额
     */
    public String total;
    /**
     * 商户优惠金额
     */
    public String totalDscSelf;
    /**
     * 平台优惠金额
     */
    public String totalDscChannel;
    /**
     * 实收金额
     */
    public String totalRecv;
    /**
     * 结算金额
     */
    public String totalSettle;
    /**
     * 平台订单号
     */
    public String tradeNo;

    /**
     * 退款总金额
     */
    public String totalRefund;

    /**
     * 支付信息
     */
    public ArrayList<PayData> pay;

    /**
     * 券信息
     */
    public ArrayList<CouponData> coupon;


    /**
     * 处理 WebAPI 接口返回数据。(支付)
     *
     * @param resultData 接口返回数据
     * @return 支付结果信息
     */
    public static PayResultData handleResult(PayResultData resultData) {
        PayResultData payResultData = new PayResultData();
        payResultData.buyerLogonId = resultData.buyerLogonId;
        payResultData.buyerUserId = resultData.buyerUserId;
        payResultData.outTradeNo = resultData.outTradeNo;
        payResultData.payCode = resultData.payCode;
        payResultData.payName = resultData.payName;
        payResultData.saleTime = resultData.saleTime;
        payResultData.total = resultData.total;
        payResultData.totalDscSelf = resultData.totalDscSelf == null ? "0" : resultData.totalDscSelf;
        payResultData.totalDscChannel = resultData.totalDscChannel == null ? "0" : resultData.totalDscChannel;
        payResultData.totalRecv = resultData.totalRecv;
        payResultData.totalSettle = resultData.totalSettle;
        payResultData.tradeNo = resultData.tradeNo;

        payResultData.pay = new ArrayList<>();
        payResultData.coupon = new ArrayList<>();

        if(null != resultData.getPay()){
            for(PayData payData1:resultData.getPay()) {
                PayData payData = new PayData();
                payData.payCode = payData1.payCode;
                payData.payName = payData1.payName;
                payData.totalPaid = payData1.totalPaid;

                payResultData.pay.add(payData);
            }
        }

        if(null != resultData.getCoupon()) {
            for(CouponData couponData1:resultData.getCoupon()) {
                CouponData couponData = new CouponData();
                couponData.type = couponData1.type;
                couponData.value = couponData1.value;
                couponData.tag = couponData1.tag;
                couponData.align = couponData1.align;
                couponData.outTradeNo = payResultData.outTradeNo;

                payResultData.coupon.add(couponData);
            }
        }


//        CouponData couponData2 = new CouponData();
//        couponData2.type = "code";
//        couponData2.value = "httpasdhflajdla";
//        payResultData.coupon.add(couponData2);
//
//        CouponData couponData3 = new CouponData();
//        couponData3.type = "line";
//        couponData3.tag = "dash";//couponData1.tag;
//        payResultData.coupon.add(couponData3);
//
//        CouponData couponData4 = new CouponData();
//        couponData4.type = "line";
//        couponData4.tag = "solid";//couponData1.tag;
//        payResultData.coupon.add(couponData4);
//
//        CouponData couponData5 = new CouponData();
//        couponData5.type = "line";
//        couponData5.tag = "empty";//couponData1.tag;
//        payResultData.coupon.add(couponData5);
//
//        CouponData couponData6 = new CouponData();
//        couponData6.type = "line";
//        couponData6.tag = "solid";//couponData1.tag;
//        payResultData.coupon.add(couponData6);
//
//        CouponData couponData1 = new CouponData();
//        couponData1.type = "text";
//        couponData1.value = "恭喜发财发财发大财呀";
//        couponData1.tag = "2";//couponData1.tag;
//        couponData1.align = "center";
//        payResultData.coupon.add(couponData1);
        return payResultData;
    }

    /**
     * 处理 WebAPI 接口返回数据。(退款)
     *
     * @param resultData 接口返回数据
     * @return 退款结果信息
     */
    public static PayResultData handleRefundResult(PayResultData resultData) {
        PayResultData payResultData = new PayResultData();
        payResultData.buyerLogonId = resultData.buyerLogonId;
        payResultData.buyerUserId = resultData.buyerUserId;
        payResultData.outTradeNo = resultData.outTradeNo;
        payResultData.payCode = resultData.payCode;
        payResultData.payName = resultData.payName;
        payResultData.saleTime = resultData.saleTime;
        payResultData.totalDscSelf = resultData.totalDscSelf == null ? "0" : resultData.totalDscSelf;
        payResultData.totalDscChannel = resultData.totalDscChannel == null ? "0" : resultData.totalDscChannel;
        payResultData.totalRefund = resultData.totalRefund;
        payResultData.totalSettle = resultData.totalSettle;
        payResultData.tradeNo = resultData.tradeNo;
        return payResultData;
    }


    @Override
    public String toString() {
        return "PayResultData{" +
                "buyerLogonId='" + buyerLogonId + '\'' +
                ", buyerUserId='" + buyerUserId + '\'' +
                ", outTradeNo='" + outTradeNo + '\'' +
                ", payCode='" + payCode + '\'' +
                ", payName='" + payName + '\'' +
                ", saleTime='" + saleTime + '\'' +
                ", total='" + total + '\'' +
                ", totalDscSelf='" + totalDscSelf + '\'' +
                ", totalDscChannel='" + totalDscChannel + '\'' +
                ", totalRecv='" + totalRecv + '\'' +
                ", totalSettle='" + totalSettle + '\'' +
                ", tradeNo='" + tradeNo + '\'' +
                ", totalRefund='" + totalRefund + '\'' +
                '}';
    }
}
