/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.plu;

import com.hisense.crm.vip.Vip;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.plu.PluV2;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;

/**
 * 查询商品优惠的请求数据。
 *
 * @author zhangsai    2017-09-03
 * @author zhangsai    2018-01-08 增加生成 WebAPI 请求的工厂方法。
 * @author guolina     2019-01-09 HIPOSAPP-916 查询商品优惠接口调整
 * @author wangyaogang 2020-04-10 HIAPOSCMP-660 会员促销接口增加vipCardNo，barCode
 */
class PluDiscountRequestData {
    /**
     * 会员卡类型编码
     */
    public String cardType;
    /**
     * 会员卡号
     */
    public String vipCardNo;
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 交易商品
     */
    public ArrayList<PluV2> plu;
    /**
     * 收款机号
     */
    public String posNo;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 专柜ID
     */
    public String shopId;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param shopId    专柜 ID
     * @param vip       会员卡
     * @param pluModels 商品
     * @return WebAPI 请求
     */
    public static Request<PluDiscountRequestData> request(String shopId, String saleNo, Vip vip, List<BaseSalePluModel> pluModels) {
        PluDiscountRequestData data = new PluDiscountRequestData();
        data.orgCode = Context.orgCode();
        data.shopId = shopId;
        data.cardType = vip.card().cardType().code();
        data.vipCardNo = vip.card().vipCardNo();
        data.posNo = Context.posNo();
        data.saleNo = RSAService.encrypt(saleNo);

        data.plu = new ArrayList<>();

        for (BaseSalePluModel pluModel : pluModels) {
            PluV2 pluData = new PluV2();
            pluData.barCode = pluModel.getBarCode();
            pluData.brandCode = pluModel.getBrandCode();
            pluData.classCode = pluModel.getClassCode();
            pluData.itemNo = pluModel.getItemNo();
            pluData.pageNo = pluModel.getPageNo();
            pluData.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluData.pluId = RSAService.encrypt(pluModel.getPluId());
            pluData.price = RSAService.encrypt(pluModel.getPrice());
            pluData.quantity = RSAService.encrypt(pluModel.getQuantity());
            pluData.tagPrice = RSAService.encrypt(pluModel.getTagPrice());
            pluData.vipPrice = pluModel.getVipPrice();
            pluData.total = RSAService.encrypt(pluModel.getTotal());

            data.plu.add(pluData);
        }

        return WebApi.sign(data);
    }
}
