/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.plu;

import com.hisense.common.util.StringUtils;
import com.hisense.crm.vip.Vip;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

/**
 * 查询商品的请求数据。
 *
 * @author zhangsai    2017-09-02
 * @author zhangsai    2018-01-08 增加生成 WebAPI 请求的工厂方法。
 * @author wangyaogang 2018-12-06 HIPOSAPP-885 商品查询接口调整
 * @author wangyaogang 2018-12-17 HIPOSAPP-911 属性由私有改为公有
 * @author guolina     2020-02-27 HIAPOSCMP-553 与中台联调商品查询和商品促销计算接口
 * @author wangyaogang 2020-04-10 HIAPOSCMP-660 流水查询接口增加vipCardNo
 * @author guolina     2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 */
class PluQueryRequestData {
    /**
     * 组织编码
     */
    public String orgCode;
    /**
     * 商品编码、条码、货号
     */
    public String pluCode;
    /**
     * 专柜编码
     */
    public String shopCode;
    /**
     * 专柜编码中台
     */
    public String depCode;
    /**
     * 会员卡类型编码
     */
    public String cardType;
    /**
     * 会员卡号
     */
    public String vipCardNo;
    /**
     * 收款机号
     */
    public String posNo;
    /**
     * 流水号
     */
    public String saleNo;
    /**
     * 促销类型
     */
    public String proType;
    /**
     * 促销计算方式  0 – 返回不包含促销 1 – 返回包含促销
     */
    public String queryType;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param pluCode  商品编码
     * @param shopCode 专柜编码
     * @return WebAPI 请求
     */
    public static Request<PluQueryRequestData> request(String pluCode, String shopCode) {
        PluQueryRequestData data = new PluQueryRequestData();
        data.orgCode = Context.orgCode();
        if (WebApi.isAmwAndAip()) {
            data.depCode = shopCode;
        } else {
            data.shopCode = shopCode;
        }


        data.pluCode = pluCode;

        return WebApi.sign(data);
    }

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param pluCode  商品编码
     * @param shopCode 专柜编码
     * @return WebAPI 请求
     */
    public static Request<PluQueryRequestData> request(String queryType, String saleNo, Vip vip, String pluCode, String shopCode) {
        PluQueryRequestData data = new PluQueryRequestData();
        data.queryType = queryType;
        if (!StringUtils.isEmpty(saleNo)) {
            data.saleNo = RSAService.encrypt(saleNo);
        }
        data.cardType = vip == null ? null : vip.card().cardType().code();
        data.vipCardNo = vip == null ? null : vip.card().vipCardNo();
        data.orgCode = Context.orgCode();
        if (WebApi.isAmwAndAip()) {
            data.depCode = shopCode;
            data.proType = "RPP";
        } else {
            data.shopCode = shopCode;
        }
        data.pluCode = pluCode;
        data.posNo = Context.posNo();
        return WebApi.sign(data);
    }
}
