/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.plu;

import com.hisense.common.boot.GlobalException;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.plu.PluType;
import com.hisense.webapi.WebApi;

/**
 * 查询商品的返回结果。
 *
 * @author zhangsai    2017-09-02
 * @author zhangsai    2018-01-08 增加处理 WebAPI 返回数据的工厂方法。
 * @author zhangsai    2018-02-08 HIPOSAPP-321 支持称重商品
 * @author zhangsai    2018-06-13 HIPOSAPP-519 设置商品的 pluType 商品类型属性
 * @author guolina     2018-12-21 HIPOSAPP-839 小票增加促销打印
 * @author zhangsai    2019-01-07 HIPOSAPP-957 增加会员促销属性
 * @author guolina     2019-01-24 HIPOSAPP-977 条码称重商品支持商品促销和会员促销
 * @author guolina     2019-04-09 HIPOSAPP-1071 增加商品范围设置异常捕获，防止闪退
 * @author wangyaogang 2019-05-14 HIPOSAPP-1094 易收银前台能否销售根据中间件返回的商品状态进行处理,且去掉商品缓存
 * @author wangyaogang 2019-06-05 HIPOSAPP-1154 易收银销售小票-打印商品"货号"字段，初始化时增加货号属性
 * @author wangyaogang 2019-12-13 HIAPOSCMP-429 计件商品不弹框
 * @author xieliyan    2019-12-13 商品包类型更改为string
 */
public class PluQueryResultData {
    /**
     * 货号
     */
    public String cargoNo;
    /**
     * 商品编码
     */
    public String pluCode;
    /**
     * 商品ID
     */
    public String pluId;
    /**
     * 商品名称
     */
    public String pluName;

    public String pluAbbrName;
    /**
     * 助记码
     */
    public String pluMN;
    /**
     * 商品类型
     * 0 - 普通商品
     * 1 - 商品包
     * 2 - 商品明细
     */
    public String pluType;
    /**
     * 条码
     */
    public String barCode;
    /**
     * 品牌编码
     */
    public String brandCode;
    /**
     * 品类编码
     */
    public String classCode;

    /**
     * 部门编码
     */
    public String depCode;
    /**
     * 部门ID
     */
    public String depId;
    /**
     * 部门名称
     */
    public String depName;

    /**
     * 专柜编码
     */
    public String shopCode;
    /**
     * 专柜ID
     */
    public String shopId;
    /**
     * 专柜名称
     */
    public String shopName;

    /**
     * 商品单位
     */
    public String unit;

    public String pluWeight;        //商品重量，单位kg

    /**
     * 规格
     */
    public String spec;

    /**
     * 单价
     */
    public String price;

    public String vipPrice;
    /**
     * 最低价
     */
    public String minPrice;
    /**
     * 最高价
     */
    public String maxPrice;

    /**
     * 是否称重商品
     */
    public int isWeight;

    public String isWeight13;  // 是否是13位称重商品；0-不是；1-是   标识用
    public String isWeight18;  // 是否是18位称重商品；0-不是；1-是   标识用

    public int isCountQuantity;
    /**
     * 商品重量
     */
    public String quantity = "0.00";

    public String weight;

    /**
     * 是否普通优惠
     * 0 - 无优惠
     * 1 - 普通优惠
     */
    public int isNormalDsc;

    /**
     * 是否参与满减促销
     * 0 - 不参与
     * 1 - 参与
     */
    public int isTotalDsc;
    /**
     * 是否允许销售
     * 0 - 不允许
     * 1 - 允许
     */
    public int isCanSale;
    /**
     * 是否允许退货
     * 0 - 不允许
     * 1 - 允许
     */
    public int isCanReturn;
    /**
     * 优惠类型
     * 0 - 优惠率
     * 1 - 优惠价格
     */
    public String dscType;
    /**
     * 促销单号
     */
    public String dscCode;
    /**
     * 优惠率
     */
    public String dscRate;
    /**
     * 优惠价格
     */
    public String dscPrice;

    public String dscProName;

    public String dscProType;

    public String dscProPrice;      //优惠后单价，即发生价

    /**
     * 商品金额（实付金额）
     */
    public String total = "0.00";

    /**
     * 处理 WebAPI 接口返回数据。
     *
     * @param resultData 接口返回数据
     * @return 商品列表
     */
    public static BaseSalePluModel handleResult(PluQueryResultData resultData) throws GlobalException {
        if (resultData.isCanSale == 0) {
            throw new GlobalException("该商品不允许销售");
        }
        if (BigDecimalUtils.isEquals0(resultData.quantity)) {
            throw new GlobalException("商品数量不能为 0");
        }
        try {
            checkPriceRange(resultData.minPrice, resultData.maxPrice, resultData.price);
        } catch (Exception e) {
            throw new GlobalException(e.getMessage());
        }

        BaseSalePluModel pluModel = new BaseSalePluModel();
        pluModel.setBarCode(resultData.barCode);
        pluModel.setPluId(resultData.pluId);
        pluModel.setPluCode(resultData.pluCode);
        pluModel.setPluName(resultData.pluName);
        pluModel.setPluMn(resultData.pluMN);
        pluModel.setPluType(PluType.valueOfDefined(resultData.pluType));
        pluModel.setBrandCode(resultData.brandCode);
        pluModel.setClassCode(resultData.classCode);
        pluModel.setDepCode(resultData.depCode);
        pluModel.setDepId(resultData.depId);
        pluModel.setDepName(resultData.depName);
        if (WebApi.isAmwAndAip()) {
            pluModel.setShopCode(resultData.depCode);
            pluModel.setShopId(resultData.depId);
            pluModel.setShopName(resultData.depName);
        } else {
            pluModel.setShopCode(resultData.shopCode);
            pluModel.setShopId(resultData.shopId);
            pluModel.setShopName(resultData.shopName);
        }

        pluModel.setTagPrice(resultData.price);
        pluModel.setPrice(resultData.dscProPrice);
        pluModel.setVipPrice(resultData.vipPrice);
        pluModel.setMinPrice(resultData.minPrice);
        pluModel.setMaxPrice(resultData.maxPrice);
        pluModel.setQuantity(resultData.quantity);
        pluModel.setWeight(resultData.weight);
        pluModel.setSpec(resultData.spec);
        pluModel.setUnit(resultData.unit);
        pluModel.setPluWeight(resultData.pluWeight);
        pluModel.setTotal(resultData.total);
        pluModel.setIsNormalDsc(resultData.isNormalDsc);
        pluModel.setIsTotalDsc(resultData.isTotalDsc);
        pluModel.setIsWeight(resultData.isWeight);

        if (WebApi.isAmwAndAip() && pluModel.isWeight()) {
            pluModel.setWeightNoCode(false);
        } else {
            pluModel.setWeightNoCode(!("1".equals(resultData.isWeight13) || "1".equals(resultData.isWeight18)));
        }

        pluModel.setIsCountQuantity(resultData.isCountQuantity);
        pluModel.setCargoNo(resultData.cargoNo);
        pluModel.setPluAbbrName(resultData.pluAbbrName);

        if (resultData.isNormalDsc == 1) {
            SalePluDscModel pluDscModel = new SalePluDscModel();
            pluDscModel.setDscType(resultData.dscType);
            pluDscModel.setDscCode(resultData.dscCode);
            pluDscModel.setDscNum(resultData.quantity);
            pluDscModel.setDscProType(resultData.dscProType);
            pluDscModel.setDscProName(resultData.dscProName);
            pluDscModel.setDscPrice(resultData.dscProPrice);
            pluDscModel.setDscTotal(resultData.dscPrice);
            pluDscModel.setDscRate(resultData.dscRate);

            pluModel.addPluDsc(pluDscModel);
        } else {
            pluModel.calcTotalDsc();
        }

        return pluModel;
    }

    private static void checkPriceRange(String minPrice, String maxPrice, String price) {
        if (BigDecimalUtils.isLess0(minPrice)) {
            throw new IllegalArgumentException("minPrice 不能小于零");
        }
        if (BigDecimalUtils.isLess0(maxPrice)) {
            throw new IllegalArgumentException("maxPrice 不能小于零");
        }
//        if (BigDecimalUtils.isGreater(minPrice, maxPrice)) {
//            throw new IllegalArgumentException("minPrice 不能大于 maxPrice");
//        }
//        if (BigDecimalUtils.isGreater(minPrice, price)) {
//            throw new IllegalArgumentException("minPrice 不能大于 price");
//        }
//        if (BigDecimalUtils.isGreater0(maxPrice) && BigDecimalUtils.isLess(maxPrice, price)) {
//            throw new IllegalArgumentException("maxPrice 不能小于 price");
//        }
    }
}
