/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.report;

import com.hisense.hipos.model.SalePluModel;
import com.hisense.hipos.report.SaleReport;
import com.hisense.hipos.report.ShopSaleReport;
import com.hisense.hipos.report.ShopSaleReportItem;
import com.hisense.hipos.sale.TranType;
import com.hisense.webapi.Callback;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Response;

/**
 * 报表查询服务。
 *
 * @author zhangsai    2017-09-06
 * @author zhangsai    2018-12-28 HIPOSAPP-946 新增专柜报表；及报表异步查询方法
 * @author wangyaogang 2019-09-25 HIPOSAPP-1292 增加部门报表和商品销售汇总报表
 * @author wangyaogang 2019-10-17 HIPOSAPP-1324 全场收银时,shopcode传 *
 */
public class ReportService {

    private ReportService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回指定查询日期的销售报表。
     *
     * @param queryDate 查询日期。
     * @param tranType  交易类型
     * @param callback  查询销售报表的回调方法。
     */
    public static void sale(final String queryDate, final TranType tranType, final Callback<SaleReport> callback) {
        Request<SaleReportRequestData> request = SaleReportRequestData
                .request(queryDate, tranType);

        Call<Result<SaleReportResultData>> call;
        if (tranType.equals(TranType.Sale)) {
            call = WebApi.create(IReportService.class).sale(request);
        } else {
            call = WebApi.create(IReportService.class).presale(request);
        }

        WebApi.requestAsync(call, new retrofit2.Callback<Result<SaleReportResultData>>() {
            @Override
            public void onResponse(Call<Result<SaleReportResultData>> call, Response<Result<SaleReportResultData>> response) {
                callback.onSuccess(SaleReportResultData.handlerResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<SaleReportResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },SaleReportResultData.class);
    }

    /**
     * 返回指定查询日期的收银员报表。
     *
     * @param cshCode   收款员编码。
     * @param queryDate 查询日期。
     * @param callback  查询收银员报表的回调方法。
     */
    public static void cashier(final String cshCode, final String queryDate, final Callback<SaleReport> callback) {
        Request<SaleReportRequestData> request = SaleReportRequestData
                .request(cshCode, queryDate, TranType.PreSale);

        WebApi.requestAsync(WebApi.create(IReportService.class).cashier(request), new retrofit2.Callback<Result<SaleReportResultData>>() {
            @Override
            public void onResponse(Call<Result<SaleReportResultData>> call, Response<Result<SaleReportResultData>> response) {
                callback.onSuccess(SaleReportResultData.handlerResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<SaleReportResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },SaleReportResultData.class);
    }

    /**
     * 返回指定查询日期的专柜报表。
     *
     * @param queryDate 查询日期。
     * @param callback  查询专柜报表的回调方法。
     */
    public static void shop(final String queryDate, final Callback<ShopSaleReport> callback) {
        Request<ShopSaleReportRequestData> request = ShopSaleReportRequestData.request(queryDate);

        WebApi.requestAsync(WebApi.create(IReportService.class).shop(request), new retrofit2.Callback<Result<List<ShopSaleReportResultItem>>>() {
            @Override
            public void onResponse(Call<Result<List<ShopSaleReportResultItem>>> call, Response<Result<List<ShopSaleReportResultItem>>> response) {
                List<ShopSaleReportItem> items = new ArrayList<>();
                for (ShopSaleReportResultItem item : response.body().data) {

                    SalePluModel pluModel = new SalePluModel();
                    pluModel.setPluCode(item.pluCode);
                    pluModel.setPluName(item.pluName);

                    items.add(new ShopSaleReportItem(pluModel, Double.valueOf(item.totalRecv)));
                }
                callback.onSuccess(new ShopSaleReport(items));
            }

            @Override
            public void onFailure(Call<Result<List<ShopSaleReportResultItem>>> call, Throwable t) {
                callback.onError(t);
            }
        },ShopSaleReportResultItem.class);
    }

    /**
     * 返回指定查询日期的商品销售报表。
     *
     * @param bgnDate  查询日期。
     * @param callback 查询专柜报表的回调方法。
     */
    public static void plu(final String bgnDate, String shopCode, final Callback<PluReportResultData> callback) {
        Request<PluReportRequestData> request = PluReportRequestData.request(bgnDate, shopCode);

        WebApi.requestAsync(WebApi.create(IReportService.class).plu(request), new retrofit2.Callback<Result<PluReportResultData>>() {
            @Override
            public void onResponse(Call<Result<PluReportResultData>> call, Response<Result<PluReportResultData>> response) {

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<PluReportResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },PluReportResultData.class);
    }

    /**
     * 返回指定查询日期的商品销售报表。
     *
     * @param bgnDate  查询日期。
     * @param callback 查询专柜报表的回调方法。
     */
    public static void dep(final String bgnDate, String shopCode, final Callback<DepReportResultData> callback) {
        Request<PluReportRequestData> request = PluReportRequestData.request(bgnDate, shopCode);

        WebApi.requestAsync(WebApi.create(IReportService.class).dep(request), new retrofit2.Callback<Result<DepReportResultData>>() {
            @Override
            public void onResponse(Call<Result<DepReportResultData>> call, Response<Result<DepReportResultData>> response) {

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<DepReportResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },DepReportResultData.class);
    }
}
