/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.report;

import com.hisense.hipos.report.SaleReport;

import java.math.BigDecimal;
import java.util.List;

/**
 * 销售报表查询返回结果。
 *
 * @author zhangsai 2017-09-06
 */
class SaleReportResultData {
    /**合计应收金额*/
    public String total;
    /**合计优惠金额*/
    public String totalDsc;
    /**合计实收金额*/
    public String totalRecv;
    /**合计退货金额*/
    public String totalRtn;
    /**总交易笔数*/
    public int totalTrans;
    /**总退货交易笔数*/
    public int totalTransRtn;

    /**支付信息*/
    public List<SaleReportResultPayment> payments;

    /**
     * 处理 WebAPI 接口返回数据。
     * @param resultData 接口返回数据
     * @return 身份验证结果
     */
    public static SaleReport handlerResult(SaleReportResultData resultData) {
        double totalRtn = Double.valueOf(resultData.totalRtn);
        double totalRecv = calcTotalPaid(Double.valueOf(resultData.totalRecv), totalRtn);
        SaleReport report = new SaleReport.Builder(resultData.totalTrans, Double.valueOf(resultData.total))
                .discount(Double.valueOf(resultData.totalDsc))
                .received(totalRecv)
                .returned(resultData.totalTransRtn, totalRtn)
                .build();

        for (SaleReportResultPayment payment: resultData.payments) {
            double totalPaymentRtn = Double.valueOf(payment.totalRtn);
            double totalPaymentPaid = calcTotalPaid(Double.valueOf(payment.totalPaid), totalPaymentRtn);
            report.addPayment(new com.hisense.hipos.report.SaleReportPayment.Builder(payment.payCode, payment.payName)
                    .paid(totalPaymentPaid)
                    .received(Double.valueOf(payment.totalRecv))
                    .returned(totalPaymentRtn)
                    .build());
        }

        return report;
    }

    /**
     * 计算合计支付金额。
     *
     * @param totalPaid 支付金额（已减去退货）
     * @param totalRtn  退货金额
     */
    private static double calcTotalPaid(double totalPaid, double totalRtn) {
        // 退货金额为负数，此处使用减法计算合计支付金额
        return new BigDecimal(totalPaid).subtract(new BigDecimal(totalRtn)).doubleValue();
    }
}
