/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.sale;

import com.hisense.webapi.Request;
import com.hisense.webapi.Result;

import java.util.List;

import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.POST;

/**
 * 交易服务的 retrofit 接口。
 *
 * @author zhangsai 2017-09-03
 * @author zhangsai 2018-01-24 HIPOSAPP-302 增加日结接口
 * @author zhangsai 2018-02-24 HIPOSAPP-334 增加流水查询接口
 * @author zhangsai 2018-06-15 HIPOSAPP-538 增加满送赠券接口
 * @author zhangsai 2018-06-20 HIPOSAPP-565 增加满减促销接口
 * @author zhangsai 2018-07-26 HIPOSAPP-698 增加流水上传接口
 * @author guolina  2018-08-10 HIPOSAPP-715 流水查询界面添加扫描核销功能
 * @author guolina  2018-12-25 HIPOSAPP-850 获取新流水号兼容POS中间件,规则可以在中间件设置
 * @author guolina  2020-01-09 HIAPOSCMP-490 慧收银增加整单促销接口
 */
interface ISaleService {

    /**
     * 返回日结的 retrofit 调用。
     *
     * @param request 日结请求
     * @return 日结的 retrofit 调用
     */
    @POST("api/sale/accounting")
    Call<Result<String>> accounting(@Body Request<AccountingRequestData> request);

    /**
     * 返回当前结账日期的 retrofit 调用。
     *
     * @param request 获取当前结账日期的请求
     * @return 获取当前结账日期的 retrofit 调用
     */
    @POST("api/sale/lastAccountingDate")
    Call<Result<AccountingDateResultData>> lastAccountingDate(@Body Request<AccountingDateRequestData> request);

    /**
     * 返回流水查询的 retrofit 调用。
     *
     * @param request 流水查询请求
     * @return 流水查询的 retrofit 调用
     */
    @POST("api/sale/query")
    Call<Result<SaleQueryResultData>> query(@Body Request<SaleQueryRequestData> request);

    /**
     * 返回流水查询的 retrofit 调用。(归集接口)
     *
     * @param request 流水查询请求
     * @return 流水查询的 retrofit 调用
     */
    @POST("api/rpt/saleQuery")
    Call<Result<SaleQueryResultData>> rptQuery(@Body Request<RptSaleQueryRequestData> request);

    /**
     * 开始交易
     *
     * @param request 开始交易请求
     * @return 开始交易的 retrofit 调用
     */
    @POST("api/sale/begin")
    Call<Result<SaleBeginResultData>> begin(@Body Request<SaleBeginRequestData> request);

    /**
     * 取消交易
     *
     * @param request 取消交易请求
     * @return 取消交易的 retrofit 调用
     */
    @POST("api/sale/cancel")
    Call<Result<Object>> cancel(@Body Request<SaleCancelRequestData> request);

    /**
     * 返回交易满送赠券的 retrofit 调用。
     *
     * @param request 满送赠券请求
     * @return 交易满送赠券的 retrofit 调用
     */
    @POST("api/sale/rebate")
    Call<Result<List<SaleRebateResultCoupon>>> rebate(@Body Request<SaleRebateRequestData> request);

    /**
     * 满减接口
     *
     * @param request 满减请求
     * @return 满减的 retrofit 调用
     */
    @POST("api/sale/totalreduction")
    Call<Result<TotalReductionResultData>> totalReduction(@Body Request<TotalReductionRequestData> request);

    /**
     * 整单促销接口
     *
     * @param request 整单促销请求
     * @return 满减的 retrofit 调用
     */
    @POST("api/sale/promotionCalc")
    Call<Result<PromotionCalcResultData>> promotionCalc(@Body Request<PromotionCalcRequestData> request);

    /**
     * 返回退货流水上传的 retrofit 调用。
     *
     * @param request 退货流水上传请求
     * @return 退货流水上传的 retrofit 调用
     */
    @POST("api/sale/return")
    Call<Result<Object>> returns(@Body Request<SaleSubmitRequestData> request);

    /**
     * 返回退货流水上传的 retrofit 调用。(归集接口)
     *
     * @param request 退货流水上传请求
     * @return 退货流水上传的 retrofit 调用
     */
    @POST("api/rpt/refundUpload")
    Call<Result<Object>> rptReturns(@Body Request<SaleSubmitRequestData> request);

    /**
     * 返回交易上传的 retrofit 调用。
     *
     * @param request 交易上传请求
     * @return 交易上传的 retrofit 调用
     */
    @POST("api/sale/submit")
    Call<Result<SaleSubmitResultData>> submit(@Body Request<SaleSubmitRequestData> request);

    /**
     * 返回交易上传的 retrofit 调用。(归集接口)
     *
     * @param request 交易上传请求
     * @return 交易上传的 retrofit 调用
     */
    @POST("api/rpt/saleUpload")
    Call<Result<SaleSubmitResultData>> rptSubmit(@Body Request<SaleSubmitRequestData> request);

    /**
     * 返回交易完成的 retrofit 调用。
     * （可搭配api/sale/promotionCalc 整单促销接口， 用于 限量控制 ）
     *
     * @param request 交易完成请求
     * @return 交易上传的 retrofit 调用
     */
    @POST("api/sale/complete")
    Call<Result<Object>> complete(@Body Request<SaleCompleteRequestData> request);

    /**
     * 全场限量查询接口
     * (用于查询 该方案是否还有可用兑换次数)
     *
     * @param request 全场限量查询请求
     * @return 交易上传的 retrofit 调用
     */
    @POST("api/sale/globalTypeQuery")
    Call<Result<List<GlobalTypeQueryResultData>>> globalTypeQuery(@Body Request<GlobalTypeQueryRequestData> request);

    /**
     * 礼品券详情查询
     */
    @POST("crm/coupon/certiDtl")
    Call<Result<CertiDtlQueryResultData>> giftQuery(@Body Request<CertiDtlQueryRequestData> request);
    /**
     * 礼品券详情查询
     */
    @POST("crm/coupon/certiConsume")
    Call<Result<Object>> consumeCerti(@Body Request<CertiConsumeRequestData> request);
}
