package com.hisense.hipos.webapi.sale;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.model.SalePluModel;
import com.hisense.hipos.plu.Plu;
import com.hisense.hipos.plu.PluDsc;
import com.hisense.hipos.plu.PluGift;
import com.hisense.hipos.plu.PluGiftAction;

import java.util.ArrayList;
import java.util.List;

import lombok.Data;

/**
 * @author guolina          2020-01-09 HIAPOSCMP-490 慧收银增加整单促销接口
 */
@Data
public class PromotionCalcModel {

    private String totalDsc;
    public List<SalePluModel> plu;
    private List<PluGiftAction> pluGiftActions;

    public static PromotionCalcModel handleResult(PromotionCalcResultData resultData) {
        if (resultData == null) {
            return null;
        }

        PromotionCalcModel promotionCalcModel = new PromotionCalcModel();
        promotionCalcModel.totalDsc = resultData.totalDsc;

        promotionCalcModel.plu = new ArrayList<>();
        for (Plu plu : resultData.plu) {
            SalePluModel pluModel = new SalePluModel();
            pluModel.setItemNo(plu.itemNo);
            pluModel.setPageNo(plu.pageNo);
            pluModel.setPluCode(plu.pluCode);
            pluModel.setTagPrice(plu.tagPrice);
            pluModel.setPrice(plu.price);
            pluModel.setQuantity(plu.quantity);
            pluModel.setTotal(plu.total);
            pluModel.setTotalRecv(plu.totalRecv);
            pluModel.setTotalDsc(plu.totalDsc);
            if (!ListUtils.isEmpty(plu.pluDsc)) {
                for (PluDsc pluDscTemp: plu.pluDsc) {
                    SalePluDscModel pluDscItem = new SalePluDscModel();
                    pluDscItem.setDscCode(pluDscTemp.getDscCode());
                    pluDscItem.setDscNum(pluDscTemp.getDscNum());
                    pluDscItem.setDscProName(pluDscTemp.getDscProName());
                    pluDscItem.setDscProType(pluDscTemp.getDscProType());
                    pluDscItem.setPageNo(pluModel.getPageNo());
                    pluDscItem.setItemNo(pluModel.getItemNo());
                    pluDscItem.setTotalDsc(pluDscTemp.getTotalDsc());
                    pluDscItem.setGlobalType(pluDscTemp.getGlobalType());
                    pluDscItem.setPluCode(plu.pluCode);
                    pluModel.getPluDscs().add(pluDscItem);
                }
            }
            if (BigDecimalUtils.isNotEquals0(plu.getTotal())) {
                promotionCalcModel.plu.add(pluModel);
            }
        }


        promotionCalcModel.pluGiftActions = new ArrayList<>();
        if (!ListUtils.isEmpty(resultData.getPluGift())) {
            for (int i = 0; i < resultData.getPluGift().size(); i++) {
                PluGift pluGift = resultData.getPluGift().get(i);
                for (PluGiftAction pluGiftAction : pluGift.getActions()) {
                    pluGiftAction.setDscCode(pluGift.getDscCode());
                    pluGiftAction.setTotal(pluGift.getQuantity());
                    pluGiftAction.setDscProType(pluGift.getDscProType());
                    pluGiftAction.setDscProName(pluGift.getDscProName());
                    pluGiftAction.setGlobalType(pluGift.getGlobalType());
                    pluGiftAction.setGroup(String.valueOf(i));
                    pluGiftAction.setMemberCount(String.valueOf(pluGift.getActions().size()));
                    pluGiftAction.setGroupCount(String.valueOf(resultData.getPluGift().size()));

                    promotionCalcModel.pluGiftActions.add(pluGiftAction);
                }
            }
        }

        return promotionCalcModel;
    }
}
