package com.hisense.hipos.webapi.sale;

import com.hisense.common.util.ListUtils;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.plu.PluDsc;
import com.hisense.hipos.plu.PluV2;
import com.hisense.hipos.sale.CalcType;
import com.hisense.hipos.webapi.Context;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;

import lombok.Data;

/**
 * @author guolina          2020-01-09 HIAPOSCMP-490 慧收银增加整单促销接口
 * @author guolina          2020-01-13 HIAPOSCMP-495 易收银调整整单促销接口
 * @author guolina          2020-02-27 HIAPOSCMP-553 与中台联调商品查询和商品促销计算接口
 */
@Data
public class PromotionCalcRequestData {

    public String cardType;             //卡类型
    public String vipCardNo;               //会员卡号
    public String orgCode;              //组织编码
    public List<PluV2> plu;             //交易商品
    public String posNo;                //收款机号
    public String shopId;               //专柜id
    public String saleNo;               //流水号
    public String saleTime;             //销售日期
    public String calcType;             //促销计算方式 0-录入时的计算 1-合计时的计算
    public String totalReductionWay;             //满减记录方式 0 – 记录为优惠1 – 记录为支付方式


    /**
     * 返回一个新的 WebAPI 请求。（中山益华）
     *
     * @return WebAPI 请求
     */
    public static Request<PromotionCalcRequestData> request(CalcType calcType, List<BaseSalePluModel> pluModels, String totalReductionWay) {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();

        PromotionCalcRequestData saleReductionRequestData = new PromotionCalcRequestData();
        saleReductionRequestData.plu = new ArrayList<>();

        if (saleModel.getVip() != null) {
            saleReductionRequestData.cardType = saleModel.getVip().getCard().cardType().code();
            saleReductionRequestData.vipCardNo = saleModel.getVip().getCard().vipCardNo();
        }
        saleReductionRequestData.totalReductionWay = totalReductionWay.equals("支付方式") ? "1" : "0";
        saleReductionRequestData.calcType = String.valueOf(calcType == null ? "1" : calcType.value());
        saleReductionRequestData.orgCode = Context.orgCode();
        if (!WebApi.isAmwAndAip()) {
            saleReductionRequestData.shopId = Context.shopCode();
        }
        for (BaseSalePluModel pluModel : pluModels) {
            PluV2 pluItem = new PluV2();
            pluItem.brandCode = pluModel.getBrandCode();
            pluItem.classCode = pluModel.getClassCode();
            pluItem.depCode = pluModel.getDepCode();
            pluItem.depId = pluModel.getDepId();
            pluItem.itemNo = pluModel.getItemNo();
            pluItem.pageNo = pluModel.getPageNo();
            pluItem.pluCode = pluModel.getPluCode();
            pluItem.pluId = pluModel.getPluId();
            pluItem.price = pluModel.getPrice();
            pluItem.quantity = pluModel.getQuantity();
            pluItem.barCode = pluModel.getBarCode();
            pluItem.total = pluModel.getTotal();
            pluItem.totalRecv = pluModel.getTotalRecv();
            pluItem.tagPrice = pluModel.getTagPrice();
            pluItem.vipPrice = pluModel.getVipPrice();
            if (!WebApi.isAmwAndAip()) {
                pluItem.shopCode = pluModel.getShopCode();
                pluItem.shopId = pluModel.getShopId();
            }
            pluItem.pluDsc = new ArrayList<>();
            if (!ListUtils.isEmpty(pluModel.getPluDscs())) {
                for (SalePluDscModel pluDscModel : pluModel.getPluDscs()) {
                    PluDsc pluDscItem = new PluDsc();
                    pluDscItem.dscCode = pluDscModel.getDscCode();
                    pluDscItem.dscNum = pluDscModel.getDscNum();
                    pluDscItem.dscProName = pluDscModel.getDscProName();
                    pluDscItem.dscProType = pluDscModel.getDscProType();
                    pluDscItem.pageNo = pluModel.getPageNo();
                    pluDscItem.itemNo = pluModel.getItemNo();
                    pluDscItem.totalDsc = pluModel.getTotalDsc();

                    pluItem.pluDsc.add(pluDscItem);
                }
            }
            saleReductionRequestData.plu.add(pluItem);


        }

        saleReductionRequestData.posNo = Context.posNo();
        saleReductionRequestData.saleNo = saleModel.getSaleNo();
        saleReductionRequestData.saleTime = saleModel.getSaleTime();

        return WebApi.sign(saleReductionRequestData);
    }
}
