/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.sale;

import com.hisense.common.util.CustomUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;

import java.util.ArrayList;

import lombok.Data;

/**
 * 交易流水数据。
 *
 * @author zhangsai    2018-02-24 HIPOSAPP-334
 * @author zhangsai    2018-07-24 HIPOSAPP-698
 * @author guolina     2019-04-03 HIPOSAPP-1077 增加dataSrc字段区分不同应用的流水
 * @author wangyaogang 2019-08-22 HIPOSAPP-1286 流水上传接口增加vipCardNo
 * @author guolina     2020-03-17 HIAPOSCMP-563 流水上传增加cshName字段
 * @author liuyazeng   2020-09-25 HIAPOSCMP-1141 慧收银流水上传添加vipMobile字段
 */
@Data
public class SaleData {
    /**
     * 结账日期
     */
    public String accDate;

    /**
     * 会员号
     */
    public String vipCardNo;

    /**
     * 会员手机号
     */
    public String vipMobile;

    /**
     * 会员卡号(卡面号)
     */
    public String cardNo;

    /**
     * 会员卡号类型
     */
    public int cardNoType;

    /**
     * 会员卡类型
     */
    public String cardType;

    /**
     * 数据来源
     */
    public String dataSrc;

    /**
     * 收银员编码
     */
    public String cshCode;

    /**
     * 收银员 ID
     */
    public String cshId;

    /**
     * 收银员名称
     */
    public String cshName;

    /**
     * 组织编码
     */
    public String orgCode;

    /**
     * 收款机号
     */
    public String posNo;

    /**
     * 流水号
     */
    public String saleNo;

    /**
     * 销售日期
     */
    public String saleDate;

    /**
     * 应收合计
     */
    public String total;

    /**
     * 找零合计
     */
    public String totalChange;

    /**
     * 优惠合计
     */
    public String totalDsc;

    /**
     * 支付合计
     */
    public String totalPaid;

    /**
     * 实收合计
     */
    public String totalRecv;

    /**
     * 抹零合计
     */
    public String totalRound;

    /**
     * 交易类型
     */
    public String tranType;

    /**
     * 备注
     */
    public String remark;

    /**
     * 扩展属性
     */
    public String udp1;
    /**
     * 扩展属性
     */
    public String udp2;
    /**
     * 扩展属性
     */
    public String udp3;
    /**
     * 扩展属性
     */
    public String udp4;
    /**
     * 扩展属性
     */
    public String udp5;
    /**
     * 原流水号
     */
    public String oldSaleNo;

    public static ArrayList<SaleData> buildSaleData(BaseSaleModel saleModel) {
        MainSaleModel mainSaleModel = MainSaleModel.getInstance();

        ArrayList<SaleData> saleDataList = new ArrayList<>();

        SaleData saleData = new SaleData();
        saleData.accDate = saleModel.getAccDate();

        String cardNo = "";
        String vipCardNo = "";
        String cardType = "";
        if (null != saleModel.getVip() && null != saleModel.getVip().card() && null != saleModel.getVip().card().vipCardNo()) {
            cardNo = saleModel.getVip().card().cardFaceNo();
            vipCardNo = saleModel.getVip().card().vipCardNo();
            if (!StringUtils.isEmpty(saleModel.getVip().getMobile())) {
                saleData.vipMobile = saleModel.getVip().getMobile();
            }
            cardType = saleModel.getVip().card().cardType().code();
        }
        saleData.cardNo = RSAService.encrypt(cardNo);
        saleData.cardNoType = 1;                  //与传递的cardNo对应，此处cardNo是会员号，所以应该传3  0-卡内号；1-卡面号；2-手机号；3-会员号
        saleData.vipCardNo = vipCardNo;
        saleData.cardType = cardType;

        saleData.dataSrc = CustomUtils.isUniformStyle() ? "3" : "2";//2易收银;3慧收银
        saleData.cshCode = saleModel.getCashier().getUserCode();
        saleData.cshId = saleModel.getCashier().getUserId();
        saleData.cshName = saleModel.getUdp1();
        saleData.orgCode = Context.orgCode();
        saleData.posNo = saleModel.getPosNo();
        saleData.saleNo = RSAService.encrypt(saleModel.getSaleNo());
        if (saleModel instanceof ReturnSaleModel) {
            saleData.oldSaleNo = ((ReturnSaleModel) saleModel).oldSaleNo();
        }
        saleData.saleDate = saleModel.getSaleTime();

        saleData.total = RSAService.encrypt(saleModel.getTotal());
        saleData.totalChange = RSAService.encrypt(saleModel.getTotalChange());
        saleData.totalDsc = RSAService.encrypt(saleModel.getTotalDsc());
        saleData.totalPaid = RSAService.encrypt(saleModel.getTotalPaid());
        saleData.totalRecv = RSAService.encrypt(saleModel.getTotalRecv());
        saleData.totalRound = RSAService.encrypt(saleModel.getTotalRound());
        saleData.tranType = String.valueOf(saleModel.getTranType().value());
        saleData.remark = saleModel.getRemark();
        saleData.udp1 = saleModel.getUdp1();
        saleData.udp2 = saleModel.getUdp2();
        saleData.udp3 = saleModel.getUdp3();
        saleData.udp4 = saleModel.getUdp4(); // 电子发票提取码
        saleData.udp5 = saleModel.getUdp5();

        saleDataList.add(saleData);

        return saleDataList;
    }

    public static ArrayList<SaleData> buildSaleData(ArrayList<SaleData> sale) {
        ArrayList<SaleData> saleDataList = new ArrayList<>();

        SaleData saleData = sale.get(0);
        saleData.cardNo = RSAService.encrypt(saleData.cardNo);
        saleData.cardNoType = 1;                  //与传递的cardNo对应，此处cardNo是会员号，所以应该传3  0-卡内号；1-卡面号；2-手机号；3-会员号

        saleData.dataSrc = "3";//2代表易收银
        saleData.saleNo = RSAService.encrypt(saleData.getSaleNo());
        saleData.total = RSAService.encrypt(saleData.getTotal());
        saleData.totalChange = RSAService.encrypt(saleData.getTotalChange());
        saleData.totalDsc = RSAService.encrypt(saleData.getTotalDsc());
        saleData.totalPaid = RSAService.encrypt(saleData.getTotalPaid());
        saleData.totalRecv = RSAService.encrypt(saleData.getTotalRecv());
        saleData.totalRound = RSAService.encrypt(saleData.getTotalRound());

        saleDataList.add(saleData);

        return saleDataList;
    }
}
