/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.sale;

import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;

import java.util.ArrayList;

import lombok.Data;

/**
 * 交易流水商品信息。
 *
 * @author zhangsai    2018-02-24 HIPOSAPP-334
 * @author zhangsai    2018-07-24 HIPOSAPP-698
 * @author wangyaogang 2019-06-14 HIPOSAPP-1098 增加AuthNo字段
 */
@Data
public class SalePluData {
    /**
     * 退货工号
     */
    public String authNo;
    /**
     * 条码
     */
    public String barCode;

    /**
     * 品牌编码
     */
    public String brandCode;

    /**
     * 品类编码
     */
    public String classCode;

    /**
     * 营业员编码
     */
    public String clkCode;

    /**
     * 营业员 ID
     */
    public String clkId;

    /**
     * 部门编码
     */
    public String depCode;

    /**
     * 部门 ID
     */
    public String depId;

    /**
     * 部门名称
     */
    public String depName;

    /**
     * 发票号
     */
    public String invNo;

    /**
     * 行号
     */
    public String itemNo;

    /**
     * 组织编码
     */
    public String orgCode;

    /**
     * 页号
     */
    public String pageNo;

    /**
     * 商品编码
     */
    public String pluCode;

    /**
     * 商品 ID
     */
    public String pluId;

    /**
     * 商品名称
     */
    public String pluName;

    /**
     * 商品类型
     */
    public String pluType;

    /**
     * 发生价格
     */
    public String price;

    /**
     * 数量
     */
    public String quantity;

    /**
     * 流水号
     */
    public String saleNo;

    /**
     * 专柜编码
     */
    public String shopCode;

    /**
     * 专柜ID
     */
    public String shopId;

    /**
     * 规格
     */
    public String spec;

    /**
     * 商品标价
     */
    public String tagPrice;

    /**
     * 应收合计
     */
    public String total;

    /**
     * 优惠合计
     */
    public String totalDsc;

    /**
     * 实收合计
     */
    public String totalRecv;

    /**
     * 抹零合计
     */
    public String totalRound;

    /**
     * 交易类型
     */
    public String tranType;

    /**
     * 单位
     */
    public String unit;

    /**
     * 备注
     */
    public String remark;
    /**
     * 扩展属性
     */
    public String udp1;
    /**
     * 扩展属性
     */
    public String udp2;
    /**
     * 扩展属性
     */
    public String udp3;
    /**
     * 扩展属性
     */
    public String udp4;
    /**
     * 扩展属性
     */
    public String udp5;

    public static ArrayList<SalePluData> buildSalePluData(BaseSaleModel saleModel) {
        ArrayList<SalePluData> pluDataList = new ArrayList<>();

        for (BaseSalePluModel pluModel : saleModel.getPlus()) {
            SalePluData pluData = new SalePluData();
            pluData.authNo = (saleModel instanceof ReturnSaleModel) ? ((ReturnSaleModel) saleModel).getAuthNo() : "";
            pluData.barCode = pluModel.getBarCode();
            pluData.brandCode = pluModel.getBrandCode();
            pluData.classCode = pluModel.getClassCode();
            pluData.clkCode = saleModel.getCashier().getUserCode();
            pluData.clkId = saleModel.getCashier().getUserId();
            pluData.depCode = pluModel.getDepCode();
            pluData.depId = pluModel.getDepId();
            pluData.depName = pluModel.getDepName();
            pluData.itemNo = String.valueOf(pluModel.getItemNo());
            pluData.orgCode = Context.orgCode();
            pluData.pageNo = String.valueOf(pluModel.getPageNo());
            pluData.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluData.pluId = RSAService.encrypt(pluModel.getPluId());
            pluData.pluName = pluModel.getPluName();
            pluData.pluType = pluModel.getPluType().value();
            pluData.price = RSAService.encrypt(pluModel.getPrice());
            pluData.quantity = RSAService.encrypt(pluModel.getQuantity());
            pluData.saleNo = RSAService.encrypt(saleModel.getSaleNo());
            pluData.shopCode = RSAService.encrypt(pluModel.getShopCode());
            pluData.shopId = pluModel.getShopId();
            pluData.spec = pluModel.getSpec();
            pluData.tagPrice = String.valueOf(pluModel.getTagPrice());
            pluData.total = RSAService.encrypt(pluModel.getTotal());
            pluData.totalDsc = RSAService.encrypt(pluModel.getTotalDsc());
            pluData.totalRecv = RSAService.encrypt(pluModel.getTotalRecv());
            pluData.totalRound = RSAService.encrypt(pluModel.getTotalRound());
            pluData.tranType = String.valueOf(saleModel.getTranType().value());
            pluData.unit = pluModel.getUnit();
            pluData.remark = pluModel.getRemark();

            //UDP1保存是否称重商品
            pluData.udp1 = pluModel.isWeightAndNotCount() ? "1" : "0";
            pluData.udp2 = pluModel.getUdp2();
            pluData.udp3 = pluModel.getUdp3();
            pluData.udp4 = pluModel.getUdp4();
            pluData.udp5 = pluModel.getUdp5();

            pluDataList.add(pluData);
        }

        return pluDataList;
    }

    public static ArrayList<SalePluData> buildSalePluData(ArrayList<SalePluData> plu) {
        ArrayList<SalePluData> pluDataList = new ArrayList<>();

        for (SalePluData pluData : plu) {
            pluData.pluCode = RSAService.encrypt(pluData.getPluCode());
            pluData.pluId = RSAService.encrypt(pluData.getPluId());
            pluData.price = RSAService.encrypt(pluData.getPrice());
            pluData.quantity = RSAService.encrypt(pluData.getQuantity());
            pluData.saleNo = RSAService.encrypt(pluData.getSaleNo());
            pluData.shopCode = RSAService.encrypt(pluData.getShopCode());
            pluData.total = RSAService.encrypt(pluData.getTotal());
            pluData.totalDsc = RSAService.encrypt(pluData.getTotalDsc());
            pluData.totalRecv = RSAService.encrypt(pluData.getTotalRecv());
            pluData.totalRound = RSAService.encrypt(pluData.getTotalRound());

            pluDataList.add(pluData);
        }

        return pluDataList;
    }
}
