/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.sale;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.crm.card.CardNoType;
import com.hisense.crm.card.VipCard;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.model.SalePluDscModel;
import com.hisense.hipos.model.SalePluModel;
import com.hisense.hipos.plu.PluType;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.User;

import java.util.List;

/**
 * 流水查询的返回结果数据。
 *
 * @author zhangsai    2018-02-24 HIPOSAPP-334
 * @author xieliyan    2018-03-14 HIPOSAPP-355 重打印小票的时间显示为进行该笔交易的时间
 * @author xieliyan    2018-07-04 HIPOSAPP-630 退货数量调整为正数，避免合计计算错误
 * @author zhangsai    2018-12-21 HIPOSAPP-699 支持退货流水查询
 * @author guolina     2018-12-21 HIPOSAPP-839 小票增加促销打印
 * @author zhangsai    2018-12-26 HIPOSAPP-931 流水查询组装流水时新增营业员参数处理逻辑
 * @author zhangsai    2019-01-07 HIPOSAPP-956 支持根据商品分摊计算商品实际发生价
 * @author zhangsai    2019-01-08 HIPOSAPP-966 除商品优惠外，其它促销计入分摊
 * @author guolina     2019-03-18 HIPOSAPP-1054 流水查询的total和pluTotal不再次四舍五入，保留查询的原值
 * @author wangyaogang 2019-05-15 HIPOSAPP-1087 优化[控制每台机器收款的小数位]逻辑,去掉isNeedRound
 * @author wangyaogang 2019-06-05 HIPOSAPP-1154 易收银销售小票-打印商品"货号"字段，初始化时增加货号属性
 * @author zhangsai    2019-06-13 HIPOSAPP-1169 Sale.payments() 属性返回类型改为 ISalePayment
 * @author zhangsai    2019-06-18 HIPOSAPP-1175 商品优惠允许负值（例如：交易四舍五入分摊金额）
 * @author wangyaogang 2019-08-30 HIPOSAPP-1284 查询流水时支付表记录Remark字段
 * @author wangyaogang 2019-11-15 HIAPOSCMP-289 流水查询不再取绝对值
 */
class SaleQueryResultData {
    /**
     * 流水集合。
     */
    public List<SaleData> sale;

    /**
     * 商品集合。
     */
    public List<SalePluData> plu;

    /**
     * 商品优惠集合。
     */
    public List<SalePluDiscountData> pluDsc;

    /**
     * 支付信息集合。
     */
    public List<SalePayData> pay;


    /**
     * 处理 WebAPI 接口返回数据。
     *
     * @param resultData 接口返回数据
     * @return 如果返回结果包含流水则解析后直接返回。否则，返回 null。
     */
    public static BaseSaleModel handleResult(SaleQueryResultData resultData) {
        SaleModel saleModel = new SaleModel();

        BaseSaleModel result = null;
        if (resultData != null && resultData.sale.size() > 0) {
            SaleData saleData = resultData.sale.get(0);

            String saleNo;
            boolean isReturn = false;
            if (Double.valueOf(saleData.total) > 0) {
                saleNo = saleData.saleNo;
            } else {
                saleNo = saleData.udp2;
                isReturn = true;
            }

            VipCard card = null;
            if (saleData.cardNo != null && saleData.cardNo.length() > 0) {
                card = new VipCard.Builder(saleData.cardNo, "")
                        .cardType(saleData.cardType, null)
                        .cardNoType(CardNoType.valueOf(saleData.cardNoType))
                        .build();
            }

            saleModel.setSaleNo(saleNo);
            saleModel.setOrgCode(saleData.orgCode);
            saleModel.setPosNo(saleData.posNo);
            saleModel.setCashier(new User(saleData.cshId, saleData.cshCode, null));
            saleModel.setAccDate(saleData.accDate);
            saleModel.setTranType(TranType.valueOf(Integer.valueOf(saleData.tranType)));
            if (card != null) {
                saleModel.addVip(card);
            }

            saleModel.setSaleTime(saleData.saleDate);
            saleModel.setUdp1(saleData.udp1);
            saleModel.setUdp2(saleData.udp2);
            saleModel.setUdp3(saleData.udp3);
            saleModel.setUdp4(saleData.udp4);
            saleModel.setUdp5(saleData.udp5);


            for (SalePluData pluData : resultData.plu) {
                SalePluModel salePluModel = new SalePluModel();
                salePluModel.setBarCode(pluData.barCode);
                salePluModel.setBrandCode(pluData.brandCode);
                salePluModel.setClassCode(pluData.classCode);
                salePluModel.setClkCode(pluData.clkCode);
                salePluModel.setClkId(pluData.clkId);
                salePluModel.setDepCode(pluData.depCode);
                salePluModel.setDepId(pluData.depId);
                salePluModel.setDepName(pluData.depName);
                salePluModel.setItemNo(Integer.parseInt(pluData.itemNo));
                salePluModel.setPageNo(Integer.parseInt(pluData.pageNo));
                salePluModel.setPluCode(pluData.pluCode);
                salePluModel.setPluId(pluData.pluId);
                salePluModel.setPluName(pluData.pluName);
                salePluModel.setPluType(PluType.valueOfDefined(pluData.pluType));
                salePluModel.setPrice(pluData.price);
                salePluModel.setQuantity(pluData.quantity);
                salePluModel.setShopCode(pluData.shopCode);
                salePluModel.setShopId(pluData.shopId);
                salePluModel.setSpec(pluData.spec);
                salePluModel.setUnit(pluData.unit);
                salePluModel.setUdp5(pluData.udp5);
                salePluModel.setTagPrice(pluData.tagPrice);
                salePluModel.setTotalRecv(pluData.totalRecv);
                salePluModel.setTotal(pluData.total);
                salePluModel.setTotalDsc(pluData.totalDsc);
                salePluModel.setTotalRound(pluData.totalRound);
                salePluModel.setRemark(pluData.remark);
                salePluModel.setIsWeight(StringUtils.toInt(pluData.udp1, 0));

                for (SalePluDiscountData pluDscData : resultData.pluDsc) {
                    SalePluDscModel salePluDscModel = new SalePluDscModel();
                    salePluDscModel.setDscType("1");
                    salePluDscModel.setDscCode(pluDscData.dscCode);
                    salePluDscModel.setDscProType(pluDscData.dscType);
                    salePluDscModel.setDscProName(pluDscData.dscName);
                    salePluDscModel.setPageNo(Integer.parseInt(pluDscData.pageNo));
                    salePluDscModel.setItemNo(Integer.valueOf(pluDscData.itemNo));
                    salePluDscModel.setPluId(pluDscData.pluId);
                    salePluDscModel.setTotalDsc(pluDscData.totalDsc);

                    if (BigDecimalUtils.isEquals(pluDscData.itemNo, pluData.itemNo)) {
                        salePluModel.addPluDsc(salePluDscModel);
                    }
                }

                saleModel.addPlu(salePluModel);
            }

            for (SalePayData payData : resultData.pay) {
                SalePayModel salePayModel = new SalePayModel();
                salePayModel.setPayCode(payData.payCode);
                salePayModel.setPayName(payData.payName);
                salePayModel.setRemark(payData.remark);
                salePayModel.setPayType(payData.payType);
                salePayModel.setBuyerLogonId(payData.payNo);
                salePayModel.setSerialNo(Integer.parseInt(payData.serialNo));
                salePayModel.setTotalPaid(payData.totalPaid);
                salePayModel.setTotalRecv(payData.totalRecv);
                salePayModel.setUdp1(payData.udp1);
                salePayModel.setUdp2(payData.udp2);
                salePayModel.setUdp3(payData.udp3);
                salePayModel.setUdp4(payData.udp4);
                salePayModel.setUdp5(payData.udp5);

                saleModel.addPay(salePayModel);
            }

            if (isReturn) {
                ReturnSaleModel returnSaleModel = new ReturnSaleModel();
                returnSaleModel.setSaleNo(saleData.saleNo);
                returnSaleModel.setSaleModel(saleModel);
                returnSaleModel.setSaleTime(saleModel.getSaleTime());
                returnSaleModel.setPlus(saleModel.getPlus());
                returnSaleModel.setPays(saleModel.getPays());

                returnSaleModel.setUdp2(saleModel.getSaleNo());
                returnSaleModel.setUdp4(saleModel.getUdp4());
                result = returnSaleModel;

            } else {
                result = saleModel;
            }
        }

        return result;
    }
}
