/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.sale;

import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.model.SalePayModel;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;

/**
 * 交易满送赠券请求数据。
 *
 * @author zhangsai 2018-06-15 HIPOSAPP-538
 * @author zhangsai 2018-12-19 HIPOSAPP-698  交易商品类型改为 ISaleItem
 * @author zhangsai 2019-06-13 HIPOSAPP-1169 Sale.payments() 属性返回类型改为 ISalePayment
 */
class SaleRebateRequestData {
    /**
     * 会员卡号
     */
    public String cardNo;

    /**
     * 会员类型
     */
    public String cardType;

    /**
     * 组织编码
     */
    public String orgCode;

    /**
     * 流水号
     */
    public String saleNo;

    /**
     * 支付信息
     */
    public ArrayList<SaleRebateRequestPayData> salePay;

    /**
     * 商品信息
     */
    public ArrayList<SaleRebateRequestPluData> salePlu;

    /**
     * 销售日期
     */
    public String saleTime;

    /**
     * 用户编码
     */
    public String userCode;

    /**
     * 用户ID
     */
    public String userId;

    /**
     * 用户名称
     */
    public String userName;

    public static Request<SaleRebateRequestData> request() {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();

        SaleRebateRequestData data = new SaleRebateRequestData();
        if (saleModel.isVip()) {
            data.cardNo = RSAService.encrypt(saleModel.getVip().card().cardFaceNo());
            data.cardType = saleModel.getVip().card().cardType().code();
        } else {
            data.cardNo = RSAService.encrypt("");
            data.cardType = "";
        }
        data.orgCode = Context.orgCode();
        data.saleNo = RSAService.encrypt(saleModel.getSaleNo());
        data.saleTime = RSAService.encrypt(saleModel.getSaleTime());
        data.userCode = RSAService.encrypt(saleModel.getCashier().code());
        data.userId = saleModel.getCashier().id();
        data.userName = saleModel.getCashier().name();

        data.salePay = new ArrayList<>();
        for (SalePayModel payModel : saleModel.getPays()) {
            SaleRebateRequestPayData payData = new SaleRebateRequestPayData();
            payData.payCode = RSAService.encrypt(payModel.getPayCode());
            payData.totalPaid = RSAService.encrypt(payModel.getTotalPaid());

            data.salePay.add(payData);
        }

        data.salePlu = new ArrayList<>();
        for (BaseSalePluModel pluModel : saleModel.getPlus()) {
            SaleRebateRequestPluData pluData = new SaleRebateRequestPluData();
            pluData.depCode = pluModel.getDepCode();
            pluData.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluData.pluId = RSAService.encrypt(pluModel.getPluId());
            pluData.quantity = RSAService.encrypt(pluModel.getQuantity());
            pluData.shopCode = RSAService.encrypt(pluModel.getShopCode());
            pluData.total = RSAService.encrypt(pluModel.getTotalRecv());

            data.salePlu.add(pluData);
        }

        return WebApi.sign(data);
    }
}
