/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.webapi.sale;

import com.hisense.hipos.coupon.Coupon;
import com.hisense.hipos.coupon.CouponType;
import com.hisense.hipos.coupon.GiftCoupon;
import com.hisense.hipos.coupon.ICoupon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 交易满送赠券返回的赠券。
 *
 * @author zhangsai     2018-06-15 HIPOSAPP-538
 * @author zhangsai     2018-11-13 HIPOSAPP-822  满送赠券接口返回结果支持礼品券。
 * @author guolina      2018-11-14 HIPOSAPP-823  小票增加礼品券打印
 * @author wangyaogang  2019-06-12 HIPOSAPP-1114 增加qType券类型字段
 * @author wangyaogang  2019-06-24 HIPOSAPP-1090 增加礼品名称,活动描述字段,更改礼品数量字段;增加字符串按指定长度添加换行符的方法
 */
public class SaleRebateResultCoupon {

    /**
     * 促销方案名称
     */
    public String name;

    /**
     * 赠券编号
     */
    public String no;

    /**
     * 校验码
     */
    public String checkCode;

    /**
     * 面额。金额券时返回，其它情况下返回null。
     */
    public String value;

    /**
     * 用券开始日期
     */
    public String beginDate;

    /**
     * 用券结束日期
     */
    public String endDate;

    /**
     * 会员卡号
     */
    public String cardNo;

    /**
     * 会员卡类型
     */
    public String cardType;

    /**
     * 是否打印。0-不打印；1-打印
     */
    public int isPrint;

    /**
     * 促销说明
     */
    public String remark;

    /**
     * 券类型
     */
    public String qType;

    /**
     * 礼品券名称
     */
    public String couponName;
    /**
     * 礼品券数量,赠送张数。礼品券时返回，其它情况下返回null。
     */
    public String couponCount;

    /**
     * 活动描述
     */
    public String couponDesc;


    /**
     * 处理 WebAPI 接口返回数据。
     *
     * @param resultData 接口返回数据
     * @return 赠券集合。如果不赠券，则返回的集合为空集合
     */
    static Collection<ICoupon> handleResult(List<SaleRebateResultCoupon> resultData) {
        ArrayList<ICoupon> coupons = new ArrayList<>();

        for (SaleRebateResultCoupon c : resultData) {
            ICoupon coupon;
            if (c.couponCount == null || "".equals(c.couponCount)) {
                // 金额券
                coupon = new Coupon.Builder()
                        .coupon(c.no, c.checkCode, c.name)
                        .values(Double.valueOf(c.value))
                        .beginAt(c.beginDate)
                        .endsAt(c.endDate)
                        .isPrint(c.isPrint == 1)
                        .remark(getFormatStr(c.remark, 16))
                        .couponDesc(c.couponDesc)
                        .type(CouponType.valueOf(Integer.valueOf(c.qType != null ? c.qType : "0")))
                        .build();
            } else {
                // 礼品券
                coupon = new GiftCoupon.Builder()
                        .coupon(c.no, c.name)
                        .count(Integer.valueOf(c.couponCount))
                        .beginAt(c.beginDate)
                        .endsAt(c.endDate)
                        .isPrint(c.isPrint == 1)
                        .remark(getFormatStr(c.remark, 16))
                        .couponDesc(c.couponDesc)
                        .giftName(c.couponName != null ? c.couponName : "")
                        .build();
            }

            if (coupon instanceof Coupon) {
                coupons.add(coupon);
            } else if (coupon instanceof GiftCoupon) {
                coupons.add(coupon);
            }
        }
        return coupons;
    }

    /**
     * 把原始字符串按照指定长度添加换行符
     *
     * @param inputString 原始字符串
     * @param length      指定长度
     * @return
     */
    public static String getFormatStr(String inputString, int length) {
        if (inputString == null || "".equals(inputString)) {
            return "";
        }
        int size = inputString.length() / length;
        if (inputString.length() % length != 0) {
            size += 1;
        }
        return getStr(inputString, length, size);
    }


    /**
     * 把原始字符串按照指定长度添加换行符
     *
     * @param inputString 原始字符串
     * @param length      指定长度
     * @param size        指定列表大小
     * @return
     */
    private static String getStr(String inputString, int length, int size) {
        StringBuilder sb = new StringBuilder();
        for (int index = 0; index < size; index++) {
            String childStr = subString(inputString, index * length, (index + 1) * length);
            if (childStr != null) {
                sb.append(childStr);
                sb.append("\n");
            }
        }
        return sb.toString();
    }

    /**
     * 分割字符串，如果开始位置大于字符串长度，返回空
     *
     * @param str 原始字符串
     * @param f   开始位置
     * @param t   结束位置
     * @return
     */
    private static String subString(String str, int f, int t) {
        if (f > str.length()) {
            return null;
        }
        if (t > str.length()) {
            return str.substring(f, str.length());
        } else {
            return str.substring(f, t);
        }
    }
}
