/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.sale;


import com.hisense.common.util.DateUtils;
import com.hisense.common.util.JSONUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hipos.coupon.ICoupon;
import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.ReturnSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.sale.CalcType;
import com.hisense.hipos.webapi.plu.PluQueryResultData;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.text.ParseException;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.concurrent.TimeUnit;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import sun.rmi.runtime.Log;

/**
 * 交易服务。
 *
 * @author zhangsai    2017-09-03
 * @author zhangsai    2017-12-21 HIPOSAPP-250 增加获取当前结账日期的方法
 * @author zhangsai    2018-01-24 HIPOSAPP-302 增加日结和异步获取结账日期的方法
 * @author zhangsai    2018-02-24 HIPOSAPP-334 增加流水查询
 * @author zhangsai    2018-06-15 HIPOSAPP-538 增加满送赠券
 * @author zhangsai    2018-06-15 HIPOSAPP-539 增加异步获取当前结账日期的方法
 * @author zhangsai    2018-06-20 HIPOSAPP-565 增加满减促销
 * @author zhangsai    2018-07-26 HIPOSAPP-698 增加流水上传接口
 * @author guolina     2018-08-10 HIPOSAPP-715 流水查询界面添加扫描核销功能
 * @author guolina     2018-11-14 HIPOSAPP-823 小票增加礼品券打印
 * @author zhangsai    2018-12-21 HIPOSAPP-699 增加退货流水上传
 * @author guolina     2018-12-25 HIPOSAPP-850 获取新流水号兼容POS中间件,规则可以在中间件设置
 * @author zhangsai    2018-12-26 HIPOSAPP-931 普通流水上传/退货流水上传删除营业员参数
 * @author wangyaogang 2019-06-12 HIPOSAPP-1165 调整销售和退货流水上传的网络超时时间为2分钟
 * @author zhangsai    2019-06-12 HIPOSAPP-1166 删除扫码购核销
 * @author wangyaogang 2019-11-26 HIAPOSCMP-344 修复因scrm流水上传完毕数据处理方法未调用导致的券无法打印问题
 * @author guolina     2020-01-09 HIAPOSCMP-490 慧收银增加整单促销接口
 * @author guolina     2020-01-13 HIAPOSCMP-495 易收银调整整单促销接口
 * @author guolina     2020-05-26 HIPOSAPP-1408 重打小票限制为本收款台需求
 */
public class SaleService {

    private SaleService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 日结。
     *
     * @param callback 日结的回调方法
     */
    public static void accounting(final com.hisense.webapi.Callback<String> callback) {
        Request<AccountingRequestData> request = AccountingRequestData.request();

        WebApi.requestAsync(WebApi.create(ISaleService.class).accounting(request), new Callback<Result<String>>() {
            @Override
            public void onResponse(Call<Result<String>> call, Response<Result<String>> response) {
                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<String>> call, Throwable t) {
                callback.onError(t);
            }
        }, String.class);
    }

    /**
     * 根据最后一次结账日期计算并返回当前的结账日期。
     *
     * @param lastAccDate 最后一次结账的日期
     * @return 当前的结账日期
     * @throws WebApiException 如果无法解析日期
     */
    private static String getAccountingDate(String lastAccDate) throws WebApiException {
        if (StringUtils.isEmpty(lastAccDate)) {
            throw new WebApiException("日期字符串为空，无法解析日期");
        }
        try {
            Calendar c = new GregorianCalendar();
            c.setTime(WebApi.parseDate(lastAccDate));
            c.add(Calendar.DAY_OF_MONTH, 1);
            return WebApi.formatDate(c.getTime());
        } catch (ParseException ex) {
            throw new WebApiException("无法解析日期", ex);
        }
    }

    /**
     * 获取当前的结账日期。
     *
     * @param callback 获取当前结账日期的回调方法
     */
    public static void getAccountingDate(final com.hisense.webapi.Callback<String> callback) {
        Request<AccountingDateRequestData> request = AccountingDateRequestData.request();

        WebApi.requestAsync(WebApi.create(ISaleService.class).lastAccountingDate(request), new Callback<Result<AccountingDateResultData>>() {
            @Override
            public void onResponse(Call<Result<AccountingDateResultData>> call, Response<Result<AccountingDateResultData>> response) {
                try {
                    callback.onSuccess(getAccountingDate(response.body().data.lastAccountingDate));
                } catch (WebApiException t) {
                    callback.onError(t);
                }
            }

            @Override
            public void onFailure(Call<Result<AccountingDateResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, AccountingDateResultData.class);
    }

    /**
     * 交易开始
     *
     * @param accDate  结账日期
     * @param cdkey    cdkey
     * @param operator 操作员
     * @param callback 回调函数
     */
    public static void saleBegin(String accDate, String cdkey, String operator, String mac, final com.hisense.webapi.Callback<SaleBeginResultData> callback) {
        Request<SaleBeginRequestData> request = SaleBeginRequestData.request(accDate, cdkey, operator, mac);
        WebApi.requestAsync(WebApi.create(ISaleService.class).begin(request), new Callback<Result<SaleBeginResultData>>() {
            @Override
            public void onResponse(Call<Result<SaleBeginResultData>> call, Response<Result<SaleBeginResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("获取流水号返回参数为空"));
                    return;
                }

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<SaleBeginResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, SaleBeginResultData.class);
    }

    /**
     * 取消交易
     *
     * @param saleNo   流水号
     * @param operator 操作员
     * @param callback 回调函数
     */
    public static void saleCancel(String saleNo, String operator, final com.hisense.webapi.Callback<Object> callback) {
        Request<SaleCancelRequestData> request = SaleCancelRequestData.request(saleNo, operator);

        WebApi.requestAsync(WebApi.create(ISaleService.class).cancel(request), new Callback<Result<Object>>() {
            @Override
            public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("取消交易返回参数为空"));
                    return;
                }

                callback.onSuccess(new Object());
            }

            @Override
            public void onFailure(Call<Result<Object>> call, Throwable t) {
                callback.onError(t);
            }
        }, Object.class);
    }

    /**
     * 满减优惠
     *
     * @param callback
     */
    public static void saleTotalReduction(CalcType calcType, List<BaseSalePluModel> pluModels, final com.hisense.webapi.Callback<TotalReductionModel> callback) {
        Request<TotalReductionRequestData> request = TotalReductionRequestData.request(calcType, pluModels);

        WebApi.requestAsync(WebApi.create(ISaleService.class).totalReduction(request), new Callback<Result<TotalReductionResultData>>() {
            @Override
            public void onResponse(Call<Result<TotalReductionResultData>> call, Response<Result<TotalReductionResultData>> response) {
                try {
                    if (response.body() == null) {
                        callback.onError(new Throwable("整单促销返回参数为空"));
                        return;
                    }

                    callback.onSuccess(TotalReductionModel.handleResult(response.body().data));
                } catch (Exception e) {

                    LoggerUtils.getLogger().debug(JSONUtils.javaBeanToJSON(response.body()));
                    LoggerUtils.getLogger().error("整单异常", e);
                    callback.onError(e);
                }
            }

            @Override
            public void onFailure(Call<Result<TotalReductionResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, TotalReductionResultData.class);
    }


    /**
     * 整单促销
     *
     * @param callback
     */
    public static void promotionCalc(CalcType calcType, List<BaseSalePluModel> pluModels, String totalReductionWay, final com.hisense.webapi.Callback<PromotionCalcModel> callback) {
        Request<PromotionCalcRequestData> request = PromotionCalcRequestData.request(calcType, pluModels, totalReductionWay);

        WebApi.requestAsync(WebApi.create(ISaleService.class).promotionCalc(request), new Callback<Result<PromotionCalcResultData>>() {
            @Override
            public void onResponse(Call<Result<PromotionCalcResultData>> call, Response<Result<PromotionCalcResultData>> response) {
                try {
                    if (response.body() == null) {
                        callback.onError(new Throwable("整单促销返回参数为空"));
                        return;
                    }

                    callback.onSuccess(PromotionCalcModel.handleResult(response.body().data));
                } catch (Exception e) {
                    callback.onError(e);
                }
            }

            @Override
            public void onFailure(Call<Result<PromotionCalcResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, PromotionCalcResultData.class);
    }

    /**
     * 上传流水。
     *
     * @param isUseRpt 是否使用归集接口
     * @param callback 流水上传的回调方法
     */
    public static void submit(boolean isUseRpt, boolean isToVip, SaleModel sale, final com.hisense.webapi.Callback<SaleSubmitResultData> callback) {
        if (isUseRpt) {
            LoggerUtils.getLogger().debug("rpt-------交易上传流水");
            Request<SaleSubmitRequestData> request = SaleSubmitRequestData.request(isToVip, sale);
            ISaleService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ISaleService.class);
            WebApi.requestAsync(service.rptSubmit(request), new Callback<Result<SaleSubmitResultData>>() {
                @Override
                public void onResponse(Call<Result<SaleSubmitResultData>> call, Response<Result<SaleSubmitResultData>> response) {
                    LoggerUtils.getLogger().debug("rpt-------交易上传流水成功");
                    callback.onSuccess(SaleSubmitResultData.handleResult(response.body().data));
                }

                @Override
                public void onFailure(Call<Result<SaleSubmitResultData>> call, Throwable t) {
                    callback.onError(t);
                }
            }, SaleSubmitResultData.class);
        } else {
            Request<SaleSubmitRequestData> request = SaleSubmitRequestData.request(isToVip, sale);
            ISaleService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ISaleService.class);
            WebApi.requestAsync(service.submit(request), new Callback<Result<SaleSubmitResultData>>() {
                @Override
                public void onResponse(Call<Result<SaleSubmitResultData>> call, Response<Result<SaleSubmitResultData>> response) {
                    callback.onSuccess(SaleSubmitResultData.handleResult(response.body().data));
                }

                @Override
                public void onFailure(Call<Result<SaleSubmitResultData>> call, Throwable t) {
                    callback.onError(t);
                }
            }, SaleSubmitResultData.class);
        }

    }

    /**
     * 获取最后一次结账的日期。
     *
     * @param callback 获取最后一次结账日期的回调方法
     */
    public static void getLastAccountingDate(final com.hisense.webapi.Callback<String> callback) {
        Request<AccountingDateRequestData> request = AccountingDateRequestData.request();

        WebApi.requestAsync(WebApi.create(ISaleService.class).lastAccountingDate(request), new Callback<Result<AccountingDateResultData>>() {
            @Override
            public void onResponse(Call<Result<AccountingDateResultData>> call, Response<Result<AccountingDateResultData>> response) {
                callback.onSuccess(response.body().data.lastAccountingDate);
            }

            @Override
            public void onFailure(Call<Result<AccountingDateResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, AccountingDateResultData.class);
    }

    /**
     * 查询 {@code saleNo} 指定的流水。
     *
     * @param useRpt   是否使用归集接口
     * @param saleNo   流水号
     * @param days     流水距当前日期的天数
     * @param callback 查询流水的回调方法
     */
    public static void query(boolean useRpt, String saleNo, int days, String posNo, final com.hisense.webapi.Callback<BaseSaleModel> callback) {
        if (useRpt) {
            // rpt 归集接口
            LoggerUtils.getLogger().debug("rpt------流水查询");
            Request<RptSaleQueryRequestData> request = RptSaleQueryRequestData.request(saleNo, posNo, DateUtils.dateFormat("yyyy-MM-dd"));
            WebApi.requestAsync(WebApi.create(ISaleService.class).rptQuery(request), new Callback<Result<SaleQueryResultData>>() {
                @Override
                public void onResponse(Call<Result<SaleQueryResultData>> call, Response<Result<SaleQueryResultData>> response) {
                    if (response.body() == null) {
                        callback.onError(new Throwable("流水查询返回参数为空"));
                        return;
                    }
                    try {
                        callback.onSuccess(SaleQueryResultData.handleResult(response.body().data));
                        LoggerUtils.getLogger().debug("rpt------流水查询成功");
                    } catch (Exception e) {
                        callback.onError(new Throwable(e.getMessage()));
                    }
                }

                @Override
                public void onFailure(Call<Result<SaleQueryResultData>> call, Throwable t) {
                    callback.onError(t);
                }
            }, SaleQueryResultData.class);
        } else {
            Request<SaleQueryRequestData> request = SaleQueryRequestData.request(saleNo, days, posNo);
            WebApi.requestAsync(WebApi.create(ISaleService.class).query(request), new Callback<Result<SaleQueryResultData>>() {
                @Override
                public void onResponse(Call<Result<SaleQueryResultData>> call, Response<Result<SaleQueryResultData>> response) {
                    if (response.body() == null) {
                        callback.onError(new Throwable("流水查询返回参数为空"));
                        return;
                    }
                    try {
                        callback.onSuccess(SaleQueryResultData.handleResult(response.body().data));
                    } catch (Exception e) {
                        callback.onError(new Throwable(e.getMessage()));
                    }
                }

                @Override
                public void onFailure(Call<Result<SaleQueryResultData>> call, Throwable t) {
                    callback.onError(t);
                }
            }, SaleQueryResultData.class);
        }


    }

    /**
     * 满送赠券。
     *
     * @param callback 满送赠券的回调方法
     */
    public static void rebate(final com.hisense.webapi.Callback<Collection<ICoupon>> callback) {
        Request<SaleRebateRequestData> request = SaleRebateRequestData.request();

        WebApi.requestAsync(WebApi.create(ISaleService.class).rebate(request), new Callback<Result<List<SaleRebateResultCoupon>>>() {
            @Override
            public void onResponse(Call<Result<List<SaleRebateResultCoupon>>> call, Response<Result<List<SaleRebateResultCoupon>>> response) {
                callback.onSuccess(SaleRebateResultCoupon.handleResult(response.body().data));
            }

            @Override
            public void onFailure(Call<Result<List<SaleRebateResultCoupon>>> call, Throwable t) {
                callback.onError(t);
            }
        }, SaleRebateResultCoupon.class);
    }

    /**
     * 上传退货流水。
     *
     * @param isUseRpt 是否使用归集接口
     * @param isToVip  是否会员流水
     * @param callback 流水上传的回调方法
     */
    public static void returns(boolean isUseRpt, boolean isToVip, ReturnSaleModel sale, final com.hisense.webapi.Callback<Object> callback) {
        if (isUseRpt) {
            LoggerUtils.getLogger().debug("rpt-----上传退货流水");
            Request<SaleSubmitRequestData> request = SaleSubmitRequestData.request(isToVip, sale);
            ISaleService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ISaleService.class);
            WebApi.requestAsync(service.rptReturns(request), new Callback<Result<Object>>() {
                @Override
                public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                    LoggerUtils.getLogger().debug("rpt-----上传退货流水成功");
                    callback.onSuccess(null);
                }

                @Override
                public void onFailure(Call<Result<Object>> call, Throwable t) {
                    callback.onError(t);
                }
            }, Object.class);
        } else {
            Request<SaleSubmitRequestData> request = SaleSubmitRequestData.request(isToVip, sale);
            ISaleService service = WebApi.newInstance(120, TimeUnit.SECONDS).createService(ISaleService.class);
            WebApi.requestAsync(service.returns(request), new Callback<Result<Object>>() {
                @Override
                public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                    callback.onSuccess(null);
                }

                @Override
                public void onFailure(Call<Result<Object>> call, Throwable t) {
                    callback.onError(t);
                }
            }, Object.class);
        }

    }

    /**
     * 完成交易（可用做 限量控制）
     *
     * @param saleNo    流水号
     * @param vipCardNo 会员卡号
     * @param saleDate  销售日期
     * @param pluDsc    商品优惠
     * @param callback  回调函数
     */
    public static void saleComplete(String saleNo, String vipCardNo, String saleDate, List<CompletePluDscData> pluDsc, final com.hisense.webapi.Callback<Object> callback) {
        Request<SaleCompleteRequestData> request = SaleCompleteRequestData.request(saleNo, vipCardNo, saleDate, pluDsc);

        WebApi.requestAsync(WebApi.create(ISaleService.class).complete(request), new Callback<Result<Object>>() {
            @Override
            public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("完成交易返回参数为空"));
                    return;
                }

                callback.onSuccess(new Object());
            }

            @Override
            public void onFailure(Call<Result<Object>> call, Throwable t) {
                callback.onError(t);
            }
        }, Object.class);
    }

    /**
     * 全场限量查询接口(查询方案可用兑换次数)
     * （不限量的返回int 最大值，其他返回剩余次数，不做精确控制，可能为负）
     *
     * @param saleNo    流水号
     * @param vipCardNo 会员卡号
     * @param saleDate  销售日期
     * @param pluDsc    商品优惠
     * @param callback  回调函数
     */
    public static void globalTypeQuery(String saleNo, String vipCardNo, String saleDate, List<GlobalTypeQueryPluDscData> pluDsc, final com.hisense.webapi.Callback<List<GlobalTypeQueryResultData>> callback) {
        Request<GlobalTypeQueryRequestData> request = GlobalTypeQueryRequestData.request(saleNo, vipCardNo, saleDate, pluDsc);

        WebApi.requestAsync(WebApi.create(ISaleService.class).globalTypeQuery(request), new Callback<Result<List<GlobalTypeQueryResultData>>>() {
            @Override
            public void onResponse(Call<Result<List<GlobalTypeQueryResultData>>> call, Response<Result<List<GlobalTypeQueryResultData>>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("全场限量查询返回参数为空"));
                    return;
                }

                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<List<GlobalTypeQueryResultData>>> call, Throwable t) {
                callback.onError(t);
            }
        }, GlobalTypeQueryResultData.class);
    }

    /**
     * 礼品券查询接口
     *
     * @param orgCode   组织编号
     * @param certiCode 礼品券码
     * @param posNo     柜台编码
     * @param saleNo    流水号
     * @param sign      sign值
     */
    public static void giftQuery(String orgCode, String certiCode, String posNo, String saleNo, String sign, final com.hisense.webapi.Callback<CertiDtlQueryResultData> callback) {
        Request<CertiDtlQueryRequestData> request = CertiDtlQueryRequestData.request(orgCode, certiCode, posNo, saleNo, sign);

        WebApi.requestAsync(WebApi.create(ISaleService.class).giftQuery(request), new Callback<Result<CertiDtlQueryResultData>>() {
            @Override
            public void onResponse(Call<Result<CertiDtlQueryResultData>> call, Response<Result<CertiDtlQueryResultData>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("全场限量查询返回参数为空"));
                    return;
                }
                callback.onSuccess(response.body().data);
            }

            @Override
            public void onFailure(Call<Result<CertiDtlQueryResultData>> call, Throwable t) {
                callback.onError(t);
            }
        }, CertiDtlQueryResultData.class);
    }

    /**
     * 礼品券核销接口
     *
     * @param orgCode   组织编号
     * @param certiCode 礼品券码
     * @param sign      sign值
     */
    public static void consumeCerti(String orgCode, String certiCode, String saleNo, String bizBillNo, String origOrderAmount, String consumeAmount, String vipInfoId, String sign, final com.hisense.webapi.Callback<Object> callback) {
        Request<CertiConsumeRequestData> request = CertiConsumeRequestData.request(orgCode, certiCode, saleNo, bizBillNo, origOrderAmount, consumeAmount, vipInfoId, sign);

        WebApi.requestAsync(WebApi.create(ISaleService.class).consumeCerti(request), new Callback<Result<Object>>() {
            @Override
            public void onResponse(Call<Result<Object>> call, Response<Result<Object>> response) {
                if (response.body() == null) {
                    callback.onError(new Throwable("全场限量查询返回参数为空"));
                    return;
                }
                callback.onSuccess(new Object());
            }

            @Override
            public void onFailure(Call<Result<Object>> call, Throwable t) {
                callback.onError(t);
            }
        }, CertiDtlQueryResultData.class);
    }

}
