/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.sale;

import com.hisense.hipos.model.BaseSaleModel;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 流水上传请求数据。
 *
 * @author zhangsai    2018-07-26 HIPOSAPP-698
 * @author zhangsai    2018-12-26 HIPOSAPP-931 生成 WebAPI 请求时删除营业员参数
 * @author zhangsai    2019-01-02 HIPOSAPP-947 修复商品流水表应收/实收金额记录错误
 * @author guolina     2019-04-03 HIPOSAPP-1077 增加dataSrc字段区分不同应用的流水，非必传，不传默认为1，0 - 普通POS，1 - PADPOS，2 - 易收银，3 - 慧收银
 * @author wangyaogang 2019-06-05 HIPOSAPP-1154 易收银销售小票-打印商品"货号"字段，
 * @author zhangsai    2019-06-13 HIPOSAPP-1169 Sale.payments() 属性返回类型改为 ISalePayment
 * @author wangyaogang 2019-06-14 HIPOSAPP-1098 流水上传时plu中增加AuthNo字段
 * @author wangyaogang 2019-08-22 HIPOSAPP-1286 流水上传接口增加vipCardNo
 * @author wangyaogang 2019-08-30 HIPOSAPP-1284 流水上传时Remark传实际值
 * @author dukunbiao(null)  2019-08-30  重构
 */
public class SaleSubmitRequestData {

    /**
     * 数据类型。
     */
    public int dataType;

    /**
     * 交易数据集合。
     */
    public ArrayList<SaleData> sale;

    /**
     * 交易商品集合。
     */
    public ArrayList<SalePluData> plu;

    /***
     * 交易商品优惠集合。
     */
    public ArrayList<SalePluDiscountData> pluDsc;

    /**
     * 交易支付信息集合。
     */
    public ArrayList<SalePayData> pay;

    /**
     * 返回一个新的 WebAPI 请求。
     *
     * @param isToVip 是否上传会员系统
     * @return WebAPI 请求
     */
    public static Request<SaleSubmitRequestData> request(boolean isToVip, BaseSaleModel saleModel) {
        SaleSubmitRequestData data = new SaleSubmitRequestData();
        data.dataType = (isToVip) ? 1 : 0;
        data.sale = SaleData.buildSaleData(saleModel);
        data.plu = SalePluData.buildSalePluData(saleModel);
        data.pay = SalePayData.buildSalePayData(saleModel);
        if (!isToVip) {
            data.pluDsc = SalePluDiscountData.buildSalePluDscData(saleModel);
        }

        return WebApi.sign(data);
    }

    public static Request<SaleSubmitRequestData> request(SaleSubmitRequestData requestData) {
        SaleSubmitRequestData data = new SaleSubmitRequestData();
        data.dataType = requestData.dataType;
        data.sale = SaleData.buildSaleData(requestData.sale);
        data.plu = SalePluData.buildSalePluData(requestData.plu);
        data.pay = SalePayData.buildSalePayData(requestData.pay);
        data.pluDsc = SalePluDiscountData.buildSalePluDscData(requestData.pluDsc);

        return WebApi.sign(data);
    }

    /**
     * 返回一个新的 WebAPI 请求。流水重传
     *
     * @param isToVip 是否上传会员系统
     * @return WebAPI 请求
     */
    public static Request<SaleSubmitRequestData> request(List<SaleData> saleDataList, List<SalePluData> salePluDataList, List<SalePayData> payDataList, List<SalePluDiscountData> dscs, boolean isToVip) {
        SaleSubmitRequestData data = new SaleSubmitRequestData();
        data.dataType = (isToVip) ? 1 : 0;
        data.sale = new ArrayList<>(saleDataList);
        data.plu = new ArrayList<>(salePluDataList);
        data.pay = new ArrayList<>(payDataList);
        data.pluDsc = new ArrayList<>(dscs);

        return WebApi.sign(data);
    }

    private static double calcTotal(double d1, double d2) {
        return new BigDecimal(d1).add(new BigDecimal(d2)).doubleValue();
    }
}
