package com.hisense.hipos.webapi.sale;

import com.hisense.hipos.model.BaseSalePluModel;
import com.hisense.hipos.model.MainSaleModel;
import com.hisense.hipos.model.SaleModel;
import com.hisense.hipos.plu.PluV2;
import com.hisense.hipos.sale.CalcType;
import com.hisense.hipos.webapi.Context;
import com.hisense.hipos.webapi.RSAService;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;

import java.util.ArrayList;
import java.util.List;

import lombok.Data;

/**
 * 满减促销请求
 *
 * @author xieliyan 2018-07-09
 */
@Data
public class TotalReductionRequestData {

    public String cardType;             //卡类型
    public String orgCode;              //组织编码
    public List<PluV2> plu;             //交易商品
    public String posNo;                //收款机号
    public String saleNo;               //流水号
    public String saleTime;             //销售日期


    /**
     * 返回一个新的 WebAPI 请求。（中山益华）
     *
     * @return WebAPI 请求
     */
    public static Request<TotalReductionRequestData> request(CalcType calcType, List<BaseSalePluModel> pluModels) {
        SaleModel saleModel = MainSaleModel.getInstance().getSaleModel();

        TotalReductionRequestData saleReductionRequestData = new TotalReductionRequestData();
        saleReductionRequestData.plu = new ArrayList<>();

        if (saleModel.getVip() != null) {
            saleReductionRequestData.cardType = saleModel.getVip().getCard().cardType().code();
            if (WebApi.isAmwAndAip()) {
            }

        }
        saleReductionRequestData.orgCode = Context.orgCode();

        for (BaseSalePluModel pluModel : pluModels) {
            PluV2 pluItem = new PluV2();
            pluItem.brandCode = pluModel.getBrandCode();
            pluItem.classCode = pluModel.getClassCode();
            pluItem.depCode = pluModel.getDepCode();
            pluItem.depId = pluModel.getDepId();
            pluItem.itemNo = pluModel.getItemNo();
            pluItem.pageNo = pluModel.getPageNo();
            pluItem.pluCode = RSAService.encrypt(pluModel.getPluCode());
            pluItem.pluId = RSAService.encrypt(pluModel.getPluId());
            pluItem.price = RSAService.encrypt(pluModel.getPrice());
            pluItem.quantity = RSAService.encrypt(pluModel.getQuantity());
            if (!WebApi.isAmwAndAip()) {
                pluItem.shopCode = pluModel.getShopCode();
                pluItem.shopId = pluModel.getShopId();
            }

            pluItem.total = RSAService.encrypt(pluModel.getTotal());
            pluItem.tagPrice = RSAService.encrypt(pluModel.getTagPrice());
            pluItem.vipPrice = pluModel.getVipPrice();

            saleReductionRequestData.plu.add(pluItem);


        }

        saleReductionRequestData.posNo = Context.posNo();
        saleReductionRequestData.saleNo = RSAService.encrypt(saleModel.getSaleNo());
        saleReductionRequestData.saleTime = saleModel.getSaleTime();

        return WebApi.sign(saleReductionRequestData);
    }
}
