/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.service;

import com.hisense.webapi.Callback;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.RetryCallback;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.Locale;

import retrofit2.Call;
import retrofit2.Response;

/**
 * WebAPI 服务。
 *
 * @author zhangsai    2017-09-02
 * @author zhangsai    2017-10-25 增加服务在线检测异步调用
 * @author zhangsai    2018-03-28 HIPOSAPP-369 更改 online 接口返回结果
 * @author zhangsai    2018-05-04 HIPOSAPP-433 服务在线检测接口返回结果新增业务系统代码
 * @author zhangsai    2018-06-16 HIPOSAPP-552 增加在线检测重试方法
 * @author guolina     2018-08-29 HIPOSAPP-731 修复因服务未连接导致的闪退
 * @author wangyaogang 2019-05-24 HIPOSAPP-1127 优化获取WebAPI 服务实例的方法
 * @author wangyaogang 2019-08-09 HIPOSAPP-1264 调整online接口超时时间
 */
public class Service {

    private Service() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 检查服务是否在线。如果无法连接服务，则自动重试。
     *
     * @param maxRetry 最大重试次数
     * @param interval 两次重试之间的时间间隔(毫秒)
     * @param callback 回调方法
     */
    public static void online(final int maxRetry, final int interval,
                              final RetryCallback<OnlineResultData> callback) {
        online(callback, maxRetry < 1 ? 1 : maxRetry, 1, interval);
    }

    /**
     * 检查服务是否在线。如果无法连接服务，则自动重试。
     *
     * @param callback     回调方法
     * @param maxRetry     最大重试次数
     * @param currentRetry 当前重试次数
     * @param interval     两次重试之间的时间间隔(毫秒)
     */
    private static void online(final RetryCallback<OnlineResultData> callback,
                               final int maxRetry, final int currentRetry, final int interval) {
        Service.online(new Callback<OnlineResultData>() {
            @Override
            public void onSuccess(OnlineResultData s) {
                callback.onSuccess(s);
            }

            @Override
            public void onError(Throwable throwable) {
                if (maxRetry == currentRetry) {
                    callback.onError(throwable);
                } else {
                    callback.onProgress(currentRetry, String.format(Locale.CHINESE,
                            "[%d/%d]无法连接后台服务。正在重试", currentRetry, maxRetry));
                    try {
                        Thread.sleep(interval);
                        online(callback, maxRetry, currentRetry + 1, interval);
                    } catch (InterruptedException ex) {
                        callback.onError(ex);
                    }
                }
            }
        });
    }

    public static void online(final Callback<OnlineResultData> callback) {
        Request request = WebApi.sign();
        WebApi.requestAsync(WebApi.create(IService.class).online(request), new retrofit2.Callback<Result<OnlineResultData>>() {
            @Override
            public void onResponse(Call<Result<OnlineResultData>> call, Response<Result<OnlineResultData>> response) {
                OnlineResultData resultData = new OnlineResultData();
                if (response.body() != null && response.body().data != null) {
                    resultData = response.body().data;
                    resultData.apiVersion = resultData.apiVersion == null ? "5.3.6.76.17" : resultData.apiVersion;
                }
                callback.onSuccess(resultData);
            }

            @Override
            public void onFailure(Call<Result<OnlineResultData>> call, Throwable t) {
                callback.onError(t);
            }
        },OnlineResultData.class);
    }
}
