/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.template;

import com.hisense.hipos.template.models.PrintTemplate;
import com.hisense.hipos.template.models.PrintTemplateType;
import com.hisense.webapi.Request;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

/**
 * 打印模板服务。
 *
 * @author zhangsai 2017-09-30
 */
public class TemplateService {

    private TemplateService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 检查本地打印模板版本是否与服务器版本一致。如果不一致，则返回最新版本的模板内容。
     *
     * @param type    打印模板类型
     * @param version 本地打印模板的版本号
     * @return 服务器最新版本的模板内容。如果服务器版本与本地一致，则返回 null。
     * @throws WebApiException 如果 WebAPI 执行过程中产生错误或返回错误。
     */
    public static PrintTemplate checkUpdate(PrintTemplateType type, int version) throws WebApiException {
        CheckUpdateRequestData requestData = new CheckUpdateRequestData();
        requestData.clientVersion = version;
        requestData.type = type.value();

        Request<CheckUpdateRequestData> request = WebApi.sign(requestData);

        CheckUpdateResultData resultData = WebApi.request(WebApi.create(ITemplateService.class).checkUpdate(request));
        if (resultData != null) {
            return new PrintTemplate(type, resultData.version, resultData.content);
        }

        return null;
    }
}
