/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.webapi.vip;

import com.hisense.hipos.coupon.Coupon;
import com.hisense.webapi.Request;
import com.hisense.webapi.Result;
import com.hisense.webapi.WebApi;
import com.hisense.webapi.WebApiException;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * 会员服务。
 *
 * @author zhangsai    2017-11-22
 * @author wangyaogang 2019-06-27 HIPOSAPP-1181 生成券信息时,若可用金额大于券的面值,进行提示
 */
public class VipService {

    private VipService() {
        // NO CODE HERE. 避免初始化。
    }

    /**
     * 返回指定会员当前交易的可使用赠券。
     *
     * @param cardFaceNo 会员卡面号
     * @param callback   会员赠券查询的回调方法
     */
    public static void getCoupons(String cardFaceNo, final com.hisense.webapi.Callback<List<Coupon>> callback) {
        Request<VipCouponsRequestData> request = VipCouponsRequestData.request(cardFaceNo);

        WebApi.requestAsync(WebApi.create(IVipService.class).getCoupons(request), new Callback<Result<List<VipCouponsResultCoupon>>>() {
            @Override
            public void onResponse(Call<Result<List<VipCouponsResultCoupon>>> call, Response<Result<List<VipCouponsResultCoupon>>> response) {
                try {
                    List<Coupon> coupons = new ArrayList<>();
                    for (VipCouponsResultCoupon resultCoupon : response.body().data) {
                        coupons.add(VipCouponsResultCoupon.handlerResult(resultCoupon));
                    }

                    callback.onSuccess(coupons);
                } catch (Exception e) {
                    callback.onError(new Throwable(e.getMessage()));
                }
            }

            @Override
            public void onFailure(Call<Result<List<VipCouponsResultCoupon>>> call, Throwable t) {
                callback.onError(t);
            }
        },VipCouponsResultCoupon.class);
    }
}
