/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import java.io.Serializable;

import lombok.Getter;

/**
 * 赠券的基类。
 *
 * @author zhangsai    2018-11-13 HIPOSAPP-822
 * @author guolina     2018-11-20 HIPOSAPP-833 解决因券号为空导致的赠券核销错误
 * @author guolina     2019-03-25 HIPOSAPP-1048 修复手动添加会员券再次选中时未传校验码导致的用券失败
 * @author wangyaogang 2019-06-12 HIPOSAPP-1114 增加qType券类型字段
 * @author wangyaogang 2019-06-24 HIPOSAPP-1090 增加活动描述字段
 */
@Getter
public abstract class CouponBase implements ICoupon, Serializable {
    // 券号
    protected String qNo;

    // 校验码
    protected String checkCode;

    // 名称
    protected String name;
    // 开始日期
    protected String beginDate;
    // 截止日期
    protected String endDate;

    // 是否打印
    protected boolean isPrint;
    // 备注
    protected String remark;

    // 赠券类型
    protected CouponType qType;

    //活动描述
    protected String couponDesc;



    /**
     * 返回赠券类型。
     *
     * @return 赠券类型
     */
    public CouponType type() {
        return this.qType;
    }

    /**
     * 返回赠券名称。
     *
     * @return 赠券名称
     */
    public String name() {
        return this.name;
    }

    /**
     * 返回券号。
     *
     * @return 券号
     */
    public String no() {
        return this.qNo;
    }

    /**
     * 返回校验码。
     *
     * @return 校验码
     */
    public String checkCode() {
        return this.checkCode;
    }

    /**
     * 返回使用开始日期。
     *
     * @return 使用开始日期
     */
    public String beginDate() {
        return this.beginDate;
    }

    /**
     * 返回使用截止日期。
     *
     * @return 使用截止日期
     */
    public String endDate() {
        return this.endDate;
    }

    /**
     * 返回是否打印赠券。
     *
     * @return true 表示打印；false 表示不打印
     */
    public boolean isPrint() {
        return this.isPrint;
    }

    /**
     * 返回备注。
     *
     * @return 备注
     */
    public String remark() {
        return this.remark;
    }

    /**
     * 活动描述
     *
     * @return 活动描述
     */
    public String couponDesc() {
        return couponDesc;
    }


    public CouponType qType() {
        return qType;
    }

    public void qType(CouponType qType) {
        this.qType = qType;
    }
}
