/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import java.io.Serializable;
import java.security.InvalidParameterException;

/**
 * 赠券类型。
 *
 * @author zhangsai    2017-11-24
 * @author wangyaogang 2019-06-12 HIPOSAPP-1114 增加枚举值 Gift(1) 礼品券
 */
public enum CouponType implements Serializable {

    /**
     * 金额券。
     */
    Cash(0),
    /**
     * 礼品券
     */
    Gift(1),
    /**
     * 折扣券。
     */
    Discount(2);

    private int value;

    /**
     * 初始化一个 CouponType 枚举。
     *
     * @param value 赠券类型对应的值。
     */
    CouponType(int value) {
        this.value = value;
    }

    /**
     * 返回赠券类型对应的值。
     *
     * @return 赠券类型对应的值
     */
    public int value() {
        return this.value;
    }

    /**
     * 返回值对应的赠券类型。
     *
     * @param value 待转换的值
     * @return 值对应的赠券类型
     */
    public static CouponType valueOf(int value) {
        if (value == CouponType.Cash.value()) {
            return CouponType.Cash;
        } else if (value == CouponType.Discount.value()) {
            return CouponType.Discount;
        } else if (value == CouponType.Gift.value()) {
            return CouponType.Gift;
        } else {
            throw new InvalidParameterException("无效的赠券类型的值");
        }
    }
}
