/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import java.io.Serializable;

/**
 * 礼品券。
 *
 * @author zhangsai    2018-11-13 HIPOSAPP-822
 * @author wangyaogang 2019-06-12 HIPOSAPP-1114 增加qType券类型字段
 * @author wangyaogang 2019-06-24 HIPOSAPP-1090 增加活动描述.礼品名称字段
 */
public class GiftCoupon extends CouponBase {

    // 赠送张数
    private int count;
    //礼品名字
    private String giftName;

    /**
     * 返回赠送张数。
     *
     * @return 赠送张数
     */
    public int count() {
        return this.count;
    }

    /**
     * 返回礼品名字
     *
     * @return 礼品名字
     */
    public String giftName() {
        return this.giftName;
    }

    /**
     * 使用 Builder 模式初始化一个 {@code GiftCoupon} 的实例。
     */
    public static class Builder implements Serializable {
        // 券号
        private String qNo;
        // 赠券张数
        private int count;
        // 名称
        private String name;
        // 开始日期
        private String beginDate;
        // 截止日期
        private String endDate;

        // 是否打印
        private boolean isPrint;
        // 备注
        private String remark;

        //礼品名字
        private String giftName;
        //活动描述
        private String couponDesc;

        /**
         * 初始化一个 {@code Coupon.Builder} 的实例。
         */
        public Builder() {
        }

        /**
         * 设置券号、校验码和名称。
         *
         * @param qNo  券号
         * @param name 券名称
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder coupon(String qNo, String name) {
            this.qNo = qNo;
            this.name = name;
            return this;
        }

        /**
         * 设置赠送张数。
         *
         * @param value 赠送张数
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder count(int value) {
            this.count = value;
            return this;
        }

        /**
         * 设置赠券使用开始日期。
         *
         * @param date 赠券使用开始日期
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder beginAt(String date) {
            this.beginDate = date;
            return this;
        }

        /**
         * 设置赠券使用截止日期。
         *
         * @param date 赠券使用截止日期
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder endsAt(String date) {
            this.endDate = date;
            return this;
        }

        /**
         * 设置是否打印赠券。
         *
         * @param value true 表示打印；false 表示不打印
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder isPrint(boolean value) {
            this.isPrint = value;
            return this;
        }

        /**
         * 设置备注。
         *
         * @param value 备注
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder remark(String value) {
            this.remark = value;
            return this;
        }

        /**
         * 设置礼品名字。
         *
         * @param giftName 备注
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder giftName(String giftName) {
            this.giftName = giftName;
            return this;
        }

        /**
         * 设置活动描述。
         *
         * @param couponDesc 活动描述
         * @return {@code Coupon.Builder} 的实例。
         */
        public Builder couponDesc(String couponDesc) {
            this.couponDesc = couponDesc;
            return this;
        }

        /**
         * 构建一个 {@code GiftCoupon} 的实例。
         */
        public GiftCoupon build() {
            GiftCoupon coupon = new GiftCoupon();
            coupon.qNo = this.qNo;
            coupon.name = this.name;
            coupon.count = this.count;
            coupon.beginDate = this.beginDate;
            coupon.endDate = this.endDate;
            coupon.isPrint = this.isPrint;
            coupon.remark = this.remark;
            coupon.qType = CouponType.Gift;
            coupon.giftName = this.giftName;
            coupon.couponDesc = this.couponDesc;
            return coupon;
        }
    }
}
