/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.coupon;

import java.io.Serializable;

/**
 * 赠券商品分摊。
 *
 * @author zhangsai 2017-11-22
 * @author zhangsai 2017-12-20 HIPOSAPP-245
 */
public class SaleItemShare implements Serializable {
    // 页号
    private int pageNo;
    // 行号
    private int itemNo;
    // 商品编码
    private String pluCode;
    // 商品 ID
    private String pluId;
    // 分摊金额
    private double value;
    // 核销金额
    private double useValue;
    // 应收合计
    private double total;
    // 支付金额
    private double totalPaid;


    /**
     * 返回页号。
     *
     * @return 页号
     */
    public int pageNo() {
        return this.pageNo;
    }

    /**
     * 返回行号。
     *
     * @return 行号
     */
    public int itemNo() {
        return this.itemNo;
    }

    /**
     * 返回商品编码。
     *
     * @return 商品编码
     */
    public String pluCode() {
        return this.pluCode;
    }

    /**
     * 返回商品 ID。
     *
     * @return 商品 ID
     */
    public String pluId() {
        return this.pluId;
    }

    /**
     * 返回分摊金额。
     *
     * @return 分摊金额
     */
    public double value() {
        return this.value;
    }

    /**
     * 返回核销金额。
     *
     * @return 核销金额
     */
    public double useValue() {
        return this.useValue;
    }

    /**
     * 核销分摊金额。
     *
     * @param value 核销金额
     */
    public void use(double value) {
        if (value < 0 | value > this.value) {
            throw new IllegalArgumentException("核销金额不能大于分摊金额");
        }

        this.useValue = value;
    }

    /**
     * 取消核销分摊。
     *
     * @return 核销金额
     */
    public double cancel() {
        double cancelled = Double.valueOf(this.useValue).doubleValue();
        this.useValue = 0;
        return cancelled;
    }

    /**
     * 返回应收合计。
     *
     * @return 应收合计
     */
    public double total() {
        return this.total;
    }

    /**
     * 返回合计支付金额。
     *
     * @return 合计支付金额
     */
    public double totalPaid() {
        return this.totalPaid;
    }


    /**
     * 使用 Builder 模式初始化一个 {@code SaleItemShare} 的实例。
     */
    public static class Builder implements Serializable {
        // 页号
        private int pageNo;
        // 行号
        private int itemNo;
        // 商品编码
        private String pluCode;
        // 商品 ID
        private String pluId;
        // 分摊金额
        private double value;
        // 应收合计
        private double total;
        // 支付金额
        private double totalPaid;

        /**
         * 初始化一个 {@code SaleItemShare.Builder} 的实例。
         */
        public Builder() {

        }

        /**
         * 设置销售商品。
         *
         * @param pluCode 商品编码
         * @param pluId   商品 ID
         * @return {@code SaleItemShare.Builder} 的实例。
         */
        public Builder plu(String pluCode, String pluId) {
            this.pluCode = pluCode;
            this.pluId = pluId;
            return this;
        }

        /**
         * 设置商品在流水中的位置。
         *
         * @param pageNo 页号
         * @param itemNo 行号
         * @return {@code SaleItemShare.Builder} 的实例。
         */
        public Builder item(int pageNo, int itemNo) {
            this.pageNo = pageNo;
            this.itemNo = itemNo;
            return this;
        }

        /**
         * 设置分摊金额。
         *
         * @param value 分摊金额
         * @return {@code SaleItemShare.Builder} 的实例。
         */
        public Builder share(double value) {
            this.value = value;
            return this;
        }

        public Builder total(double total, double totalPaid) {
            this.total = total;
            this.totalPaid = totalPaid;
            return this;
        }

        public Builder total(String total, String totalPaid) {
            this.total = Double.parseDouble(total);
            this.totalPaid = Double.parseDouble(totalPaid);
            return this;
        }

        /**
         * 构建一个 {@code SaleItemShare} 的实例。
         */
        public SaleItemShare build() {
            SaleItemShare share = new SaleItemShare();
            share.pluCode = this.pluCode;
            share.pluId = this.pluId;
            share.pageNo = this.pageNo;
            share.itemNo = this.itemNo;
            share.value = this.value;
            share.total = this.total;
            share.totalPaid = this.totalPaid;

            return share;
        }

    }
}
