package com.hisense.hipos.model;

import com.alibaba.fastjson.JSONObject;
import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.crm.card.VipCard;
import com.hisense.crm.vip.Vip;
import com.hisense.hipos.coupon.HxPayCoupon;
import com.hisense.hipos.payment.PayMethod;
import com.hisense.hipos.plu.PluDscType;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.user.User;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import lombok.Data;

/**
 * 重构之后的 SaleModel
 *
 * @author dukunbiao(null)  2019-08-16
 * @author wangyaogang      2019-12-27 HIAPOSCMP-455 标准版支付功能开发,增加对找零的处理
 * @author guolina          2020-02-27 HIAPOSCMP-553 与中台联调商品查询和商品促销计算接口
 * @author guolina          2020-05-18 HIAPOSCMP-702 吉麦隆淘鲜达营销方案对接涉及慧收银调整增加支付明细和赠券打印
 */
@Data
public class BaseSaleModel implements Serializable {

    protected String posNo;                               //收款机号
    protected User cashier;                               //收银员

    protected String accDate;                             //结账日期
    protected String saleNo;                              //流水号
    protected String invoiceNo;                           //小票号

    protected String saleDate;                            //销售日期，yyyy-MM=dd
    protected String saleTime;                            //销售时间，yyyy-MM=dd hh:mm:ss

    protected String total;                               //应收合计
    protected String totalDsc;                            //优惠合计
    protected String totalRecv;                           //实收合计
    protected String totalPaid;                           //支付合计
    protected String totalUnPaid;                         //未付金额
    protected String totalRound;                          //抹零合计
    protected String totalChange;                         //找零合计

    protected TranType tranType;                          //交易类型

    protected String remark;                              //备注
    protected String udp1;                                //扩展字段1
    protected String udp2;                                //扩展字段2
    protected String udp3;                                //扩展字段3
    protected String udp4;                                //扩展字段4
    protected String udp5;                                //扩展字段5

    //打印使用
    protected String expensePoint;                        //消费积分
    protected String parkingPoint;                        //停车积分
    protected String couponPoint;                         //赠券积分
    protected String lastUsablePoint;                     //上次可用积分
    protected String vipPoints;                           //当前累计积分

    protected Vip vip;                                    //FIXME null 会员

    protected List<BaseSalePluModel> plus = new ArrayList<>();
    protected List<SalePayModel> pays = new ArrayList<>();
    protected List<SalePayModel> paysPrint = new ArrayList<>();

    protected List<SalePluDscModel> pluDscs = new ArrayList<>();    ////用于保存汇总之后的优惠信息，如果取某个商品的优惠信息取plus.pluDscs

    protected PayMethod payMethod;
    private List<HxPayCoupon> hxPayCoupons = new ArrayList<>();        //石基淘鲜达购后返券

    public void setPlus(List<BaseSalePluModel> plus) {
        this.plus = plus;
        calcTotal();
    }

    public void setPays(List<SalePayModel> pays) {
        this.pays = pays;
        calcUnPaid();
    }

    public void setPaysPrint(List<SalePayModel> paysPrint) {
        this.paysPrint = paysPrint;
    }

    public void addPlu(BaseSalePluModel pluModel) {
        if (this.plus == null) {
            this.plus = new ArrayList<>();
        }
        if (pluModel.getItemNo() == 0) {
            pluModel.setPageNo(1);
            pluModel.setItemNo(plus.size() + 1);
            pluModel.setTotalPaid("0");
        }
        if (!ListUtils.isEmpty(pluModel.pluDscs)) {
            for (SalePluDscModel pluDscModel : pluModel.pluDscs) {
                pluDscModel.setPageNo(pluModel.pageNo);
                pluDscModel.setItemNo(pluModel.itemNo);
            }
        }

        plus.add(pluModel);

        calcTotal();
    }

    public void addHxPayCoupon(HxPayCoupon hxPayCoupon) {
        if (this.hxPayCoupons == null) {
            this.hxPayCoupons = new ArrayList<>();
        }


        hxPayCoupons.add(hxPayCoupon);

    }

    public void clearPlu() {
        this.plus.clear();
        calcTotal();
    }

    public void addPluBySql(BaseSalePluModel pluModel) {
        if (this.plus == null) {
            this.plus = new ArrayList<>();
        }

        plus.add(pluModel);
    }

    public void updatePlu(BaseSalePluModel pluModel) {
        int index = queryIndex(pluModel);

        updatePlu(index, pluModel);
    }

    public void updatePlu(int index, BaseSalePluModel pluModel) {
        if (this.plus == null) {
            this.plus = new ArrayList<>();
            return;
        }
        if (plus.size() > index) {
            plus.set(index, pluModel);
        } else {
            plus.add(index, pluModel);
        }


        calcTotal();
    }

    public void changeQuantity(int index, int quantity) {
        this.getPlu(index).changeQuantity(quantity);

        calcTotal();
    }

    public void changeQuantityReturn(int index, int quantity) {
        ReturnSalePluModel returnSalePluModel = (ReturnSalePluModel) getPlu(index);
        returnSalePluModel.changeQuantity(quantity);

        calcTotal();
    }

    public BaseSalePluModel getPlu(int index) {
        return plus.get(index);
    }

    public BaseSalePluModel getLastPlu() {
        return plus.get(plus.size() - 1);
    }

    public BaseSalePluModel getLastPluWithFen() {
        for (int i = plus.size() - 1; i >= 0; i--) {
            BaseSalePluModel pluModel = plus.get(i);
            if (pluModel.isHasFen()) {
                return pluModel;
            }
        }

        return getLastPlu();
    }

    public int pluSize() {
        if (plus == null) {
            return 0;
        }

        return plus.size();
    }

    public void removePlu(int index) {
        if (index > plus.size()) {
            return;
        }

        this.plus.remove(index);

        for (int i = index; i < plus.size(); i++) {
            BaseSalePluModel pluModel = plus.get(i);
            pluModel.setItemNo(i + 1);

            pluModel.updatePluDsc();
            this.plus.set(i, pluModel);
        }

        calcTotal();
    }

    public void addAbandonFenDsc(int scale) {
        removeAbandonFenDsc();

        String totalRecv = BigDecimalUtils.formatScale(this.getTotalRecv(), scale);

        if (BigDecimalUtils.isEquals(totalRecv, this.getTotalRecv())) {
            return;
        }

        String totalRecvTemp = this.getTotalRecv();
        String totalDsc = BigDecimalUtils.subtract(totalRecvTemp, totalRecv);
        String totalDscLeft = totalDsc;

        //分摊舍分优惠
        for (int i = 0; i < this.plus.size(); i++) {
            BaseSalePluModel pluModel = plus.get(i);

            String pluTotalDsc;
            //已经是最后一个商品了，剩余优惠直接归入最后一个商品
            if (i == plus.size() - 1) {
                pluTotalDsc = totalDscLeft;
            } else {
                pluTotalDsc = BigDecimalUtils.divide(BigDecimalUtils.multiply(totalDsc, pluModel.totalRecv), totalRecvTemp, 2, BigDecimal.ROUND_DOWN);
            }

            if (BigDecimalUtils.isGreater0(totalDscLeft) && BigDecimalUtils.isGreater(pluTotalDsc, totalDscLeft)) {
                pluTotalDsc = totalDscLeft;
            }
            if (BigDecimalUtils.isLess0(totalDscLeft) && BigDecimalUtils.isGreater(totalDscLeft, pluTotalDsc)) {
                pluTotalDsc = totalDscLeft;
            }

            //如果当前商品可分摊金额为0，不添加优惠
            if (BigDecimalUtils.isEquals0(pluTotalDsc)) {
                continue;
            }

            SalePluDscModel pluDscModel = new SalePluDscModel("", "1", "舍分优惠", pluTotalDsc);
            pluModel.addPluDsc(pluDscModel);
            updatePlu(pluModel);

            totalDscLeft = BigDecimalUtils.subtract(totalDscLeft, pluTotalDsc);
            if (BigDecimalUtils.isEquals0(totalDscLeft)) {
                break;
            }
        }
    }

    public void removeAbandonFenDsc() {
        if (this.plus == null) {
            this.plus = new ArrayList<>();
        }

        for (BaseSalePluModel pluModel : this.plus) {
            if (ListUtils.isEmpty(pluModel.pluDscs)) {
                continue;
            }

            Iterator<SalePluDscModel> iterator = pluModel.pluDscs.iterator();
            while (iterator.hasNext()) {
                SalePluDscModel pluDscModel = iterator.next();

                if ("舍分优惠".equals(pluDscModel.getDscProName())) {
                    iterator.remove();
                }
            }

            pluModel.calcTotalDsc();
        }

        calcTotal();
    }

    public void updatePluDsc(List<SalePluDscModel> pluDscModels) {
        if (ListUtils.isEmpty(pluDscModels)) {
            return;
        }

        for (SalePluDscModel pluDscModel : pluDscModels) {
            int i = 0;
            for (BaseSalePluModel pluModel : this.plus) {
                //保险起见，匹配下行号页号
                //saleModel.getPlu(pluDsc.itemNo - 1).addPluDsc(pluDscModel);
                if (pluModel.pluCode.equals(pluDscModel.getPluCode()) &&
                        pluModel.getItemNo() == (pluDscModel.getItemNo()) &&
                        pluModel.getPageNo() == (pluDscModel.getPageNo())) {
                    pluModel.addPluDsc(pluDscModel);
                    pluDscModel.setPageNo(pluModel.getPageNo());
                    pluDscModel.setItemNo(pluModel.getItemNo());
                    this.updatePlu(i, pluModel);
                }
                i++;
            }
        }
    }

    public String updatePluDscNoMj(List<SalePluDscModel> pluDscModels) {
        if (ListUtils.isEmpty(pluDscModels)) {
            return "0";
        }

        String totalDscMj = "0";
        for (SalePluDscModel pluDscModel : pluDscModels) {
            if (PluDscType.TOTAL_REDUCTION.equals(pluDscModel.getDscProType())) {
                totalDscMj = BigDecimalUtils.add(totalDscMj, pluDscModel.getTotalDsc());
                continue;
            }

            int i = 0;
            for (BaseSalePluModel pluModel : this.plus) {
                if (pluModel.getItemNo() == pluDscModel.getItemNo() && pluModel.getPageNo() == pluDscModel.getPageNo()) {
                    pluModel.addPluDsc(pluDscModel);
                    this.updatePlu(i, pluModel);
                }
                i++;
            }
        }

        return totalDscMj;
    }

    public void clearPluDsc() {
        if (!ListUtils.isEmpty(plus)) {
            for (BaseSalePluModel pluModel : plus) {
                pluModel.clearPluDsc();
            }

            calcTotal();
        }
    }

    public void removeHjDsc() {
        if (ListUtils.isEmpty(plus)) {
            return;
        }

        for (BaseSalePluModel pluModel : plus) {
            pluModel.removeHjDsc();
        }

        calcTotal();
    }

    public void increasePlu(int index) {
        int count = getPlu(index).count() + 1;

        changeQuantity(index, count);
    }

    public void decreasePlu(int index) {
        int count = getPlu(index).count() - 1;

        changeQuantity(index, count);
    }

    public void calcTotal() {
        String total = "0";
        String totalDsc = "0";
        String totalRecv = "0";
        for (int i = 0; i < plus.size(); i++) {
            BaseSalePluModel pluModel = plus.get(i);
            total = BigDecimalUtils.add(total, pluModel.getTotal());
            totalDsc = BigDecimalUtils.add(totalDsc, pluModel.getTotalDsc());
            totalRecv = BigDecimalUtils.add(totalRecv, pluModel.getTotalRecv());
        }

        this.total = BigDecimalUtils.decimalFormat(total);
        this.totalDsc = BigDecimalUtils.decimalFormat(totalDsc);
        this.totalRecv = BigDecimalUtils.decimalFormat(totalRecv);

        calcUnPaid();
    }

    public void calcUnPaid() {
        String totalPaid = "0";
        for (int i = 0; i < pays.size(); i++) {
            SalePayModel payModel = pays.get(i);
            totalPaid = BigDecimalUtils.add(totalPaid, payModel.getTotalPaid());
        }

        this.totalPaid = BigDecimalUtils.decimalFormat(totalPaid);
        this.totalUnPaid = BigDecimalUtils.decimalFormat(BigDecimalUtils.subtract(this.totalRecv, this.totalPaid));

        this.totalRound = BigDecimalUtils.decimalFormat("0");
        this.totalChange = BigDecimalUtils.decimalFormat("0");

        //未付金额小于0时,表明多收,此时需将未付金额修改为0,多收金额赋值给找零(应该只发生在现金支付时)
        if (BigDecimalUtils.isLess0(this.totalUnPaid) && !ListUtils.isEmpty(pays)) {
            this.totalChange = BigDecimalUtils.decimalFormat(BigDecimalUtils.subtract("0", this.totalUnPaid));
            this.totalUnPaid = BigDecimalUtils.decimalFormat("0");
            //存在找零时,修正该支付方式的实收金额
            SalePayModel payModel = pays.get(pays.size() - 1);
            payModel.setTotalRecv(BigDecimalUtils.subtract(payModel.getTotalPaid(), this.totalChange));
        }
    }

    /**
     * 返回合计销售件数。
     *
     * @return 合计销售件数
     */
    public int calcCount() {
        int count = 0;
        for (BaseSalePluModel pluModel : plus) {
            count += pluModel.count();
        }
        return count;
    }

    /**
     * 返回合计销售数量。
     *
     * @return 合计销售数量
     */
    public String calcQuantity() {
        String quantity = "0";
        for (BaseSalePluModel pluModel : plus) {
            quantity = BigDecimalUtils.add(quantity, pluModel.getQuantity());
        }

        return quantity;
    }

    /**
     * 返回指定商品编码的商品行号
     *
     * @param pluCode 商品编码
     * @return pluItem 行号
     */
    public int queryPluItemNo(String pluCode) {
        int pluItemNo = 0;
        for (BaseSalePluModel pluModel : plus) {
            if (pluModel.getPluCode().equals(pluCode)) {
                pluItemNo = pluModel.getItemNo();
                break;
            }
        }

        return pluItemNo;
    }

    public int queryIndex(BaseSalePluModel pluModel) {
        int index = 0;
        for (BaseSalePluModel pluModelTemp : plus) {
            if (pluModel.getItemNo() == pluModelTemp.getItemNo()) {
                return index;
            }
            index++;
        }

        return index;
    }

    public boolean isVip() {
        return this.vip != null;
    }

    public boolean isNotValid() {
        return this.calcCount() < 1 || BigDecimalUtils.isLess(this.totalRecv, "0.001");
    }

    public SalePayModel getPay(int index) {
        return pays.get(index);
    }

    public SalePayModel getPayPringt(int index) {
        return paysPrint.get(index);
    }

    public SalePayModel getLastPay() {
        return pays.get(pays.size() - 1);
    }

    public int paySize() {
        if (pays == null) {
            return 0;
        }

        return pays.size();
    }

    public void removePay(int index) {
        if (index > pays.size()) {
            return;
        }
       if(null!= this.paysPrint){
           Iterator<SalePayModel> iterator = this.paysPrint.iterator();
           while (iterator.hasNext()) {
               SalePayModel salePayModel = iterator.next();
               if (this.pays.get(index).outTradeNo.equals(salePayModel.outTradeNo)) {
                   iterator.remove();
               }
           }
       }


        if(null!= this.hxPayCoupons){
            Iterator<HxPayCoupon> iterator = this.hxPayCoupons.iterator();
            while (iterator.hasNext()) {
                HxPayCoupon hxPayCoupon = iterator.next();
                if (this.pays.get(index).outTradeNo.equals(hxPayCoupon.outTradeNo)) {
                    iterator.remove();
                }
            }
        }

        this.pays.remove(index);

        for (int i = index; i < pays.size(); i++) {
            SalePayModel payModel = pays.get(i);
            payModel.setSerialNo(i + 1);

            this.pays.set(i, payModel);
        }

        calcUnPaid();
    }

    public void removePayPrint(int index) {
        if (index > paysPrint.size()) {
            return;
        }

        this.paysPrint.remove(index);

        for (int i = index; i < paysPrint.size(); i++) {
            SalePayModel payModel = paysPrint.get(i);
            payModel.setSerialNo(i + 1);

            this.paysPrint.set(i, payModel);
        }
    }

    public void removeLastPay() {
        removePay(pays.size() - 1);
    }

    public void removeLastPayPrint() {
        removePayPrint(paysPrint.size() - 1);
    }

    public void addPay(SalePayModel payModel) {
        if (this.pays == null) {
            this.pays = new ArrayList<>();
        }

        payModel.setSerialNo(pays.size() + 1);
        if (payModel.getTotalDscSelf() == null) {
            payModel.setTotalDscSelf("0.00");
        }
        if (payModel.getTotalDscChannel() == null) {
            payModel.setTotalDscChannel("0.00");
        }

        pays.add(payModel);

        calcUnPaid();
    }

    public void addPayPrint(SalePayModel payModel) {
        if (this.paysPrint == null) {
            this.paysPrint = new ArrayList<>();
        }

        payModel.setSerialNo(paysPrint.size() + 1);
        if (payModel.getTotalDscSelf() == null) {
            payModel.setTotalDscSelf("0.00");
        }
        if (payModel.getTotalDscChannel() == null) {
            payModel.setTotalDscChannel("0.00");
        }

        paysPrint.add(payModel);

    }

    public void addPayNotUpdate(SalePayModel payModel) {
        if (this.pays == null) {
            this.pays = new ArrayList<>();
        }

        payModel.setSerialNo(pays.size() + 1);
        if (payModel.getTotalDscSelf() == null) {
            payModel.setTotalDscSelf("0.00");
        }
        if (payModel.getTotalDscChannel() == null) {
            payModel.setTotalDscChannel("0.00");
        }

        pays.add(payModel);

    }

    public void addPayBySql(SalePayModel payModel) {
        if (this.pays == null) {
            this.pays = new ArrayList<>();
        }

        pays.add(payModel);
    }

    public void addPluDscBySql(SalePluDscModel pluDscModel) {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
        }

        pluDscs.add(pluDscModel);
    }

    public void addVip(VipCard card) {
        this.vip = new Vip.Builder("")
                .birthday("")
                .mobile("")
                .message("", "")
                .card(card)
                .build();
    }

    public boolean hasAllPaid() {
        try {
            return BigDecimalUtils.isLess(this.totalUnPaid, "0.001");
        } catch (Exception e) {
            return true;
        }
    }

    public boolean hasAlreadyPaid() {
        try {
            return BigDecimalUtils.isGreater(this.totalPaid, "0.001");
        } catch (Exception e) {
            return false;
        }
    }

    public boolean allowTotalDsc() {
        if (this.isVip()) {
            return true;
        }

        for (BaseSalePluModel pluModel : this.plus) {
            if (pluModel.isTotalDsc()) {
                return true;
            }
        }

        return false;
    }

    public String totalReduction() {
        for (SalePayModel payModel : this.pays) {
            if (PayMethod.CODE_TOTAL_REDUCTION.equals(payModel.getPayCode())) {
                return payModel.getTotalPaid();
            }
        }

        return "0.00";
    }

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }
}
