package com.hisense.hipos.model;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.ListUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hipos.coupon.SaleItemShare;
import com.hisense.hipos.plu.PluType;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import lombok.Data;

/**
 * 重构之后的 BaseSalePluModel
 *
 * @author dukunbiao(null)  2019-08-16
 * @author guolina          2020-01-09 HIAPOSCMP-490 慧收银增加整单促销接口
 * @author guolina          2020-04-21 HIAPOSCMP-649 利客来易收银允许改价需求
 */
@Data
public class BaseSalePluModel implements Serializable {

    protected String barCode;                             //条码
    protected String pluIncode;                           //原始条码
    protected String cargoNo;                             //货号
    protected String pluBarCode;                          //商品条码(称重商品barCode存的大码，pluBarCode为实际条码)
    protected String pluId;                               //商品ID
    protected String pluCode;                             //商品编码
    protected String pluName;                             //商品名称
    protected String pluAbbrName;                          //商品名称简称
    protected String pluMn;                               //商品助记码
    protected PluType pluType;                            //商品类型 0-普通商品；1-商品包；2-商品包明细

    protected String brandCode;                           //品牌编码
    protected String classCode;                           //品类编码

    protected String clkCode;                              //营业员编码
    protected String clkId;                                //营业员 ID

    protected String depCode;                             //部门编码
    protected String depId;                               //部门ID
    protected String depName;                             //部门名称
    protected String shopCode;                            //专柜编码
    protected String shopId;                              //专柜ID
    protected String shopName;                            //专柜名

    protected int pageNo;                                 //页号
    protected int itemNo;                                 //行号

    protected String tagPrice;                            //原价
    protected String price;                               //发生价
    protected String vipPrice;                            //会员价
    protected String minPrice;                            //最低价
    protected String maxPrice;                            //最高价
    protected String quantity;                            //数量
    protected String weight;                              //重量

    protected String spec;                                //规格
    protected String unit;                                //单位
    protected String pluWeight;                           //商品规格，单位kg（一包饼干，重250g，此处应该为0.25）

    protected String total;                               //应收合计
    protected String totalDsc;                            //优惠合计
    protected String totalRecv;                           //实收合计
    protected String totalPaid;                           //支付金额
    protected String totalRound;                          //抹零合计

    protected int isNormalDsc;                            //是否是普通优惠；0-无优惠；1-普通优惠
    protected int isTotalDsc;                             //是否参与满减促销；0-不参与；1-参与

    protected int isWeight;                               //是否是称重商品；0-普通商品；1-称重商品
    protected int isCountQuantity;                        //是否参与按计件处理
    protected boolean weightNoCode;                       //是否中台解析过称重条码,true:未解析过

    protected String remark;                              //备注
    protected String udp1;                                //扩展字段1
    protected String udp2;                                //扩展字段2
    protected String udp3;                                //扩展字段3
    protected String udp4;                                //扩展字段4
    protected String udp5;                                //扩展字段5

    protected List<SalePluDscModel> pluDscs = new ArrayList<>();                //优惠信息

    public void setQuantity(String quantity) {
        this.quantity = BigDecimalUtils.format(quantity);
    }

    public void addPluDsc(SalePluDscModel pluDscModel) {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
        }

        updateCalcPluDsc(pluDscModel);

        if (!BigDecimalUtils.isEquals(pluDscModel.getTotalDsc(), "0")) {
            pluDscModel.setPageNo(this.pageNo);
            pluDscModel.setItemNo(this.itemNo);
            pluDscModel.setSerialNo(pluDscs.size() + 1);
            pluDscs.add(pluDscModel);
        }

        calcTotalDsc();
    }

    public void setCargoNo(String cargoNo) {
        this.udp5 = this.cargoNo = cargoNo;
    }

    public void setUdp5(String udp5) {
        this.cargoNo = this.udp5 = udp5;
    }

    public void updatePluDsc() {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
            return;
        }

        for (SalePluDscModel pluDscModel : pluDscs) {
            pluDscModel.setItemNo(this.getItemNo());
        }
    }

    public void updatePluDsc(int index, SalePluDscModel pluDscModel) {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
            return;
        }

        pluDscModel.setPageNo(pluDscs.get(index).getPageNo());
        pluDscModel.setItemNo(pluDscs.get(index).getItemNo());
        pluDscModel.setSerialNo(pluDscs.get(index).getSerialNo());
        pluDscs.set(index, pluDscModel);

        calcTotalDsc();
    }

    public void replacePtxsDsc(SalePluDscModel pluDscModel) {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
        }

        if (pluDscModel == null) {
            return;
        }

        if (ListUtils.isEmpty(this.pluDscs)) {
            addPluDsc(pluDscModel);
            return;
        }

        updateCalcPluDsc(pluDscModel);

        for (SalePluDscModel salePluDscModel : this.pluDscs) {
            if (salePluDscModel.isPtxsDsc()) {
                int index = queryPluDscIndex(salePluDscModel);

                if (BigDecimalUtils.isGreater(getPluDsc(index).getDscPrice(), pluDscModel.getDscPrice())) {
                    updatePluDsc(index, pluDscModel);
                }
                return;
            }
        }

        addPluDsc(pluDscModel);
    }

    public void replacePluDsc(int index, SalePluDscModel pluDscModel) {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
        }

        if (pluDscModel == null) {
            return;
        }

        if (ListUtils.isEmpty(this.pluDscs)) {
            addPluDsc(pluDscModel);
            return;
        }

        updateCalcPluDsc(pluDscModel);

        if (BigDecimalUtils.isGreater(getPluDsc(index).getDscPrice(), pluDscModel.getDscPrice())) {
            updatePluDsc(index, pluDscModel);
        }
    }

    public void updateCalcPluDsc(SalePluDscModel pluDscModel) {
        if ("0".equals(pluDscModel.getDscType())) {
            String yhRate = BigDecimalUtils.subtract("100", pluDscModel.getDscRate());
            String dscPrice = StringUtils.changeF2Y(BigDecimalUtils.multiply(this.tagPrice, yhRate));

            pluDscModel.setDscPrice(dscPrice);
        }

        if (StringUtils.isEmpty(pluDscModel.getDscNum()) || BigDecimalUtils.isEquals0(pluDscModel.getDscNum())) {
            pluDscModel.setDscNum(this.getQuantity());
        }

        if (StringUtils.isEmpty(pluDscModel.getTotalDsc()) || BigDecimalUtils.isEquals0(pluDscModel.getTotalDsc())) {
            if (StringUtils.isEmpty(pluDscModel.getDscTotal()) || BigDecimalUtils.isEquals0(pluDscModel.getDscTotal())) {
                if (pluDscModel.getDscPrice() == null || pluDscModel.getDscPrice().length() == 0) {
                    pluDscModel.setDscPrice("0");
                }
                String priceDsc = BigDecimalUtils.subtract(this.tagPrice, pluDscModel.getDscPrice());
                pluDscModel.setTotalDsc(BigDecimalUtils.multiply(priceDsc, pluDscModel.getDscNum()));
            } else {
                pluDscModel.setTotalDsc(BigDecimalUtils.subtract(this.total, pluDscModel.getDscTotal()));
            }
        }

        if (StringUtils.isEmpty(pluDscModel.getDscTotal()) || BigDecimalUtils.isEquals0(pluDscModel.getDscTotal())) {
            pluDscModel.setDscTotal(BigDecimalUtils.subtract(this.total, pluDscModel.getTotalDsc()));
        }

        if (StringUtils.isEmpty(pluDscModel.getDscPrice()) || BigDecimalUtils.isEquals0(pluDscModel.getDscPrice())) {
            pluDscModel.setDscPrice(BigDecimalUtils.divide(pluDscModel.getDscTotal(), pluDscModel.getDscNum()));
        }
    }

    public void updateCalcPluDscByNum() {
        if (ListUtils.isEmpty(this.pluDscs)) {
            return;
        }

        SalePluDscModel pluDscModel = this.getPluDsc(0);
        if ("0".equals(pluDscModel.getDscType())) {
            String yhRate = BigDecimalUtils.subtract("100", pluDscModel.getDscRate());
            String dscPrice = StringUtils.changeF2Y(BigDecimalUtils.multiply(this.tagPrice, yhRate));

            pluDscModel.setDscPrice(dscPrice);
        }

        pluDscModel.setDscNum(this.getQuantity());
        pluDscModel.setDscTotal(BigDecimalUtils.multiply(pluDscModel.getDscPrice(), pluDscModel.getDscNum()));
        pluDscModel.setTotalDsc(BigDecimalUtils.multiply(pluDscModel.getTotalDsc(), pluDscModel.getDscNum()));
    }

    public void clearPluDsc() {
        this.pluDscs.clear();

        calcTotalDsc();
    }

    public void removeHjDsc() {
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
            return;
        }

        Iterator<SalePluDscModel> iterator = pluDscs.iterator();
        while (iterator.hasNext()) {
            SalePluDscModel pluDscModel = iterator.next();

            if (!pluDscModel.isPtxsDsc()) {
                iterator.remove();
            }
        }

        updatePluDscSerialNo();
    }

    private void updatePluDscSerialNo() {
        if (ListUtils.isEmpty(pluDscs)) {
            calcTotalDsc();
            return;
        }

        int i = 1;
        for (SalePluDscModel pluDscModel : pluDscs) {
            pluDscModel.setSerialNo(i);

            i++;
        }

        calcTotalDsc();
    }

    public SalePluDscModel getPluDsc(int index) {
        return pluDscs.get(index);
    }

    public int queryPluDscIndex(SalePluDscModel pluDscModel) {
        int index = 0;
        for (SalePluDscModel pluDscModelTemp : this.pluDscs) {
            if (pluDscModelTemp.getSerialNo() == pluDscModel.getSerialNo()) {
                return index;
            }
            index++;
        }

        return index;
    }

    public int pluDscSize() {
        if (pluDscs == null) {
            return 0;
        }

        return pluDscs.size();
    }

    private void calcTotal() {
        this.total = BigDecimalUtils.decimalFormat(BigDecimalUtils.multiply(this.tagPrice, this.quantity));

        String totalDsc = "0";

        for (int i = 0; i < pluDscs.size(); i++) {
            SalePluDscModel pluDscModel = pluDscs.get(i);
            LoggerUtils.getLogger().debug("商品促销优惠" + pluDscModel.toString());
            if (pluDscModel.isPtxsDsc()) {
                pluDscModel.setTotalDsc(BigDecimalUtils.multiply(BigDecimalUtils.divide(pluDscModel.getTotalDsc(), pluDscModel.getDscNum()), this.quantity));
                pluDscModel.setDscNum(this.quantity);

                pluDscs.set(i, pluDscModel);
            }

            totalDsc = BigDecimalUtils.add(totalDsc, pluDscModel.getTotalDsc());
        }

        this.totalDsc = BigDecimalUtils.decimalFormat(totalDsc);
        this.totalRecv = BigDecimalUtils.decimalFormat(BigDecimalUtils.subtract(this.total, this.totalDsc));
        this.totalPaid = BigDecimalUtils.decimalFormat("0");
    }

    public void calcTotalDscbyTotal(String totalRecv) {
        this.total = BigDecimalUtils.decimalFormat(this.total);
        this.totalRecv = BigDecimalUtils.decimalFormat(totalRecv);
        this.totalDsc = BigDecimalUtils.decimalFormat(BigDecimalUtils.subtract(this.total, this.totalRecv));
        this.totalRound = BigDecimalUtils.decimalFormat("0");
        this.price = BigDecimalUtils.decimalFormat(BigDecimalUtils.divide(this.totalRecv, this.quantity));
    }

    public void calcTotal(SalePluModel salePluModel) {
        this.total = BigDecimalUtils.decimalFormat(BigDecimalUtils.multiply(this.tagPrice, this.quantity));

        String totalDsc = "0";

        if (("0").equals(quantity)) {
            pluDscs.clear();
        } else {
            if (salePluModel.getPluDscs().size() > 0) {
                pluDscs = new ArrayList<>(salePluModel.getPluDscs());
            }

            for (int i = 0; i < pluDscs.size(); i++) {
                SalePluDscModel pluDscModel = pluDscs.get(i);
                LoggerUtils.getLogger().debug("商品促销优惠" + pluDscModel.toString());
                if (pluDscModel.isPtxsDsc()) {
                    pluDscModel.setTotalDsc(BigDecimalUtils.multiply(BigDecimalUtils.divide(pluDscModel.getTotalDsc(), pluDscModel.getDscNum()), this.quantity));
                    pluDscModel.setDscNum(this.quantity);

                    pluDscs.set(i, pluDscModel);
                }

                totalDsc = BigDecimalUtils.add(totalDsc, pluDscModel.getTotalDsc());
            }

        }


        this.totalDsc = BigDecimalUtils.decimalFormat(totalDsc);
        this.totalRecv = BigDecimalUtils.decimalFormat(BigDecimalUtils.subtract(this.total, this.totalDsc));
        this.totalPaid = BigDecimalUtils.decimalFormat("0");
    }

    public void calcTotalDsc() {
        this.total = BigDecimalUtils.decimalFormat(this.total);

        String totalDsc = "0";
        for (int i = 0; i < pluDscs.size(); i++) {
            SalePluDscModel pluDscModel = pluDscs.get(i);
            if (StringUtils.isEmpty(pluDscModel.getTotalDsc())) {
                pluDscModel.setTotalDsc(BigDecimalUtils.subtract(this.total, pluDscModel.getDscTotal()));

                pluDscs.set(i, pluDscModel);
            }

            totalDsc = BigDecimalUtils.add(totalDsc, pluDscModel.getTotalDsc());
        }

        this.totalDsc = BigDecimalUtils.decimalFormat(totalDsc);
        this.totalRecv = BigDecimalUtils.decimalFormat(BigDecimalUtils.subtract(this.total, this.totalDsc));

        this.totalRound = BigDecimalUtils.decimalFormat("0");

        this.price = BigDecimalUtils.decimalFormat(BigDecimalUtils.divide(this.totalRecv, this.quantity));
    }

    public boolean isNormalDsc() {
        return isThisDsc("10");
    }

    public boolean isVipDsc() {
        return isThisDsc("20") || isThisDsc("94");
    }

    private boolean isThisDsc(String dscProType) {
        if (ListUtils.isEmpty(pluDscs) || StringUtils.isEmpty(dscProType)) {
            return false;
        }

        for (SalePluDscModel pluDscModel : pluDscs) {
            if (dscProType.equals(pluDscModel.getDscProType())) {
                return true;
            }
        }

        return false;
    }

    public boolean isTotalDsc() {
        return this.isTotalDsc == 1;
    }

    public boolean isHasTotalDsc() {
        return BigDecimalUtils.isGreater(this.totalDsc, "0");
    }

    public boolean isWeight() {
        return this.isWeight == 1;
    }

    public boolean isWeightNoCode() {
        return isWeight() && this.weightNoCode;
    }

    public boolean isCountQuantity() {
        return this.isCountQuantity == 1;
    }

    public boolean isHasFen() {
        String totalRecv = BigDecimalUtils.formatScale(this.totalRecv, 1);

        return BigDecimalUtils.isNotEquals(totalRecv, this.totalRecv);
    }

    public boolean isWeightOrEquivalent() {
        return (isWeight() && !isCountQuantity()) || (!StringUtils.isInteger(BigDecimalUtils.format(this.quantity)));
    }

    //称重商品
    public boolean isWeightGoods() {
        return (isWeight()) || (!StringUtils.isInteger(BigDecimalUtils.format(this.quantity)));
    }

    public boolean isWeightAndNotCount() {
        return isWeight() && !isCountQuantity();
    }

    public boolean isLimitDecreaseQuantity() {
        return isWeightOrEquivalent() || BigDecimalUtils.isEquals(this.quantity, "1");
    }

    public void changeQuantity(int quantity) {
        if (quantity >= 0) {
            this.quantity = String.valueOf(quantity);
        } else {
            this.quantity = "0";
        }

        calcTotal();
    }

    public void weightPlu(String quantity) {
        this.quantity = quantity;
        calcTotal();
    }

    public String increaseQuantity() {
        return BigDecimalUtils.add(this.quantity, "1");
    }

    public String decreaseQuantity() {
        return BigDecimalUtils.subtract(this.quantity, "1");
    }

    public int count() {
        if (isWeightOrEquivalent()) {
            return 1;
        } else {
            return StringUtils.toInt(this.quantity, 1);
        }
    }

    public double use(SaleItemShare share) {
        double paid = this.pay(share.value());
        share.use(paid);
        return paid;
    }

    /**
     * 支付。
     *
     * @param value 支付的金额
     * @return 实际支付金额
     */
    public double pay(double value) {
        double actualUsed = value;

        String totalPaid = BigDecimalUtils.add(this.totalPaid, String.valueOf(value));

        if (BigDecimalUtils.isGreater(totalPaid, this.total)) {
            actualUsed = Double.valueOf(BigDecimalUtils.subtract(this.total, this.totalPaid));
            this.totalPaid = this.total;
        } else {
            this.totalPaid = totalPaid;
        }

        return actualUsed;
    }

    public void cancel(SaleItemShare share) {
        this.pay(share.cancel() * -1);
    }

    public void setTotal(String total) {
        this.total = BigDecimalUtils.decimalFormat(total);
    }

    public void setTotalDsc(String totalDsc) {
        this.totalDsc = BigDecimalUtils.decimalFormat(totalDsc);
    }

    public void setTotalRecv(String totalRecv) {
        this.totalRecv = BigDecimalUtils.decimalFormat(totalRecv);
    }

    public void setTotalPaid(String totalPaid) {
        this.totalPaid = BigDecimalUtils.decimalFormat(totalPaid);
    }

    public void setTotalRound(String totalRound) {
        this.totalRound = BigDecimalUtils.decimalFormat(totalRound);
    }

    public void setTotal(double total) {
        this.total = BigDecimalUtils.decimalFormat(total);
    }

    public void setTotalDsc(double totalDsc) {
        this.totalDsc = BigDecimalUtils.decimalFormat(totalDsc);
    }

    public void setTotalRecv(double totalRecv) {
        this.totalRecv = BigDecimalUtils.decimalFormat(totalRecv);
    }

    public void setTotalPaid(double totalPaid) {
        this.totalPaid = BigDecimalUtils.decimalFormat(totalPaid);
    }

    public void setTotalRound(double totalRound) {
        this.totalRound = BigDecimalUtils.decimalFormat(totalRound);
    }


}
