package com.hisense.hipos.model;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hipos.payment.PayMethod;

import java.math.BigDecimal;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

import lombok.Data;

/**
 * 重构之后的 SaleModel
 *
 * @author dukunbiao(null) 2019-08-16
 * @author wangyaogang     2019-11-15 HIAPOSCMP-258 组装退货流水时,增加默认值
 */
@Data
public class ReturnSaleModel extends BaseSaleModel {

    private SaleModel saleModel;
    protected String authNo;                    //退货工号

    public boolean hasPassCoupons;              ///预判券失败时是否点击确定
    public boolean hasPassPoints;               //预判积分失败时是否点击确定

    public void setSaleModel(SaleModel saleModel) {
        this.saleModel = saleModel;
        this.posNo = (this.posNo == null) ? this.saleModel.getPosNo() : this.posNo;
        this.cashier = (this.cashier == null) ? this.saleModel.getCashier() : this.cashier;
        this.accDate = (this.accDate == null) ? this.saleModel.getAccDate() : this.accDate;
        this.vip = (this.vip == null) ? this.saleModel.getVip() : this.vip;
        this.tranType = (this.tranType == null) ? this.saleModel.getTranType() : this.tranType;
    }

    public String oldSaleNo() {
        if (saleModel != null) {
            return saleModel.getSaleNo();
        }
        return "";
    }

    /**
     * 返回是否原交易商品全退。只检查商品条数，不检查商品数量。
     *
     * @return true 表示全退；false 表示退部分商品。
     */
    public boolean isReturnAll() {
        if (saleModel == null) {
            return false;
        }
        return pluSize() == this.saleModel.pluSize();
    }

    public ReturnSalePluModel add(BaseSalePluModel baseSalePluModel) {
        ReturnSalePluModel returnSalePluModel = new ReturnSalePluModel();
//        SalePluModel salePluModel=new SalePluModel();
//        try {
//         salePluModel=   (SalePluModel) baseSalePluModel.clone();
//        }catch (Exception e){
//            LoggerUtils.getLogger().debug("saleplumodel 克隆异常");
//        }

        returnSalePluModel.setSalePluModel((SalePluModel)baseSalePluModel);
        returnSalePluModel.setBarCode(baseSalePluModel.getBarCode());
        returnSalePluModel.setBrandCode(baseSalePluModel.getBrandCode());
        returnSalePluModel.setClassCode(baseSalePluModel.getClassCode());
        returnSalePluModel.setClkCode(baseSalePluModel.getClkCode());
        returnSalePluModel.setClkId(baseSalePluModel.getClkId());
        returnSalePluModel.setDepCode(baseSalePluModel.getDepCode());
        returnSalePluModel.setDepId(baseSalePluModel.getDepId());
        returnSalePluModel.setDepName(baseSalePluModel.getDepName());
        returnSalePluModel.setPageNo(baseSalePluModel.getPageNo());
        returnSalePluModel.setItemNo(baseSalePluModel.getItemNo());
        returnSalePluModel.setPluCode(baseSalePluModel.getPluCode());
        returnSalePluModel.setPluId(baseSalePluModel.getPluId());
        returnSalePluModel.setPluName(baseSalePluModel.getPluName());
        returnSalePluModel.setPluType(baseSalePluModel.getPluType());
        returnSalePluModel.setPrice(baseSalePluModel.getPrice());
        returnSalePluModel.setQuantity(BigDecimalUtils.absToString(baseSalePluModel.getQuantity()));
        returnSalePluModel.setShopCode(baseSalePluModel.getShopCode());
        returnSalePluModel.setShopId(baseSalePluModel.getShopId());
        returnSalePluModel.setSpec(baseSalePluModel.getSpec());
        returnSalePluModel.setUnit(baseSalePluModel.getUnit());
        returnSalePluModel.setUdp5(baseSalePluModel.getUdp5());
        returnSalePluModel.setTagPrice(BigDecimalUtils.absToString(baseSalePluModel.getTagPrice()));
        returnSalePluModel.setTotalRecv(baseSalePluModel.getTotalRecv());
        returnSalePluModel.setTotal(BigDecimalUtils.absToString(baseSalePluModel.getTotal()));
        returnSalePluModel.setTotalDsc(BigDecimalUtils.absToString(baseSalePluModel.getTotalDsc()));
        returnSalePluModel.setTotalRound(BigDecimalUtils.absToString(baseSalePluModel.getTotalRound()));
        returnSalePluModel.setPluDscs(new ArrayList<>(baseSalePluModel.getPluDscs()));

        return returnSalePluModel;

    }

    /**
     * 少退一件商品。
     */
    public void returnLess(int position, BaseSalePluModel salePluModel) {

        if (salePluModel.isWeightOrEquivalent()) {
            changeQuantityReturn(position, 0);
        } else {
            this.changeQuantityReturn(position, Integer.parseInt(salePluModel.getQuantity()) - 1);
        }

    }


    /**
     * 少退一件商品。
     */
    public void returnMore(int position, BaseSalePluModel salePluModel) {

        if (salePluModel.isWeightOrEquivalent()) {
            changeQuantityReturn(position, Integer.parseInt(salePluModel.getQuantity()));
        } else {
            this.changeQuantityReturn(position, Integer.parseInt(salePluModel.getQuantity()) + 1);
        }

    }

    /**
     * 按照支付方式和支付号计算可退金额
     *
     * @param method 支付方式
     * @param payNo  支付号
     * @return 可退金额
     */
    public String getSuggestAmount(PayMethod method, String payNo) {
        if (this.saleModel == null || this.saleModel.getPays() == null || this.saleModel.getPays().size() == 0) {
            return "0";
        }
        BigDecimal suggestRefunding = BigDecimal.ZERO;
        for (SalePayModel salePayModel : this.saleModel.getPays()) {
            if (salePayModel.getPayCode().equals(method.code()) &&
                    (salePayModel.getBuyerLogonId().equals(payNo))) {
                suggestRefunding = suggestRefunding.add(BigDecimal.valueOf(salePayModel.totalUnRefund()));
            }
        }
        return suggestRefunding.stripTrailingZeros().toPlainString();
    }

    /**
     * 根据traceNo 查找原始交易,金额也使用原始金额
     * 针对利客来一点通新增的查询方式
     *
     * @param traceNo
     * @return
     * @throws Exception
     */
    public SalePayModel refundSuggest(String traceNo) throws Exception {
        if (this.saleModel == null || this.saleModel.getPays() == null || this.saleModel.getPays().size() == 0) {
            LoggerUtils.getLogger().debug("traceNo_refundSuggest:原始交易数据为空");
            throw new Exception("未找到可退款的原始交易");
        }

        // 查找traceNo相同的原始交易,即oldOutTradeNo
        for (SalePayModel salePayModel : this.saleModel.getPays()) {
            if (traceNo.equals(salePayModel.getOutTradeNo())) {
                LoggerUtils.getLogger().debug("traceNo_refundSuggest:找到原始交易:" + salePayModel);
                return salePayModel;
            }
        }

        LoggerUtils.getLogger().debug("traceNo_refundSuggest不该出现未找到");
        // 正常情况下应该不会出现这种情况
        throw new Exception("未找到可退款的原始交易");
    }

    /**
     * 查找指定支付方式,指定支付号的原始交易
     *
     * @param method 支付方式
     * @param payNo  支付号
     * @return 原始交易集合
     * @throws Exception
     */
    public Collection<SalePayModel> refundSuggest(PayMethod method, String payNo) throws Exception {
        if (this.saleModel == null || this.saleModel.getPays() == null || this.saleModel.getPays().size() == 0) {
            LoggerUtils.getLogger().debug("payNo_refundSuggest原始交易数据为空");
            throw new Exception("未找到可退款的原始交易");
        }
        Collection<SalePayModel> payments = new ArrayList<>();
        for (SalePayModel payment : this.saleModel.getPays()) {
            if (payment.getPayCode().equals(method.code()) &&
                    payment.getBuyerLogonId().equals(payNo)) {
                payments.add(payment);
            }
        }
        if (payments.size() > 0) {
            LoggerUtils.getLogger().debug("payNo_refundSuggest:找到原始交易");
            return payments;
        } else {
            LoggerUtils.getLogger().debug("payNo_refundSuggest未找到使用该卡支付的原始交易");
            throw new Exception("未找到使用该卡支付的原始交易");
        }
    }

    /**
     * 查找单笔可退金额大于 {@param amount} 的原始支付交易。
     *
     * @param method 支付方式
     * @param amount 欲退款金额
     * @return 单笔可退金额大于退款金额的原始支付交易
     * @throws Exception 如果未找到对应的原始支付交易
     */
    public SalePayModel refundSuggest(PayMethod method, double amount) throws Exception {
        if (this.saleModel == null || this.saleModel.getPays() == null || this.saleModel.getPays().size() == 0) {
            LoggerUtils.getLogger().debug("amount_refundSuggest原始交易数据为空");
            return null;
        }

        // 查找退款金额小于单笔可退金额的原始交易
        for (SalePayModel payment : this.saleModel.getPays()) {
            if (payment.getPayCode().equals(method.code()) &&
                    amount <= payment.totalUnRefund()) {
                LoggerUtils.getLogger().debug("amount_refundSuggest:找到原始交易:" + payment);
                return payment;
            }
        }

        // 查找可退金额最大的原始交易
        double suggestRefunding = 0;
        for (SalePayModel payment : this.saleModel.getPays()) {
            if (payment.getPayCode().equals(method.code()) &&
                    (payment).totalUnRefund() > suggestRefunding) {
                suggestRefunding = payment.totalUnRefund();
            }
        }

        if (suggestRefunding > 0) {
            LoggerUtils.getLogger().debug("amount_refundSuggest:建议金额:" + suggestRefunding);
            throw new Exception(String.format(Locale.CHINESE,
                    "退款金额大于单笔可退金额。建议将退款金额调整为%.2f", suggestRefunding));
        } else {
            // 正常情况下应该不会出现这种情况
            LoggerUtils.getLogger().debug("amount_refundSuggest:不该出现未找到");
            throw new Exception("未找到可退款的原始交易");
        }
    }

    /**
     * 查找单笔可退金额大于 {@param amount} 的原始支付交易。
     *
     * @param method  支付方式
     * @param traceNo 交易凭证号
     * @return 单笔可退金额大于退款金额的原始支付交易
     * @throws Exception 如果未找到对应的原始支付交易
     */
    public SalePayModel refundSuggestByTraceNo(PayMethod method, String traceNo) {
        if (this.saleModel == null || this.saleModel.getPays() == null || this.saleModel.getPays().size() == 0) {
            LoggerUtils.getLogger().debug("amount_refundSuggest原始交易数据为空");
            return null;
        }

        // 查找退款金额小于单笔可退金额的原始交易
        for (SalePayModel payment : this.saleModel.getPays()) {
            if (payment.getPayCode().equals(method.code()) &&
                    traceNo.equals(payment.getOutTradeNo())) {
                LoggerUtils.getLogger().debug("amount_refundSuggest:找到原始交易:" + payment);
                return payment;
            }
        }
        return null;

    }

    /**
     * 退款
     *
     * @param refundPayModel 退款
     * @param oldPaymentList 退款对应的原支付交易
     * @return 退款交易
     */
    public SalePayModel refund(SalePayModel refundPayModel, Collection<SalePayModel> oldPaymentList) {
        //将退款交易存储在salepaymodel中
        addPay(refundPayModel);
        // 将退款分摊到原支付交易上
        double refundAmount = Double.valueOf(refundPayModel.getTotalPaid());
        for (SalePayModel oldPayModel : oldPaymentList) {


            refundAmount = oldPayModel.refund(refundPayModel, refundAmount);
            if (refundAmount == 0) {
                break;
            }
        }
        return refundPayModel;
    }

    /**
     * 退款。
     *
     * @param refundPayModel 退款
     * @param oldPayment     退款对应的原支付交易
     * @return 退款交易
     */
    public SalePayModel refund(SalePayModel refundPayModel, SalePayModel oldPayment) {
        if (oldPayment == null) {
            return this.refund(refundPayModel);
        }
        if (BigDecimalUtils.isGreater(refundPayModel.getTotalPaid(), oldPayment.getTotalPaid())) {
            // 调用方应保证不会出现此种情况。如果出现，说明调用方需要修改逻辑。
            throw new InvalidParameterException("退款金额不能大于原支付金额");
        }
        //退款时的列表
        addPay(refundPayModel);

        oldPayment.refund(refundPayModel, Double.valueOf(refundPayModel.getTotalPaid()));

        return refundPayModel;
    }

    /**
     * 退款。
     *
     * @param refund 退款
     * @return 退款交易
     */
    public SalePayModel refund(SalePayModel refund) {

        addPay(refund);

        // 将退款分摊到原支付交易上
        double refundAmount = Double.valueOf(refund.getTotalPaid());
        //检测原支付方式是否包含该支付
        for (SalePayModel salePayment : saleModel.getPays()) {
            refundAmount = salePayment.refund(refund, refundAmount);
            if (refundAmount == 0) {
                break;
            }
        }
        //原支付方式若不包含该支付，refundAmount会大于0，执行下边代码，将预退货支付方式增加到原支付方式中（totalpaid必须为0）
        if (refundAmount > 0) {

            // 原交易流水没有对应的支付方式，如：原交易使用现金支付，退货时选择退款到积分卡，此时，在支付列表中新增一个退款对应的支付方式
            SalePayModel salePayModel = new SalePayModel();
            salePayModel.setBuyerLogonId("");
            salePayModel.setPayCode(refund.getPayCode());
            salePayModel.setRemark(refund.getRemark());
            salePayModel.setPayType(refund.getPayType());
            salePayModel.setPayName(refund.getPayName());
            salePayModel.setTotalPaid("0");
            salePayModel.setTotalRecv("0");
            saleModel.addPayNotUpdate(salePayModel);
            saleModel.getLastPay().refund(refund, refundAmount);
        }

        return refund;
    }

    /**
     * 满减退款。当退款金额小于满减金额时，按退款金额退；当退款金额大于满减金额时，按满减金额退。
     *
     * @param refundAmount 退款金额
     * @return 退款交易
     */
    public void refundTotalReduction(String refundAmount) {
        String amount = BigDecimalUtils.ternaryOperatorGreater(refundAmount, "0");
        if (BigDecimalUtils.isEquals0(amount)) {
            return;
        }

        SalePayModel totalReduction = null;
        for (SalePayModel payment : saleModel.getPays()) {
            if (PayMethod.isTotalReduction(payment.getPayCode())) {
                totalReduction = payment;
                break;
            }
        }

        if (totalReduction != null) {
            String reduction = BigDecimal.valueOf(Double.valueOf(this.getTotal()))
                    .divide(BigDecimal.valueOf(Double.valueOf(this.saleModel.getTotal())), 6, BigDecimal.ROUND_HALF_UP)
                    .multiply(BigDecimal.valueOf(Double.valueOf(totalReduction.getTotalPaid())))
                    .setScale(2, BigDecimal.ROUND_HALF_UP)
                    .stripTrailingZeros().toPlainString();

            SalePayModel salePayModel = new SalePayModel();
            salePayModel.setBuyerLogonId("");
            salePayModel.setPayCode(totalReduction.getPayCode());
            salePayModel.setRemark(totalReduction.getRemark());
            salePayModel.setPayType(totalReduction.getPayType());
            salePayModel.setPayName(totalReduction.getPayName());
            salePayModel.setTotalPaid(BigDecimalUtils.ternaryOperatorLess(reduction, amount));
            salePayModel.setTotalRecv(salePayModel.getTotalPaid());
            addPay(salePayModel);
        }
    }
}
