package com.hisense.hipos.model;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.hipos.plu.PluDscType;
import com.hisense.hipos.sale.ISaleItemDiscount;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import lombok.Data;

/**
 * 重构之后的 BaseSalePluModel
 *
 * @author dukunbiao(null)  2019-08-16
 * @author xieliyan        2019-01-07 HIAPOSCMP-521 修复调整商品数量，右侧合计金额更新的问题
 */
@Data
public class ReturnSalePluModel extends BaseSalePluModel {

    private SalePluModel salePluModel;

    private void calcDiscount() {
        this.salePluModel.calcTotalDsc();
    }

    public void changeQuantity(String quantity) {
        if (BigDecimalUtils.isGreaterAndEquals0(quantity)) {
            this.quantity = quantity;
        } else {
            this.quantity = "0";
        }

        calcTotal(salePluModel);
    }

    /**
     * 少退一件商品。
     */
    public void returnMore(BaseSaleModel baseSaleModel) {
        if (isWeightOrEquivalent()) {
            changeQuantityReturn(getQuantity(), baseSaleModel);
        } else {
            changeQuantityReturn(BigDecimalUtils.add(getQuantity(), "1"), baseSaleModel);
        }
    }

    /**
     * 少退一件商品。
     */
    public void returnLess(BaseSaleModel baseSaleModel) {
        if (isWeightOrEquivalent()) {
            changeQuantityReturn("0", baseSaleModel);
        } else {
            changeQuantityReturn(BigDecimalUtils.subtract(getQuantity(), "1"), baseSaleModel);
        }
    }

    public void changeQuantityReturn(String quantity, BaseSaleModel baseSaleModel) {
        changeQuantity(quantity);

        baseSaleModel.calcTotal();
    }

    /**
     * 退款。
     *
     * @param total 合计退款金额
     */
    public void returnTotal(double total) {
        this.setTotal(total);
        this.setTotalPaid(total);
        this.internalCalcPrice();
        this.calcDiscount();
    }

    /**
     * 计算退货商品单价。
     */
    private void internalCalcPrice() {
        if (Double.valueOf(getQuantity()) > 0) {
            BigDecimal total = new BigDecimal(this.totalRecv);
            for (SalePluDscModel discount : salePluModel.getPluDscs()) {
                if (discount.getDscType().equals(PluDscType.TOTAL_REDUCTION)) {
                    total = total.add(new BigDecimal(discount.getDscTotal()));
                }
            }
            BigDecimal quantity = new BigDecimal(this.quantity);
            setPrice(String.format(Locale.CHINESE, "%.2f", total.divide(quantity, 4).doubleValue()));
        }
    }
}
