package com.hisense.hipos.model;

import com.hisense.common.util.ListUtils;
import com.hisense.crm.card.UseLimitPlu;
import com.hisense.hipos.coupon.UsableCoupon;
import com.hisense.hipos.coupon.UsableCouponList;
import com.hisense.hipos.pay.PayUi;
import com.hisense.hipos.plu.PluGiftAction;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import lombok.Data;

/**
 * 重构之后的 SaleModel
 *
 * @author dukunbiao(null)  2019-08-16
 */
@Data
public class SaleModel extends BaseSaleModel {

    //组织号,仅流水查询返回使用,其他情况请使用ShopInfo
    private String orgCode;

    private List<PluGiftAction> pluGiftActions = new ArrayList<>();

    private List<PayUi> payUiOften = new ArrayList<>();
    private List<PayUi> payUiOther = new ArrayList<>();

    private UsableCouponList usableCoupons = new UsableCouponList();        //核销赠券列表
    private List<UsableCoupon> usableCouponScrm = new ArrayList<>();        //SCRM用券信息

    private static volatile SaleModel mInstance;

    public SaleModel() {
        init();
    }

    @Override
    public List<BaseSalePluModel> getPlus() {
        return plus;
    }

    public void init() {
        if (this.plus == null) {
            this.plus = new ArrayList<>();
        }
        if (this.pays == null) {
            this.pays = new ArrayList<>();
        }
        if (this.pluGiftActions == null) {
            this.pluGiftActions = new ArrayList<>();
        }
        if (this.pluDscs == null) {
            this.pluDscs = new ArrayList<>();
        }
        if (this.payUiOften == null) {
            this.payUiOften = new ArrayList<>();
        }
        if (this.payUiOther == null) {
            this.payUiOther = new ArrayList<>();
        }
        if (this.usableCoupons == null) {
            this.usableCoupons = new UsableCouponList();
        }
        if (this.usableCouponScrm == null) {
            this.usableCouponScrm = new ArrayList<>();
        }

        calcTotal();
    }

    public void clearPluGift() {
        if (!ListUtils.isEmpty(pluGiftActions)) {
            pluGiftActions.clear();
        }
    }

    public void initPayMethodList() {
        payUiOften = new ArrayList<>();
        payUiOther = new ArrayList<>();
    }

    //当前限制品牌储值卡,支付商品列表
    private List<UseLimitPlu> tempPluList;

    //当前限制品牌储值卡,已支付所有列表
    private List<UseLimitPlu> pluList;

    public void setTempPluList(List<UseLimitPlu> tempPluList) {
        this.tempPluList = tempPluList;
    }

    public List<UseLimitPlu> getPluList() {
        return pluList;
    }

    public void setPluList(List<UseLimitPlu> pluList) {
        this.pluList = pluList;
    }

    /**
     * 分摊限制支付金额
     *
     * @param amount 该笔支付金额
     */
    public void cardShare(double amount) {
        if (tempPluList != null) {
            BigDecimal discountTotal = BigDecimal.valueOf(amount);
            BigDecimal total = BigDecimal.ZERO;
            BigDecimal tempDiscount = BigDecimal.ZERO;
            //保证pluList不为空, 找到相同pluCode 金额相加,否则直接添加
            List<UseLimitPlu> pluList = getPluList();
            if (pluList == null) {
                pluList = new ArrayList<>();
            }
            //计算总限制金额
            for (UseLimitPlu plu : tempPluList) {
                total = total.add(new BigDecimal(plu.totalLimit));
            }
            //准备分摊
            for (int i = 0; i < tempPluList.size(); i++) {
                UseLimitPlu plu = tempPluList.get(i);
                // 除最后一个商品外，其余商品按照商品金额占总金额百分比分摊优惠金额
                if ((i + 1) != tempPluList.size()) {
                    // 计算单个商品所占优惠金额百分比
                    BigDecimal discountPercent = new BigDecimal(plu.totalLimit).divide(total, 2, BigDecimal.ROUND_HALF_UP);
                    //计算优惠金额
                    BigDecimal paid = discountPercent.multiply(discountTotal);
                    tempDiscount = tempDiscount.add(paid);
                    plu.totalDiscount(plu.totalDiscount().add(paid));
                } else {
                    plu.totalDiscount(plu.totalDiscount().add(discountTotal.subtract(tempDiscount)));
                }

                if (pluList.isEmpty()) {
                    pluList.add(plu);
                } else {
                    boolean hasFound = false;
                    for (int j = 0; j < pluList.size(); j++) {
                        UseLimitPlu limitPlu = pluList.get(j);
                        if (plu.pluCode.equals(limitPlu.pluCode)) {
                            hasFound = true;
                            limitPlu.totalDiscount(limitPlu.totalDiscount().add(plu.totalDiscount()));
                            break;
                        }
                    }
                    if (!hasFound) {
                        pluList.add(plu);
                    }
                }
            }
            //将历次分摊记录下来
            setPluList(pluList);
        }
    }
}
