package com.hisense.hipos.model;

import com.hisense.common.util.BigDecimalUtils;
import com.hisense.common.util.DateUtils;
import com.hisense.common.util.LoggerUtils;
import com.hisense.common.util.StringUtils;
import com.hisense.hipos.payment.PayMethod;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 重构之后的 SalePayModel
 *
 * @author dukunbiao(null)  2019-08-16
 */

public class SalePayModel extends SalePayModelBase {

    private Map<SalePayModelBase, Double> refunds = new HashMap<>();

    public SalePayModel() {

    }

    public SalePayModel(PayMethod payMethod, String payNo, String totalPaid) {
        this(payMethod, payNo, totalPaid, null, null, null);
    }

    public SalePayModel(PayMethod payMethod, String payNo, String totalPaid, String outTradeNo, String refNo, String balance) {
        this.payCode = payMethod.code();
        this.payName = payMethod.name();
        this.remark = payMethod.remark();
        this.payType = payMethod.payType();
        this.buyerLogonId = payNo;
        this.outTradeNo = outTradeNo;
        this.tradeNo = refNo;
        this.balance = balance;

        this.setTotal(totalPaid);
        this.setTotalRecv(totalPaid);
        this.setTotalPaid(totalPaid);
    }

    public SalePayModel(PayMethod payMethod, String totalPaid, String outTradeNo, String saleNoTemp, int serialNo) {
        this(payMethod, null, totalPaid, outTradeNo, null, saleNoTemp, "0", serialNo);
    }

    public SalePayModel(PayMethod payMethod, String payNo, String totalPaid, String outTradeNo, String refNo, String saleNoTemp, String balance, int serialNo) {
        this.payCode = payMethod.code();
        this.payName = payMethod.name();
        this.remark = payMethod.remark();
        this.payType = payMethod.payType();
        this.buyerLogonId = payNo;
        this.payTime = DateUtils.dateFormat();
        this.outTradeNo = outTradeNo;
        this.tradeNo = refNo;
        this.saleNoTemp = saleNoTemp;
        this.serialNo = serialNo;
        this.balance = balance;

        this.setTotal(totalPaid);
        this.setTotalRecv(totalPaid);
        this.setTotalPaid(totalPaid);
    }

    /**
     * 退款。
     *
     * @param refund 退款交易
     * @param amount 退款金额
     * @return 当前支付交易分摊的退款金额
     */
    public double refund(SalePayModel refund, double amount) {
        if (refund == null) {
            return amount;
        }

        // 支付方式不同时不分摊
        boolean code = refund.getPayCode().equals(this.getPayCode());

        // 支付方式不同时不分摊
        if (!code) {
            return amount;
        }

        // 原支付金额为 0 时，表示原交易中没有使用此支付方式，此时，分摊全部退款金额

        String totalPaid = getTotalPaid();
        if (totalPaid.equals("0.00")) {


            this.refunds.put(refund, amount);
            return 0;
        }

        // 当前支付交易全部退款后，不处理，直接返回
        double totalUnRefund = this.totalUnRefund();
        if (totalUnRefund == 0) {
            return amount;
        }

        if (amount <= totalUnRefund) {
            // 退款金额 <= 未退金额时，分摊全部退款金额
            this.refunds.put(refund, amount);
            return 0;
        } else {
            // 退款金额 > 未退金额时，仅分摊未退金额部分
            this.refunds.put(refund, totalUnRefund);
            return BigDecimal.valueOf(amount)
                    .subtract(BigDecimal.valueOf(totalUnRefund))
                    .doubleValue();
        }
    }

    /**
     * 返回合计退款金额。
     *
     * @return 合计退款金额
     */
    public double totalRefund() {

        double total = 0.0;
        try {

            for (double refund : refunds.values()) {

                total = BigDecimalUtils.add(total, refund);
            }
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("退款金额异常" + e.toString());
        }

        return total;


    }

    /**
     * 返回未退金额。
     *
     * @return 未退金额
     */
    public double totalUnRefund() {
        if (StringUtils.isEmpty(getTotalPaid())) {
            return 0;
        }
        return Double.valueOf(BigDecimalUtils.subtract(getTotalPaid(), String.valueOf(totalRefund())));
    }

    @Override
    public String toString() {
        return "SalePayModel{" +
                "payCode='" + payCode + '\'' +
                ", payName='" + payName + '\'' +
                ", remark='" + remark + '\'' +
                ", outTradeNo='" + outTradeNo + '\'' +
                ", tradeNo='" + tradeNo + '\'' +
                ", mchId='" + mchId + '\'' +
                ", cardNo=" + cardNo +
                ", balance='" + balance + '\'' +
                ", payTime='" + payTime + '\'' +
                ", serialNo=" + serialNo +
                ", total='" + total + '\'' +
                ", totalRecv='" + totalRecv + '\'' +
                ", totalPaid='" + totalPaid + '\'' +
                ", totalDscSelf='" + totalDscSelf + '\'' +
                ", totalDscChannel='" + totalDscChannel + '\'' +
                ", udp1='" + udp1 + '\'' +
                ", udp2='" + udp2 + '\'' +
                ", udp3='" + udp3 + '\'' +
                ", udp4='" + udp4 + '\'' +
                ", udp5='" + udp5 + '\'' +
                '}';
    }
}

