/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.payment;

import java.io.Serializable;

/**
 * 支付方式。
 *
 * @author zhangsai    2017-11-22
 * @author zhangsai    2017-12-22
 * @author zhangsai    2018-06-21 HIPOSAPP-576 增加满减支付静态方法
 * @author lizhixin    2018-06-25 HIPOSAPP-520 新增工行支付方式：积分支付与分期付款
 * @author guolina     2018-06-29 HIPOSAPP-604 修复无限满减
 * @author wangyaogang 2018-12-27 HIPOSAPP-876 中行银联二维码支付对接,增加云闪付支付方式
 * @author guolina  2019-03-14 HIPOSAPP-1052 支持其他支付方式
 * @author wangyaogang 2019-08-05 HIPOSAPP-1285 增加单独设置code和name的方法
 * @author dukunbiao(null)  2019-10-28  重构
 */
public class PayMethod implements Serializable {

    //未知支付方式
    public static final String UNKNOWN = "UNKNOWN";

    //慧收银专用支付方式
    public static final String ALIFACEPAY = "$alifacepay";                                                          //阿里人脸识别
    public static final String WXFACEPAY = "$wxfacepay";                                                            //微信人脸识别

    //现金类支付方式，不需调用接口
    public static final String CASH = "$cash";                                                                      //现金
    public static final String OTHER = "$other";                                                                    //系统外的其他支付方式，例如衢州东方自印券
    public static final String CODE_TOTAL_REDUCTION = "99";                                                         //满减（作为支付方式记录）

    //卡、券类支付方式，调用相关接口
    public static final String CARD = "$card";                                                                      //储值卡
    public static final String VIP_POINTS = "$jfzf";                                                                //会员积分
    public static final String VIP_COUPON = "$vipzq";                                                               //会员券一段式核销
    public static final String VIP_POINTS_SCRM = "InterType=AllInPay;CallDllName=HsQTSCRMPay;CardType=Jplu";        //scrm会员积分
    public static final String SCRM_REMAIN_PAY = "InterType=AllInPay;CallDllName=HsQTSCRMPay;CardType=Vplu";        //scrm余额支付
    public static final String SCRM_COUPON = "InterType=AllInPay;CallDllName=HsQTSCRMPay;CardType=Qplu";            //scrm优惠券

    public static final String HISENSE_CLOUND_PAY = "InterType=AllInPay;CallDllName=HsCloudPay;CardType=Cplu";      //海信云闪付

    //银联类（银行或全通道）支付方式，调用中间件接口或对应渠道支付接口
    public static final String ALIPAY = "$zfb";                                                                     //支付宝
    public static final String WXPAY = "$wx";                                                                       //微信支付
    public static final String MOBILEPAY = "$mobilepay";                                                            //聚合支付

    //银联类（全通道）支付方式，调用中间件接口
    public static final String YI_DIAN_TONG = "HsLKLAppPay";                                                        //利客来,一点通支付方式,pos5 使用
    public static final String POS_TONG = "InterType=AllInPay;CallDllName=HsSeaWayJHPay;CardType=Pplu";         // pos通,专指走 海信pay通道的银商pos通（银商单品营销使用）


    //银联类（银行通道）支付方式，调用对应渠道支付接口
    public static final String UNION_CCB_CARD = "$uccbcard";                                                        //建行直连银行卡
    public static final String UNION_CCB_SCAN = "$uccbmobilepay";                                                   //建行直连聚合支付
    public static final String UNION_COUPON = "$upcoupon";                                                          //银联优惠券
    public static final String UNION_PAY = "$yl";                                                                   //银联卡（储蓄卡或信用卡）
    public static final String UNION_CLOUD_PAY = "$ysf";                                                            //云闪付(银联二维码支付)
    public static final String UNION_PREPAY_CARD = "$yfk";                                                          //银联预付卡
    public static final String UNION_WALLET = "$upwallet";                                                          //银联钱包
    public static final String UNION_VIP_CARD_1 = "$membercard1";                                                   //银联本地会员卡1
    public static final String UNION_VIP_CARD_2 = "$membercard2";                                                   //银联本地会员卡2
    public static final String UNION_POINTS = "$yhkjf";                                                             //银行卡积分支付
    public static final String UNION_INSTALLMENT = "$yhkfq";                                                        //银行卡分期付款
    public static final String BOCSCANPAY = "$bocscanpay";                                                          //微信扫码
    public static final String UMSCARD = "$umscard";                                                                //银联卡

    //不再支持的支付方式
    public static final String COUPON = "$cmpzq";                                                                   //赠券
    public static final String HXZF = "$hxzf";

    // 支付方式编码
    private String code;
    // 支付方式名称
    private String name;
    // 备注
    private String remark;
    // 支付类型
    private String payType;

    public PayMethod(String code, String name) {
        this.code = code;
        this.name = name;
    }

    public PayMethod(String code, String name, String remark) {
        this.code = code;
        this.name = name;
        this.remark = remark;
    }

    public PayMethod(String code, String name, String remark, String payType) {
        this.code = code;
        this.name = name;
        this.remark = remark;
        this.payType = payType;
    }

    /****************************************** 根据payType判断支付方式类型 ******************************************/

    /**
     * payType==0 现金类支付方式
     *
     * @return
     */
    public boolean isCashByPayType() {
        return isCashByPayType(this.payType);
    }

    public static boolean isCashByPayType(String payType) {
        return "0".equals(payType);
    }

    /**
     * 将支付方式作为现金处理。
     */
    public void asCash() {
        this.remark(PayMethod.CASH);
    }

    /**
     * payType==1 储值卡类支付方式
     *
     * @return
     */
    public boolean isCardByPayType() {
        return isCardByPayType(this.payType);
    }

    public static boolean isCardByPayType(String payType) {
        return "1".equals(payType);
    }

    /**
     * payType==2 银联卡类支付方式
     * payType==9 线上支付方式
     *
     * @return
     */
    public boolean isAllInPayByPayType() {
        return isAllInPayByPayType(this.payType);
    }

    public static boolean isAllInPayByPayType(String payType) {
        return "2".equals(payType) || "9".equals(payType);
    }

    /**
     * payType==4 购物券类支付方式
     *
     * @return
     */
    public boolean isCouponByPayType() {
        return isCouponByPayType(this.payType);
    }

    public static boolean isCouponByPayType(String payType) {
        return "4".equals(payType);
    }

    /****************************************** 慧收银专用支付方式 ******************************************/

    public boolean isHsyType() {
        return isAliFacePay() || isWxFacePay() ;
    }

    /**
     * 返回支付方式是否为支付宝人脸支付
     *
     * @return true 如果支付方式为支付宝人脸支付
     */
    public boolean isAliFacePay() {
        return PayMethod.isAliFacePay(this.remark());
    }

    public static boolean isAliFacePay(String remark) {
        return PayMethod.ALIFACEPAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为微信人脸支付
     *
     * @return true 如果支付方式为微信人脸支付
     */
    public boolean isWxFacePay() {
        return PayMethod.isWxFacePay(this.remark());
    }

    public static boolean isWxFacePay(String remark) {
        return PayMethod.WXFACEPAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /****************************************** 现金类支付方式 ******************************************/

    /**
     * 现金类支付方式
     *
     * @return
     */
    public boolean isCashType() {
        return isCash() || isOther() || isTotalReduction();
    }

    /**
     * 返回支付方式是否为现金
     *
     * @return true 如果支付方式为现金
     */
    public boolean isCash() {
        return PayMethod.isCash(this.remark);
    }

    public static boolean isCash(String remark) {
        return PayMethod.CASH.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为其他
     *
     * @return true 如果支付方式为其他
     */
    public boolean isOther() {
        return PayMethod.isOther(this.remark);
    }

    public static boolean isOther(String remark) {
        return PayMethod.OTHER.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为满减
     *
     * @return true 如果支付方式为满减则返回
     */
    public boolean isTotalReduction() {
        return PayMethod.CODE_TOTAL_REDUCTION.equals(this.code);
    }

    /****************************************** 卡、券类支付方式 ******************************************/

    /**
     * 储值卡支付方式
     *
     * @return
     */
    public boolean isCardType() {
        return isCard();
    }

    public static boolean isCardType(String remark) {
        return isCard(remark.trim());
    }

    /**
     * 积分支付方式
     *
     * @return
     */
    public boolean isPointsType() {
        return isVipPoints() || isScrmVipPoints();
    }

    public static boolean isPointsType(String remark) {
        return isVipPoints(remark) || isScrmVipPoints(remark.trim());
    }

    /**
     * 余额支付方式
     *
     * @return
     */
    public boolean isRemainType() {
        return isScrmRemain();
    }

    public static boolean isRemainType(String remark) {
        return isScrmRemain(remark.trim());
    }

    /**
     * 券支付方式
     *
     * @return
     */
    public boolean isCouponType() {
        return isVipCoupon() || isScrmCoupon() || isCoupon();
    }

    /**
     * 返回支付方式是否为储值卡
     *
     * @return true 如果支付方式为储值卡
     */
    public boolean isCard() {
        return PayMethod.isCard(this.remark);
    }

    public static boolean isCard(String remark) {
        return PayMethod.CARD.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为会员积分
     *
     * @return true 如果支付方式为会员积分
     */
    public boolean isVipPoints() {
        return PayMethod.isVipPoints(this.remark);
    }

    public static boolean isVipPoints(String remark) {
        return PayMethod.VIP_POINTS.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为会员券（一段式核销）
     *
     * @return true 如果支付方式为会员券（一段式核销）
     */
    public boolean isVipCoupon() {
        return PayMethod.isVipCoupon(this.remark);
    }

    public static boolean isVipCoupon(String remark) {
        return PayMethod.VIP_COUPON.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为SCRM积分支付
     *
     * @return true 如果支付方式为SCRM积分支付
     */
    public boolean isScrmVipPoints() {
        return PayMethod.isScrmVipPoints(this.remark);
    }

    public static boolean isScrmVipPoints(String remark) {
        return remark.compareToIgnoreCase(PayMethod.VIP_POINTS_SCRM) == 0;
    }

    /**
     * 返回支付方式是否为SCRM余额支付
     *
     * @return true 如果支付方式为SCRM余额支付
     */
    public boolean isScrmRemain() {
        return PayMethod.isScrmRemain(this.remark);
    }

    public static boolean isScrmRemain(String remark) {
        return PayMethod.SCRM_REMAIN_PAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为SCRM券支付
     *
     * @return true 如果支付方式为SCRM券支付
     */
    public boolean isScrmCoupon() {
        return PayMethod.isScrmCoupon(this.remark);
    }

    public static boolean isScrmCoupon(String remark) {
        return PayMethod.SCRM_COUPON.compareToIgnoreCase(remark.trim()) == 0;
    }

    /****************************************** 银联类（银行或全通道）支付方式 ******************************************/

    /**
     * 银联类（银行或全通道）支付方式
     *
     * @return
     */
    public boolean isAllOrChannelType() {
        return isAliPay() || isWxPay() || isMobilePay();
    }

    public static boolean isAllOrChannelType(String remark) {
        return isAliPay(remark.trim()) || isWxPay(remark.trim()) || isMobilePay(remark.trim());
    }

    /**
     * 返回支付方式是否为支付宝支付
     *
     * @return true 如果支付方式为支付宝支付
     */
    public boolean isAliPay() {
        return PayMethod.isAliPay(this.remark);
    }

    public static boolean isAliPay(String remark) {
        return PayMethod.ALIPAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为微信支付
     *
     * @return true 如果支付方式为微信支付
     */
    public boolean isWxPay() {
        return PayMethod.isWxPay(this.remark);
    }

    public static boolean isWxPay(String remark) {
        return PayMethod.WXPAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为聚合支付
     *
     * @return true 如果支付方式为聚合支付
     */
    public boolean isMobilePay() {
        return PayMethod.isMobilePay(this.remark());
    }

    public static boolean isMobilePay(String remark) {
        return PayMethod.MOBILEPAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /****************************************** 银联类（全通道）支付方式 ******************************************/

    /**
     * 银联类全通道支付
     * InterType=AllInPay;CallDllName=HsLKLAppPay;CardType=Cplu
     *
     * @return
     */
    public boolean isAllInPayType() {
        return this.remark.startsWith("InterType=AllInPay");
    }

    public static boolean isAllInPayType(String remark) {
        return remark.startsWith("InterType=AllInPay");
    }

    public String getAllInPayRemark() {
        return getAllInPayRemark(this.remark);
    }

    public static String getAllInPayRemark(String remark) {
        String[] payRemarks = remark.split(";");
        if (payRemarks.length > 0) {
            for (String payRemark : payRemarks) {
                if (payRemark.startsWith("CallDllName")) {
                    return payRemark.substring("CallDllName=".length()).toLowerCase();
                }
            }
        }

        return remark;
    }

    /**
     * 返回支付方式是否为一点通
     *
     * @return true 如果支付方式为一点通
     */
    public boolean isYdt() {
        return PayMethod.isYdt(this.remark);
    }

    public static boolean isYdt(String remark) {
        return PayMethod.YI_DIAN_TONG.compareToIgnoreCase(getAllInPayRemark(remark.trim())) == 0;
    }

    /**
     * 是否POS通
     *
     * @param remark 支付方式备注
     * @return true 如果支付方式为POS通
     */
    public static boolean isPOSTong(String remark) {
        return remark.compareToIgnoreCase(PayMethod.POS_TONG) == 0;
    }

    /**
     * 返回支付方式是POS通
     *
     * @return true 如果支付方式为pos通
     */
    public boolean isPOSTong() {
        return PayMethod.isPOSTong(this.remark);
    }
    /****************************************** 银联类（银行通道）支付方式 ******************************************/

    /**
     * 银联类（银行通道）支付方式
     *
     * @return
     */
    public boolean isUnionType() {
        return isUnionPay() || isUnionCloudPay();
    }

    public static boolean isUnionType(String remark) {
        return isUnionPay(remark) || isUnionCloudPay(remark);
    }

    /**
     * 银联卡支付方式
     *
     * @return
     */
    public boolean isUnionCardType() {
        return isUnionPrepayCard() || isUnionVipCard1() || isUnionVipCard2();
    }

    public static boolean isUnionCardType(String remark) {
        return isUnionPrepayCard(remark) || isUnionVipCard1(remark) || isUnionVipCard2(remark);
    }

    /**
     * 建联直连支付方式
     *
     * @return
     */
    public boolean isUccbType() {
        return isUccbCard() || isUccbScan();
    }

    public static boolean isUccbType(String remark) {
        return isUccbCard(remark) || isUccbScan(remark);
    }

    /**
     * 是否银行卡（走建行直连通道）
     *
     * @return true 如果支付方式为银行卡（走建行直连通道）
     */
    public boolean isUccbCard() {
        return PayMethod.isUccbCard(this.remark);
    }

    /**
     * 返回支付方式是否为满减。
     *
     * @return 如果支付方式为满减则返回 true
     */
    public static boolean isTotalReduction(String code) {
        return CODE_TOTAL_REDUCTION.equals(code);
    }


    public static boolean isUccbCard(String remark) {
        return PayMethod.UNION_CCB_CARD.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 是否聚合支付（走建行直连通道）
     *
     * @return true 如果支付方式为聚合支付（走建行直连通道）
     */
    public boolean isUccbScan() {
        return PayMethod.isUccbScan(this.remark);
    }

    public static boolean isUccbScan(String remark) {
        return PayMethod.UNION_CCB_SCAN.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为银联优惠券
     *
     * @return true 如果支付方式为银联优惠券
     */
    public boolean isUnionCoupon() {
        return PayMethod.isUnionCoupon(this.remark);
    }

    public static boolean isUnionCoupon(String remark) {
        return PayMethod.UNION_COUPON.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为银联卡
     *
     * @return true 如果支付方式为银联卡
     */
    public boolean isUnionPay() {
        return PayMethod.isUnionPay(this.remark);
    }

    public static boolean isUnionPay(String remark) {
        return remark.compareToIgnoreCase(PayMethod.UNION_PAY) == 0;
    }

    /**
     * 返回支付方式是否为云闪付(银联二维码支付)
     *
     * @return true 如果支付方式为云闪付(银联二维码支付)
     */
    public boolean isUnionCloudPay() {
        return PayMethod.isUnionCloudPay(this.remark);
    }

    public static boolean isUnionCloudPay(String remark) {
        return PayMethod.UNION_CLOUD_PAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为银联预付卡
     *
     * @return true 如果支付方式为银联预付卡
     */
    public boolean isUnionPrepayCard() {
        return PayMethod.isUnionPrepayCard(this.remark);
    }

    public static boolean isUnionPrepayCard(String remark) {
        return PayMethod.UNION_PREPAY_CARD.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为银联钱包
     *
     * @return true 如果支付方式为银联钱包
     */
    public boolean isUnionWallet() {
        return PayMethod.isUnionWallet(this.remark);
    }

    public static boolean isUnionWallet(String remark) {
        return PayMethod.UNION_WALLET.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为银联本地会员卡1
     *
     * @return true 如果支付方式为银联本地会员卡1
     */
    public boolean isUnionVipCard1() {
        return PayMethod.isUnionVipCard1(this.remark);
    }

    public static boolean isUnionVipCard1(String remark) {
        return PayMethod.UNION_VIP_CARD_1.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为银联本地会员卡2
     *
     * @return true 如果支付方式为银联本地会员卡2
     */
    public boolean isUnionVipCard2() {
        return PayMethod.isUnionVipCard2(this.remark);
    }

    public static boolean isUnionVipCard2(String remark) {
        return PayMethod.UNION_VIP_CARD_2.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为工行积分支付
     *
     * @return true 如果支付方式为工行积分支付
     */
    public boolean isIcbcPointPay() {
        return PayMethod.isUnionPoints(this.remark);
    }

    public static boolean isUnionPoints(String remark) {
        return PayMethod.UNION_POINTS.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回支付方式是否为工行分期付款
     *
     * @return true 如果支付方式为工行分期付款
     */
    public boolean isIcbcInstallmentPay() {
        return PayMethod.isUnionInstallment(this.remark);
    }

    public static boolean isUnionInstallment(String remark) {
        return PayMethod.UNION_INSTALLMENT.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回是否为中行扫码支付
     *
     * @return true 如果支付方式为中行扫码支付
     */
    public boolean isHsCloudPay() {
        return PayMethod.isHsCloudPay(this.remark());
    }

    public static boolean isHsCloudPay(String remark) {
        return PayMethod.HISENSE_CLOUND_PAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回是否为中行扫码支付
     *
     * @return true 如果支付方式为中行扫码支付
     */
    public boolean isBocScanPay() {
        return PayMethod.isBocScanPay(this.remark());
    }

    public static boolean isBocScanPay(String remark) {
        return PayMethod.BOCSCANPAY.compareToIgnoreCase(remark.trim()) == 0;
    }

    /**
     * 返回是否为中行扫码支付
     *
     * @return
     */
    public boolean isUmsCard() {
        return PayMethod.isUmsCard(this.remark());
    }

    public static boolean isUmsCard(String remark) {
        return PayMethod.UMSCARD.compareToIgnoreCase(remark.trim()) == 0;
    }

    /****************************************** 不再支持的支付方式 ******************************************/

    public boolean isAbandon() {
        return isCoupon() || isHxzf();
    }

    public boolean isCoupon() {
        return PayMethod.isCoupon(this.remark);
    }

    public static boolean isCoupon(String remark) {
        return PayMethod.COUPON.compareToIgnoreCase(remark.trim()) == 0;
    }

    public boolean isHxzf() {
        return PayMethod.isHxzf(this.remark);
    }

    public static boolean isHxzf(String remark) {
        return PayMethod.HXZF.compareToIgnoreCase(remark.trim()) == 0;
    }

    /****************************************** END ******************************************/

    public String code() {
        return this.code;
    }

    public void code(String code) {
        this.code = code;
    }

    public String name() {
        return this.name;
    }

    public void name(String name) {
        this.name = name;
    }

    public String remark() {
        return this.remark;
    }

    public String payRemark() {
        if (this.isAllInPayType()) {
            return getAllInPayRemark();
        } else {
            return this.remark;
        }
    }

    public void remark(String value) {
        this.remark = value;
    }

    public String payType() {
        return payType;
    }

    public void payType(String payType) {
        this.payType = payType;
    }

    @Override
    public String toString() {
        return "PayMethod{" +
                "code='" + code + '\'' +
                ", name='" + name + '\'' +
                ", remark='" + remark + '\'' +
                ", payType='" + payType + '\'' +
                '}';
    }
}
