/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.payment;

/**
 * 表示一笔支付信息。
 *
 * @author xieliyan 2018-01-25
 * @author zhangsai 2018-02-24 HIPOSAPP-334
 * @author zhangsai 2018-12-07 HIPOSAPP-888 新增traceNo和refNo属性
 */
public class Payment {

    private PayMethod payMethod;// 支付方式
    private String payNo;       // 支付号
    private String traceNo;     // 交易凭证号
    private String refNo;       // 交易参考号
    private double amount;      // 支付金额
    private double balance;     // 支付后余额

    /**
     * 初始化一个 {@code Payment} 的实例。
     *
     * @param payMethod 支付方式
     * @param payNo     支付号
     * @param amount    支付金额
     */
    public Payment(PayMethod payMethod, String payNo, double amount) {
        this.payMethod = payMethod;
        this.payNo = payNo;
        this.amount = amount;
    }

    /**
     * 初始化一个 {@code Payment} 的实例。
     *
     * @param payMethod 支付方式
     * @param payNo     支付号
     * @param traceNo   交易凭证号
     * @param refNo     交易参考号
     * @param amount    支付金额
     * @param balance   支付后余额
     */
    public Payment(PayMethod payMethod, String payNo, String traceNo, String refNo, double amount, double balance) {
        this(payMethod, payNo, amount);
        this.traceNo = traceNo;
        this.refNo = refNo;
        this.balance = balance;
    }

    /**
     * 返回支付方式。
     *
     * @return 支付方式
     */
    public PayMethod payMethod() {
        return this.payMethod;
    }

    /**
     * 返回支付号。
     *
     * @return 支付号
     */
    public String payNo() {
        return this.payNo;
    }

    /**
     * 返回交易参考号。
     *
     * @return 交易参考号
     */
    public String refNo() {
        return this.refNo;
    }

    /**
     * 返回交易凭证号。
     *
     * @return 交易凭证号
     */
    public String traceNo() {
        return this.traceNo;
    }

    /**
     * 返回支付金额。
     *
     * @return 支付金额
     */
    public double amount() {
        return this.amount;
    }

    /**
     * 返回支付后余额。
     *
     * @return 支付后余额
     */
    public double balance() {
        return this.balance;
    }
}
