package com.hisense.hipos.plu;

import com.hisense.common.util.BigDecimalUtils;

import java.io.Serializable;

import lombok.Data;

/**
 * 赠品/换购行为节点
 *
 * @author xieliyan 2018-11-13
 */
@Data
public class PluGiftAction implements Serializable {

    /**
     * 条件递增量
     */
    private String addConNum;
    /**
     * 递增满减量
     */
    private String addDscNum;
    /**
     * 条件递增类型
     */
    private String addDscType;
    /**
     * 条件基础数量
     */
    private String baseConNum;
    /**
     * 条件基础类型
     */
    private String baseConType;
    /**
     * 赠送量
     */
    private String baseDscNum;
    /**
     * 赠送类型
     */
    private String baseDscType;
    /**
     * 促销单号
     */
    private String dscCode;
    /**
     * 折扣价
     */
    private String dscPrice;
    /**
     * 优惠类型
     */
    private String dscType;
    /**
     * 折扣率
     */
    private String dscRate;
    /**
     * 换购商品编码
     */
    private String pluCode;
    /**
     * 商品名称
     */
    private String pluName;
    /**
     * 最大参与量
     */
    private String saleMaxIn;
    /**
     * 序号
     */
    private String serialNo;
    /**
     * 数量
     */
    private String quantity;

    private String total;

    public String dscProName;
    public String dscProType;
    /**
     * 全局限量类型
     */
    public String globalType;

    /**
     * 所属组别,  用于组合促销时 返回多个方案的情况, 即 必选加 多选一
     */
    public String group;
    /**
     * 组内数量
     */
    public String memberCount;
    /**
     * 组数量
     */
    public String groupCount;

    public void changeQuantity(int quantity) {
        if (quantity >= 0) {
            this.quantity = String.valueOf(quantity);
        } else {
            this.quantity = "0";
        }
    }

    public String increaseQuantity() {
        return BigDecimalUtils.add(this.quantity, "1");
    }

    public String decreaseQuantity() {
        return BigDecimalUtils.subtract(this.quantity, "1");
    }

    /**
     * 返回条件递增量
     *
     * @return 条件递增量
     */
    public String addConNum() {
        return this.addConNum;
    }

    /**
     * 返回递增满减量
     *
     * @return 递增满减量
     */
    public String addDscNum() {
        return this.addDscNum;
    }

    /**
     * 返回条件递增类型
     *
     * @return 条件递增类型
     */
    public String addDscType() {
        return this.addDscType;
    }

    /**
     * 返回条件基础数量
     *
     * @return 条件基础数量
     */
    public String baseConNum() {
        return this.baseConNum;
    }

    /**
     * 返回条件基础类型
     *
     * @return 条件基础类型
     */
    public String baseConType() {
        return this.baseConType;
    }

    /**
     * 返回赠送量
     *
     * @return 赠送量
     */
    public String baseDscNum() {
        return this.baseDscNum;
    }

    /**
     * 返回赠送类型
     *
     * @return 赠送类型
     */
    public String baseDscType() {
        return this.baseDscType;
    }

    /**
     * 返回促销单号
     *
     * @return 促销单号
     */
    public String dscCode() {
        return this.dscCode;
    }

    /**
     * 返回折扣价
     *
     * @return 折扣价
     */
    public String dscPrice() {
        return this.dscPrice;
    }

    /**
     * 返回优惠类型
     *
     * @return 优惠类型
     */
    public String dscType() {
        return this.dscType;
    }

    /**
     * 返回折扣率
     *
     * @return 折扣率
     */
    public String dscRate() {
        return this.dscRate;
    }

    /**
     * 返回换购商品编码
     *
     * @return 换购商品编码
     */
    public String pluCode() {
        return this.pluCode;
    }

    /**
     * 返回商品名称
     *
     * @return 商品名称
     */
    public String pluName() {
        return this.pluName;
    }

    /**
     * 返回最大参与量
     *
     * @return 最大参与量
     */
    public String saleMaxIn() {
        return this.saleMaxIn;
    }

    /**
     * 返回序号
     *
     * @return 序号
     */
    public String serialNo() {
        return this.serialNo;
    }

    /**
     * 换购数量
     *
     * @return 数量
     */
    public String quantity() {
        return this.quantity;
    }

    public String total() {
        return this.total;
    }
}
