/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.plu;


import java.security.InvalidParameterException;

/**
 * 商品类型。
 *
 * @author zhangsai    2018-06-13 HIPOSAPP-519
 * @author wangyaogang 2019-11-19 HIAPOSCMP-271 增加商品包明细类型2,不包含的类型抛出异常并提示
 * @author xieliyan    2019-12-13 pluType改为string类型
 */
public enum PluType {
    /**
     * 普通商品。
     */
    Normal("0"),

    /**
     * 商品包。
     */
    Package("1"),

    /**
     * 商品包明细。
     */
    PackageDetail("2");


    // 表示商品类型的值。
    private final String type;


    /**
     * 初始化一个 {@code PluType} 的实例。
     *
     * @param type 表示商品类型的值。
     */
    PluType(String type) {
        this.type = type;
    }

    /**
     * 返回值对应的商品类型。
     *
     * @param value 待转换的值
     * @return 值对应的商品类型
     */
    public static PluType valueOfDefined(String value) {
        if (value.equals(PluType.Normal.value())) {
            return PluType.Normal;
        } else if (value.equals(PluType.Package.value())) {
            return PluType.Package;
        } else if (value.equals(PluType.PackageDetail.value())) {
            return PluType.PackageDetail;
        } else {
            throw new InvalidParameterException("无效的商品类型值");
        }
    }

    /**
     * 返回商品类型的值。
     *
     * @return 商品类型的值。
     */
    public String value() {
        return this.type;
    }


}
