/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.report;

/**
 * 报表支付统计项。
 *
 * @author zhangsai 2017-10-12
 */
public class Payment {
    // 支付方式编码
    private String payCode;
    // 支付方式名称
    private String payName;
    // 合计支付金额
    private double totalPaid;
    // 合计实收金额
    private double totalReceived;
    // 合计退货金额
    private double totalReturned;


    /**
     * 初始化一个 {@code Payment} 的实例。
     */
    private Payment() {
        // NO CODE HERE. 避免初始化。
    }


    /**
     * 返回支付方式编码。
     * @return 支付方式编码
     */
    public String code() {
        return payCode;
    }

    /**
     * 返回支付方式名称。
     * @return 支付方式名称
     */
    public String name() {
        return payName;
    }

    /**
     * 返回合计支付金额。
     * @return 合计支付金额
     */
    public double totalPaid() {
        return totalPaid;
    }

    /**
     * 返回合计实收金额。
     * @return 合计实收金额
     */
    public double totalReceived() {
        return totalReceived;
    }

    /**
     * 返回合计退货金额。
     * @return 合计退货金额
     */
    public double totalReturned() {
        return totalReturned;
    }


    /**
     * 使用 Builder 模式初始化一个 {@code Payment} 的实例。
     */
    public static class Builder {
        // 支付方式编码
        private String payCode;
        // 支付方式名称
        private String payName;
        // 合计支付金额
        private double totalPaid;
        // 合计实收金额
        private double totalReceived;
        // 合计退货金额
        private double totalReturned;


        /**
         * 初始化一个 {@code Payment.Builder} 的实例。
         * @param payCode 支付方式编码
         * @param payName 支付方式名称
         */
        public Builder(String payCode, String payName) {
            this.payCode = payCode;
            this.payName = payName;
        }


        /**
         * 设置合计支付金额。
         * @param paid 合计支付金额
         * @return {@code Payment.Builder} 的实例。
         */
        public Builder paid(double paid) {
            this.totalPaid = paid;
            return this;
        }

        /**
         * 设置合计实收金额。
         * @param received 合计实收金额
         * @return {@code Payment.Builder} 的实例。
         */
        public Builder received(double received) {
            this.totalReceived = received;
            return this;
        }

        /**
         * 设置合计退货金额。
         * @param returned 合计退货金额
         * @return {@code Payment.Builder} 的实例。
         */
        public Builder returned(double returned) {
            this.totalReturned = returned;
            return this;
        }

        /**
         * 构建一个 {@code Payment} 的实例。
         */
        public Payment build() {
            Payment payment = new Payment();
            payment.payCode = this.payCode;
            payment.payName = this.payName;
            payment.totalPaid = this.totalPaid;
            payment.totalReceived = this.totalReceived;
            payment.totalReturned = this.totalReturned;

            return payment;
        }
    }
}
