package com.hisense.hipos.report;

import java.math.BigDecimal;
import java.util.Collection;

/**
 * 专柜销售报表。
 *
 * @author zhangsai 2018-12-28 HIPOSAPP-946
 */
public class ShopSaleReport {

    // 销售商品集合
    private Collection<ShopSaleReportItem> saleReportItems;
    // 合计实收金额
    private double total;

    /**
     * 初始化一个 {@code ShopSaleReport} 的实例。
     *
     * @param reportItems 销售商品集合
     */
    public ShopSaleReport(Collection<ShopSaleReportItem> reportItems) {
        this.saleReportItems = reportItems;
        this.total = calcTotal(reportItems);
    }

    /**
     * 计算合计实收金额。
     *
     * @param reportItems 销售商品集合
     * @return 合计实收金额
     */
    private static double calcTotal(Collection<ShopSaleReportItem> reportItems) {
        if (reportItems == null || reportItems.size() == 0) {
            return 0;
        }

        BigDecimal result = BigDecimal.ZERO;
        for (ShopSaleReportItem item : reportItems) {
            result = result.add(new BigDecimal(item.totalRecv()));
        }
        return result.doubleValue();
    }

    /**
     * 返回销售商品集合。
     *
     * @return 销售商品集合
     */
    public Collection<ShopSaleReportItem> items() {
        return this.saleReportItems;
    }

    /**
     * 返回合计实收金额。
     *
     * @return 合计实收金额
     */
    public double total() {
        return this.total;
    }
}
