/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.sale;

import com.hisense.common.util.StringUtils;

import java.io.Serializable;

/**
 * 满减类型。
 *
 * @author wangyaogang 2019-12-10  去掉会员促销接口,统一满减促销接口
 */
public enum CalcType implements Serializable {
    /**
     * 录入时
     */
    Enter(0),
    /**
     * 合计时
     */
    Total(1);

    // 表示满减类型的值。
    private final int type;


    /**
     * 初始化一个 {@code CalcType} 的实例。
     *
     * @param type 表示交易类型的值。
     */
    CalcType(int type) {
        this.type = type;
    }


    /**
     * 返回满减类型的值。
     *
     * @return 满减类型的值。
     */
    public int value() {
        return this.type;
    }

    /**
     * 返回当前交易类型是否为录入。
     *
     * @return true 表示为录入
     */
    public boolean isEnter() {
        return this == CalcType.Enter;
    }

    /**
     * 返回当前交易类型是否为合计。
     *
     * @return true 表示为合计
     */
    public boolean isTotal() {
        return this == CalcType.Total;
    }

    public static CalcType valueOf(int type) {
        if (type == CalcType.Enter.type) {
            return CalcType.Enter;
        } else {
            return CalcType.Total;
        }
    }

    public static CalcType valueOfType(String type) {
        if (StringUtils.toInt(type, CalcType.Enter.type) == CalcType.Enter.type) {
            return CalcType.Enter;
        } else {
            return CalcType.Total;
        }
    }
}
