/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.sale;

/**
 * 销售商品顺序号提供者程序。
 *
 * @author zhangsai    2017-11-22
 * @author wangyaogang 2019-02-02 HIPOSAPP-990 优化退货流水添加行号的逻辑,保证最终流水从给定初始值开始
 */
public class SaleItemNoProvider implements ISaleItemNoProvider {

    private int current; // 当前值
    private int min;     // 最小值
    private int next;    // 下一个值
    private int initialValue = 1; //初始值

    /**
     * 初始化一个 {@code SaleItemNoProvider} 的实例。
     */
    public SaleItemNoProvider() {
        this(1);
    }

    /**
     * 初始化一个 {@code SaleItemNoProvider} 的实例。
     *
     * @param initialValue 初始值
     */
    public SaleItemNoProvider(int initialValue) {
        this.initialValue = initialValue;
        this.reset();
    }

    /**
     * 返回当前的顺序号。
     *
     * @return 当前的顺序号
     */
    @Override
    public int current() {
        return this.current;
    }

    /**
     * 返回最小值。
     *
     * @return 最小值
     */
    @Override
    public int min() {
        return this.min;
    }


    /**
     * 在当前顺序号基础上减去 1。
     */
    @Override
    public void decrease() {
        this.next = this.current;
        this.current--;
    }

    /**
     * 返回当前的顺序号，并在当前顺序号基础上自动加 1。
     *
     * @return 当前的顺序号
     */
    @Override
    public int next() {
        this.current = this.next;
        this.next++;
        return this.current;
    }

    /**
     * 重置当前行号提供器
     */
    @Override
    public void reset() {
        this.current = this.initialValue - 1;
        this.next = this.initialValue;

        this.min = this.current;
    }
}
