/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.sale;

import com.hisense.common.util.StringUtils;

import java.io.Serializable;

/**
 * 交易类型。
 *
 * @author zhangsai 2018-02-24
 * @author zhangsai 2018-12-21 HIPOSAPP-699 增加 isPreSale() 和 isSale() 的快捷方法
 */
public enum TranType implements Serializable {
    /**
     * 销售/CRM退货/非整单退货
     */
    Sale(1),
    /**
     * SCRM整单退货
     */
    SCRMReturn(2),

    /**
     * 预销售
     */
    PreSale(9),
    /**
     * SCRM整单退货
     */
    SCRMPreReturn(6);

    // 表示交易类型的值。
    private final int type;


    /**
     * 初始化一个 {@code DiscountType} 的实例。
     *
     * @param type 表示交易类型的值。
     */
    TranType(int type) {
        this.type = type;
    }


    /**
     * 返回交易类型的值。
     *
     * @return 交易类型的值。
     */
    public int value() {
        return this.type;
    }

    /**
     * 返回当前交易类型是否为预销售。
     *
     * @return true 表示为预销售
     */
    public boolean isPreSale() {
        return this == TranType.PreSale;
    }

    /**
     * 返回当前交易类型是否为销售。
     *
     * @return true 表示为销售
     */
    public boolean isSale() {
        return this == TranType.Sale;
    }

    /**
     * 返回当前交易类型是否为SCRM整单退货。
     *
     * @return true 表示为销售
     */
    public boolean isScrmReturn() {
        return this == TranType.SCRMReturn;
    }

    public static TranType valueOf(int type) {
        if (type == TranType.Sale.type) {
            return TranType.Sale;
        } else {
            return TranType.PreSale;
        }
    }

    public static TranType valueOfType(String type) {
        if (StringUtils.toInt(type, TranType.Sale.type) == TranType.Sale.type) {
            return TranType.Sale;
        } else {
            return TranType.PreSale;
        }
    }
}
