/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.sale;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * 用户自定义属性。
 *
 * @author zhangsai 2018-02-24
 * @author zhangsai 2018-12-19 HIPOSAPP-698 自定义属性默认为 null
 */
public class UserDefinedProperty implements Serializable{
    // 默认自定义属性个数
    private static final int DEFAULT_SIZE = 5;

    // 存储自定义属性的集合
    private ArrayList<String> udp;
    // 自定义属性的个数
    private int size;

    public UserDefinedProperty() {
        this(DEFAULT_SIZE);
    }

    public UserDefinedProperty(int size) {
        if (size < 0) {
            this.size = DEFAULT_SIZE;
        } else {
            this.size = size;
        }

        this.udp = new ArrayList<>();
        for (int i = 0; i <= size; i++) {
            this.udp.add(null);
        }
    }


    /**
     * 返回 {@code index} 指定位置的用户自定义属性的值。
     *
     * @param index 用户自定义属性的索引
     * @return 指定位置的用户自定义属性的值
     */
    public String get(int index) {
        if (index >=0 && index <= this.size) {
            return this.udp.get(index);
        } else {
            return null;
        }
    }

    /**
     * 设置 {@code index} 指定位置的用户自定义属性的值。
     *
     * @param index 用户自定义属性的索引
     * @param value 用户自定义属性的值
     */
    public void set(int index, String value) {
        if (index >=0 && index <= this.size) {
            this.udp.set(index, value);
        }
    }
}
