/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.sale.models;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * 商定天下/商海纵横系统的流水号提供程序。
 *
 * @author zhangsai 2018-01-11 HIPOSAPP-271
 */
public class CmpSaleNoProvider extends SaleNoProvider {

    private CmpSaleNoSerialProvider serialNoProvider;

    /**
     * 初始化一个 {@code CmpSaleNoProvider} 的实例。
     *
     * @param repository 流水顺序号资源库
     */
    public CmpSaleNoProvider(ISerialNoRepository repository) {
        super(repository);

        serialNoProvider = new CmpSaleNoSerialProvider();
    }


    /**
     * 根据当前值计算下一个顺序号。
     *
     * @param serialNo 顺序号的当前值
     * @return 下一个顺序号
     */
    @Override
    protected int next(int serialNo) {
        return serialNoProvider.next(serialNo);
    }

    /**
     * 返回一个新的流水号。
     *
     * @param orgCode  组织编码
     * @param posNo    收款机号
     * @param saleTime 销售时间
     * @param serialNo 顺序号
     * @return 流水号
     */
    @Override
    public String format(String orgCode, String posNo, Date saleTime, int serialNo) {
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd", Locale.CHINESE);
        return df.format(saleTime) + posNo + new DecimalFormat("0000").format(serialNo);
    }

    /**
     * 从流水号中提取顺序号。
     *
     * @param saleNo 流水号
     * @return 顺序号
     */
    @Override
    public int extractSerialNo(String saleNo) {
        if (saleNo == null) {
            return -1;
        }
        return Integer.valueOf(saleNo.substring(saleNo.length() - 4));
    }

}
