/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.sale.models;

import com.hisense.crm.card.VipCard;
import com.hisense.hipos.sale.TranType;
import com.hisense.hipos.sale.UserDefinedProperty;
import com.hisense.hipos.user.User;

import java.util.Collection;

/**
 * 交易流水接口。
 *
 * @author zhangsai    2018-12-19 HIPOSAPP-698
 * @author wangyaogang 2019-05-13 HIPOSAPP-1087 控制每台机器收款的小数位
 * @author zhangsai    2019-06-13 HIPOSAPP-1169 支付信息集合改为 Collection&lt;ISalePayment&gt;
 */
public interface ISale {

    /**
     * 返回流水号。
     *
     * @return 流水号
     */
    String saleNo();

    /**
     * 返回组织编码。
     *
     * @return 组织编码
     */
    String orgCode();

    /**
     * 返回收款机号。
     *
     * @return 收款机号
     */
    String posNo();

    /**
     * 返回结账日期。
     *
     * @return 结账日期
     */
    String accDate();

    /**
     * 返回交易时间。
     *
     * @return 交易时间
     */
    String saleTime();

    /**
     * 返回是否会员交易。
     *
     * @return true 表示会员交易；false 表示普通交易
     */
    boolean isVip();

    /**
     * 返回会员卡信息。
     *
     * @return 会员卡信息
     */
    VipCard vipCard();

    /**
     * 返回收银员。
     *
     * @return 收银员
     */
    User cashier();

    /**
     * 返回交易类型。
     *
     * @return 交易类型
     */
    TranType tranType();

    /**
     * 返回交易扩展属性。
     *
     * @return 交易扩展属性
     */
    UserDefinedProperty udp();

    /**
     * 返回商品件数。
     *
     * @return 商品件数
     */
    int totalCount();

    /**
     * 返回商品数量。
     *
     * @return 商品数量
     */
    double totalQuantity();

    /**
     * 返回交易应收合计。
     *
     * @return 交易应收合计
     */
    double total();

    /**
     * 返回交易合计支付金额。
     *
     * @return 交易合计支付金额
     */
    double totalPaid();

    /**
     * 返回交易合计优惠金额。
     *
     * @return 交易合计优惠金额
     */
    double totalDsc();
}
