/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.hipos.sale.models;

import java.util.Date;

/**
 * 提供流水号的接口。
 *
 * @author zhangsai 2018-01-11 HIPOSAPP-271
 */
public interface ISaleNoProvider {

    /**
     * 使用指定信息格式化流水号。
     *
     * @param orgCode  组织编码
     * @param posNo    收款机号
     * @param saleTime 销售时间
     * @param serialNo 顺序号
     * @return 流水号
     */
    String format(String orgCode, String posNo, Date saleTime, int serialNo);

    /**
     * 保存流水号。
     *
     * @param saleNo 流水号
     */
    int extractSerialNo(String saleNo);

    /**
     * 返回指定收款机的当前流水顺序号。
     *
     * @param orgCode 门店编码
     * @param posNo   收款机号
     * @return 指定收款机的当前流水顺序号。-1 表示没有对应的记录。
     */
    int getSerialNo(String orgCode, String posNo);

    /**
     * 返回一个新的流水号。
     *
     * @param orgCode         组织编码
     * @param posNo           收款机号
     * @param saleTime        销售时间
     * @param defaultSerialNo 无法找到顺序号时的默认顺序号
     * @return 流水号
     */
    String next(String orgCode, String posNo, Date saleTime, int defaultSerialNo);

    /**
     * 保存流水号。
     *
     * @param orgCode 组织编码
     * @param posNo   收款机号
     * @param saleNo  流水号
     */
    void save(String orgCode, String posNo, String saleNo);
}
