/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.shop;

/**
 * 门店信息。
 *
 * @author guolina 2019-03-27
 */
public class Shop {

    // 组织编码
    private String orgCode;
    // 组织名称
    private String orgName;
    // 地址
    private String address;
    // 地理位置
    private Location location;
    // 电话
    private String telephone;

    /**
     * 初始化一个 {@code Shop} 的实例。
     */
    private Shop() {

    }

    /**
     * 地址。
     *
     * @return 地址。
     */
    public String address() {
        return this.address;
    }

    /**
     * 地理位置。
     *
     * @return 地理位置。
     */
    public Location location() {
        return this.location;
    }

    /**
     * 组织编码。
     *
     * @return 组织编码。
     */
    public String orgCode() {
        return this.orgCode;
    }

    /**
     * 组织名称。
     *
     * @return 组织名称。
     */
    public String orgName() {
        return this.orgName;
    }

    /**
     * 电话。
     *
     * @return 电话。
     */
    public String telephone() {
        return this.telephone;
    }


    public static class Builder {
        // 地址
        private String address;
        // 地理位置
        private Location location;
        // 组织编码
        private String orgCode;
        // 组织名称
        private String orgName;
        // 电话
        private String telephone;

        /**
         * 初始化一个 {@code Shop.Builder} 的实例。
         *
         * @param orgCode 组织编码
         * @param orgName 组织名称
         */
        public Builder(String orgCode, String  orgName) {
            this.orgCode = orgCode;
            this.orgName = orgName;
        }

        /**
         * 门店地址。
         *
         * @param address 门店地址
         * @return {@code Shop.Builder} 的实例。
         */
        public Builder address(String address) {
            this.address = address;
            return this;
        }

        /**
         * 门店地理位置。
         *
         * @param longitude 门店经度
         * @param latitude  门店纬度
         * @return {@code Shop.Builder} 的实例。
         */
        public Builder location(double longitude, double latitude) {
            this.location = new Location(longitude, latitude);
            return this;
        }

        /**
         * 门店电话。
         *
         * @param telephone 门店电话
         * @return {@code Shop.Builder} 的实例。
         */
        public Builder telephone(String telephone) {
            this.telephone = telephone;
            return this;
        }


        /**
         * 构建一个 {@code Shop} 的实例。
         */
        public Shop build() {
            Shop shop = new Shop();

            shop.orgCode = this.orgCode;
            shop.orgName = this.orgName;
            shop.location = this.location;
            shop.address = this.address;
            shop.telephone = this.telephone;

            return shop;
        }
    }
}
