/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.user;

import java.io.Serializable;
import java.util.Collection;

import lombok.Data;

/**
 * 用户。
 *
 * @author zhangsai    2017-10-12
 * @author wangyaogang 2019-01-07 HIPOSAPP-851 用户账号权限控制优化
 */
@Data
public class User implements Serializable {
    // 用户编码
    private String userCode;
    // 用户 ID
    private String userId;
    // 用户名称
    private String userName;
    //权限编码集合
    private Collection<String> privilegeList;

    /**
     * 初始化一个 {@code User} 的实例。
     *
     * @param userCode 用户编码
     */
    public User(String userCode) {
        this(null, userCode, null);
    }

    /**
     * 初始化一个 {@code User} 的实例。
     *
     * @param userId   用户 ID
     * @param userCode 用户编码
     * @param userName 用户名称
     */
    public User(String userId, String userCode, String userName) {
        this.userId = userId;
        this.userCode = userCode;
        this.userName = userName;
    }

    /**
     * 返回用户 ID。
     *
     * @return 用户 ID
     */
    public String id() {
        return this.userId;
    }

    /**
     * 返回用户编码。
     *
     * @return 用户编码
     */
    public String code() {
        return this.userCode;
    }

    /**
     * 返回用户名称。
     *
     * @return 用户名称
     */
    public String name() {
        return this.userName;
    }

    /**
     * 获取用户权限列表
     *
     * @return 权限列表
     */
    public Collection<String> privilegeList() {
        return this.privilegeList;
    }

    /**
     * 是否具有某项权限
     *
     * @param code 指定权限编码
     * @return true 有  false  无
     */
    public boolean hasPrivilege(String code) {
        if (privilegeList != null) {
            for (String s : privilegeList) {
                if (code.equals(s)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 使用 Builder 模式初始化一个 {@code User} 的实例。
     */
    public static class Builder {
        // 用户编码
        private String userCode;
        // 用户 ID
        private String userId;
        // 用户名称
        private String userName;
        //权限编码集合
        private Collection<String> privilegeList;

        /**
         * 初始化一个 {@code User.Builder} 的实例。
         *
         * @param userCode 用户编码
         */
        public Builder(String userCode) {
            this.userCode = userCode;
        }

        /**
         * 初始化一个 {@code User.Builder} 的实例。
         *
         * @param userCode 用户编码
         * @param userId   用户ID
         * @param userName 用户名称
         */
        public Builder(String userCode, String userId, String userName) {
            this.userCode = userCode;
            this.userId = userId;
            this.userName = userName;
        }

        /**
         * 设置权限列表
         *
         * @param privilegeList 权限列表
         * @return {@code User.Builder} 的实例。
         */
        public Builder setPrivilegeList(Collection<String> privilegeList) {
            this.privilegeList = privilegeList;
            return this;
        }

        /**
         * 构建一个 {@code User} 的实例。
         *
         * @return User实例
         */
        public User build() {
            User user = new User(this.userCode);
            user.userId = this.userId;
            user.userName = this.userName;
            user.privilegeList = this.privilegeList;
            return user;
        }
    }
}
