/***************************************************************************************************
 * Copyright (c) 2003-2017 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.hipos.yst;

/**
 * 赢商通配置。
 *
 * @author zhangsai 2017-10-12
 */
public class YstConfig {
    // 赢商通账号
    private String account;
    // 加密密钥
    private String key;
    // 服务地址
    private String url;


    /**
     * 初始化一个 {@code YstConfig} 的实例。
     */
    private YstConfig() {
        // NO CODE HERE. 避免初始化。
    }


    /**
     * 返回赢商通账号。
     * @return 赢商通账号。
     */
    public String account() {
        return this.account;
    }

    /**
     * 返回加密密钥。
     * @return 加密密钥。
     */
    public String key() {
        return this.key;
    }

    /**
     * 返回赢商通服务地址。
     * @return 赢商通服务地址。
     */
    public String url() {
        return this.url;
    }


    /**
     * 使用 Builder 模式初始化一个 {@code YstConfig} 的实例。
     */
    public static class Builder {
        // 赢商通账号
        private String account;
        // 加密密钥
        private String key;
        // 服务地址
        private String url;


        /**
         * 初始化一个 {@code YstConfig.Builder} 的实例。
         * @param url 赢商通服务地址。
         */
        public Builder(String url) {
            this.url = url;
        }


        /**
         * 设置赢商通账号/密钥。
         * @param account   账号
         * @param key       加密密钥
         * @return {@code YstConfig.Builder} 的实例。
         */
        public Builder account(String account, String key) {
            this.account = account;
            this.key = key;

            return this;
        }

        /**
         * 构建一个 {@code YstConfig} 的实例。
         */
        public YstConfig build() {
            YstConfig config = new YstConfig();
            config.url = this.url;
            config.account = this.account;
            config.key = this.key;

            return config;
        }
    }

}
