/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.boc;

import android.content.Context;

import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1CardReaderProvider;

/**
 * 中行读卡器
 *
 * @author wangyaogang 2018-11-10 HIPOSAPP-815 支持中行读卡
 * @author wangyaogang 2018-12-01 HIPOSAPP-814 更新中行刷卡接口
 */
public class BOCCardReader implements ICardReader {

    /**
     * Mifare1 非接触式磁卡读卡器
     */
    private M1CardReader m1CardReader;

    /**
     * 初始化一个 {@code BOCCardReader} 的实例。
     *
     * @param provider Mifare1 非接触式磁卡读取器提供程序
     * @param version  Mifare1 版本
     */
    public BOCCardReader(M1CardReaderProvider provider, int version) {
        m1CardReader = new M1CardReaderV2(version, provider);
    }

    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {

        if (m1CardReader != null) {
            m1CardReader.open(context);
        }
    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {

        if (m1CardReader != null) {
            m1CardReader.read(password, callback);
        }
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {

        if (m1CardReader != null) {
            m1CardReader.start();
        }
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {

        if (m1CardReader != null) {
            m1CardReader.stop();
        }
    }


    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {

        if (m1CardReader != null) {
            m1CardReader.close();
        }
    }
}
