/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.boc;

import android.content.Context;

import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;

/**
 * Mifare1 非接触式磁卡读取器的基类。
 *
 * @author wangyaogang 2018-11-09 HIPOSAPP-815 支持中行IC卡读卡
 * @author wangyaogang 2018-12-01 HIPOSAPP-814 更新中行刷卡接口
 */
abstract class M1CardReader implements ICardReader {

    /**
     * 是否正在监听。避免启动多个监听线程。
     */
    private static volatile boolean IS_RUNNING = false;
    /**
     * 是否已读取卡号。
     */
    protected volatile boolean isRead = false;

    /**
     * 监听程序
     */
    private PiccListener cardListener;

    /**
     * 读卡密码
     */
    private String password;
    /**
     * 读取卡号的回调方法
     */
    private CardNoCallback callback;

    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {

    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {

        this.password = password;
        this.callback = callback;

        this.start();
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
       /* if (!DeviceService.Holder.getInstance().isConnection()) {
            return;
        }*/
        // 如果已启动，则立即退出。避免启动多个监听线程。
        if (IS_RUNNING) {
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;
        isRead = false;

        cardListener = new PiccListener();
        cardListener.start();
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }

        IS_RUNNING = false;
    }

    /**
     * 关闭读卡器。 & 关闭非接模块
     */
    @Override
    public void close() {
        this.stop();

    }

    /**
     * 使用读卡密码读取卡内数据。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    protected abstract void innerRead(String password, CardNoCallback callback);

    /**
     * 非接触式磁卡监听程序。
     *
     * @author wangyaogang 2018-11-09 中行
     */
    private class PiccListener extends CardListener {

        /**
         * 在一个监听周期后探测并读卡。
         */
        @Override
        protected void onListening() {
            if (isRead) {
                return;
            }
            M1CardReader.this.innerRead(password, callback);

        }

    }

}
