/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.payment.boc;

import android.os.Bundle;

import com.hisense.pos.payment.PayResult;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * 中行收单应用返回的交易结果。
 *
 * @author zhangsai    2018-12-17 HIPOSAPP-914
 * @author guolina     2019-04-03 HIPOSAPP-1076 中行微信支付宝隔日退款返回参数调整
 * @author wangyaogang 2019-09-11 HIPOSAPP-1301 支持中行支付查询
 */
public class BocPayResult extends PayResult {

    private Bundle bundle;

    /**
     * 初始化一个 {@code BocPayResult} 的实例。
     *
     * @param bundle 返回数据
     */
    BocPayResult(Bundle bundle) {
        if (bundle != null) {
            this.bundle = bundle;
        } else {
            this.bundle = new Bundle();
        }
    }


    /**
     * 交易总金额。与上送的 TxnAmt 值相同，格式也相同。
     */
    @Override
    public double amount() {
        return bundle.getDouble("TotalAmt");
    }

    /**
     * 返回卡号。
     */
    @Override
    public String cardNo() {
        String payerID = bundle.getString("PayerID", null);
        if (payerID == null) {
            payerID = bundle.getString("ChannelTxnNo", null);
            if (payerID == null) {
                String temp = bundle.getString("MultData");
                //payerID = temp.substring(0, temp.indexOf("|"));
                JSONObject transData = null;
                try {
                    transData = new JSONObject(temp);
                    if (transData.get("MD001") != null && transData.get("MD001") != "") {
                        payerID = transData.getString("MD001");
                    } else if (transData.get("MD006") != null && transData.get("MD006") != "") {
                        payerID = transData.getString("MD006");
                    } else {
                        payerID = "";
                    }
                } catch (JSONException e) {
                    payerID = "";
                    e.printStackTrace();
                }


            }
        }
        return payerID;
    }

    /**
     * 原交易信息（撤销/退款用）。
     */
    @Override
    public String refNo() {
        return bundle.getString("MultData");
    }

    /**
     * 原交易信息（撤销/退款用）。
     */
    @Override
    public String traceNo() {
        return bundle.getString("MultData");
    }

    /**
     * 终端号。
     */
    @Override
    public String termNo() {
        return bundle.getString("TerminalID");
    }

    @Override
    public String outTradeNo() {
        return null;
    }

    @Override
    public String mchId() {
        return null;
    }

    @Override
    public String payType() {
        return null;
    }

    /**
     * 返回当前交易是否为消费。
     *
     * @return true 表示消费，false 表示其它。
     */
    @Override
    public boolean isPayment() {
        return TransType.PAY.equals(bundle.getString("TxnType"));
    }

    /**
     * 返回当前交易是否为消费撤销。
     *
     * @return true 表示消费撤销，false 表示其它。
     */
    @Override
    public boolean isCancellation() {
        return TransType.CANCEL.equals(bundle.getString("TxnType"));
    }

    /**
     * 返回当前交易是否为退货。
     *
     * @return true 表示退货，false 表示其它。
     */
    @Override
    public boolean isRefunding() {
        return TransType.REFUND.equals(bundle.getString("TxnType"));
    }

    /**
     * 返回当前交易是否为查询。
     *
     * @return true 表示查询，false 表示其它。
     */
    @Override
    public boolean isQuery() {
        return TransType.QUERY.equals(bundle.getString("TxnType"));
    }
}
