/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.payment.boc;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;

import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PayResult;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.hisense.pos.print.boc.BOCDeviceHelper;

import java.util.Date;
import java.util.Map;

/**
 * 中行收单应用支付提供程序。
 *
 * @author zhangsai    2018-12-17 HIPOSAPP-914
 * @author wangyaogang 2019-05-27 HIPOSAPP-1140 调整中行版打印服务逻辑,调用收单行渠道的操作前解绑打印服务,返回结果后重新绑定服务
 * @author wangyaogang 2019-09-11 HIPOSAPP-1301 支持中行支付查询
 */
public class BocPaymentProvider extends BocProviderBase implements IPaymentProvider {

    private PayMode payMode; // 支付模式
    private String TxnType;//交易类型

    /**
     * 初始化一个 {@code BocPaymentProvider} 的实例。
     *
     * @param activity Activity 活动
     * @param payMode  支付模式
     */
    public BocPaymentProvider(Activity activity, PayMode payMode) {
        super(activity);
        this.payMode = payMode;
    }


    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 支付结果
     * @throws PaymentProviderException 支付异常
     */
    @Override
    public PayResult handleResult(Intent data) throws PaymentProviderException {
        BOCDeviceHelper.getInstance().bindDeviceService();
        if (null == data) {
            if (TransType.PAY.equals(this.TxnType) || TransType.QUERY.equals(this.TxnType)) {
                throw new PaymentProviderException(PayResult.CODE_NO_DATA, "未返回任何数据");
            } else {
                throw new PaymentProviderException("未返回任何数据");
            }
        }

        //XX,交易失败直接返回,X0 ,如果是支付或者查询则进行查询
        Bundle bundle = data.getExtras();
        if ("XX".equals(bundle.getString("RespCode"))) {
            throw new PaymentProviderException(bundle.getString("RespDesc"));
        } else if ("X0".equals(bundle.getString("RespCode"))) {
            if (TransType.PAY.equals(this.TxnType) || TransType.QUERY.equals(this.TxnType)) {
                throw new PaymentProviderException(PayResult.CODE_NO_DATA, bundle.getString("RespDesc"));
            } else {
                throw new PaymentProviderException(bundle.getString("RespDesc"));
            }
        }

        return new BocPayResult(bundle);
    }

    /**
     * 返回是否支持撤销。
     *
     * @return true 表示支持撤销，false 表示不支持撤销
     */
    @Override
    public boolean supportCancellation() {
        return true;
    }

    /**
     * 返回是否支持交易查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    @Override
    public boolean supportPayQuery() {
        return true;
    }

    /**
     * 返回是否支持撤销查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    @Override
    public boolean supportCancelQuery() {
        return true;
    }

    /**
     * 返回是否支持退款查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    @Override
    public boolean supportRefundQuery() {
        return true;
    }

    /**
     * 消费。
     *
     * @param payRequest 交易金额
     * @param handler    支付处理程序
     */
    @Override
    public void pay(PayRequest payRequest, PaymentHandler handler) throws PaymentProviderException {
        this.TxnType = TransType.PAY;
        Uri uri = Uri.parse(BASE_URI
                + "payTrans"
                + "?TxnType=" + TransType.PAY
                + "&PayMode=" + payMode.value()
                + "&TxnAmt=" + formatAmount(payRequest.getAmount())
                + "&MerchantTxnNo=" + payRequest.getUniqueNo()
                + "&TxnReqTime=" + getCurrentTime());
        callTrans(activity, uri, BOC_REQUEST_CODE);
    }

    @Override
    public void payQuery(String uniqueNo, PaymentHandler handler) throws PaymentProviderException {
        this.TxnType = TransType.QUERY;
        Uri uri = Uri.parse(BASE_URI
                + "payTrans"
                + "?TxnType=" + TransType.QUERY
                + "&PayMode=" + payMode.value()
                + "&OrgTxnType=" + TransType.PAY
                + "&OrgMerchantTxnNo=" + uniqueNo);
        callTrans(activity, uri, BOC_REQUEST_CODE);
    }

    /**
     * 退货。
     *
     * @param payRequest 退款金额
     * @param refNo      参考号
     * @param transDate  交易日期
     * @param params     扩展参数
     * @param handler    退款处理程序
     */
    @Override
    public void refund(PayRequest payRequest, String refNo, Date transDate, Map<String, String> params,
                       PaymentHandler handler) throws PaymentProviderException {
        this.TxnType = TransType.REFUND;
        Uri uri = Uri.parse(BASE_URI
                + "payTrans"
                + "?TxnType=" + TransType.REFUND
                + "&PayMode=" + payMode.value()
                // + "&MerchantTxnNo=" + refundRequest.getMerchantTxnNo()
                + "&TxnAmt=" + formatAmount(payRequest.getAmount())
                + "&OrgMultData=" + refNo
                + "&TxnReqTime=" + getCurrentTime());
        callTrans(activity, uri, BOC_REQUEST_CODE);
    }

    /**
     * 撤销。
     *
     * @param payRequest 退款金额
     * @param traceNo    原交易凭证号
     * @param params     扩展参数
     * @param handler    撤销处理程序
     */
    @Override
    public void cancel(PayRequest payRequest, String traceNo, Map<String, String> params,
                       PaymentHandler handler) throws PaymentProviderException {
        this.TxnType = TransType.CANCEL;
        Uri uri = Uri.parse(BASE_URI
                + "payTrans"
                + "?TxnType=" + TransType.CANCEL
                + "&PayMode=" + payMode.value()
                // + "&MerchantTxnNo=" + cancelRequest.getMerchantTxnNo()
                // + "&TxnAmt=" + getFormatMoney(cancelRequest.getTxnAmt())
                + "&OrgMultData=" + traceNo
                + "&TxnReqTime=" + getCurrentTime());
        callTrans(activity, uri, BOC_REQUEST_CODE);
    }
}
