/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.payment.boc;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;

import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.hisense.pos.print.boc.BOCDeviceHelper;

import java.util.HashMap;
import java.util.Map;

/**
 * 中行收单支付功能提供程序。
 *
 * @author wangyaogang 2018-12-18 HIPOSAPP-915 中行支持重打印签购单
 * @author wangyaogang 2019-05-27 HIPOSAPP-1140 调整中行版打印服务逻辑,调用收单行渠道的操作前解绑打印服务,返回结果后重新绑定服务
 */
public class BocPaymentServiceProvider extends BocProviderBase implements IPaymentServiceProvider {
    private final static String PROVIDER_NAME = "中行收单";

    /**
     * 初始化一个 {@code BocPaymentServiceProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    public BocPaymentServiceProvider(Activity activity) {
        super(activity);
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 返回结果字典
     * @throws PaymentProviderException 支付服务异常
     */
    @Override
    public Map handleResult(Intent data) throws PaymentProviderException {
        BOCDeviceHelper.getInstance().bindDeviceService();
        if (data == null) {
            throw new PaymentProviderException("未返回任何数据");
        }

        Bundle bundle = data.getExtras();
        if (!"00".equals(bundle.getString("RespCode"))) {
            throw new PaymentProviderException(bundle.getString("RespDesc"));
        }

        Map<String, String> map = new HashMap<>();
        return map;
    }

    /**
     * 返回提供程序名称。
     *
     * @return 提供程序名称
     */
    @Override
    public String providerName() {
        return PROVIDER_NAME;
    }

    /**
     * 重打印签购单。
     *
     * @param traceNo 交易凭证号
     */
    @Override
    public void reprintReceipt(String traceNo, PaymentHandler handler) throws PaymentProviderException {
        String payMode = "1";
        if (traceNo != null) {
            if (traceNo.contains("WP")) {
                payMode = "7";
            } else if (traceNo.contains("ZP")) {
                payMode = "6";
            } else {
                payMode = "1";
            }
        }
        Uri uri = Uri.parse(BASE_URI
                + "payTrans"
                + "?TxnType=" + TransType.REPRINT
                + "&PayMode=" + payMode
                + "&TxnReqTime=" + getCurrentTime());
        Log.d("wyg", "重打印:" + uri.toString());
        callTrans(activity, uri, BOC_REQUEST_CODE);
    }

    /**
     * 重打印结算单。
     */
    @Override
    public void reprintSettlement() {
        //中行不支持
    }

    /**
     * 打印交易汇总。
     */
    @Override
    public void printTransSummary(PaymentHandler handler) throws PaymentProviderException {
        //中行不支持
    }

    /**
     * 查询银行卡余额。
     */
    @Override
    public void queryBalance(PaymentHandler handler) throws PaymentProviderException {
        //中行不支持
    }

    /**
     * 结算。
     */
    @Override
    public void settle(PaymentHandler handler) throws PaymentProviderException {
        Uri uri = Uri.parse(BASE_URI
                + "payTrans"
                + "?TxnType=" + TransType.SETTLE
                + "&PayMode=" + "1"
                + "&TxnReqTime=" + getCurrentTime());
        callTrans(activity, uri, BOC_REQUEST_CODE);
    }

    /**
     * 换班。
     */
    @Override
    public void shift() {
        //中行不支持
    }

    /**
     * 班次查询
     */
    @Override
    public void queryBatchNo() {
        //不支持
    }

    /**
     * 交易查询
     *
     * @param traceNo            凭证号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单；
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void tradeQuery(String traceNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        //不支持
    }

    /**
     * @param orderNo            商户订单号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void scanTradeQuery(String orderNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        //不支持
    }
}
