/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.payment.boc;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;

import com.hisense.pos.payment.PaymentProvider;
import com.hisense.pos.print.boc.BOCDeviceHelper;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * 中行收单应用提供程序的基类。
 *
 * @author zhangsai    2018-12-18 HIPOSAPP-914
 * @author wangyaogang 2019-05-27 HIPOSAPP-1140 调整中行版打印服务逻辑,调用收单行渠道的操作前解绑打印服务,返回结果后重新绑定服务
 */
abstract class BocProviderBase extends PaymentProvider {

    private final static String REQUEST_TIME_FORMAT = "yyyy/MM/dd HH:mm:ss";
    private final static SimpleDateFormat REQUEST_TIME_FORMATTER = new SimpleDateFormat(REQUEST_TIME_FORMAT, Locale.CHINESE);

    /**
     * 跳转Activity所需URI的公共部分
     */
    static final String BASE_URI = "sssoft://sssoft.uri.activity/";

    /**
     * 支付的请求码
     */
    static final int BOC_REQUEST_CODE = 2002;


    /**
     * 初始化一个 {@code BocProviderBase} 的实例。
     *
     * @param activity Activity 活动
     */
    BocProviderBase(Activity activity) {
        super(activity);
    }

    /**
     * 调用 POS 交易接口。
     *
     * @param activity    Activity
     * @param uri         调用地址
     * @param requestCode Activity 请求编码
     */
    static void callTrans(Activity activity, Uri uri, int requestCode) {
        BOCDeviceHelper.getInstance().unBindDeviceService();

        Intent intent = new Intent("android.sssoft.schemeurl.activity");
        intent.setData(uri);
        activity.startActivityForResult(intent, requestCode);
    }

    /**
     * 格式化交易金额。
     *
     * @param amount 交易金额
     * @return 符合中行收单请求标准的交易金额
     */
    static String formatAmount(double amount) {
        DecimalFormat df = new DecimalFormat("##############0.00");
        return df.format(amount);
    }

    /**
     * 获取当前时间。
     *
     * @return "yyyy/MM/dd  HH:mm:ss" 格式的字符串
     */
    static String getCurrentTime() {
        return REQUEST_TIME_FORMATTER.format(new Date());
    }


    /**
     * 返回 Android 请求编码。
     *
     * @return Android 请求编码
     */
    public int requestCode() {
        return BOC_REQUEST_CODE;
    }
}
