/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.print.boc;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.os.RemoteException;
import android.os.SystemClock;
import android.util.Log;

import com.boc.aidl.deviceService.AidlDeviceService;
import com.boc.aidl.printer.AidlPrinter;
import com.landi.bocapilibrary.DeviceService;

/**
 * 中行设备驱动调用
 *
 * @author wangyaogang 2018-11-09 HIPOSAPP-813 新增中行收单功能-读卡、打印
 * @author wangyaogang 2018-11-16 HIPOSAPP-817 优化中行打印逻辑
 * @author wangyaogang 2018-11-19 HIPOSAPP-817 优化打印状态的判断逻辑
 * @author wangyaogang 2018-12-01 HIPOSAPP-814 增加中行刷卡接口服务绑定和解绑方法
 * @author wangyaogang 2018-12-19 HIPOSAPP-897 更新中行打印接口,修复主控占用问题
 * @author guolina     2019-04-02 HIPOSAPP-1073 修复因刚安装未配置好组织编码等参数时日志上传CMP5导致的闪退
 * @author wangyaogang 2018-05-13 HIPOSAPP-1091 衢州东方-易收银功耗问题排查并完善
 * @author wangyaogang 2019-05-27 HIPOSAPP-1140 调整中行版打印服务逻辑,调用收单行渠道的操作前解绑打印服务,返回结果后重新绑定服务
 */
public class BOCDeviceHelper {
    private final String TAG = "TradeLog";
    private static BOCDeviceHelper deviceHelper;
    private Context mContext;
    private static AidlDeviceService deviceService;

    private BOCDeviceHelper() {
    }

    public static BOCDeviceHelper getInstance() {
        if (deviceHelper == null) {
            synchronized (BOCDeviceHelper.class) {
                if (deviceHelper == null) {
                    deviceHelper = new BOCDeviceHelper();
                }
            }
        }
        return deviceHelper;
    }

    public BOCDeviceHelper init(Context context) {
        mContext = context;
        return getInstance();
    }

    private ServiceConnection serviceConnection = new ServiceConnection() {

        @Override
        public void onServiceDisconnected(ComponentName name) {
            deviceService = null;
            Log.d(TAG, "boc服务断开");
//            bindDeviceService();
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            deviceService = AidlDeviceService.Stub.asInterface(service);
            linkToDeath(service);
            Log.d(TAG, "boc服务连接");
        }

        private void linkToDeath(IBinder service) {
            try {
                service.linkToDeath(new IBinder.DeathRecipient() {
                    @Override
                    public void binderDied() {
                        Log.d(TAG, "boc服务Death");
                        //bindDeviceService();
                    }
                }, 0);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    };

    /**
     * 绑定服务
     */
    public void bindDeviceService() {
        if (deviceService == null) {
            Intent intent = new Intent();
            intent.setAction("com.boc.DeviceService");
            intent.setPackage("com.boc.spos.service");
            mContext.bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);
            Log.d(TAG, "boc开始绑定服务");
        } else {
            Log.d(TAG, "boc服务已经启动");
        }
        SystemClock.sleep(300);
    }

    /**
     * 解绑服务
     */
    public void unBindDeviceService() {
        if (deviceService != null) {
            Log.d(TAG, "解绑boc服务");
            try {
                mContext.unbindService(serviceConnection);
            } catch (Exception e) {
                e.printStackTrace();
                Log.d(TAG, "boc服务解绑异常" + e.getMessage());
            } finally {
                deviceService = null;
            }

        } else {
            Log.d(TAG, "boc服务未启动");
        }
        SystemClock.sleep(300);
    }

    public AidlPrinter getNewPrinter() {
        AidlPrinter printerModule = null;
        try {
            printerModule = AidlPrinter.Stub.asInterface(deviceService.getPrinter());
        } catch (Exception e1) {
            e1.printStackTrace();
            Log.d(TAG, "获取boc打印机错误" + e1.getMessage());
        }
        return printerModule;
    }

    /**
     * 绑定刷卡服务
     */
    public void bind() {
        Log.d(TAG, "绑定刷卡服务");
        com.landi.bocapilibrary.DeviceService.Holder.getInstance().bindDeviceService(mContext);
        //  SystemClock.sleep(200);
    }

    /**
     * 解绑刷卡服务
     */
    public void unBind() {
        Log.d(TAG, "关闭非接载波,解绑刷卡服务");
        try {
            if (DeviceService.Holder.getInstance().getCardReader() != null) {
                DeviceService.Holder.getInstance().getCardReader().closeCardReader();
            }
        } catch (RemoteException e) {
            e.printStackTrace();
            Log.d(TAG, "关闭非接载波异常," + e.toString());
        }
        com.landi.bocapilibrary.DeviceService.Holder.getInstance().unbindService();
        SystemClock.sleep(800);
    }
}
