/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.ccb;

import android.content.Context;
import android.os.Bundle;
import android.os.ConditionVariable;
import android.os.IBinder;
import android.os.RemoteException;

import com.ccb.deviceservice.aidl.IDeviceService;
import com.ccb.deviceservice.aidl.magreader.IMagCardReader;
import com.ccb.deviceservice.aidl.magreader.OnSearchListener;
import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;

/**
 * {功能描述}
 *
 * @author zhangsai 2018-01-18
 */
class MagCardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;

    // 监听程序
    private MagCardListener cardListener;

    // 读取卡号的回调方法
    private CardNoCallback callback;

    private IMagCardReader magCardReader;
    private IDeviceService deviceService;

    public MagCardReader(IDeviceService deviceService) {
        this.deviceService = deviceService;
    }

    @Override
    public void open(final Context context) {

    }

    @Override
    public void read(String password, CardNoCallback callback) {
        try {
            IBinder binder = this.deviceService.getMagCardReader();
            magCardReader = IMagCardReader.Stub.asInterface(binder);

            this.callback = callback;

            this.start();
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("ccb_MagCardReader_read异常:" + e.toString());
        }
    }

    @Override
    public void start() {
        if (magCardReader == null) {
            LoggerUtils.getLogger().debug("ccb_MagCardReader_start异常:magCardReader为null");
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;

        cardListener = new MagCardListener(magCardReader, callback);
        cardListener.start();
    }

    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }

        IS_RUNNING = false;
    }

    @Override
    public void close() {
        this.stop();
    }

    /**
     * 磁条卡读卡处理
     *
     * @author xieliyan 2017-09-25 HIPOSAPP-160
     */
    private static class MagCardListener extends CardListener {

        private CardNoCallback callback;
        private IMagCardReader card;
        private ConditionVariable conditionVariable = new ConditionVariable();

        public MagCardListener(IMagCardReader card, CardNoCallback callback) {
            this.card = card;
            this.callback = callback;
        }

        @Override
        protected void onListening() {
            try {
                card.searchCard(10000000, new OnSearchListener.Stub() {

                    @Override
                    public void onSuccess(Bundle bundle) throws RemoteException {
                        String cardNo = bundle.getString("TRACK2");
                        LoggerUtils.getLogger().debug("ccb_MagCardReader_onSuccess:" + cardNo);
                        if (!"".equals(cardNo)) {
                            callback.onSuccess(cardNo);
                        }
                        conditionVariable.open();
                    }

                    @Override
                    public void onError(int i) throws RemoteException {
                        conditionVariable.open();
                        LoggerUtils.getLogger().debug("ccb_MagCardReader_searchCard:" + i);
                    }

                    @Override
                    public void onTimeout() throws RemoteException {
                        conditionVariable.open();
                        LoggerUtils.getLogger().debug("ccb_MagCardReader_onTimeout");
                    }
                });
            } catch (RemoteException e) {
                e.printStackTrace();
                LoggerUtils.getLogger().debug("ccb_MagCardReader_searchCard异常:" + e.toString());
                conditionVariable.open();
            }
            conditionVariable.block();
        }

        @Override
        protected void onStop() {
            conditionVariable.close();
        }
    }
}
