/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.ccb;

import com.hisense.pos.payment.PayResult;

import org.json.JSONException;
import org.json.JSONObject;

import java.math.BigDecimal;

/**
 * 建行收单应用返回的交易结果。
 *
 * @author zhangsai    2018-11-26 HIPOSAPP-888
 * @author wangyaogang 2019-08-19 HIPOSAPP-1225 银联商务增加支付查询,修改了父类,其他行一同修改
 */
public class CcbPayResult extends PayResult {

    private String appName;         // 应用名称
    private String transId;         // 交易ID
    private JSONObject transData;   // 交易结果
    private double amount;          // 交易金额

    /**
     * 初始化一个 {@code CcbPayResult} 的实例。
     *
     * @param appName   应用名称
     * @param transId   交易ID
     * @param transData 交易结果
     */
    public CcbPayResult(String appName, String transId, JSONObject transData) {
        this.appName = appName;
        this.transId = transId;

        if (transData != null) {
            this.transData = transData;
        } else {
            this.transData = new JSONObject();
        }

        this.amount = this.parseAmount("amt");
    }

    /**
     * 返回交易金额。
     *
     * @return 交易金额
     */
    @Override
    public double amount() {
        return this.amount;
    }

    /**
     * 返回卡号。
     *
     * @return 卡号
     */
    @Override
    public String cardNo() {
        return this.getString("cardNo");
    }

    /**
     * 返回参考号。
     *
     * @return 参考号
     */
    @Override
    public String refNo() {
        if (transId.equals("消费")) {
            return this.getString("refNo");
        } else {
            return this.getString("wxAliPayOrderNo");
        }
    }

    /**
     * 返回交易凭证号。
     *
     * @return 交易凭证号
     */
    @Override
    public String traceNo() {
        return this.getString("traceNo");
    }

    /**
     * 返回终端号。
     *
     * @return 终端号
     */
    @Override
    public String termNo() {
        return null;
    }

    @Override
    public String outTradeNo() {
        return null;
    }

    @Override
    public String mchId() {
        return null;
    }


    /**
     * 返回当前交易是否为消费。
     *
     * @return true 表示消费，false 表示其它。
     */
    @Override
    public boolean isPayment() {
        return "消费".equals(this.transId) ||
                "聚合扫码支付".equals(this.transId);
    }

    /**
     * 返回当前交易是否为消费撤销。
     *
     * @return true 表示消费撤销，false 表示其它。
     */
    @Override
    public boolean isCancellation() {
        return "撤销".equals(this.transId);
    }

    /**
     * 返回当前交易是否为退货。
     *
     * @return true 表示退货，false 表示其它。
     */
    @Override
    public boolean isRefunding() {
        return "退货".equals(this.transId);
    }

    /**
     * 返回当前交易是否为查询
     *
     * @return true 表示查询，false 表示其它。
     */
    @Override
    public boolean isQuery() {
        return false;
    }

    private String getString(String name) {
        return this.getString(name, null);
    }

    private String getString(String name, String defaultValue) {
        try {
            if (this.transData.has(name)) {
                return this.transData.getString(name);
            } else {
                return defaultValue;
            }
        } catch (JSONException ex) {
            return null;
        }
    }

    private double parseAmount(String name) {
        String amt = this.getString(name, "0");
        BigDecimal p = new BigDecimal(amt);
        BigDecimal q = new BigDecimal(100);
        return p.divide(q, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    @Override
    public String payType() {
        return this.getString("payChannel");
    }
}
