/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.ccb;

import android.app.Activity;
import android.content.Intent;


import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PayResult;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;

import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 建行收单应用支付提供程序的基类。
 *
 * @author zhangsai    2018-11-26 HIPOSAPP-888
 * @author wangyaogang 2019-08-19 HIPOSAPP-1225 银联商务增加支付查询,修改了父类,其他行一同修改
 * @author xieliyan    2019-09-10 HIPOSAPP-1293 增加建行支付结果得处理
 */
abstract class CcbPaymentProvider extends CcbProviderBase implements IPaymentProvider {

    private final static String REFUND_DATE_FORMAT = "MMdd";
    final static SimpleDateFormat REFUND_DATE_FORMATTER = new SimpleDateFormat(REFUND_DATE_FORMAT, Locale.CHINESE);
    private String transId; // 当前交易类型,

    /**
     * 初始化一个 {@code CcbPaymentProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    CcbPaymentProvider(Activity activity) {
        super(activity);
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 支付结果
     * @throws PaymentProviderException 支付异常
     */
    @Override
    public PayResult handleResult(Intent data) throws PaymentProviderException {
        if (null == data) {
            if (TransId.CANCELLATION.equals(this.transId)) {
                throw new PaymentProviderException("未返回任何数据");
            } else {
                throw new PaymentProviderException(PayResult.CODE_NO_DATA, "未返回任何数据");
            }
        }

        Map<String, String> map = filterTransResult(data);

        if (!"0".equals(map.get("resultCode"))) {
            throw new PaymentProviderException("resultMsg:" + map.get("resultMsg"));
        }

        if (null == map.get("transData")) {
            throw new PaymentProviderException("返回transData为空");
        }

        try {
            String sJson = map.get("transData");
            JSONObject transData = new JSONObject(sJson);

            if (transData.get("resCode") == null) {
                throw new PaymentProviderException("返回resCode为空");
            }

            if (!transData.getString("resCode").equals("00")) {
                String resCode = transData.getString("resCode");
                String resDesc = transData.getString("resMsg");
                throw new PaymentProviderException(String.format(Locale.CHINESE, "(%s) %s", resCode, resDesc));
            }
            return new CcbPayResult(map.get("appName"), map.get("transId"), transData);
        } catch (JSONException ex) {
            throw new PaymentProviderException("JSONException" + ex);
        }

    }

    @Override
    public boolean supportPayQuery() {
        return false;
    }

    @Override
    public boolean supportCancelQuery() {
        return false;
    }

    @Override
    public boolean supportRefundQuery() {
        return false;
    }

    /**
     * 支付。
     *
     * @param transId    交易ID
     * @param payRequest 支付金额
     */
    protected void pay(String transId, PayRequest payRequest) {
        HashMap<String, String> map = new HashMap<>();
        map.put("amt", makeFieldAmount(String.valueOf(payRequest.getAmount())));

        this.callPosTrans(transId, map);
    }
    /**
     * 调用 POS 交易接口。
     *
     * @param transId 交易ID
     * @param map     扩展参数
     */
    private void callPosTrans(String transId, HashMap<String, String> map) {
        this.transId = transId;
        this.callTrans(transId,map);

    }
    @Override
    public void payQuery(String uniqueNo, PaymentHandler handler) {
        //暂不对接查询方法
    }

    /**
     * 银行标准化金额
     *
     * @param amount 金额
     * @return amount.substring() 标准化后的金额
     */
    static String makeFieldAmount(String amount) {
        String[] part = amount.split("\\.");
        switch (part.length) {
            case 1:
                return ("0000000000" + amount + "00").substring(amount.length());
            case 2:
                String digit = part[1].length() >= 2 ? part[1].substring(0, 2)
                        : (part[1] + "00").substring(0, 2);
                amount = "000000000000" + part[0] + digit;
                return amount.substring(amount.length() - 12);
        }
        System.out.println(part.length);
        return null;
    }
}
