/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.ccb;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Intent;
import android.os.Bundle;

import com.hisense.pos.payment.PaymentProvider;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

/**
 * 建行收单应用提供程序的基类。
 *
 * @author zhangsai 2018-11-26 HIPOSAPP-888
 * @author xieliyan 2019-09-10 HIPOSAPP-1302 增加建行结果返回处理方法
 */
abstract class CcbProviderBase extends PaymentProvider {

    static final int TRANS_REQUEST_CODE = 2001; // TODO: 此值为易收银自定义
    private static final String APP_NAME = "建行收单应用";

    /**
     * 初始化一个 {@code CcbProviderBase} 的实例。
     *
     * @param activity Activity 活动
     */
    CcbProviderBase(Activity activity) {
        super(activity);
    }


    /**
     * 返回 Android 请求编码。
     *
     * @return Android 请求编码
     */
    public int requestCode() {
        return TRANS_REQUEST_CODE;
    }

    /**
     * 调用建行收单。
     *
     * @param transId 交易ID
     */
    protected void callTrans(String transId) {
        this.callTrans(transId, (HashMap) null);
    }

    /**
     * 调用建行收单。
     *
     * @param transId 交易ID
     * @param map     交易数据
     */
    protected void callTrans(String transId, HashMap map) {
        JSONObject jsonObject = null;
        if(map == null) {
            map = new HashMap();
        }
        map.put("projectTag", "QqhrBDHipos");
        map.put("authIndex", "88888888");
        map.put("authCode", "hk6Vq64T6TEeIEWaPEBU1CF6OjpJMVJmni68OcfB6NXTpV6+eOZuT8gQtfhkG8Ge+IGcdjG8dp9APlXkj7SR4BVGwuhQejEMYBYOXHxZ9qApnR3RJvmO/CrFpSleLbFtoALuLEHSc7p3MX+RRZKv+OdOyRo9Lt3LSl+p2U8O7C0=");
        jsonObject = new JSONObject(map);
        this.callTrans(transId, jsonObject);
    }

    /**
     * 调用建行收单。
     *
     * @param transId    交易Id
     * @param jsonObject 交易数据
     */
    protected void callTrans(String transId, JSONObject jsonObject) {
        ComponentName compName = new ComponentName("com.ccb.smartpos.bankpay",
                "com.ccb.smartpos.bankpay.ui.MainActivity");

        Intent intent = new Intent();
        intent.setComponent(compName);
        intent.putExtra("appName", APP_NAME);
        intent.putExtra("transId", transId);
        intent.putExtra("transData", jsonObject.toString());


        super.activity.startActivityForResult(intent, TRANS_REQUEST_CODE);
    }

    /**
     * 返回交易结果
     *
     * @param intent 事件
     * @return map   交易结果
     */
    protected static Map<String, String> filterTransResult(Intent intent) {
        HashMap<String, String> map = new HashMap<>();
        Bundle bundle = intent.getExtras();
        if (bundle != null) {
            if (bundle.containsKey("appName")) {
                map.put("appName", bundle.getString("appName"));
            }
            if (bundle.containsKey("transId")) {
                map.put("transId", bundle.getString("transId"));
            }
            if (bundle.containsKey("resultCode")) {
                map.put("resultCode", bundle.getString("resultCode"));
            }
            if (bundle.containsKey("resultMsg")) {
                map.put("resultMsg", bundle.getString("resultMsg"));
            }
            if (bundle.containsKey("transData")) {
                map.put("transData", bundle.getString("transData"));
            }
        }
        return map;

    }

}
