package com.hisense.pos.print.ccb;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Binder;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.widget.Toast;

import com.hisense.common.util.LoggerUtils;
import com.ccb.deviceservice.aidl.IDeviceService;
import com.ccb.deviceservice.aidl.beeper.IBeeper;
import com.ccb.deviceservice.aidl.deviceinfo.IDeviceInfo;
import com.ccb.deviceservice.aidl.insertreader.IInsertCardReader;
import com.ccb.deviceservice.aidl.led.ILed;
import com.ccb.deviceservice.aidl.magreader.IMagCardReader;
import com.ccb.deviceservice.aidl.pboc.IPBOC;
import com.ccb.deviceservice.aidl.pinpad.IPinpad;
import com.ccb.deviceservice.aidl.printer.IPrinter;
import com.ccb.deviceservice.aidl.rfreader.IRFCardReader;
import com.ccb.deviceservice.aidl.scanner.IScanner;
import com.ccb.deviceservice.aidl.serialport.ISerialPort;

/**
 * 联迪驱动调用
 *
 * @author land
 * @author wangyaogang 2019-05-14 HIPOSAPP-1095 易收银各类Toast提示信息改为弹窗提示,方便记录
 * @author wangyaogang 2019-09-05 HIPOSAPP-1287 建行服务断开后重新绑定
 */
public class CCBDeviceHelper {
    private static final String TAG = "TradeLog";

    private static final String ACTION_DEVICE_SERVICE = "com.ccb.device_service";
    private static final String PACKAGE_DEVICE_SERVICE = "com.ccb.deviceservice";

    private static CCBDeviceHelper ccbDeviceHelper = new CCBDeviceHelper();
    private Context mContext;
    private boolean isBindService;
    private IDeviceService mDeviceService;
    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            isBindService = true;
            mDeviceService = IDeviceService.Stub.asInterface(service);
            LoggerUtils.getLogger().debug("-------------onServiceConnected----------" + isBindService);
            linkToDeath(service);

        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            isBindService = false;
            LoggerUtils.getLogger().debug("-------------onServiceDisconnected----------");
        }

        private void linkToDeath(IBinder service) {
            try {
                service.linkToDeath(new IBinder.DeathRecipient() {
                    @Override
                    public void binderDied() {
                        isBindService = false;
                        LoggerUtils.getLogger().debug("-------------binder service is dead!!!----------");
                        bindService();
                    }
                }, 0);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    };

    private CCBDeviceHelper() {
    }

    public static CCBDeviceHelper getInstance() {
        return ccbDeviceHelper;
    }

    public IDeviceService getDeviceService() {
        return mDeviceService;
    }

    public void init(Context context) {
        mContext = context;
    }

    public void bindService() {
        if (!isBindService) {
            Intent service = new Intent(ACTION_DEVICE_SERVICE);
            service.setPackage(PACKAGE_DEVICE_SERVICE);
            boolean bindSucc = mContext.bindService(service, mConnection, Context.BIND_AUTO_CREATE);
            if (bindSucc) {
                Log.d(TAG, "-------------bind service success----------");
            } else {
                Log.e(TAG, "-------------bind service failed----------");
            }
        }
    }

    public void unbindService() {
        if (isBindService) {
            Log.d(TAG, "-------------unbind service success----------");
            mContext.unbindService(mConnection);
            isBindService = false;
        }
    }

    public IBeeper getBeeper() {
        try {
            IBinder binder = mDeviceService.getBeeper();
            return IBeeper.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getBeeper SecurityException---------------");
            return null;
        }
    }


    public IMagCardReader getMagCardReader() {
        try {
            IBinder binder = mDeviceService.getMagCardReader();
            return IMagCardReader.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getMagCardReader SecurityException---------------");
            return null;
        }
    }


    public IDeviceInfo getDeviceInfo() {
        try {
            IBinder binder = mDeviceService.getDeviceInfo();
            return IDeviceInfo.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getDeviceInfo SecurityException---------------");
            return null;
        }
    }


    public ILed getLED() {
        try {
            IBinder binder = mDeviceService.getLed();
            return ILed.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getLed SecurityException---------------");
            return null;
        }
    }


    public IInsertCardReader getInsertCardReader() {
        try {
            IBinder binder = mDeviceService.getInsertCardReader();
            return IInsertCardReader.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getInsertCardReader SecurityException---------------");
            return null;
        }
    }


    public IRFCardReader getRFCardReader() {
        try {
            IBinder binder = mDeviceService.getRFCardReader();
            return IRFCardReader.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getRFCardReader SecurityException---------------");
            return null;
        }
    }


    public IScanner getFrontScanner() {
        try {
            IBinder binder = mDeviceService.getScanner(1);
            return IScanner.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getScanner SecurityException---------------");
            return null;
        }
    }


    public IScanner getRearScanner() {
        try {
            IBinder binder = mDeviceService.getScanner(0);
            return IScanner.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getScanner SecurityException---------------");
            return null;
        }
    }


    public ISerialPort getSerialPort() {
        try {
            IBinder binder = mDeviceService.getSerialPort();
            return ISerialPort.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getSerialPort SecurityException---------------");
            return null;
        }
    }


    public IPrinter getPrinter() {
        try {
            lockService();
            IBinder binder = mDeviceService.getPrinter();
            return IPrinter.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();

            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Log.d(TAG, "-------------------getPrinter SecurityException---------------");
            LoggerUtils.getLogger().debug("打印异常"+e.toString());
            return null;
        }
    }


    public IPinpad getPinpad(int kapId) {
        try {
            IBinder binder = mDeviceService.getPinpad(kapId);
            return IPinpad.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getPinpad SecurityException---------------");
            return null;
        }
    }


    public IPBOC getPBOC() {
        try {
            IBinder binder = mDeviceService.getPBOC();
            return IPBOC.Stub.asInterface(binder);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getPBOC SecurityException---------------");
            return null;
        }
    }

    /**
     * 打印刷卡调用时需要锁定服务
     */
    public void lockService() {
        if (isBindService) {
            try {
                mDeviceService.lock(new Bundle(), new Binder());
                LoggerUtils.getLogger().debug("lock成功");
            } catch (Exception e) {
                e.printStackTrace();
                LoggerUtils.getLogger().debug(e.getMessage());
            }
        } else {
            LoggerUtils.getLogger().debug("CCBService 未绑定成功，请重新登录 isBindService" + isBindService);
        }
    }

    /**
     * 打印刷卡调用结束时需要解锁服务
     */
    public void unlockService() {
        if (isBindService) {
            try {
                mDeviceService.unlock();
                LoggerUtils.getLogger().debug("unLock成功");
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            LoggerUtils.getLogger().debug("CCBService 未绑定成功，请重新登录 isBindService" + isBindService);

        }
    }
}
