package com.hisense.pos.print.ccb;

import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.ConditionVariable;
import android.os.RemoteException;

import com.ccb.deviceservice.aidl.printer.IPrinter;
import com.ccb.deviceservice.aidl.printer.OnPrintListener;
import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.print.IPrintProvider;

/**
 * 建行打印提供程序
 *
 * @author xieliyan  2019-11-13
 */
public class CcbPrintProvider implements IPrintProvider {


    private IPrinter iPrinter;

    private static ConditionVariable conditionVariable = new ConditionVariable();

    private static OnPrintListener printerListener = new OnPrintListener.Stub() {
        @Override
        public void onFinish() throws RemoteException {
            conditionVariable.open();
        }

        @Override
        public void onError(int code) throws RemoteException {
            conditionVariable.open();
        }
    };


    @Override
    public boolean initPrinter(int prnBaudRate, int prnPort, int prnType, int feedLine) {
        iPrinter = CCBDeviceHelper.getInstance().getPrinter();
        return iPrinter != null;
    }

    @Override
    public void cutPaper() {

    }

    @Override
    public void closePrinter() {

    }

    @Override
    public void addText(String text, int align, int font) {
        try {
            Bundle format = new Bundle();
            format.putInt("font", font);
            format.putInt("align", align);
            format.putBoolean("underline", false);   //是否加下划线
            format.putBoolean("autoTrunc", false);   //是否自动截行
            iPrinter.addText(format, text);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加文本异常" + e.getMessage());
        }

    }

    @Override
    public void addBarCode(String barCode) {
        try {
            Bundle format = new Bundle();
            format.putInt("align", 1);
            iPrinter.addBarCode(format, barCode);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加条形码异常" + e.getMessage());
        }
    }

    @Override
    public void addQrCode(String qrCode, int offset, int expectedHeight) {
        try {
            Bundle format = new Bundle();
            format.putInt("align", 1);
            format.putInt("offset", offset);
            format.putInt("expectedHeight", expectedHeight);
            iPrinter.addQrCode(format, qrCode);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加二维码异常" + e.getMessage());
        }
    }

    @Override
    public void addImage(byte[] imageData, Bitmap bitmap, int offset, int width, int height) {
        try {
            Bundle format = new Bundle();
            format.putInt("align", 1);
            format.putInt("offset", offset);
            iPrinter.addImage(format, imageData);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加二维码异常" + e.getMessage());
        }


    }


    @Override
    public int getStatus() {
        try {
            return iPrinter.getStatus();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("获取打印状态异常" + e.getMessage());
            return -1;
        }
    }

    @Override
    public void feedLine(int lines) {
        try {
            iPrinter.feedLine(lines);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("换行异常" + e.getMessage());
        }
    }

    @Override
    public void startPrint() {
        try {
            iPrinter.startPrint(printerListener);
            conditionVariable.block(30000);//此方法不调用会引起打印数据不清除
            conditionVariable.close();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("启动打印异常" + e.getMessage());
        } finally {
            CCBDeviceHelper.getInstance().unlockService();
        }
    }

}
