/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.hi98;

import android.content.Context;

import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.magic.MagCard;
import com.hisense.pos.magic.TrackResult;

/**
 * {功能描述}
 *
 * @author zhangsai 2018-01-18
 */
class MagCardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;

    private MagCard card;

    // 监听程序
    private MagCardListener cardListener;

    // 读取卡号的回调方法
    private CardNoCallback callback;


    @Override
    public void open(Context context) {
        card = new MagCard();
        int net = card.Mag_Open();
        if (net != com.hisense.pos.magic.MagCard.MAG_OK) {
            card = null;
            return;
        }
    }

    @Override
    public void read(String password, CardNoCallback callback) {
        this.callback = callback;

        this.start();
    }

    @Override
    public void start() {
        if (card == null) {
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;

        cardListener = new MagCardListener(card, callback);
        cardListener.start();
    }

    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }

        IS_RUNNING = false;
    }

    @Override
    public void close() {
        this.stop();

        if (card != null) {
            card.Mag_Close();
        }
    }

    /**
     * 磁条卡读卡处理
     *
     * @author xieliyan 2017-09-25 HIPOSAPP-160
     */
    private static class MagCardListener extends CardListener {

        private CardNoCallback callback;
        private MagCard card;

        public MagCardListener(MagCard card, CardNoCallback callback) {
            this.card = card;
            this.callback = callback;
        }

        @Override
        protected void onListening() {
            TrackResult result = new TrackResult();
            byte[] readMsr2;

            if (card.Mag_Swiped() == 0) {
                int net = card.Mag_Read(result);
                if (net == 0) {
                    String cardNo = "";
                    if (result.isTrackValid(2)) {
                        readMsr2 = result.getTrackData(2);
                        cardNo = new String(readMsr2);
                    }

                    this.callback.onSuccess(cardNo);
                }
            }
        }
    }
}
