/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.hk316;

import android.content.Context;

import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1CardReaderProvider;

/**
 * 316读卡器
 *
 * @author wangyaogang 2020-01-10 HIAPOSCMP-482 支持磁条卡/非接卡刷卡功能
 */
public class Hk316CardReader implements ICardReader {

    // 磁条卡读卡器
    private MagCardReader magCardReader;

    // Mifare1 非接触式磁卡读卡器
    private M1CardReader m1CardReader;


    /**
     * 初始化一个 {@code CcbCardReader} 的实例。
     *
     * @param provider Mifare1 非接触式磁卡读取器提供程序
     * @param version  Mifare1 版本
     */
    public Hk316CardReader(M1CardReaderProvider provider, int version) {
        magCardReader = new MagCardReader();
        switch (version) {
            case 0:
                m1CardReader = new M1CardReaderV1();
                break;
            case 2:
            default:
                m1CardReader = new M1CardReaderV2(provider);
                break;
        }
    }

    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {
        if (magCardReader != null) {
            magCardReader.open(context);
        }

        if (m1CardReader != null) {
            m1CardReader.open(context);
        }
    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {

        if (magCardReader != null) {
            magCardReader.read(password, callback);
        }

        if (m1CardReader != null) {
            m1CardReader.read(password, callback);
        }

        this.start();
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
        if (magCardReader != null) {
            magCardReader.start();
        }

        if (m1CardReader != null) {
            m1CardReader.start();
        }
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (magCardReader != null) {
            magCardReader.stop();
        }

        if (m1CardReader != null) {
            m1CardReader.stop();
        }
    }

    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {
        if (magCardReader != null) {
            magCardReader.close();
        }

        if (m1CardReader != null) {
            m1CardReader.close();
        }

    }
}
