/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.hk316;

import android.content.Context;
import android.os.Bundle;
import android.os.ConditionVariable;
import android.os.RemoteException;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.print.hk316.HI316DeviceHelper;
import com.histonepos.npsdk.bind.Const;
import com.histonepos.npsdk.msr.IMsrEventListener;
import com.histonepos.npsdk.msr.IMsrService;

/**
 * hk316磁条卡读卡器
 *
 * @author wangyaogang 2020-01-10 HIAPOSCMP-482 支持磁条卡/非接卡刷卡功能
 */
class MagCardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;

    // 监听程序
    private MagCardListener cardListener;

    // 读取卡号的回调方法
    private static CardNoCallback callback;

    private IMsrService magCardReader;

    public MagCardReader() {

    }

    @Override
    public void open(final Context context) {
        try {
            magCardReader = HI316DeviceHelper.getInstance().getMsr();
            int iret = magCardReader.openMsr();
            if (iret != 0) {
                LoggerUtils.getLogger().debug("316_MagCardReader_open失败:" + iret);
            }
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("316_MagCardReader_open异常:" + e.toString());
        }
    }

    @Override
    public void read(String password, CardNoCallback callback) {

        MagCardReader.callback = callback;
        this.start();
    }

    @Override
    public void start() {
        LoggerUtils.getLogger().debug("316_MagCardReader_start");
        if (magCardReader == null) {
            LoggerUtils.getLogger().debug("316_MagCardReader_start异常:magCardReader为null");
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            LoggerUtils.getLogger().debug("316_MagCardReader_IS_RUNNING返回");
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;

        cardListener = new MagCardListener(magCardReader, callback);
        cardListener.start();
    }

    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();
        }
        IS_RUNNING = false;
    }

    @Override
    public void close() {
        this.stop();
        LoggerUtils.getLogger().debug("316_MagCardReader_close");
    }

    /**
     * 磁条卡读卡处理
     *
     * @author xieliyan 2017-09-25 HIPOSAPP-160
     */
    private class MagCardListener extends CardListener {

        private CardNoCallback callback;
        private IMsrService card;
        private ConditionVariable conditionVariable = new ConditionVariable();

        public MagCardListener(IMsrService card, CardNoCallback callback) {
            this.card = card;
            this.callback = callback;
        }

        @Override
        protected void onListening() {
            try {
                card.setTimeout(60 * 60 * 24);
                card.setMsrListener(new IMsrEventListener.Stub() {
                    @Override
                    public void onTrack(int respcode, Bundle bundle) throws RemoteException {
                        if (respcode != 0) {
                            LoggerUtils.getLogger().debug("316_MagCardReader_onTrack失败:" + respcode);
                        } else {
                            byte[] data = bundle.getByteArray(Const.TRACK2);
                            if (data != null) {
                                String cardNo = new String(data);
                                LoggerUtils.getLogger().debug("316_MagCardReader_onSuccess:" + cardNo);
                                if (!"".equals(cardNo) && MagCardReader.callback != null) {
                                    MagCardReader.callback.onSuccess(cardNo);
                                } else {
                                    LoggerUtils.getLogger().debug("316_MagCardReader_onSuccess:callback为空");
                                }
                            } else {
                                LoggerUtils.getLogger().debug("316_MagCardReader_onSuccess:data为null");
                            }
                        }
                        conditionVariable.open();
                    }

                    @Override
                    public void onTimeout() throws RemoteException {
                        LoggerUtils.getLogger().debug("316_MagCardReader_onTimeout");
                        card.closeMsr();
                        conditionVariable.open();
                    }
                });

            } catch (RemoteException e) {
                e.printStackTrace();
                LoggerUtils.getLogger().debug("ccb_MagCardReader_searchCard异常:" + e.toString());
                conditionVariable.open();
            }
            conditionVariable.block();
        }

        @Override
        protected void onStop() {
            conditionVariable.close();
        }
    }
}
