/***************************************************************************************************
 * Copyright (c) 2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.print.hk316;

import android.content.Context;
import android.os.Bundle;

import com.hisense.common.util.LoggerUtils;
import com.histonepos.npsdk.api.IServiceManager;
import com.histonepos.npsdk.bind.Const;
import com.histonepos.npsdk.bind.IServiceConnector;
import com.histonepos.npsdk.bind.PrinterConsts;
import com.histonepos.npsdk.bind.SmartService;
import com.histonepos.npsdk.msr.IMsrService;
import com.histonepos.npsdk.picccard.IPiccCardService;
import com.histonepos.npsdk.printer.IPrinterService;


/**
 * hk316设备驱动调用
 *
 * @author wangyaogang 2019-12-30 HIAPOSCMP-465 支持hk316设备打印功能
 * @author wangyaogang 2020-01-10 HIAPOSCMP-482 支持磁条卡/非接卡刷卡功能
 */
public class HI316DeviceHelper {
    private static HI316DeviceHelper deviceHelper;
    private Context mContext;
    private static IServiceManager sm;

    private HI316DeviceHelper() {
    }

    public static HI316DeviceHelper getInstance() {
        if (deviceHelper == null) {
            synchronized (HI316DeviceHelper.class) {
                if (deviceHelper == null) {
                    deviceHelper = new HI316DeviceHelper();
                }
            }
        }
        return deviceHelper;
    }

    public HI316DeviceHelper init(Context context) {
        mContext = context;
        return getInstance();
    }

    /**
     * 绑定服务
     */
    public void bindDeviceService() {
        if (sm == null && mContext != null) {
//            LoggerUtils.getLogger().debug("hk316开始绑定服务");
            SmartService.getInstance().register(mContext, new IServiceConnector() {

                @Override
                public void onServiceDisconnected() {
//                    LoggerUtils.getLogger().debug("hk316已解绑");
                    sm = null;
                    bindDeviceService();
                }

                @Override
                public void onServiceConnected(IServiceManager serviceManager) {
//                    LoggerUtils.getLogger().debug("hk316已绑定");
                    /**
                     * Step 2: get apiHander
                     */
                    sm = serviceManager;
                }
            });
        } else {
//            LoggerUtils.getLogger().debug("hk316服务已经启动");
        }
    }

    public static void setSm(IServiceManager sm) {
        HI316DeviceHelper.sm = sm;
    }

    /**
     * 解绑服务
     */
    public void unBindDeviceService() {
        if (sm != null) {
//            LoggerUtils.getLogger().debug("解绑hk316服务");
            SmartService.getInstance().unregister();
        } else {
//            LoggerUtils.getLogger().debug("hk316服务未启动");
        }
    }

    /**
     * 获取打印机
     */
    public IPrinterService getPrinter() {
        IPrinterService printerModule = null;
        try {
            printerModule = IPrinterService.Stub.asInterface(sm.getService(Const.PRINTER));

            // tell sdk the widthmode(58mm or 80mm) of printer in use.
            Bundle bundle = new Bundle();
            String widthType = PrinterConsts.PRT_CFG_VALUE_WIDTH_MODE_80;
            bundle.putString(PrinterConsts.PRT_CFG_KEY_WIDTH_MODE, widthType);
            printerModule.setConfig(bundle);

        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("获取hk316打印机错误" + e.getMessage());
        }
        return printerModule;
    }

    /**
     * 获取磁条卡读卡器
     */
    public IMsrService getMsr() {
        try {
            return IMsrService.Stub.asInterface(sm.getService(Const.MSR));
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("获取hk316磁条卡读卡器错误" + e.getMessage());
        }
        return null;
    }

    /**
     * 获取非接卡读卡器
     */
    public IPiccCardService getPiccCard() {
        try {
            return IPiccCardService.Stub.asInterface(sm.getService(Const.PICC));
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("获取hk316非接卡读卡器错误" + e.getMessage());
        }
        return null;
    }
}
