package com.hisense.pos.hs6500;

import android.graphics.Bitmap;

import android.os.RemoteException;

import com.hisense.common.util.LoggerUtils;
import com.hisense.hs650service.aidl.Printer;
import com.hisense.pos.print.IPrintProvider;


/**
 * HS大屏系列打印提供程序
 *
 * @author xieliyan  2019-11-13
 */
public class HS650PrintProvider implements IPrintProvider {


    private Printer printer;


    @Override
    public boolean initPrinter(int prnBaudRate, int prnPort, int prnType, int feedLine) {
        boolean result;
        result = HS650DeviceHelper.getInstance().initPrinter(prnBaudRate, prnPort, prnType, feedLine);
        printer = HS650DeviceHelper.getInstance().getPrinter();
        return result;


    }

    @Override
    public void addText(String text, int align, int font) {
        try {
            LoggerUtils.getLogger().debug("打印文本");
            printer.printTextStr(text, 0, align);
            printer.feedPaper(1, 0);
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("添加文本异常" + e.getMessage());
        }
    }

    @Override
    public void addBarCode(String barCode) {

        try {
            LoggerUtils.getLogger().debug("打印条形码");
            printer.printBar(barCode);
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 printBar exception----------" + e.toString());
        }

    }

    @Override
    public void addQrCode(String qrCode, int offset, int expectedHeight) {
        try {
            LoggerUtils.getLogger().debug("打印二维码");
            printer.printQR(qrCode, 6);
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 printQR exception----------" + e.toString());
        }
    }

    @Override
    public void addImage(byte[] imageData, Bitmap bitmap, int offset, int width, int height) {
        try {
            LoggerUtils.getLogger().debug("打印图片");
            printer.printImage(bitmap, 1);
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 printImage exception----------" + e.toString());
        }
    }

    @Override
    public int getStatus() {

        try {
            LoggerUtils.getLogger().debug("获取打印状态");

            return printer.printStatus();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("获取打印状态异常" + e.getMessage());
            return -1;
        }

    }

    @Override
    public void feedLine(int lines) {
        try {
            LoggerUtils.getLogger().debug("走纸 lines");
            printer.feedPaper(lines, 0);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("走纸异常" + e.getMessage());
        }
    }

    @Override
    public void startPrint() {

        LoggerUtils.getLogger().debug("启动打印");
    }

    @Override
    public void cutPaper() {
        try {
            LoggerUtils.getLogger().debug("切纸打印");
            printer.cutPaper();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("切纸异常" + e.getMessage());
        }

    }

    @Override
    public void closePrinter() {
        try {
            printer.closePrinter();
        } catch (RemoteException e) {
            LoggerUtils.getLogger().debug("-------------hs650 closePrinter exception----------" + e.toString());
        }
    }
}
