package com.icbc.smartpos.transservice.aidl;

import com.icbc.smartpos.transservice.aidl.TransHandler;

/**
 * 交易调用接口
 * @author: icbc
 */
interface ITransService {

        /**
         * 启动银行卡交易
         * @param transType - 交易类型
         * <ul>
         * <li>LOGON(4001)            -   签到 </li>
         * <li>MULTI_PURCHASE(8888)   -   收银 </li>
         * <li>PURCHASE(1001)         -   银行卡消费 </li>
         * <li>INQUERY(2002)          -   余额查询 </li>
         * <li>POS_VOID(1101)         -   当日撤销 </li>
         * <li>REFUND(1102)           -   隔日退货 </li>
         * <li>QRPURCHASE(1053)       -   二维码消费 </li>
         * <li>QRINQUERY(2024)        -   二维码查询 </li>
         * <li>QRREFUND(1054)         -   二维码退货 </li>
         * <li>INTEGRAL_PURCHASE(3004)-   积分消费 </li>
         * <li>INTEGRAL_QUERY(2004)   -   积分查询 </li>
         * <li>INTEGRAL_REFUND(1110)  -   积分退货 </li>
         * <li>INSTALLMENT(1003)      -   分期付款 </li>
         * <li>PREAUTH(1011)          -   预授权 </li>
         * <li>ADDPREAUTH(1014)       -   追加预授权 </li>
         * <li>PREAUTHEND(1012)       -   预授权确认 </li>
         * <li>PREAUTHVOID(1103)      -   预授权隔日取消 </li>
         * <li>PREAUTHQUERY(2009)     -   查原预授权卡号 </li>
         * <li>QUERY_TRANS_REC(2003)  -   查询交易记录 </li>
         * </ul>
         *
         * @param transData - 交易数据
         * <ul>
         * <li>APP_NAME(String)：      调用方应用名称 </li>
         * <li>TRANS_SEQUENCE(long)：  调用方的交易顺序号 </li>
         * <li>AMOUNT(long)：          交易金额(分) </li>
         * <li>NEED_PRINT(boolean)：   是否需要打印凭条，默认为true，如不打印需要有特别权限控制 </li>
         * <li>NEED_CONFIRM(boolean)：   在反交易中需要对原交易信息进行确认（当外部应用直接传入了所有信息），默认为true </li>
         * <li>OLD_TRANS_SEQUENCE(long)：原交易顺序号 </li>
         * <li>OLD_AUTH_ID(String)：   原授权码 </li>
         * <li>OLD_REF_NO(String)：    原交易检索参考号 </li>
         * <li>OLD_TERM_NO(String)：   原交易终端号 </li>
         * <li>OLD_TRANS_DATE(long)：  原交易日期(yyyyMMdd) </li>
         * </ul>
         *
         * @param handler - 交易回调处理器
         */
        long startTrans(String transType,
                        in Bundle ctrlData,
                        in Bundle transData,
                        in TransHandler handler);
}