/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.icbc;

import android.content.Context;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1CardReaderProvider;
import com.icbc.smartpos.deviceservice.aidl.IDeviceService;

/**
 * Hi98 读卡器，同时支持磁条卡和 Mifare1 非接触式磁卡。
 *
 * @author lizhixin HIPOSAPP-596 2018-06-22 新增工行收单功能-读卡
 */
public class ICBCCardReader implements ICardReader {

    private static final String TAG = "ICBCCardReader";

    // 磁条卡读卡器
    private MagCardReader magCardReader;

    // Mifare1 非接触式磁卡读卡器
    private M1CardReader m1CardReader;


    private IDeviceService deviceService;

    /**
     * 初始化一个 {@code CcbCardReader} 的实例。
     *
     * @param provider   Mifare1 非接触式磁卡读取器提供程序
     * @param version    Mifare1 版本
     * @param deviceType 设备类型 LANDI 联迪 AISINO 艾体威尔
     */
    public ICBCCardReader(IDeviceService deviceService, M1CardReaderProvider provider, int version, String deviceType) {
        this.deviceService = deviceService;
        magCardReader = new MagCardReader(deviceService, deviceType);
        switch (version) {
            case 0:
                m1CardReader = new M1CardReaderV1(deviceService, deviceType);
                break;
            case 2:
                m1CardReader = new M1CardReaderV2(deviceService, provider, deviceType);
                break;
        }
    }

    /**
     * 打开并初始化读卡器。
     *
     * @param context Context
     */
    @Override
    public void open(Context context) {
        if (magCardReader != null) {
            magCardReader.open(context);
        }

        if (m1CardReader != null) {
            m1CardReader.open(context);
        }
    }

    /**
     * 读取卡号。
     *
     * @param password 读卡密码
     * @param callback 读取卡号的回调方法
     */
    @Override
    public void read(String password, CardNoCallback callback) {
        if (this.deviceService == null) {
            LoggerUtils.getLogger().debug("-------------------deviceService is NULL-------------------------");
            return;
        }

        /*try {
            this.deviceService.lock(new Bundle(), new Binder());*/

        if (magCardReader != null) {
            magCardReader.read(password, callback);
        }

        if (m1CardReader != null) {
            m1CardReader.read(password, callback);
        }

        start();
        /*} catch (RemoteException e) {

        }*/
    }

    /**
     * 开始读卡。
     */
    @Override
    public void start() {
        if (magCardReader != null) {
            magCardReader.start();
        }

        if (m1CardReader != null) {
            m1CardReader.start();
        }
    }

    /**
     * 停止读卡。
     */
    @Override
    public void stop() {
        if (magCardReader != null) {
            magCardReader.stop();
        }

        if (m1CardReader != null) {
            m1CardReader.stop();
        }
    }

    /**
     * 关闭读卡器。
     */
    @Override
    public void close() {
        if (magCardReader != null) {
            magCardReader.close();
        }

        if (m1CardReader != null) {
            m1CardReader.close();
        }

        /*if (this.deviceService != null) {
            try {
                this.deviceService.unlock();
            } catch (RemoteException e) {
                Log.e("", e.toString());
            }
        }*/
    }
}
