/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.icbc;

import android.content.Context;
import android.os.Bundle;
import android.os.RemoteException;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardListener;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.icbc.smartpos.deviceservice.aidl.IDeviceService;
import com.icbc.smartpos.deviceservice.aidl.IMagCardReader;

/**
 * {功能描述}
 *
 * @author lizhixin 2018-06-22 HIPOSAPP-596 新增工行收单功能-读卡
 * @author lizhixin 2018-08-07 HIPOSAPP-712 支持艾体威尔设备类型
 * @author guolina  2018-10-30 HIPOSAPP-805 解决因未停止查找卡导致的工行第一次储值卡刷卡没反应的问题
 */
class MagCardReader implements ICardReader {

    private static final String TAG = "MagCardReader";

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;
    //设备类型 LANDI 联迪 AISINO 艾体威尔
    private String deviceType;

    // 监听程序
    private MagCardListener cardListener;

    // 读取卡号的回调方法
    private CardNoCallback callback;

    private IMagCardReader magCardReader;
    private IDeviceService deviceService;

    public MagCardReader(IDeviceService deviceService, String deviceType) {
        this.deviceService = deviceService;
        this.deviceType = deviceType;
    }

    @Override
    public void open(final Context context) {

    }

    @Override
    public void read(String password, CardNoCallback callback) {
        try {
            magCardReader = deviceService.getMagCardReader();

            /*IBinder binder = this.deviceService.getMagCardReader();
            magCardReader = IMagCardReader.Stub.asInterface(binder);*/

            this.callback = callback;

            start();
        } catch (RemoteException e) {

        }
    }

    @Override
    public void start() {
        if (magCardReader == null) {
            LoggerUtils.getLogger().debug("-------------------magCardReader is NULL-------------------------");
            return;
        }
        if (IS_RUNNING) { // 如果已启动，则立即退出。避免启动多个监听线程。
            return;
        }

        // 启动立即修改允许状态。避免启动多个监听线程。
        IS_RUNNING = true;

        cardListener = new MagCardListener(magCardReader, callback);
        cardListener.start();
    }

    @Override
    public void stop() {
        if (cardListener != null) {
            cardListener.stopListen();

            if ("AISINO".equals(deviceType)) {
                //艾体威尔设备需要停止监听
                try {
                    magCardReader.stopSearch();
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }

        }

        IS_RUNNING = false;
    }

    @Override
    public void close() {
        this.stop();
    }

    /**
     * 磁条卡读卡处理
     *
     * @author xieliyan 2017-09-25 HIPOSAPP-160
     */
    private static class MagCardListener extends CardListener {

        private CardNoCallback callback;
        private IMagCardReader card;

        public MagCardListener(IMagCardReader card, CardNoCallback callback) {
            this.card = card;
            this.callback = callback;
        }

        @Override
        protected void onListening() {
            try {

                card.searchCard(10, new com.icbc.smartpos.deviceservice.aidl.MagCardListener.Stub() {
                    @Override
                    public void onSuccess(Bundle track) throws RemoteException {
                        String cardNo = track.getString("TRACK2");
                        LoggerUtils.getLogger().debug("icbc磁条卡号:" + cardNo);
                        if (!"".equals(cardNo) && null != callback) {
                            callback.onSuccess(cardNo);
                        }
                    }

                    @Override
                    public void onError(int error, String message) throws RemoteException {
                    }

                    @Override
                    public void onTimeout() throws RemoteException {
                    }
                });

            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }

        @Override
        protected void onStop() {
            try {
                card.stopSearch();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    }
}
