/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import android.os.Bundle;

/**
 * 工行收单接口返回的基本结果数据。
 *
 * @author zhangsai 2018-11-27 HIPOSAPP-888
 */
public class BaseResult extends ResultBase {
    private final static long RESULT_SUCCESS = 0;
    private final static long RESULT_FAILURE = 1;


    /**
     * 初始化一个 {@code IcbcInstallmentProvider} 的实例。
     *
     * @param bundle 接口返回的基本结果数据
     */
    BaseResult(Bundle bundle) {
        super();
        this.setBundle(bundle);
    }


    /**
     * 返回调用是否成功。
     *
     * @return 成功返回 true，失败返回 false
     */
    public boolean isSuccess() {
        return this.result() == RESULT_SUCCESS;
    }

    /**
     * 返回结果类型。
     */
    public long result() {
        return super.bundle.getLong("RESULT", RESULT_FAILURE);
    }

    /**
     * 返回交易描述，交易失败时将会输出对应的失败信息描述。
     */
    public String description() {
        return super.bundle.getString("DESCRIPTION");
    }

    /**
     * 返回交易时间。格式：yyyyMMddHHmmss
     */
    public String transTime() {
        return super.bundle.getString("TRANS_TIME");
    }

    /**
     * 返回终端对应的商户名称。
     */
    public String custName() {
        return super.bundle.getString("CUST_NAME");
    }

    /**
     * 返回终端对应的12位商户号。
     */
    public String custNo() {
        return super.bundle.getString("CUST_NO");
    }

    /**
     * 返回终端对应的15位终端号。
     */
    public String termNo() {
        return super.bundle.getString("TERM_NO");
    }

    /**
     * 返回交易顺序号。
     */
    public long transSequence() {
        return super.bundle.getLong("TRANS_SEQUENCE", 1);
    }
}
