/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.RemoteException;

import com.alibaba.fastjson.JSON;
import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.hisense.pos.payment.icbc.bean.StOrgItem;
import com.hisense.pos.payment.icbc.bean.StOrgItemTransItem;
import com.hisense.pos.payment.icbc.bean.StQRCodeItem;
import com.hisense.pos.payment.icbc.bean.StResultOuter;
import com.icbc.smartpos.transservice.aidl.TransHandler;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 工行收单支付服务提供程序。
 *
 * @author zhangsai 2018-11-27 HIPOSAPP-888
 */
public class IcbcPaymentServiceProvider extends IcbcProviderBase implements IPaymentServiceProvider {

    private final static String PROVIDER_NAME = "工行收单";

    private static final String TRANS_TYPE_PRINT = "START_PRINT";       //打印
    private static final String TRANS_TYPE_INQUERY = "INQUERY";         //余额查询
    private static final String TRANS_TYPE_REPRINT = "REPRINT";         //重打印签购单
    private static final String TRANS_TYPE_STATISTICS = "STATISTICS";   //交易汇总
    private static final String TRANS_TYPE_SETTLEMENT = "SETTLEMENT";   //结账

    private final static String DATE_FORMAT = "yyyyMMdd";
    private final static String PRINT_DATE_FORMAT = "yyyyMMdd HH:mm:ss";
    private final static SimpleDateFormat DATE_FORMATTER = new SimpleDateFormat(DATE_FORMAT, Locale.CHINESE);
    private final static SimpleDateFormat PRINT_DATE_FORMATTER = new SimpleDateFormat(PRINT_DATE_FORMAT, Locale.CHINESE);


    /**
     * 初始化一个 {@code IcbcPaymentServiceProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    public IcbcPaymentServiceProvider(Activity activity) {
        super(activity);
    }

    /**
     * 将分转换成元，保留两位小数
     *
     * @param amount 分
     * @return 元
     */
    private static String transFen2Yuan(int amount) {
        int flag = 0;
        String amString = String.valueOf(amount);
        if (amString.charAt(0) == '-') {
            flag = 1;
            amString = amString.substring(1);
        }
        StringBuilder result = new StringBuilder();
        if (amString.length() == 1) {
            result.append("0.0").append(amString);
        } else if (amString.length() == 2) {
            result.append("0.").append(amString);
        } else {
            String intString = amString.substring(0, amString.length() - 2);
            for (int i = 1; i <= intString.length(); i++) {
                if ((i - 1) % 3 == 0 && i != 1) {
                    result.append(",");
                }
                result.append(intString.substring(intString.length() - i, intString.length() - i + 1));
            }
            result.reverse().append(".").append(amString.substring(amString.length() - 2));
        }
        if (flag == 1) {
            return "-" + result.toString();
        } else {
            return result.toString();
        }
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 返回结果字典
     * @throws PaymentProviderException 支付服务异常
     */
    @Override
    public Map handleResult(Intent data) throws PaymentProviderException {
        this.handleBaseResult(data.getBundleExtra("baseResult"));
        Map<String, String> map = new HashMap<>();
        return map;
    }

    /**
     * 返回提供程序名称。
     *
     * @return 提供程序名称
     */
    @Override
    public String providerName() {
        return PROVIDER_NAME;
    }

    /**
     * 打印交易汇总。
     */
    @Override
    public void printTransSummary(final PaymentHandler handler) throws PaymentProviderException {
        Bundle transData = new Bundle();

        Date today = new Date();
        transData.putString("START_DATE", DATE_FORMATTER.format(today));
        transData.putString("END_DATE", DATE_FORMATTER.format(today));
        transData.putString("BATCH_NUM,", null);
        IcbcService.startTrans(TRANS_TYPE_STATISTICS, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(Bundle baseResult, Bundle transResult, Bundle extraInfo) throws RemoteException {
                if (handler != null) {
                    handler.onFinish(baseResult, transResult, extraInfo);
                }
            }
        });
    }

    /**
     * 打印交易汇总。
     *
     * @param baseResultBundle  交易汇总统计接口返回的基础数据
     * @param transResultBundle 交易汇总统计接口返回的交易数据
     * @param handler           打印交易汇总后的处理程序
     * @throws PaymentProviderException 如果请求未成功，或解析返回数据失败
     */
    public void printTransSummary(Bundle baseResultBundle, Bundle transResultBundle,
                                  final PaymentHandler handler) throws PaymentProviderException {

        BaseResult baseResult = this.handleBaseResult(baseResultBundle);
        TransResult transResult = this.handleResult(TransResult.class, transResultBundle);

        // 组装返回数据
        StResultOuter resultOuter = JSON.parseObject(transResult.collection(), StResultOuter.class);

        if (resultOuter == null) {
            throw new PaymentProviderException("解析数据失败，请稍后重试");
        }

        Bundle transData = new Bundle();
        transData.putStringArrayList("PRINT_DATA", createPrintStrList(resultOuter, baseResult));

        IcbcService.startTrans(TRANS_TYPE_PRINT, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) throws RemoteException {
                if (handler != null) {
                    IcbcPaymentServiceProvider.super.activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            handler.onFinish(baseResult, transResult, extraInfo);
                        }
                    });
                }
            }
        });
    }

    /**
     * 根据交易汇总接口返回的数据组装打印文本集合
     *
     * @param resultOuter 交易汇总接口返回的数据
     * @param baseResult  基本结果数据
     * @return ArrayList<String>
     */
    private ArrayList<String> createPrintStrList(StResultOuter resultOuter, BaseResult baseResult) {
        ArrayList<String> printData = new ArrayList<>();
        if (resultOuter != null) {

            //开头空行
            printData.add("{\"type\":1,\"text\":[{\"data\":\"\"}]}");

            //大标题 大字号 居中
            printData.add("{\"type\":1,\"text\":[{\"font\":2,\"align\":1,\"data\":\"交易汇总\"}]}");

            //小标题
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"商户名称 %s\"}]}", baseResult.custName()));
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"商户编号 %s\"}]}", baseResult.custNo()));
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"终端编号 %s\"}]}", baseResult.termNo()));
            printData.add(String.format("{\"type\":1,\"text\":[{\"data\":\"打印时间 %s\"}]}", PRINT_DATE_FORMATTER.format(new Date())));
            printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
            printData.add("{\"type\":1,\"text\":[{\"data\":\"内卡        笔数        金额\"}]}");

            if (resultOuter.org != null && resultOuter.org.size() > 0) {
                // 银行卡交易数据
                for (StOrgItem orgItem : resultOuter.org) {
                    if ("CUP".equals(orgItem.org_name)) {
                        // 只统计中国银联的
                        int totalCnt = 0;//合计笔数
                        if (orgItem.trans != null) {
                            for (StOrgItemTransItem transItem : orgItem.trans) {

                                if ("PURCHASE".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"消费        %d         %s\"}]}", transItem.trans_cnt, transFen2Yuan(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("POS_VOID".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"撤销        %d         %s\"}]}", transItem.trans_cnt, transFen2Yuan(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("REFUND".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"退货        %d         %s\"}]}", transItem.trans_cnt, transFen2Yuan(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("INTEGRAL_PURCHASE".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"积分        %d         %s\"}]}", transItem.trans_cnt, transFen2Yuan(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("INSTALLMENT".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"分期        %d         %s\"}]}", transItem.trans_cnt, transFen2Yuan(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                } else if ("INTEGRAL_REFUND".equals(transItem.trans_name)) {
                                    printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"积分退货    %d         %s\"}]}", transItem.trans_cnt, transFen2Yuan(transItem.trans_amt)));
                                    totalCnt += transItem.trans_cnt;
                                }
                            }
                        }

                        //总计
                        printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
                        printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"合计        %d         %s\"}]}", totalCnt, transFen2Yuan(orgItem.org_amt)));
                        printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
                    }
                }
            }

            if (resultOuter.qrcode != null && resultOuter.qrcode.size() > 0) {
                // 二维码交易数据
                StQRCodeItem qrCodeItem = resultOuter.qrcode.get(0);

                printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"二维码消费   %d         %s    \"}]}", qrCodeItem.qr_cnt, transFen2Yuan(qrCodeItem.qr_amt)));
                printData.add(String.format(Locale.CHINESE, "{\"type\":1,\"text\":[{\"data\":\"二维码退货   %d         %s    \"}]}", qrCodeItem.qr_refund_cnt, transFen2Yuan(qrCodeItem.qr_refund_amt)));
                printData.add("{\"type\":1,\"text\":[{\"data\":\"--------------------------------\"}]}");
            }

            //结尾空行
            printData.add("{\"type\":1,\"text\":[{\"data\":\"----------x----------x----------\"}]}");
            printData.add("{\"type\":1,\"text\":[{\"font\":2,\"data\":\" \"}]}");
            printData.add("{\"type\":1,\"text\":[{\"font\":2,\"data\":\" \"}]}");
            printData.add("{\"type\":1,\"text\":[{\"font\":2,\"data\":\" \"}]}");
        }

        return printData;
    }

    /**
     * 查询银行卡余额。
     */
    @Override
    public void queryBalance(final PaymentHandler handler) throws PaymentProviderException {
        IcbcService.startTrans(TRANS_TYPE_INQUERY, new Bundle(), new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) {
                if (handler != null) {
                    handler.onFinish(baseResult, transResult, extraInfo);
                }
            }
        });
    }

    /**
     * 重打印签购单。
     *
     * @param traceNo 交易凭证号
     */
    @Override
    public void reprintReceipt(String traceNo, final PaymentHandler handler) throws PaymentProviderException {
        Bundle transData = new Bundle();
//        transData.putString("OLD_REF_NO,", refNo);
        IcbcService.startTrans(TRANS_TYPE_REPRINT, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(Bundle baseResult, Bundle transResult, Bundle extraInfo) {
                if (handler != null) {
                    handler.onFinish(baseResult, transResult, extraInfo);
                }
            }
        });
    }

    /**
     * 重打印结算单。
     */
    @Override
    public void reprintSettlement() {
        // 工行收单未提供【重打印结算单】接口
    }

    /**
     * 结算。
     */
    @Override
    public void settle(final PaymentHandler handler) throws PaymentProviderException {
        Bundle transData = new Bundle();
        IcbcService.startTrans(TRANS_TYPE_SETTLEMENT, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) {
                if (handler != null) {
                    handler.onFinish(baseResult, transResult, extraInfo);
                }
            }
        });
    }

    /**
     * 换班。
     */
    @Override
    public void shift() {
        // 工行收单未提供【换班】接口
    }

    /**
     * 班次查询
     */
    @Override
    public void queryBatchNo() {
        //不支持
    }

    /**
     * 交易查询
     *
     * @param traceNo            凭证号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单；
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void tradeQuery(String traceNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        //不支持
    }

    /**
     * @param orderNo            商户订单号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void scanTradeQuery(String orderNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        //不支持
    }
}
