/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import android.app.Activity;
import android.os.Bundle;

import com.hisense.pos.payment.PaymentProvider;
import com.hisense.pos.payment.PaymentProviderException;

/**
 * 工行收单提供程序的基类。
 *
 * @author zhangsai 2018-11-28 HIPOSAPP-888
 */
abstract class IcbcProviderBase extends PaymentProvider {

    /**
     * 初始化一个 {@code IcbcProviderBase} 的实例。
     *
     * @param activity Activity 活动
     */
    IcbcProviderBase(Activity activity) {
        super(activity);
    }


    /**
     * 返回 Android 请求编码。
     *
     * @return Android 请求编码
     */
    public int requestCode() {
        return 0;
    }

    /**
     * 处理接口返回的基本结果数据。
     *
     * @param bundle 基本结果数据
     * @return 基本结果数据的实例
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    public BaseResult handleBaseResult(Bundle bundle) throws PaymentProviderException {
        if (bundle == null) {
            throw new PaymentProviderException("基本结果数据为空");
        }

        BaseResult baseResult = new BaseResult(bundle);
        if (!baseResult.isSuccess()) {
            throw new PaymentProviderException(baseResult.description());
        }

        return baseResult;
    }

    /**
     * 处理接口返回的结果数据。
     *
     * @param cls    接口返回的结果数据对应的类型
     * @param bundle 接口返回的结果数据
     * @param <T>    接口返回的结果数据转换的类型
     * @return 结果数据的实例
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    public <T extends ResultBase> T handleResult(Class<T> cls, Bundle bundle) throws PaymentProviderException {
        T result;

        try {
            result = cls.newInstance();
            result.setBundle(bundle);
        } catch (Exception ex) {
            result = null;
        }

        return result;
    }
}
