/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import android.app.Activity;
import android.os.Bundle;
import android.os.RemoteException;

import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;
import com.icbc.smartpos.transservice.aidl.TransHandler;

import java.util.Date;
import java.util.Map;

/**
 * 工行收单二维码消费提供程序。
 *
 * @author zhangsai 2018-11-27 HIPOSAPP-888
 */
public class IcbcQRPurchaseProvider extends IcbcPaymentProvider implements IPaymentProvider {

    /**
     * 初始化一个 {@code IcbcQRPurchaseProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    public IcbcQRPurchaseProvider(Activity activity) {
        super(activity);
    }


    /**
     * 返回是否支持撤销。
     *
     * @return true 表示支持撤销，false 表示不支持撤销
     */
    @Override
    public boolean supportCancellation() {
        return false;
    }

    /**
     * 消费。
     *
     * @param payRequest 交易金额
     * @param handler    消费结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    @Override
    public void pay(PayRequest payRequest, PaymentHandler handler) throws PaymentProviderException {
        this.pay(TransListType.QRPURCHASE, payRequest, handler);
    }

    /**
     * 隔日退货。
     *
     * @param payRequest 退款金额
     * @param refNo      参考号
     * @param transDate  交易日期
     * @param params     扩展参数
     * @param handler    隔日退货结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    @Override
    public void refund(PayRequest payRequest, String refNo, Date transDate, Map<String, String> params,
                       final PaymentHandler handler) throws PaymentProviderException {
        if (params == null) {
            throw new PaymentProviderException("未提交交易参数");
        }

        Bundle transData = new Bundle();
        transData.putLong("AMOUNT", Long.valueOf(makeFieldAmount(String.valueOf(payRequest.getAmount()))));
        transData.putString("OLD_QRCODE_ORDER", params.get("PAY_NO"));
        transData.putString("LAST_OF_CARDNO", "");

        IcbcService.startTrans(TransType.QRREFUND, transData, new TransHandler.Stub() {
            @Override
            public void onFinish(final Bundle baseResult, final Bundle transResult, final Bundle extraInfo) throws RemoteException {
                if (handler != null) {
                    activity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            handler.onFinish(baseResult, transResult, extraInfo);
                        }
                    });
                }
            }
        });
    }

    /**
     * 当日撤销。
     *
     * @param payRequest 退款金额
     * @param traceNo    原交易凭证号
     * @param params     扩展参数
     * @param handler    撤销结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    @Override
    public void cancel(PayRequest payRequest, String traceNo, Map<String, String> params,
                       PaymentHandler handler) throws PaymentProviderException {
        // 工行收单当日撤销与隔日退货均通过【二维码退货】接口实现
        this.refund(payRequest, traceNo, new Date(), params, handler);
    }
}
