/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment.icbc;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;

import com.hisense.pos.payment.PaymentProviderException;
import com.icbc.smartpos.transservice.aidl.ITransService;
import com.icbc.smartpos.transservice.aidl.TransHandler;

/**
 * 工行收单服务。
 *
 * @author zhangsai 2018-11-27 HIPOSAPP-888
 */
public final class IcbcService {
    static IcbcService icbcService;     // 工行收单服务的实例
    private ITransService transService; // 交易服务
    private Context context;            // Context
    private Bundle ctrlData;            // 控制参数

    private ServiceConnection connection = new ServiceConnection() {
        @Override
        public void onServiceDisconnected(ComponentName name) {
//            CommonTools.toastShort("工行收单服务中断");
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            transService = ITransService.Stub.asInterface(service);
//            CommonTools.toastShort("工行收单服务绑定成功");
        }
    };

    /**
     * 初始化一个 {@code IcbcService} 的实例。
     */
    private IcbcService() {

    }

    /**
     * 返回 {@code IcbcService} 工行收单服务的实例。
     */
    static IcbcService getInstance() {
        if (icbcService == null) {
            synchronized (IcbcService.class) {
                if (icbcService == null) {
                    icbcService = new IcbcService();
                }
            }
        }
        return icbcService;
    }

    /**
     * 绑定工行收单服务。
     *
     * @param context Context
     * @param appName 外部应用名称
     */
    public static void bind(Context context, String appName) {
        Intent intent = new Intent();
        intent.setAction("com.icbc.smartpos.transservice.TransService");
        intent.setPackage("com.icbc.smartpos.bankpay");
        context.bindService(intent, IcbcService.getInstance().init(context, appName).connection,
                Context.BIND_AUTO_CREATE);
    }

    /**
     * 解绑工行收单服务。
     */
    public static void unbind() {
        IcbcService.getInstance().context.unbindService(IcbcService.getInstance().connection);
    }

    /**
     * 调用交易接口。
     *
     * @param type      交易类型
     * @param transData 交易数据
     * @param handler   交易结果处理程序
     * @throws PaymentProviderException 如果请求失败，或数据解析错误
     */
    static void startTrans(String type, Bundle transData, TransHandler handler) throws PaymentProviderException {
        try {
            if (IcbcService.getInstance().transService == null) {
                throw new PaymentProviderException("未发现指定服务，请检查工行paybank是否安装");
            }

            IcbcService.getInstance().transService.startTrans(type,
                    IcbcService.getInstance().ctrlData, transData, handler);
        } catch (RemoteException e) {
            e.printStackTrace();
            throw new PaymentProviderException("工行服务调用异常：" + e.getLocalizedMessage(), e);
        }
    }

    /**
     * 返回控制参数。
     *
     * @param appName 外部应用名称
     * @return 控制参数
     */
    private static Bundle getCtrlData(String appName) {
        Bundle ctrlData = new Bundle();
        ctrlData.putString("APP_NAME", appName);
        ctrlData.putString("AIDL_VER", "V1.0.2");
        return ctrlData;
    }

    /**
     * 初始化 {@code IcbcService} 工行收单服务。
     *
     * @param context Context
     * @param appName 外部应用名称
     * @return 工行收单服务的实例
     */
    private IcbcService init(Context context, String appName) {
        this.context = context;
        this.ctrlData = getCtrlData(appName);
        return this;
    }
}
