package com.hisense.pos.print.icbc;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.widget.Toast;


import com.icbc.smartpos.deviceservice.aidl.IBeeper;
import com.icbc.smartpos.deviceservice.aidl.IDeviceInfo;
import com.icbc.smartpos.deviceservice.aidl.IDeviceService;
import com.icbc.smartpos.deviceservice.aidl.IInsertCardReader;
import com.icbc.smartpos.deviceservice.aidl.ILed;
import com.icbc.smartpos.deviceservice.aidl.IMagCardReader;
import com.icbc.smartpos.deviceservice.aidl.IPBOC;
import com.icbc.smartpos.deviceservice.aidl.IPinpad;
import com.icbc.smartpos.deviceservice.aidl.IPrinter;
import com.icbc.smartpos.deviceservice.aidl.IRFCardReader;
import com.icbc.smartpos.deviceservice.aidl.IScanner;
import com.icbc.smartpos.deviceservice.aidl.ISerialPort;

/**
 * 工行联迪驱动调用
 *
 * @author lizhixin 20180625 HIPOSAPP-596 新增工行收单功能-读卡
 */
public class ICBCDeviceHelper {
    private static final String TAG = "DeviceServiceHelper";

    private static final String ACTION_DEVICE_SERVICE = "com.icbc.smartpos.device_service";
    private static final String PACKAGE_DEVICE_SERVICE = "com.icbc.smartpos.deviceservice";

    private static ICBCDeviceHelper deviceHelper;
    private Context mContext;
    private boolean isBindService;
    private IDeviceService mDeviceService;
    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            isBindService = true;
            mDeviceService = IDeviceService.Stub.asInterface(service);
           // LoggerUtils.getLogger().debug("-------------onICBCServiceConnected----------" + isBindService);
            Log.d(TAG, "-------------onICBCServiceConnected----------");
            linkToDeath(service);
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            isBindService = false;
            Log.d(TAG, "-------------onICBCServiceDisconnected----------");
        }

        private void linkToDeath(IBinder service) {
            try {
                service.linkToDeath(new IBinder.DeathRecipient() {
                    @Override
                    public void binderDied() {
                        isBindService = false;
                        Log.d(TAG, "-------------binder service is dead!!!----------");
                    }
                }, 0);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    };

    private ICBCDeviceHelper() {
    }

    public static ICBCDeviceHelper getInstance() {
        if (deviceHelper == null) {
            synchronized (ICBCDeviceHelper.class) {
                if (deviceHelper == null) {
                    deviceHelper = new ICBCDeviceHelper();
                }
            }
        }
        return deviceHelper;
    }

    public IDeviceService getDeviceService() {
        return mDeviceService;
    }

    public ICBCDeviceHelper init(Context context) {
        mContext = context;
        return deviceHelper;
    }

    public void bindICBCDeviceService() {
        if (!isBindService) {
            Intent service = new Intent(ACTION_DEVICE_SERVICE);
            service.setPackage(PACKAGE_DEVICE_SERVICE);
            boolean bindSucc = mContext.bindService(service, mConnection, Context.BIND_AUTO_CREATE);
            if (bindSucc) {
                Log.d(TAG, "-------------bind ICBCDevice service success----------");
            } else {
                Log.e(TAG, "-------------bind ICBCDevice service failed----------");
            }
        }
    }

    public void unbindICBCDeviceService() {
        if (isBindService) {
            Log.d(TAG, "-------------unbind ICBCDevice service success----------");
            mContext.unbindService(mConnection);
            isBindService = false;
        }
    }

    public IBeeper getBeeper() {
        try {
            return mDeviceService.getBeeper();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getBeeper SecurityException---------------");
            return null;
        }
    }

    ;

    public IMagCardReader getMagCardReader() {
        try {
            return mDeviceService.getMagCardReader();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getMagCardReader SecurityException---------------");
            return null;
        }
    }

    ;

    public IDeviceInfo getDeviceInfo() {
        try {
            return mDeviceService.getDeviceInfo();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getDeviceInfo SecurityException---------------");
            return null;
        }
    }

    ;

    public ILed getLED() {
        try {
            return mDeviceService.getLed();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getLed SecurityException---------------");
            return null;
        }
    }

    ;


    public IInsertCardReader getInsertCardReader() {
        try {
            return mDeviceService.getInsertCardReader();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getInsertCardReader SecurityException---------------");
            return null;
        }
    }

    ;

    public IRFCardReader getRFCardReader() {
        try {
            return mDeviceService.getRFCardReader();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getRFCardReader SecurityException---------------");
            return null;
        }
    }

    ;

    public IScanner getFrontScanner() {
        try {
            return mDeviceService.getScanner(1);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getScanner SecurityException---------------");
            return null;
        }
    }

    ;

    public IScanner getRearScanner() {
        try {
            return mDeviceService.getScanner(0);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getScanner SecurityException---------------");
            return null;
        }
    }

    ;

    public ISerialPort getSerialPort() {
        try {
            return mDeviceService.getSerialPort();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getSerialPort SecurityException---------------");
            return null;
        }
    }


    public IPrinter getPrinter() {
        try {
            return mDeviceService.getPrinter();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getPrinter SecurityException---------------");
            return null;
        }
    }


    public IPinpad getPinpad(int kapId) {
        try {
            return mDeviceService.getPinpad(kapId);
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getPinpad SecurityException---------------");
            return null;
        }
    }


    public IPBOC getPBOC() {
        try {
            return mDeviceService.getPBOC();
        } catch (RemoteException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            return null;
        } catch (SecurityException e) {
            e.printStackTrace();
            Toast.makeText(mContext, e.getMessage(), Toast.LENGTH_LONG).show();
            Log.d(TAG, "-------------------getPBOC SecurityException---------------");
            return null;
        }
    }

    /**
     * 打印刷卡调用时需要锁定服务
     */
    /*public void lockService() {
        if (isBindService) {
            try {
                mDeviceService.lock(new Bundle(), new Binder());
                TGlobal.getLogger().debug("lock成功");
            } catch (Exception e) {
                e.printStackTrace();
                TGlobal.getLogger().debug(e.getMessage());
            }
        } else {
            TGlobal.getLogger().debug("ICBCService 未绑定成功，请重新登录 isBindService"+isBindService);
            CommonTools.toastShort(App.getContext(), "ICBCService 未绑定成功，请重新登录");
        }
    }*/

    /**
     * 打印刷卡调用结束时需要解锁服务
     */
    /*public void unlockService() {
        if (isBindService) {
            try {
                mDeviceService.unlock();
                TGlobal.getLogger().debug("unLock成功");
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            TGlobal.getLogger().debug("ICBCService 未绑定成功，请重新登录 isBindService"+isBindService);
            CommonTools.toastShort(App.getContext(), "ICBCService 未绑定成功，请重新登录");
        }
    }*/
}
