package com.hisense.pos.print.icbc;

import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.ConditionVariable;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.print.IPrintProvider;
import com.icbc.smartpos.deviceservice.aidl.IPrinter;
import com.icbc.smartpos.deviceservice.aidl.PrinterListener;

/**
 * 工行打印提供程序
 *
 * @author xieliyan  2019-11-13
 */
public class ICBCPrintProvider implements IPrintProvider {


    private IPrinter iPrinter;
    private static ConditionVariable conditionVariable = new ConditionVariable();

    private PrinterListener printerListener = new PrinterListener.Stub() {
        @Override
        public void onFinish() {
            conditionVariable.open();
        }

        @Override
        public void onError(int error) {
            conditionVariable.open();
        }
    };


    @Override
    public boolean initPrinter(int prnBaudRate, int prnPort, int prnType, int feedLine) {
        iPrinter = ICBCDeviceHelper.getInstance().getPrinter();
        return iPrinter != null;
    }

    @Override
    public void cutPaper() {

    }

    @Override
    public void closePrinter() {

    }

    @Override
    public void addText(String text, int align, int font) {

        try {
            Bundle format = new Bundle();
            format.putInt("font", font);
            format.putInt("align", align);
            format.putBoolean("newline", true);   //是否自动换行
            iPrinter.addText(format, text);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加文本异常" + e.getMessage());
        }

    }

    @Override
    public void addBarCode(String barCode) {
        try {
            Bundle format = new Bundle();
            format.putInt("align", 1);
            format.putInt("height", 80);
            iPrinter.addBarCode(format, barCode);
        } catch (Exception e) {

            LoggerUtils.getLogger().debug("添加文本异常" + e.getMessage());
        }
    }

    @Override
    public void addQrCode(String qrCode, int offset, int expectedHeight) {
        try {
            Bundle format = new Bundle();
            format.putInt("align", 1);
            format.putInt("expectedHeight", expectedHeight);
            iPrinter.addQrCode(format, qrCode);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加文本异常" + e.getMessage());
        }
    }

    @Override
    public void addImage(byte[] imageData, Bitmap bitmap, int offset, int width, int height) {
        try {
            Bundle format = new Bundle();
            format.putInt("align", 1);
            format.putInt("offset", offset);
            format.putInt("width", width);
            format.putInt("height", height);
            iPrinter.addImage(format, imageData);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("添加二维码异常" + e.getMessage());
        }
    }

    @Override
    public int getStatus() {

        try {
            return iPrinter.getStatus();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("获取打印状态异常" + e.getMessage());
            return -1;
        }

    }

    @Override
    public void feedLine(int lines) {
        try {
            iPrinter.feedLine(lines);
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("换行异常" + e.getMessage());
        }
    }

    @Override
    public void startPrint() {
        try {
            iPrinter.startPrint(printerListener);
            conditionVariable.block(30000);//此方法不调用会引起打印数据不清除
            conditionVariable.close();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("启动打印异常" + e.getMessage());
        }

    }

}
