/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.ums;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.M1Card;
import com.ums.upos.sdk.card.m1.AuthEntity;
import com.ums.upos.sdk.card.m1.BlockEntity;
import com.ums.upos.sdk.card.m1.KeyTypeEnum;
import com.ums.upos.sdk.card.m1.M1CardManager;
import com.ums.upos.sdk.cardslot.CardInfoEntity;
import com.ums.upos.sdk.utils.common.ByteUtils;

/**
 * 用于读取CRM制卡标准1.0的 Mifare1 非接触式磁卡读取器。
 *
 * @author zhangsai    2018-04-25 HIPOSAPP-504
 * @author wangyaogang 2019-05-28 HIPOSAPP-1142 优化银联商务储值卡刷卡结果回调逻辑
 * @author wangyaogang 2019-12-09 HIAPOSCMP-408 卡号判空
 */
class M1CardReader {

    /**
     * 读卡
     */
    public void read(final String password, CardInfoEntity arg1, final CardNoCallback callback) {
        M1CardManager m1CardManager = new M1CardManager();

        try {
            String pwd = password + "FFFFFF";
            byte[] blockData = readData(m1CardManager, M1Card.SECTOR_CARD_NO, M1Card.BLOCK_CARD_NO, pwd, arg1);
            String cardNo = M1Card.decrypt(new String(blockData));
            LoggerUtils.getLogger().debug("ums刷卡:" + cardNo);
            if (callback != null && !"".equals(cardNo)) {
                callback.onSuccess(cardNo);
            }
        } catch (Exception e2) {
            LoggerUtils.getLogger().debug("ums刷卡read异常:" + e2.toString());
            e2.printStackTrace();
        }
    }

    protected byte[] readData(M1CardManager cardManager, int sector, int block, String password, CardInfoEntity arg1) {
        try {
            AuthEntity authEntity = new AuthEntity();
            authEntity.setBlkNo(sector);//扇区号
            authEntity.setKeyType(KeyTypeEnum.KEY_A);//密钥类型
            authEntity.setPwd(ByteUtils.hexString2ByteArray(password));//密钥
            authEntity.setSerialNo(arg1.getCardNo().trim());
            cardManager.authority(authEntity);

            BlockEntity blockEntity = new BlockEntity();
            blockEntity.setBlkData(new byte[16]);//块中内容
            blockEntity.setBlkNo(block);   //块号
            cardManager.readBlock(blockEntity);

            return blockEntity.getBlkData();
        } catch (Exception e) {
            LoggerUtils.getLogger().debug("ums刷卡readData异常:" + e.toString());
        }

        return null;
    }

}
