/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.ums;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.M1Card;
import com.hisense.pos.card.M1CardBlockPasswordCallback;
import com.hisense.pos.card.M1CardReaderProvider;
import com.ums.upos.sdk.card.m1.M1CardManager;
import com.ums.upos.sdk.cardslot.CardInfoEntity;
import com.ums.upos.sdk.utils.common.ByteUtils;

/**
 * 用于读取CRM制卡标准2.0的 Mifare1 非接触式磁卡读取器。
 *
 * @author zhangsai    2018-04-26 HIPOSAPP-504
 * @author wangyaogang 2019-05-28 HIPOSAPP-1142 优化银联商务储值卡刷卡结果回调逻辑
 */
class M1CardReaderV2 extends M1CardReader {

    // 磁卡读取提供程序
    private M1CardReaderProvider provider;

    /**
     * 初始化一个 {@code M1CardReaderV2} 的实例。
     *
     * @param provider 磁卡读取提供程序
     */
    public M1CardReaderV2(M1CardReaderProvider provider) {
        this.provider = provider;
    }

    @Override
    public void read(final String password, final CardInfoEntity arg1, final CardNoCallback callback) {
        final M1CardManager m1CardManager = new M1CardManager();
        try {
            byte[] authData = readData(m1CardManager, M1Card.SECTOR_AUTH, M1Card.BLOCK_AUTH, password, arg1);
            String icNo = ByteUtils.bytes2HexString(authData);

            this.provider.getBlockPassword(icNo, new M1CardBlockPasswordCallback() {
                @Override
                public void onSuccess(String blockPwd) {
                    byte[] blockData = readData(m1CardManager, M1Card.SECTOR_CARD_NO, M1Card.BLOCK_CARD_NO, blockPwd, arg1);
                    String cardNo = ByteUtils.bytes2HexString(blockData);
                    if (!"".equals(cardNo)) {
                        cardNo = cardNo.substring(0, cardNo.length() - 8);
                        if (callback != null) {
                            LoggerUtils.getLogger().debug("umsV2刷卡:" + cardNo);
                            callback.onSuccess(cardNo);
                        }
                    }
                }
            });

        } catch (Exception e2) {
            e2.printStackTrace();
            LoggerUtils.getLogger().debug("ums刷卡V2read异常:" + e2.toString());
        }
    }

}
