/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card.ums;

import android.content.Context;
import android.os.Bundle;
import android.widget.Toast;

import com.hisense.common.util.LoggerUtils;
import com.hisense.pos.card.CardNoCallback;
import com.hisense.pos.card.ICardReader;
import com.hisense.pos.card.M1Card;
import com.hisense.pos.card.M1CardReaderProvider;
import com.ums.upos.sdk.cardslot.CardInfoEntity;
import com.ums.upos.sdk.cardslot.CardSlotManager;
import com.ums.upos.sdk.cardslot.CardSlotTypeEnum;
import com.ums.upos.sdk.cardslot.CardTypeEnum;
import com.ums.upos.sdk.cardslot.OnCardInfoListener;
import com.ums.upos.sdk.cardslot.SwipeSlotOptions;
import com.ums.upos.sdk.exception.CallServiceException;
import com.ums.upos.sdk.exception.SdkException;
import com.ums.upos.sdk.system.BaseSystemManager;
import com.ums.upos.sdk.system.OnServiceStatusListener;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * 银联商务读卡器，同时支持磁条卡和 Mifare1 非接触式磁卡。
 *
 * @author zhangsai    2018-04-25 HIPOSAPP-504
 * @author wangyaogang 2019-05-28 HIPOSAPP-1142 优化银联商务储值卡刷卡结果回调逻辑
 * @author wangyaogang 2019-07-02 HIPOSAPP-1185 银联商务读卡异常时进行提示
 * @author wangyaogang 2019-12-09 HIAPOSCMP-408 卡号判空
 */
public class UmsCardReader implements ICardReader {

    // 是否正在监听。避免启动多个监听线程。
    private static volatile boolean IS_RUNNING = false;

    private CardSlotManager cardSlotManager;//刷卡器
    private Context context;
    private M1CardReader m1CardReader;

    private String password;
    private CardNoCallback callback;

    public UmsCardReader(int version, M1CardReaderProvider provider) {
        cardSlotManager = new CardSlotManager();
        switch (version) {
            case M1Card.VERSION1:
                m1CardReader = new M1CardReader();
                break;
            case M1Card.VERSION2:
                m1CardReader = new M1CardReaderV2(provider);
                break;
        }
    }

    @Override
    public void open(final Context context) {
        try {
            this.context = context;

            Map<CardSlotTypeEnum, Bundle> options = new HashMap<>();
            Bundle bundle = new Bundle();
            bundle.putBoolean(SwipeSlotOptions.LRC_CHECK, false);
            options.put(CardSlotTypeEnum.SWIPE, bundle);
            cardSlotManager.setConfig(options);
        } catch (CallServiceException | SdkException e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("ums刷卡open1异常:" + e.toString());
        } catch (Exception e) {
            e.printStackTrace();
            LoggerUtils.getLogger().debug("ums刷卡open2异常:" + e.toString());
        }
    }

    @Override
    public void read(final String password, final CardNoCallback callback) {
        this.password = password;
        this.callback = callback;

        this.start();
    }

    @Override
    public void start() {
        if (IS_RUNNING) {
            return;
        }

        try {
            BaseSystemManager
                    .getInstance()
                    .deviceServiceLogin(
                            this.context, null, "99999998",
                            new OnServiceStatusListener() {
                                @Override
                                public void onStatus(int arg0) {
                                    if (0 == arg0 || 2 == arg0 || 100 == arg0) {
                                        IS_RUNNING = true;
                                        searchCardInfo(password, callback);
                                    } else {
                                        //  Toast.makeText(activity, "刷卡器出错", Toast.LENGTH_LONG).show();
                                    }
                                }
                            });

        } catch (SdkException e) {
            e.printStackTrace();
            showToast(e);
        } catch (Exception e) {
            e.printStackTrace();
//            showToast(e);
        }
    }

    /**
     * 提示错误信息
     *
     * @param e 异常
     */
    private void showToast(Exception e) {
        if (context != null) {
            Toast.makeText(context, "读卡异常,请退出重试! " + e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public void stop() {
        try {
            cardSlotManager.stopRead();
        } catch (Exception e) {
            e.printStackTrace();
        }

        IS_RUNNING = false;
    }

    @Override
    public void close() {
        this.stop();
    }

    private void searchCardInfo(final String password, final CardNoCallback callback) {
        try {
            Set<CardSlotTypeEnum> slotTypes = new HashSet<>();
            slotTypes.add(CardSlotTypeEnum.SWIPE);
            slotTypes.add(CardSlotTypeEnum.RF);//射频卡
            slotTypes.add(CardSlotTypeEnum.ICC1);

            Set<CardTypeEnum> cardTypes = new HashSet<>();
            cardTypes.add(CardTypeEnum.MAG_CARD);
            cardTypes.add(CardTypeEnum.M1CARD);

            int timeout = 0;

            cardSlotManager.readCard(slotTypes, cardTypes, timeout,
                    new OnCardInfoListener() {

                        @Override
                        public void onCardInfo(int arg0, CardInfoEntity arg1) {
                            if (0 != arg0) {
//                                callback.onError(null);
                            } else {
                                switch (arg1.getActuralEnterType()) {
                                    case MAG_CARD:
                                        String cardNo = arg1.getTk2().trim();
                                        LoggerUtils.getLogger().debug("ums磁条卡:" + cardNo);
                                        if (callback != null) {
                                            callback.onSuccess(cardNo);
                                        }
                                        break;
                                    case M1CARD:
                                        if (m1CardReader != null) {
                                            m1CardReader.read(password, arg1, callback);
                                        }
                                        break;
                                    default:
                                        break;
                                }
                            }
                        }
                    }, null);

        } catch (CallServiceException | SdkException e) {
            e.printStackTrace();
            showToast(e);
        } catch (Exception e) {
            e.printStackTrace();
           // showToast(e);
        }
    }
}
