/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.unionpay.payment.ccb;

import android.os.Bundle;

import com.hisense.pos.payment.PayResult;

import java.math.BigDecimal;

/**
 * 建行直联应用返回的交易结果。
 *
 * @author xieliyan  2019-10-18 HIPOSAPP-1328
 */
public class UnionCcbPayResult extends PayResult {

    private String transId;         // 交易ID
    private Bundle bundle;


    /**
     * 初始化一个 {@code UnionCcbPayResult} 的实例。
     *
     * @param transId 交易ID
     * @param bundle
     */
    UnionCcbPayResult(Bundle bundle, String transId) {
        if (bundle != null) {
            this.bundle = bundle;
        } else {
            this.bundle = new Bundle();
        }
        this.transId = transId;
    }

    /**
     * 返回交易金额。
     *
     * @return 交易金额
     */
    @Override
    public double amount() {
        return parseAmount("amount");
    }

    /**
     * 返回卡号。
     *
     * @return 卡号
     */
    @Override
    public String cardNo() {
        if (bundle.containsKey("cardNo")) {
            return bundle.getString("cardNo");
        } else if (bundle.containsKey("transaction_id")) {
            return bundle.getString("transaction_id");
        }
        return "";

    }

    /**
     * 返回参考号。
     *
     * @return 参考号
     */
    @Override
    public String refNo() {
        if (bundle.containsKey("referenceNo")) {
            return bundle.getString("referenceNo");
        }

        return "";

    }

    /**
     * 返回交易凭证号。
     *
     * @return 交易凭证号
     */
    @Override
    public String traceNo() {
        if (bundle.containsKey("traceNo")) {
            return bundle.getString("traceNo");
        }

        return "";
    }

    /**
     * 返回终端号。
     *
     * @return 终端号
     */
    @Override
    public String termNo() {
        if (bundle.containsKey("terminalId")) {
            return bundle.getString("terminalId");
        }

        return "";
    }

    /**
     * 返回当前交易是否为消费。
     *
     * @return true 表示消费，false 表示其它。
     */
    @Override
    public boolean isPayment() {
        return TransId.CONSUME.equals(this.transId) ||
                TransId.CONSUME_SCAN.equals(this.transId);
    }

    /**
     * 返回商户订单号
     *
     * @return 商户订单号。
     */
    @Override
    public String outTradeNo() {
        if (bundle.containsKey("out_trade_no")) {
            return bundle.getString("out_trade_no");
        }

        return "";
    }

    /**
     * 返回商户号
     *
     * @return 商户号。
     */
    @Override
    public String mchId() {
        if (bundle.containsKey("mch_id")) {
            return bundle.getString("mch_id");
        }

        return "";
    }

    /**
     * 返回平台订单号
     *
     * @return 平台订单号。
     */
    public String transactionId() {
        if (bundle.containsKey("transaction_id")) {
            return bundle.getString("transaction_id");
        }

        return "";
    }

    /**
     * 返回平台订单号
     *
     * @return 平台订单号。
     */
    @Override
    public String payType() {
        if (bundle.containsKey("payType")) {
            return bundle.getString("payType");
        }

        return "";
    }

    /**
     * 返回当前交易是否为消费撤销。
     *
     * @return true 表示消费撤销，false 表示其它。
     */
    @Override
    public boolean isCancellation() {
        return TransId.CANCELLATION.equals(this.transId);
    }

    /**
     * 返回当前交易是否为退货。
     *
     * @return true 表示退货，false 表示其它。
     */
    @Override
    public boolean isRefunding() {
        return TransId.REFUNDING.equals(this.transId);
    }

    /**
     * 返回当前交易是否为查询
     *
     * @return true 表示查询，false 表示其它。
     */
    @Override
    public boolean isQuery() {
        return false;
    }

    private String getString(String name) {
        return this.getString(name, null);
    }

    private String getString(String name, String defaultValue) {

        if (bundle.containsKey(name)) {
            return bundle.getString(name);
        } else {
            return defaultValue;
        }

    }

    /**
     * 格式化金额
     *
     * @param name 待格式化字段
     * @return double 格式化后值
     */
    private double parseAmount(String name) {
        String amt = this.getString(name, "0");
        BigDecimal p = new BigDecimal(amt);
        BigDecimal q = new BigDecimal(100);
        return p.divide(q, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }


}
