/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.unionpay.payment.ccb;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;

import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PayResult;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 建行直联应用支付提供程序的基类。
 *
 * @author xieliyan  2019-09-10 HIPOSAPP-1328
 */
abstract class UnionCcbPaymentProvider extends UnionCcbProviderBase implements IPaymentProvider {

    private final static String REFUND_DATE_FORMAT = "MMdd";
    final static SimpleDateFormat REFUND_DATE_FORMATTER = new SimpleDateFormat(REFUND_DATE_FORMAT, Locale.CHINESE);
    private String transId; // 当前交易类型,

    /**
     * 初始化一个 {@code UnionCcbPaymentProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    UnionCcbPaymentProvider(Activity activity) {
        super(activity);
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 支付结果
     * @throws PaymentProviderException 支付异常
     */
    @Override
    public PayResult handleResult(Intent data) throws PaymentProviderException {
        if (null == data) {
            throw new PaymentProviderException("未返回任何数据");
        }

        Bundle bundle = data.getExtras();
        if (bundle.containsKey("reason")) {
            throw new PaymentProviderException(bundle.getString("reason"));
        }

        return new UnionCcbPayResult(bundle, transId);


    }

    @Override
    public boolean supportPayQuery() {
        return false;
    }

    @Override
    public boolean supportCancelQuery() {
        return false;
    }

    @Override
    public boolean supportRefundQuery() {
        return false;
    }

    /**
     * 支付。
     *
     * @param transId    交易ID
     * @param payRequest 支付金额
     */
    protected void pay(String transId, PayRequest payRequest) {
        HashMap<String, String> map = new HashMap<>();
        map.put("amt", makeFieldAmount(String.valueOf(payRequest.getAmount())));

        this.callPosTrans(transId, map);
    }


    /**
     * 撤销。
     *
     * @param transId    交易ID
     * @param payRequest 支付金额
     * @param traceNo    交易凭证号
     */
    protected void cancel(String transId, PayRequest payRequest, String traceNo, Map<String, String> params, PaymentHandler handler) {
        HashMap<String, String> map = new HashMap<>();
        map.put("amt", makeFieldAmount(String.valueOf(payRequest.getAmount())));
        map.put("oldTrace", traceNo);
        this.callPosTrans(transId, map);
    }

    /**
     * 退款。
     *
     * @param transId    交易ID
     * @param payRequest 支付金额
     * @param refNo      交易参考号
     * @param transDate  交易日期
     */
    public void refund(String transId, PayRequest payRequest, String refNo, Date transDate, Map<String, String> params, PaymentHandler handler) {

        HashMap<String, String> map = new HashMap<>();
        map.put("amt", makeFieldAmount(String.valueOf(payRequest.getAmount())));
        map.put("oldReferenceNo", refNo);
        map.put("oldDate", REFUND_DATE_FORMATTER.format(transDate));
        this.callPosTrans(transId, map);
    }

    /**
     * 调用 POS 交易接口。
     *
     * @param transId 交易ID
     * @param map     扩展参数
     */
    private void callPosTrans(String transId, HashMap<String, String> map) {
        this.transId = transId;
        this.callTrans(transId, map);

    }

    @Override
    public void payQuery(String uniqueNo, PaymentHandler handler) {
        //暂不对接查询方法
    }

    /**
     * 银行标准化金额
     *
     * @param amount 金额
     * @return amount.substring() 标准化后的金额
     */
    static String makeFieldAmount(String amount) {
        String[] part = amount.split("\\.");
        switch (part.length) {
            case 1:
                return ("0000000000" + amount + "00").substring(amount.length());
            case 2:
                String digit = part[1].length() >= 2 ? part[1].substring(0, 2)
                        : (part[1] + "00").substring(0, 2);
                amount = "000000000000" + part[0] + digit;
                return amount.substring(amount.length() - 12);
        }
        System.out.println(part.length);
        return null;
    }
}
