/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.unionpay.payment.ccb;

import android.app.Activity;
import android.content.Intent;

import com.hisense.pos.payment.IPaymentServiceProvider;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;

import java.util.Map;

import static com.hisense.pos.unionpay.payment.ccb.TransId.PRINT_TRANS_SUMMARY;
import static com.hisense.pos.unionpay.payment.ccb.TransId.PROVIDER_NAME;
import static com.hisense.pos.unionpay.payment.ccb.TransId.QUERY_CARD_BALANCE;
import static com.hisense.pos.unionpay.payment.ccb.TransId.REPRINT_SETTLEMENT;
import static com.hisense.pos.unionpay.payment.ccb.TransId.SETTLE;

/**
 * 建行直联支付功能提供程序（该功能利客来不再使用）
 *
 * @author xieliyan  2019-10-18 HIPOSAPP-1328
 */
public class UnionCcbPaymentServiceProvider extends UnionCcbProviderBase implements IPaymentServiceProvider {

    /**
     * 初始化一个 {@code UnionCcbPaymentServiceProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    public UnionCcbPaymentServiceProvider(Activity activity) {
        super(activity);
    }

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 返回结果字典
     * @throws PaymentProviderException 支付服务异常
     */
    @Override
    public Map<String, String> handleResult(Intent data) throws PaymentProviderException {
        if (data == null) {
            throw new PaymentProviderException("返回数据transData为空");
        }

        Map<String, String> map = UnionCcbPaymentServiceProvider.filterTransResult(data);


        return map;
    }

    /**
     * 返回提供程序名称。
     *
     * @return 提供程序名称
     */
    @Override
    public String providerName() {
        return PROVIDER_NAME;
    }

    /**
     * 重打印签购单。
     *
     * @param traceNo 交易凭证号
     */
    @Override
    public void reprintReceipt(String traceNo, PaymentHandler handler) {

    }

    /**
     * 重打印结算单。
     */
    @Override
    public void reprintSettlement() {
        this.callTrans(REPRINT_SETTLEMENT);
    }

    /**
     * 打印交易汇总。
     */
    @Override
    public void printTransSummary(PaymentHandler handler) {
        this.callTrans(PRINT_TRANS_SUMMARY);
    }

    /**
     * 查询银行卡余额。
     */
    @Override
    public void queryBalance(PaymentHandler handler) {
        this.callTrans(QUERY_CARD_BALANCE);
    }

    /**
     * 结算。
     */
    @Override
    public void settle(PaymentHandler handler) {
        this.callTrans(SETTLE);
    }

    /**
     * 换班。
     */
    @Override
    public void shift() {
        // 建行未提供【换班】接口
    }

    /**
     * 班次查询
     */
    @Override
    public void queryBatchNo() {
        //不支持
    }

    /**
     * 交易查询
     *
     * @param traceNo            凭证号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单；
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void tradeQuery(String traceNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        //不支持
    }

    /**
     * @param orderNo            商户订单号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    @Override
    public void scanTradeQuery(String orderNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException {
        //不支持
    }
}
