/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.unionpay.payment.ccb;

import android.app.Activity;

import com.hisense.pos.payment.IPaymentProvider;
import com.hisense.pos.payment.PayRequest;
import com.hisense.pos.payment.PaymentHandler;
import com.hisense.pos.payment.PaymentProviderException;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 建行直联应用的微信支付提供程序。
 *
 * @author xieliyan  2019-10-18 HIPOSAPP-1328
 */
public class UnionCcbScanPayProvider extends UnionCcbPaymentProvider implements IPaymentProvider {

    /**
     * 初始化一个 {@code UnionCcbScanPayProvider} 的实例。
     *
     * @param activity Activity 活动
     */
    public UnionCcbScanPayProvider(Activity activity) {
        super(activity);
    }


    /**
     * 返回是否支持撤销。
     *
     * @return true 表示支持撤销，false 表示不支持撤销
     */
    @Override
    public boolean supportCancellation() {
        return false;
    }

    /**
     * 消费。
     *
     * @param payRequest 交易金额
     */
    @Override
    public void pay(PayRequest payRequest, PaymentHandler handler) {
        this.pay(TransId.CONSUME_SCAN, payRequest);
    }

    /**
     * 退货。
     *
     * @param payRequest 退款金额
     * @param refNo      参考号
     * @param transDate  交易日期
     * @param params     扩展参数
     */
    @Override
    public void refund(PayRequest payRequest, String refNo, Date transDate, Map<String, String> params, PaymentHandler handler) {
        HashMap<String, String> map = new HashMap<>();
        map.put("amt", makeFieldAmount(String.valueOf(payRequest.getAmount())));
        map.put("refund_fee",makeFieldAmount(params.get("refund_fee")));
        map.put("mch_id",params.get("mch_id"));
        map.put("out_trade_no",params.get("out_trade_no"));
        this.callTrans(TransId.REFUNDING_SCAN, map);
    }

    /**
     * 撤销。
     *
     * @param payRequest 退款金额
     * @param traceNo    原交易凭证号
     * @param params     扩展参数
     */
    @Override
    public void cancel(PayRequest payRequest, String traceNo, Map<String, String> params, PaymentHandler handler) {
        try {
            throw new PaymentProviderException("此支付方式不支持撤销，请支付完成后进行退货操作");
        }catch (Exception e){

        }
    }
}