/***************************************************************************************************
 * Copyright (c) 2003-2019 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.database.sqlite;

import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.Collection;
import java.util.Map;

/**
 * SQLite 数据库。
 *
 * @author zhangsai 2019-05-29 HIPOSAPP-1174
 */
public class SQLiteHelper extends SQLiteOpenHelper {

    ISQLiteDatabaseProvider provider;

    /**
     * 初始化一个 {@code SQLiteHelper} 的实例。
     *
     * @param provider SQLite 数据库结构提供程序的实例
     */
    public SQLiteHelper(ISQLiteDatabaseProvider provider) {
        super(provider.context(), provider.getDatabaseName(), null, provider.version());
        this.provider = provider;
    }

    /**
     * 创建数据库。
     *
     * @param sqLiteDatabase SQLite 数据库
     */
    @Override
    public void onCreate(SQLiteDatabase sqLiteDatabase) {
        Collection<String> sqlList = SQLiteDatabaseProvider.getOnCreateSql(this.provider);
        if (sqlList != null) {
            for (String createSql : sqlList) {
                sqLiteDatabase.execSQL(createSql);
            }
        }
    }

    /**
     * 升级数据库。
     *
     * @param sqLiteDatabase SQLite 数据库
     * @param oldVersion     升级前版本
     * @param newVersion     升级后版本
     */
    @Override
    public void onUpgrade(SQLiteDatabase sqLiteDatabase, int oldVersion, int newVersion) {
        Map<Integer, Collection<String>> upgradeSqlMap = SQLiteDatabaseProvider
                .getOnUpgradeSql(this.provider, oldVersion, newVersion);

        // 按 version 顺序执行符合版本要求的升级语句
        for (Integer version : upgradeSqlMap.keySet()) {
            for (String upgradeSql : upgradeSqlMap.get(version)) {
                sqLiteDatabase.execSQL(upgradeSql);
            }
        }
    }
}
