/***************************************************************************************************
 * Copyright (c) 2003-2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/

package com.hisense.pos.card;

/**
 * Mifare1 卡工具类。
 *
 * @author zhangsai 2018-04-25
 */
public class M1Card {

    /**
     * 存储认证的区块。
     */
    public final static int BLOCK_AUTH = 0;

    /**
     * 存储卡号的区块。
     */
    public final static int BLOCK_CARD_NO = 6;

    /**
     * 存储认证数据的存储区。
     */
    public final static int SECTOR_AUTH = 0;

    /**
     * 存储卡号数据的存储区。
     */
    public final static int SECTOR_CARD_NO = 1;

    /**
     * M1 卡版本
     */
    public final static int VERSION1 = 0;

    /**
     * M1 卡版本
     */
    public final static int VERSION2 = 2;

    /**
     * 使用默认配置解密数据。
     *
     * @param encrypted 加密数据
     * @return 解密后的数据
     */
    public static String decrypt(String encrypted) {
        return M1Card.decrypt(encrypted, 6789);
    }

    /**
     * 使用 {@code key} 解密数据。
     *
     * @param encrypted 加密数据
     * @param key       key
     * @return 解密后的数据
     */
    public static String decrypt(String encrypted, int key) {
        String result = "";
        for (int i = 0; i < encrypted.length(); i++) {
            result += (char) (encrypted.codePointAt(i) ^ (key >> 8));
            key = encrypted.codePointAt(i) + key;
        }
        return result;
    }

    /**
     * 将读卡密码转为 SDK 支持的二进制格式。
     *
     * @param pwd 读卡密码
     * @return 二进制格式的读卡密码
     */
    public static byte[] parseKey(String pwd) {
        int len = pwd.length() / 2;
        byte keyData[] = new byte[len];
        for (int i = 0; i < pwd.length(); i += 2) {
            int number = Integer.valueOf(pwd.substring(i, i + 2), 16);
            byte b = (byte) number;
            keyData[i / 2] = b;
        }
        return keyData;
    }
}
