/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment;

import android.content.Intent;

import java.util.Date;
import java.util.Map;

/**
 * 支付提供程序。
 *
 * @author zhangsai    2018-11-24 HIPOSAPP-888
 * @author zhangsai    2018-02-01 HIPOSAPP-898 增加 supportCancellation() 是否支持当日撤销属性
 * @author wangyaogang 2019-08-19 HIPOSAPP-1225  银联商务增加支付查询
 */
public interface IPaymentProvider {

    /**
     * 返回 Activity 请求编码。
     *
     * @return Activity 请求编码
     */
    int requestCode();

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 支付结果
     * @throws PaymentProviderException 支付异常
     */
    PayResult handleResult(Intent data) throws PaymentProviderException;

    /**
     * 返回是否支持撤销。
     *
     * @return true 表示支持撤销，false 表示不支持撤销
     */
    boolean supportCancellation();

    /**
     * 返回是否支持支付查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    boolean supportPayQuery();

    /**
     * 返回是否支持撤销查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    boolean supportCancelQuery();

    /**
     * 返回是否支持退款查询。
     *
     * @return true 表示支持查询，false 表示不支持查询
     */
    boolean supportRefundQuery();

    /**
     * 消费。
     *
     * @param payRequest 交易请求
     */
    void pay(PayRequest payRequest, PaymentHandler handler) throws PaymentProviderException;

    /**
     * 交易查询
     *
     * @param uniqueNo 传入的外部订单号
     * @param handler  结果处理
     * @throws PaymentProviderException
     */
    void payQuery(String uniqueNo, PaymentHandler handler) throws PaymentProviderException;

    /**
     * 退货。
     *
     * @param payRequest 退款请求
     * @param refNo      参考号
     * @param transDate  交易日期
     * @param params     扩展参数
     */
    void refund(PayRequest payRequest, String refNo, Date transDate, Map<String, String> params,
                PaymentHandler handler) throws PaymentProviderException;

    /**
     * 撤销。
     *
     * @param payRequest 撤销请求
     * @param traceNo    原交易凭证号
     * @param params     扩展参数
     */
    void cancel(PayRequest payRequest, String traceNo, Map<String, String> params,
                PaymentHandler handler) throws PaymentProviderException;
}
