/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment;

import android.content.Intent;

import java.util.Map;

/**
 * 支付功能提供程序接口。
 *
 * @author zhangsai 2018-11-23 HIPOSAPP-888
 */
public interface IPaymentServiceProvider {

    /**
     * 处理 Activity 请求返回的结果。
     *
     * @param data Activity 请求返回的结果
     * @return 返回结果字典
     * @throws PaymentProviderException 支付服务异常
     */
    Map handleResult(Intent data) throws PaymentProviderException;

    /**
     * 返回提供程序名称。
     *
     * @return 提供程序名称
     */
    String providerName();

    /**
     * 返回 Activity 请求编码。
     *
     * @return Activity 请求编码
     */
    int requestCode();

    /**
     * 重打印签购单。
     *
     * @param traceNo 交易凭证号
     */
    void reprintReceipt(String traceNo, PaymentHandler handler) throws PaymentProviderException;

    /**
     * 重打印结算单。
     */
    void reprintSettlement();

    /**
     * 打印交易汇总。
     */
    void printTransSummary(PaymentHandler handler) throws PaymentProviderException;

    /**
     * 查询银行卡余额。
     */
    void queryBalance(PaymentHandler handler) throws PaymentProviderException;

    /**
     * 结算。
     */
    void settle(PaymentHandler handler) throws PaymentProviderException;

    /**
     * 换班。
     */
    void shift();

    /**
     * 班次查询
     */
    void queryBatchNo();

    /**
     * 交易查询(理论上银行卡 pos通都可以)
     *
     * @param traceNo            凭证号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单；
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    void tradeQuery(String traceNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException;

    /**
     * 订单查询（pos 通专用）
     *
     * @param orderNo            商户订单号
     * @param isNeedPrintReceipt 是否打单
     *                           true表示POS不打单
     *                           false或者不存在时表示由POS打单
     * @param handler
     * @throws PaymentProviderException
     */
    void scanTradeQuery(String orderNo, boolean isNeedPrintReceipt, PaymentHandler handler) throws PaymentProviderException;
}
