/***************************************************************************************************
 * Copyright (c) 2018 青岛海信智能商用系统股份有限公司 版权所有
 **************************************************************************************************/
package com.hisense.pos.payment;

/**
 * 支付结果。
 *
 * @author zhangsai 2018-11-24 HIPOSAPP-888
 * @author wangyaogang 2020-04-14 HIAPOSCMP-654 增加利客来银联商务支付异常（超时等）查询功能
 */
public abstract class PayResult {
    /**
     * 收单行未返回结果时的异常code
     */
    public static final String CODE_NO_DATA = "404";
    /**
     * 交易超时，请重试(银商)
     */
    public static final String CODE_TRADE_TIMEOUT = "68";
    /**
     * 发卡方超时（银商）
     */
    public static final String CODE_ISSUER_TIMEOUT = "98";
    /**
     * 通讯异常(银商)
     */
    public static final String CODE_COMM_ANOMALY = "FG";

    /**
     * 是否需要查询，暂只有银商的超时code
     *
     * @param code 返回code
     * @return true 需要查询
     */
    public static boolean needPayQuery(String code) {
        return PayResult.CODE_COMM_ANOMALY.equalsIgnoreCase(code)
                || PayResult.CODE_TRADE_TIMEOUT.equalsIgnoreCase(code)
                || PayResult.CODE_ISSUER_TIMEOUT.equalsIgnoreCase(code);
    }

    /**
     * 返回交易金额。
     *
     * @return 交易金额
     */
    public abstract double amount();

    /**
     * 返回卡号。
     *
     * @return 卡号
     */
    public abstract String cardNo();

    /**
     * 返回参考号。
     *
     * @return 参考号
     */
    public abstract String refNo();

    /**
     * 返回交易凭证号。
     *
     * @return 交易凭证号
     */
    public abstract String traceNo();

    /**
     * 返回终端号。
     *
     * @return 终端号
     */
    public abstract String termNo();

    /**
     * 返回商户订单号。
     *
     * @return 商户订单号
     */
    public abstract String outTradeNo();

    /**
     * 返回商户号。
     *
     * @return 商户号
     */
    public abstract String mchId();

    /**
     * 返回聚合支付的支付方式。
     *
     * @return 聚合支付的支付方式
     */
    public abstract String payType();

    /**
     * 返回当前交易是否为消费。
     *
     * @return true 表示消费，false 表示其它。
     */
    public abstract boolean isPayment();

    /**
     * 返回当前交易是否为消费撤销。
     *
     * @return true 表示消费撤销，false 表示其它。
     */
    public abstract boolean isCancellation();

    /**
     * 返回当前交易是否为退货。
     *
     * @return true 表示退货，false 表示其它。
     */
    public abstract boolean isRefunding();

    /**
     * 返回当前交易是否为查询
     *
     * @return true 表示查询，false 表示其它。
     */
    public abstract boolean isQuery();
}
