package com.hisense.database.sqlite;

import org.junit.Test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import static org.junit.Assert.*;

/**
 * Created by zhangsai on 2019-05-29.
 */

public class SQLiteDatabaseProviderTest {

    public class TestProvider extends SQLiteDatabaseProvider {
        public TestProvider() {
            super(null, "test.db", 1);
        }

        @OnCreate
        public String createTable1() {
            return "create table1";
        }

        @OnCreate
        @OnUpgrade(version = 2)
        public String createTable2() {
            return "create table2";
        }

        @OnUpgrade(version = 1)
        public String alterTalbe1() {
            return "alter table1";
        }

        @OnUpgrade(version = 3)
        public Collection<String> alterTable3() {
            ArrayList<String> sql = new ArrayList<>();
            sql.add("alter table3-1");
            sql.add("alter table3-2");
            return sql;
        }
    }

    @Test
    public void onCreate_test() {
        ISQLiteDatabaseProvider provider = new TestProvider();
        Collection<String> onCreateSql = SQLiteDatabaseProvider.getOnCreateSql(provider);
        assertNotNull(onCreateSql);
        assertEquals(2, onCreateSql.size());
    }

    @Test
    public void onUpgrade_test() {
        ISQLiteDatabaseProvider provider = new TestProvider();
        Map<Integer, Collection<String>> upgradeSqlMap = SQLiteDatabaseProvider
                .getOnUpgradeSql(provider, 0, 24);
        assertNotNull(upgradeSqlMap);
        assertEquals(3, upgradeSqlMap.size());
        assertEquals(1, upgradeSqlMap.get(1).size());
        assertEquals(1, upgradeSqlMap.get(2).size());
        assertEquals(2, upgradeSqlMap.get(3).size());
        assertNull(upgradeSqlMap.get(0));
    }
}
